/* Generated by Cython 0.21.1 */

#define PY_SSIZE_T_CLEAN
#ifndef CYTHON_USE_PYLONG_INTERNALS
#ifdef PYLONG_BITS_IN_DIGIT
#define CYTHON_USE_PYLONG_INTERNALS 0
#else
#include "pyconfig.h"
#ifdef PYLONG_BITS_IN_DIGIT
#define CYTHON_USE_PYLONG_INTERNALS 1
#else
#define CYTHON_USE_PYLONG_INTERNALS 0
#endif
#endif
#endif
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03020000)
    #error Cython requires Python 2.6+ or Python 3.2+.
#else
#define CYTHON_ABI "0_21_1"
#include <stddef.h>
#ifndef offsetof
#define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
#define CYTHON_COMPILING_IN_PYPY 1
#define CYTHON_COMPILING_IN_CPYTHON 0
#else
#define CYTHON_COMPILING_IN_PYPY 0
#define CYTHON_COMPILING_IN_CPYTHON 1
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600
#define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos) \
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos) \
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#if PY_MAJOR_VERSION >= 3
  #define Py_TPFLAGS_CHECKTYPES 0
  #define Py_TPFLAGS_HAVE_INDEX 0
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#if PY_VERSION_HEX < 0x030400a1 && !defined(Py_TPFLAGS_HAVE_FINALIZE)
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ? \
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
#else
  #define CYTHON_PEP393_ENABLED 0
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
  #define __Pyx_PyFrozenSet_Size(s)         PyObject_Size(s)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ? \
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
  #define __Pyx_PyFrozenSet_Size(s)         PySet_Size(s)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#ifndef CYTHON_INLINE
  #if defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  /* Initialize NaN. The sign is irrelevant, an exponent with all bits 1 and
   a nonzero mantissa means NaN. If the first bit in the mantissa is 1, it is
   a quiet NaN. */
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#ifdef __cplusplus
template<typename T>
void __Pyx_call_destructor(T* x) {
    x->~T();
}
#endif


#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
#define _USE_MATH_DEFINES
#endif
#include <math.h>
#define __PYX_HAVE__sklearn__neighbors__kd_tree
#define __PYX_HAVE_API__sklearn__neighbors__kd_tree
#include "string.h"
#include "stdio.h"
#include "stdlib.h"
#include "numpy/arrayobject.h"
#include "numpy/ufuncobject.h"
#include "math.h"
#include "pythread.h"
#include "pystate.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
typedef struct {PyObject **p; char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (    \
    (sizeof(type) < sizeof(Py_ssize_t))  ||             \
    (sizeof(type) > sizeof(Py_ssize_t) &&               \
          likely(v < (type)PY_SSIZE_T_MAX ||            \
                 v == (type)PY_SSIZE_T_MAX)  &&         \
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||       \
                                v == (type)PY_SSIZE_T_MIN)))  ||  \
    (sizeof(type) == sizeof(Py_ssize_t) &&              \
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||        \
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyObject_AsSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromUString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromUString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromUString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromUString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromUString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if PY_MAJOR_VERSION < 3
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
#define __Pyx_Py_UNICODE_strlen Py_UNICODE_strlen
#endif
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_Owned_Py_None(b) (Py_INCREF(Py_None), Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? (Py_INCREF(Py_True), Py_True) : (Py_INCREF(Py_False), Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_COMPILING_IN_CPYTHON
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */

static PyObject *__pyx_m;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "sklearn/neighbors/binary_tree.pxi",
  "sklearn/neighbors/kd_tree.pyx",
  "__init__.pxd",
  "sklearn/neighbors/stringsource",
  "sklearn/neighbors/stringsource",
  "type.pxd",
  "sklearn/neighbors/dist_metrics.pxd",
};
#define IS_UNSIGNED(type) (((type) -1) > 0)
struct __Pyx_StructField_;
#define __PYX_BUF_FLAGS_PACKED_STRUCT (1 << 0)
typedef struct {
  const char* name;
  struct __Pyx_StructField_* fields;
  size_t size;
  size_t arraysize[8];
  int ndim;
  char typegroup;
  char is_unsigned;
  int flags;
} __Pyx_TypeInfo;
typedef struct __Pyx_StructField_ {
  __Pyx_TypeInfo* type;
  const char* name;
  size_t offset;
} __Pyx_StructField;
typedef struct {
  __Pyx_StructField* field;
  size_t parent_offset;
} __Pyx_BufFmt_StackElem;
typedef struct {
  __Pyx_StructField root;
  __Pyx_BufFmt_StackElem* head;
  size_t fmt_offset;
  size_t new_count, enc_count;
  size_t struct_alignment;
  int is_complex;
  char enc_type;
  char new_packmode;
  char enc_packmode;
  char is_valid_array;
} __Pyx_BufFmt_Context;

struct __pyx_memoryview_obj;
typedef struct {
  struct __pyx_memoryview_obj *memview;
  char *data;
  Py_ssize_t shape[8];
  Py_ssize_t strides[8];
  Py_ssize_t suboffsets[8];
} __Pyx_memviewslice;

#include <pythread.h>
#ifndef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 1
#endif
#define __pyx_atomic_int_type int
#if CYTHON_ATOMICS && __GNUC__ >= 4 && (__GNUC_MINOR__ > 1 ||           \
                    (__GNUC_MINOR__ == 1 && __GNUC_PATCHLEVEL >= 2)) && \
                    !defined(__i386__)
    #define __pyx_atomic_incr_aligned(value, lock) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_decr_aligned(value, lock) __sync_fetch_and_sub(value, 1)
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using GNU atomics"
    #endif
#elif CYTHON_ATOMICS && MSC_VER
    #include <Windows.h>
    #define __pyx_atomic_int_type LONG
    #define __pyx_atomic_incr_aligned(value, lock) InterlockedIncrement(value)
    #define __pyx_atomic_decr_aligned(value, lock) InterlockedDecrement(value)
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using MSVC atomics"
    #endif
#elif CYTHON_ATOMICS && (defined(__ICC) || defined(__INTEL_COMPILER)) && 0
    #define __pyx_atomic_incr_aligned(value, lock) _InterlockedIncrement(value)
    #define __pyx_atomic_decr_aligned(value, lock) _InterlockedDecrement(value)
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using Intel atomics"
    #endif
#else
    #undef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 0
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Not using atomics"
    #endif
#endif
typedef volatile __pyx_atomic_int_type __pyx_atomic_int;
#if CYTHON_ATOMICS
    #define __pyx_add_acquisition_count(memview) \
             __pyx_atomic_incr_aligned(__pyx_get_slice_count_pointer(memview), memview->lock)
    #define __pyx_sub_acquisition_count(memview) \
            __pyx_atomic_decr_aligned(__pyx_get_slice_count_pointer(memview), memview->lock)
#else
    #define __pyx_add_acquisition_count(memview) \
            __pyx_add_acquisition_count_locked(__pyx_get_slice_count_pointer(memview), memview->lock)
    #define __pyx_sub_acquisition_count(memview) \
            __pyx_sub_acquisition_count_locked(__pyx_get_slice_count_pointer(memview), memview->lock)
#endif


/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":723
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":724
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":725
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":726
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":730
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":731
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":732
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":733
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":737
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":738
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":747
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":748
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":749
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":751
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":752
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":753
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":755
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":756
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":758
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":759
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":760
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;

/* "typedefs.pxd":5
 * 
 * # Floating point/data type
 * ctypedef np.float64_t DTYPE_t  # WARNING: should match DTYPE in typedefs.pyx             # <<<<<<<<<<<<<<
 * 
 * cdef enum:
 */
typedef __pyx_t_5numpy_float64_t __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t;

/* "typedefs.pxd":13
 * # Index/integer type.
 * #  WARNING: ITYPE_t must be a signed integer type or you will have a bad time!
 * ctypedef np.intp_t ITYPE_t  # WARNING: should match ITYPE in typedefs.pyx             # <<<<<<<<<<<<<<
 * 
 * # Fused type for certain operations
 */
typedef __pyx_t_5numpy_intp_t __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t;
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif


/*--- Type declarations ---*/
struct __pyx_obj_7sklearn_9neighbors_12dist_metrics_DistanceMetric;
struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap;
struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap;
struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree;
struct __pyx_obj_7sklearn_9neighbors_7kd_tree_KDTree;
struct __pyx_array_obj;
struct __pyx_MemviewEnum_obj;
struct __pyx_memoryview_obj;
struct __pyx_memoryviewslice_obj;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":762
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":763
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":764
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":766
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;

/* "typedefs.pxd":7
 * ctypedef np.float64_t DTYPE_t  # WARNING: should match DTYPE in typedefs.pyx
 * 
 * cdef enum:             # <<<<<<<<<<<<<<
 *     DTYPECODE = np.NPY_FLOAT64
 *     ITYPECODE = np.NPY_INTP
 */
enum  {

  /* "typedefs.pxd":9
 * cdef enum:
 *     DTYPECODE = np.NPY_FLOAT64
 *     ITYPECODE = np.NPY_INTP             # <<<<<<<<<<<<<<
 * 
 * # Index/integer type.
 */
  __pyx_e_7sklearn_9neighbors_8typedefs_DTYPECODE = NPY_FLOAT64,
  __pyx_e_7sklearn_9neighbors_8typedefs_ITYPECODE = NPY_INTP
};
struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t;
struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t;

/* "binary_tree.pxi":385
 * #       for all valid d and h.  Note that for precision, the log of both
 * #       the kernel and kernel norm is returned.
 * cdef enum KernelType:             # <<<<<<<<<<<<<<
 *     GAUSSIAN_KERNEL = 1
 *     TOPHAT_KERNEL = 2
 */
enum __pyx_t_7sklearn_9neighbors_7kd_tree_KernelType {
  __pyx_e_7sklearn_9neighbors_7kd_tree_GAUSSIAN_KERNEL = 1,
  __pyx_e_7sklearn_9neighbors_7kd_tree_TOPHAT_KERNEL = 2,
  __pyx_e_7sklearn_9neighbors_7kd_tree_EPANECHNIKOV_KERNEL = 3,
  __pyx_e_7sklearn_9neighbors_7kd_tree_EXPONENTIAL_KERNEL = 4,
  __pyx_e_7sklearn_9neighbors_7kd_tree_LINEAR_KERNEL = 5,
  __pyx_e_7sklearn_9neighbors_7kd_tree_COSINE_KERNEL = 6
};

/* "binary_tree.pxi":169
 * 
 * # Some compound datatypes used below:
 * cdef struct NodeHeapData_t:             # <<<<<<<<<<<<<<
 *     DTYPE_t val
 *     ITYPE_t i1
 */
struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t val;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t i1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t i2;
};

/* "binary_tree.pxi":188
 *                          'itemsize': sizeof(NodeHeapData_t)})
 * 
 * cdef struct NodeData_t:             # <<<<<<<<<<<<<<
 *     ITYPE_t idx_start
 *     ITYPE_t idx_end
 */
struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t idx_start;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t idx_end;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t is_leaf;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t radius;
};

/* "dist_metrics.pxd":48
 * ######################################################################
 * # DistanceMetric base class
 * cdef class DistanceMetric:             # <<<<<<<<<<<<<<
 *     # The following attributes are required for a few of the subclasses.
 *     # we must define them here so that cython's limited polymorphism will work.
 */
struct __pyx_obj_7sklearn_9neighbors_12dist_metrics_DistanceMetric {
  PyObject_HEAD
  struct __pyx_vtabstruct_7sklearn_9neighbors_12dist_metrics_DistanceMetric *__pyx_vtab;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t p;
  PyArrayObject *vec;
  PyArrayObject *mat;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *vec_ptr;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *mat_ptr;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t size;
  PyObject *func;
  PyObject *kwargs;
};


/* "binary_tree.pxi":560
 * 
 * 
 * cdef class NeighborsHeap:             # <<<<<<<<<<<<<<
 *     """A max-heap structure to keep track of distances/indices of neighbors
 * 
 */
struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap {
  PyObject_HEAD
  struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_vtab;
  PyArrayObject *distances_arr;
  PyArrayObject *indices_arr;
  __Pyx_memviewslice distances;
  __Pyx_memviewslice indices;
};


/* "binary_tree.pxi":864
 * 
 * 
 * cdef class NodeHeap:             # <<<<<<<<<<<<<<
 *     """NodeHeap
 * 
 */
struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap {
  PyObject_HEAD
  struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_vtab;
  PyArrayObject *data_arr;
  __Pyx_memviewslice data;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t n;
};


/* "binary_tree.pxi":1002
 * ######################################################################
 * # Binary Tree class
 * cdef class BinaryTree:             # <<<<<<<<<<<<<<
 * 
 *     cdef np.ndarray data_arr
 */
struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree {
  PyObject_HEAD
  struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_vtab;
  PyArrayObject *data_arr;
  PyArrayObject *idx_array_arr;
  PyArrayObject *node_data_arr;
  PyArrayObject *node_bounds_arr;
  __Pyx_memviewslice data;
  __Pyx_memviewslice idx_array;
  __Pyx_memviewslice node_data;
  __Pyx_memviewslice node_bounds;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t leaf_size;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t n_levels;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t n_nodes;
  struct __pyx_obj_7sklearn_9neighbors_12dist_metrics_DistanceMetric *dist_metric;
  int euclidean;
  int n_trims;
  int n_leaves;
  int n_splits;
  int n_calls;
};


/* "sklearn/neighbors/kd_tree.pyx":21
 * include "binary_tree.pxi"
 * 
 * cdef class KDTree(BinaryTree):             # <<<<<<<<<<<<<<
 *     __doc__ = CLASS_DOC.format(**DOC_DICT)
 *     pass
 */
struct __pyx_obj_7sklearn_9neighbors_7kd_tree_KDTree {
  struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree __pyx_base;
};


/* "View.MemoryView":99
 * 
 * @cname("__pyx_array")
 * cdef class array:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */
struct __pyx_array_obj {
  PyObject_HEAD
  char *data;
  Py_ssize_t len;
  char *format;
  int ndim;
  Py_ssize_t *_shape;
  Py_ssize_t *_strides;
  Py_ssize_t itemsize;
  PyObject *mode;
  PyObject *_format;
  void (*callback_free_data)(void *);
  int free_data;
  int dtype_is_object;
};


/* "View.MemoryView":269
 * 
 * @cname('__pyx_MemviewEnum')
 * cdef class Enum(object):             # <<<<<<<<<<<<<<
 *     cdef object name
 *     def __init__(self, name):
 */
struct __pyx_MemviewEnum_obj {
  PyObject_HEAD
  PyObject *name;
};


/* "View.MemoryView":302
 * 
 * @cname('__pyx_memoryview')
 * cdef class memoryview(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef object obj
 */
struct __pyx_memoryview_obj {
  PyObject_HEAD
  struct __pyx_vtabstruct_memoryview *__pyx_vtab;
  PyObject *obj;
  PyObject *_size;
  PyObject *_array_interface;
  PyThread_type_lock lock;
  __pyx_atomic_int acquisition_count[2];
  __pyx_atomic_int *acquisition_count_aligned_p;
  Py_buffer view;
  int flags;
  int dtype_is_object;
  __Pyx_TypeInfo *typeinfo;
};


/* "View.MemoryView":922
 * 
 * @cname('__pyx_memoryviewslice')
 * cdef class _memoryviewslice(memoryview):             # <<<<<<<<<<<<<<
 *     "Internal class for passing memoryview slices to Python"
 * 
 */
struct __pyx_memoryviewslice_obj {
  struct __pyx_memoryview_obj __pyx_base;
  __Pyx_memviewslice from_slice;
  PyObject *from_object;
  PyObject *(*to_object_func)(char *);
  int (*to_dtype_func)(char *, PyObject *);
};



/* "dist_metrics.pxd":48
 * ######################################################################
 * # DistanceMetric base class
 * cdef class DistanceMetric:             # <<<<<<<<<<<<<<
 *     # The following attributes are required for a few of the subclasses.
 *     # we must define them here so that cython's limited polymorphism will work.
 */

struct __pyx_vtabstruct_7sklearn_9neighbors_12dist_metrics_DistanceMetric {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t (*dist)(struct __pyx_obj_7sklearn_9neighbors_12dist_metrics_DistanceMetric *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t);
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t (*rdist)(struct __pyx_obj_7sklearn_9neighbors_12dist_metrics_DistanceMetric *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t);
  int (*pdist)(struct __pyx_obj_7sklearn_9neighbors_12dist_metrics_DistanceMetric *, __Pyx_memviewslice, __Pyx_memviewslice);
  int (*cdist)(struct __pyx_obj_7sklearn_9neighbors_12dist_metrics_DistanceMetric *, __Pyx_memviewslice, __Pyx_memviewslice, __Pyx_memviewslice);
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t (*_rdist_to_dist)(struct __pyx_obj_7sklearn_9neighbors_12dist_metrics_DistanceMetric *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t);
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t (*_dist_to_rdist)(struct __pyx_obj_7sklearn_9neighbors_12dist_metrics_DistanceMetric *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t);
};
static struct __pyx_vtabstruct_7sklearn_9neighbors_12dist_metrics_DistanceMetric *__pyx_vtabptr_7sklearn_9neighbors_12dist_metrics_DistanceMetric;


/* "binary_tree.pxi":560
 * 
 * 
 * cdef class NeighborsHeap:             # <<<<<<<<<<<<<<
 *     """A max-heap structure to keep track of distances/indices of neighbors
 * 
 */

struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NeighborsHeap {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t (*largest)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t);
  int (*_push)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t);
  int (*_sort)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *);
};
static struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_vtabptr_7sklearn_9neighbors_7kd_tree_NeighborsHeap;
static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_largest(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t);


/* "binary_tree.pxi":864
 * 
 * 
 * cdef class NodeHeap:             # <<<<<<<<<<<<<<
 *     """NodeHeap
 * 
 */

struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NodeHeap {
  int (*resize)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t);
  int (*push)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *, struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t);
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t (*peek)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *);
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t (*pop)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *);
  void (*clear)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *);
};
static struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_vtabptr_7sklearn_9neighbors_7kd_tree_NodeHeap;


/* "binary_tree.pxi":1002
 * ######################################################################
 * # Binary Tree class
 * cdef class BinaryTree:             # <<<<<<<<<<<<<<
 * 
 *     cdef np.ndarray data_arr
 */

struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t (*dist)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t);
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t (*rdist)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t);
  int (*_recursive_build)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t);
  int (*_query_single_depthfirst)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t);
  int (*_query_single_breadthfirst)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *);
  int (*_query_dual_depthfirst)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __Pyx_memviewslice, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t);
  int (*_query_dual_breadthfirst)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *);
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t (*_query_radius_single)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, int, int);
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t (*_kde_single_breadthfirst)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, enum __pyx_t_7sklearn_9neighbors_7kd_tree_KernelType, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *);
  int (*_kde_single_depthfirst)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, enum __pyx_t_7sklearn_9neighbors_7kd_tree_KernelType, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *);
  int (*_two_point_single)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t);
  int (*_two_point_dual)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t);
};
static struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_vtabptr_7sklearn_9neighbors_7kd_tree_BinaryTree;
static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree_dist(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t);
static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree_rdist(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t);


/* "sklearn/neighbors/kd_tree.pyx":21
 * include "binary_tree.pxi"
 * 
 * cdef class KDTree(BinaryTree):             # <<<<<<<<<<<<<<
 *     __doc__ = CLASS_DOC.format(**DOC_DICT)
 *     pass
 */

struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_KDTree {
  struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree __pyx_base;
};
static struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_KDTree *__pyx_vtabptr_7sklearn_9neighbors_7kd_tree_KDTree;


/* "View.MemoryView":302
 * 
 * @cname('__pyx_memoryview')
 * cdef class memoryview(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef object obj
 */

struct __pyx_vtabstruct_memoryview {
  char *(*get_item_pointer)(struct __pyx_memoryview_obj *, PyObject *);
  PyObject *(*is_slice)(struct __pyx_memoryview_obj *, PyObject *);
  PyObject *(*setitem_slice_assignment)(struct __pyx_memoryview_obj *, PyObject *, PyObject *);
  PyObject *(*setitem_slice_assign_scalar)(struct __pyx_memoryview_obj *, struct __pyx_memoryview_obj *, PyObject *);
  PyObject *(*setitem_indexed)(struct __pyx_memoryview_obj *, PyObject *, PyObject *);
  PyObject *(*convert_item_to_object)(struct __pyx_memoryview_obj *, char *);
  PyObject *(*assign_item_from_object)(struct __pyx_memoryview_obj *, char *, PyObject *);
};
static struct __pyx_vtabstruct_memoryview *__pyx_vtabptr_memoryview;


/* "View.MemoryView":922
 * 
 * @cname('__pyx_memoryviewslice')
 * cdef class _memoryviewslice(memoryview):             # <<<<<<<<<<<<<<
 *     "Internal class for passing memoryview slices to Python"
 * 
 */

struct __pyx_vtabstruct__memoryviewslice {
  struct __pyx_vtabstruct_memoryview __pyx_base;
};
static struct __pyx_vtabstruct__memoryviewslice *__pyx_vtabptr__memoryviewslice;
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil) \
          if (acquire_gil) { \
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure(); \
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__); \
              PyGILState_Release(__pyx_gilstate_save); \
          } else { \
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__); \
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil) \
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext() \
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {                            \
        PyObject *tmp = (PyObject *) r;                         \
        r = v; __Pyx_XDECREF(tmp);                              \
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {                             \
        PyObject *tmp = (PyObject *) r;                         \
        r = v; __Pyx_DECREF(tmp);                               \
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name);

static CYTHON_INLINE int  __Pyx_GetBufferAndValidate(Py_buffer* buf, PyObject* obj,
    __Pyx_TypeInfo* dtype, int flags, int nd, int cast, __Pyx_BufFmt_StackElem* stack);
static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info);

#define __Pyx_BUF_MAX_NDIMS %(BUF_MAX_NDIMS)d
#define __Pyx_MEMVIEW_DIRECT   1
#define __Pyx_MEMVIEW_PTR      2
#define __Pyx_MEMVIEW_FULL     4
#define __Pyx_MEMVIEW_CONTIG   8
#define __Pyx_MEMVIEW_STRIDED  16
#define __Pyx_MEMVIEW_FOLLOW   32
#define __Pyx_IS_C_CONTIG 1
#define __Pyx_IS_F_CONTIG 2
static int __Pyx_init_memviewslice(
                struct __pyx_memoryview_obj *memview,
                int ndim,
                __Pyx_memviewslice *memviewslice,
                int memview_is_new_reference);
static CYTHON_INLINE int __pyx_add_acquisition_count_locked(
    __pyx_atomic_int *acquisition_count, PyThread_type_lock lock);
static CYTHON_INLINE int __pyx_sub_acquisition_count_locked(
    __pyx_atomic_int *acquisition_count, PyThread_type_lock lock);
#define __pyx_get_slice_count_pointer(memview) (memview->acquisition_count_aligned_p)
#define __pyx_get_slice_count(memview) (*__pyx_get_slice_count_pointer(memview))
#define __PYX_INC_MEMVIEW(slice, have_gil) __Pyx_INC_MEMVIEW(slice, have_gil, __LINE__)
#define __PYX_XDEC_MEMVIEW(slice, have_gil) __Pyx_XDEC_MEMVIEW(slice, have_gil, __LINE__)
static CYTHON_INLINE void __Pyx_INC_MEMVIEW(__Pyx_memviewslice *, int, int);
static CYTHON_INLINE void __Pyx_XDEC_MEMVIEW(__Pyx_memviewslice *, int, int);

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb);

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[], \
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args, \
    const char* function_name);

#include <string.h>

static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

static CYTHON_INLINE int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback);

static CYTHON_INLINE int __Pyx_PySequence_Contains(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck) \
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ? \
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) : \
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) : \
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck) \
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ? \
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) : \
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck) \
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ? \
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) : \
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname);

static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

static CYTHON_INLINE int __Pyx_IterFinish(void);

static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck) \
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ? \
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) : \
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) : \
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static CYTHON_INLINE int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact);

#define UNARY_NEG_WOULD_OVERFLOW(x)            (((x) < 0) & ((unsigned long)(x) == 0-(unsigned long)(x)))

static CYTHON_UNUSED int __pyx_array_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static PyObject *get_memview(PyObject *__pyx_v_self); /*proto*/
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors));

static CYTHON_INLINE void __Pyx_ExceptionSave(PyObject **type, PyObject **value, PyObject **tb);
static void __Pyx_ExceptionReset(PyObject *type, PyObject *value, PyObject *tb);

static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);

static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);

static CYTHON_UNUSED int __pyx_memoryview_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static PyObject *__pyx_memoryview_transpose(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview__get__base(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_shape(PyObject *__pyx_v_self); /*proto*/
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

static PyObject *__pyx_memoryview_get_strides(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_suboffsets(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_ndim(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_itemsize(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_nbytes(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_size(PyObject *__pyx_v_self); /*proto*/
static CYTHON_INLINE int __Pyx_PyList_Extend(PyObject* L, PyObject* v) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject* none = _PyList_Extend((PyListObject*)L, v);
    if (unlikely(!none))
        return -1;
    Py_DECREF(none);
    return 0;
#else
    return PyList_SetSlice(L, PY_SSIZE_T_MAX, PY_SSIZE_T_MAX, v);
#endif
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

static PyObject *__pyx_memoryviewslice__get__base(PyObject *__pyx_v_self); /*proto*/
#if PY_MAJOR_VERSION < 3
#define __Pyx_PyString_Join __Pyx_PyBytes_Join
#define __Pyx_PyBaseString_Join(s, v) (PyUnicode_CheckExact(s) ? PyUnicode_Join(s, v) : __Pyx_PyBytes_Join(s, v))
#else
#define __Pyx_PyString_Join PyUnicode_Join
#define __Pyx_PyBaseString_Join PyUnicode_Join
#endif
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION < 3
    #define __Pyx_PyBytes_Join _PyString_Join
    #else
    #define __Pyx_PyBytes_Join _PyBytes_Join
    #endif
#else
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values);
#endif

static int __Pyx_SetVtable(PyObject *dict, void *vtable);

static void* __Pyx_GetVtable(PyObject *dict);

static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

typedef struct {
    int code_line;
    PyCodeObject* code_object;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

static int __pyx_typeinfo_cmp(__Pyx_TypeInfo *a, __Pyx_TypeInfo *b);

static int __Pyx_ValidateAndInit_memviewslice(
                int *axes_specs,
                int c_or_f_flag,
                int buf_flags,
                int ndim,
                __Pyx_TypeInfo *dtype,
                __Pyx_BufFmt_StackElem stack[],
                __Pyx_memviewslice *memviewslice,
                PyObject *original_obj);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dc_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_d_d_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t(PyObject *);

typedef struct {
  Py_ssize_t shape, strides, suboffsets;
} __Pyx_Buf_DimInfo;
typedef struct {
  size_t refcount;
  Py_buffer pybuffer;
} __Pyx_Buffer;
typedef struct {
  __Pyx_Buffer *rcbuffer;
  char *data;
  __Pyx_Buf_DimInfo diminfo[8];
} __Pyx_LocalBuf_ND;

#if PY_MAJOR_VERSION < 3
    static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags);
    static void __Pyx_ReleaseBuffer(Py_buffer *view);
#else
    #define __Pyx_GetBuffer PyObject_GetBuffer
    #define __Pyx_ReleaseBuffer PyBuffer_Release
#endif


static Py_ssize_t __Pyx_zeros[] = {0, 0, 0, 0, 0, 0, 0, 0};
static Py_ssize_t __Pyx_minusones[] = {-1, -1, -1, -1, -1, -1, -1, -1};

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

static CYTHON_INLINE Py_intptr_t __Pyx_PyInt_As_Py_intptr_t(PyObject *);

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_d_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_d_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t(PyObject *);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Py_intptr_t(Py_intptr_t value);

static PyObject *__pyx_memview_get_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t(const char *itemp);
static int __pyx_memview_set_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t(const char *itemp, PyObject *obj);

static CYTHON_INLINE long __Pyx_pow_long(long, long); /* proto */

static PyObject *__pyx_memview_get_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t(const char *itemp);
static int __pyx_memview_set_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t(const char *itemp, PyObject *obj);

struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t;
static PyObject* __pyx_convert__to_py___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t(struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t s);
struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t;
static struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t __pyx_convert__from_py___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t(PyObject *);

static PyObject *__pyx_memview_get_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t(const char *itemp);
static int __pyx_memview_set_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t(const char *itemp, PyObject *obj);

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if (defined(_WIN32) || defined(__clang__)) && defined(__cplusplus) && CYTHON_CCOMPLEX
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

#if CYTHON_CCOMPLEX
    #define __Pyx_c_eqf(a, b)   ((a)==(b))
    #define __Pyx_c_sumf(a, b)  ((a)+(b))
    #define __Pyx_c_difff(a, b) ((a)-(b))
    #define __Pyx_c_prodf(a, b) ((a)*(b))
    #define __Pyx_c_quotf(a, b) ((a)/(b))
    #define __Pyx_c_negf(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zerof(z) ((z)==(float)0)
    #define __Pyx_c_conjf(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_absf(z)     (::std::abs(z))
        #define __Pyx_c_powf(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zerof(z) ((z)==0)
    #define __Pyx_c_conjf(z)    (conjf(z))
    #if 1
        #define __Pyx_c_absf(z)     (cabsf(z))
        #define __Pyx_c_powf(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eqf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sumf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_difff(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prodf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quotf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_negf(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zerof(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conjf(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_absf(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_powf(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);

#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq(a, b)   ((a)==(b))
    #define __Pyx_c_sum(a, b)  ((a)+(b))
    #define __Pyx_c_diff(a, b) ((a)-(b))
    #define __Pyx_c_prod(a, b) ((a)*(b))
    #define __Pyx_c_quot(a, b) ((a)/(b))
    #define __Pyx_c_neg(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero(z) ((z)==(double)0)
    #define __Pyx_c_conj(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs(z)     (::std::abs(z))
        #define __Pyx_c_pow(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero(z) ((z)==0)
    #define __Pyx_c_conj(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs(z)     (cabs(z))
        #define __Pyx_c_pow(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

static int __pyx_memviewslice_is_contig(const __Pyx_memviewslice *mvs,
                                        char order, int ndim);

static int __pyx_slices_overlap(__Pyx_memviewslice *slice1,
                                __Pyx_memviewslice *slice2,
                                int ndim, size_t itemsize);

static __Pyx_memviewslice
__pyx_memoryview_copy_new_contig(const __Pyx_memviewslice *from_mvs,
                                 const char *mode, int ndim,
                                 size_t sizeof_dtype, int contig_flag,
                                 int dtype_is_object);

static CYTHON_INLINE PyObject *__pyx_capsule_create(void *p, const char *sig);

static CYTHON_INLINE char __Pyx_PyInt_As_char(PyObject *);

struct __pyx_typeinfo_string {
    char string[3];
};
static struct __pyx_typeinfo_string __Pyx_TypeInfoToFormat(__Pyx_TypeInfo *type);

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dc_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t(PyObject *);

static int __Pyx_check_binary_version(void);

#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

static PyObject *__Pyx_ImportModule(const char *name);

static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_largest(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_row); /* proto*/
static int __pyx_f_7sklearn_9neighbors_7kd_tree_13NeighborsHeap__push(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_row, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_val, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_val); /* proto*/
static int __pyx_f_7sklearn_9neighbors_7kd_tree_13NeighborsHeap__sort(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_self); /* proto*/
static int __pyx_f_7sklearn_9neighbors_7kd_tree_8NodeHeap_resize(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_new_size); /* proto*/
static int __pyx_f_7sklearn_9neighbors_7kd_tree_8NodeHeap_push(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_self, struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t __pyx_v_data); /* proto*/
static struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t __pyx_f_7sklearn_9neighbors_7kd_tree_8NodeHeap_peek(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_self); /* proto*/
static struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t __pyx_f_7sklearn_9neighbors_7kd_tree_8NodeHeap_pop(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_self); /* proto*/
static void __pyx_f_7sklearn_9neighbors_7kd_tree_8NodeHeap_clear(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_self); /* proto*/
static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree_dist(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_x1, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_x2, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_size); /* proto*/
static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree_rdist(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_x1, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_x2, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_size); /* proto*/
static int __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__recursive_build(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_idx_start, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_idx_end); /* proto*/
static int __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__query_single_depthfirst(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_pt, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_heap, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_reduced_dist_LB); /* proto*/
static int __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__query_single_breadthfirst(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_pt, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_heap, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_nodeheap); /* proto*/
static int __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__query_dual_depthfirst(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node1, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_other, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node2, __Pyx_memviewslice __pyx_v_bounds, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_heap, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_reduced_dist_LB); /* proto*/
static int __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__query_dual_breadthfirst(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_other, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_heap, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_nodeheap); /* proto*/
static __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__query_radius_single(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_r, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_v_indices, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_distances, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_count, int __pyx_v_count_only, int __pyx_v_return_distance); /* proto*/
static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__kde_single_breadthfirst(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt, enum __pyx_t_7sklearn_9neighbors_7kd_tree_KernelType __pyx_v_kernel, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_h, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_log_knorm, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_log_atol, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_log_rtol, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_nodeheap, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_node_log_min_bounds, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_node_log_bound_spreads); /* proto*/
static int __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__kde_single_depthfirst(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt, enum __pyx_t_7sklearn_9neighbors_7kd_tree_KernelType __pyx_v_kernel, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_h, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_log_knorm, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_log_atol, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_log_rtol, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_local_log_min_bound, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_local_log_bound_spread, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_global_log_min_bound, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_global_log_bound_spread); /* proto*/
static int __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__two_point_single(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_r, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_v_count, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_min, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_max); /* proto*/
static int __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__two_point_dual(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node1, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_other, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node2, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_r, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_v_count, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_min, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_max); /* proto*/
static char *__pyx_memoryview_get_item_pointer(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index); /* proto*/
static PyObject *__pyx_memoryview_is_slice(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj); /* proto*/
static PyObject *__pyx_memoryview_setitem_slice_assignment(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_dst, PyObject *__pyx_v_src); /* proto*/
static PyObject *__pyx_memoryview_setitem_slice_assign_scalar(struct __pyx_memoryview_obj *__pyx_v_self, struct __pyx_memoryview_obj *__pyx_v_dst, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_memoryview_setitem_indexed(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_memoryview_convert_item_to_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp); /* proto*/
static PyObject *__pyx_memoryview_assign_item_from_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_memoryviewslice_convert_item_to_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp); /* proto*/
static PyObject *__pyx_memoryviewslice_assign_item_from_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value); /* proto*/

/* Module declarations from 'cython.view' */
static struct __pyx_array_obj *__pyx_array_new(PyObject *, Py_ssize_t, char *, char *, char *); /*proto*/

/* Module declarations from 'cython' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'cpython.object' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;
static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *, char *, char *, int *); /*proto*/

/* Module declarations from 'libc.math' */

/* Module declarations from 'sklearn.utils.lgamma' */
static double (*__pyx_f_7sklearn_5utils_6lgamma_lgamma)(double); /*proto*/

/* Module declarations from 'sklearn.neighbors.typedefs' */

/* Module declarations from 'sklearn.neighbors.dist_metrics' */
static PyTypeObject *__pyx_ptype_7sklearn_9neighbors_12dist_metrics_DistanceMetric = 0;
static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_12dist_metrics_euclidean_dist(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t); /*proto*/
static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_12dist_metrics_euclidean_rdist(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t); /*proto*/
static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_12dist_metrics_euclidean_dist_to_rdist(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t); /*proto*/
static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_12dist_metrics_euclidean_rdist_to_dist(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t); /*proto*/

/* Module declarations from 'sklearn.neighbors.kd_tree' */
static PyTypeObject *__pyx_ptype_7sklearn_9neighbors_7kd_tree_NeighborsHeap = 0;
static PyTypeObject *__pyx_ptype_7sklearn_9neighbors_7kd_tree_NodeHeap = 0;
static PyTypeObject *__pyx_ptype_7sklearn_9neighbors_7kd_tree_BinaryTree = 0;
static PyTypeObject *__pyx_ptype_7sklearn_9neighbors_7kd_tree_KDTree = 0;
static PyTypeObject *__pyx_array_type = 0;
static PyTypeObject *__pyx_MemviewEnum_type = 0;
static PyTypeObject *__pyx_memoryview_type = 0;
static PyTypeObject *__pyx_memoryviewslice_type = 0;
static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_7sklearn_9neighbors_7kd_tree_INF;
static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_7sklearn_9neighbors_7kd_tree_NEG_INF;
static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_7sklearn_9neighbors_7kd_tree_PI;
static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_7sklearn_9neighbors_7kd_tree_ROOT_2PI;
static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_7sklearn_9neighbors_7kd_tree_LOG_PI;
static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_7sklearn_9neighbors_7kd_tree_LOG_2PI;
static struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t __pyx_v_7sklearn_9neighbors_7kd_tree_nhd_tmp;
static struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t __pyx_v_7sklearn_9neighbors_7kd_tree_nd_tmp;
static PyObject *generic = 0;
static PyObject *strided = 0;
static PyObject *indirect = 0;
static PyObject *contiguous = 0;
static PyObject *indirect_contiguous = 0;
static __Pyx_memviewslice __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_1D(PyArrayObject *); /*proto*/
static __Pyx_memviewslice __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_2D(PyArrayObject *); /*proto*/
static __Pyx_memviewslice __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_3D(PyArrayObject *); /*proto*/
static __Pyx_memviewslice __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_ITYPE_1D(PyArrayObject *); /*proto*/
static __Pyx_memviewslice __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_ITYPE_2D(PyArrayObject *); /*proto*/
static __Pyx_memviewslice __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_NodeHeapData_1D(PyArrayObject *); /*proto*/
static __Pyx_memviewslice __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_NodeData_1D(PyArrayObject *); /*proto*/
static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_logaddexp(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t); /*proto*/
static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_logsubexp(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t); /*proto*/
static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_log_gaussian_kernel(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t); /*proto*/
static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_log_tophat_kernel(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t); /*proto*/
static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_log_epanechnikov_kernel(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t); /*proto*/
static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_log_exponential_kernel(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t); /*proto*/
static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_log_linear_kernel(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t); /*proto*/
static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_log_cosine_kernel(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t); /*proto*/
static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_compute_log_kernel(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, enum __pyx_t_7sklearn_9neighbors_7kd_tree_KernelType); /*proto*/
static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_logVn(__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t); /*proto*/
static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_logSn(__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t); /*proto*/
static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree__log_kernel_norm(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, enum __pyx_t_7sklearn_9neighbors_7kd_tree_KernelType); /*proto*/
static CYTHON_INLINE void __pyx_f_7sklearn_9neighbors_7kd_tree_dual_swap(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t); /*proto*/
static int __pyx_f_7sklearn_9neighbors_7kd_tree__simultaneous_sort(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t); /*proto*/
static __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_find_node_split_dim(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t); /*proto*/
static int __pyx_f_7sklearn_9neighbors_7kd_tree_partition_node_indices(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t); /*proto*/
static CYTHON_INLINE void __pyx_f_7sklearn_9neighbors_7kd_tree_swap_nodes(struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t); /*proto*/
static CYTHON_INLINE double __pyx_f_7sklearn_9neighbors_7kd_tree_fmin(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_7sklearn_9neighbors_7kd_tree_fmax(double, double); /*proto*/
static int __pyx_f_7sklearn_9neighbors_7kd_tree_allocate_data(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t); /*proto*/
static int __pyx_f_7sklearn_9neighbors_7kd_tree_init_node(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t); /*proto*/
static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_min_rdist(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *); /*proto*/
static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_min_dist(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *); /*proto*/
static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_max_rdist(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *); /*proto*/
static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_max_dist(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *); /*proto*/
static CYTHON_INLINE int __pyx_f_7sklearn_9neighbors_7kd_tree_min_max_dist(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *); /*proto*/
static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_min_rdist_dual(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t); /*proto*/
static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_min_dist_dual(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t); /*proto*/
static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_max_rdist_dual(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t); /*proto*/
static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_max_dist_dual(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t); /*proto*/
static CYTHON_INLINE void __pyx_fuse_0__pyx_f_7sklearn_9neighbors_7kd_tree_swap(__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *, int, int); /*proto*/
static struct __pyx_array_obj *__pyx_array_new(PyObject *, Py_ssize_t, char *, char *, char *); /*proto*/
static void *__pyx_align_pointer(void *, size_t); /*proto*/
static PyObject *__pyx_memoryview_new(PyObject *, int, int, __Pyx_TypeInfo *); /*proto*/
static CYTHON_INLINE int __pyx_memoryview_check(PyObject *); /*proto*/
static PyObject *_unellipsify(PyObject *, int); /*proto*/
static PyObject *assert_direct_dimensions(Py_ssize_t *, int); /*proto*/
static struct __pyx_memoryview_obj *__pyx_memview_slice(struct __pyx_memoryview_obj *, PyObject *); /*proto*/
static int __pyx_memoryview_slice_memviewslice(__Pyx_memviewslice *, Py_ssize_t, Py_ssize_t, Py_ssize_t, int, int, int *, Py_ssize_t, Py_ssize_t, Py_ssize_t, int, int, int, int); /*proto*/
static char *__pyx_pybuffer_index(Py_buffer *, char *, Py_ssize_t, Py_ssize_t); /*proto*/
static int __pyx_memslice_transpose(__Pyx_memviewslice *); /*proto*/
static PyObject *__pyx_memoryview_fromslice(__Pyx_memviewslice, int, PyObject *(*)(char *), int (*)(char *, PyObject *), int); /*proto*/
static __Pyx_memviewslice *__pyx_memoryview_get_slice_from_memoryview(struct __pyx_memoryview_obj *, __Pyx_memviewslice *); /*proto*/
static void __pyx_memoryview_slice_copy(struct __pyx_memoryview_obj *, __Pyx_memviewslice *); /*proto*/
static PyObject *__pyx_memoryview_copy_object(struct __pyx_memoryview_obj *); /*proto*/
static PyObject *__pyx_memoryview_copy_object_from_slice(struct __pyx_memoryview_obj *, __Pyx_memviewslice *); /*proto*/
static Py_ssize_t abs_py_ssize_t(Py_ssize_t); /*proto*/
static char __pyx_get_best_slice_order(__Pyx_memviewslice *, int); /*proto*/
static void _copy_strided_to_strided(char *, Py_ssize_t *, char *, Py_ssize_t *, Py_ssize_t *, Py_ssize_t *, int, size_t); /*proto*/
static void copy_strided_to_strided(__Pyx_memviewslice *, __Pyx_memviewslice *, int, size_t); /*proto*/
static Py_ssize_t __pyx_memoryview_slice_get_size(__Pyx_memviewslice *, int); /*proto*/
static Py_ssize_t __pyx_fill_contig_strides_array(Py_ssize_t *, Py_ssize_t *, Py_ssize_t, int, char); /*proto*/
static void *__pyx_memoryview_copy_data_to_temp(__Pyx_memviewslice *, __Pyx_memviewslice *, char, int); /*proto*/
static int __pyx_memoryview_err_extents(int, Py_ssize_t, Py_ssize_t); /*proto*/
static int __pyx_memoryview_err_dim(PyObject *, char *, int); /*proto*/
static int __pyx_memoryview_err(PyObject *, char *); /*proto*/
static int __pyx_memoryview_copy_contents(__Pyx_memviewslice, __Pyx_memviewslice, int, int, int); /*proto*/
static void __pyx_memoryview_broadcast_leading(__Pyx_memviewslice *, int, int); /*proto*/
static void __pyx_memoryview_refcount_copying(__Pyx_memviewslice *, int, int, int); /*proto*/
static void __pyx_memoryview_refcount_objects_in_slice_with_gil(char *, Py_ssize_t *, Py_ssize_t *, int, int); /*proto*/
static void __pyx_memoryview_refcount_objects_in_slice(char *, Py_ssize_t *, Py_ssize_t *, int, int); /*proto*/
static void __pyx_memoryview_slice_assign_scalar(__Pyx_memviewslice *, int, size_t, void *, int); /*proto*/
static void __pyx_memoryview__slice_assign_scalar(char *, Py_ssize_t *, Py_ssize_t *, int, size_t, void *); /*proto*/
static PyObject *__pyx_format_from_typeinfo(__Pyx_TypeInfo *); /*proto*/
static __Pyx_TypeInfo __Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t = { "DTYPE_t", NULL, sizeof(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t), { 0 }, 0, 'R', 0, 0 };
static __Pyx_TypeInfo __Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t = { "ITYPE_t", NULL, sizeof(__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t), { 0 }, 0, IS_UNSIGNED(__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t) ? 'U' : 'I', IS_UNSIGNED(__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t), 0 };
static __Pyx_StructField __Pyx_StructFields_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t[] = {
  {&__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, "val", offsetof(struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t, val)},
  {&__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, "i1", offsetof(struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t, i1)},
  {&__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, "i2", offsetof(struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t, i2)},
  {NULL, NULL, 0}
};
static __Pyx_TypeInfo __Pyx_TypeInfo_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t = { "NodeHeapData_t", __Pyx_StructFields_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t, sizeof(struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t), { 0 }, 0, 'S', 0, 0 };
static __Pyx_StructField __Pyx_StructFields_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t[] = {
  {&__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, "idx_start", offsetof(struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t, idx_start)},
  {&__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, "idx_end", offsetof(struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t, idx_end)},
  {&__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, "is_leaf", offsetof(struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t, is_leaf)},
  {&__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, "radius", offsetof(struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t, radius)},
  {NULL, NULL, 0}
};
static __Pyx_TypeInfo __Pyx_TypeInfo_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t = { "NodeData_t", __Pyx_StructFields_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t, sizeof(struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t), { 0 }, 0, 'S', 0, 0 };
#define __Pyx_MODULE_NAME "sklearn.neighbors.kd_tree"
int __pyx_module_is_main_sklearn__neighbors__kd_tree = 0;

/* Implementation of 'sklearn.neighbors.kd_tree' */
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_RuntimeError;
static PyObject *__pyx_builtin_MemoryError;
static PyObject *__pyx_builtin_enumerate;
static PyObject *__pyx_builtin_Ellipsis;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_xrange;
static PyObject *__pyx_builtin_id;
static PyObject *__pyx_builtin_IndexError;
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_kernel_norm(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_h, PyObject *__pyx_v_d, PyObject *__pyx_v_kernel, PyObject *__pyx_v_return_log); /* proto */
static int __pyx_pf_7sklearn_9neighbors_7kd_tree_13NeighborsHeap___cinit__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_self); /* proto */
static int __pyx_pf_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_2__init__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_self, PyObject *__pyx_v_n_pts, PyObject *__pyx_v_n_nbrs); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_4get_arrays(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_self, PyObject *__pyx_v_sort); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_6push(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_row, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_val, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_val); /* proto */
static int __pyx_pf_7sklearn_9neighbors_7kd_tree_8NodeHeap___cinit__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_self); /* proto */
static int __pyx_pf_7sklearn_9neighbors_7kd_tree_8NodeHeap_2__init__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_self, PyObject *__pyx_v_size_guess); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_2newObj(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_obj); /* proto */
static int __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree___cinit__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self); /* proto */
static int __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_2__init__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_leaf_size, PyObject *__pyx_v_metric, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_4__reduce__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_6__getstate__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_8__setstate__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, PyObject *__pyx_v_state); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_10get_tree_stats(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_12reset_n_calls(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_14get_n_calls(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_16get_arrays(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_18query(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, PyObject *__pyx_v_X, PyObject *__pyx_v_k, PyObject *__pyx_v_return_distance, PyObject *__pyx_v_dualtree, PyObject *__pyx_v_breadth_first, PyObject *__pyx_v_sort_results); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_20query_radius(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, PyObject *__pyx_v_X, PyObject *__pyx_v_r, PyObject *__pyx_v_return_distance, int __pyx_v_count_only, int __pyx_v_sort_results); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_22kernel_density(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, PyObject *__pyx_v_X, PyObject *__pyx_v_h, PyObject *__pyx_v_kernel, PyObject *__pyx_v_atol, PyObject *__pyx_v_rtol, PyObject *__pyx_v_breadth_first, PyObject *__pyx_v_return_log); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_24two_point_correlation(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, PyObject *__pyx_v_X, PyObject *__pyx_v_r, PyObject *__pyx_v_dualtree); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_4data___get__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_9idx_array___get__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self); /* proto */
static int __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_9idx_array_2__set__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_9node_data___get__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self); /* proto */
static int __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_9node_data_2__set__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_11node_bounds___get__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self); /* proto */
static int __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_11node_bounds_2__set__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_4load_heap(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_X, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_k); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_6simultaneous_sort(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_distances, __Pyx_memviewslice __pyx_v_indices); /* proto */
static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_8nodeheap_sort(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_vals); /* proto */
static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info); /* proto */
static int __pyx_array_MemoryView_5array___cinit__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_shape, Py_ssize_t __pyx_v_itemsize, PyObject *__pyx_v_format, PyObject *__pyx_v_mode, int __pyx_v_allocate_buffer); /* proto */
static int __pyx_array_getbuffer_MemoryView_5array_2__getbuffer__(struct __pyx_array_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static void __pyx_array_MemoryView_5array_4__dealloc__(struct __pyx_array_obj *__pyx_v_self); /* proto */
static PyObject *get_memview_MemoryView_5array_7memview___get__(struct __pyx_array_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_array_MemoryView_5array_6__getattr__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_attr); /* proto */
static PyObject *__pyx_array_MemoryView_5array_8__getitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item); /* proto */
static int __pyx_array_MemoryView_5array_10__setitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value); /* proto */
static int __pyx_MemviewEnum_MemoryView_4Enum___init__(struct __pyx_MemviewEnum_obj *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_MemviewEnum_MemoryView_4Enum_2__repr__(struct __pyx_MemviewEnum_obj *__pyx_v_self); /* proto */
static int __pyx_memoryview_MemoryView_10memoryview___cinit__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj, int __pyx_v_flags, int __pyx_v_dtype_is_object); /* proto */
static void __pyx_memoryview_MemoryView_10memoryview_2__dealloc__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview_MemoryView_10memoryview_4__getitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index); /* proto */
static int __pyx_memoryview_MemoryView_10memoryview_6__setitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /* proto */
static int __pyx_memoryview_getbuffer_MemoryView_10memoryview_8__getbuffer__(struct __pyx_memoryview_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static PyObject *__pyx_memoryview_transpose_MemoryView_10memoryview_1T___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview__get__base_MemoryView_10memoryview_4base___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview_get_shape_MemoryView_10memoryview_5shape___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview_get_strides_MemoryView_10memoryview_7strides___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview_get_suboffsets_MemoryView_10memoryview_10suboffsets___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview_get_ndim_MemoryView_10memoryview_4ndim___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview_get_itemsize_MemoryView_10memoryview_8itemsize___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview_get_nbytes_MemoryView_10memoryview_6nbytes___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview_get_size_MemoryView_10memoryview_4size___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_memoryview_MemoryView_10memoryview_10__len__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview_MemoryView_10memoryview_12__repr__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview_MemoryView_10memoryview_14__str__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview_MemoryView_10memoryview_16is_c_contig(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview_MemoryView_10memoryview_18is_f_contig(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview_MemoryView_10memoryview_20copy(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview_MemoryView_10memoryview_22copy_fortran(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static void __pyx_memoryviewslice_MemoryView_16_memoryviewslice___dealloc__(struct __pyx_memoryviewslice_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryviewslice__get__base_MemoryView_16_memoryviewslice_4base___get__(struct __pyx_memoryviewslice_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_tp_new_7sklearn_9neighbors_7kd_tree_NeighborsHeap(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7sklearn_9neighbors_7kd_tree_NodeHeap(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7sklearn_9neighbors_7kd_tree_BinaryTree(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7sklearn_9neighbors_7kd_tree_KDTree(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_array(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_Enum(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_memoryview(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new__memoryviewslice(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static char __pyx_k_B[] = "B";
static char __pyx_k_C[] = "C";
static char __pyx_k_H[] = "H";
static char __pyx_k_I[] = "I";
static char __pyx_k_L[] = "L";
static char __pyx_k_O[] = "O";
static char __pyx_k_Q[] = "Q";
static char __pyx_k_T[] = "T{";
  static char __pyx_k_X[] = "X";
  static char __pyx_k_b[] = "b";
  static char __pyx_k_c[] = "c";
  static char __pyx_k_d[] = "d";
  static char __pyx_k_f[] = "f";
  static char __pyx_k_g[] = "g";
  static char __pyx_k_h[] = "h";
  static char __pyx_k_i[] = "i";
  static char __pyx_k_j[] = "j";
  static char __pyx_k_k[] = "k";
  static char __pyx_k_l[] = "l";
  static char __pyx_k_q[] = "q";
  static char __pyx_k_r[] = "r";
  static char __pyx_k_s[] = "(%s)";
  static char __pyx_k_Zd[] = "Zd";
  static char __pyx_k_Zf[] = "Zf";
  static char __pyx_k_Zg[] = "Zg";
  static char __pyx_k_i1[] = "i1";
  static char __pyx_k_i2[] = "i2";
  static char __pyx_k_id[] = "id";
  static char __pyx_k_np[] = "np";
  static char __pyx_k_pi[] = "pi";
  static char __pyx_k__50[] = "^";
  static char __pyx_k__51[] = "";
  static char __pyx_k__52[] = ":";
static char __pyx_k__53[] = "}";
static char __pyx_k__54[] = ",";
static char __pyx_k_all[] = "__all__";
static char __pyx_k_doc[] = "__doc__";
static char __pyx_k_exp[] = "exp";
static char __pyx_k_inf[] = "inf";
static char __pyx_k_new[] = "__new__";
static char __pyx_k_obj[] = "obj";
static char __pyx_k_row[] = "row";
static char __pyx_k_val[] = "val";
static char __pyx_k_atol[] = "atol";
static char __pyx_k_base[] = "base";
static char __pyx_k_copy[] = "copy";
static char __pyx_k_data[] = "data";
static char __pyx_k_heap[] = "heap";
static char __pyx_k_join[] = "join";
static char __pyx_k_log2[] = "log2";
static char __pyx_k_main[] = "__main__";
static char __pyx_k_mode[] = "mode";
static char __pyx_k_name[] = "__name__";
static char __pyx_k_ndim[] = "ndim";
static char __pyx_k_pack[] = "pack";
static char __pyx_k_rtol[] = "rtol";
static char __pyx_k_size[] = "size";
static char __pyx_k_sort[] = "sort";
static char __pyx_k_step[] = "step";
static char __pyx_k_stop[] = "stop";
static char __pyx_k_test[] = "__test__";
static char __pyx_k_vals[] = "vals";
static char __pyx_k_warn[] = "warn";
static char __pyx_k_DTYPE[] = "DTYPE";
static char __pyx_k_ITYPE[] = "ITYPE";
static char __pyx_k_class[] = "__class__";
static char __pyx_k_dtype[] = "dtype";
static char __pyx_k_empty[] = "empty";
static char __pyx_k_error[] = "error";
static char __pyx_k_flags[] = "flags";
static char __pyx_k_i_val[] = "i_val";
static char __pyx_k_n_pts[] = "n_pts";
static char __pyx_k_names[] = "names";
static char __pyx_k_numpy[] = "numpy";
static char __pyx_k_order[] = "order";
static char __pyx_k_range[] = "range";
static char __pyx_k_shape[] = "shape";
static char __pyx_k_start[] = "start";
static char __pyx_k_utils[] = "utils";
static char __pyx_k_zeros[] = "zeros";
static char __pyx_k_KDTree[] = "KDTree";
static char __pyx_k_arange[] = "arange";
static char __pyx_k_cosine[] = "cosine";
static char __pyx_k_format[] = "format";
static char __pyx_k_import[] = "__import__";
static char __pyx_k_kernel[] = "kernel";
static char __pyx_k_linear[] = "linear";
static char __pyx_k_metric[] = "metric";
static char __pyx_k_n_nbrs[] = "n_nbrs";
static char __pyx_k_name_2[] = "name";
static char __pyx_k_newObj[] = "newObj";
static char __pyx_k_object[] = "object";
static char __pyx_k_radius[] = "radius";
static char __pyx_k_result[] = "result";
static char __pyx_k_struct[] = "struct";
static char __pyx_k_tophat[] = "tophat";
static char __pyx_k_unpack[] = "unpack";
static char __pyx_k_xrange[] = "xrange";
static char __pyx_k_argsort[] = "argsort";
static char __pyx_k_asarray[] = "asarray";
static char __pyx_k_formats[] = "formats";
static char __pyx_k_fortran[] = "fortran";
static char __pyx_k_idx_end[] = "idx_end";
static char __pyx_k_indices[] = "indices";
static char __pyx_k_is_leaf[] = "is_leaf";
static char __pyx_k_kd_tree[] = "kd_tree";
static char __pyx_k_memview[] = "memview";
static char __pyx_k_offsets[] = "offsets";
static char __pyx_k_reshape[] = "reshape";
static char __pyx_k_DOC_DICT[] = "DOC_DICT";
static char __pyx_k_Ellipsis[] = "Ellipsis";
static char __pyx_k_NodeData[] = "NodeData";
static char __pyx_k_dualtree[] = "dualtree";
static char __pyx_k_gaussian[] = "gaussian";
static char __pyx_k_getstate[] = "__getstate__";
static char __pyx_k_itemsize[] = "itemsize";
static char __pyx_k_typedefs[] = "typedefs";
static char __pyx_k_warnings[] = "warnings";
static char __pyx_k_CLASS_DOC[] = "CLASS_DOC";
static char __pyx_k_TypeError[] = "TypeError";
static char __pyx_k_distances[] = "distances";
static char __pyx_k_enumerate[] = "enumerate";
static char __pyx_k_idx_start[] = "idx_start";
static char __pyx_k_leaf_size[] = "leaf_size";
static char __pyx_k_load_heap[] = "load_heap";
static char __pyx_k_minkowski[] = "minkowski";
static char __pyx_k_node_data[] = "node_data";
static char __pyx_k_BinaryTree[] = "BinaryTree";
static char __pyx_k_IndexError[] = "IndexError";
static char __pyx_k_ValueError[] = "ValueError";
static char __pyx_k_atleast_1d[] = "atleast_1d";
static char __pyx_k_count_only[] = "count_only";
static char __pyx_k_get_arrays[] = "get_arrays";
static char __pyx_k_get_metric[] = "get_metric";
static char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static char __pyx_k_return_log[] = "return_log";
static char __pyx_k_size_guess[] = "size_guess";
static char __pyx_k_zeros_like[] = "zeros_like";
static char __pyx_k_MemoryError[] = "MemoryError";
static char __pyx_k_binary_tree[] = "binary_tree";
static char __pyx_k_check_array[] = "check_array";
static char __pyx_k_exponential[] = "exponential";
static char __pyx_k_kernel_norm[] = "kernel_norm";
static char __pyx_k_vals_sorted[] = "vals_sorted";
static char __pyx_k_NodeHeapData[] = "NodeHeapData";
static char __pyx_k_RuntimeError[] = "RuntimeError";
static char __pyx_k_dist_metrics[] = "dist_metrics";
static char __pyx_k_epanechnikov[] = "epanechnikov";
static char __pyx_k_sort_results[] = "sort_results";
static char __pyx_k_VALID_METRICS[] = "VALID_METRICS";
static char __pyx_k_breadth_first[] = "breadth_first";
static char __pyx_k_nodeheap_sort[] = "nodeheap_sort";
static char __pyx_k_pyx_getbuffer[] = "__pyx_getbuffer";
static char __pyx_k_rdist_to_dist[] = "rdist_to_dist";
static char __pyx_k_valid_metrics[] = "valid_metrics";
static char __pyx_k_allocate_buffer[] = "allocate_buffer";
static char __pyx_k_dtype_is_object[] = "dtype_is_object";
static char __pyx_k_return_distance[] = "return_distance";
static char __pyx_k_VALID_METRIC_IDS[] = "VALID_METRIC_IDS";
static char __pyx_k_ChebyshevDistance[] = "ChebyshevDistance";
static char __pyx_k_EuclideanDistance[] = "EuclideanDistance";
static char __pyx_k_ManhattanDistance[] = "ManhattanDistance";
static char __pyx_k_MinkowskiDistance[] = "MinkowskiDistance";
static char __pyx_k_simultaneous_sort[] = "simultaneous_sort";
static char __pyx_k_strided_and_direct[] = "<strided and direct>";
static char __pyx_k_X_is_an_empty_array[] = "X is an empty array";
static char __pyx_k_get_valid_metric_ids[] = "get_valid_metric_ids";
static char __pyx_k_strided_and_indirect[] = "<strided and indirect>";
static char __pyx_k_contiguous_and_direct[] = "<contiguous and direct>";
static char __pyx_k_kernel_not_recognized[] = "kernel not recognized";
static char __pyx_k_MemoryView_of_r_object[] = "<MemoryView of %r object>";
static char __pyx_k_MemoryView_of_r_at_0x_x[] = "<MemoryView of %r at 0x%x>";
static char __pyx_k_contiguous_and_indirect[] = "<contiguous and indirect>";
static char __pyx_k_kernel_s_not_recognized[] = "kernel = '%s' not recognized";
static char __pyx_k_Cannot_index_with_type_s[] = "Cannot index with type '%s'";
static char __pyx_k_cannot_pop_on_empty_heap[] = "cannot pop on empty heap";
static char __pyx_k_getbuffer_obj_view_flags[] = "getbuffer(obj, view, flags)";
static char __pyx_k_Dimension_d_is_not_direct[] = "Dimension %d is not direct";
static char __pyx_k_Invalid_shape_in_axis_d_d[] = "Invalid shape in axis %d: %d.";
static char __pyx_k_sklearn_neighbors_kd_tree[] = "sklearn.neighbors.kd_tree";
static char __pyx_k_BinaryTree_query_line_1233[] = "BinaryTree.query (line 1233)";
static char __pyx_k_Index_out_of_bounds_axis_d[] = "Index out of bounds (axis %d)";
static char __pyx_k_Kernel_code_not_recognized[] = "Kernel code not recognized";
static char __pyx_k_Step_may_not_be_zero_axis_d[] = "Step may not be zero (axis %d)";
static char __pyx_k_itemsize_0_for_cython_array[] = "itemsize <= 0 for cython.array";
static char __pyx_k_ndarray_is_not_C_contiguous[] = "ndarray is not C contiguous";
static char __pyx_k_unable_to_allocate_array_data[] = "unable to allocate array data.";
static char __pyx_k_home_varoquau_dev_scikit_learn[] = "/home/varoquau/dev/scikit-learn/sklearn/neighbors/binary_tree.pxi";
static char __pyx_k_kernel_density_self_X_h_kernel[] = "\n        kernel_density(self, X, h, kernel='gaussian', atol=0, rtol=1E-8,\n                       breadth_first=True, return_log=False)\n\n        Compute the kernel density estimate at points X with the given kernel,\n        using the distance metric specified at tree creation.\n\n        Parameters\n        ----------\n        X : array_like\n            An array of points to query.  Last dimension should match dimension\n            of training data.\n        h : float\n            the bandwidth of the kernel\n        kernel : string\n            specify the kernel to use.  Options are\n            - 'gaussian'\n            - 'tophat'\n            - 'epanechnikov'\n            - 'exponential'\n            - 'linear'\n            - 'cosine'\n            Default is kernel = 'gaussian'\n        atol, rtol : float (default = 0)\n            Specify the desired relative and absolute tolerance of the result.\n            If the true result is K_true, then the returned result K_ret\n            satisfies ``abs(K_true - K_ret) < atol + rtol * K_ret``\n            The default is zero (i.e. machine precision) for both.\n        breadth_first : boolean (default = False)\n            if True, use a breadth-first search.  If False (default) use a\n            depth-first search.  Breadth-first is generally faster for\n            compact kernels and/or high tolerances.\n        return_log : boolean (default = False)\n            return the logarithm of the result.  This can be more accurate\n            than returning the result itself for narrow kernels.\n\n        Returns\n        -------\n        density : ndarray\n            The array of (log)-density evaluations, shape = X.shape[:-1]\n\n        Examples\n        --------\n        Compute a gaussian kernel density estimate:\n\n        >>> import numpy as np\n        >>> np.random.seed(1)\n        >>> X = np.random.random((100, 3))\n        >>> tree = BinaryTree(X)           # doctest: +SKIP\n        >>> tree.kernel_den""sity(X[:3], h=0.1, kernel='gaussian')\n        array([ 6.94114649,  7.83281226,  7.2071716 ])\n        ";
static char __pyx_k_strided_and_direct_or_indirect[] = "<strided and direct or indirect>";
static char __pyx_k_BinaryTree_for_fast_generalized[] = "{BinaryTree} for fast generalized N-point problems\n\n{BinaryTree}(X, leaf_size=40, metric='minkowski', \\**kwargs)\n\nParameters\n----------\nX : array-like, shape = [n_samples, n_features]\n    n_samples is the number of points in the data set, and\n    n_features is the dimension of the parameter space.\n    Note: if X is a C-contiguous array of doubles then data will\n    not be copied. Otherwise, an internal copy will be made.\n\nleaf_size : positive integer (default = 20)\n    Number of points at which to switch to brute-force. Changing\n    leaf_size will not affect the results of a query, but can\n    significantly impact the speed of a query and the memory required\n    to store the constructed tree.  The amount of memory needed to\n    store the tree scales as approximately n_samples / leaf_size.\n    For a specified ``leaf_size``, a leaf node is guaranteed to\n    satisfy ``leaf_size <= n_points <= 2 * leaf_size``, except in\n    the case that ``n_samples < leaf_size``.\n\nmetric : string or DistanceMetric object\n    the distance metric to use for the tree.  Default='minkowski'\n    with p=2 (that is, a euclidean metric). See the documentation\n    of the DistanceMetric class for a list of available metrics.\n    {binary_tree}.valid_metrics gives a list of the metrics which\n    are valid for {BinaryTree}.\n\nAdditional keywords are passed to the distance metric class.\n\nAttributes\n----------\ndata : np.ndarray\n    The training data\n\nExamples\n--------\nQuery for k-nearest neighbors\n\n    >>> import numpy as np\n\n    >>> np.random.seed(0)\n    >>> X = np.random.random((10, 3))  # 10 points in 3 dimensions\n    >>> tree = {BinaryTree}(X, leaf_size=2)              # doctest: +SKIP\n    >>> dist, ind = tree.query(X[0], k=3)                # doctest: +SKIP\n    >>> print ind  # indices of 3 closest neighbors\n    [0 3 1]\n    >>> print dist  # distances to 3 closest neighbors\n    [ 0.          0.19662693  0.29473397]\n\nPickle and Unpickle a tree.  N""ote that the state of the tree is saved in the\npickle operation: the tree needs not be rebuilt upon unpickling.\n\n    >>> import numpy as np\n    >>> import pickle\n    >>> np.random.seed(0)\n    >>> X = np.random.random((10, 3))  # 10 points in 3 dimensions\n    >>> tree = {BinaryTree}(X, leaf_size=2)        # doctest: +SKIP\n    >>> s = pickle.dumps(tree)                     # doctest: +SKIP\n    >>> tree_copy = pickle.loads(s)                # doctest: +SKIP\n    >>> dist, ind = tree_copy.query(X[0], k=3)     # doctest: +SKIP\n    >>> print ind  # indices of 3 closest neighbors\n    [0 3 1]\n    >>> print dist  # distances to 3 closest neighbors\n    [ 0.          0.19662693  0.29473397]\n\nQuery for neighbors within a given radius\n\n    >>> import numpy as np\n    >>> np.random.seed(0)\n    >>> X = np.random.random((10, 3))  # 10 points in 3 dimensions\n    >>> tree = {BinaryTree}(X, leaf_size=2)     # doctest: +SKIP\n    >>> print tree.query_radius(X[0], r=0.3, count_only=True)\n    3\n    >>> ind = tree.query_radius(X[0], r=0.3)  # doctest: +SKIP\n    >>> print ind  # indices of neighbors within distance 0.3\n    [3 0 1]\n\n\nCompute a gaussian kernel density estimate:\n\n    >>> import numpy as np\n    >>> np.random.seed(1)\n    >>> X = np.random.random((100, 3))\n    >>> tree = {BinaryTree}(X)                # doctest: +SKIP\n    >>> tree.kernel_density(X[:3], h=0.1, kernel='gaussian')\n    array([ 6.94114649,  7.83281226,  7.2071716 ])\n\nCompute a two-point auto-correlation function\n\n    >>> import numpy as np\n    >>> np.random.seed(0)\n    >>> X = np.random.random((30, 3))\n    >>> r = np.linspace(0, 1, 5)\n    >>> tree = {BinaryTree}(X)                # doctest: +SKIP\n    >>> tree.two_point_correlation(X, r)\n    array([ 30,  62, 278, 580, 820])\n\n";
static char __pyx_k_Fatal_count_too_big_this_should[] = "Fatal: count too big: this should never happen";
static char __pyx_k_k_must_be_less_than_or_equal_to[] = "k must be less than or equal to the number of training points";
static char __pyx_k_query_X_k_1_return_distance_Tru[] = "\n        query(X, k=1, return_distance=True,\n              dualtree=False, breadth_first=False)\n\n        query the tree for the k nearest neighbors\n\n        Parameters\n        ----------\n        X : array-like, last dimension self.dim\n            An array of points to query\n        k : integer  (default = 1)\n            The number of nearest neighbors to return\n        return_distance : boolean (default = True)\n            if True, return a tuple (d, i) of distances and indices\n            if False, return array i\n        dualtree : boolean (default = False)\n            if True, use the dual tree formalism for the query: a tree is\n            built for the query points, and the pair of trees is used to\n            efficiently search this space.  This can lead to better\n            performance as the number of points grows large.\n        breadth_first : boolean (default = False)\n            if True, then query the nodes in a breadth-first manner.\n            Otherwise, query the nodes in a depth-first manner.\n        sort_results : boolean (default = True)\n            if True, then distances and indices of each point are sorted\n            on return, so that the first column contains the closest points.\n            Otherwise, neighbors are returned in an arbitrary order.\n\n        Returns\n        -------\n        i    : if return_distance == False\n        (d,i) : if return_distance == True\n\n        d : array of doubles - shape: x.shape[:-1] + (k,)\n            each entry gives the list of distances to the\n            neighbors of the corresponding point\n\n        i : array of integers - shape: x.shape[:-1] + (k,)\n            each entry gives the list of indices of\n            neighbors of the corresponding point\n\n        Examples\n        --------\n        Query for k-nearest neighbors\n\n            >>> import numpy as np\n            >>> np.random.seed(0)\n            >>> X = np.random.random((10, 3))  # 10 points in 3 dimension""s\n            >>> tree = BinaryTree(X, leaf_size=2)    # doctest: +SKIP\n            >>> dist, ind = tree.query(X[0], k=3)    # doctest: +SKIP\n            >>> print ind  # indices of 3 closest neighbors\n            [0 3 1]\n            >>> print dist  # distances to 3 closest neighbors\n            [ 0.          0.19662693  0.29473397]\n        ";
static char __pyx_k_query_data_dimension_must_match[] = "query data dimension must match training data dimension";
static char __pyx_k_query_radius_self_X_r_count_onl[] = "\n        query_radius(self, X, r, count_only = False):\n\n        query the tree for neighbors within a radius r\n\n        Parameters\n        ----------\n        X : array-like, last dimension self.dim\n            An array of points to query\n        r : distance within which neighbors are returned\n            r can be a single value, or an array of values of shape\n            x.shape[:-1] if different radii are desired for each point.\n        return_distance : boolean (default = False)\n            if True,  return distances to neighbors of each point\n            if False, return only neighbors\n            Note that unlike the query() method, setting return_distance=True\n            here adds to the computation time.  Not all distances need to be\n            calculated explicitly for return_distance=False.  Results are\n            not sorted by default: see ``sort_results`` keyword.\n        count_only : boolean (default = False)\n            if True,  return only the count of points within distance r\n            if False, return the indices of all points within distance r\n            If return_distance==True, setting count_only=True will\n            result in an error.\n        sort_results : boolean (default = False)\n            if True, the distances and indices will be sorted before being\n            returned.  If False, the results will not be sorted.  If\n            return_distance == False, setting sort_results = True will\n            result in an error.\n\n        Returns\n        -------\n        count       : if count_only == True\n        ind         : if count_only == False and return_distance == False\n        (ind, dist) : if count_only == False and return_distance == True\n\n        count : array of integers, shape = X.shape[:-1]\n            each entry gives the number of neighbors within\n            a distance r of the corresponding point.\n\n        ind : array of objects, shape = X.shape[:-1]\n            each element is a num""py integer array listing the indices of\n            neighbors of the corresponding point.  Note that unlike\n            the results of a k-neighbors query, the returned neighbors\n            are not sorted by distance by default.\n\n        dist : array of objects, shape = X.shape[:-1]\n            each element is a numpy double array\n            listing the distances corresponding to indices in i.\n\n        Examples\n        --------\n        Query for neighbors in a given radius\n\n        >>> import numpy as np\n        >>> np.random.seed(0)\n        >>> X = np.random.random((10, 3))  # 10 points in 3 dimensions\n        >>> tree = BinaryTree(X, leaf_size=2)     # doctest: +SKIP\n        >>> print tree.query_radius(X[0], r=0.3, count_only=True)\n        3\n        >>> ind = tree.query_radius(X[0], r=0.3)  # doctest: +SKIP\n        >>> print ind  # indices of neighbors within distance 0.3\n        [3 0 1]\n        ";
static char __pyx_k_r_must_be_a_1_dimensional_array[] = "r must be a 1-dimensional array";
static char __pyx_k_return_distance_must_be_True_if[] = "return_distance must be True if sort_results is True";
static char __pyx_k_unknown_dtype_code_in_numpy_pxd[] = "unknown dtype code in numpy.pxd (%d)";
static char __pyx_k_All_dimensions_preceding_dimensi[] = "All dimensions preceding dimension %d must be indexed and not sliced";
static char __pyx_k_BinaryTree_kernel_density_line_1[] = "BinaryTree.kernel_density (line 1505)";
static char __pyx_k_BinaryTree_query_radius_line_135[] = "BinaryTree.query_radius (line 1359)";
static char __pyx_k_BinaryTree_two_point_correlation[] = "BinaryTree.two_point_correlation (line 1656)";
static char __pyx_k_Buffer_view_does_not_expose_stri[] = "Buffer view does not expose strides";
static char __pyx_k_Can_only_create_a_buffer_that_is[] = "Can only create a buffer that is contiguous in memory.";
static char __pyx_k_Cannot_transpose_memoryview_with[] = "Cannot transpose memoryview with indirect dimensions";
static char __pyx_k_Compute_the_two_point_correlatio[] = "Compute the two-point correlation function\n\n        Parameters\n        ----------\n        X : array_like\n            An array of points to query.  Last dimension should match dimension\n            of training data.\n        r : array_like\n            A one-dimensional array of distances\n        dualtree : boolean (default = False)\n            If true, use a dualtree algorithm.  Otherwise, use a single-tree\n            algorithm.  Dual tree algorithms can have better scaling for\n            large N.\n\n        Returns\n        -------\n        counts : ndarray\n            counts[i] contains the number of pairs of points with distance\n            less than or equal to r[i]\n\n        Examples\n        --------\n        Compute the two-point autocorrelation function of X:\n\n        >>> import numpy as np\n        >>> np.random.seed(0)\n        >>> X = np.random.random((30, 3))\n        >>> r = np.linspace(0, 1, 5)\n        >>> tree = BinaryTree(X)     # doctest: +SKIP\n        >>> tree.two_point_correlation(X, r)\n        array([ 30,  62, 278, 580, 820])\n        ";
static char __pyx_k_Empty_shape_tuple_for_cython_arr[] = "Empty shape tuple for cython.array";
static char __pyx_k_Fatal_count_out_of_range_This_sh[] = "Fatal: count out of range. This should never happen.";
static char __pyx_k_Format_string_allocated_too_shor[] = "Format string allocated too short, see comment in numpy.pxd";
static char __pyx_k_Indirect_dimensions_not_supporte[] = "Indirect dimensions not supported";
static char __pyx_k_Internal_memory_layout_is_flawed[] = "Internal: memory layout is flawed: not enough nodes allocated";
static char __pyx_k_Invalid_mode_expected_c_or_fortr[] = "Invalid mode, expected 'c' or 'fortran', got %s";
static char __pyx_k_Non_native_byte_order_not_suppor[] = "Non-native byte order not supported";
static char __pyx_k_Out_of_bounds_on_buffer_access_a[] = "Out of bounds on buffer access (axis %d)";
static char __pyx_k_Unable_to_convert_item_to_object[] = "Unable to convert item to object";
static char __pyx_k_count_only_and_return_distance_c[] = "count_only and return_distance cannot both be true";
static char __pyx_k_got_differing_extents_in_dimensi[] = "got differing extents in dimension %d (got %d and %d)";
static char __pyx_k_leaf_size_must_be_greater_than_o[] = "leaf_size must be greater than or equal to 1";
static char __pyx_k_metric_metric_is_not_valid_for_B[] = "metric {metric} is not valid for {BinaryTree}";
static char __pyx_k_ndarray_is_not_Fortran_contiguou[] = "ndarray is not Fortran contiguous";
static char __pyx_k_r_must_be_broadcastable_to_X_sha[] = "r must be broadcastable to X.shape";
static char __pyx_k_unable_to_allocate_shape_and_str[] = "unable to allocate shape and strides.";
static char __pyx_k_Format_string_allocated_too_shor_2[] = "Format string allocated too short.";
static char __pyx_k_Internal_memory_layout_is_flawed_2[] = "Internal: memory layout is flawed: too many nodes allocated";
static PyObject *__pyx_n_s_BinaryTree;
static PyObject *__pyx_kp_s_BinaryTree_for_fast_generalized;
static PyObject *__pyx_kp_u_BinaryTree_kernel_density_line_1;
static PyObject *__pyx_kp_u_BinaryTree_query_line_1233;
static PyObject *__pyx_kp_u_BinaryTree_query_radius_line_135;
static PyObject *__pyx_kp_u_BinaryTree_two_point_correlation;
static PyObject *__pyx_kp_s_Buffer_view_does_not_expose_stri;
static PyObject *__pyx_n_s_C;
static PyObject *__pyx_n_s_CLASS_DOC;
static PyObject *__pyx_kp_s_Can_only_create_a_buffer_that_is;
static PyObject *__pyx_kp_s_Cannot_index_with_type_s;
static PyObject *__pyx_n_s_ChebyshevDistance;
static PyObject *__pyx_kp_u_Compute_the_two_point_correlatio;
static PyObject *__pyx_n_s_DOC_DICT;
static PyObject *__pyx_n_s_DTYPE;
static PyObject *__pyx_n_s_Ellipsis;
static PyObject *__pyx_kp_s_Empty_shape_tuple_for_cython_arr;
static PyObject *__pyx_n_s_EuclideanDistance;
static PyObject *__pyx_kp_s_Fatal_count_out_of_range_This_sh;
static PyObject *__pyx_kp_s_Fatal_count_too_big_this_should;
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor;
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor_2;
static PyObject *__pyx_n_s_ITYPE;
static PyObject *__pyx_n_s_IndexError;
static PyObject *__pyx_kp_s_Indirect_dimensions_not_supporte;
static PyObject *__pyx_kp_s_Internal_memory_layout_is_flawed;
static PyObject *__pyx_kp_s_Internal_memory_layout_is_flawed_2;
static PyObject *__pyx_kp_s_Invalid_mode_expected_c_or_fortr;
static PyObject *__pyx_kp_s_Invalid_shape_in_axis_d_d;
static PyObject *__pyx_n_s_KDTree;
static PyObject *__pyx_kp_s_Kernel_code_not_recognized;
static PyObject *__pyx_n_s_ManhattanDistance;
static PyObject *__pyx_n_s_MemoryError;
static PyObject *__pyx_kp_s_MemoryView_of_r_at_0x_x;
static PyObject *__pyx_kp_s_MemoryView_of_r_object;
static PyObject *__pyx_n_s_MinkowskiDistance;
static PyObject *__pyx_n_s_NodeData;
static PyObject *__pyx_n_s_NodeHeapData;
static PyObject *__pyx_kp_u_Non_native_byte_order_not_suppor;
static PyObject *__pyx_n_b_O;
static PyObject *__pyx_kp_s_Out_of_bounds_on_buffer_access_a;
static PyObject *__pyx_n_s_RuntimeError;
static PyObject *__pyx_kp_b_T;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_kp_s_Unable_to_convert_item_to_object;
static PyObject *__pyx_n_s_VALID_METRICS;
static PyObject *__pyx_n_s_VALID_METRIC_IDS;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_X;
static PyObject *__pyx_kp_s_X_is_an_empty_array;
static PyObject *__pyx_kp_b__50;
static PyObject *__pyx_kp_b__51;
static PyObject *__pyx_kp_b__52;
static PyObject *__pyx_kp_b__53;
static PyObject *__pyx_kp_u__54;
static PyObject *__pyx_n_s_all;
static PyObject *__pyx_n_s_allocate_buffer;
static PyObject *__pyx_n_s_arange;
static PyObject *__pyx_n_s_argsort;
static PyObject *__pyx_n_s_asarray;
static PyObject *__pyx_n_s_atleast_1d;
static PyObject *__pyx_n_s_atol;
static PyObject *__pyx_n_s_base;
static PyObject *__pyx_n_s_binary_tree;
static PyObject *__pyx_n_s_breadth_first;
static PyObject *__pyx_n_s_c;
static PyObject *__pyx_n_u_c;
static PyObject *__pyx_kp_s_cannot_pop_on_empty_heap;
static PyObject *__pyx_n_s_check_array;
static PyObject *__pyx_n_s_class;
static PyObject *__pyx_kp_s_contiguous_and_direct;
static PyObject *__pyx_kp_s_contiguous_and_indirect;
static PyObject *__pyx_n_s_copy;
static PyObject *__pyx_n_s_cosine;
static PyObject *__pyx_n_s_count_only;
static PyObject *__pyx_kp_s_count_only_and_return_distance_c;
static PyObject *__pyx_n_s_d;
static PyObject *__pyx_n_s_data;
static PyObject *__pyx_n_s_dist_metrics;
static PyObject *__pyx_n_s_distances;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_dtype;
static PyObject *__pyx_n_s_dtype_is_object;
static PyObject *__pyx_n_s_dualtree;
static PyObject *__pyx_n_s_empty;
static PyObject *__pyx_n_s_enumerate;
static PyObject *__pyx_n_s_epanechnikov;
static PyObject *__pyx_n_s_error;
static PyObject *__pyx_n_s_exp;
static PyObject *__pyx_n_s_exponential;
static PyObject *__pyx_n_s_flags;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_formats;
static PyObject *__pyx_n_s_fortran;
static PyObject *__pyx_n_u_fortran;
static PyObject *__pyx_n_s_gaussian;
static PyObject *__pyx_n_s_get_arrays;
static PyObject *__pyx_n_s_get_metric;
static PyObject *__pyx_n_s_get_valid_metric_ids;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_kp_s_got_differing_extents_in_dimensi;
static PyObject *__pyx_n_s_h;
static PyObject *__pyx_n_s_heap;
static PyObject *__pyx_kp_s_home_varoquau_dev_scikit_learn;
static PyObject *__pyx_n_s_i;
static PyObject *__pyx_n_s_i1;
static PyObject *__pyx_n_s_i2;
static PyObject *__pyx_n_s_i_val;
static PyObject *__pyx_n_s_id;
static PyObject *__pyx_n_s_idx_end;
static PyObject *__pyx_n_s_idx_start;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_indices;
static PyObject *__pyx_n_s_inf;
static PyObject *__pyx_n_s_is_leaf;
static PyObject *__pyx_n_s_itemsize;
static PyObject *__pyx_kp_s_itemsize_0_for_cython_array;
static PyObject *__pyx_n_s_j;
static PyObject *__pyx_n_s_join;
static PyObject *__pyx_n_s_k;
static PyObject *__pyx_kp_s_k_must_be_less_than_or_equal_to;
static PyObject *__pyx_n_s_kd_tree;
static PyObject *__pyx_n_s_kernel;
static PyObject *__pyx_kp_u_kernel_density_self_X_h_kernel;
static PyObject *__pyx_n_s_kernel_norm;
static PyObject *__pyx_kp_s_kernel_not_recognized;
static PyObject *__pyx_kp_s_kernel_s_not_recognized;
static PyObject *__pyx_n_s_leaf_size;
static PyObject *__pyx_kp_s_leaf_size_must_be_greater_than_o;
static PyObject *__pyx_n_s_linear;
static PyObject *__pyx_n_s_load_heap;
static PyObject *__pyx_n_s_log2;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_memview;
static PyObject *__pyx_n_s_metric;
static PyObject *__pyx_kp_s_metric_metric_is_not_valid_for_B;
static PyObject *__pyx_n_s_minkowski;
static PyObject *__pyx_n_s_mode;
static PyObject *__pyx_n_s_n_nbrs;
static PyObject *__pyx_n_s_n_pts;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_name_2;
static PyObject *__pyx_n_s_names;
static PyObject *__pyx_kp_u_ndarray_is_not_C_contiguous;
static PyObject *__pyx_kp_u_ndarray_is_not_Fortran_contiguou;
static PyObject *__pyx_n_s_ndim;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_newObj;
static PyObject *__pyx_n_s_node_data;
static PyObject *__pyx_n_s_nodeheap_sort;
static PyObject *__pyx_n_s_np;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_n_s_obj;
static PyObject *__pyx_n_s_object;
static PyObject *__pyx_n_s_offsets;
static PyObject *__pyx_n_s_order;
static PyObject *__pyx_n_s_pack;
static PyObject *__pyx_n_s_pi;
static PyObject *__pyx_n_s_pyx_getbuffer;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_kp_u_query_X_k_1_return_distance_Tru;
static PyObject *__pyx_kp_s_query_data_dimension_must_match;
static PyObject *__pyx_kp_u_query_radius_self_X_r_count_onl;
static PyObject *__pyx_n_s_r;
static PyObject *__pyx_kp_s_r_must_be_a_1_dimensional_array;
static PyObject *__pyx_kp_s_r_must_be_broadcastable_to_X_sha;
static PyObject *__pyx_n_s_radius;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_rdist_to_dist;
static PyObject *__pyx_n_s_reshape;
static PyObject *__pyx_n_s_result;
static PyObject *__pyx_n_s_return_distance;
static PyObject *__pyx_kp_s_return_distance_must_be_True_if;
static PyObject *__pyx_n_s_return_log;
static PyObject *__pyx_n_s_row;
static PyObject *__pyx_n_s_rtol;
static PyObject *__pyx_kp_u_s;
static PyObject *__pyx_n_s_shape;
static PyObject *__pyx_n_s_simultaneous_sort;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_n_s_size_guess;
static PyObject *__pyx_n_s_sklearn_neighbors_kd_tree;
static PyObject *__pyx_n_s_sort;
static PyObject *__pyx_n_s_sort_results;
static PyObject *__pyx_n_s_start;
static PyObject *__pyx_n_s_step;
static PyObject *__pyx_n_s_stop;
static PyObject *__pyx_kp_s_strided_and_direct;
static PyObject *__pyx_kp_s_strided_and_direct_or_indirect;
static PyObject *__pyx_kp_s_strided_and_indirect;
static PyObject *__pyx_n_s_struct;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_tophat;
static PyObject *__pyx_n_s_typedefs;
static PyObject *__pyx_kp_s_unable_to_allocate_array_data;
static PyObject *__pyx_kp_s_unable_to_allocate_shape_and_str;
static PyObject *__pyx_kp_u_unknown_dtype_code_in_numpy_pxd;
static PyObject *__pyx_n_s_unpack;
static PyObject *__pyx_n_s_utils;
static PyObject *__pyx_n_s_val;
static PyObject *__pyx_n_s_valid_metrics;
static PyObject *__pyx_n_s_vals;
static PyObject *__pyx_n_s_vals_sorted;
static PyObject *__pyx_n_s_warn;
static PyObject *__pyx_n_s_warnings;
static PyObject *__pyx_n_s_xrange;
static PyObject *__pyx_n_s_zeros;
static PyObject *__pyx_n_s_zeros_like;
static PyObject *__pyx_float_1Eneg_8;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_40;
static PyObject *__pyx_int_100;
static PyObject *__pyx_int_neg_1;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_slice__46;
static PyObject *__pyx_slice__47;
static PyObject *__pyx_slice__48;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__30;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__32;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_tuple__34;
static PyObject *__pyx_tuple__35;
static PyObject *__pyx_tuple__36;
static PyObject *__pyx_tuple__37;
static PyObject *__pyx_tuple__38;
static PyObject *__pyx_tuple__39;
static PyObject *__pyx_tuple__40;
static PyObject *__pyx_tuple__41;
static PyObject *__pyx_tuple__42;
static PyObject *__pyx_tuple__43;
static PyObject *__pyx_tuple__44;
static PyObject *__pyx_tuple__45;
static PyObject *__pyx_tuple__49;
static PyObject *__pyx_tuple__55;
static PyObject *__pyx_tuple__57;
static PyObject *__pyx_tuple__59;
static PyObject *__pyx_tuple__61;
static PyObject *__pyx_tuple__63;
static PyObject *__pyx_tuple__65;
static PyObject *__pyx_tuple__66;
static PyObject *__pyx_tuple__67;
static PyObject *__pyx_tuple__68;
static PyObject *__pyx_tuple__69;
static PyObject *__pyx_codeobj__56;
static PyObject *__pyx_codeobj__58;
static PyObject *__pyx_codeobj__60;
static PyObject *__pyx_codeobj__62;
static PyObject *__pyx_codeobj__64;

/* "binary_tree.pxi":212
 * ######################################################################
 * # Numpy 1.3-1.4 compatibility utilities
 * cdef DTYPE_t[::1] get_memview_DTYPE_1D(             # <<<<<<<<<<<<<<
 *                                np.ndarray[DTYPE_t, ndim=1, mode='c'] X):
 *     return <DTYPE_t[:X.shape[0]:1]> (<DTYPE_t*> X.data)
 */

static __Pyx_memviewslice __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_1D(PyArrayObject *__pyx_v_X) {
  __Pyx_LocalBuf_ND __pyx_pybuffernd_X;
  __Pyx_Buffer __pyx_pybuffer_X;
  __Pyx_memviewslice __pyx_r = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_RefNannyDeclarations
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_t_1;
  struct __pyx_array_obj *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_memview_DTYPE_1D", 0);
  __pyx_pybuffer_X.pybuffer.buf = NULL;
  __pyx_pybuffer_X.refcount = 0;
  __pyx_pybuffernd_X.data = NULL;
  __pyx_pybuffernd_X.rcbuffer = &__pyx_pybuffer_X;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_X.rcbuffer->pybuffer, (PyObject*)__pyx_v_X, &__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 212; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_X.diminfo[0].strides = __pyx_pybuffernd_X.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_X.diminfo[0].shape = __pyx_pybuffernd_X.rcbuffer->pybuffer.shape[0];

  /* "binary_tree.pxi":214
 * cdef DTYPE_t[::1] get_memview_DTYPE_1D(
 *                                np.ndarray[DTYPE_t, ndim=1, mode='c'] X):
 *     return <DTYPE_t[:X.shape[0]:1]> (<DTYPE_t*> X.data)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = ((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *)__pyx_v_X->data);
  if (!__pyx_t_1) {
    PyErr_SetString(PyExc_ValueError,"Cannot create cython.array from NULL pointer");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 214; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_4 = __pyx_format_from_typeinfo(&__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t);
  __pyx_t_3 = Py_BuildValue((char*) "("  __PYX_BUILD_PY_SSIZE_T  ")", ((Py_ssize_t)(__pyx_v_X->dimensions[0])));
  if (unlikely(!__pyx_t_4 || !__pyx_t_3 || !PyBytes_AsString(__pyx_t_4))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 214; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __pyx_array_new(__pyx_t_3, sizeof(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t), PyBytes_AS_STRING(__pyx_t_4), (char *) "fortran", (char *) __pyx_t_1);
  if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 214; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t(((PyObject *)__pyx_t_2));
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 214; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(((PyObject *)__pyx_t_2)); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;
  goto __pyx_L0;

  /* "binary_tree.pxi":212
 * ######################################################################
 * # Numpy 1.3-1.4 compatibility utilities
 * cdef DTYPE_t[::1] get_memview_DTYPE_1D(             # <<<<<<<<<<<<<<
 *                                np.ndarray[DTYPE_t, ndim=1, mode='c'] X):
 *     return <DTYPE_t[:X.shape[0]:1]> (<DTYPE_t*> X.data)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(((PyObject *)__pyx_t_2));
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_X.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __pyx_r.data = NULL;
  __pyx_r.memview = NULL;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.get_memview_DTYPE_1D", __pyx_clineno, __pyx_lineno, __pyx_filename);

  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_X.rcbuffer->pybuffer);
  if (unlikely(!__pyx_r.memview)) {
    PyErr_SetString(PyExc_TypeError,"Memoryview return value is not initialized");
  }
  __pyx_L2:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":217
 * 
 * 
 * cdef DTYPE_t[:, ::1] get_memview_DTYPE_2D(             # <<<<<<<<<<<<<<
 *                                np.ndarray[DTYPE_t, ndim=2, mode='c'] X):
 *     return <DTYPE_t[:X.shape[0], :X.shape[1]:1]> (<DTYPE_t*> X.data)
 */

static __Pyx_memviewslice __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_2D(PyArrayObject *__pyx_v_X) {
  __Pyx_LocalBuf_ND __pyx_pybuffernd_X;
  __Pyx_Buffer __pyx_pybuffer_X;
  __Pyx_memviewslice __pyx_r = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_RefNannyDeclarations
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_t_1;
  struct __pyx_array_obj *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_memview_DTYPE_2D", 0);
  __pyx_pybuffer_X.pybuffer.buf = NULL;
  __pyx_pybuffer_X.refcount = 0;
  __pyx_pybuffernd_X.data = NULL;
  __pyx_pybuffernd_X.rcbuffer = &__pyx_pybuffer_X;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_X.rcbuffer->pybuffer, (PyObject*)__pyx_v_X, &__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 217; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_X.diminfo[0].strides = __pyx_pybuffernd_X.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_X.diminfo[0].shape = __pyx_pybuffernd_X.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_X.diminfo[1].strides = __pyx_pybuffernd_X.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_X.diminfo[1].shape = __pyx_pybuffernd_X.rcbuffer->pybuffer.shape[1];

  /* "binary_tree.pxi":219
 * cdef DTYPE_t[:, ::1] get_memview_DTYPE_2D(
 *                                np.ndarray[DTYPE_t, ndim=2, mode='c'] X):
 *     return <DTYPE_t[:X.shape[0], :X.shape[1]:1]> (<DTYPE_t*> X.data)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = ((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *)__pyx_v_X->data);
  if (!__pyx_t_1) {
    PyErr_SetString(PyExc_ValueError,"Cannot create cython.array from NULL pointer");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_4 = __pyx_format_from_typeinfo(&__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t);
  __pyx_t_3 = Py_BuildValue((char*) "("  __PYX_BUILD_PY_SSIZE_T  __PYX_BUILD_PY_SSIZE_T  ")", ((Py_ssize_t)(__pyx_v_X->dimensions[0])), ((Py_ssize_t)(__pyx_v_X->dimensions[1])));
  if (unlikely(!__pyx_t_4 || !__pyx_t_3 || !PyBytes_AsString(__pyx_t_4))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __pyx_array_new(__pyx_t_3, sizeof(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t), PyBytes_AS_STRING(__pyx_t_4), (char *) "c", (char *) __pyx_t_1);
  if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_d_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t(((PyObject *)__pyx_t_2));
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(((PyObject *)__pyx_t_2)); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;
  goto __pyx_L0;

  /* "binary_tree.pxi":217
 * 
 * 
 * cdef DTYPE_t[:, ::1] get_memview_DTYPE_2D(             # <<<<<<<<<<<<<<
 *                                np.ndarray[DTYPE_t, ndim=2, mode='c'] X):
 *     return <DTYPE_t[:X.shape[0], :X.shape[1]:1]> (<DTYPE_t*> X.data)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(((PyObject *)__pyx_t_2));
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_X.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __pyx_r.data = NULL;
  __pyx_r.memview = NULL;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.get_memview_DTYPE_2D", __pyx_clineno, __pyx_lineno, __pyx_filename);

  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_X.rcbuffer->pybuffer);
  if (unlikely(!__pyx_r.memview)) {
    PyErr_SetString(PyExc_TypeError,"Memoryview return value is not initialized");
  }
  __pyx_L2:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":222
 * 
 * 
 * cdef DTYPE_t[:, :, ::1] get_memview_DTYPE_3D(             # <<<<<<<<<<<<<<
 *                                np.ndarray[DTYPE_t, ndim=3, mode='c'] X):
 *     return <DTYPE_t[:X.shape[0], :X.shape[1], :X.shape[2]:1]>\
 */

static __Pyx_memviewslice __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_3D(PyArrayObject *__pyx_v_X) {
  __Pyx_LocalBuf_ND __pyx_pybuffernd_X;
  __Pyx_Buffer __pyx_pybuffer_X;
  __Pyx_memviewslice __pyx_r = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_RefNannyDeclarations
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_t_1;
  struct __pyx_array_obj *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_memview_DTYPE_3D", 0);
  __pyx_pybuffer_X.pybuffer.buf = NULL;
  __pyx_pybuffer_X.refcount = 0;
  __pyx_pybuffernd_X.data = NULL;
  __pyx_pybuffernd_X.rcbuffer = &__pyx_pybuffer_X;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_X.rcbuffer->pybuffer, (PyObject*)__pyx_v_X, &__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 3, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_X.diminfo[0].strides = __pyx_pybuffernd_X.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_X.diminfo[0].shape = __pyx_pybuffernd_X.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_X.diminfo[1].strides = __pyx_pybuffernd_X.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_X.diminfo[1].shape = __pyx_pybuffernd_X.rcbuffer->pybuffer.shape[1]; __pyx_pybuffernd_X.diminfo[2].strides = __pyx_pybuffernd_X.rcbuffer->pybuffer.strides[2]; __pyx_pybuffernd_X.diminfo[2].shape = __pyx_pybuffernd_X.rcbuffer->pybuffer.shape[2];

  /* "binary_tree.pxi":225
 *                                np.ndarray[DTYPE_t, ndim=3, mode='c'] X):
 *     return <DTYPE_t[:X.shape[0], :X.shape[1], :X.shape[2]:1]>\
 *                                                        (<DTYPE_t*> X.data)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = ((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *)__pyx_v_X->data);

  /* "binary_tree.pxi":224
 * cdef DTYPE_t[:, :, ::1] get_memview_DTYPE_3D(
 *                                np.ndarray[DTYPE_t, ndim=3, mode='c'] X):
 *     return <DTYPE_t[:X.shape[0], :X.shape[1], :X.shape[2]:1]>\             # <<<<<<<<<<<<<<
 *                                                        (<DTYPE_t*> X.data)
 * 
 */
  if (!__pyx_t_1) {
    PyErr_SetString(PyExc_ValueError,"Cannot create cython.array from NULL pointer");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 225; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_4 = __pyx_format_from_typeinfo(&__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t);
  __pyx_t_3 = Py_BuildValue((char*) "("  __PYX_BUILD_PY_SSIZE_T  __PYX_BUILD_PY_SSIZE_T  __PYX_BUILD_PY_SSIZE_T  ")", ((Py_ssize_t)(__pyx_v_X->dimensions[0])), ((Py_ssize_t)(__pyx_v_X->dimensions[1])), ((Py_ssize_t)(__pyx_v_X->dimensions[2])));
  if (unlikely(!__pyx_t_4 || !__pyx_t_3 || !PyBytes_AsString(__pyx_t_4))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 224; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __pyx_array_new(__pyx_t_3, sizeof(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t), PyBytes_AS_STRING(__pyx_t_4), (char *) "c", (char *) __pyx_t_1);
  if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 224; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_d_d_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t(((PyObject *)__pyx_t_2));
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 224; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(((PyObject *)__pyx_t_2)); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;
  goto __pyx_L0;

  /* "binary_tree.pxi":222
 * 
 * 
 * cdef DTYPE_t[:, :, ::1] get_memview_DTYPE_3D(             # <<<<<<<<<<<<<<
 *                                np.ndarray[DTYPE_t, ndim=3, mode='c'] X):
 *     return <DTYPE_t[:X.shape[0], :X.shape[1], :X.shape[2]:1]>\
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(((PyObject *)__pyx_t_2));
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_X.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __pyx_r.data = NULL;
  __pyx_r.memview = NULL;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.get_memview_DTYPE_3D", __pyx_clineno, __pyx_lineno, __pyx_filename);

  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_X.rcbuffer->pybuffer);
  if (unlikely(!__pyx_r.memview)) {
    PyErr_SetString(PyExc_TypeError,"Memoryview return value is not initialized");
  }
  __pyx_L2:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":228
 * 
 * 
 * cdef ITYPE_t[::1] get_memview_ITYPE_1D(             # <<<<<<<<<<<<<<
 *                                np.ndarray[ITYPE_t, ndim=1, mode='c'] X):
 *     return <ITYPE_t[:X.shape[0]:1]> (<ITYPE_t*> X.data)
 */

static __Pyx_memviewslice __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_ITYPE_1D(PyArrayObject *__pyx_v_X) {
  __Pyx_LocalBuf_ND __pyx_pybuffernd_X;
  __Pyx_Buffer __pyx_pybuffer_X;
  __Pyx_memviewslice __pyx_r = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_RefNannyDeclarations
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_t_1;
  struct __pyx_array_obj *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_memview_ITYPE_1D", 0);
  __pyx_pybuffer_X.pybuffer.buf = NULL;
  __pyx_pybuffer_X.refcount = 0;
  __pyx_pybuffernd_X.data = NULL;
  __pyx_pybuffernd_X.rcbuffer = &__pyx_pybuffer_X;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_X.rcbuffer->pybuffer, (PyObject*)__pyx_v_X, &__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 228; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_X.diminfo[0].strides = __pyx_pybuffernd_X.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_X.diminfo[0].shape = __pyx_pybuffernd_X.rcbuffer->pybuffer.shape[0];

  /* "binary_tree.pxi":230
 * cdef ITYPE_t[::1] get_memview_ITYPE_1D(
 *                                np.ndarray[ITYPE_t, ndim=1, mode='c'] X):
 *     return <ITYPE_t[:X.shape[0]:1]> (<ITYPE_t*> X.data)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = ((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *)__pyx_v_X->data);
  if (!__pyx_t_1) {
    PyErr_SetString(PyExc_ValueError,"Cannot create cython.array from NULL pointer");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 230; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_4 = __pyx_format_from_typeinfo(&__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t);
  __pyx_t_3 = Py_BuildValue((char*) "("  __PYX_BUILD_PY_SSIZE_T  ")", ((Py_ssize_t)(__pyx_v_X->dimensions[0])));
  if (unlikely(!__pyx_t_4 || !__pyx_t_3 || !PyBytes_AsString(__pyx_t_4))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 230; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __pyx_array_new(__pyx_t_3, sizeof(__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t), PyBytes_AS_STRING(__pyx_t_4), (char *) "fortran", (char *) __pyx_t_1);
  if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 230; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t(((PyObject *)__pyx_t_2));
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 230; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(((PyObject *)__pyx_t_2)); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;
  goto __pyx_L0;

  /* "binary_tree.pxi":228
 * 
 * 
 * cdef ITYPE_t[::1] get_memview_ITYPE_1D(             # <<<<<<<<<<<<<<
 *                                np.ndarray[ITYPE_t, ndim=1, mode='c'] X):
 *     return <ITYPE_t[:X.shape[0]:1]> (<ITYPE_t*> X.data)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(((PyObject *)__pyx_t_2));
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_X.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __pyx_r.data = NULL;
  __pyx_r.memview = NULL;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.get_memview_ITYPE_1D", __pyx_clineno, __pyx_lineno, __pyx_filename);

  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_X.rcbuffer->pybuffer);
  if (unlikely(!__pyx_r.memview)) {
    PyErr_SetString(PyExc_TypeError,"Memoryview return value is not initialized");
  }
  __pyx_L2:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":233
 * 
 * 
 * cdef ITYPE_t[:, ::1] get_memview_ITYPE_2D(             # <<<<<<<<<<<<<<
 *                                np.ndarray[ITYPE_t, ndim=2, mode='c'] X):
 *     return <ITYPE_t[:X.shape[0], :X.shape[1]:1]> (<ITYPE_t*> X.data)
 */

static __Pyx_memviewslice __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_ITYPE_2D(PyArrayObject *__pyx_v_X) {
  __Pyx_LocalBuf_ND __pyx_pybuffernd_X;
  __Pyx_Buffer __pyx_pybuffer_X;
  __Pyx_memviewslice __pyx_r = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_RefNannyDeclarations
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_t_1;
  struct __pyx_array_obj *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_memview_ITYPE_2D", 0);
  __pyx_pybuffer_X.pybuffer.buf = NULL;
  __pyx_pybuffer_X.refcount = 0;
  __pyx_pybuffernd_X.data = NULL;
  __pyx_pybuffernd_X.rcbuffer = &__pyx_pybuffer_X;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_X.rcbuffer->pybuffer, (PyObject*)__pyx_v_X, &__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_X.diminfo[0].strides = __pyx_pybuffernd_X.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_X.diminfo[0].shape = __pyx_pybuffernd_X.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_X.diminfo[1].strides = __pyx_pybuffernd_X.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_X.diminfo[1].shape = __pyx_pybuffernd_X.rcbuffer->pybuffer.shape[1];

  /* "binary_tree.pxi":235
 * cdef ITYPE_t[:, ::1] get_memview_ITYPE_2D(
 *                                np.ndarray[ITYPE_t, ndim=2, mode='c'] X):
 *     return <ITYPE_t[:X.shape[0], :X.shape[1]:1]> (<ITYPE_t*> X.data)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = ((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *)__pyx_v_X->data);
  if (!__pyx_t_1) {
    PyErr_SetString(PyExc_ValueError,"Cannot create cython.array from NULL pointer");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 235; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_4 = __pyx_format_from_typeinfo(&__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t);
  __pyx_t_3 = Py_BuildValue((char*) "("  __PYX_BUILD_PY_SSIZE_T  __PYX_BUILD_PY_SSIZE_T  ")", ((Py_ssize_t)(__pyx_v_X->dimensions[0])), ((Py_ssize_t)(__pyx_v_X->dimensions[1])));
  if (unlikely(!__pyx_t_4 || !__pyx_t_3 || !PyBytes_AsString(__pyx_t_4))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 235; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __pyx_array_new(__pyx_t_3, sizeof(__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t), PyBytes_AS_STRING(__pyx_t_4), (char *) "c", (char *) __pyx_t_1);
  if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 235; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_d_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t(((PyObject *)__pyx_t_2));
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 235; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(((PyObject *)__pyx_t_2)); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;
  goto __pyx_L0;

  /* "binary_tree.pxi":233
 * 
 * 
 * cdef ITYPE_t[:, ::1] get_memview_ITYPE_2D(             # <<<<<<<<<<<<<<
 *                                np.ndarray[ITYPE_t, ndim=2, mode='c'] X):
 *     return <ITYPE_t[:X.shape[0], :X.shape[1]:1]> (<ITYPE_t*> X.data)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(((PyObject *)__pyx_t_2));
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_X.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __pyx_r.data = NULL;
  __pyx_r.memview = NULL;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.get_memview_ITYPE_2D", __pyx_clineno, __pyx_lineno, __pyx_filename);

  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_X.rcbuffer->pybuffer);
  if (unlikely(!__pyx_r.memview)) {
    PyErr_SetString(PyExc_TypeError,"Memoryview return value is not initialized");
  }
  __pyx_L2:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":238
 * 
 * 
 * cdef NodeHeapData_t[::1] get_memview_NodeHeapData_1D(             # <<<<<<<<<<<<<<
 *                     np.ndarray[NodeHeapData_t, ndim=1, mode='c'] X):
 *     return <NodeHeapData_t[:X.shape[0]:1]> (<NodeHeapData_t*> X.data)
 */

static __Pyx_memviewslice __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_NodeHeapData_1D(PyArrayObject *__pyx_v_X) {
  __Pyx_LocalBuf_ND __pyx_pybuffernd_X;
  __Pyx_Buffer __pyx_pybuffer_X;
  __Pyx_memviewslice __pyx_r = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_RefNannyDeclarations
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t *__pyx_t_1;
  struct __pyx_array_obj *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_memview_NodeHeapData_1D", 0);
  __pyx_pybuffer_X.pybuffer.buf = NULL;
  __pyx_pybuffer_X.refcount = 0;
  __pyx_pybuffernd_X.data = NULL;
  __pyx_pybuffernd_X.rcbuffer = &__pyx_pybuffer_X;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[2];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_X.rcbuffer->pybuffer, (PyObject*)__pyx_v_X, &__Pyx_TypeInfo_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 238; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_X.diminfo[0].strides = __pyx_pybuffernd_X.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_X.diminfo[0].shape = __pyx_pybuffernd_X.rcbuffer->pybuffer.shape[0];

  /* "binary_tree.pxi":240
 * cdef NodeHeapData_t[::1] get_memview_NodeHeapData_1D(
 *                     np.ndarray[NodeHeapData_t, ndim=1, mode='c'] X):
 *     return <NodeHeapData_t[:X.shape[0]:1]> (<NodeHeapData_t*> X.data)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = ((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t *)__pyx_v_X->data);
  if (!__pyx_t_1) {
    PyErr_SetString(PyExc_ValueError,"Cannot create cython.array from NULL pointer");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 240; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_4 = __pyx_format_from_typeinfo(&__Pyx_TypeInfo_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t);
  __pyx_t_3 = Py_BuildValue((char*) "("  __PYX_BUILD_PY_SSIZE_T  ")", ((Py_ssize_t)(__pyx_v_X->dimensions[0])));
  if (unlikely(!__pyx_t_4 || !__pyx_t_3 || !PyBytes_AsString(__pyx_t_4))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 240; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __pyx_array_new(__pyx_t_3, sizeof(struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t), PyBytes_AS_STRING(__pyx_t_4), (char *) "fortran", (char *) __pyx_t_1);
  if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 240; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_dc_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t(((PyObject *)__pyx_t_2));
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 240; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(((PyObject *)__pyx_t_2)); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;
  goto __pyx_L0;

  /* "binary_tree.pxi":238
 * 
 * 
 * cdef NodeHeapData_t[::1] get_memview_NodeHeapData_1D(             # <<<<<<<<<<<<<<
 *                     np.ndarray[NodeHeapData_t, ndim=1, mode='c'] X):
 *     return <NodeHeapData_t[:X.shape[0]:1]> (<NodeHeapData_t*> X.data)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(((PyObject *)__pyx_t_2));
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_X.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __pyx_r.data = NULL;
  __pyx_r.memview = NULL;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.get_memview_NodeHeapData_1D", __pyx_clineno, __pyx_lineno, __pyx_filename);

  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_X.rcbuffer->pybuffer);
  if (unlikely(!__pyx_r.memview)) {
    PyErr_SetString(PyExc_TypeError,"Memoryview return value is not initialized");
  }
  __pyx_L2:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":243
 * 
 * 
 * cdef NodeData_t[::1] get_memview_NodeData_1D(             # <<<<<<<<<<<<<<
 *                     np.ndarray[NodeData_t, ndim=1, mode='c'] X):
 *     return <NodeData_t[:X.shape[0]:1]> (<NodeData_t*> X.data)
 */

static __Pyx_memviewslice __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_NodeData_1D(PyArrayObject *__pyx_v_X) {
  __Pyx_LocalBuf_ND __pyx_pybuffernd_X;
  __Pyx_Buffer __pyx_pybuffer_X;
  __Pyx_memviewslice __pyx_r = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_RefNannyDeclarations
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *__pyx_t_1;
  struct __pyx_array_obj *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_memview_NodeData_1D", 0);
  __pyx_pybuffer_X.pybuffer.buf = NULL;
  __pyx_pybuffer_X.refcount = 0;
  __pyx_pybuffernd_X.data = NULL;
  __pyx_pybuffernd_X.rcbuffer = &__pyx_pybuffer_X;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[2];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_X.rcbuffer->pybuffer, (PyObject*)__pyx_v_X, &__Pyx_TypeInfo_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 1, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_X.diminfo[0].strides = __pyx_pybuffernd_X.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_X.diminfo[0].shape = __pyx_pybuffernd_X.rcbuffer->pybuffer.shape[0];

  /* "binary_tree.pxi":245
 * cdef NodeData_t[::1] get_memview_NodeData_1D(
 *                     np.ndarray[NodeData_t, ndim=1, mode='c'] X):
 *     return <NodeData_t[:X.shape[0]:1]> (<NodeData_t*> X.data)             # <<<<<<<<<<<<<<
 * 
 * ######################################################################
 */
  __pyx_t_1 = ((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *)__pyx_v_X->data);
  if (!__pyx_t_1) {
    PyErr_SetString(PyExc_ValueError,"Cannot create cython.array from NULL pointer");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 245; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_4 = __pyx_format_from_typeinfo(&__Pyx_TypeInfo_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t);
  __pyx_t_3 = Py_BuildValue((char*) "("  __PYX_BUILD_PY_SSIZE_T  ")", ((Py_ssize_t)(__pyx_v_X->dimensions[0])));
  if (unlikely(!__pyx_t_4 || !__pyx_t_3 || !PyBytes_AsString(__pyx_t_4))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 245; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __pyx_array_new(__pyx_t_3, sizeof(struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t), PyBytes_AS_STRING(__pyx_t_4), (char *) "fortran", (char *) __pyx_t_1);
  if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 245; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_dc_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t(((PyObject *)__pyx_t_2));
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 245; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(((PyObject *)__pyx_t_2)); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;
  goto __pyx_L0;

  /* "binary_tree.pxi":243
 * 
 * 
 * cdef NodeData_t[::1] get_memview_NodeData_1D(             # <<<<<<<<<<<<<<
 *                     np.ndarray[NodeData_t, ndim=1, mode='c'] X):
 *     return <NodeData_t[:X.shape[0]:1]> (<NodeData_t*> X.data)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(((PyObject *)__pyx_t_2));
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_X.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __pyx_r.data = NULL;
  __pyx_r.memview = NULL;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.get_memview_NodeData_1D", __pyx_clineno, __pyx_lineno, __pyx_filename);

  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_X.rcbuffer->pybuffer);
  if (unlikely(!__pyx_r.memview)) {
    PyErr_SetString(PyExc_TypeError,"Memoryview return value is not initialized");
  }
  __pyx_L2:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":359
 * ######################################################################
 * # Utility functions
 * cdef DTYPE_t logaddexp(DTYPE_t x1, DTYPE_t x2):             # <<<<<<<<<<<<<<
 *     """logaddexp(x1, x2) -> log(exp(x1) + exp(x2))"""
 *     cdef DTYPE_t a = fmax(x1, x2)
 */

static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_logaddexp(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_x1, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_x2) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_a;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("logaddexp", 0);

  /* "binary_tree.pxi":361
 * cdef DTYPE_t logaddexp(DTYPE_t x1, DTYPE_t x2):
 *     """logaddexp(x1, x2) -> log(exp(x1) + exp(x2))"""
 *     cdef DTYPE_t a = fmax(x1, x2)             # <<<<<<<<<<<<<<
 *     if a == NEG_INF:
 *         return NEG_INF
 */
  __pyx_v_a = __pyx_f_7sklearn_9neighbors_7kd_tree_fmax(__pyx_v_x1, __pyx_v_x2);

  /* "binary_tree.pxi":362
 *     """logaddexp(x1, x2) -> log(exp(x1) + exp(x2))"""
 *     cdef DTYPE_t a = fmax(x1, x2)
 *     if a == NEG_INF:             # <<<<<<<<<<<<<<
 *         return NEG_INF
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_a == __pyx_v_7sklearn_9neighbors_7kd_tree_NEG_INF) != 0);
  if (__pyx_t_1) {

    /* "binary_tree.pxi":363
 *     cdef DTYPE_t a = fmax(x1, x2)
 *     if a == NEG_INF:
 *         return NEG_INF             # <<<<<<<<<<<<<<
 *     else:
 *         return a + log(exp(x1 - a) + exp(x2 - a))
 */
    __pyx_r = __pyx_v_7sklearn_9neighbors_7kd_tree_NEG_INF;
    goto __pyx_L0;
  }
  /*else*/ {

    /* "binary_tree.pxi":365
 *         return NEG_INF
 *     else:
 *         return a + log(exp(x1 - a) + exp(x2 - a))             # <<<<<<<<<<<<<<
 * 
 * cdef DTYPE_t logsubexp(DTYPE_t x1, DTYPE_t x2):
 */
    __pyx_r = (__pyx_v_a + log((exp((__pyx_v_x1 - __pyx_v_a)) + exp((__pyx_v_x2 - __pyx_v_a)))));
    goto __pyx_L0;
  }

  /* "binary_tree.pxi":359
 * ######################################################################
 * # Utility functions
 * cdef DTYPE_t logaddexp(DTYPE_t x1, DTYPE_t x2):             # <<<<<<<<<<<<<<
 *     """logaddexp(x1, x2) -> log(exp(x1) + exp(x2))"""
 *     cdef DTYPE_t a = fmax(x1, x2)
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":367
 *         return a + log(exp(x1 - a) + exp(x2 - a))
 * 
 * cdef DTYPE_t logsubexp(DTYPE_t x1, DTYPE_t x2):             # <<<<<<<<<<<<<<
 *     """logsubexp(x1, x2) -> log(exp(x1) - exp(x2))"""
 *     if x1 <= x2:
 */

static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_logsubexp(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_x1, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_x2) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("logsubexp", 0);

  /* "binary_tree.pxi":369
 * cdef DTYPE_t logsubexp(DTYPE_t x1, DTYPE_t x2):
 *     """logsubexp(x1, x2) -> log(exp(x1) - exp(x2))"""
 *     if x1 <= x2:             # <<<<<<<<<<<<<<
 *         return NEG_INF
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_x1 <= __pyx_v_x2) != 0);
  if (__pyx_t_1) {

    /* "binary_tree.pxi":370
 *     """logsubexp(x1, x2) -> log(exp(x1) - exp(x2))"""
 *     if x1 <= x2:
 *         return NEG_INF             # <<<<<<<<<<<<<<
 *     else:
 *         return x1 + log(1 - exp(x2 - x1))
 */
    __pyx_r = __pyx_v_7sklearn_9neighbors_7kd_tree_NEG_INF;
    goto __pyx_L0;
  }
  /*else*/ {

    /* "binary_tree.pxi":372
 *         return NEG_INF
 *     else:
 *         return x1 + log(1 - exp(x2 - x1))             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_r = (__pyx_v_x1 + log((1.0 - exp((__pyx_v_x2 - __pyx_v_x1)))));
    goto __pyx_L0;
  }

  /* "binary_tree.pxi":367
 *         return a + log(exp(x1 - a) + exp(x2 - a))
 * 
 * cdef DTYPE_t logsubexp(DTYPE_t x1, DTYPE_t x2):             # <<<<<<<<<<<<<<
 *     """logsubexp(x1, x2) -> log(exp(x1) - exp(x2))"""
 *     if x1 <= x2:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":394
 * 
 * 
 * cdef inline DTYPE_t log_gaussian_kernel(DTYPE_t dist, DTYPE_t h):             # <<<<<<<<<<<<<<
 *     """log of the gaussian kernel for bandwidth h (unnormalized)"""
 *     return -0.5 * (dist * dist) / (h * h)
 */

static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_log_gaussian_kernel(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_h) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("log_gaussian_kernel", 0);

  /* "binary_tree.pxi":396
 * cdef inline DTYPE_t log_gaussian_kernel(DTYPE_t dist, DTYPE_t h):
 *     """log of the gaussian kernel for bandwidth h (unnormalized)"""
 *     return -0.5 * (dist * dist) / (h * h)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((-0.5 * (__pyx_v_dist * __pyx_v_dist)) / (__pyx_v_h * __pyx_v_h));
  goto __pyx_L0;

  /* "binary_tree.pxi":394
 * 
 * 
 * cdef inline DTYPE_t log_gaussian_kernel(DTYPE_t dist, DTYPE_t h):             # <<<<<<<<<<<<<<
 *     """log of the gaussian kernel for bandwidth h (unnormalized)"""
 *     return -0.5 * (dist * dist) / (h * h)
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":399
 * 
 * 
 * cdef inline DTYPE_t log_tophat_kernel(DTYPE_t dist, DTYPE_t h):             # <<<<<<<<<<<<<<
 *     """log of the tophat kernel for bandwidth h (unnormalized)"""
 *     if dist < h:
 */

static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_log_tophat_kernel(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_h) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("log_tophat_kernel", 0);

  /* "binary_tree.pxi":401
 * cdef inline DTYPE_t log_tophat_kernel(DTYPE_t dist, DTYPE_t h):
 *     """log of the tophat kernel for bandwidth h (unnormalized)"""
 *     if dist < h:             # <<<<<<<<<<<<<<
 *         return 0.0
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_dist < __pyx_v_h) != 0);
  if (__pyx_t_1) {

    /* "binary_tree.pxi":402
 *     """log of the tophat kernel for bandwidth h (unnormalized)"""
 *     if dist < h:
 *         return 0.0             # <<<<<<<<<<<<<<
 *     else:
 *         return NEG_INF
 */
    __pyx_r = 0.0;
    goto __pyx_L0;
  }
  /*else*/ {

    /* "binary_tree.pxi":404
 *         return 0.0
 *     else:
 *         return NEG_INF             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_r = __pyx_v_7sklearn_9neighbors_7kd_tree_NEG_INF;
    goto __pyx_L0;
  }

  /* "binary_tree.pxi":399
 * 
 * 
 * cdef inline DTYPE_t log_tophat_kernel(DTYPE_t dist, DTYPE_t h):             # <<<<<<<<<<<<<<
 *     """log of the tophat kernel for bandwidth h (unnormalized)"""
 *     if dist < h:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":407
 * 
 * 
 * cdef inline DTYPE_t log_epanechnikov_kernel(DTYPE_t dist, DTYPE_t h):             # <<<<<<<<<<<<<<
 *     """log of the epanechnikov kernel for bandwidth h (unnormalized)"""
 *     if dist < h:
 */

static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_log_epanechnikov_kernel(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_h) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("log_epanechnikov_kernel", 0);

  /* "binary_tree.pxi":409
 * cdef inline DTYPE_t log_epanechnikov_kernel(DTYPE_t dist, DTYPE_t h):
 *     """log of the epanechnikov kernel for bandwidth h (unnormalized)"""
 *     if dist < h:             # <<<<<<<<<<<<<<
 *         return log(1.0 - (dist * dist) / (h * h))
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_dist < __pyx_v_h) != 0);
  if (__pyx_t_1) {

    /* "binary_tree.pxi":410
 *     """log of the epanechnikov kernel for bandwidth h (unnormalized)"""
 *     if dist < h:
 *         return log(1.0 - (dist * dist) / (h * h))             # <<<<<<<<<<<<<<
 *     else:
 *         return NEG_INF
 */
    __pyx_r = log((1.0 - ((__pyx_v_dist * __pyx_v_dist) / (__pyx_v_h * __pyx_v_h))));
    goto __pyx_L0;
  }
  /*else*/ {

    /* "binary_tree.pxi":412
 *         return log(1.0 - (dist * dist) / (h * h))
 *     else:
 *         return NEG_INF             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_r = __pyx_v_7sklearn_9neighbors_7kd_tree_NEG_INF;
    goto __pyx_L0;
  }

  /* "binary_tree.pxi":407
 * 
 * 
 * cdef inline DTYPE_t log_epanechnikov_kernel(DTYPE_t dist, DTYPE_t h):             # <<<<<<<<<<<<<<
 *     """log of the epanechnikov kernel for bandwidth h (unnormalized)"""
 *     if dist < h:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":415
 * 
 * 
 * cdef inline DTYPE_t log_exponential_kernel(DTYPE_t dist, DTYPE_t h):             # <<<<<<<<<<<<<<
 *     """log of the exponential kernel for bandwidth h (unnormalized)"""
 *     return -dist / h
 */

static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_log_exponential_kernel(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_h) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("log_exponential_kernel", 0);

  /* "binary_tree.pxi":417
 * cdef inline DTYPE_t log_exponential_kernel(DTYPE_t dist, DTYPE_t h):
 *     """log of the exponential kernel for bandwidth h (unnormalized)"""
 *     return -dist / h             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((-__pyx_v_dist) / __pyx_v_h);
  goto __pyx_L0;

  /* "binary_tree.pxi":415
 * 
 * 
 * cdef inline DTYPE_t log_exponential_kernel(DTYPE_t dist, DTYPE_t h):             # <<<<<<<<<<<<<<
 *     """log of the exponential kernel for bandwidth h (unnormalized)"""
 *     return -dist / h
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":420
 * 
 * 
 * cdef inline DTYPE_t log_linear_kernel(DTYPE_t dist, DTYPE_t h):             # <<<<<<<<<<<<<<
 *     """log of the linear kernel for bandwidth h (unnormalized)"""
 *     if dist < h:
 */

static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_log_linear_kernel(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_h) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("log_linear_kernel", 0);

  /* "binary_tree.pxi":422
 * cdef inline DTYPE_t log_linear_kernel(DTYPE_t dist, DTYPE_t h):
 *     """log of the linear kernel for bandwidth h (unnormalized)"""
 *     if dist < h:             # <<<<<<<<<<<<<<
 *         return log(1 - dist / h)
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_dist < __pyx_v_h) != 0);
  if (__pyx_t_1) {

    /* "binary_tree.pxi":423
 *     """log of the linear kernel for bandwidth h (unnormalized)"""
 *     if dist < h:
 *         return log(1 - dist / h)             # <<<<<<<<<<<<<<
 *     else:
 *         return NEG_INF
 */
    __pyx_r = log((1.0 - (__pyx_v_dist / __pyx_v_h)));
    goto __pyx_L0;
  }
  /*else*/ {

    /* "binary_tree.pxi":425
 *         return log(1 - dist / h)
 *     else:
 *         return NEG_INF             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_r = __pyx_v_7sklearn_9neighbors_7kd_tree_NEG_INF;
    goto __pyx_L0;
  }

  /* "binary_tree.pxi":420
 * 
 * 
 * cdef inline DTYPE_t log_linear_kernel(DTYPE_t dist, DTYPE_t h):             # <<<<<<<<<<<<<<
 *     """log of the linear kernel for bandwidth h (unnormalized)"""
 *     if dist < h:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":428
 * 
 * 
 * cdef inline DTYPE_t log_cosine_kernel(DTYPE_t dist, DTYPE_t h):             # <<<<<<<<<<<<<<
 *     """log of the cosine kernel for bandwidth h (unnormalized)"""
 *     if dist < h:
 */

static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_log_cosine_kernel(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_h) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("log_cosine_kernel", 0);

  /* "binary_tree.pxi":430
 * cdef inline DTYPE_t log_cosine_kernel(DTYPE_t dist, DTYPE_t h):
 *     """log of the cosine kernel for bandwidth h (unnormalized)"""
 *     if dist < h:             # <<<<<<<<<<<<<<
 *         return log(cos(0.5 * PI * dist / h))
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_dist < __pyx_v_h) != 0);
  if (__pyx_t_1) {

    /* "binary_tree.pxi":431
 *     """log of the cosine kernel for bandwidth h (unnormalized)"""
 *     if dist < h:
 *         return log(cos(0.5 * PI * dist / h))             # <<<<<<<<<<<<<<
 *     else:
 *         return NEG_INF
 */
    __pyx_r = log(cos((((0.5 * __pyx_v_7sklearn_9neighbors_7kd_tree_PI) * __pyx_v_dist) / __pyx_v_h)));
    goto __pyx_L0;
  }
  /*else*/ {

    /* "binary_tree.pxi":433
 *         return log(cos(0.5 * PI * dist / h))
 *     else:
 *         return NEG_INF             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_r = __pyx_v_7sklearn_9neighbors_7kd_tree_NEG_INF;
    goto __pyx_L0;
  }

  /* "binary_tree.pxi":428
 * 
 * 
 * cdef inline DTYPE_t log_cosine_kernel(DTYPE_t dist, DTYPE_t h):             # <<<<<<<<<<<<<<
 *     """log of the cosine kernel for bandwidth h (unnormalized)"""
 *     if dist < h:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":436
 * 
 * 
 * cdef inline DTYPE_t compute_log_kernel(DTYPE_t dist, DTYPE_t h,             # <<<<<<<<<<<<<<
 *                                        KernelType kernel):
 *     """Given a KernelType enumeration, compute the appropriate log-kernel"""
 */

static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_compute_log_kernel(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_h, enum __pyx_t_7sklearn_9neighbors_7kd_tree_KernelType __pyx_v_kernel) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compute_log_kernel", 0);

  /* "binary_tree.pxi":449
 *     elif kernel == LINEAR_KERNEL:
 *         return log_linear_kernel(dist, h)
 *     elif kernel == COSINE_KERNEL:             # <<<<<<<<<<<<<<
 *         return log_cosine_kernel(dist, h)
 * 
 */
  switch (__pyx_v_kernel) {

    /* "binary_tree.pxi":439
 *                                        KernelType kernel):
 *     """Given a KernelType enumeration, compute the appropriate log-kernel"""
 *     if kernel == GAUSSIAN_KERNEL:             # <<<<<<<<<<<<<<
 *         return log_gaussian_kernel(dist, h)
 *     elif kernel == TOPHAT_KERNEL:
 */
    case __pyx_e_7sklearn_9neighbors_7kd_tree_GAUSSIAN_KERNEL:

    /* "binary_tree.pxi":440
 *     """Given a KernelType enumeration, compute the appropriate log-kernel"""
 *     if kernel == GAUSSIAN_KERNEL:
 *         return log_gaussian_kernel(dist, h)             # <<<<<<<<<<<<<<
 *     elif kernel == TOPHAT_KERNEL:
 *         return log_tophat_kernel(dist, h)
 */
    __pyx_r = __pyx_f_7sklearn_9neighbors_7kd_tree_log_gaussian_kernel(__pyx_v_dist, __pyx_v_h);
    goto __pyx_L0;
    break;

    /* "binary_tree.pxi":441
 *     if kernel == GAUSSIAN_KERNEL:
 *         return log_gaussian_kernel(dist, h)
 *     elif kernel == TOPHAT_KERNEL:             # <<<<<<<<<<<<<<
 *         return log_tophat_kernel(dist, h)
 *     elif kernel == EPANECHNIKOV_KERNEL:
 */
    case __pyx_e_7sklearn_9neighbors_7kd_tree_TOPHAT_KERNEL:

    /* "binary_tree.pxi":442
 *         return log_gaussian_kernel(dist, h)
 *     elif kernel == TOPHAT_KERNEL:
 *         return log_tophat_kernel(dist, h)             # <<<<<<<<<<<<<<
 *     elif kernel == EPANECHNIKOV_KERNEL:
 *         return log_epanechnikov_kernel(dist, h)
 */
    __pyx_r = __pyx_f_7sklearn_9neighbors_7kd_tree_log_tophat_kernel(__pyx_v_dist, __pyx_v_h);
    goto __pyx_L0;
    break;

    /* "binary_tree.pxi":443
 *     elif kernel == TOPHAT_KERNEL:
 *         return log_tophat_kernel(dist, h)
 *     elif kernel == EPANECHNIKOV_KERNEL:             # <<<<<<<<<<<<<<
 *         return log_epanechnikov_kernel(dist, h)
 *     elif kernel == EXPONENTIAL_KERNEL:
 */
    case __pyx_e_7sklearn_9neighbors_7kd_tree_EPANECHNIKOV_KERNEL:

    /* "binary_tree.pxi":444
 *         return log_tophat_kernel(dist, h)
 *     elif kernel == EPANECHNIKOV_KERNEL:
 *         return log_epanechnikov_kernel(dist, h)             # <<<<<<<<<<<<<<
 *     elif kernel == EXPONENTIAL_KERNEL:
 *         return log_exponential_kernel(dist, h)
 */
    __pyx_r = __pyx_f_7sklearn_9neighbors_7kd_tree_log_epanechnikov_kernel(__pyx_v_dist, __pyx_v_h);
    goto __pyx_L0;
    break;

    /* "binary_tree.pxi":445
 *     elif kernel == EPANECHNIKOV_KERNEL:
 *         return log_epanechnikov_kernel(dist, h)
 *     elif kernel == EXPONENTIAL_KERNEL:             # <<<<<<<<<<<<<<
 *         return log_exponential_kernel(dist, h)
 *     elif kernel == LINEAR_KERNEL:
 */
    case __pyx_e_7sklearn_9neighbors_7kd_tree_EXPONENTIAL_KERNEL:

    /* "binary_tree.pxi":446
 *         return log_epanechnikov_kernel(dist, h)
 *     elif kernel == EXPONENTIAL_KERNEL:
 *         return log_exponential_kernel(dist, h)             # <<<<<<<<<<<<<<
 *     elif kernel == LINEAR_KERNEL:
 *         return log_linear_kernel(dist, h)
 */
    __pyx_r = __pyx_f_7sklearn_9neighbors_7kd_tree_log_exponential_kernel(__pyx_v_dist, __pyx_v_h);
    goto __pyx_L0;
    break;

    /* "binary_tree.pxi":447
 *     elif kernel == EXPONENTIAL_KERNEL:
 *         return log_exponential_kernel(dist, h)
 *     elif kernel == LINEAR_KERNEL:             # <<<<<<<<<<<<<<
 *         return log_linear_kernel(dist, h)
 *     elif kernel == COSINE_KERNEL:
 */
    case __pyx_e_7sklearn_9neighbors_7kd_tree_LINEAR_KERNEL:

    /* "binary_tree.pxi":448
 *         return log_exponential_kernel(dist, h)
 *     elif kernel == LINEAR_KERNEL:
 *         return log_linear_kernel(dist, h)             # <<<<<<<<<<<<<<
 *     elif kernel == COSINE_KERNEL:
 *         return log_cosine_kernel(dist, h)
 */
    __pyx_r = __pyx_f_7sklearn_9neighbors_7kd_tree_log_linear_kernel(__pyx_v_dist, __pyx_v_h);
    goto __pyx_L0;
    break;

    /* "binary_tree.pxi":449
 *     elif kernel == LINEAR_KERNEL:
 *         return log_linear_kernel(dist, h)
 *     elif kernel == COSINE_KERNEL:             # <<<<<<<<<<<<<<
 *         return log_cosine_kernel(dist, h)
 * 
 */
    case __pyx_e_7sklearn_9neighbors_7kd_tree_COSINE_KERNEL:

    /* "binary_tree.pxi":450
 *         return log_linear_kernel(dist, h)
 *     elif kernel == COSINE_KERNEL:
 *         return log_cosine_kernel(dist, h)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_r = __pyx_f_7sklearn_9neighbors_7kd_tree_log_cosine_kernel(__pyx_v_dist, __pyx_v_h);
    goto __pyx_L0;
    break;
    default: break;
  }

  /* "binary_tree.pxi":436
 * 
 * 
 * cdef inline DTYPE_t compute_log_kernel(DTYPE_t dist, DTYPE_t h,             # <<<<<<<<<<<<<<
 *                                        KernelType kernel):
 *     """Given a KernelType enumeration, compute the appropriate log-kernel"""
 */

  /* function exit code */
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":456
 * # Kernel norms are defined via the volume element V_n
 * # and surface element S_(n-1) of an n-sphere.
 * cdef DTYPE_t logVn(ITYPE_t n):             # <<<<<<<<<<<<<<
 *     """V_n = pi^(n/2) / gamma(n/2 - 1)"""
 *     return 0.5 * n * LOG_PI - lgamma(0.5 * n + 1)
 */

static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_logVn(__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("logVn", 0);

  /* "binary_tree.pxi":458
 * cdef DTYPE_t logVn(ITYPE_t n):
 *     """V_n = pi^(n/2) / gamma(n/2 - 1)"""
 *     return 0.5 * n * LOG_PI - lgamma(0.5 * n + 1)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = (((0.5 * __pyx_v_n) * __pyx_v_7sklearn_9neighbors_7kd_tree_LOG_PI) - __pyx_f_7sklearn_5utils_6lgamma_lgamma(((0.5 * __pyx_v_n) + 1.0)));
  goto __pyx_L0;

  /* "binary_tree.pxi":456
 * # Kernel norms are defined via the volume element V_n
 * # and surface element S_(n-1) of an n-sphere.
 * cdef DTYPE_t logVn(ITYPE_t n):             # <<<<<<<<<<<<<<
 *     """V_n = pi^(n/2) / gamma(n/2 - 1)"""
 *     return 0.5 * n * LOG_PI - lgamma(0.5 * n + 1)
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":461
 * 
 * 
 * cdef DTYPE_t logSn(ITYPE_t n):             # <<<<<<<<<<<<<<
 *     """V_(n+1) = int_0^1 S_n r^n dr"""
 *     return LOG_2PI + logVn(n - 1)
 */

static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_logSn(__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("logSn", 0);

  /* "binary_tree.pxi":463
 * cdef DTYPE_t logSn(ITYPE_t n):
 *     """V_(n+1) = int_0^1 S_n r^n dr"""
 *     return LOG_2PI + logVn(n - 1)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = (__pyx_v_7sklearn_9neighbors_7kd_tree_LOG_2PI + __pyx_f_7sklearn_9neighbors_7kd_tree_logVn((__pyx_v_n - 1)));
  goto __pyx_L0;

  /* "binary_tree.pxi":461
 * 
 * 
 * cdef DTYPE_t logSn(ITYPE_t n):             # <<<<<<<<<<<<<<
 *     """V_(n+1) = int_0^1 S_n r^n dr"""
 *     return LOG_2PI + logVn(n - 1)
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":466
 * 
 * 
 * cdef DTYPE_t _log_kernel_norm(DTYPE_t h, ITYPE_t d,             # <<<<<<<<<<<<<<
 *                               KernelType kernel) except -1:
 *     """Given a KernelType enumeration, compute the kernel normalization.
 */

static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree__log_kernel_norm(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_h, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_d, enum __pyx_t_7sklearn_9neighbors_7kd_tree_KernelType __pyx_v_kernel) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_tmp;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_factor;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_k;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_log_kernel_norm", 0);

  /* "binary_tree.pxi":472
 *     h is the bandwidth, d is the dimension.
 *     """
 *     cdef DTYPE_t tmp, factor = 0             # <<<<<<<<<<<<<<
 *     cdef ITYPE_t k
 *     if kernel == GAUSSIAN_KERNEL:
 */
  __pyx_v_factor = 0.0;

  /* "binary_tree.pxi":484
 *     elif kernel == LINEAR_KERNEL:
 *         factor = logVn(d) - log(d + 1.)
 *     elif kernel == COSINE_KERNEL:             # <<<<<<<<<<<<<<
 *         # this is derived from a chain rule integration
 *         factor = 0
 */
  switch (__pyx_v_kernel) {

    /* "binary_tree.pxi":474
 *     cdef DTYPE_t tmp, factor = 0
 *     cdef ITYPE_t k
 *     if kernel == GAUSSIAN_KERNEL:             # <<<<<<<<<<<<<<
 *         factor = 0.5 * d * LOG_2PI
 *     elif kernel == TOPHAT_KERNEL:
 */
    case __pyx_e_7sklearn_9neighbors_7kd_tree_GAUSSIAN_KERNEL:

    /* "binary_tree.pxi":475
 *     cdef ITYPE_t k
 *     if kernel == GAUSSIAN_KERNEL:
 *         factor = 0.5 * d * LOG_2PI             # <<<<<<<<<<<<<<
 *     elif kernel == TOPHAT_KERNEL:
 *         factor = logVn(d)
 */
    __pyx_v_factor = ((0.5 * __pyx_v_d) * __pyx_v_7sklearn_9neighbors_7kd_tree_LOG_2PI);
    break;

    /* "binary_tree.pxi":476
 *     if kernel == GAUSSIAN_KERNEL:
 *         factor = 0.5 * d * LOG_2PI
 *     elif kernel == TOPHAT_KERNEL:             # <<<<<<<<<<<<<<
 *         factor = logVn(d)
 *     elif kernel == EPANECHNIKOV_KERNEL:
 */
    case __pyx_e_7sklearn_9neighbors_7kd_tree_TOPHAT_KERNEL:

    /* "binary_tree.pxi":477
 *         factor = 0.5 * d * LOG_2PI
 *     elif kernel == TOPHAT_KERNEL:
 *         factor = logVn(d)             # <<<<<<<<<<<<<<
 *     elif kernel == EPANECHNIKOV_KERNEL:
 *         factor = logVn(d) + log(2. / (d + 2.))
 */
    __pyx_v_factor = __pyx_f_7sklearn_9neighbors_7kd_tree_logVn(__pyx_v_d);
    break;

    /* "binary_tree.pxi":478
 *     elif kernel == TOPHAT_KERNEL:
 *         factor = logVn(d)
 *     elif kernel == EPANECHNIKOV_KERNEL:             # <<<<<<<<<<<<<<
 *         factor = logVn(d) + log(2. / (d + 2.))
 *     elif kernel == EXPONENTIAL_KERNEL:
 */
    case __pyx_e_7sklearn_9neighbors_7kd_tree_EPANECHNIKOV_KERNEL:

    /* "binary_tree.pxi":479
 *         factor = logVn(d)
 *     elif kernel == EPANECHNIKOV_KERNEL:
 *         factor = logVn(d) + log(2. / (d + 2.))             # <<<<<<<<<<<<<<
 *     elif kernel == EXPONENTIAL_KERNEL:
 *         factor = logSn(d - 1) + lgamma(d)
 */
    __pyx_v_factor = (__pyx_f_7sklearn_9neighbors_7kd_tree_logVn(__pyx_v_d) + log((2. / (__pyx_v_d + 2.))));
    break;

    /* "binary_tree.pxi":480
 *     elif kernel == EPANECHNIKOV_KERNEL:
 *         factor = logVn(d) + log(2. / (d + 2.))
 *     elif kernel == EXPONENTIAL_KERNEL:             # <<<<<<<<<<<<<<
 *         factor = logSn(d - 1) + lgamma(d)
 *     elif kernel == LINEAR_KERNEL:
 */
    case __pyx_e_7sklearn_9neighbors_7kd_tree_EXPONENTIAL_KERNEL:

    /* "binary_tree.pxi":481
 *         factor = logVn(d) + log(2. / (d + 2.))
 *     elif kernel == EXPONENTIAL_KERNEL:
 *         factor = logSn(d - 1) + lgamma(d)             # <<<<<<<<<<<<<<
 *     elif kernel == LINEAR_KERNEL:
 *         factor = logVn(d) - log(d + 1.)
 */
    __pyx_v_factor = (__pyx_f_7sklearn_9neighbors_7kd_tree_logSn((__pyx_v_d - 1)) + __pyx_f_7sklearn_5utils_6lgamma_lgamma(__pyx_v_d));
    break;

    /* "binary_tree.pxi":482
 *     elif kernel == EXPONENTIAL_KERNEL:
 *         factor = logSn(d - 1) + lgamma(d)
 *     elif kernel == LINEAR_KERNEL:             # <<<<<<<<<<<<<<
 *         factor = logVn(d) - log(d + 1.)
 *     elif kernel == COSINE_KERNEL:
 */
    case __pyx_e_7sklearn_9neighbors_7kd_tree_LINEAR_KERNEL:

    /* "binary_tree.pxi":483
 *         factor = logSn(d - 1) + lgamma(d)
 *     elif kernel == LINEAR_KERNEL:
 *         factor = logVn(d) - log(d + 1.)             # <<<<<<<<<<<<<<
 *     elif kernel == COSINE_KERNEL:
 *         # this is derived from a chain rule integration
 */
    __pyx_v_factor = (__pyx_f_7sklearn_9neighbors_7kd_tree_logVn(__pyx_v_d) - log((__pyx_v_d + 1.)));
    break;

    /* "binary_tree.pxi":484
 *     elif kernel == LINEAR_KERNEL:
 *         factor = logVn(d) - log(d + 1.)
 *     elif kernel == COSINE_KERNEL:             # <<<<<<<<<<<<<<
 *         # this is derived from a chain rule integration
 *         factor = 0
 */
    case __pyx_e_7sklearn_9neighbors_7kd_tree_COSINE_KERNEL:

    /* "binary_tree.pxi":486
 *     elif kernel == COSINE_KERNEL:
 *         # this is derived from a chain rule integration
 *         factor = 0             # <<<<<<<<<<<<<<
 *         tmp = 2. / PI
 *         for k in range(1, d + 1, 2):
 */
    __pyx_v_factor = 0.0;

    /* "binary_tree.pxi":487
 *         # this is derived from a chain rule integration
 *         factor = 0
 *         tmp = 2. / PI             # <<<<<<<<<<<<<<
 *         for k in range(1, d + 1, 2):
 *             factor += tmp
 */
    __pyx_v_tmp = (2. / __pyx_v_7sklearn_9neighbors_7kd_tree_PI);

    /* "binary_tree.pxi":488
 *         factor = 0
 *         tmp = 2. / PI
 *         for k in range(1, d + 1, 2):             # <<<<<<<<<<<<<<
 *             factor += tmp
 *             tmp *= -(d - k) * (d - k - 1) * (2. / PI) ** 2
 */
    __pyx_t_1 = (__pyx_v_d + 1);
    for (__pyx_t_2 = 1; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=2) {
      __pyx_v_k = __pyx_t_2;

      /* "binary_tree.pxi":489
 *         tmp = 2. / PI
 *         for k in range(1, d + 1, 2):
 *             factor += tmp             # <<<<<<<<<<<<<<
 *             tmp *= -(d - k) * (d - k - 1) * (2. / PI) ** 2
 *         factor = log(factor) + logSn(d - 1)
 */
      __pyx_v_factor = (__pyx_v_factor + __pyx_v_tmp);

      /* "binary_tree.pxi":490
 *         for k in range(1, d + 1, 2):
 *             factor += tmp
 *             tmp *= -(d - k) * (d - k - 1) * (2. / PI) ** 2             # <<<<<<<<<<<<<<
 *         factor = log(factor) + logSn(d - 1)
 *     else:
 */
      __pyx_v_tmp = (__pyx_v_tmp * (((-(__pyx_v_d - __pyx_v_k)) * ((__pyx_v_d - __pyx_v_k) - 1)) * pow((2. / __pyx_v_7sklearn_9neighbors_7kd_tree_PI), 2.0)));
    }

    /* "binary_tree.pxi":491
 *             factor += tmp
 *             tmp *= -(d - k) * (d - k - 1) * (2. / PI) ** 2
 *         factor = log(factor) + logSn(d - 1)             # <<<<<<<<<<<<<<
 *     else:
 *         raise ValueError("Kernel code not recognized")
 */
    __pyx_v_factor = (log(__pyx_v_factor) + __pyx_f_7sklearn_9neighbors_7kd_tree_logSn((__pyx_v_d - 1)));
    break;
    default:

    /* "binary_tree.pxi":493
 *         factor = log(factor) + logSn(d - 1)
 *     else:
 *         raise ValueError("Kernel code not recognized")             # <<<<<<<<<<<<<<
 *     return -factor - d * log(h)
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 493; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 493; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    break;
  }

  /* "binary_tree.pxi":494
 *     else:
 *         raise ValueError("Kernel code not recognized")
 *     return -factor - d * log(h)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((-__pyx_v_factor) - (__pyx_v_d * log(__pyx_v_h)));
  goto __pyx_L0;

  /* "binary_tree.pxi":466
 * 
 * 
 * cdef DTYPE_t _log_kernel_norm(DTYPE_t h, ITYPE_t d,             # <<<<<<<<<<<<<<
 *                               KernelType kernel) except -1:
 *     """Given a KernelType enumeration, compute the kernel normalization.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree._log_kernel_norm", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1.0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":497
 * 
 * 
 * def kernel_norm(h, d, kernel, return_log=False):             # <<<<<<<<<<<<<<
 *     """Given a string specification of a kernel, compute the normalization.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_1kernel_norm(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7sklearn_9neighbors_7kd_tree_kernel_norm[] = "Given a string specification of a kernel, compute the normalization.\n\n    Parameters\n    ----------\n    h : float\n        the bandwidth of the kernel\n    d : int\n        the dimension of the space in which the kernel norm is computed\n    kernel : string\n        The kernel identifier.  Must be one of\n        ['gaussian'|'tophat'|'epanechnikov'|\n         'exponential'|'linear'|'cosine']\n    return_log : boolean\n        if True, return the log of the kernel norm.  Otherwise, return the\n        kernel norm.\n    Returns\n    -------\n    knorm or log_knorm : float\n        the kernel norm or logarithm of the kernel norm.\n    ";
static PyMethodDef __pyx_mdef_7sklearn_9neighbors_7kd_tree_1kernel_norm = {"kernel_norm", (PyCFunction)__pyx_pw_7sklearn_9neighbors_7kd_tree_1kernel_norm, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7sklearn_9neighbors_7kd_tree_kernel_norm};
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_1kernel_norm(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_h = 0;
  PyObject *__pyx_v_d = 0;
  PyObject *__pyx_v_kernel = 0;
  PyObject *__pyx_v_return_log = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("kernel_norm (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_h,&__pyx_n_s_d,&__pyx_n_s_kernel,&__pyx_n_s_return_log,0};
    PyObject* values[4] = {0,0,0,0};
    values[3] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_h)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("kernel_norm", 0, 3, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 497; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kernel)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("kernel_norm", 0, 3, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 497; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_return_log);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "kernel_norm") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 497; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_h = values[0];
    __pyx_v_d = values[1];
    __pyx_v_kernel = values[2];
    __pyx_v_return_log = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("kernel_norm", 0, 3, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 497; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.kernel_norm", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_kernel_norm(__pyx_self, __pyx_v_h, __pyx_v_d, __pyx_v_kernel, __pyx_v_return_log);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_kernel_norm(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_h, PyObject *__pyx_v_d, PyObject *__pyx_v_kernel, PyObject *__pyx_v_return_log) {
  PyObject *__pyx_v_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_3;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("kernel_norm", 0);

  /* "binary_tree.pxi":518
 *         the kernel norm or logarithm of the kernel norm.
 *     """
 *     if kernel == 'gaussian':             # <<<<<<<<<<<<<<
 *         result = _log_kernel_norm(h, d, GAUSSIAN_KERNEL)
 *     elif kernel == 'tophat':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_kernel, __pyx_n_s_gaussian, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 518; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_1) {

    /* "binary_tree.pxi":519
 *     """
 *     if kernel == 'gaussian':
 *         result = _log_kernel_norm(h, d, GAUSSIAN_KERNEL)             # <<<<<<<<<<<<<<
 *     elif kernel == 'tophat':
 *         result = _log_kernel_norm(h, d, TOPHAT_KERNEL)
 */
    __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_v_h); if (unlikely((__pyx_t_2 == (npy_float64)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 519; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = __Pyx_PyInt_As_Py_intptr_t(__pyx_v_d); if (unlikely((__pyx_t_3 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 519; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree__log_kernel_norm(__pyx_t_2, __pyx_t_3, __pyx_e_7sklearn_9neighbors_7kd_tree_GAUSSIAN_KERNEL); if (unlikely(__pyx_t_4 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 519; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_5 = PyFloat_FromDouble(__pyx_t_4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 519; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_v_result = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L3;
  }

  /* "binary_tree.pxi":520
 *     if kernel == 'gaussian':
 *         result = _log_kernel_norm(h, d, GAUSSIAN_KERNEL)
 *     elif kernel == 'tophat':             # <<<<<<<<<<<<<<
 *         result = _log_kernel_norm(h, d, TOPHAT_KERNEL)
 *     elif kernel == 'epanechnikov':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_kernel, __pyx_n_s_tophat, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 520; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_1) {

    /* "binary_tree.pxi":521
 *         result = _log_kernel_norm(h, d, GAUSSIAN_KERNEL)
 *     elif kernel == 'tophat':
 *         result = _log_kernel_norm(h, d, TOPHAT_KERNEL)             # <<<<<<<<<<<<<<
 *     elif kernel == 'epanechnikov':
 *         result = _log_kernel_norm(h, d, EPANECHNIKOV_KERNEL)
 */
    __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_h); if (unlikely((__pyx_t_4 == (npy_float64)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 521; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = __Pyx_PyInt_As_Py_intptr_t(__pyx_v_d); if (unlikely((__pyx_t_3 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 521; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_2 = __pyx_f_7sklearn_9neighbors_7kd_tree__log_kernel_norm(__pyx_t_4, __pyx_t_3, __pyx_e_7sklearn_9neighbors_7kd_tree_TOPHAT_KERNEL); if (unlikely(__pyx_t_2 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 521; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_5 = PyFloat_FromDouble(__pyx_t_2); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 521; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_v_result = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L3;
  }

  /* "binary_tree.pxi":522
 *     elif kernel == 'tophat':
 *         result = _log_kernel_norm(h, d, TOPHAT_KERNEL)
 *     elif kernel == 'epanechnikov':             # <<<<<<<<<<<<<<
 *         result = _log_kernel_norm(h, d, EPANECHNIKOV_KERNEL)
 *     elif kernel == 'exponential':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_kernel, __pyx_n_s_epanechnikov, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 522; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_1) {

    /* "binary_tree.pxi":523
 *         result = _log_kernel_norm(h, d, TOPHAT_KERNEL)
 *     elif kernel == 'epanechnikov':
 *         result = _log_kernel_norm(h, d, EPANECHNIKOV_KERNEL)             # <<<<<<<<<<<<<<
 *     elif kernel == 'exponential':
 *         result = _log_kernel_norm(h, d, EXPONENTIAL_KERNEL)
 */
    __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_v_h); if (unlikely((__pyx_t_2 == (npy_float64)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 523; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = __Pyx_PyInt_As_Py_intptr_t(__pyx_v_d); if (unlikely((__pyx_t_3 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 523; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree__log_kernel_norm(__pyx_t_2, __pyx_t_3, __pyx_e_7sklearn_9neighbors_7kd_tree_EPANECHNIKOV_KERNEL); if (unlikely(__pyx_t_4 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 523; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_5 = PyFloat_FromDouble(__pyx_t_4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 523; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_v_result = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L3;
  }

  /* "binary_tree.pxi":524
 *     elif kernel == 'epanechnikov':
 *         result = _log_kernel_norm(h, d, EPANECHNIKOV_KERNEL)
 *     elif kernel == 'exponential':             # <<<<<<<<<<<<<<
 *         result = _log_kernel_norm(h, d, EXPONENTIAL_KERNEL)
 *     elif kernel == 'linear':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_kernel, __pyx_n_s_exponential, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 524; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_1) {

    /* "binary_tree.pxi":525
 *         result = _log_kernel_norm(h, d, EPANECHNIKOV_KERNEL)
 *     elif kernel == 'exponential':
 *         result = _log_kernel_norm(h, d, EXPONENTIAL_KERNEL)             # <<<<<<<<<<<<<<
 *     elif kernel == 'linear':
 *         result = _log_kernel_norm(h, d, LINEAR_KERNEL)
 */
    __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_h); if (unlikely((__pyx_t_4 == (npy_float64)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 525; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = __Pyx_PyInt_As_Py_intptr_t(__pyx_v_d); if (unlikely((__pyx_t_3 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 525; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_2 = __pyx_f_7sklearn_9neighbors_7kd_tree__log_kernel_norm(__pyx_t_4, __pyx_t_3, __pyx_e_7sklearn_9neighbors_7kd_tree_EXPONENTIAL_KERNEL); if (unlikely(__pyx_t_2 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 525; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_5 = PyFloat_FromDouble(__pyx_t_2); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 525; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_v_result = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L3;
  }

  /* "binary_tree.pxi":526
 *     elif kernel == 'exponential':
 *         result = _log_kernel_norm(h, d, EXPONENTIAL_KERNEL)
 *     elif kernel == 'linear':             # <<<<<<<<<<<<<<
 *         result = _log_kernel_norm(h, d, LINEAR_KERNEL)
 *     elif kernel == 'cosine':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_kernel, __pyx_n_s_linear, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 526; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_1) {

    /* "binary_tree.pxi":527
 *         result = _log_kernel_norm(h, d, EXPONENTIAL_KERNEL)
 *     elif kernel == 'linear':
 *         result = _log_kernel_norm(h, d, LINEAR_KERNEL)             # <<<<<<<<<<<<<<
 *     elif kernel == 'cosine':
 *         result = _log_kernel_norm(h, d, COSINE_KERNEL)
 */
    __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_v_h); if (unlikely((__pyx_t_2 == (npy_float64)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 527; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = __Pyx_PyInt_As_Py_intptr_t(__pyx_v_d); if (unlikely((__pyx_t_3 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 527; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree__log_kernel_norm(__pyx_t_2, __pyx_t_3, __pyx_e_7sklearn_9neighbors_7kd_tree_LINEAR_KERNEL); if (unlikely(__pyx_t_4 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 527; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_5 = PyFloat_FromDouble(__pyx_t_4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 527; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_v_result = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L3;
  }

  /* "binary_tree.pxi":528
 *     elif kernel == 'linear':
 *         result = _log_kernel_norm(h, d, LINEAR_KERNEL)
 *     elif kernel == 'cosine':             # <<<<<<<<<<<<<<
 *         result = _log_kernel_norm(h, d, COSINE_KERNEL)
 *     else:
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_kernel, __pyx_n_s_cosine, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 528; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_1) {

    /* "binary_tree.pxi":529
 *         result = _log_kernel_norm(h, d, LINEAR_KERNEL)
 *     elif kernel == 'cosine':
 *         result = _log_kernel_norm(h, d, COSINE_KERNEL)             # <<<<<<<<<<<<<<
 *     else:
 *         raise ValueError('kernel not recognized')
 */
    __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_v_h); if (unlikely((__pyx_t_4 == (npy_float64)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = __Pyx_PyInt_As_Py_intptr_t(__pyx_v_d); if (unlikely((__pyx_t_3 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_2 = __pyx_f_7sklearn_9neighbors_7kd_tree__log_kernel_norm(__pyx_t_4, __pyx_t_3, __pyx_e_7sklearn_9neighbors_7kd_tree_COSINE_KERNEL); if (unlikely(__pyx_t_2 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_5 = PyFloat_FromDouble(__pyx_t_2); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_v_result = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "binary_tree.pxi":531
 *         result = _log_kernel_norm(h, d, COSINE_KERNEL)
 *     else:
 *         raise ValueError('kernel not recognized')             # <<<<<<<<<<<<<<
 * 
 *     if return_log:
 */
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L3:;

  /* "binary_tree.pxi":533
 *         raise ValueError('kernel not recognized')
 * 
 *     if return_log:             # <<<<<<<<<<<<<<
 *         return result
 *     else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_return_log); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 533; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_1) {

    /* "binary_tree.pxi":534
 * 
 *     if return_log:
 *         return result             # <<<<<<<<<<<<<<
 *     else:
 *         return np.exp(result)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_result);
    __pyx_r = __pyx_v_result;
    goto __pyx_L0;
  }
  /*else*/ {

    /* "binary_tree.pxi":536
 *         return result
 *     else:
 *         return np.exp(result)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 536; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_exp); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 536; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    if (!__pyx_t_6) {
      __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_v_result); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 536; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
    } else {
      __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 536; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __Pyx_GIVEREF(__pyx_t_6); __pyx_t_6 = NULL;
      __Pyx_INCREF(__pyx_v_result);
      PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_v_result);
      __Pyx_GIVEREF(__pyx_v_result);
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_8, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 536; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "binary_tree.pxi":497
 * 
 * 
 * def kernel_norm(h, d, kernel, return_log=False):             # <<<<<<<<<<<<<<
 *     """Given a string specification of a kernel, compute the normalization.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.kernel_norm", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":541
 * ######################################################################
 * # Tree Utility Routines
 * cdef inline void swap(DITYPE_t* arr, ITYPE_t i1, ITYPE_t i2):             # <<<<<<<<<<<<<<
 *     """swap the values at index i1 and i2 of arr"""
 *     cdef DITYPE_t tmp = arr[i1]
 */

static CYTHON_INLINE void __pyx_fuse_0__pyx_f_7sklearn_9neighbors_7kd_tree_swap(__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_v_arr, int __pyx_v_i1, int __pyx_v_i2) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_tmp;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fuse_0swap", 0);

  /* "binary_tree.pxi":543
 * cdef inline void swap(DITYPE_t* arr, ITYPE_t i1, ITYPE_t i2):
 *     """swap the values at index i1 and i2 of arr"""
 *     cdef DITYPE_t tmp = arr[i1]             # <<<<<<<<<<<<<<
 *     arr[i1] = arr[i2]
 *     arr[i2] = tmp
 */
  __pyx_v_tmp = (__pyx_v_arr[__pyx_v_i1]);

  /* "binary_tree.pxi":544
 *     """swap the values at index i1 and i2 of arr"""
 *     cdef DITYPE_t tmp = arr[i1]
 *     arr[i1] = arr[i2]             # <<<<<<<<<<<<<<
 *     arr[i2] = tmp
 * 
 */
  (__pyx_v_arr[__pyx_v_i1]) = (__pyx_v_arr[__pyx_v_i2]);

  /* "binary_tree.pxi":545
 *     cdef DITYPE_t tmp = arr[i1]
 *     arr[i1] = arr[i2]
 *     arr[i2] = tmp             # <<<<<<<<<<<<<<
 * 
 * 
 */
  (__pyx_v_arr[__pyx_v_i2]) = __pyx_v_tmp;

  /* "binary_tree.pxi":541
 * ######################################################################
 * # Tree Utility Routines
 * cdef inline void swap(DITYPE_t* arr, ITYPE_t i1, ITYPE_t i2):             # <<<<<<<<<<<<<<
 *     """swap the values at index i1 and i2 of arr"""
 *     cdef DITYPE_t tmp = arr[i1]
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "binary_tree.pxi":548
 * 
 * 
 * cdef inline void dual_swap(DTYPE_t* darr, ITYPE_t* iarr,             # <<<<<<<<<<<<<<
 *                            ITYPE_t i1, ITYPE_t i2):
 *     """swap the values at inex i1 and i2 of both darr and iarr"""
 */

static CYTHON_INLINE void __pyx_f_7sklearn_9neighbors_7kd_tree_dual_swap(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_darr, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_v_iarr, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i1, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i2) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dtmp;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_itmp;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("dual_swap", 0);

  /* "binary_tree.pxi":551
 *                            ITYPE_t i1, ITYPE_t i2):
 *     """swap the values at inex i1 and i2 of both darr and iarr"""
 *     cdef DTYPE_t dtmp = darr[i1]             # <<<<<<<<<<<<<<
 *     darr[i1] = darr[i2]
 *     darr[i2] = dtmp
 */
  __pyx_v_dtmp = (__pyx_v_darr[__pyx_v_i1]);

  /* "binary_tree.pxi":552
 *     """swap the values at inex i1 and i2 of both darr and iarr"""
 *     cdef DTYPE_t dtmp = darr[i1]
 *     darr[i1] = darr[i2]             # <<<<<<<<<<<<<<
 *     darr[i2] = dtmp
 * 
 */
  (__pyx_v_darr[__pyx_v_i1]) = (__pyx_v_darr[__pyx_v_i2]);

  /* "binary_tree.pxi":553
 *     cdef DTYPE_t dtmp = darr[i1]
 *     darr[i1] = darr[i2]
 *     darr[i2] = dtmp             # <<<<<<<<<<<<<<
 * 
 *     cdef ITYPE_t itmp = iarr[i1]
 */
  (__pyx_v_darr[__pyx_v_i2]) = __pyx_v_dtmp;

  /* "binary_tree.pxi":555
 *     darr[i2] = dtmp
 * 
 *     cdef ITYPE_t itmp = iarr[i1]             # <<<<<<<<<<<<<<
 *     iarr[i1] = iarr[i2]
 *     iarr[i2] = itmp
 */
  __pyx_v_itmp = (__pyx_v_iarr[__pyx_v_i1]);

  /* "binary_tree.pxi":556
 * 
 *     cdef ITYPE_t itmp = iarr[i1]
 *     iarr[i1] = iarr[i2]             # <<<<<<<<<<<<<<
 *     iarr[i2] = itmp
 * 
 */
  (__pyx_v_iarr[__pyx_v_i1]) = (__pyx_v_iarr[__pyx_v_i2]);

  /* "binary_tree.pxi":557
 *     cdef ITYPE_t itmp = iarr[i1]
 *     iarr[i1] = iarr[i2]
 *     iarr[i2] = itmp             # <<<<<<<<<<<<<<
 * 
 * 
 */
  (__pyx_v_iarr[__pyx_v_i2]) = __pyx_v_itmp;

  /* "binary_tree.pxi":548
 * 
 * 
 * cdef inline void dual_swap(DTYPE_t* darr, ITYPE_t* iarr,             # <<<<<<<<<<<<<<
 *                            ITYPE_t i1, ITYPE_t i2):
 *     """swap the values at inex i1 and i2 of both darr and iarr"""
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "binary_tree.pxi":581
 *     cdef ITYPE_t[:, ::1] indices
 * 
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *         self.distances_arr = np.zeros((1, 1), dtype=DTYPE, order='C')
 *         self.indices_arr = np.zeros((1, 1), dtype=ITYPE, order='C')
 */

/* Python wrapper */
static int __pyx_pw_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_13NeighborsHeap___cinit__(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7sklearn_9neighbors_7kd_tree_13NeighborsHeap___cinit__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_memviewslice __pyx_t_4 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "binary_tree.pxi":582
 * 
 *     def __cinit__(self):
 *         self.distances_arr = np.zeros((1, 1), dtype=DTYPE, order='C')             # <<<<<<<<<<<<<<
 *         self.indices_arr = np.zeros((1, 1), dtype=ITYPE, order='C')
 *         self.distances = get_memview_DTYPE_2D(self.distances_arr)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 582; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 582; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 582; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 582; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 582; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 582; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__4, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 582; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 582; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->distances_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->distances_arr));
  __pyx_v_self->distances_arr = ((PyArrayObject *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "binary_tree.pxi":583
 *     def __cinit__(self):
 *         self.distances_arr = np.zeros((1, 1), dtype=DTYPE, order='C')
 *         self.indices_arr = np.zeros((1, 1), dtype=ITYPE, order='C')             # <<<<<<<<<<<<<<
 *         self.distances = get_memview_DTYPE_2D(self.distances_arr)
 *         self.indices = get_memview_ITYPE_2D(self.indices_arr)
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__6, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->indices_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->indices_arr));
  __pyx_v_self->indices_arr = ((PyArrayObject *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "binary_tree.pxi":584
 *         self.distances_arr = np.zeros((1, 1), dtype=DTYPE, order='C')
 *         self.indices_arr = np.zeros((1, 1), dtype=ITYPE, order='C')
 *         self.distances = get_memview_DTYPE_2D(self.distances_arr)             # <<<<<<<<<<<<<<
 *         self.indices = get_memview_ITYPE_2D(self.indices_arr)
 * 
 */
  __pyx_t_2 = ((PyObject *)__pyx_v_self->distances_arr);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_2D(((PyArrayObject *)__pyx_t_2)); if (unlikely(!__pyx_t_4.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 584; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->distances, 0);
  __pyx_v_self->distances = __pyx_t_4;
  __pyx_t_4.memview = NULL;
  __pyx_t_4.data = NULL;

  /* "binary_tree.pxi":585
 *         self.indices_arr = np.zeros((1, 1), dtype=ITYPE, order='C')
 *         self.distances = get_memview_DTYPE_2D(self.distances_arr)
 *         self.indices = get_memview_ITYPE_2D(self.indices_arr)             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, n_pts, n_nbrs):
 */
  __pyx_t_2 = ((PyObject *)__pyx_v_self->indices_arr);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_5 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_ITYPE_2D(((PyArrayObject *)__pyx_t_2)); if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 585; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->indices, 0);
  __pyx_v_self->indices = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "binary_tree.pxi":581
 *     cdef ITYPE_t[:, ::1] indices
 * 
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *         self.distances_arr = np.zeros((1, 1), dtype=DTYPE, order='C')
 *         self.indices_arr = np.zeros((1, 1), dtype=ITYPE, order='C')
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __PYX_XDEC_MEMVIEW(&__pyx_t_4, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.NeighborsHeap.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":587
 *         self.indices = get_memview_ITYPE_2D(self.indices_arr)
 * 
 *     def __init__(self, n_pts, n_nbrs):             # <<<<<<<<<<<<<<
 *         self.distances_arr = np.inf + np.zeros((n_pts, n_nbrs), dtype=DTYPE,
 *                                                order='C')
 */

/* Python wrapper */
static int __pyx_pw_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_n_pts = 0;
  PyObject *__pyx_v_n_nbrs = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_n_pts,&__pyx_n_s_n_nbrs,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_n_pts)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_n_nbrs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 587; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 587; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_n_pts = values[0];
    __pyx_v_n_nbrs = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 587; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.NeighborsHeap.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_2__init__(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *)__pyx_v_self), __pyx_v_n_pts, __pyx_v_n_nbrs);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_2__init__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_self, PyObject *__pyx_v_n_pts, PyObject *__pyx_v_n_nbrs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "binary_tree.pxi":588
 * 
 *     def __init__(self, n_pts, n_nbrs):
 *         self.distances_arr = np.inf + np.zeros((n_pts, n_nbrs), dtype=DTYPE,             # <<<<<<<<<<<<<<
 *                                                order='C')
 *         self.indices_arr = np.zeros((n_pts, n_nbrs), dtype=ITYPE, order='C')
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_inf); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_n_pts);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_n_pts);
  __Pyx_GIVEREF(__pyx_v_n_pts);
  __Pyx_INCREF(__pyx_v_n_nbrs);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_n_nbrs);
  __Pyx_GIVEREF(__pyx_v_n_nbrs);
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, __pyx_t_1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Add(__pyx_t_2, __pyx_t_5); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->distances_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->distances_arr));
  __pyx_v_self->distances_arr = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "binary_tree.pxi":590
 *         self.distances_arr = np.inf + np.zeros((n_pts, n_nbrs), dtype=DTYPE,
 *                                                order='C')
 *         self.indices_arr = np.zeros((n_pts, n_nbrs), dtype=ITYPE, order='C')             # <<<<<<<<<<<<<<
 *         self.distances = get_memview_DTYPE_2D(self.distances_arr)
 *         self.indices = get_memview_ITYPE_2D(self.indices_arr)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 590; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 590; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 590; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_n_pts);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_n_pts);
  __Pyx_GIVEREF(__pyx_v_n_pts);
  __Pyx_INCREF(__pyx_v_n_nbrs);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_n_nbrs);
  __Pyx_GIVEREF(__pyx_v_n_nbrs);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 590; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 590; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 590; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 590; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 590; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 590; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 590; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->indices_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->indices_arr));
  __pyx_v_self->indices_arr = ((PyArrayObject *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "binary_tree.pxi":591
 *                                                order='C')
 *         self.indices_arr = np.zeros((n_pts, n_nbrs), dtype=ITYPE, order='C')
 *         self.distances = get_memview_DTYPE_2D(self.distances_arr)             # <<<<<<<<<<<<<<
 *         self.indices = get_memview_ITYPE_2D(self.indices_arr)
 * 
 */
  __pyx_t_4 = ((PyObject *)__pyx_v_self->distances_arr);
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_6 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_2D(((PyArrayObject *)__pyx_t_4)); if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->distances, 0);
  __pyx_v_self->distances = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "binary_tree.pxi":592
 *         self.indices_arr = np.zeros((n_pts, n_nbrs), dtype=ITYPE, order='C')
 *         self.distances = get_memview_DTYPE_2D(self.distances_arr)
 *         self.indices = get_memview_ITYPE_2D(self.indices_arr)             # <<<<<<<<<<<<<<
 * 
 *     def get_arrays(self, sort=True):
 */
  __pyx_t_4 = ((PyObject *)__pyx_v_self->indices_arr);
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_7 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_ITYPE_2D(((PyArrayObject *)__pyx_t_4)); if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->indices, 0);
  __pyx_v_self->indices = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "binary_tree.pxi":587
 *         self.indices = get_memview_ITYPE_2D(self.indices_arr)
 * 
 *     def __init__(self, n_pts, n_nbrs):             # <<<<<<<<<<<<<<
 *         self.distances_arr = np.inf + np.zeros((n_pts, n_nbrs), dtype=DTYPE,
 *                                                order='C')
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.NeighborsHeap.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":594
 *         self.indices = get_memview_ITYPE_2D(self.indices_arr)
 * 
 *     def get_arrays(self, sort=True):             # <<<<<<<<<<<<<<
 *         """Get the arrays of distances and indices within the heap.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_5get_arrays(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_4get_arrays[] = "Get the arrays of distances and indices within the heap.\n\n        If sort=True, then simultaneously sort the indices and distances,\n        so the closer points are listed first.\n        ";
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_5get_arrays(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_sort = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_arrays (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_sort,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sort);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_arrays") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 594; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_sort = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_arrays", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 594; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.NeighborsHeap.get_arrays", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_4get_arrays(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *)__pyx_v_self), __pyx_v_sort);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_4get_arrays(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_self, PyObject *__pyx_v_sort) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_arrays", 0);

  /* "binary_tree.pxi":600
 *         so the closer points are listed first.
 *         """
 *         if sort:             # <<<<<<<<<<<<<<
 *             self._sort()
 *         return self.distances_arr, self.indices_arr
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_sort); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 600; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_1) {

    /* "binary_tree.pxi":601
 *         """
 *         if sort:
 *             self._sort()             # <<<<<<<<<<<<<<
 *         return self.distances_arr, self.indices_arr
 * 
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NeighborsHeap *)__pyx_v_self->__pyx_vtab)->_sort(__pyx_v_self); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 601; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "binary_tree.pxi":602
 *         if sort:
 *             self._sort()
 *         return self.distances_arr, self.indices_arr             # <<<<<<<<<<<<<<
 * 
 *     cdef inline DTYPE_t largest(self, ITYPE_t row) nogil except -1:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 602; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->distances_arr));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_v_self->distances_arr));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->distances_arr));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->indices_arr));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->indices_arr));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->indices_arr));
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":594
 *         self.indices = get_memview_ITYPE_2D(self.indices_arr)
 * 
 *     def get_arrays(self, sort=True):             # <<<<<<<<<<<<<<
 *         """Get the arrays of distances and indices within the heap.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.NeighborsHeap.get_arrays", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":604
 *         return self.distances_arr, self.indices_arr
 * 
 *     cdef inline DTYPE_t largest(self, ITYPE_t row) nogil except -1:             # <<<<<<<<<<<<<<
 *         """Return the largest distance in the given row"""
 *         return self.distances[row, 0]
 */

static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_largest(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_row) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "binary_tree.pxi":606
 *     cdef inline DTYPE_t largest(self, ITYPE_t row) nogil except -1:
 *         """Return the largest distance in the given row"""
 *         return self.distances[row, 0]             # <<<<<<<<<<<<<<
 * 
 *     def push(self, ITYPE_t row, DTYPE_t val, ITYPE_t i_val):
 */
  if (unlikely(!__pyx_v_self->distances.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 606; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_1 = __pyx_v_row;
  __pyx_t_2 = 0;
  __pyx_r = (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_self->distances.data + __pyx_t_1 * __pyx_v_self->distances.strides[0]) )) + __pyx_t_2)) )));
  goto __pyx_L0;

  /* "binary_tree.pxi":604
 *         return self.distances_arr, self.indices_arr
 * 
 *     cdef inline DTYPE_t largest(self, ITYPE_t row) nogil except -1:             # <<<<<<<<<<<<<<
 *         """Return the largest distance in the given row"""
 *         return self.distances[row, 0]
 */

  /* function exit code */
  __pyx_L1_error:;
  {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
    #endif
    __Pyx_AddTraceback("sklearn.neighbors.kd_tree.NeighborsHeap.largest", __pyx_clineno, __pyx_lineno, __pyx_filename);
    #ifdef WITH_THREAD
    PyGILState_Release(__pyx_gilstate_save);
    #endif
  }
  __pyx_r = -1.0;
  __pyx_L0:;
  return __pyx_r;
}

/* "binary_tree.pxi":608
 *         return self.distances[row, 0]
 * 
 *     def push(self, ITYPE_t row, DTYPE_t val, ITYPE_t i_val):             # <<<<<<<<<<<<<<
 *         return self._push(row, val, i_val)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_7push(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_7push(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_row;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_val;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_val;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_row,&__pyx_n_s_val,&__pyx_n_s_i_val,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_row)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_val)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("push", 1, 3, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 608; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_i_val)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("push", 1, 3, 3, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 608; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "push") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 608; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_row = __Pyx_PyInt_As_Py_intptr_t(values[0]); if (unlikely((__pyx_v_row == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 608; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_val = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_val == (npy_float64)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 608; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_i_val = __Pyx_PyInt_As_Py_intptr_t(values[2]); if (unlikely((__pyx_v_i_val == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 608; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("push", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 608; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.NeighborsHeap.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_6push(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *)__pyx_v_self), __pyx_v_row, __pyx_v_val, __pyx_v_i_val);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_6push(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_row, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_val, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_val) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);

  /* "binary_tree.pxi":609
 * 
 *     def push(self, ITYPE_t row, DTYPE_t val, ITYPE_t i_val):
 *         return self._push(row, val, i_val)             # <<<<<<<<<<<<<<
 * 
 *     cdef int _push(self, ITYPE_t row, DTYPE_t val,
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NeighborsHeap *)__pyx_v_self->__pyx_vtab)->_push(__pyx_v_self, __pyx_v_row, __pyx_v_val, __pyx_v_i_val); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 609; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 609; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":608
 *         return self.distances[row, 0]
 * 
 *     def push(self, ITYPE_t row, DTYPE_t val, ITYPE_t i_val):             # <<<<<<<<<<<<<<
 *         return self._push(row, val, i_val)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.NeighborsHeap.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":611
 *         return self._push(row, val, i_val)
 * 
 *     cdef int _push(self, ITYPE_t row, DTYPE_t val,             # <<<<<<<<<<<<<<
 *                    ITYPE_t i_val) nogil except -1:
 *         """push (val, i_val) into the given row"""
 */

static int __pyx_f_7sklearn_9neighbors_7kd_tree_13NeighborsHeap__push(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_row, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_val, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_val) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_ic1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_ic2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_swap;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_size;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_dist_arr;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_v_ind_arr;
  int __pyx_r;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "binary_tree.pxi":615
 *         """push (val, i_val) into the given row"""
 *         cdef ITYPE_t i, ic1, ic2, i_swap
 *         cdef ITYPE_t size = self.distances.shape[1]             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t* dist_arr = &self.distances[row, 0]
 *         cdef ITYPE_t* ind_arr = &self.indices[row, 0]
 */
  if (unlikely(!__pyx_v_self->distances.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 615; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_size = (__pyx_v_self->distances.shape[1]);

  /* "binary_tree.pxi":616
 *         cdef ITYPE_t i, ic1, ic2, i_swap
 *         cdef ITYPE_t size = self.distances.shape[1]
 *         cdef DTYPE_t* dist_arr = &self.distances[row, 0]             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t* ind_arr = &self.indices[row, 0]
 * 
 */
  if (unlikely(!__pyx_v_self->distances.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 616; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_1 = __pyx_v_row;
  __pyx_t_2 = 0;
  __pyx_v_dist_arr = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_self->distances.data + __pyx_t_1 * __pyx_v_self->distances.strides[0]) )) + __pyx_t_2)) ))));

  /* "binary_tree.pxi":617
 *         cdef ITYPE_t size = self.distances.shape[1]
 *         cdef DTYPE_t* dist_arr = &self.distances[row, 0]
 *         cdef ITYPE_t* ind_arr = &self.indices[row, 0]             # <<<<<<<<<<<<<<
 * 
 *         # check if val should be in heap
 */
  if (unlikely(!__pyx_v_self->indices.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 617; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_3 = __pyx_v_row;
  __pyx_t_4 = 0;
  __pyx_v_ind_arr = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ (__pyx_v_self->indices.data + __pyx_t_3 * __pyx_v_self->indices.strides[0]) )) + __pyx_t_4)) ))));

  /* "binary_tree.pxi":620
 * 
 *         # check if val should be in heap
 *         if val > dist_arr[0]:             # <<<<<<<<<<<<<<
 *             return 0
 * 
 */
  __pyx_t_5 = ((__pyx_v_val > (__pyx_v_dist_arr[0])) != 0);
  if (__pyx_t_5) {

    /* "binary_tree.pxi":621
 *         # check if val should be in heap
 *         if val > dist_arr[0]:
 *             return 0             # <<<<<<<<<<<<<<
 * 
 *         # insert val at position zero
 */
    __pyx_r = 0;
    goto __pyx_L0;
  }

  /* "binary_tree.pxi":624
 * 
 *         # insert val at position zero
 *         dist_arr[0] = val             # <<<<<<<<<<<<<<
 *         ind_arr[0] = i_val
 * 
 */
  (__pyx_v_dist_arr[0]) = __pyx_v_val;

  /* "binary_tree.pxi":625
 *         # insert val at position zero
 *         dist_arr[0] = val
 *         ind_arr[0] = i_val             # <<<<<<<<<<<<<<
 * 
 *         #descend the heap, swapping values until the max heap criterion is met
 */
  (__pyx_v_ind_arr[0]) = __pyx_v_i_val;

  /* "binary_tree.pxi":628
 * 
 *         #descend the heap, swapping values until the max heap criterion is met
 *         i = 0             # <<<<<<<<<<<<<<
 *         while True:
 *             ic1 = 2 * i + 1
 */
  __pyx_v_i = 0;

  /* "binary_tree.pxi":629
 *         #descend the heap, swapping values until the max heap criterion is met
 *         i = 0
 *         while True:             # <<<<<<<<<<<<<<
 *             ic1 = 2 * i + 1
 *             ic2 = ic1 + 1
 */
  while (1) {

    /* "binary_tree.pxi":630
 *         i = 0
 *         while True:
 *             ic1 = 2 * i + 1             # <<<<<<<<<<<<<<
 *             ic2 = ic1 + 1
 * 
 */
    __pyx_v_ic1 = ((2 * __pyx_v_i) + 1);

    /* "binary_tree.pxi":631
 *         while True:
 *             ic1 = 2 * i + 1
 *             ic2 = ic1 + 1             # <<<<<<<<<<<<<<
 * 
 *             if ic1 >= size:
 */
    __pyx_v_ic2 = (__pyx_v_ic1 + 1);

    /* "binary_tree.pxi":633
 *             ic2 = ic1 + 1
 * 
 *             if ic1 >= size:             # <<<<<<<<<<<<<<
 *                 break
 *             elif ic2 >= size:
 */
    __pyx_t_5 = ((__pyx_v_ic1 >= __pyx_v_size) != 0);
    if (__pyx_t_5) {

      /* "binary_tree.pxi":634
 * 
 *             if ic1 >= size:
 *                 break             # <<<<<<<<<<<<<<
 *             elif ic2 >= size:
 *                 if dist_arr[ic1] > val:
 */
      goto __pyx_L5_break;
    }

    /* "binary_tree.pxi":635
 *             if ic1 >= size:
 *                 break
 *             elif ic2 >= size:             # <<<<<<<<<<<<<<
 *                 if dist_arr[ic1] > val:
 *                     i_swap = ic1
 */
    __pyx_t_5 = ((__pyx_v_ic2 >= __pyx_v_size) != 0);
    if (__pyx_t_5) {

      /* "binary_tree.pxi":636
 *                 break
 *             elif ic2 >= size:
 *                 if dist_arr[ic1] > val:             # <<<<<<<<<<<<<<
 *                     i_swap = ic1
 *                 else:
 */
      __pyx_t_5 = (((__pyx_v_dist_arr[__pyx_v_ic1]) > __pyx_v_val) != 0);
      if (__pyx_t_5) {

        /* "binary_tree.pxi":637
 *             elif ic2 >= size:
 *                 if dist_arr[ic1] > val:
 *                     i_swap = ic1             # <<<<<<<<<<<<<<
 *                 else:
 *                     break
 */
        __pyx_v_i_swap = __pyx_v_ic1;
        goto __pyx_L7;
      }
      /*else*/ {

        /* "binary_tree.pxi":639
 *                     i_swap = ic1
 *                 else:
 *                     break             # <<<<<<<<<<<<<<
 *             elif dist_arr[ic1] >= dist_arr[ic2]:
 *                 if val < dist_arr[ic1]:
 */
        goto __pyx_L5_break;
      }
      __pyx_L7:;
      goto __pyx_L6;
    }

    /* "binary_tree.pxi":640
 *                 else:
 *                     break
 *             elif dist_arr[ic1] >= dist_arr[ic2]:             # <<<<<<<<<<<<<<
 *                 if val < dist_arr[ic1]:
 *                     i_swap = ic1
 */
    __pyx_t_5 = (((__pyx_v_dist_arr[__pyx_v_ic1]) >= (__pyx_v_dist_arr[__pyx_v_ic2])) != 0);
    if (__pyx_t_5) {

      /* "binary_tree.pxi":641
 *                     break
 *             elif dist_arr[ic1] >= dist_arr[ic2]:
 *                 if val < dist_arr[ic1]:             # <<<<<<<<<<<<<<
 *                     i_swap = ic1
 *                 else:
 */
      __pyx_t_5 = ((__pyx_v_val < (__pyx_v_dist_arr[__pyx_v_ic1])) != 0);
      if (__pyx_t_5) {

        /* "binary_tree.pxi":642
 *             elif dist_arr[ic1] >= dist_arr[ic2]:
 *                 if val < dist_arr[ic1]:
 *                     i_swap = ic1             # <<<<<<<<<<<<<<
 *                 else:
 *                     break
 */
        __pyx_v_i_swap = __pyx_v_ic1;
        goto __pyx_L8;
      }
      /*else*/ {

        /* "binary_tree.pxi":644
 *                     i_swap = ic1
 *                 else:
 *                     break             # <<<<<<<<<<<<<<
 *             else:
 *                 if val < dist_arr[ic2]:
 */
        goto __pyx_L5_break;
      }
      __pyx_L8:;
      goto __pyx_L6;
    }
    /*else*/ {

      /* "binary_tree.pxi":646
 *                     break
 *             else:
 *                 if val < dist_arr[ic2]:             # <<<<<<<<<<<<<<
 *                     i_swap = ic2
 *                 else:
 */
      __pyx_t_5 = ((__pyx_v_val < (__pyx_v_dist_arr[__pyx_v_ic2])) != 0);
      if (__pyx_t_5) {

        /* "binary_tree.pxi":647
 *             else:
 *                 if val < dist_arr[ic2]:
 *                     i_swap = ic2             # <<<<<<<<<<<<<<
 *                 else:
 *                     break
 */
        __pyx_v_i_swap = __pyx_v_ic2;
        goto __pyx_L9;
      }
      /*else*/ {

        /* "binary_tree.pxi":649
 *                     i_swap = ic2
 *                 else:
 *                     break             # <<<<<<<<<<<<<<
 * 
 *             dist_arr[i] = dist_arr[i_swap]
 */
        goto __pyx_L5_break;
      }
      __pyx_L9:;
    }
    __pyx_L6:;

    /* "binary_tree.pxi":651
 *                     break
 * 
 *             dist_arr[i] = dist_arr[i_swap]             # <<<<<<<<<<<<<<
 *             ind_arr[i] = ind_arr[i_swap]
 * 
 */
    (__pyx_v_dist_arr[__pyx_v_i]) = (__pyx_v_dist_arr[__pyx_v_i_swap]);

    /* "binary_tree.pxi":652
 * 
 *             dist_arr[i] = dist_arr[i_swap]
 *             ind_arr[i] = ind_arr[i_swap]             # <<<<<<<<<<<<<<
 * 
 *             i = i_swap
 */
    (__pyx_v_ind_arr[__pyx_v_i]) = (__pyx_v_ind_arr[__pyx_v_i_swap]);

    /* "binary_tree.pxi":654
 *             ind_arr[i] = ind_arr[i_swap]
 * 
 *             i = i_swap             # <<<<<<<<<<<<<<
 * 
 *         dist_arr[i] = val
 */
    __pyx_v_i = __pyx_v_i_swap;
  }
  __pyx_L5_break:;

  /* "binary_tree.pxi":656
 *             i = i_swap
 * 
 *         dist_arr[i] = val             # <<<<<<<<<<<<<<
 *         ind_arr[i] = i_val
 * 
 */
  (__pyx_v_dist_arr[__pyx_v_i]) = __pyx_v_val;

  /* "binary_tree.pxi":657
 * 
 *         dist_arr[i] = val
 *         ind_arr[i] = i_val             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
  (__pyx_v_ind_arr[__pyx_v_i]) = __pyx_v_i_val;

  /* "binary_tree.pxi":659
 *         ind_arr[i] = i_val
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     cdef int _sort(self) except -1:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":611
 *         return self._push(row, val, i_val)
 * 
 *     cdef int _push(self, ITYPE_t row, DTYPE_t val,             # <<<<<<<<<<<<<<
 *                    ITYPE_t i_val) nogil except -1:
 *         """push (val, i_val) into the given row"""
 */

  /* function exit code */
  __pyx_L1_error:;
  {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
    #endif
    __Pyx_AddTraceback("sklearn.neighbors.kd_tree.NeighborsHeap._push", __pyx_clineno, __pyx_lineno, __pyx_filename);
    #ifdef WITH_THREAD
    PyGILState_Release(__pyx_gilstate_save);
    #endif
  }
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "binary_tree.pxi":661
 *         return 0
 * 
 *     cdef int _sort(self) except -1:             # <<<<<<<<<<<<<<
 *         """simultaneously sort the distances and indices"""
 *         cdef DTYPE_t[:, ::1] distances = self.distances
 */

static int __pyx_f_7sklearn_9neighbors_7kd_tree_13NeighborsHeap__sort(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_self) {
  __Pyx_memviewslice __pyx_v_distances = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_indices = { 0, 0, { 0 }, { 0 }, { 0 } };
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_row;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_memviewslice __pyx_t_1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_2 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_3;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_4;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_sort", 0);

  /* "binary_tree.pxi":663
 *     cdef int _sort(self) except -1:
 *         """simultaneously sort the distances and indices"""
 *         cdef DTYPE_t[:, ::1] distances = self.distances             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t[:, ::1] indices = self.indices
 *         cdef ITYPE_t row
 */
  if (unlikely(!__pyx_v_self->distances.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 663; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_1 = __pyx_v_self->distances;
  __PYX_INC_MEMVIEW(&__pyx_t_1, 1);
  __pyx_v_distances = __pyx_t_1;
  __pyx_t_1.memview = NULL;
  __pyx_t_1.data = NULL;

  /* "binary_tree.pxi":664
 *         """simultaneously sort the distances and indices"""
 *         cdef DTYPE_t[:, ::1] distances = self.distances
 *         cdef ITYPE_t[:, ::1] indices = self.indices             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t row
 *         for row in range(distances.shape[0]):
 */
  if (unlikely(!__pyx_v_self->indices.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 664; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_2 = __pyx_v_self->indices;
  __PYX_INC_MEMVIEW(&__pyx_t_2, 1);
  __pyx_v_indices = __pyx_t_2;
  __pyx_t_2.memview = NULL;
  __pyx_t_2.data = NULL;

  /* "binary_tree.pxi":666
 *         cdef ITYPE_t[:, ::1] indices = self.indices
 *         cdef ITYPE_t row
 *         for row in range(distances.shape[0]):             # <<<<<<<<<<<<<<
 *             _simultaneous_sort(&distances[row, 0],
 *                                &indices[row, 0],
 */
  __pyx_t_3 = (__pyx_v_distances.shape[0]);
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_row = __pyx_t_4;

    /* "binary_tree.pxi":667
 *         cdef ITYPE_t row
 *         for row in range(distances.shape[0]):
 *             _simultaneous_sort(&distances[row, 0],             # <<<<<<<<<<<<<<
 *                                &indices[row, 0],
 *                                distances.shape[1])
 */
    __pyx_t_5 = __pyx_v_row;
    __pyx_t_6 = 0;

    /* "binary_tree.pxi":668
 *         for row in range(distances.shape[0]):
 *             _simultaneous_sort(&distances[row, 0],
 *                                &indices[row, 0],             # <<<<<<<<<<<<<<
 *                                distances.shape[1])
 *         return 0
 */
    __pyx_t_7 = __pyx_v_row;
    __pyx_t_8 = 0;

    /* "binary_tree.pxi":667
 *         cdef ITYPE_t row
 *         for row in range(distances.shape[0]):
 *             _simultaneous_sort(&distances[row, 0],             # <<<<<<<<<<<<<<
 *                                &indices[row, 0],
 *                                distances.shape[1])
 */
    __pyx_t_9 = __pyx_f_7sklearn_9neighbors_7kd_tree__simultaneous_sort((&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_distances.data + __pyx_t_5 * __pyx_v_distances.strides[0]) )) + __pyx_t_6)) )))), (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ (__pyx_v_indices.data + __pyx_t_7 * __pyx_v_indices.strides[0]) )) + __pyx_t_8)) )))), (__pyx_v_distances.shape[1])); if (unlikely(__pyx_t_9 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 667; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "binary_tree.pxi":670
 *                                &indices[row, 0],
 *                                distances.shape[1])
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":661
 *         return 0
 * 
 *     cdef int _sort(self) except -1:             # <<<<<<<<<<<<<<
 *         """simultaneously sort the distances and indices"""
 *         cdef DTYPE_t[:, ::1] distances = self.distances
 */

  /* function exit code */
  __pyx_L1_error:;
  __PYX_XDEC_MEMVIEW(&__pyx_t_1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_2, 1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.NeighborsHeap._sort", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_distances, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_indices, 1);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":673
 * 
 * 
 * cdef int _simultaneous_sort(DTYPE_t* dist, ITYPE_t* idx,             # <<<<<<<<<<<<<<
 *                             ITYPE_t size) except -1:
 *     """
 */

static int __pyx_f_7sklearn_9neighbors_7kd_tree__simultaneous_sort(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_dist, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_v_idx, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_size) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_pivot_idx;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_store_idx;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_pivot_val;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  long __pyx_t_2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_simultaneous_sort", 0);

  /* "binary_tree.pxi":688
 * 
 *     # in the small-array case, do things efficiently
 *     if size <= 1:             # <<<<<<<<<<<<<<
 *         pass
 *     elif size == 2:
 */
  __pyx_t_1 = ((__pyx_v_size <= 1) != 0);
  if (__pyx_t_1) {
    goto __pyx_L3;
  }

  /* "binary_tree.pxi":690
 *     if size <= 1:
 *         pass
 *     elif size == 2:             # <<<<<<<<<<<<<<
 *         if dist[0] > dist[1]:
 *             dual_swap(dist, idx, 0, 1)
 */
  __pyx_t_1 = ((__pyx_v_size == 2) != 0);
  if (__pyx_t_1) {

    /* "binary_tree.pxi":691
 *         pass
 *     elif size == 2:
 *         if dist[0] > dist[1]:             # <<<<<<<<<<<<<<
 *             dual_swap(dist, idx, 0, 1)
 *     elif size == 3:
 */
    __pyx_t_1 = (((__pyx_v_dist[0]) > (__pyx_v_dist[1])) != 0);
    if (__pyx_t_1) {

      /* "binary_tree.pxi":692
 *     elif size == 2:
 *         if dist[0] > dist[1]:
 *             dual_swap(dist, idx, 0, 1)             # <<<<<<<<<<<<<<
 *     elif size == 3:
 *         if dist[0] > dist[1]:
 */
      __pyx_f_7sklearn_9neighbors_7kd_tree_dual_swap(__pyx_v_dist, __pyx_v_idx, 0, 1);
      goto __pyx_L4;
    }
    __pyx_L4:;
    goto __pyx_L3;
  }

  /* "binary_tree.pxi":693
 *         if dist[0] > dist[1]:
 *             dual_swap(dist, idx, 0, 1)
 *     elif size == 3:             # <<<<<<<<<<<<<<
 *         if dist[0] > dist[1]:
 *             dual_swap(dist, idx, 0, 1)
 */
  __pyx_t_1 = ((__pyx_v_size == 3) != 0);
  if (__pyx_t_1) {

    /* "binary_tree.pxi":694
 *             dual_swap(dist, idx, 0, 1)
 *     elif size == 3:
 *         if dist[0] > dist[1]:             # <<<<<<<<<<<<<<
 *             dual_swap(dist, idx, 0, 1)
 *         if dist[1] > dist[2]:
 */
    __pyx_t_1 = (((__pyx_v_dist[0]) > (__pyx_v_dist[1])) != 0);
    if (__pyx_t_1) {

      /* "binary_tree.pxi":695
 *     elif size == 3:
 *         if dist[0] > dist[1]:
 *             dual_swap(dist, idx, 0, 1)             # <<<<<<<<<<<<<<
 *         if dist[1] > dist[2]:
 *             dual_swap(dist, idx, 1, 2)
 */
      __pyx_f_7sklearn_9neighbors_7kd_tree_dual_swap(__pyx_v_dist, __pyx_v_idx, 0, 1);
      goto __pyx_L5;
    }
    __pyx_L5:;

    /* "binary_tree.pxi":696
 *         if dist[0] > dist[1]:
 *             dual_swap(dist, idx, 0, 1)
 *         if dist[1] > dist[2]:             # <<<<<<<<<<<<<<
 *             dual_swap(dist, idx, 1, 2)
 *             if dist[0] > dist[1]:
 */
    __pyx_t_1 = (((__pyx_v_dist[1]) > (__pyx_v_dist[2])) != 0);
    if (__pyx_t_1) {

      /* "binary_tree.pxi":697
 *             dual_swap(dist, idx, 0, 1)
 *         if dist[1] > dist[2]:
 *             dual_swap(dist, idx, 1, 2)             # <<<<<<<<<<<<<<
 *             if dist[0] > dist[1]:
 *                 dual_swap(dist, idx, 0, 1)
 */
      __pyx_f_7sklearn_9neighbors_7kd_tree_dual_swap(__pyx_v_dist, __pyx_v_idx, 1, 2);

      /* "binary_tree.pxi":698
 *         if dist[1] > dist[2]:
 *             dual_swap(dist, idx, 1, 2)
 *             if dist[0] > dist[1]:             # <<<<<<<<<<<<<<
 *                 dual_swap(dist, idx, 0, 1)
 *     else:
 */
      __pyx_t_1 = (((__pyx_v_dist[0]) > (__pyx_v_dist[1])) != 0);
      if (__pyx_t_1) {

        /* "binary_tree.pxi":699
 *             dual_swap(dist, idx, 1, 2)
 *             if dist[0] > dist[1]:
 *                 dual_swap(dist, idx, 0, 1)             # <<<<<<<<<<<<<<
 *     else:
 *         # Determine the pivot using the median-of-three rule.
 */
        __pyx_f_7sklearn_9neighbors_7kd_tree_dual_swap(__pyx_v_dist, __pyx_v_idx, 0, 1);
        goto __pyx_L7;
      }
      __pyx_L7:;
      goto __pyx_L6;
    }
    __pyx_L6:;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "binary_tree.pxi":705
 *         # the middle (the pivot value) is moved to the end, and the largest
 *         # is moved to the pivot index.
 *         pivot_idx = size / 2             # <<<<<<<<<<<<<<
 *         if dist[0] > dist[size - 1]:
 *             dual_swap(dist, idx, 0, size - 1)
 */
    __pyx_v_pivot_idx = (__pyx_v_size / 2);

    /* "binary_tree.pxi":706
 *         # is moved to the pivot index.
 *         pivot_idx = size / 2
 *         if dist[0] > dist[size - 1]:             # <<<<<<<<<<<<<<
 *             dual_swap(dist, idx, 0, size - 1)
 *         if dist[size - 1] > dist[pivot_idx]:
 */
    __pyx_t_1 = (((__pyx_v_dist[0]) > (__pyx_v_dist[(__pyx_v_size - 1)])) != 0);
    if (__pyx_t_1) {

      /* "binary_tree.pxi":707
 *         pivot_idx = size / 2
 *         if dist[0] > dist[size - 1]:
 *             dual_swap(dist, idx, 0, size - 1)             # <<<<<<<<<<<<<<
 *         if dist[size - 1] > dist[pivot_idx]:
 *             dual_swap(dist, idx, size - 1, pivot_idx)
 */
      __pyx_f_7sklearn_9neighbors_7kd_tree_dual_swap(__pyx_v_dist, __pyx_v_idx, 0, (__pyx_v_size - 1));
      goto __pyx_L8;
    }
    __pyx_L8:;

    /* "binary_tree.pxi":708
 *         if dist[0] > dist[size - 1]:
 *             dual_swap(dist, idx, 0, size - 1)
 *         if dist[size - 1] > dist[pivot_idx]:             # <<<<<<<<<<<<<<
 *             dual_swap(dist, idx, size - 1, pivot_idx)
 *             if dist[0] > dist[size - 1]:
 */
    __pyx_t_1 = (((__pyx_v_dist[(__pyx_v_size - 1)]) > (__pyx_v_dist[__pyx_v_pivot_idx])) != 0);
    if (__pyx_t_1) {

      /* "binary_tree.pxi":709
 *             dual_swap(dist, idx, 0, size - 1)
 *         if dist[size - 1] > dist[pivot_idx]:
 *             dual_swap(dist, idx, size - 1, pivot_idx)             # <<<<<<<<<<<<<<
 *             if dist[0] > dist[size - 1]:
 *                 dual_swap(dist, idx, 0, size - 1)
 */
      __pyx_f_7sklearn_9neighbors_7kd_tree_dual_swap(__pyx_v_dist, __pyx_v_idx, (__pyx_v_size - 1), __pyx_v_pivot_idx);

      /* "binary_tree.pxi":710
 *         if dist[size - 1] > dist[pivot_idx]:
 *             dual_swap(dist, idx, size - 1, pivot_idx)
 *             if dist[0] > dist[size - 1]:             # <<<<<<<<<<<<<<
 *                 dual_swap(dist, idx, 0, size - 1)
 *         pivot_val = dist[size - 1]
 */
      __pyx_t_1 = (((__pyx_v_dist[0]) > (__pyx_v_dist[(__pyx_v_size - 1)])) != 0);
      if (__pyx_t_1) {

        /* "binary_tree.pxi":711
 *             dual_swap(dist, idx, size - 1, pivot_idx)
 *             if dist[0] > dist[size - 1]:
 *                 dual_swap(dist, idx, 0, size - 1)             # <<<<<<<<<<<<<<
 *         pivot_val = dist[size - 1]
 * 
 */
        __pyx_f_7sklearn_9neighbors_7kd_tree_dual_swap(__pyx_v_dist, __pyx_v_idx, 0, (__pyx_v_size - 1));
        goto __pyx_L10;
      }
      __pyx_L10:;
      goto __pyx_L9;
    }
    __pyx_L9:;

    /* "binary_tree.pxi":712
 *             if dist[0] > dist[size - 1]:
 *                 dual_swap(dist, idx, 0, size - 1)
 *         pivot_val = dist[size - 1]             # <<<<<<<<<<<<<<
 * 
 *         # partition indices about pivot.  At the end of this operation,
 */
    __pyx_v_pivot_val = (__pyx_v_dist[(__pyx_v_size - 1)]);

    /* "binary_tree.pxi":717
 *         # pivot_idx will contain the pivot value, everything to the left
 *         # will be smaller, and everything to the right will be larger.
 *         store_idx = 0             # <<<<<<<<<<<<<<
 *         for i in range(size - 1):
 *             if dist[i] < pivot_val:
 */
    __pyx_v_store_idx = 0;

    /* "binary_tree.pxi":718
 *         # will be smaller, and everything to the right will be larger.
 *         store_idx = 0
 *         for i in range(size - 1):             # <<<<<<<<<<<<<<
 *             if dist[i] < pivot_val:
 *                 dual_swap(dist, idx, i, store_idx)
 */
    __pyx_t_2 = (__pyx_v_size - 1);
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "binary_tree.pxi":719
 *         store_idx = 0
 *         for i in range(size - 1):
 *             if dist[i] < pivot_val:             # <<<<<<<<<<<<<<
 *                 dual_swap(dist, idx, i, store_idx)
 *                 store_idx += 1
 */
      __pyx_t_1 = (((__pyx_v_dist[__pyx_v_i]) < __pyx_v_pivot_val) != 0);
      if (__pyx_t_1) {

        /* "binary_tree.pxi":720
 *         for i in range(size - 1):
 *             if dist[i] < pivot_val:
 *                 dual_swap(dist, idx, i, store_idx)             # <<<<<<<<<<<<<<
 *                 store_idx += 1
 *         dual_swap(dist, idx, store_idx, size - 1)
 */
        __pyx_f_7sklearn_9neighbors_7kd_tree_dual_swap(__pyx_v_dist, __pyx_v_idx, __pyx_v_i, __pyx_v_store_idx);

        /* "binary_tree.pxi":721
 *             if dist[i] < pivot_val:
 *                 dual_swap(dist, idx, i, store_idx)
 *                 store_idx += 1             # <<<<<<<<<<<<<<
 *         dual_swap(dist, idx, store_idx, size - 1)
 *         pivot_idx = store_idx
 */
        __pyx_v_store_idx = (__pyx_v_store_idx + 1);
        goto __pyx_L13;
      }
      __pyx_L13:;
    }

    /* "binary_tree.pxi":722
 *                 dual_swap(dist, idx, i, store_idx)
 *                 store_idx += 1
 *         dual_swap(dist, idx, store_idx, size - 1)             # <<<<<<<<<<<<<<
 *         pivot_idx = store_idx
 * 
 */
    __pyx_f_7sklearn_9neighbors_7kd_tree_dual_swap(__pyx_v_dist, __pyx_v_idx, __pyx_v_store_idx, (__pyx_v_size - 1));

    /* "binary_tree.pxi":723
 *                 store_idx += 1
 *         dual_swap(dist, idx, store_idx, size - 1)
 *         pivot_idx = store_idx             # <<<<<<<<<<<<<<
 * 
 *         # recursively sort each side of the pivot
 */
    __pyx_v_pivot_idx = __pyx_v_store_idx;

    /* "binary_tree.pxi":726
 * 
 *         # recursively sort each side of the pivot
 *         if pivot_idx > 1:             # <<<<<<<<<<<<<<
 *             _simultaneous_sort(dist, idx, pivot_idx)
 *         if pivot_idx + 2 < size:
 */
    __pyx_t_1 = ((__pyx_v_pivot_idx > 1) != 0);
    if (__pyx_t_1) {

      /* "binary_tree.pxi":727
 *         # recursively sort each side of the pivot
 *         if pivot_idx > 1:
 *             _simultaneous_sort(dist, idx, pivot_idx)             # <<<<<<<<<<<<<<
 *         if pivot_idx + 2 < size:
 *             _simultaneous_sort(dist + pivot_idx + 1,
 */
      __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree__simultaneous_sort(__pyx_v_dist, __pyx_v_idx, __pyx_v_pivot_idx); if (unlikely(__pyx_t_4 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 727; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      goto __pyx_L14;
    }
    __pyx_L14:;

    /* "binary_tree.pxi":728
 *         if pivot_idx > 1:
 *             _simultaneous_sort(dist, idx, pivot_idx)
 *         if pivot_idx + 2 < size:             # <<<<<<<<<<<<<<
 *             _simultaneous_sort(dist + pivot_idx + 1,
 *                                idx + pivot_idx + 1,
 */
    __pyx_t_1 = (((__pyx_v_pivot_idx + 2) < __pyx_v_size) != 0);
    if (__pyx_t_1) {

      /* "binary_tree.pxi":729
 *             _simultaneous_sort(dist, idx, pivot_idx)
 *         if pivot_idx + 2 < size:
 *             _simultaneous_sort(dist + pivot_idx + 1,             # <<<<<<<<<<<<<<
 *                                idx + pivot_idx + 1,
 *                                size - pivot_idx - 1)
 */
      __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree__simultaneous_sort(((__pyx_v_dist + __pyx_v_pivot_idx) + 1), ((__pyx_v_idx + __pyx_v_pivot_idx) + 1), ((__pyx_v_size - __pyx_v_pivot_idx) - 1)); if (unlikely(__pyx_t_4 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 729; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      goto __pyx_L15;
    }
    __pyx_L15:;
  }
  __pyx_L3:;

  /* "binary_tree.pxi":732
 *                                idx + pivot_idx + 1,
 *                                size - pivot_idx - 1)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * #------------------------------------------------------------
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":673
 * 
 * 
 * cdef int _simultaneous_sort(DTYPE_t* dist, ITYPE_t* idx,             # <<<<<<<<<<<<<<
 *                             ITYPE_t size) except -1:
 *     """
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree._simultaneous_sort", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":738
 * #  this computes the equivalent of
 * #  j_max = np.argmax(np.max(data, 0) - np.min(data, 0))
 * cdef ITYPE_t find_node_split_dim(DTYPE_t* data,             # <<<<<<<<<<<<<<
 *                                  ITYPE_t* node_indices,
 *                                  ITYPE_t n_features,
 */

static __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_find_node_split_dim(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_data, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_v_node_indices, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_features, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_points) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_min_val;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_max_val;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_val;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_spread;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_max_spread;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_j;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_j_max;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_3;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_4;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("find_node_split_dim", 0);

  /* "binary_tree.pxi":771
 *     cdef ITYPE_t i, j, j_max
 * 
 *     j_max = 0             # <<<<<<<<<<<<<<
 *     max_spread = 0
 * 
 */
  __pyx_v_j_max = 0;

  /* "binary_tree.pxi":772
 * 
 *     j_max = 0
 *     max_spread = 0             # <<<<<<<<<<<<<<
 * 
 *     for j in range(n_features):
 */
  __pyx_v_max_spread = 0.0;

  /* "binary_tree.pxi":774
 *     max_spread = 0
 * 
 *     for j in range(n_features):             # <<<<<<<<<<<<<<
 *         max_val = data[node_indices[0] * n_features + j]
 *         min_val = max_val
 */
  __pyx_t_1 = __pyx_v_n_features;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_j = __pyx_t_2;

    /* "binary_tree.pxi":775
 * 
 *     for j in range(n_features):
 *         max_val = data[node_indices[0] * n_features + j]             # <<<<<<<<<<<<<<
 *         min_val = max_val
 *         for i in range(1, n_points):
 */
    __pyx_v_max_val = (__pyx_v_data[(((__pyx_v_node_indices[0]) * __pyx_v_n_features) + __pyx_v_j)]);

    /* "binary_tree.pxi":776
 *     for j in range(n_features):
 *         max_val = data[node_indices[0] * n_features + j]
 *         min_val = max_val             # <<<<<<<<<<<<<<
 *         for i in range(1, n_points):
 *             val = data[node_indices[i] * n_features + j]
 */
    __pyx_v_min_val = __pyx_v_max_val;

    /* "binary_tree.pxi":777
 *         max_val = data[node_indices[0] * n_features + j]
 *         min_val = max_val
 *         for i in range(1, n_points):             # <<<<<<<<<<<<<<
 *             val = data[node_indices[i] * n_features + j]
 *             max_val = fmax(max_val, val)
 */
    __pyx_t_3 = __pyx_v_n_points;
    for (__pyx_t_4 = 1; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_i = __pyx_t_4;

      /* "binary_tree.pxi":778
 *         min_val = max_val
 *         for i in range(1, n_points):
 *             val = data[node_indices[i] * n_features + j]             # <<<<<<<<<<<<<<
 *             max_val = fmax(max_val, val)
 *             min_val = fmin(min_val, val)
 */
      __pyx_v_val = (__pyx_v_data[(((__pyx_v_node_indices[__pyx_v_i]) * __pyx_v_n_features) + __pyx_v_j)]);

      /* "binary_tree.pxi":779
 *         for i in range(1, n_points):
 *             val = data[node_indices[i] * n_features + j]
 *             max_val = fmax(max_val, val)             # <<<<<<<<<<<<<<
 *             min_val = fmin(min_val, val)
 *         spread = max_val - min_val
 */
      __pyx_v_max_val = __pyx_f_7sklearn_9neighbors_7kd_tree_fmax(__pyx_v_max_val, __pyx_v_val);

      /* "binary_tree.pxi":780
 *             val = data[node_indices[i] * n_features + j]
 *             max_val = fmax(max_val, val)
 *             min_val = fmin(min_val, val)             # <<<<<<<<<<<<<<
 *         spread = max_val - min_val
 *         if spread > max_spread:
 */
      __pyx_v_min_val = __pyx_f_7sklearn_9neighbors_7kd_tree_fmin(__pyx_v_min_val, __pyx_v_val);
    }

    /* "binary_tree.pxi":781
 *             max_val = fmax(max_val, val)
 *             min_val = fmin(min_val, val)
 *         spread = max_val - min_val             # <<<<<<<<<<<<<<
 *         if spread > max_spread:
 *             max_spread = spread
 */
    __pyx_v_spread = (__pyx_v_max_val - __pyx_v_min_val);

    /* "binary_tree.pxi":782
 *             min_val = fmin(min_val, val)
 *         spread = max_val - min_val
 *         if spread > max_spread:             # <<<<<<<<<<<<<<
 *             max_spread = spread
 *             j_max = j
 */
    __pyx_t_5 = ((__pyx_v_spread > __pyx_v_max_spread) != 0);
    if (__pyx_t_5) {

      /* "binary_tree.pxi":783
 *         spread = max_val - min_val
 *         if spread > max_spread:
 *             max_spread = spread             # <<<<<<<<<<<<<<
 *             j_max = j
 *     return j_max
 */
      __pyx_v_max_spread = __pyx_v_spread;

      /* "binary_tree.pxi":784
 *         if spread > max_spread:
 *             max_spread = spread
 *             j_max = j             # <<<<<<<<<<<<<<
 *     return j_max
 * 
 */
      __pyx_v_j_max = __pyx_v_j;
      goto __pyx_L7;
    }
    __pyx_L7:;
  }

  /* "binary_tree.pxi":785
 *             max_spread = spread
 *             j_max = j
 *     return j_max             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_j_max;
  goto __pyx_L0;

  /* "binary_tree.pxi":738
 * #  this computes the equivalent of
 * #  j_max = np.argmax(np.max(data, 0) - np.min(data, 0))
 * cdef ITYPE_t find_node_split_dim(DTYPE_t* data,             # <<<<<<<<<<<<<<
 *                                  ITYPE_t* node_indices,
 *                                  ITYPE_t n_features,
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":788
 * 
 * 
 * cdef int partition_node_indices(DTYPE_t* data,             # <<<<<<<<<<<<<<
 *                                 ITYPE_t* node_indices,
 *                                 ITYPE_t split_dim,
 */

static int __pyx_f_7sklearn_9neighbors_7kd_tree_partition_node_indices(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_data, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_v_node_indices, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_split_dim, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_split_index, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_features, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_points) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_left;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_right;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_midindex;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_d1;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_d2;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_2;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("partition_node_indices", 0);

  /* "binary_tree.pxi":833
 *     cdef ITYPE_t left, right, midindex, i
 *     cdef DTYPE_t d1, d2
 *     left = 0             # <<<<<<<<<<<<<<
 *     right = n_points - 1
 * 
 */
  __pyx_v_left = 0;

  /* "binary_tree.pxi":834
 *     cdef DTYPE_t d1, d2
 *     left = 0
 *     right = n_points - 1             # <<<<<<<<<<<<<<
 * 
 *     while True:
 */
  __pyx_v_right = (__pyx_v_n_points - 1);

  /* "binary_tree.pxi":836
 *     right = n_points - 1
 * 
 *     while True:             # <<<<<<<<<<<<<<
 *         midindex = left
 *         for i in range(left, right):
 */
  while (1) {

    /* "binary_tree.pxi":837
 * 
 *     while True:
 *         midindex = left             # <<<<<<<<<<<<<<
 *         for i in range(left, right):
 *             d1 = data[node_indices[i] * n_features + split_dim]
 */
    __pyx_v_midindex = __pyx_v_left;

    /* "binary_tree.pxi":838
 *     while True:
 *         midindex = left
 *         for i in range(left, right):             # <<<<<<<<<<<<<<
 *             d1 = data[node_indices[i] * n_features + split_dim]
 *             d2 = data[node_indices[right] * n_features + split_dim]
 */
    __pyx_t_1 = __pyx_v_right;
    for (__pyx_t_2 = __pyx_v_left; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
      __pyx_v_i = __pyx_t_2;

      /* "binary_tree.pxi":839
 *         midindex = left
 *         for i in range(left, right):
 *             d1 = data[node_indices[i] * n_features + split_dim]             # <<<<<<<<<<<<<<
 *             d2 = data[node_indices[right] * n_features + split_dim]
 *             if d1 < d2:
 */
      __pyx_v_d1 = (__pyx_v_data[(((__pyx_v_node_indices[__pyx_v_i]) * __pyx_v_n_features) + __pyx_v_split_dim)]);

      /* "binary_tree.pxi":840
 *         for i in range(left, right):
 *             d1 = data[node_indices[i] * n_features + split_dim]
 *             d2 = data[node_indices[right] * n_features + split_dim]             # <<<<<<<<<<<<<<
 *             if d1 < d2:
 *                 swap(node_indices, i, midindex)
 */
      __pyx_v_d2 = (__pyx_v_data[(((__pyx_v_node_indices[__pyx_v_right]) * __pyx_v_n_features) + __pyx_v_split_dim)]);

      /* "binary_tree.pxi":841
 *             d1 = data[node_indices[i] * n_features + split_dim]
 *             d2 = data[node_indices[right] * n_features + split_dim]
 *             if d1 < d2:             # <<<<<<<<<<<<<<
 *                 swap(node_indices, i, midindex)
 *                 midindex += 1
 */
      __pyx_t_3 = ((__pyx_v_d1 < __pyx_v_d2) != 0);
      if (__pyx_t_3) {

        /* "binary_tree.pxi":842
 *             d2 = data[node_indices[right] * n_features + split_dim]
 *             if d1 < d2:
 *                 swap(node_indices, i, midindex)             # <<<<<<<<<<<<<<
 *                 midindex += 1
 *         swap(node_indices, midindex, right)
 */
        __pyx_fuse_0__pyx_f_7sklearn_9neighbors_7kd_tree_swap(__pyx_v_node_indices, __pyx_v_i, __pyx_v_midindex);

        /* "binary_tree.pxi":843
 *             if d1 < d2:
 *                 swap(node_indices, i, midindex)
 *                 midindex += 1             # <<<<<<<<<<<<<<
 *         swap(node_indices, midindex, right)
 *         if midindex == split_index:
 */
        __pyx_v_midindex = (__pyx_v_midindex + 1);
        goto __pyx_L7;
      }
      __pyx_L7:;
    }

    /* "binary_tree.pxi":844
 *                 swap(node_indices, i, midindex)
 *                 midindex += 1
 *         swap(node_indices, midindex, right)             # <<<<<<<<<<<<<<
 *         if midindex == split_index:
 *             break
 */
    __pyx_fuse_0__pyx_f_7sklearn_9neighbors_7kd_tree_swap(__pyx_v_node_indices, __pyx_v_midindex, __pyx_v_right);

    /* "binary_tree.pxi":845
 *                 midindex += 1
 *         swap(node_indices, midindex, right)
 *         if midindex == split_index:             # <<<<<<<<<<<<<<
 *             break
 *         elif midindex < split_index:
 */
    __pyx_t_3 = ((__pyx_v_midindex == __pyx_v_split_index) != 0);
    if (__pyx_t_3) {

      /* "binary_tree.pxi":846
 *         swap(node_indices, midindex, right)
 *         if midindex == split_index:
 *             break             # <<<<<<<<<<<<<<
 *         elif midindex < split_index:
 *             left = midindex + 1
 */
      goto __pyx_L4_break;
    }

    /* "binary_tree.pxi":847
 *         if midindex == split_index:
 *             break
 *         elif midindex < split_index:             # <<<<<<<<<<<<<<
 *             left = midindex + 1
 *         else:
 */
    __pyx_t_3 = ((__pyx_v_midindex < __pyx_v_split_index) != 0);
    if (__pyx_t_3) {

      /* "binary_tree.pxi":848
 *             break
 *         elif midindex < split_index:
 *             left = midindex + 1             # <<<<<<<<<<<<<<
 *         else:
 *             right = midindex - 1
 */
      __pyx_v_left = (__pyx_v_midindex + 1);
      goto __pyx_L8;
    }
    /*else*/ {

      /* "binary_tree.pxi":850
 *             left = midindex + 1
 *         else:
 *             right = midindex - 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
      __pyx_v_right = (__pyx_v_midindex - 1);
    }
    __pyx_L8:;
  }
  __pyx_L4_break:;

  /* "binary_tree.pxi":852
 *             right = midindex - 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":788
 * 
 * 
 * cdef int partition_node_indices(DTYPE_t* data,             # <<<<<<<<<<<<<<
 *                                 ITYPE_t* node_indices,
 *                                 ITYPE_t split_dim,
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":858
 * # NodeHeap : min-heap used to keep track of nodes during
 * #            breadth-first query
 * cdef inline void swap_nodes(NodeHeapData_t* arr, ITYPE_t i1, ITYPE_t i2):             # <<<<<<<<<<<<<<
 *     cdef NodeHeapData_t tmp = arr[i1]
 *     arr[i1] = arr[i2]
 */

static CYTHON_INLINE void __pyx_f_7sklearn_9neighbors_7kd_tree_swap_nodes(struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t *__pyx_v_arr, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i1, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i2) {
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t __pyx_v_tmp;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("swap_nodes", 0);

  /* "binary_tree.pxi":859
 * #            breadth-first query
 * cdef inline void swap_nodes(NodeHeapData_t* arr, ITYPE_t i1, ITYPE_t i2):
 *     cdef NodeHeapData_t tmp = arr[i1]             # <<<<<<<<<<<<<<
 *     arr[i1] = arr[i2]
 *     arr[i2] = tmp
 */
  __pyx_v_tmp = (__pyx_v_arr[__pyx_v_i1]);

  /* "binary_tree.pxi":860
 * cdef inline void swap_nodes(NodeHeapData_t* arr, ITYPE_t i1, ITYPE_t i2):
 *     cdef NodeHeapData_t tmp = arr[i1]
 *     arr[i1] = arr[i2]             # <<<<<<<<<<<<<<
 *     arr[i2] = tmp
 * 
 */
  (__pyx_v_arr[__pyx_v_i1]) = (__pyx_v_arr[__pyx_v_i2]);

  /* "binary_tree.pxi":861
 *     cdef NodeHeapData_t tmp = arr[i1]
 *     arr[i1] = arr[i2]
 *     arr[i2] = tmp             # <<<<<<<<<<<<<<
 * 
 * 
 */
  (__pyx_v_arr[__pyx_v_i2]) = __pyx_v_tmp;

  /* "binary_tree.pxi":858
 * # NodeHeap : min-heap used to keep track of nodes during
 * #            breadth-first query
 * cdef inline void swap_nodes(NodeHeapData_t* arr, ITYPE_t i1, ITYPE_t i2):             # <<<<<<<<<<<<<<
 *     cdef NodeHeapData_t tmp = arr[i1]
 *     arr[i1] = arr[i2]
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "binary_tree.pxi":881
 *     cdef ITYPE_t n
 * 
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *         self.data_arr = np.zeros(1, dtype=NodeHeapData, order='C')
 *         self.data = get_memview_NodeHeapData_1D(self.data_arr)
 */

/* Python wrapper */
static int __pyx_pw_7sklearn_9neighbors_7kd_tree_8NodeHeap_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7sklearn_9neighbors_7kd_tree_8NodeHeap_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_8NodeHeap___cinit__(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7sklearn_9neighbors_7kd_tree_8NodeHeap___cinit__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_memviewslice __pyx_t_4 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "binary_tree.pxi":882
 * 
 *     def __cinit__(self):
 *         self.data_arr = np.zeros(1, dtype=NodeHeapData, order='C')             # <<<<<<<<<<<<<<
 *         self.data = get_memview_NodeHeapData_1D(self.data_arr)
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_NodeHeapData); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__7, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->data_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->data_arr));
  __pyx_v_self->data_arr = ((PyArrayObject *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "binary_tree.pxi":883
 *     def __cinit__(self):
 *         self.data_arr = np.zeros(1, dtype=NodeHeapData, order='C')
 *         self.data = get_memview_NodeHeapData_1D(self.data_arr)             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, size_guess=100):
 */
  __pyx_t_3 = ((PyObject *)__pyx_v_self->data_arr);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_NodeHeapData_1D(((PyArrayObject *)__pyx_t_3)); if (unlikely(!__pyx_t_4.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 883; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->data, 0);
  __pyx_v_self->data = __pyx_t_4;
  __pyx_t_4.memview = NULL;
  __pyx_t_4.data = NULL;

  /* "binary_tree.pxi":881
 *     cdef ITYPE_t n
 * 
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *         self.data_arr = np.zeros(1, dtype=NodeHeapData, order='C')
 *         self.data = get_memview_NodeHeapData_1D(self.data_arr)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __PYX_XDEC_MEMVIEW(&__pyx_t_4, 1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.NodeHeap.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":885
 *         self.data = get_memview_NodeHeapData_1D(self.data_arr)
 * 
 *     def __init__(self, size_guess=100):             # <<<<<<<<<<<<<<
 *         size_guess = max(size_guess, 1)  # need space for at least one item
 *         self.data_arr = np.zeros(size_guess, dtype=NodeHeapData, order='C')
 */

/* Python wrapper */
static int __pyx_pw_7sklearn_9neighbors_7kd_tree_8NodeHeap_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7sklearn_9neighbors_7kd_tree_8NodeHeap_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_size_guess = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_size_guess,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)__pyx_int_100);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size_guess);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 885; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_size_guess = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 885; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.NodeHeap.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_8NodeHeap_2__init__(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_v_self), __pyx_v_size_guess);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7sklearn_9neighbors_7kd_tree_8NodeHeap_2__init__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_self, PyObject *__pyx_v_size_guess) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);
  __Pyx_INCREF(__pyx_v_size_guess);

  /* "binary_tree.pxi":886
 * 
 *     def __init__(self, size_guess=100):
 *         size_guess = max(size_guess, 1)  # need space for at least one item             # <<<<<<<<<<<<<<
 *         self.data_arr = np.zeros(size_guess, dtype=NodeHeapData, order='C')
 *         self.data = get_memview_NodeHeapData_1D(self.data_arr)
 */
  __pyx_t_1 = 1;
  __Pyx_INCREF(__pyx_v_size_guess);
  __pyx_t_2 = __pyx_v_size_guess;
  __pyx_t_4 = __Pyx_PyInt_From_long(__pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 886; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_4, __pyx_t_2, Py_GT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 886; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 886; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (__pyx_t_6) {
    __pyx_t_5 = __Pyx_PyInt_From_long(__pyx_t_1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 886; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __pyx_t_5;
    __pyx_t_5 = 0;
  } else {
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = __pyx_t_2;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_t_3;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF_SET(__pyx_v_size_guess, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "binary_tree.pxi":887
 *     def __init__(self, size_guess=100):
 *         size_guess = max(size_guess, 1)  # need space for at least one item
 *         self.data_arr = np.zeros(size_guess, dtype=NodeHeapData, order='C')             # <<<<<<<<<<<<<<
 *         self.data = get_memview_NodeHeapData_1D(self.data_arr)
 *         self.n = size_guess
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 887; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_zeros); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 887; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 887; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_size_guess);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_size_guess);
  __Pyx_GIVEREF(__pyx_v_size_guess);
  __pyx_t_5 = PyDict_New(); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 887; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_NodeHeapData); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 887; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 887; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 887; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_2, __pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 887; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 887; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->data_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->data_arr));
  __pyx_v_self->data_arr = ((PyArrayObject *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "binary_tree.pxi":888
 *         size_guess = max(size_guess, 1)  # need space for at least one item
 *         self.data_arr = np.zeros(size_guess, dtype=NodeHeapData, order='C')
 *         self.data = get_memview_NodeHeapData_1D(self.data_arr)             # <<<<<<<<<<<<<<
 *         self.n = size_guess
 *         self.clear()
 */
  __pyx_t_4 = ((PyObject *)__pyx_v_self->data_arr);
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_7 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_NodeHeapData_1D(((PyArrayObject *)__pyx_t_4)); if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->data, 0);
  __pyx_v_self->data = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "binary_tree.pxi":889
 *         self.data_arr = np.zeros(size_guess, dtype=NodeHeapData, order='C')
 *         self.data = get_memview_NodeHeapData_1D(self.data_arr)
 *         self.n = size_guess             # <<<<<<<<<<<<<<
 *         self.clear()
 * 
 */
  __pyx_t_8 = __Pyx_PyInt_As_Py_intptr_t(__pyx_v_size_guess); if (unlikely((__pyx_t_8 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_self->n = __pyx_t_8;

  /* "binary_tree.pxi":890
 *         self.data = get_memview_NodeHeapData_1D(self.data_arr)
 *         self.n = size_guess
 *         self.clear()             # <<<<<<<<<<<<<<
 * 
 *     cdef int resize(self, ITYPE_t new_size) except -1:
 */
  ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_v_self->__pyx_vtab)->clear(__pyx_v_self);

  /* "binary_tree.pxi":885
 *         self.data = get_memview_NodeHeapData_1D(self.data_arr)
 * 
 *     def __init__(self, size_guess=100):             # <<<<<<<<<<<<<<
 *         size_guess = max(size_guess, 1)  # need space for at least one item
 *         self.data_arr = np.zeros(size_guess, dtype=NodeHeapData, order='C')
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.NodeHeap.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_size_guess);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":892
 *         self.clear()
 * 
 *     cdef int resize(self, ITYPE_t new_size) except -1:             # <<<<<<<<<<<<<<
 *         """Resize the heap to be either larger or smaller"""
 *         cdef NodeHeapData_t *data_ptr
 */

static int __pyx_f_7sklearn_9neighbors_7kd_tree_8NodeHeap_resize(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_new_size) {
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t *__pyx_v_data_ptr;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t *__pyx_v_new_data_ptr;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_size;
  PyArrayObject *__pyx_v_new_data_arr = 0;
  __Pyx_memviewslice __pyx_v_new_data = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_t_6;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_10;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_11;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("resize", 0);

  /* "binary_tree.pxi":897
 *         cdef NodeHeapData_t *new_data_ptr
 *         cdef ITYPE_t i
 *         cdef ITYPE_t size = self.data.shape[0]             # <<<<<<<<<<<<<<
 *         cdef np.ndarray new_data_arr = np.zeros(new_size,
 *                                                 dtype=NodeHeapData)
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 897; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_size = (__pyx_v_self->data.shape[0]);

  /* "binary_tree.pxi":898
 *         cdef ITYPE_t i
 *         cdef ITYPE_t size = self.data.shape[0]
 *         cdef np.ndarray new_data_arr = np.zeros(new_size,             # <<<<<<<<<<<<<<
 *                                                 dtype=NodeHeapData)
 *         cdef NodeHeapData_t[::1] new_data =\
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 898; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 898; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_new_size); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 898; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 898; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 898; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);

  /* "binary_tree.pxi":899
 *         cdef ITYPE_t size = self.data.shape[0]
 *         cdef np.ndarray new_data_arr = np.zeros(new_size,
 *                                                 dtype=NodeHeapData)             # <<<<<<<<<<<<<<
 *         cdef NodeHeapData_t[::1] new_data =\
 *                                     get_memview_NodeHeapData_1D(new_data_arr)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_NodeHeapData); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 899; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 898; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "binary_tree.pxi":898
 *         cdef ITYPE_t i
 *         cdef ITYPE_t size = self.data.shape[0]
 *         cdef np.ndarray new_data_arr = np.zeros(new_size,             # <<<<<<<<<<<<<<
 *                                                 dtype=NodeHeapData)
 *         cdef NodeHeapData_t[::1] new_data =\
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 898; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 898; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_new_data_arr = ((PyArrayObject *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "binary_tree.pxi":901
 *                                                 dtype=NodeHeapData)
 *         cdef NodeHeapData_t[::1] new_data =\
 *                                     get_memview_NodeHeapData_1D(new_data_arr)             # <<<<<<<<<<<<<<
 * 
 *         if size > 0 and new_size > 0:
 */
  __pyx_t_5 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_NodeHeapData_1D(((PyArrayObject *)__pyx_v_new_data_arr)); if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 901; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_new_data = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "binary_tree.pxi":903
 *                                     get_memview_NodeHeapData_1D(new_data_arr)
 * 
 *         if size > 0 and new_size > 0:             # <<<<<<<<<<<<<<
 *             data_ptr = &self.data[0]
 *             new_data_ptr = &new_data[0]
 */
  __pyx_t_7 = ((__pyx_v_size > 0) != 0);
  if (__pyx_t_7) {
  } else {
    __pyx_t_6 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_7 = ((__pyx_v_new_size > 0) != 0);
  __pyx_t_6 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_6) {

    /* "binary_tree.pxi":904
 * 
 *         if size > 0 and new_size > 0:
 *             data_ptr = &self.data[0]             # <<<<<<<<<<<<<<
 *             new_data_ptr = &new_data[0]
 *             for i in range(min(size, new_size)):
 */
    if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 904; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
    __pyx_t_8 = 0;
    __pyx_v_data_ptr = (&(*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t *) __pyx_v_self->data.data) + __pyx_t_8)) ))));

    /* "binary_tree.pxi":905
 *         if size > 0 and new_size > 0:
 *             data_ptr = &self.data[0]
 *             new_data_ptr = &new_data[0]             # <<<<<<<<<<<<<<
 *             for i in range(min(size, new_size)):
 *                 new_data_ptr[i] = data_ptr[i]
 */
    __pyx_t_9 = 0;
    __pyx_v_new_data_ptr = (&(*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t *) __pyx_v_new_data.data) + __pyx_t_9)) ))));

    /* "binary_tree.pxi":906
 *             data_ptr = &self.data[0]
 *             new_data_ptr = &new_data[0]
 *             for i in range(min(size, new_size)):             # <<<<<<<<<<<<<<
 *                 new_data_ptr[i] = data_ptr[i]
 * 
 */
    __pyx_t_10 = __pyx_v_new_size;
    __pyx_t_11 = __pyx_v_size;
    if (((__pyx_t_10 < __pyx_t_11) != 0)) {
      __pyx_t_12 = __pyx_t_10;
    } else {
      __pyx_t_12 = __pyx_t_11;
    }
    __pyx_t_10 = __pyx_t_12;
    for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_10; __pyx_t_12+=1) {
      __pyx_v_i = __pyx_t_12;

      /* "binary_tree.pxi":907
 *             new_data_ptr = &new_data[0]
 *             for i in range(min(size, new_size)):
 *                 new_data_ptr[i] = data_ptr[i]             # <<<<<<<<<<<<<<
 * 
 *         if new_size < size:
 */
      (__pyx_v_new_data_ptr[__pyx_v_i]) = (__pyx_v_data_ptr[__pyx_v_i]);
    }
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "binary_tree.pxi":909
 *                 new_data_ptr[i] = data_ptr[i]
 * 
 *         if new_size < size:             # <<<<<<<<<<<<<<
 *             self.n = new_size
 * 
 */
  __pyx_t_6 = ((__pyx_v_new_size < __pyx_v_size) != 0);
  if (__pyx_t_6) {

    /* "binary_tree.pxi":910
 * 
 *         if new_size < size:
 *             self.n = new_size             # <<<<<<<<<<<<<<
 * 
 *         self.data = new_data
 */
    __pyx_v_self->n = __pyx_v_new_size;
    goto __pyx_L8;
  }
  __pyx_L8:;

  /* "binary_tree.pxi":912
 *             self.n = new_size
 * 
 *         self.data = new_data             # <<<<<<<<<<<<<<
 *         self.data_arr = new_data_arr
 *         return 0
 */
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->data, 0);
  __PYX_INC_MEMVIEW(&__pyx_v_new_data, 0);
  __pyx_v_self->data = __pyx_v_new_data;

  /* "binary_tree.pxi":913
 * 
 *         self.data = new_data
 *         self.data_arr = new_data_arr             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_new_data_arr));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_new_data_arr));
  __Pyx_GOTREF(__pyx_v_self->data_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->data_arr));
  __pyx_v_self->data_arr = __pyx_v_new_data_arr;

  /* "binary_tree.pxi":914
 *         self.data = new_data
 *         self.data_arr = new_data_arr
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     cdef int push(self, NodeHeapData_t data) except -1:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":892
 *         self.clear()
 * 
 *     cdef int resize(self, ITYPE_t new_size) except -1:             # <<<<<<<<<<<<<<
 *         """Resize the heap to be either larger or smaller"""
 *         cdef NodeHeapData_t *data_ptr
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.NodeHeap.resize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_new_data_arr);
  __PYX_XDEC_MEMVIEW(&__pyx_v_new_data, 1);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":916
 *         return 0
 * 
 *     cdef int push(self, NodeHeapData_t data) except -1:             # <<<<<<<<<<<<<<
 *         """Push a new item onto the heap"""
 *         cdef ITYPE_t i, i_parent
 */

static int __pyx_f_7sklearn_9neighbors_7kd_tree_8NodeHeap_push(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_self, struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t __pyx_v_data) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_parent;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t *__pyx_v_data_arr;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);

  /* "binary_tree.pxi":920
 *         cdef ITYPE_t i, i_parent
 *         cdef NodeHeapData_t* data_arr
 *         self.n += 1             # <<<<<<<<<<<<<<
 *         if self.n > self.data.shape[0]:
 *             self.resize(2 * self.n)
 */
  __pyx_v_self->n = (__pyx_v_self->n + 1);

  /* "binary_tree.pxi":921
 *         cdef NodeHeapData_t* data_arr
 *         self.n += 1
 *         if self.n > self.data.shape[0]:             # <<<<<<<<<<<<<<
 *             self.resize(2 * self.n)
 * 
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 921; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_1 = ((__pyx_v_self->n > (__pyx_v_self->data.shape[0])) != 0);
  if (__pyx_t_1) {

    /* "binary_tree.pxi":922
 *         self.n += 1
 *         if self.n > self.data.shape[0]:
 *             self.resize(2 * self.n)             # <<<<<<<<<<<<<<
 * 
 *         # put the new element at the end,
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_v_self->__pyx_vtab)->resize(__pyx_v_self, (2 * __pyx_v_self->n)); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 922; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "binary_tree.pxi":926
 *         # put the new element at the end,
 *         # and then perform swaps until the heap is in order
 *         data_arr = &self.data[0]             # <<<<<<<<<<<<<<
 *         i = self.n - 1
 *         data_arr[i] = data
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_3 = 0;
  __pyx_v_data_arr = (&(*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t *) __pyx_v_self->data.data) + __pyx_t_3)) ))));

  /* "binary_tree.pxi":927
 *         # and then perform swaps until the heap is in order
 *         data_arr = &self.data[0]
 *         i = self.n - 1             # <<<<<<<<<<<<<<
 *         data_arr[i] = data
 * 
 */
  __pyx_v_i = (__pyx_v_self->n - 1);

  /* "binary_tree.pxi":928
 *         data_arr = &self.data[0]
 *         i = self.n - 1
 *         data_arr[i] = data             # <<<<<<<<<<<<<<
 * 
 *         while i > 0:
 */
  (__pyx_v_data_arr[__pyx_v_i]) = __pyx_v_data;

  /* "binary_tree.pxi":930
 *         data_arr[i] = data
 * 
 *         while i > 0:             # <<<<<<<<<<<<<<
 *             i_parent = (i - 1) // 2
 *             if data_arr[i_parent].val <= data_arr[i].val:
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_i > 0) != 0);
    if (!__pyx_t_1) break;

    /* "binary_tree.pxi":931
 * 
 *         while i > 0:
 *             i_parent = (i - 1) // 2             # <<<<<<<<<<<<<<
 *             if data_arr[i_parent].val <= data_arr[i].val:
 *                 break
 */
    __pyx_v_i_parent = ((__pyx_v_i - 1) / 2);

    /* "binary_tree.pxi":932
 *         while i > 0:
 *             i_parent = (i - 1) // 2
 *             if data_arr[i_parent].val <= data_arr[i].val:             # <<<<<<<<<<<<<<
 *                 break
 *             else:
 */
    __pyx_t_1 = (((__pyx_v_data_arr[__pyx_v_i_parent]).val <= (__pyx_v_data_arr[__pyx_v_i]).val) != 0);
    if (__pyx_t_1) {

      /* "binary_tree.pxi":933
 *             i_parent = (i - 1) // 2
 *             if data_arr[i_parent].val <= data_arr[i].val:
 *                 break             # <<<<<<<<<<<<<<
 *             else:
 *                 swap_nodes(data_arr, i, i_parent)
 */
      goto __pyx_L5_break;
    }
    /*else*/ {

      /* "binary_tree.pxi":935
 *                 break
 *             else:
 *                 swap_nodes(data_arr, i, i_parent)             # <<<<<<<<<<<<<<
 *                 i = i_parent
 *         return 0
 */
      __pyx_f_7sklearn_9neighbors_7kd_tree_swap_nodes(__pyx_v_data_arr, __pyx_v_i, __pyx_v_i_parent);

      /* "binary_tree.pxi":936
 *             else:
 *                 swap_nodes(data_arr, i, i_parent)
 *                 i = i_parent             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
      __pyx_v_i = __pyx_v_i_parent;
    }
  }
  __pyx_L5_break:;

  /* "binary_tree.pxi":937
 *                 swap_nodes(data_arr, i, i_parent)
 *                 i = i_parent
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     cdef NodeHeapData_t peek(self):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":916
 *         return 0
 * 
 *     cdef int push(self, NodeHeapData_t data) except -1:             # <<<<<<<<<<<<<<
 *         """Push a new item onto the heap"""
 *         cdef ITYPE_t i, i_parent
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.NodeHeap.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":939
 *         return 0
 * 
 *     cdef NodeHeapData_t peek(self):             # <<<<<<<<<<<<<<
 *         """Peek at the root of the heap, without removing it"""
 *         return self.data[0]
 */

static struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t __pyx_f_7sklearn_9neighbors_7kd_tree_8NodeHeap_peek(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_self) {
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("peek", 0);

  /* "binary_tree.pxi":941
 *     cdef NodeHeapData_t peek(self):
 *         """Peek at the root of the heap, without removing it"""
 *         return self.data[0]             # <<<<<<<<<<<<<<
 * 
 *     cdef NodeHeapData_t pop(self):
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 941; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_1 = 0;
  __pyx_r = (*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t *) __pyx_v_self->data.data) + __pyx_t_1)) )));
  goto __pyx_L0;

  /* "binary_tree.pxi":939
 *         return 0
 * 
 *     cdef NodeHeapData_t peek(self):             # <<<<<<<<<<<<<<
 *         """Peek at the root of the heap, without removing it"""
 *         return self.data[0]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("sklearn.neighbors.kd_tree.NodeHeap.peek", __pyx_clineno, __pyx_lineno, __pyx_filename, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":943
 *         return self.data[0]
 * 
 *     cdef NodeHeapData_t pop(self):             # <<<<<<<<<<<<<<
 *         """Remove the root of the heap, and update the remaining nodes"""
 *         if self.n == 0:
 */

static struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t __pyx_f_7sklearn_9neighbors_7kd_tree_8NodeHeap_pop(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_self) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_child1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_child2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_swap;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t *__pyx_v_data_arr;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t __pyx_v_popped_element;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pop", 0);

  /* "binary_tree.pxi":945
 *     cdef NodeHeapData_t pop(self):
 *         """Remove the root of the heap, and update the remaining nodes"""
 *         if self.n == 0:             # <<<<<<<<<<<<<<
 *             raise ValueError('cannot pop on empty heap')
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->n == 0) != 0);
  if (__pyx_t_1) {

    /* "binary_tree.pxi":946
 *         """Remove the root of the heap, and update the remaining nodes"""
 *         if self.n == 0:
 *             raise ValueError('cannot pop on empty heap')             # <<<<<<<<<<<<<<
 * 
 *         cdef ITYPE_t i, i_child1, i_child2, i_swap
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "binary_tree.pxi":949
 * 
 *         cdef ITYPE_t i, i_child1, i_child2, i_swap
 *         cdef NodeHeapData_t* data_arr = &self.data[0]             # <<<<<<<<<<<<<<
 *         cdef NodeHeapData_t popped_element = data_arr[0]
 * 
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 949; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_3 = 0;
  __pyx_v_data_arr = (&(*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t *) __pyx_v_self->data.data) + __pyx_t_3)) ))));

  /* "binary_tree.pxi":950
 *         cdef ITYPE_t i, i_child1, i_child2, i_swap
 *         cdef NodeHeapData_t* data_arr = &self.data[0]
 *         cdef NodeHeapData_t popped_element = data_arr[0]             # <<<<<<<<<<<<<<
 * 
 *         # pop off the first element, move the last element to the front,
 */
  __pyx_v_popped_element = (__pyx_v_data_arr[0]);

  /* "binary_tree.pxi":954
 *         # pop off the first element, move the last element to the front,
 *         # and then perform swaps until the heap is back in order
 *         data_arr[0] = data_arr[self.n - 1]             # <<<<<<<<<<<<<<
 *         self.n -= 1
 * 
 */
  (__pyx_v_data_arr[0]) = (__pyx_v_data_arr[(__pyx_v_self->n - 1)]);

  /* "binary_tree.pxi":955
 *         # and then perform swaps until the heap is back in order
 *         data_arr[0] = data_arr[self.n - 1]
 *         self.n -= 1             # <<<<<<<<<<<<<<
 * 
 *         i = 0
 */
  __pyx_v_self->n = (__pyx_v_self->n - 1);

  /* "binary_tree.pxi":957
 *         self.n -= 1
 * 
 *         i = 0             # <<<<<<<<<<<<<<
 * 
 *         while (i < self.n):
 */
  __pyx_v_i = 0;

  /* "binary_tree.pxi":959
 *         i = 0
 * 
 *         while (i < self.n):             # <<<<<<<<<<<<<<
 *             i_child1 = 2 * i + 1
 *             i_child2 = 2 * i + 2
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_i < __pyx_v_self->n) != 0);
    if (!__pyx_t_1) break;

    /* "binary_tree.pxi":960
 * 
 *         while (i < self.n):
 *             i_child1 = 2 * i + 1             # <<<<<<<<<<<<<<
 *             i_child2 = 2 * i + 2
 *             i_swap = 0
 */
    __pyx_v_i_child1 = ((2 * __pyx_v_i) + 1);

    /* "binary_tree.pxi":961
 *         while (i < self.n):
 *             i_child1 = 2 * i + 1
 *             i_child2 = 2 * i + 2             # <<<<<<<<<<<<<<
 *             i_swap = 0
 * 
 */
    __pyx_v_i_child2 = ((2 * __pyx_v_i) + 2);

    /* "binary_tree.pxi":962
 *             i_child1 = 2 * i + 1
 *             i_child2 = 2 * i + 2
 *             i_swap = 0             # <<<<<<<<<<<<<<
 * 
 *             if i_child2 < self.n:
 */
    __pyx_v_i_swap = 0;

    /* "binary_tree.pxi":964
 *             i_swap = 0
 * 
 *             if i_child2 < self.n:             # <<<<<<<<<<<<<<
 *                 if data_arr[i_child1].val <= data_arr[i_child2].val:
 *                     i_swap = i_child1
 */
    __pyx_t_1 = ((__pyx_v_i_child2 < __pyx_v_self->n) != 0);
    if (__pyx_t_1) {

      /* "binary_tree.pxi":965
 * 
 *             if i_child2 < self.n:
 *                 if data_arr[i_child1].val <= data_arr[i_child2].val:             # <<<<<<<<<<<<<<
 *                     i_swap = i_child1
 *                 else:
 */
      __pyx_t_1 = (((__pyx_v_data_arr[__pyx_v_i_child1]).val <= (__pyx_v_data_arr[__pyx_v_i_child2]).val) != 0);
      if (__pyx_t_1) {

        /* "binary_tree.pxi":966
 *             if i_child2 < self.n:
 *                 if data_arr[i_child1].val <= data_arr[i_child2].val:
 *                     i_swap = i_child1             # <<<<<<<<<<<<<<
 *                 else:
 *                     i_swap = i_child2
 */
        __pyx_v_i_swap = __pyx_v_i_child1;
        goto __pyx_L7;
      }
      /*else*/ {

        /* "binary_tree.pxi":968
 *                     i_swap = i_child1
 *                 else:
 *                     i_swap = i_child2             # <<<<<<<<<<<<<<
 *             elif i_child1 < self.n:
 *                 i_swap = i_child1
 */
        __pyx_v_i_swap = __pyx_v_i_child2;
      }
      __pyx_L7:;
      goto __pyx_L6;
    }

    /* "binary_tree.pxi":969
 *                 else:
 *                     i_swap = i_child2
 *             elif i_child1 < self.n:             # <<<<<<<<<<<<<<
 *                 i_swap = i_child1
 *             else:
 */
    __pyx_t_1 = ((__pyx_v_i_child1 < __pyx_v_self->n) != 0);
    if (__pyx_t_1) {

      /* "binary_tree.pxi":970
 *                     i_swap = i_child2
 *             elif i_child1 < self.n:
 *                 i_swap = i_child1             # <<<<<<<<<<<<<<
 *             else:
 *                 break
 */
      __pyx_v_i_swap = __pyx_v_i_child1;
      goto __pyx_L6;
    }
    /*else*/ {

      /* "binary_tree.pxi":972
 *                 i_swap = i_child1
 *             else:
 *                 break             # <<<<<<<<<<<<<<
 * 
 *             if (i_swap > 0) and (data_arr[i_swap].val <= data_arr[i].val):
 */
      goto __pyx_L5_break;
    }
    __pyx_L6:;

    /* "binary_tree.pxi":974
 *                 break
 * 
 *             if (i_swap > 0) and (data_arr[i_swap].val <= data_arr[i].val):             # <<<<<<<<<<<<<<
 *                 swap_nodes(data_arr, i, i_swap)
 *                 i = i_swap
 */
    __pyx_t_4 = ((__pyx_v_i_swap > 0) != 0);
    if (__pyx_t_4) {
    } else {
      __pyx_t_1 = __pyx_t_4;
      goto __pyx_L9_bool_binop_done;
    }
    __pyx_t_4 = (((__pyx_v_data_arr[__pyx_v_i_swap]).val <= (__pyx_v_data_arr[__pyx_v_i]).val) != 0);
    __pyx_t_1 = __pyx_t_4;
    __pyx_L9_bool_binop_done:;
    if (__pyx_t_1) {

      /* "binary_tree.pxi":975
 * 
 *             if (i_swap > 0) and (data_arr[i_swap].val <= data_arr[i].val):
 *                 swap_nodes(data_arr, i, i_swap)             # <<<<<<<<<<<<<<
 *                 i = i_swap
 *             else:
 */
      __pyx_f_7sklearn_9neighbors_7kd_tree_swap_nodes(__pyx_v_data_arr, __pyx_v_i, __pyx_v_i_swap);

      /* "binary_tree.pxi":976
 *             if (i_swap > 0) and (data_arr[i_swap].val <= data_arr[i].val):
 *                 swap_nodes(data_arr, i, i_swap)
 *                 i = i_swap             # <<<<<<<<<<<<<<
 *             else:
 *                 break
 */
      __pyx_v_i = __pyx_v_i_swap;
      goto __pyx_L8;
    }
    /*else*/ {

      /* "binary_tree.pxi":978
 *                 i = i_swap
 *             else:
 *                 break             # <<<<<<<<<<<<<<
 * 
 *         return popped_element
 */
      goto __pyx_L5_break;
    }
    __pyx_L8:;
  }
  __pyx_L5_break:;

  /* "binary_tree.pxi":980
 *                 break
 * 
 *         return popped_element             # <<<<<<<<<<<<<<
 * 
 *     cdef void clear(self):
 */
  __pyx_r = __pyx_v_popped_element;
  goto __pyx_L0;

  /* "binary_tree.pxi":943
 *         return self.data[0]
 * 
 *     cdef NodeHeapData_t pop(self):             # <<<<<<<<<<<<<<
 *         """Remove the root of the heap, and update the remaining nodes"""
 *         if self.n == 0:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_WriteUnraisable("sklearn.neighbors.kd_tree.NodeHeap.pop", __pyx_clineno, __pyx_lineno, __pyx_filename, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":982
 *         return popped_element
 * 
 *     cdef void clear(self):             # <<<<<<<<<<<<<<
 *         """Clear the heap"""
 *         self.n = 0
 */

static void __pyx_f_7sklearn_9neighbors_7kd_tree_8NodeHeap_clear(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("clear", 0);

  /* "binary_tree.pxi":984
 *     cdef void clear(self):
 *         """Clear the heap"""
 *         self.n = 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_self->n = 0;

  /* "binary_tree.pxi":982
 *         return popped_element
 * 
 *     cdef void clear(self):             # <<<<<<<<<<<<<<
 *         """Clear the heap"""
 *         self.n = 0
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "binary_tree.pxi":990
 * # newObj function
 * #  this is a helper function for pickling
 * def newObj(obj):             # <<<<<<<<<<<<<<
 *     return obj.__new__(obj)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_3newObj(PyObject *__pyx_self, PyObject *__pyx_v_obj); /*proto*/
static PyMethodDef __pyx_mdef_7sklearn_9neighbors_7kd_tree_3newObj = {"newObj", (PyCFunction)__pyx_pw_7sklearn_9neighbors_7kd_tree_3newObj, METH_O, 0};
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_3newObj(PyObject *__pyx_self, PyObject *__pyx_v_obj) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("newObj (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_2newObj(__pyx_self, ((PyObject *)__pyx_v_obj));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_2newObj(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_obj) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("newObj", 0);

  /* "binary_tree.pxi":991
 * #  this is a helper function for pickling
 * def newObj(obj):
 *     return obj.__new__(obj)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_obj, __pyx_n_s_new); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 991; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_obj); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 991; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 991; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __Pyx_GIVEREF(__pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_INCREF(__pyx_v_obj);
    PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_obj);
    __Pyx_GIVEREF(__pyx_v_obj);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 991; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":990
 * # newObj function
 * #  this is a helper function for pickling
 * def newObj(obj):             # <<<<<<<<<<<<<<
 *     return obj.__new__(obj)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.newObj", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1031
 *     # Use cinit to initialize all arrays to empty: this will prevent memory
 *     # errors and seg-faults in rare cases where __init__ is not called
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *         self.data_arr = np.empty((1, 1), dtype=DTYPE, order='C')
 *         self.idx_array_arr = np.empty(1, dtype=ITYPE, order='C')
 */

/* Python wrapper */
static int __pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree___cinit__(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree___cinit__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_memviewslice __pyx_t_4 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "binary_tree.pxi":1032
 *     # errors and seg-faults in rare cases where __init__ is not called
 *     def __cinit__(self):
 *         self.data_arr = np.empty((1, 1), dtype=DTYPE, order='C')             # <<<<<<<<<<<<<<
 *         self.idx_array_arr = np.empty(1, dtype=ITYPE, order='C')
 *         self.node_data_arr = np.empty(1, dtype=NodeData, order='C')
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1032; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1032; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1032; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1032; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1032; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1032; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__10, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1032; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1032; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->data_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->data_arr));
  __pyx_v_self->data_arr = ((PyArrayObject *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "binary_tree.pxi":1033
 *     def __cinit__(self):
 *         self.data_arr = np.empty((1, 1), dtype=DTYPE, order='C')
 *         self.idx_array_arr = np.empty(1, dtype=ITYPE, order='C')             # <<<<<<<<<<<<<<
 *         self.node_data_arr = np.empty(1, dtype=NodeData, order='C')
 *         self.node_bounds_arr = np.empty((1, 1, 1), dtype=DTYPE)
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1033; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_empty); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1033; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1033; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1033; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1033; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1033; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__11, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1033; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1033; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->idx_array_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->idx_array_arr));
  __pyx_v_self->idx_array_arr = ((PyArrayObject *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "binary_tree.pxi":1034
 *         self.data_arr = np.empty((1, 1), dtype=DTYPE, order='C')
 *         self.idx_array_arr = np.empty(1, dtype=ITYPE, order='C')
 *         self.node_data_arr = np.empty(1, dtype=NodeData, order='C')             # <<<<<<<<<<<<<<
 *         self.node_bounds_arr = np.empty((1, 1, 1), dtype=DTYPE)
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1034; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1034; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1034; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_NodeData); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1034; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1034; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1034; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__12, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1034; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1034; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->node_data_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->node_data_arr));
  __pyx_v_self->node_data_arr = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "binary_tree.pxi":1035
 *         self.idx_array_arr = np.empty(1, dtype=ITYPE, order='C')
 *         self.node_data_arr = np.empty(1, dtype=NodeData, order='C')
 *         self.node_bounds_arr = np.empty((1, 1, 1), dtype=DTYPE)             # <<<<<<<<<<<<<<
 * 
 *         self.data = get_memview_DTYPE_2D(self.data_arr)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__14, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->node_bounds_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->node_bounds_arr));
  __pyx_v_self->node_bounds_arr = ((PyArrayObject *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "binary_tree.pxi":1037
 *         self.node_bounds_arr = np.empty((1, 1, 1), dtype=DTYPE)
 * 
 *         self.data = get_memview_DTYPE_2D(self.data_arr)             # <<<<<<<<<<<<<<
 *         self.idx_array = get_memview_ITYPE_1D(self.idx_array_arr)
 *         self.node_data = get_memview_NodeData_1D(self.node_data_arr)
 */
  __pyx_t_3 = ((PyObject *)__pyx_v_self->data_arr);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_2D(((PyArrayObject *)__pyx_t_3)); if (unlikely(!__pyx_t_4.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1037; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->data, 0);
  __pyx_v_self->data = __pyx_t_4;
  __pyx_t_4.memview = NULL;
  __pyx_t_4.data = NULL;

  /* "binary_tree.pxi":1038
 * 
 *         self.data = get_memview_DTYPE_2D(self.data_arr)
 *         self.idx_array = get_memview_ITYPE_1D(self.idx_array_arr)             # <<<<<<<<<<<<<<
 *         self.node_data = get_memview_NodeData_1D(self.node_data_arr)
 *         self.node_bounds = get_memview_DTYPE_3D(self.node_bounds_arr)
 */
  __pyx_t_3 = ((PyObject *)__pyx_v_self->idx_array_arr);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_5 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_ITYPE_1D(((PyArrayObject *)__pyx_t_3)); if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1038; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->idx_array, 0);
  __pyx_v_self->idx_array = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "binary_tree.pxi":1039
 *         self.data = get_memview_DTYPE_2D(self.data_arr)
 *         self.idx_array = get_memview_ITYPE_1D(self.idx_array_arr)
 *         self.node_data = get_memview_NodeData_1D(self.node_data_arr)             # <<<<<<<<<<<<<<
 *         self.node_bounds = get_memview_DTYPE_3D(self.node_bounds_arr)
 * 
 */
  __pyx_t_3 = ((PyObject *)__pyx_v_self->node_data_arr);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_6 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_NodeData_1D(((PyArrayObject *)__pyx_t_3)); if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1039; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->node_data, 0);
  __pyx_v_self->node_data = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "binary_tree.pxi":1040
 *         self.idx_array = get_memview_ITYPE_1D(self.idx_array_arr)
 *         self.node_data = get_memview_NodeData_1D(self.node_data_arr)
 *         self.node_bounds = get_memview_DTYPE_3D(self.node_bounds_arr)             # <<<<<<<<<<<<<<
 * 
 *         self.leaf_size = 0
 */
  __pyx_t_3 = ((PyObject *)__pyx_v_self->node_bounds_arr);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_7 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_3D(((PyArrayObject *)__pyx_t_3)); if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1040; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->node_bounds, 0);
  __pyx_v_self->node_bounds = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "binary_tree.pxi":1042
 *         self.node_bounds = get_memview_DTYPE_3D(self.node_bounds_arr)
 * 
 *         self.leaf_size = 0             # <<<<<<<<<<<<<<
 *         self.n_levels = 0
 *         self.n_nodes = 0
 */
  __pyx_v_self->leaf_size = 0;

  /* "binary_tree.pxi":1043
 * 
 *         self.leaf_size = 0
 *         self.n_levels = 0             # <<<<<<<<<<<<<<
 *         self.n_nodes = 0
 * 
 */
  __pyx_v_self->n_levels = 0;

  /* "binary_tree.pxi":1044
 *         self.leaf_size = 0
 *         self.n_levels = 0
 *         self.n_nodes = 0             # <<<<<<<<<<<<<<
 * 
 *         self.euclidean = False
 */
  __pyx_v_self->n_nodes = 0;

  /* "binary_tree.pxi":1046
 *         self.n_nodes = 0
 * 
 *         self.euclidean = False             # <<<<<<<<<<<<<<
 * 
 *         self.n_trims = 0
 */
  __pyx_v_self->euclidean = 0;

  /* "binary_tree.pxi":1048
 *         self.euclidean = False
 * 
 *         self.n_trims = 0             # <<<<<<<<<<<<<<
 *         self.n_leaves = 0
 *         self.n_splits = 0
 */
  __pyx_v_self->n_trims = 0;

  /* "binary_tree.pxi":1049
 * 
 *         self.n_trims = 0
 *         self.n_leaves = 0             # <<<<<<<<<<<<<<
 *         self.n_splits = 0
 *         self.n_calls = 0
 */
  __pyx_v_self->n_leaves = 0;

  /* "binary_tree.pxi":1050
 *         self.n_trims = 0
 *         self.n_leaves = 0
 *         self.n_splits = 0             # <<<<<<<<<<<<<<
 *         self.n_calls = 0
 * 
 */
  __pyx_v_self->n_splits = 0;

  /* "binary_tree.pxi":1051
 *         self.n_leaves = 0
 *         self.n_splits = 0
 *         self.n_calls = 0             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, data,
 */
  __pyx_v_self->n_calls = 0;

  /* "binary_tree.pxi":1031
 *     # Use cinit to initialize all arrays to empty: this will prevent memory
 *     # errors and seg-faults in rare cases where __init__ is not called
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *         self.data_arr = np.empty((1, 1), dtype=DTYPE, order='C')
 *         self.idx_array_arr = np.empty(1, dtype=ITYPE, order='C')
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __PYX_XDEC_MEMVIEW(&__pyx_t_4, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1053
 *         self.n_calls = 0
 * 
 *     def __init__(self, data,             # <<<<<<<<<<<<<<
 *                  leaf_size=40, metric='minkowski', **kwargs):
 *         self.data_arr = np.asarray(data, dtype=DTYPE, order='C')
 */

/* Python wrapper */
static int __pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_leaf_size = 0;
  PyObject *__pyx_v_metric = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_leaf_size,&__pyx_n_s_metric,0};
    PyObject* values[3] = {0,0,0};
    values[1] = ((PyObject *)__pyx_int_40);
    values[2] = ((PyObject *)__pyx_n_s_minkowski);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_leaf_size);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_metric);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1053; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_data = values[0];
    __pyx_v_leaf_size = values[1];
    __pyx_v_metric = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1053; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_2__init__(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self), __pyx_v_data, __pyx_v_leaf_size, __pyx_v_metric, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_2__init__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, PyObject *__pyx_v_data, PyObject *__pyx_v_leaf_size, PyObject *__pyx_v_metric, PyObject *__pyx_v_kwargs) {
  PyObject *__pyx_v_n_samples = NULL;
  Py_ssize_t __pyx_v_n_features;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  double __pyx_t_11;
  __Pyx_memviewslice __pyx_t_12 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_13 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);
  __Pyx_INCREF(__pyx_v_metric);

  /* "binary_tree.pxi":1055
 *     def __init__(self, data,
 *                  leaf_size=40, metric='minkowski', **kwargs):
 *         self.data_arr = np.asarray(data, dtype=DTYPE, order='C')             # <<<<<<<<<<<<<<
 *         self.data = get_memview_DTYPE_2D(self.data_arr)
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1055; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_asarray); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1055; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1055; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_data);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_data);
  __Pyx_GIVEREF(__pyx_v_data);
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1055; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1055; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1055; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1055; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1055; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1055; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->data_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->data_arr));
  __pyx_v_self->data_arr = ((PyArrayObject *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "binary_tree.pxi":1056
 *                  leaf_size=40, metric='minkowski', **kwargs):
 *         self.data_arr = np.asarray(data, dtype=DTYPE, order='C')
 *         self.data = get_memview_DTYPE_2D(self.data_arr)             # <<<<<<<<<<<<<<
 * 
 *         self.leaf_size = leaf_size
 */
  __pyx_t_4 = ((PyObject *)__pyx_v_self->data_arr);
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_5 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_2D(((PyArrayObject *)__pyx_t_4)); if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1056; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->data, 0);
  __pyx_v_self->data = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "binary_tree.pxi":1058
 *         self.data = get_memview_DTYPE_2D(self.data_arr)
 * 
 *         self.leaf_size = leaf_size             # <<<<<<<<<<<<<<
 *         self.dist_metric = DistanceMetric.get_metric(metric, **kwargs)
 *         self.euclidean = (self.dist_metric.__class__.__name__
 */
  __pyx_t_6 = __Pyx_PyInt_As_Py_intptr_t(__pyx_v_leaf_size); if (unlikely((__pyx_t_6 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1058; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_self->leaf_size = __pyx_t_6;

  /* "binary_tree.pxi":1059
 * 
 *         self.leaf_size = leaf_size
 *         self.dist_metric = DistanceMetric.get_metric(metric, **kwargs)             # <<<<<<<<<<<<<<
 *         self.euclidean = (self.dist_metric.__class__.__name__
 *                           == 'EuclideanDistance')
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)((PyObject*)__pyx_ptype_7sklearn_9neighbors_12dist_metrics_DistanceMetric)), __pyx_n_s_get_metric); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1059; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1059; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_metric);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_metric);
  __Pyx_GIVEREF(__pyx_v_metric);
  __pyx_t_1 = __pyx_v_kwargs;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1059; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_7sklearn_9neighbors_12dist_metrics_DistanceMetric))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1059; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->dist_metric);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->dist_metric));
  __pyx_v_self->dist_metric = ((struct __pyx_obj_7sklearn_9neighbors_12dist_metrics_DistanceMetric *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "binary_tree.pxi":1060
 *         self.leaf_size = leaf_size
 *         self.dist_metric = DistanceMetric.get_metric(metric, **kwargs)
 *         self.euclidean = (self.dist_metric.__class__.__name__             # <<<<<<<<<<<<<<
 *                           == 'EuclideanDistance')
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->dist_metric), __pyx_n_s_class); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1060; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_name); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1060; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_n_s_EuclideanDistance, Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1061; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "binary_tree.pxi":1061
 *         self.dist_metric = DistanceMetric.get_metric(metric, **kwargs)
 *         self.euclidean = (self.dist_metric.__class__.__name__
 *                           == 'EuclideanDistance')             # <<<<<<<<<<<<<<
 * 
 *         metric = self.dist_metric.__class__.__name__
 */
  __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1061; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "binary_tree.pxi":1060
 *         self.leaf_size = leaf_size
 *         self.dist_metric = DistanceMetric.get_metric(metric, **kwargs)
 *         self.euclidean = (self.dist_metric.__class__.__name__             # <<<<<<<<<<<<<<
 *                           == 'EuclideanDistance')
 * 
 */
  __pyx_v_self->euclidean = __pyx_t_7;

  /* "binary_tree.pxi":1063
 *                           == 'EuclideanDistance')
 * 
 *         metric = self.dist_metric.__class__.__name__             # <<<<<<<<<<<<<<
 *         if metric not in VALID_METRICS:
 *             raise ValueError('metric {metric} is not valid for '
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->dist_metric), __pyx_n_s_class); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1063; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_name); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1063; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_metric, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "binary_tree.pxi":1064
 * 
 *         metric = self.dist_metric.__class__.__name__
 *         if metric not in VALID_METRICS:             # <<<<<<<<<<<<<<
 *             raise ValueError('metric {metric} is not valid for '
 *                              '{BinaryTree}'.format(metric=metric,
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_VALID_METRICS); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1064; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = (__Pyx_PySequence_Contains(__pyx_v_metric, __pyx_t_1, Py_NE)); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1064; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "binary_tree.pxi":1066
 *         if metric not in VALID_METRICS:
 *             raise ValueError('metric {metric} is not valid for '
 *                              '{BinaryTree}'.format(metric=metric,             # <<<<<<<<<<<<<<
 *                                                    **DOC_DICT))
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_metric_metric_is_not_valid_for_B, __pyx_n_s_format); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1066; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);

    /* "binary_tree.pxi":1067
 *             raise ValueError('metric {metric} is not valid for '
 *                              '{BinaryTree}'.format(metric=metric,
 *                                                    **DOC_DICT))             # <<<<<<<<<<<<<<
 * 
 *         # validate data
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_DOC_DICT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1067; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    if (unlikely(__pyx_t_3 == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "argument after ** must be a mapping, not NoneType");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1067; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    if (likely(PyDict_Check(__pyx_t_3))) {
      __pyx_t_2 = PyDict_Copy(__pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1066; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      __pyx_t_2 = PyObject_CallFunctionObjArgs((PyObject*)&PyDict_Type, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1066; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "binary_tree.pxi":1066
 *         if metric not in VALID_METRICS:
 *             raise ValueError('metric {metric} is not valid for '
 *                              '{BinaryTree}'.format(metric=metric,             # <<<<<<<<<<<<<<
 *                                                    **DOC_DICT))
 * 
 */
    if (unlikely(PyDict_GetItem(__pyx_t_2, __pyx_n_s_metric))) {
      __Pyx_RaiseDoubleKeywordsError("function", __pyx_n_s_metric); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1066; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_metric, __pyx_v_metric) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1066; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1066; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "binary_tree.pxi":1065
 *         metric = self.dist_metric.__class__.__name__
 *         if metric not in VALID_METRICS:
 *             raise ValueError('metric {metric} is not valid for '             # <<<<<<<<<<<<<<
 *                              '{BinaryTree}'.format(metric=metric,
 *                                                    **DOC_DICT))
 */
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1065; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1065; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1065; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "binary_tree.pxi":1070
 * 
 *         # validate data
 *         if self.data.size == 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("X is an empty array")
 * 
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1070; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_self->data, 2, (PyObject *(*)(char *)) __pyx_memview_get_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, (int (*)(char *, PyObject *)) __pyx_memview_set_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1070; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_size); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1070; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_2, __pyx_int_0, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1070; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1070; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_9) {

    /* "binary_tree.pxi":1071
 *         # validate data
 *         if self.data.size == 0:
 *             raise ValueError("X is an empty array")             # <<<<<<<<<<<<<<
 * 
 *         if leaf_size < 1:
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1071; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1071; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "binary_tree.pxi":1073
 *             raise ValueError("X is an empty array")
 * 
 *         if leaf_size < 1:             # <<<<<<<<<<<<<<
 *             raise ValueError("leaf_size must be greater than or equal to 1")
 * 
 */
  __pyx_t_3 = PyObject_RichCompare(__pyx_v_leaf_size, __pyx_int_1, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1073; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1073; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_9) {

    /* "binary_tree.pxi":1074
 * 
 *         if leaf_size < 1:
 *             raise ValueError("leaf_size must be greater than or equal to 1")             # <<<<<<<<<<<<<<
 * 
 *         n_samples = self.data.shape[0]
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1074; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1074; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "binary_tree.pxi":1076
 *             raise ValueError("leaf_size must be greater than or equal to 1")
 * 
 *         n_samples = self.data.shape[0]             # <<<<<<<<<<<<<<
 *         n_features = self.data.shape[1]
 * 
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1076; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_3 = PyInt_FromSsize_t((__pyx_v_self->data.shape[0])); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1076; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_n_samples = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "binary_tree.pxi":1077
 * 
 *         n_samples = self.data.shape[0]
 *         n_features = self.data.shape[1]             # <<<<<<<<<<<<<<
 * 
 *         # determine number of levels in the tree, and from this
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1077; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_n_features = (__pyx_v_self->data.shape[1]);

  /* "binary_tree.pxi":1082
 *         # the number of nodes in the tree.  This results in leaf nodes
 *         # with numbers of points betweeen leaf_size and 2 * leaf_size
 *         self.n_levels = np.log2(fmax(1, (n_samples - 1) / self.leaf_size)) + 1             # <<<<<<<<<<<<<<
 *         self.n_nodes = (2 ** self.n_levels) - 1
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_log2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyNumber_Subtract(__pyx_v_n_samples, __pyx_int_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->leaf_size); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_10 = __Pyx_PyNumber_Divide(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_11 = __pyx_PyFloat_AsDouble(__pyx_t_10); if (unlikely((__pyx_t_11 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyFloat_FromDouble(__pyx_f_7sklearn_9neighbors_7kd_tree_fmax(1.0, __pyx_t_11)); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_10); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    __pyx_t_2 = PyTuple_New(1+1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4); __Pyx_GIVEREF(__pyx_t_4); __pyx_t_4 = NULL;
    PyTuple_SET_ITEM(__pyx_t_2, 0+1, __pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_10);
    __pyx_t_10 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Add(__pyx_t_3, __pyx_int_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_1); if (unlikely((__pyx_t_6 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->n_levels = __pyx_t_6;

  /* "binary_tree.pxi":1083
 *         # with numbers of points betweeen leaf_size and 2 * leaf_size
 *         self.n_levels = np.log2(fmax(1, (n_samples - 1) / self.leaf_size)) + 1
 *         self.n_nodes = (2 ** self.n_levels) - 1             # <<<<<<<<<<<<<<
 * 
 *         # allocate arrays for storage
 */
  __pyx_v_self->n_nodes = (__Pyx_pow_long(2, ((long)__pyx_v_self->n_levels)) - 1);

  /* "binary_tree.pxi":1086
 * 
 *         # allocate arrays for storage
 *         self.idx_array_arr = np.arange(n_samples, dtype=ITYPE)             # <<<<<<<<<<<<<<
 *         self.idx_array = get_memview_ITYPE_1D(self.idx_array_arr)
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1086; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_arange); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1086; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1086; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_n_samples);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_n_samples);
  __Pyx_GIVEREF(__pyx_v_n_samples);
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1086; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1086; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_10) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1086; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1086; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(((__pyx_t_10) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_10, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1086; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_10);
  __Pyx_GOTREF(__pyx_v_self->idx_array_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->idx_array_arr));
  __pyx_v_self->idx_array_arr = ((PyArrayObject *)__pyx_t_10);
  __pyx_t_10 = 0;

  /* "binary_tree.pxi":1087
 *         # allocate arrays for storage
 *         self.idx_array_arr = np.arange(n_samples, dtype=ITYPE)
 *         self.idx_array = get_memview_ITYPE_1D(self.idx_array_arr)             # <<<<<<<<<<<<<<
 * 
 *         self.node_data_arr = np.zeros(self.n_nodes, dtype=NodeData)
 */
  __pyx_t_10 = ((PyObject *)__pyx_v_self->idx_array_arr);
  __Pyx_INCREF(__pyx_t_10);
  __pyx_t_12 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_ITYPE_1D(((PyArrayObject *)__pyx_t_10)); if (unlikely(!__pyx_t_12.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1087; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->idx_array, 0);
  __pyx_v_self->idx_array = __pyx_t_12;
  __pyx_t_12.memview = NULL;
  __pyx_t_12.data = NULL;

  /* "binary_tree.pxi":1089
 *         self.idx_array = get_memview_ITYPE_1D(self.idx_array_arr)
 * 
 *         self.node_data_arr = np.zeros(self.n_nodes, dtype=NodeData)             # <<<<<<<<<<<<<<
 *         self.node_data = get_memview_NodeData_1D(self.node_data_arr)
 * 
 */
  __pyx_t_10 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1089; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1089; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->n_nodes); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1089; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1089; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_10);
  __pyx_t_10 = 0;
  __pyx_t_10 = PyDict_New(); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1089; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_NodeData); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1089; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_10, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1089; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, __pyx_t_10); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1089; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1089; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->node_data_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->node_data_arr));
  __pyx_v_self->node_data_arr = ((PyArrayObject *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "binary_tree.pxi":1090
 * 
 *         self.node_data_arr = np.zeros(self.n_nodes, dtype=NodeData)
 *         self.node_data = get_memview_NodeData_1D(self.node_data_arr)             # <<<<<<<<<<<<<<
 * 
 *         # Allocate tree-specific data
 */
  __pyx_t_3 = ((PyObject *)__pyx_v_self->node_data_arr);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_13 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_NodeData_1D(((PyArrayObject *)__pyx_t_3)); if (unlikely(!__pyx_t_13.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1090; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->node_data, 0);
  __pyx_v_self->node_data = __pyx_t_13;
  __pyx_t_13.memview = NULL;
  __pyx_t_13.data = NULL;

  /* "binary_tree.pxi":1093
 * 
 *         # Allocate tree-specific data
 *         allocate_data(self, self.n_nodes, n_features)             # <<<<<<<<<<<<<<
 *         self._recursive_build(0, 0, n_samples)
 * 
 */
  __pyx_t_7 = __pyx_f_7sklearn_9neighbors_7kd_tree_allocate_data(__pyx_v_self, __pyx_v_self->n_nodes, __pyx_v_n_features); if (unlikely(__pyx_t_7 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1093; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "binary_tree.pxi":1094
 *         # Allocate tree-specific data
 *         allocate_data(self, self.n_nodes, n_features)
 *         self._recursive_build(0, 0, n_samples)             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
 */
  __pyx_t_6 = __Pyx_PyInt_As_Py_intptr_t(__pyx_v_n_samples); if (unlikely((__pyx_t_6 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1094; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_7 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_recursive_build(__pyx_v_self, 0, 0, __pyx_t_6); if (unlikely(__pyx_t_7 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1094; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "binary_tree.pxi":1053
 *         self.n_calls = 0
 * 
 *     def __init__(self, data,             # <<<<<<<<<<<<<<
 *                  leaf_size=40, metric='minkowski', **kwargs):
 *         self.data_arr = np.asarray(data, dtype=DTYPE, order='C')
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  __Pyx_XDECREF(__pyx_t_10);
  __PYX_XDEC_MEMVIEW(&__pyx_t_12, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_13, 1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_n_samples);
  __Pyx_XDECREF(__pyx_v_metric);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1096
 *         self._recursive_build(0, 0, n_samples)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         """
 *         reduce method used for pickling
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_5__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_7sklearn_9neighbors_7kd_tree_10BinaryTree_4__reduce__[] = "\n        reduce method used for pickling\n        ";
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_5__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_4__reduce__(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_4__reduce__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "binary_tree.pxi":1100
 *         reduce method used for pickling
 *         """
 *         return (newObj, (BinaryTree,), self.__getstate__())             # <<<<<<<<<<<<<<
 * 
 *     def __getstate__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_newObj); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1100; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1100; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)((PyObject*)__pyx_ptype_7sklearn_9neighbors_7kd_tree_BinaryTree)));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)((PyObject*)__pyx_ptype_7sklearn_9neighbors_7kd_tree_BinaryTree)));
  __Pyx_GIVEREF(((PyObject *)((PyObject*)__pyx_ptype_7sklearn_9neighbors_7kd_tree_BinaryTree)));
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getstate); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1100; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (__pyx_t_5) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1100; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else {
    __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1100; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1100; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":1096
 *         self._recursive_build(0, 0, n_samples)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         """
 *         reduce method used for pickling
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1102
 *         return (newObj, (BinaryTree,), self.__getstate__())
 * 
 *     def __getstate__(self):             # <<<<<<<<<<<<<<
 *         """
 *         get state for pickling
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_7__getstate__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_7sklearn_9neighbors_7kd_tree_10BinaryTree_6__getstate__[] = "\n        get state for pickling\n        ";
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_7__getstate__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getstate__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_6__getstate__(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_6__getstate__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getstate__", 0);

  /* "binary_tree.pxi":1106
 *         get state for pickling
 *         """
 *         return (self.data_arr,             # <<<<<<<<<<<<<<
 *                 self.idx_array_arr,
 *                 self.node_data_arr,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "binary_tree.pxi":1110
 *                 self.node_data_arr,
 *                 self.node_bounds_arr,
 *                 int(self.leaf_size),             # <<<<<<<<<<<<<<
 *                 int(self.n_levels),
 *                 int(self.n_nodes),
 */
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->leaf_size); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)((PyObject*)(&PyInt_Type))), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "binary_tree.pxi":1111
 *                 self.node_bounds_arr,
 *                 int(self.leaf_size),
 *                 int(self.n_levels),             # <<<<<<<<<<<<<<
 *                 int(self.n_nodes),
 *                 int(self.n_trims),
 */
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->n_levels); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)((PyObject*)(&PyInt_Type))), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "binary_tree.pxi":1112
 *                 int(self.leaf_size),
 *                 int(self.n_levels),
 *                 int(self.n_nodes),             # <<<<<<<<<<<<<<
 *                 int(self.n_trims),
 *                 int(self.n_leaves),
 */
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->n_nodes); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)((PyObject*)(&PyInt_Type))), __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "binary_tree.pxi":1113
 *                 int(self.n_levels),
 *                 int(self.n_nodes),
 *                 int(self.n_trims),             # <<<<<<<<<<<<<<
 *                 int(self.n_leaves),
 *                 int(self.n_splits),
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_self->n_trims); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1113; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1113; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)((PyObject*)(&PyInt_Type))), __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1113; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "binary_tree.pxi":1114
 *                 int(self.n_nodes),
 *                 int(self.n_trims),
 *                 int(self.n_leaves),             # <<<<<<<<<<<<<<
 *                 int(self.n_splits),
 *                 int(self.n_calls),
 */
  __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->n_leaves); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1114; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1114; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_5);
  __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)((PyObject*)(&PyInt_Type))), __pyx_t_6, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1114; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "binary_tree.pxi":1115
 *                 int(self.n_trims),
 *                 int(self.n_leaves),
 *                 int(self.n_splits),             # <<<<<<<<<<<<<<
 *                 int(self.n_calls),
 *                 self.dist_metric)
 */
  __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_self->n_splits); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_6);
  __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_Call(((PyObject *)((PyObject*)(&PyInt_Type))), __pyx_t_7, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "binary_tree.pxi":1116
 *                 int(self.n_leaves),
 *                 int(self.n_splits),
 *                 int(self.n_calls),             # <<<<<<<<<<<<<<
 *                 self.dist_metric)
 * 
 */
  __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_self->n_calls); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1116; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1116; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_7);
  __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_Call(((PyObject *)((PyObject*)(&PyInt_Type))), __pyx_t_8, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1116; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "binary_tree.pxi":1106
 *         get state for pickling
 *         """
 *         return (self.data_arr,             # <<<<<<<<<<<<<<
 *                 self.idx_array_arr,
 *                 self.node_data_arr,
 */
  __pyx_t_8 = PyTuple_New(12); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1106; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->data_arr));
  PyTuple_SET_ITEM(__pyx_t_8, 0, ((PyObject *)__pyx_v_self->data_arr));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->data_arr));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->idx_array_arr));
  PyTuple_SET_ITEM(__pyx_t_8, 1, ((PyObject *)__pyx_v_self->idx_array_arr));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->idx_array_arr));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->node_data_arr));
  PyTuple_SET_ITEM(__pyx_t_8, 2, ((PyObject *)__pyx_v_self->node_data_arr));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->node_data_arr));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->node_bounds_arr));
  PyTuple_SET_ITEM(__pyx_t_8, 3, ((PyObject *)__pyx_v_self->node_bounds_arr));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->node_bounds_arr));
  PyTuple_SET_ITEM(__pyx_t_8, 4, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_8, 5, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_8, 6, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_8, 7, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_8, 8, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_8, 9, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_8, 10, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_7);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->dist_metric));
  PyTuple_SET_ITEM(__pyx_t_8, 11, ((PyObject *)__pyx_v_self->dist_metric));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->dist_metric));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_r = __pyx_t_8;
  __pyx_t_8 = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":1102
 *         return (newObj, (BinaryTree,), self.__getstate__())
 * 
 *     def __getstate__(self):             # <<<<<<<<<<<<<<
 *         """
 *         get state for pickling
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.__getstate__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1119
 *                 self.dist_metric)
 * 
 *     def __setstate__(self, state):             # <<<<<<<<<<<<<<
 *         """
 *         set state for pickling
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_9__setstate__(PyObject *__pyx_v_self, PyObject *__pyx_v_state); /*proto*/
static char __pyx_doc_7sklearn_9neighbors_7kd_tree_10BinaryTree_8__setstate__[] = "\n        set state for pickling\n        ";
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_9__setstate__(PyObject *__pyx_v_self, PyObject *__pyx_v_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_8__setstate__(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self), ((PyObject *)__pyx_v_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_8__setstate__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, PyObject *__pyx_v_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_memviewslice __pyx_t_2 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_3 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_4 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate__", 0);

  /* "binary_tree.pxi":1123
 *         set state for pickling
 *         """
 *         self.data_arr = state[0]             # <<<<<<<<<<<<<<
 *         self.idx_array_arr = state[1]
 *         self.node_data_arr = state[2]
 */
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1123; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->data_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->data_arr));
  __pyx_v_self->data_arr = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "binary_tree.pxi":1124
 *         """
 *         self.data_arr = state[0]
 *         self.idx_array_arr = state[1]             # <<<<<<<<<<<<<<
 *         self.node_data_arr = state[2]
 *         self.node_bounds_arr = state[3]
 */
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1124; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->idx_array_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->idx_array_arr));
  __pyx_v_self->idx_array_arr = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "binary_tree.pxi":1125
 *         self.data_arr = state[0]
 *         self.idx_array_arr = state[1]
 *         self.node_data_arr = state[2]             # <<<<<<<<<<<<<<
 *         self.node_bounds_arr = state[3]
 * 
 */
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1125; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->node_data_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->node_data_arr));
  __pyx_v_self->node_data_arr = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "binary_tree.pxi":1126
 *         self.idx_array_arr = state[1]
 *         self.node_data_arr = state[2]
 *         self.node_bounds_arr = state[3]             # <<<<<<<<<<<<<<
 * 
 *         self.data = get_memview_DTYPE_2D(self.data_arr)
 */
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1126; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1126; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->node_bounds_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->node_bounds_arr));
  __pyx_v_self->node_bounds_arr = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "binary_tree.pxi":1128
 *         self.node_bounds_arr = state[3]
 * 
 *         self.data = get_memview_DTYPE_2D(self.data_arr)             # <<<<<<<<<<<<<<
 *         self.idx_array = get_memview_ITYPE_1D(self.idx_array_arr)
 *         self.node_data = get_memview_NodeData_1D(self.node_data_arr)
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_self->data_arr);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_2D(((PyArrayObject *)__pyx_t_1)); if (unlikely(!__pyx_t_2.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1128; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->data, 0);
  __pyx_v_self->data = __pyx_t_2;
  __pyx_t_2.memview = NULL;
  __pyx_t_2.data = NULL;

  /* "binary_tree.pxi":1129
 * 
 *         self.data = get_memview_DTYPE_2D(self.data_arr)
 *         self.idx_array = get_memview_ITYPE_1D(self.idx_array_arr)             # <<<<<<<<<<<<<<
 *         self.node_data = get_memview_NodeData_1D(self.node_data_arr)
 *         self.node_bounds = get_memview_DTYPE_3D(self.node_bounds_arr)
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_self->idx_array_arr);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_3 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_ITYPE_1D(((PyArrayObject *)__pyx_t_1)); if (unlikely(!__pyx_t_3.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1129; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->idx_array, 0);
  __pyx_v_self->idx_array = __pyx_t_3;
  __pyx_t_3.memview = NULL;
  __pyx_t_3.data = NULL;

  /* "binary_tree.pxi":1130
 *         self.data = get_memview_DTYPE_2D(self.data_arr)
 *         self.idx_array = get_memview_ITYPE_1D(self.idx_array_arr)
 *         self.node_data = get_memview_NodeData_1D(self.node_data_arr)             # <<<<<<<<<<<<<<
 *         self.node_bounds = get_memview_DTYPE_3D(self.node_bounds_arr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_self->node_data_arr);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_NodeData_1D(((PyArrayObject *)__pyx_t_1)); if (unlikely(!__pyx_t_4.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1130; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->node_data, 0);
  __pyx_v_self->node_data = __pyx_t_4;
  __pyx_t_4.memview = NULL;
  __pyx_t_4.data = NULL;

  /* "binary_tree.pxi":1131
 *         self.idx_array = get_memview_ITYPE_1D(self.idx_array_arr)
 *         self.node_data = get_memview_NodeData_1D(self.node_data_arr)
 *         self.node_bounds = get_memview_DTYPE_3D(self.node_bounds_arr)             # <<<<<<<<<<<<<<
 * 
 *         self.leaf_size = state[4]
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_self->node_bounds_arr);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_5 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_3D(((PyArrayObject *)__pyx_t_1)); if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->node_bounds, 0);
  __pyx_v_self->node_bounds = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "binary_tree.pxi":1133
 *         self.node_bounds = get_memview_DTYPE_3D(self.node_bounds_arr)
 * 
 *         self.leaf_size = state[4]             # <<<<<<<<<<<<<<
 *         self.n_levels = state[5]
 *         self.n_nodes = state[6]
 */
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1133; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_1); if (unlikely((__pyx_t_6 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1133; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->leaf_size = __pyx_t_6;

  /* "binary_tree.pxi":1134
 * 
 *         self.leaf_size = state[4]
 *         self.n_levels = state[5]             # <<<<<<<<<<<<<<
 *         self.n_nodes = state[6]
 *         self.n_trims = state[7]
 */
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1134; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_1); if (unlikely((__pyx_t_6 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1134; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->n_levels = __pyx_t_6;

  /* "binary_tree.pxi":1135
 *         self.leaf_size = state[4]
 *         self.n_levels = state[5]
 *         self.n_nodes = state[6]             # <<<<<<<<<<<<<<
 *         self.n_trims = state[7]
 *         self.n_leaves = state[8]
 */
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1135; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_1); if (unlikely((__pyx_t_6 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->n_nodes = __pyx_t_6;

  /* "binary_tree.pxi":1136
 *         self.n_levels = state[5]
 *         self.n_nodes = state[6]
 *         self.n_trims = state[7]             # <<<<<<<<<<<<<<
 *         self.n_leaves = state[8]
 *         self.n_splits = state[9]
 */
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1136; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1136; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->n_trims = __pyx_t_7;

  /* "binary_tree.pxi":1137
 *         self.n_nodes = state[6]
 *         self.n_trims = state[7]
 *         self.n_leaves = state[8]             # <<<<<<<<<<<<<<
 *         self.n_splits = state[9]
 *         self.n_calls = state[10]
 */
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1137; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1137; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->n_leaves = __pyx_t_7;

  /* "binary_tree.pxi":1138
 *         self.n_trims = state[7]
 *         self.n_leaves = state[8]
 *         self.n_splits = state[9]             # <<<<<<<<<<<<<<
 *         self.n_calls = state[10]
 *         self.dist_metric = state[11]
 */
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1138; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1138; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->n_splits = __pyx_t_7;

  /* "binary_tree.pxi":1139
 *         self.n_leaves = state[8]
 *         self.n_splits = state[9]
 *         self.n_calls = state[10]             # <<<<<<<<<<<<<<
 *         self.dist_metric = state[11]
 *         self.euclidean = (self.dist_metric.__class__.__name__
 */
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1139; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1139; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->n_calls = __pyx_t_7;

  /* "binary_tree.pxi":1140
 *         self.n_splits = state[9]
 *         self.n_calls = state[10]
 *         self.dist_metric = state[11]             # <<<<<<<<<<<<<<
 *         self.euclidean = (self.dist_metric.__class__.__name__
 *                           == 'EuclideanDistance')
 */
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1140; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_7sklearn_9neighbors_12dist_metrics_DistanceMetric))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1140; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->dist_metric);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->dist_metric));
  __pyx_v_self->dist_metric = ((struct __pyx_obj_7sklearn_9neighbors_12dist_metrics_DistanceMetric *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "binary_tree.pxi":1141
 *         self.n_calls = state[10]
 *         self.dist_metric = state[11]
 *         self.euclidean = (self.dist_metric.__class__.__name__             # <<<<<<<<<<<<<<
 *                           == 'EuclideanDistance')
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->dist_metric), __pyx_n_s_class); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1141; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_name); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1141; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_8, __pyx_n_s_EuclideanDistance, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1142; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "binary_tree.pxi":1142
 *         self.dist_metric = state[11]
 *         self.euclidean = (self.dist_metric.__class__.__name__
 *                           == 'EuclideanDistance')             # <<<<<<<<<<<<<<
 * 
 *     def get_tree_stats(self):
 */
  __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1142; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "binary_tree.pxi":1141
 *         self.n_calls = state[10]
 *         self.dist_metric = state[11]
 *         self.euclidean = (self.dist_metric.__class__.__name__             # <<<<<<<<<<<<<<
 *                           == 'EuclideanDistance')
 * 
 */
  __pyx_v_self->euclidean = __pyx_t_7;

  /* "binary_tree.pxi":1119
 *                 self.dist_metric)
 * 
 *     def __setstate__(self, state):             # <<<<<<<<<<<<<<
 *         """
 *         set state for pickling
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_2, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_3, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_4, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.__setstate__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1144
 *                           == 'EuclideanDistance')
 * 
 *     def get_tree_stats(self):             # <<<<<<<<<<<<<<
 *         return (self.n_trims, self.n_leaves, self.n_splits)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_11get_tree_stats(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_11get_tree_stats(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_tree_stats (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_10get_tree_stats(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_10get_tree_stats(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_tree_stats", 0);

  /* "binary_tree.pxi":1145
 * 
 *     def get_tree_stats(self):
 *         return (self.n_trims, self.n_leaves, self.n_splits)             # <<<<<<<<<<<<<<
 * 
 *     def reset_n_calls(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->n_trims); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1145; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->n_leaves); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1145; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->n_splits); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1145; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1145; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":1144
 *                           == 'EuclideanDistance')
 * 
 *     def get_tree_stats(self):             # <<<<<<<<<<<<<<
 *         return (self.n_trims, self.n_leaves, self.n_splits)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.get_tree_stats", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1147
 *         return (self.n_trims, self.n_leaves, self.n_splits)
 * 
 *     def reset_n_calls(self):             # <<<<<<<<<<<<<<
 *         self.n_calls = 0
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_13reset_n_calls(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_13reset_n_calls(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("reset_n_calls (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_12reset_n_calls(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_12reset_n_calls(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("reset_n_calls", 0);

  /* "binary_tree.pxi":1148
 * 
 *     def reset_n_calls(self):
 *         self.n_calls = 0             # <<<<<<<<<<<<<<
 * 
 *     def get_n_calls(self):
 */
  __pyx_v_self->n_calls = 0;

  /* "binary_tree.pxi":1147
 *         return (self.n_trims, self.n_leaves, self.n_splits)
 * 
 *     def reset_n_calls(self):             # <<<<<<<<<<<<<<
 *         self.n_calls = 0
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1150
 *         self.n_calls = 0
 * 
 *     def get_n_calls(self):             # <<<<<<<<<<<<<<
 *         return self.n_calls
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_15get_n_calls(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_15get_n_calls(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_n_calls (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_14get_n_calls(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_14get_n_calls(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_n_calls", 0);

  /* "binary_tree.pxi":1151
 * 
 *     def get_n_calls(self):
 *         return self.n_calls             # <<<<<<<<<<<<<<
 * 
 *     def get_arrays(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->n_calls); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1151; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":1150
 *         self.n_calls = 0
 * 
 *     def get_n_calls(self):             # <<<<<<<<<<<<<<
 *         return self.n_calls
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.get_n_calls", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1153
 *         return self.n_calls
 * 
 *     def get_arrays(self):             # <<<<<<<<<<<<<<
 *         return (self.data_arr, self.idx_array_arr,
 *                 self.node_data_arr, self.node_bounds_arr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_17get_arrays(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_17get_arrays(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_arrays (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_16get_arrays(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_16get_arrays(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_arrays", 0);

  /* "binary_tree.pxi":1154
 * 
 *     def get_arrays(self):
 *         return (self.data_arr, self.idx_array_arr,             # <<<<<<<<<<<<<<
 *                 self.node_data_arr, self.node_bounds_arr)
 * 
 */
  __Pyx_XDECREF(__pyx_r);

  /* "binary_tree.pxi":1155
 *     def get_arrays(self):
 *         return (self.data_arr, self.idx_array_arr,
 *                 self.node_data_arr, self.node_bounds_arr)             # <<<<<<<<<<<<<<
 * 
 *     cdef inline DTYPE_t dist(self, DTYPE_t* x1, DTYPE_t* x2,
 */
  __pyx_t_1 = PyTuple_New(4); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1154; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->data_arr));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self->data_arr));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->data_arr));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->idx_array_arr));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self->idx_array_arr));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->idx_array_arr));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->node_data_arr));
  PyTuple_SET_ITEM(__pyx_t_1, 2, ((PyObject *)__pyx_v_self->node_data_arr));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->node_data_arr));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->node_bounds_arr));
  PyTuple_SET_ITEM(__pyx_t_1, 3, ((PyObject *)__pyx_v_self->node_bounds_arr));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->node_bounds_arr));
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":1153
 *         return self.n_calls
 * 
 *     def get_arrays(self):             # <<<<<<<<<<<<<<
 *         return (self.data_arr, self.idx_array_arr,
 *                 self.node_data_arr, self.node_bounds_arr)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.get_arrays", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1157
 *                 self.node_data_arr, self.node_bounds_arr)
 * 
 *     cdef inline DTYPE_t dist(self, DTYPE_t* x1, DTYPE_t* x2,             # <<<<<<<<<<<<<<
 *                              ITYPE_t size) nogil except -1:
 *         """Compute the distance between arrays x1 and x2"""
 */

static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree_dist(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_x1, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_x2, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_size) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  int __pyx_t_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "binary_tree.pxi":1160
 *                              ITYPE_t size) nogil except -1:
 *         """Compute the distance between arrays x1 and x2"""
 *         self.n_calls += 1             # <<<<<<<<<<<<<<
 *         if self.euclidean:
 *             return euclidean_dist(x1, x2, size)
 */
  __pyx_v_self->n_calls = (__pyx_v_self->n_calls + 1);

  /* "binary_tree.pxi":1161
 *         """Compute the distance between arrays x1 and x2"""
 *         self.n_calls += 1
 *         if self.euclidean:             # <<<<<<<<<<<<<<
 *             return euclidean_dist(x1, x2, size)
 *         else:
 */
  __pyx_t_1 = (__pyx_v_self->euclidean != 0);
  if (__pyx_t_1) {

    /* "binary_tree.pxi":1162
 *         self.n_calls += 1
 *         if self.euclidean:
 *             return euclidean_dist(x1, x2, size)             # <<<<<<<<<<<<<<
 *         else:
 *             return self.dist_metric.dist(x1, x2, size)
 */
    __pyx_t_2 = __pyx_f_7sklearn_9neighbors_12dist_metrics_euclidean_dist(__pyx_v_x1, __pyx_v_x2, __pyx_v_size); if (unlikely(__pyx_t_2 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1162; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_r = __pyx_t_2;
    goto __pyx_L0;
  }
  /*else*/ {

    /* "binary_tree.pxi":1164
 *             return euclidean_dist(x1, x2, size)
 *         else:
 *             return self.dist_metric.dist(x1, x2, size)             # <<<<<<<<<<<<<<
 * 
 *     cdef inline DTYPE_t rdist(self, DTYPE_t* x1, DTYPE_t* x2,
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_12dist_metrics_DistanceMetric *)__pyx_v_self->dist_metric->__pyx_vtab)->dist(__pyx_v_self->dist_metric, __pyx_v_x1, __pyx_v_x2, __pyx_v_size); if (unlikely(__pyx_t_2 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_r = __pyx_t_2;
    goto __pyx_L0;
  }

  /* "binary_tree.pxi":1157
 *                 self.node_data_arr, self.node_bounds_arr)
 * 
 *     cdef inline DTYPE_t dist(self, DTYPE_t* x1, DTYPE_t* x2,             # <<<<<<<<<<<<<<
 *                              ITYPE_t size) nogil except -1:
 *         """Compute the distance between arrays x1 and x2"""
 */

  /* function exit code */
  __pyx_L1_error:;
  {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
    #endif
    __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.dist", __pyx_clineno, __pyx_lineno, __pyx_filename);
    #ifdef WITH_THREAD
    PyGILState_Release(__pyx_gilstate_save);
    #endif
  }
  __pyx_r = -1.0;
  __pyx_L0:;
  return __pyx_r;
}

/* "binary_tree.pxi":1166
 *             return self.dist_metric.dist(x1, x2, size)
 * 
 *     cdef inline DTYPE_t rdist(self, DTYPE_t* x1, DTYPE_t* x2,             # <<<<<<<<<<<<<<
 *                               ITYPE_t size) nogil except -1:
 *         """Compute the reduced distance between arrays x1 and x2.
 */

static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree_rdist(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_x1, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_x2, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_size) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  int __pyx_t_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "binary_tree.pxi":1175
 *         distance for the Euclidean metric is the squared-euclidean distance.
 *         """
 *         self.n_calls += 1             # <<<<<<<<<<<<<<
 *         if self.euclidean:
 *             return euclidean_rdist(x1, x2, size)
 */
  __pyx_v_self->n_calls = (__pyx_v_self->n_calls + 1);

  /* "binary_tree.pxi":1176
 *         """
 *         self.n_calls += 1
 *         if self.euclidean:             # <<<<<<<<<<<<<<
 *             return euclidean_rdist(x1, x2, size)
 *         else:
 */
  __pyx_t_1 = (__pyx_v_self->euclidean != 0);
  if (__pyx_t_1) {

    /* "binary_tree.pxi":1177
 *         self.n_calls += 1
 *         if self.euclidean:
 *             return euclidean_rdist(x1, x2, size)             # <<<<<<<<<<<<<<
 *         else:
 *             return self.dist_metric.rdist(x1, x2, size)
 */
    __pyx_t_2 = __pyx_f_7sklearn_9neighbors_12dist_metrics_euclidean_rdist(__pyx_v_x1, __pyx_v_x2, __pyx_v_size); if (unlikely(__pyx_t_2 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1177; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_r = __pyx_t_2;
    goto __pyx_L0;
  }
  /*else*/ {

    /* "binary_tree.pxi":1179
 *             return euclidean_rdist(x1, x2, size)
 *         else:
 *             return self.dist_metric.rdist(x1, x2, size)             # <<<<<<<<<<<<<<
 * 
 *     cdef int _recursive_build(self, ITYPE_t i_node, ITYPE_t idx_start,
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_12dist_metrics_DistanceMetric *)__pyx_v_self->dist_metric->__pyx_vtab)->rdist(__pyx_v_self->dist_metric, __pyx_v_x1, __pyx_v_x2, __pyx_v_size); if (unlikely(__pyx_t_2 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1179; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_r = __pyx_t_2;
    goto __pyx_L0;
  }

  /* "binary_tree.pxi":1166
 *             return self.dist_metric.dist(x1, x2, size)
 * 
 *     cdef inline DTYPE_t rdist(self, DTYPE_t* x1, DTYPE_t* x2,             # <<<<<<<<<<<<<<
 *                               ITYPE_t size) nogil except -1:
 *         """Compute the reduced distance between arrays x1 and x2.
 */

  /* function exit code */
  __pyx_L1_error:;
  {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
    #endif
    __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.rdist", __pyx_clineno, __pyx_lineno, __pyx_filename);
    #ifdef WITH_THREAD
    PyGILState_Release(__pyx_gilstate_save);
    #endif
  }
  __pyx_r = -1.0;
  __pyx_L0:;
  return __pyx_r;
}

/* "binary_tree.pxi":1181
 *             return self.dist_metric.rdist(x1, x2, size)
 * 
 *     cdef int _recursive_build(self, ITYPE_t i_node, ITYPE_t idx_start,             # <<<<<<<<<<<<<<
 *                               ITYPE_t idx_end) except -1:
 *         """Recursively build the tree.
 */

static int __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__recursive_build(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_idx_start, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_idx_end) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_features;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_points;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_mid;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_v_idx_array;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_data;
  PyObject *__pyx_v_warnings = NULL;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_max;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_9;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_10;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_recursive_build", 0);

  /* "binary_tree.pxi":1194
 *         """
 *         cdef ITYPE_t imax
 *         cdef ITYPE_t n_features = self.data.shape[1]             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t n_points = idx_end - idx_start
 *         cdef ITYPE_t n_mid = n_points / 2
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1194; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_n_features = (__pyx_v_self->data.shape[1]);

  /* "binary_tree.pxi":1195
 *         cdef ITYPE_t imax
 *         cdef ITYPE_t n_features = self.data.shape[1]
 *         cdef ITYPE_t n_points = idx_end - idx_start             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t n_mid = n_points / 2
 *         cdef ITYPE_t* idx_array = &self.idx_array[idx_start]
 */
  __pyx_v_n_points = (__pyx_v_idx_end - __pyx_v_idx_start);

  /* "binary_tree.pxi":1196
 *         cdef ITYPE_t n_features = self.data.shape[1]
 *         cdef ITYPE_t n_points = idx_end - idx_start
 *         cdef ITYPE_t n_mid = n_points / 2             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t* idx_array = &self.idx_array[idx_start]
 *         cdef DTYPE_t* data = &self.data[0, 0]
 */
  __pyx_v_n_mid = (__pyx_v_n_points / 2);

  /* "binary_tree.pxi":1197
 *         cdef ITYPE_t n_points = idx_end - idx_start
 *         cdef ITYPE_t n_mid = n_points / 2
 *         cdef ITYPE_t* idx_array = &self.idx_array[idx_start]             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t* data = &self.data[0, 0]
 * 
 */
  if (unlikely(!__pyx_v_self->idx_array.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1197; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_1 = __pyx_v_idx_start;
  __pyx_v_idx_array = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_self->idx_array.data) + __pyx_t_1)) ))));

  /* "binary_tree.pxi":1198
 *         cdef ITYPE_t n_mid = n_points / 2
 *         cdef ITYPE_t* idx_array = &self.idx_array[idx_start]
 *         cdef DTYPE_t* data = &self.data[0, 0]             # <<<<<<<<<<<<<<
 * 
 *         # initialize node data
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1198; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_data = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_self->data.data + __pyx_t_2 * __pyx_v_self->data.strides[0]) )) + __pyx_t_3)) ))));

  /* "binary_tree.pxi":1201
 * 
 *         # initialize node data
 *         init_node(self, i_node, idx_start, idx_end)             # <<<<<<<<<<<<<<
 * 
 *         if 2 * i_node + 1 >= self.n_nodes:
 */
  __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree_init_node(__pyx_v_self, __pyx_v_i_node, __pyx_v_idx_start, __pyx_v_idx_end); if (unlikely(__pyx_t_4 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1201; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "binary_tree.pxi":1203
 *         init_node(self, i_node, idx_start, idx_end)
 * 
 *         if 2 * i_node + 1 >= self.n_nodes:             # <<<<<<<<<<<<<<
 *             self.node_data[i_node].is_leaf = True
 *             if idx_end - idx_start > 2 * self.leaf_size:
 */
  __pyx_t_5 = ((((2 * __pyx_v_i_node) + 1) >= __pyx_v_self->n_nodes) != 0);
  if (__pyx_t_5) {

    /* "binary_tree.pxi":1204
 * 
 *         if 2 * i_node + 1 >= self.n_nodes:
 *             self.node_data[i_node].is_leaf = True             # <<<<<<<<<<<<<<
 *             if idx_end - idx_start > 2 * self.leaf_size:
 *                 # this shouldn't happen if our memory allocation is correct
 */
    if (unlikely(!__pyx_v_self->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1204; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
    __pyx_t_6 = __pyx_v_i_node;
    (*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_self->node_data.data) + __pyx_t_6)) ))).is_leaf = 1;

    /* "binary_tree.pxi":1205
 *         if 2 * i_node + 1 >= self.n_nodes:
 *             self.node_data[i_node].is_leaf = True
 *             if idx_end - idx_start > 2 * self.leaf_size:             # <<<<<<<<<<<<<<
 *                 # this shouldn't happen if our memory allocation is correct
 *                 # we'll proactively prevent memory errors, but raise a
 */
    __pyx_t_5 = (((__pyx_v_idx_end - __pyx_v_idx_start) > (2 * __pyx_v_self->leaf_size)) != 0);
    if (__pyx_t_5) {

      /* "binary_tree.pxi":1209
 *                 # we'll proactively prevent memory errors, but raise a
 *                 # warning saying we're doing so.
 *                 import warnings             # <<<<<<<<<<<<<<
 *                 warnings.warn("Internal: memory layout is flawed: "
 *                               "not enough nodes allocated")
 */
      __pyx_t_7 = __Pyx_Import(__pyx_n_s_warnings, 0, -1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1209; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_v_warnings = __pyx_t_7;
      __pyx_t_7 = 0;

      /* "binary_tree.pxi":1210
 *                 # warning saying we're doing so.
 *                 import warnings
 *                 warnings.warn("Internal: memory layout is flawed: "             # <<<<<<<<<<<<<<
 *                               "not enough nodes allocated")
 * 
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_warnings, __pyx_n_s_warn); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1210; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1210; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      goto __pyx_L4;
    }
    __pyx_L4:;
    goto __pyx_L3;
  }

  /* "binary_tree.pxi":1213
 *                               "not enough nodes allocated")
 * 
 *         elif idx_end - idx_start < 2:             # <<<<<<<<<<<<<<
 *             # again, this shouldn't happen if our memory allocation
 *             # is correct.  Raise a warning.
 */
  __pyx_t_5 = (((__pyx_v_idx_end - __pyx_v_idx_start) < 2) != 0);
  if (__pyx_t_5) {

    /* "binary_tree.pxi":1216
 *             # again, this shouldn't happen if our memory allocation
 *             # is correct.  Raise a warning.
 *             import warnings             # <<<<<<<<<<<<<<
 *             warnings.warn("Internal: memory layout is flawed: "
 *                           "too many nodes allocated")
 */
    __pyx_t_8 = __Pyx_Import(__pyx_n_s_warnings, 0, -1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1216; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_v_warnings = __pyx_t_8;
    __pyx_t_8 = 0;

    /* "binary_tree.pxi":1217
 *             # is correct.  Raise a warning.
 *             import warnings
 *             warnings.warn("Internal: memory layout is flawed: "             # <<<<<<<<<<<<<<
 *                           "too many nodes allocated")
 *             self.node_data[i_node].is_leaf = True
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_warnings, __pyx_n_s_warn); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1217; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1217; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "binary_tree.pxi":1219
 *             warnings.warn("Internal: memory layout is flawed: "
 *                           "too many nodes allocated")
 *             self.node_data[i_node].is_leaf = True             # <<<<<<<<<<<<<<
 * 
 *         else:
 */
    if (unlikely(!__pyx_v_self->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
    __pyx_t_9 = __pyx_v_i_node;
    (*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_self->node_data.data) + __pyx_t_9)) ))).is_leaf = 1;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "binary_tree.pxi":1223
 *         else:
 *             # split node and recursively construct child nodes.
 *             self.node_data[i_node].is_leaf = False             # <<<<<<<<<<<<<<
 *             i_max = find_node_split_dim(data, idx_array,
 *                                         n_features, n_points)
 */
    if (unlikely(!__pyx_v_self->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
    __pyx_t_10 = __pyx_v_i_node;
    (*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_self->node_data.data) + __pyx_t_10)) ))).is_leaf = 0;

    /* "binary_tree.pxi":1224
 *             # split node and recursively construct child nodes.
 *             self.node_data[i_node].is_leaf = False
 *             i_max = find_node_split_dim(data, idx_array,             # <<<<<<<<<<<<<<
 *                                         n_features, n_points)
 *             partition_node_indices(data, idx_array, i_max, n_mid,
 */
    __pyx_t_11 = __pyx_f_7sklearn_9neighbors_7kd_tree_find_node_split_dim(__pyx_v_data, __pyx_v_idx_array, __pyx_v_n_features, __pyx_v_n_points); if (unlikely(__pyx_t_11 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1224; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_i_max = __pyx_t_11;

    /* "binary_tree.pxi":1226
 *             i_max = find_node_split_dim(data, idx_array,
 *                                         n_features, n_points)
 *             partition_node_indices(data, idx_array, i_max, n_mid,             # <<<<<<<<<<<<<<
 *                                    n_features, n_points)
 *             self._recursive_build(2 * i_node + 1,
 */
    __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree_partition_node_indices(__pyx_v_data, __pyx_v_idx_array, __pyx_v_i_max, __pyx_v_n_mid, __pyx_v_n_features, __pyx_v_n_points); if (unlikely(__pyx_t_4 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1226; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "binary_tree.pxi":1228
 *             partition_node_indices(data, idx_array, i_max, n_mid,
 *                                    n_features, n_points)
 *             self._recursive_build(2 * i_node + 1,             # <<<<<<<<<<<<<<
 *                                   idx_start, idx_start + n_mid)
 *             self._recursive_build(2 * i_node + 2,
 */
    __pyx_t_4 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_recursive_build(__pyx_v_self, ((2 * __pyx_v_i_node) + 1), __pyx_v_idx_start, (__pyx_v_idx_start + __pyx_v_n_mid)); if (unlikely(__pyx_t_4 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1228; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "binary_tree.pxi":1230
 *             self._recursive_build(2 * i_node + 1,
 *                                   idx_start, idx_start + n_mid)
 *             self._recursive_build(2 * i_node + 2,             # <<<<<<<<<<<<<<
 *                                   idx_start + n_mid, idx_end)
 * 
 */
    __pyx_t_4 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_recursive_build(__pyx_v_self, ((2 * __pyx_v_i_node) + 2), (__pyx_v_idx_start + __pyx_v_n_mid), __pyx_v_idx_end); if (unlikely(__pyx_t_4 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1230; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L3:;

  /* "binary_tree.pxi":1181
 *             return self.dist_metric.rdist(x1, x2, size)
 * 
 *     cdef int _recursive_build(self, ITYPE_t i_node, ITYPE_t idx_start,             # <<<<<<<<<<<<<<
 *                               ITYPE_t idx_end) except -1:
 *         """Recursively build the tree.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree._recursive_build", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_warnings);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1233
 *                                   idx_start + n_mid, idx_end)
 * 
 *     def query(self, X, k=1, return_distance=True,             # <<<<<<<<<<<<<<
 *               dualtree=False, breadth_first=False,
 *               sort_results=True):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_19query(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7sklearn_9neighbors_7kd_tree_10BinaryTree_18query[] = "\n        query(X, k=1, return_distance=True,\n              dualtree=False, breadth_first=False)\n\n        query the tree for the k nearest neighbors\n\n        Parameters\n        ----------\n        X : array-like, last dimension self.dim\n            An array of points to query\n        k : integer  (default = 1)\n            The number of nearest neighbors to return\n        return_distance : boolean (default = True)\n            if True, return a tuple (d, i) of distances and indices\n            if False, return array i\n        dualtree : boolean (default = False)\n            if True, use the dual tree formalism for the query: a tree is\n            built for the query points, and the pair of trees is used to\n            efficiently search this space.  This can lead to better\n            performance as the number of points grows large.\n        breadth_first : boolean (default = False)\n            if True, then query the nodes in a breadth-first manner.\n            Otherwise, query the nodes in a depth-first manner.\n        sort_results : boolean (default = True)\n            if True, then distances and indices of each point are sorted\n            on return, so that the first column contains the closest points.\n            Otherwise, neighbors are returned in an arbitrary order.\n\n        Returns\n        -------\n        i    : if return_distance == False\n        (d,i) : if return_distance == True\n\n        d : array of doubles - shape: x.shape[:-1] + (k,)\n            each entry gives the list of distances to the\n            neighbors of the corresponding point\n\n        i : array of integers - shape: x.shape[:-1] + (k,)\n            each entry gives the list of indices of\n            neighbors of the corresponding point\n\n        Examples\n        --------\n        Query for k-nearest neighbors\n\n            >>> import numpy as np\n            >>> np.random.seed(0)\n            >>> X = np.random.random((10, 3))  # 10 points in 3 dimension""s\n            >>> tree = BinaryTree(X, leaf_size=2)    # doctest: +SKIP\n            >>> dist, ind = tree.query(X[0], k=3)    # doctest: +SKIP\n            >>> print ind  # indices of 3 closest neighbors\n            [0 3 1]\n            >>> print dist  # distances to 3 closest neighbors\n            [ 0.          0.19662693  0.29473397]\n        ";
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_19query(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_X = 0;
  PyObject *__pyx_v_k = 0;
  PyObject *__pyx_v_return_distance = 0;
  PyObject *__pyx_v_dualtree = 0;
  PyObject *__pyx_v_breadth_first = 0;
  PyObject *__pyx_v_sort_results = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("query (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_X,&__pyx_n_s_k,&__pyx_n_s_return_distance,&__pyx_n_s_dualtree,&__pyx_n_s_breadth_first,&__pyx_n_s_sort_results,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    values[1] = ((PyObject *)__pyx_int_1);
    values[2] = ((PyObject *)Py_True);

    /* "binary_tree.pxi":1234
 * 
 *     def query(self, X, k=1, return_distance=True,
 *               dualtree=False, breadth_first=False,             # <<<<<<<<<<<<<<
 *               sort_results=True):
 *         """
 */
    values[3] = ((PyObject *)Py_False);
    values[4] = ((PyObject *)Py_False);

    /* "binary_tree.pxi":1235
 *     def query(self, X, k=1, return_distance=True,
 *               dualtree=False, breadth_first=False,
 *               sort_results=True):             # <<<<<<<<<<<<<<
 *         """
 *         query(X, k=1, return_distance=True,
 */
    values[5] = ((PyObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_X)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_k);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_return_distance);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_dualtree);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_breadth_first);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sort_results);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "query") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1233; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_X = values[0];
    __pyx_v_k = values[1];
    __pyx_v_return_distance = values[2];
    __pyx_v_dualtree = values[3];
    __pyx_v_breadth_first = values[4];
    __pyx_v_sort_results = values[5];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("query", 0, 1, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1233; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.query", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_18query(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self), __pyx_v_X, __pyx_v_k, __pyx_v_return_distance, __pyx_v_dualtree, __pyx_v_breadth_first, __pyx_v_sort_results);

  /* "binary_tree.pxi":1233
 *                                   idx_start + n_mid, idx_end)
 * 
 *     def query(self, X, k=1, return_distance=True,             # <<<<<<<<<<<<<<
 *               dualtree=False, breadth_first=False,
 *               sort_results=True):
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_18query(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, PyObject *__pyx_v_X, PyObject *__pyx_v_k, PyObject *__pyx_v_return_distance, PyObject *__pyx_v_dualtree, PyObject *__pyx_v_breadth_first, PyObject *__pyx_v_sort_results) {
  PyObject *__pyx_v_np_Xarr = NULL;
  __Pyx_memviewslice __pyx_v_Xarr = { 0, 0, { 0 }, { 0 }, { 0 } };
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_reduced_dist_LB;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt;
  struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_heap = 0;
  struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_nodeheap = 0;
  __Pyx_memviewslice __pyx_v_bounds = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_v_other = NULL;
  PyObject *__pyx_v_distances = NULL;
  PyObject *__pyx_v_indices = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_t_8;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_memviewslice __pyx_t_11 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_15;
  PyObject *(*__pyx_t_16)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("query", 0);
  __Pyx_INCREF(__pyx_v_X);

  /* "binary_tree.pxi":1292
 *         """
 *         # XXX: we should allow X to be a pre-built tree.
 *         X = check_array(X, dtype=DTYPE, order='C')             # <<<<<<<<<<<<<<
 * 
 *         if X.shape[X.ndim - 1] != self.data.shape[1]:
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_check_array); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_X);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_X);
  __Pyx_GIVEREF(__pyx_v_X);
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF_SET(__pyx_v_X, __pyx_t_4);
  __pyx_t_4 = 0;

  /* "binary_tree.pxi":1294
 *         X = check_array(X, dtype=DTYPE, order='C')
 * 
 *         if X.shape[X.ndim - 1] != self.data.shape[1]:             # <<<<<<<<<<<<<<
 *             raise ValueError("query data dimension must "
 *                              "match training data dimension")
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_shape); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_ndim); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyNumber_Subtract(__pyx_t_3, __pyx_int_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyObject_GetItem(__pyx_t_4, __pyx_t_2); if (unlikely(__pyx_t_3 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_2 = PyInt_FromSsize_t((__pyx_v_self->data.shape[1])); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_t_2, Py_NE); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_5) {

    /* "binary_tree.pxi":1295
 * 
 *         if X.shape[X.ndim - 1] != self.data.shape[1]:
 *             raise ValueError("query data dimension must "             # <<<<<<<<<<<<<<
 *                              "match training data dimension")
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1295; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1295; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "binary_tree.pxi":1298
 *                              "match training data dimension")
 * 
 *         if self.data.shape[0] < k:             # <<<<<<<<<<<<<<
 *             raise ValueError("k must be less than or equal "
 *                              "to the number of training points")
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_4 = PyInt_FromSsize_t((__pyx_v_self->data.shape[0])); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_4, __pyx_v_k, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_5) {

    /* "binary_tree.pxi":1299
 * 
 *         if self.data.shape[0] < k:
 *             raise ValueError("k must be less than or equal "             # <<<<<<<<<<<<<<
 *                              "to the number of training points")
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__20, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "binary_tree.pxi":1303
 * 
 *         # flatten X, and save original shape information
 *         np_Xarr = X.reshape((-1, self.data.shape[1]))             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t[:, ::1] Xarr = get_memview_DTYPE_2D(np_Xarr)
 *         cdef DTYPE_t reduced_dist_LB
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_reshape); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_3 = PyInt_FromSsize_t((__pyx_v_self->data.shape[1])); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_neg_1);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_int_neg_1);
  __Pyx_GIVEREF(__pyx_int_neg_1);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3); __Pyx_GIVEREF(__pyx_t_3); __pyx_t_3 = NULL;
    PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_np_Xarr = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "binary_tree.pxi":1304
 *         # flatten X, and save original shape information
 *         np_Xarr = X.reshape((-1, self.data.shape[1]))
 *         cdef DTYPE_t[:, ::1] Xarr = get_memview_DTYPE_2D(np_Xarr)             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t reduced_dist_LB
 *         cdef ITYPE_t i
 */
  if (!(likely(((__pyx_v_np_Xarr) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_np_Xarr, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1304; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_7 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_2D(((PyArrayObject *)__pyx_v_np_Xarr)); if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1304; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_Xarr = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "binary_tree.pxi":1310
 * 
 *         # initialize heap for neighbors
 *         cdef NeighborsHeap heap = NeighborsHeap(Xarr.shape[0], k)             # <<<<<<<<<<<<<<
 * 
 *         # node heap for breadth-first queries
 */
  __pyx_t_2 = PyInt_FromSsize_t((__pyx_v_Xarr.shape[0])); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1310; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1310; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_k);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_k);
  __Pyx_GIVEREF(__pyx_v_k);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)((PyObject*)__pyx_ptype_7sklearn_9neighbors_7kd_tree_NeighborsHeap)), __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1310; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_heap = ((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "binary_tree.pxi":1314
 *         # node heap for breadth-first queries
 *         cdef NodeHeap nodeheap
 *         if breadth_first:             # <<<<<<<<<<<<<<
 *             nodeheap = NodeHeap(self.data.shape[0] // self.leaf_size)
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_breadth_first); if (unlikely(__pyx_t_5 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_5) {

    /* "binary_tree.pxi":1315
 *         cdef NodeHeap nodeheap
 *         if breadth_first:
 *             nodeheap = NodeHeap(self.data.shape[0] // self.leaf_size)             # <<<<<<<<<<<<<<
 * 
 *         # bounds is needed for the dual tree algorithm
 */
    if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1315; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
    __pyx_t_2 = PyInt_FromSsize_t(((__pyx_v_self->data.shape[0]) / __pyx_v_self->leaf_size)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1315; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1315; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)((PyObject*)__pyx_ptype_7sklearn_9neighbors_7kd_tree_NodeHeap)), __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1315; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_nodeheap = ((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L5;
  }
  __pyx_L5:;

  /* "binary_tree.pxi":1320
 *         cdef DTYPE_t[::1] bounds
 * 
 *         self.n_trims = 0             # <<<<<<<<<<<<<<
 *         self.n_leaves = 0
 *         self.n_splits = 0
 */
  __pyx_v_self->n_trims = 0;

  /* "binary_tree.pxi":1321
 * 
 *         self.n_trims = 0
 *         self.n_leaves = 0             # <<<<<<<<<<<<<<
 *         self.n_splits = 0
 * 
 */
  __pyx_v_self->n_leaves = 0;

  /* "binary_tree.pxi":1322
 *         self.n_trims = 0
 *         self.n_leaves = 0
 *         self.n_splits = 0             # <<<<<<<<<<<<<<
 * 
 *         if dualtree:
 */
  __pyx_v_self->n_splits = 0;

  /* "binary_tree.pxi":1324
 *         self.n_splits = 0
 * 
 *         if dualtree:             # <<<<<<<<<<<<<<
 *             other = self.__class__(np_Xarr, metric=self.dist_metric,
 *                                    leaf_size=self.leaf_size)
 */
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_dualtree); if (unlikely(__pyx_t_5 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1324; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_5) {

    /* "binary_tree.pxi":1325
 * 
 *         if dualtree:
 *             other = self.__class__(np_Xarr, metric=self.dist_metric,             # <<<<<<<<<<<<<<
 *                                    leaf_size=self.leaf_size)
 *             if breadth_first:
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1325; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1325; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v_np_Xarr);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_np_Xarr);
    __Pyx_GIVEREF(__pyx_v_np_Xarr);
    __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1325; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_metric, ((PyObject *)__pyx_v_self->dist_metric)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1325; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "binary_tree.pxi":1326
 *         if dualtree:
 *             other = self.__class__(np_Xarr, metric=self.dist_metric,
 *                                    leaf_size=self.leaf_size)             # <<<<<<<<<<<<<<
 *             if breadth_first:
 *                 self._query_dual_breadthfirst(other, heap, nodeheap)
 */
    __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->leaf_size); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1326; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_leaf_size, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1325; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "binary_tree.pxi":1325
 * 
 *         if dualtree:
 *             other = self.__class__(np_Xarr, metric=self.dist_metric,             # <<<<<<<<<<<<<<
 *                                    leaf_size=self.leaf_size)
 *             if breadth_first:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1325; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_other = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "binary_tree.pxi":1327
 *             other = self.__class__(np_Xarr, metric=self.dist_metric,
 *                                    leaf_size=self.leaf_size)
 *             if breadth_first:             # <<<<<<<<<<<<<<
 *                 self._query_dual_breadthfirst(other, heap, nodeheap)
 *             else:
 */
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_breadth_first); if (unlikely(__pyx_t_5 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1327; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (__pyx_t_5) {

      /* "binary_tree.pxi":1328
 *                                    leaf_size=self.leaf_size)
 *             if breadth_first:
 *                 self._query_dual_breadthfirst(other, heap, nodeheap)             # <<<<<<<<<<<<<<
 *             else:
 *                 reduced_dist_LB = min_rdist_dual(self, 0, other, 0)
 */
      if (!(likely(((__pyx_v_other) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_other, __pyx_ptype_7sklearn_9neighbors_7kd_tree_BinaryTree))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1328; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      if (unlikely(!__pyx_v_nodeheap)) { __Pyx_RaiseUnboundLocalError("nodeheap"); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1328; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }
      __pyx_t_8 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_query_dual_breadthfirst(__pyx_v_self, ((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_other), __pyx_v_heap, __pyx_v_nodeheap); if (unlikely(__pyx_t_8 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1328; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      goto __pyx_L7;
    }
    /*else*/ {

      /* "binary_tree.pxi":1330
 *                 self._query_dual_breadthfirst(other, heap, nodeheap)
 *             else:
 *                 reduced_dist_LB = min_rdist_dual(self, 0, other, 0)             # <<<<<<<<<<<<<<
 *                 bounds = np.inf + np.zeros(other.node_data.shape[0])
 *                 self._query_dual_depthfirst(0, other, 0, bounds,
 */
      if (!(likely(((__pyx_v_other) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_other, __pyx_ptype_7sklearn_9neighbors_7kd_tree_BinaryTree))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1330; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_t_9 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_rdist_dual(__pyx_v_self, 0, ((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_other), 0); if (unlikely(__pyx_t_9 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1330; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_v_reduced_dist_LB = __pyx_t_9;

      /* "binary_tree.pxi":1331
 *             else:
 *                 reduced_dist_LB = min_rdist_dual(self, 0, other, 0)
 *                 bounds = np.inf + np.zeros(other.node_data.shape[0])             # <<<<<<<<<<<<<<
 *                 self._query_dual_depthfirst(0, other, 0, bounds,
 *                                             heap, reduced_dist_LB)
 */
      __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1331; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_inf); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1331; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1331; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1331; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_other, __pyx_n_s_node_data); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1331; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1331; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_GetItemInt(__pyx_t_3, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_4 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1331; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      if (!__pyx_t_3) {
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1331; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else {
        __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1331; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_10);
        PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_3); __Pyx_GIVEREF(__pyx_t_3); __pyx_t_3 = NULL;
        PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_10, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1331; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = PyNumber_Add(__pyx_t_6, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1331; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_11 = __Pyx_PyObject_to_MemoryviewSlice_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t(__pyx_t_2);
      if (unlikely(!__pyx_t_11.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1331; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_v_bounds = __pyx_t_11;
      __pyx_t_11.memview = NULL;
      __pyx_t_11.data = NULL;

      /* "binary_tree.pxi":1332
 *                 reduced_dist_LB = min_rdist_dual(self, 0, other, 0)
 *                 bounds = np.inf + np.zeros(other.node_data.shape[0])
 *                 self._query_dual_depthfirst(0, other, 0, bounds,             # <<<<<<<<<<<<<<
 *                                             heap, reduced_dist_LB)
 * 
 */
      if (!(likely(((__pyx_v_other) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_other, __pyx_ptype_7sklearn_9neighbors_7kd_tree_BinaryTree))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "binary_tree.pxi":1333
 *                 bounds = np.inf + np.zeros(other.node_data.shape[0])
 *                 self._query_dual_depthfirst(0, other, 0, bounds,
 *                                             heap, reduced_dist_LB)             # <<<<<<<<<<<<<<
 * 
 *         else:
 */
      __pyx_t_8 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_query_dual_depthfirst(__pyx_v_self, 0, ((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_other), 0, __pyx_v_bounds, __pyx_v_heap, __pyx_v_reduced_dist_LB); if (unlikely(__pyx_t_8 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_L7:;
    goto __pyx_L6;
  }
  /*else*/ {

    /* "binary_tree.pxi":1336
 * 
 *         else:
 *             pt = &Xarr[0, 0]             # <<<<<<<<<<<<<<
 *             if breadth_first:
 *                 for i in range(Xarr.shape[0]):
 */
    __pyx_t_12 = 0;
    __pyx_t_13 = 0;
    __pyx_v_pt = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_Xarr.data + __pyx_t_12 * __pyx_v_Xarr.strides[0]) )) + __pyx_t_13)) ))));

    /* "binary_tree.pxi":1337
 *         else:
 *             pt = &Xarr[0, 0]
 *             if breadth_first:             # <<<<<<<<<<<<<<
 *                 for i in range(Xarr.shape[0]):
 *                     self._query_single_breadthfirst(pt, i, heap, nodeheap)
 */
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_breadth_first); if (unlikely(__pyx_t_5 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (__pyx_t_5) {

      /* "binary_tree.pxi":1338
 *             pt = &Xarr[0, 0]
 *             if breadth_first:
 *                 for i in range(Xarr.shape[0]):             # <<<<<<<<<<<<<<
 *                     self._query_single_breadthfirst(pt, i, heap, nodeheap)
 *                     pt += Xarr.shape[1]
 */
      __pyx_t_14 = (__pyx_v_Xarr.shape[0]);
      for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
        __pyx_v_i = __pyx_t_15;

        /* "binary_tree.pxi":1339
 *             if breadth_first:
 *                 for i in range(Xarr.shape[0]):
 *                     self._query_single_breadthfirst(pt, i, heap, nodeheap)             # <<<<<<<<<<<<<<
 *                     pt += Xarr.shape[1]
 *             else:
 */
        if (unlikely(!__pyx_v_nodeheap)) { __Pyx_RaiseUnboundLocalError("nodeheap"); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1339; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }
        __pyx_t_8 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_query_single_breadthfirst(__pyx_v_self, __pyx_v_pt, __pyx_v_i, __pyx_v_heap, __pyx_v_nodeheap); if (unlikely(__pyx_t_8 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1339; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

        /* "binary_tree.pxi":1340
 *                 for i in range(Xarr.shape[0]):
 *                     self._query_single_breadthfirst(pt, i, heap, nodeheap)
 *                     pt += Xarr.shape[1]             # <<<<<<<<<<<<<<
 *             else:
 *                 with nogil:
 */
        __pyx_v_pt = (__pyx_v_pt + (__pyx_v_Xarr.shape[1]));
      }
      goto __pyx_L8;
    }
    /*else*/ {

      /* "binary_tree.pxi":1342
 *                     pt += Xarr.shape[1]
 *             else:
 *                 with nogil:             # <<<<<<<<<<<<<<
 *                     for i in range(Xarr.shape[0]):
 *                         reduced_dist_LB = min_rdist(self, 0, pt)
 */
      {
          #ifdef WITH_THREAD
          PyThreadState *_save;
          Py_UNBLOCK_THREADS
          #endif
          /*try:*/ {

            /* "binary_tree.pxi":1343
 *             else:
 *                 with nogil:
 *                     for i in range(Xarr.shape[0]):             # <<<<<<<<<<<<<<
 *                         reduced_dist_LB = min_rdist(self, 0, pt)
 *                         self._query_single_depthfirst(0, pt, i, heap,
 */
            __pyx_t_14 = (__pyx_v_Xarr.shape[0]);
            for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
              __pyx_v_i = __pyx_t_15;

              /* "binary_tree.pxi":1344
 *                 with nogil:
 *                     for i in range(Xarr.shape[0]):
 *                         reduced_dist_LB = min_rdist(self, 0, pt)             # <<<<<<<<<<<<<<
 *                         self._query_single_depthfirst(0, pt, i, heap,
 *                                                       reduced_dist_LB)
 */
              __pyx_t_9 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_rdist(__pyx_v_self, 0, __pyx_v_pt); if (unlikely(__pyx_t_9 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1344; __pyx_clineno = __LINE__; goto __pyx_L12_error;}
              __pyx_v_reduced_dist_LB = __pyx_t_9;

              /* "binary_tree.pxi":1345
 *                     for i in range(Xarr.shape[0]):
 *                         reduced_dist_LB = min_rdist(self, 0, pt)
 *                         self._query_single_depthfirst(0, pt, i, heap,             # <<<<<<<<<<<<<<
 *                                                       reduced_dist_LB)
 *                         pt += Xarr.shape[1]
 */
              __pyx_t_8 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_query_single_depthfirst(__pyx_v_self, 0, __pyx_v_pt, __pyx_v_i, __pyx_v_heap, __pyx_v_reduced_dist_LB); if (unlikely(__pyx_t_8 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L12_error;}

              /* "binary_tree.pxi":1347
 *                         self._query_single_depthfirst(0, pt, i, heap,
 *                                                       reduced_dist_LB)
 *                         pt += Xarr.shape[1]             # <<<<<<<<<<<<<<
 * 
 *         distances, indices = heap.get_arrays(sort=sort_results)
 */
              __pyx_v_pt = (__pyx_v_pt + (__pyx_v_Xarr.shape[1]));
            }
          }

          /* "binary_tree.pxi":1342
 *                     pt += Xarr.shape[1]
 *             else:
 *                 with nogil:             # <<<<<<<<<<<<<<
 *                     for i in range(Xarr.shape[0]):
 *                         reduced_dist_LB = min_rdist(self, 0, pt)
 */
          /*finally:*/ {
            /*normal exit:*/{
              #ifdef WITH_THREAD
              Py_BLOCK_THREADS
              #endif
              goto __pyx_L13;
            }
            __pyx_L12_error: {
              #ifdef WITH_THREAD
              Py_BLOCK_THREADS
              #endif
              goto __pyx_L1_error;
            }
            __pyx_L13:;
          }
      }
    }
    __pyx_L8:;
  }
  __pyx_L6:;

  /* "binary_tree.pxi":1349
 *                         pt += Xarr.shape[1]
 * 
 *         distances, indices = heap.get_arrays(sort=sort_results)             # <<<<<<<<<<<<<<
 *         distances = self.dist_metric.rdist_to_dist(distances)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_heap), __pyx_n_s_get_arrays); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1349; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1349; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_sort, __pyx_v_sort_results) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1349; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1349; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if ((likely(PyTuple_CheckExact(__pyx_t_6))) || (PyList_CheckExact(__pyx_t_6))) {
    PyObject* sequence = __pyx_t_6;
    #if CYTHON_COMPILING_IN_CPYTHON
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1349; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    #if CYTHON_COMPILING_IN_CPYTHON
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_1 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_2 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_2);
    #else
    __pyx_t_1 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1349; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1349; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    #endif
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_10 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1349; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_16 = Py_TYPE(__pyx_t_10)->tp_iternext;
    index = 0; __pyx_t_1 = __pyx_t_16(__pyx_t_10); if (unlikely(!__pyx_t_1)) goto __pyx_L16_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_1);
    index = 1; __pyx_t_2 = __pyx_t_16(__pyx_t_10); if (unlikely(!__pyx_t_2)) goto __pyx_L16_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_16(__pyx_t_10), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1349; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_16 = NULL;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    goto __pyx_L17_unpacking_done;
    __pyx_L16_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_16 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1349; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_L17_unpacking_done:;
  }
  __pyx_v_distances = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_v_indices = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "binary_tree.pxi":1350
 * 
 *         distances, indices = heap.get_arrays(sort=sort_results)
 *         distances = self.dist_metric.rdist_to_dist(distances)             # <<<<<<<<<<<<<<
 * 
 *         # deflatten results
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->dist_metric), __pyx_n_s_rdist_to_dist); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1350; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_1) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_distances); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1350; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1350; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_1); __Pyx_GIVEREF(__pyx_t_1); __pyx_t_1 = NULL;
    __Pyx_INCREF(__pyx_v_distances);
    PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_v_distances);
    __Pyx_GIVEREF(__pyx_v_distances);
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_10, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1350; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_distances, __pyx_t_6);
  __pyx_t_6 = 0;

  /* "binary_tree.pxi":1353
 * 
 *         # deflatten results
 *         if return_distance:             # <<<<<<<<<<<<<<
 *             return (distances.reshape(X.shape[:X.ndim - 1] + (k,)),
 *                     indices.reshape(X.shape[:X.ndim - 1] + (k,)))
 */
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_return_distance); if (unlikely(__pyx_t_5 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1353; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_5) {

    /* "binary_tree.pxi":1354
 *         # deflatten results
 *         if return_distance:
 *             return (distances.reshape(X.shape[:X.ndim - 1] + (k,)),             # <<<<<<<<<<<<<<
 *                     indices.reshape(X.shape[:X.ndim - 1] + (k,)))
 *         else:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_distances, __pyx_n_s_reshape); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1354; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_shape); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1354; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_ndim); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1354; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = PyNumber_Subtract(__pyx_t_1, __pyx_int_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1354; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_t_10, 0, 0, NULL, &__pyx_t_4, NULL, 0, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1354; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1354; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v_k);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_k);
    __Pyx_GIVEREF(__pyx_v_k);
    __pyx_t_10 = PyNumber_Add(__pyx_t_1, __pyx_t_4); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1354; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_4) {
      __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_10); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1354; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_GOTREF(__pyx_t_6);
    } else {
      __pyx_t_1 = PyTuple_New(1+1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1354; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4); __Pyx_GIVEREF(__pyx_t_4); __pyx_t_4 = NULL;
      PyTuple_SET_ITEM(__pyx_t_1, 0+1, __pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_10);
      __pyx_t_10 = 0;
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1354; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "binary_tree.pxi":1355
 *         if return_distance:
 *             return (distances.reshape(X.shape[:X.ndim - 1] + (k,)),
 *                     indices.reshape(X.shape[:X.ndim - 1] + (k,)))             # <<<<<<<<<<<<<<
 *         else:
 *             return indices.reshape(X.shape[:X.ndim - 1] + (k,))
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_indices, __pyx_n_s_reshape); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1355; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_shape); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1355; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_ndim); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1355; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = PyNumber_Subtract(__pyx_t_4, __pyx_int_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1355; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetSlice(__pyx_t_10, 0, 0, NULL, &__pyx_t_3, NULL, 0, 0, 0); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1355; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1355; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_k);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_k);
    __Pyx_GIVEREF(__pyx_v_k);
    __pyx_t_10 = PyNumber_Add(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1355; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    if (!__pyx_t_3) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_10); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1355; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1355; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __Pyx_GIVEREF(__pyx_t_3); __pyx_t_3 = NULL;
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_10);
      __pyx_t_10 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1355; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "binary_tree.pxi":1354
 *         # deflatten results
 *         if return_distance:
 *             return (distances.reshape(X.shape[:X.ndim - 1] + (k,)),             # <<<<<<<<<<<<<<
 *                     indices.reshape(X.shape[:X.ndim - 1] + (k,)))
 *         else:
 */
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1354; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_2);
    __pyx_t_6 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }
  /*else*/ {

    /* "binary_tree.pxi":1357
 *                     indices.reshape(X.shape[:X.ndim - 1] + (k,)))
 *         else:
 *             return indices.reshape(X.shape[:X.ndim - 1] + (k,))             # <<<<<<<<<<<<<<
 * 
 *     def query_radius(self, X, r, return_distance=False,
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_indices, __pyx_n_s_reshape); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_shape); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_ndim); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_10 = PyNumber_Subtract(__pyx_t_4, __pyx_int_1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetSlice(__pyx_t_6, 0, 0, NULL, &__pyx_t_10, NULL, 0, 0, 0); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = PyTuple_New(1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_INCREF(__pyx_v_k);
    PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_v_k);
    __Pyx_GIVEREF(__pyx_v_k);
    __pyx_t_6 = PyNumber_Add(__pyx_t_4, __pyx_t_10); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_10) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_6); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_10); __Pyx_GIVEREF(__pyx_t_10); __pyx_t_10 = NULL;
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "binary_tree.pxi":1233
 *                                   idx_start + n_mid, idx_end)
 * 
 *     def query(self, X, k=1, return_distance=True,             # <<<<<<<<<<<<<<
 *               dualtree=False, breadth_first=False,
 *               sort_results=True):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_XDECREF(__pyx_t_10);
  __PYX_XDEC_MEMVIEW(&__pyx_t_11, 1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.query", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_np_Xarr);
  __PYX_XDEC_MEMVIEW(&__pyx_v_Xarr, 1);
  __Pyx_XDECREF((PyObject *)__pyx_v_heap);
  __Pyx_XDECREF((PyObject *)__pyx_v_nodeheap);
  __PYX_XDEC_MEMVIEW(&__pyx_v_bounds, 1);
  __Pyx_XDECREF(__pyx_v_other);
  __Pyx_XDECREF(__pyx_v_distances);
  __Pyx_XDECREF(__pyx_v_indices);
  __Pyx_XDECREF(__pyx_v_X);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1359
 *             return indices.reshape(X.shape[:X.ndim - 1] + (k,))
 * 
 *     def query_radius(self, X, r, return_distance=False,             # <<<<<<<<<<<<<<
 *                      int count_only=False, int sort_results=False):
 *         """
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_21query_radius(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7sklearn_9neighbors_7kd_tree_10BinaryTree_20query_radius[] = "\n        query_radius(self, X, r, count_only = False):\n\n        query the tree for neighbors within a radius r\n\n        Parameters\n        ----------\n        X : array-like, last dimension self.dim\n            An array of points to query\n        r : distance within which neighbors are returned\n            r can be a single value, or an array of values of shape\n            x.shape[:-1] if different radii are desired for each point.\n        return_distance : boolean (default = False)\n            if True,  return distances to neighbors of each point\n            if False, return only neighbors\n            Note that unlike the query() method, setting return_distance=True\n            here adds to the computation time.  Not all distances need to be\n            calculated explicitly for return_distance=False.  Results are\n            not sorted by default: see ``sort_results`` keyword.\n        count_only : boolean (default = False)\n            if True,  return only the count of points within distance r\n            if False, return the indices of all points within distance r\n            If return_distance==True, setting count_only=True will\n            result in an error.\n        sort_results : boolean (default = False)\n            if True, the distances and indices will be sorted before being\n            returned.  If False, the results will not be sorted.  If\n            return_distance == False, setting sort_results = True will\n            result in an error.\n\n        Returns\n        -------\n        count       : if count_only == True\n        ind         : if count_only == False and return_distance == False\n        (ind, dist) : if count_only == False and return_distance == True\n\n        count : array of integers, shape = X.shape[:-1]\n            each entry gives the number of neighbors within\n            a distance r of the corresponding point.\n\n        ind : array of objects, shape = X.shape[:-1]\n            each element is a num""py integer array listing the indices of\n            neighbors of the corresponding point.  Note that unlike\n            the results of a k-neighbors query, the returned neighbors\n            are not sorted by distance by default.\n\n        dist : array of objects, shape = X.shape[:-1]\n            each element is a numpy double array\n            listing the distances corresponding to indices in i.\n\n        Examples\n        --------\n        Query for neighbors in a given radius\n\n        >>> import numpy as np\n        >>> np.random.seed(0)\n        >>> X = np.random.random((10, 3))  # 10 points in 3 dimensions\n        >>> tree = BinaryTree(X, leaf_size=2)     # doctest: +SKIP\n        >>> print tree.query_radius(X[0], r=0.3, count_only=True)\n        3\n        >>> ind = tree.query_radius(X[0], r=0.3)  # doctest: +SKIP\n        >>> print ind  # indices of neighbors within distance 0.3\n        [3 0 1]\n        ";
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_21query_radius(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_X = 0;
  PyObject *__pyx_v_r = 0;
  PyObject *__pyx_v_return_distance = 0;
  int __pyx_v_count_only;
  int __pyx_v_sort_results;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("query_radius (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_X,&__pyx_n_s_r,&__pyx_n_s_return_distance,&__pyx_n_s_count_only,&__pyx_n_s_sort_results,0};
    PyObject* values[5] = {0,0,0,0,0};
    values[2] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_X)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_r)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("query_radius", 0, 2, 5, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1359; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_return_distance);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_count_only);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sort_results);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "query_radius") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1359; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_X = values[0];
    __pyx_v_r = values[1];
    __pyx_v_return_distance = values[2];
    if (values[3]) {
      __pyx_v_count_only = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_count_only == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1360; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {

      /* "binary_tree.pxi":1360
 * 
 *     def query_radius(self, X, r, return_distance=False,
 *                      int count_only=False, int sort_results=False):             # <<<<<<<<<<<<<<
 *         """
 *         query_radius(self, X, r, count_only = False):
 */
      __pyx_v_count_only = ((int)0);
    }
    if (values[4]) {
      __pyx_v_sort_results = __Pyx_PyInt_As_int(values[4]); if (unlikely((__pyx_v_sort_results == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1360; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_sort_results = ((int)0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("query_radius", 0, 2, 5, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1359; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.query_radius", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_20query_radius(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self), __pyx_v_X, __pyx_v_r, __pyx_v_return_distance, __pyx_v_count_only, __pyx_v_sort_results);

  /* "binary_tree.pxi":1359
 *             return indices.reshape(X.shape[:X.ndim - 1] + (k,))
 * 
 *     def query_radius(self, X, r, return_distance=False,             # <<<<<<<<<<<<<<
 *                      int count_only=False, int sort_results=False):
 *         """
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_20query_radius(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, PyObject *__pyx_v_X, PyObject *__pyx_v_r, PyObject *__pyx_v_return_distance, int __pyx_v_count_only, int __pyx_v_sort_results) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i;
  CYTHON_UNUSED __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_count_i;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_features;
  __Pyx_memviewslice __pyx_v_dist_arr_i = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_idx_arr_i = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_counts = { 0, 0, { 0 }, { 0 }, { 0 } };
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt;
  __Pyx_memviewslice __pyx_v_Xarr = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_v_rarr_np = NULL;
  __Pyx_memviewslice __pyx_v_rarr = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_v_indices = NULL;
  PyObject *__pyx_v_distances = NULL;
  PyObject *__pyx_v_np_idx_arr = NULL;
  PyObject *__pyx_v_np_dist_arr = NULL;
  PyObject *__pyx_v_counts_arr = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_memviewslice __pyx_t_9 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_10 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_11 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_15;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  int __pyx_t_19;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_20;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_24;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_25;
  PyObject *__pyx_t_26 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("query_radius", 0);
  __Pyx_INCREF(__pyx_v_X);
  __Pyx_INCREF(__pyx_v_r);

  /* "binary_tree.pxi":1425
 *         [3 0 1]
 *         """
 *         if count_only and return_distance:             # <<<<<<<<<<<<<<
 *             raise ValueError("count_only and return_distance "
 *                              "cannot both be true")
 */
  __pyx_t_2 = (__pyx_v_count_only != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_return_distance); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1425; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "binary_tree.pxi":1426
 *         """
 *         if count_only and return_distance:
 *             raise ValueError("count_only and return_distance "             # <<<<<<<<<<<<<<
 *                              "cannot both be true")
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1426; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1426; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "binary_tree.pxi":1429
 *                              "cannot both be true")
 * 
 *         if sort_results and not return_distance:             # <<<<<<<<<<<<<<
 *             raise ValueError("return_distance must be True "
 *                              "if sort_results is True")
 */
  __pyx_t_2 = (__pyx_v_sort_results != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_return_distance); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1429; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = ((!__pyx_t_2) != 0);
  __pyx_t_1 = __pyx_t_4;
  __pyx_L7_bool_binop_done:;
  if (__pyx_t_1) {

    /* "binary_tree.pxi":1430
 * 
 *         if sort_results and not return_distance:
 *             raise ValueError("return_distance must be True "             # <<<<<<<<<<<<<<
 *                              "if sort_results is True")
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__22, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1430; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1430; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "binary_tree.pxi":1433
 *                              "if sort_results is True")
 * 
 *         cdef ITYPE_t i, count_i = 0             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t n_features = self.data.shape[1]
 *         cdef DTYPE_t[::1] dist_arr_i
 */
  __pyx_v_count_i = 0;

  /* "binary_tree.pxi":1434
 * 
 *         cdef ITYPE_t i, count_i = 0
 *         cdef ITYPE_t n_features = self.data.shape[1]             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t[::1] dist_arr_i
 *         cdef ITYPE_t[::1] idx_arr_i, counts
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_n_features = (__pyx_v_self->data.shape[1]);

  /* "binary_tree.pxi":1440
 * 
 *         # validate X and prepare for query
 *         X = check_array(X, dtype=DTYPE, order='C')             # <<<<<<<<<<<<<<
 * 
 *         if X.shape[X.ndim - 1] != self.data.shape[1]:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_check_array); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1440; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1440; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_X);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_X);
  __Pyx_GIVEREF(__pyx_v_X);
  __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1440; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1440; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_dtype, __pyx_t_7) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1440; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1440; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1440; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF_SET(__pyx_v_X, __pyx_t_7);
  __pyx_t_7 = 0;

  /* "binary_tree.pxi":1442
 *         X = check_array(X, dtype=DTYPE, order='C')
 * 
 *         if X.shape[X.ndim - 1] != self.data.shape[1]:             # <<<<<<<<<<<<<<
 *             raise ValueError("query data dimension must "
 *                              "match training data dimension")
 */
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_shape); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_ndim); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = PyNumber_Subtract(__pyx_t_6, __pyx_int_1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = PyObject_GetItem(__pyx_t_7, __pyx_t_5); if (unlikely(__pyx_t_6 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1442; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_5 = PyInt_FromSsize_t((__pyx_v_self->data.shape[1])); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_7 = PyObject_RichCompare(__pyx_t_6, __pyx_t_5, Py_NE); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__pyx_t_1) {

    /* "binary_tree.pxi":1443
 * 
 *         if X.shape[X.ndim - 1] != self.data.shape[1]:
 *             raise ValueError("query data dimension must "             # <<<<<<<<<<<<<<
 *                              "match training data dimension")
 * 
 */
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__23, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1443; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_Raise(__pyx_t_7, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1443; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "binary_tree.pxi":1447
 * 
 *         cdef DTYPE_t[:, ::1] Xarr =\
 *                 get_memview_DTYPE_2D(X.reshape((-1, self.data.shape[1])))             # <<<<<<<<<<<<<<
 * 
 *         # prepare r for query
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_reshape); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1447; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1447; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_6 = PyInt_FromSsize_t((__pyx_v_self->data.shape[1])); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1447; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1447; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_int_neg_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_int_neg_1);
  __Pyx_GIVEREF(__pyx_int_neg_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_6);
  __pyx_t_6 = 0;
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_3); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1447; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_7);
  } else {
    __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1447; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __Pyx_GIVEREF(__pyx_t_6); __pyx_t_6 = NULL;
    PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1447; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (!(likely(((__pyx_t_7) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_7, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1447; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_9 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_2D(((PyArrayObject *)__pyx_t_7)); if (unlikely(!__pyx_t_9.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1447; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_Xarr = __pyx_t_9;
  __pyx_t_9.memview = NULL;
  __pyx_t_9.data = NULL;

  /* "binary_tree.pxi":1450
 * 
 *         # prepare r for query
 *         r = np.asarray(r, dtype=DTYPE, order='C')             # <<<<<<<<<<<<<<
 *         r = np.atleast_1d(r)
 *         if r.shape == (1,):
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1450; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_asarray); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1450; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1450; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_INCREF(__pyx_v_r);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_v_r);
  __Pyx_GIVEREF(__pyx_v_r);
  __pyx_t_8 = PyDict_New(); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1450; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1450; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1450; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1450; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, __pyx_t_8); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1450; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF_SET(__pyx_v_r, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "binary_tree.pxi":1451
 *         # prepare r for query
 *         r = np.asarray(r, dtype=DTYPE, order='C')
 *         r = np.atleast_1d(r)             # <<<<<<<<<<<<<<
 *         if r.shape == (1,):
 *             r = r[0] + np.zeros(X.shape[:X.ndim - 1], dtype=DTYPE)
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1451; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_atleast_1d); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1451; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
    }
  }
  if (!__pyx_t_8) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_v_r); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1451; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1451; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_8); __Pyx_GIVEREF(__pyx_t_8); __pyx_t_8 = NULL;
    __Pyx_INCREF(__pyx_v_r);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_r);
    __Pyx_GIVEREF(__pyx_v_r);
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1451; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF_SET(__pyx_v_r, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "binary_tree.pxi":1452
 *         r = np.asarray(r, dtype=DTYPE, order='C')
 *         r = np.atleast_1d(r)
 *         if r.shape == (1,):             # <<<<<<<<<<<<<<
 *             r = r[0] + np.zeros(X.shape[:X.ndim - 1], dtype=DTYPE)
 *         else:
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_r, __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1452; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = PyObject_RichCompare(__pyx_t_3, __pyx_tuple__24, Py_EQ); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1452; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1452; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__pyx_t_1) {

    /* "binary_tree.pxi":1453
 *         r = np.atleast_1d(r)
 *         if r.shape == (1,):
 *             r = r[0] + np.zeros(X.shape[:X.ndim - 1], dtype=DTYPE)             # <<<<<<<<<<<<<<
 *         else:
 *             if r.shape != X.shape[:X.ndim - 1]:
 */
    __pyx_t_7 = __Pyx_GetItemInt(__pyx_v_r, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_7 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1453; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_6 = PyNumber_Subtract(__pyx_t_8, __pyx_int_1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_GetSlice(__pyx_t_3, 0, 0, NULL, &__pyx_t_6, NULL, 0, 0, 0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = PyDict_New(); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_6, __pyx_t_8); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = PyNumber_Add(__pyx_t_7, __pyx_t_3); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_r, __pyx_t_8);
    __pyx_t_8 = 0;
    goto __pyx_L10;
  }
  /*else*/ {

    /* "binary_tree.pxi":1455
 *             r = r[0] + np.zeros(X.shape[:X.ndim - 1], dtype=DTYPE)
 *         else:
 *             if r.shape != X.shape[:X.ndim - 1]:             # <<<<<<<<<<<<<<
 *                 raise ValueError("r must be broadcastable to X.shape")
 * 
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_r, __pyx_n_s_shape); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1455; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1455; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_ndim); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1455; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = PyNumber_Subtract(__pyx_t_7, __pyx_int_1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1455; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyObject_GetSlice(__pyx_t_3, 0, 0, NULL, &__pyx_t_6, NULL, 0, 0, 0); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1455; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyObject_RichCompare(__pyx_t_8, __pyx_t_7, Py_NE); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1455; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1455; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (__pyx_t_1) {

      /* "binary_tree.pxi":1456
 *         else:
 *             if r.shape != X.shape[:X.ndim - 1]:
 *                 raise ValueError("r must be broadcastable to X.shape")             # <<<<<<<<<<<<<<
 * 
 *         rarr_np = r.reshape(-1)  # store explicitly to keep in scope
 */
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__25, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1456; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1456; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __pyx_L10:;

  /* "binary_tree.pxi":1458
 *                 raise ValueError("r must be broadcastable to X.shape")
 * 
 *         rarr_np = r.reshape(-1)  # store explicitly to keep in scope             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t[::1] rarr = get_memview_DTYPE_1D(rarr_np)
 * 
 */
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_r, __pyx_n_s_reshape); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1458; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1458; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_rarr_np = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "binary_tree.pxi":1459
 * 
 *         rarr_np = r.reshape(-1)  # store explicitly to keep in scope
 *         cdef DTYPE_t[::1] rarr = get_memview_DTYPE_1D(rarr_np)             # <<<<<<<<<<<<<<
 * 
 *         # prepare variables for iteration
 */
  if (!(likely(((__pyx_v_rarr_np) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_rarr_np, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_10 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_1D(((PyArrayObject *)__pyx_v_rarr_np)); if (unlikely(!__pyx_t_10.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_rarr = __pyx_t_10;
  __pyx_t_10.memview = NULL;
  __pyx_t_10.data = NULL;

  /* "binary_tree.pxi":1462
 * 
 *         # prepare variables for iteration
 *         if not count_only:             # <<<<<<<<<<<<<<
 *             indices = np.zeros(Xarr.shape[0], dtype='object')
 *             if return_distance:
 */
  __pyx_t_1 = ((!(__pyx_v_count_only != 0)) != 0);
  if (__pyx_t_1) {

    /* "binary_tree.pxi":1463
 *         # prepare variables for iteration
 *         if not count_only:
 *             indices = np.zeros(Xarr.shape[0], dtype='object')             # <<<<<<<<<<<<<<
 *             if return_distance:
 *                 distances = np.zeros(Xarr.shape[0], dtype='object')
 */
    __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_zeros); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = PyInt_FromSsize_t((__pyx_v_Xarr.shape[0])); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_7 = PyDict_New(); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_dtype, __pyx_n_s_object) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_v_indices = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "binary_tree.pxi":1464
 *         if not count_only:
 *             indices = np.zeros(Xarr.shape[0], dtype='object')
 *             if return_distance:             # <<<<<<<<<<<<<<
 *                 distances = np.zeros(Xarr.shape[0], dtype='object')
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_return_distance); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1464; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (__pyx_t_1) {

      /* "binary_tree.pxi":1465
 *             indices = np.zeros(Xarr.shape[0], dtype='object')
 *             if return_distance:
 *                 distances = np.zeros(Xarr.shape[0], dtype='object')             # <<<<<<<<<<<<<<
 * 
 *         np_idx_arr = np.zeros(self.data.shape[0], dtype=ITYPE)
 */
      __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1465; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1465; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = PyInt_FromSsize_t((__pyx_v_Xarr.shape[0])); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1465; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1465; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1465; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_n_s_object) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1465; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_8, __pyx_t_3); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1465; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_v_distances = __pyx_t_6;
      __pyx_t_6 = 0;
      goto __pyx_L13;
    }
    __pyx_L13:;
    goto __pyx_L12;
  }
  __pyx_L12:;

  /* "binary_tree.pxi":1467
 *                 distances = np.zeros(Xarr.shape[0], dtype='object')
 * 
 *         np_idx_arr = np.zeros(self.data.shape[0], dtype=ITYPE)             # <<<<<<<<<<<<<<
 *         idx_arr_i = get_memview_ITYPE_1D(np_idx_arr)
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1467; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_zeros); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1467; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1467; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_6 = PyInt_FromSsize_t((__pyx_v_self->data.shape[0])); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1467; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1467; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_6);
  __pyx_t_6 = 0;
  __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1467; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1467; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_dtype, __pyx_t_7) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1467; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_8, __pyx_t_6); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1467; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_np_idx_arr = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "binary_tree.pxi":1468
 * 
 *         np_idx_arr = np.zeros(self.data.shape[0], dtype=ITYPE)
 *         idx_arr_i = get_memview_ITYPE_1D(np_idx_arr)             # <<<<<<<<<<<<<<
 * 
 *         np_dist_arr = np.zeros(self.data.shape[0], dtype=DTYPE)
 */
  if (!(likely(((__pyx_v_np_idx_arr) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_np_idx_arr, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1468; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_11 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_ITYPE_1D(((PyArrayObject *)__pyx_v_np_idx_arr)); if (unlikely(!__pyx_t_11.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1468; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_idx_arr_i = __pyx_t_11;
  __pyx_t_11.memview = NULL;
  __pyx_t_11.data = NULL;

  /* "binary_tree.pxi":1470
 *         idx_arr_i = get_memview_ITYPE_1D(np_idx_arr)
 * 
 *         np_dist_arr = np.zeros(self.data.shape[0], dtype=DTYPE)             # <<<<<<<<<<<<<<
 *         dist_arr_i = get_memview_DTYPE_1D(np_dist_arr)
 * 
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1470; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_zeros); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1470; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1470; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_7 = PyInt_FromSsize_t((__pyx_v_self->data.shape[0])); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1470; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1470; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_7);
  __pyx_t_7 = 0;
  __pyx_t_7 = PyDict_New(); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1470; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1470; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1470; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1470; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_np_dist_arr = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "binary_tree.pxi":1471
 * 
 *         np_dist_arr = np.zeros(self.data.shape[0], dtype=DTYPE)
 *         dist_arr_i = get_memview_DTYPE_1D(np_dist_arr)             # <<<<<<<<<<<<<<
 * 
 *         counts_arr = np.zeros(Xarr.shape[0], dtype=ITYPE)
 */
  if (!(likely(((__pyx_v_np_dist_arr) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_np_dist_arr, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1471; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_10 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_1D(((PyArrayObject *)__pyx_v_np_dist_arr)); if (unlikely(!__pyx_t_10.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1471; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_dist_arr_i = __pyx_t_10;
  __pyx_t_10.memview = NULL;
  __pyx_t_10.data = NULL;

  /* "binary_tree.pxi":1473
 *         dist_arr_i = get_memview_DTYPE_1D(np_dist_arr)
 * 
 *         counts_arr = np.zeros(Xarr.shape[0], dtype=ITYPE)             # <<<<<<<<<<<<<<
 *         counts = get_memview_ITYPE_1D(counts_arr)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyInt_FromSsize_t((__pyx_v_Xarr.shape[0])); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_8, __pyx_t_3); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_counts_arr = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "binary_tree.pxi":1474
 * 
 *         counts_arr = np.zeros(Xarr.shape[0], dtype=ITYPE)
 *         counts = get_memview_ITYPE_1D(counts_arr)             # <<<<<<<<<<<<<<
 * 
 *         pt = &Xarr[0, 0]
 */
  if (!(likely(((__pyx_v_counts_arr) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_counts_arr, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_11 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_ITYPE_1D(((PyArrayObject *)__pyx_v_counts_arr)); if (unlikely(!__pyx_t_11.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_counts = __pyx_t_11;
  __pyx_t_11.memview = NULL;
  __pyx_t_11.data = NULL;

  /* "binary_tree.pxi":1476
 *         counts = get_memview_ITYPE_1D(counts_arr)
 * 
 *         pt = &Xarr[0, 0]             # <<<<<<<<<<<<<<
 *         for i in range(Xarr.shape[0]):
 *             counts[i] = self._query_radius_single(0, pt, rarr[i],
 */
  __pyx_t_12 = 0;
  __pyx_t_13 = 0;
  __pyx_v_pt = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_Xarr.data + __pyx_t_12 * __pyx_v_Xarr.strides[0]) )) + __pyx_t_13)) ))));

  /* "binary_tree.pxi":1477
 * 
 *         pt = &Xarr[0, 0]
 *         for i in range(Xarr.shape[0]):             # <<<<<<<<<<<<<<
 *             counts[i] = self._query_radius_single(0, pt, rarr[i],
 *                                                   &idx_arr_i[0],
 */
  __pyx_t_14 = (__pyx_v_Xarr.shape[0]);
  for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
    __pyx_v_i = __pyx_t_15;

    /* "binary_tree.pxi":1478
 *         pt = &Xarr[0, 0]
 *         for i in range(Xarr.shape[0]):
 *             counts[i] = self._query_radius_single(0, pt, rarr[i],             # <<<<<<<<<<<<<<
 *                                                   &idx_arr_i[0],
 *                                                   &dist_arr_i[0],
 */
    __pyx_t_16 = __pyx_v_i;

    /* "binary_tree.pxi":1479
 *         for i in range(Xarr.shape[0]):
 *             counts[i] = self._query_radius_single(0, pt, rarr[i],
 *                                                   &idx_arr_i[0],             # <<<<<<<<<<<<<<
 *                                                   &dist_arr_i[0],
 *                                                   0, count_only,
 */
    __pyx_t_17 = 0;

    /* "binary_tree.pxi":1480
 *             counts[i] = self._query_radius_single(0, pt, rarr[i],
 *                                                   &idx_arr_i[0],
 *                                                   &dist_arr_i[0],             # <<<<<<<<<<<<<<
 *                                                   0, count_only,
 *                                                   return_distance)
 */
    __pyx_t_18 = 0;

    /* "binary_tree.pxi":1482
 *                                                   &dist_arr_i[0],
 *                                                   0, count_only,
 *                                                   return_distance)             # <<<<<<<<<<<<<<
 *             pt += n_features
 * 
 */
    __pyx_t_19 = __Pyx_PyInt_As_int(__pyx_v_return_distance); if (unlikely((__pyx_t_19 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1482; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "binary_tree.pxi":1478
 *         pt = &Xarr[0, 0]
 *         for i in range(Xarr.shape[0]):
 *             counts[i] = self._query_radius_single(0, pt, rarr[i],             # <<<<<<<<<<<<<<
 *                                                   &idx_arr_i[0],
 *                                                   &dist_arr_i[0],
 */
    __pyx_t_20 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_query_radius_single(__pyx_v_self, 0, __pyx_v_pt, (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_rarr.data) + __pyx_t_16)) ))), (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_idx_arr_i.data) + __pyx_t_17)) )))), (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_dist_arr_i.data) + __pyx_t_18)) )))), 0, __pyx_v_count_only, __pyx_t_19); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_21 = __pyx_v_i;
    *((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_counts.data) + __pyx_t_21)) )) = __pyx_t_20;

    /* "binary_tree.pxi":1483
 *                                                   0, count_only,
 *                                                   return_distance)
 *             pt += n_features             # <<<<<<<<<<<<<<
 * 
 *             if count_only:
 */
    __pyx_v_pt = (__pyx_v_pt + __pyx_v_n_features);

    /* "binary_tree.pxi":1485
 *             pt += n_features
 * 
 *             if count_only:             # <<<<<<<<<<<<<<
 *                 pass
 *             else:
 */
    __pyx_t_1 = (__pyx_v_count_only != 0);
    if (__pyx_t_1) {
      goto __pyx_L16;
    }
    /*else*/ {

      /* "binary_tree.pxi":1488
 *                 pass
 *             else:
 *                 if sort_results:             # <<<<<<<<<<<<<<
 *                     _simultaneous_sort(&dist_arr_i[0], &idx_arr_i[0],
 *                                        counts[i])
 */
      __pyx_t_1 = (__pyx_v_sort_results != 0);
      if (__pyx_t_1) {

        /* "binary_tree.pxi":1489
 *             else:
 *                 if sort_results:
 *                     _simultaneous_sort(&dist_arr_i[0], &idx_arr_i[0],             # <<<<<<<<<<<<<<
 *                                        counts[i])
 * 
 */
        __pyx_t_22 = 0;
        __pyx_t_23 = 0;

        /* "binary_tree.pxi":1490
 *                 if sort_results:
 *                     _simultaneous_sort(&dist_arr_i[0], &idx_arr_i[0],
 *                                        counts[i])             # <<<<<<<<<<<<<<
 * 
 *                 indices[i] = np_idx_arr[:counts[i]].copy()
 */
        __pyx_t_20 = __pyx_v_i;

        /* "binary_tree.pxi":1489
 *             else:
 *                 if sort_results:
 *                     _simultaneous_sort(&dist_arr_i[0], &idx_arr_i[0],             # <<<<<<<<<<<<<<
 *                                        counts[i])
 * 
 */
        __pyx_t_19 = __pyx_f_7sklearn_9neighbors_7kd_tree__simultaneous_sort((&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_dist_arr_i.data) + __pyx_t_22)) )))), (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_idx_arr_i.data) + __pyx_t_23)) )))), (*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_counts.data) + __pyx_t_20)) )))); if (unlikely(__pyx_t_19 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1489; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L17;
      }
      __pyx_L17:;

      /* "binary_tree.pxi":1492
 *                                        counts[i])
 * 
 *                 indices[i] = np_idx_arr[:counts[i]].copy()             # <<<<<<<<<<<<<<
 *                 if return_distance:
 *                     distances[i] = np_dist_arr[:counts[i]].copy()
 */
      __pyx_t_24 = __pyx_v_i;
      __pyx_t_3 = __Pyx_PyObject_GetSlice(__pyx_v_np_idx_arr, 0, (*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_counts.data) + __pyx_t_24)) ))), NULL, NULL, NULL, 0, 1, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1492; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_copy); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1492; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_8);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_8, function);
        }
      }
      if (__pyx_t_3) {
        __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_3); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1492; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else {
        __pyx_t_6 = __Pyx_PyObject_CallNoArg(__pyx_t_8); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1492; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_v_indices)) { __Pyx_RaiseUnboundLocalError("indices"); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1492; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }
      if (unlikely(__Pyx_SetItemInt(__pyx_v_indices, __pyx_v_i, __pyx_t_6, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, 1, __Pyx_PyInt_From_Py_intptr_t, 0, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1492; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "binary_tree.pxi":1493
 * 
 *                 indices[i] = np_idx_arr[:counts[i]].copy()
 *                 if return_distance:             # <<<<<<<<<<<<<<
 *                     distances[i] = np_dist_arr[:counts[i]].copy()
 * 
 */
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_return_distance); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1493; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      if (__pyx_t_1) {

        /* "binary_tree.pxi":1494
 *                 indices[i] = np_idx_arr[:counts[i]].copy()
 *                 if return_distance:
 *                     distances[i] = np_dist_arr[:counts[i]].copy()             # <<<<<<<<<<<<<<
 * 
 *         # deflatten results
 */
        __pyx_t_25 = __pyx_v_i;
        __pyx_t_8 = __Pyx_PyObject_GetSlice(__pyx_v_np_dist_arr, 0, (*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_counts.data) + __pyx_t_25)) ))), NULL, NULL, NULL, 0, 1, 0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1494; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_copy); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1494; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_8 = NULL;
        if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_8)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_8);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        if (__pyx_t_8) {
          __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_8); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1494; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        } else {
          __pyx_t_6 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1494; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_v_distances)) { __Pyx_RaiseUnboundLocalError("distances"); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1494; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }
        if (unlikely(__Pyx_SetItemInt(__pyx_v_distances, __pyx_v_i, __pyx_t_6, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, 1, __Pyx_PyInt_From_Py_intptr_t, 0, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1494; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        goto __pyx_L18;
      }
      __pyx_L18:;
    }
    __pyx_L16:;
  }

  /* "binary_tree.pxi":1497
 * 
 *         # deflatten results
 *         if count_only:             # <<<<<<<<<<<<<<
 *             return counts_arr.reshape(X.shape[:X.ndim - 1])
 *         elif return_distance:
 */
  __pyx_t_1 = (__pyx_v_count_only != 0);
  if (__pyx_t_1) {

    /* "binary_tree.pxi":1498
 *         # deflatten results
 *         if count_only:
 *             return counts_arr.reshape(X.shape[:X.ndim - 1])             # <<<<<<<<<<<<<<
 *         elif return_distance:
 *             return (indices.reshape(X.shape[:X.ndim - 1]),
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_counts_arr, __pyx_n_s_reshape); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1498; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_shape); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1498; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_ndim); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1498; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_5 = PyNumber_Subtract(__pyx_t_7, __pyx_int_1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1498; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyObject_GetSlice(__pyx_t_8, 0, 0, NULL, &__pyx_t_5, NULL, 0, 0, 0); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1498; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_7); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1498; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_6);
    } else {
      __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1498; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __Pyx_GIVEREF(__pyx_t_5); __pyx_t_5 = NULL;
      PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_7);
      __pyx_t_7 = 0;
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_8, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1498; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;
  }

  /* "binary_tree.pxi":1499
 *         if count_only:
 *             return counts_arr.reshape(X.shape[:X.ndim - 1])
 *         elif return_distance:             # <<<<<<<<<<<<<<
 *             return (indices.reshape(X.shape[:X.ndim - 1]),
 *                     distances.reshape(X.shape[:X.ndim - 1]))
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_return_distance); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1499; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_1) {

    /* "binary_tree.pxi":1500
 *             return counts_arr.reshape(X.shape[:X.ndim - 1])
 *         elif return_distance:
 *             return (indices.reshape(X.shape[:X.ndim - 1]),             # <<<<<<<<<<<<<<
 *                     distances.reshape(X.shape[:X.ndim - 1]))
 *         else:
 */
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(!__pyx_v_indices)) { __Pyx_RaiseUnboundLocalError("indices"); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1500; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_indices, __pyx_n_s_reshape); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1500; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_shape); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1500; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_ndim); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1500; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_5 = PyNumber_Subtract(__pyx_t_7, __pyx_int_1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1500; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyObject_GetSlice(__pyx_t_8, 0, 0, NULL, &__pyx_t_5, NULL, 0, 0, 0); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1500; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_7); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1500; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_6);
    } else {
      __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1500; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __Pyx_GIVEREF(__pyx_t_5); __pyx_t_5 = NULL;
      PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_7);
      __pyx_t_7 = 0;
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_8, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1500; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "binary_tree.pxi":1501
 *         elif return_distance:
 *             return (indices.reshape(X.shape[:X.ndim - 1]),
 *                     distances.reshape(X.shape[:X.ndim - 1]))             # <<<<<<<<<<<<<<
 *         else:
 *             return indices.reshape(X.shape[:X.ndim - 1])
 */
    if (unlikely(!__pyx_v_distances)) { __Pyx_RaiseUnboundLocalError("distances"); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1501; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_distances, __pyx_n_s_reshape); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1501; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_shape); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1501; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_ndim); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1501; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_26 = PyNumber_Subtract(__pyx_t_5, __pyx_int_1); if (unlikely(!__pyx_t_26)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1501; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_26);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_GetSlice(__pyx_t_7, 0, 0, NULL, &__pyx_t_26, NULL, 0, 0, 0); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1501; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_26); __pyx_t_26 = 0;
    __pyx_t_26 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_26 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_26)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_26);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    if (!__pyx_t_26) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1501; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1501; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_26); __Pyx_GIVEREF(__pyx_t_26); __pyx_t_26 = NULL;
      PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_7, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1501; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "binary_tree.pxi":1500
 *             return counts_arr.reshape(X.shape[:X.ndim - 1])
 *         elif return_distance:
 *             return (indices.reshape(X.shape[:X.ndim - 1]),             # <<<<<<<<<<<<<<
 *                     distances.reshape(X.shape[:X.ndim - 1]))
 *         else:
 */
    __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1500; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_6 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }
  /*else*/ {

    /* "binary_tree.pxi":1503
 *                     distances.reshape(X.shape[:X.ndim - 1]))
 *         else:
 *             return indices.reshape(X.shape[:X.ndim - 1])             # <<<<<<<<<<<<<<
 * 
 *     def kernel_density(self, X, h, kernel='gaussian',
 */
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(!__pyx_v_indices)) { __Pyx_RaiseUnboundLocalError("indices"); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1503; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_indices, __pyx_n_s_reshape); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1503; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_shape); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1503; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_ndim); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1503; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_5 = PyNumber_Subtract(__pyx_t_7, __pyx_int_1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1503; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyObject_GetSlice(__pyx_t_6, 0, 0, NULL, &__pyx_t_5, NULL, 0, 0, 0); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1503; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_7); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1503; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_8);
    } else {
      __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1503; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __Pyx_GIVEREF(__pyx_t_5); __pyx_t_5 = NULL;
      PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_7);
      __pyx_t_7 = 0;
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1503; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* "binary_tree.pxi":1359
 *             return indices.reshape(X.shape[:X.ndim - 1] + (k,))
 * 
 *     def query_radius(self, X, r, return_distance=False,             # <<<<<<<<<<<<<<
 *                      int count_only=False, int sort_results=False):
 *         """
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __PYX_XDEC_MEMVIEW(&__pyx_t_9, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_10, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_11, 1);
  __Pyx_XDECREF(__pyx_t_26);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.query_radius", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_dist_arr_i, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_idx_arr_i, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_counts, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_Xarr, 1);
  __Pyx_XDECREF(__pyx_v_rarr_np);
  __PYX_XDEC_MEMVIEW(&__pyx_v_rarr, 1);
  __Pyx_XDECREF(__pyx_v_indices);
  __Pyx_XDECREF(__pyx_v_distances);
  __Pyx_XDECREF(__pyx_v_np_idx_arr);
  __Pyx_XDECREF(__pyx_v_np_dist_arr);
  __Pyx_XDECREF(__pyx_v_counts_arr);
  __Pyx_XDECREF(__pyx_v_X);
  __Pyx_XDECREF(__pyx_v_r);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1505
 *             return indices.reshape(X.shape[:X.ndim - 1])
 * 
 *     def kernel_density(self, X, h, kernel='gaussian',             # <<<<<<<<<<<<<<
 *                        atol=0, rtol=1E-8,
 *                        breadth_first=True, return_log=False):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_23kernel_density(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7sklearn_9neighbors_7kd_tree_10BinaryTree_22kernel_density[] = "\n        kernel_density(self, X, h, kernel='gaussian', atol=0, rtol=1E-8,\n                       breadth_first=True, return_log=False)\n\n        Compute the kernel density estimate at points X with the given kernel,\n        using the distance metric specified at tree creation.\n\n        Parameters\n        ----------\n        X : array_like\n            An array of points to query.  Last dimension should match dimension\n            of training data.\n        h : float\n            the bandwidth of the kernel\n        kernel : string\n            specify the kernel to use.  Options are\n            - 'gaussian'\n            - 'tophat'\n            - 'epanechnikov'\n            - 'exponential'\n            - 'linear'\n            - 'cosine'\n            Default is kernel = 'gaussian'\n        atol, rtol : float (default = 0)\n            Specify the desired relative and absolute tolerance of the result.\n            If the true result is K_true, then the returned result K_ret\n            satisfies ``abs(K_true - K_ret) < atol + rtol * K_ret``\n            The default is zero (i.e. machine precision) for both.\n        breadth_first : boolean (default = False)\n            if True, use a breadth-first search.  If False (default) use a\n            depth-first search.  Breadth-first is generally faster for\n            compact kernels and/or high tolerances.\n        return_log : boolean (default = False)\n            return the logarithm of the result.  This can be more accurate\n            than returning the result itself for narrow kernels.\n\n        Returns\n        -------\n        density : ndarray\n            The array of (log)-density evaluations, shape = X.shape[:-1]\n\n        Examples\n        --------\n        Compute a gaussian kernel density estimate:\n\n        >>> import numpy as np\n        >>> np.random.seed(1)\n        >>> X = np.random.random((100, 3))\n        >>> tree = BinaryTree(X)           # doctest: +SKIP\n        >>> tree.kernel_den""sity(X[:3], h=0.1, kernel='gaussian')\n        array([ 6.94114649,  7.83281226,  7.2071716 ])\n        ";
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_23kernel_density(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_X = 0;
  PyObject *__pyx_v_h = 0;
  PyObject *__pyx_v_kernel = 0;
  PyObject *__pyx_v_atol = 0;
  PyObject *__pyx_v_rtol = 0;
  PyObject *__pyx_v_breadth_first = 0;
  PyObject *__pyx_v_return_log = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("kernel_density (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_X,&__pyx_n_s_h,&__pyx_n_s_kernel,&__pyx_n_s_atol,&__pyx_n_s_rtol,&__pyx_n_s_breadth_first,&__pyx_n_s_return_log,0};
    PyObject* values[7] = {0,0,0,0,0,0,0};
    values[2] = ((PyObject *)__pyx_n_s_gaussian);
    values[3] = ((PyObject *)__pyx_int_0);
    values[4] = ((PyObject *)__pyx_float_1Eneg_8);

    /* "binary_tree.pxi":1507
 *     def kernel_density(self, X, h, kernel='gaussian',
 *                        atol=0, rtol=1E-8,
 *                        breadth_first=True, return_log=False):             # <<<<<<<<<<<<<<
 *         """
 *         kernel_density(self, X, h, kernel='gaussian', atol=0, rtol=1E-8,
 */
    values[5] = ((PyObject *)Py_True);
    values[6] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_X)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_h)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("kernel_density", 0, 2, 7, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1505; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kernel);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_atol);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_rtol);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_breadth_first);
          if (value) { values[5] = value; kw_args--; }
        }
        case  6:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_return_log);
          if (value) { values[6] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "kernel_density") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1505; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_X = values[0];
    __pyx_v_h = values[1];
    __pyx_v_kernel = values[2];
    __pyx_v_atol = values[3];
    __pyx_v_rtol = values[4];
    __pyx_v_breadth_first = values[5];
    __pyx_v_return_log = values[6];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("kernel_density", 0, 2, 7, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1505; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.kernel_density", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_22kernel_density(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self), __pyx_v_X, __pyx_v_h, __pyx_v_kernel, __pyx_v_atol, __pyx_v_rtol, __pyx_v_breadth_first, __pyx_v_return_log);

  /* "binary_tree.pxi":1505
 *             return indices.reshape(X.shape[:X.ndim - 1])
 * 
 *     def kernel_density(self, X, h, kernel='gaussian',             # <<<<<<<<<<<<<<
 *                        atol=0, rtol=1E-8,
 *                        breadth_first=True, return_log=False):
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_22kernel_density(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, PyObject *__pyx_v_X, PyObject *__pyx_v_h, PyObject *__pyx_v_kernel, PyObject *__pyx_v_atol, PyObject *__pyx_v_rtol, PyObject *__pyx_v_breadth_first, PyObject *__pyx_v_return_log) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_h_c;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_log_atol;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_log_rtol;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_log_min_bound;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_log_max_bound;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_log_bound_spread;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_LB;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_UB;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_samples;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_features;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i;
  enum __pyx_t_7sklearn_9neighbors_7kd_tree_KernelType __pyx_v_kernel_c;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_log_knorm;
  PyObject *__pyx_v_Xarr_np = NULL;
  __Pyx_memviewslice __pyx_v_Xarr = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_v_log_density_arr = NULL;
  __Pyx_memviewslice __pyx_v_log_density = { 0, 0, { 0 }, { 0 }, { 0 } };
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt;
  struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_nodeheap = 0;
  __Pyx_memviewslice __pyx_v_node_log_min_bounds = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_node_bound_widths = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_v_node_log_min_bounds_arr = NULL;
  PyObject *__pyx_v_node_bound_widths_arr = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_1;
  double __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_memviewslice __pyx_t_9 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_10 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  Py_ssize_t __pyx_t_14;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_18;
  int __pyx_t_19;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_20;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_21;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("kernel_density", 0);
  __Pyx_INCREF(__pyx_v_X);

  /* "binary_tree.pxi":1560
 *         array([ 6.94114649,  7.83281226,  7.2071716 ])
 *         """
 *         cdef DTYPE_t h_c = h             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t log_atol = log(atol)
 *         cdef DTYPE_t log_rtol = log(rtol)
 */
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_h); if (unlikely((__pyx_t_1 == (npy_float64)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1560; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_h_c = __pyx_t_1;

  /* "binary_tree.pxi":1561
 *         """
 *         cdef DTYPE_t h_c = h
 *         cdef DTYPE_t log_atol = log(atol)             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t log_rtol = log(rtol)
 *         cdef DTYPE_t log_min_bound, log_max_bound, log_bound_spread
 */
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_v_atol); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1561; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_log_atol = log(__pyx_t_2);

  /* "binary_tree.pxi":1562
 *         cdef DTYPE_t h_c = h
 *         cdef DTYPE_t log_atol = log(atol)
 *         cdef DTYPE_t log_rtol = log(rtol)             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t log_min_bound, log_max_bound, log_bound_spread
 *         cdef DTYPE_t dist_LB = 0, dist_UB = 0
 */
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_v_rtol); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1562; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_log_rtol = log(__pyx_t_2);

  /* "binary_tree.pxi":1564
 *         cdef DTYPE_t log_rtol = log(rtol)
 *         cdef DTYPE_t log_min_bound, log_max_bound, log_bound_spread
 *         cdef DTYPE_t dist_LB = 0, dist_UB = 0             # <<<<<<<<<<<<<<
 * 
 *         cdef ITYPE_t n_samples = self.data.shape[0]
 */
  __pyx_v_dist_LB = 0.0;
  __pyx_v_dist_UB = 0.0;

  /* "binary_tree.pxi":1566
 *         cdef DTYPE_t dist_LB = 0, dist_UB = 0
 * 
 *         cdef ITYPE_t n_samples = self.data.shape[0]             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t n_features = self.data.shape[1]
 *         cdef ITYPE_t i
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1566; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_n_samples = (__pyx_v_self->data.shape[0]);

  /* "binary_tree.pxi":1567
 * 
 *         cdef ITYPE_t n_samples = self.data.shape[0]
 *         cdef ITYPE_t n_features = self.data.shape[1]             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t i
 *         cdef KernelType kernel_c
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_n_features = (__pyx_v_self->data.shape[1]);

  /* "binary_tree.pxi":1572
 * 
 *         # validate kernel
 *         if kernel == 'gaussian':             # <<<<<<<<<<<<<<
 *             kernel_c = GAUSSIAN_KERNEL
 *         elif kernel == 'tophat':
 */
  __pyx_t_3 = (__Pyx_PyString_Equals(__pyx_v_kernel, __pyx_n_s_gaussian, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1572; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_3) {

    /* "binary_tree.pxi":1573
 *         # validate kernel
 *         if kernel == 'gaussian':
 *             kernel_c = GAUSSIAN_KERNEL             # <<<<<<<<<<<<<<
 *         elif kernel == 'tophat':
 *             kernel_c = TOPHAT_KERNEL
 */
    __pyx_v_kernel_c = __pyx_e_7sklearn_9neighbors_7kd_tree_GAUSSIAN_KERNEL;
    goto __pyx_L3;
  }

  /* "binary_tree.pxi":1574
 *         if kernel == 'gaussian':
 *             kernel_c = GAUSSIAN_KERNEL
 *         elif kernel == 'tophat':             # <<<<<<<<<<<<<<
 *             kernel_c = TOPHAT_KERNEL
 *         elif kernel == 'epanechnikov':
 */
  __pyx_t_3 = (__Pyx_PyString_Equals(__pyx_v_kernel, __pyx_n_s_tophat, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_3) {

    /* "binary_tree.pxi":1575
 *             kernel_c = GAUSSIAN_KERNEL
 *         elif kernel == 'tophat':
 *             kernel_c = TOPHAT_KERNEL             # <<<<<<<<<<<<<<
 *         elif kernel == 'epanechnikov':
 *             kernel_c = EPANECHNIKOV_KERNEL
 */
    __pyx_v_kernel_c = __pyx_e_7sklearn_9neighbors_7kd_tree_TOPHAT_KERNEL;
    goto __pyx_L3;
  }

  /* "binary_tree.pxi":1576
 *         elif kernel == 'tophat':
 *             kernel_c = TOPHAT_KERNEL
 *         elif kernel == 'epanechnikov':             # <<<<<<<<<<<<<<
 *             kernel_c = EPANECHNIKOV_KERNEL
 *         elif kernel == 'exponential':
 */
  __pyx_t_3 = (__Pyx_PyString_Equals(__pyx_v_kernel, __pyx_n_s_epanechnikov, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1576; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_3) {

    /* "binary_tree.pxi":1577
 *             kernel_c = TOPHAT_KERNEL
 *         elif kernel == 'epanechnikov':
 *             kernel_c = EPANECHNIKOV_KERNEL             # <<<<<<<<<<<<<<
 *         elif kernel == 'exponential':
 *             kernel_c = EXPONENTIAL_KERNEL
 */
    __pyx_v_kernel_c = __pyx_e_7sklearn_9neighbors_7kd_tree_EPANECHNIKOV_KERNEL;
    goto __pyx_L3;
  }

  /* "binary_tree.pxi":1578
 *         elif kernel == 'epanechnikov':
 *             kernel_c = EPANECHNIKOV_KERNEL
 *         elif kernel == 'exponential':             # <<<<<<<<<<<<<<
 *             kernel_c = EXPONENTIAL_KERNEL
 *         elif kernel == 'linear':
 */
  __pyx_t_3 = (__Pyx_PyString_Equals(__pyx_v_kernel, __pyx_n_s_exponential, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1578; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_3) {

    /* "binary_tree.pxi":1579
 *             kernel_c = EPANECHNIKOV_KERNEL
 *         elif kernel == 'exponential':
 *             kernel_c = EXPONENTIAL_KERNEL             # <<<<<<<<<<<<<<
 *         elif kernel == 'linear':
 *             kernel_c = LINEAR_KERNEL
 */
    __pyx_v_kernel_c = __pyx_e_7sklearn_9neighbors_7kd_tree_EXPONENTIAL_KERNEL;
    goto __pyx_L3;
  }

  /* "binary_tree.pxi":1580
 *         elif kernel == 'exponential':
 *             kernel_c = EXPONENTIAL_KERNEL
 *         elif kernel == 'linear':             # <<<<<<<<<<<<<<
 *             kernel_c = LINEAR_KERNEL
 *         elif kernel == 'cosine':
 */
  __pyx_t_3 = (__Pyx_PyString_Equals(__pyx_v_kernel, __pyx_n_s_linear, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1580; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_3) {

    /* "binary_tree.pxi":1581
 *             kernel_c = EXPONENTIAL_KERNEL
 *         elif kernel == 'linear':
 *             kernel_c = LINEAR_KERNEL             # <<<<<<<<<<<<<<
 *         elif kernel == 'cosine':
 *             kernel_c = COSINE_KERNEL
 */
    __pyx_v_kernel_c = __pyx_e_7sklearn_9neighbors_7kd_tree_LINEAR_KERNEL;
    goto __pyx_L3;
  }

  /* "binary_tree.pxi":1582
 *         elif kernel == 'linear':
 *             kernel_c = LINEAR_KERNEL
 *         elif kernel == 'cosine':             # <<<<<<<<<<<<<<
 *             kernel_c = COSINE_KERNEL
 *         else:
 */
  __pyx_t_3 = (__Pyx_PyString_Equals(__pyx_v_kernel, __pyx_n_s_cosine, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1582; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_3) {

    /* "binary_tree.pxi":1583
 *             kernel_c = LINEAR_KERNEL
 *         elif kernel == 'cosine':
 *             kernel_c = COSINE_KERNEL             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError("kernel = '%s' not recognized" % kernel)
 */
    __pyx_v_kernel_c = __pyx_e_7sklearn_9neighbors_7kd_tree_COSINE_KERNEL;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "binary_tree.pxi":1585
 *             kernel_c = COSINE_KERNEL
 *         else:
 *             raise ValueError("kernel = '%s' not recognized" % kernel)             # <<<<<<<<<<<<<<
 * 
 *         cdef DTYPE_t log_knorm = _log_kernel_norm(h_c, n_features, kernel_c)
 */
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_kernel_s_not_recognized, __pyx_v_kernel); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1585; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1585; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1585; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1585; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L3:;

  /* "binary_tree.pxi":1587
 *             raise ValueError("kernel = '%s' not recognized" % kernel)
 * 
 *         cdef DTYPE_t log_knorm = _log_kernel_norm(h_c, n_features, kernel_c)             # <<<<<<<<<<<<<<
 * 
 *         # validate X and prepare for query
 */
  __pyx_t_1 = __pyx_f_7sklearn_9neighbors_7kd_tree__log_kernel_norm(__pyx_v_h_c, __pyx_v_n_features, __pyx_v_kernel_c); if (unlikely(__pyx_t_1 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1587; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_log_knorm = __pyx_t_1;

  /* "binary_tree.pxi":1590
 * 
 *         # validate X and prepare for query
 *         X = check_array(X, dtype=DTYPE, order='C')             # <<<<<<<<<<<<<<
 * 
 *         if X.shape[X.ndim - 1] != n_features:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_check_array); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1590; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1590; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_X);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_X);
  __Pyx_GIVEREF(__pyx_v_X);
  __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1590; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1590; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_dtype, __pyx_t_7) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1590; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1590; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1590; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF_SET(__pyx_v_X, __pyx_t_7);
  __pyx_t_7 = 0;

  /* "binary_tree.pxi":1592
 *         X = check_array(X, dtype=DTYPE, order='C')
 * 
 *         if X.shape[X.ndim - 1] != n_features:             # <<<<<<<<<<<<<<
 *             raise ValueError("query data dimension must "
 *                              "match training data dimension")
 */
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_shape); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_ndim); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = PyNumber_Subtract(__pyx_t_6, __pyx_int_1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = PyObject_GetItem(__pyx_t_7, __pyx_t_5); if (unlikely(__pyx_t_6 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1592; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n_features); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_7 = PyObject_RichCompare(__pyx_t_6, __pyx_t_5, Py_NE); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__pyx_t_3) {

    /* "binary_tree.pxi":1593
 * 
 *         if X.shape[X.ndim - 1] != n_features:
 *             raise ValueError("query data dimension must "             # <<<<<<<<<<<<<<
 *                              "match training data dimension")
 *         Xarr_np = X.reshape((-1, n_features))
 */
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__27, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1593; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_Raise(__pyx_t_7, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1593; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "binary_tree.pxi":1595
 *             raise ValueError("query data dimension must "
 *                              "match training data dimension")
 *         Xarr_np = X.reshape((-1, n_features))             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t[:, ::1] Xarr = get_memview_DTYPE_2D(Xarr_np)
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_reshape); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n_features); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_int_neg_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_int_neg_1);
  __Pyx_GIVEREF(__pyx_int_neg_1);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_6);
  __pyx_t_6 = 0;
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_7);
  } else {
    __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __Pyx_GIVEREF(__pyx_t_6); __pyx_t_6 = NULL;
    PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_Xarr_np = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "binary_tree.pxi":1596
 *                              "match training data dimension")
 *         Xarr_np = X.reshape((-1, n_features))
 *         cdef DTYPE_t[:, ::1] Xarr = get_memview_DTYPE_2D(Xarr_np)             # <<<<<<<<<<<<<<
 * 
 *         log_density_arr = np.zeros(Xarr.shape[0], dtype=DTYPE)
 */
  if (!(likely(((__pyx_v_Xarr_np) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_Xarr_np, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_9 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_2D(((PyArrayObject *)__pyx_v_Xarr_np)); if (unlikely(!__pyx_t_9.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_Xarr = __pyx_t_9;
  __pyx_t_9.memview = NULL;
  __pyx_t_9.data = NULL;

  /* "binary_tree.pxi":1598
 *         cdef DTYPE_t[:, ::1] Xarr = get_memview_DTYPE_2D(Xarr_np)
 * 
 *         log_density_arr = np.zeros(Xarr.shape[0], dtype=DTYPE)             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t[::1] log_density = get_memview_DTYPE_1D(log_density_arr)
 * 
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1598; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1598; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyInt_FromSsize_t((__pyx_v_Xarr.shape[0])); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1598; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1598; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_7);
  __pyx_t_7 = 0;
  __pyx_t_7 = PyDict_New(); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1598; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1598; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1598; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1598; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_log_density_arr = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "binary_tree.pxi":1599
 * 
 *         log_density_arr = np.zeros(Xarr.shape[0], dtype=DTYPE)
 *         cdef DTYPE_t[::1] log_density = get_memview_DTYPE_1D(log_density_arr)             # <<<<<<<<<<<<<<
 * 
 *         cdef DTYPE_t* pt = &Xarr[0, 0]
 */
  if (!(likely(((__pyx_v_log_density_arr) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_log_density_arr, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1599; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_10 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_1D(((PyArrayObject *)__pyx_v_log_density_arr)); if (unlikely(!__pyx_t_10.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1599; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_log_density = __pyx_t_10;
  __pyx_t_10.memview = NULL;
  __pyx_t_10.data = NULL;

  /* "binary_tree.pxi":1601
 *         cdef DTYPE_t[::1] log_density = get_memview_DTYPE_1D(log_density_arr)
 * 
 *         cdef DTYPE_t* pt = &Xarr[0, 0]             # <<<<<<<<<<<<<<
 * 
 *         cdef NodeHeap nodeheap
 */
  __pyx_t_11 = 0;
  __pyx_t_12 = 0;
  __pyx_v_pt = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_Xarr.data + __pyx_t_11 * __pyx_v_Xarr.strides[0]) )) + __pyx_t_12)) ))));

  /* "binary_tree.pxi":1604
 * 
 *         cdef NodeHeap nodeheap
 *         if breadth_first:             # <<<<<<<<<<<<<<
 *             nodeheap = NodeHeap(self.data.shape[0] // self.leaf_size)
 *         cdef DTYPE_t[::1] node_log_min_bounds
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_breadth_first); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1604; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_3) {

    /* "binary_tree.pxi":1605
 *         cdef NodeHeap nodeheap
 *         if breadth_first:
 *             nodeheap = NodeHeap(self.data.shape[0] // self.leaf_size)             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t[::1] node_log_min_bounds
 *         cdef DTYPE_t[::1] node_bound_widths
 */
    if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1605; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
    __pyx_t_4 = PyInt_FromSsize_t(((__pyx_v_self->data.shape[0]) / __pyx_v_self->leaf_size)); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1605; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1605; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)((PyObject*)__pyx_ptype_7sklearn_9neighbors_7kd_tree_NodeHeap)), __pyx_t_7, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1605; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_v_nodeheap = ((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_t_4);
    __pyx_t_4 = 0;
    goto __pyx_L5;
  }
  __pyx_L5:;

  /* "binary_tree.pxi":1611
 *         #       this is difficult because of the need to cache values
 *         #       computed between node pairs.
 *         if breadth_first:             # <<<<<<<<<<<<<<
 *             node_log_min_bounds_arr = -np.inf + np.zeros(self.n_nodes)
 *             node_log_min_bounds = get_memview_DTYPE_1D(node_log_min_bounds_arr)
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_breadth_first); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1611; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_3) {

    /* "binary_tree.pxi":1612
 *         #       computed between node pairs.
 *         if breadth_first:
 *             node_log_min_bounds_arr = -np.inf + np.zeros(self.n_nodes)             # <<<<<<<<<<<<<<
 *             node_log_min_bounds = get_memview_DTYPE_1D(node_log_min_bounds_arr)
 *             node_bound_widths_arr = np.zeros(self.n_nodes)
 */
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1612; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_inf); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1612; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyNumber_Negative(__pyx_t_7); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1612; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1612; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1612; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->n_nodes); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1612; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_6 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    if (!__pyx_t_6) {
      __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_8); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1612; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_7);
    } else {
      __pyx_t_13 = PyTuple_New(1+1); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1612; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_13);
      PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_6); __Pyx_GIVEREF(__pyx_t_6); __pyx_t_6 = NULL;
      PyTuple_SET_ITEM(__pyx_t_13, 0+1, __pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_8);
      __pyx_t_8 = 0;
      __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_13, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1612; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyNumber_Add(__pyx_t_4, __pyx_t_7); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1612; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_v_node_log_min_bounds_arr = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "binary_tree.pxi":1613
 *         if breadth_first:
 *             node_log_min_bounds_arr = -np.inf + np.zeros(self.n_nodes)
 *             node_log_min_bounds = get_memview_DTYPE_1D(node_log_min_bounds_arr)             # <<<<<<<<<<<<<<
 *             node_bound_widths_arr = np.zeros(self.n_nodes)
 *             node_bound_widths = get_memview_DTYPE_1D(node_bound_widths_arr)
 */
    if (!(likely(((__pyx_v_node_log_min_bounds_arr) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_node_log_min_bounds_arr, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1613; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_10 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_1D(((PyArrayObject *)__pyx_v_node_log_min_bounds_arr)); if (unlikely(!__pyx_t_10.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1613; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_node_log_min_bounds = __pyx_t_10;
    __pyx_t_10.memview = NULL;
    __pyx_t_10.data = NULL;

    /* "binary_tree.pxi":1614
 *             node_log_min_bounds_arr = -np.inf + np.zeros(self.n_nodes)
 *             node_log_min_bounds = get_memview_DTYPE_1D(node_log_min_bounds_arr)
 *             node_bound_widths_arr = np.zeros(self.n_nodes)             # <<<<<<<<<<<<<<
 *             node_bound_widths = get_memview_DTYPE_1D(node_bound_widths_arr)
 *             for i in range(Xarr.shape[0]):
 */
    __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->n_nodes); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_13 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_13)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_13);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_13) {
      __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_7); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_5);
    } else {
      __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_13); __Pyx_GIVEREF(__pyx_t_13); __pyx_t_13 = NULL;
      PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_7);
      __pyx_t_7 = 0;
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_8, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_node_bound_widths_arr = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "binary_tree.pxi":1615
 *             node_log_min_bounds = get_memview_DTYPE_1D(node_log_min_bounds_arr)
 *             node_bound_widths_arr = np.zeros(self.n_nodes)
 *             node_bound_widths = get_memview_DTYPE_1D(node_bound_widths_arr)             # <<<<<<<<<<<<<<
 *             for i in range(Xarr.shape[0]):
 *                 log_density[i] = self._kde_single_breadthfirst(
 */
    if (!(likely(((__pyx_v_node_bound_widths_arr) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_node_bound_widths_arr, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1615; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_10 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_1D(((PyArrayObject *)__pyx_v_node_bound_widths_arr)); if (unlikely(!__pyx_t_10.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1615; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_node_bound_widths = __pyx_t_10;
    __pyx_t_10.memview = NULL;
    __pyx_t_10.data = NULL;

    /* "binary_tree.pxi":1616
 *             node_bound_widths_arr = np.zeros(self.n_nodes)
 *             node_bound_widths = get_memview_DTYPE_1D(node_bound_widths_arr)
 *             for i in range(Xarr.shape[0]):             # <<<<<<<<<<<<<<
 *                 log_density[i] = self._kde_single_breadthfirst(
 *                                             pt, kernel_c, h_c,
 */
    __pyx_t_14 = (__pyx_v_Xarr.shape[0]);
    for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
      __pyx_v_i = __pyx_t_15;

      /* "binary_tree.pxi":1620
 *                                             pt, kernel_c, h_c,
 *                                             log_knorm, log_atol, log_rtol,
 *                                             nodeheap,             # <<<<<<<<<<<<<<
 *                                             &node_log_min_bounds[0],
 *                                             &node_bound_widths[0])
 */
      if (unlikely(!__pyx_v_nodeheap)) { __Pyx_RaiseUnboundLocalError("nodeheap"); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1620; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }

      /* "binary_tree.pxi":1621
 *                                             log_knorm, log_atol, log_rtol,
 *                                             nodeheap,
 *                                             &node_log_min_bounds[0],             # <<<<<<<<<<<<<<
 *                                             &node_bound_widths[0])
 *                 pt += n_features
 */
      __pyx_t_16 = 0;

      /* "binary_tree.pxi":1622
 *                                             nodeheap,
 *                                             &node_log_min_bounds[0],
 *                                             &node_bound_widths[0])             # <<<<<<<<<<<<<<
 *                 pt += n_features
 *         else:
 */
      __pyx_t_17 = 0;

      /* "binary_tree.pxi":1617
 *             node_bound_widths = get_memview_DTYPE_1D(node_bound_widths_arr)
 *             for i in range(Xarr.shape[0]):
 *                 log_density[i] = self._kde_single_breadthfirst(             # <<<<<<<<<<<<<<
 *                                             pt, kernel_c, h_c,
 *                                             log_knorm, log_atol, log_rtol,
 */
      __pyx_t_18 = __pyx_v_i;
      *((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_log_density.data) + __pyx_t_18)) )) = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_kde_single_breadthfirst(__pyx_v_self, __pyx_v_pt, __pyx_v_kernel_c, __pyx_v_h_c, __pyx_v_log_knorm, __pyx_v_log_atol, __pyx_v_log_rtol, __pyx_v_nodeheap, (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_node_log_min_bounds.data) + __pyx_t_16)) )))), (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_node_bound_widths.data) + __pyx_t_17)) )))));

      /* "binary_tree.pxi":1623
 *                                             &node_log_min_bounds[0],
 *                                             &node_bound_widths[0])
 *                 pt += n_features             # <<<<<<<<<<<<<<
 *         else:
 *             for i in range(Xarr.shape[0]):
 */
      __pyx_v_pt = (__pyx_v_pt + __pyx_v_n_features);
    }
    goto __pyx_L6;
  }
  /*else*/ {

    /* "binary_tree.pxi":1625
 *                 pt += n_features
 *         else:
 *             for i in range(Xarr.shape[0]):             # <<<<<<<<<<<<<<
 *                 min_max_dist(self, 0, pt, &dist_LB, &dist_UB)
 *                 # compute max & min bounds on density within top node
 */
    __pyx_t_14 = (__pyx_v_Xarr.shape[0]);
    for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
      __pyx_v_i = __pyx_t_15;

      /* "binary_tree.pxi":1626
 *         else:
 *             for i in range(Xarr.shape[0]):
 *                 min_max_dist(self, 0, pt, &dist_LB, &dist_UB)             # <<<<<<<<<<<<<<
 *                 # compute max & min bounds on density within top node
 *                 log_min_bound = (log(n_samples) +
 */
      __pyx_t_19 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_max_dist(__pyx_v_self, 0, __pyx_v_pt, (&__pyx_v_dist_LB), (&__pyx_v_dist_UB)); if (unlikely(__pyx_t_19 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "binary_tree.pxi":1628
 *                 min_max_dist(self, 0, pt, &dist_LB, &dist_UB)
 *                 # compute max & min bounds on density within top node
 *                 log_min_bound = (log(n_samples) +             # <<<<<<<<<<<<<<
 *                                  compute_log_kernel(dist_UB,
 *                                                     h_c, kernel_c))
 */
      __pyx_v_log_min_bound = (log(__pyx_v_n_samples) + __pyx_f_7sklearn_9neighbors_7kd_tree_compute_log_kernel(__pyx_v_dist_UB, __pyx_v_h_c, __pyx_v_kernel_c));

      /* "binary_tree.pxi":1631
 *                                  compute_log_kernel(dist_UB,
 *                                                     h_c, kernel_c))
 *                 log_max_bound = (log(n_samples) +             # <<<<<<<<<<<<<<
 *                                  compute_log_kernel(dist_LB,
 *                                                     h_c, kernel_c))
 */
      __pyx_v_log_max_bound = (log(__pyx_v_n_samples) + __pyx_f_7sklearn_9neighbors_7kd_tree_compute_log_kernel(__pyx_v_dist_LB, __pyx_v_h_c, __pyx_v_kernel_c));

      /* "binary_tree.pxi":1634
 *                                  compute_log_kernel(dist_LB,
 *                                                     h_c, kernel_c))
 *                 log_bound_spread = logsubexp(log_max_bound, log_min_bound)             # <<<<<<<<<<<<<<
 *                 self._kde_single_depthfirst(0, pt, kernel_c, h_c,
 *                                             log_knorm, log_atol, log_rtol,
 */
      __pyx_v_log_bound_spread = __pyx_f_7sklearn_9neighbors_7kd_tree_logsubexp(__pyx_v_log_max_bound, __pyx_v_log_min_bound);

      /* "binary_tree.pxi":1635
 *                                                     h_c, kernel_c))
 *                 log_bound_spread = logsubexp(log_max_bound, log_min_bound)
 *                 self._kde_single_depthfirst(0, pt, kernel_c, h_c,             # <<<<<<<<<<<<<<
 *                                             log_knorm, log_atol, log_rtol,
 *                                             log_min_bound,
 */
      __pyx_t_19 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_kde_single_depthfirst(__pyx_v_self, 0, __pyx_v_pt, __pyx_v_kernel_c, __pyx_v_h_c, __pyx_v_log_knorm, __pyx_v_log_atol, __pyx_v_log_rtol, __pyx_v_log_min_bound, __pyx_v_log_bound_spread, (&__pyx_v_log_min_bound), (&__pyx_v_log_bound_spread)); if (unlikely(__pyx_t_19 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1635; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "binary_tree.pxi":1641
 *                                             &log_min_bound,
 *                                             &log_bound_spread)
 *                 log_density[i] = logaddexp(log_min_bound,             # <<<<<<<<<<<<<<
 *                                            log_bound_spread - log(2))
 *                 pt += n_features
 */
      __pyx_t_20 = __pyx_v_i;
      *((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_log_density.data) + __pyx_t_20)) )) = __pyx_f_7sklearn_9neighbors_7kd_tree_logaddexp(__pyx_v_log_min_bound, (__pyx_v_log_bound_spread - log(2.0)));

      /* "binary_tree.pxi":1643
 *                 log_density[i] = logaddexp(log_min_bound,
 *                                            log_bound_spread - log(2))
 *                 pt += n_features             # <<<<<<<<<<<<<<
 * 
 *         # normalize the results
 */
      __pyx_v_pt = (__pyx_v_pt + __pyx_v_n_features);
    }
  }
  __pyx_L6:;

  /* "binary_tree.pxi":1646
 * 
 *         # normalize the results
 *         for i in range(log_density.shape[0]):             # <<<<<<<<<<<<<<
 *             log_density[i] += log_knorm
 * 
 */
  __pyx_t_14 = (__pyx_v_log_density.shape[0]);
  for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
    __pyx_v_i = __pyx_t_15;

    /* "binary_tree.pxi":1647
 *         # normalize the results
 *         for i in range(log_density.shape[0]):
 *             log_density[i] += log_knorm             # <<<<<<<<<<<<<<
 * 
 *         log_density_arr = log_density_arr.reshape(X.shape[:X.ndim - 1])
 */
    __pyx_t_21 = __pyx_v_i;
    *((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_log_density.data) + __pyx_t_21)) )) += __pyx_v_log_knorm;
  }

  /* "binary_tree.pxi":1649
 *             log_density[i] += log_knorm
 * 
 *         log_density_arr = log_density_arr.reshape(X.shape[:X.ndim - 1])             # <<<<<<<<<<<<<<
 * 
 *         if return_log:
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_log_density_arr, __pyx_n_s_reshape); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1649; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_shape); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1649; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_ndim); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1649; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_13 = PyNumber_Subtract(__pyx_t_7, __pyx_int_1); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1649; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_GetSlice(__pyx_t_8, 0, 0, NULL, &__pyx_t_13, NULL, 0, 0, 0); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1649; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_13)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_13);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_13) {
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_7); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1649; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_5);
  } else {
    __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1649; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_13); __Pyx_GIVEREF(__pyx_t_13); __pyx_t_13 = NULL;
    PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_8, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1649; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF_SET(__pyx_v_log_density_arr, __pyx_t_5);
  __pyx_t_5 = 0;

  /* "binary_tree.pxi":1651
 *         log_density_arr = log_density_arr.reshape(X.shape[:X.ndim - 1])
 * 
 *         if return_log:             # <<<<<<<<<<<<<<
 *             return log_density_arr
 *         else:
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_return_log); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1651; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_3) {

    /* "binary_tree.pxi":1652
 * 
 *         if return_log:
 *             return log_density_arr             # <<<<<<<<<<<<<<
 *         else:
 *             return np.exp(log_density_arr)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_log_density_arr);
    __pyx_r = __pyx_v_log_density_arr;
    goto __pyx_L0;
  }
  /*else*/ {

    /* "binary_tree.pxi":1654
 *             return log_density_arr
 *         else:
 *             return np.exp(log_density_arr)             # <<<<<<<<<<<<<<
 * 
 *     def two_point_correlation(self, X, r, dualtree=False):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1654; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_exp); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1654; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    if (!__pyx_t_4) {
      __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_v_log_density_arr); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1654; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
    } else {
      __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1654; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_4); __Pyx_GIVEREF(__pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_INCREF(__pyx_v_log_density_arr);
      PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_v_log_density_arr);
      __Pyx_GIVEREF(__pyx_v_log_density_arr);
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_7, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1654; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "binary_tree.pxi":1505
 *             return indices.reshape(X.shape[:X.ndim - 1])
 * 
 *     def kernel_density(self, X, h, kernel='gaussian',             # <<<<<<<<<<<<<<
 *                        atol=0, rtol=1E-8,
 *                        breadth_first=True, return_log=False):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __PYX_XDEC_MEMVIEW(&__pyx_t_9, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_10, 1);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.kernel_density", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_Xarr_np);
  __PYX_XDEC_MEMVIEW(&__pyx_v_Xarr, 1);
  __Pyx_XDECREF(__pyx_v_log_density_arr);
  __PYX_XDEC_MEMVIEW(&__pyx_v_log_density, 1);
  __Pyx_XDECREF((PyObject *)__pyx_v_nodeheap);
  __PYX_XDEC_MEMVIEW(&__pyx_v_node_log_min_bounds, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_node_bound_widths, 1);
  __Pyx_XDECREF(__pyx_v_node_log_min_bounds_arr);
  __Pyx_XDECREF(__pyx_v_node_bound_widths_arr);
  __Pyx_XDECREF(__pyx_v_X);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1656
 *             return np.exp(log_density_arr)
 * 
 *     def two_point_correlation(self, X, r, dualtree=False):             # <<<<<<<<<<<<<<
 *         """Compute the two-point correlation function
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_25two_point_correlation(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7sklearn_9neighbors_7kd_tree_10BinaryTree_24two_point_correlation[] = "Compute the two-point correlation function\n\n        Parameters\n        ----------\n        X : array_like\n            An array of points to query.  Last dimension should match dimension\n            of training data.\n        r : array_like\n            A one-dimensional array of distances\n        dualtree : boolean (default = False)\n            If true, use a dualtree algorithm.  Otherwise, use a single-tree\n            algorithm.  Dual tree algorithms can have better scaling for\n            large N.\n\n        Returns\n        -------\n        counts : ndarray\n            counts[i] contains the number of pairs of points with distance\n            less than or equal to r[i]\n\n        Examples\n        --------\n        Compute the two-point autocorrelation function of X:\n\n        >>> import numpy as np\n        >>> np.random.seed(0)\n        >>> X = np.random.random((30, 3))\n        >>> r = np.linspace(0, 1, 5)\n        >>> tree = BinaryTree(X)     # doctest: +SKIP\n        >>> tree.two_point_correlation(X, r)\n        array([ 30,  62, 278, 580, 820])\n        ";
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_25two_point_correlation(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_X = 0;
  PyObject *__pyx_v_r = 0;
  PyObject *__pyx_v_dualtree = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("two_point_correlation (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_X,&__pyx_n_s_r,&__pyx_n_s_dualtree,0};
    PyObject* values[3] = {0,0,0};
    values[2] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_X)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_r)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("two_point_correlation", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1656; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_dualtree);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "two_point_correlation") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1656; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_X = values[0];
    __pyx_v_r = values[1];
    __pyx_v_dualtree = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("two_point_correlation", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1656; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.two_point_correlation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_24two_point_correlation(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self), __pyx_v_X, __pyx_v_r, __pyx_v_dualtree);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_24two_point_correlation(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, PyObject *__pyx_v_X, PyObject *__pyx_v_r, PyObject *__pyx_v_dualtree) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_features;
  CYTHON_UNUSED __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i;
  PyObject *__pyx_v_np_Xarr = NULL;
  __Pyx_memviewslice __pyx_v_Xarr = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_v_i_rsort = NULL;
  PyObject *__pyx_v_rarr_np = NULL;
  __Pyx_memviewslice __pyx_v_rarr = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_v_count = NULL;
  __Pyx_memviewslice __pyx_v_carr = { 0, 0, { 0 }, { 0 }, { 0 } };
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt;
  PyObject *__pyx_v_other = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_8 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_9 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  int __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("two_point_correlation", 0);
  __Pyx_INCREF(__pyx_v_X);
  __Pyx_INCREF(__pyx_v_r);

  /* "binary_tree.pxi":1689
 *         array([ 30,  62, 278, 580, 820])
 *         """
 *         cdef ITYPE_t n_features = self.data.shape[1]             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t i
 * 
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1689; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_n_features = (__pyx_v_self->data.shape[1]);

  /* "binary_tree.pxi":1693
 * 
 *         # validate X and prepare for query
 *         X = check_array(X, dtype=DTYPE, order='C')             # <<<<<<<<<<<<<<
 * 
 *         if X.shape[X.ndim - 1] != self.data.shape[1]:
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_check_array); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1693; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1693; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_X);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_X);
  __Pyx_GIVEREF(__pyx_v_X);
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1693; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1693; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1693; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1693; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1693; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF_SET(__pyx_v_X, __pyx_t_4);
  __pyx_t_4 = 0;

  /* "binary_tree.pxi":1695
 *         X = check_array(X, dtype=DTYPE, order='C')
 * 
 *         if X.shape[X.ndim - 1] != self.data.shape[1]:             # <<<<<<<<<<<<<<
 *             raise ValueError("query data dimension must "
 *                              "match training data dimension")
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_shape); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_ndim); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyNumber_Subtract(__pyx_t_3, __pyx_int_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyObject_GetItem(__pyx_t_4, __pyx_t_2); if (unlikely(__pyx_t_3 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_2 = PyInt_FromSsize_t((__pyx_v_self->data.shape[1])); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_t_2, Py_NE); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_5) {

    /* "binary_tree.pxi":1696
 * 
 *         if X.shape[X.ndim - 1] != self.data.shape[1]:
 *             raise ValueError("query data dimension must "             # <<<<<<<<<<<<<<
 *                              "match training data dimension")
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__28, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "binary_tree.pxi":1699
 *                              "match training data dimension")
 * 
 *         np_Xarr = X.reshape((-1, self.data.shape[1]))             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t[:, ::1] Xarr = get_memview_DTYPE_2D(np_Xarr)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_reshape); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1699; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1699; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_3 = PyInt_FromSsize_t((__pyx_v_self->data.shape[1])); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1699; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1699; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_neg_1);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_int_neg_1);
  __Pyx_GIVEREF(__pyx_int_neg_1);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1699; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_4);
  } else {
    __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1699; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3); __Pyx_GIVEREF(__pyx_t_3); __pyx_t_3 = NULL;
    PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1699; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_np_Xarr = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "binary_tree.pxi":1700
 * 
 *         np_Xarr = X.reshape((-1, self.data.shape[1]))
 *         cdef DTYPE_t[:, ::1] Xarr = get_memview_DTYPE_2D(np_Xarr)             # <<<<<<<<<<<<<<
 * 
 *         # prepare r for query
 */
  if (!(likely(((__pyx_v_np_Xarr) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_np_Xarr, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1700; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_7 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_2D(((PyArrayObject *)__pyx_v_np_Xarr)); if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1700; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_Xarr = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "binary_tree.pxi":1703
 * 
 *         # prepare r for query
 *         r = np.asarray(r, dtype=DTYPE, order='C')             # <<<<<<<<<<<<<<
 *         r = np.atleast_1d(r)
 *         if r.ndim != 1:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1703; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1703; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1703; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_r);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_r);
  __Pyx_GIVEREF(__pyx_v_r);
  __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1703; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1703; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1703; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1703; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1703; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF_SET(__pyx_v_r, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "binary_tree.pxi":1704
 *         # prepare r for query
 *         r = np.asarray(r, dtype=DTYPE, order='C')
 *         r = np.atleast_1d(r)             # <<<<<<<<<<<<<<
 *         if r.ndim != 1:
 *             raise ValueError("r must be a 1-dimensional array")
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1704; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_atleast_1d); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1704; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_r); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1704; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_2 = PyTuple_New(1+1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1704; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_6); __Pyx_GIVEREF(__pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_INCREF(__pyx_v_r);
    PyTuple_SET_ITEM(__pyx_t_2, 0+1, __pyx_v_r);
    __Pyx_GIVEREF(__pyx_v_r);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1704; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF_SET(__pyx_v_r, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "binary_tree.pxi":1705
 *         r = np.asarray(r, dtype=DTYPE, order='C')
 *         r = np.atleast_1d(r)
 *         if r.ndim != 1:             # <<<<<<<<<<<<<<
 *             raise ValueError("r must be a 1-dimensional array")
 *         i_rsort = np.argsort(r)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_r, __pyx_n_s_ndim); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1705; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyObject_RichCompare(__pyx_t_1, __pyx_int_1, Py_NE); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1705; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1705; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_5) {

    /* "binary_tree.pxi":1706
 *         r = np.atleast_1d(r)
 *         if r.ndim != 1:
 *             raise ValueError("r must be a 1-dimensional array")             # <<<<<<<<<<<<<<
 *         i_rsort = np.argsort(r)
 *         rarr_np = r[i_rsort]  # needed to keep memory in scope
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__29, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1706; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1706; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "binary_tree.pxi":1707
 *         if r.ndim != 1:
 *             raise ValueError("r must be a 1-dimensional array")
 *         i_rsort = np.argsort(r)             # <<<<<<<<<<<<<<
 *         rarr_np = r[i_rsort]  # needed to keep memory in scope
 *         cdef DTYPE_t[::1] rarr = get_memview_DTYPE_1D(rarr_np)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1707; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_argsort); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1707; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_1) {
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_r); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1707; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
  } else {
    __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1707; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1); __Pyx_GIVEREF(__pyx_t_1); __pyx_t_1 = NULL;
    __Pyx_INCREF(__pyx_v_r);
    PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_v_r);
    __Pyx_GIVEREF(__pyx_v_r);
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1707; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_i_rsort = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "binary_tree.pxi":1708
 *             raise ValueError("r must be a 1-dimensional array")
 *         i_rsort = np.argsort(r)
 *         rarr_np = r[i_rsort]  # needed to keep memory in scope             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t[::1] rarr = get_memview_DTYPE_1D(rarr_np)
 * 
 */
  __pyx_t_4 = PyObject_GetItem(__pyx_v_r, __pyx_v_i_rsort); if (unlikely(__pyx_t_4 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1708; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v_rarr_np = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "binary_tree.pxi":1709
 *         i_rsort = np.argsort(r)
 *         rarr_np = r[i_rsort]  # needed to keep memory in scope
 *         cdef DTYPE_t[::1] rarr = get_memview_DTYPE_1D(rarr_np)             # <<<<<<<<<<<<<<
 * 
 *         # create array to hold counts
 */
  if (!(likely(((__pyx_v_rarr_np) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_rarr_np, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1709; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_8 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_1D(((PyArrayObject *)__pyx_v_rarr_np)); if (unlikely(!__pyx_t_8.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1709; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_rarr = __pyx_t_8;
  __pyx_t_8.memview = NULL;
  __pyx_t_8.data = NULL;

  /* "binary_tree.pxi":1712
 * 
 *         # create array to hold counts
 *         count = np.zeros(r.shape[0], dtype=ITYPE)             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t[::1] carr = get_memview_ITYPE_1D(count)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1712; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1712; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_r, __pyx_n_s_shape); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1712; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_GetItemInt(__pyx_t_4, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_6 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1712; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1712; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_6);
  __pyx_t_6 = 0;
  __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1712; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1712; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1712; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1712; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_count = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "binary_tree.pxi":1713
 *         # create array to hold counts
 *         count = np.zeros(r.shape[0], dtype=ITYPE)
 *         cdef ITYPE_t[::1] carr = get_memview_ITYPE_1D(count)             # <<<<<<<<<<<<<<
 * 
 *         cdef DTYPE_t* pt = &Xarr[0, 0]
 */
  if (!(likely(((__pyx_v_count) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_count, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1713; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_9 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_ITYPE_1D(((PyArrayObject *)__pyx_v_count)); if (unlikely(!__pyx_t_9.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1713; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_carr = __pyx_t_9;
  __pyx_t_9.memview = NULL;
  __pyx_t_9.data = NULL;

  /* "binary_tree.pxi":1715
 *         cdef ITYPE_t[::1] carr = get_memview_ITYPE_1D(count)
 * 
 *         cdef DTYPE_t* pt = &Xarr[0, 0]             # <<<<<<<<<<<<<<
 * 
 *         if dualtree:
 */
  __pyx_t_10 = 0;
  __pyx_t_11 = 0;
  __pyx_v_pt = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_Xarr.data + __pyx_t_10 * __pyx_v_Xarr.strides[0]) )) + __pyx_t_11)) ))));

  /* "binary_tree.pxi":1717
 *         cdef DTYPE_t* pt = &Xarr[0, 0]
 * 
 *         if dualtree:             # <<<<<<<<<<<<<<
 *             other = self.__class__(Xarr, metric=self.dist_metric,
 *                                    leaf_size=self.leaf_size)
 */
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_dualtree); if (unlikely(__pyx_t_5 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1717; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_5) {

    /* "binary_tree.pxi":1718
 * 
 *         if dualtree:
 *             other = self.__class__(Xarr, metric=self.dist_metric,             # <<<<<<<<<<<<<<
 *                                    leaf_size=self.leaf_size)
 *             self._two_point_dual(0, other, 0, &rarr[0], &carr[0],
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_Xarr, 2, (PyObject *(*)(char *)) __pyx_memview_get_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, (int (*)(char *, PyObject *)) __pyx_memview_set_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_metric, ((PyObject *)__pyx_v_self->dist_metric)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "binary_tree.pxi":1719
 *         if dualtree:
 *             other = self.__class__(Xarr, metric=self.dist_metric,
 *                                    leaf_size=self.leaf_size)             # <<<<<<<<<<<<<<
 *             self._two_point_dual(0, other, 0, &rarr[0], &carr[0],
 *                                  0, rarr.shape[0])
 */
    __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->leaf_size); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1719; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_leaf_size, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "binary_tree.pxi":1718
 * 
 *         if dualtree:
 *             other = self.__class__(Xarr, metric=self.dist_metric,             # <<<<<<<<<<<<<<
 *                                    leaf_size=self.leaf_size)
 *             self._two_point_dual(0, other, 0, &rarr[0], &carr[0],
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_other = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "binary_tree.pxi":1720
 *             other = self.__class__(Xarr, metric=self.dist_metric,
 *                                    leaf_size=self.leaf_size)
 *             self._two_point_dual(0, other, 0, &rarr[0], &carr[0],             # <<<<<<<<<<<<<<
 *                                  0, rarr.shape[0])
 *         else:
 */
    if (!(likely(((__pyx_v_other) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_other, __pyx_ptype_7sklearn_9neighbors_7kd_tree_BinaryTree))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1720; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_12 = 0;
    __pyx_t_13 = 0;

    /* "binary_tree.pxi":1721
 *                                    leaf_size=self.leaf_size)
 *             self._two_point_dual(0, other, 0, &rarr[0], &carr[0],
 *                                  0, rarr.shape[0])             # <<<<<<<<<<<<<<
 *         else:
 *             for i in range(Xarr.shape[0]):
 */
    __pyx_t_14 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_two_point_dual(__pyx_v_self, 0, ((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_other), 0, (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_rarr.data) + __pyx_t_12)) )))), (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_carr.data) + __pyx_t_13)) )))), 0, (__pyx_v_rarr.shape[0])); if (unlikely(__pyx_t_14 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1720; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L5;
  }
  /*else*/ {

    /* "binary_tree.pxi":1723
 *                                  0, rarr.shape[0])
 *         else:
 *             for i in range(Xarr.shape[0]):             # <<<<<<<<<<<<<<
 *                 self._two_point_single(0, pt, &rarr[0], &carr[0],
 *                                        0, rarr.shape[0])
 */
    __pyx_t_15 = (__pyx_v_Xarr.shape[0]);
    for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
      __pyx_v_i = __pyx_t_16;

      /* "binary_tree.pxi":1724
 *         else:
 *             for i in range(Xarr.shape[0]):
 *                 self._two_point_single(0, pt, &rarr[0], &carr[0],             # <<<<<<<<<<<<<<
 *                                        0, rarr.shape[0])
 *                 pt += n_features
 */
      __pyx_t_17 = 0;
      __pyx_t_18 = 0;

      /* "binary_tree.pxi":1725
 *             for i in range(Xarr.shape[0]):
 *                 self._two_point_single(0, pt, &rarr[0], &carr[0],
 *                                        0, rarr.shape[0])             # <<<<<<<<<<<<<<
 *                 pt += n_features
 * 
 */
      __pyx_t_14 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_two_point_single(__pyx_v_self, 0, __pyx_v_pt, (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_rarr.data) + __pyx_t_17)) )))), (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_carr.data) + __pyx_t_18)) )))), 0, (__pyx_v_rarr.shape[0])); if (unlikely(__pyx_t_14 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1724; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "binary_tree.pxi":1726
 *                 self._two_point_single(0, pt, &rarr[0], &carr[0],
 *                                        0, rarr.shape[0])
 *                 pt += n_features             # <<<<<<<<<<<<<<
 * 
 *         return count
 */
      __pyx_v_pt = (__pyx_v_pt + __pyx_v_n_features);
    }
  }
  __pyx_L5:;

  /* "binary_tree.pxi":1728
 *                 pt += n_features
 * 
 *         return count             # <<<<<<<<<<<<<<
 * 
 *     cdef int _query_single_depthfirst(self, ITYPE_t i_node,
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_count);
  __pyx_r = __pyx_v_count;
  goto __pyx_L0;

  /* "binary_tree.pxi":1656
 *             return np.exp(log_density_arr)
 * 
 *     def two_point_correlation(self, X, r, dualtree=False):             # <<<<<<<<<<<<<<
 *         """Compute the two-point correlation function
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_8, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_9, 1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.two_point_correlation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_np_Xarr);
  __PYX_XDEC_MEMVIEW(&__pyx_v_Xarr, 1);
  __Pyx_XDECREF(__pyx_v_i_rsort);
  __Pyx_XDECREF(__pyx_v_rarr_np);
  __PYX_XDEC_MEMVIEW(&__pyx_v_rarr, 1);
  __Pyx_XDECREF(__pyx_v_count);
  __PYX_XDEC_MEMVIEW(&__pyx_v_carr, 1);
  __Pyx_XDECREF(__pyx_v_other);
  __Pyx_XDECREF(__pyx_v_X);
  __Pyx_XDECREF(__pyx_v_r);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1730
 *         return count
 * 
 *     cdef int _query_single_depthfirst(self, ITYPE_t i_node,             # <<<<<<<<<<<<<<
 *                                       DTYPE_t* pt, ITYPE_t i_pt,
 *                                       NeighborsHeap heap,
 */

static int __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__query_single_depthfirst(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_pt, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_heap, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_reduced_dist_LB) {
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t __pyx_v_node_info;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_pt;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_reduced_dist_LB_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_reduced_dist_LB_2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i2;
  CYTHON_UNUSED __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_data;
  int __pyx_r;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_4;
  int __pyx_t_5;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_6;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_7;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_8;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_11;
  int __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "binary_tree.pxi":1735
 *                                       DTYPE_t reduced_dist_LB) nogil except -1:
 *         """Recursive Single-tree k-neighbors query, depth-first approach"""
 *         cdef NodeData_t node_info = self.node_data[i_node]             # <<<<<<<<<<<<<<
 * 
 *         cdef DTYPE_t dist_pt, reduced_dist_LB_1, reduced_dist_LB_2
 */
  if (unlikely(!__pyx_v_self->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1735; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_1 = __pyx_v_i_node;
  __pyx_v_node_info = (*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_self->node_data.data) + __pyx_t_1)) )));

  /* "binary_tree.pxi":1740
 *         cdef ITYPE_t i, i1, i2
 * 
 *         cdef DTYPE_t* data = &self.data[0, 0]             # <<<<<<<<<<<<<<
 * 
 *         #------------------------------------------------------------
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1740; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_data = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_self->data.data + __pyx_t_2 * __pyx_v_self->data.strides[0]) )) + __pyx_t_3)) ))));

  /* "binary_tree.pxi":1745
 *         # Case 1: query point is outside node radius:
 *         #         trim it from the query
 *         if reduced_dist_LB > heap.largest(i_pt):             # <<<<<<<<<<<<<<
 *             self.n_trims += 1
 * 
 */
  __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_largest(__pyx_v_heap, __pyx_v_i_pt); if (unlikely(__pyx_t_4 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1745; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_5 = ((__pyx_v_reduced_dist_LB > __pyx_t_4) != 0);
  if (__pyx_t_5) {

    /* "binary_tree.pxi":1746
 *         #         trim it from the query
 *         if reduced_dist_LB > heap.largest(i_pt):
 *             self.n_trims += 1             # <<<<<<<<<<<<<<
 * 
 *         #------------------------------------------------------------
 */
    __pyx_v_self->n_trims = (__pyx_v_self->n_trims + 1);
    goto __pyx_L3;
  }

  /* "binary_tree.pxi":1750
 *         #------------------------------------------------------------
 *         # Case 2: this is a leaf node.  Update set of nearby points
 *         elif node_info.is_leaf:             # <<<<<<<<<<<<<<
 *             self.n_leaves += 1
 *             for i in range(node_info.idx_start, node_info.idx_end):
 */
  __pyx_t_5 = (__pyx_v_node_info.is_leaf != 0);
  if (__pyx_t_5) {

    /* "binary_tree.pxi":1751
 *         # Case 2: this is a leaf node.  Update set of nearby points
 *         elif node_info.is_leaf:
 *             self.n_leaves += 1             # <<<<<<<<<<<<<<
 *             for i in range(node_info.idx_start, node_info.idx_end):
 *                 dist_pt = self.rdist(pt,
 */
    __pyx_v_self->n_leaves = (__pyx_v_self->n_leaves + 1);

    /* "binary_tree.pxi":1752
 *         elif node_info.is_leaf:
 *             self.n_leaves += 1
 *             for i in range(node_info.idx_start, node_info.idx_end):             # <<<<<<<<<<<<<<
 *                 dist_pt = self.rdist(pt,
 *                                      &self.data[self.idx_array[i], 0],
 */
    __pyx_t_6 = __pyx_v_node_info.idx_end;
    for (__pyx_t_7 = __pyx_v_node_info.idx_start; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
      __pyx_v_i = __pyx_t_7;

      /* "binary_tree.pxi":1754
 *             for i in range(node_info.idx_start, node_info.idx_end):
 *                 dist_pt = self.rdist(pt,
 *                                      &self.data[self.idx_array[i], 0],             # <<<<<<<<<<<<<<
 *                                      self.data.shape[1])
 *                 if dist_pt < heap.largest(i_pt):
 */
      if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1754; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      if (unlikely(!__pyx_v_self->idx_array.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1754; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_8 = __pyx_v_i;
      __pyx_t_9 = (*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_self->idx_array.data) + __pyx_t_8)) )));
      __pyx_t_10 = 0;

      /* "binary_tree.pxi":1755
 *                 dist_pt = self.rdist(pt,
 *                                      &self.data[self.idx_array[i], 0],
 *                                      self.data.shape[1])             # <<<<<<<<<<<<<<
 *                 if dist_pt < heap.largest(i_pt):
 *                     heap._push(i_pt, dist_pt, self.idx_array[i])
 */
      if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1755; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}

      /* "binary_tree.pxi":1753
 *             self.n_leaves += 1
 *             for i in range(node_info.idx_start, node_info.idx_end):
 *                 dist_pt = self.rdist(pt,             # <<<<<<<<<<<<<<
 *                                      &self.data[self.idx_array[i], 0],
 *                                      self.data.shape[1])
 */
      __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree_rdist(__pyx_v_self, __pyx_v_pt, (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_self->data.data + __pyx_t_9 * __pyx_v_self->data.strides[0]) )) + __pyx_t_10)) )))), (__pyx_v_self->data.shape[1])); if (unlikely(__pyx_t_4 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1753; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_v_dist_pt = __pyx_t_4;

      /* "binary_tree.pxi":1756
 *                                      &self.data[self.idx_array[i], 0],
 *                                      self.data.shape[1])
 *                 if dist_pt < heap.largest(i_pt):             # <<<<<<<<<<<<<<
 *                     heap._push(i_pt, dist_pt, self.idx_array[i])
 * 
 */
      __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_largest(__pyx_v_heap, __pyx_v_i_pt); if (unlikely(__pyx_t_4 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1756; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_t_5 = ((__pyx_v_dist_pt < __pyx_t_4) != 0);
      if (__pyx_t_5) {

        /* "binary_tree.pxi":1757
 *                                      self.data.shape[1])
 *                 if dist_pt < heap.largest(i_pt):
 *                     heap._push(i_pt, dist_pt, self.idx_array[i])             # <<<<<<<<<<<<<<
 * 
 *         #------------------------------------------------------------
 */
        if (unlikely(!__pyx_v_self->idx_array.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1757; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
        __pyx_t_11 = __pyx_v_i;
        __pyx_t_12 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NeighborsHeap *)__pyx_v_heap->__pyx_vtab)->_push(__pyx_v_heap, __pyx_v_i_pt, __pyx_v_dist_pt, (*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_self->idx_array.data) + __pyx_t_11)) )))); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1757; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L3;
  }
  /*else*/ {

    /* "binary_tree.pxi":1763
 *         #         starting with the closest
 *         else:
 *             self.n_splits += 1             # <<<<<<<<<<<<<<
 *             i1 = 2 * i_node + 1
 *             i2 = i1 + 1
 */
    __pyx_v_self->n_splits = (__pyx_v_self->n_splits + 1);

    /* "binary_tree.pxi":1764
 *         else:
 *             self.n_splits += 1
 *             i1 = 2 * i_node + 1             # <<<<<<<<<<<<<<
 *             i2 = i1 + 1
 *             reduced_dist_LB_1 = min_rdist(self, i1, pt)
 */
    __pyx_v_i1 = ((2 * __pyx_v_i_node) + 1);

    /* "binary_tree.pxi":1765
 *             self.n_splits += 1
 *             i1 = 2 * i_node + 1
 *             i2 = i1 + 1             # <<<<<<<<<<<<<<
 *             reduced_dist_LB_1 = min_rdist(self, i1, pt)
 *             reduced_dist_LB_2 = min_rdist(self, i2, pt)
 */
    __pyx_v_i2 = (__pyx_v_i1 + 1);

    /* "binary_tree.pxi":1766
 *             i1 = 2 * i_node + 1
 *             i2 = i1 + 1
 *             reduced_dist_LB_1 = min_rdist(self, i1, pt)             # <<<<<<<<<<<<<<
 *             reduced_dist_LB_2 = min_rdist(self, i2, pt)
 * 
 */
    __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_rdist(__pyx_v_self, __pyx_v_i1, __pyx_v_pt); if (unlikely(__pyx_t_4 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1766; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_reduced_dist_LB_1 = __pyx_t_4;

    /* "binary_tree.pxi":1767
 *             i2 = i1 + 1
 *             reduced_dist_LB_1 = min_rdist(self, i1, pt)
 *             reduced_dist_LB_2 = min_rdist(self, i2, pt)             # <<<<<<<<<<<<<<
 * 
 *             # recursively query subnodes
 */
    __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_rdist(__pyx_v_self, __pyx_v_i2, __pyx_v_pt); if (unlikely(__pyx_t_4 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_reduced_dist_LB_2 = __pyx_t_4;

    /* "binary_tree.pxi":1770
 * 
 *             # recursively query subnodes
 *             if reduced_dist_LB_1 <= reduced_dist_LB_2:             # <<<<<<<<<<<<<<
 *                 self._query_single_depthfirst(i1, pt, i_pt, heap,
 *                                               reduced_dist_LB_1)
 */
    __pyx_t_5 = ((__pyx_v_reduced_dist_LB_1 <= __pyx_v_reduced_dist_LB_2) != 0);
    if (__pyx_t_5) {

      /* "binary_tree.pxi":1771
 *             # recursively query subnodes
 *             if reduced_dist_LB_1 <= reduced_dist_LB_2:
 *                 self._query_single_depthfirst(i1, pt, i_pt, heap,             # <<<<<<<<<<<<<<
 *                                               reduced_dist_LB_1)
 *                 self._query_single_depthfirst(i2, pt, i_pt, heap,
 */
      __pyx_t_12 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_query_single_depthfirst(__pyx_v_self, __pyx_v_i1, __pyx_v_pt, __pyx_v_i_pt, __pyx_v_heap, __pyx_v_reduced_dist_LB_1); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1771; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "binary_tree.pxi":1773
 *                 self._query_single_depthfirst(i1, pt, i_pt, heap,
 *                                               reduced_dist_LB_1)
 *                 self._query_single_depthfirst(i2, pt, i_pt, heap,             # <<<<<<<<<<<<<<
 *                                               reduced_dist_LB_2)
 *             else:
 */
      __pyx_t_12 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_query_single_depthfirst(__pyx_v_self, __pyx_v_i2, __pyx_v_pt, __pyx_v_i_pt, __pyx_v_heap, __pyx_v_reduced_dist_LB_2); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1773; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      goto __pyx_L7;
    }
    /*else*/ {

      /* "binary_tree.pxi":1776
 *                                               reduced_dist_LB_2)
 *             else:
 *                 self._query_single_depthfirst(i2, pt, i_pt, heap,             # <<<<<<<<<<<<<<
 *                                               reduced_dist_LB_2)
 *                 self._query_single_depthfirst(i1, pt, i_pt, heap,
 */
      __pyx_t_12 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_query_single_depthfirst(__pyx_v_self, __pyx_v_i2, __pyx_v_pt, __pyx_v_i_pt, __pyx_v_heap, __pyx_v_reduced_dist_LB_2); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1776; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "binary_tree.pxi":1778
 *                 self._query_single_depthfirst(i2, pt, i_pt, heap,
 *                                               reduced_dist_LB_2)
 *                 self._query_single_depthfirst(i1, pt, i_pt, heap,             # <<<<<<<<<<<<<<
 *                                               reduced_dist_LB_1)
 *         return 0
 */
      __pyx_t_12 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_query_single_depthfirst(__pyx_v_self, __pyx_v_i1, __pyx_v_pt, __pyx_v_i_pt, __pyx_v_heap, __pyx_v_reduced_dist_LB_1); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1778; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_L7:;
  }
  __pyx_L3:;

  /* "binary_tree.pxi":1780
 *                 self._query_single_depthfirst(i1, pt, i_pt, heap,
 *                                               reduced_dist_LB_1)
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     cdef int _query_single_breadthfirst(self, DTYPE_t* pt,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":1730
 *         return count
 * 
 *     cdef int _query_single_depthfirst(self, ITYPE_t i_node,             # <<<<<<<<<<<<<<
 *                                       DTYPE_t* pt, ITYPE_t i_pt,
 *                                       NeighborsHeap heap,
 */

  /* function exit code */
  __pyx_L1_error:;
  {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
    #endif
    __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree._query_single_depthfirst", __pyx_clineno, __pyx_lineno, __pyx_filename);
    #ifdef WITH_THREAD
    PyGILState_Release(__pyx_gilstate_save);
    #endif
  }
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "binary_tree.pxi":1782
 *         return 0
 * 
 *     cdef int _query_single_breadthfirst(self, DTYPE_t* pt,             # <<<<<<<<<<<<<<
 *                                         ITYPE_t i_pt,
 *                                         NeighborsHeap heap,
 */

static int __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__query_single_breadthfirst(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_pt, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_heap, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_nodeheap) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_pt;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_reduced_dist_LB;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *__pyx_v_node_data;
  CYTHON_UNUSED __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_data;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t __pyx_v_nodeheap_item;
  CYTHON_UNUSED struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t __pyx_v_node_info;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_7;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_8;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_9;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_12;
  long __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_query_single_breadthfirst", 0);

  /* "binary_tree.pxi":1789
 *         cdef ITYPE_t i, i_node
 *         cdef DTYPE_t dist_pt, reduced_dist_LB
 *         cdef NodeData_t* node_data = &self.node_data[0]             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t* data = &self.data[0, 0]
 * 
 */
  if (unlikely(!__pyx_v_self->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1789; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_1 = 0;
  __pyx_v_node_data = (&(*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_self->node_data.data) + __pyx_t_1)) ))));

  /* "binary_tree.pxi":1790
 *         cdef DTYPE_t dist_pt, reduced_dist_LB
 *         cdef NodeData_t* node_data = &self.node_data[0]
 *         cdef DTYPE_t* data = &self.data[0, 0]             # <<<<<<<<<<<<<<
 * 
 *         # Set up the node heap and push the head node onto it
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1790; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_data = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_self->data.data + __pyx_t_2 * __pyx_v_self->data.strides[0]) )) + __pyx_t_3)) ))));

  /* "binary_tree.pxi":1794
 *         # Set up the node heap and push the head node onto it
 *         cdef NodeHeapData_t nodeheap_item
 *         nodeheap_item.val = min_rdist(self, 0, pt)             # <<<<<<<<<<<<<<
 *         nodeheap_item.i1 = 0
 *         nodeheap.push(nodeheap_item)
 */
  __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_rdist(__pyx_v_self, 0, __pyx_v_pt); if (unlikely(__pyx_t_4 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_nodeheap_item.val = __pyx_t_4;

  /* "binary_tree.pxi":1795
 *         cdef NodeHeapData_t nodeheap_item
 *         nodeheap_item.val = min_rdist(self, 0, pt)
 *         nodeheap_item.i1 = 0             # <<<<<<<<<<<<<<
 *         nodeheap.push(nodeheap_item)
 * 
 */
  __pyx_v_nodeheap_item.i1 = 0;

  /* "binary_tree.pxi":1796
 *         nodeheap_item.val = min_rdist(self, 0, pt)
 *         nodeheap_item.i1 = 0
 *         nodeheap.push(nodeheap_item)             # <<<<<<<<<<<<<<
 * 
 *         while nodeheap.n > 0:
 */
  __pyx_t_5 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_v_nodeheap->__pyx_vtab)->push(__pyx_v_nodeheap, __pyx_v_nodeheap_item); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "binary_tree.pxi":1798
 *         nodeheap.push(nodeheap_item)
 * 
 *         while nodeheap.n > 0:             # <<<<<<<<<<<<<<
 *             nodeheap_item = nodeheap.pop()
 *             reduced_dist_LB = nodeheap_item.val
 */
  while (1) {
    __pyx_t_6 = ((__pyx_v_nodeheap->n > 0) != 0);
    if (!__pyx_t_6) break;

    /* "binary_tree.pxi":1799
 * 
 *         while nodeheap.n > 0:
 *             nodeheap_item = nodeheap.pop()             # <<<<<<<<<<<<<<
 *             reduced_dist_LB = nodeheap_item.val
 *             i_node = nodeheap_item.i1
 */
    __pyx_v_nodeheap_item = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_v_nodeheap->__pyx_vtab)->pop(__pyx_v_nodeheap);

    /* "binary_tree.pxi":1800
 *         while nodeheap.n > 0:
 *             nodeheap_item = nodeheap.pop()
 *             reduced_dist_LB = nodeheap_item.val             # <<<<<<<<<<<<<<
 *             i_node = nodeheap_item.i1
 *             node_info = node_data[i_node]
 */
    __pyx_t_4 = __pyx_v_nodeheap_item.val;
    __pyx_v_reduced_dist_LB = __pyx_t_4;

    /* "binary_tree.pxi":1801
 *             nodeheap_item = nodeheap.pop()
 *             reduced_dist_LB = nodeheap_item.val
 *             i_node = nodeheap_item.i1             # <<<<<<<<<<<<<<
 *             node_info = node_data[i_node]
 * 
 */
    __pyx_t_7 = __pyx_v_nodeheap_item.i1;
    __pyx_v_i_node = __pyx_t_7;

    /* "binary_tree.pxi":1802
 *             reduced_dist_LB = nodeheap_item.val
 *             i_node = nodeheap_item.i1
 *             node_info = node_data[i_node]             # <<<<<<<<<<<<<<
 * 
 *             #------------------------------------------------------------
 */
    __pyx_v_node_info = (__pyx_v_node_data[__pyx_v_i_node]);

    /* "binary_tree.pxi":1807
 *             # Case 1: query point is outside node radius:
 *             #         trim it from the query
 *             if reduced_dist_LB > heap.largest(i_pt):             # <<<<<<<<<<<<<<
 *                 self.n_trims += 1
 * 
 */
    __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_largest(__pyx_v_heap, __pyx_v_i_pt); if (unlikely(__pyx_t_4 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1807; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_6 = ((__pyx_v_reduced_dist_LB > __pyx_t_4) != 0);
    if (__pyx_t_6) {

      /* "binary_tree.pxi":1808
 *             #         trim it from the query
 *             if reduced_dist_LB > heap.largest(i_pt):
 *                 self.n_trims += 1             # <<<<<<<<<<<<<<
 * 
 *             #------------------------------------------------------------
 */
      __pyx_v_self->n_trims = (__pyx_v_self->n_trims + 1);
      goto __pyx_L5;
    }

    /* "binary_tree.pxi":1812
 *             #------------------------------------------------------------
 *             # Case 2: this is a leaf node.  Update set of nearby points
 *             elif node_data[i_node].is_leaf:             # <<<<<<<<<<<<<<
 *                 self.n_leaves += 1
 *                 for i in range(node_data[i_node].idx_start,
 */
    __pyx_t_6 = ((__pyx_v_node_data[__pyx_v_i_node]).is_leaf != 0);
    if (__pyx_t_6) {

      /* "binary_tree.pxi":1813
 *             # Case 2: this is a leaf node.  Update set of nearby points
 *             elif node_data[i_node].is_leaf:
 *                 self.n_leaves += 1             # <<<<<<<<<<<<<<
 *                 for i in range(node_data[i_node].idx_start,
 *                                node_data[i_node].idx_end):
 */
      __pyx_v_self->n_leaves = (__pyx_v_self->n_leaves + 1);

      /* "binary_tree.pxi":1815
 *                 self.n_leaves += 1
 *                 for i in range(node_data[i_node].idx_start,
 *                                node_data[i_node].idx_end):             # <<<<<<<<<<<<<<
 *                     dist_pt = self.rdist(pt,
 *                                          &self.data[self.idx_array[i], 0],
 */
      __pyx_t_7 = (__pyx_v_node_data[__pyx_v_i_node]).idx_end;

      /* "binary_tree.pxi":1814
 *             elif node_data[i_node].is_leaf:
 *                 self.n_leaves += 1
 *                 for i in range(node_data[i_node].idx_start,             # <<<<<<<<<<<<<<
 *                                node_data[i_node].idx_end):
 *                     dist_pt = self.rdist(pt,
 */
      for (__pyx_t_8 = (__pyx_v_node_data[__pyx_v_i_node]).idx_start; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
        __pyx_v_i = __pyx_t_8;

        /* "binary_tree.pxi":1817
 *                                node_data[i_node].idx_end):
 *                     dist_pt = self.rdist(pt,
 *                                          &self.data[self.idx_array[i], 0],             # <<<<<<<<<<<<<<
 *                                          self.data.shape[1])
 *                     if dist_pt < heap.largest(i_pt):
 */
        if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1817; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
        if (unlikely(!__pyx_v_self->idx_array.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1817; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
        __pyx_t_9 = __pyx_v_i;
        __pyx_t_10 = (*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_self->idx_array.data) + __pyx_t_9)) )));
        __pyx_t_11 = 0;

        /* "binary_tree.pxi":1818
 *                     dist_pt = self.rdist(pt,
 *                                          &self.data[self.idx_array[i], 0],
 *                                          self.data.shape[1])             # <<<<<<<<<<<<<<
 *                     if dist_pt < heap.largest(i_pt):
 *                         heap._push(i_pt, dist_pt, self.idx_array[i])
 */
        if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1818; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}

        /* "binary_tree.pxi":1816
 *                 for i in range(node_data[i_node].idx_start,
 *                                node_data[i_node].idx_end):
 *                     dist_pt = self.rdist(pt,             # <<<<<<<<<<<<<<
 *                                          &self.data[self.idx_array[i], 0],
 *                                          self.data.shape[1])
 */
        __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree_rdist(__pyx_v_self, __pyx_v_pt, (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_self->data.data + __pyx_t_10 * __pyx_v_self->data.strides[0]) )) + __pyx_t_11)) )))), (__pyx_v_self->data.shape[1])); if (unlikely(__pyx_t_4 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1816; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_v_dist_pt = __pyx_t_4;

        /* "binary_tree.pxi":1819
 *                                          &self.data[self.idx_array[i], 0],
 *                                          self.data.shape[1])
 *                     if dist_pt < heap.largest(i_pt):             # <<<<<<<<<<<<<<
 *                         heap._push(i_pt, dist_pt, self.idx_array[i])
 * 
 */
        __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_largest(__pyx_v_heap, __pyx_v_i_pt); if (unlikely(__pyx_t_4 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1819; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_6 = ((__pyx_v_dist_pt < __pyx_t_4) != 0);
        if (__pyx_t_6) {

          /* "binary_tree.pxi":1820
 *                                          self.data.shape[1])
 *                     if dist_pt < heap.largest(i_pt):
 *                         heap._push(i_pt, dist_pt, self.idx_array[i])             # <<<<<<<<<<<<<<
 * 
 *             #------------------------------------------------------------
 */
          if (unlikely(!__pyx_v_self->idx_array.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1820; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_5 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NeighborsHeap *)__pyx_v_heap->__pyx_vtab)->_push(__pyx_v_heap, __pyx_v_i_pt, __pyx_v_dist_pt, (*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_self->idx_array.data) + __pyx_t_12)) )))); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1820; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          goto __pyx_L8;
        }
        __pyx_L8:;
      }
      goto __pyx_L5;
    }
    /*else*/ {

      /* "binary_tree.pxi":1825
 *             # Case 3: Node is not a leaf.  Add subnodes to the node heap
 *             else:
 *                 self.n_splits += 1             # <<<<<<<<<<<<<<
 *                 for i in range(2 * i_node + 1, 2 * i_node + 3):
 *                     nodeheap_item.i1 = i
 */
      __pyx_v_self->n_splits = (__pyx_v_self->n_splits + 1);

      /* "binary_tree.pxi":1826
 *             else:
 *                 self.n_splits += 1
 *                 for i in range(2 * i_node + 1, 2 * i_node + 3):             # <<<<<<<<<<<<<<
 *                     nodeheap_item.i1 = i
 *                     nodeheap_item.val = min_rdist(self, i, pt)
 */
      __pyx_t_13 = ((2 * __pyx_v_i_node) + 3);
      for (__pyx_t_7 = ((2 * __pyx_v_i_node) + 1); __pyx_t_7 < __pyx_t_13; __pyx_t_7+=1) {
        __pyx_v_i = __pyx_t_7;

        /* "binary_tree.pxi":1827
 *                 self.n_splits += 1
 *                 for i in range(2 * i_node + 1, 2 * i_node + 3):
 *                     nodeheap_item.i1 = i             # <<<<<<<<<<<<<<
 *                     nodeheap_item.val = min_rdist(self, i, pt)
 *                     nodeheap.push(nodeheap_item)
 */
        __pyx_v_nodeheap_item.i1 = __pyx_v_i;

        /* "binary_tree.pxi":1828
 *                 for i in range(2 * i_node + 1, 2 * i_node + 3):
 *                     nodeheap_item.i1 = i
 *                     nodeheap_item.val = min_rdist(self, i, pt)             # <<<<<<<<<<<<<<
 *                     nodeheap.push(nodeheap_item)
 *         return 0
 */
        __pyx_t_4 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_rdist(__pyx_v_self, __pyx_v_i, __pyx_v_pt); if (unlikely(__pyx_t_4 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_v_nodeheap_item.val = __pyx_t_4;

        /* "binary_tree.pxi":1829
 *                     nodeheap_item.i1 = i
 *                     nodeheap_item.val = min_rdist(self, i, pt)
 *                     nodeheap.push(nodeheap_item)             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
        __pyx_t_5 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_v_nodeheap->__pyx_vtab)->push(__pyx_v_nodeheap, __pyx_v_nodeheap_item); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
    }
    __pyx_L5:;
  }

  /* "binary_tree.pxi":1830
 *                     nodeheap_item.val = min_rdist(self, i, pt)
 *                     nodeheap.push(nodeheap_item)
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     cdef int _query_dual_depthfirst(self, ITYPE_t i_node1,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":1782
 *         return 0
 * 
 *     cdef int _query_single_breadthfirst(self, DTYPE_t* pt,             # <<<<<<<<<<<<<<
 *                                         ITYPE_t i_pt,
 *                                         NeighborsHeap heap,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree._query_single_breadthfirst", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1832
 *         return 0
 * 
 *     cdef int _query_dual_depthfirst(self, ITYPE_t i_node1,             # <<<<<<<<<<<<<<
 *                                     BinaryTree other, ITYPE_t i_node2,
 *                                     DTYPE_t[::1] bounds,
 */

static int __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__query_dual_depthfirst(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node1, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_other, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node2, __Pyx_memviewslice __pyx_v_bounds, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_heap, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_reduced_dist_LB) {
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t __pyx_v_node_info1;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t __pyx_v_node_info2;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_data1;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_data2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_features;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_bound_max;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_pt;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_reduced_dist_LB1;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_reduced_dist_LB2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_pt;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_parent;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_10;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_11;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_12;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_13;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_14;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_15;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_16;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_17;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_18;
  int __pyx_t_19;
  long __pyx_t_20;
  long __pyx_t_21;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_query_dual_depthfirst", 0);

  /* "binary_tree.pxi":1841
 *         # bounds[i] is the largest distance among any of the
 *         # current neighbors in node i of the other tree.
 *         cdef NodeData_t node_info1 = self.node_data[i_node1]             # <<<<<<<<<<<<<<
 *         cdef NodeData_t node_info2 = other.node_data[i_node2]
 * 
 */
  if (unlikely(!__pyx_v_self->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_1 = __pyx_v_i_node1;
  __pyx_v_node_info1 = (*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_self->node_data.data) + __pyx_t_1)) )));

  /* "binary_tree.pxi":1842
 *         # current neighbors in node i of the other tree.
 *         cdef NodeData_t node_info1 = self.node_data[i_node1]
 *         cdef NodeData_t node_info2 = other.node_data[i_node2]             # <<<<<<<<<<<<<<
 * 
 *         cdef DTYPE_t* data1 = &self.data[0, 0]
 */
  if (unlikely(!__pyx_v_other->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_2 = __pyx_v_i_node2;
  __pyx_v_node_info2 = (*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_other->node_data.data) + __pyx_t_2)) )));

  /* "binary_tree.pxi":1844
 *         cdef NodeData_t node_info2 = other.node_data[i_node2]
 * 
 *         cdef DTYPE_t* data1 = &self.data[0, 0]             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t* data2 = &other.data[0, 0]
 *         cdef ITYPE_t n_features = self.data.shape[1]
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_v_data1 = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_self->data.data + __pyx_t_3 * __pyx_v_self->data.strides[0]) )) + __pyx_t_4)) ))));

  /* "binary_tree.pxi":1845
 * 
 *         cdef DTYPE_t* data1 = &self.data[0, 0]
 *         cdef DTYPE_t* data2 = &other.data[0, 0]             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t n_features = self.data.shape[1]
 * 
 */
  if (unlikely(!__pyx_v_other->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1845; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_v_data2 = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_other->data.data + __pyx_t_5 * __pyx_v_other->data.strides[0]) )) + __pyx_t_6)) ))));

  /* "binary_tree.pxi":1846
 *         cdef DTYPE_t* data1 = &self.data[0, 0]
 *         cdef DTYPE_t* data2 = &other.data[0, 0]
 *         cdef ITYPE_t n_features = self.data.shape[1]             # <<<<<<<<<<<<<<
 * 
 *         cdef DTYPE_t bound_max, dist_pt, reduced_dist_LB1, reduced_dist_LB2
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1846; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_n_features = (__pyx_v_self->data.shape[1]);

  /* "binary_tree.pxi":1854
 *         # Case 1: nodes are further apart than the current bound:
 *         #         trim both from the query
 *         if reduced_dist_LB > bounds[i_node2]:             # <<<<<<<<<<<<<<
 *             pass
 * 
 */
  __pyx_t_7 = __pyx_v_i_node2;
  __pyx_t_8 = ((__pyx_v_reduced_dist_LB > (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_bounds.data) + __pyx_t_7)) )))) != 0);
  if (__pyx_t_8) {
    goto __pyx_L3;
  }

  /* "binary_tree.pxi":1860
 *         # Case 2: both nodes are leaves:
 *         #         do a brute-force search comparing all pairs
 *         elif node_info1.is_leaf and node_info2.is_leaf:             # <<<<<<<<<<<<<<
 *             bounds[i_node2] = 0
 * 
 */
  __pyx_t_9 = (__pyx_v_node_info1.is_leaf != 0);
  if (__pyx_t_9) {
  } else {
    __pyx_t_8 = __pyx_t_9;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_9 = (__pyx_v_node_info2.is_leaf != 0);
  __pyx_t_8 = __pyx_t_9;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_8) {

    /* "binary_tree.pxi":1861
 *         #         do a brute-force search comparing all pairs
 *         elif node_info1.is_leaf and node_info2.is_leaf:
 *             bounds[i_node2] = 0             # <<<<<<<<<<<<<<
 * 
 *             for i2 in range(node_info2.idx_start, node_info2.idx_end):
 */
    __pyx_t_10 = __pyx_v_i_node2;
    *((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_bounds.data) + __pyx_t_10)) )) = 0.0;

    /* "binary_tree.pxi":1863
 *             bounds[i_node2] = 0
 * 
 *             for i2 in range(node_info2.idx_start, node_info2.idx_end):             # <<<<<<<<<<<<<<
 *                 i_pt = other.idx_array[i2]
 * 
 */
    __pyx_t_11 = __pyx_v_node_info2.idx_end;
    for (__pyx_t_12 = __pyx_v_node_info2.idx_start; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
      __pyx_v_i2 = __pyx_t_12;

      /* "binary_tree.pxi":1864
 * 
 *             for i2 in range(node_info2.idx_start, node_info2.idx_end):
 *                 i_pt = other.idx_array[i2]             # <<<<<<<<<<<<<<
 * 
 *                 if heap.largest(i_pt) <= reduced_dist_LB:
 */
      if (unlikely(!__pyx_v_other->idx_array.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1864; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_13 = __pyx_v_i2;
      __pyx_v_i_pt = (*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_other->idx_array.data) + __pyx_t_13)) )));

      /* "binary_tree.pxi":1866
 *                 i_pt = other.idx_array[i2]
 * 
 *                 if heap.largest(i_pt) <= reduced_dist_LB:             # <<<<<<<<<<<<<<
 *                     continue
 * 
 */
      __pyx_t_14 = __pyx_f_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_largest(__pyx_v_heap, __pyx_v_i_pt); if (unlikely(__pyx_t_14 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1866; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_t_8 = ((__pyx_t_14 <= __pyx_v_reduced_dist_LB) != 0);
      if (__pyx_t_8) {

        /* "binary_tree.pxi":1867
 * 
 *                 if heap.largest(i_pt) <= reduced_dist_LB:
 *                     continue             # <<<<<<<<<<<<<<
 * 
 *                 for i1 in range(node_info1.idx_start, node_info1.idx_end):
 */
        goto __pyx_L6_continue;
      }

      /* "binary_tree.pxi":1869
 *                     continue
 * 
 *                 for i1 in range(node_info1.idx_start, node_info1.idx_end):             # <<<<<<<<<<<<<<
 *                     dist_pt = self.rdist(
 *                         data1 + n_features * self.idx_array[i1],
 */
      __pyx_t_15 = __pyx_v_node_info1.idx_end;
      for (__pyx_t_16 = __pyx_v_node_info1.idx_start; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
        __pyx_v_i1 = __pyx_t_16;

        /* "binary_tree.pxi":1871
 *                 for i1 in range(node_info1.idx_start, node_info1.idx_end):
 *                     dist_pt = self.rdist(
 *                         data1 + n_features * self.idx_array[i1],             # <<<<<<<<<<<<<<
 *                         data2 + n_features * i_pt,
 *                         n_features)
 */
        if (unlikely(!__pyx_v_self->idx_array.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1871; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
        __pyx_t_17 = __pyx_v_i1;

        /* "binary_tree.pxi":1870
 * 
 *                 for i1 in range(node_info1.idx_start, node_info1.idx_end):
 *                     dist_pt = self.rdist(             # <<<<<<<<<<<<<<
 *                         data1 + n_features * self.idx_array[i1],
 *                         data2 + n_features * i_pt,
 */
        __pyx_t_14 = __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree_rdist(__pyx_v_self, (__pyx_v_data1 + (__pyx_v_n_features * (*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_self->idx_array.data) + __pyx_t_17)) ))))), (__pyx_v_data2 + (__pyx_v_n_features * __pyx_v_i_pt)), __pyx_v_n_features); if (unlikely(__pyx_t_14 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1870; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_v_dist_pt = __pyx_t_14;

        /* "binary_tree.pxi":1874
 *                         data2 + n_features * i_pt,
 *                         n_features)
 *                     if dist_pt < heap.largest(i_pt):             # <<<<<<<<<<<<<<
 *                         heap._push(i_pt, dist_pt, self.idx_array[i1])
 * 
 */
        __pyx_t_14 = __pyx_f_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_largest(__pyx_v_heap, __pyx_v_i_pt); if (unlikely(__pyx_t_14 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1874; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_8 = ((__pyx_v_dist_pt < __pyx_t_14) != 0);
        if (__pyx_t_8) {

          /* "binary_tree.pxi":1875
 *                         n_features)
 *                     if dist_pt < heap.largest(i_pt):
 *                         heap._push(i_pt, dist_pt, self.idx_array[i1])             # <<<<<<<<<<<<<<
 * 
 *                 # keep track of node bound
 */
          if (unlikely(!__pyx_v_self->idx_array.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1875; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
          __pyx_t_18 = __pyx_v_i1;
          __pyx_t_19 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NeighborsHeap *)__pyx_v_heap->__pyx_vtab)->_push(__pyx_v_heap, __pyx_v_i_pt, __pyx_v_dist_pt, (*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_self->idx_array.data) + __pyx_t_18)) )))); if (unlikely(__pyx_t_19 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1875; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          goto __pyx_L11;
        }
        __pyx_L11:;
      }

      /* "binary_tree.pxi":1878
 * 
 *                 # keep track of node bound
 *                 bounds[i_node2] = fmax(bounds[i_node2],             # <<<<<<<<<<<<<<
 *                                        heap.largest(i_pt))
 * 
 */
      __pyx_t_15 = __pyx_v_i_node2;

      /* "binary_tree.pxi":1879
 *                 # keep track of node bound
 *                 bounds[i_node2] = fmax(bounds[i_node2],
 *                                        heap.largest(i_pt))             # <<<<<<<<<<<<<<
 * 
 *             # update bounds up the tree
 */
      __pyx_t_14 = __pyx_f_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_largest(__pyx_v_heap, __pyx_v_i_pt); if (unlikely(__pyx_t_14 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1879; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "binary_tree.pxi":1878
 * 
 *                 # keep track of node bound
 *                 bounds[i_node2] = fmax(bounds[i_node2],             # <<<<<<<<<<<<<<
 *                                        heap.largest(i_pt))
 * 
 */
      __pyx_t_16 = __pyx_v_i_node2;
      *((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_bounds.data) + __pyx_t_16)) )) = __pyx_f_7sklearn_9neighbors_7kd_tree_fmax((*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_bounds.data) + __pyx_t_15)) ))), __pyx_t_14);
      __pyx_L6_continue:;
    }

    /* "binary_tree.pxi":1882
 * 
 *             # update bounds up the tree
 *             while i_node2 > 0:             # <<<<<<<<<<<<<<
 *                 i_parent = (i_node2 - 1) // 2
 *                 bound_max = fmax(bounds[2 * i_parent + 1],
 */
    while (1) {
      __pyx_t_8 = ((__pyx_v_i_node2 > 0) != 0);
      if (!__pyx_t_8) break;

      /* "binary_tree.pxi":1883
 *             # update bounds up the tree
 *             while i_node2 > 0:
 *                 i_parent = (i_node2 - 1) // 2             # <<<<<<<<<<<<<<
 *                 bound_max = fmax(bounds[2 * i_parent + 1],
 *                                  bounds[2 * i_parent + 2])
 */
      __pyx_v_i_parent = ((__pyx_v_i_node2 - 1) / 2);

      /* "binary_tree.pxi":1884
 *             while i_node2 > 0:
 *                 i_parent = (i_node2 - 1) // 2
 *                 bound_max = fmax(bounds[2 * i_parent + 1],             # <<<<<<<<<<<<<<
 *                                  bounds[2 * i_parent + 2])
 *                 if bound_max < bounds[i_parent]:
 */
      __pyx_t_20 = ((2 * __pyx_v_i_parent) + 1);

      /* "binary_tree.pxi":1885
 *                 i_parent = (i_node2 - 1) // 2
 *                 bound_max = fmax(bounds[2 * i_parent + 1],
 *                                  bounds[2 * i_parent + 2])             # <<<<<<<<<<<<<<
 *                 if bound_max < bounds[i_parent]:
 *                     bounds[i_parent] = bound_max
 */
      __pyx_t_21 = ((2 * __pyx_v_i_parent) + 2);

      /* "binary_tree.pxi":1884
 *             while i_node2 > 0:
 *                 i_parent = (i_node2 - 1) // 2
 *                 bound_max = fmax(bounds[2 * i_parent + 1],             # <<<<<<<<<<<<<<
 *                                  bounds[2 * i_parent + 2])
 *                 if bound_max < bounds[i_parent]:
 */
      __pyx_v_bound_max = __pyx_f_7sklearn_9neighbors_7kd_tree_fmax((*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_bounds.data) + __pyx_t_20)) ))), (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_bounds.data) + __pyx_t_21)) ))));

      /* "binary_tree.pxi":1886
 *                 bound_max = fmax(bounds[2 * i_parent + 1],
 *                                  bounds[2 * i_parent + 2])
 *                 if bound_max < bounds[i_parent]:             # <<<<<<<<<<<<<<
 *                     bounds[i_parent] = bound_max
 *                     i_node2 = i_parent
 */
      __pyx_t_11 = __pyx_v_i_parent;
      __pyx_t_8 = ((__pyx_v_bound_max < (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_bounds.data) + __pyx_t_11)) )))) != 0);
      if (__pyx_t_8) {

        /* "binary_tree.pxi":1887
 *                                  bounds[2 * i_parent + 2])
 *                 if bound_max < bounds[i_parent]:
 *                     bounds[i_parent] = bound_max             # <<<<<<<<<<<<<<
 *                     i_node2 = i_parent
 *                 else:
 */
        __pyx_t_12 = __pyx_v_i_parent;
        *((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_bounds.data) + __pyx_t_12)) )) = __pyx_v_bound_max;

        /* "binary_tree.pxi":1888
 *                 if bound_max < bounds[i_parent]:
 *                     bounds[i_parent] = bound_max
 *                     i_node2 = i_parent             # <<<<<<<<<<<<<<
 *                 else:
 *                     break
 */
        __pyx_v_i_node2 = __pyx_v_i_parent;
        goto __pyx_L14;
      }
      /*else*/ {

        /* "binary_tree.pxi":1890
 *                     i_node2 = i_parent
 *                 else:
 *                     break             # <<<<<<<<<<<<<<
 * 
 *         #------------------------------------------------------------
 */
        goto __pyx_L13_break;
      }
      __pyx_L14:;
    }
    __pyx_L13_break:;
    goto __pyx_L3;
  }

  /* "binary_tree.pxi":1895
 *         # Case 3a: node 1 is a leaf or is smaller: split node 2 and
 *         #          recursively query, starting with the nearest subnode
 *         elif node_info1.is_leaf or (not node_info2.is_leaf             # <<<<<<<<<<<<<<
 *                                     and node_info2.radius > node_info1.radius):
 *             reduced_dist_LB1 = min_rdist_dual(self, i_node1,
 */
  __pyx_t_9 = (__pyx_v_node_info1.is_leaf != 0);
  if (!__pyx_t_9) {
  } else {
    __pyx_t_8 = __pyx_t_9;
    goto __pyx_L15_bool_binop_done;
  }

  /* "binary_tree.pxi":1896
 *         #          recursively query, starting with the nearest subnode
 *         elif node_info1.is_leaf or (not node_info2.is_leaf
 *                                     and node_info2.radius > node_info1.radius):             # <<<<<<<<<<<<<<
 *             reduced_dist_LB1 = min_rdist_dual(self, i_node1,
 *                                               other, 2 * i_node2 + 1)
 */
  __pyx_t_9 = ((!(__pyx_v_node_info2.is_leaf != 0)) != 0);
  if (__pyx_t_9) {
  } else {
    __pyx_t_8 = __pyx_t_9;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_9 = ((__pyx_v_node_info2.radius > __pyx_v_node_info1.radius) != 0);
  __pyx_t_8 = __pyx_t_9;
  __pyx_L15_bool_binop_done:;
  if (__pyx_t_8) {

    /* "binary_tree.pxi":1897
 *         elif node_info1.is_leaf or (not node_info2.is_leaf
 *                                     and node_info2.radius > node_info1.radius):
 *             reduced_dist_LB1 = min_rdist_dual(self, i_node1,             # <<<<<<<<<<<<<<
 *                                               other, 2 * i_node2 + 1)
 *             reduced_dist_LB2 = min_rdist_dual(self, i_node1,
 */
    __pyx_t_14 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_rdist_dual(__pyx_v_self, __pyx_v_i_node1, __pyx_v_other, ((2 * __pyx_v_i_node2) + 1)); if (unlikely(__pyx_t_14 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1897; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_reduced_dist_LB1 = __pyx_t_14;

    /* "binary_tree.pxi":1899
 *             reduced_dist_LB1 = min_rdist_dual(self, i_node1,
 *                                               other, 2 * i_node2 + 1)
 *             reduced_dist_LB2 = min_rdist_dual(self, i_node1,             # <<<<<<<<<<<<<<
 *                                               other, 2 * i_node2 + 2)
 * 
 */
    __pyx_t_14 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_rdist_dual(__pyx_v_self, __pyx_v_i_node1, __pyx_v_other, ((2 * __pyx_v_i_node2) + 2)); if (unlikely(__pyx_t_14 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1899; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_reduced_dist_LB2 = __pyx_t_14;

    /* "binary_tree.pxi":1902
 *                                               other, 2 * i_node2 + 2)
 * 
 *             if reduced_dist_LB1 < reduced_dist_LB2:             # <<<<<<<<<<<<<<
 *                 self._query_dual_depthfirst(i_node1, other, 2 * i_node2 + 1,
 *                                             bounds, heap, reduced_dist_LB1)
 */
    __pyx_t_8 = ((__pyx_v_reduced_dist_LB1 < __pyx_v_reduced_dist_LB2) != 0);
    if (__pyx_t_8) {

      /* "binary_tree.pxi":1903
 * 
 *             if reduced_dist_LB1 < reduced_dist_LB2:
 *                 self._query_dual_depthfirst(i_node1, other, 2 * i_node2 + 1,             # <<<<<<<<<<<<<<
 *                                             bounds, heap, reduced_dist_LB1)
 *                 self._query_dual_depthfirst(i_node1, other, 2 * i_node2 + 2,
 */
      __pyx_t_19 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_query_dual_depthfirst(__pyx_v_self, __pyx_v_i_node1, __pyx_v_other, ((2 * __pyx_v_i_node2) + 1), __pyx_v_bounds, __pyx_v_heap, __pyx_v_reduced_dist_LB1); if (unlikely(__pyx_t_19 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "binary_tree.pxi":1905
 *                 self._query_dual_depthfirst(i_node1, other, 2 * i_node2 + 1,
 *                                             bounds, heap, reduced_dist_LB1)
 *                 self._query_dual_depthfirst(i_node1, other, 2 * i_node2 + 2,             # <<<<<<<<<<<<<<
 *                                             bounds, heap, reduced_dist_LB2)
 *             else:
 */
      __pyx_t_19 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_query_dual_depthfirst(__pyx_v_self, __pyx_v_i_node1, __pyx_v_other, ((2 * __pyx_v_i_node2) + 2), __pyx_v_bounds, __pyx_v_heap, __pyx_v_reduced_dist_LB2); if (unlikely(__pyx_t_19 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1905; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      goto __pyx_L18;
    }
    /*else*/ {

      /* "binary_tree.pxi":1908
 *                                             bounds, heap, reduced_dist_LB2)
 *             else:
 *                 self._query_dual_depthfirst(i_node1, other, 2 * i_node2 + 2,             # <<<<<<<<<<<<<<
 *                                             bounds, heap, reduced_dist_LB2)
 *                 self._query_dual_depthfirst(i_node1, other, 2 * i_node2 + 1,
 */
      __pyx_t_19 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_query_dual_depthfirst(__pyx_v_self, __pyx_v_i_node1, __pyx_v_other, ((2 * __pyx_v_i_node2) + 2), __pyx_v_bounds, __pyx_v_heap, __pyx_v_reduced_dist_LB2); if (unlikely(__pyx_t_19 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1908; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "binary_tree.pxi":1910
 *                 self._query_dual_depthfirst(i_node1, other, 2 * i_node2 + 2,
 *                                             bounds, heap, reduced_dist_LB2)
 *                 self._query_dual_depthfirst(i_node1, other, 2 * i_node2 + 1,             # <<<<<<<<<<<<<<
 *                                             bounds, heap, reduced_dist_LB1)
 * 
 */
      __pyx_t_19 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_query_dual_depthfirst(__pyx_v_self, __pyx_v_i_node1, __pyx_v_other, ((2 * __pyx_v_i_node2) + 1), __pyx_v_bounds, __pyx_v_heap, __pyx_v_reduced_dist_LB1); if (unlikely(__pyx_t_19 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_L18:;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "binary_tree.pxi":1917
 *         #          recursively query, starting with the nearest subnode
 *         else:
 *             reduced_dist_LB1 = min_rdist_dual(self, 2 * i_node1 + 1,             # <<<<<<<<<<<<<<
 *                                               other, i_node2)
 *             reduced_dist_LB2 = min_rdist_dual(self, 2 * i_node1 + 2,
 */
    __pyx_t_14 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_rdist_dual(__pyx_v_self, ((2 * __pyx_v_i_node1) + 1), __pyx_v_other, __pyx_v_i_node2); if (unlikely(__pyx_t_14 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1917; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_reduced_dist_LB1 = __pyx_t_14;

    /* "binary_tree.pxi":1919
 *             reduced_dist_LB1 = min_rdist_dual(self, 2 * i_node1 + 1,
 *                                               other, i_node2)
 *             reduced_dist_LB2 = min_rdist_dual(self, 2 * i_node1 + 2,             # <<<<<<<<<<<<<<
 *                                               other, i_node2)
 * 
 */
    __pyx_t_14 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_rdist_dual(__pyx_v_self, ((2 * __pyx_v_i_node1) + 2), __pyx_v_other, __pyx_v_i_node2); if (unlikely(__pyx_t_14 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1919; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_reduced_dist_LB2 = __pyx_t_14;

    /* "binary_tree.pxi":1922
 *                                               other, i_node2)
 * 
 *             if reduced_dist_LB1 < reduced_dist_LB2:             # <<<<<<<<<<<<<<
 *                 self._query_dual_depthfirst(2 * i_node1 + 1, other, i_node2,
 *                                             bounds, heap, reduced_dist_LB1)
 */
    __pyx_t_8 = ((__pyx_v_reduced_dist_LB1 < __pyx_v_reduced_dist_LB2) != 0);
    if (__pyx_t_8) {

      /* "binary_tree.pxi":1923
 * 
 *             if reduced_dist_LB1 < reduced_dist_LB2:
 *                 self._query_dual_depthfirst(2 * i_node1 + 1, other, i_node2,             # <<<<<<<<<<<<<<
 *                                             bounds, heap, reduced_dist_LB1)
 *                 self._query_dual_depthfirst(2 * i_node1 + 2, other, i_node2,
 */
      __pyx_t_19 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_query_dual_depthfirst(__pyx_v_self, ((2 * __pyx_v_i_node1) + 1), __pyx_v_other, __pyx_v_i_node2, __pyx_v_bounds, __pyx_v_heap, __pyx_v_reduced_dist_LB1); if (unlikely(__pyx_t_19 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1923; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "binary_tree.pxi":1925
 *                 self._query_dual_depthfirst(2 * i_node1 + 1, other, i_node2,
 *                                             bounds, heap, reduced_dist_LB1)
 *                 self._query_dual_depthfirst(2 * i_node1 + 2, other, i_node2,             # <<<<<<<<<<<<<<
 *                                             bounds, heap, reduced_dist_LB2)
 *             else:
 */
      __pyx_t_19 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_query_dual_depthfirst(__pyx_v_self, ((2 * __pyx_v_i_node1) + 2), __pyx_v_other, __pyx_v_i_node2, __pyx_v_bounds, __pyx_v_heap, __pyx_v_reduced_dist_LB2); if (unlikely(__pyx_t_19 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1925; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      goto __pyx_L19;
    }
    /*else*/ {

      /* "binary_tree.pxi":1928
 *                                             bounds, heap, reduced_dist_LB2)
 *             else:
 *                 self._query_dual_depthfirst(2 * i_node1 + 2, other, i_node2,             # <<<<<<<<<<<<<<
 *                                             bounds, heap, reduced_dist_LB2)
 *                 self._query_dual_depthfirst(2 * i_node1 + 1, other, i_node2,
 */
      __pyx_t_19 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_query_dual_depthfirst(__pyx_v_self, ((2 * __pyx_v_i_node1) + 2), __pyx_v_other, __pyx_v_i_node2, __pyx_v_bounds, __pyx_v_heap, __pyx_v_reduced_dist_LB2); if (unlikely(__pyx_t_19 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "binary_tree.pxi":1930
 *                 self._query_dual_depthfirst(2 * i_node1 + 2, other, i_node2,
 *                                             bounds, heap, reduced_dist_LB2)
 *                 self._query_dual_depthfirst(2 * i_node1 + 1, other, i_node2,             # <<<<<<<<<<<<<<
 *                                             bounds, heap, reduced_dist_LB1)
 *         return 0
 */
      __pyx_t_19 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_query_dual_depthfirst(__pyx_v_self, ((2 * __pyx_v_i_node1) + 1), __pyx_v_other, __pyx_v_i_node2, __pyx_v_bounds, __pyx_v_heap, __pyx_v_reduced_dist_LB1); if (unlikely(__pyx_t_19 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1930; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_L19:;
  }
  __pyx_L3:;

  /* "binary_tree.pxi":1932
 *                 self._query_dual_depthfirst(2 * i_node1 + 1, other, i_node2,
 *                                             bounds, heap, reduced_dist_LB1)
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     cdef int _query_dual_breadthfirst(self, BinaryTree other,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":1832
 *         return 0
 * 
 *     cdef int _query_dual_depthfirst(self, ITYPE_t i_node1,             # <<<<<<<<<<<<<<
 *                                     BinaryTree other, ITYPE_t i_node2,
 *                                     DTYPE_t[::1] bounds,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree._query_dual_depthfirst", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1934
 *         return 0
 * 
 *     cdef int _query_dual_breadthfirst(self, BinaryTree other,             # <<<<<<<<<<<<<<
 *                                       NeighborsHeap heap,
 *                                       NodeHeap nodeheap) except -1:
 */

static int __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__query_dual_breadthfirst(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_other, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_heap, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_nodeheap) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_pt;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_pt;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_reduced_dist_LB;
  __Pyx_memviewslice __pyx_v_bounds = { 0, 0, { 0 }, { 0 }, { 0 } };
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *__pyx_v_node_data1;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *__pyx_v_node_data2;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t __pyx_v_node_info1;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t __pyx_v_node_info2;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_data1;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_data2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_features;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t __pyx_v_nodeheap_item;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_14;
  int __pyx_t_15;
  int __pyx_t_16;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_17;
  int __pyx_t_18;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_19;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_20;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_21;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_22;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_23;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_24;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_25;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_26;
  long __pyx_t_27;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_query_dual_breadthfirst", 0);

  /* "binary_tree.pxi":1940
 *         cdef ITYPE_t i, i1, i2, i_node1, i_node2, i_pt
 *         cdef DTYPE_t dist_pt, reduced_dist_LB
 *         cdef DTYPE_t[::1] bounds = np.inf + np.zeros(other.node_data.shape[0])             # <<<<<<<<<<<<<<
 *         cdef NodeData_t* node_data1 = &self.node_data[0]
 *         cdef NodeData_t* node_data2 = &other.node_data[0]
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1940; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_inf); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1940; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1940; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1940; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_v_other->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1940; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_3 = PyInt_FromSsize_t((__pyx_v_other->node_data.shape[0])); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1940; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_5) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1940; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1940; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __Pyx_GIVEREF(__pyx_t_5); __pyx_t_5 = NULL;
    PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1940; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyNumber_Add(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1940; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t(__pyx_t_4);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1940; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_bounds = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "binary_tree.pxi":1941
 *         cdef DTYPE_t dist_pt, reduced_dist_LB
 *         cdef DTYPE_t[::1] bounds = np.inf + np.zeros(other.node_data.shape[0])
 *         cdef NodeData_t* node_data1 = &self.node_data[0]             # <<<<<<<<<<<<<<
 *         cdef NodeData_t* node_data2 = &other.node_data[0]
 *         cdef NodeData_t node_info1, node_info2
 */
  if (unlikely(!__pyx_v_self->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1941; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_8 = 0;
  __pyx_v_node_data1 = (&(*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_self->node_data.data) + __pyx_t_8)) ))));

  /* "binary_tree.pxi":1942
 *         cdef DTYPE_t[::1] bounds = np.inf + np.zeros(other.node_data.shape[0])
 *         cdef NodeData_t* node_data1 = &self.node_data[0]
 *         cdef NodeData_t* node_data2 = &other.node_data[0]             # <<<<<<<<<<<<<<
 *         cdef NodeData_t node_info1, node_info2
 *         cdef DTYPE_t* data1 = &self.data[0, 0]
 */
  if (unlikely(!__pyx_v_other->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1942; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_9 = 0;
  __pyx_v_node_data2 = (&(*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_other->node_data.data) + __pyx_t_9)) ))));

  /* "binary_tree.pxi":1944
 *         cdef NodeData_t* node_data2 = &other.node_data[0]
 *         cdef NodeData_t node_info1, node_info2
 *         cdef DTYPE_t* data1 = &self.data[0, 0]             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t* data2 = &other.data[0, 0]
 *         cdef ITYPE_t n_features = self.data.shape[1]
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1944; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_10 = 0;
  __pyx_t_11 = 0;
  __pyx_v_data1 = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_self->data.data + __pyx_t_10 * __pyx_v_self->data.strides[0]) )) + __pyx_t_11)) ))));

  /* "binary_tree.pxi":1945
 *         cdef NodeData_t node_info1, node_info2
 *         cdef DTYPE_t* data1 = &self.data[0, 0]
 *         cdef DTYPE_t* data2 = &other.data[0, 0]             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t n_features = self.data.shape[1]
 * 
 */
  if (unlikely(!__pyx_v_other->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1945; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_12 = 0;
  __pyx_t_13 = 0;
  __pyx_v_data2 = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_other->data.data + __pyx_t_12 * __pyx_v_other->data.strides[0]) )) + __pyx_t_13)) ))));

  /* "binary_tree.pxi":1946
 *         cdef DTYPE_t* data1 = &self.data[0, 0]
 *         cdef DTYPE_t* data2 = &other.data[0, 0]
 *         cdef ITYPE_t n_features = self.data.shape[1]             # <<<<<<<<<<<<<<
 * 
 *         # Set up the node heap and push the head nodes onto it
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_n_features = (__pyx_v_self->data.shape[1]);

  /* "binary_tree.pxi":1950
 *         # Set up the node heap and push the head nodes onto it
 *         cdef NodeHeapData_t nodeheap_item
 *         nodeheap_item.val = min_rdist_dual(self, 0, other, 0)             # <<<<<<<<<<<<<<
 *         nodeheap_item.i1 = 0
 *         nodeheap_item.i2 = 0
 */
  __pyx_t_14 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_rdist_dual(__pyx_v_self, 0, __pyx_v_other, 0); if (unlikely(__pyx_t_14 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1950; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_nodeheap_item.val = __pyx_t_14;

  /* "binary_tree.pxi":1951
 *         cdef NodeHeapData_t nodeheap_item
 *         nodeheap_item.val = min_rdist_dual(self, 0, other, 0)
 *         nodeheap_item.i1 = 0             # <<<<<<<<<<<<<<
 *         nodeheap_item.i2 = 0
 *         nodeheap.push(nodeheap_item)
 */
  __pyx_v_nodeheap_item.i1 = 0;

  /* "binary_tree.pxi":1952
 *         nodeheap_item.val = min_rdist_dual(self, 0, other, 0)
 *         nodeheap_item.i1 = 0
 *         nodeheap_item.i2 = 0             # <<<<<<<<<<<<<<
 *         nodeheap.push(nodeheap_item)
 * 
 */
  __pyx_v_nodeheap_item.i2 = 0;

  /* "binary_tree.pxi":1953
 *         nodeheap_item.i1 = 0
 *         nodeheap_item.i2 = 0
 *         nodeheap.push(nodeheap_item)             # <<<<<<<<<<<<<<
 * 
 *         while nodeheap.n > 0:
 */
  __pyx_t_15 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_v_nodeheap->__pyx_vtab)->push(__pyx_v_nodeheap, __pyx_v_nodeheap_item); if (unlikely(__pyx_t_15 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1953; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "binary_tree.pxi":1955
 *         nodeheap.push(nodeheap_item)
 * 
 *         while nodeheap.n > 0:             # <<<<<<<<<<<<<<
 *             nodeheap_item = nodeheap.pop()
 *             reduced_dist_LB = nodeheap_item.val
 */
  while (1) {
    __pyx_t_16 = ((__pyx_v_nodeheap->n > 0) != 0);
    if (!__pyx_t_16) break;

    /* "binary_tree.pxi":1956
 * 
 *         while nodeheap.n > 0:
 *             nodeheap_item = nodeheap.pop()             # <<<<<<<<<<<<<<
 *             reduced_dist_LB = nodeheap_item.val
 *             i_node1 = nodeheap_item.i1
 */
    __pyx_v_nodeheap_item = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_v_nodeheap->__pyx_vtab)->pop(__pyx_v_nodeheap);

    /* "binary_tree.pxi":1957
 *         while nodeheap.n > 0:
 *             nodeheap_item = nodeheap.pop()
 *             reduced_dist_LB = nodeheap_item.val             # <<<<<<<<<<<<<<
 *             i_node1 = nodeheap_item.i1
 *             i_node2 = nodeheap_item.i2
 */
    __pyx_t_14 = __pyx_v_nodeheap_item.val;
    __pyx_v_reduced_dist_LB = __pyx_t_14;

    /* "binary_tree.pxi":1958
 *             nodeheap_item = nodeheap.pop()
 *             reduced_dist_LB = nodeheap_item.val
 *             i_node1 = nodeheap_item.i1             # <<<<<<<<<<<<<<
 *             i_node2 = nodeheap_item.i2
 * 
 */
    __pyx_t_17 = __pyx_v_nodeheap_item.i1;
    __pyx_v_i_node1 = __pyx_t_17;

    /* "binary_tree.pxi":1959
 *             reduced_dist_LB = nodeheap_item.val
 *             i_node1 = nodeheap_item.i1
 *             i_node2 = nodeheap_item.i2             # <<<<<<<<<<<<<<
 * 
 *             node_info1 = node_data1[i_node1]
 */
    __pyx_t_17 = __pyx_v_nodeheap_item.i2;
    __pyx_v_i_node2 = __pyx_t_17;

    /* "binary_tree.pxi":1961
 *             i_node2 = nodeheap_item.i2
 * 
 *             node_info1 = node_data1[i_node1]             # <<<<<<<<<<<<<<
 *             node_info2 = node_data2[i_node2]
 * 
 */
    __pyx_v_node_info1 = (__pyx_v_node_data1[__pyx_v_i_node1]);

    /* "binary_tree.pxi":1962
 * 
 *             node_info1 = node_data1[i_node1]
 *             node_info2 = node_data2[i_node2]             # <<<<<<<<<<<<<<
 * 
 *             #------------------------------------------------------------
 */
    __pyx_v_node_info2 = (__pyx_v_node_data2[__pyx_v_i_node2]);

    /* "binary_tree.pxi":1967
 *             # Case 1: nodes are further apart than the current bound:
 *             #         trim both from the query
 *             if reduced_dist_LB > bounds[i_node2]:             # <<<<<<<<<<<<<<
 *                 pass
 * 
 */
    __pyx_t_17 = __pyx_v_i_node2;
    __pyx_t_16 = ((__pyx_v_reduced_dist_LB > (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_bounds.data) + __pyx_t_17)) )))) != 0);
    if (__pyx_t_16) {
      goto __pyx_L5;
    }

    /* "binary_tree.pxi":1973
 *             # Case 2: both nodes are leaves:
 *             #         do a brute-force search comparing all pairs
 *             elif node_info1.is_leaf and node_info2.is_leaf:             # <<<<<<<<<<<<<<
 *                 bounds[i_node2] = -1
 * 
 */
    __pyx_t_18 = (__pyx_v_node_info1.is_leaf != 0);
    if (__pyx_t_18) {
    } else {
      __pyx_t_16 = __pyx_t_18;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_18 = (__pyx_v_node_info2.is_leaf != 0);
    __pyx_t_16 = __pyx_t_18;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_16) {

      /* "binary_tree.pxi":1974
 *             #         do a brute-force search comparing all pairs
 *             elif node_info1.is_leaf and node_info2.is_leaf:
 *                 bounds[i_node2] = -1             # <<<<<<<<<<<<<<
 * 
 *                 for i2 in range(node_info2.idx_start, node_info2.idx_end):
 */
      __pyx_t_19 = __pyx_v_i_node2;
      *((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_bounds.data) + __pyx_t_19)) )) = -1.0;

      /* "binary_tree.pxi":1976
 *                 bounds[i_node2] = -1
 * 
 *                 for i2 in range(node_info2.idx_start, node_info2.idx_end):             # <<<<<<<<<<<<<<
 *                     i_pt = other.idx_array[i2]
 * 
 */
      __pyx_t_20 = __pyx_v_node_info2.idx_end;
      for (__pyx_t_21 = __pyx_v_node_info2.idx_start; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
        __pyx_v_i2 = __pyx_t_21;

        /* "binary_tree.pxi":1977
 * 
 *                 for i2 in range(node_info2.idx_start, node_info2.idx_end):
 *                     i_pt = other.idx_array[i2]             # <<<<<<<<<<<<<<
 * 
 *                     if heap.largest(i_pt) <= reduced_dist_LB:
 */
        if (unlikely(!__pyx_v_other->idx_array.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1977; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
        __pyx_t_22 = __pyx_v_i2;
        __pyx_v_i_pt = (*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_other->idx_array.data) + __pyx_t_22)) )));

        /* "binary_tree.pxi":1979
 *                     i_pt = other.idx_array[i2]
 * 
 *                     if heap.largest(i_pt) <= reduced_dist_LB:             # <<<<<<<<<<<<<<
 *                         continue
 * 
 */
        __pyx_t_14 = __pyx_f_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_largest(__pyx_v_heap, __pyx_v_i_pt); if (unlikely(__pyx_t_14 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1979; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_16 = ((__pyx_t_14 <= __pyx_v_reduced_dist_LB) != 0);
        if (__pyx_t_16) {

          /* "binary_tree.pxi":1980
 * 
 *                     if heap.largest(i_pt) <= reduced_dist_LB:
 *                         continue             # <<<<<<<<<<<<<<
 * 
 *                     for i1 in range(node_info1.idx_start, node_info1.idx_end):
 */
          goto __pyx_L8_continue;
        }

        /* "binary_tree.pxi":1982
 *                         continue
 * 
 *                     for i1 in range(node_info1.idx_start, node_info1.idx_end):             # <<<<<<<<<<<<<<
 *                         dist_pt = self.rdist(
 *                             data1 + n_features * self.idx_array[i1],
 */
        __pyx_t_23 = __pyx_v_node_info1.idx_end;
        for (__pyx_t_24 = __pyx_v_node_info1.idx_start; __pyx_t_24 < __pyx_t_23; __pyx_t_24+=1) {
          __pyx_v_i1 = __pyx_t_24;

          /* "binary_tree.pxi":1984
 *                     for i1 in range(node_info1.idx_start, node_info1.idx_end):
 *                         dist_pt = self.rdist(
 *                             data1 + n_features * self.idx_array[i1],             # <<<<<<<<<<<<<<
 *                             data2 + n_features * i_pt,
 *                             n_features)
 */
          if (unlikely(!__pyx_v_self->idx_array.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1984; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
          __pyx_t_25 = __pyx_v_i1;

          /* "binary_tree.pxi":1983
 * 
 *                     for i1 in range(node_info1.idx_start, node_info1.idx_end):
 *                         dist_pt = self.rdist(             # <<<<<<<<<<<<<<
 *                             data1 + n_features * self.idx_array[i1],
 *                             data2 + n_features * i_pt,
 */
          __pyx_t_14 = __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree_rdist(__pyx_v_self, (__pyx_v_data1 + (__pyx_v_n_features * (*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_self->idx_array.data) + __pyx_t_25)) ))))), (__pyx_v_data2 + (__pyx_v_n_features * __pyx_v_i_pt)), __pyx_v_n_features); if (unlikely(__pyx_t_14 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1983; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __pyx_v_dist_pt = __pyx_t_14;

          /* "binary_tree.pxi":1987
 *                             data2 + n_features * i_pt,
 *                             n_features)
 *                         if dist_pt < heap.largest(i_pt):             # <<<<<<<<<<<<<<
 *                             heap._push(i_pt, dist_pt, self.idx_array[i1])
 * 
 */
          __pyx_t_14 = __pyx_f_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_largest(__pyx_v_heap, __pyx_v_i_pt); if (unlikely(__pyx_t_14 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1987; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __pyx_t_16 = ((__pyx_v_dist_pt < __pyx_t_14) != 0);
          if (__pyx_t_16) {

            /* "binary_tree.pxi":1988
 *                             n_features)
 *                         if dist_pt < heap.largest(i_pt):
 *                             heap._push(i_pt, dist_pt, self.idx_array[i1])             # <<<<<<<<<<<<<<
 * 
 *                     # keep track of node bound
 */
            if (unlikely(!__pyx_v_self->idx_array.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1988; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
            __pyx_t_26 = __pyx_v_i1;
            __pyx_t_15 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NeighborsHeap *)__pyx_v_heap->__pyx_vtab)->_push(__pyx_v_heap, __pyx_v_i_pt, __pyx_v_dist_pt, (*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_self->idx_array.data) + __pyx_t_26)) )))); if (unlikely(__pyx_t_15 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1988; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L13;
          }
          __pyx_L13:;
        }

        /* "binary_tree.pxi":1991
 * 
 *                     # keep track of node bound
 *                     bounds[i_node2] = fmax(bounds[i_node2],             # <<<<<<<<<<<<<<
 *                                            heap.largest(i_pt))
 * 
 */
        __pyx_t_23 = __pyx_v_i_node2;

        /* "binary_tree.pxi":1992
 *                     # keep track of node bound
 *                     bounds[i_node2] = fmax(bounds[i_node2],
 *                                            heap.largest(i_pt))             # <<<<<<<<<<<<<<
 * 
 *             #------------------------------------------------------------
 */
        __pyx_t_14 = __pyx_f_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_largest(__pyx_v_heap, __pyx_v_i_pt); if (unlikely(__pyx_t_14 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1992; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

        /* "binary_tree.pxi":1991
 * 
 *                     # keep track of node bound
 *                     bounds[i_node2] = fmax(bounds[i_node2],             # <<<<<<<<<<<<<<
 *                                            heap.largest(i_pt))
 * 
 */
        __pyx_t_24 = __pyx_v_i_node2;
        *((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_bounds.data) + __pyx_t_24)) )) = __pyx_f_7sklearn_9neighbors_7kd_tree_fmax((*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_bounds.data) + __pyx_t_23)) ))), __pyx_t_14);
        __pyx_L8_continue:;
      }
      goto __pyx_L5;
    }

    /* "binary_tree.pxi":1997
 *             # Case 3a: node 1 is a leaf or is smaller: split node 2 and
 *             #          recursively query, starting with the nearest subnode
 *             elif node_info1.is_leaf or (not node_info2.is_leaf             # <<<<<<<<<<<<<<
 *                                         and (node_info2.radius
 *                                              > node_info1.radius)):
 */
    __pyx_t_18 = (__pyx_v_node_info1.is_leaf != 0);
    if (!__pyx_t_18) {
    } else {
      __pyx_t_16 = __pyx_t_18;
      goto __pyx_L14_bool_binop_done;
    }

    /* "binary_tree.pxi":1998
 *             #          recursively query, starting with the nearest subnode
 *             elif node_info1.is_leaf or (not node_info2.is_leaf
 *                                         and (node_info2.radius             # <<<<<<<<<<<<<<
 *                                              > node_info1.radius)):
 *                 nodeheap_item.i1 = i_node1
 */
    __pyx_t_18 = ((!(__pyx_v_node_info2.is_leaf != 0)) != 0);
    if (__pyx_t_18) {
    } else {
      __pyx_t_16 = __pyx_t_18;
      goto __pyx_L14_bool_binop_done;
    }

    /* "binary_tree.pxi":1999
 *             elif node_info1.is_leaf or (not node_info2.is_leaf
 *                                         and (node_info2.radius
 *                                              > node_info1.radius)):             # <<<<<<<<<<<<<<
 *                 nodeheap_item.i1 = i_node1
 *                 for i2 in range(2 * i_node2 + 1, 2 * i_node2 + 3):
 */
    __pyx_t_18 = ((__pyx_v_node_info2.radius > __pyx_v_node_info1.radius) != 0);
    __pyx_t_16 = __pyx_t_18;
    __pyx_L14_bool_binop_done:;
    if (__pyx_t_16) {

      /* "binary_tree.pxi":2000
 *                                         and (node_info2.radius
 *                                              > node_info1.radius)):
 *                 nodeheap_item.i1 = i_node1             # <<<<<<<<<<<<<<
 *                 for i2 in range(2 * i_node2 + 1, 2 * i_node2 + 3):
 *                     nodeheap_item.i2 = i2
 */
      __pyx_v_nodeheap_item.i1 = __pyx_v_i_node1;

      /* "binary_tree.pxi":2001
 *                                              > node_info1.radius)):
 *                 nodeheap_item.i1 = i_node1
 *                 for i2 in range(2 * i_node2 + 1, 2 * i_node2 + 3):             # <<<<<<<<<<<<<<
 *                     nodeheap_item.i2 = i2
 *                     nodeheap_item.val = min_rdist_dual(self, i_node1,
 */
      __pyx_t_27 = ((2 * __pyx_v_i_node2) + 3);
      for (__pyx_t_20 = ((2 * __pyx_v_i_node2) + 1); __pyx_t_20 < __pyx_t_27; __pyx_t_20+=1) {
        __pyx_v_i2 = __pyx_t_20;

        /* "binary_tree.pxi":2002
 *                 nodeheap_item.i1 = i_node1
 *                 for i2 in range(2 * i_node2 + 1, 2 * i_node2 + 3):
 *                     nodeheap_item.i2 = i2             # <<<<<<<<<<<<<<
 *                     nodeheap_item.val = min_rdist_dual(self, i_node1,
 *                                                        other, i2)
 */
        __pyx_v_nodeheap_item.i2 = __pyx_v_i2;

        /* "binary_tree.pxi":2003
 *                 for i2 in range(2 * i_node2 + 1, 2 * i_node2 + 3):
 *                     nodeheap_item.i2 = i2
 *                     nodeheap_item.val = min_rdist_dual(self, i_node1,             # <<<<<<<<<<<<<<
 *                                                        other, i2)
 *                     nodeheap.push(nodeheap_item)
 */
        __pyx_t_14 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_rdist_dual(__pyx_v_self, __pyx_v_i_node1, __pyx_v_other, __pyx_v_i2); if (unlikely(__pyx_t_14 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2003; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_v_nodeheap_item.val = __pyx_t_14;

        /* "binary_tree.pxi":2005
 *                     nodeheap_item.val = min_rdist_dual(self, i_node1,
 *                                                        other, i2)
 *                     nodeheap.push(nodeheap_item)             # <<<<<<<<<<<<<<
 * 
 *             #------------------------------------------------------------
 */
        __pyx_t_15 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_v_nodeheap->__pyx_vtab)->push(__pyx_v_nodeheap, __pyx_v_nodeheap_item); if (unlikely(__pyx_t_15 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2005; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      goto __pyx_L5;
    }
    /*else*/ {

      /* "binary_tree.pxi":2011
 *             #          recursively query, starting with the nearest subnode
 *             else:
 *                 nodeheap_item.i2 = i_node2             # <<<<<<<<<<<<<<
 *                 for i1 in range(2 * i_node1 + 1, 2 * i_node1 + 3):
 *                     nodeheap_item.i1 = i1
 */
      __pyx_v_nodeheap_item.i2 = __pyx_v_i_node2;

      /* "binary_tree.pxi":2012
 *             else:
 *                 nodeheap_item.i2 = i_node2
 *                 for i1 in range(2 * i_node1 + 1, 2 * i_node1 + 3):             # <<<<<<<<<<<<<<
 *                     nodeheap_item.i1 = i1
 *                     nodeheap_item.val = min_rdist_dual(self, i1,
 */
      __pyx_t_27 = ((2 * __pyx_v_i_node1) + 3);
      for (__pyx_t_20 = ((2 * __pyx_v_i_node1) + 1); __pyx_t_20 < __pyx_t_27; __pyx_t_20+=1) {
        __pyx_v_i1 = __pyx_t_20;

        /* "binary_tree.pxi":2013
 *                 nodeheap_item.i2 = i_node2
 *                 for i1 in range(2 * i_node1 + 1, 2 * i_node1 + 3):
 *                     nodeheap_item.i1 = i1             # <<<<<<<<<<<<<<
 *                     nodeheap_item.val = min_rdist_dual(self, i1,
 *                                                        other, i_node2)
 */
        __pyx_v_nodeheap_item.i1 = __pyx_v_i1;

        /* "binary_tree.pxi":2014
 *                 for i1 in range(2 * i_node1 + 1, 2 * i_node1 + 3):
 *                     nodeheap_item.i1 = i1
 *                     nodeheap_item.val = min_rdist_dual(self, i1,             # <<<<<<<<<<<<<<
 *                                                        other, i_node2)
 *                     nodeheap.push(nodeheap_item)
 */
        __pyx_t_14 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_rdist_dual(__pyx_v_self, __pyx_v_i1, __pyx_v_other, __pyx_v_i_node2); if (unlikely(__pyx_t_14 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2014; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_v_nodeheap_item.val = __pyx_t_14;

        /* "binary_tree.pxi":2016
 *                     nodeheap_item.val = min_rdist_dual(self, i1,
 *                                                        other, i_node2)
 *                     nodeheap.push(nodeheap_item)             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
        __pyx_t_15 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_v_nodeheap->__pyx_vtab)->push(__pyx_v_nodeheap, __pyx_v_nodeheap_item); if (unlikely(__pyx_t_15 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2016; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
    }
    __pyx_L5:;
  }

  /* "binary_tree.pxi":2017
 *                                                        other, i_node2)
 *                     nodeheap.push(nodeheap_item)
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     cdef ITYPE_t _query_radius_single(self,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":1934
 *         return 0
 * 
 *     cdef int _query_dual_breadthfirst(self, BinaryTree other,             # <<<<<<<<<<<<<<
 *                                       NeighborsHeap heap,
 *                                       NodeHeap nodeheap) except -1:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree._query_dual_breadthfirst", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_bounds, 1);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":2019
 *         return 0
 * 
 *     cdef ITYPE_t _query_radius_single(self,             # <<<<<<<<<<<<<<
 *                                       ITYPE_t i_node,
 *                                       DTYPE_t* pt, DTYPE_t r,
 */

static __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__query_radius_single(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_r, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_v_indices, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_distances, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_count, int __pyx_v_count_only, int __pyx_v_return_distance) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_data;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_v_idx_array;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_features;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t __pyx_v_node_info;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_reduced_r;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_pt;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_LB;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_UB;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_7;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_query_radius_single", 0);

  /* "binary_tree.pxi":2028
 *                                       int return_distance) except -1:
 *         """recursive single-tree radius query, depth-first"""
 *         cdef DTYPE_t* data = &self.data[0, 0]             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t* idx_array = &self.idx_array[0]
 *         cdef ITYPE_t n_features = self.data.shape[1]
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2028; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_data = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_self->data.data + __pyx_t_1 * __pyx_v_self->data.strides[0]) )) + __pyx_t_2)) ))));

  /* "binary_tree.pxi":2029
 *         """recursive single-tree radius query, depth-first"""
 *         cdef DTYPE_t* data = &self.data[0, 0]
 *         cdef ITYPE_t* idx_array = &self.idx_array[0]             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t n_features = self.data.shape[1]
 *         cdef NodeData_t node_info = self.node_data[i_node]
 */
  if (unlikely(!__pyx_v_self->idx_array.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2029; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_3 = 0;
  __pyx_v_idx_array = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_self->idx_array.data) + __pyx_t_3)) ))));

  /* "binary_tree.pxi":2030
 *         cdef DTYPE_t* data = &self.data[0, 0]
 *         cdef ITYPE_t* idx_array = &self.idx_array[0]
 *         cdef ITYPE_t n_features = self.data.shape[1]             # <<<<<<<<<<<<<<
 *         cdef NodeData_t node_info = self.node_data[i_node]
 * 
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2030; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_n_features = (__pyx_v_self->data.shape[1]);

  /* "binary_tree.pxi":2031
 *         cdef ITYPE_t* idx_array = &self.idx_array[0]
 *         cdef ITYPE_t n_features = self.data.shape[1]
 *         cdef NodeData_t node_info = self.node_data[i_node]             # <<<<<<<<<<<<<<
 * 
 *         cdef ITYPE_t i
 */
  if (unlikely(!__pyx_v_self->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2031; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_4 = __pyx_v_i_node;
  __pyx_v_node_info = (*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_self->node_data.data) + __pyx_t_4)) )));

  /* "binary_tree.pxi":2036
 *         cdef DTYPE_t reduced_r
 * 
 *         cdef DTYPE_t dist_pt, dist_LB = 0, dist_UB = 0             # <<<<<<<<<<<<<<
 *         min_max_dist(self, i_node, pt, &dist_LB, &dist_UB)
 * 
 */
  __pyx_v_dist_LB = 0.0;
  __pyx_v_dist_UB = 0.0;

  /* "binary_tree.pxi":2037
 * 
 *         cdef DTYPE_t dist_pt, dist_LB = 0, dist_UB = 0
 *         min_max_dist(self, i_node, pt, &dist_LB, &dist_UB)             # <<<<<<<<<<<<<<
 * 
 *         #------------------------------------------------------------
 */
  __pyx_t_5 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_max_dist(__pyx_v_self, __pyx_v_i_node, __pyx_v_pt, (&__pyx_v_dist_LB), (&__pyx_v_dist_UB)); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2037; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "binary_tree.pxi":2042
 *         # Case 1: all node points are outside distance r.
 *         #         prune this branch.
 *         if dist_LB > r:             # <<<<<<<<<<<<<<
 *             pass
 * 
 */
  __pyx_t_6 = ((__pyx_v_dist_LB > __pyx_v_r) != 0);
  if (__pyx_t_6) {
    goto __pyx_L3;
  }

  /* "binary_tree.pxi":2048
 *         # Case 2: all node points are within distance r
 *         #         add all points to neighbors
 *         elif dist_UB <= r:             # <<<<<<<<<<<<<<
 *             if count_only:
 *                 count += (node_info.idx_end - node_info.idx_start)
 */
  __pyx_t_6 = ((__pyx_v_dist_UB <= __pyx_v_r) != 0);
  if (__pyx_t_6) {

    /* "binary_tree.pxi":2049
 *         #         add all points to neighbors
 *         elif dist_UB <= r:
 *             if count_only:             # <<<<<<<<<<<<<<
 *                 count += (node_info.idx_end - node_info.idx_start)
 *             else:
 */
    __pyx_t_6 = (__pyx_v_count_only != 0);
    if (__pyx_t_6) {

      /* "binary_tree.pxi":2050
 *         elif dist_UB <= r:
 *             if count_only:
 *                 count += (node_info.idx_end - node_info.idx_start)             # <<<<<<<<<<<<<<
 *             else:
 *                 for i in range(node_info.idx_start, node_info.idx_end):
 */
      __pyx_v_count = (__pyx_v_count + (__pyx_v_node_info.idx_end - __pyx_v_node_info.idx_start));
      goto __pyx_L4;
    }
    /*else*/ {

      /* "binary_tree.pxi":2052
 *                 count += (node_info.idx_end - node_info.idx_start)
 *             else:
 *                 for i in range(node_info.idx_start, node_info.idx_end):             # <<<<<<<<<<<<<<
 *                     if (count < 0) or (count >= self.data.shape[0]):
 *                         raise ValueError("Fatal: count too big: "
 */
      __pyx_t_7 = __pyx_v_node_info.idx_end;
      for (__pyx_t_8 = __pyx_v_node_info.idx_start; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
        __pyx_v_i = __pyx_t_8;

        /* "binary_tree.pxi":2053
 *             else:
 *                 for i in range(node_info.idx_start, node_info.idx_end):
 *                     if (count < 0) or (count >= self.data.shape[0]):             # <<<<<<<<<<<<<<
 *                         raise ValueError("Fatal: count too big: "
 *                                          "this should never happen")
 */
        __pyx_t_9 = ((__pyx_v_count < 0) != 0);
        if (!__pyx_t_9) {
        } else {
          __pyx_t_6 = __pyx_t_9;
          goto __pyx_L8_bool_binop_done;
        }
        if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2053; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
        __pyx_t_9 = ((__pyx_v_count >= (__pyx_v_self->data.shape[0])) != 0);
        __pyx_t_6 = __pyx_t_9;
        __pyx_L8_bool_binop_done:;
        if (__pyx_t_6) {

          /* "binary_tree.pxi":2054
 *                 for i in range(node_info.idx_start, node_info.idx_end):
 *                     if (count < 0) or (count >= self.data.shape[0]):
 *                         raise ValueError("Fatal: count too big: "             # <<<<<<<<<<<<<<
 *                                          "this should never happen")
 *                     indices[count] = idx_array[i]
 */
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__30, NULL); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_Raise(__pyx_t_10, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }

        /* "binary_tree.pxi":2056
 *                         raise ValueError("Fatal: count too big: "
 *                                          "this should never happen")
 *                     indices[count] = idx_array[i]             # <<<<<<<<<<<<<<
 *                     if return_distance:
 *                         distances[count] = self.dist(pt, (data + n_features
 */
        (__pyx_v_indices[__pyx_v_count]) = (__pyx_v_idx_array[__pyx_v_i]);

        /* "binary_tree.pxi":2057
 *                                          "this should never happen")
 *                     indices[count] = idx_array[i]
 *                     if return_distance:             # <<<<<<<<<<<<<<
 *                         distances[count] = self.dist(pt, (data + n_features
 *                                                           * idx_array[i]),
 */
        __pyx_t_6 = (__pyx_v_return_distance != 0);
        if (__pyx_t_6) {

          /* "binary_tree.pxi":2058
 *                     indices[count] = idx_array[i]
 *                     if return_distance:
 *                         distances[count] = self.dist(pt, (data + n_features             # <<<<<<<<<<<<<<
 *                                                           * idx_array[i]),
 *                                                      n_features)
 */
          __pyx_t_11 = __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree_dist(__pyx_v_self, __pyx_v_pt, (__pyx_v_data + (__pyx_v_n_features * (__pyx_v_idx_array[__pyx_v_i]))), __pyx_v_n_features); if (unlikely(__pyx_t_11 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2058; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          (__pyx_v_distances[__pyx_v_count]) = __pyx_t_11;
          goto __pyx_L10;
        }
        __pyx_L10:;

        /* "binary_tree.pxi":2061
 *                                                           * idx_array[i]),
 *                                                      n_features)
 *                     count += 1             # <<<<<<<<<<<<<<
 * 
 *         #------------------------------------------------------------
 */
        __pyx_v_count = (__pyx_v_count + 1);
      }
    }
    __pyx_L4:;
    goto __pyx_L3;
  }

  /* "binary_tree.pxi":2066
 *         # Case 3: this is a leaf node.  Go through all points to
 *         #         determine if they fall within radius
 *         elif node_info.is_leaf:             # <<<<<<<<<<<<<<
 *             reduced_r = self.dist_metric._dist_to_rdist(r)
 * 
 */
  __pyx_t_6 = (__pyx_v_node_info.is_leaf != 0);
  if (__pyx_t_6) {

    /* "binary_tree.pxi":2067
 *         #         determine if they fall within radius
 *         elif node_info.is_leaf:
 *             reduced_r = self.dist_metric._dist_to_rdist(r)             # <<<<<<<<<<<<<<
 * 
 *             for i in range(node_info.idx_start, node_info.idx_end):
 */
    __pyx_t_11 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_12dist_metrics_DistanceMetric *)__pyx_v_self->dist_metric->__pyx_vtab)->_dist_to_rdist(__pyx_v_self->dist_metric, __pyx_v_r); if (unlikely(__pyx_t_11 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2067; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_reduced_r = __pyx_t_11;

    /* "binary_tree.pxi":2069
 *             reduced_r = self.dist_metric._dist_to_rdist(r)
 * 
 *             for i in range(node_info.idx_start, node_info.idx_end):             # <<<<<<<<<<<<<<
 *                 dist_pt = self.rdist(pt, (data + n_features * idx_array[i]),
 *                                      n_features)
 */
    __pyx_t_7 = __pyx_v_node_info.idx_end;
    for (__pyx_t_8 = __pyx_v_node_info.idx_start; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
      __pyx_v_i = __pyx_t_8;

      /* "binary_tree.pxi":2070
 * 
 *             for i in range(node_info.idx_start, node_info.idx_end):
 *                 dist_pt = self.rdist(pt, (data + n_features * idx_array[i]),             # <<<<<<<<<<<<<<
 *                                      n_features)
 *                 if dist_pt <= reduced_r:
 */
      __pyx_t_11 = __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree_rdist(__pyx_v_self, __pyx_v_pt, (__pyx_v_data + (__pyx_v_n_features * (__pyx_v_idx_array[__pyx_v_i]))), __pyx_v_n_features); if (unlikely(__pyx_t_11 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2070; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_v_dist_pt = __pyx_t_11;

      /* "binary_tree.pxi":2072
 *                 dist_pt = self.rdist(pt, (data + n_features * idx_array[i]),
 *                                      n_features)
 *                 if dist_pt <= reduced_r:             # <<<<<<<<<<<<<<
 *                     if (count < 0) or (count >= self.data.shape[0]):
 *                         raise ValueError("Fatal: count out of range. "
 */
      __pyx_t_6 = ((__pyx_v_dist_pt <= __pyx_v_reduced_r) != 0);
      if (__pyx_t_6) {

        /* "binary_tree.pxi":2073
 *                                      n_features)
 *                 if dist_pt <= reduced_r:
 *                     if (count < 0) or (count >= self.data.shape[0]):             # <<<<<<<<<<<<<<
 *                         raise ValueError("Fatal: count out of range. "
 *                                          "This should never happen.")
 */
        __pyx_t_9 = ((__pyx_v_count < 0) != 0);
        if (!__pyx_t_9) {
        } else {
          __pyx_t_6 = __pyx_t_9;
          goto __pyx_L15_bool_binop_done;
        }
        if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2073; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
        __pyx_t_9 = ((__pyx_v_count >= (__pyx_v_self->data.shape[0])) != 0);
        __pyx_t_6 = __pyx_t_9;
        __pyx_L15_bool_binop_done:;
        if (__pyx_t_6) {

          /* "binary_tree.pxi":2074
 *                 if dist_pt <= reduced_r:
 *                     if (count < 0) or (count >= self.data.shape[0]):
 *                         raise ValueError("Fatal: count out of range. "             # <<<<<<<<<<<<<<
 *                                          "This should never happen.")
 *                     if count_only:
 */
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__31, NULL); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2074; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_Raise(__pyx_t_10, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2074; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }

        /* "binary_tree.pxi":2076
 *                         raise ValueError("Fatal: count out of range. "
 *                                          "This should never happen.")
 *                     if count_only:             # <<<<<<<<<<<<<<
 *                         pass
 *                     else:
 */
        __pyx_t_6 = (__pyx_v_count_only != 0);
        if (__pyx_t_6) {
          goto __pyx_L17;
        }
        /*else*/ {

          /* "binary_tree.pxi":2079
 *                         pass
 *                     else:
 *                         indices[count] = idx_array[i]             # <<<<<<<<<<<<<<
 *                         if return_distance:
 *                             distances[count] =\
 */
          (__pyx_v_indices[__pyx_v_count]) = (__pyx_v_idx_array[__pyx_v_i]);

          /* "binary_tree.pxi":2080
 *                     else:
 *                         indices[count] = idx_array[i]
 *                         if return_distance:             # <<<<<<<<<<<<<<
 *                             distances[count] =\
 *                                 self.dist_metric._rdist_to_dist(dist_pt)
 */
          __pyx_t_6 = (__pyx_v_return_distance != 0);
          if (__pyx_t_6) {

            /* "binary_tree.pxi":2082
 *                         if return_distance:
 *                             distances[count] =\
 *                                 self.dist_metric._rdist_to_dist(dist_pt)             # <<<<<<<<<<<<<<
 *                     count += 1
 * 
 */
            __pyx_t_11 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_12dist_metrics_DistanceMetric *)__pyx_v_self->dist_metric->__pyx_vtab)->_rdist_to_dist(__pyx_v_self->dist_metric, __pyx_v_dist_pt); if (unlikely(__pyx_t_11 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

            /* "binary_tree.pxi":2081
 *                         indices[count] = idx_array[i]
 *                         if return_distance:
 *                             distances[count] =\             # <<<<<<<<<<<<<<
 *                                 self.dist_metric._rdist_to_dist(dist_pt)
 *                     count += 1
 */
            (__pyx_v_distances[__pyx_v_count]) = __pyx_t_11;
            goto __pyx_L18;
          }
          __pyx_L18:;
        }
        __pyx_L17:;

        /* "binary_tree.pxi":2083
 *                             distances[count] =\
 *                                 self.dist_metric._rdist_to_dist(dist_pt)
 *                     count += 1             # <<<<<<<<<<<<<<
 * 
 *         #------------------------------------------------------------
 */
        __pyx_v_count = (__pyx_v_count + 1);
        goto __pyx_L13;
      }
      __pyx_L13:;
    }
    goto __pyx_L3;
  }
  /*else*/ {

    /* "binary_tree.pxi":2088
 *         # Case 4: Node is not a leaf.  Recursively query subnodes
 *         else:
 *             count = self._query_radius_single(2 * i_node + 1, pt, r,             # <<<<<<<<<<<<<<
 *                                               indices, distances, count,
 *                                               count_only, return_distance)
 */
    __pyx_t_7 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_query_radius_single(__pyx_v_self, ((2 * __pyx_v_i_node) + 1), __pyx_v_pt, __pyx_v_r, __pyx_v_indices, __pyx_v_distances, __pyx_v_count, __pyx_v_count_only, __pyx_v_return_distance); if (unlikely(__pyx_t_7 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2088; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_count = __pyx_t_7;

    /* "binary_tree.pxi":2091
 *                                               indices, distances, count,
 *                                               count_only, return_distance)
 *             count = self._query_radius_single(2 * i_node + 2, pt, r,             # <<<<<<<<<<<<<<
 *                                               indices, distances, count,
 *                                               count_only, return_distance)
 */
    __pyx_t_7 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_query_radius_single(__pyx_v_self, ((2 * __pyx_v_i_node) + 2), __pyx_v_pt, __pyx_v_r, __pyx_v_indices, __pyx_v_distances, __pyx_v_count, __pyx_v_count_only, __pyx_v_return_distance); if (unlikely(__pyx_t_7 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2091; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_count = __pyx_t_7;
  }
  __pyx_L3:;

  /* "binary_tree.pxi":2095
 *                                               count_only, return_distance)
 * 
 *         return count             # <<<<<<<<<<<<<<
 * 
 *     cdef DTYPE_t _kde_single_breadthfirst(self, DTYPE_t* pt,
 */
  __pyx_r = __pyx_v_count;
  goto __pyx_L0;

  /* "binary_tree.pxi":2019
 *         return 0
 * 
 *     cdef ITYPE_t _query_radius_single(self,             # <<<<<<<<<<<<<<
 *                                       ITYPE_t i_node,
 *                                       DTYPE_t* pt, DTYPE_t r,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree._query_radius_single", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":2097
 *         return count
 * 
 *     cdef DTYPE_t _kde_single_breadthfirst(self, DTYPE_t* pt,             # <<<<<<<<<<<<<<
 *                                           KernelType kernel, DTYPE_t h,
 *                                           DTYPE_t log_knorm,
 */

static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__kde_single_breadthfirst(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt, enum __pyx_t_7sklearn_9neighbors_7kd_tree_KernelType __pyx_v_kernel, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_h, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_log_knorm, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_log_atol, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_log_rtol, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_nodeheap, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_node_log_min_bounds, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_node_log_bound_spreads) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_N1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_N2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_global_log_min_bound;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_global_log_bound_spread;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_global_log_max_bound;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_data;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_v_idx_array;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *__pyx_v_node_data;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_N;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_features;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t __pyx_v_node_info;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_pt;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_log_density;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_LB_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_LB_2;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_UB_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_UB_2;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t __pyx_v_nodeheap_item;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_8;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_kde_single_breadthfirst", 0);

  /* "binary_tree.pxi":2116
 *         cdef DTYPE_t global_log_max_bound
 * 
 *         cdef DTYPE_t* data = &self.data[0, 0]             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t* idx_array = &self.idx_array[0]
 *         cdef NodeData_t* node_data = &self.node_data[0]
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2116; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_data = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_self->data.data + __pyx_t_1 * __pyx_v_self->data.strides[0]) )) + __pyx_t_2)) ))));

  /* "binary_tree.pxi":2117
 * 
 *         cdef DTYPE_t* data = &self.data[0, 0]
 *         cdef ITYPE_t* idx_array = &self.idx_array[0]             # <<<<<<<<<<<<<<
 *         cdef NodeData_t* node_data = &self.node_data[0]
 *         cdef ITYPE_t N = self.data.shape[0]
 */
  if (unlikely(!__pyx_v_self->idx_array.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_3 = 0;
  __pyx_v_idx_array = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_self->idx_array.data) + __pyx_t_3)) ))));

  /* "binary_tree.pxi":2118
 *         cdef DTYPE_t* data = &self.data[0, 0]
 *         cdef ITYPE_t* idx_array = &self.idx_array[0]
 *         cdef NodeData_t* node_data = &self.node_data[0]             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t N = self.data.shape[0]
 *         cdef ITYPE_t n_features = self.data.shape[1]
 */
  if (unlikely(!__pyx_v_self->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2118; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_4 = 0;
  __pyx_v_node_data = (&(*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_self->node_data.data) + __pyx_t_4)) ))));

  /* "binary_tree.pxi":2119
 *         cdef ITYPE_t* idx_array = &self.idx_array[0]
 *         cdef NodeData_t* node_data = &self.node_data[0]
 *         cdef ITYPE_t N = self.data.shape[0]             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t n_features = self.data.shape[1]
 * 
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2119; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_N = (__pyx_v_self->data.shape[0]);

  /* "binary_tree.pxi":2120
 *         cdef NodeData_t* node_data = &self.node_data[0]
 *         cdef ITYPE_t N = self.data.shape[0]
 *         cdef ITYPE_t n_features = self.data.shape[1]             # <<<<<<<<<<<<<<
 * 
 *         cdef NodeData_t node_info
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2120; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_n_features = (__pyx_v_self->data.shape[1]);

  /* "binary_tree.pxi":2124
 *         cdef NodeData_t node_info
 *         cdef DTYPE_t dist_pt, log_density
 *         cdef DTYPE_t dist_LB_1 = 0, dist_LB_2 = 0             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t dist_UB_1 = 0, dist_UB_2 = 0
 * 
 */
  __pyx_v_dist_LB_1 = 0.0;
  __pyx_v_dist_LB_2 = 0.0;

  /* "binary_tree.pxi":2125
 *         cdef DTYPE_t dist_pt, log_density
 *         cdef DTYPE_t dist_LB_1 = 0, dist_LB_2 = 0
 *         cdef DTYPE_t dist_UB_1 = 0, dist_UB_2 = 0             # <<<<<<<<<<<<<<
 * 
 *         cdef DTYPE_t dist_UB, dist_LB
 */
  __pyx_v_dist_UB_1 = 0.0;
  __pyx_v_dist_UB_2 = 0.0;

  /* "binary_tree.pxi":2131
 *         # push the top node to the heap
 *         cdef NodeHeapData_t nodeheap_item
 *         nodeheap_item.val = min_dist(self, 0, pt)             # <<<<<<<<<<<<<<
 *         nodeheap_item.i1 = 0
 *         nodeheap.push(nodeheap_item)
 */
  __pyx_t_5 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_dist(__pyx_v_self, 0, __pyx_v_pt); if (unlikely(__pyx_t_5 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_nodeheap_item.val = __pyx_t_5;

  /* "binary_tree.pxi":2132
 *         cdef NodeHeapData_t nodeheap_item
 *         nodeheap_item.val = min_dist(self, 0, pt)
 *         nodeheap_item.i1 = 0             # <<<<<<<<<<<<<<
 *         nodeheap.push(nodeheap_item)
 * 
 */
  __pyx_v_nodeheap_item.i1 = 0;

  /* "binary_tree.pxi":2133
 *         nodeheap_item.val = min_dist(self, 0, pt)
 *         nodeheap_item.i1 = 0
 *         nodeheap.push(nodeheap_item)             # <<<<<<<<<<<<<<
 * 
 *         global_log_min_bound = log(N) + compute_log_kernel(max_dist(self,
 */
  __pyx_t_6 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_v_nodeheap->__pyx_vtab)->push(__pyx_v_nodeheap, __pyx_v_nodeheap_item); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2133; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "binary_tree.pxi":2135
 *         nodeheap.push(nodeheap_item)
 * 
 *         global_log_min_bound = log(N) + compute_log_kernel(max_dist(self,             # <<<<<<<<<<<<<<
 *                                                                     0, pt),
 *                                                            h, kernel)
 */
  __pyx_t_5 = __pyx_f_7sklearn_9neighbors_7kd_tree_max_dist(__pyx_v_self, 0, __pyx_v_pt); if (unlikely(__pyx_t_5 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "binary_tree.pxi":2137
 *         global_log_min_bound = log(N) + compute_log_kernel(max_dist(self,
 *                                                                     0, pt),
 *                                                            h, kernel)             # <<<<<<<<<<<<<<
 *         global_log_max_bound = log(N) + compute_log_kernel(nodeheap_item.val,
 *                                                            h, kernel)
 */
  __pyx_v_global_log_min_bound = (log(__pyx_v_N) + __pyx_f_7sklearn_9neighbors_7kd_tree_compute_log_kernel(__pyx_t_5, __pyx_v_h, __pyx_v_kernel));

  /* "binary_tree.pxi":2138
 *                                                                     0, pt),
 *                                                            h, kernel)
 *         global_log_max_bound = log(N) + compute_log_kernel(nodeheap_item.val,             # <<<<<<<<<<<<<<
 *                                                            h, kernel)
 *         global_log_bound_spread = logsubexp(global_log_max_bound,
 */
  __pyx_v_global_log_max_bound = (log(__pyx_v_N) + __pyx_f_7sklearn_9neighbors_7kd_tree_compute_log_kernel(__pyx_v_nodeheap_item.val, __pyx_v_h, __pyx_v_kernel));

  /* "binary_tree.pxi":2140
 *         global_log_max_bound = log(N) + compute_log_kernel(nodeheap_item.val,
 *                                                            h, kernel)
 *         global_log_bound_spread = logsubexp(global_log_max_bound,             # <<<<<<<<<<<<<<
 *                                             global_log_min_bound)
 * 
 */
  __pyx_v_global_log_bound_spread = __pyx_f_7sklearn_9neighbors_7kd_tree_logsubexp(__pyx_v_global_log_max_bound, __pyx_v_global_log_min_bound);

  /* "binary_tree.pxi":2143
 *                                             global_log_min_bound)
 * 
 *         node_log_min_bounds[0] = global_log_min_bound             # <<<<<<<<<<<<<<
 *         node_log_bound_spreads[0] = global_log_bound_spread
 * 
 */
  (__pyx_v_node_log_min_bounds[0]) = __pyx_v_global_log_min_bound;

  /* "binary_tree.pxi":2144
 * 
 *         node_log_min_bounds[0] = global_log_min_bound
 *         node_log_bound_spreads[0] = global_log_bound_spread             # <<<<<<<<<<<<<<
 * 
 *         while nodeheap.n > 0:
 */
  (__pyx_v_node_log_bound_spreads[0]) = __pyx_v_global_log_bound_spread;

  /* "binary_tree.pxi":2146
 *         node_log_bound_spreads[0] = global_log_bound_spread
 * 
 *         while nodeheap.n > 0:             # <<<<<<<<<<<<<<
 *             nodeheap_item = nodeheap.pop()
 *             i_node = nodeheap_item.i1
 */
  while (1) {
    __pyx_t_7 = ((__pyx_v_nodeheap->n > 0) != 0);
    if (!__pyx_t_7) break;

    /* "binary_tree.pxi":2147
 * 
 *         while nodeheap.n > 0:
 *             nodeheap_item = nodeheap.pop()             # <<<<<<<<<<<<<<
 *             i_node = nodeheap_item.i1
 * 
 */
    __pyx_v_nodeheap_item = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_v_nodeheap->__pyx_vtab)->pop(__pyx_v_nodeheap);

    /* "binary_tree.pxi":2148
 *         while nodeheap.n > 0:
 *             nodeheap_item = nodeheap.pop()
 *             i_node = nodeheap_item.i1             # <<<<<<<<<<<<<<
 * 
 *             node_info = node_data[i_node]
 */
    __pyx_t_8 = __pyx_v_nodeheap_item.i1;
    __pyx_v_i_node = __pyx_t_8;

    /* "binary_tree.pxi":2150
 *             i_node = nodeheap_item.i1
 * 
 *             node_info = node_data[i_node]             # <<<<<<<<<<<<<<
 *             N1 = node_info.idx_end - node_info.idx_start
 * 
 */
    __pyx_v_node_info = (__pyx_v_node_data[__pyx_v_i_node]);

    /* "binary_tree.pxi":2151
 * 
 *             node_info = node_data[i_node]
 *             N1 = node_info.idx_end - node_info.idx_start             # <<<<<<<<<<<<<<
 * 
 *             #------------------------------------------------------------
 */
    __pyx_v_N1 = (__pyx_v_node_info.idx_end - __pyx_v_node_info.idx_start);

    /* "binary_tree.pxi":2156
 *             # Case 1: local bounds are equal to within per-point tolerance.
 *             if (log_knorm + node_log_bound_spreads[i_node] - log(N1) + log(N)
 *                 <= logaddexp(log_atol, (log_rtol + log_knorm             # <<<<<<<<<<<<<<
 *                                         + node_log_min_bounds[i_node]))):
 *                 pass
 */
    __pyx_t_7 = (((((__pyx_v_log_knorm + (__pyx_v_node_log_bound_spreads[__pyx_v_i_node])) - log(__pyx_v_N1)) + log(__pyx_v_N)) <= __pyx_f_7sklearn_9neighbors_7kd_tree_logaddexp(__pyx_v_log_atol, ((__pyx_v_log_rtol + __pyx_v_log_knorm) + (__pyx_v_node_log_min_bounds[__pyx_v_i_node])))) != 0);
    if (__pyx_t_7) {
      goto __pyx_L5;
    }

    /* "binary_tree.pxi":2163
 *             # Case 2: global bounds are within rtol & atol.
 *             elif (log_knorm + global_log_bound_spread
 *                   <= logaddexp(log_atol,             # <<<<<<<<<<<<<<
 *                                log_rtol + log_knorm + global_log_min_bound)):
 *                 break
 */
    __pyx_t_7 = (((__pyx_v_log_knorm + __pyx_v_global_log_bound_spread) <= __pyx_f_7sklearn_9neighbors_7kd_tree_logaddexp(__pyx_v_log_atol, ((__pyx_v_log_rtol + __pyx_v_log_knorm) + __pyx_v_global_log_min_bound))) != 0);
    if (__pyx_t_7) {

      /* "binary_tree.pxi":2165
 *                   <= logaddexp(log_atol,
 *                                log_rtol + log_knorm + global_log_min_bound)):
 *                 break             # <<<<<<<<<<<<<<
 * 
 *             #------------------------------------------------------------
 */
      goto __pyx_L4_break;
    }

    /* "binary_tree.pxi":2169
 *             #------------------------------------------------------------
 *             # Case 3: node is a leaf. Count contributions from all points
 *             elif node_info.is_leaf:             # <<<<<<<<<<<<<<
 *                 global_log_min_bound =\
 *                     logsubexp(global_log_min_bound,
 */
    __pyx_t_7 = (__pyx_v_node_info.is_leaf != 0);
    if (__pyx_t_7) {

      /* "binary_tree.pxi":2171
 *             elif node_info.is_leaf:
 *                 global_log_min_bound =\
 *                     logsubexp(global_log_min_bound,             # <<<<<<<<<<<<<<
 *                               node_log_min_bounds[i_node])
 *                 global_log_bound_spread =\
 */
      __pyx_v_global_log_min_bound = __pyx_f_7sklearn_9neighbors_7kd_tree_logsubexp(__pyx_v_global_log_min_bound, (__pyx_v_node_log_min_bounds[__pyx_v_i_node]));

      /* "binary_tree.pxi":2174
 *                               node_log_min_bounds[i_node])
 *                 global_log_bound_spread =\
 *                     logsubexp(global_log_bound_spread,             # <<<<<<<<<<<<<<
 *                               node_log_bound_spreads[i_node])
 *                 for i in range(node_info.idx_start, node_info.idx_end):
 */
      __pyx_v_global_log_bound_spread = __pyx_f_7sklearn_9neighbors_7kd_tree_logsubexp(__pyx_v_global_log_bound_spread, (__pyx_v_node_log_bound_spreads[__pyx_v_i_node]));

      /* "binary_tree.pxi":2176
 *                     logsubexp(global_log_bound_spread,
 *                               node_log_bound_spreads[i_node])
 *                 for i in range(node_info.idx_start, node_info.idx_end):             # <<<<<<<<<<<<<<
 *                     dist_pt = self.dist(pt, data + n_features * idx_array[i],
 *                                         n_features)
 */
      __pyx_t_8 = __pyx_v_node_info.idx_end;
      for (__pyx_t_9 = __pyx_v_node_info.idx_start; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
        __pyx_v_i = __pyx_t_9;

        /* "binary_tree.pxi":2177
 *                               node_log_bound_spreads[i_node])
 *                 for i in range(node_info.idx_start, node_info.idx_end):
 *                     dist_pt = self.dist(pt, data + n_features * idx_array[i],             # <<<<<<<<<<<<<<
 *                                         n_features)
 *                     log_density = compute_log_kernel(dist_pt, h, kernel)
 */
        __pyx_t_5 = __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree_dist(__pyx_v_self, __pyx_v_pt, (__pyx_v_data + (__pyx_v_n_features * (__pyx_v_idx_array[__pyx_v_i]))), __pyx_v_n_features); if (unlikely(__pyx_t_5 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2177; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_v_dist_pt = __pyx_t_5;

        /* "binary_tree.pxi":2179
 *                     dist_pt = self.dist(pt, data + n_features * idx_array[i],
 *                                         n_features)
 *                     log_density = compute_log_kernel(dist_pt, h, kernel)             # <<<<<<<<<<<<<<
 *                     global_log_min_bound = logaddexp(global_log_min_bound,
 *                                                      log_density)
 */
        __pyx_v_log_density = __pyx_f_7sklearn_9neighbors_7kd_tree_compute_log_kernel(__pyx_v_dist_pt, __pyx_v_h, __pyx_v_kernel);

        /* "binary_tree.pxi":2180
 *                                         n_features)
 *                     log_density = compute_log_kernel(dist_pt, h, kernel)
 *                     global_log_min_bound = logaddexp(global_log_min_bound,             # <<<<<<<<<<<<<<
 *                                                      log_density)
 * 
 */
        __pyx_v_global_log_min_bound = __pyx_f_7sklearn_9neighbors_7kd_tree_logaddexp(__pyx_v_global_log_min_bound, __pyx_v_log_density);
      }
      goto __pyx_L5;
    }
    /*else*/ {

      /* "binary_tree.pxi":2186
 *             # Case 4: split node and query subnodes
 *             else:
 *                 i1 = 2 * i_node + 1             # <<<<<<<<<<<<<<
 *                 i2 = 2 * i_node + 2
 * 
 */
      __pyx_v_i1 = ((2 * __pyx_v_i_node) + 1);

      /* "binary_tree.pxi":2187
 *             else:
 *                 i1 = 2 * i_node + 1
 *                 i2 = 2 * i_node + 2             # <<<<<<<<<<<<<<
 * 
 *                 N1 = node_data[i1].idx_end - node_data[i1].idx_start
 */
      __pyx_v_i2 = ((2 * __pyx_v_i_node) + 2);

      /* "binary_tree.pxi":2189
 *                 i2 = 2 * i_node + 2
 * 
 *                 N1 = node_data[i1].idx_end - node_data[i1].idx_start             # <<<<<<<<<<<<<<
 *                 N2 = node_data[i2].idx_end - node_data[i2].idx_start
 * 
 */
      __pyx_v_N1 = ((__pyx_v_node_data[__pyx_v_i1]).idx_end - (__pyx_v_node_data[__pyx_v_i1]).idx_start);

      /* "binary_tree.pxi":2190
 * 
 *                 N1 = node_data[i1].idx_end - node_data[i1].idx_start
 *                 N2 = node_data[i2].idx_end - node_data[i2].idx_start             # <<<<<<<<<<<<<<
 * 
 *                 min_max_dist(self, i1, pt, &dist_LB_1, &dist_UB_1)
 */
      __pyx_v_N2 = ((__pyx_v_node_data[__pyx_v_i2]).idx_end - (__pyx_v_node_data[__pyx_v_i2]).idx_start);

      /* "binary_tree.pxi":2192
 *                 N2 = node_data[i2].idx_end - node_data[i2].idx_start
 * 
 *                 min_max_dist(self, i1, pt, &dist_LB_1, &dist_UB_1)             # <<<<<<<<<<<<<<
 *                 min_max_dist(self, i2, pt, &dist_LB_2, &dist_UB_2)
 * 
 */
      __pyx_t_6 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_max_dist(__pyx_v_self, __pyx_v_i1, __pyx_v_pt, (&__pyx_v_dist_LB_1), (&__pyx_v_dist_UB_1)); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2192; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "binary_tree.pxi":2193
 * 
 *                 min_max_dist(self, i1, pt, &dist_LB_1, &dist_UB_1)
 *                 min_max_dist(self, i2, pt, &dist_LB_2, &dist_UB_2)             # <<<<<<<<<<<<<<
 * 
 *                 node_log_min_bounds[i1] = (log(N1) +
 */
      __pyx_t_6 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_max_dist(__pyx_v_self, __pyx_v_i2, __pyx_v_pt, (&__pyx_v_dist_LB_2), (&__pyx_v_dist_UB_2)); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2193; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "binary_tree.pxi":2195
 *                 min_max_dist(self, i2, pt, &dist_LB_2, &dist_UB_2)
 * 
 *                 node_log_min_bounds[i1] = (log(N1) +             # <<<<<<<<<<<<<<
 *                                            compute_log_kernel(dist_UB_1,
 *                                                               h, kernel))
 */
      (__pyx_v_node_log_min_bounds[__pyx_v_i1]) = (log(__pyx_v_N1) + __pyx_f_7sklearn_9neighbors_7kd_tree_compute_log_kernel(__pyx_v_dist_UB_1, __pyx_v_h, __pyx_v_kernel));

      /* "binary_tree.pxi":2198
 *                                            compute_log_kernel(dist_UB_1,
 *                                                               h, kernel))
 *                 node_log_bound_spreads[i1] = (log(N1) +             # <<<<<<<<<<<<<<
 *                                               compute_log_kernel(dist_LB_1,
 *                                                                  h, kernel))
 */
      (__pyx_v_node_log_bound_spreads[__pyx_v_i1]) = (log(__pyx_v_N1) + __pyx_f_7sklearn_9neighbors_7kd_tree_compute_log_kernel(__pyx_v_dist_LB_1, __pyx_v_h, __pyx_v_kernel));

      /* "binary_tree.pxi":2202
 *                                                                  h, kernel))
 * 
 *                 node_log_min_bounds[i2] = (log(N2) +             # <<<<<<<<<<<<<<
 *                                            compute_log_kernel(dist_UB_2,
 *                                                               h, kernel))
 */
      (__pyx_v_node_log_min_bounds[__pyx_v_i2]) = (log(__pyx_v_N2) + __pyx_f_7sklearn_9neighbors_7kd_tree_compute_log_kernel(__pyx_v_dist_UB_2, __pyx_v_h, __pyx_v_kernel));

      /* "binary_tree.pxi":2205
 *                                            compute_log_kernel(dist_UB_2,
 *                                                               h, kernel))
 *                 node_log_bound_spreads[i2] = (log(N2) +             # <<<<<<<<<<<<<<
 *                                               compute_log_kernel(dist_LB_2,
 *                                                                  h, kernel))
 */
      (__pyx_v_node_log_bound_spreads[__pyx_v_i2]) = (log(__pyx_v_N2) + __pyx_f_7sklearn_9neighbors_7kd_tree_compute_log_kernel(__pyx_v_dist_LB_2, __pyx_v_h, __pyx_v_kernel));

      /* "binary_tree.pxi":2209
 *                                                                  h, kernel))
 * 
 *                 global_log_min_bound = logsubexp(global_log_min_bound,             # <<<<<<<<<<<<<<
 *                                                  node_log_min_bounds[i_node])
 *                 global_log_min_bound = logaddexp(global_log_min_bound,
 */
      __pyx_v_global_log_min_bound = __pyx_f_7sklearn_9neighbors_7kd_tree_logsubexp(__pyx_v_global_log_min_bound, (__pyx_v_node_log_min_bounds[__pyx_v_i_node]));

      /* "binary_tree.pxi":2211
 *                 global_log_min_bound = logsubexp(global_log_min_bound,
 *                                                  node_log_min_bounds[i_node])
 *                 global_log_min_bound = logaddexp(global_log_min_bound,             # <<<<<<<<<<<<<<
 *                                                  node_log_min_bounds[i1])
 *                 global_log_min_bound = logaddexp(global_log_min_bound,
 */
      __pyx_v_global_log_min_bound = __pyx_f_7sklearn_9neighbors_7kd_tree_logaddexp(__pyx_v_global_log_min_bound, (__pyx_v_node_log_min_bounds[__pyx_v_i1]));

      /* "binary_tree.pxi":2213
 *                 global_log_min_bound = logaddexp(global_log_min_bound,
 *                                                  node_log_min_bounds[i1])
 *                 global_log_min_bound = logaddexp(global_log_min_bound,             # <<<<<<<<<<<<<<
 *                                                  node_log_min_bounds[i2])
 * 
 */
      __pyx_v_global_log_min_bound = __pyx_f_7sklearn_9neighbors_7kd_tree_logaddexp(__pyx_v_global_log_min_bound, (__pyx_v_node_log_min_bounds[__pyx_v_i2]));

      /* "binary_tree.pxi":2217
 * 
 *                 global_log_bound_spread =\
 *                     logsubexp(global_log_bound_spread,             # <<<<<<<<<<<<<<
 *                               node_log_bound_spreads[i_node])
 *                 global_log_bound_spread = logaddexp(global_log_bound_spread,
 */
      __pyx_v_global_log_bound_spread = __pyx_f_7sklearn_9neighbors_7kd_tree_logsubexp(__pyx_v_global_log_bound_spread, (__pyx_v_node_log_bound_spreads[__pyx_v_i_node]));

      /* "binary_tree.pxi":2219
 *                     logsubexp(global_log_bound_spread,
 *                               node_log_bound_spreads[i_node])
 *                 global_log_bound_spread = logaddexp(global_log_bound_spread,             # <<<<<<<<<<<<<<
 *                                                     node_log_bound_spreads[i1])
 *                 global_log_bound_spread = logaddexp(global_log_bound_spread,
 */
      __pyx_v_global_log_bound_spread = __pyx_f_7sklearn_9neighbors_7kd_tree_logaddexp(__pyx_v_global_log_bound_spread, (__pyx_v_node_log_bound_spreads[__pyx_v_i1]));

      /* "binary_tree.pxi":2221
 *                 global_log_bound_spread = logaddexp(global_log_bound_spread,
 *                                                     node_log_bound_spreads[i1])
 *                 global_log_bound_spread = logaddexp(global_log_bound_spread,             # <<<<<<<<<<<<<<
 *                                                     node_log_bound_spreads[i2])
 * 
 */
      __pyx_v_global_log_bound_spread = __pyx_f_7sklearn_9neighbors_7kd_tree_logaddexp(__pyx_v_global_log_bound_spread, (__pyx_v_node_log_bound_spreads[__pyx_v_i2]));

      /* "binary_tree.pxi":2225
 * 
 *                 # TODO: rank by the spread rather than the distance?
 *                 nodeheap_item.val = dist_LB_1             # <<<<<<<<<<<<<<
 *                 nodeheap_item.i1 = i1
 *                 nodeheap.push(nodeheap_item)
 */
      __pyx_v_nodeheap_item.val = __pyx_v_dist_LB_1;

      /* "binary_tree.pxi":2226
 *                 # TODO: rank by the spread rather than the distance?
 *                 nodeheap_item.val = dist_LB_1
 *                 nodeheap_item.i1 = i1             # <<<<<<<<<<<<<<
 *                 nodeheap.push(nodeheap_item)
 * 
 */
      __pyx_v_nodeheap_item.i1 = __pyx_v_i1;

      /* "binary_tree.pxi":2227
 *                 nodeheap_item.val = dist_LB_1
 *                 nodeheap_item.i1 = i1
 *                 nodeheap.push(nodeheap_item)             # <<<<<<<<<<<<<<
 * 
 *                 nodeheap_item.val = dist_LB_2
 */
      __pyx_t_6 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_v_nodeheap->__pyx_vtab)->push(__pyx_v_nodeheap, __pyx_v_nodeheap_item); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "binary_tree.pxi":2229
 *                 nodeheap.push(nodeheap_item)
 * 
 *                 nodeheap_item.val = dist_LB_2             # <<<<<<<<<<<<<<
 *                 nodeheap_item.i1 = i2
 *                 nodeheap.push(nodeheap_item)
 */
      __pyx_v_nodeheap_item.val = __pyx_v_dist_LB_2;

      /* "binary_tree.pxi":2230
 * 
 *                 nodeheap_item.val = dist_LB_2
 *                 nodeheap_item.i1 = i2             # <<<<<<<<<<<<<<
 *                 nodeheap.push(nodeheap_item)
 * 
 */
      __pyx_v_nodeheap_item.i1 = __pyx_v_i2;

      /* "binary_tree.pxi":2231
 *                 nodeheap_item.val = dist_LB_2
 *                 nodeheap_item.i1 = i2
 *                 nodeheap.push(nodeheap_item)             # <<<<<<<<<<<<<<
 * 
 *         nodeheap.clear()
 */
      __pyx_t_6 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_v_nodeheap->__pyx_vtab)->push(__pyx_v_nodeheap, __pyx_v_nodeheap_item); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_L5:;
  }
  __pyx_L4_break:;

  /* "binary_tree.pxi":2233
 *                 nodeheap.push(nodeheap_item)
 * 
 *         nodeheap.clear()             # <<<<<<<<<<<<<<
 *         return logaddexp(global_log_min_bound,
 *                          global_log_bound_spread - log(2))
 */
  ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_v_nodeheap->__pyx_vtab)->clear(__pyx_v_nodeheap);

  /* "binary_tree.pxi":2234
 * 
 *         nodeheap.clear()
 *         return logaddexp(global_log_min_bound,             # <<<<<<<<<<<<<<
 *                          global_log_bound_spread - log(2))
 * 
 */
  __pyx_r = __pyx_f_7sklearn_9neighbors_7kd_tree_logaddexp(__pyx_v_global_log_min_bound, (__pyx_v_global_log_bound_spread - log(2.0)));
  goto __pyx_L0;

  /* "binary_tree.pxi":2097
 *         return count
 * 
 *     cdef DTYPE_t _kde_single_breadthfirst(self, DTYPE_t* pt,             # <<<<<<<<<<<<<<
 *                                           KernelType kernel, DTYPE_t h,
 *                                           DTYPE_t log_knorm,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("sklearn.neighbors.kd_tree.BinaryTree._kde_single_breadthfirst", __pyx_clineno, __pyx_lineno, __pyx_filename, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":2237
 *                          global_log_bound_spread - log(2))
 * 
 *     cdef int _kde_single_depthfirst(             # <<<<<<<<<<<<<<
 *                    self, ITYPE_t i_node, DTYPE_t* pt,
 *                    KernelType kernel, DTYPE_t h,
 */

static int __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__kde_single_depthfirst(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt, enum __pyx_t_7sklearn_9neighbors_7kd_tree_KernelType __pyx_v_kernel, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_h, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_log_knorm, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_log_atol, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_log_rtol, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_local_log_min_bound, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_local_log_bound_spread, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_global_log_min_bound, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_global_log_bound_spread) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_N1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_N2;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_data;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_v_idx_array;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_features;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t __pyx_v_node_info;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_pt;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_log_dens_contribution;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_child1_log_min_bound;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_child2_log_min_bound;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_child1_log_bound_spread;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_child2_log_bound_spread;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_UB;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_LB;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_4;
  int __pyx_t_5;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_6;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_7;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_8;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_9;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_kde_single_depthfirst", 0);

  /* "binary_tree.pxi":2254
 *         cdef ITYPE_t i, i1, i2, N1, N2
 * 
 *         cdef DTYPE_t* data = &self.data[0, 0]             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t* idx_array = &self.idx_array[0]
 *         cdef ITYPE_t n_features = self.data.shape[1]
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2254; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_data = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_self->data.data + __pyx_t_1 * __pyx_v_self->data.strides[0]) )) + __pyx_t_2)) ))));

  /* "binary_tree.pxi":2255
 * 
 *         cdef DTYPE_t* data = &self.data[0, 0]
 *         cdef ITYPE_t* idx_array = &self.idx_array[0]             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t n_features = self.data.shape[1]
 * 
 */
  if (unlikely(!__pyx_v_self->idx_array.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2255; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_3 = 0;
  __pyx_v_idx_array = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_self->idx_array.data) + __pyx_t_3)) ))));

  /* "binary_tree.pxi":2256
 *         cdef DTYPE_t* data = &self.data[0, 0]
 *         cdef ITYPE_t* idx_array = &self.idx_array[0]
 *         cdef ITYPE_t n_features = self.data.shape[1]             # <<<<<<<<<<<<<<
 * 
 *         cdef NodeData_t node_info = self.node_data[i_node]
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_n_features = (__pyx_v_self->data.shape[1]);

  /* "binary_tree.pxi":2258
 *         cdef ITYPE_t n_features = self.data.shape[1]
 * 
 *         cdef NodeData_t node_info = self.node_data[i_node]             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t dist_pt, log_dens_contribution
 * 
 */
  if (unlikely(!__pyx_v_self->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_4 = __pyx_v_i_node;
  __pyx_v_node_info = (*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_self->node_data.data) + __pyx_t_4)) )));

  /* "binary_tree.pxi":2263
 *         cdef DTYPE_t child1_log_min_bound, child2_log_min_bound
 *         cdef DTYPE_t child1_log_bound_spread, child2_log_bound_spread
 *         cdef DTYPE_t dist_UB = 0, dist_LB = 0             # <<<<<<<<<<<<<<
 * 
 *         N1 = node_info.idx_end - node_info.idx_start
 */
  __pyx_v_dist_UB = 0.0;
  __pyx_v_dist_LB = 0.0;

  /* "binary_tree.pxi":2265
 *         cdef DTYPE_t dist_UB = 0, dist_LB = 0
 * 
 *         N1 = node_info.idx_end - node_info.idx_start             # <<<<<<<<<<<<<<
 *         N2 = self.data.shape[0]
 * 
 */
  __pyx_v_N1 = (__pyx_v_node_info.idx_end - __pyx_v_node_info.idx_start);

  /* "binary_tree.pxi":2266
 * 
 *         N1 = node_info.idx_end - node_info.idx_start
 *         N2 = self.data.shape[0]             # <<<<<<<<<<<<<<
 * 
 *         #------------------------------------------------------------
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2266; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_N2 = (__pyx_v_self->data.shape[0]);

  /* "binary_tree.pxi":2271
 *         # Case 1: local bounds are equal to within errors.  Return
 *         if (log_knorm + local_log_bound_spread - log(N1) + log(N2)
 *             <= logaddexp(log_atol, (log_rtol + log_knorm             # <<<<<<<<<<<<<<
 *                                     + local_log_min_bound))):
 *             pass
 */
  __pyx_t_5 = (((((__pyx_v_log_knorm + __pyx_v_local_log_bound_spread) - log(__pyx_v_N1)) + log(__pyx_v_N2)) <= __pyx_f_7sklearn_9neighbors_7kd_tree_logaddexp(__pyx_v_log_atol, ((__pyx_v_log_rtol + __pyx_v_log_knorm) + __pyx_v_local_log_min_bound))) != 0);
  if (__pyx_t_5) {
    goto __pyx_L3;
  }

  /* "binary_tree.pxi":2278
 *         # Case 2: global bounds are within rtol & atol. Return
 *         elif (log_knorm + global_log_bound_spread[0]
 *             <= logaddexp(log_atol, (log_rtol + log_knorm             # <<<<<<<<<<<<<<
 *                                     + global_log_min_bound[0]))):
 *             pass
 */
  __pyx_t_5 = (((__pyx_v_log_knorm + (__pyx_v_global_log_bound_spread[0])) <= __pyx_f_7sklearn_9neighbors_7kd_tree_logaddexp(__pyx_v_log_atol, ((__pyx_v_log_rtol + __pyx_v_log_knorm) + (__pyx_v_global_log_min_bound[0])))) != 0);
  if (__pyx_t_5) {
    goto __pyx_L3;
  }

  /* "binary_tree.pxi":2284
 *         #------------------------------------------------------------
 *         # Case 3: node is a leaf. Count contributions from all points
 *         elif node_info.is_leaf:             # <<<<<<<<<<<<<<
 *             global_log_min_bound[0] = logsubexp(global_log_min_bound[0],
 *                                                 local_log_min_bound)
 */
  __pyx_t_5 = (__pyx_v_node_info.is_leaf != 0);
  if (__pyx_t_5) {

    /* "binary_tree.pxi":2285
 *         # Case 3: node is a leaf. Count contributions from all points
 *         elif node_info.is_leaf:
 *             global_log_min_bound[0] = logsubexp(global_log_min_bound[0],             # <<<<<<<<<<<<<<
 *                                                 local_log_min_bound)
 *             global_log_bound_spread[0] = logsubexp(global_log_bound_spread[0],
 */
    (__pyx_v_global_log_min_bound[0]) = __pyx_f_7sklearn_9neighbors_7kd_tree_logsubexp((__pyx_v_global_log_min_bound[0]), __pyx_v_local_log_min_bound);

    /* "binary_tree.pxi":2287
 *             global_log_min_bound[0] = logsubexp(global_log_min_bound[0],
 *                                                 local_log_min_bound)
 *             global_log_bound_spread[0] = logsubexp(global_log_bound_spread[0],             # <<<<<<<<<<<<<<
 *                                                    local_log_bound_spread)
 *             for i in range(node_info.idx_start, node_info.idx_end):
 */
    (__pyx_v_global_log_bound_spread[0]) = __pyx_f_7sklearn_9neighbors_7kd_tree_logsubexp((__pyx_v_global_log_bound_spread[0]), __pyx_v_local_log_bound_spread);

    /* "binary_tree.pxi":2289
 *             global_log_bound_spread[0] = logsubexp(global_log_bound_spread[0],
 *                                                    local_log_bound_spread)
 *             for i in range(node_info.idx_start, node_info.idx_end):             # <<<<<<<<<<<<<<
 *                 dist_pt = self.dist(pt, (data + n_features * idx_array[i]),
 *                                     n_features)
 */
    __pyx_t_6 = __pyx_v_node_info.idx_end;
    for (__pyx_t_7 = __pyx_v_node_info.idx_start; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
      __pyx_v_i = __pyx_t_7;

      /* "binary_tree.pxi":2290
 *                                                    local_log_bound_spread)
 *             for i in range(node_info.idx_start, node_info.idx_end):
 *                 dist_pt = self.dist(pt, (data + n_features * idx_array[i]),             # <<<<<<<<<<<<<<
 *                                     n_features)
 *                 log_dens_contribution = compute_log_kernel(dist_pt, h, kernel)
 */
      __pyx_t_8 = __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree_dist(__pyx_v_self, __pyx_v_pt, (__pyx_v_data + (__pyx_v_n_features * (__pyx_v_idx_array[__pyx_v_i]))), __pyx_v_n_features); if (unlikely(__pyx_t_8 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_v_dist_pt = __pyx_t_8;

      /* "binary_tree.pxi":2292
 *                 dist_pt = self.dist(pt, (data + n_features * idx_array[i]),
 *                                     n_features)
 *                 log_dens_contribution = compute_log_kernel(dist_pt, h, kernel)             # <<<<<<<<<<<<<<
 *                 global_log_min_bound[0] = logaddexp(global_log_min_bound[0],
 *                                                     log_dens_contribution)
 */
      __pyx_v_log_dens_contribution = __pyx_f_7sklearn_9neighbors_7kd_tree_compute_log_kernel(__pyx_v_dist_pt, __pyx_v_h, __pyx_v_kernel);

      /* "binary_tree.pxi":2293
 *                                     n_features)
 *                 log_dens_contribution = compute_log_kernel(dist_pt, h, kernel)
 *                 global_log_min_bound[0] = logaddexp(global_log_min_bound[0],             # <<<<<<<<<<<<<<
 *                                                     log_dens_contribution)
 * 
 */
      (__pyx_v_global_log_min_bound[0]) = __pyx_f_7sklearn_9neighbors_7kd_tree_logaddexp((__pyx_v_global_log_min_bound[0]), __pyx_v_log_dens_contribution);
    }
    goto __pyx_L3;
  }
  /*else*/ {

    /* "binary_tree.pxi":2299
 *         # Case 4: split node and query subnodes
 *         else:
 *             i1 = 2 * i_node + 1             # <<<<<<<<<<<<<<
 *             i2 = 2 * i_node + 2
 * 
 */
    __pyx_v_i1 = ((2 * __pyx_v_i_node) + 1);

    /* "binary_tree.pxi":2300
 *         else:
 *             i1 = 2 * i_node + 1
 *             i2 = 2 * i_node + 2             # <<<<<<<<<<<<<<
 * 
 *             N1 = self.node_data[i1].idx_end - self.node_data[i1].idx_start
 */
    __pyx_v_i2 = ((2 * __pyx_v_i_node) + 2);

    /* "binary_tree.pxi":2302
 *             i2 = 2 * i_node + 2
 * 
 *             N1 = self.node_data[i1].idx_end - self.node_data[i1].idx_start             # <<<<<<<<<<<<<<
 *             N2 = self.node_data[i2].idx_end - self.node_data[i2].idx_start
 * 
 */
    if (unlikely(!__pyx_v_self->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2302; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
    __pyx_t_6 = __pyx_v_i1;
    if (unlikely(!__pyx_v_self->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2302; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
    __pyx_t_7 = __pyx_v_i1;
    __pyx_v_N1 = ((*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_self->node_data.data) + __pyx_t_6)) ))).idx_end - (*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_self->node_data.data) + __pyx_t_7)) ))).idx_start);

    /* "binary_tree.pxi":2303
 * 
 *             N1 = self.node_data[i1].idx_end - self.node_data[i1].idx_start
 *             N2 = self.node_data[i2].idx_end - self.node_data[i2].idx_start             # <<<<<<<<<<<<<<
 * 
 *             min_max_dist(self, i1, pt, &dist_LB, &dist_UB)
 */
    if (unlikely(!__pyx_v_self->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
    __pyx_t_9 = __pyx_v_i2;
    if (unlikely(!__pyx_v_self->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
    __pyx_t_10 = __pyx_v_i2;
    __pyx_v_N2 = ((*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_self->node_data.data) + __pyx_t_9)) ))).idx_end - (*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_self->node_data.data) + __pyx_t_10)) ))).idx_start);

    /* "binary_tree.pxi":2305
 *             N2 = self.node_data[i2].idx_end - self.node_data[i2].idx_start
 * 
 *             min_max_dist(self, i1, pt, &dist_LB, &dist_UB)             # <<<<<<<<<<<<<<
 *             child1_log_min_bound = log(N1) + compute_log_kernel(dist_UB, h,
 *                                                                 kernel)
 */
    __pyx_t_11 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_max_dist(__pyx_v_self, __pyx_v_i1, __pyx_v_pt, (&__pyx_v_dist_LB), (&__pyx_v_dist_UB)); if (unlikely(__pyx_t_11 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2305; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "binary_tree.pxi":2306
 * 
 *             min_max_dist(self, i1, pt, &dist_LB, &dist_UB)
 *             child1_log_min_bound = log(N1) + compute_log_kernel(dist_UB, h,             # <<<<<<<<<<<<<<
 *                                                                 kernel)
 *             child1_log_bound_spread = logsubexp(log(N1) +
 */
    __pyx_v_child1_log_min_bound = (log(__pyx_v_N1) + __pyx_f_7sklearn_9neighbors_7kd_tree_compute_log_kernel(__pyx_v_dist_UB, __pyx_v_h, __pyx_v_kernel));

    /* "binary_tree.pxi":2308
 *             child1_log_min_bound = log(N1) + compute_log_kernel(dist_UB, h,
 *                                                                 kernel)
 *             child1_log_bound_spread = logsubexp(log(N1) +             # <<<<<<<<<<<<<<
 *                                                 compute_log_kernel(dist_LB, h,
 *                                                                    kernel),
 */
    __pyx_v_child1_log_bound_spread = __pyx_f_7sklearn_9neighbors_7kd_tree_logsubexp((log(__pyx_v_N1) + __pyx_f_7sklearn_9neighbors_7kd_tree_compute_log_kernel(__pyx_v_dist_LB, __pyx_v_h, __pyx_v_kernel)), __pyx_v_child1_log_min_bound);

    /* "binary_tree.pxi":2313
 *                                                 child1_log_min_bound)
 * 
 *             min_max_dist(self, i2, pt, &dist_LB, &dist_UB)             # <<<<<<<<<<<<<<
 *             child2_log_min_bound = log(N2) + compute_log_kernel(dist_UB, h,
 *                                                                 kernel)
 */
    __pyx_t_11 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_max_dist(__pyx_v_self, __pyx_v_i2, __pyx_v_pt, (&__pyx_v_dist_LB), (&__pyx_v_dist_UB)); if (unlikely(__pyx_t_11 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "binary_tree.pxi":2314
 * 
 *             min_max_dist(self, i2, pt, &dist_LB, &dist_UB)
 *             child2_log_min_bound = log(N2) + compute_log_kernel(dist_UB, h,             # <<<<<<<<<<<<<<
 *                                                                 kernel)
 *             child2_log_bound_spread = logsubexp(log(N2) +
 */
    __pyx_v_child2_log_min_bound = (log(__pyx_v_N2) + __pyx_f_7sklearn_9neighbors_7kd_tree_compute_log_kernel(__pyx_v_dist_UB, __pyx_v_h, __pyx_v_kernel));

    /* "binary_tree.pxi":2316
 *             child2_log_min_bound = log(N2) + compute_log_kernel(dist_UB, h,
 *                                                                 kernel)
 *             child2_log_bound_spread = logsubexp(log(N2) +             # <<<<<<<<<<<<<<
 *                                                 compute_log_kernel(dist_LB, h,
 *                                                                    kernel),
 */
    __pyx_v_child2_log_bound_spread = __pyx_f_7sklearn_9neighbors_7kd_tree_logsubexp((log(__pyx_v_N2) + __pyx_f_7sklearn_9neighbors_7kd_tree_compute_log_kernel(__pyx_v_dist_LB, __pyx_v_h, __pyx_v_kernel)), __pyx_v_child2_log_min_bound);

    /* "binary_tree.pxi":2321
 *                                                 child2_log_min_bound)
 * 
 *             global_log_min_bound[0] = logsubexp(global_log_min_bound[0],             # <<<<<<<<<<<<<<
 *                                                 local_log_min_bound)
 *             global_log_min_bound[0] = logaddexp(global_log_min_bound[0],
 */
    (__pyx_v_global_log_min_bound[0]) = __pyx_f_7sklearn_9neighbors_7kd_tree_logsubexp((__pyx_v_global_log_min_bound[0]), __pyx_v_local_log_min_bound);

    /* "binary_tree.pxi":2323
 *             global_log_min_bound[0] = logsubexp(global_log_min_bound[0],
 *                                                 local_log_min_bound)
 *             global_log_min_bound[0] = logaddexp(global_log_min_bound[0],             # <<<<<<<<<<<<<<
 *                                                 child1_log_min_bound)
 *             global_log_min_bound[0] = logaddexp(global_log_min_bound[0],
 */
    (__pyx_v_global_log_min_bound[0]) = __pyx_f_7sklearn_9neighbors_7kd_tree_logaddexp((__pyx_v_global_log_min_bound[0]), __pyx_v_child1_log_min_bound);

    /* "binary_tree.pxi":2325
 *             global_log_min_bound[0] = logaddexp(global_log_min_bound[0],
 *                                                 child1_log_min_bound)
 *             global_log_min_bound[0] = logaddexp(global_log_min_bound[0],             # <<<<<<<<<<<<<<
 *                                                 child2_log_min_bound)
 * 
 */
    (__pyx_v_global_log_min_bound[0]) = __pyx_f_7sklearn_9neighbors_7kd_tree_logaddexp((__pyx_v_global_log_min_bound[0]), __pyx_v_child2_log_min_bound);

    /* "binary_tree.pxi":2328
 *                                                 child2_log_min_bound)
 * 
 *             global_log_bound_spread[0] = logsubexp(global_log_bound_spread[0],             # <<<<<<<<<<<<<<
 *                                                    local_log_bound_spread)
 *             global_log_bound_spread[0] = logaddexp(global_log_bound_spread[0],
 */
    (__pyx_v_global_log_bound_spread[0]) = __pyx_f_7sklearn_9neighbors_7kd_tree_logsubexp((__pyx_v_global_log_bound_spread[0]), __pyx_v_local_log_bound_spread);

    /* "binary_tree.pxi":2330
 *             global_log_bound_spread[0] = logsubexp(global_log_bound_spread[0],
 *                                                    local_log_bound_spread)
 *             global_log_bound_spread[0] = logaddexp(global_log_bound_spread[0],             # <<<<<<<<<<<<<<
 *                                                    child1_log_bound_spread)
 *             global_log_bound_spread[0] = logaddexp(global_log_bound_spread[0],
 */
    (__pyx_v_global_log_bound_spread[0]) = __pyx_f_7sklearn_9neighbors_7kd_tree_logaddexp((__pyx_v_global_log_bound_spread[0]), __pyx_v_child1_log_bound_spread);

    /* "binary_tree.pxi":2332
 *             global_log_bound_spread[0] = logaddexp(global_log_bound_spread[0],
 *                                                    child1_log_bound_spread)
 *             global_log_bound_spread[0] = logaddexp(global_log_bound_spread[0],             # <<<<<<<<<<<<<<
 *                                                    child2_log_bound_spread)
 * 
 */
    (__pyx_v_global_log_bound_spread[0]) = __pyx_f_7sklearn_9neighbors_7kd_tree_logaddexp((__pyx_v_global_log_bound_spread[0]), __pyx_v_child2_log_bound_spread);

    /* "binary_tree.pxi":2335
 *                                                    child2_log_bound_spread)
 * 
 *             self._kde_single_depthfirst(i1, pt, kernel, h, log_knorm,             # <<<<<<<<<<<<<<
 *                                         log_atol, log_rtol,
 *                                         child1_log_min_bound,
 */
    __pyx_t_11 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_kde_single_depthfirst(__pyx_v_self, __pyx_v_i1, __pyx_v_pt, __pyx_v_kernel, __pyx_v_h, __pyx_v_log_knorm, __pyx_v_log_atol, __pyx_v_log_rtol, __pyx_v_child1_log_min_bound, __pyx_v_child1_log_bound_spread, __pyx_v_global_log_min_bound, __pyx_v_global_log_bound_spread); if (unlikely(__pyx_t_11 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2335; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "binary_tree.pxi":2341
 *                                         global_log_min_bound,
 *                                         global_log_bound_spread)
 *             self._kde_single_depthfirst(i2, pt, kernel, h, log_knorm,             # <<<<<<<<<<<<<<
 *                                         log_atol, log_rtol,
 *                                         child2_log_min_bound,
 */
    __pyx_t_11 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_kde_single_depthfirst(__pyx_v_self, __pyx_v_i2, __pyx_v_pt, __pyx_v_kernel, __pyx_v_h, __pyx_v_log_knorm, __pyx_v_log_atol, __pyx_v_log_rtol, __pyx_v_child2_log_min_bound, __pyx_v_child2_log_bound_spread, __pyx_v_global_log_min_bound, __pyx_v_global_log_bound_spread); if (unlikely(__pyx_t_11 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2341; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L3:;

  /* "binary_tree.pxi":2347
 *                                         global_log_min_bound,
 *                                         global_log_bound_spread)
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     cdef int _two_point_single(self, ITYPE_t i_node, DTYPE_t* pt, DTYPE_t* r,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":2237
 *                          global_log_bound_spread - log(2))
 * 
 *     cdef int _kde_single_depthfirst(             # <<<<<<<<<<<<<<
 *                    self, ITYPE_t i_node, DTYPE_t* pt,
 *                    KernelType kernel, DTYPE_t h,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree._kde_single_depthfirst", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":2349
 *         return 0
 * 
 *     cdef int _two_point_single(self, ITYPE_t i_node, DTYPE_t* pt, DTYPE_t* r,             # <<<<<<<<<<<<<<
 *                                ITYPE_t* count, ITYPE_t i_min,
 *                                ITYPE_t i_max) except -1:
 */

static int __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__two_point_single(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_r, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_v_count, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_min, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_max) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_data;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_v_idx_array;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_features;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t __pyx_v_node_info;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_j;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_Npts;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_pt;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_LB;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_UB;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  long __pyx_t_7;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_8;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_9;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_10;
  int __pyx_t_11;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_two_point_single", 0);

  /* "binary_tree.pxi":2353
 *                                ITYPE_t i_max) except -1:
 *         """recursive single-tree two-point correlation function query"""
 *         cdef DTYPE_t* data = &self.data[0, 0]             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t* idx_array = &self.idx_array[0]
 *         cdef ITYPE_t n_features = self.data.shape[1]
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2353; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_data = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_self->data.data + __pyx_t_1 * __pyx_v_self->data.strides[0]) )) + __pyx_t_2)) ))));

  /* "binary_tree.pxi":2354
 *         """recursive single-tree two-point correlation function query"""
 *         cdef DTYPE_t* data = &self.data[0, 0]
 *         cdef ITYPE_t* idx_array = &self.idx_array[0]             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t n_features = self.data.shape[1]
 *         cdef NodeData_t node_info = self.node_data[i_node]
 */
  if (unlikely(!__pyx_v_self->idx_array.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2354; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_3 = 0;
  __pyx_v_idx_array = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_self->idx_array.data) + __pyx_t_3)) ))));

  /* "binary_tree.pxi":2355
 *         cdef DTYPE_t* data = &self.data[0, 0]
 *         cdef ITYPE_t* idx_array = &self.idx_array[0]
 *         cdef ITYPE_t n_features = self.data.shape[1]             # <<<<<<<<<<<<<<
 *         cdef NodeData_t node_info = self.node_data[i_node]
 * 
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2355; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_n_features = (__pyx_v_self->data.shape[1]);

  /* "binary_tree.pxi":2356
 *         cdef ITYPE_t* idx_array = &self.idx_array[0]
 *         cdef ITYPE_t n_features = self.data.shape[1]
 *         cdef NodeData_t node_info = self.node_data[i_node]             # <<<<<<<<<<<<<<
 * 
 *         cdef ITYPE_t i, j, Npts
 */
  if (unlikely(!__pyx_v_self->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2356; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_4 = __pyx_v_i_node;
  __pyx_v_node_info = (*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_self->node_data.data) + __pyx_t_4)) )));

  /* "binary_tree.pxi":2361
 *         cdef DTYPE_t reduced_r
 * 
 *         cdef DTYPE_t dist_pt, dist_LB = 0, dist_UB = 0             # <<<<<<<<<<<<<<
 *         min_max_dist(self, i_node, pt, &dist_LB, &dist_UB)
 * 
 */
  __pyx_v_dist_LB = 0.0;
  __pyx_v_dist_UB = 0.0;

  /* "binary_tree.pxi":2362
 * 
 *         cdef DTYPE_t dist_pt, dist_LB = 0, dist_UB = 0
 *         min_max_dist(self, i_node, pt, &dist_LB, &dist_UB)             # <<<<<<<<<<<<<<
 * 
 *         #------------------------------------------------------------
 */
  __pyx_t_5 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_max_dist(__pyx_v_self, __pyx_v_i_node, __pyx_v_pt, (&__pyx_v_dist_LB), (&__pyx_v_dist_UB)); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2362; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "binary_tree.pxi":2366
 *         #------------------------------------------------------------
 *         # Go through bounds and check for cuts
 *         while i_min < i_max:             # <<<<<<<<<<<<<<
 *             if dist_LB > r[i_min]:
 *                 i_min += 1
 */
  while (1) {
    __pyx_t_6 = ((__pyx_v_i_min < __pyx_v_i_max) != 0);
    if (!__pyx_t_6) break;

    /* "binary_tree.pxi":2367
 *         # Go through bounds and check for cuts
 *         while i_min < i_max:
 *             if dist_LB > r[i_min]:             # <<<<<<<<<<<<<<
 *                 i_min += 1
 *             else:
 */
    __pyx_t_6 = ((__pyx_v_dist_LB > (__pyx_v_r[__pyx_v_i_min])) != 0);
    if (__pyx_t_6) {

      /* "binary_tree.pxi":2368
 *         while i_min < i_max:
 *             if dist_LB > r[i_min]:
 *                 i_min += 1             # <<<<<<<<<<<<<<
 *             else:
 *                 break
 */
      __pyx_v_i_min = (__pyx_v_i_min + 1);
      goto __pyx_L5;
    }
    /*else*/ {

      /* "binary_tree.pxi":2370
 *                 i_min += 1
 *             else:
 *                 break             # <<<<<<<<<<<<<<
 * 
 *         while i_max > i_min:
 */
      goto __pyx_L4_break;
    }
    __pyx_L5:;
  }
  __pyx_L4_break:;

  /* "binary_tree.pxi":2372
 *                 break
 * 
 *         while i_max > i_min:             # <<<<<<<<<<<<<<
 *             Npts = (node_info.idx_end - node_info.idx_start)
 *             if dist_UB <= r[i_max - 1]:
 */
  while (1) {
    __pyx_t_6 = ((__pyx_v_i_max > __pyx_v_i_min) != 0);
    if (!__pyx_t_6) break;

    /* "binary_tree.pxi":2373
 * 
 *         while i_max > i_min:
 *             Npts = (node_info.idx_end - node_info.idx_start)             # <<<<<<<<<<<<<<
 *             if dist_UB <= r[i_max - 1]:
 *                 count[i_max - 1] += Npts
 */
    __pyx_v_Npts = (__pyx_v_node_info.idx_end - __pyx_v_node_info.idx_start);

    /* "binary_tree.pxi":2374
 *         while i_max > i_min:
 *             Npts = (node_info.idx_end - node_info.idx_start)
 *             if dist_UB <= r[i_max - 1]:             # <<<<<<<<<<<<<<
 *                 count[i_max - 1] += Npts
 *                 i_max -= 1
 */
    __pyx_t_6 = ((__pyx_v_dist_UB <= (__pyx_v_r[(__pyx_v_i_max - 1)])) != 0);
    if (__pyx_t_6) {

      /* "binary_tree.pxi":2375
 *             Npts = (node_info.idx_end - node_info.idx_start)
 *             if dist_UB <= r[i_max - 1]:
 *                 count[i_max - 1] += Npts             # <<<<<<<<<<<<<<
 *                 i_max -= 1
 *             else:
 */
      __pyx_t_7 = (__pyx_v_i_max - 1);
      (__pyx_v_count[__pyx_t_7]) = ((__pyx_v_count[__pyx_t_7]) + __pyx_v_Npts);

      /* "binary_tree.pxi":2376
 *             if dist_UB <= r[i_max - 1]:
 *                 count[i_max - 1] += Npts
 *                 i_max -= 1             # <<<<<<<<<<<<<<
 *             else:
 *                 break
 */
      __pyx_v_i_max = (__pyx_v_i_max - 1);
      goto __pyx_L8;
    }
    /*else*/ {

      /* "binary_tree.pxi":2378
 *                 i_max -= 1
 *             else:
 *                 break             # <<<<<<<<<<<<<<
 * 
 *         if i_min < i_max:
 */
      goto __pyx_L7_break;
    }
    __pyx_L8:;
  }
  __pyx_L7_break:;

  /* "binary_tree.pxi":2380
 *                 break
 * 
 *         if i_min < i_max:             # <<<<<<<<<<<<<<
 *             # If node is a leaf, go through all points
 *             if node_info.is_leaf:
 */
  __pyx_t_6 = ((__pyx_v_i_min < __pyx_v_i_max) != 0);
  if (__pyx_t_6) {

    /* "binary_tree.pxi":2382
 *         if i_min < i_max:
 *             # If node is a leaf, go through all points
 *             if node_info.is_leaf:             # <<<<<<<<<<<<<<
 *                 for i in range(node_info.idx_start, node_info.idx_end):
 *                     dist_pt = self.dist(pt, (data + n_features * idx_array[i]),
 */
    __pyx_t_6 = (__pyx_v_node_info.is_leaf != 0);
    if (__pyx_t_6) {

      /* "binary_tree.pxi":2383
 *             # If node is a leaf, go through all points
 *             if node_info.is_leaf:
 *                 for i in range(node_info.idx_start, node_info.idx_end):             # <<<<<<<<<<<<<<
 *                     dist_pt = self.dist(pt, (data + n_features * idx_array[i]),
 *                                         n_features)
 */
      __pyx_t_8 = __pyx_v_node_info.idx_end;
      for (__pyx_t_9 = __pyx_v_node_info.idx_start; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
        __pyx_v_i = __pyx_t_9;

        /* "binary_tree.pxi":2384
 *             if node_info.is_leaf:
 *                 for i in range(node_info.idx_start, node_info.idx_end):
 *                     dist_pt = self.dist(pt, (data + n_features * idx_array[i]),             # <<<<<<<<<<<<<<
 *                                         n_features)
 *                     j = i_max - 1
 */
        __pyx_t_10 = __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree_dist(__pyx_v_self, __pyx_v_pt, (__pyx_v_data + (__pyx_v_n_features * (__pyx_v_idx_array[__pyx_v_i]))), __pyx_v_n_features); if (unlikely(__pyx_t_10 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2384; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_v_dist_pt = __pyx_t_10;

        /* "binary_tree.pxi":2386
 *                     dist_pt = self.dist(pt, (data + n_features * idx_array[i]),
 *                                         n_features)
 *                     j = i_max - 1             # <<<<<<<<<<<<<<
 *                     while (j >= i_min) and (dist_pt <= r[j]):
 *                         count[j] += 1
 */
        __pyx_v_j = (__pyx_v_i_max - 1);

        /* "binary_tree.pxi":2387
 *                                         n_features)
 *                     j = i_max - 1
 *                     while (j >= i_min) and (dist_pt <= r[j]):             # <<<<<<<<<<<<<<
 *                         count[j] += 1
 *                         j -= 1
 */
        while (1) {
          __pyx_t_11 = ((__pyx_v_j >= __pyx_v_i_min) != 0);
          if (__pyx_t_11) {
          } else {
            __pyx_t_6 = __pyx_t_11;
            goto __pyx_L15_bool_binop_done;
          }
          __pyx_t_11 = ((__pyx_v_dist_pt <= (__pyx_v_r[__pyx_v_j])) != 0);
          __pyx_t_6 = __pyx_t_11;
          __pyx_L15_bool_binop_done:;
          if (!__pyx_t_6) break;

          /* "binary_tree.pxi":2388
 *                     j = i_max - 1
 *                     while (j >= i_min) and (dist_pt <= r[j]):
 *                         count[j] += 1             # <<<<<<<<<<<<<<
 *                         j -= 1
 * 
 */
          __pyx_t_12 = __pyx_v_j;
          (__pyx_v_count[__pyx_t_12]) = ((__pyx_v_count[__pyx_t_12]) + 1);

          /* "binary_tree.pxi":2389
 *                     while (j >= i_min) and (dist_pt <= r[j]):
 *                         count[j] += 1
 *                         j -= 1             # <<<<<<<<<<<<<<
 * 
 *             else:
 */
          __pyx_v_j = (__pyx_v_j - 1);
        }
      }
      goto __pyx_L10;
    }
    /*else*/ {

      /* "binary_tree.pxi":2392
 * 
 *             else:
 *                 self._two_point_single(2 * i_node + 1, pt, r,             # <<<<<<<<<<<<<<
 *                                        count, i_min, i_max)
 *                 self._two_point_single(2 * i_node + 2, pt, r,
 */
      __pyx_t_5 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_two_point_single(__pyx_v_self, ((2 * __pyx_v_i_node) + 1), __pyx_v_pt, __pyx_v_r, __pyx_v_count, __pyx_v_i_min, __pyx_v_i_max); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2392; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "binary_tree.pxi":2394
 *                 self._two_point_single(2 * i_node + 1, pt, r,
 *                                        count, i_min, i_max)
 *                 self._two_point_single(2 * i_node + 2, pt, r,             # <<<<<<<<<<<<<<
 *                                        count, i_min, i_max)
 *         return 0
 */
      __pyx_t_5 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_two_point_single(__pyx_v_self, ((2 * __pyx_v_i_node) + 2), __pyx_v_pt, __pyx_v_r, __pyx_v_count, __pyx_v_i_min, __pyx_v_i_max); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2394; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_L10:;
    goto __pyx_L9;
  }
  __pyx_L9:;

  /* "binary_tree.pxi":2396
 *                 self._two_point_single(2 * i_node + 2, pt, r,
 *                                        count, i_min, i_max)
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     cdef int _two_point_dual(self, ITYPE_t i_node1,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":2349
 *         return 0
 * 
 *     cdef int _two_point_single(self, ITYPE_t i_node, DTYPE_t* pt, DTYPE_t* r,             # <<<<<<<<<<<<<<
 *                                ITYPE_t* count, ITYPE_t i_min,
 *                                ITYPE_t i_max) except -1:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree._two_point_single", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":2398
 *         return 0
 * 
 *     cdef int _two_point_dual(self, ITYPE_t i_node1,             # <<<<<<<<<<<<<<
 *                              BinaryTree other, ITYPE_t i_node2,
 *                              DTYPE_t* r, ITYPE_t* count,
 */

static int __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__two_point_dual(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node1, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_other, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node2, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_r, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_v_count, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_min, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_max) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_data1;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_data2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_v_idx_array1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_v_idx_array2;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t __pyx_v_node_info1;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t __pyx_v_node_info2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_features;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_j;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_Npts;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_pt;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_LB;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist_UB;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_7;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_8;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_9;
  int __pyx_t_10;
  long __pyx_t_11;
  int __pyx_t_12;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_13;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_14;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_15;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_16;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_17;
  int __pyx_t_18;
  long __pyx_t_19;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_two_point_dual", 0);

  /* "binary_tree.pxi":2403
 *                              ITYPE_t i_min, ITYPE_t i_max) except -1:
 *         """recursive dual-tree two-point correlation function query"""
 *         cdef DTYPE_t* data1 = &self.data[0, 0]             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t* data2 = &other.data[0, 0]
 *         cdef ITYPE_t* idx_array1 = &self.idx_array[0]
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2403; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_data1 = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_self->data.data + __pyx_t_1 * __pyx_v_self->data.strides[0]) )) + __pyx_t_2)) ))));

  /* "binary_tree.pxi":2404
 *         """recursive dual-tree two-point correlation function query"""
 *         cdef DTYPE_t* data1 = &self.data[0, 0]
 *         cdef DTYPE_t* data2 = &other.data[0, 0]             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t* idx_array1 = &self.idx_array[0]
 *         cdef ITYPE_t* idx_array2 = &other.idx_array[0]
 */
  if (unlikely(!__pyx_v_other->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2404; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_v_data2 = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_other->data.data + __pyx_t_3 * __pyx_v_other->data.strides[0]) )) + __pyx_t_4)) ))));

  /* "binary_tree.pxi":2405
 *         cdef DTYPE_t* data1 = &self.data[0, 0]
 *         cdef DTYPE_t* data2 = &other.data[0, 0]
 *         cdef ITYPE_t* idx_array1 = &self.idx_array[0]             # <<<<<<<<<<<<<<
 *         cdef ITYPE_t* idx_array2 = &other.idx_array[0]
 *         cdef NodeData_t node_info1 = self.node_data[i_node1]
 */
  if (unlikely(!__pyx_v_self->idx_array.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2405; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_5 = 0;
  __pyx_v_idx_array1 = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_self->idx_array.data) + __pyx_t_5)) ))));

  /* "binary_tree.pxi":2406
 *         cdef DTYPE_t* data2 = &other.data[0, 0]
 *         cdef ITYPE_t* idx_array1 = &self.idx_array[0]
 *         cdef ITYPE_t* idx_array2 = &other.idx_array[0]             # <<<<<<<<<<<<<<
 *         cdef NodeData_t node_info1 = self.node_data[i_node1]
 *         cdef NodeData_t node_info2 = other.node_data[i_node2]
 */
  if (unlikely(!__pyx_v_other->idx_array.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2406; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_6 = 0;
  __pyx_v_idx_array2 = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_other->idx_array.data) + __pyx_t_6)) ))));

  /* "binary_tree.pxi":2407
 *         cdef ITYPE_t* idx_array1 = &self.idx_array[0]
 *         cdef ITYPE_t* idx_array2 = &other.idx_array[0]
 *         cdef NodeData_t node_info1 = self.node_data[i_node1]             # <<<<<<<<<<<<<<
 *         cdef NodeData_t node_info2 = other.node_data[i_node2]
 * 
 */
  if (unlikely(!__pyx_v_self->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2407; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_7 = __pyx_v_i_node1;
  __pyx_v_node_info1 = (*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_self->node_data.data) + __pyx_t_7)) )));

  /* "binary_tree.pxi":2408
 *         cdef ITYPE_t* idx_array2 = &other.idx_array[0]
 *         cdef NodeData_t node_info1 = self.node_data[i_node1]
 *         cdef NodeData_t node_info2 = other.node_data[i_node2]             # <<<<<<<<<<<<<<
 * 
 *         cdef ITYPE_t n_features = self.data.shape[1]
 */
  if (unlikely(!__pyx_v_other->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2408; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_8 = __pyx_v_i_node2;
  __pyx_v_node_info2 = (*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_other->node_data.data) + __pyx_t_8)) )));

  /* "binary_tree.pxi":2410
 *         cdef NodeData_t node_info2 = other.node_data[i_node2]
 * 
 *         cdef ITYPE_t n_features = self.data.shape[1]             # <<<<<<<<<<<<<<
 * 
 *         cdef ITYPE_t i1, i2, j, Npts
 */
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 2410; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_n_features = (__pyx_v_self->data.shape[1]);

  /* "binary_tree.pxi":2415
 *         cdef DTYPE_t reduced_r
 * 
 *         cdef DTYPE_t dist_pt, dist_LB = 0, dist_UB = 0             # <<<<<<<<<<<<<<
 *         dist_LB = min_dist_dual(self, i_node1, other, i_node2)
 *         dist_UB = max_dist_dual(self, i_node1, other, i_node2)
 */
  __pyx_v_dist_LB = 0.0;
  __pyx_v_dist_UB = 0.0;

  /* "binary_tree.pxi":2416
 * 
 *         cdef DTYPE_t dist_pt, dist_LB = 0, dist_UB = 0
 *         dist_LB = min_dist_dual(self, i_node1, other, i_node2)             # <<<<<<<<<<<<<<
 *         dist_UB = max_dist_dual(self, i_node1, other, i_node2)
 * 
 */
  __pyx_t_9 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_dist_dual(__pyx_v_self, __pyx_v_i_node1, __pyx_v_other, __pyx_v_i_node2); if (unlikely(__pyx_t_9 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_dist_LB = __pyx_t_9;

  /* "binary_tree.pxi":2417
 *         cdef DTYPE_t dist_pt, dist_LB = 0, dist_UB = 0
 *         dist_LB = min_dist_dual(self, i_node1, other, i_node2)
 *         dist_UB = max_dist_dual(self, i_node1, other, i_node2)             # <<<<<<<<<<<<<<
 * 
 *         #------------------------------------------------------------
 */
  __pyx_t_9 = __pyx_f_7sklearn_9neighbors_7kd_tree_max_dist_dual(__pyx_v_self, __pyx_v_i_node1, __pyx_v_other, __pyx_v_i_node2); if (unlikely(__pyx_t_9 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_dist_UB = __pyx_t_9;

  /* "binary_tree.pxi":2421
 *         #------------------------------------------------------------
 *         # Go through bounds and check for cuts
 *         while i_min < i_max:             # <<<<<<<<<<<<<<
 *             if dist_LB > r[i_min]:
 *                 i_min += 1
 */
  while (1) {
    __pyx_t_10 = ((__pyx_v_i_min < __pyx_v_i_max) != 0);
    if (!__pyx_t_10) break;

    /* "binary_tree.pxi":2422
 *         # Go through bounds and check for cuts
 *         while i_min < i_max:
 *             if dist_LB > r[i_min]:             # <<<<<<<<<<<<<<
 *                 i_min += 1
 *             else:
 */
    __pyx_t_10 = ((__pyx_v_dist_LB > (__pyx_v_r[__pyx_v_i_min])) != 0);
    if (__pyx_t_10) {

      /* "binary_tree.pxi":2423
 *         while i_min < i_max:
 *             if dist_LB > r[i_min]:
 *                 i_min += 1             # <<<<<<<<<<<<<<
 *             else:
 *                 break
 */
      __pyx_v_i_min = (__pyx_v_i_min + 1);
      goto __pyx_L5;
    }
    /*else*/ {

      /* "binary_tree.pxi":2425
 *                 i_min += 1
 *             else:
 *                 break             # <<<<<<<<<<<<<<
 * 
 *         while i_max > i_min:
 */
      goto __pyx_L4_break;
    }
    __pyx_L5:;
  }
  __pyx_L4_break:;

  /* "binary_tree.pxi":2427
 *                 break
 * 
 *         while i_max > i_min:             # <<<<<<<<<<<<<<
 *             Npts = ((node_info1.idx_end - node_info1.idx_start)
 *                     * (node_info2.idx_end - node_info2.idx_start))
 */
  while (1) {
    __pyx_t_10 = ((__pyx_v_i_max > __pyx_v_i_min) != 0);
    if (!__pyx_t_10) break;

    /* "binary_tree.pxi":2429
 *         while i_max > i_min:
 *             Npts = ((node_info1.idx_end - node_info1.idx_start)
 *                     * (node_info2.idx_end - node_info2.idx_start))             # <<<<<<<<<<<<<<
 *             if dist_UB <= r[i_max - 1]:
 *                 count[i_max - 1] += Npts
 */
    __pyx_v_Npts = ((__pyx_v_node_info1.idx_end - __pyx_v_node_info1.idx_start) * (__pyx_v_node_info2.idx_end - __pyx_v_node_info2.idx_start));

    /* "binary_tree.pxi":2430
 *             Npts = ((node_info1.idx_end - node_info1.idx_start)
 *                     * (node_info2.idx_end - node_info2.idx_start))
 *             if dist_UB <= r[i_max - 1]:             # <<<<<<<<<<<<<<
 *                 count[i_max - 1] += Npts
 *                 i_max -= 1
 */
    __pyx_t_10 = ((__pyx_v_dist_UB <= (__pyx_v_r[(__pyx_v_i_max - 1)])) != 0);
    if (__pyx_t_10) {

      /* "binary_tree.pxi":2431
 *                     * (node_info2.idx_end - node_info2.idx_start))
 *             if dist_UB <= r[i_max - 1]:
 *                 count[i_max - 1] += Npts             # <<<<<<<<<<<<<<
 *                 i_max -= 1
 *             else:
 */
      __pyx_t_11 = (__pyx_v_i_max - 1);
      (__pyx_v_count[__pyx_t_11]) = ((__pyx_v_count[__pyx_t_11]) + __pyx_v_Npts);

      /* "binary_tree.pxi":2432
 *             if dist_UB <= r[i_max - 1]:
 *                 count[i_max - 1] += Npts
 *                 i_max -= 1             # <<<<<<<<<<<<<<
 *             else:
 *                 break
 */
      __pyx_v_i_max = (__pyx_v_i_max - 1);
      goto __pyx_L8;
    }
    /*else*/ {

      /* "binary_tree.pxi":2434
 *                 i_max -= 1
 *             else:
 *                 break             # <<<<<<<<<<<<<<
 * 
 *         if i_min < i_max:
 */
      goto __pyx_L7_break;
    }
    __pyx_L8:;
  }
  __pyx_L7_break:;

  /* "binary_tree.pxi":2436
 *                 break
 * 
 *         if i_min < i_max:             # <<<<<<<<<<<<<<
 *             if node_info1.is_leaf and node_info2.is_leaf:
 *                 # If both nodes are leaves, go through all points
 */
  __pyx_t_10 = ((__pyx_v_i_min < __pyx_v_i_max) != 0);
  if (__pyx_t_10) {

    /* "binary_tree.pxi":2437
 * 
 *         if i_min < i_max:
 *             if node_info1.is_leaf and node_info2.is_leaf:             # <<<<<<<<<<<<<<
 *                 # If both nodes are leaves, go through all points
 *                 for i1 in range(node_info1.idx_start, node_info1.idx_end):
 */
    __pyx_t_12 = (__pyx_v_node_info1.is_leaf != 0);
    if (__pyx_t_12) {
    } else {
      __pyx_t_10 = __pyx_t_12;
      goto __pyx_L11_bool_binop_done;
    }
    __pyx_t_12 = (__pyx_v_node_info2.is_leaf != 0);
    __pyx_t_10 = __pyx_t_12;
    __pyx_L11_bool_binop_done:;
    if (__pyx_t_10) {

      /* "binary_tree.pxi":2439
 *             if node_info1.is_leaf and node_info2.is_leaf:
 *                 # If both nodes are leaves, go through all points
 *                 for i1 in range(node_info1.idx_start, node_info1.idx_end):             # <<<<<<<<<<<<<<
 *                     for i2 in range(node_info2.idx_start, node_info2.idx_end):
 *                         dist_pt = self.dist((data1 + n_features
 */
      __pyx_t_13 = __pyx_v_node_info1.idx_end;
      for (__pyx_t_14 = __pyx_v_node_info1.idx_start; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
        __pyx_v_i1 = __pyx_t_14;

        /* "binary_tree.pxi":2440
 *                 # If both nodes are leaves, go through all points
 *                 for i1 in range(node_info1.idx_start, node_info1.idx_end):
 *                     for i2 in range(node_info2.idx_start, node_info2.idx_end):             # <<<<<<<<<<<<<<
 *                         dist_pt = self.dist((data1 + n_features
 *                                              * idx_array1[i1]),
 */
        __pyx_t_15 = __pyx_v_node_info2.idx_end;
        for (__pyx_t_16 = __pyx_v_node_info2.idx_start; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i2 = __pyx_t_16;

          /* "binary_tree.pxi":2441
 *                 for i1 in range(node_info1.idx_start, node_info1.idx_end):
 *                     for i2 in range(node_info2.idx_start, node_info2.idx_end):
 *                         dist_pt = self.dist((data1 + n_features             # <<<<<<<<<<<<<<
 *                                              * idx_array1[i1]),
 *                                             (data2 + n_features
 */
          __pyx_t_9 = __pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree_dist(__pyx_v_self, (__pyx_v_data1 + (__pyx_v_n_features * (__pyx_v_idx_array1[__pyx_v_i1]))), (__pyx_v_data2 + (__pyx_v_n_features * (__pyx_v_idx_array2[__pyx_v_i2]))), __pyx_v_n_features); if (unlikely(__pyx_t_9 == -1.0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2441; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __pyx_v_dist_pt = __pyx_t_9;

          /* "binary_tree.pxi":2446
 *                                              * idx_array2[i2]),
 *                                             n_features)
 *                         j = i_max - 1             # <<<<<<<<<<<<<<
 *                         while (j >= i_min) and (dist_pt <= r[j]):
 *                             count[j] += 1
 */
          __pyx_v_j = (__pyx_v_i_max - 1);

          /* "binary_tree.pxi":2447
 *                                             n_features)
 *                         j = i_max - 1
 *                         while (j >= i_min) and (dist_pt <= r[j]):             # <<<<<<<<<<<<<<
 *                             count[j] += 1
 *                             j -= 1
 */
          while (1) {
            __pyx_t_12 = ((__pyx_v_j >= __pyx_v_i_min) != 0);
            if (__pyx_t_12) {
            } else {
              __pyx_t_10 = __pyx_t_12;
              goto __pyx_L19_bool_binop_done;
            }
            __pyx_t_12 = ((__pyx_v_dist_pt <= (__pyx_v_r[__pyx_v_j])) != 0);
            __pyx_t_10 = __pyx_t_12;
            __pyx_L19_bool_binop_done:;
            if (!__pyx_t_10) break;

            /* "binary_tree.pxi":2448
 *                         j = i_max - 1
 *                         while (j >= i_min) and (dist_pt <= r[j]):
 *                             count[j] += 1             # <<<<<<<<<<<<<<
 *                             j -= 1
 * 
 */
            __pyx_t_17 = __pyx_v_j;
            (__pyx_v_count[__pyx_t_17]) = ((__pyx_v_count[__pyx_t_17]) + 1);

            /* "binary_tree.pxi":2449
 *                         while (j >= i_min) and (dist_pt <= r[j]):
 *                             count[j] += 1
 *                             j -= 1             # <<<<<<<<<<<<<<
 * 
 *             elif node_info1.is_leaf:
 */
            __pyx_v_j = (__pyx_v_j - 1);
          }
        }
      }
      goto __pyx_L10;
    }

    /* "binary_tree.pxi":2451
 *                             j -= 1
 * 
 *             elif node_info1.is_leaf:             # <<<<<<<<<<<<<<
 *                 # If only one is a leaf, split the other
 *                 for i2 in range(2 * i_node2 + 1, 2 * i_node2 + 3):
 */
    __pyx_t_10 = (__pyx_v_node_info1.is_leaf != 0);
    if (__pyx_t_10) {

      /* "binary_tree.pxi":2453
 *             elif node_info1.is_leaf:
 *                 # If only one is a leaf, split the other
 *                 for i2 in range(2 * i_node2 + 1, 2 * i_node2 + 3):             # <<<<<<<<<<<<<<
 *                     self._two_point_dual(i_node1, other, i2,
 *                                          r, count, i_min, i_max)
 */
      __pyx_t_11 = ((2 * __pyx_v_i_node2) + 3);
      for (__pyx_t_13 = ((2 * __pyx_v_i_node2) + 1); __pyx_t_13 < __pyx_t_11; __pyx_t_13+=1) {
        __pyx_v_i2 = __pyx_t_13;

        /* "binary_tree.pxi":2454
 *                 # If only one is a leaf, split the other
 *                 for i2 in range(2 * i_node2 + 1, 2 * i_node2 + 3):
 *                     self._two_point_dual(i_node1, other, i2,             # <<<<<<<<<<<<<<
 *                                          r, count, i_min, i_max)
 * 
 */
        __pyx_t_18 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_two_point_dual(__pyx_v_self, __pyx_v_i_node1, __pyx_v_other, __pyx_v_i2, __pyx_v_r, __pyx_v_count, __pyx_v_i_min, __pyx_v_i_max); if (unlikely(__pyx_t_18 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2454; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      goto __pyx_L10;
    }

    /* "binary_tree.pxi":2457
 *                                          r, count, i_min, i_max)
 * 
 *             elif node_info2.is_leaf:             # <<<<<<<<<<<<<<
 *                 for i1 in range(2 * i_node1 + 1, 2 * i_node1 + 3):
 *                     self._two_point_dual(i1, other, i_node2,
 */
    __pyx_t_10 = (__pyx_v_node_info2.is_leaf != 0);
    if (__pyx_t_10) {

      /* "binary_tree.pxi":2458
 * 
 *             elif node_info2.is_leaf:
 *                 for i1 in range(2 * i_node1 + 1, 2 * i_node1 + 3):             # <<<<<<<<<<<<<<
 *                     self._two_point_dual(i1, other, i_node2,
 *                                          r, count, i_min, i_max)
 */
      __pyx_t_11 = ((2 * __pyx_v_i_node1) + 3);
      for (__pyx_t_13 = ((2 * __pyx_v_i_node1) + 1); __pyx_t_13 < __pyx_t_11; __pyx_t_13+=1) {
        __pyx_v_i1 = __pyx_t_13;

        /* "binary_tree.pxi":2459
 *             elif node_info2.is_leaf:
 *                 for i1 in range(2 * i_node1 + 1, 2 * i_node1 + 3):
 *                     self._two_point_dual(i1, other, i_node2,             # <<<<<<<<<<<<<<
 *                                          r, count, i_min, i_max)
 * 
 */
        __pyx_t_18 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_two_point_dual(__pyx_v_self, __pyx_v_i1, __pyx_v_other, __pyx_v_i_node2, __pyx_v_r, __pyx_v_count, __pyx_v_i_min, __pyx_v_i_max); if (unlikely(__pyx_t_18 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      goto __pyx_L10;
    }
    /*else*/ {

      /* "binary_tree.pxi":2464
 *             else:
 *                  # neither is a leaf: split & query both
 *                 for i1 in range(2 * i_node1 + 1, 2 * i_node1 + 3):             # <<<<<<<<<<<<<<
 *                     for i2 in range(2 * i_node2 + 1, 2 * i_node2 + 3):
 *                         self._two_point_dual(i1, other, i2,
 */
      __pyx_t_11 = ((2 * __pyx_v_i_node1) + 3);
      for (__pyx_t_13 = ((2 * __pyx_v_i_node1) + 1); __pyx_t_13 < __pyx_t_11; __pyx_t_13+=1) {
        __pyx_v_i1 = __pyx_t_13;

        /* "binary_tree.pxi":2465
 *                  # neither is a leaf: split & query both
 *                 for i1 in range(2 * i_node1 + 1, 2 * i_node1 + 3):
 *                     for i2 in range(2 * i_node2 + 1, 2 * i_node2 + 3):             # <<<<<<<<<<<<<<
 *                         self._two_point_dual(i1, other, i2,
 *                                              r, count, i_min, i_max)
 */
        __pyx_t_19 = ((2 * __pyx_v_i_node2) + 3);
        for (__pyx_t_14 = ((2 * __pyx_v_i_node2) + 1); __pyx_t_14 < __pyx_t_19; __pyx_t_14+=1) {
          __pyx_v_i2 = __pyx_t_14;

          /* "binary_tree.pxi":2466
 *                 for i1 in range(2 * i_node1 + 1, 2 * i_node1 + 3):
 *                     for i2 in range(2 * i_node2 + 1, 2 * i_node2 + 3):
 *                         self._two_point_dual(i1, other, i2,             # <<<<<<<<<<<<<<
 *                                              r, count, i_min, i_max)
 *         return 0
 */
          __pyx_t_18 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self->__pyx_vtab)->_two_point_dual(__pyx_v_self, __pyx_v_i1, __pyx_v_other, __pyx_v_i2, __pyx_v_r, __pyx_v_count, __pyx_v_i_min, __pyx_v_i_max); if (unlikely(__pyx_t_18 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2466; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
      }
    }
    __pyx_L10:;
    goto __pyx_L9;
  }
  __pyx_L9:;

  /* "binary_tree.pxi":2468
 *                         self._two_point_dual(i1, other, i2,
 *                                              r, count, i_min, i_max)
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":2398
 *         return 0
 * 
 *     cdef int _two_point_dual(self, ITYPE_t i_node1,             # <<<<<<<<<<<<<<
 *                              BinaryTree other, ITYPE_t i_node2,
 *                              DTYPE_t* r, ITYPE_t* count,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree._two_point_dual", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1009
 *     cdef np.ndarray node_bounds_arr
 * 
 *     cdef readonly DTYPE_t[:, ::1] data             # <<<<<<<<<<<<<<
 *     cdef public ITYPE_t[::1] idx_array
 *     cdef public NodeData_t[::1] node_data
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_4data_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_4data_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_4data___get__(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_4data___get__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(!__pyx_v_self->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1009; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_self->data, 2, (PyObject *(*)(char *)) __pyx_memview_get_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, (int (*)(char *, PyObject *)) __pyx_memview_set_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1009; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.data.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1010
 * 
 *     cdef readonly DTYPE_t[:, ::1] data
 *     cdef public ITYPE_t[::1] idx_array             # <<<<<<<<<<<<<<
 *     cdef public NodeData_t[::1] node_data
 *     cdef public DTYPE_t[:, :, ::1] node_bounds
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_9idx_array_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_9idx_array_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_9idx_array___get__(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_9idx_array___get__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(!__pyx_v_self->idx_array.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1010; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_self->idx_array, 1, (PyObject *(*)(char *)) __pyx_memview_get_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, (int (*)(char *, PyObject *)) __pyx_memview_set_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1010; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.idx_array.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_9idx_array_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_9idx_array_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_9idx_array_2__set__(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_9idx_array_2__set__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_memviewslice __pyx_t_1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_to_MemoryviewSlice_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t(__pyx_v_value);
  if (unlikely(!__pyx_t_1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1010; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->idx_array, 0);
  __pyx_v_self->idx_array = __pyx_t_1;
  __pyx_t_1.memview = NULL;
  __pyx_t_1.data = NULL;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __PYX_XDEC_MEMVIEW(&__pyx_t_1, 1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.idx_array.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1011
 *     cdef readonly DTYPE_t[:, ::1] data
 *     cdef public ITYPE_t[::1] idx_array
 *     cdef public NodeData_t[::1] node_data             # <<<<<<<<<<<<<<
 *     cdef public DTYPE_t[:, :, ::1] node_bounds
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_9node_data_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_9node_data_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_9node_data___get__(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_9node_data___get__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(!__pyx_v_self->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1011; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_self->node_data, 1, (PyObject *(*)(char *)) __pyx_memview_get_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t, (int (*)(char *, PyObject *)) __pyx_memview_set_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1011; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.node_data.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_9node_data_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_9node_data_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_9node_data_2__set__(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_9node_data_2__set__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_memviewslice __pyx_t_1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_to_MemoryviewSlice_dc_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t(__pyx_v_value);
  if (unlikely(!__pyx_t_1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1011; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->node_data, 0);
  __pyx_v_self->node_data = __pyx_t_1;
  __pyx_t_1.memview = NULL;
  __pyx_t_1.data = NULL;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __PYX_XDEC_MEMVIEW(&__pyx_t_1, 1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.node_data.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":1012
 *     cdef public ITYPE_t[::1] idx_array
 *     cdef public NodeData_t[::1] node_data
 *     cdef public DTYPE_t[:, :, ::1] node_bounds             # <<<<<<<<<<<<<<
 * 
 *     cdef ITYPE_t leaf_size
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_11node_bounds_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_11node_bounds_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_11node_bounds___get__(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_11node_bounds___get__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(!__pyx_v_self->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[0]; __pyx_lineno = 1012; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_self->node_bounds, 3, (PyObject *(*)(char *)) __pyx_memview_get_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, (int (*)(char *, PyObject *)) __pyx_memview_set_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1012; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.node_bounds.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_11node_bounds_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_11node_bounds_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_11node_bounds_2__set__(((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7sklearn_9neighbors_7kd_tree_10BinaryTree_11node_bounds_2__set__(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_memviewslice __pyx_t_1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_to_MemoryviewSlice_d_d_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t(__pyx_v_value);
  if (unlikely(!__pyx_t_1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1012; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __PYX_XDEC_MEMVIEW(&__pyx_v_self->node_bounds, 0);
  __pyx_v_self->node_bounds = __pyx_t_1;
  __pyx_t_1.memview = NULL;
  __pyx_t_1.data = NULL;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __PYX_XDEC_MEMVIEW(&__pyx_t_1, 1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.BinaryTree.node_bounds.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":2474
 * # Python functions for benchmarking and testing C implementations
 * 
 * def load_heap(DTYPE_t[:, ::1] X, ITYPE_t k):             # <<<<<<<<<<<<<<
 *     """test fully loading the heap"""
 *     assert k <= X.shape[1]
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_5load_heap(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7sklearn_9neighbors_7kd_tree_4load_heap[] = "test fully loading the heap";
static PyMethodDef __pyx_mdef_7sklearn_9neighbors_7kd_tree_5load_heap = {"load_heap", (PyCFunction)__pyx_pw_7sklearn_9neighbors_7kd_tree_5load_heap, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7sklearn_9neighbors_7kd_tree_4load_heap};
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_5load_heap(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_X = { 0, 0, { 0 }, { 0 }, { 0 } };
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_k;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("load_heap (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_X,&__pyx_n_s_k,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_X)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_k)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("load_heap", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "load_heap") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_X = __Pyx_PyObject_to_MemoryviewSlice_d_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t(values[0]); if (unlikely(!__pyx_v_X.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_k = __Pyx_PyInt_As_Py_intptr_t(values[1]); if (unlikely((__pyx_v_k == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("load_heap", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.load_heap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_4load_heap(__pyx_self, __pyx_v_X, __pyx_v_k);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_4load_heap(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_X, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_k) {
  struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *__pyx_v_heap = 0;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_j;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_7;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_8;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_9;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("load_heap", 0);

  /* "binary_tree.pxi":2476
 * def load_heap(DTYPE_t[:, ::1] X, ITYPE_t k):
 *     """test fully loading the heap"""
 *     assert k <= X.shape[1]             # <<<<<<<<<<<<<<
 *     cdef NeighborsHeap heap = NeighborsHeap(X.shape[0], k)
 *     cdef ITYPE_t i, j
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_k <= (__pyx_v_X.shape[1])) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif

  /* "binary_tree.pxi":2477
 *     """test fully loading the heap"""
 *     assert k <= X.shape[1]
 *     cdef NeighborsHeap heap = NeighborsHeap(X.shape[0], k)             # <<<<<<<<<<<<<<
 *     cdef ITYPE_t i, j
 *     for i in range(X.shape[0]):
 */
  __pyx_t_1 = PyInt_FromSsize_t((__pyx_v_X.shape[0])); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_k); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)((PyObject*)__pyx_ptype_7sklearn_9neighbors_7kd_tree_NeighborsHeap)), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_heap = ((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "binary_tree.pxi":2479
 *     cdef NeighborsHeap heap = NeighborsHeap(X.shape[0], k)
 *     cdef ITYPE_t i, j
 *     for i in range(X.shape[0]):             # <<<<<<<<<<<<<<
 *         for j in range(X.shape[1]):
 *             heap._push(i, X[i, j], j)
 */
  __pyx_t_4 = (__pyx_v_X.shape[0]);
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_i = __pyx_t_5;

    /* "binary_tree.pxi":2480
 *     cdef ITYPE_t i, j
 *     for i in range(X.shape[0]):
 *         for j in range(X.shape[1]):             # <<<<<<<<<<<<<<
 *             heap._push(i, X[i, j], j)
 *     return heap.get_arrays()
 */
    __pyx_t_6 = (__pyx_v_X.shape[1]);
    for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
      __pyx_v_j = __pyx_t_7;

      /* "binary_tree.pxi":2481
 *     for i in range(X.shape[0]):
 *         for j in range(X.shape[1]):
 *             heap._push(i, X[i, j], j)             # <<<<<<<<<<<<<<
 *     return heap.get_arrays()
 * 
 */
      __pyx_t_8 = __pyx_v_i;
      __pyx_t_9 = __pyx_v_j;
      __pyx_t_10 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NeighborsHeap *)__pyx_v_heap->__pyx_vtab)->_push(__pyx_v_heap, __pyx_v_i, (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_X.data + __pyx_t_8 * __pyx_v_X.strides[0]) )) + __pyx_t_9)) ))), __pyx_v_j); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2481; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }

  /* "binary_tree.pxi":2482
 *         for j in range(X.shape[1]):
 *             heap._push(i, X[i, j], j)
 *     return heap.get_arrays()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_heap), __pyx_n_s_get_arrays); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2482; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_1) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2482; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2482; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":2474
 * # Python functions for benchmarking and testing C implementations
 * 
 * def load_heap(DTYPE_t[:, ::1] X, ITYPE_t k):             # <<<<<<<<<<<<<<
 *     """test fully loading the heap"""
 *     assert k <= X.shape[1]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.load_heap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_heap);
  __PYX_XDEC_MEMVIEW(&__pyx_v_X, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":2485
 * 
 * 
 * def simultaneous_sort(DTYPE_t[:, ::1] distances, ITYPE_t[:, ::1] indices):             # <<<<<<<<<<<<<<
 *     """In-place simultaneous sort the given row of the arrays
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_7simultaneous_sort(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7sklearn_9neighbors_7kd_tree_6simultaneous_sort[] = "In-place simultaneous sort the given row of the arrays\n\n    This python wrapper exists primarily to enable unit testing\n    of the _simultaneous_sort C routine.\n    ";
static PyMethodDef __pyx_mdef_7sklearn_9neighbors_7kd_tree_7simultaneous_sort = {"simultaneous_sort", (PyCFunction)__pyx_pw_7sklearn_9neighbors_7kd_tree_7simultaneous_sort, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7sklearn_9neighbors_7kd_tree_6simultaneous_sort};
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_7simultaneous_sort(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_distances = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_indices = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("simultaneous_sort (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_distances,&__pyx_n_s_indices,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_distances)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_indices)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("simultaneous_sort", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2485; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "simultaneous_sort") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2485; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_distances = __Pyx_PyObject_to_MemoryviewSlice_d_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t(values[0]); if (unlikely(!__pyx_v_distances.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2485; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_indices = __Pyx_PyObject_to_MemoryviewSlice_d_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t(values[1]); if (unlikely(!__pyx_v_indices.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2485; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("simultaneous_sort", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2485; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.simultaneous_sort", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_6simultaneous_sort(__pyx_self, __pyx_v_distances, __pyx_v_indices);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_6simultaneous_sort(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_distances, __Pyx_memviewslice __pyx_v_indices) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_row;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("simultaneous_sort", 0);

  /* "binary_tree.pxi":2491
 *     of the _simultaneous_sort C routine.
 *     """
 *     assert distances.shape[0] == indices.shape[0]             # <<<<<<<<<<<<<<
 *     assert distances.shape[1] == indices.shape[1]
 *     cdef ITYPE_t row
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!(((__pyx_v_distances.shape[0]) == (__pyx_v_indices.shape[0])) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2491; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif

  /* "binary_tree.pxi":2492
 *     """
 *     assert distances.shape[0] == indices.shape[0]
 *     assert distances.shape[1] == indices.shape[1]             # <<<<<<<<<<<<<<
 *     cdef ITYPE_t row
 *     for row in range(distances.shape[0]):
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!(((__pyx_v_distances.shape[1]) == (__pyx_v_indices.shape[1])) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2492; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif

  /* "binary_tree.pxi":2494
 *     assert distances.shape[1] == indices.shape[1]
 *     cdef ITYPE_t row
 *     for row in range(distances.shape[0]):             # <<<<<<<<<<<<<<
 *         _simultaneous_sort(&distances[row, 0],
 *                            &indices[row, 0],
 */
  __pyx_t_1 = (__pyx_v_distances.shape[0]);
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_row = __pyx_t_2;

    /* "binary_tree.pxi":2495
 *     cdef ITYPE_t row
 *     for row in range(distances.shape[0]):
 *         _simultaneous_sort(&distances[row, 0],             # <<<<<<<<<<<<<<
 *                            &indices[row, 0],
 *                            distances.shape[1])
 */
    __pyx_t_3 = __pyx_v_row;
    __pyx_t_4 = 0;

    /* "binary_tree.pxi":2496
 *     for row in range(distances.shape[0]):
 *         _simultaneous_sort(&distances[row, 0],
 *                            &indices[row, 0],             # <<<<<<<<<<<<<<
 *                            distances.shape[1])
 * 
 */
    __pyx_t_5 = __pyx_v_row;
    __pyx_t_6 = 0;

    /* "binary_tree.pxi":2495
 *     cdef ITYPE_t row
 *     for row in range(distances.shape[0]):
 *         _simultaneous_sort(&distances[row, 0],             # <<<<<<<<<<<<<<
 *                            &indices[row, 0],
 *                            distances.shape[1])
 */
    __pyx_t_7 = __pyx_f_7sklearn_9neighbors_7kd_tree__simultaneous_sort((&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_distances.data + __pyx_t_3 * __pyx_v_distances.strides[0]) )) + __pyx_t_4)) )))), (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ (__pyx_v_indices.data + __pyx_t_5 * __pyx_v_indices.strides[0]) )) + __pyx_t_6)) )))), (__pyx_v_distances.shape[1])); if (unlikely(__pyx_t_7 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2495; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "binary_tree.pxi":2485
 * 
 * 
 * def simultaneous_sort(DTYPE_t[:, ::1] distances, ITYPE_t[:, ::1] indices):             # <<<<<<<<<<<<<<
 *     """In-place simultaneous sort the given row of the arrays
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.simultaneous_sort", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_distances, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_indices, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":2500
 * 
 * 
 * def nodeheap_sort(DTYPE_t[::1] vals):             # <<<<<<<<<<<<<<
 *     """In-place reverse sort of vals using NodeHeap"""
 *     cdef ITYPE_t[::1] indices = np.zeros(vals.shape[0], dtype=ITYPE)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_9nodeheap_sort(PyObject *__pyx_self, PyObject *__pyx_arg_vals); /*proto*/
static char __pyx_doc_7sklearn_9neighbors_7kd_tree_8nodeheap_sort[] = "In-place reverse sort of vals using NodeHeap";
static PyMethodDef __pyx_mdef_7sklearn_9neighbors_7kd_tree_9nodeheap_sort = {"nodeheap_sort", (PyCFunction)__pyx_pw_7sklearn_9neighbors_7kd_tree_9nodeheap_sort, METH_O, __pyx_doc_7sklearn_9neighbors_7kd_tree_8nodeheap_sort};
static PyObject *__pyx_pw_7sklearn_9neighbors_7kd_tree_9nodeheap_sort(PyObject *__pyx_self, PyObject *__pyx_arg_vals) {
  __Pyx_memviewslice __pyx_v_vals = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("nodeheap_sort (wrapper)", 0);
  assert(__pyx_arg_vals); {
    __pyx_v_vals = __Pyx_PyObject_to_MemoryviewSlice_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t(__pyx_arg_vals); if (unlikely(!__pyx_v_vals.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2500; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.nodeheap_sort", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7sklearn_9neighbors_7kd_tree_8nodeheap_sort(__pyx_self, __pyx_v_vals);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_9neighbors_7kd_tree_8nodeheap_sort(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_vals) {
  __Pyx_memviewslice __pyx_v_indices = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_vals_sorted = { 0, 0, { 0 }, { 0 }, { 0 } };
  struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *__pyx_v_heap = 0;
  struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t __pyx_v_data;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_t_6 = NULL;
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_8;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_9;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_10;
  int __pyx_t_11;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_12;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_13;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_14;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("nodeheap_sort", 0);

  /* "binary_tree.pxi":2502
 * def nodeheap_sort(DTYPE_t[::1] vals):
 *     """In-place reverse sort of vals using NodeHeap"""
 *     cdef ITYPE_t[::1] indices = np.zeros(vals.shape[0], dtype=ITYPE)             # <<<<<<<<<<<<<<
 *     cdef DTYPE_t[::1] vals_sorted = np.zeros_like(vals)
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2502; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2502; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyInt_FromSsize_t((__pyx_v_vals.shape[0])); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2502; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2502; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2502; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2502; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2502; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2502; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t(__pyx_t_4);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2502; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_indices = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "binary_tree.pxi":2503
 *     """In-place reverse sort of vals using NodeHeap"""
 *     cdef ITYPE_t[::1] indices = np.zeros(vals.shape[0], dtype=ITYPE)
 *     cdef DTYPE_t[::1] vals_sorted = np.zeros_like(vals)             # <<<<<<<<<<<<<<
 * 
 *     # use initial size 0 to check corner case
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2503; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros_like); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2503; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_vals, 1, (PyObject *(*)(char *)) __pyx_memview_get_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, (int (*)(char *, PyObject *)) __pyx_memview_set_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2503; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_2) {
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2503; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_4);
  } else {
    __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2503; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2); __Pyx_GIVEREF(__pyx_t_2); __pyx_t_2 = NULL;
    PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2503; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t(__pyx_t_4);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2503; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_vals_sorted = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "binary_tree.pxi":2506
 * 
 *     # use initial size 0 to check corner case
 *     cdef NodeHeap heap = NodeHeap(0)             # <<<<<<<<<<<<<<
 *     cdef NodeHeapData_t data
 *     cdef ITYPE_t i
 */
  __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)((PyObject*)__pyx_ptype_7sklearn_9neighbors_7kd_tree_NodeHeap)), __pyx_tuple__32, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2506; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v_heap = ((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "binary_tree.pxi":2509
 *     cdef NodeHeapData_t data
 *     cdef ITYPE_t i
 *     for i in range(vals.shape[0]):             # <<<<<<<<<<<<<<
 *         data.val = vals[i]
 *         data.i1 = i
 */
  __pyx_t_8 = (__pyx_v_vals.shape[0]);
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_i = __pyx_t_9;

    /* "binary_tree.pxi":2510
 *     cdef ITYPE_t i
 *     for i in range(vals.shape[0]):
 *         data.val = vals[i]             # <<<<<<<<<<<<<<
 *         data.i1 = i
 *         data.i2 = i + 1
 */
    __pyx_t_10 = __pyx_v_i;
    __pyx_v_data.val = (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_vals.data) + __pyx_t_10)) )));

    /* "binary_tree.pxi":2511
 *     for i in range(vals.shape[0]):
 *         data.val = vals[i]
 *         data.i1 = i             # <<<<<<<<<<<<<<
 *         data.i2 = i + 1
 *         heap.push(data)
 */
    __pyx_v_data.i1 = __pyx_v_i;

    /* "binary_tree.pxi":2512
 *         data.val = vals[i]
 *         data.i1 = i
 *         data.i2 = i + 1             # <<<<<<<<<<<<<<
 *         heap.push(data)
 * 
 */
    __pyx_v_data.i2 = (__pyx_v_i + 1);

    /* "binary_tree.pxi":2513
 *         data.i1 = i
 *         data.i2 = i + 1
 *         heap.push(data)             # <<<<<<<<<<<<<<
 * 
 *     for i in range(vals.shape[0]):
 */
    __pyx_t_11 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_v_heap->__pyx_vtab)->push(__pyx_v_heap, __pyx_v_data); if (unlikely(__pyx_t_11 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2513; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "binary_tree.pxi":2515
 *         heap.push(data)
 * 
 *     for i in range(vals.shape[0]):             # <<<<<<<<<<<<<<
 *         data = heap.pop()
 *         vals_sorted[i] = data.val
 */
  __pyx_t_8 = (__pyx_v_vals.shape[0]);
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_i = __pyx_t_9;

    /* "binary_tree.pxi":2516
 * 
 *     for i in range(vals.shape[0]):
 *         data = heap.pop()             # <<<<<<<<<<<<<<
 *         vals_sorted[i] = data.val
 *         indices[i] = data.i1
 */
    __pyx_v_data = ((struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NodeHeap *)__pyx_v_heap->__pyx_vtab)->pop(__pyx_v_heap);

    /* "binary_tree.pxi":2517
 *     for i in range(vals.shape[0]):
 *         data = heap.pop()
 *         vals_sorted[i] = data.val             # <<<<<<<<<<<<<<
 *         indices[i] = data.i1
 * 
 */
    __pyx_t_12 = __pyx_v_data.val;
    __pyx_t_13 = __pyx_v_i;
    *((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) __pyx_v_vals_sorted.data) + __pyx_t_13)) )) = __pyx_t_12;

    /* "binary_tree.pxi":2518
 *         data = heap.pop()
 *         vals_sorted[i] = data.val
 *         indices[i] = data.i1             # <<<<<<<<<<<<<<
 * 
 *     return np.asarray(vals_sorted), np.asarray(indices)
 */
    __pyx_t_14 = __pyx_v_data.i1;
    __pyx_t_15 = __pyx_v_i;
    *((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_indices.data) + __pyx_t_15)) )) = __pyx_t_14;
  }

  /* "binary_tree.pxi":2520
 *         indices[i] = data.i1
 * 
 *     return np.asarray(vals_sorted), np.asarray(indices)             # <<<<<<<<<<<<<<
 * 
 * # Reimplementation for MSVC support
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2520; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_asarray); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2520; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_vals_sorted, 1, (PyObject *(*)(char *)) __pyx_memview_get_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, (int (*)(char *, PyObject *)) __pyx_memview_set_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2520; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  if (!__pyx_t_1) {
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2520; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_4);
  } else {
    __pyx_t_2 = PyTuple_New(1+1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2520; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1); __Pyx_GIVEREF(__pyx_t_1); __pyx_t_1 = NULL;
    PyTuple_SET_ITEM(__pyx_t_2, 0+1, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_2, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2520; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2520; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2520; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_indices, 1, (PyObject *(*)(char *)) __pyx_memview_get_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, (int (*)(char *, PyObject *)) __pyx_memview_set_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2520; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_1) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2520; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_16 = PyTuple_New(1+1); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2520; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_16);
    PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_t_1); __Pyx_GIVEREF(__pyx_t_1); __pyx_t_1 = NULL;
    PyTuple_SET_ITEM(__pyx_t_16, 0+1, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_16, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2520; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2520; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_6);
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "binary_tree.pxi":2500
 * 
 * 
 * def nodeheap_sort(DTYPE_t[::1] vals):             # <<<<<<<<<<<<<<
 *     """In-place reverse sort of vals using NodeHeap"""
 *     cdef ITYPE_t[::1] indices = np.zeros(vals.shape[0], dtype=ITYPE)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  __Pyx_XDECREF(__pyx_t_6);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.nodeheap_sort", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_vals, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_indices, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_vals_sorted, 1);
  __Pyx_XDECREF((PyObject *)__pyx_v_heap);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":2523
 * 
 * # Reimplementation for MSVC support
 * cdef inline double fmin(double a, double b):             # <<<<<<<<<<<<<<
 *     return min(a, b)
 * 
 */

static CYTHON_INLINE double __pyx_f_7sklearn_9neighbors_7kd_tree_fmin(double __pyx_v_a, double __pyx_v_b) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  double __pyx_t_2;
  double __pyx_t_3;
  __Pyx_RefNannySetupContext("fmin", 0);

  /* "binary_tree.pxi":2524
 * # Reimplementation for MSVC support
 * cdef inline double fmin(double a, double b):
 *     return min(a, b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double fmax(double a, double b) nogil:
 */
  __pyx_t_1 = __pyx_v_b;
  __pyx_t_2 = __pyx_v_a;
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_r = __pyx_t_3;
  goto __pyx_L0;

  /* "binary_tree.pxi":2523
 * 
 * # Reimplementation for MSVC support
 * cdef inline double fmin(double a, double b):             # <<<<<<<<<<<<<<
 *     return min(a, b)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "binary_tree.pxi":2526
 *     return min(a, b)
 * 
 * cdef inline double fmax(double a, double b) nogil:             # <<<<<<<<<<<<<<
 *     return max(a, b)
 */

static CYTHON_INLINE double __pyx_f_7sklearn_9neighbors_7kd_tree_fmax(double __pyx_v_a, double __pyx_v_b) {
  double __pyx_r;
  double __pyx_t_1;
  double __pyx_t_2;
  double __pyx_t_3;

  /* "binary_tree.pxi":2527
 * 
 * cdef inline double fmax(double a, double b) nogil:
 *     return max(a, b)             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __pyx_v_b;
  __pyx_t_2 = __pyx_v_a;
  if (((__pyx_t_1 > __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_r = __pyx_t_3;
  goto __pyx_L0;

  /* "binary_tree.pxi":2526
 *     return min(a, b)
 * 
 * cdef inline double fmax(double a, double b) nogil:             # <<<<<<<<<<<<<<
 *     return max(a, b)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "sklearn/neighbors/kd_tree.pyx":36
 * 
 * 
 * cdef int allocate_data(BinaryTree tree, ITYPE_t n_nodes,             # <<<<<<<<<<<<<<
 *                        ITYPE_t n_features) except -1:
 *     """Allocate arrays needed for the KD Tree"""
 */

static int __pyx_f_7sklearn_9neighbors_7kd_tree_allocate_data(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_tree, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_nodes, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_features) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("allocate_data", 0);

  /* "sklearn/neighbors/kd_tree.pyx":39
 *                        ITYPE_t n_features) except -1:
 *     """Allocate arrays needed for the KD Tree"""
 *     tree.node_bounds_arr = np.zeros((2, n_nodes, n_features), dtype=DTYPE)             # <<<<<<<<<<<<<<
 *     tree.node_bounds = get_memview_DTYPE_3D(tree.node_bounds_arr)
 *     return 0
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 39; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 39; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n_nodes); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 39; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n_features); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 39; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 39; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 39; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 39; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 39; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 39; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 39; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 39; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_tree->node_bounds_arr);
  __Pyx_DECREF(((PyObject *)__pyx_v_tree->node_bounds_arr));
  __pyx_v_tree->node_bounds_arr = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sklearn/neighbors/kd_tree.pyx":40
 *     """Allocate arrays needed for the KD Tree"""
 *     tree.node_bounds_arr = np.zeros((2, n_nodes, n_features), dtype=DTYPE)
 *     tree.node_bounds = get_memview_DTYPE_3D(tree.node_bounds_arr)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_tree->node_bounds_arr);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_5 = __pyx_f_7sklearn_9neighbors_7kd_tree_get_memview_DTYPE_3D(((PyArrayObject *)__pyx_t_1)); if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 40; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_XDEC_MEMVIEW(&__pyx_v_tree->node_bounds, 0);
  __pyx_v_tree->node_bounds = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "sklearn/neighbors/kd_tree.pyx":41
 *     tree.node_bounds_arr = np.zeros((2, n_nodes, n_features), dtype=DTYPE)
 *     tree.node_bounds = get_memview_DTYPE_3D(tree.node_bounds_arr)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "sklearn/neighbors/kd_tree.pyx":36
 * 
 * 
 * cdef int allocate_data(BinaryTree tree, ITYPE_t n_nodes,             # <<<<<<<<<<<<<<
 *                        ITYPE_t n_features) except -1:
 *     """Allocate arrays needed for the KD Tree"""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.allocate_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/neighbors/kd_tree.pyx":44
 * 
 * 
 * cdef int init_node(BinaryTree tree, ITYPE_t i_node,             # <<<<<<<<<<<<<<
 *                    ITYPE_t idx_start, ITYPE_t idx_end) except -1:
 *     """Initialize the node for the dataset stored in tree.data"""
 */

static int __pyx_f_7sklearn_9neighbors_7kd_tree_init_node(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_tree, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_idx_start, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_idx_end) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_features;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_j;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_rad;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_lower_bounds;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_upper_bounds;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_data;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *__pyx_v_idx_array;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_data_row;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_10;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_11;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_12;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_13;
  int __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("init_node", 0);

  /* "sklearn/neighbors/kd_tree.pyx":47
 *                    ITYPE_t idx_start, ITYPE_t idx_end) except -1:
 *     """Initialize the node for the dataset stored in tree.data"""
 *     cdef ITYPE_t n_features = tree.data.shape[1]             # <<<<<<<<<<<<<<
 *     cdef ITYPE_t i, j
 *     cdef DTYPE_t rad = 0
 */
  if (unlikely(!__pyx_v_tree->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 47; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_n_features = (__pyx_v_tree->data.shape[1]);

  /* "sklearn/neighbors/kd_tree.pyx":49
 *     cdef ITYPE_t n_features = tree.data.shape[1]
 *     cdef ITYPE_t i, j
 *     cdef DTYPE_t rad = 0             # <<<<<<<<<<<<<<
 * 
 *     cdef DTYPE_t* lower_bounds = &tree.node_bounds[0, i_node, 0]
 */
  __pyx_v_rad = 0.0;

  /* "sklearn/neighbors/kd_tree.pyx":51
 *     cdef DTYPE_t rad = 0
 * 
 *     cdef DTYPE_t* lower_bounds = &tree.node_bounds[0, i_node, 0]             # <<<<<<<<<<<<<<
 *     cdef DTYPE_t* upper_bounds = &tree.node_bounds[1, i_node, 0]
 *     cdef DTYPE_t* data = &tree.data[0, 0]
 */
  if (unlikely(!__pyx_v_tree->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 51; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_1 = 0;
  __pyx_t_2 = __pyx_v_i_node;
  __pyx_t_3 = 0;
  __pyx_v_lower_bounds = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree->node_bounds.data + __pyx_t_1 * __pyx_v_tree->node_bounds.strides[0]) ) + __pyx_t_2 * __pyx_v_tree->node_bounds.strides[1]) )) + __pyx_t_3)) ))));

  /* "sklearn/neighbors/kd_tree.pyx":52
 * 
 *     cdef DTYPE_t* lower_bounds = &tree.node_bounds[0, i_node, 0]
 *     cdef DTYPE_t* upper_bounds = &tree.node_bounds[1, i_node, 0]             # <<<<<<<<<<<<<<
 *     cdef DTYPE_t* data = &tree.data[0, 0]
 *     cdef ITYPE_t* idx_array = &tree.idx_array[0]
 */
  if (unlikely(!__pyx_v_tree->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 52; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_4 = 1;
  __pyx_t_5 = __pyx_v_i_node;
  __pyx_t_6 = 0;
  __pyx_v_upper_bounds = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree->node_bounds.data + __pyx_t_4 * __pyx_v_tree->node_bounds.strides[0]) ) + __pyx_t_5 * __pyx_v_tree->node_bounds.strides[1]) )) + __pyx_t_6)) ))));

  /* "sklearn/neighbors/kd_tree.pyx":53
 *     cdef DTYPE_t* lower_bounds = &tree.node_bounds[0, i_node, 0]
 *     cdef DTYPE_t* upper_bounds = &tree.node_bounds[1, i_node, 0]
 *     cdef DTYPE_t* data = &tree.data[0, 0]             # <<<<<<<<<<<<<<
 *     cdef ITYPE_t* idx_array = &tree.idx_array[0]
 * 
 */
  if (unlikely(!__pyx_v_tree->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_v_data = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=0 */ (__pyx_v_tree->data.data + __pyx_t_7 * __pyx_v_tree->data.strides[0]) )) + __pyx_t_8)) ))));

  /* "sklearn/neighbors/kd_tree.pyx":54
 *     cdef DTYPE_t* upper_bounds = &tree.node_bounds[1, i_node, 0]
 *     cdef DTYPE_t* data = &tree.data[0, 0]
 *     cdef ITYPE_t* idx_array = &tree.idx_array[0]             # <<<<<<<<<<<<<<
 * 
 *     cdef DTYPE_t* data_row
 */
  if (unlikely(!__pyx_v_tree->idx_array.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_9 = 0;
  __pyx_v_idx_array = (&(*((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) ( /* dim=0 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) __pyx_v_tree->idx_array.data) + __pyx_t_9)) ))));

  /* "sklearn/neighbors/kd_tree.pyx":59
 * 
 *     # determine Node bounds
 *     for j in range(n_features):             # <<<<<<<<<<<<<<
 *         lower_bounds[j] = INF
 *         upper_bounds[j] = -INF
 */
  __pyx_t_10 = __pyx_v_n_features;
  for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
    __pyx_v_j = __pyx_t_11;

    /* "sklearn/neighbors/kd_tree.pyx":60
 *     # determine Node bounds
 *     for j in range(n_features):
 *         lower_bounds[j] = INF             # <<<<<<<<<<<<<<
 *         upper_bounds[j] = -INF
 * 
 */
    (__pyx_v_lower_bounds[__pyx_v_j]) = __pyx_v_7sklearn_9neighbors_7kd_tree_INF;

    /* "sklearn/neighbors/kd_tree.pyx":61
 *     for j in range(n_features):
 *         lower_bounds[j] = INF
 *         upper_bounds[j] = -INF             # <<<<<<<<<<<<<<
 * 
 *     # Compute the actual data range.  At build time, this is slightly
 */
    (__pyx_v_upper_bounds[__pyx_v_j]) = (-__pyx_v_7sklearn_9neighbors_7kd_tree_INF);
  }

  /* "sklearn/neighbors/kd_tree.pyx":66
 *     # slower than using the previously-computed bounds of the parent node,
 *     # but leads to more compact trees and thus faster queries.
 *     for i in range(idx_start, idx_end):             # <<<<<<<<<<<<<<
 *         data_row = data + idx_array[i] * n_features
 *         for j in range(n_features):
 */
  __pyx_t_10 = __pyx_v_idx_end;
  for (__pyx_t_11 = __pyx_v_idx_start; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
    __pyx_v_i = __pyx_t_11;

    /* "sklearn/neighbors/kd_tree.pyx":67
 *     # but leads to more compact trees and thus faster queries.
 *     for i in range(idx_start, idx_end):
 *         data_row = data + idx_array[i] * n_features             # <<<<<<<<<<<<<<
 *         for j in range(n_features):
 *             lower_bounds[j] = fmin(lower_bounds[j], data_row[j])
 */
    __pyx_v_data_row = (__pyx_v_data + ((__pyx_v_idx_array[__pyx_v_i]) * __pyx_v_n_features));

    /* "sklearn/neighbors/kd_tree.pyx":68
 *     for i in range(idx_start, idx_end):
 *         data_row = data + idx_array[i] * n_features
 *         for j in range(n_features):             # <<<<<<<<<<<<<<
 *             lower_bounds[j] = fmin(lower_bounds[j], data_row[j])
 *             upper_bounds[j] = fmax(upper_bounds[j], data_row[j])
 */
    __pyx_t_12 = __pyx_v_n_features;
    for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
      __pyx_v_j = __pyx_t_13;

      /* "sklearn/neighbors/kd_tree.pyx":69
 *         data_row = data + idx_array[i] * n_features
 *         for j in range(n_features):
 *             lower_bounds[j] = fmin(lower_bounds[j], data_row[j])             # <<<<<<<<<<<<<<
 *             upper_bounds[j] = fmax(upper_bounds[j], data_row[j])
 *         if tree.dist_metric.p == INF:
 */
      (__pyx_v_lower_bounds[__pyx_v_j]) = __pyx_f_7sklearn_9neighbors_7kd_tree_fmin((__pyx_v_lower_bounds[__pyx_v_j]), (__pyx_v_data_row[__pyx_v_j]));

      /* "sklearn/neighbors/kd_tree.pyx":70
 *         for j in range(n_features):
 *             lower_bounds[j] = fmin(lower_bounds[j], data_row[j])
 *             upper_bounds[j] = fmax(upper_bounds[j], data_row[j])             # <<<<<<<<<<<<<<
 *         if tree.dist_metric.p == INF:
 *             rad = fmax(rad, 0.5 * (upper_bounds[j] - lower_bounds[j]))
 */
      (__pyx_v_upper_bounds[__pyx_v_j]) = __pyx_f_7sklearn_9neighbors_7kd_tree_fmax((__pyx_v_upper_bounds[__pyx_v_j]), (__pyx_v_data_row[__pyx_v_j]));
    }

    /* "sklearn/neighbors/kd_tree.pyx":71
 *             lower_bounds[j] = fmin(lower_bounds[j], data_row[j])
 *             upper_bounds[j] = fmax(upper_bounds[j], data_row[j])
 *         if tree.dist_metric.p == INF:             # <<<<<<<<<<<<<<
 *             rad = fmax(rad, 0.5 * (upper_bounds[j] - lower_bounds[j]))
 *         else:
 */
    __pyx_t_14 = ((__pyx_v_tree->dist_metric->p == __pyx_v_7sklearn_9neighbors_7kd_tree_INF) != 0);
    if (__pyx_t_14) {

      /* "sklearn/neighbors/kd_tree.pyx":72
 *             upper_bounds[j] = fmax(upper_bounds[j], data_row[j])
 *         if tree.dist_metric.p == INF:
 *             rad = fmax(rad, 0.5 * (upper_bounds[j] - lower_bounds[j]))             # <<<<<<<<<<<<<<
 *         else:
 *             rad += pow(0.5 * abs(upper_bounds[j] - lower_bounds[j]),
 */
      __pyx_v_rad = __pyx_f_7sklearn_9neighbors_7kd_tree_fmax(__pyx_v_rad, (0.5 * ((__pyx_v_upper_bounds[__pyx_v_j]) - (__pyx_v_lower_bounds[__pyx_v_j]))));
      goto __pyx_L9;
    }
    /*else*/ {

      /* "sklearn/neighbors/kd_tree.pyx":74
 *             rad = fmax(rad, 0.5 * (upper_bounds[j] - lower_bounds[j]))
 *         else:
 *             rad += pow(0.5 * abs(upper_bounds[j] - lower_bounds[j]),             # <<<<<<<<<<<<<<
 *                        tree.dist_metric.p)
 * 
 */
      __pyx_v_rad = (__pyx_v_rad + pow((0.5 * fabs(((__pyx_v_upper_bounds[__pyx_v_j]) - (__pyx_v_lower_bounds[__pyx_v_j])))), __pyx_v_tree->dist_metric->p));
    }
    __pyx_L9:;
  }

  /* "sklearn/neighbors/kd_tree.pyx":77
 *                        tree.dist_metric.p)
 * 
 *     tree.node_data[i_node].idx_start = idx_start             # <<<<<<<<<<<<<<
 *     tree.node_data[i_node].idx_end = idx_end
 * 
 */
  if (unlikely(!__pyx_v_tree->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 77; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_10 = __pyx_v_i_node;
  (*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_tree->node_data.data) + __pyx_t_10)) ))).idx_start = __pyx_v_idx_start;

  /* "sklearn/neighbors/kd_tree.pyx":78
 * 
 *     tree.node_data[i_node].idx_start = idx_start
 *     tree.node_data[i_node].idx_end = idx_end             # <<<<<<<<<<<<<<
 * 
 *     # The radius will hold the size of the circumscribed hypersphere measured
 */
  if (unlikely(!__pyx_v_tree->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 78; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_11 = __pyx_v_i_node;
  (*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_tree->node_data.data) + __pyx_t_11)) ))).idx_end = __pyx_v_idx_end;

  /* "sklearn/neighbors/kd_tree.pyx":83
 *     # with the specified metric: in querying, this is used as a measure of the
 *     # size of each node when deciding which nodes to split.
 *     tree.node_data[i_node].radius = pow(rad, 1. / tree.dist_metric.p)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  if (unlikely(!__pyx_v_tree->node_data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 83; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_t_12 = __pyx_v_i_node;
  (*((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) ( /* dim=0 */ ((char *) (((struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) __pyx_v_tree->node_data.data) + __pyx_t_12)) ))).radius = pow(__pyx_v_rad, (1. / __pyx_v_tree->dist_metric->p));

  /* "sklearn/neighbors/kd_tree.pyx":84
 *     # size of each node when deciding which nodes to split.
 *     tree.node_data[i_node].radius = pow(rad, 1. / tree.dist_metric.p)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "sklearn/neighbors/kd_tree.pyx":44
 * 
 * 
 * cdef int init_node(BinaryTree tree, ITYPE_t i_node,             # <<<<<<<<<<<<<<
 *                    ITYPE_t idx_start, ITYPE_t idx_end) except -1:
 *     """Initialize the node for the dataset stored in tree.data"""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.init_node", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/neighbors/kd_tree.pyx":87
 * 
 * 
 * cdef DTYPE_t min_rdist(BinaryTree tree, ITYPE_t i_node,             # <<<<<<<<<<<<<<
 *                        DTYPE_t* pt) nogil except -1:
 *     """Compute the minimum reduced-distance between a point and a node"""
 */

static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_min_rdist(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_tree, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_features;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_d;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_d_lo;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_d_hi;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_rdist;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_j;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  int __pyx_t_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_5;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_8;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_11;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_14;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "sklearn/neighbors/kd_tree.pyx":90
 *                        DTYPE_t* pt) nogil except -1:
 *     """Compute the minimum reduced-distance between a point and a node"""
 *     cdef ITYPE_t n_features = tree.data.shape[1]             # <<<<<<<<<<<<<<
 *     cdef DTYPE_t d, d_lo, d_hi, rdist=0.0
 *     cdef ITYPE_t j
 */
  if (unlikely(!__pyx_v_tree->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 90; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_n_features = (__pyx_v_tree->data.shape[1]);

  /* "sklearn/neighbors/kd_tree.pyx":91
 *     """Compute the minimum reduced-distance between a point and a node"""
 *     cdef ITYPE_t n_features = tree.data.shape[1]
 *     cdef DTYPE_t d, d_lo, d_hi, rdist=0.0             # <<<<<<<<<<<<<<
 *     cdef ITYPE_t j
 * 
 */
  __pyx_v_rdist = 0.0;

  /* "sklearn/neighbors/kd_tree.pyx":94
 *     cdef ITYPE_t j
 * 
 *     if tree.dist_metric.p == INF:             # <<<<<<<<<<<<<<
 *         for j in range(n_features):
 *             d_lo = tree.node_bounds[0, i_node, j] - pt[j]
 */
  __pyx_t_1 = ((__pyx_v_tree->dist_metric->p == __pyx_v_7sklearn_9neighbors_7kd_tree_INF) != 0);
  if (__pyx_t_1) {

    /* "sklearn/neighbors/kd_tree.pyx":95
 * 
 *     if tree.dist_metric.p == INF:
 *         for j in range(n_features):             # <<<<<<<<<<<<<<
 *             d_lo = tree.node_bounds[0, i_node, j] - pt[j]
 *             d_hi = pt[j] - tree.node_bounds[1, i_node, j]
 */
    __pyx_t_2 = __pyx_v_n_features;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_j = __pyx_t_3;

      /* "sklearn/neighbors/kd_tree.pyx":96
 *     if tree.dist_metric.p == INF:
 *         for j in range(n_features):
 *             d_lo = tree.node_bounds[0, i_node, j] - pt[j]             # <<<<<<<<<<<<<<
 *             d_hi = pt[j] - tree.node_bounds[1, i_node, j]
 *             d = (d_lo + fabs(d_lo)) + (d_hi + fabs(d_hi))
 */
      if (unlikely(!__pyx_v_tree->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 96; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_4 = 0;
      __pyx_t_5 = __pyx_v_i_node;
      __pyx_t_6 = __pyx_v_j;
      __pyx_v_d_lo = ((*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree->node_bounds.data + __pyx_t_4 * __pyx_v_tree->node_bounds.strides[0]) ) + __pyx_t_5 * __pyx_v_tree->node_bounds.strides[1]) )) + __pyx_t_6)) ))) - (__pyx_v_pt[__pyx_v_j]));

      /* "sklearn/neighbors/kd_tree.pyx":97
 *         for j in range(n_features):
 *             d_lo = tree.node_bounds[0, i_node, j] - pt[j]
 *             d_hi = pt[j] - tree.node_bounds[1, i_node, j]             # <<<<<<<<<<<<<<
 *             d = (d_lo + fabs(d_lo)) + (d_hi + fabs(d_hi))
 *             rdist = fmax(rdist, 0.5 * d)
 */
      if (unlikely(!__pyx_v_tree->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 97; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_7 = 1;
      __pyx_t_8 = __pyx_v_i_node;
      __pyx_t_9 = __pyx_v_j;
      __pyx_v_d_hi = ((__pyx_v_pt[__pyx_v_j]) - (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree->node_bounds.data + __pyx_t_7 * __pyx_v_tree->node_bounds.strides[0]) ) + __pyx_t_8 * __pyx_v_tree->node_bounds.strides[1]) )) + __pyx_t_9)) ))));

      /* "sklearn/neighbors/kd_tree.pyx":98
 *             d_lo = tree.node_bounds[0, i_node, j] - pt[j]
 *             d_hi = pt[j] - tree.node_bounds[1, i_node, j]
 *             d = (d_lo + fabs(d_lo)) + (d_hi + fabs(d_hi))             # <<<<<<<<<<<<<<
 *             rdist = fmax(rdist, 0.5 * d)
 *     else:
 */
      __pyx_v_d = ((__pyx_v_d_lo + fabs(__pyx_v_d_lo)) + (__pyx_v_d_hi + fabs(__pyx_v_d_hi)));

      /* "sklearn/neighbors/kd_tree.pyx":99
 *             d_hi = pt[j] - tree.node_bounds[1, i_node, j]
 *             d = (d_lo + fabs(d_lo)) + (d_hi + fabs(d_hi))
 *             rdist = fmax(rdist, 0.5 * d)             # <<<<<<<<<<<<<<
 *     else:
 *         # here we'll use the fact that x + abs(x) = 2 * max(x, 0)
 */
      __pyx_v_rdist = __pyx_f_7sklearn_9neighbors_7kd_tree_fmax(__pyx_v_rdist, (0.5 * __pyx_v_d));
    }
    goto __pyx_L3;
  }
  /*else*/ {

    /* "sklearn/neighbors/kd_tree.pyx":102
 *     else:
 *         # here we'll use the fact that x + abs(x) = 2 * max(x, 0)
 *         for j in range(n_features):             # <<<<<<<<<<<<<<
 *             d_lo = tree.node_bounds[0, i_node, j] - pt[j]
 *             d_hi = pt[j] - tree.node_bounds[1, i_node, j]
 */
    __pyx_t_2 = __pyx_v_n_features;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_j = __pyx_t_3;

      /* "sklearn/neighbors/kd_tree.pyx":103
 *         # here we'll use the fact that x + abs(x) = 2 * max(x, 0)
 *         for j in range(n_features):
 *             d_lo = tree.node_bounds[0, i_node, j] - pt[j]             # <<<<<<<<<<<<<<
 *             d_hi = pt[j] - tree.node_bounds[1, i_node, j]
 *             d = (d_lo + fabs(d_lo)) + (d_hi + fabs(d_hi))
 */
      if (unlikely(!__pyx_v_tree->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 103; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_10 = 0;
      __pyx_t_11 = __pyx_v_i_node;
      __pyx_t_12 = __pyx_v_j;
      __pyx_v_d_lo = ((*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree->node_bounds.data + __pyx_t_10 * __pyx_v_tree->node_bounds.strides[0]) ) + __pyx_t_11 * __pyx_v_tree->node_bounds.strides[1]) )) + __pyx_t_12)) ))) - (__pyx_v_pt[__pyx_v_j]));

      /* "sklearn/neighbors/kd_tree.pyx":104
 *         for j in range(n_features):
 *             d_lo = tree.node_bounds[0, i_node, j] - pt[j]
 *             d_hi = pt[j] - tree.node_bounds[1, i_node, j]             # <<<<<<<<<<<<<<
 *             d = (d_lo + fabs(d_lo)) + (d_hi + fabs(d_hi))
 *             rdist += pow(0.5 * d, tree.dist_metric.p)
 */
      if (unlikely(!__pyx_v_tree->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 104; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_13 = 1;
      __pyx_t_14 = __pyx_v_i_node;
      __pyx_t_15 = __pyx_v_j;
      __pyx_v_d_hi = ((__pyx_v_pt[__pyx_v_j]) - (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree->node_bounds.data + __pyx_t_13 * __pyx_v_tree->node_bounds.strides[0]) ) + __pyx_t_14 * __pyx_v_tree->node_bounds.strides[1]) )) + __pyx_t_15)) ))));

      /* "sklearn/neighbors/kd_tree.pyx":105
 *             d_lo = tree.node_bounds[0, i_node, j] - pt[j]
 *             d_hi = pt[j] - tree.node_bounds[1, i_node, j]
 *             d = (d_lo + fabs(d_lo)) + (d_hi + fabs(d_hi))             # <<<<<<<<<<<<<<
 *             rdist += pow(0.5 * d, tree.dist_metric.p)
 * 
 */
      __pyx_v_d = ((__pyx_v_d_lo + fabs(__pyx_v_d_lo)) + (__pyx_v_d_hi + fabs(__pyx_v_d_hi)));

      /* "sklearn/neighbors/kd_tree.pyx":106
 *             d_hi = pt[j] - tree.node_bounds[1, i_node, j]
 *             d = (d_lo + fabs(d_lo)) + (d_hi + fabs(d_hi))
 *             rdist += pow(0.5 * d, tree.dist_metric.p)             # <<<<<<<<<<<<<<
 * 
 *     return rdist
 */
      __pyx_v_rdist = (__pyx_v_rdist + pow((0.5 * __pyx_v_d), __pyx_v_tree->dist_metric->p));
    }
  }
  __pyx_L3:;

  /* "sklearn/neighbors/kd_tree.pyx":108
 *             rdist += pow(0.5 * d, tree.dist_metric.p)
 * 
 *     return rdist             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_rdist;
  goto __pyx_L0;

  /* "sklearn/neighbors/kd_tree.pyx":87
 * 
 * 
 * cdef DTYPE_t min_rdist(BinaryTree tree, ITYPE_t i_node,             # <<<<<<<<<<<<<<
 *                        DTYPE_t* pt) nogil except -1:
 *     """Compute the minimum reduced-distance between a point and a node"""
 */

  /* function exit code */
  __pyx_L1_error:;
  {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
    #endif
    __Pyx_AddTraceback("sklearn.neighbors.kd_tree.min_rdist", __pyx_clineno, __pyx_lineno, __pyx_filename);
    #ifdef WITH_THREAD
    PyGILState_Release(__pyx_gilstate_save);
    #endif
  }
  __pyx_r = -1.0;
  __pyx_L0:;
  return __pyx_r;
}

/* "sklearn/neighbors/kd_tree.pyx":111
 * 
 * 
 * cdef DTYPE_t min_dist(BinaryTree tree, ITYPE_t i_node, DTYPE_t* pt) except -1:             # <<<<<<<<<<<<<<
 *     """Compute the minimum distance between a point and a node"""
 *     if tree.dist_metric.p == INF:
 */

static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_min_dist(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_tree, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("min_dist", 0);

  /* "sklearn/neighbors/kd_tree.pyx":113
 * cdef DTYPE_t min_dist(BinaryTree tree, ITYPE_t i_node, DTYPE_t* pt) except -1:
 *     """Compute the minimum distance between a point and a node"""
 *     if tree.dist_metric.p == INF:             # <<<<<<<<<<<<<<
 *         return min_rdist(tree, i_node, pt)
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_tree->dist_metric->p == __pyx_v_7sklearn_9neighbors_7kd_tree_INF) != 0);
  if (__pyx_t_1) {

    /* "sklearn/neighbors/kd_tree.pyx":114
 *     """Compute the minimum distance between a point and a node"""
 *     if tree.dist_metric.p == INF:
 *         return min_rdist(tree, i_node, pt)             # <<<<<<<<<<<<<<
 *     else:
 *         return pow(min_rdist(tree, i_node, pt), 1. / tree.dist_metric.p)
 */
    __pyx_t_2 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_rdist(__pyx_v_tree, __pyx_v_i_node, __pyx_v_pt); if (unlikely(__pyx_t_2 == -1.0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 114; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_r = __pyx_t_2;
    goto __pyx_L0;
  }
  /*else*/ {

    /* "sklearn/neighbors/kd_tree.pyx":116
 *         return min_rdist(tree, i_node, pt)
 *     else:
 *         return pow(min_rdist(tree, i_node, pt), 1. / tree.dist_metric.p)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_2 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_rdist(__pyx_v_tree, __pyx_v_i_node, __pyx_v_pt); if (unlikely(__pyx_t_2 == -1.0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 116; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_r = pow(__pyx_t_2, (1. / __pyx_v_tree->dist_metric->p));
    goto __pyx_L0;
  }

  /* "sklearn/neighbors/kd_tree.pyx":111
 * 
 * 
 * cdef DTYPE_t min_dist(BinaryTree tree, ITYPE_t i_node, DTYPE_t* pt) except -1:             # <<<<<<<<<<<<<<
 *     """Compute the minimum distance between a point and a node"""
 *     if tree.dist_metric.p == INF:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.min_dist", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1.0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/neighbors/kd_tree.pyx":119
 * 
 * 
 * cdef DTYPE_t max_rdist(BinaryTree tree,             # <<<<<<<<<<<<<<
 *                        ITYPE_t i_node, DTYPE_t* pt) except -1:
 *     """Compute the maximum reduced-distance between a point and a node"""
 */

static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_max_rdist(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_tree, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_features;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_d_lo;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_d_hi;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_rdist;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_j;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_5;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_8;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_11;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_14;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("max_rdist", 0);

  /* "sklearn/neighbors/kd_tree.pyx":122
 *                        ITYPE_t i_node, DTYPE_t* pt) except -1:
 *     """Compute the maximum reduced-distance between a point and a node"""
 *     cdef ITYPE_t n_features = tree.data.shape[1]             # <<<<<<<<<<<<<<
 * 
 *     cdef DTYPE_t d, d_lo, d_hi, rdist=0.0
 */
  if (unlikely(!__pyx_v_tree->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 122; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_n_features = (__pyx_v_tree->data.shape[1]);

  /* "sklearn/neighbors/kd_tree.pyx":124
 *     cdef ITYPE_t n_features = tree.data.shape[1]
 * 
 *     cdef DTYPE_t d, d_lo, d_hi, rdist=0.0             # <<<<<<<<<<<<<<
 *     cdef ITYPE_t j
 * 
 */
  __pyx_v_rdist = 0.0;

  /* "sklearn/neighbors/kd_tree.pyx":127
 *     cdef ITYPE_t j
 * 
 *     if tree.dist_metric.p == INF:             # <<<<<<<<<<<<<<
 *         for j in range(n_features):
 *             rdist = fmax(rdist, fabs(pt[j] - tree.node_bounds[0, i_node, j]))
 */
  __pyx_t_1 = ((__pyx_v_tree->dist_metric->p == __pyx_v_7sklearn_9neighbors_7kd_tree_INF) != 0);
  if (__pyx_t_1) {

    /* "sklearn/neighbors/kd_tree.pyx":128
 * 
 *     if tree.dist_metric.p == INF:
 *         for j in range(n_features):             # <<<<<<<<<<<<<<
 *             rdist = fmax(rdist, fabs(pt[j] - tree.node_bounds[0, i_node, j]))
 *             rdist = fmax(rdist, fabs(pt[j] - tree.node_bounds[1, i_node, j]))
 */
    __pyx_t_2 = __pyx_v_n_features;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_j = __pyx_t_3;

      /* "sklearn/neighbors/kd_tree.pyx":129
 *     if tree.dist_metric.p == INF:
 *         for j in range(n_features):
 *             rdist = fmax(rdist, fabs(pt[j] - tree.node_bounds[0, i_node, j]))             # <<<<<<<<<<<<<<
 *             rdist = fmax(rdist, fabs(pt[j] - tree.node_bounds[1, i_node, j]))
 *     else:
 */
      if (unlikely(!__pyx_v_tree->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 129; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_4 = 0;
      __pyx_t_5 = __pyx_v_i_node;
      __pyx_t_6 = __pyx_v_j;
      __pyx_v_rdist = __pyx_f_7sklearn_9neighbors_7kd_tree_fmax(__pyx_v_rdist, fabs(((__pyx_v_pt[__pyx_v_j]) - (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree->node_bounds.data + __pyx_t_4 * __pyx_v_tree->node_bounds.strides[0]) ) + __pyx_t_5 * __pyx_v_tree->node_bounds.strides[1]) )) + __pyx_t_6)) ))))));

      /* "sklearn/neighbors/kd_tree.pyx":130
 *         for j in range(n_features):
 *             rdist = fmax(rdist, fabs(pt[j] - tree.node_bounds[0, i_node, j]))
 *             rdist = fmax(rdist, fabs(pt[j] - tree.node_bounds[1, i_node, j]))             # <<<<<<<<<<<<<<
 *     else:
 *         for j in range(n_features):
 */
      if (unlikely(!__pyx_v_tree->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 130; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_7 = 1;
      __pyx_t_8 = __pyx_v_i_node;
      __pyx_t_9 = __pyx_v_j;
      __pyx_v_rdist = __pyx_f_7sklearn_9neighbors_7kd_tree_fmax(__pyx_v_rdist, fabs(((__pyx_v_pt[__pyx_v_j]) - (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree->node_bounds.data + __pyx_t_7 * __pyx_v_tree->node_bounds.strides[0]) ) + __pyx_t_8 * __pyx_v_tree->node_bounds.strides[1]) )) + __pyx_t_9)) ))))));
    }
    goto __pyx_L3;
  }
  /*else*/ {

    /* "sklearn/neighbors/kd_tree.pyx":132
 *             rdist = fmax(rdist, fabs(pt[j] - tree.node_bounds[1, i_node, j]))
 *     else:
 *         for j in range(n_features):             # <<<<<<<<<<<<<<
 *             d_lo = fabs(pt[j] - tree.node_bounds[0, i_node, j])
 *             d_hi = fabs(pt[j] - tree.node_bounds[1, i_node, j])
 */
    __pyx_t_2 = __pyx_v_n_features;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_j = __pyx_t_3;

      /* "sklearn/neighbors/kd_tree.pyx":133
 *     else:
 *         for j in range(n_features):
 *             d_lo = fabs(pt[j] - tree.node_bounds[0, i_node, j])             # <<<<<<<<<<<<<<
 *             d_hi = fabs(pt[j] - tree.node_bounds[1, i_node, j])
 *             rdist += pow(fmax(d_lo, d_hi), tree.dist_metric.p)
 */
      if (unlikely(!__pyx_v_tree->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 133; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_10 = 0;
      __pyx_t_11 = __pyx_v_i_node;
      __pyx_t_12 = __pyx_v_j;
      __pyx_v_d_lo = fabs(((__pyx_v_pt[__pyx_v_j]) - (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree->node_bounds.data + __pyx_t_10 * __pyx_v_tree->node_bounds.strides[0]) ) + __pyx_t_11 * __pyx_v_tree->node_bounds.strides[1]) )) + __pyx_t_12)) )))));

      /* "sklearn/neighbors/kd_tree.pyx":134
 *         for j in range(n_features):
 *             d_lo = fabs(pt[j] - tree.node_bounds[0, i_node, j])
 *             d_hi = fabs(pt[j] - tree.node_bounds[1, i_node, j])             # <<<<<<<<<<<<<<
 *             rdist += pow(fmax(d_lo, d_hi), tree.dist_metric.p)
 * 
 */
      if (unlikely(!__pyx_v_tree->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 134; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_13 = 1;
      __pyx_t_14 = __pyx_v_i_node;
      __pyx_t_15 = __pyx_v_j;
      __pyx_v_d_hi = fabs(((__pyx_v_pt[__pyx_v_j]) - (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree->node_bounds.data + __pyx_t_13 * __pyx_v_tree->node_bounds.strides[0]) ) + __pyx_t_14 * __pyx_v_tree->node_bounds.strides[1]) )) + __pyx_t_15)) )))));

      /* "sklearn/neighbors/kd_tree.pyx":135
 *             d_lo = fabs(pt[j] - tree.node_bounds[0, i_node, j])
 *             d_hi = fabs(pt[j] - tree.node_bounds[1, i_node, j])
 *             rdist += pow(fmax(d_lo, d_hi), tree.dist_metric.p)             # <<<<<<<<<<<<<<
 * 
 *     return rdist
 */
      __pyx_v_rdist = (__pyx_v_rdist + pow(__pyx_f_7sklearn_9neighbors_7kd_tree_fmax(__pyx_v_d_lo, __pyx_v_d_hi), __pyx_v_tree->dist_metric->p));
    }
  }
  __pyx_L3:;

  /* "sklearn/neighbors/kd_tree.pyx":137
 *             rdist += pow(fmax(d_lo, d_hi), tree.dist_metric.p)
 * 
 *     return rdist             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_rdist;
  goto __pyx_L0;

  /* "sklearn/neighbors/kd_tree.pyx":119
 * 
 * 
 * cdef DTYPE_t max_rdist(BinaryTree tree,             # <<<<<<<<<<<<<<
 *                        ITYPE_t i_node, DTYPE_t* pt) except -1:
 *     """Compute the maximum reduced-distance between a point and a node"""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.max_rdist", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1.0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/neighbors/kd_tree.pyx":140
 * 
 * 
 * cdef DTYPE_t max_dist(BinaryTree tree, ITYPE_t i_node, DTYPE_t* pt) except -1:             # <<<<<<<<<<<<<<
 *     """Compute the maximum distance between a point and a node"""
 *     if tree.dist_metric.p == INF:
 */

static __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_max_dist(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_tree, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("max_dist", 0);

  /* "sklearn/neighbors/kd_tree.pyx":142
 * cdef DTYPE_t max_dist(BinaryTree tree, ITYPE_t i_node, DTYPE_t* pt) except -1:
 *     """Compute the maximum distance between a point and a node"""
 *     if tree.dist_metric.p == INF:             # <<<<<<<<<<<<<<
 *         return max_rdist(tree, i_node, pt)
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_tree->dist_metric->p == __pyx_v_7sklearn_9neighbors_7kd_tree_INF) != 0);
  if (__pyx_t_1) {

    /* "sklearn/neighbors/kd_tree.pyx":143
 *     """Compute the maximum distance between a point and a node"""
 *     if tree.dist_metric.p == INF:
 *         return max_rdist(tree, i_node, pt)             # <<<<<<<<<<<<<<
 *     else:
 *         return pow(max_rdist(tree, i_node, pt), 1. / tree.dist_metric.p)
 */
    __pyx_t_2 = __pyx_f_7sklearn_9neighbors_7kd_tree_max_rdist(__pyx_v_tree, __pyx_v_i_node, __pyx_v_pt); if (unlikely(__pyx_t_2 == -1.0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 143; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_r = __pyx_t_2;
    goto __pyx_L0;
  }
  /*else*/ {

    /* "sklearn/neighbors/kd_tree.pyx":145
 *         return max_rdist(tree, i_node, pt)
 *     else:
 *         return pow(max_rdist(tree, i_node, pt), 1. / tree.dist_metric.p)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_2 = __pyx_f_7sklearn_9neighbors_7kd_tree_max_rdist(__pyx_v_tree, __pyx_v_i_node, __pyx_v_pt); if (unlikely(__pyx_t_2 == -1.0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 145; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_r = pow(__pyx_t_2, (1. / __pyx_v_tree->dist_metric->p));
    goto __pyx_L0;
  }

  /* "sklearn/neighbors/kd_tree.pyx":140
 * 
 * 
 * cdef DTYPE_t max_dist(BinaryTree tree, ITYPE_t i_node, DTYPE_t* pt) except -1:             # <<<<<<<<<<<<<<
 *     """Compute the maximum distance between a point and a node"""
 *     if tree.dist_metric.p == INF:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.max_dist", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1.0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/neighbors/kd_tree.pyx":148
 * 
 * 
 * cdef inline int min_max_dist(BinaryTree tree, ITYPE_t i_node, DTYPE_t* pt,             # <<<<<<<<<<<<<<
 *                              DTYPE_t* min_dist, DTYPE_t* max_dist) except -1:
 *     """Compute the minimum and maximum distance between a point and a node"""
 */

static CYTHON_INLINE int __pyx_f_7sklearn_9neighbors_7kd_tree_min_max_dist(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_tree, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_pt, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_min_dist, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_max_dist) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_features;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_d;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_d_lo;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_d_hi;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_j;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_5;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_8;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_11;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_14;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_17;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_20;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_21;
  long __pyx_t_22;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("min_max_dist", 0);

  /* "sklearn/neighbors/kd_tree.pyx":151
 *                              DTYPE_t* min_dist, DTYPE_t* max_dist) except -1:
 *     """Compute the minimum and maximum distance between a point and a node"""
 *     cdef ITYPE_t n_features = tree.data.shape[1]             # <<<<<<<<<<<<<<
 * 
 *     cdef DTYPE_t d, d_lo, d_hi
 */
  if (unlikely(!__pyx_v_tree->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 151; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_n_features = (__pyx_v_tree->data.shape[1]);

  /* "sklearn/neighbors/kd_tree.pyx":156
 *     cdef ITYPE_t j
 * 
 *     min_dist[0] = 0.0             # <<<<<<<<<<<<<<
 *     max_dist[0] = 0.0
 * 
 */
  (__pyx_v_min_dist[0]) = 0.0;

  /* "sklearn/neighbors/kd_tree.pyx":157
 * 
 *     min_dist[0] = 0.0
 *     max_dist[0] = 0.0             # <<<<<<<<<<<<<<
 * 
 *     if tree.dist_metric.p == INF:
 */
  (__pyx_v_max_dist[0]) = 0.0;

  /* "sklearn/neighbors/kd_tree.pyx":159
 *     max_dist[0] = 0.0
 * 
 *     if tree.dist_metric.p == INF:             # <<<<<<<<<<<<<<
 *         for j in range(n_features):
 *             d_lo = tree.node_bounds[0, i_node, j] - pt[j]
 */
  __pyx_t_1 = ((__pyx_v_tree->dist_metric->p == __pyx_v_7sklearn_9neighbors_7kd_tree_INF) != 0);
  if (__pyx_t_1) {

    /* "sklearn/neighbors/kd_tree.pyx":160
 * 
 *     if tree.dist_metric.p == INF:
 *         for j in range(n_features):             # <<<<<<<<<<<<<<
 *             d_lo = tree.node_bounds[0, i_node, j] - pt[j]
 *             d_hi = pt[j] - tree.node_bounds[1, i_node, j]
 */
    __pyx_t_2 = __pyx_v_n_features;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_j = __pyx_t_3;

      /* "sklearn/neighbors/kd_tree.pyx":161
 *     if tree.dist_metric.p == INF:
 *         for j in range(n_features):
 *             d_lo = tree.node_bounds[0, i_node, j] - pt[j]             # <<<<<<<<<<<<<<
 *             d_hi = pt[j] - tree.node_bounds[1, i_node, j]
 *             d = (d_lo + fabs(d_lo)) + (d_hi + fabs(d_hi))
 */
      if (unlikely(!__pyx_v_tree->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 161; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_4 = 0;
      __pyx_t_5 = __pyx_v_i_node;
      __pyx_t_6 = __pyx_v_j;
      __pyx_v_d_lo = ((*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree->node_bounds.data + __pyx_t_4 * __pyx_v_tree->node_bounds.strides[0]) ) + __pyx_t_5 * __pyx_v_tree->node_bounds.strides[1]) )) + __pyx_t_6)) ))) - (__pyx_v_pt[__pyx_v_j]));

      /* "sklearn/neighbors/kd_tree.pyx":162
 *         for j in range(n_features):
 *             d_lo = tree.node_bounds[0, i_node, j] - pt[j]
 *             d_hi = pt[j] - tree.node_bounds[1, i_node, j]             # <<<<<<<<<<<<<<
 *             d = (d_lo + fabs(d_lo)) + (d_hi + fabs(d_hi))
 *             min_dist[0] = fmax(min_dist[0], 0.5 * d)
 */
      if (unlikely(!__pyx_v_tree->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 162; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_7 = 1;
      __pyx_t_8 = __pyx_v_i_node;
      __pyx_t_9 = __pyx_v_j;
      __pyx_v_d_hi = ((__pyx_v_pt[__pyx_v_j]) - (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree->node_bounds.data + __pyx_t_7 * __pyx_v_tree->node_bounds.strides[0]) ) + __pyx_t_8 * __pyx_v_tree->node_bounds.strides[1]) )) + __pyx_t_9)) ))));

      /* "sklearn/neighbors/kd_tree.pyx":163
 *             d_lo = tree.node_bounds[0, i_node, j] - pt[j]
 *             d_hi = pt[j] - tree.node_bounds[1, i_node, j]
 *             d = (d_lo + fabs(d_lo)) + (d_hi + fabs(d_hi))             # <<<<<<<<<<<<<<
 *             min_dist[0] = fmax(min_dist[0], 0.5 * d)
 *             max_dist[0] = fmax(max_dist[0],
 */
      __pyx_v_d = ((__pyx_v_d_lo + fabs(__pyx_v_d_lo)) + (__pyx_v_d_hi + fabs(__pyx_v_d_hi)));

      /* "sklearn/neighbors/kd_tree.pyx":164
 *             d_hi = pt[j] - tree.node_bounds[1, i_node, j]
 *             d = (d_lo + fabs(d_lo)) + (d_hi + fabs(d_hi))
 *             min_dist[0] = fmax(min_dist[0], 0.5 * d)             # <<<<<<<<<<<<<<
 *             max_dist[0] = fmax(max_dist[0],
 *                                fabs(pt[j] - tree.node_bounds[0, i_node, j]))
 */
      (__pyx_v_min_dist[0]) = __pyx_f_7sklearn_9neighbors_7kd_tree_fmax((__pyx_v_min_dist[0]), (0.5 * __pyx_v_d));

      /* "sklearn/neighbors/kd_tree.pyx":166
 *             min_dist[0] = fmax(min_dist[0], 0.5 * d)
 *             max_dist[0] = fmax(max_dist[0],
 *                                fabs(pt[j] - tree.node_bounds[0, i_node, j]))             # <<<<<<<<<<<<<<
 *             max_dist[0] = fmax(max_dist[0],
 *                                fabs(pt[j] - tree.node_bounds[1, i_node, j]))
 */
      if (unlikely(!__pyx_v_tree->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 166; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_10 = 0;
      __pyx_t_11 = __pyx_v_i_node;
      __pyx_t_12 = __pyx_v_j;

      /* "sklearn/neighbors/kd_tree.pyx":165
 *             d = (d_lo + fabs(d_lo)) + (d_hi + fabs(d_hi))
 *             min_dist[0] = fmax(min_dist[0], 0.5 * d)
 *             max_dist[0] = fmax(max_dist[0],             # <<<<<<<<<<<<<<
 *                                fabs(pt[j] - tree.node_bounds[0, i_node, j]))
 *             max_dist[0] = fmax(max_dist[0],
 */
      (__pyx_v_max_dist[0]) = __pyx_f_7sklearn_9neighbors_7kd_tree_fmax((__pyx_v_max_dist[0]), fabs(((__pyx_v_pt[__pyx_v_j]) - (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree->node_bounds.data + __pyx_t_10 * __pyx_v_tree->node_bounds.strides[0]) ) + __pyx_t_11 * __pyx_v_tree->node_bounds.strides[1]) )) + __pyx_t_12)) ))))));

      /* "sklearn/neighbors/kd_tree.pyx":168
 *                                fabs(pt[j] - tree.node_bounds[0, i_node, j]))
 *             max_dist[0] = fmax(max_dist[0],
 *                                fabs(pt[j] - tree.node_bounds[1, i_node, j]))             # <<<<<<<<<<<<<<
 *     else:
 *         # as above, use the fact that x + abs(x) = 2 * max(x, 0)
 */
      if (unlikely(!__pyx_v_tree->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_13 = 1;
      __pyx_t_14 = __pyx_v_i_node;
      __pyx_t_15 = __pyx_v_j;

      /* "sklearn/neighbors/kd_tree.pyx":167
 *             max_dist[0] = fmax(max_dist[0],
 *                                fabs(pt[j] - tree.node_bounds[0, i_node, j]))
 *             max_dist[0] = fmax(max_dist[0],             # <<<<<<<<<<<<<<
 *                                fabs(pt[j] - tree.node_bounds[1, i_node, j]))
 *     else:
 */
      (__pyx_v_max_dist[0]) = __pyx_f_7sklearn_9neighbors_7kd_tree_fmax((__pyx_v_max_dist[0]), fabs(((__pyx_v_pt[__pyx_v_j]) - (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree->node_bounds.data + __pyx_t_13 * __pyx_v_tree->node_bounds.strides[0]) ) + __pyx_t_14 * __pyx_v_tree->node_bounds.strides[1]) )) + __pyx_t_15)) ))))));
    }
    goto __pyx_L3;
  }
  /*else*/ {

    /* "sklearn/neighbors/kd_tree.pyx":171
 *     else:
 *         # as above, use the fact that x + abs(x) = 2 * max(x, 0)
 *         for j in range(n_features):             # <<<<<<<<<<<<<<
 *             d_lo = tree.node_bounds[0, i_node, j] - pt[j]
 *             d_hi = pt[j] - tree.node_bounds[1, i_node, j]
 */
    __pyx_t_2 = __pyx_v_n_features;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_j = __pyx_t_3;

      /* "sklearn/neighbors/kd_tree.pyx":172
 *         # as above, use the fact that x + abs(x) = 2 * max(x, 0)
 *         for j in range(n_features):
 *             d_lo = tree.node_bounds[0, i_node, j] - pt[j]             # <<<<<<<<<<<<<<
 *             d_hi = pt[j] - tree.node_bounds[1, i_node, j]
 *             d = (d_lo + fabs(d_lo)) + (d_hi + fabs(d_hi))
 */
      if (unlikely(!__pyx_v_tree->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_16 = 0;
      __pyx_t_17 = __pyx_v_i_node;
      __pyx_t_18 = __pyx_v_j;
      __pyx_v_d_lo = ((*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree->node_bounds.data + __pyx_t_16 * __pyx_v_tree->node_bounds.strides[0]) ) + __pyx_t_17 * __pyx_v_tree->node_bounds.strides[1]) )) + __pyx_t_18)) ))) - (__pyx_v_pt[__pyx_v_j]));

      /* "sklearn/neighbors/kd_tree.pyx":173
 *         for j in range(n_features):
 *             d_lo = tree.node_bounds[0, i_node, j] - pt[j]
 *             d_hi = pt[j] - tree.node_bounds[1, i_node, j]             # <<<<<<<<<<<<<<
 *             d = (d_lo + fabs(d_lo)) + (d_hi + fabs(d_hi))
 *             min_dist[0] += pow(0.5 * d, tree.dist_metric.p)
 */
      if (unlikely(!__pyx_v_tree->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 173; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_19 = 1;
      __pyx_t_20 = __pyx_v_i_node;
      __pyx_t_21 = __pyx_v_j;
      __pyx_v_d_hi = ((__pyx_v_pt[__pyx_v_j]) - (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree->node_bounds.data + __pyx_t_19 * __pyx_v_tree->node_bounds.strides[0]) ) + __pyx_t_20 * __pyx_v_tree->node_bounds.strides[1]) )) + __pyx_t_21)) ))));

      /* "sklearn/neighbors/kd_tree.pyx":174
 *             d_lo = tree.node_bounds[0, i_node, j] - pt[j]
 *             d_hi = pt[j] - tree.node_bounds[1, i_node, j]
 *             d = (d_lo + fabs(d_lo)) + (d_hi + fabs(d_hi))             # <<<<<<<<<<<<<<
 *             min_dist[0] += pow(0.5 * d, tree.dist_metric.p)
 *             max_dist[0] += pow(fmax(fabs(d_lo), fabs(d_hi)),
 */
      __pyx_v_d = ((__pyx_v_d_lo + fabs(__pyx_v_d_lo)) + (__pyx_v_d_hi + fabs(__pyx_v_d_hi)));

      /* "sklearn/neighbors/kd_tree.pyx":175
 *             d_hi = pt[j] - tree.node_bounds[1, i_node, j]
 *             d = (d_lo + fabs(d_lo)) + (d_hi + fabs(d_hi))
 *             min_dist[0] += pow(0.5 * d, tree.dist_metric.p)             # <<<<<<<<<<<<<<
 *             max_dist[0] += pow(fmax(fabs(d_lo), fabs(d_hi)),
 *                                tree.dist_metric.p)
 */
      __pyx_t_22 = 0;
      (__pyx_v_min_dist[__pyx_t_22]) = ((__pyx_v_min_dist[__pyx_t_22]) + pow((0.5 * __pyx_v_d), __pyx_v_tree->dist_metric->p));

      /* "sklearn/neighbors/kd_tree.pyx":176
 *             d = (d_lo + fabs(d_lo)) + (d_hi + fabs(d_hi))
 *             min_dist[0] += pow(0.5 * d, tree.dist_metric.p)
 *             max_dist[0] += pow(fmax(fabs(d_lo), fabs(d_hi)),             # <<<<<<<<<<<<<<
 *                                tree.dist_metric.p)
 * 
 */
      __pyx_t_22 = 0;

      /* "sklearn/neighbors/kd_tree.pyx":177
 *             min_dist[0] += pow(0.5 * d, tree.dist_metric.p)
 *             max_dist[0] += pow(fmax(fabs(d_lo), fabs(d_hi)),
 *                                tree.dist_metric.p)             # <<<<<<<<<<<<<<
 * 
 *         min_dist[0] = pow(min_dist[0], 1. / tree.dist_metric.p)
 */
      (__pyx_v_max_dist[__pyx_t_22]) = ((__pyx_v_max_dist[__pyx_t_22]) + pow(__pyx_f_7sklearn_9neighbors_7kd_tree_fmax(fabs(__pyx_v_d_lo), fabs(__pyx_v_d_hi)), __pyx_v_tree->dist_metric->p));
    }

    /* "sklearn/neighbors/kd_tree.pyx":179
 *                                tree.dist_metric.p)
 * 
 *         min_dist[0] = pow(min_dist[0], 1. / tree.dist_metric.p)             # <<<<<<<<<<<<<<
 *         max_dist[0] = pow(max_dist[0], 1. / tree.dist_metric.p)
 * 
 */
    (__pyx_v_min_dist[0]) = pow((__pyx_v_min_dist[0]), (1. / __pyx_v_tree->dist_metric->p));

    /* "sklearn/neighbors/kd_tree.pyx":180
 * 
 *         min_dist[0] = pow(min_dist[0], 1. / tree.dist_metric.p)
 *         max_dist[0] = pow(max_dist[0], 1. / tree.dist_metric.p)             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
    (__pyx_v_max_dist[0]) = pow((__pyx_v_max_dist[0]), (1. / __pyx_v_tree->dist_metric->p));
  }
  __pyx_L3:;

  /* "sklearn/neighbors/kd_tree.pyx":182
 *         max_dist[0] = pow(max_dist[0], 1. / tree.dist_metric.p)
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "sklearn/neighbors/kd_tree.pyx":148
 * 
 * 
 * cdef inline int min_max_dist(BinaryTree tree, ITYPE_t i_node, DTYPE_t* pt,             # <<<<<<<<<<<<<<
 *                              DTYPE_t* min_dist, DTYPE_t* max_dist) except -1:
 *     """Compute the minimum and maximum distance between a point and a node"""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.min_max_dist", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/neighbors/kd_tree.pyx":185
 * 
 * 
 * cdef inline DTYPE_t min_rdist_dual(BinaryTree tree1, ITYPE_t i_node1,             # <<<<<<<<<<<<<<
 *                                    BinaryTree tree2, ITYPE_t i_node2) except -1:
 *     """Compute the minimum reduced distance between two nodes"""
 */

static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_min_rdist_dual(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_tree1, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node1, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_tree2, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node2) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_features;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_d;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_d1;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_d2;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_rdist;
  CYTHON_UNUSED __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_zero;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_j;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_5;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_8;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_11;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_14;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_17;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_20;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_23;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_26;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_27;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("min_rdist_dual", 0);

  /* "sklearn/neighbors/kd_tree.pyx":188
 *                                    BinaryTree tree2, ITYPE_t i_node2) except -1:
 *     """Compute the minimum reduced distance between two nodes"""
 *     cdef ITYPE_t n_features = tree1.data.shape[1]             # <<<<<<<<<<<<<<
 * 
 *     cdef DTYPE_t d, d1, d2, rdist=0.0
 */
  if (unlikely(!__pyx_v_tree1->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_n_features = (__pyx_v_tree1->data.shape[1]);

  /* "sklearn/neighbors/kd_tree.pyx":190
 *     cdef ITYPE_t n_features = tree1.data.shape[1]
 * 
 *     cdef DTYPE_t d, d1, d2, rdist=0.0             # <<<<<<<<<<<<<<
 *     cdef DTYPE_t zero = 0.0
 *     cdef ITYPE_t j
 */
  __pyx_v_rdist = 0.0;

  /* "sklearn/neighbors/kd_tree.pyx":191
 * 
 *     cdef DTYPE_t d, d1, d2, rdist=0.0
 *     cdef DTYPE_t zero = 0.0             # <<<<<<<<<<<<<<
 *     cdef ITYPE_t j
 * 
 */
  __pyx_v_zero = 0.0;

  /* "sklearn/neighbors/kd_tree.pyx":194
 *     cdef ITYPE_t j
 * 
 *     if tree1.dist_metric.p == INF:             # <<<<<<<<<<<<<<
 *         for j in range(n_features):
 *             d1 = (tree1.node_bounds[0, i_node1, j]
 */
  __pyx_t_1 = ((__pyx_v_tree1->dist_metric->p == __pyx_v_7sklearn_9neighbors_7kd_tree_INF) != 0);
  if (__pyx_t_1) {

    /* "sklearn/neighbors/kd_tree.pyx":195
 * 
 *     if tree1.dist_metric.p == INF:
 *         for j in range(n_features):             # <<<<<<<<<<<<<<
 *             d1 = (tree1.node_bounds[0, i_node1, j]
 *                   - tree2.node_bounds[1, i_node2, j])
 */
    __pyx_t_2 = __pyx_v_n_features;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_j = __pyx_t_3;

      /* "sklearn/neighbors/kd_tree.pyx":196
 *     if tree1.dist_metric.p == INF:
 *         for j in range(n_features):
 *             d1 = (tree1.node_bounds[0, i_node1, j]             # <<<<<<<<<<<<<<
 *                   - tree2.node_bounds[1, i_node2, j])
 *             d2 = (tree2.node_bounds[0, i_node2, j]
 */
      if (unlikely(!__pyx_v_tree1->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 196; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_4 = 0;
      __pyx_t_5 = __pyx_v_i_node1;
      __pyx_t_6 = __pyx_v_j;

      /* "sklearn/neighbors/kd_tree.pyx":197
 *         for j in range(n_features):
 *             d1 = (tree1.node_bounds[0, i_node1, j]
 *                   - tree2.node_bounds[1, i_node2, j])             # <<<<<<<<<<<<<<
 *             d2 = (tree2.node_bounds[0, i_node2, j]
 *                   - tree1.node_bounds[1, i_node1, j])
 */
      if (unlikely(!__pyx_v_tree2->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 197; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_7 = 1;
      __pyx_t_8 = __pyx_v_i_node2;
      __pyx_t_9 = __pyx_v_j;
      __pyx_v_d1 = ((*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree1->node_bounds.data + __pyx_t_4 * __pyx_v_tree1->node_bounds.strides[0]) ) + __pyx_t_5 * __pyx_v_tree1->node_bounds.strides[1]) )) + __pyx_t_6)) ))) - (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree2->node_bounds.data + __pyx_t_7 * __pyx_v_tree2->node_bounds.strides[0]) ) + __pyx_t_8 * __pyx_v_tree2->node_bounds.strides[1]) )) + __pyx_t_9)) ))));

      /* "sklearn/neighbors/kd_tree.pyx":198
 *             d1 = (tree1.node_bounds[0, i_node1, j]
 *                   - tree2.node_bounds[1, i_node2, j])
 *             d2 = (tree2.node_bounds[0, i_node2, j]             # <<<<<<<<<<<<<<
 *                   - tree1.node_bounds[1, i_node1, j])
 *             d = (d1 + fabs(d1)) + (d2 + fabs(d2))
 */
      if (unlikely(!__pyx_v_tree2->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 198; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_10 = 0;
      __pyx_t_11 = __pyx_v_i_node2;
      __pyx_t_12 = __pyx_v_j;

      /* "sklearn/neighbors/kd_tree.pyx":199
 *                   - tree2.node_bounds[1, i_node2, j])
 *             d2 = (tree2.node_bounds[0, i_node2, j]
 *                   - tree1.node_bounds[1, i_node1, j])             # <<<<<<<<<<<<<<
 *             d = (d1 + fabs(d1)) + (d2 + fabs(d2))
 * 
 */
      if (unlikely(!__pyx_v_tree1->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 199; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_13 = 1;
      __pyx_t_14 = __pyx_v_i_node1;
      __pyx_t_15 = __pyx_v_j;
      __pyx_v_d2 = ((*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree2->node_bounds.data + __pyx_t_10 * __pyx_v_tree2->node_bounds.strides[0]) ) + __pyx_t_11 * __pyx_v_tree2->node_bounds.strides[1]) )) + __pyx_t_12)) ))) - (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree1->node_bounds.data + __pyx_t_13 * __pyx_v_tree1->node_bounds.strides[0]) ) + __pyx_t_14 * __pyx_v_tree1->node_bounds.strides[1]) )) + __pyx_t_15)) ))));

      /* "sklearn/neighbors/kd_tree.pyx":200
 *             d2 = (tree2.node_bounds[0, i_node2, j]
 *                   - tree1.node_bounds[1, i_node1, j])
 *             d = (d1 + fabs(d1)) + (d2 + fabs(d2))             # <<<<<<<<<<<<<<
 * 
 *             rdist = fmax(rdist, 0.5 * d)
 */
      __pyx_v_d = ((__pyx_v_d1 + fabs(__pyx_v_d1)) + (__pyx_v_d2 + fabs(__pyx_v_d2)));

      /* "sklearn/neighbors/kd_tree.pyx":202
 *             d = (d1 + fabs(d1)) + (d2 + fabs(d2))
 * 
 *             rdist = fmax(rdist, 0.5 * d)             # <<<<<<<<<<<<<<
 *     else:
 *         # here we'll use the fact that x + abs(x) = 2 * max(x, 0)
 */
      __pyx_v_rdist = __pyx_f_7sklearn_9neighbors_7kd_tree_fmax(__pyx_v_rdist, (0.5 * __pyx_v_d));
    }
    goto __pyx_L3;
  }
  /*else*/ {

    /* "sklearn/neighbors/kd_tree.pyx":205
 *     else:
 *         # here we'll use the fact that x + abs(x) = 2 * max(x, 0)
 *         for j in range(n_features):             # <<<<<<<<<<<<<<
 *             d1 = (tree1.node_bounds[0, i_node1, j]
 *                   - tree2.node_bounds[1, i_node2, j])
 */
    __pyx_t_2 = __pyx_v_n_features;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_j = __pyx_t_3;

      /* "sklearn/neighbors/kd_tree.pyx":206
 *         # here we'll use the fact that x + abs(x) = 2 * max(x, 0)
 *         for j in range(n_features):
 *             d1 = (tree1.node_bounds[0, i_node1, j]             # <<<<<<<<<<<<<<
 *                   - tree2.node_bounds[1, i_node2, j])
 *             d2 = (tree2.node_bounds[0, i_node2, j]
 */
      if (unlikely(!__pyx_v_tree1->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 206; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_16 = 0;
      __pyx_t_17 = __pyx_v_i_node1;
      __pyx_t_18 = __pyx_v_j;

      /* "sklearn/neighbors/kd_tree.pyx":207
 *         for j in range(n_features):
 *             d1 = (tree1.node_bounds[0, i_node1, j]
 *                   - tree2.node_bounds[1, i_node2, j])             # <<<<<<<<<<<<<<
 *             d2 = (tree2.node_bounds[0, i_node2, j]
 *                   - tree1.node_bounds[1, i_node1, j])
 */
      if (unlikely(!__pyx_v_tree2->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 207; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_19 = 1;
      __pyx_t_20 = __pyx_v_i_node2;
      __pyx_t_21 = __pyx_v_j;
      __pyx_v_d1 = ((*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree1->node_bounds.data + __pyx_t_16 * __pyx_v_tree1->node_bounds.strides[0]) ) + __pyx_t_17 * __pyx_v_tree1->node_bounds.strides[1]) )) + __pyx_t_18)) ))) - (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree2->node_bounds.data + __pyx_t_19 * __pyx_v_tree2->node_bounds.strides[0]) ) + __pyx_t_20 * __pyx_v_tree2->node_bounds.strides[1]) )) + __pyx_t_21)) ))));

      /* "sklearn/neighbors/kd_tree.pyx":208
 *             d1 = (tree1.node_bounds[0, i_node1, j]
 *                   - tree2.node_bounds[1, i_node2, j])
 *             d2 = (tree2.node_bounds[0, i_node2, j]             # <<<<<<<<<<<<<<
 *                   - tree1.node_bounds[1, i_node1, j])
 *             d = (d1 + fabs(d1)) + (d2 + fabs(d2))
 */
      if (unlikely(!__pyx_v_tree2->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 208; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_22 = 0;
      __pyx_t_23 = __pyx_v_i_node2;
      __pyx_t_24 = __pyx_v_j;

      /* "sklearn/neighbors/kd_tree.pyx":209
 *                   - tree2.node_bounds[1, i_node2, j])
 *             d2 = (tree2.node_bounds[0, i_node2, j]
 *                   - tree1.node_bounds[1, i_node1, j])             # <<<<<<<<<<<<<<
 *             d = (d1 + fabs(d1)) + (d2 + fabs(d2))
 * 
 */
      if (unlikely(!__pyx_v_tree1->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 209; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_25 = 1;
      __pyx_t_26 = __pyx_v_i_node1;
      __pyx_t_27 = __pyx_v_j;
      __pyx_v_d2 = ((*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree2->node_bounds.data + __pyx_t_22 * __pyx_v_tree2->node_bounds.strides[0]) ) + __pyx_t_23 * __pyx_v_tree2->node_bounds.strides[1]) )) + __pyx_t_24)) ))) - (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree1->node_bounds.data + __pyx_t_25 * __pyx_v_tree1->node_bounds.strides[0]) ) + __pyx_t_26 * __pyx_v_tree1->node_bounds.strides[1]) )) + __pyx_t_27)) ))));

      /* "sklearn/neighbors/kd_tree.pyx":210
 *             d2 = (tree2.node_bounds[0, i_node2, j]
 *                   - tree1.node_bounds[1, i_node1, j])
 *             d = (d1 + fabs(d1)) + (d2 + fabs(d2))             # <<<<<<<<<<<<<<
 * 
 *             rdist += pow(0.5 * d, tree1.dist_metric.p)
 */
      __pyx_v_d = ((__pyx_v_d1 + fabs(__pyx_v_d1)) + (__pyx_v_d2 + fabs(__pyx_v_d2)));

      /* "sklearn/neighbors/kd_tree.pyx":212
 *             d = (d1 + fabs(d1)) + (d2 + fabs(d2))
 * 
 *             rdist += pow(0.5 * d, tree1.dist_metric.p)             # <<<<<<<<<<<<<<
 * 
 *     return rdist
 */
      __pyx_v_rdist = (__pyx_v_rdist + pow((0.5 * __pyx_v_d), __pyx_v_tree1->dist_metric->p));
    }
  }
  __pyx_L3:;

  /* "sklearn/neighbors/kd_tree.pyx":214
 *             rdist += pow(0.5 * d, tree1.dist_metric.p)
 * 
 *     return rdist             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_rdist;
  goto __pyx_L0;

  /* "sklearn/neighbors/kd_tree.pyx":185
 * 
 * 
 * cdef inline DTYPE_t min_rdist_dual(BinaryTree tree1, ITYPE_t i_node1,             # <<<<<<<<<<<<<<
 *                                    BinaryTree tree2, ITYPE_t i_node2) except -1:
 *     """Compute the minimum reduced distance between two nodes"""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.min_rdist_dual", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1.0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/neighbors/kd_tree.pyx":217
 * 
 * 
 * cdef inline DTYPE_t min_dist_dual(BinaryTree tree1, ITYPE_t i_node1,             # <<<<<<<<<<<<<<
 *                                   BinaryTree tree2, ITYPE_t i_node2) except -1:
 *     """Compute the minimum distance between two nodes"""
 */

static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_min_dist_dual(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_tree1, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node1, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_tree2, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node2) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("min_dist_dual", 0);

  /* "sklearn/neighbors/kd_tree.pyx":220
 *                                   BinaryTree tree2, ITYPE_t i_node2) except -1:
 *     """Compute the minimum distance between two nodes"""
 *     return tree1.dist_metric._rdist_to_dist(min_rdist_dual(tree1, i_node1,             # <<<<<<<<<<<<<<
 *                                                            tree2, i_node2))
 * 
 */
  __pyx_t_1 = __pyx_f_7sklearn_9neighbors_7kd_tree_min_rdist_dual(__pyx_v_tree1, __pyx_v_i_node1, __pyx_v_tree2, __pyx_v_i_node2); if (unlikely(__pyx_t_1 == -1.0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_12dist_metrics_DistanceMetric *)__pyx_v_tree1->dist_metric->__pyx_vtab)->_rdist_to_dist(__pyx_v_tree1->dist_metric, __pyx_t_1); if (unlikely(__pyx_t_2 == -1.0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "sklearn/neighbors/kd_tree.pyx":217
 * 
 * 
 * cdef inline DTYPE_t min_dist_dual(BinaryTree tree1, ITYPE_t i_node1,             # <<<<<<<<<<<<<<
 *                                   BinaryTree tree2, ITYPE_t i_node2) except -1:
 *     """Compute the minimum distance between two nodes"""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.min_dist_dual", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1.0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/neighbors/kd_tree.pyx":224
 * 
 * 
 * cdef inline DTYPE_t max_rdist_dual(BinaryTree tree1, ITYPE_t i_node1,             # <<<<<<<<<<<<<<
 *                                    BinaryTree tree2, ITYPE_t i_node2) except -1:
 *     """Compute the maximum reduced distance between two nodes"""
 */

static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_max_rdist_dual(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_tree1, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node1, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_tree2, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node2) {
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_n_features;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_d1;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_d2;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_rdist;
  CYTHON_UNUSED __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_zero;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_j;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_2;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_5;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_8;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_11;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_14;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_17;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_20;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_23;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_26;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_27;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("max_rdist_dual", 0);

  /* "sklearn/neighbors/kd_tree.pyx":227
 *                                    BinaryTree tree2, ITYPE_t i_node2) except -1:
 *     """Compute the maximum reduced distance between two nodes"""
 *     cdef ITYPE_t n_features = tree1.data.shape[1]             # <<<<<<<<<<<<<<
 * 
 *     cdef DTYPE_t d, d1, d2, rdist=0.0
 */
  if (unlikely(!__pyx_v_tree1->data.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
  __pyx_v_n_features = (__pyx_v_tree1->data.shape[1]);

  /* "sklearn/neighbors/kd_tree.pyx":229
 *     cdef ITYPE_t n_features = tree1.data.shape[1]
 * 
 *     cdef DTYPE_t d, d1, d2, rdist=0.0             # <<<<<<<<<<<<<<
 *     cdef DTYPE_t zero = 0.0
 *     cdef ITYPE_t j
 */
  __pyx_v_rdist = 0.0;

  /* "sklearn/neighbors/kd_tree.pyx":230
 * 
 *     cdef DTYPE_t d, d1, d2, rdist=0.0
 *     cdef DTYPE_t zero = 0.0             # <<<<<<<<<<<<<<
 *     cdef ITYPE_t j
 * 
 */
  __pyx_v_zero = 0.0;

  /* "sklearn/neighbors/kd_tree.pyx":233
 *     cdef ITYPE_t j
 * 
 *     if tree1.dist_metric.p == INF:             # <<<<<<<<<<<<<<
 *         for j in range(n_features):
 *             rdist = fmax(rdist, fabs(tree1.node_bounds[0, i_node1, j]
 */
  __pyx_t_1 = ((__pyx_v_tree1->dist_metric->p == __pyx_v_7sklearn_9neighbors_7kd_tree_INF) != 0);
  if (__pyx_t_1) {

    /* "sklearn/neighbors/kd_tree.pyx":234
 * 
 *     if tree1.dist_metric.p == INF:
 *         for j in range(n_features):             # <<<<<<<<<<<<<<
 *             rdist = fmax(rdist, fabs(tree1.node_bounds[0, i_node1, j]
 *                                      - tree2.node_bounds[1, i_node2, j]))
 */
    __pyx_t_2 = __pyx_v_n_features;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_j = __pyx_t_3;

      /* "sklearn/neighbors/kd_tree.pyx":235
 *     if tree1.dist_metric.p == INF:
 *         for j in range(n_features):
 *             rdist = fmax(rdist, fabs(tree1.node_bounds[0, i_node1, j]             # <<<<<<<<<<<<<<
 *                                      - tree2.node_bounds[1, i_node2, j]))
 *             rdist = fmax(rdist, fabs(tree1.node_bounds[1, i_node1, j]
 */
      if (unlikely(!__pyx_v_tree1->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 235; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_4 = 0;
      __pyx_t_5 = __pyx_v_i_node1;
      __pyx_t_6 = __pyx_v_j;

      /* "sklearn/neighbors/kd_tree.pyx":236
 *         for j in range(n_features):
 *             rdist = fmax(rdist, fabs(tree1.node_bounds[0, i_node1, j]
 *                                      - tree2.node_bounds[1, i_node2, j]))             # <<<<<<<<<<<<<<
 *             rdist = fmax(rdist, fabs(tree1.node_bounds[1, i_node1, j]
 *                                      - tree2.node_bounds[0, i_node2, j]))
 */
      if (unlikely(!__pyx_v_tree2->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 236; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_7 = 1;
      __pyx_t_8 = __pyx_v_i_node2;
      __pyx_t_9 = __pyx_v_j;

      /* "sklearn/neighbors/kd_tree.pyx":235
 *     if tree1.dist_metric.p == INF:
 *         for j in range(n_features):
 *             rdist = fmax(rdist, fabs(tree1.node_bounds[0, i_node1, j]             # <<<<<<<<<<<<<<
 *                                      - tree2.node_bounds[1, i_node2, j]))
 *             rdist = fmax(rdist, fabs(tree1.node_bounds[1, i_node1, j]
 */
      __pyx_v_rdist = __pyx_f_7sklearn_9neighbors_7kd_tree_fmax(__pyx_v_rdist, fabs(((*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree1->node_bounds.data + __pyx_t_4 * __pyx_v_tree1->node_bounds.strides[0]) ) + __pyx_t_5 * __pyx_v_tree1->node_bounds.strides[1]) )) + __pyx_t_6)) ))) - (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree2->node_bounds.data + __pyx_t_7 * __pyx_v_tree2->node_bounds.strides[0]) ) + __pyx_t_8 * __pyx_v_tree2->node_bounds.strides[1]) )) + __pyx_t_9)) ))))));

      /* "sklearn/neighbors/kd_tree.pyx":237
 *             rdist = fmax(rdist, fabs(tree1.node_bounds[0, i_node1, j]
 *                                      - tree2.node_bounds[1, i_node2, j]))
 *             rdist = fmax(rdist, fabs(tree1.node_bounds[1, i_node1, j]             # <<<<<<<<<<<<<<
 *                                      - tree2.node_bounds[0, i_node2, j]))
 *     else:
 */
      if (unlikely(!__pyx_v_tree1->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 237; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_10 = 1;
      __pyx_t_11 = __pyx_v_i_node1;
      __pyx_t_12 = __pyx_v_j;

      /* "sklearn/neighbors/kd_tree.pyx":238
 *                                      - tree2.node_bounds[1, i_node2, j]))
 *             rdist = fmax(rdist, fabs(tree1.node_bounds[1, i_node1, j]
 *                                      - tree2.node_bounds[0, i_node2, j]))             # <<<<<<<<<<<<<<
 *     else:
 *         for j in range(n_features):
 */
      if (unlikely(!__pyx_v_tree2->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 238; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_13 = 0;
      __pyx_t_14 = __pyx_v_i_node2;
      __pyx_t_15 = __pyx_v_j;

      /* "sklearn/neighbors/kd_tree.pyx":237
 *             rdist = fmax(rdist, fabs(tree1.node_bounds[0, i_node1, j]
 *                                      - tree2.node_bounds[1, i_node2, j]))
 *             rdist = fmax(rdist, fabs(tree1.node_bounds[1, i_node1, j]             # <<<<<<<<<<<<<<
 *                                      - tree2.node_bounds[0, i_node2, j]))
 *     else:
 */
      __pyx_v_rdist = __pyx_f_7sklearn_9neighbors_7kd_tree_fmax(__pyx_v_rdist, fabs(((*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree1->node_bounds.data + __pyx_t_10 * __pyx_v_tree1->node_bounds.strides[0]) ) + __pyx_t_11 * __pyx_v_tree1->node_bounds.strides[1]) )) + __pyx_t_12)) ))) - (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree2->node_bounds.data + __pyx_t_13 * __pyx_v_tree2->node_bounds.strides[0]) ) + __pyx_t_14 * __pyx_v_tree2->node_bounds.strides[1]) )) + __pyx_t_15)) ))))));
    }
    goto __pyx_L3;
  }
  /*else*/ {

    /* "sklearn/neighbors/kd_tree.pyx":240
 *                                      - tree2.node_bounds[0, i_node2, j]))
 *     else:
 *         for j in range(n_features):             # <<<<<<<<<<<<<<
 *             d1 = fabs(tree1.node_bounds[0, i_node1, j]
 *                       - tree2.node_bounds[1, i_node2, j])
 */
    __pyx_t_2 = __pyx_v_n_features;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_j = __pyx_t_3;

      /* "sklearn/neighbors/kd_tree.pyx":241
 *     else:
 *         for j in range(n_features):
 *             d1 = fabs(tree1.node_bounds[0, i_node1, j]             # <<<<<<<<<<<<<<
 *                       - tree2.node_bounds[1, i_node2, j])
 *             d2 = fabs(tree1.node_bounds[1, i_node1, j]
 */
      if (unlikely(!__pyx_v_tree1->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_16 = 0;
      __pyx_t_17 = __pyx_v_i_node1;
      __pyx_t_18 = __pyx_v_j;

      /* "sklearn/neighbors/kd_tree.pyx":242
 *         for j in range(n_features):
 *             d1 = fabs(tree1.node_bounds[0, i_node1, j]
 *                       - tree2.node_bounds[1, i_node2, j])             # <<<<<<<<<<<<<<
 *             d2 = fabs(tree1.node_bounds[1, i_node1, j]
 *                       - tree2.node_bounds[0, i_node2, j])
 */
      if (unlikely(!__pyx_v_tree2->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 242; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_19 = 1;
      __pyx_t_20 = __pyx_v_i_node2;
      __pyx_t_21 = __pyx_v_j;

      /* "sklearn/neighbors/kd_tree.pyx":241
 *     else:
 *         for j in range(n_features):
 *             d1 = fabs(tree1.node_bounds[0, i_node1, j]             # <<<<<<<<<<<<<<
 *                       - tree2.node_bounds[1, i_node2, j])
 *             d2 = fabs(tree1.node_bounds[1, i_node1, j]
 */
      __pyx_v_d1 = fabs(((*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree1->node_bounds.data + __pyx_t_16 * __pyx_v_tree1->node_bounds.strides[0]) ) + __pyx_t_17 * __pyx_v_tree1->node_bounds.strides[1]) )) + __pyx_t_18)) ))) - (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree2->node_bounds.data + __pyx_t_19 * __pyx_v_tree2->node_bounds.strides[0]) ) + __pyx_t_20 * __pyx_v_tree2->node_bounds.strides[1]) )) + __pyx_t_21)) )))));

      /* "sklearn/neighbors/kd_tree.pyx":243
 *             d1 = fabs(tree1.node_bounds[0, i_node1, j]
 *                       - tree2.node_bounds[1, i_node2, j])
 *             d2 = fabs(tree1.node_bounds[1, i_node1, j]             # <<<<<<<<<<<<<<
 *                       - tree2.node_bounds[0, i_node2, j])
 *             rdist += pow(fmax(d1, d2), tree1.dist_metric.p)
 */
      if (unlikely(!__pyx_v_tree1->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_22 = 1;
      __pyx_t_23 = __pyx_v_i_node1;
      __pyx_t_24 = __pyx_v_j;

      /* "sklearn/neighbors/kd_tree.pyx":244
 *                       - tree2.node_bounds[1, i_node2, j])
 *             d2 = fabs(tree1.node_bounds[1, i_node1, j]
 *                       - tree2.node_bounds[0, i_node2, j])             # <<<<<<<<<<<<<<
 *             rdist += pow(fmax(d1, d2), tree1.dist_metric.p)
 * 
 */
      if (unlikely(!__pyx_v_tree2->node_bounds.memview)) {PyErr_SetString(PyExc_AttributeError,"Memoryview is not initialized");{__pyx_filename = __pyx_f[1]; __pyx_lineno = 244; __pyx_clineno = __LINE__; goto __pyx_L1_error;}}
      __pyx_t_25 = 0;
      __pyx_t_26 = __pyx_v_i_node2;
      __pyx_t_27 = __pyx_v_j;

      /* "sklearn/neighbors/kd_tree.pyx":243
 *             d1 = fabs(tree1.node_bounds[0, i_node1, j]
 *                       - tree2.node_bounds[1, i_node2, j])
 *             d2 = fabs(tree1.node_bounds[1, i_node1, j]             # <<<<<<<<<<<<<<
 *                       - tree2.node_bounds[0, i_node2, j])
 *             rdist += pow(fmax(d1, d2), tree1.dist_metric.p)
 */
      __pyx_v_d2 = fabs(((*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree1->node_bounds.data + __pyx_t_22 * __pyx_v_tree1->node_bounds.strides[0]) ) + __pyx_t_23 * __pyx_v_tree1->node_bounds.strides[1]) )) + __pyx_t_24)) ))) - (*((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=2 */ ((char *) (((__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_tree2->node_bounds.data + __pyx_t_25 * __pyx_v_tree2->node_bounds.strides[0]) ) + __pyx_t_26 * __pyx_v_tree2->node_bounds.strides[1]) )) + __pyx_t_27)) )))));

      /* "sklearn/neighbors/kd_tree.pyx":245
 *             d2 = fabs(tree1.node_bounds[1, i_node1, j]
 *                       - tree2.node_bounds[0, i_node2, j])
 *             rdist += pow(fmax(d1, d2), tree1.dist_metric.p)             # <<<<<<<<<<<<<<
 * 
 *     return rdist
 */
      __pyx_v_rdist = (__pyx_v_rdist + pow(__pyx_f_7sklearn_9neighbors_7kd_tree_fmax(__pyx_v_d1, __pyx_v_d2), __pyx_v_tree1->dist_metric->p));
    }
  }
  __pyx_L3:;

  /* "sklearn/neighbors/kd_tree.pyx":247
 *             rdist += pow(fmax(d1, d2), tree1.dist_metric.p)
 * 
 *     return rdist             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_rdist;
  goto __pyx_L0;

  /* "sklearn/neighbors/kd_tree.pyx":224
 * 
 * 
 * cdef inline DTYPE_t max_rdist_dual(BinaryTree tree1, ITYPE_t i_node1,             # <<<<<<<<<<<<<<
 *                                    BinaryTree tree2, ITYPE_t i_node2) except -1:
 *     """Compute the maximum reduced distance between two nodes"""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.max_rdist_dual", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1.0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/neighbors/kd_tree.pyx":250
 * 
 * 
 * cdef inline DTYPE_t max_dist_dual(BinaryTree tree1, ITYPE_t i_node1,             # <<<<<<<<<<<<<<
 *                                   BinaryTree tree2, ITYPE_t i_node2) except -1:
 *     """Compute the maximum distance between two nodes"""
 */

static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_7kd_tree_max_dist_dual(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_tree1, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node1, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *__pyx_v_tree2, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_i_node2) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_1;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("max_dist_dual", 0);

  /* "sklearn/neighbors/kd_tree.pyx":253
 *                                   BinaryTree tree2, ITYPE_t i_node2) except -1:
 *     """Compute the maximum distance between two nodes"""
 *     return tree1.dist_metric._rdist_to_dist(max_rdist_dual(tree1, i_node1,             # <<<<<<<<<<<<<<
 *                                                            tree2, i_node2))
 */
  __pyx_t_1 = __pyx_f_7sklearn_9neighbors_7kd_tree_max_rdist_dual(__pyx_v_tree1, __pyx_v_i_node1, __pyx_v_tree2, __pyx_v_i_node2); if (unlikely(__pyx_t_1 == -1.0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 253; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((struct __pyx_vtabstruct_7sklearn_9neighbors_12dist_metrics_DistanceMetric *)__pyx_v_tree1->dist_metric->__pyx_vtab)->_rdist_to_dist(__pyx_v_tree1->dist_metric, __pyx_t_1); if (unlikely(__pyx_t_2 == -1.0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 253; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "sklearn/neighbors/kd_tree.pyx":250
 * 
 * 
 * cdef inline DTYPE_t max_dist_dual(BinaryTree tree1, ITYPE_t i_node1,             # <<<<<<<<<<<<<<
 *                                   BinaryTree tree2, ITYPE_t i_node2) except -1:
 *     """Compute the maximum distance between two nodes"""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("sklearn.neighbors.kd_tree.max_dist_dual", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1.0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dist_metrics.pxd":18
 * #  We use these for the default (euclidean) case so that they can be
 * #  inlined.  This leads to faster computation for the most common case
 * cdef inline DTYPE_t euclidean_dist(DTYPE_t* x1, DTYPE_t* x2,             # <<<<<<<<<<<<<<
 *                                    ITYPE_t size) nogil except -1:
 *     cdef DTYPE_t tmp, d=0
 */

static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_12dist_metrics_euclidean_dist(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_x1, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_x2, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_size) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_tmp;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_d;
  __pyx_t_5numpy_intp_t __pyx_v_j;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_1;
  __pyx_t_5numpy_intp_t __pyx_t_2;

  /* "dist_metrics.pxd":20
 * cdef inline DTYPE_t euclidean_dist(DTYPE_t* x1, DTYPE_t* x2,
 *                                    ITYPE_t size) nogil except -1:
 *     cdef DTYPE_t tmp, d=0             # <<<<<<<<<<<<<<
 *     cdef np.intp_t j
 *     for j in range(size):
 */
  __pyx_v_d = 0.0;

  /* "dist_metrics.pxd":22
 *     cdef DTYPE_t tmp, d=0
 *     cdef np.intp_t j
 *     for j in range(size):             # <<<<<<<<<<<<<<
 *         tmp = x1[j] - x2[j]
 *         d += tmp * tmp
 */
  __pyx_t_1 = __pyx_v_size;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_j = __pyx_t_2;

    /* "dist_metrics.pxd":23
 *     cdef np.intp_t j
 *     for j in range(size):
 *         tmp = x1[j] - x2[j]             # <<<<<<<<<<<<<<
 *         d += tmp * tmp
 *     return sqrt(d)
 */
    __pyx_v_tmp = ((__pyx_v_x1[__pyx_v_j]) - (__pyx_v_x2[__pyx_v_j]));

    /* "dist_metrics.pxd":24
 *     for j in range(size):
 *         tmp = x1[j] - x2[j]
 *         d += tmp * tmp             # <<<<<<<<<<<<<<
 *     return sqrt(d)
 * 
 */
    __pyx_v_d = (__pyx_v_d + (__pyx_v_tmp * __pyx_v_tmp));
  }

  /* "dist_metrics.pxd":25
 *         tmp = x1[j] - x2[j]
 *         d += tmp * tmp
 *     return sqrt(d)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = sqrt(__pyx_v_d);
  goto __pyx_L0;

  /* "dist_metrics.pxd":18
 * #  We use these for the default (euclidean) case so that they can be
 * #  inlined.  This leads to faster computation for the most common case
 * cdef inline DTYPE_t euclidean_dist(DTYPE_t* x1, DTYPE_t* x2,             # <<<<<<<<<<<<<<
 *                                    ITYPE_t size) nogil except -1:
 *     cdef DTYPE_t tmp, d=0
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "dist_metrics.pxd":28
 * 
 * 
 * cdef inline DTYPE_t euclidean_rdist(DTYPE_t* x1, DTYPE_t* x2,             # <<<<<<<<<<<<<<
 *                                     ITYPE_t size) nogil except -1:
 *     cdef DTYPE_t tmp, d=0
 */

static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_12dist_metrics_euclidean_rdist(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_x1, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *__pyx_v_x2, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_v_size) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_tmp;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_d;
  __pyx_t_5numpy_intp_t __pyx_v_j;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t __pyx_t_1;
  __pyx_t_5numpy_intp_t __pyx_t_2;

  /* "dist_metrics.pxd":30
 * cdef inline DTYPE_t euclidean_rdist(DTYPE_t* x1, DTYPE_t* x2,
 *                                     ITYPE_t size) nogil except -1:
 *     cdef DTYPE_t tmp, d=0             # <<<<<<<<<<<<<<
 *     cdef np.intp_t j
 *     for j in range(size):
 */
  __pyx_v_d = 0.0;

  /* "dist_metrics.pxd":32
 *     cdef DTYPE_t tmp, d=0
 *     cdef np.intp_t j
 *     for j in range(size):             # <<<<<<<<<<<<<<
 *         tmp = x1[j] - x2[j]
 *         d += tmp * tmp
 */
  __pyx_t_1 = __pyx_v_size;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_j = __pyx_t_2;

    /* "dist_metrics.pxd":33
 *     cdef np.intp_t j
 *     for j in range(size):
 *         tmp = x1[j] - x2[j]             # <<<<<<<<<<<<<<
 *         d += tmp * tmp
 *     return d
 */
    __pyx_v_tmp = ((__pyx_v_x1[__pyx_v_j]) - (__pyx_v_x2[__pyx_v_j]));

    /* "dist_metrics.pxd":34
 *     for j in range(size):
 *         tmp = x1[j] - x2[j]
 *         d += tmp * tmp             # <<<<<<<<<<<<<<
 *     return d
 * 
 */
    __pyx_v_d = (__pyx_v_d + (__pyx_v_tmp * __pyx_v_tmp));
  }

  /* "dist_metrics.pxd":35
 *         tmp = x1[j] - x2[j]
 *         d += tmp * tmp
 *     return d             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_d;
  goto __pyx_L0;

  /* "dist_metrics.pxd":28
 * 
 * 
 * cdef inline DTYPE_t euclidean_rdist(DTYPE_t* x1, DTYPE_t* x2,             # <<<<<<<<<<<<<<
 *                                     ITYPE_t size) nogil except -1:
 *     cdef DTYPE_t tmp, d=0
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "dist_metrics.pxd":38
 * 
 * 
 * cdef inline DTYPE_t euclidean_dist_to_rdist(DTYPE_t dist) nogil except -1:             # <<<<<<<<<<<<<<
 *     return dist * dist
 * 
 */

static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_12dist_metrics_euclidean_dist_to_rdist(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;

  /* "dist_metrics.pxd":39
 * 
 * cdef inline DTYPE_t euclidean_dist_to_rdist(DTYPE_t dist) nogil except -1:
 *     return dist * dist             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = (__pyx_v_dist * __pyx_v_dist);
  goto __pyx_L0;

  /* "dist_metrics.pxd":38
 * 
 * 
 * cdef inline DTYPE_t euclidean_dist_to_rdist(DTYPE_t dist) nogil except -1:             # <<<<<<<<<<<<<<
 *     return dist * dist
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "dist_metrics.pxd":42
 * 
 * 
 * cdef inline DTYPE_t euclidean_rdist_to_dist(DTYPE_t dist) except -1:             # <<<<<<<<<<<<<<
 *     return sqrt(dist)
 * 
 */

static CYTHON_INLINE __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_f_7sklearn_9neighbors_12dist_metrics_euclidean_rdist_to_dist(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_v_dist) {
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("euclidean_rdist_to_dist", 0);

  /* "dist_metrics.pxd":43
 * 
 * cdef inline DTYPE_t euclidean_rdist_to_dist(DTYPE_t dist) except -1:
 *     return sqrt(dist)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = sqrt(__pyx_v_dist);
  goto __pyx_L0;

  /* "dist_metrics.pxd":42
 * 
 * 
 * cdef inline DTYPE_t euclidean_rdist_to_dist(DTYPE_t dist) except -1:             # <<<<<<<<<<<<<<
 *     return sqrt(dist)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":194
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fullfill the PEP.
 */

/* Python wrapper */
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5numpy_7ndarray___getbuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_v_copy_shape;
  int __pyx_v_i;
  int __pyx_v_ndim;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  int __pyx_v_t;
  char *__pyx_v_f;
  PyArray_Descr *__pyx_v_descr = 0;
  int __pyx_v_offset;
  int __pyx_v_hasfields;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  char *__pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  if (__pyx_v_info != NULL) {
    __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(__pyx_v_info->obj);
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":200
 *             # of flags
 * 
 *             if info == NULL: return             # <<<<<<<<<<<<<<
 * 
 *             cdef int copy_shape, i, ndim
 */
  __pyx_t_1 = ((__pyx_v_info == NULL) != 0);
  if (__pyx_t_1) {
    __pyx_r = 0;
    goto __pyx_L0;
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":203
 * 
 *             cdef int copy_shape, i, ndim
 *             cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 */
  __pyx_v_endian_detector = 1;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":204
 *             cdef int copy_shape, i, ndim
 *             cdef int endian_detector = 1
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 * 
 *             ndim = PyArray_NDIM(self)
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":206
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 *             ndim = PyArray_NDIM(self)             # <<<<<<<<<<<<<<
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_v_ndim = PyArray_NDIM(__pyx_v_self);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":208
 *             ndim = PyArray_NDIM(self)
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 copy_shape = 1
 *             else:
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":209
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 copy_shape = 1             # <<<<<<<<<<<<<<
 *             else:
 *                 copy_shape = 0
 */
    __pyx_v_copy_shape = 1;
    goto __pyx_L4;
  }
  /*else*/ {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":211
 *                 copy_shape = 1
 *             else:
 *                 copy_shape = 0             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 */
    __pyx_v_copy_shape = 0;
  }
  __pyx_L4:;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":213
 *                 copy_shape = 0
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_C_CONTIGUOUS) == PyBUF_C_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L6_bool_binop_done;
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":214
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_C_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_1) {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":215
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__33, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[2]; __pyx_lineno = 215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":217
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_F_CONTIGUOUS) == PyBUF_F_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":218
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_F_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_1) {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":219
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__34, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[2]; __pyx_lineno = 219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":221
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 *             info.buf = PyArray_DATA(self)             # <<<<<<<<<<<<<<
 *             info.ndim = ndim
 *             if copy_shape:
 */
  __pyx_v_info->buf = PyArray_DATA(__pyx_v_self);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":222
 * 
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim             # <<<<<<<<<<<<<<
 *             if copy_shape:
 *                 # Allocate new buffer for strides and shape info.
 */
  __pyx_v_info->ndim = __pyx_v_ndim;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":223
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim
 *             if copy_shape:             # <<<<<<<<<<<<<<
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 */
  __pyx_t_1 = (__pyx_v_copy_shape != 0);
  if (__pyx_t_1) {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":226
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)             # <<<<<<<<<<<<<<
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 */
    __pyx_v_info->strides = ((Py_ssize_t *)malloc((((sizeof(Py_ssize_t)) * ((size_t)__pyx_v_ndim)) * 2)));

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":227
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)
 *                 info.shape = info.strides + ndim             # <<<<<<<<<<<<<<
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 */
    __pyx_v_info->shape = (__pyx_v_info->strides + __pyx_v_ndim);

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":228
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):             # <<<<<<<<<<<<<<
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 */
    __pyx_t_4 = __pyx_v_ndim;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_i = __pyx_t_5;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":229
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]             # <<<<<<<<<<<<<<
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 */
      (__pyx_v_info->strides[__pyx_v_i]) = (PyArray_STRIDES(__pyx_v_self)[__pyx_v_i]);

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":230
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]             # <<<<<<<<<<<<<<
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 */
      (__pyx_v_info->shape[__pyx_v_i]) = (PyArray_DIMS(__pyx_v_self)[__pyx_v_i]);
    }
    goto __pyx_L11;
  }
  /*else*/ {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":232
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)             # <<<<<<<<<<<<<<
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 */
    __pyx_v_info->strides = ((Py_ssize_t *)PyArray_STRIDES(__pyx_v_self));

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":233
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)             # <<<<<<<<<<<<<<
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 */
    __pyx_v_info->shape = ((Py_ssize_t *)PyArray_DIMS(__pyx_v_self));
  }
  __pyx_L11:;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":234
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL             # <<<<<<<<<<<<<<
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 */
  __pyx_v_info->suboffsets = NULL;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":235
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)             # <<<<<<<<<<<<<<
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 * 
 */
  __pyx_v_info->itemsize = PyArray_ITEMSIZE(__pyx_v_self);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":236
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)             # <<<<<<<<<<<<<<
 * 
 *             cdef int t
 */
  __pyx_v_info->readonly = (!(PyArray_ISWRITEABLE(__pyx_v_self) != 0));

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":239
 * 
 *             cdef int t
 *             cdef char* f = NULL             # <<<<<<<<<<<<<<
 *             cdef dtype descr = self.descr
 *             cdef list stack
 */
  __pyx_v_f = NULL;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":240
 *             cdef int t
 *             cdef char* f = NULL
 *             cdef dtype descr = self.descr             # <<<<<<<<<<<<<<
 *             cdef list stack
 *             cdef int offset
 */
  __pyx_t_3 = ((PyObject *)__pyx_v_self->descr);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_v_descr = ((PyArray_Descr *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":244
 *             cdef int offset
 * 
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)             # <<<<<<<<<<<<<<
 * 
 *             if not hasfields and not copy_shape:
 */
  __pyx_v_hasfields = PyDataType_HASFIELDS(__pyx_v_descr);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":246
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)
 * 
 *             if not hasfields and not copy_shape:             # <<<<<<<<<<<<<<
 *                 # do not call releasebuffer
 *                 info.obj = None
 */
  __pyx_t_2 = ((!(__pyx_v_hasfields != 0)) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_2 = ((!(__pyx_v_copy_shape != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L15_bool_binop_done:;
  if (__pyx_t_1) {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":248
 *             if not hasfields and not copy_shape:
 *                 # do not call releasebuffer
 *                 info.obj = None             # <<<<<<<<<<<<<<
 *             else:
 *                 # need to call releasebuffer
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj);
    __pyx_v_info->obj = Py_None;
    goto __pyx_L14;
  }
  /*else*/ {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":251
 *             else:
 *                 # need to call releasebuffer
 *                 info.obj = self             # <<<<<<<<<<<<<<
 * 
 *             if not hasfields:
 */
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj);
    __pyx_v_info->obj = ((PyObject *)__pyx_v_self);
  }
  __pyx_L14:;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":253
 *                 info.obj = self
 * 
 *             if not hasfields:             # <<<<<<<<<<<<<<
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 */
  __pyx_t_1 = ((!(__pyx_v_hasfields != 0)) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":254
 * 
 *             if not hasfields:
 *                 t = descr.type_num             # <<<<<<<<<<<<<<
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 */
    __pyx_t_4 = __pyx_v_descr->type_num;
    __pyx_v_t = __pyx_t_4;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":255
 *             if not hasfields:
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '>') != 0);
    if (!__pyx_t_2) {
      goto __pyx_L20_next_or;
    } else {
    }
    __pyx_t_2 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L19_bool_binop_done;
    }
    __pyx_L20_next_or:;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":256
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '<') != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L19_bool_binop_done;
    }
    __pyx_t_2 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L19_bool_binop_done:;
    if (__pyx_t_1) {

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":257
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__35, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[2]; __pyx_lineno = 257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":274
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"             # <<<<<<<<<<<<<<
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
    switch (__pyx_v_t) {

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":258
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 */
      case NPY_BYTE:
      __pyx_v_f = __pyx_k_b;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":259
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 */
      case NPY_UBYTE:
      __pyx_v_f = __pyx_k_B;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":260
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 */
      case NPY_SHORT:
      __pyx_v_f = __pyx_k_h;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":261
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 */
      case NPY_USHORT:
      __pyx_v_f = __pyx_k_H;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":262
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 */
      case NPY_INT:
      __pyx_v_f = __pyx_k_i;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":263
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 */
      case NPY_UINT:
      __pyx_v_f = __pyx_k_I;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":264
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 */
      case NPY_LONG:
      __pyx_v_f = __pyx_k_l;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":265
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 */
      case NPY_ULONG:
      __pyx_v_f = __pyx_k_L;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":266
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 */
      case NPY_LONGLONG:
      __pyx_v_f = __pyx_k_q;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":267
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 */
      case NPY_ULONGLONG:
      __pyx_v_f = __pyx_k_Q;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":268
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 */
      case NPY_FLOAT:
      __pyx_v_f = __pyx_k_f;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":269
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 */
      case NPY_DOUBLE:
      __pyx_v_f = __pyx_k_d;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":270
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 */
      case NPY_LONGDOUBLE:
      __pyx_v_f = __pyx_k_g;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":271
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 */
      case NPY_CFLOAT:
      __pyx_v_f = __pyx_k_Zf;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":272
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"
 */
      case NPY_CDOUBLE:
      __pyx_v_f = __pyx_k_Zd;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":273
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 */
      case NPY_CLONGDOUBLE:
      __pyx_v_f = __pyx_k_Zg;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":274
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"             # <<<<<<<<<<<<<<
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      case NPY_OBJECT:
      __pyx_v_f = __pyx_k_O;
      break;
      default:

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":276
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *                 info.format = f
 *                 return
 */
      __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_t); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 276; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_6 = PyUnicode_Format(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_t_3); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 276; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 276; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_3, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 276; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      {__pyx_filename = __pyx_f[2]; __pyx_lineno = 276; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      break;
    }

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":277
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f             # <<<<<<<<<<<<<<
 *                 return
 *             else:
 */
    __pyx_v_info->format = __pyx_v_f;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":278
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f
 *                 return             # <<<<<<<<<<<<<<
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 */
    __pyx_r = 0;
    goto __pyx_L0;
  }
  /*else*/ {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":280
 *                 return
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)             # <<<<<<<<<<<<<<
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 */
    __pyx_v_info->format = ((char *)malloc(255));

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":281
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment             # <<<<<<<<<<<<<<
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,
 */
    (__pyx_v_info->format[0]) = '^';

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":282
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0             # <<<<<<<<<<<<<<
 *                 f = _util_dtypestring(descr, info.format + 1,
 *                                       info.format + _buffer_format_string_len,
 */
    __pyx_v_offset = 0;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":283
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,             # <<<<<<<<<<<<<<
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 */
    __pyx_t_7 = __pyx_f_5numpy__util_dtypestring(__pyx_v_descr, (__pyx_v_info->format + 1), (__pyx_v_info->format + 255), (&__pyx_v_offset)); if (unlikely(__pyx_t_7 == NULL)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 283; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_f = __pyx_t_7;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":286
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 *                 f[0] = c'\0' # Terminate format string             # <<<<<<<<<<<<<<
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 */
    (__pyx_v_f[0]) = '\x00';
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":194
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fullfill the PEP.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("numpy.ndarray.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  if (__pyx_v_info != NULL && __pyx_v_info->obj != NULL) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = NULL;
  }
  goto __pyx_L2;
  __pyx_L0:;
  if (__pyx_v_info != NULL && __pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(Py_None);
    __Pyx_DECREF(Py_None); __pyx_v_info->obj = NULL;
  }
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_descr);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":288
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 */

/* Python wrapper */
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info); /*proto*/
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__releasebuffer__ (wrapper)", 0);
  __pyx_pf_5numpy_7ndarray_2__releasebuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__releasebuffer__", 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":289
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_t_1 = (PyArray_HASFIELDS(__pyx_v_self) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":290
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)             # <<<<<<<<<<<<<<
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 stdlib.free(info.strides)
 */
    free(__pyx_v_info->format);
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":291
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.strides)
 *                 # info.shape was stored after info.strides in the same block
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":292
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 stdlib.free(info.strides)             # <<<<<<<<<<<<<<
 *                 # info.shape was stored after info.strides in the same block
 * 
 */
    free(__pyx_v_info->strides);
    goto __pyx_L4;
  }
  __pyx_L4:;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":288
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":768
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":769
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 769; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":768
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":771
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":772
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 772; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":771
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":774
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":775
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 775; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":774
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":777
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":778
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 778; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":777
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":780
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":781
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 781; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":780
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":783
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *__pyx_v_descr, char *__pyx_v_f, char *__pyx_v_end, int *__pyx_v_offset) {
  PyArray_Descr *__pyx_v_child = 0;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  PyObject *__pyx_v_fields = 0;
  PyObject *__pyx_v_childname = NULL;
  PyObject *__pyx_v_new_offset = NULL;
  PyObject *__pyx_v_t = NULL;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  long __pyx_t_8;
  char *__pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_util_dtypestring", 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":790
 *     cdef int delta_offset
 *     cdef tuple i
 *     cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 *     cdef tuple fields
 */
  __pyx_v_endian_detector = 1;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":791
 *     cdef tuple i
 *     cdef int endian_detector = 1
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 *     cdef tuple fields
 * 
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":794
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  if (unlikely(__pyx_v_descr->names == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[2]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __pyx_v_descr->names; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    #endif
    __Pyx_XDECREF_SET(__pyx_v_childname, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":795
 * 
 *     for childname in descr.names:
 *         fields = descr.fields[childname]             # <<<<<<<<<<<<<<
 *         child, new_offset = fields
 * 
 */
    __pyx_t_3 = PyObject_GetItem(__pyx_v_descr->fields, __pyx_v_childname); if (unlikely(__pyx_t_3 == NULL)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyTuple_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_t_3)->tp_name), 0))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_fields, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":796
 *     for childname in descr.names:
 *         fields = descr.fields[childname]
 *         child, new_offset = fields             # <<<<<<<<<<<<<<
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 */
    if (likely(__pyx_v_fields != Py_None)) {
      PyObject* sequence = __pyx_v_fields;
      #if CYTHON_COMPILING_IN_CPYTHON
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        {__pyx_filename = __pyx_f[2]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      #if CYTHON_COMPILING_IN_CPYTHON
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      #else
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      #endif
    } else {
      __Pyx_RaiseNoneNotIterableError(); {__pyx_filename = __pyx_f[2]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_dtype))))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_child, ((PyArray_Descr *)__pyx_t_3));
    __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_new_offset, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":798
 *         child, new_offset = fields
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 */
    __pyx_t_4 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = PyNumber_Subtract(__pyx_v_new_offset, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = ((((__pyx_v_end - __pyx_v_f) - ((int)__pyx_t_5)) < 15) != 0);
    if (__pyx_t_6) {

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":799
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__36, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[2]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":801
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '>') != 0);
    if (!__pyx_t_7) {
      goto __pyx_L8_next_or;
    } else {
    }
    __pyx_t_7 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_L8_next_or:;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":802
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *             raise ValueError(u"Non-native byte order not supported")
 *             # One could encode it in the format string and have Cython
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '<') != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_7 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L7_bool_binop_done:;
    if (__pyx_t_6) {

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":803
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *             # One could encode it in the format string and have Cython
 *             # complain instead, BUT: < and > in format strings also imply
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__37, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[2]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":813
 * 
 *         # Output padding bytes
 *         while offset[0] < new_offset:             # <<<<<<<<<<<<<<
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 */
    while (1) {
      __pyx_t_3 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_v_new_offset, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (!__pyx_t_6) break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":814
 *         # Output padding bytes
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte             # <<<<<<<<<<<<<<
 *             f += 1
 *             offset[0] += 1
 */
      (__pyx_v_f[0]) = 120;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":815
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte
 *             f += 1             # <<<<<<<<<<<<<<
 *             offset[0] += 1
 * 
 */
      __pyx_v_f = (__pyx_v_f + 1);

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":816
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 *             offset[0] += 1             # <<<<<<<<<<<<<<
 * 
 *         offset[0] += child.itemsize
 */
      __pyx_t_8 = 0;
      (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + 1);
    }

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":818
 *             offset[0] += 1
 * 
 *         offset[0] += child.itemsize             # <<<<<<<<<<<<<<
 * 
 *         if not PyDataType_HASFIELDS(child):
 */
    __pyx_t_8 = 0;
    (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + __pyx_v_child->elsize);

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":820
 *         offset[0] += child.itemsize
 * 
 *         if not PyDataType_HASFIELDS(child):             # <<<<<<<<<<<<<<
 *             t = child.type_num
 *             if end - f < 5:
 */
    __pyx_t_6 = ((!(PyDataType_HASFIELDS(__pyx_v_child) != 0)) != 0);
    if (__pyx_t_6) {

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":821
 * 
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num             # <<<<<<<<<<<<<<
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")
 */
      __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_child->type_num); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 821; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_XDECREF_SET(__pyx_v_t, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":822
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num
 *             if end - f < 5:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError(u"Format string allocated too short.")
 * 
 */
      __pyx_t_6 = (((__pyx_v_end - __pyx_v_f) < 5) != 0);
      if (__pyx_t_6) {

        /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":823
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__38, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_Raise(__pyx_t_4, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        {__pyx_filename = __pyx_f[2]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":826
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 */
      __pyx_t_4 = PyInt_FromLong(NPY_BYTE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 98;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":827
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"             # <<<<<<<<<<<<<<
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 */
      __pyx_t_3 = PyInt_FromLong(NPY_UBYTE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 66;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":828
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"             # <<<<<<<<<<<<<<
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 */
      __pyx_t_4 = PyInt_FromLong(NPY_SHORT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 104;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":829
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"             # <<<<<<<<<<<<<<
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 */
      __pyx_t_3 = PyInt_FromLong(NPY_USHORT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 72;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":830
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 */
      __pyx_t_4 = PyInt_FromLong(NPY_INT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 105;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":831
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 */
      __pyx_t_3 = PyInt_FromLong(NPY_UINT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 73;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":832
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 */
      __pyx_t_4 = PyInt_FromLong(NPY_LONG); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 108;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":833
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 */
      __pyx_t_3 = PyInt_FromLong(NPY_ULONG); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 76;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":834
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 */
      __pyx_t_4 = PyInt_FromLong(NPY_LONGLONG); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 113;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":835
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 */
      __pyx_t_3 = PyInt_FromLong(NPY_ULONGLONG); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 81;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":836
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"             # <<<<<<<<<<<<<<
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 */
      __pyx_t_4 = PyInt_FromLong(NPY_FLOAT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 102;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":837
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 */
      __pyx_t_3 = PyInt_FromLong(NPY_DOUBLE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 100;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":838
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"             # <<<<<<<<<<<<<<
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 */
      __pyx_t_4 = PyInt_FromLong(NPY_LONGDOUBLE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 103;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":839
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf             # <<<<<<<<<<<<<<
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 */
      __pyx_t_3 = PyInt_FromLong(NPY_CFLOAT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 102;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":840
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd             # <<<<<<<<<<<<<<
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 */
      __pyx_t_4 = PyInt_FromLong(NPY_CDOUBLE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 100;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":841
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg             # <<<<<<<<<<<<<<
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 */
      __pyx_t_3 = PyInt_FromLong(NPY_CLONGDOUBLE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 103;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":842
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"             # <<<<<<<<<<<<<<
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      __pyx_t_4 = PyInt_FromLong(NPY_OBJECT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 79;
        goto __pyx_L15;
      }
      /*else*/ {

        /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":844
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *             f += 1
 *         else:
 */
        __pyx_t_3 = PyUnicode_Format(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_v_t); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_Raise(__pyx_t_3, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        {__pyx_filename = __pyx_f[2]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_L15:;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":845
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *             f += 1             # <<<<<<<<<<<<<<
 *         else:
 *             # Cython ignores struct boundary information ("T{...}"),
 */
      __pyx_v_f = (__pyx_v_f + 1);
      goto __pyx_L13;
    }
    /*else*/ {

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":849
 *             # Cython ignores struct boundary information ("T{...}"),
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)             # <<<<<<<<<<<<<<
 *     return f
 * 
 */
      __pyx_t_9 = __pyx_f_5numpy__util_dtypestring(__pyx_v_child, __pyx_v_f, __pyx_v_end, __pyx_v_offset); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 849; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_v_f = __pyx_t_9;
    }
    __pyx_L13:;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":794
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":850
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)
 *     return f             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_f;
  goto __pyx_L0;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":783
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("numpy._util_dtypestring", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child);
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XDECREF(__pyx_v_childname);
  __Pyx_XDECREF(__pyx_v_new_offset);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":966
 * 
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *      cdef PyObject* baseptr
 *      if base is None:
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  PyObject *__pyx_v_baseptr;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":968
 * cdef inline void set_array_base(ndarray arr, object base):
 *      cdef PyObject* baseptr
 *      if base is None:             # <<<<<<<<<<<<<<
 *          baseptr = NULL
 *      else:
 */
  __pyx_t_1 = (__pyx_v_base == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":969
 *      cdef PyObject* baseptr
 *      if base is None:
 *          baseptr = NULL             # <<<<<<<<<<<<<<
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!
 */
    __pyx_v_baseptr = NULL;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":971
 *          baseptr = NULL
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!             # <<<<<<<<<<<<<<
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)
 */
    Py_INCREF(__pyx_v_base);

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":972
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!
 *          baseptr = <PyObject*>base             # <<<<<<<<<<<<<<
 *      Py_XDECREF(arr.base)
 *      arr.base = baseptr
 */
    __pyx_v_baseptr = ((PyObject *)__pyx_v_base);
  }
  __pyx_L3:;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":973
 *          Py_INCREF(base) # important to do this before decref below!
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)             # <<<<<<<<<<<<<<
 *      arr.base = baseptr
 * 
 */
  Py_XDECREF(__pyx_v_arr->base);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":974
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)
 *      arr.base = baseptr             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  __pyx_v_arr->base = __pyx_v_baseptr;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":966
 * 
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *      cdef PyObject* baseptr
 *      if base is None:
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":976
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":977
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_arr->base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":978
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     else:
 *         return <object>arr.base
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_None);
    __pyx_r = Py_None;
    goto __pyx_L0;
  }
  /*else*/ {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":980
 *         return None
 *     else:
 *         return <object>arr.base             # <<<<<<<<<<<<<<
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_arr->base));
    __pyx_r = ((PyObject *)__pyx_v_arr->base);
    goto __pyx_L0;
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":976
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":116
 *         cdef bint dtype_is_object
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,             # <<<<<<<<<<<<<<
 *                   mode="c", bint allocate_buffer=True):
 * 
 */

/* Python wrapper */
static int __pyx_array___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_array___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_shape = 0;
  Py_ssize_t __pyx_v_itemsize;
  PyObject *__pyx_v_format = 0;
  PyObject *__pyx_v_mode = 0;
  int __pyx_v_allocate_buffer;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_shape,&__pyx_n_s_itemsize,&__pyx_n_s_format,&__pyx_n_s_mode,&__pyx_n_s_allocate_buffer,0};
    PyObject* values[5] = {0,0,0,0,0};
    values[3] = ((PyObject *)__pyx_n_s_c);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_shape)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_itemsize)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 3, 5, 1); {__pyx_filename = __pyx_f[3]; __pyx_lineno = 116; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_format)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 3, 5, 2); {__pyx_filename = __pyx_f[3]; __pyx_lineno = 116; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_mode);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_allocate_buffer);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 116; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_shape = ((PyObject*)values[0]);
    __pyx_v_itemsize = __Pyx_PyIndex_AsSsize_t(values[1]); if (unlikely((__pyx_v_itemsize == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 116; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_format = values[2];
    __pyx_v_mode = values[3];
    if (values[4]) {
      __pyx_v_allocate_buffer = __Pyx_PyObject_IsTrue(values[4]); if (unlikely((__pyx_v_allocate_buffer == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 117; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {

      /* "View.MemoryView":117
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,
 *                   mode="c", bint allocate_buffer=True):             # <<<<<<<<<<<<<<
 * 
 *         cdef int idx
 */
      __pyx_v_allocate_buffer = ((int)1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 3, 5, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[3]; __pyx_lineno = 116; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("View.MemoryView.array.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_shape), (&PyTuple_Type), 1, "shape", 1))) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 116; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(((PyObject *)__pyx_v_format) == Py_None)) {
    PyErr_Format(PyExc_TypeError, "Argument '%.200s' must not be None", "format"); {__pyx_filename = __pyx_f[3]; __pyx_lineno = 116; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_r = __pyx_array_MemoryView_5array___cinit__(((struct __pyx_array_obj *)__pyx_v_self), __pyx_v_shape, __pyx_v_itemsize, __pyx_v_format, __pyx_v_mode, __pyx_v_allocate_buffer);

  /* "View.MemoryView":116
 *         cdef bint dtype_is_object
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,             # <<<<<<<<<<<<<<
 *                   mode="c", bint allocate_buffer=True):
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_array_MemoryView_5array___cinit__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_shape, Py_ssize_t __pyx_v_itemsize, PyObject *__pyx_v_format, PyObject *__pyx_v_mode, int __pyx_v_allocate_buffer) {
  int __pyx_v_idx;
  Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_dim;
  PyObject **__pyx_v_p;
  char __pyx_v_order;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  char *__pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);
  __Pyx_INCREF(__pyx_v_format);

  /* "View.MemoryView":123
 *         cdef PyObject **p
 * 
 *         self.ndim = <int> len(shape)             # <<<<<<<<<<<<<<
 *         self.itemsize = itemsize
 * 
 */
  if (unlikely(__pyx_v_shape == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[3]; __pyx_lineno = 123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_shape); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_self->ndim = ((int)__pyx_t_1);

  /* "View.MemoryView":124
 * 
 *         self.ndim = <int> len(shape)
 *         self.itemsize = itemsize             # <<<<<<<<<<<<<<
 * 
 *         if not self.ndim:
 */
  __pyx_v_self->itemsize = __pyx_v_itemsize;

  /* "View.MemoryView":126
 *         self.itemsize = itemsize
 * 
 *         if not self.ndim:             # <<<<<<<<<<<<<<
 *             raise ValueError("Empty shape tuple for cython.array")
 * 
 */
  __pyx_t_2 = ((!(__pyx_v_self->ndim != 0)) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":127
 * 
 *         if not self.ndim:
 *             raise ValueError("Empty shape tuple for cython.array")             # <<<<<<<<<<<<<<
 * 
 *         if itemsize <= 0:
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__39, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[3]; __pyx_lineno = 127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "View.MemoryView":129
 *             raise ValueError("Empty shape tuple for cython.array")
 * 
 *         if itemsize <= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("itemsize <= 0 for cython.array")
 * 
 */
  __pyx_t_2 = ((__pyx_v_itemsize <= 0) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":130
 * 
 *         if itemsize <= 0:
 *             raise ValueError("itemsize <= 0 for cython.array")             # <<<<<<<<<<<<<<
 * 
 *         if isinstance(format, unicode):
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__40, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 130; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[3]; __pyx_lineno = 130; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "View.MemoryView":132
 *             raise ValueError("itemsize <= 0 for cython.array")
 * 
 *         if isinstance(format, unicode):             # <<<<<<<<<<<<<<
 *             format = (<unicode>format).encode('ASCII')
 *         self._format = format  # keep a reference to the byte string
 */
  __pyx_t_2 = PyUnicode_Check(__pyx_v_format); 
  __pyx_t_4 = (__pyx_t_2 != 0);
  if (__pyx_t_4) {

    /* "View.MemoryView":133
 * 
 *         if isinstance(format, unicode):
 *             format = (<unicode>format).encode('ASCII')             # <<<<<<<<<<<<<<
 *         self._format = format  # keep a reference to the byte string
 *         self.format = self._format
 */
    if (unlikely(__pyx_v_format == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%s'", "encode");
      {__pyx_filename = __pyx_f[3]; __pyx_lineno = 133; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_3 = PyUnicode_AsASCIIString(((PyObject*)__pyx_v_format)); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 133; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_format, __pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L5;
  }
  __pyx_L5:;

  /* "View.MemoryView":134
 *         if isinstance(format, unicode):
 *             format = (<unicode>format).encode('ASCII')
 *         self._format = format  # keep a reference to the byte string             # <<<<<<<<<<<<<<
 *         self.format = self._format
 * 
 */
  if (!(likely(PyBytes_CheckExact(__pyx_v_format))||((__pyx_v_format) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_v_format)->tp_name), 0))) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 134; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __pyx_v_format;
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->_format);
  __Pyx_DECREF(__pyx_v_self->_format);
  __pyx_v_self->_format = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "View.MemoryView":135
 *             format = (<unicode>format).encode('ASCII')
 *         self._format = format  # keep a reference to the byte string
 *         self.format = self._format             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_AsString(__pyx_v_self->_format); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_self->format = __pyx_t_5;

  /* "View.MemoryView":138
 * 
 * 
 *         self._shape = <Py_ssize_t *> PyMem_Malloc(sizeof(Py_ssize_t)*self.ndim*2)             # <<<<<<<<<<<<<<
 *         self._strides = self._shape + self.ndim
 * 
 */
  __pyx_v_self->_shape = ((Py_ssize_t *)PyMem_Malloc((((sizeof(Py_ssize_t)) * __pyx_v_self->ndim) * 2)));

  /* "View.MemoryView":139
 * 
 *         self._shape = <Py_ssize_t *> PyMem_Malloc(sizeof(Py_ssize_t)*self.ndim*2)
 *         self._strides = self._shape + self.ndim             # <<<<<<<<<<<<<<
 * 
 *         if not self._shape:
 */
  __pyx_v_self->_strides = (__pyx_v_self->_shape + __pyx_v_self->ndim);

  /* "View.MemoryView":141
 *         self._strides = self._shape + self.ndim
 * 
 *         if not self._shape:             # <<<<<<<<<<<<<<
 *             raise MemoryError("unable to allocate shape and strides.")
 * 
 */
  __pyx_t_4 = ((!(__pyx_v_self->_shape != 0)) != 0);
  if (__pyx_t_4) {

    /* "View.MemoryView":142
 * 
 *         if not self._shape:
 *             raise MemoryError("unable to allocate shape and strides.")             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__41, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 142; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[3]; __pyx_lineno = 142; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "View.MemoryView":145
 * 
 * 
 *         for idx, dim in enumerate(shape):             # <<<<<<<<<<<<<<
 *             if dim <= 0:
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 */
  __pyx_t_6 = 0;
  __pyx_t_3 = __pyx_v_shape; __Pyx_INCREF(__pyx_t_3); __pyx_t_1 = 0;
  for (;;) {
    if (__pyx_t_1 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_1); __Pyx_INCREF(__pyx_t_7); __pyx_t_1++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 145; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    #else
    __pyx_t_7 = PySequence_ITEM(__pyx_t_3, __pyx_t_1); __pyx_t_1++; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 145; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    #endif
    __pyx_t_8 = __Pyx_PyIndex_AsSsize_t(__pyx_t_7); if (unlikely((__pyx_t_8 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 145; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_v_dim = __pyx_t_8;
    __pyx_v_idx = __pyx_t_6;
    __pyx_t_6 = (__pyx_t_6 + 1);

    /* "View.MemoryView":146
 * 
 *         for idx, dim in enumerate(shape):
 *             if dim <= 0:             # <<<<<<<<<<<<<<
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 *             self._shape[idx] = dim
 */
    __pyx_t_4 = ((__pyx_v_dim <= 0) != 0);
    if (__pyx_t_4) {

      /* "View.MemoryView":147
 *         for idx, dim in enumerate(shape):
 *             if dim <= 0:
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))             # <<<<<<<<<<<<<<
 *             self._shape[idx] = dim
 * 
 */
      __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_idx); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_9 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_10);
      PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_7);
      PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_9);
      __Pyx_GIVEREF(__pyx_t_9);
      __pyx_t_7 = 0;
      __pyx_t_9 = 0;
      __pyx_t_9 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_shape_in_axis_d_d, __pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_10 = PyTuple_New(1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_10);
      PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9);
      __Pyx_GIVEREF(__pyx_t_9);
      __pyx_t_9 = 0;
      __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_10, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_Raise(__pyx_t_9, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      {__pyx_filename = __pyx_f[3]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }

    /* "View.MemoryView":148
 *             if dim <= 0:
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 *             self._shape[idx] = dim             # <<<<<<<<<<<<<<
 * 
 *         cdef char order
 */
    (__pyx_v_self->_shape[__pyx_v_idx]) = __pyx_v_dim;

    /* "View.MemoryView":145
 * 
 * 
 *         for idx, dim in enumerate(shape):             # <<<<<<<<<<<<<<
 *             if dim <= 0:
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 */
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "View.MemoryView":151
 * 
 *         cdef char order
 *         if mode == 'fortran':             # <<<<<<<<<<<<<<
 *             order = b'F'
 *             self.mode = u'fortran'
 */
  __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_mode, __pyx_n_s_fortran, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 151; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_4) {

    /* "View.MemoryView":152
 *         cdef char order
 *         if mode == 'fortran':
 *             order = b'F'             # <<<<<<<<<<<<<<
 *             self.mode = u'fortran'
 *         elif mode == 'c':
 */
    __pyx_v_order = 'F';

    /* "View.MemoryView":153
 *         if mode == 'fortran':
 *             order = b'F'
 *             self.mode = u'fortran'             # <<<<<<<<<<<<<<
 *         elif mode == 'c':
 *             order = b'C'
 */
    __Pyx_INCREF(__pyx_n_u_fortran);
    __Pyx_GIVEREF(__pyx_n_u_fortran);
    __Pyx_GOTREF(__pyx_v_self->mode);
    __Pyx_DECREF(__pyx_v_self->mode);
    __pyx_v_self->mode = __pyx_n_u_fortran;
    goto __pyx_L10;
  }

  /* "View.MemoryView":154
 *             order = b'F'
 *             self.mode = u'fortran'
 *         elif mode == 'c':             # <<<<<<<<<<<<<<
 *             order = b'C'
 *             self.mode = u'c'
 */
  __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_mode, __pyx_n_s_c, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_4) {

    /* "View.MemoryView":155
 *             self.mode = u'fortran'
 *         elif mode == 'c':
 *             order = b'C'             # <<<<<<<<<<<<<<
 *             self.mode = u'c'
 *         else:
 */
    __pyx_v_order = 'C';

    /* "View.MemoryView":156
 *         elif mode == 'c':
 *             order = b'C'
 *             self.mode = u'c'             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError("Invalid mode, expected 'c' or 'fortran', got %s" % mode)
 */
    __Pyx_INCREF(__pyx_n_u_c);
    __Pyx_GIVEREF(__pyx_n_u_c);
    __Pyx_GOTREF(__pyx_v_self->mode);
    __Pyx_DECREF(__pyx_v_self->mode);
    __pyx_v_self->mode = __pyx_n_u_c;
    goto __pyx_L10;
  }
  /*else*/ {

    /* "View.MemoryView":158
 *             self.mode = u'c'
 *         else:
 *             raise ValueError("Invalid mode, expected 'c' or 'fortran', got %s" % mode)             # <<<<<<<<<<<<<<
 * 
 *         self.len = fill_contig_strides_array(self._shape, self._strides,
 */
    __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_mode_expected_c_or_fortr, __pyx_v_mode); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 158; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 158; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 158; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[3]; __pyx_lineno = 158; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L10:;

  /* "View.MemoryView":160
 *             raise ValueError("Invalid mode, expected 'c' or 'fortran', got %s" % mode)
 * 
 *         self.len = fill_contig_strides_array(self._shape, self._strides,             # <<<<<<<<<<<<<<
 *                                              itemsize, self.ndim, order)
 * 
 */
  __pyx_v_self->len = __pyx_fill_contig_strides_array(__pyx_v_self->_shape, __pyx_v_self->_strides, __pyx_v_itemsize, __pyx_v_self->ndim, __pyx_v_order);

  /* "View.MemoryView":163
 *                                              itemsize, self.ndim, order)
 * 
 *         self.free_data = allocate_buffer             # <<<<<<<<<<<<<<
 *         self.dtype_is_object = format == b'O'
 *         if allocate_buffer:
 */
  __pyx_v_self->free_data = __pyx_v_allocate_buffer;

  /* "View.MemoryView":164
 * 
 *         self.free_data = allocate_buffer
 *         self.dtype_is_object = format == b'O'             # <<<<<<<<<<<<<<
 *         if allocate_buffer:
 * 
 */
  __pyx_t_3 = PyObject_RichCompare(__pyx_v_format, __pyx_n_b_O, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_self->dtype_is_object = __pyx_t_4;

  /* "View.MemoryView":165
 *         self.free_data = allocate_buffer
 *         self.dtype_is_object = format == b'O'
 *         if allocate_buffer:             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_4 = (__pyx_v_allocate_buffer != 0);
  if (__pyx_t_4) {

    /* "View.MemoryView":168
 * 
 * 
 *             self.data = <char *>malloc(self.len)             # <<<<<<<<<<<<<<
 *             if not self.data:
 *                 raise MemoryError("unable to allocate array data.")
 */
    __pyx_v_self->data = ((char *)malloc(__pyx_v_self->len));

    /* "View.MemoryView":169
 * 
 *             self.data = <char *>malloc(self.len)
 *             if not self.data:             # <<<<<<<<<<<<<<
 *                 raise MemoryError("unable to allocate array data.")
 * 
 */
    __pyx_t_4 = ((!(__pyx_v_self->data != 0)) != 0);
    if (__pyx_t_4) {

      /* "View.MemoryView":170
 *             self.data = <char *>malloc(self.len)
 *             if not self.data:
 *                 raise MemoryError("unable to allocate array data.")             # <<<<<<<<<<<<<<
 * 
 *             if self.dtype_is_object:
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__42, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[3]; __pyx_lineno = 170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }

    /* "View.MemoryView":172
 *                 raise MemoryError("unable to allocate array data.")
 * 
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 p = <PyObject **> self.data
 *                 for i in range(self.len / itemsize):
 */
    __pyx_t_4 = (__pyx_v_self->dtype_is_object != 0);
    if (__pyx_t_4) {

      /* "View.MemoryView":173
 * 
 *             if self.dtype_is_object:
 *                 p = <PyObject **> self.data             # <<<<<<<<<<<<<<
 *                 for i in range(self.len / itemsize):
 *                     p[i] = Py_None
 */
      __pyx_v_p = ((PyObject **)__pyx_v_self->data);

      /* "View.MemoryView":174
 *             if self.dtype_is_object:
 *                 p = <PyObject **> self.data
 *                 for i in range(self.len / itemsize):             # <<<<<<<<<<<<<<
 *                     p[i] = Py_None
 *                     Py_INCREF(Py_None)
 */
      if (unlikely(__pyx_v_itemsize == 0)) {
        #ifdef WITH_THREAD
        PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
        #endif
        PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
        #ifdef WITH_THREAD
        PyGILState_Release(__pyx_gilstate_save);
        #endif
        {__pyx_filename = __pyx_f[3]; __pyx_lineno = 174; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      else if (sizeof(Py_ssize_t) == sizeof(long) && unlikely(__pyx_v_itemsize == -1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_self->len))) {
        #ifdef WITH_THREAD
        PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
        #endif
        PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
        #ifdef WITH_THREAD
        PyGILState_Release(__pyx_gilstate_save);
        #endif
        {__pyx_filename = __pyx_f[3]; __pyx_lineno = 174; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_1 = (__pyx_v_self->len / __pyx_v_itemsize);
      for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_1; __pyx_t_8+=1) {
        __pyx_v_i = __pyx_t_8;

        /* "View.MemoryView":175
 *                 p = <PyObject **> self.data
 *                 for i in range(self.len / itemsize):
 *                     p[i] = Py_None             # <<<<<<<<<<<<<<
 *                     Py_INCREF(Py_None)
 * 
 */
        (__pyx_v_p[__pyx_v_i]) = Py_None;

        /* "View.MemoryView":176
 *                 for i in range(self.len / itemsize):
 *                     p[i] = Py_None
 *                     Py_INCREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *     @cname('getbuffer')
 */
        Py_INCREF(Py_None);
      }
      goto __pyx_L13;
    }
    __pyx_L13:;
    goto __pyx_L11;
  }
  __pyx_L11:;

  /* "View.MemoryView":116
 *         cdef bint dtype_is_object
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,             # <<<<<<<<<<<<<<
 *                   mode="c", bint allocate_buffer=True):
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("View.MemoryView.array.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_format);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":179
 * 
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):             # <<<<<<<<<<<<<<
 *         cdef int bufmode = -1
 *         if self.mode == u"c":
 */

/* Python wrapper */
static CYTHON_UNUSED int __pyx_array_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static CYTHON_UNUSED int __pyx_array_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = __pyx_array_getbuffer_MemoryView_5array_2__getbuffer__(((struct __pyx_array_obj *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_array_getbuffer_MemoryView_5array_2__getbuffer__(struct __pyx_array_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_v_bufmode;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  Py_ssize_t *__pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  if (__pyx_v_info != NULL) {
    __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(__pyx_v_info->obj);
  }

  /* "View.MemoryView":180
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         cdef int bufmode = -1             # <<<<<<<<<<<<<<
 *         if self.mode == u"c":
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 */
  __pyx_v_bufmode = -1;

  /* "View.MemoryView":181
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         cdef int bufmode = -1
 *         if self.mode == u"c":             # <<<<<<<<<<<<<<
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":
 */
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_self->mode, __pyx_n_u_c, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":182
 *         cdef int bufmode = -1
 *         if self.mode == u"c":
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS             # <<<<<<<<<<<<<<
 *         elif self.mode == u"fortran":
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 */
    __pyx_v_bufmode = (PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS);
    goto __pyx_L3;
  }

  /* "View.MemoryView":183
 *         if self.mode == u"c":
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":             # <<<<<<<<<<<<<<
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):
 */
  __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_v_self->mode, __pyx_n_u_fortran, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":184
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS             # <<<<<<<<<<<<<<
 *         if not (flags & bufmode):
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 */
    __pyx_v_bufmode = (PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS);
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "View.MemoryView":185
 *         elif self.mode == u"fortran":
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):             # <<<<<<<<<<<<<<
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 *         info.buf = self.data
 */
  __pyx_t_1 = ((!((__pyx_v_flags & __pyx_v_bufmode) != 0)) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":186
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")             # <<<<<<<<<<<<<<
 *         info.buf = self.data
 *         info.len = self.len
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__43, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[3]; __pyx_lineno = 186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "View.MemoryView":187
 *         if not (flags & bufmode):
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 *         info.buf = self.data             # <<<<<<<<<<<<<<
 *         info.len = self.len
 *         info.ndim = self.ndim
 */
  __pyx_t_4 = __pyx_v_self->data;
  __pyx_v_info->buf = __pyx_t_4;

  /* "View.MemoryView":188
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 *         info.buf = self.data
 *         info.len = self.len             # <<<<<<<<<<<<<<
 *         info.ndim = self.ndim
 *         info.shape = self._shape
 */
  __pyx_t_5 = __pyx_v_self->len;
  __pyx_v_info->len = __pyx_t_5;

  /* "View.MemoryView":189
 *         info.buf = self.data
 *         info.len = self.len
 *         info.ndim = self.ndim             # <<<<<<<<<<<<<<
 *         info.shape = self._shape
 *         info.strides = self._strides
 */
  __pyx_t_6 = __pyx_v_self->ndim;
  __pyx_v_info->ndim = __pyx_t_6;

  /* "View.MemoryView":190
 *         info.len = self.len
 *         info.ndim = self.ndim
 *         info.shape = self._shape             # <<<<<<<<<<<<<<
 *         info.strides = self._strides
 *         info.suboffsets = NULL
 */
  __pyx_t_7 = __pyx_v_self->_shape;
  __pyx_v_info->shape = __pyx_t_7;

  /* "View.MemoryView":191
 *         info.ndim = self.ndim
 *         info.shape = self._shape
 *         info.strides = self._strides             # <<<<<<<<<<<<<<
 *         info.suboffsets = NULL
 *         info.itemsize = self.itemsize
 */
  __pyx_t_7 = __pyx_v_self->_strides;
  __pyx_v_info->strides = __pyx_t_7;

  /* "View.MemoryView":192
 *         info.shape = self._shape
 *         info.strides = self._strides
 *         info.suboffsets = NULL             # <<<<<<<<<<<<<<
 *         info.itemsize = self.itemsize
 *         info.readonly = 0
 */
  __pyx_v_info->suboffsets = NULL;

  /* "View.MemoryView":193
 *         info.strides = self._strides
 *         info.suboffsets = NULL
 *         info.itemsize = self.itemsize             # <<<<<<<<<<<<<<
 *         info.readonly = 0
 * 
 */
  __pyx_t_5 = __pyx_v_self->itemsize;
  __pyx_v_info->itemsize = __pyx_t_5;

  /* "View.MemoryView":194
 *         info.suboffsets = NULL
 *         info.itemsize = self.itemsize
 *         info.readonly = 0             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_FORMAT:
 */
  __pyx_v_info->readonly = 0;

  /* "View.MemoryView":196
 *         info.readonly = 0
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             info.format = self.format
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_FORMAT) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":197
 * 
 *         if flags & PyBUF_FORMAT:
 *             info.format = self.format             # <<<<<<<<<<<<<<
 *         else:
 *             info.format = NULL
 */
    __pyx_t_4 = __pyx_v_self->format;
    __pyx_v_info->format = __pyx_t_4;
    goto __pyx_L5;
  }
  /*else*/ {

    /* "View.MemoryView":199
 *             info.format = self.format
 *         else:
 *             info.format = NULL             # <<<<<<<<<<<<<<
 * 
 *         info.obj = self
 */
    __pyx_v_info->format = NULL;
  }
  __pyx_L5:;

  /* "View.MemoryView":201
 *             info.format = NULL
 * 
 *         info.obj = self             # <<<<<<<<<<<<<<
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_array_getbuffer, "getbuffer(obj, view, flags)")
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  __Pyx_GOTREF(__pyx_v_info->obj);
  __Pyx_DECREF(__pyx_v_info->obj);
  __pyx_v_info->obj = ((PyObject *)__pyx_v_self);

  /* "View.MemoryView":179
 * 
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):             # <<<<<<<<<<<<<<
 *         cdef int bufmode = -1
 *         if self.mode == u"c":
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.array.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  if (__pyx_v_info != NULL && __pyx_v_info->obj != NULL) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = NULL;
  }
  goto __pyx_L2;
  __pyx_L0:;
  if (__pyx_v_info != NULL && __pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(Py_None);
    __Pyx_DECREF(Py_None); __pyx_v_info->obj = NULL;
  }
  __pyx_L2:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":205
 *     __pyx_getbuffer = capsule(<void *> &__pyx_array_getbuffer, "getbuffer(obj, view, flags)")
 * 
 *     def __dealloc__(array self):             # <<<<<<<<<<<<<<
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 */

/* Python wrapper */
static void __pyx_array___dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_array___dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_array_MemoryView_5array_4__dealloc__(((struct __pyx_array_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_array_MemoryView_5array_4__dealloc__(struct __pyx_array_obj *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "View.MemoryView":206
 * 
 *     def __dealloc__(array self):
 *         if self.callback_free_data != NULL:             # <<<<<<<<<<<<<<
 *             self.callback_free_data(self.data)
 *         elif self.free_data:
 */
  __pyx_t_1 = ((__pyx_v_self->callback_free_data != NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":207
 *     def __dealloc__(array self):
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)             # <<<<<<<<<<<<<<
 *         elif self.free_data:
 *             if self.dtype_is_object:
 */
    __pyx_v_self->callback_free_data(__pyx_v_self->data);
    goto __pyx_L3;
  }

  /* "View.MemoryView":208
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 *         elif self.free_data:             # <<<<<<<<<<<<<<
 *             if self.dtype_is_object:
 *                 refcount_objects_in_slice(self.data, self._shape,
 */
  __pyx_t_1 = (__pyx_v_self->free_data != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":209
 *             self.callback_free_data(self.data)
 *         elif self.free_data:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 refcount_objects_in_slice(self.data, self._shape,
 *                                           self._strides, self.ndim, False)
 */
    __pyx_t_1 = (__pyx_v_self->dtype_is_object != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":210
 *         elif self.free_data:
 *             if self.dtype_is_object:
 *                 refcount_objects_in_slice(self.data, self._shape,             # <<<<<<<<<<<<<<
 *                                           self._strides, self.ndim, False)
 *             free(self.data)
 */
      __pyx_memoryview_refcount_objects_in_slice(__pyx_v_self->data, __pyx_v_self->_shape, __pyx_v_self->_strides, __pyx_v_self->ndim, 0);
      goto __pyx_L4;
    }
    __pyx_L4:;

    /* "View.MemoryView":212
 *                 refcount_objects_in_slice(self.data, self._shape,
 *                                           self._strides, self.ndim, False)
 *             free(self.data)             # <<<<<<<<<<<<<<
 *         PyMem_Free(self._shape)
 * 
 */
    free(__pyx_v_self->data);
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "View.MemoryView":213
 *                                           self._strides, self.ndim, False)
 *             free(self.data)
 *         PyMem_Free(self._shape)             # <<<<<<<<<<<<<<
 * 
 *     property memview:
 */
  PyMem_Free(__pyx_v_self->_shape);

  /* "View.MemoryView":205
 *     __pyx_getbuffer = capsule(<void *> &__pyx_array_getbuffer, "getbuffer(obj, view, flags)")
 * 
 *     def __dealloc__(array self):             # <<<<<<<<<<<<<<
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "View.MemoryView":217
 *     property memview:
 *         @cname('get_memview')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 * 
 *             flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE
 */

/* Python wrapper */
static PyObject *get_memview(PyObject *__pyx_v_self); /*proto*/
static PyObject *get_memview(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = get_memview_MemoryView_5array_7memview___get__(((struct __pyx_array_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *get_memview_MemoryView_5array_7memview___get__(struct __pyx_array_obj *__pyx_v_self) {
  int __pyx_v_flags;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":219
 *         def __get__(self):
 * 
 *             flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE             # <<<<<<<<<<<<<<
 *             return  memoryview(self, flags, self.dtype_is_object)
 * 
 */
  __pyx_v_flags = ((PyBUF_ANY_CONTIGUOUS | PyBUF_FORMAT) | PyBUF_WRITABLE);

  /* "View.MemoryView":220
 * 
 *             flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE
 *             return  memoryview(self, flags, self.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_flags); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->dtype_is_object); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)((PyObject *)__pyx_memoryview_type)), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":217
 *     property memview:
 *         @cname('get_memview')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 * 
 *             flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.array.memview.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":223
 * 
 * 
 *     def __getattr__(self, attr):             # <<<<<<<<<<<<<<
 *         return getattr(self.memview, attr)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_array___getattr__(PyObject *__pyx_v_self, PyObject *__pyx_v_attr); /*proto*/
static PyObject *__pyx_array___getattr__(PyObject *__pyx_v_self, PyObject *__pyx_v_attr) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getattr__ (wrapper)", 0);
  __pyx_r = __pyx_array_MemoryView_5array_6__getattr__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v_attr));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_array_MemoryView_5array_6__getattr__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_attr) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getattr__", 0);

  /* "View.MemoryView":224
 * 
 *     def __getattr__(self, attr):
 *         return getattr(self.memview, attr)             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, item):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_memview); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 224; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetAttr(__pyx_t_1, __pyx_v_attr); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 224; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":223
 * 
 * 
 *     def __getattr__(self, attr):             # <<<<<<<<<<<<<<
 *         return getattr(self.memview, attr)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.array.__getattr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":226
 *         return getattr(self.memview, attr)
 * 
 *     def __getitem__(self, item):             # <<<<<<<<<<<<<<
 *         return self.memview[item]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_array___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item); /*proto*/
static PyObject *__pyx_array___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_array_MemoryView_5array_8__getitem__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v_item));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_array_MemoryView_5array_8__getitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "View.MemoryView":227
 * 
 *     def __getitem__(self, item):
 *         return self.memview[item]             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(self, item, value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_memview); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_GetItem(__pyx_t_1, __pyx_v_item); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":226
 *         return getattr(self.memview, attr)
 * 
 *     def __getitem__(self, item):             # <<<<<<<<<<<<<<
 *         return self.memview[item]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.array.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":229
 *         return self.memview[item]
 * 
 *     def __setitem__(self, item, value):             # <<<<<<<<<<<<<<
 *         self.memview[item] = value
 * 
 */

/* Python wrapper */
static int __pyx_array___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value); /*proto*/
static int __pyx_array___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_r = __pyx_array_MemoryView_5array_10__setitem__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v_item), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_array_MemoryView_5array_10__setitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setitem__", 0);

  /* "View.MemoryView":230
 * 
 *     def __setitem__(self, item, value):
 *         self.memview[item] = value             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_memview); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 230; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(PyObject_SetItem(__pyx_t_1, __pyx_v_item, __pyx_v_value) < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 230; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "View.MemoryView":229
 *         return self.memview[item]
 * 
 *     def __setitem__(self, item, value):             # <<<<<<<<<<<<<<
 *         self.memview[item] = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.array.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":234
 * 
 * @cname("__pyx_array_new")
 * cdef array array_cwrapper(tuple shape, Py_ssize_t itemsize, char *format,             # <<<<<<<<<<<<<<
 *                           char *mode, char *buf):
 *     cdef array result
 */

static struct __pyx_array_obj *__pyx_array_new(PyObject *__pyx_v_shape, Py_ssize_t __pyx_v_itemsize, char *__pyx_v_format, char *__pyx_v_mode, char *__pyx_v_buf) {
  struct __pyx_array_obj *__pyx_v_result = 0;
  struct __pyx_array_obj *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("array_cwrapper", 0);

  /* "View.MemoryView":238
 *     cdef array result
 * 
 *     if buf == NULL:             # <<<<<<<<<<<<<<
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_buf == NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":239
 * 
 *     if buf == NULL:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))             # <<<<<<<<<<<<<<
 *     else:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),
 */
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_itemsize); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyBytes_FromString(__pyx_v_format); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_decode_c_string(__pyx_v_mode, 0, strlen(__pyx_v_mode), NULL, NULL, PyUnicode_DecodeASCII); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v_shape);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_shape);
    __Pyx_GIVEREF(__pyx_v_shape);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)((PyObject *)__pyx_array_type)), __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_result = ((struct __pyx_array_obj *)__pyx_t_4);
    __pyx_t_4 = 0;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "View.MemoryView":241
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))
 *     else:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),             # <<<<<<<<<<<<<<
 *                        allocate_buffer=False)
 *         result.data = buf
 */
    __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_itemsize); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyBytes_FromString(__pyx_v_format); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __Pyx_decode_c_string(__pyx_v_mode, 0, strlen(__pyx_v_mode), NULL, NULL, PyUnicode_DecodeASCII); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = PyTuple_New(4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_shape);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_shape);
    __Pyx_GIVEREF(__pyx_v_shape);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_t_3 = 0;
    __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);

    /* "View.MemoryView":242
 *     else:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),
 *                        allocate_buffer=False)             # <<<<<<<<<<<<<<
 *         result.data = buf
 * 
 */
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_allocate_buffer, Py_False) < 0) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "View.MemoryView":241
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))
 *     else:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),             # <<<<<<<<<<<<<<
 *                        allocate_buffer=False)
 *         result.data = buf
 */
    __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)((PyObject *)__pyx_array_type)), __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_result = ((struct __pyx_array_obj *)__pyx_t_5);
    __pyx_t_5 = 0;

    /* "View.MemoryView":243
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),
 *                        allocate_buffer=False)
 *         result.data = buf             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
    __pyx_v_result->data = __pyx_v_buf;
  }
  __pyx_L3:;

  /* "View.MemoryView":245
 *         result.data = buf
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_result));
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "View.MemoryView":234
 * 
 * @cname("__pyx_array_new")
 * cdef array array_cwrapper(tuple shape, Py_ssize_t itemsize, char *format,             # <<<<<<<<<<<<<<
 *                           char *mode, char *buf):
 *     cdef array result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.array_cwrapper", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":271
 * cdef class Enum(object):
 *     cdef object name
 *     def __init__(self, name):             # <<<<<<<<<<<<<<
 *         self.name = name
 *     def __repr__(self):
 */

/* Python wrapper */
static int __pyx_MemviewEnum___init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_MemviewEnum___init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name_2,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name_2)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 271; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_name = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[3]; __pyx_lineno = 271; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("View.MemoryView.Enum.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_MemviewEnum_MemoryView_4Enum___init__(((struct __pyx_MemviewEnum_obj *)__pyx_v_self), __pyx_v_name);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_MemviewEnum_MemoryView_4Enum___init__(struct __pyx_MemviewEnum_obj *__pyx_v_self, PyObject *__pyx_v_name) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "View.MemoryView":272
 *     cdef object name
 *     def __init__(self, name):
 *         self.name = name             # <<<<<<<<<<<<<<
 *     def __repr__(self):
 *         return self.name
 */
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = __pyx_v_name;

  /* "View.MemoryView":271
 * cdef class Enum(object):
 *     cdef object name
 *     def __init__(self, name):             # <<<<<<<<<<<<<<
 *         self.name = name
 *     def __repr__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":273
 *     def __init__(self, name):
 *         self.name = name
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return self.name
 * 
 */

/* Python wrapper */
static PyObject *__pyx_MemviewEnum___repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_MemviewEnum___repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_MemviewEnum_MemoryView_4Enum_2__repr__(((struct __pyx_MemviewEnum_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_MemviewEnum_MemoryView_4Enum_2__repr__(struct __pyx_MemviewEnum_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "View.MemoryView":274
 *         self.name = name
 *     def __repr__(self):
 *         return self.name             # <<<<<<<<<<<<<<
 * 
 * cdef generic = Enum("<strided and direct or indirect>")
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->name);
  __pyx_r = __pyx_v_self->name;
  goto __pyx_L0;

  /* "View.MemoryView":273
 *     def __init__(self, name):
 *         self.name = name
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return self.name
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":288
 * 
 * @cname('__pyx_align_pointer')
 * cdef void *align_pointer(void *memory, size_t alignment) nogil:             # <<<<<<<<<<<<<<
 *     "Align pointer memory on a given boundary"
 *     cdef Py_intptr_t aligned_p = <Py_intptr_t> memory
 */

static void *__pyx_align_pointer(void *__pyx_v_memory, size_t __pyx_v_alignment) {
  Py_intptr_t __pyx_v_aligned_p;
  size_t __pyx_v_offset;
  void *__pyx_r;
  int __pyx_t_1;

  /* "View.MemoryView":290
 * cdef void *align_pointer(void *memory, size_t alignment) nogil:
 *     "Align pointer memory on a given boundary"
 *     cdef Py_intptr_t aligned_p = <Py_intptr_t> memory             # <<<<<<<<<<<<<<
 *     cdef size_t offset
 * 
 */
  __pyx_v_aligned_p = ((Py_intptr_t)__pyx_v_memory);

  /* "View.MemoryView":294
 * 
 *     with cython.cdivision(True):
 *         offset = aligned_p % alignment             # <<<<<<<<<<<<<<
 * 
 *     if offset > 0:
 */
  __pyx_v_offset = (__pyx_v_aligned_p % __pyx_v_alignment);

  /* "View.MemoryView":296
 *         offset = aligned_p % alignment
 * 
 *     if offset > 0:             # <<<<<<<<<<<<<<
 *         aligned_p += alignment - offset
 * 
 */
  __pyx_t_1 = ((__pyx_v_offset > 0) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":297
 * 
 *     if offset > 0:
 *         aligned_p += alignment - offset             # <<<<<<<<<<<<<<
 * 
 *     return <void *> aligned_p
 */
    __pyx_v_aligned_p = (__pyx_v_aligned_p + (__pyx_v_alignment - __pyx_v_offset));
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "View.MemoryView":299
 *         aligned_p += alignment - offset
 * 
 *     return <void *> aligned_p             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview')
 */
  __pyx_r = ((void *)__pyx_v_aligned_p);
  goto __pyx_L0;

  /* "View.MemoryView":288
 * 
 * @cname('__pyx_align_pointer')
 * cdef void *align_pointer(void *memory, size_t alignment) nogil:             # <<<<<<<<<<<<<<
 *     "Align pointer memory on a given boundary"
 *     cdef Py_intptr_t aligned_p = <Py_intptr_t> memory
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":317
 *     cdef __Pyx_TypeInfo *typeinfo
 * 
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):             # <<<<<<<<<<<<<<
 *         self.obj = obj
 *         self.flags = flags
 */

/* Python wrapper */
static int __pyx_memoryview___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_memoryview___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_obj = 0;
  int __pyx_v_flags;
  int __pyx_v_dtype_is_object;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_obj,&__pyx_n_s_flags,&__pyx_n_s_dtype_is_object,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_obj)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_flags)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 3, 1); {__pyx_filename = __pyx_f[3]; __pyx_lineno = 317; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_dtype_is_object);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 317; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_obj = values[0];
    __pyx_v_flags = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_flags == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 317; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_dtype_is_object = __Pyx_PyObject_IsTrue(values[2]); if (unlikely((__pyx_v_dtype_is_object == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 317; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_dtype_is_object = ((int)0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[3]; __pyx_lineno = 317; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("View.MemoryView.memoryview.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_memoryview_MemoryView_10memoryview___cinit__(((struct __pyx_memoryview_obj *)__pyx_v_self), __pyx_v_obj, __pyx_v_flags, __pyx_v_dtype_is_object);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_memoryview_MemoryView_10memoryview___cinit__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj, int __pyx_v_flags, int __pyx_v_dtype_is_object) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "View.MemoryView":318
 * 
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):
 *         self.obj = obj             # <<<<<<<<<<<<<<
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:
 */
  __Pyx_INCREF(__pyx_v_obj);
  __Pyx_GIVEREF(__pyx_v_obj);
  __Pyx_GOTREF(__pyx_v_self->obj);
  __Pyx_DECREF(__pyx_v_self->obj);
  __pyx_v_self->obj = __pyx_v_obj;

  /* "View.MemoryView":319
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):
 *         self.obj = obj
 *         self.flags = flags             # <<<<<<<<<<<<<<
 *         if type(self) is memoryview or obj is not None:
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 */
  __pyx_v_self->flags = __pyx_v_flags;

  /* "View.MemoryView":320
 *         self.obj = obj
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:             # <<<<<<<<<<<<<<
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:
 */
  __pyx_t_2 = (((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))) == ((PyObject *)((PyObject *)__pyx_memoryview_type)));
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (__pyx_v_obj != Py_None);
  __pyx_t_2 = (__pyx_t_3 != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "View.MemoryView":321
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:
 *             __Pyx_GetBuffer(obj, &self.view, flags)             # <<<<<<<<<<<<<<
 *             if <PyObject *> self.view.obj == NULL:
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 */
    __pyx_t_4 = __Pyx_GetBuffer(__pyx_v_obj, (&__pyx_v_self->view), __pyx_v_flags); if (unlikely(__pyx_t_4 == -1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 321; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "View.MemoryView":322
 *         if type(self) is memoryview or obj is not None:
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:             # <<<<<<<<<<<<<<
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 *                 Py_INCREF(Py_None)
 */
    __pyx_t_1 = ((((PyObject *)__pyx_v_self->view.obj) == NULL) != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":323
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:
 *                 (<__pyx_buffer *> &self.view).obj = Py_None             # <<<<<<<<<<<<<<
 *                 Py_INCREF(Py_None)
 * 
 */
      ((Py_buffer *)(&__pyx_v_self->view))->obj = Py_None;

      /* "View.MemoryView":324
 *             if <PyObject *> self.view.obj == NULL:
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 *                 Py_INCREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *         self.lock = PyThread_allocate_lock()
 */
      Py_INCREF(Py_None);
      goto __pyx_L6;
    }
    __pyx_L6:;
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "View.MemoryView":326
 *                 Py_INCREF(Py_None)
 * 
 *         self.lock = PyThread_allocate_lock()             # <<<<<<<<<<<<<<
 *         if self.lock == NULL:
 *             raise MemoryError
 */
  __pyx_v_self->lock = PyThread_allocate_lock();

  /* "View.MemoryView":327
 * 
 *         self.lock = PyThread_allocate_lock()
 *         if self.lock == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->lock == NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":328
 *         self.lock = PyThread_allocate_lock()
 *         if self.lock == NULL:
 *             raise MemoryError             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_FORMAT:
 */
    PyErr_NoMemory(); {__pyx_filename = __pyx_f[3]; __pyx_lineno = 328; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "View.MemoryView":330
 *             raise MemoryError
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             self.dtype_is_object = self.view.format == b'O'
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_FORMAT) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":331
 * 
 *         if flags & PyBUF_FORMAT:
 *             self.dtype_is_object = self.view.format == b'O'             # <<<<<<<<<<<<<<
 *         else:
 *             self.dtype_is_object = dtype_is_object
 */
    __pyx_t_5 = __Pyx_PyBytes_FromString(__pyx_v_self->view.format); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 331; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyObject_RichCompare(__pyx_t_5, __pyx_n_b_O, Py_EQ); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 331; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 331; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_self->dtype_is_object = __pyx_t_1;
    goto __pyx_L8;
  }
  /*else*/ {

    /* "View.MemoryView":333
 *             self.dtype_is_object = self.view.format == b'O'
 *         else:
 *             self.dtype_is_object = dtype_is_object             # <<<<<<<<<<<<<<
 * 
 *         self.acquisition_count_aligned_p = <__pyx_atomic_int *> align_pointer(
 */
    __pyx_v_self->dtype_is_object = __pyx_v_dtype_is_object;
  }
  __pyx_L8:;

  /* "View.MemoryView":335
 *             self.dtype_is_object = dtype_is_object
 * 
 *         self.acquisition_count_aligned_p = <__pyx_atomic_int *> align_pointer(             # <<<<<<<<<<<<<<
 *                   <void *> &self.acquisition_count[0], sizeof(__pyx_atomic_int))
 *         self.typeinfo = NULL
 */
  __pyx_v_self->acquisition_count_aligned_p = ((__pyx_atomic_int *)__pyx_align_pointer(((void *)(&(__pyx_v_self->acquisition_count[0]))), (sizeof(__pyx_atomic_int))));

  /* "View.MemoryView":337
 *         self.acquisition_count_aligned_p = <__pyx_atomic_int *> align_pointer(
 *                   <void *> &self.acquisition_count[0], sizeof(__pyx_atomic_int))
 *         self.typeinfo = NULL             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(memoryview self):
 */
  __pyx_v_self->typeinfo = NULL;

  /* "View.MemoryView":317
 *     cdef __Pyx_TypeInfo *typeinfo
 * 
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):             # <<<<<<<<<<<<<<
 *         self.obj = obj
 *         self.flags = flags
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":339
 *         self.typeinfo = NULL
 * 
 *     def __dealloc__(memoryview self):             # <<<<<<<<<<<<<<
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)
 */

/* Python wrapper */
static void __pyx_memoryview___dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_memoryview___dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_memoryview_MemoryView_10memoryview_2__dealloc__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_memoryview_MemoryView_10memoryview_2__dealloc__(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "View.MemoryView":340
 * 
 *     def __dealloc__(memoryview self):
 *         if self.obj is not None:             # <<<<<<<<<<<<<<
 *             __Pyx_ReleaseBuffer(&self.view)
 * 
 */
  __pyx_t_1 = (__pyx_v_self->obj != Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":341
 *     def __dealloc__(memoryview self):
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)             # <<<<<<<<<<<<<<
 * 
 *         if self.lock != NULL:
 */
    __Pyx_ReleaseBuffer((&__pyx_v_self->view));
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "View.MemoryView":343
 *             __Pyx_ReleaseBuffer(&self.view)
 * 
 *         if self.lock != NULL:             # <<<<<<<<<<<<<<
 *             PyThread_free_lock(self.lock)
 * 
 */
  __pyx_t_2 = ((__pyx_v_self->lock != NULL) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":344
 * 
 *         if self.lock != NULL:
 *             PyThread_free_lock(self.lock)             # <<<<<<<<<<<<<<
 * 
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:
 */
    PyThread_free_lock(__pyx_v_self->lock);
    goto __pyx_L4;
  }
  __pyx_L4:;

  /* "View.MemoryView":339
 *         self.typeinfo = NULL
 * 
 *     def __dealloc__(memoryview self):             # <<<<<<<<<<<<<<
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "View.MemoryView":346
 *             PyThread_free_lock(self.lock)
 * 
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t dim
 *         cdef char *itemp = <char *> self.view.buf
 */

static char *__pyx_memoryview_get_item_pointer(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index) {
  Py_ssize_t __pyx_v_dim;
  char *__pyx_v_itemp;
  PyObject *__pyx_v_idx = NULL;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  char *__pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_item_pointer", 0);

  /* "View.MemoryView":348
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:
 *         cdef Py_ssize_t dim
 *         cdef char *itemp = <char *> self.view.buf             # <<<<<<<<<<<<<<
 * 
 *         for dim, idx in enumerate(index):
 */
  __pyx_v_itemp = ((char *)__pyx_v_self->view.buf);

  /* "View.MemoryView":350
 *         cdef char *itemp = <char *> self.view.buf
 * 
 *         for dim, idx in enumerate(index):             # <<<<<<<<<<<<<<
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)
 * 
 */
  __pyx_t_1 = 0;
  if (likely(PyList_CheckExact(__pyx_v_index)) || PyTuple_CheckExact(__pyx_v_index)) {
    __pyx_t_2 = __pyx_v_index; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_index); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 350; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 350; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_5 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 350; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 350; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 350; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 350; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #endif
      }
    } else {
      __pyx_t_5 = __pyx_t_4(__pyx_t_2);
      if (unlikely(!__pyx_t_5)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[3]; __pyx_lineno = 350; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_5);
    }
    __Pyx_XDECREF_SET(__pyx_v_idx, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_v_dim = __pyx_t_1;
    __pyx_t_1 = (__pyx_t_1 + 1);

    /* "View.MemoryView":351
 * 
 *         for dim, idx in enumerate(index):
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)             # <<<<<<<<<<<<<<
 * 
 *         return itemp
 */
    __pyx_t_6 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_6 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 351; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_7 = __pyx_pybuffer_index((&__pyx_v_self->view), __pyx_v_itemp, __pyx_t_6, __pyx_v_dim); if (unlikely(__pyx_t_7 == NULL)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 351; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_itemp = __pyx_t_7;

    /* "View.MemoryView":350
 *         cdef char *itemp = <char *> self.view.buf
 * 
 *         for dim, idx in enumerate(index):             # <<<<<<<<<<<<<<
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":353
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)
 * 
 *         return itemp             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_itemp;
  goto __pyx_L0;

  /* "View.MemoryView":346
 *             PyThread_free_lock(self.lock)
 * 
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t dim
 *         cdef char *itemp = <char *> self.view.buf
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.memoryview.get_item_pointer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_idx);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":356
 * 
 * 
 *     def __getitem__(memoryview self, object index):             # <<<<<<<<<<<<<<
 *         if index is Ellipsis:
 *             return self
 */

/* Python wrapper */
static PyObject *__pyx_memoryview___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index); /*proto*/
static PyObject *__pyx_memoryview___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview_MemoryView_10memoryview_4__getitem__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((PyObject *)__pyx_v_index));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview_MemoryView_10memoryview_4__getitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index) {
  PyObject *__pyx_v_have_slices = NULL;
  PyObject *__pyx_v_indices = NULL;
  char *__pyx_v_itemp;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  char *__pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "View.MemoryView":357
 * 
 *     def __getitem__(memoryview self, object index):
 *         if index is Ellipsis:             # <<<<<<<<<<<<<<
 *             return self
 * 
 */
  __pyx_t_1 = (__pyx_v_index == __pyx_builtin_Ellipsis);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":358
 *     def __getitem__(memoryview self, object index):
 *         if index is Ellipsis:
 *             return self             # <<<<<<<<<<<<<<
 * 
 *         have_slices, indices = _unellipsify(index, self.view.ndim)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __pyx_r = ((PyObject *)__pyx_v_self);
    goto __pyx_L0;
  }

  /* "View.MemoryView":360
 *             return self
 * 
 *         have_slices, indices = _unellipsify(index, self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *         cdef char *itemp
 */
  __pyx_t_3 = _unellipsify(__pyx_v_index, __pyx_v_self->view.ndim); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (likely(__pyx_t_3 != Py_None)) {
    PyObject* sequence = __pyx_t_3;
    #if CYTHON_COMPILING_IN_CPYTHON
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      {__pyx_filename = __pyx_f[3]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 
    __pyx_t_5 = PyTuple_GET_ITEM(sequence, 1); 
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_5);
    #else
    __pyx_t_4 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    #endif
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __Pyx_RaiseNoneNotIterableError(); {__pyx_filename = __pyx_f[3]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_v_have_slices = __pyx_t_4;
  __pyx_t_4 = 0;
  __pyx_v_indices = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "View.MemoryView":363
 * 
 *         cdef char *itemp
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             return memview_slice(self, indices)
 *         else:
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_have_slices); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 363; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_2) {

    /* "View.MemoryView":364
 *         cdef char *itemp
 *         if have_slices:
 *             return memview_slice(self, indices)             # <<<<<<<<<<<<<<
 *         else:
 *             itemp = self.get_item_pointer(indices)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = ((PyObject *)__pyx_memview_slice(__pyx_v_self, __pyx_v_indices)); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 364; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }
  /*else*/ {

    /* "View.MemoryView":366
 *             return memview_slice(self, indices)
 *         else:
 *             itemp = self.get_item_pointer(indices)             # <<<<<<<<<<<<<<
 *             return self.convert_item_to_object(itemp)
 * 
 */
    __pyx_t_6 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->get_item_pointer(__pyx_v_self, __pyx_v_indices); if (unlikely(__pyx_t_6 == NULL)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_itemp = __pyx_t_6;

    /* "View.MemoryView":367
 *         else:
 *             itemp = self.get_item_pointer(indices)
 *             return self.convert_item_to_object(itemp)             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(memoryview self, object index, object value):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->convert_item_to_object(__pyx_v_self, __pyx_v_itemp); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 367; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "View.MemoryView":356
 * 
 * 
 *     def __getitem__(memoryview self, object index):             # <<<<<<<<<<<<<<
 *         if index is Ellipsis:
 *             return self
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_have_slices);
  __Pyx_XDECREF(__pyx_v_indices);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":369
 *             return self.convert_item_to_object(itemp)
 * 
 *     def __setitem__(memoryview self, object index, object value):             # <<<<<<<<<<<<<<
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 * 
 */

/* Python wrapper */
static int __pyx_memoryview___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /*proto*/
static int __pyx_memoryview___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview_MemoryView_10memoryview_6__setitem__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((PyObject *)__pyx_v_index), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_memoryview_MemoryView_10memoryview_6__setitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_have_slices = NULL;
  PyObject *__pyx_v_obj = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setitem__", 0);
  __Pyx_INCREF(__pyx_v_index);

  /* "View.MemoryView":370
 * 
 *     def __setitem__(memoryview self, object index, object value):
 *         have_slices, index = _unellipsify(index, self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *         if have_slices:
 */
  __pyx_t_1 = _unellipsify(__pyx_v_index, __pyx_v_self->view.ndim); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 370; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(__pyx_t_1 != Py_None)) {
    PyObject* sequence = __pyx_t_1;
    #if CYTHON_COMPILING_IN_CPYTHON
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      {__pyx_filename = __pyx_f[3]; __pyx_lineno = 370; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
    __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    #else
    __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 370; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 370; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    __Pyx_RaiseNoneNotIterableError(); {__pyx_filename = __pyx_f[3]; __pyx_lineno = 370; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_v_have_slices = __pyx_t_2;
  __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_index, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "View.MemoryView":372
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 * 
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             obj = self.is_slice(value)
 *             if obj:
 */
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_have_slices); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 372; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_4) {

    /* "View.MemoryView":373
 * 
 *         if have_slices:
 *             obj = self.is_slice(value)             # <<<<<<<<<<<<<<
 *             if obj:
 *                 self.setitem_slice_assignment(self[index], obj)
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->is_slice(__pyx_v_self, __pyx_v_value); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 373; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_obj = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "View.MemoryView":374
 *         if have_slices:
 *             obj = self.is_slice(value)
 *             if obj:             # <<<<<<<<<<<<<<
 *                 self.setitem_slice_assignment(self[index], obj)
 *             else:
 */
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_obj); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 374; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (__pyx_t_4) {

      /* "View.MemoryView":375
 *             obj = self.is_slice(value)
 *             if obj:
 *                 self.setitem_slice_assignment(self[index], obj)             # <<<<<<<<<<<<<<
 *             else:
 *                 self.setitem_slice_assign_scalar(self[index], value)
 */
      __pyx_t_1 = PyObject_GetItem(((PyObject *)__pyx_v_self), __pyx_v_index); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 375; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_3 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->setitem_slice_assignment(__pyx_v_self, __pyx_t_1, __pyx_v_obj); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 375; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      goto __pyx_L4;
    }
    /*else*/ {

      /* "View.MemoryView":377
 *                 self.setitem_slice_assignment(self[index], obj)
 *             else:
 *                 self.setitem_slice_assign_scalar(self[index], value)             # <<<<<<<<<<<<<<
 *         else:
 *             self.setitem_indexed(index, value)
 */
      __pyx_t_3 = PyObject_GetItem(((PyObject *)__pyx_v_self), __pyx_v_index); if (unlikely(__pyx_t_3 == NULL)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 377; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
      __Pyx_GOTREF(__pyx_t_3);
      if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_memoryview_type))))) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 377; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_t_1 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->setitem_slice_assign_scalar(__pyx_v_self, ((struct __pyx_memoryview_obj *)__pyx_t_3), __pyx_v_value); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 377; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __pyx_L4:;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "View.MemoryView":379
 *                 self.setitem_slice_assign_scalar(self[index], value)
 *         else:
 *             self.setitem_indexed(index, value)             # <<<<<<<<<<<<<<
 * 
 *     cdef is_slice(self, obj):
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->setitem_indexed(__pyx_v_self, __pyx_v_index, __pyx_v_value); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 379; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "View.MemoryView":369
 *             return self.convert_item_to_object(itemp)
 * 
 *     def __setitem__(memoryview self, object index, object value):             # <<<<<<<<<<<<<<
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_have_slices);
  __Pyx_XDECREF(__pyx_v_obj);
  __Pyx_XDECREF(__pyx_v_index);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":381
 *             self.setitem_indexed(index, value)
 * 
 *     cdef is_slice(self, obj):             # <<<<<<<<<<<<<<
 *         if not isinstance(obj, memoryview):
 *             try:
 */

static PyObject *__pyx_memoryview_is_slice(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_slice", 0);
  __Pyx_INCREF(__pyx_v_obj);

  /* "View.MemoryView":382
 * 
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):             # <<<<<<<<<<<<<<
 *             try:
 *                 obj = memoryview(obj, self.flags|PyBUF_ANY_CONTIGUOUS,
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_obj, ((PyObject *)__pyx_memoryview_type)); 
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":383
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):
 *             try:             # <<<<<<<<<<<<<<
 *                 obj = memoryview(obj, self.flags|PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
 */
    {
      __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_5);
      /*try:*/ {

        /* "View.MemoryView":384
 *         if not isinstance(obj, memoryview):
 *             try:
 *                 obj = memoryview(obj, self.flags|PyBUF_ANY_CONTIGUOUS,             # <<<<<<<<<<<<<<
 *                                  self.dtype_is_object)
 *             except TypeError:
 */
        __pyx_t_6 = __Pyx_PyInt_From_int((__pyx_v_self->flags | PyBUF_ANY_CONTIGUOUS)); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 384; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
        __Pyx_GOTREF(__pyx_t_6);

        /* "View.MemoryView":385
 *             try:
 *                 obj = memoryview(obj, self.flags|PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)             # <<<<<<<<<<<<<<
 *             except TypeError:
 *                 return None
 */
        __pyx_t_7 = __Pyx_PyBool_FromLong(__pyx_v_self->dtype_is_object); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 385; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
        __Pyx_GOTREF(__pyx_t_7);

        /* "View.MemoryView":384
 *         if not isinstance(obj, memoryview):
 *             try:
 *                 obj = memoryview(obj, self.flags|PyBUF_ANY_CONTIGUOUS,             # <<<<<<<<<<<<<<
 *                                  self.dtype_is_object)
 *             except TypeError:
 */
        __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 384; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_INCREF(__pyx_v_obj);
        PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_v_obj);
        __Pyx_GIVEREF(__pyx_v_obj);
        PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_t_7);
        __Pyx_GIVEREF(__pyx_t_7);
        __pyx_t_6 = 0;
        __pyx_t_7 = 0;
        __pyx_t_7 = __Pyx_PyObject_Call(((PyObject *)((PyObject *)__pyx_memoryview_type)), __pyx_t_8, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 384; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF_SET(__pyx_v_obj, __pyx_t_7);
        __pyx_t_7 = 0;
      }
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      goto __pyx_L11_try_end;
      __pyx_L4_error:;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "View.MemoryView":386
 *                 obj = memoryview(obj, self.flags|PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
 *             except TypeError:             # <<<<<<<<<<<<<<
 *                 return None
 * 
 */
      __pyx_t_9 = PyErr_ExceptionMatches(__pyx_builtin_TypeError);
      if (__pyx_t_9) {
        __Pyx_AddTraceback("View.MemoryView.memoryview.is_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_8, &__pyx_t_6) < 0) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 386; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_GOTREF(__pyx_t_6);

        /* "View.MemoryView":387
 *                                  self.dtype_is_object)
 *             except TypeError:
 *                 return None             # <<<<<<<<<<<<<<
 * 
 *         return obj
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(Py_None);
        __pyx_r = Py_None;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L7_except_return;
      }
      goto __pyx_L6_except_error;
      __pyx_L6_except_error:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L1_error;
      __pyx_L7_except_return:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L0;
      __pyx_L11_try_end:;
    }
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "View.MemoryView":389
 *                 return None
 * 
 *         return obj             # <<<<<<<<<<<<<<
 * 
 *     cdef setitem_slice_assignment(self, dst, src):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_obj);
  __pyx_r = __pyx_v_obj;
  goto __pyx_L0;

  /* "View.MemoryView":381
 *             self.setitem_indexed(index, value)
 * 
 *     cdef is_slice(self, obj):             # <<<<<<<<<<<<<<
 *         if not isinstance(obj, memoryview):
 *             try:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("View.MemoryView.memoryview.is_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_obj);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":391
 *         return obj
 * 
 *     cdef setitem_slice_assignment(self, dst, src):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice dst_slice
 *         cdef __Pyx_memviewslice src_slice
 */

static PyObject *__pyx_memoryview_setitem_slice_assignment(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_dst, PyObject *__pyx_v_src) {
  __Pyx_memviewslice __pyx_v_dst_slice;
  __Pyx_memviewslice __pyx_v_src_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setitem_slice_assignment", 0);

  /* "View.MemoryView":395
 *         cdef __Pyx_memviewslice src_slice
 * 
 *         memoryview_copy_contents(get_slice_from_memview(src, &src_slice)[0],             # <<<<<<<<<<<<<<
 *                                  get_slice_from_memview(dst, &dst_slice)[0],
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 */
  if (!(likely(((__pyx_v_src) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_src, __pyx_memoryview_type))))) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 395; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "View.MemoryView":396
 * 
 *         memoryview_copy_contents(get_slice_from_memview(src, &src_slice)[0],
 *                                  get_slice_from_memview(dst, &dst_slice)[0],             # <<<<<<<<<<<<<<
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 * 
 */
  if (!(likely(((__pyx_v_dst) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_dst, __pyx_memoryview_type))))) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "View.MemoryView":397
 *         memoryview_copy_contents(get_slice_from_memview(src, &src_slice)[0],
 *                                  get_slice_from_memview(dst, &dst_slice)[0],
 *                                  src.ndim, dst.ndim, self.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_src, __pyx_n_s_ndim); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dst, __pyx_n_s_ndim); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "View.MemoryView":395
 *         cdef __Pyx_memviewslice src_slice
 * 
 *         memoryview_copy_contents(get_slice_from_memview(src, &src_slice)[0],             # <<<<<<<<<<<<<<
 *                                  get_slice_from_memview(dst, &dst_slice)[0],
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 */
  __pyx_t_4 = __pyx_memoryview_copy_contents((__pyx_memoryview_get_slice_from_memoryview(((struct __pyx_memoryview_obj *)__pyx_v_src), (&__pyx_v_src_slice))[0]), (__pyx_memoryview_get_slice_from_memoryview(((struct __pyx_memoryview_obj *)__pyx_v_dst), (&__pyx_v_dst_slice))[0]), __pyx_t_2, __pyx_t_3, __pyx_v_self->dtype_is_object); if (unlikely(__pyx_t_4 == -1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 395; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "View.MemoryView":391
 *         return obj
 * 
 *     cdef setitem_slice_assignment(self, dst, src):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice dst_slice
 *         cdef __Pyx_memviewslice src_slice
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.setitem_slice_assignment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":399
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 * 
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):             # <<<<<<<<<<<<<<
 *         cdef int array[128]
 *         cdef void *tmp = NULL
 */

static PyObject *__pyx_memoryview_setitem_slice_assign_scalar(struct __pyx_memoryview_obj *__pyx_v_self, struct __pyx_memoryview_obj *__pyx_v_dst, PyObject *__pyx_v_value) {
  int __pyx_v_array[128];
  void *__pyx_v_tmp;
  void *__pyx_v_item;
  __Pyx_memviewslice *__pyx_v_dst_slice;
  __Pyx_memviewslice __pyx_v_tmp_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  char const *__pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setitem_slice_assign_scalar", 0);

  /* "View.MemoryView":401
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):
 *         cdef int array[128]
 *         cdef void *tmp = NULL             # <<<<<<<<<<<<<<
 *         cdef void *item
 * 
 */
  __pyx_v_tmp = NULL;

  /* "View.MemoryView":406
 *         cdef __Pyx_memviewslice *dst_slice
 *         cdef __Pyx_memviewslice tmp_slice
 *         dst_slice = get_slice_from_memview(dst, &tmp_slice)             # <<<<<<<<<<<<<<
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):
 */
  __pyx_v_dst_slice = __pyx_memoryview_get_slice_from_memoryview(__pyx_v_dst, (&__pyx_v_tmp_slice));

  /* "View.MemoryView":408
 *         dst_slice = get_slice_from_memview(dst, &tmp_slice)
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):             # <<<<<<<<<<<<<<
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:
 */
  __pyx_t_1 = ((((size_t)__pyx_v_self->view.itemsize) > (sizeof(__pyx_v_array))) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":409
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):
 *             tmp = PyMem_Malloc(self.view.itemsize)             # <<<<<<<<<<<<<<
 *             if tmp == NULL:
 *                 raise MemoryError
 */
    __pyx_v_tmp = PyMem_Malloc(__pyx_v_self->view.itemsize);

    /* "View.MemoryView":410
 *         if <size_t>self.view.itemsize > sizeof(array):
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 *             item = tmp
 */
    __pyx_t_1 = ((__pyx_v_tmp == NULL) != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":411
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:
 *                 raise MemoryError             # <<<<<<<<<<<<<<
 *             item = tmp
 *         else:
 */
      PyErr_NoMemory(); {__pyx_filename = __pyx_f[3]; __pyx_lineno = 411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }

    /* "View.MemoryView":412
 *             if tmp == NULL:
 *                 raise MemoryError
 *             item = tmp             # <<<<<<<<<<<<<<
 *         else:
 *             item = <void *> array
 */
    __pyx_v_item = __pyx_v_tmp;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "View.MemoryView":414
 *             item = tmp
 *         else:
 *             item = <void *> array             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
    __pyx_v_item = ((void *)__pyx_v_array);
  }
  __pyx_L3:;

  /* "View.MemoryView":416
 *             item = <void *> array
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             if self.dtype_is_object:
 *                 (<PyObject **> item)[0] = <PyObject *> value
 */
  /*try:*/ {

    /* "View.MemoryView":417
 * 
 *         try:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 (<PyObject **> item)[0] = <PyObject *> value
 *             else:
 */
    __pyx_t_1 = (__pyx_v_self->dtype_is_object != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":418
 *         try:
 *             if self.dtype_is_object:
 *                 (<PyObject **> item)[0] = <PyObject *> value             # <<<<<<<<<<<<<<
 *             else:
 *                 self.assign_item_from_object(<char *> item, value)
 */
      (((PyObject **)__pyx_v_item)[0]) = ((PyObject *)__pyx_v_value);
      goto __pyx_L8;
    }
    /*else*/ {

      /* "View.MemoryView":420
 *                 (<PyObject **> item)[0] = <PyObject *> value
 *             else:
 *                 self.assign_item_from_object(<char *> item, value)             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->assign_item_from_object(__pyx_v_self, ((char *)__pyx_v_item), __pyx_v_value); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 420; __pyx_clineno = __LINE__; goto __pyx_L6_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __pyx_L8:;

    /* "View.MemoryView":424
 * 
 * 
 *             if self.view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,
 */
    __pyx_t_1 = ((__pyx_v_self->view.suboffsets != NULL) != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":425
 * 
 *             if self.view.suboffsets != NULL:
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)             # <<<<<<<<<<<<<<
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,
 *                                 item, self.dtype_is_object)
 */
      __pyx_t_2 = assert_direct_dimensions(__pyx_v_self->view.suboffsets, __pyx_v_self->view.ndim); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 425; __pyx_clineno = __LINE__; goto __pyx_L6_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      goto __pyx_L9;
    }
    __pyx_L9:;

    /* "View.MemoryView":426
 *             if self.view.suboffsets != NULL:
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,             # <<<<<<<<<<<<<<
 *                                 item, self.dtype_is_object)
 *         finally:
 */
    __pyx_memoryview_slice_assign_scalar(__pyx_v_dst_slice, __pyx_v_dst->view.ndim, __pyx_v_self->view.itemsize, __pyx_v_item, __pyx_v_self->dtype_is_object);
  }

  /* "View.MemoryView":429
 *                                 item, self.dtype_is_object)
 *         finally:
 *             PyMem_Free(tmp)             # <<<<<<<<<<<<<<
 * 
 *     cdef setitem_indexed(self, index, value):
 */
  /*finally:*/ {
    /*normal exit:*/{
      PyMem_Free(__pyx_v_tmp);
      goto __pyx_L7;
    }
    /*exception exit:*/{
      __pyx_L6_error:;
      __pyx_t_6 = 0; __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8) < 0)) __Pyx_ErrFetch(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_11);
      __pyx_t_3 = __pyx_lineno; __pyx_t_4 = __pyx_clineno; __pyx_t_5 = __pyx_filename;
      {
        PyMem_Free(__pyx_v_tmp);
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
      }
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_XGIVEREF(__pyx_t_8);
      __Pyx_ErrRestore(__pyx_t_6, __pyx_t_7, __pyx_t_8);
      __pyx_t_6 = 0; __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0;
      __pyx_lineno = __pyx_t_3; __pyx_clineno = __pyx_t_4; __pyx_filename = __pyx_t_5;
      goto __pyx_L1_error;
    }
    __pyx_L7:;
  }

  /* "View.MemoryView":399
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 * 
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):             # <<<<<<<<<<<<<<
 *         cdef int array[128]
 *         cdef void *tmp = NULL
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.setitem_slice_assign_scalar", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":431
 *             PyMem_Free(tmp)
 * 
 *     cdef setitem_indexed(self, index, value):             # <<<<<<<<<<<<<<
 *         cdef char *itemp = self.get_item_pointer(index)
 *         self.assign_item_from_object(itemp, value)
 */

static PyObject *__pyx_memoryview_setitem_indexed(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {
  char *__pyx_v_itemp;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char *__pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setitem_indexed", 0);

  /* "View.MemoryView":432
 * 
 *     cdef setitem_indexed(self, index, value):
 *         cdef char *itemp = self.get_item_pointer(index)             # <<<<<<<<<<<<<<
 *         self.assign_item_from_object(itemp, value)
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->get_item_pointer(__pyx_v_self, __pyx_v_index); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 432; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_itemp = __pyx_t_1;

  /* "View.MemoryView":433
 *     cdef setitem_indexed(self, index, value):
 *         cdef char *itemp = self.get_item_pointer(index)
 *         self.assign_item_from_object(itemp, value)             # <<<<<<<<<<<<<<
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->assign_item_from_object(__pyx_v_self, __pyx_v_itemp, __pyx_v_value); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 433; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":431
 *             PyMem_Free(tmp)
 * 
 *     cdef setitem_indexed(self, index, value):             # <<<<<<<<<<<<<<
 *         cdef char *itemp = self.get_item_pointer(index)
 *         self.assign_item_from_object(itemp, value)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.setitem_indexed", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":435
 *         self.assign_item_from_object(itemp, value)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 */

static PyObject *__pyx_memoryview_convert_item_to_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp) {
  PyObject *__pyx_v_struct = NULL;
  PyObject *__pyx_v_bytesitem = 0;
  PyObject *__pyx_v_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("convert_item_to_object", 0);

  /* "View.MemoryView":438
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 *         import struct             # <<<<<<<<<<<<<<
 *         cdef bytes bytesitem
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_struct, 0, -1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 438; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_struct = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "View.MemoryView":441
 *         cdef bytes bytesitem
 * 
 *         bytesitem = itemp[:self.view.itemsize]             # <<<<<<<<<<<<<<
 *         try:
 *             result = struct.unpack(self.view.format, bytesitem)
 */
  __pyx_t_1 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_itemp + 0, __pyx_v_self->view.itemsize - 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 441; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_bytesitem = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "View.MemoryView":442
 * 
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:             # <<<<<<<<<<<<<<
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 */
  {
    __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    /*try:*/ {

      /* "View.MemoryView":443
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:
 *             result = struct.unpack(self.view.format, bytesitem)             # <<<<<<<<<<<<<<
 *         except struct.error:
 *             raise ValueError("Unable to convert item to object")
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_unpack); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 443; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = __Pyx_PyBytes_FromString(__pyx_v_self->view.format); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 443; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = NULL;
      __pyx_t_8 = 0;
      if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
          __pyx_t_8 = 1;
        }
      }
      __pyx_t_9 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 443; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      __Pyx_GOTREF(__pyx_t_9);
      if (__pyx_t_7) {
        PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __Pyx_GIVEREF(__pyx_t_7); __pyx_t_7 = NULL;
      }
      PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_6);
      __Pyx_INCREF(__pyx_v_bytesitem);
      PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_v_bytesitem);
      __Pyx_GIVEREF(__pyx_v_bytesitem);
      __pyx_t_6 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 443; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_v_result = __pyx_t_1;
      __pyx_t_1 = 0;
    }
    /*else:*/ {

      /* "View.MemoryView":447
 *             raise ValueError("Unable to convert item to object")
 *         else:
 *             if len(self.view.format) == 1:             # <<<<<<<<<<<<<<
 *                 return result[0]
 *             return result
 */
      __pyx_t_10 = strlen(__pyx_v_self->view.format); 
      __pyx_t_11 = ((__pyx_t_10 == 1) != 0);
      if (__pyx_t_11) {

        /* "View.MemoryView":448
 *         else:
 *             if len(self.view.format) == 1:
 *                 return result[0]             # <<<<<<<<<<<<<<
 *             return result
 * 
 */
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_result, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 448; __pyx_clineno = __LINE__; goto __pyx_L5_except_error;};
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_r = __pyx_t_1;
        __pyx_t_1 = 0;
        goto __pyx_L6_except_return;
      }

      /* "View.MemoryView":449
 *             if len(self.view.format) == 1:
 *                 return result[0]
 *             return result             # <<<<<<<<<<<<<<
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_result);
      __pyx_r = __pyx_v_result;
      goto __pyx_L6_except_return;
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "View.MemoryView":444
 *         try:
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:             # <<<<<<<<<<<<<<
 *             raise ValueError("Unable to convert item to object")
 *         else:
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_error); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 444; __pyx_clineno = __LINE__; goto __pyx_L5_except_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_12 = PyErr_ExceptionMatches(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_12) {
      __Pyx_AddTraceback("View.MemoryView.memoryview.convert_item_to_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_5, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 444; __pyx_clineno = __LINE__; goto __pyx_L5_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_9);

      /* "View.MemoryView":445
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 *             raise ValueError("Unable to convert item to object")             # <<<<<<<<<<<<<<
 *         else:
 *             if len(self.view.format) == 1:
 */
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__44, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L5_except_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      {__pyx_filename = __pyx_f[3]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L5_except_error;}
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L1_error;
    __pyx_L6_except_return:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L0;
  }

  /* "View.MemoryView":435
 *         self.assign_item_from_object(itemp, value)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("View.MemoryView.memoryview.convert_item_to_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_struct);
  __Pyx_XDECREF(__pyx_v_bytesitem);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":451
 *             return result
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 */

static PyObject *__pyx_memoryview_assign_item_from_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_struct = NULL;
  char __pyx_v_c;
  PyObject *__pyx_v_bytesvalue = 0;
  Py_ssize_t __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  char *__pyx_t_10;
  char *__pyx_t_11;
  char *__pyx_t_12;
  char *__pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("assign_item_from_object", 0);

  /* "View.MemoryView":454
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 *         import struct             # <<<<<<<<<<<<<<
 *         cdef char c
 *         cdef bytes bytesvalue
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_struct, 0, -1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 454; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_struct = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "View.MemoryView":459
 *         cdef Py_ssize_t i
 * 
 *         if isinstance(value, tuple):             # <<<<<<<<<<<<<<
 *             bytesvalue = struct.pack(self.view.format, *value)
 *         else:
 */
  __pyx_t_2 = PyTuple_Check(__pyx_v_value); 
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "View.MemoryView":460
 * 
 *         if isinstance(value, tuple):
 *             bytesvalue = struct.pack(self.view.format, *value)             # <<<<<<<<<<<<<<
 *         else:
 *             bytesvalue = struct.pack(self.view.format, value)
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_pack); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 460; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyBytes_FromString(__pyx_v_self->view.format); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 460; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 460; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = PySequence_Tuple(__pyx_v_value); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 460; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyNumber_Add(__pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 460; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_6, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 460; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (!(likely(PyBytes_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_4)->tp_name), 0))) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 460; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_bytesvalue = ((PyObject*)__pyx_t_4);
    __pyx_t_4 = 0;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "View.MemoryView":462
 *             bytesvalue = struct.pack(self.view.format, *value)
 *         else:
 *             bytesvalue = struct.pack(self.view.format, value)             # <<<<<<<<<<<<<<
 * 
 *         for i, c in enumerate(bytesvalue):
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_pack); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_self->view.format); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = NULL;
    __pyx_t_7 = 0;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
        __pyx_t_7 = 1;
      }
    }
    __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_5) {
      PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __Pyx_GIVEREF(__pyx_t_5); __pyx_t_5 = NULL;
    }
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_value);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_v_value);
    __Pyx_GIVEREF(__pyx_v_value);
    __pyx_t_1 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (!(likely(PyBytes_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_4)->tp_name), 0))) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_bytesvalue = ((PyObject*)__pyx_t_4);
    __pyx_t_4 = 0;
  }
  __pyx_L3:;

  /* "View.MemoryView":464
 *             bytesvalue = struct.pack(self.view.format, value)
 * 
 *         for i, c in enumerate(bytesvalue):             # <<<<<<<<<<<<<<
 *             itemp[i] = c
 * 
 */
  __pyx_t_7 = 0;
  if (unlikely(__pyx_v_bytesvalue == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' is not iterable");
    {__pyx_filename = __pyx_f[3]; __pyx_lineno = 464; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_INCREF(__pyx_v_bytesvalue);
  __pyx_t_9 = __pyx_v_bytesvalue;
  __pyx_t_11 = PyBytes_AS_STRING(__pyx_t_9);
  __pyx_t_12 = (__pyx_t_11 + PyBytes_GET_SIZE(__pyx_t_9));
  for (__pyx_t_13 = __pyx_t_11; __pyx_t_13 < __pyx_t_12; __pyx_t_13++) {
    __pyx_t_10 = __pyx_t_13;
    __pyx_v_c = (__pyx_t_10[0]);

    /* "View.MemoryView":465
 * 
 *         for i, c in enumerate(bytesvalue):
 *             itemp[i] = c             # <<<<<<<<<<<<<<
 * 
 *     @cname('getbuffer')
 */
    __pyx_v_i = __pyx_t_7;

    /* "View.MemoryView":464
 *             bytesvalue = struct.pack(self.view.format, value)
 * 
 *         for i, c in enumerate(bytesvalue):             # <<<<<<<<<<<<<<
 *             itemp[i] = c
 * 
 */
    __pyx_t_7 = (__pyx_t_7 + 1);

    /* "View.MemoryView":465
 * 
 *         for i, c in enumerate(bytesvalue):
 *             itemp[i] = c             # <<<<<<<<<<<<<<
 * 
 *     @cname('getbuffer')
 */
    (__pyx_v_itemp[__pyx_v_i]) = __pyx_v_c;
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "View.MemoryView":451
 *             return result
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("View.MemoryView.memoryview.assign_item_from_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_struct);
  __Pyx_XDECREF(__pyx_v_bytesvalue);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":468
 * 
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):             # <<<<<<<<<<<<<<
 *         if flags & PyBUF_STRIDES:
 *             info.shape = self.view.shape
 */

/* Python wrapper */
static CYTHON_UNUSED int __pyx_memoryview_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static CYTHON_UNUSED int __pyx_memoryview_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview_getbuffer_MemoryView_10memoryview_8__getbuffer__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_memoryview_getbuffer_MemoryView_10memoryview_8__getbuffer__(struct __pyx_memoryview_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  Py_ssize_t *__pyx_t_2;
  char *__pyx_t_3;
  void *__pyx_t_4;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  if (__pyx_v_info != NULL) {
    __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(__pyx_v_info->obj);
  }

  /* "View.MemoryView":469
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_STRIDES:             # <<<<<<<<<<<<<<
 *             info.shape = self.view.shape
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_STRIDES) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":470
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_STRIDES:
 *             info.shape = self.view.shape             # <<<<<<<<<<<<<<
 *         else:
 *             info.shape = NULL
 */
    __pyx_t_2 = __pyx_v_self->view.shape;
    __pyx_v_info->shape = __pyx_t_2;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "View.MemoryView":472
 *             info.shape = self.view.shape
 *         else:
 *             info.shape = NULL             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_STRIDES:
 */
    __pyx_v_info->shape = NULL;
  }
  __pyx_L3:;

  /* "View.MemoryView":474
 *             info.shape = NULL
 * 
 *         if flags & PyBUF_STRIDES:             # <<<<<<<<<<<<<<
 *             info.strides = self.view.strides
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_STRIDES) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":475
 * 
 *         if flags & PyBUF_STRIDES:
 *             info.strides = self.view.strides             # <<<<<<<<<<<<<<
 *         else:
 *             info.strides = NULL
 */
    __pyx_t_2 = __pyx_v_self->view.strides;
    __pyx_v_info->strides = __pyx_t_2;
    goto __pyx_L4;
  }
  /*else*/ {

    /* "View.MemoryView":477
 *             info.strides = self.view.strides
 *         else:
 *             info.strides = NULL             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_INDIRECT:
 */
    __pyx_v_info->strides = NULL;
  }
  __pyx_L4:;

  /* "View.MemoryView":479
 *             info.strides = NULL
 * 
 *         if flags & PyBUF_INDIRECT:             # <<<<<<<<<<<<<<
 *             info.suboffsets = self.view.suboffsets
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_INDIRECT) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":480
 * 
 *         if flags & PyBUF_INDIRECT:
 *             info.suboffsets = self.view.suboffsets             # <<<<<<<<<<<<<<
 *         else:
 *             info.suboffsets = NULL
 */
    __pyx_t_2 = __pyx_v_self->view.suboffsets;
    __pyx_v_info->suboffsets = __pyx_t_2;
    goto __pyx_L5;
  }
  /*else*/ {

    /* "View.MemoryView":482
 *             info.suboffsets = self.view.suboffsets
 *         else:
 *             info.suboffsets = NULL             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_FORMAT:
 */
    __pyx_v_info->suboffsets = NULL;
  }
  __pyx_L5:;

  /* "View.MemoryView":484
 *             info.suboffsets = NULL
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             info.format = self.view.format
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_FORMAT) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":485
 * 
 *         if flags & PyBUF_FORMAT:
 *             info.format = self.view.format             # <<<<<<<<<<<<<<
 *         else:
 *             info.format = NULL
 */
    __pyx_t_3 = __pyx_v_self->view.format;
    __pyx_v_info->format = __pyx_t_3;
    goto __pyx_L6;
  }
  /*else*/ {

    /* "View.MemoryView":487
 *             info.format = self.view.format
 *         else:
 *             info.format = NULL             # <<<<<<<<<<<<<<
 * 
 *         info.buf = self.view.buf
 */
    __pyx_v_info->format = NULL;
  }
  __pyx_L6:;

  /* "View.MemoryView":489
 *             info.format = NULL
 * 
 *         info.buf = self.view.buf             # <<<<<<<<<<<<<<
 *         info.ndim = self.view.ndim
 *         info.itemsize = self.view.itemsize
 */
  __pyx_t_4 = __pyx_v_self->view.buf;
  __pyx_v_info->buf = __pyx_t_4;

  /* "View.MemoryView":490
 * 
 *         info.buf = self.view.buf
 *         info.ndim = self.view.ndim             # <<<<<<<<<<<<<<
 *         info.itemsize = self.view.itemsize
 *         info.len = self.view.len
 */
  __pyx_t_5 = __pyx_v_self->view.ndim;
  __pyx_v_info->ndim = __pyx_t_5;

  /* "View.MemoryView":491
 *         info.buf = self.view.buf
 *         info.ndim = self.view.ndim
 *         info.itemsize = self.view.itemsize             # <<<<<<<<<<<<<<
 *         info.len = self.view.len
 *         info.readonly = 0
 */
  __pyx_t_6 = __pyx_v_self->view.itemsize;
  __pyx_v_info->itemsize = __pyx_t_6;

  /* "View.MemoryView":492
 *         info.ndim = self.view.ndim
 *         info.itemsize = self.view.itemsize
 *         info.len = self.view.len             # <<<<<<<<<<<<<<
 *         info.readonly = 0
 *         info.obj = self
 */
  __pyx_t_6 = __pyx_v_self->view.len;
  __pyx_v_info->len = __pyx_t_6;

  /* "View.MemoryView":493
 *         info.itemsize = self.view.itemsize
 *         info.len = self.view.len
 *         info.readonly = 0             # <<<<<<<<<<<<<<
 *         info.obj = self
 * 
 */
  __pyx_v_info->readonly = 0;

  /* "View.MemoryView":494
 *         info.len = self.view.len
 *         info.readonly = 0
 *         info.obj = self             # <<<<<<<<<<<<<<
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_memoryview_getbuffer, "getbuffer(obj, view, flags)")
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  __Pyx_GOTREF(__pyx_v_info->obj);
  __Pyx_DECREF(__pyx_v_info->obj);
  __pyx_v_info->obj = ((PyObject *)__pyx_v_self);

  /* "View.MemoryView":468
 * 
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):             # <<<<<<<<<<<<<<
 *         if flags & PyBUF_STRIDES:
 *             info.shape = self.view.shape
 */

  /* function exit code */
  __pyx_r = 0;
  if (__pyx_v_info != NULL && __pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(Py_None);
    __Pyx_DECREF(Py_None); __pyx_v_info->obj = NULL;
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":501
 *     property T:
 *         @cname('__pyx_memoryview_transpose')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             cdef _memoryviewslice result = memoryview_copy(self)
 *             transpose_memslice(&result.from_slice)
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_transpose(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_transpose(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview_transpose_MemoryView_10memoryview_1T___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview_transpose_MemoryView_10memoryview_1T___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  struct __pyx_memoryviewslice_obj *__pyx_v_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":502
 *         @cname('__pyx_memoryview_transpose')
 *         def __get__(self):
 *             cdef _memoryviewslice result = memoryview_copy(self)             # <<<<<<<<<<<<<<
 *             transpose_memslice(&result.from_slice)
 *             return result
 */
  __pyx_t_1 = __pyx_memoryview_copy_object(__pyx_v_self); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 502; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_memoryviewslice_type))))) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 502; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_result = ((struct __pyx_memoryviewslice_obj *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "View.MemoryView":503
 *         def __get__(self):
 *             cdef _memoryviewslice result = memoryview_copy(self)
 *             transpose_memslice(&result.from_slice)             # <<<<<<<<<<<<<<
 *             return result
 * 
 */
  __pyx_t_2 = __pyx_memslice_transpose((&__pyx_v_result->from_slice)); if (unlikely(__pyx_t_2 == 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 503; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "View.MemoryView":504
 *             cdef _memoryviewslice result = memoryview_copy(self)
 *             transpose_memslice(&result.from_slice)
 *             return result             # <<<<<<<<<<<<<<
 * 
 *     property base:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_result));
  __pyx_r = ((PyObject *)__pyx_v_result);
  goto __pyx_L0;

  /* "View.MemoryView":501
 *     property T:
 *         @cname('__pyx_memoryview_transpose')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             cdef _memoryviewslice result = memoryview_copy(self)
 *             transpose_memslice(&result.from_slice)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.T.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":508
 *     property base:
 *         @cname('__pyx_memoryview__get__base')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.obj
 * 
 */

/* Python wrapper */
static PyObject *__pyx_memoryview__get__base(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview__get__base(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview__get__base_MemoryView_10memoryview_4base___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview__get__base_MemoryView_10memoryview_4base___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":509
 *         @cname('__pyx_memoryview__get__base')
 *         def __get__(self):
 *             return self.obj             # <<<<<<<<<<<<<<
 * 
 *     property shape:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->obj);
  __pyx_r = __pyx_v_self->obj;
  goto __pyx_L0;

  /* "View.MemoryView":508
 *     property base:
 *         @cname('__pyx_memoryview__get__base')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.obj
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":513
 *     property shape:
 *         @cname('__pyx_memoryview_get_shape')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return tuple([self.view.shape[i] for i in xrange(self.view.ndim)])
 * 
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_get_shape(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_shape(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview_get_shape_MemoryView_10memoryview_5shape___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview_get_shape_MemoryView_10memoryview_5shape___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":514
 *         @cname('__pyx_memoryview_get_shape')
 *         def __get__(self):
 *             return tuple([self.view.shape[i] for i in xrange(self.view.ndim)])             # <<<<<<<<<<<<<<
 * 
 *     property strides:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 514; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_v_self->view.ndim;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;
    __pyx_t_4 = PyInt_FromSsize_t((__pyx_v_self->view.shape[__pyx_v_i])); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 514; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_4))) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 514; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __pyx_t_4 = PyList_AsTuple(((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 514; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":513
 *     property shape:
 *         @cname('__pyx_memoryview_get_shape')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return tuple([self.view.shape[i] for i in xrange(self.view.ndim)])
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("View.MemoryView.memoryview.shape.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":518
 *     property strides:
 *         @cname('__pyx_memoryview_get_strides')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.view.strides == NULL:
 * 
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_get_strides(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_strides(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview_get_strides_MemoryView_10memoryview_7strides___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview_get_strides_MemoryView_10memoryview_7strides___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":519
 *         @cname('__pyx_memoryview_get_strides')
 *         def __get__(self):
 *             if self.view.strides == NULL:             # <<<<<<<<<<<<<<
 * 
 *                 raise ValueError("Buffer view does not expose strides")
 */
  __pyx_t_1 = ((__pyx_v_self->view.strides == NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":521
 *             if self.view.strides == NULL:
 * 
 *                 raise ValueError("Buffer view does not expose strides")             # <<<<<<<<<<<<<<
 * 
 *             return tuple([self.view.strides[i] for i in xrange(self.view.ndim)])
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__45, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 521; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    {__pyx_filename = __pyx_f[3]; __pyx_lineno = 521; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "View.MemoryView":523
 *                 raise ValueError("Buffer view does not expose strides")
 * 
 *             return tuple([self.view.strides[i] for i in xrange(self.view.ndim)])             # <<<<<<<<<<<<<<
 * 
 *     property suboffsets:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 523; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_v_self->view.ndim;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;
    __pyx_t_5 = PyInt_FromSsize_t((__pyx_v_self->view.strides[__pyx_v_i])); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 523; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_t_5))) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 523; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __pyx_t_5 = PyList_AsTuple(((PyObject*)__pyx_t_2)); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 523; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":518
 *     property strides:
 *         @cname('__pyx_memoryview_get_strides')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.view.strides == NULL:
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.memoryview.strides.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":527
 *     property suboffsets:
 *         @cname('__pyx_memoryview_get_suboffsets')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.view.suboffsets == NULL:
 *                 return [-1] * self.view.ndim
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_get_suboffsets(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_suboffsets(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview_get_suboffsets_MemoryView_10memoryview_10suboffsets___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview_get_suboffsets_MemoryView_10memoryview_10suboffsets___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":528
 *         @cname('__pyx_memoryview_get_suboffsets')
 *         def __get__(self):
 *             if self.view.suboffsets == NULL:             # <<<<<<<<<<<<<<
 *                 return [-1] * self.view.ndim
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->view.suboffsets == NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":529
 *         def __get__(self):
 *             if self.view.suboffsets == NULL:
 *                 return [-1] * self.view.ndim             # <<<<<<<<<<<<<<
 * 
 *             return tuple([self.view.suboffsets[i] for i in xrange(self.view.ndim)])
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = PyList_New(1 * ((__pyx_v_self->view.ndim<0) ? 0:__pyx_v_self->view.ndim)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    { Py_ssize_t __pyx_temp;
      for (__pyx_temp=0; __pyx_temp < __pyx_v_self->view.ndim; __pyx_temp++) {
        __Pyx_INCREF(__pyx_int_neg_1);
        PyList_SET_ITEM(__pyx_t_2, __pyx_temp, __pyx_int_neg_1);
        __Pyx_GIVEREF(__pyx_int_neg_1);
      }
    }
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "View.MemoryView":531
 *                 return [-1] * self.view.ndim
 * 
 *             return tuple([self.view.suboffsets[i] for i in xrange(self.view.ndim)])             # <<<<<<<<<<<<<<
 * 
 *     property ndim:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_v_self->view.ndim;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;
    __pyx_t_5 = PyInt_FromSsize_t((__pyx_v_self->view.suboffsets[__pyx_v_i])); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_t_5))) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __pyx_t_5 = PyList_AsTuple(((PyObject*)__pyx_t_2)); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":527
 *     property suboffsets:
 *         @cname('__pyx_memoryview_get_suboffsets')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.view.suboffsets == NULL:
 *                 return [-1] * self.view.ndim
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.memoryview.suboffsets.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":535
 *     property ndim:
 *         @cname('__pyx_memoryview_get_ndim')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.view.ndim
 * 
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_get_ndim(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_ndim(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview_get_ndim_MemoryView_10memoryview_4ndim___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview_get_ndim_MemoryView_10memoryview_4ndim___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":536
 *         @cname('__pyx_memoryview_get_ndim')
 *         def __get__(self):
 *             return self.view.ndim             # <<<<<<<<<<<<<<
 * 
 *     property itemsize:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->view.ndim); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 536; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":535
 *     property ndim:
 *         @cname('__pyx_memoryview_get_ndim')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.view.ndim
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.ndim.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":540
 *     property itemsize:
 *         @cname('__pyx_memoryview_get_itemsize')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.view.itemsize
 * 
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_get_itemsize(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_itemsize(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview_get_itemsize_MemoryView_10memoryview_8itemsize___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview_get_itemsize_MemoryView_10memoryview_8itemsize___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":541
 *         @cname('__pyx_memoryview_get_itemsize')
 *         def __get__(self):
 *             return self.view.itemsize             # <<<<<<<<<<<<<<
 * 
 *     property nbytes:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_self->view.itemsize); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 541; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":540
 *     property itemsize:
 *         @cname('__pyx_memoryview_get_itemsize')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.view.itemsize
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.itemsize.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":545
 *     property nbytes:
 *         @cname('__pyx_memoryview_get_nbytes')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.size * self.view.itemsize
 * 
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_get_nbytes(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_nbytes(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview_get_nbytes_MemoryView_10memoryview_6nbytes___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview_get_nbytes_MemoryView_10memoryview_6nbytes___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":546
 *         @cname('__pyx_memoryview_get_nbytes')
 *         def __get__(self):
 *             return self.size * self.view.itemsize             # <<<<<<<<<<<<<<
 * 
 *     property size:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_size); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 546; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_self->view.itemsize); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 546; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Multiply(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 546; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":545
 *     property nbytes:
 *         @cname('__pyx_memoryview_get_nbytes')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.size * self.view.itemsize
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview.nbytes.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":550
 *     property size:
 *         @cname('__pyx_memoryview_get_size')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self._size is None:
 *                 result = 1
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_get_size(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_size(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview_get_size_MemoryView_10memoryview_4size___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview_get_size_MemoryView_10memoryview_4size___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_v_result = NULL;
  PyObject *__pyx_v_length = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":551
 *         @cname('__pyx_memoryview_get_size')
 *         def __get__(self):
 *             if self._size is None:             # <<<<<<<<<<<<<<
 *                 result = 1
 * 
 */
  __pyx_t_1 = (__pyx_v_self->_size == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":552
 *         def __get__(self):
 *             if self._size is None:
 *                 result = 1             # <<<<<<<<<<<<<<
 * 
 *                 for length in self.shape:
 */
    __Pyx_INCREF(__pyx_int_1);
    __pyx_v_result = __pyx_int_1;

    /* "View.MemoryView":554
 *                 result = 1
 * 
 *                 for length in self.shape:             # <<<<<<<<<<<<<<
 *                     result *= length
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 554; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
      __pyx_t_4 = __pyx_t_3; __Pyx_INCREF(__pyx_t_4); __pyx_t_5 = 0;
      __pyx_t_6 = NULL;
    } else {
      __pyx_t_5 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 554; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 554; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    for (;;) {
      if (likely(!__pyx_t_6)) {
        if (likely(PyList_CheckExact(__pyx_t_4))) {
          if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_3 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_3); __pyx_t_5++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 554; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_3 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 554; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #endif
        } else {
          if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_3); __pyx_t_5++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 554; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_3 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 554; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #endif
        }
      } else {
        __pyx_t_3 = __pyx_t_6(__pyx_t_4);
        if (unlikely(!__pyx_t_3)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[3]; __pyx_lineno = 554; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_XDECREF_SET(__pyx_v_length, __pyx_t_3);
      __pyx_t_3 = 0;

      /* "View.MemoryView":555
 * 
 *                 for length in self.shape:
 *                     result *= length             # <<<<<<<<<<<<<<
 * 
 *                 self._size = result
 */
      __pyx_t_3 = PyNumber_InPlaceMultiply(__pyx_v_result, __pyx_v_length); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 555; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF_SET(__pyx_v_result, __pyx_t_3);
      __pyx_t_3 = 0;

      /* "View.MemoryView":554
 *                 result = 1
 * 
 *                 for length in self.shape:             # <<<<<<<<<<<<<<
 *                     result *= length
 * 
 */
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "View.MemoryView":557
 *                     result *= length
 * 
 *                 self._size = result             # <<<<<<<<<<<<<<
 * 
 *             return self._size
 */
    __Pyx_INCREF(__pyx_v_result);
    __Pyx_GIVEREF(__pyx_v_result);
    __Pyx_GOTREF(__pyx_v_self->_size);
    __Pyx_DECREF(__pyx_v_self->_size);
    __pyx_v_self->_size = __pyx_v_result;
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "View.MemoryView":559
 *                 self._size = result
 * 
 *             return self._size             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_size);
  __pyx_r = __pyx_v_self->_size;
  goto __pyx_L0;

  /* "View.MemoryView":550
 *     property size:
 *         @cname('__pyx_memoryview_get_size')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self._size is None:
 *                 result = 1
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("View.MemoryView.memoryview.size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_length);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":561
 *             return self._size
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         if self.view.ndim >= 1:
 *             return self.view.shape[0]
 */

/* Python wrapper */
static Py_ssize_t __pyx_memoryview___len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_memoryview___len__(PyObject *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview_MemoryView_10memoryview_10__len__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_memoryview_MemoryView_10memoryview_10__len__(struct __pyx_memoryview_obj *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "View.MemoryView":562
 * 
 *     def __len__(self):
 *         if self.view.ndim >= 1:             # <<<<<<<<<<<<<<
 *             return self.view.shape[0]
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->view.ndim >= 1) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":563
 *     def __len__(self):
 *         if self.view.ndim >= 1:
 *             return self.view.shape[0]             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
    __pyx_r = (__pyx_v_self->view.shape[0]);
    goto __pyx_L0;
  }

  /* "View.MemoryView":565
 *             return self.view.shape[0]
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "View.MemoryView":561
 *             return self._size
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         if self.view.ndim >= 1:
 *             return self.view.shape[0]
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":567
 *         return 0
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,
 *                                                id(self))
 */

/* Python wrapper */
static PyObject *__pyx_memoryview___repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview___repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview_MemoryView_10memoryview_12__repr__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview_MemoryView_10memoryview_12__repr__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "View.MemoryView":568
 * 
 *     def __repr__(self):
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,             # <<<<<<<<<<<<<<
 *                                                id(self))
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_base); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 568; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_class); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 568; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_name); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 568; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":569
 *     def __repr__(self):
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,
 *                                                id(self))             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 569; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_id, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 569; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":568
 * 
 *     def __repr__(self):
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,             # <<<<<<<<<<<<<<
 *                                                id(self))
 * 
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 568; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_MemoryView_of_r_at_0x_x, __pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 568; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":567
 *         return 0
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,
 *                                                id(self))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":571
 *                                                id(self))
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r object>" % (self.base.__class__.__name__,)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_memoryview___str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview___str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview_MemoryView_10memoryview_14__str__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview_MemoryView_10memoryview_14__str__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "View.MemoryView":572
 * 
 *     def __str__(self):
 *         return "<MemoryView of %r object>" % (self.base.__class__.__name__,)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_base); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 572; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_class); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 572; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_name); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 572; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 572; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_MemoryView_of_r_object, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 572; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":571
 *                                                id(self))
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r object>" % (self.base.__class__.__name__,)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":575
 * 
 * 
 *     def is_c_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_is_c_contig(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_memoryview_is_c_contig(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_c_contig (wrapper)", 0);
  __pyx_r = __pyx_memoryview_MemoryView_10memoryview_16is_c_contig(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview_MemoryView_10memoryview_16is_c_contig(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice *__pyx_v_mslice;
  __Pyx_memviewslice __pyx_v_tmp;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_c_contig", 0);

  /* "View.MemoryView":578
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)             # <<<<<<<<<<<<<<
 *         return slice_is_contig(mslice, 'C', self.view.ndim)
 * 
 */
  __pyx_v_mslice = __pyx_memoryview_get_slice_from_memoryview(__pyx_v_self, (&__pyx_v_tmp));

  /* "View.MemoryView":579
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)
 *         return slice_is_contig(mslice, 'C', self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *     def is_f_contig(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_memviewslice_is_contig(__pyx_v_mslice, 'C', __pyx_v_self->view.ndim)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 579; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":575
 * 
 * 
 *     def is_c_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.is_c_contig", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":581
 *         return slice_is_contig(mslice, 'C', self.view.ndim)
 * 
 *     def is_f_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_is_f_contig(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_memoryview_is_f_contig(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_f_contig (wrapper)", 0);
  __pyx_r = __pyx_memoryview_MemoryView_10memoryview_18is_f_contig(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview_MemoryView_10memoryview_18is_f_contig(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice *__pyx_v_mslice;
  __Pyx_memviewslice __pyx_v_tmp;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_f_contig", 0);

  /* "View.MemoryView":584
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)             # <<<<<<<<<<<<<<
 *         return slice_is_contig(mslice, 'F', self.view.ndim)
 * 
 */
  __pyx_v_mslice = __pyx_memoryview_get_slice_from_memoryview(__pyx_v_self, (&__pyx_v_tmp));

  /* "View.MemoryView":585
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)
 *         return slice_is_contig(mslice, 'F', self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *     def copy(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_memviewslice_is_contig(__pyx_v_mslice, 'F', __pyx_v_self->view.ndim)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 585; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":581
 *         return slice_is_contig(mslice, 'C', self.view.ndim)
 * 
 *     def is_f_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.is_f_contig", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":587
 *         return slice_is_contig(mslice, 'F', self.view.ndim)
 * 
 *     def copy(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice mslice
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_memoryview_copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy (wrapper)", 0);
  __pyx_r = __pyx_memoryview_MemoryView_10memoryview_20copy(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview_MemoryView_10memoryview_20copy(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice __pyx_v_mslice;
  int __pyx_v_flags;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_memviewslice __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy", 0);

  /* "View.MemoryView":589
 *     def copy(self):
 *         cdef __Pyx_memviewslice mslice
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS             # <<<<<<<<<<<<<<
 * 
 *         slice_copy(self, &mslice)
 */
  __pyx_v_flags = (__pyx_v_self->flags & (~PyBUF_F_CONTIGUOUS));

  /* "View.MemoryView":591
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS
 * 
 *         slice_copy(self, &mslice)             # <<<<<<<<<<<<<<
 *         mslice = slice_copy_contig(&mslice, "c", self.view.ndim,
 *                                    self.view.itemsize,
 */
  __pyx_memoryview_slice_copy(__pyx_v_self, (&__pyx_v_mslice));

  /* "View.MemoryView":592
 * 
 *         slice_copy(self, &mslice)
 *         mslice = slice_copy_contig(&mslice, "c", self.view.ndim,             # <<<<<<<<<<<<<<
 *                                    self.view.itemsize,
 *                                    flags|PyBUF_C_CONTIGUOUS,
 */
  __pyx_t_1 = __pyx_memoryview_copy_new_contig((&__pyx_v_mslice), __pyx_k_c, __pyx_v_self->view.ndim, __pyx_v_self->view.itemsize, (__pyx_v_flags | PyBUF_C_CONTIGUOUS), __pyx_v_self->dtype_is_object); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_mslice = __pyx_t_1;

  /* "View.MemoryView":597
 *                                    self.dtype_is_object)
 * 
 *         return memoryview_copy_from_slice(self, &mslice)             # <<<<<<<<<<<<<<
 * 
 *     def copy_fortran(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_memoryview_copy_object_from_slice(__pyx_v_self, (&__pyx_v_mslice)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 597; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":587
 *         return slice_is_contig(mslice, 'F', self.view.ndim)
 * 
 *     def copy(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice mslice
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":599
 *         return memoryview_copy_from_slice(self, &mslice)
 * 
 *     def copy_fortran(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice src, dst
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_copy_fortran(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_memoryview_copy_fortran(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy_fortran (wrapper)", 0);
  __pyx_r = __pyx_memoryview_MemoryView_10memoryview_22copy_fortran(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview_MemoryView_10memoryview_22copy_fortran(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice __pyx_v_src;
  __Pyx_memviewslice __pyx_v_dst;
  int __pyx_v_flags;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_memviewslice __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_fortran", 0);

  /* "View.MemoryView":601
 *     def copy_fortran(self):
 *         cdef __Pyx_memviewslice src, dst
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS             # <<<<<<<<<<<<<<
 * 
 *         slice_copy(self, &src)
 */
  __pyx_v_flags = (__pyx_v_self->flags & (~PyBUF_C_CONTIGUOUS));

  /* "View.MemoryView":603
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS
 * 
 *         slice_copy(self, &src)             # <<<<<<<<<<<<<<
 *         dst = slice_copy_contig(&src, "fortran", self.view.ndim,
 *                                 self.view.itemsize,
 */
  __pyx_memoryview_slice_copy(__pyx_v_self, (&__pyx_v_src));

  /* "View.MemoryView":604
 * 
 *         slice_copy(self, &src)
 *         dst = slice_copy_contig(&src, "fortran", self.view.ndim,             # <<<<<<<<<<<<<<
 *                                 self.view.itemsize,
 *                                 flags|PyBUF_F_CONTIGUOUS,
 */
  __pyx_t_1 = __pyx_memoryview_copy_new_contig((&__pyx_v_src), __pyx_k_fortran, __pyx_v_self->view.ndim, __pyx_v_self->view.itemsize, (__pyx_v_flags | PyBUF_F_CONTIGUOUS), __pyx_v_self->dtype_is_object); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 604; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_dst = __pyx_t_1;

  /* "View.MemoryView":609
 *                                 self.dtype_is_object)
 * 
 *         return memoryview_copy_from_slice(self, &dst)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_memoryview_copy_object_from_slice(__pyx_v_self, (&__pyx_v_dst)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 609; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":599
 *         return memoryview_copy_from_slice(self, &mslice)
 * 
 *     def copy_fortran(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice src, dst
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.copy_fortran", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":613
 * 
 * @cname('__pyx_memoryview_new')
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, __Pyx_TypeInfo *typeinfo):             # <<<<<<<<<<<<<<
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo
 */

static PyObject *__pyx_memoryview_new(PyObject *__pyx_v_o, int __pyx_v_flags, int __pyx_v_dtype_is_object, __Pyx_TypeInfo *__pyx_v_typeinfo) {
  struct __pyx_memoryview_obj *__pyx_v_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memoryview_cwrapper", 0);

  /* "View.MemoryView":614
 * @cname('__pyx_memoryview_new')
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, __Pyx_TypeInfo *typeinfo):
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)             # <<<<<<<<<<<<<<
 *     result.typeinfo = typeinfo
 *     return result
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_flags); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_dtype_is_object); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_o);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_o);
  __Pyx_GIVEREF(__pyx_v_o);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)((PyObject *)__pyx_memoryview_type)), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_result = ((struct __pyx_memoryview_obj *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "View.MemoryView":615
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, __Pyx_TypeInfo *typeinfo):
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo             # <<<<<<<<<<<<<<
 *     return result
 * 
 */
  __pyx_v_result->typeinfo = __pyx_v_typeinfo;

  /* "View.MemoryView":616
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo
 *     return result             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_check')
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_result));
  __pyx_r = ((PyObject *)__pyx_v_result);
  goto __pyx_L0;

  /* "View.MemoryView":613
 * 
 * @cname('__pyx_memoryview_new')
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, __Pyx_TypeInfo *typeinfo):             # <<<<<<<<<<<<<<
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview_cwrapper", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":619
 * 
 * @cname('__pyx_memoryview_check')
 * cdef inline bint memoryview_check(object o):             # <<<<<<<<<<<<<<
 *     return isinstance(o, memoryview)
 * 
 */

static CYTHON_INLINE int __pyx_memoryview_check(PyObject *__pyx_v_o) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("memoryview_check", 0);

  /* "View.MemoryView":620
 * @cname('__pyx_memoryview_check')
 * cdef inline bint memoryview_check(object o):
 *     return isinstance(o, memoryview)             # <<<<<<<<<<<<<<
 * 
 * cdef tuple _unellipsify(object index, int ndim):
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_o, ((PyObject *)__pyx_memoryview_type)); 
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "View.MemoryView":619
 * 
 * @cname('__pyx_memoryview_check')
 * cdef inline bint memoryview_check(object o):             # <<<<<<<<<<<<<<
 *     return isinstance(o, memoryview)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":622
 *     return isinstance(o, memoryview)
 * 
 * cdef tuple _unellipsify(object index, int ndim):             # <<<<<<<<<<<<<<
 *     """
 *     Replace all ellipses with full slices and fill incomplete indices with
 */

static PyObject *_unellipsify(PyObject *__pyx_v_index, int __pyx_v_ndim) {
  PyObject *__pyx_v_tup = NULL;
  PyObject *__pyx_v_result = NULL;
  int __pyx_v_have_slices;
  int __pyx_v_seen_ellipsis;
  CYTHON_UNUSED PyObject *__pyx_v_idx = NULL;
  PyObject *__pyx_v_item = NULL;
  Py_ssize_t __pyx_v_nslices;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_unellipsify", 0);

  /* "View.MemoryView":627
 *     full slices.
 *     """
 *     if not isinstance(index, tuple):             # <<<<<<<<<<<<<<
 *         tup = (index,)
 *     else:
 */
  __pyx_t_1 = PyTuple_Check(__pyx_v_index); 
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":628
 *     """
 *     if not isinstance(index, tuple):
 *         tup = (index,)             # <<<<<<<<<<<<<<
 *     else:
 *         tup = index
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 628; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_index);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_index);
    __Pyx_GIVEREF(__pyx_v_index);
    __pyx_v_tup = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "View.MemoryView":630
 *         tup = (index,)
 *     else:
 *         tup = index             # <<<<<<<<<<<<<<
 * 
 *     result = []
 */
    __Pyx_INCREF(__pyx_v_index);
    __pyx_v_tup = __pyx_v_index;
  }
  __pyx_L3:;

  /* "View.MemoryView":632
 *         tup = index
 * 
 *     result = []             # <<<<<<<<<<<<<<
 *     have_slices = False
 *     seen_ellipsis = False
 */
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 632; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_result = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "View.MemoryView":633
 * 
 *     result = []
 *     have_slices = False             # <<<<<<<<<<<<<<
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):
 */
  __pyx_v_have_slices = 0;

  /* "View.MemoryView":634
 *     result = []
 *     have_slices = False
 *     seen_ellipsis = False             # <<<<<<<<<<<<<<
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:
 */
  __pyx_v_seen_ellipsis = 0;

  /* "View.MemoryView":635
 *     have_slices = False
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):             # <<<<<<<<<<<<<<
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_t_3 = __pyx_int_0;
  if (likely(PyList_CheckExact(__pyx_v_tup)) || PyTuple_CheckExact(__pyx_v_tup)) {
    __pyx_t_4 = __pyx_v_tup; __Pyx_INCREF(__pyx_t_4); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_v_tup); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 635; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 635; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_7 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 635; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 635; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #endif
      } else {
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 635; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 635; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #endif
      }
    } else {
      __pyx_t_7 = __pyx_t_6(__pyx_t_4);
      if (unlikely(!__pyx_t_7)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[3]; __pyx_lineno = 635; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_7);
    }
    __Pyx_XDECREF_SET(__pyx_v_item, __pyx_t_7);
    __pyx_t_7 = 0;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_idx, __pyx_t_3);
    __pyx_t_7 = PyNumber_Add(__pyx_t_3, __pyx_int_1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 635; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3);
    __pyx_t_3 = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "View.MemoryView":636
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:             # <<<<<<<<<<<<<<
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 */
    __pyx_t_2 = (__pyx_v_item == __pyx_builtin_Ellipsis);
    __pyx_t_1 = (__pyx_t_2 != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":637
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:
 *             if not seen_ellipsis:             # <<<<<<<<<<<<<<
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 *                 seen_ellipsis = True
 */
      __pyx_t_1 = ((!(__pyx_v_seen_ellipsis != 0)) != 0);
      if (__pyx_t_1) {

        /* "View.MemoryView":638
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))             # <<<<<<<<<<<<<<
 *                 seen_ellipsis = True
 *             else:
 */
        __pyx_t_8 = PyObject_Length(__pyx_v_tup); if (unlikely(__pyx_t_8 == -1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 638; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_7 = PyList_New(1 * ((((__pyx_v_ndim - __pyx_t_8) + 1)<0) ? 0:((__pyx_v_ndim - __pyx_t_8) + 1))); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 638; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_7);
        { Py_ssize_t __pyx_temp;
          for (__pyx_temp=0; __pyx_temp < ((__pyx_v_ndim - __pyx_t_8) + 1); __pyx_temp++) {
            __Pyx_INCREF(__pyx_slice__46);
            PyList_SET_ITEM(__pyx_t_7, __pyx_temp, __pyx_slice__46);
            __Pyx_GIVEREF(__pyx_slice__46);
          }
        }
        __pyx_t_9 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_t_7); if (unlikely(__pyx_t_9 == -1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 638; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

        /* "View.MemoryView":639
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 *                 seen_ellipsis = True             # <<<<<<<<<<<<<<
 *             else:
 *                 result.append(slice(None))
 */
        __pyx_v_seen_ellipsis = 1;
        goto __pyx_L7;
      }
      /*else*/ {

        /* "View.MemoryView":641
 *                 seen_ellipsis = True
 *             else:
 *                 result.append(slice(None))             # <<<<<<<<<<<<<<
 *             have_slices = True
 *         else:
 */
        __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_result, __pyx_slice__47); if (unlikely(__pyx_t_9 == -1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 641; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_L7:;

      /* "View.MemoryView":642
 *             else:
 *                 result.append(slice(None))
 *             have_slices = True             # <<<<<<<<<<<<<<
 *         else:
 *             if not isinstance(item, slice) and not PyIndex_Check(item):
 */
      __pyx_v_have_slices = 1;
      goto __pyx_L6;
    }
    /*else*/ {

      /* "View.MemoryView":644
 *             have_slices = True
 *         else:
 *             if not isinstance(item, slice) and not PyIndex_Check(item):             # <<<<<<<<<<<<<<
 *                 raise TypeError("Cannot index with type '%s'" % type(item))
 * 
 */
      __pyx_t_2 = PySlice_Check(__pyx_v_item); 
      __pyx_t_10 = ((!(__pyx_t_2 != 0)) != 0);
      if (__pyx_t_10) {
      } else {
        __pyx_t_1 = __pyx_t_10;
        goto __pyx_L9_bool_binop_done;
      }
      __pyx_t_10 = ((!(PyIndex_Check(__pyx_v_item) != 0)) != 0);
      __pyx_t_1 = __pyx_t_10;
      __pyx_L9_bool_binop_done:;
      if (__pyx_t_1) {

        /* "View.MemoryView":645
 *         else:
 *             if not isinstance(item, slice) and not PyIndex_Check(item):
 *                 raise TypeError("Cannot index with type '%s'" % type(item))             # <<<<<<<<<<<<<<
 * 
 *             have_slices = have_slices or isinstance(item, slice)
 */
        __pyx_t_7 = __Pyx_PyString_Format(__pyx_kp_s_Cannot_index_with_type_s, ((PyObject *)Py_TYPE(__pyx_v_item))); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 645; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_11 = PyTuple_New(1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 645; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_11);
        PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_7);
        __Pyx_GIVEREF(__pyx_t_7);
        __pyx_t_7 = 0;
        __pyx_t_7 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_11, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 645; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_Raise(__pyx_t_7, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        {__pyx_filename = __pyx_f[3]; __pyx_lineno = 645; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }

      /* "View.MemoryView":647
 *                 raise TypeError("Cannot index with type '%s'" % type(item))
 * 
 *             have_slices = have_slices or isinstance(item, slice)             # <<<<<<<<<<<<<<
 *             result.append(item)
 * 
 */
      __pyx_t_10 = (__pyx_v_have_slices != 0);
      if (!__pyx_t_10) {
      } else {
        __pyx_t_1 = __pyx_t_10;
        goto __pyx_L11_bool_binop_done;
      }
      __pyx_t_10 = PySlice_Check(__pyx_v_item); 
      __pyx_t_2 = (__pyx_t_10 != 0);
      __pyx_t_1 = __pyx_t_2;
      __pyx_L11_bool_binop_done:;
      __pyx_v_have_slices = __pyx_t_1;

      /* "View.MemoryView":648
 * 
 *             have_slices = have_slices or isinstance(item, slice)
 *             result.append(item)             # <<<<<<<<<<<<<<
 * 
 *     nslices = ndim - len(result)
 */
      __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_result, __pyx_v_item); if (unlikely(__pyx_t_9 == -1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 648; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_L6:;

    /* "View.MemoryView":635
 *     have_slices = False
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):             # <<<<<<<<<<<<<<
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 */
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "View.MemoryView":650
 *             result.append(item)
 * 
 *     nslices = ndim - len(result)             # <<<<<<<<<<<<<<
 *     if nslices:
 *         result.extend([slice(None)] * nslices)
 */
  __pyx_t_5 = PyList_GET_SIZE(__pyx_v_result); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 650; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_nslices = (__pyx_v_ndim - __pyx_t_5);

  /* "View.MemoryView":651
 * 
 *     nslices = ndim - len(result)
 *     if nslices:             # <<<<<<<<<<<<<<
 *         result.extend([slice(None)] * nslices)
 * 
 */
  __pyx_t_1 = (__pyx_v_nslices != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":652
 *     nslices = ndim - len(result)
 *     if nslices:
 *         result.extend([slice(None)] * nslices)             # <<<<<<<<<<<<<<
 * 
 *     return have_slices or nslices, tuple(result)
 */
    __pyx_t_3 = PyList_New(1 * ((__pyx_v_nslices<0) ? 0:__pyx_v_nslices)); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 652; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    { Py_ssize_t __pyx_temp;
      for (__pyx_temp=0; __pyx_temp < __pyx_v_nslices; __pyx_temp++) {
        __Pyx_INCREF(__pyx_slice__48);
        PyList_SET_ITEM(__pyx_t_3, __pyx_temp, __pyx_slice__48);
        __Pyx_GIVEREF(__pyx_slice__48);
      }
    }
    __pyx_t_9 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_t_3); if (unlikely(__pyx_t_9 == -1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 652; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L13;
  }
  __pyx_L13:;

  /* "View.MemoryView":654
 *         result.extend([slice(None)] * nslices)
 * 
 *     return have_slices or nslices, tuple(result)             # <<<<<<<<<<<<<<
 * 
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):
 */
  __Pyx_XDECREF(__pyx_r);
  if (!__pyx_v_have_slices) {
  } else {
    __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_have_slices); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 654; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L14_bool_binop_done;
  }
  __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_nslices); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 654; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __pyx_t_4;
  __pyx_t_4 = 0;
  __pyx_L14_bool_binop_done:;
  __pyx_t_4 = PyList_AsTuple(__pyx_v_result); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 654; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 654; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_r = ((PyObject*)__pyx_t_7);
  __pyx_t_7 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":622
 *     return isinstance(o, memoryview)
 * 
 * cdef tuple _unellipsify(object index, int ndim):             # <<<<<<<<<<<<<<
 *     """
 *     Replace all ellipses with full slices and fill incomplete indices with
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("View.MemoryView._unellipsify", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tup);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_idx);
  __Pyx_XDECREF(__pyx_v_item);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":656
 *     return have_slices or nslices, tuple(result)
 * 
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):             # <<<<<<<<<<<<<<
 *     cdef int i
 *     for i in range(ndim):
 */

static PyObject *assert_direct_dimensions(Py_ssize_t *__pyx_v_suboffsets, int __pyx_v_ndim) {
  int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("assert_direct_dimensions", 0);

  /* "View.MemoryView":658
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):
 *     cdef int i
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         if suboffsets[i] >= 0:
 *             raise ValueError("Indirect dimensions not supported")
 */
  __pyx_t_1 = __pyx_v_ndim;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "View.MemoryView":659
 *     cdef int i
 *     for i in range(ndim):
 *         if suboffsets[i] >= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("Indirect dimensions not supported")
 * 
 */
    __pyx_t_3 = (((__pyx_v_suboffsets[__pyx_v_i]) >= 0) != 0);
    if (__pyx_t_3) {

      /* "View.MemoryView":660
 *     for i in range(ndim):
 *         if suboffsets[i] >= 0:
 *             raise ValueError("Indirect dimensions not supported")             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__49, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 660; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_Raise(__pyx_t_4, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      {__pyx_filename = __pyx_f[3]; __pyx_lineno = 660; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }

  /* "View.MemoryView":656
 *     return have_slices or nslices, tuple(result)
 * 
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):             # <<<<<<<<<<<<<<
 *     cdef int i
 *     for i in range(ndim):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("View.MemoryView.assert_direct_dimensions", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":667
 * 
 * @cname('__pyx_memview_slice')
 * cdef memoryview memview_slice(memoryview memview, object indices):             # <<<<<<<<<<<<<<
 *     cdef int new_ndim = 0, suboffset_dim = -1, dim
 *     cdef bint negative_step
 */

static struct __pyx_memoryview_obj *__pyx_memview_slice(struct __pyx_memoryview_obj *__pyx_v_memview, PyObject *__pyx_v_indices) {
  int __pyx_v_new_ndim;
  int __pyx_v_suboffset_dim;
  int __pyx_v_dim;
  __Pyx_memviewslice __pyx_v_src;
  __Pyx_memviewslice __pyx_v_dst;
  __Pyx_memviewslice *__pyx_v_p_src;
  struct __pyx_memoryviewslice_obj *__pyx_v_memviewsliceobj = 0;
  __Pyx_memviewslice *__pyx_v_p_dst;
  int *__pyx_v_p_suboffset_dim;
  Py_ssize_t __pyx_v_start;
  Py_ssize_t __pyx_v_stop;
  Py_ssize_t __pyx_v_step;
  int __pyx_v_have_start;
  int __pyx_v_have_stop;
  int __pyx_v_have_step;
  PyObject *__pyx_v_index = NULL;
  struct __pyx_memoryview_obj *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  struct __pyx_memoryview_obj *__pyx_t_4;
  char *__pyx_t_5;
  int __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  int __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memview_slice", 0);

  /* "View.MemoryView":668
 * @cname('__pyx_memview_slice')
 * cdef memoryview memview_slice(memoryview memview, object indices):
 *     cdef int new_ndim = 0, suboffset_dim = -1, dim             # <<<<<<<<<<<<<<
 *     cdef bint negative_step
 *     cdef __Pyx_memviewslice src, dst
 */
  __pyx_v_new_ndim = 0;
  __pyx_v_suboffset_dim = -1;

  /* "View.MemoryView":675
 * 
 * 
 *     memset(&dst, 0, sizeof(dst))             # <<<<<<<<<<<<<<
 * 
 *     cdef _memoryviewslice memviewsliceobj
 */
  memset((&__pyx_v_dst), 0, (sizeof(__pyx_v_dst)));

  /* "View.MemoryView":679
 *     cdef _memoryviewslice memviewsliceobj
 * 
 *     assert memview.view.ndim > 0             # <<<<<<<<<<<<<<
 * 
 *     if isinstance(memview, _memoryviewslice):
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_memview->view.ndim > 0) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      {__pyx_filename = __pyx_f[3]; __pyx_lineno = 679; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif

  /* "View.MemoryView":681
 *     assert memview.view.ndim > 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         memviewsliceobj = memview
 *         p_src = &memviewsliceobj.from_slice
 */
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), ((PyObject *)__pyx_memoryviewslice_type)); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":682
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         memviewsliceobj = memview             # <<<<<<<<<<<<<<
 *         p_src = &memviewsliceobj.from_slice
 *     else:
 */
    if (!(likely(((((PyObject *)__pyx_v_memview)) == Py_None) || likely(__Pyx_TypeTest(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type))))) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 682; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = ((PyObject *)__pyx_v_memview);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_memviewsliceobj = ((struct __pyx_memoryviewslice_obj *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "View.MemoryView":683
 *     if isinstance(memview, _memoryviewslice):
 *         memviewsliceobj = memview
 *         p_src = &memviewsliceobj.from_slice             # <<<<<<<<<<<<<<
 *     else:
 *         slice_copy(memview, &src)
 */
    __pyx_v_p_src = (&__pyx_v_memviewsliceobj->from_slice);
    goto __pyx_L3;
  }
  /*else*/ {

    /* "View.MemoryView":685
 *         p_src = &memviewsliceobj.from_slice
 *     else:
 *         slice_copy(memview, &src)             # <<<<<<<<<<<<<<
 *         p_src = &src
 * 
 */
    __pyx_memoryview_slice_copy(__pyx_v_memview, (&__pyx_v_src));

    /* "View.MemoryView":686
 *     else:
 *         slice_copy(memview, &src)
 *         p_src = &src             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_v_p_src = (&__pyx_v_src);
  }
  __pyx_L3:;

  /* "View.MemoryView":692
 * 
 * 
 *     dst.memview = p_src.memview             # <<<<<<<<<<<<<<
 *     dst.data = p_src.data
 * 
 */
  __pyx_t_4 = __pyx_v_p_src->memview;
  __pyx_v_dst.memview = __pyx_t_4;

  /* "View.MemoryView":693
 * 
 *     dst.memview = p_src.memview
 *     dst.data = p_src.data             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_5 = __pyx_v_p_src->data;
  __pyx_v_dst.data = __pyx_t_5;

  /* "View.MemoryView":698
 * 
 * 
 *     cdef __Pyx_memviewslice *p_dst = &dst             # <<<<<<<<<<<<<<
 *     cdef int *p_suboffset_dim = &suboffset_dim
 *     cdef Py_ssize_t start, stop, step
 */
  __pyx_v_p_dst = (&__pyx_v_dst);

  /* "View.MemoryView":699
 * 
 *     cdef __Pyx_memviewslice *p_dst = &dst
 *     cdef int *p_suboffset_dim = &suboffset_dim             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t start, stop, step
 *     cdef bint have_start, have_stop, have_step
 */
  __pyx_v_p_suboffset_dim = (&__pyx_v_suboffset_dim);

  /* "View.MemoryView":703
 *     cdef bint have_start, have_stop, have_step
 * 
 *     for dim, index in enumerate(indices):             # <<<<<<<<<<<<<<
 *         if PyIndex_Check(index):
 *             slice_memviewslice(
 */
  __pyx_t_6 = 0;
  if (likely(PyList_CheckExact(__pyx_v_indices)) || PyTuple_CheckExact(__pyx_v_indices)) {
    __pyx_t_3 = __pyx_v_indices; __Pyx_INCREF(__pyx_t_3); __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_7 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_v_indices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 703; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 703; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_9 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_7); __Pyx_INCREF(__pyx_t_9); __pyx_t_7++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 703; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_9 = PySequence_ITEM(__pyx_t_3, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 703; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #endif
      } else {
        if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_9 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_7); __Pyx_INCREF(__pyx_t_9); __pyx_t_7++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 703; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_9 = PySequence_ITEM(__pyx_t_3, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 703; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #endif
      }
    } else {
      __pyx_t_9 = __pyx_t_8(__pyx_t_3);
      if (unlikely(!__pyx_t_9)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[3]; __pyx_lineno = 703; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_9);
    }
    __Pyx_XDECREF_SET(__pyx_v_index, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_dim = __pyx_t_6;
    __pyx_t_6 = (__pyx_t_6 + 1);

    /* "View.MemoryView":704
 * 
 *     for dim, index in enumerate(indices):
 *         if PyIndex_Check(index):             # <<<<<<<<<<<<<<
 *             slice_memviewslice(
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 */
    __pyx_t_2 = (PyIndex_Check(__pyx_v_index) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":708
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 *                 dim, new_ndim, p_suboffset_dim,
 *                 index, 0, 0, # start, stop, step             # <<<<<<<<<<<<<<
 *                 0, 0, 0, # have_{start,stop,step}
 *                 False)
 */
      __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_v_index); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 708; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "View.MemoryView":705
 *     for dim, index in enumerate(indices):
 *         if PyIndex_Check(index):
 *             slice_memviewslice(             # <<<<<<<<<<<<<<
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 *                 dim, new_ndim, p_suboffset_dim,
 */
      __pyx_t_11 = __pyx_memoryview_slice_memviewslice(__pyx_v_p_dst, (__pyx_v_p_src->shape[__pyx_v_dim]), (__pyx_v_p_src->strides[__pyx_v_dim]), (__pyx_v_p_src->suboffsets[__pyx_v_dim]), __pyx_v_dim, __pyx_v_new_ndim, __pyx_v_p_suboffset_dim, __pyx_t_10, 0, 0, 0, 0, 0, 0); if (unlikely(__pyx_t_11 == -1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 705; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      goto __pyx_L6;
    }

    /* "View.MemoryView":711
 *                 0, 0, 0, # have_{start,stop,step}
 *                 False)
 *         elif index is None:             # <<<<<<<<<<<<<<
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0
 */
    __pyx_t_2 = (__pyx_v_index == Py_None);
    __pyx_t_1 = (__pyx_t_2 != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":712
 *                 False)
 *         elif index is None:
 *             p_dst.shape[new_ndim] = 1             # <<<<<<<<<<<<<<
 *             p_dst.strides[new_ndim] = 0
 *             p_dst.suboffsets[new_ndim] = -1
 */
      (__pyx_v_p_dst->shape[__pyx_v_new_ndim]) = 1;

      /* "View.MemoryView":713
 *         elif index is None:
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0             # <<<<<<<<<<<<<<
 *             p_dst.suboffsets[new_ndim] = -1
 *             new_ndim += 1
 */
      (__pyx_v_p_dst->strides[__pyx_v_new_ndim]) = 0;

      /* "View.MemoryView":714
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0
 *             p_dst.suboffsets[new_ndim] = -1             # <<<<<<<<<<<<<<
 *             new_ndim += 1
 *         else:
 */
      (__pyx_v_p_dst->suboffsets[__pyx_v_new_ndim]) = -1;

      /* "View.MemoryView":715
 *             p_dst.strides[new_ndim] = 0
 *             p_dst.suboffsets[new_ndim] = -1
 *             new_ndim += 1             # <<<<<<<<<<<<<<
 *         else:
 *             start = index.start or 0
 */
      __pyx_v_new_ndim = (__pyx_v_new_ndim + 1);
      goto __pyx_L6;
    }
    /*else*/ {

      /* "View.MemoryView":717
 *             new_ndim += 1
 *         else:
 *             start = index.start or 0             # <<<<<<<<<<<<<<
 *             stop = index.stop or 0
 *             step = index.step or 0
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_start); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 717; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 717; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      if (!__pyx_t_1) {
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else {
        __pyx_t_12 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 717; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_10 = __pyx_t_12;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        goto __pyx_L7_bool_binop_done;
      }
      __pyx_t_10 = 0;
      __pyx_L7_bool_binop_done:;
      __pyx_v_start = __pyx_t_10;

      /* "View.MemoryView":718
 *         else:
 *             start = index.start or 0
 *             stop = index.stop or 0             # <<<<<<<<<<<<<<
 *             step = index.step or 0
 * 
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_stop); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      if (!__pyx_t_1) {
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else {
        __pyx_t_12 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_10 = __pyx_t_12;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        goto __pyx_L9_bool_binop_done;
      }
      __pyx_t_10 = 0;
      __pyx_L9_bool_binop_done:;
      __pyx_v_stop = __pyx_t_10;

      /* "View.MemoryView":719
 *             start = index.start or 0
 *             stop = index.stop or 0
 *             step = index.step or 0             # <<<<<<<<<<<<<<
 * 
 *             have_start = index.start is not None
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_step); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 719; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 719; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      if (!__pyx_t_1) {
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else {
        __pyx_t_12 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 719; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_10 = __pyx_t_12;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        goto __pyx_L11_bool_binop_done;
      }
      __pyx_t_10 = 0;
      __pyx_L11_bool_binop_done:;
      __pyx_v_step = __pyx_t_10;

      /* "View.MemoryView":721
 *             step = index.step or 0
 * 
 *             have_start = index.start is not None             # <<<<<<<<<<<<<<
 *             have_stop = index.stop is not None
 *             have_step = index.step is not None
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_start); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 721; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_1 = (__pyx_t_9 != Py_None);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_v_have_start = __pyx_t_1;

      /* "View.MemoryView":722
 * 
 *             have_start = index.start is not None
 *             have_stop = index.stop is not None             # <<<<<<<<<<<<<<
 *             have_step = index.step is not None
 * 
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_stop); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 722; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_1 = (__pyx_t_9 != Py_None);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_v_have_stop = __pyx_t_1;

      /* "View.MemoryView":723
 *             have_start = index.start is not None
 *             have_stop = index.stop is not None
 *             have_step = index.step is not None             # <<<<<<<<<<<<<<
 * 
 *             slice_memviewslice(
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_step); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 723; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_1 = (__pyx_t_9 != Py_None);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_v_have_step = __pyx_t_1;

      /* "View.MemoryView":725
 *             have_step = index.step is not None
 * 
 *             slice_memviewslice(             # <<<<<<<<<<<<<<
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 *                 dim, new_ndim, p_suboffset_dim,
 */
      __pyx_t_11 = __pyx_memoryview_slice_memviewslice(__pyx_v_p_dst, (__pyx_v_p_src->shape[__pyx_v_dim]), (__pyx_v_p_src->strides[__pyx_v_dim]), (__pyx_v_p_src->suboffsets[__pyx_v_dim]), __pyx_v_dim, __pyx_v_new_ndim, __pyx_v_p_suboffset_dim, __pyx_v_start, __pyx_v_stop, __pyx_v_step, __pyx_v_have_start, __pyx_v_have_stop, __pyx_v_have_step, 1); if (unlikely(__pyx_t_11 == -1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 725; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "View.MemoryView":731
 *                 have_start, have_stop, have_step,
 *                 True)
 *             new_ndim += 1             # <<<<<<<<<<<<<<
 * 
 *     if isinstance(memview, _memoryviewslice):
 */
      __pyx_v_new_ndim = (__pyx_v_new_ndim + 1);
    }
    __pyx_L6:;

    /* "View.MemoryView":703
 *     cdef bint have_start, have_stop, have_step
 * 
 *     for dim, index in enumerate(indices):             # <<<<<<<<<<<<<<
 *         if PyIndex_Check(index):
 *             slice_memviewslice(
 */
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "View.MemoryView":733
 *             new_ndim += 1
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,
 */
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), ((PyObject *)__pyx_memoryviewslice_type)); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":734
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         return memoryview_fromslice(dst, new_ndim,             # <<<<<<<<<<<<<<
 *                                     memviewsliceobj.to_object_func,
 *                                     memviewsliceobj.to_dtype_func,
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));

    /* "View.MemoryView":735
 *     if isinstance(memview, _memoryviewslice):
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,             # <<<<<<<<<<<<<<
 *                                     memviewsliceobj.to_dtype_func,
 *                                     memview.dtype_is_object)
 */
    if (unlikely(!__pyx_v_memviewsliceobj)) { __Pyx_RaiseUnboundLocalError("memviewsliceobj"); {__pyx_filename = __pyx_f[3]; __pyx_lineno = 735; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }

    /* "View.MemoryView":736
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,
 *                                     memviewsliceobj.to_dtype_func,             # <<<<<<<<<<<<<<
 *                                     memview.dtype_is_object)
 *     else:
 */
    if (unlikely(!__pyx_v_memviewsliceobj)) { __Pyx_RaiseUnboundLocalError("memviewsliceobj"); {__pyx_filename = __pyx_f[3]; __pyx_lineno = 736; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }

    /* "View.MemoryView":734
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         return memoryview_fromslice(dst, new_ndim,             # <<<<<<<<<<<<<<
 *                                     memviewsliceobj.to_object_func,
 *                                     memviewsliceobj.to_dtype_func,
 */
    __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_dst, __pyx_v_new_ndim, __pyx_v_memviewsliceobj->to_object_func, __pyx_v_memviewsliceobj->to_dtype_func, __pyx_v_memview->dtype_is_object); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 734; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_memoryview_type))))) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 734; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_r = ((struct __pyx_memoryview_obj *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }
  /*else*/ {

    /* "View.MemoryView":739
 *                                     memview.dtype_is_object)
 *     else:
 *         return memoryview_fromslice(dst, new_ndim, NULL, NULL,             # <<<<<<<<<<<<<<
 *                                     memview.dtype_is_object)
 * 
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));

    /* "View.MemoryView":740
 *     else:
 *         return memoryview_fromslice(dst, new_ndim, NULL, NULL,
 *                                     memview.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_dst, __pyx_v_new_ndim, NULL, NULL, __pyx_v_memview->dtype_is_object); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 739; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);

    /* "View.MemoryView":739
 *                                     memview.dtype_is_object)
 *     else:
 *         return memoryview_fromslice(dst, new_ndim, NULL, NULL,             # <<<<<<<<<<<<<<
 *                                     memview.dtype_is_object)
 * 
 */
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_memoryview_type))))) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 739; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_r = ((struct __pyx_memoryview_obj *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "View.MemoryView":667
 * 
 * @cname('__pyx_memview_slice')
 * cdef memoryview memview_slice(memoryview memview, object indices):             # <<<<<<<<<<<<<<
 *     cdef int new_ndim = 0, suboffset_dim = -1, dim
 *     cdef bint negative_step
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("View.MemoryView.memview_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_memviewsliceobj);
  __Pyx_XDECREF(__pyx_v_index);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":764
 * 
 * @cname('__pyx_memoryview_slice_memviewslice')
 * cdef int slice_memviewslice(             # <<<<<<<<<<<<<<
 *         __Pyx_memviewslice *dst,
 *         Py_ssize_t shape, Py_ssize_t stride, Py_ssize_t suboffset,
 */

static int __pyx_memoryview_slice_memviewslice(__Pyx_memviewslice *__pyx_v_dst, Py_ssize_t __pyx_v_shape, Py_ssize_t __pyx_v_stride, Py_ssize_t __pyx_v_suboffset, int __pyx_v_dim, int __pyx_v_new_ndim, int *__pyx_v_suboffset_dim, Py_ssize_t __pyx_v_start, Py_ssize_t __pyx_v_stop, Py_ssize_t __pyx_v_step, int __pyx_v_have_start, int __pyx_v_have_stop, int __pyx_v_have_step, int __pyx_v_is_slice) {
  Py_ssize_t __pyx_v_new_shape;
  int __pyx_v_negative_step;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "View.MemoryView":784
 *     cdef bint negative_step
 * 
 *     if not is_slice:             # <<<<<<<<<<<<<<
 * 
 *         if start < 0:
 */
  __pyx_t_1 = ((!(__pyx_v_is_slice != 0)) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":786
 *     if not is_slice:
 * 
 *         if start < 0:             # <<<<<<<<<<<<<<
 *             start += shape
 *         if not 0 <= start < shape:
 */
    __pyx_t_1 = ((__pyx_v_start < 0) != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":787
 * 
 *         if start < 0:
 *             start += shape             # <<<<<<<<<<<<<<
 *         if not 0 <= start < shape:
 *             _err_dim(IndexError, "Index out of bounds (axis %d)", dim)
 */
      __pyx_v_start = (__pyx_v_start + __pyx_v_shape);
      goto __pyx_L4;
    }
    __pyx_L4:;

    /* "View.MemoryView":788
 *         if start < 0:
 *             start += shape
 *         if not 0 <= start < shape:             # <<<<<<<<<<<<<<
 *             _err_dim(IndexError, "Index out of bounds (axis %d)", dim)
 *     else:
 */
    __pyx_t_1 = (0 <= __pyx_v_start);
    if (__pyx_t_1) {
      __pyx_t_1 = (__pyx_v_start < __pyx_v_shape);
    }
    __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":789
 *             start += shape
 *         if not 0 <= start < shape:
 *             _err_dim(IndexError, "Index out of bounds (axis %d)", dim)             # <<<<<<<<<<<<<<
 *     else:
 * 
 */
      __pyx_t_3 = __pyx_memoryview_err_dim(__pyx_builtin_IndexError, __pyx_k_Index_out_of_bounds_axis_d, __pyx_v_dim); if (unlikely(__pyx_t_3 == -1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 789; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      goto __pyx_L5;
    }
    __pyx_L5:;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "View.MemoryView":792
 *     else:
 * 
 *         negative_step = have_step != 0 and step < 0             # <<<<<<<<<<<<<<
 * 
 *         if have_step and step == 0:
 */
    __pyx_t_1 = ((__pyx_v_have_step != 0) != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_2 = __pyx_t_1;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_step < 0) != 0);
    __pyx_t_2 = __pyx_t_1;
    __pyx_L6_bool_binop_done:;
    __pyx_v_negative_step = __pyx_t_2;

    /* "View.MemoryView":794
 *         negative_step = have_step != 0 and step < 0
 * 
 *         if have_step and step == 0:             # <<<<<<<<<<<<<<
 *             _err_dim(ValueError, "Step may not be zero (axis %d)", dim)
 * 
 */
    __pyx_t_1 = (__pyx_v_have_step != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_2 = __pyx_t_1;
      goto __pyx_L9_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_step == 0) != 0);
    __pyx_t_2 = __pyx_t_1;
    __pyx_L9_bool_binop_done:;
    if (__pyx_t_2) {

      /* "View.MemoryView":795
 * 
 *         if have_step and step == 0:
 *             _err_dim(ValueError, "Step may not be zero (axis %d)", dim)             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_t_3 = __pyx_memoryview_err_dim(__pyx_builtin_ValueError, __pyx_k_Step_may_not_be_zero_axis_d, __pyx_v_dim); if (unlikely(__pyx_t_3 == -1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      goto __pyx_L8;
    }
    __pyx_L8:;

    /* "View.MemoryView":798
 * 
 * 
 *         if have_start:             # <<<<<<<<<<<<<<
 *             if start < 0:
 *                 start += shape
 */
    __pyx_t_2 = (__pyx_v_have_start != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":799
 * 
 *         if have_start:
 *             if start < 0:             # <<<<<<<<<<<<<<
 *                 start += shape
 *                 if start < 0:
 */
      __pyx_t_2 = ((__pyx_v_start < 0) != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":800
 *         if have_start:
 *             if start < 0:
 *                 start += shape             # <<<<<<<<<<<<<<
 *                 if start < 0:
 *                     start = 0
 */
        __pyx_v_start = (__pyx_v_start + __pyx_v_shape);

        /* "View.MemoryView":801
 *             if start < 0:
 *                 start += shape
 *                 if start < 0:             # <<<<<<<<<<<<<<
 *                     start = 0
 *             elif start >= shape:
 */
        __pyx_t_2 = ((__pyx_v_start < 0) != 0);
        if (__pyx_t_2) {

          /* "View.MemoryView":802
 *                 start += shape
 *                 if start < 0:
 *                     start = 0             # <<<<<<<<<<<<<<
 *             elif start >= shape:
 *                 if negative_step:
 */
          __pyx_v_start = 0;
          goto __pyx_L13;
        }
        __pyx_L13:;
        goto __pyx_L12;
      }

      /* "View.MemoryView":803
 *                 if start < 0:
 *                     start = 0
 *             elif start >= shape:             # <<<<<<<<<<<<<<
 *                 if negative_step:
 *                     start = shape - 1
 */
      __pyx_t_2 = ((__pyx_v_start >= __pyx_v_shape) != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":804
 *                     start = 0
 *             elif start >= shape:
 *                 if negative_step:             # <<<<<<<<<<<<<<
 *                     start = shape - 1
 *                 else:
 */
        __pyx_t_2 = (__pyx_v_negative_step != 0);
        if (__pyx_t_2) {

          /* "View.MemoryView":805
 *             elif start >= shape:
 *                 if negative_step:
 *                     start = shape - 1             # <<<<<<<<<<<<<<
 *                 else:
 *                     start = shape
 */
          __pyx_v_start = (__pyx_v_shape - 1);
          goto __pyx_L14;
        }
        /*else*/ {

          /* "View.MemoryView":807
 *                     start = shape - 1
 *                 else:
 *                     start = shape             # <<<<<<<<<<<<<<
 *         else:
 *             if negative_step:
 */
          __pyx_v_start = __pyx_v_shape;
        }
        __pyx_L14:;
        goto __pyx_L12;
      }
      __pyx_L12:;
      goto __pyx_L11;
    }
    /*else*/ {

      /* "View.MemoryView":809
 *                     start = shape
 *         else:
 *             if negative_step:             # <<<<<<<<<<<<<<
 *                 start = shape - 1
 *             else:
 */
      __pyx_t_2 = (__pyx_v_negative_step != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":810
 *         else:
 *             if negative_step:
 *                 start = shape - 1             # <<<<<<<<<<<<<<
 *             else:
 *                 start = 0
 */
        __pyx_v_start = (__pyx_v_shape - 1);
        goto __pyx_L15;
      }
      /*else*/ {

        /* "View.MemoryView":812
 *                 start = shape - 1
 *             else:
 *                 start = 0             # <<<<<<<<<<<<<<
 * 
 *         if have_stop:
 */
        __pyx_v_start = 0;
      }
      __pyx_L15:;
    }
    __pyx_L11:;

    /* "View.MemoryView":814
 *                 start = 0
 * 
 *         if have_stop:             # <<<<<<<<<<<<<<
 *             if stop < 0:
 *                 stop += shape
 */
    __pyx_t_2 = (__pyx_v_have_stop != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":815
 * 
 *         if have_stop:
 *             if stop < 0:             # <<<<<<<<<<<<<<
 *                 stop += shape
 *                 if stop < 0:
 */
      __pyx_t_2 = ((__pyx_v_stop < 0) != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":816
 *         if have_stop:
 *             if stop < 0:
 *                 stop += shape             # <<<<<<<<<<<<<<
 *                 if stop < 0:
 *                     stop = 0
 */
        __pyx_v_stop = (__pyx_v_stop + __pyx_v_shape);

        /* "View.MemoryView":817
 *             if stop < 0:
 *                 stop += shape
 *                 if stop < 0:             # <<<<<<<<<<<<<<
 *                     stop = 0
 *             elif stop > shape:
 */
        __pyx_t_2 = ((__pyx_v_stop < 0) != 0);
        if (__pyx_t_2) {

          /* "View.MemoryView":818
 *                 stop += shape
 *                 if stop < 0:
 *                     stop = 0             # <<<<<<<<<<<<<<
 *             elif stop > shape:
 *                 stop = shape
 */
          __pyx_v_stop = 0;
          goto __pyx_L18;
        }
        __pyx_L18:;
        goto __pyx_L17;
      }

      /* "View.MemoryView":819
 *                 if stop < 0:
 *                     stop = 0
 *             elif stop > shape:             # <<<<<<<<<<<<<<
 *                 stop = shape
 *         else:
 */
      __pyx_t_2 = ((__pyx_v_stop > __pyx_v_shape) != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":820
 *                     stop = 0
 *             elif stop > shape:
 *                 stop = shape             # <<<<<<<<<<<<<<
 *         else:
 *             if negative_step:
 */
        __pyx_v_stop = __pyx_v_shape;
        goto __pyx_L17;
      }
      __pyx_L17:;
      goto __pyx_L16;
    }
    /*else*/ {

      /* "View.MemoryView":822
 *                 stop = shape
 *         else:
 *             if negative_step:             # <<<<<<<<<<<<<<
 *                 stop = -1
 *             else:
 */
      __pyx_t_2 = (__pyx_v_negative_step != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":823
 *         else:
 *             if negative_step:
 *                 stop = -1             # <<<<<<<<<<<<<<
 *             else:
 *                 stop = shape
 */
        __pyx_v_stop = -1;
        goto __pyx_L19;
      }
      /*else*/ {

        /* "View.MemoryView":825
 *                 stop = -1
 *             else:
 *                 stop = shape             # <<<<<<<<<<<<<<
 * 
 *         if not have_step:
 */
        __pyx_v_stop = __pyx_v_shape;
      }
      __pyx_L19:;
    }
    __pyx_L16:;

    /* "View.MemoryView":827
 *                 stop = shape
 * 
 *         if not have_step:             # <<<<<<<<<<<<<<
 *             step = 1
 * 
 */
    __pyx_t_2 = ((!(__pyx_v_have_step != 0)) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":828
 * 
 *         if not have_step:
 *             step = 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_v_step = 1;
      goto __pyx_L20;
    }
    __pyx_L20:;

    /* "View.MemoryView":832
 * 
 *         with cython.cdivision(True):
 *             new_shape = (stop - start) // step             # <<<<<<<<<<<<<<
 * 
 *             if (stop - start) - step * new_shape:
 */
    __pyx_v_new_shape = ((__pyx_v_stop - __pyx_v_start) / __pyx_v_step);

    /* "View.MemoryView":834
 *             new_shape = (stop - start) // step
 * 
 *             if (stop - start) - step * new_shape:             # <<<<<<<<<<<<<<
 *                 new_shape += 1
 * 
 */
    __pyx_t_2 = (((__pyx_v_stop - __pyx_v_start) - (__pyx_v_step * __pyx_v_new_shape)) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":835
 * 
 *             if (stop - start) - step * new_shape:
 *                 new_shape += 1             # <<<<<<<<<<<<<<
 * 
 *         if new_shape < 0:
 */
      __pyx_v_new_shape = (__pyx_v_new_shape + 1);
      goto __pyx_L21;
    }
    __pyx_L21:;

    /* "View.MemoryView":837
 *                 new_shape += 1
 * 
 *         if new_shape < 0:             # <<<<<<<<<<<<<<
 *             new_shape = 0
 * 
 */
    __pyx_t_2 = ((__pyx_v_new_shape < 0) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":838
 * 
 *         if new_shape < 0:
 *             new_shape = 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_v_new_shape = 0;
      goto __pyx_L22;
    }
    __pyx_L22:;

    /* "View.MemoryView":841
 * 
 * 
 *         dst.strides[new_ndim] = stride * step             # <<<<<<<<<<<<<<
 *         dst.shape[new_ndim] = new_shape
 *         dst.suboffsets[new_ndim] = suboffset
 */
    (__pyx_v_dst->strides[__pyx_v_new_ndim]) = (__pyx_v_stride * __pyx_v_step);

    /* "View.MemoryView":842
 * 
 *         dst.strides[new_ndim] = stride * step
 *         dst.shape[new_ndim] = new_shape             # <<<<<<<<<<<<<<
 *         dst.suboffsets[new_ndim] = suboffset
 * 
 */
    (__pyx_v_dst->shape[__pyx_v_new_ndim]) = __pyx_v_new_shape;

    /* "View.MemoryView":843
 *         dst.strides[new_ndim] = stride * step
 *         dst.shape[new_ndim] = new_shape
 *         dst.suboffsets[new_ndim] = suboffset             # <<<<<<<<<<<<<<
 * 
 * 
 */
    (__pyx_v_dst->suboffsets[__pyx_v_new_ndim]) = __pyx_v_suboffset;
  }
  __pyx_L3:;

  /* "View.MemoryView":846
 * 
 * 
 *     if suboffset_dim[0] < 0:             # <<<<<<<<<<<<<<
 *         dst.data += start * stride
 *     else:
 */
  __pyx_t_2 = (((__pyx_v_suboffset_dim[0]) < 0) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":847
 * 
 *     if suboffset_dim[0] < 0:
 *         dst.data += start * stride             # <<<<<<<<<<<<<<
 *     else:
 *         dst.suboffsets[suboffset_dim[0]] += start * stride
 */
    __pyx_v_dst->data = (__pyx_v_dst->data + (__pyx_v_start * __pyx_v_stride));
    goto __pyx_L23;
  }
  /*else*/ {

    /* "View.MemoryView":849
 *         dst.data += start * stride
 *     else:
 *         dst.suboffsets[suboffset_dim[0]] += start * stride             # <<<<<<<<<<<<<<
 * 
 *     if suboffset >= 0:
 */
    __pyx_t_3 = (__pyx_v_suboffset_dim[0]);
    (__pyx_v_dst->suboffsets[__pyx_t_3]) = ((__pyx_v_dst->suboffsets[__pyx_t_3]) + (__pyx_v_start * __pyx_v_stride));
  }
  __pyx_L23:;

  /* "View.MemoryView":851
 *         dst.suboffsets[suboffset_dim[0]] += start * stride
 * 
 *     if suboffset >= 0:             # <<<<<<<<<<<<<<
 *         if not is_slice:
 *             if new_ndim == 0:
 */
  __pyx_t_2 = ((__pyx_v_suboffset >= 0) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":852
 * 
 *     if suboffset >= 0:
 *         if not is_slice:             # <<<<<<<<<<<<<<
 *             if new_ndim == 0:
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 */
    __pyx_t_2 = ((!(__pyx_v_is_slice != 0)) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":853
 *     if suboffset >= 0:
 *         if not is_slice:
 *             if new_ndim == 0:             # <<<<<<<<<<<<<<
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 *             else:
 */
      __pyx_t_2 = ((__pyx_v_new_ndim == 0) != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":854
 *         if not is_slice:
 *             if new_ndim == 0:
 *                 dst.data = (<char **> dst.data)[0] + suboffset             # <<<<<<<<<<<<<<
 *             else:
 *                 _err_dim(IndexError, "All dimensions preceding dimension %d "
 */
        __pyx_v_dst->data = ((((char **)__pyx_v_dst->data)[0]) + __pyx_v_suboffset);
        goto __pyx_L26;
      }
      /*else*/ {

        /* "View.MemoryView":856
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 *             else:
 *                 _err_dim(IndexError, "All dimensions preceding dimension %d "             # <<<<<<<<<<<<<<
 *                                      "must be indexed and not sliced", dim)
 *         else:
 */
        __pyx_t_3 = __pyx_memoryview_err_dim(__pyx_builtin_IndexError, __pyx_k_All_dimensions_preceding_dimensi, __pyx_v_dim); if (unlikely(__pyx_t_3 == -1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 856; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_L26:;
      goto __pyx_L25;
    }
    /*else*/ {

      /* "View.MemoryView":859
 *                                      "must be indexed and not sliced", dim)
 *         else:
 *             suboffset_dim[0] = new_ndim             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
      (__pyx_v_suboffset_dim[0]) = __pyx_v_new_ndim;
    }
    __pyx_L25:;
    goto __pyx_L24;
  }
  __pyx_L24:;

  /* "View.MemoryView":861
 *             suboffset_dim[0] = new_ndim
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "View.MemoryView":764
 * 
 * @cname('__pyx_memoryview_slice_memviewslice')
 * cdef int slice_memviewslice(             # <<<<<<<<<<<<<<
 *         __Pyx_memviewslice *dst,
 *         Py_ssize_t shape, Py_ssize_t stride, Py_ssize_t suboffset,
 */

  /* function exit code */
  __pyx_L1_error:;
  {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
    #endif
    __Pyx_AddTraceback("View.MemoryView.slice_memviewslice", __pyx_clineno, __pyx_lineno, __pyx_filename);
    #ifdef WITH_THREAD
    PyGILState_Release(__pyx_gilstate_save);
    #endif
  }
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":867
 * 
 * @cname('__pyx_pybuffer_index')
 * cdef char *pybuffer_index(Py_buffer *view, char *bufp, Py_ssize_t index,             # <<<<<<<<<<<<<<
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1
 */

static char *__pyx_pybuffer_index(Py_buffer *__pyx_v_view, char *__pyx_v_bufp, Py_ssize_t __pyx_v_index, Py_ssize_t __pyx_v_dim) {
  Py_ssize_t __pyx_v_shape;
  Py_ssize_t __pyx_v_stride;
  Py_ssize_t __pyx_v_suboffset;
  Py_ssize_t __pyx_v_itemsize;
  char *__pyx_v_resultp;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pybuffer_index", 0);

  /* "View.MemoryView":869
 * cdef char *pybuffer_index(Py_buffer *view, char *bufp, Py_ssize_t index,
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t itemsize = view.itemsize
 *     cdef char *resultp
 */
  __pyx_v_suboffset = -1;

  /* "View.MemoryView":870
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1
 *     cdef Py_ssize_t itemsize = view.itemsize             # <<<<<<<<<<<<<<
 *     cdef char *resultp
 * 
 */
  __pyx_t_1 = __pyx_v_view->itemsize;
  __pyx_v_itemsize = __pyx_t_1;

  /* "View.MemoryView":873
 *     cdef char *resultp
 * 
 *     if view.ndim == 0:             # <<<<<<<<<<<<<<
 *         shape = view.len / itemsize
 *         stride = itemsize
 */
  __pyx_t_2 = ((__pyx_v_view->ndim == 0) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":874
 * 
 *     if view.ndim == 0:
 *         shape = view.len / itemsize             # <<<<<<<<<<<<<<
 *         stride = itemsize
 *     else:
 */
    if (unlikely(__pyx_v_itemsize == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
      #ifdef WITH_THREAD
      PyGILState_Release(__pyx_gilstate_save);
      #endif
      {__pyx_filename = __pyx_f[3]; __pyx_lineno = 874; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    else if (sizeof(Py_ssize_t) == sizeof(long) && unlikely(__pyx_v_itemsize == -1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_view->len))) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
      #ifdef WITH_THREAD
      PyGILState_Release(__pyx_gilstate_save);
      #endif
      {__pyx_filename = __pyx_f[3]; __pyx_lineno = 874; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_v_shape = (__pyx_v_view->len / __pyx_v_itemsize);

    /* "View.MemoryView":875
 *     if view.ndim == 0:
 *         shape = view.len / itemsize
 *         stride = itemsize             # <<<<<<<<<<<<<<
 *     else:
 *         shape = view.shape[dim]
 */
    __pyx_v_stride = __pyx_v_itemsize;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "View.MemoryView":877
 *         stride = itemsize
 *     else:
 *         shape = view.shape[dim]             # <<<<<<<<<<<<<<
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:
 */
    __pyx_v_shape = (__pyx_v_view->shape[__pyx_v_dim]);

    /* "View.MemoryView":878
 *     else:
 *         shape = view.shape[dim]
 *         stride = view.strides[dim]             # <<<<<<<<<<<<<<
 *         if view.suboffsets != NULL:
 *             suboffset = view.suboffsets[dim]
 */
    __pyx_v_stride = (__pyx_v_view->strides[__pyx_v_dim]);

    /* "View.MemoryView":879
 *         shape = view.shape[dim]
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *             suboffset = view.suboffsets[dim]
 * 
 */
    __pyx_t_2 = ((__pyx_v_view->suboffsets != NULL) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":880
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:
 *             suboffset = view.suboffsets[dim]             # <<<<<<<<<<<<<<
 * 
 *     if index < 0:
 */
      __pyx_v_suboffset = (__pyx_v_view->suboffsets[__pyx_v_dim]);
      goto __pyx_L4;
    }
    __pyx_L4:;
  }
  __pyx_L3:;

  /* "View.MemoryView":882
 *             suboffset = view.suboffsets[dim]
 * 
 *     if index < 0:             # <<<<<<<<<<<<<<
 *         index += view.shape[dim]
 *         if index < 0:
 */
  __pyx_t_2 = ((__pyx_v_index < 0) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":883
 * 
 *     if index < 0:
 *         index += view.shape[dim]             # <<<<<<<<<<<<<<
 *         if index < 0:
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 */
    __pyx_v_index = (__pyx_v_index + (__pyx_v_view->shape[__pyx_v_dim]));

    /* "View.MemoryView":884
 *     if index < 0:
 *         index += view.shape[dim]
 *         if index < 0:             # <<<<<<<<<<<<<<
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 */
    __pyx_t_2 = ((__pyx_v_index < 0) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":885
 *         index += view.shape[dim]
 *         if index < 0:
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)             # <<<<<<<<<<<<<<
 * 
 *     if index >= shape:
 */
      __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 885; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Out_of_bounds_on_buffer_access_a, __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 885; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 885; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_IndexError, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 885; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_Raise(__pyx_t_4, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      {__pyx_filename = __pyx_f[3]; __pyx_lineno = 885; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    goto __pyx_L5;
  }
  __pyx_L5:;

  /* "View.MemoryView":887
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 *     if index >= shape:             # <<<<<<<<<<<<<<
 *         raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 */
  __pyx_t_2 = ((__pyx_v_index >= __pyx_v_shape) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":888
 * 
 *     if index >= shape:
 *         raise IndexError("Out of bounds on buffer access (axis %d)" % dim)             # <<<<<<<<<<<<<<
 * 
 *     resultp = bufp + index * stride
 */
    __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_Out_of_bounds_on_buffer_access_a, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_IndexError, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[3]; __pyx_lineno = 888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "View.MemoryView":890
 *         raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 *     resultp = bufp + index * stride             # <<<<<<<<<<<<<<
 *     if suboffset >= 0:
 *         resultp = (<char **> resultp)[0] + suboffset
 */
  __pyx_v_resultp = (__pyx_v_bufp + (__pyx_v_index * __pyx_v_stride));

  /* "View.MemoryView":891
 * 
 *     resultp = bufp + index * stride
 *     if suboffset >= 0:             # <<<<<<<<<<<<<<
 *         resultp = (<char **> resultp)[0] + suboffset
 * 
 */
  __pyx_t_2 = ((__pyx_v_suboffset >= 0) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":892
 *     resultp = bufp + index * stride
 *     if suboffset >= 0:
 *         resultp = (<char **> resultp)[0] + suboffset             # <<<<<<<<<<<<<<
 * 
 *     return resultp
 */
    __pyx_v_resultp = ((((char **)__pyx_v_resultp)[0]) + __pyx_v_suboffset);
    goto __pyx_L8;
  }
  __pyx_L8:;

  /* "View.MemoryView":894
 *         resultp = (<char **> resultp)[0] + suboffset
 * 
 *     return resultp             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_resultp;
  goto __pyx_L0;

  /* "View.MemoryView":867
 * 
 * @cname('__pyx_pybuffer_index')
 * cdef char *pybuffer_index(Py_buffer *view, char *bufp, Py_ssize_t index,             # <<<<<<<<<<<<<<
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("View.MemoryView.pybuffer_index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":900
 * 
 * @cname('__pyx_memslice_transpose')
 * cdef int transpose_memslice(__Pyx_memviewslice *memslice) nogil except 0:             # <<<<<<<<<<<<<<
 *     cdef int ndim = memslice.memview.view.ndim
 * 
 */

static int __pyx_memslice_transpose(__Pyx_memviewslice *__pyx_v_memslice) {
  int __pyx_v_ndim;
  Py_ssize_t *__pyx_v_shape;
  Py_ssize_t *__pyx_v_strides;
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_r;
  int __pyx_t_1;
  Py_ssize_t *__pyx_t_2;
  long __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "View.MemoryView":901
 * @cname('__pyx_memslice_transpose')
 * cdef int transpose_memslice(__Pyx_memviewslice *memslice) nogil except 0:
 *     cdef int ndim = memslice.memview.view.ndim             # <<<<<<<<<<<<<<
 * 
 *     cdef Py_ssize_t *shape = memslice.shape
 */
  __pyx_t_1 = __pyx_v_memslice->memview->view.ndim;
  __pyx_v_ndim = __pyx_t_1;

  /* "View.MemoryView":903
 *     cdef int ndim = memslice.memview.view.ndim
 * 
 *     cdef Py_ssize_t *shape = memslice.shape             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t *strides = memslice.strides
 * 
 */
  __pyx_t_2 = __pyx_v_memslice->shape;
  __pyx_v_shape = __pyx_t_2;

  /* "View.MemoryView":904
 * 
 *     cdef Py_ssize_t *shape = memslice.shape
 *     cdef Py_ssize_t *strides = memslice.strides             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __pyx_v_memslice->strides;
  __pyx_v_strides = __pyx_t_2;

  /* "View.MemoryView":908
 * 
 *     cdef int i, j
 *     for i in range(ndim / 2):             # <<<<<<<<<<<<<<
 *         j = ndim - 1 - i
 *         strides[i], strides[j] = strides[j], strides[i]
 */
  __pyx_t_3 = (__pyx_v_ndim / 2);
  for (__pyx_t_1 = 0; __pyx_t_1 < __pyx_t_3; __pyx_t_1+=1) {
    __pyx_v_i = __pyx_t_1;

    /* "View.MemoryView":909
 *     cdef int i, j
 *     for i in range(ndim / 2):
 *         j = ndim - 1 - i             # <<<<<<<<<<<<<<
 *         strides[i], strides[j] = strides[j], strides[i]
 *         shape[i], shape[j] = shape[j], shape[i]
 */
    __pyx_v_j = ((__pyx_v_ndim - 1) - __pyx_v_i);

    /* "View.MemoryView":910
 *     for i in range(ndim / 2):
 *         j = ndim - 1 - i
 *         strides[i], strides[j] = strides[j], strides[i]             # <<<<<<<<<<<<<<
 *         shape[i], shape[j] = shape[j], shape[i]
 * 
 */
    __pyx_t_4 = (__pyx_v_strides[__pyx_v_j]);
    __pyx_t_5 = (__pyx_v_strides[__pyx_v_i]);
    (__pyx_v_strides[__pyx_v_i]) = __pyx_t_4;
    (__pyx_v_strides[__pyx_v_j]) = __pyx_t_5;

    /* "View.MemoryView":911
 *         j = ndim - 1 - i
 *         strides[i], strides[j] = strides[j], strides[i]
 *         shape[i], shape[j] = shape[j], shape[i]             # <<<<<<<<<<<<<<
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:
 */
    __pyx_t_5 = (__pyx_v_shape[__pyx_v_j]);
    __pyx_t_4 = (__pyx_v_shape[__pyx_v_i]);
    (__pyx_v_shape[__pyx_v_i]) = __pyx_t_5;
    (__pyx_v_shape[__pyx_v_j]) = __pyx_t_4;

    /* "View.MemoryView":913
 *         shape[i], shape[j] = shape[j], shape[i]
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:             # <<<<<<<<<<<<<<
 *             _err(ValueError, "Cannot transpose memoryview with indirect dimensions")
 * 
 */
    __pyx_t_7 = (((__pyx_v_memslice->suboffsets[__pyx_v_i]) >= 0) != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_7 = (((__pyx_v_memslice->suboffsets[__pyx_v_j]) >= 0) != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_6) {

      /* "View.MemoryView":914
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:
 *             _err(ValueError, "Cannot transpose memoryview with indirect dimensions")             # <<<<<<<<<<<<<<
 * 
 *     return 1
 */
      __pyx_t_8 = __pyx_memoryview_err(__pyx_builtin_ValueError, __pyx_k_Cannot_transpose_memoryview_with); if (unlikely(__pyx_t_8 == -1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 914; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      goto __pyx_L5;
    }
    __pyx_L5:;
  }

  /* "View.MemoryView":916
 *             _err(ValueError, "Cannot transpose memoryview with indirect dimensions")
 * 
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "View.MemoryView":900
 * 
 * @cname('__pyx_memslice_transpose')
 * cdef int transpose_memslice(__Pyx_memviewslice *memslice) nogil except 0:             # <<<<<<<<<<<<<<
 *     cdef int ndim = memslice.memview.view.ndim
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
    #endif
    __Pyx_AddTraceback("View.MemoryView.transpose_memslice", __pyx_clineno, __pyx_lineno, __pyx_filename);
    #ifdef WITH_THREAD
    PyGILState_Release(__pyx_gilstate_save);
    #endif
  }
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":933
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)
 * 
 */

/* Python wrapper */
static void __pyx_memoryviewslice___dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_memoryviewslice___dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_memoryviewslice_MemoryView_16_memoryviewslice___dealloc__(((struct __pyx_memoryviewslice_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_memoryviewslice_MemoryView_16_memoryviewslice___dealloc__(struct __pyx_memoryviewslice_obj *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "View.MemoryView":934
 * 
 *     def __dealloc__(self):
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)             # <<<<<<<<<<<<<<
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 */
  __PYX_XDEC_MEMVIEW((&__pyx_v_self->from_slice), 1);

  /* "View.MemoryView":933
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "View.MemoryView":936
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         if self.to_object_func != NULL:
 *             return self.to_object_func(itemp)
 */

static PyObject *__pyx_memoryviewslice_convert_item_to_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("convert_item_to_object", 0);

  /* "View.MemoryView":937
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 *         if self.to_object_func != NULL:             # <<<<<<<<<<<<<<
 *             return self.to_object_func(itemp)
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_self->to_object_func != NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":938
 *     cdef convert_item_to_object(self, char *itemp):
 *         if self.to_object_func != NULL:
 *             return self.to_object_func(itemp)             # <<<<<<<<<<<<<<
 *         else:
 *             return memoryview.convert_item_to_object(self, itemp)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_v_self->to_object_func(__pyx_v_itemp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 938; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }
  /*else*/ {

    /* "View.MemoryView":940
 *             return self.to_object_func(itemp)
 *         else:
 *             return memoryview.convert_item_to_object(self, itemp)             # <<<<<<<<<<<<<<
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_memoryview_convert_item_to_object(((struct __pyx_memoryview_obj *)__pyx_v_self), __pyx_v_itemp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 940; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "View.MemoryView":936
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         if self.to_object_func != NULL:
 *             return self.to_object_func(itemp)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.convert_item_to_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":942
 *             return memoryview.convert_item_to_object(self, itemp)
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         if self.to_dtype_func != NULL:
 *             self.to_dtype_func(itemp, value)
 */

static PyObject *__pyx_memoryviewslice_assign_item_from_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("assign_item_from_object", 0);

  /* "View.MemoryView":943
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 *         if self.to_dtype_func != NULL:             # <<<<<<<<<<<<<<
 *             self.to_dtype_func(itemp, value)
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_self->to_dtype_func != NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":944
 *     cdef assign_item_from_object(self, char *itemp, object value):
 *         if self.to_dtype_func != NULL:
 *             self.to_dtype_func(itemp, value)             # <<<<<<<<<<<<<<
 *         else:
 *             memoryview.assign_item_from_object(self, itemp, value)
 */
    __pyx_t_2 = __pyx_v_self->to_dtype_func(__pyx_v_itemp, __pyx_v_value); if (unlikely(__pyx_t_2 == 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 944; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L3;
  }
  /*else*/ {

    /* "View.MemoryView":946
 *             self.to_dtype_func(itemp, value)
 *         else:
 *             memoryview.assign_item_from_object(self, itemp, value)             # <<<<<<<<<<<<<<
 * 
 *     property base:
 */
    __pyx_t_3 = __pyx_memoryview_assign_item_from_object(((struct __pyx_memoryview_obj *)__pyx_v_self), __pyx_v_itemp, __pyx_v_value); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "View.MemoryView":942
 *             return memoryview.convert_item_to_object(self, itemp)
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         if self.to_dtype_func != NULL:
 *             self.to_dtype_func(itemp, value)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.assign_item_from_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":950
 *     property base:
 *         @cname('__pyx_memoryviewslice__get__base')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.from_object
 * 
 */

/* Python wrapper */
static PyObject *__pyx_memoryviewslice__get__base(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryviewslice__get__base(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_memoryviewslice__get__base_MemoryView_16_memoryviewslice_4base___get__(((struct __pyx_memoryviewslice_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryviewslice__get__base_MemoryView_16_memoryviewslice_4base___get__(struct __pyx_memoryviewslice_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":951
 *         @cname('__pyx_memoryviewslice__get__base')
 *         def __get__(self):
 *             return self.from_object             # <<<<<<<<<<<<<<
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_memoryview_getbuffer, "getbuffer(obj, view, flags)")
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->from_object);
  __pyx_r = __pyx_v_self->from_object;
  goto __pyx_L0;

  /* "View.MemoryView":950
 *     property base:
 *         @cname('__pyx_memoryviewslice__get__base')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.from_object
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":957
 * 
 * @cname('__pyx_memoryview_fromslice')
 * cdef memoryview_fromslice(__Pyx_memviewslice memviewslice,             # <<<<<<<<<<<<<<
 *                           int ndim,
 *                           object (*to_object_func)(char *),
 */

static PyObject *__pyx_memoryview_fromslice(__Pyx_memviewslice __pyx_v_memviewslice, int __pyx_v_ndim, PyObject *(*__pyx_v_to_object_func)(char *), int (*__pyx_v_to_dtype_func)(char *, PyObject *), int __pyx_v_dtype_is_object) {
  struct __pyx_memoryviewslice_obj *__pyx_v_result = 0;
  int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_TypeInfo *__pyx_t_4;
  Py_buffer __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memoryview_fromslice", 0);

  /* "View.MemoryView":966
 *     cdef int i
 * 
 *     if <PyObject *> memviewslice.memview == Py_None:             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
  __pyx_t_1 = ((((PyObject *)__pyx_v_memviewslice.memview) == Py_None) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":967
 * 
 *     if <PyObject *> memviewslice.memview == Py_None:
 *         return None             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_None);
    __pyx_r = Py_None;
    goto __pyx_L0;
  }

  /* "View.MemoryView":972
 * 
 * 
 *     result = _memoryviewslice(None, 0, dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 *     result.from_slice = memviewslice
 */
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_dtype_is_object); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 972; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 972; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_3, 0, Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_INCREF(__pyx_int_0);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)((PyObject *)__pyx_memoryviewslice_type)), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 972; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_result = ((struct __pyx_memoryviewslice_obj *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "View.MemoryView":974
 *     result = _memoryviewslice(None, 0, dtype_is_object)
 * 
 *     result.from_slice = memviewslice             # <<<<<<<<<<<<<<
 *     __PYX_INC_MEMVIEW(&memviewslice, 1)
 * 
 */
  __pyx_v_result->from_slice = __pyx_v_memviewslice;

  /* "View.MemoryView":975
 * 
 *     result.from_slice = memviewslice
 *     __PYX_INC_MEMVIEW(&memviewslice, 1)             # <<<<<<<<<<<<<<
 * 
 *     result.from_object = (<memoryview> memviewslice.memview).base
 */
  __PYX_INC_MEMVIEW((&__pyx_v_memviewslice), 1);

  /* "View.MemoryView":977
 *     __PYX_INC_MEMVIEW(&memviewslice, 1)
 * 
 *     result.from_object = (<memoryview> memviewslice.memview).base             # <<<<<<<<<<<<<<
 *     result.typeinfo = memviewslice.memview.typeinfo
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_memviewslice.memview), __pyx_n_s_base); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 977; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_result->from_object);
  __Pyx_DECREF(__pyx_v_result->from_object);
  __pyx_v_result->from_object = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "View.MemoryView":978
 * 
 *     result.from_object = (<memoryview> memviewslice.memview).base
 *     result.typeinfo = memviewslice.memview.typeinfo             # <<<<<<<<<<<<<<
 * 
 *     result.view = memviewslice.memview.view
 */
  __pyx_t_4 = __pyx_v_memviewslice.memview->typeinfo;
  __pyx_v_result->__pyx_base.typeinfo = __pyx_t_4;

  /* "View.MemoryView":980
 *     result.typeinfo = memviewslice.memview.typeinfo
 * 
 *     result.view = memviewslice.memview.view             # <<<<<<<<<<<<<<
 *     result.view.buf = <void *> memviewslice.data
 *     result.view.ndim = ndim
 */
  __pyx_t_5 = __pyx_v_memviewslice.memview->view;
  __pyx_v_result->__pyx_base.view = __pyx_t_5;

  /* "View.MemoryView":981
 * 
 *     result.view = memviewslice.memview.view
 *     result.view.buf = <void *> memviewslice.data             # <<<<<<<<<<<<<<
 *     result.view.ndim = ndim
 *     (<__pyx_buffer *> &result.view).obj = Py_None
 */
  __pyx_v_result->__pyx_base.view.buf = ((void *)__pyx_v_memviewslice.data);

  /* "View.MemoryView":982
 *     result.view = memviewslice.memview.view
 *     result.view.buf = <void *> memviewslice.data
 *     result.view.ndim = ndim             # <<<<<<<<<<<<<<
 *     (<__pyx_buffer *> &result.view).obj = Py_None
 *     Py_INCREF(Py_None)
 */
  __pyx_v_result->__pyx_base.view.ndim = __pyx_v_ndim;

  /* "View.MemoryView":983
 *     result.view.buf = <void *> memviewslice.data
 *     result.view.ndim = ndim
 *     (<__pyx_buffer *> &result.view).obj = Py_None             # <<<<<<<<<<<<<<
 *     Py_INCREF(Py_None)
 * 
 */
  ((Py_buffer *)(&__pyx_v_result->__pyx_base.view))->obj = Py_None;

  /* "View.MemoryView":984
 *     result.view.ndim = ndim
 *     (<__pyx_buffer *> &result.view).obj = Py_None
 *     Py_INCREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *     result.flags = PyBUF_RECORDS
 */
  Py_INCREF(Py_None);

  /* "View.MemoryView":986
 *     Py_INCREF(Py_None)
 * 
 *     result.flags = PyBUF_RECORDS             # <<<<<<<<<<<<<<
 * 
 *     result.view.shape = <Py_ssize_t *> result.from_slice.shape
 */
  __pyx_v_result->__pyx_base.flags = PyBUF_RECORDS;

  /* "View.MemoryView":988
 *     result.flags = PyBUF_RECORDS
 * 
 *     result.view.shape = <Py_ssize_t *> result.from_slice.shape             # <<<<<<<<<<<<<<
 *     result.view.strides = <Py_ssize_t *> result.from_slice.strides
 *     result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 */
  __pyx_v_result->__pyx_base.view.shape = ((Py_ssize_t *)__pyx_v_result->from_slice.shape);

  /* "View.MemoryView":989
 * 
 *     result.view.shape = <Py_ssize_t *> result.from_slice.shape
 *     result.view.strides = <Py_ssize_t *> result.from_slice.strides             # <<<<<<<<<<<<<<
 *     result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 * 
 */
  __pyx_v_result->__pyx_base.view.strides = ((Py_ssize_t *)__pyx_v_result->from_slice.strides);

  /* "View.MemoryView":990
 *     result.view.shape = <Py_ssize_t *> result.from_slice.shape
 *     result.view.strides = <Py_ssize_t *> result.from_slice.strides
 *     result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets             # <<<<<<<<<<<<<<
 * 
 *     result.view.len = result.view.itemsize
 */
  __pyx_v_result->__pyx_base.view.suboffsets = ((Py_ssize_t *)__pyx_v_result->from_slice.suboffsets);

  /* "View.MemoryView":992
 *     result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 * 
 *     result.view.len = result.view.itemsize             # <<<<<<<<<<<<<<
 *     for i in range(ndim):
 *         result.view.len *= result.view.shape[i]
 */
  __pyx_t_6 = __pyx_v_result->__pyx_base.view.itemsize;
  __pyx_v_result->__pyx_base.view.len = __pyx_t_6;

  /* "View.MemoryView":993
 * 
 *     result.view.len = result.view.itemsize
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         result.view.len *= result.view.shape[i]
 * 
 */
  __pyx_t_7 = __pyx_v_ndim;
  for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
    __pyx_v_i = __pyx_t_8;

    /* "View.MemoryView":994
 *     result.view.len = result.view.itemsize
 *     for i in range(ndim):
 *         result.view.len *= result.view.shape[i]             # <<<<<<<<<<<<<<
 * 
 *     result.to_object_func = to_object_func
 */
    __pyx_v_result->__pyx_base.view.len = (__pyx_v_result->__pyx_base.view.len * (__pyx_v_result->__pyx_base.view.shape[__pyx_v_i]));
  }

  /* "View.MemoryView":996
 *         result.view.len *= result.view.shape[i]
 * 
 *     result.to_object_func = to_object_func             # <<<<<<<<<<<<<<
 *     result.to_dtype_func = to_dtype_func
 * 
 */
  __pyx_v_result->to_object_func = __pyx_v_to_object_func;

  /* "View.MemoryView":997
 * 
 *     result.to_object_func = to_object_func
 *     result.to_dtype_func = to_dtype_func             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
  __pyx_v_result->to_dtype_func = __pyx_v_to_dtype_func;

  /* "View.MemoryView":999
 *     result.to_dtype_func = to_dtype_func
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_get_slice_from_memoryview')
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_result));
  __pyx_r = ((PyObject *)__pyx_v_result);
  goto __pyx_L0;

  /* "View.MemoryView":957
 * 
 * @cname('__pyx_memoryview_fromslice')
 * cdef memoryview_fromslice(__Pyx_memviewslice memviewslice,             # <<<<<<<<<<<<<<
 *                           int ndim,
 *                           object (*to_object_func)(char *),
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview_fromslice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":1002
 * 
 * @cname('__pyx_memoryview_get_slice_from_memoryview')
 * cdef __Pyx_memviewslice *get_slice_from_memview(memoryview memview,             # <<<<<<<<<<<<<<
 *                                                    __Pyx_memviewslice *mslice):
 *     cdef _memoryviewslice obj
 */

static __Pyx_memviewslice *__pyx_memoryview_get_slice_from_memoryview(struct __pyx_memoryview_obj *__pyx_v_memview, __Pyx_memviewslice *__pyx_v_mslice) {
  struct __pyx_memoryviewslice_obj *__pyx_v_obj = 0;
  __Pyx_memviewslice *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_slice_from_memview", 0);

  /* "View.MemoryView":1005
 *                                                    __Pyx_memviewslice *mslice):
 *     cdef _memoryviewslice obj
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         obj = memview
 *         return &obj.from_slice
 */
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), ((PyObject *)__pyx_memoryviewslice_type)); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1006
 *     cdef _memoryviewslice obj
 *     if isinstance(memview, _memoryviewslice):
 *         obj = memview             # <<<<<<<<<<<<<<
 *         return &obj.from_slice
 *     else:
 */
    if (!(likely(((((PyObject *)__pyx_v_memview)) == Py_None) || likely(__Pyx_TypeTest(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type))))) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1006; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = ((PyObject *)__pyx_v_memview);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_obj = ((struct __pyx_memoryviewslice_obj *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "View.MemoryView":1007
 *     if isinstance(memview, _memoryviewslice):
 *         obj = memview
 *         return &obj.from_slice             # <<<<<<<<<<<<<<
 *     else:
 *         slice_copy(memview, mslice)
 */
    __pyx_r = (&__pyx_v_obj->from_slice);
    goto __pyx_L0;
  }
  /*else*/ {

    /* "View.MemoryView":1009
 *         return &obj.from_slice
 *     else:
 *         slice_copy(memview, mslice)             # <<<<<<<<<<<<<<
 *         return mslice
 * 
 */
    __pyx_memoryview_slice_copy(__pyx_v_memview, __pyx_v_mslice);

    /* "View.MemoryView":1010
 *     else:
 *         slice_copy(memview, mslice)
 *         return mslice             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_slice_copy')
 */
    __pyx_r = __pyx_v_mslice;
    goto __pyx_L0;
  }

  /* "View.MemoryView":1002
 * 
 * @cname('__pyx_memoryview_get_slice_from_memoryview')
 * cdef __Pyx_memviewslice *get_slice_from_memview(memoryview memview,             # <<<<<<<<<<<<<<
 *                                                    __Pyx_memviewslice *mslice):
 *     cdef _memoryviewslice obj
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_WriteUnraisable("View.MemoryView.get_slice_from_memview", __pyx_clineno, __pyx_lineno, __pyx_filename, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_obj);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":1013
 * 
 * @cname('__pyx_memoryview_slice_copy')
 * cdef void slice_copy(memoryview memview, __Pyx_memviewslice *dst):             # <<<<<<<<<<<<<<
 *     cdef int dim
 *     cdef (Py_ssize_t*) shape, strides, suboffsets
 */

static void __pyx_memoryview_slice_copy(struct __pyx_memoryview_obj *__pyx_v_memview, __Pyx_memviewslice *__pyx_v_dst) {
  int __pyx_v_dim;
  Py_ssize_t *__pyx_v_shape;
  Py_ssize_t *__pyx_v_strides;
  Py_ssize_t *__pyx_v_suboffsets;
  __Pyx_RefNannyDeclarations
  Py_ssize_t *__pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("slice_copy", 0);

  /* "View.MemoryView":1017
 *     cdef (Py_ssize_t*) shape, strides, suboffsets
 * 
 *     shape = memview.view.shape             # <<<<<<<<<<<<<<
 *     strides = memview.view.strides
 *     suboffsets = memview.view.suboffsets
 */
  __pyx_t_1 = __pyx_v_memview->view.shape;
  __pyx_v_shape = __pyx_t_1;

  /* "View.MemoryView":1018
 * 
 *     shape = memview.view.shape
 *     strides = memview.view.strides             # <<<<<<<<<<<<<<
 *     suboffsets = memview.view.suboffsets
 * 
 */
  __pyx_t_1 = __pyx_v_memview->view.strides;
  __pyx_v_strides = __pyx_t_1;

  /* "View.MemoryView":1019
 *     shape = memview.view.shape
 *     strides = memview.view.strides
 *     suboffsets = memview.view.suboffsets             # <<<<<<<<<<<<<<
 * 
 *     dst.memview = <__pyx_memoryview *> memview
 */
  __pyx_t_1 = __pyx_v_memview->view.suboffsets;
  __pyx_v_suboffsets = __pyx_t_1;

  /* "View.MemoryView":1021
 *     suboffsets = memview.view.suboffsets
 * 
 *     dst.memview = <__pyx_memoryview *> memview             # <<<<<<<<<<<<<<
 *     dst.data = <char *> memview.view.buf
 * 
 */
  __pyx_v_dst->memview = ((struct __pyx_memoryview_obj *)__pyx_v_memview);

  /* "View.MemoryView":1022
 * 
 *     dst.memview = <__pyx_memoryview *> memview
 *     dst.data = <char *> memview.view.buf             # <<<<<<<<<<<<<<
 * 
 *     for dim in range(memview.view.ndim):
 */
  __pyx_v_dst->data = ((char *)__pyx_v_memview->view.buf);

  /* "View.MemoryView":1024
 *     dst.data = <char *> memview.view.buf
 * 
 *     for dim in range(memview.view.ndim):             # <<<<<<<<<<<<<<
 *         dst.shape[dim] = shape[dim]
 *         dst.strides[dim] = strides[dim]
 */
  __pyx_t_2 = __pyx_v_memview->view.ndim;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_dim = __pyx_t_3;

    /* "View.MemoryView":1025
 * 
 *     for dim in range(memview.view.ndim):
 *         dst.shape[dim] = shape[dim]             # <<<<<<<<<<<<<<
 *         dst.strides[dim] = strides[dim]
 *         if suboffsets == NULL:
 */
    (__pyx_v_dst->shape[__pyx_v_dim]) = (__pyx_v_shape[__pyx_v_dim]);

    /* "View.MemoryView":1026
 *     for dim in range(memview.view.ndim):
 *         dst.shape[dim] = shape[dim]
 *         dst.strides[dim] = strides[dim]             # <<<<<<<<<<<<<<
 *         if suboffsets == NULL:
 *             dst.suboffsets[dim] = -1
 */
    (__pyx_v_dst->strides[__pyx_v_dim]) = (__pyx_v_strides[__pyx_v_dim]);

    /* "View.MemoryView":1027
 *         dst.shape[dim] = shape[dim]
 *         dst.strides[dim] = strides[dim]
 *         if suboffsets == NULL:             # <<<<<<<<<<<<<<
 *             dst.suboffsets[dim] = -1
 *         else:
 */
    __pyx_t_4 = ((__pyx_v_suboffsets == NULL) != 0);
    if (__pyx_t_4) {

      /* "View.MemoryView":1028
 *         dst.strides[dim] = strides[dim]
 *         if suboffsets == NULL:
 *             dst.suboffsets[dim] = -1             # <<<<<<<<<<<<<<
 *         else:
 *             dst.suboffsets[dim] = suboffsets[dim]
 */
      (__pyx_v_dst->suboffsets[__pyx_v_dim]) = -1;
      goto __pyx_L5;
    }
    /*else*/ {

      /* "View.MemoryView":1030
 *             dst.suboffsets[dim] = -1
 *         else:
 *             dst.suboffsets[dim] = suboffsets[dim]             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_object')
 */
      (__pyx_v_dst->suboffsets[__pyx_v_dim]) = (__pyx_v_suboffsets[__pyx_v_dim]);
    }
    __pyx_L5:;
  }

  /* "View.MemoryView":1013
 * 
 * @cname('__pyx_memoryview_slice_copy')
 * cdef void slice_copy(memoryview memview, __Pyx_memviewslice *dst):             # <<<<<<<<<<<<<<
 *     cdef int dim
 *     cdef (Py_ssize_t*) shape, strides, suboffsets
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "View.MemoryView":1033
 * 
 * @cname('__pyx_memoryview_copy_object')
 * cdef memoryview_copy(memoryview memview):             # <<<<<<<<<<<<<<
 *     "Create a new memoryview object"
 *     cdef __Pyx_memviewslice memviewslice
 */

static PyObject *__pyx_memoryview_copy_object(struct __pyx_memoryview_obj *__pyx_v_memview) {
  __Pyx_memviewslice __pyx_v_memviewslice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memoryview_copy", 0);

  /* "View.MemoryView":1036
 *     "Create a new memoryview object"
 *     cdef __Pyx_memviewslice memviewslice
 *     slice_copy(memview, &memviewslice)             # <<<<<<<<<<<<<<
 *     return memoryview_copy_from_slice(memview, &memviewslice)
 * 
 */
  __pyx_memoryview_slice_copy(__pyx_v_memview, (&__pyx_v_memviewslice));

  /* "View.MemoryView":1037
 *     cdef __Pyx_memviewslice memviewslice
 *     slice_copy(memview, &memviewslice)
 *     return memoryview_copy_from_slice(memview, &memviewslice)             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_object_from_slice')
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_copy_object_from_slice(__pyx_v_memview, (&__pyx_v_memviewslice)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1037; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":1033
 * 
 * @cname('__pyx_memoryview_copy_object')
 * cdef memoryview_copy(memoryview memview):             # <<<<<<<<<<<<<<
 *     "Create a new memoryview object"
 *     cdef __Pyx_memviewslice memviewslice
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview_copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":1040
 * 
 * @cname('__pyx_memoryview_copy_object_from_slice')
 * cdef memoryview_copy_from_slice(memoryview memview, __Pyx_memviewslice *memviewslice):             # <<<<<<<<<<<<<<
 *     """
 *     Create a new memoryview object from a given memoryview object and slice.
 */

static PyObject *__pyx_memoryview_copy_object_from_slice(struct __pyx_memoryview_obj *__pyx_v_memview, __Pyx_memviewslice *__pyx_v_memviewslice) {
  PyObject *(*__pyx_v_to_object_func)(char *);
  int (*__pyx_v_to_dtype_func)(char *, PyObject *);
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *(*__pyx_t_3)(char *);
  int (*__pyx_t_4)(char *, PyObject *);
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memoryview_copy_from_slice", 0);

  /* "View.MemoryView":1047
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         to_object_func = (<_memoryviewslice> memview).to_object_func
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 */
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), ((PyObject *)__pyx_memoryviewslice_type)); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1048
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         to_object_func = (<_memoryviewslice> memview).to_object_func             # <<<<<<<<<<<<<<
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 *     else:
 */
    __pyx_t_3 = ((struct __pyx_memoryviewslice_obj *)__pyx_v_memview)->to_object_func;
    __pyx_v_to_object_func = __pyx_t_3;

    /* "View.MemoryView":1049
 *     if isinstance(memview, _memoryviewslice):
 *         to_object_func = (<_memoryviewslice> memview).to_object_func
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func             # <<<<<<<<<<<<<<
 *     else:
 *         to_object_func = NULL
 */
    __pyx_t_4 = ((struct __pyx_memoryviewslice_obj *)__pyx_v_memview)->to_dtype_func;
    __pyx_v_to_dtype_func = __pyx_t_4;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "View.MemoryView":1051
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 *     else:
 *         to_object_func = NULL             # <<<<<<<<<<<<<<
 *         to_dtype_func = NULL
 * 
 */
    __pyx_v_to_object_func = NULL;

    /* "View.MemoryView":1052
 *     else:
 *         to_object_func = NULL
 *         to_dtype_func = NULL             # <<<<<<<<<<<<<<
 * 
 *     return memoryview_fromslice(memviewslice[0], memview.view.ndim,
 */
    __pyx_v_to_dtype_func = NULL;
  }
  __pyx_L3:;

  /* "View.MemoryView":1054
 *         to_dtype_func = NULL
 * 
 *     return memoryview_fromslice(memviewslice[0], memview.view.ndim,             # <<<<<<<<<<<<<<
 *                                 to_object_func, to_dtype_func,
 *                                 memview.dtype_is_object)
 */
  __Pyx_XDECREF(__pyx_r);

  /* "View.MemoryView":1056
 *     return memoryview_fromslice(memviewslice[0], memview.view.ndim,
 *                                 to_object_func, to_dtype_func,
 *                                 memview.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_5 = __pyx_memoryview_fromslice((__pyx_v_memviewslice[0]), __pyx_v_memview->view.ndim, __pyx_v_to_object_func, __pyx_v_to_dtype_func, __pyx_v_memview->dtype_is_object); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":1040
 * 
 * @cname('__pyx_memoryview_copy_object_from_slice')
 * cdef memoryview_copy_from_slice(memoryview memview, __Pyx_memviewslice *memviewslice):             # <<<<<<<<<<<<<<
 *     """
 *     Create a new memoryview object from a given memoryview object and slice.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.memoryview_copy_from_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":1062
 * 
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:             # <<<<<<<<<<<<<<
 *     if arg < 0:
 *         return -arg
 */

static Py_ssize_t abs_py_ssize_t(Py_ssize_t __pyx_v_arg) {
  Py_ssize_t __pyx_r;
  int __pyx_t_1;

  /* "View.MemoryView":1063
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:
 *     if arg < 0:             # <<<<<<<<<<<<<<
 *         return -arg
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_arg < 0) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":1064
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:
 *     if arg < 0:
 *         return -arg             # <<<<<<<<<<<<<<
 *     else:
 *         return arg
 */
    __pyx_r = (-__pyx_v_arg);
    goto __pyx_L0;
  }
  /*else*/ {

    /* "View.MemoryView":1066
 *         return -arg
 *     else:
 *         return arg             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_get_best_slice_order')
 */
    __pyx_r = __pyx_v_arg;
    goto __pyx_L0;
  }

  /* "View.MemoryView":1062
 * 
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:             # <<<<<<<<<<<<<<
 *     if arg < 0:
 *         return -arg
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1069
 * 
 * @cname('__pyx_get_best_slice_order')
 * cdef char get_best_order(__Pyx_memviewslice *mslice, int ndim) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Figure out the best memory access order for a given slice.
 */

static char __pyx_get_best_slice_order(__Pyx_memviewslice *__pyx_v_mslice, int __pyx_v_ndim) {
  int __pyx_v_i;
  Py_ssize_t __pyx_v_c_stride;
  Py_ssize_t __pyx_v_f_stride;
  char __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "View.MemoryView":1074
 *     """
 *     cdef int i
 *     cdef Py_ssize_t c_stride = 0             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t f_stride = 0
 * 
 */
  __pyx_v_c_stride = 0;

  /* "View.MemoryView":1075
 *     cdef int i
 *     cdef Py_ssize_t c_stride = 0
 *     cdef Py_ssize_t f_stride = 0             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim - 1, -1, -1):
 */
  __pyx_v_f_stride = 0;

  /* "View.MemoryView":1077
 *     cdef Py_ssize_t f_stride = 0
 * 
 *     for i in range(ndim - 1, -1, -1):             # <<<<<<<<<<<<<<
 *         if mslice.shape[i] > 1:
 *             c_stride = mslice.strides[i]
 */
  for (__pyx_t_1 = (__pyx_v_ndim - 1); __pyx_t_1 > -1; __pyx_t_1-=1) {
    __pyx_v_i = __pyx_t_1;

    /* "View.MemoryView":1078
 * 
 *     for i in range(ndim - 1, -1, -1):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             c_stride = mslice.strides[i]
 *             break
 */
    __pyx_t_2 = (((__pyx_v_mslice->shape[__pyx_v_i]) > 1) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1079
 *     for i in range(ndim - 1, -1, -1):
 *         if mslice.shape[i] > 1:
 *             c_stride = mslice.strides[i]             # <<<<<<<<<<<<<<
 *             break
 * 
 */
      __pyx_v_c_stride = (__pyx_v_mslice->strides[__pyx_v_i]);

      /* "View.MemoryView":1080
 *         if mslice.shape[i] > 1:
 *             c_stride = mslice.strides[i]
 *             break             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim):
 */
      goto __pyx_L4_break;
    }
  }
  __pyx_L4_break:;

  /* "View.MemoryView":1082
 *             break
 * 
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         if mslice.shape[i] > 1:
 *             f_stride = mslice.strides[i]
 */
  __pyx_t_1 = __pyx_v_ndim;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_1; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "View.MemoryView":1083
 * 
 *     for i in range(ndim):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             f_stride = mslice.strides[i]
 *             break
 */
    __pyx_t_2 = (((__pyx_v_mslice->shape[__pyx_v_i]) > 1) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1084
 *     for i in range(ndim):
 *         if mslice.shape[i] > 1:
 *             f_stride = mslice.strides[i]             # <<<<<<<<<<<<<<
 *             break
 * 
 */
      __pyx_v_f_stride = (__pyx_v_mslice->strides[__pyx_v_i]);

      /* "View.MemoryView":1085
 *         if mslice.shape[i] > 1:
 *             f_stride = mslice.strides[i]
 *             break             # <<<<<<<<<<<<<<
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):
 */
      goto __pyx_L7_break;
    }
  }
  __pyx_L7_break:;

  /* "View.MemoryView":1087
 *             break
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):             # <<<<<<<<<<<<<<
 *         return 'C'
 *     else:
 */
  __pyx_t_2 = ((abs_py_ssize_t(__pyx_v_c_stride) <= abs_py_ssize_t(__pyx_v_f_stride)) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1088
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):
 *         return 'C'             # <<<<<<<<<<<<<<
 *     else:
 *         return 'F'
 */
    __pyx_r = 'C';
    goto __pyx_L0;
  }
  /*else*/ {

    /* "View.MemoryView":1090
 *         return 'C'
 *     else:
 *         return 'F'             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
    __pyx_r = 'F';
    goto __pyx_L0;
  }

  /* "View.MemoryView":1069
 * 
 * @cname('__pyx_get_best_slice_order')
 * cdef char get_best_order(__Pyx_memviewslice *mslice, int ndim) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Figure out the best memory access order for a given slice.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1093
 * 
 * @cython.cdivision(True)
 * cdef void _copy_strided_to_strided(char *src_data, Py_ssize_t *src_strides,             # <<<<<<<<<<<<<<
 *                                    char *dst_data, Py_ssize_t *dst_strides,
 *                                    Py_ssize_t *src_shape, Py_ssize_t *dst_shape,
 */

static void _copy_strided_to_strided(char *__pyx_v_src_data, Py_ssize_t *__pyx_v_src_strides, char *__pyx_v_dst_data, Py_ssize_t *__pyx_v_dst_strides, Py_ssize_t *__pyx_v_src_shape, Py_ssize_t *__pyx_v_dst_shape, int __pyx_v_ndim, size_t __pyx_v_itemsize) {
  CYTHON_UNUSED Py_ssize_t __pyx_v_i;
  CYTHON_UNUSED Py_ssize_t __pyx_v_src_extent;
  Py_ssize_t __pyx_v_dst_extent;
  Py_ssize_t __pyx_v_src_stride;
  Py_ssize_t __pyx_v_dst_stride;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;

  /* "View.MemoryView":1100
 * 
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t src_extent = src_shape[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t dst_extent = dst_shape[0]
 *     cdef Py_ssize_t src_stride = src_strides[0]
 */
  __pyx_v_src_extent = (__pyx_v_src_shape[0]);

  /* "View.MemoryView":1101
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t src_extent = src_shape[0]
 *     cdef Py_ssize_t dst_extent = dst_shape[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t src_stride = src_strides[0]
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 */
  __pyx_v_dst_extent = (__pyx_v_dst_shape[0]);

  /* "View.MemoryView":1102
 *     cdef Py_ssize_t src_extent = src_shape[0]
 *     cdef Py_ssize_t dst_extent = dst_shape[0]
 *     cdef Py_ssize_t src_stride = src_strides[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 * 
 */
  __pyx_v_src_stride = (__pyx_v_src_strides[0]);

  /* "View.MemoryView":1103
 *     cdef Py_ssize_t dst_extent = dst_shape[0]
 *     cdef Py_ssize_t src_stride = src_strides[0]
 *     cdef Py_ssize_t dst_stride = dst_strides[0]             # <<<<<<<<<<<<<<
 * 
 *     if ndim == 1:
 */
  __pyx_v_dst_stride = (__pyx_v_dst_strides[0]);

  /* "View.MemoryView":1105
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *        if (src_stride > 0 and dst_stride > 0 and
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 */
  __pyx_t_1 = ((__pyx_v_ndim == 1) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":1106
 * 
 *     if ndim == 1:
 *        if (src_stride > 0 and dst_stride > 0 and             # <<<<<<<<<<<<<<
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 */
    __pyx_t_2 = ((__pyx_v_src_stride > 0) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_dst_stride > 0) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }

    /* "View.MemoryView":1107
 *     if ndim == 1:
 *        if (src_stride > 0 and dst_stride > 0 and
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):             # <<<<<<<<<<<<<<
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 *        else:
 */
    __pyx_t_2 = (((size_t)__pyx_v_src_stride) == __pyx_v_itemsize);
    if (__pyx_t_2) {
      __pyx_t_2 = (__pyx_v_itemsize == ((size_t)__pyx_v_dst_stride));
    }
    __pyx_t_3 = (__pyx_t_2 != 0);
    __pyx_t_1 = __pyx_t_3;
    __pyx_L5_bool_binop_done:;
    if (__pyx_t_1) {

      /* "View.MemoryView":1108
 *        if (src_stride > 0 and dst_stride > 0 and
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 *            memcpy(dst_data, src_data, itemsize * dst_extent)             # <<<<<<<<<<<<<<
 *        else:
 *            for i in range(dst_extent):
 */
      memcpy(__pyx_v_dst_data, __pyx_v_src_data, (__pyx_v_itemsize * __pyx_v_dst_extent));
      goto __pyx_L4;
    }
    /*else*/ {

      /* "View.MemoryView":1110
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 *        else:
 *            for i in range(dst_extent):             # <<<<<<<<<<<<<<
 *                memcpy(dst_data, src_data, itemsize)
 *                src_data += src_stride
 */
      __pyx_t_4 = __pyx_v_dst_extent;
      for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
        __pyx_v_i = __pyx_t_5;

        /* "View.MemoryView":1111
 *        else:
 *            for i in range(dst_extent):
 *                memcpy(dst_data, src_data, itemsize)             # <<<<<<<<<<<<<<
 *                src_data += src_stride
 *                dst_data += dst_stride
 */
        memcpy(__pyx_v_dst_data, __pyx_v_src_data, __pyx_v_itemsize);

        /* "View.MemoryView":1112
 *            for i in range(dst_extent):
 *                memcpy(dst_data, src_data, itemsize)
 *                src_data += src_stride             # <<<<<<<<<<<<<<
 *                dst_data += dst_stride
 *     else:
 */
        __pyx_v_src_data = (__pyx_v_src_data + __pyx_v_src_stride);

        /* "View.MemoryView":1113
 *                memcpy(dst_data, src_data, itemsize)
 *                src_data += src_stride
 *                dst_data += dst_stride             # <<<<<<<<<<<<<<
 *     else:
 *         for i in range(dst_extent):
 */
        __pyx_v_dst_data = (__pyx_v_dst_data + __pyx_v_dst_stride);
      }
    }
    __pyx_L4:;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "View.MemoryView":1115
 *                dst_data += dst_stride
 *     else:
 *         for i in range(dst_extent):             # <<<<<<<<<<<<<<
 *             _copy_strided_to_strided(src_data, src_strides + 1,
 *                                      dst_data, dst_strides + 1,
 */
    __pyx_t_4 = __pyx_v_dst_extent;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_i = __pyx_t_5;

      /* "View.MemoryView":1116
 *     else:
 *         for i in range(dst_extent):
 *             _copy_strided_to_strided(src_data, src_strides + 1,             # <<<<<<<<<<<<<<
 *                                      dst_data, dst_strides + 1,
 *                                      src_shape + 1, dst_shape + 1,
 */
      _copy_strided_to_strided(__pyx_v_src_data, (__pyx_v_src_strides + 1), __pyx_v_dst_data, (__pyx_v_dst_strides + 1), (__pyx_v_src_shape + 1), (__pyx_v_dst_shape + 1), (__pyx_v_ndim - 1), __pyx_v_itemsize);

      /* "View.MemoryView":1120
 *                                      src_shape + 1, dst_shape + 1,
 *                                      ndim - 1, itemsize)
 *             src_data += src_stride             # <<<<<<<<<<<<<<
 *             dst_data += dst_stride
 * 
 */
      __pyx_v_src_data = (__pyx_v_src_data + __pyx_v_src_stride);

      /* "View.MemoryView":1121
 *                                      ndim - 1, itemsize)
 *             src_data += src_stride
 *             dst_data += dst_stride             # <<<<<<<<<<<<<<
 * 
 * cdef void copy_strided_to_strided(__Pyx_memviewslice *src,
 */
      __pyx_v_dst_data = (__pyx_v_dst_data + __pyx_v_dst_stride);
    }
  }
  __pyx_L3:;

  /* "View.MemoryView":1093
 * 
 * @cython.cdivision(True)
 * cdef void _copy_strided_to_strided(char *src_data, Py_ssize_t *src_strides,             # <<<<<<<<<<<<<<
 *                                    char *dst_data, Py_ssize_t *dst_strides,
 *                                    Py_ssize_t *src_shape, Py_ssize_t *dst_shape,
 */

  /* function exit code */
}

/* "View.MemoryView":1123
 *             dst_data += dst_stride
 * 
 * cdef void copy_strided_to_strided(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice *dst,
 *                                   int ndim, size_t itemsize) nogil:
 */

static void copy_strided_to_strided(__Pyx_memviewslice *__pyx_v_src, __Pyx_memviewslice *__pyx_v_dst, int __pyx_v_ndim, size_t __pyx_v_itemsize) {

  /* "View.MemoryView":1126
 *                                   __Pyx_memviewslice *dst,
 *                                   int ndim, size_t itemsize) nogil:
 *     _copy_strided_to_strided(src.data, src.strides, dst.data, dst.strides,             # <<<<<<<<<<<<<<
 *                              src.shape, dst.shape, ndim, itemsize)
 * 
 */
  _copy_strided_to_strided(__pyx_v_src->data, __pyx_v_src->strides, __pyx_v_dst->data, __pyx_v_dst->strides, __pyx_v_src->shape, __pyx_v_dst->shape, __pyx_v_ndim, __pyx_v_itemsize);

  /* "View.MemoryView":1123
 *             dst_data += dst_stride
 * 
 * cdef void copy_strided_to_strided(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice *dst,
 *                                   int ndim, size_t itemsize) nogil:
 */

  /* function exit code */
}

/* "View.MemoryView":1130
 * 
 * @cname('__pyx_memoryview_slice_get_size')
 * cdef Py_ssize_t slice_get_size(__Pyx_memviewslice *src, int ndim) nogil:             # <<<<<<<<<<<<<<
 *     "Return the size of the memory occupied by the slice in number of bytes"
 *     cdef int i
 */

static Py_ssize_t __pyx_memoryview_slice_get_size(__Pyx_memviewslice *__pyx_v_src, int __pyx_v_ndim) {
  int __pyx_v_i;
  Py_ssize_t __pyx_v_size;
  Py_ssize_t __pyx_r;
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "View.MemoryView":1133
 *     "Return the size of the memory occupied by the slice in number of bytes"
 *     cdef int i
 *     cdef Py_ssize_t size = src.memview.view.itemsize             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim):
 */
  __pyx_t_1 = __pyx_v_src->memview->view.itemsize;
  __pyx_v_size = __pyx_t_1;

  /* "View.MemoryView":1135
 *     cdef Py_ssize_t size = src.memview.view.itemsize
 * 
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         size *= src.shape[i]
 * 
 */
  __pyx_t_2 = __pyx_v_ndim;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "View.MemoryView":1136
 * 
 *     for i in range(ndim):
 *         size *= src.shape[i]             # <<<<<<<<<<<<<<
 * 
 *     return size
 */
    __pyx_v_size = (__pyx_v_size * (__pyx_v_src->shape[__pyx_v_i]));
  }

  /* "View.MemoryView":1138
 *         size *= src.shape[i]
 * 
 *     return size             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_fill_contig_strides_array')
 */
  __pyx_r = __pyx_v_size;
  goto __pyx_L0;

  /* "View.MemoryView":1130
 * 
 * @cname('__pyx_memoryview_slice_get_size')
 * cdef Py_ssize_t slice_get_size(__Pyx_memviewslice *src, int ndim) nogil:             # <<<<<<<<<<<<<<
 *     "Return the size of the memory occupied by the slice in number of bytes"
 *     cdef int i
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1141
 * 
 * @cname('__pyx_fill_contig_strides_array')
 * cdef Py_ssize_t fill_contig_strides_array(             # <<<<<<<<<<<<<<
 *                 Py_ssize_t *shape, Py_ssize_t *strides, Py_ssize_t stride,
 *                 int ndim, char order) nogil:
 */

static Py_ssize_t __pyx_fill_contig_strides_array(Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, Py_ssize_t __pyx_v_stride, int __pyx_v_ndim, char __pyx_v_order) {
  int __pyx_v_idx;
  Py_ssize_t __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "View.MemoryView":1150
 *     cdef int idx
 * 
 *     if order == 'F':             # <<<<<<<<<<<<<<
 *         for idx in range(ndim):
 *             strides[idx] = stride
 */
  __pyx_t_1 = ((__pyx_v_order == 'F') != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":1151
 * 
 *     if order == 'F':
 *         for idx in range(ndim):             # <<<<<<<<<<<<<<
 *             strides[idx] = stride
 *             stride = stride * shape[idx]
 */
    __pyx_t_2 = __pyx_v_ndim;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_idx = __pyx_t_3;

      /* "View.MemoryView":1152
 *     if order == 'F':
 *         for idx in range(ndim):
 *             strides[idx] = stride             # <<<<<<<<<<<<<<
 *             stride = stride * shape[idx]
 *     else:
 */
      (__pyx_v_strides[__pyx_v_idx]) = __pyx_v_stride;

      /* "View.MemoryView":1153
 *         for idx in range(ndim):
 *             strides[idx] = stride
 *             stride = stride * shape[idx]             # <<<<<<<<<<<<<<
 *     else:
 *         for idx in range(ndim - 1, -1, -1):
 */
      __pyx_v_stride = (__pyx_v_stride * (__pyx_v_shape[__pyx_v_idx]));
    }
    goto __pyx_L3;
  }
  /*else*/ {

    /* "View.MemoryView":1155
 *             stride = stride * shape[idx]
 *     else:
 *         for idx in range(ndim - 1, -1, -1):             # <<<<<<<<<<<<<<
 *             strides[idx] = stride
 *             stride = stride * shape[idx]
 */
    for (__pyx_t_2 = (__pyx_v_ndim - 1); __pyx_t_2 > -1; __pyx_t_2-=1) {
      __pyx_v_idx = __pyx_t_2;

      /* "View.MemoryView":1156
 *     else:
 *         for idx in range(ndim - 1, -1, -1):
 *             strides[idx] = stride             # <<<<<<<<<<<<<<
 *             stride = stride * shape[idx]
 * 
 */
      (__pyx_v_strides[__pyx_v_idx]) = __pyx_v_stride;

      /* "View.MemoryView":1157
 *         for idx in range(ndim - 1, -1, -1):
 *             strides[idx] = stride
 *             stride = stride * shape[idx]             # <<<<<<<<<<<<<<
 * 
 *     return stride
 */
      __pyx_v_stride = (__pyx_v_stride * (__pyx_v_shape[__pyx_v_idx]));
    }
  }
  __pyx_L3:;

  /* "View.MemoryView":1159
 *             stride = stride * shape[idx]
 * 
 *     return stride             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_data_to_temp')
 */
  __pyx_r = __pyx_v_stride;
  goto __pyx_L0;

  /* "View.MemoryView":1141
 * 
 * @cname('__pyx_fill_contig_strides_array')
 * cdef Py_ssize_t fill_contig_strides_array(             # <<<<<<<<<<<<<<
 *                 Py_ssize_t *shape, Py_ssize_t *strides, Py_ssize_t stride,
 *                 int ndim, char order) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1162
 * 
 * @cname('__pyx_memoryview_copy_data_to_temp')
 * cdef void *copy_data_to_temp(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                              __Pyx_memviewslice *tmpslice,
 *                              char order,
 */

static void *__pyx_memoryview_copy_data_to_temp(__Pyx_memviewslice *__pyx_v_src, __Pyx_memviewslice *__pyx_v_tmpslice, char __pyx_v_order, int __pyx_v_ndim) {
  int __pyx_v_i;
  void *__pyx_v_result;
  size_t __pyx_v_itemsize;
  size_t __pyx_v_size;
  void *__pyx_r;
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  struct __pyx_memoryview_obj *__pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "View.MemoryView":1173
 *     cdef void *result
 * 
 *     cdef size_t itemsize = src.memview.view.itemsize             # <<<<<<<<<<<<<<
 *     cdef size_t size = slice_get_size(src, ndim)
 * 
 */
  __pyx_t_1 = __pyx_v_src->memview->view.itemsize;
  __pyx_v_itemsize = __pyx_t_1;

  /* "View.MemoryView":1174
 * 
 *     cdef size_t itemsize = src.memview.view.itemsize
 *     cdef size_t size = slice_get_size(src, ndim)             # <<<<<<<<<<<<<<
 * 
 *     result = malloc(size)
 */
  __pyx_v_size = __pyx_memoryview_slice_get_size(__pyx_v_src, __pyx_v_ndim);

  /* "View.MemoryView":1176
 *     cdef size_t size = slice_get_size(src, ndim)
 * 
 *     result = malloc(size)             # <<<<<<<<<<<<<<
 *     if not result:
 *         _err(MemoryError, NULL)
 */
  __pyx_v_result = malloc(__pyx_v_size);

  /* "View.MemoryView":1177
 * 
 *     result = malloc(size)
 *     if not result:             # <<<<<<<<<<<<<<
 *         _err(MemoryError, NULL)
 * 
 */
  __pyx_t_2 = ((!(__pyx_v_result != 0)) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1178
 *     result = malloc(size)
 *     if not result:
 *         _err(MemoryError, NULL)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_3 = __pyx_memoryview_err(__pyx_builtin_MemoryError, NULL); if (unlikely(__pyx_t_3 == -1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "View.MemoryView":1181
 * 
 * 
 *     tmpslice.data = <char *> result             # <<<<<<<<<<<<<<
 *     tmpslice.memview = src.memview
 *     for i in range(ndim):
 */
  __pyx_v_tmpslice->data = ((char *)__pyx_v_result);

  /* "View.MemoryView":1182
 * 
 *     tmpslice.data = <char *> result
 *     tmpslice.memview = src.memview             # <<<<<<<<<<<<<<
 *     for i in range(ndim):
 *         tmpslice.shape[i] = src.shape[i]
 */
  __pyx_t_4 = __pyx_v_src->memview;
  __pyx_v_tmpslice->memview = __pyx_t_4;

  /* "View.MemoryView":1183
 *     tmpslice.data = <char *> result
 *     tmpslice.memview = src.memview
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         tmpslice.shape[i] = src.shape[i]
 *         tmpslice.suboffsets[i] = -1
 */
  __pyx_t_3 = __pyx_v_ndim;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_3; __pyx_t_5+=1) {
    __pyx_v_i = __pyx_t_5;

    /* "View.MemoryView":1184
 *     tmpslice.memview = src.memview
 *     for i in range(ndim):
 *         tmpslice.shape[i] = src.shape[i]             # <<<<<<<<<<<<<<
 *         tmpslice.suboffsets[i] = -1
 * 
 */
    (__pyx_v_tmpslice->shape[__pyx_v_i]) = (__pyx_v_src->shape[__pyx_v_i]);

    /* "View.MemoryView":1185
 *     for i in range(ndim):
 *         tmpslice.shape[i] = src.shape[i]
 *         tmpslice.suboffsets[i] = -1             # <<<<<<<<<<<<<<
 * 
 *     fill_contig_strides_array(&tmpslice.shape[0], &tmpslice.strides[0], itemsize,
 */
    (__pyx_v_tmpslice->suboffsets[__pyx_v_i]) = -1;
  }

  /* "View.MemoryView":1187
 *         tmpslice.suboffsets[i] = -1
 * 
 *     fill_contig_strides_array(&tmpslice.shape[0], &tmpslice.strides[0], itemsize,             # <<<<<<<<<<<<<<
 *                               ndim, order)
 * 
 */
  __pyx_fill_contig_strides_array((&(__pyx_v_tmpslice->shape[0])), (&(__pyx_v_tmpslice->strides[0])), __pyx_v_itemsize, __pyx_v_ndim, __pyx_v_order);

  /* "View.MemoryView":1191
 * 
 * 
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         if tmpslice.shape[i] == 1:
 *             tmpslice.strides[i] = 0
 */
  __pyx_t_3 = __pyx_v_ndim;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_3; __pyx_t_5+=1) {
    __pyx_v_i = __pyx_t_5;

    /* "View.MemoryView":1192
 * 
 *     for i in range(ndim):
 *         if tmpslice.shape[i] == 1:             # <<<<<<<<<<<<<<
 *             tmpslice.strides[i] = 0
 * 
 */
    __pyx_t_2 = (((__pyx_v_tmpslice->shape[__pyx_v_i]) == 1) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1193
 *     for i in range(ndim):
 *         if tmpslice.shape[i] == 1:
 *             tmpslice.strides[i] = 0             # <<<<<<<<<<<<<<
 * 
 *     if slice_is_contig(src, order, ndim):
 */
      (__pyx_v_tmpslice->strides[__pyx_v_i]) = 0;
      goto __pyx_L8;
    }
    __pyx_L8:;
  }

  /* "View.MemoryView":1195
 *             tmpslice.strides[i] = 0
 * 
 *     if slice_is_contig(src, order, ndim):             # <<<<<<<<<<<<<<
 *         memcpy(result, src.data, size)
 *     else:
 */
  __pyx_t_2 = (__pyx_memviewslice_is_contig(__pyx_v_src, __pyx_v_order, __pyx_v_ndim) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1196
 * 
 *     if slice_is_contig(src, order, ndim):
 *         memcpy(result, src.data, size)             # <<<<<<<<<<<<<<
 *     else:
 *         copy_strided_to_strided(src, tmpslice, ndim, itemsize)
 */
    memcpy(__pyx_v_result, __pyx_v_src->data, __pyx_v_size);
    goto __pyx_L9;
  }
  /*else*/ {

    /* "View.MemoryView":1198
 *         memcpy(result, src.data, size)
 *     else:
 *         copy_strided_to_strided(src, tmpslice, ndim, itemsize)             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
    copy_strided_to_strided(__pyx_v_src, __pyx_v_tmpslice, __pyx_v_ndim, __pyx_v_itemsize);
  }
  __pyx_L9:;

  /* "View.MemoryView":1200
 *         copy_strided_to_strided(src, tmpslice, ndim, itemsize)
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "View.MemoryView":1162
 * 
 * @cname('__pyx_memoryview_copy_data_to_temp')
 * cdef void *copy_data_to_temp(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                              __Pyx_memviewslice *tmpslice,
 *                              char order,
 */

  /* function exit code */
  __pyx_L1_error:;
  {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
    #endif
    __Pyx_AddTraceback("View.MemoryView.copy_data_to_temp", __pyx_clineno, __pyx_lineno, __pyx_filename);
    #ifdef WITH_THREAD
    PyGILState_Release(__pyx_gilstate_save);
    #endif
  }
  __pyx_r = NULL;
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1205
 * 
 * @cname('__pyx_memoryview_err_extents')
 * cdef int _err_extents(int i, Py_ssize_t extent1,             # <<<<<<<<<<<<<<
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError("got differing extents in dimension %d (got %d and %d)" %
 */

static int __pyx_memoryview_err_extents(int __pyx_v_i, Py_ssize_t __pyx_v_extent1, Py_ssize_t __pyx_v_extent2) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("_err_extents", 0);

  /* "View.MemoryView":1208
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError("got differing extents in dimension %d (got %d and %d)" %
 *                                                         (i, extent1, extent2))             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_err_dim')
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_i); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1208; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_extent1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1208; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_extent2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1208; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1208; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;

  /* "View.MemoryView":1207
 * cdef int _err_extents(int i, Py_ssize_t extent1,
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError("got differing extents in dimension %d (got %d and %d)" %             # <<<<<<<<<<<<<<
 *                                                         (i, extent1, extent2))
 * 
 */
  __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_got_differing_extents_in_dimensi, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1207; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1207; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1207; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_Raise(__pyx_t_3, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1207; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "View.MemoryView":1205
 * 
 * @cname('__pyx_memoryview_err_extents')
 * cdef int _err_extents(int i, Py_ssize_t extent1,             # <<<<<<<<<<<<<<
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError("got differing extents in dimension %d (got %d and %d)" %
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("View.MemoryView._err_extents", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "View.MemoryView":1211
 * 
 * @cname('__pyx_memoryview_err_dim')
 * cdef int _err_dim(object error, char *msg, int dim) except -1 with gil:             # <<<<<<<<<<<<<<
 *     raise error(msg.decode('ascii') % dim)
 * 
 */

static int __pyx_memoryview_err_dim(PyObject *__pyx_v_error, char *__pyx_v_msg, int __pyx_v_dim) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("_err_dim", 0);
  __Pyx_INCREF(__pyx_v_error);

  /* "View.MemoryView":1212
 * @cname('__pyx_memoryview_err_dim')
 * cdef int _err_dim(object error, char *msg, int dim) except -1 with gil:
 *     raise error(msg.decode('ascii') % dim)             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_err')
 */
  __pyx_t_2 = __Pyx_decode_c_string(__pyx_v_msg, 0, strlen(__pyx_v_msg), NULL, NULL, PyUnicode_DecodeASCII); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1212; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_dim); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1212; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyUnicode_Format(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1212; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_INCREF(__pyx_v_error);
  __pyx_t_3 = __pyx_v_error; __pyx_t_2 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1212; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1212; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2); __Pyx_GIVEREF(__pyx_t_2); __pyx_t_2 = NULL;
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1212; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1212; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "View.MemoryView":1211
 * 
 * @cname('__pyx_memoryview_err_dim')
 * cdef int _err_dim(object error, char *msg, int dim) except -1 with gil:             # <<<<<<<<<<<<<<
 *     raise error(msg.decode('ascii') % dim)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView._err_dim", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __Pyx_XDECREF(__pyx_v_error);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "View.MemoryView":1215
 * 
 * @cname('__pyx_memoryview_err')
 * cdef int _err(object error, char *msg) except -1 with gil:             # <<<<<<<<<<<<<<
 *     if msg != NULL:
 *         raise error(msg.decode('ascii'))
 */

static int __pyx_memoryview_err(PyObject *__pyx_v_error, char *__pyx_v_msg) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("_err", 0);
  __Pyx_INCREF(__pyx_v_error);

  /* "View.MemoryView":1216
 * @cname('__pyx_memoryview_err')
 * cdef int _err(object error, char *msg) except -1 with gil:
 *     if msg != NULL:             # <<<<<<<<<<<<<<
 *         raise error(msg.decode('ascii'))
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_msg != NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":1217
 * cdef int _err(object error, char *msg) except -1 with gil:
 *     if msg != NULL:
 *         raise error(msg.decode('ascii'))             # <<<<<<<<<<<<<<
 *     else:
 *         raise error
 */
    __pyx_t_3 = __Pyx_decode_c_string(__pyx_v_msg, 0, strlen(__pyx_v_msg), NULL, NULL, PyUnicode_DecodeASCII); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1217; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_error);
    __pyx_t_4 = __pyx_v_error; __pyx_t_5 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1217; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1217; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __Pyx_GIVEREF(__pyx_t_5); __pyx_t_5 = NULL;
      PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1217; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1217; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {

    /* "View.MemoryView":1219
 *         raise error(msg.decode('ascii'))
 *     else:
 *         raise error             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_contents')
 */
    __Pyx_Raise(__pyx_v_error, 0, 0, 0);
    {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "View.MemoryView":1215
 * 
 * @cname('__pyx_memoryview_err')
 * cdef int _err(object error, char *msg) except -1 with gil:             # <<<<<<<<<<<<<<
 *     if msg != NULL:
 *         raise error(msg.decode('ascii'))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("View.MemoryView._err", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __Pyx_XDECREF(__pyx_v_error);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "View.MemoryView":1222
 * 
 * @cname('__pyx_memoryview_copy_contents')
 * cdef int memoryview_copy_contents(__Pyx_memviewslice src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice dst,
 *                                   int src_ndim, int dst_ndim,
 */

static int __pyx_memoryview_copy_contents(__Pyx_memviewslice __pyx_v_src, __Pyx_memviewslice __pyx_v_dst, int __pyx_v_src_ndim, int __pyx_v_dst_ndim, int __pyx_v_dtype_is_object) {
  void *__pyx_v_tmpdata;
  size_t __pyx_v_itemsize;
  int __pyx_v_i;
  char __pyx_v_order;
  int __pyx_v_broadcasting;
  int __pyx_v_direct_copy;
  __Pyx_memviewslice __pyx_v_tmp;
  int __pyx_v_ndim;
  int __pyx_r;
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  void *__pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "View.MemoryView":1230
 *     Check for overlapping memory and verify the shapes.
 *     """
 *     cdef void *tmpdata = NULL             # <<<<<<<<<<<<<<
 *     cdef size_t itemsize = src.memview.view.itemsize
 *     cdef int i
 */
  __pyx_v_tmpdata = NULL;

  /* "View.MemoryView":1231
 *     """
 *     cdef void *tmpdata = NULL
 *     cdef size_t itemsize = src.memview.view.itemsize             # <<<<<<<<<<<<<<
 *     cdef int i
 *     cdef char order = get_best_order(&src, src_ndim)
 */
  __pyx_t_1 = __pyx_v_src.memview->view.itemsize;
  __pyx_v_itemsize = __pyx_t_1;

  /* "View.MemoryView":1233
 *     cdef size_t itemsize = src.memview.view.itemsize
 *     cdef int i
 *     cdef char order = get_best_order(&src, src_ndim)             # <<<<<<<<<<<<<<
 *     cdef bint broadcasting = False
 *     cdef bint direct_copy = False
 */
  __pyx_v_order = __pyx_get_best_slice_order((&__pyx_v_src), __pyx_v_src_ndim);

  /* "View.MemoryView":1234
 *     cdef int i
 *     cdef char order = get_best_order(&src, src_ndim)
 *     cdef bint broadcasting = False             # <<<<<<<<<<<<<<
 *     cdef bint direct_copy = False
 *     cdef __Pyx_memviewslice tmp
 */
  __pyx_v_broadcasting = 0;

  /* "View.MemoryView":1235
 *     cdef char order = get_best_order(&src, src_ndim)
 *     cdef bint broadcasting = False
 *     cdef bint direct_copy = False             # <<<<<<<<<<<<<<
 *     cdef __Pyx_memviewslice tmp
 * 
 */
  __pyx_v_direct_copy = 0;

  /* "View.MemoryView":1238
 *     cdef __Pyx_memviewslice tmp
 * 
 *     if src_ndim < dst_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:
 */
  __pyx_t_2 = ((__pyx_v_src_ndim < __pyx_v_dst_ndim) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1239
 * 
 *     if src_ndim < dst_ndim:
 *         broadcast_leading(&src, src_ndim, dst_ndim)             # <<<<<<<<<<<<<<
 *     elif dst_ndim < src_ndim:
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 */
    __pyx_memoryview_broadcast_leading((&__pyx_v_src), __pyx_v_src_ndim, __pyx_v_dst_ndim);
    goto __pyx_L3;
  }

  /* "View.MemoryView":1240
 *     if src_ndim < dst_ndim:
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 * 
 */
  __pyx_t_2 = ((__pyx_v_dst_ndim < __pyx_v_src_ndim) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1241
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:
 *         broadcast_leading(&dst, dst_ndim, src_ndim)             # <<<<<<<<<<<<<<
 * 
 *     cdef int ndim = max(src_ndim, dst_ndim)
 */
    __pyx_memoryview_broadcast_leading((&__pyx_v_dst), __pyx_v_dst_ndim, __pyx_v_src_ndim);
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "View.MemoryView":1243
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 * 
 *     cdef int ndim = max(src_ndim, dst_ndim)             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim):
 */
  __pyx_t_3 = __pyx_v_dst_ndim;
  __pyx_t_4 = __pyx_v_src_ndim;
  if (((__pyx_t_3 > __pyx_t_4) != 0)) {
    __pyx_t_5 = __pyx_t_3;
  } else {
    __pyx_t_5 = __pyx_t_4;
  }
  __pyx_v_ndim = __pyx_t_5;

  /* "View.MemoryView":1245
 *     cdef int ndim = max(src_ndim, dst_ndim)
 * 
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:
 */
  __pyx_t_5 = __pyx_v_ndim;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_5; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "View.MemoryView":1246
 * 
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:             # <<<<<<<<<<<<<<
 *             if src.shape[i] == 1:
 *                 broadcasting = True
 */
    __pyx_t_2 = (((__pyx_v_src.shape[__pyx_v_i]) != (__pyx_v_dst.shape[__pyx_v_i])) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1247
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:             # <<<<<<<<<<<<<<
 *                 broadcasting = True
 *                 src.strides[i] = 0
 */
      __pyx_t_2 = (((__pyx_v_src.shape[__pyx_v_i]) == 1) != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":1248
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:
 *                 broadcasting = True             # <<<<<<<<<<<<<<
 *                 src.strides[i] = 0
 *             else:
 */
        __pyx_v_broadcasting = 1;

        /* "View.MemoryView":1249
 *             if src.shape[i] == 1:
 *                 broadcasting = True
 *                 src.strides[i] = 0             # <<<<<<<<<<<<<<
 *             else:
 *                 _err_extents(i, dst.shape[i], src.shape[i])
 */
        (__pyx_v_src.strides[__pyx_v_i]) = 0;
        goto __pyx_L7;
      }
      /*else*/ {

        /* "View.MemoryView":1251
 *                 src.strides[i] = 0
 *             else:
 *                 _err_extents(i, dst.shape[i], src.shape[i])             # <<<<<<<<<<<<<<
 * 
 *         if src.suboffsets[i] >= 0:
 */
        __pyx_t_4 = __pyx_memoryview_err_extents(__pyx_v_i, (__pyx_v_dst.shape[__pyx_v_i]), (__pyx_v_src.shape[__pyx_v_i])); if (unlikely(__pyx_t_4 == -1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1251; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_L7:;
      goto __pyx_L6;
    }
    __pyx_L6:;

    /* "View.MemoryView":1253
 *                 _err_extents(i, dst.shape[i], src.shape[i])
 * 
 *         if src.suboffsets[i] >= 0:             # <<<<<<<<<<<<<<
 *             _err_dim(ValueError, "Dimension %d is not direct", i)
 * 
 */
    __pyx_t_2 = (((__pyx_v_src.suboffsets[__pyx_v_i]) >= 0) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1254
 * 
 *         if src.suboffsets[i] >= 0:
 *             _err_dim(ValueError, "Dimension %d is not direct", i)             # <<<<<<<<<<<<<<
 * 
 *     if slices_overlap(&src, &dst, ndim, itemsize):
 */
      __pyx_t_4 = __pyx_memoryview_err_dim(__pyx_builtin_ValueError, __pyx_k_Dimension_d_is_not_direct, __pyx_v_i); if (unlikely(__pyx_t_4 == -1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1254; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      goto __pyx_L8;
    }
    __pyx_L8:;
  }

  /* "View.MemoryView":1256
 *             _err_dim(ValueError, "Dimension %d is not direct", i)
 * 
 *     if slices_overlap(&src, &dst, ndim, itemsize):             # <<<<<<<<<<<<<<
 * 
 *         if not slice_is_contig(&src, order, ndim):
 */
  __pyx_t_2 = (__pyx_slices_overlap((&__pyx_v_src), (&__pyx_v_dst), __pyx_v_ndim, __pyx_v_itemsize) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1258
 *     if slices_overlap(&src, &dst, ndim, itemsize):
 * 
 *         if not slice_is_contig(&src, order, ndim):             # <<<<<<<<<<<<<<
 *             order = get_best_order(&dst, ndim)
 * 
 */
    __pyx_t_2 = ((!(__pyx_memviewslice_is_contig((&__pyx_v_src), __pyx_v_order, __pyx_v_ndim) != 0)) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1259
 * 
 *         if not slice_is_contig(&src, order, ndim):
 *             order = get_best_order(&dst, ndim)             # <<<<<<<<<<<<<<
 * 
 *         tmpdata = copy_data_to_temp(&src, &tmp, order, ndim)
 */
      __pyx_v_order = __pyx_get_best_slice_order((&__pyx_v_dst), __pyx_v_ndim);
      goto __pyx_L10;
    }
    __pyx_L10:;

    /* "View.MemoryView":1261
 *             order = get_best_order(&dst, ndim)
 * 
 *         tmpdata = copy_data_to_temp(&src, &tmp, order, ndim)             # <<<<<<<<<<<<<<
 *         src = tmp
 * 
 */
    __pyx_t_6 = __pyx_memoryview_copy_data_to_temp((&__pyx_v_src), (&__pyx_v_tmp), __pyx_v_order, __pyx_v_ndim); if (unlikely(__pyx_t_6 == NULL)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1261; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_tmpdata = __pyx_t_6;

    /* "View.MemoryView":1262
 * 
 *         tmpdata = copy_data_to_temp(&src, &tmp, order, ndim)
 *         src = tmp             # <<<<<<<<<<<<<<
 * 
 *     if not broadcasting:
 */
    __pyx_v_src = __pyx_v_tmp;
    goto __pyx_L9;
  }
  __pyx_L9:;

  /* "View.MemoryView":1264
 *         src = tmp
 * 
 *     if not broadcasting:             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = ((!(__pyx_v_broadcasting != 0)) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1267
 * 
 * 
 *         if slice_is_contig(&src, 'C', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(&dst, 'C', ndim)
 *         elif slice_is_contig(&src, 'F', ndim):
 */
    __pyx_t_2 = (__pyx_memviewslice_is_contig((&__pyx_v_src), 'C', __pyx_v_ndim) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1268
 * 
 *         if slice_is_contig(&src, 'C', ndim):
 *             direct_copy = slice_is_contig(&dst, 'C', ndim)             # <<<<<<<<<<<<<<
 *         elif slice_is_contig(&src, 'F', ndim):
 *             direct_copy = slice_is_contig(&dst, 'F', ndim)
 */
      __pyx_v_direct_copy = __pyx_memviewslice_is_contig((&__pyx_v_dst), 'C', __pyx_v_ndim);
      goto __pyx_L12;
    }

    /* "View.MemoryView":1269
 *         if slice_is_contig(&src, 'C', ndim):
 *             direct_copy = slice_is_contig(&dst, 'C', ndim)
 *         elif slice_is_contig(&src, 'F', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(&dst, 'F', ndim)
 * 
 */
    __pyx_t_2 = (__pyx_memviewslice_is_contig((&__pyx_v_src), 'F', __pyx_v_ndim) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1270
 *             direct_copy = slice_is_contig(&dst, 'C', ndim)
 *         elif slice_is_contig(&src, 'F', ndim):
 *             direct_copy = slice_is_contig(&dst, 'F', ndim)             # <<<<<<<<<<<<<<
 * 
 *         if direct_copy:
 */
      __pyx_v_direct_copy = __pyx_memviewslice_is_contig((&__pyx_v_dst), 'F', __pyx_v_ndim);
      goto __pyx_L12;
    }
    __pyx_L12:;

    /* "View.MemoryView":1272
 *             direct_copy = slice_is_contig(&dst, 'F', ndim)
 * 
 *         if direct_copy:             # <<<<<<<<<<<<<<
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, False)
 */
    __pyx_t_2 = (__pyx_v_direct_copy != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1274
 *         if direct_copy:
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, False)             # <<<<<<<<<<<<<<
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))
 *             refcount_copying(&dst, dtype_is_object, ndim, True)
 */
      __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 0);

      /* "View.MemoryView":1275
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, False)
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))             # <<<<<<<<<<<<<<
 *             refcount_copying(&dst, dtype_is_object, ndim, True)
 *             free(tmpdata)
 */
      memcpy(__pyx_v_dst.data, __pyx_v_src.data, __pyx_memoryview_slice_get_size((&__pyx_v_src), __pyx_v_ndim));

      /* "View.MemoryView":1276
 *             refcount_copying(&dst, dtype_is_object, ndim, False)
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))
 *             refcount_copying(&dst, dtype_is_object, ndim, True)             # <<<<<<<<<<<<<<
 *             free(tmpdata)
 *             return 0
 */
      __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 1);

      /* "View.MemoryView":1277
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))
 *             refcount_copying(&dst, dtype_is_object, ndim, True)
 *             free(tmpdata)             # <<<<<<<<<<<<<<
 *             return 0
 * 
 */
      free(__pyx_v_tmpdata);

      /* "View.MemoryView":1278
 *             refcount_copying(&dst, dtype_is_object, ndim, True)
 *             free(tmpdata)
 *             return 0             # <<<<<<<<<<<<<<
 * 
 *     if order == 'F' == get_best_order(&dst, ndim):
 */
      __pyx_r = 0;
      goto __pyx_L0;
    }
    goto __pyx_L11;
  }
  __pyx_L11:;

  /* "View.MemoryView":1280
 *             return 0
 * 
 *     if order == 'F' == get_best_order(&dst, ndim):             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = (__pyx_v_order == 'F');
  if (__pyx_t_2) {
    __pyx_t_2 = ('F' == __pyx_get_best_slice_order((&__pyx_v_dst), __pyx_v_ndim));
  }
  __pyx_t_7 = (__pyx_t_2 != 0);
  if (__pyx_t_7) {

    /* "View.MemoryView":1283
 * 
 * 
 *         transpose_memslice(&src)             # <<<<<<<<<<<<<<
 *         transpose_memslice(&dst)
 * 
 */
    __pyx_t_5 = __pyx_memslice_transpose((&__pyx_v_src)); if (unlikely(__pyx_t_5 == 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1283; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "View.MemoryView":1284
 * 
 *         transpose_memslice(&src)
 *         transpose_memslice(&dst)             # <<<<<<<<<<<<<<
 * 
 *     refcount_copying(&dst, dtype_is_object, ndim, False)
 */
    __pyx_t_5 = __pyx_memslice_transpose((&__pyx_v_dst)); if (unlikely(__pyx_t_5 == 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 1284; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L14;
  }
  __pyx_L14:;

  /* "View.MemoryView":1286
 *         transpose_memslice(&dst)
 * 
 *     refcount_copying(&dst, dtype_is_object, ndim, False)             # <<<<<<<<<<<<<<
 *     copy_strided_to_strided(&src, &dst, ndim, itemsize)
 *     refcount_copying(&dst, dtype_is_object, ndim, True)
 */
  __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 0);

  /* "View.MemoryView":1287
 * 
 *     refcount_copying(&dst, dtype_is_object, ndim, False)
 *     copy_strided_to_strided(&src, &dst, ndim, itemsize)             # <<<<<<<<<<<<<<
 *     refcount_copying(&dst, dtype_is_object, ndim, True)
 * 
 */
  copy_strided_to_strided((&__pyx_v_src), (&__pyx_v_dst), __pyx_v_ndim, __pyx_v_itemsize);

  /* "View.MemoryView":1288
 *     refcount_copying(&dst, dtype_is_object, ndim, False)
 *     copy_strided_to_strided(&src, &dst, ndim, itemsize)
 *     refcount_copying(&dst, dtype_is_object, ndim, True)             # <<<<<<<<<<<<<<
 * 
 *     free(tmpdata)
 */
  __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 1);

  /* "View.MemoryView":1290
 *     refcount_copying(&dst, dtype_is_object, ndim, True)
 * 
 *     free(tmpdata)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  free(__pyx_v_tmpdata);

  /* "View.MemoryView":1291
 * 
 *     free(tmpdata)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_broadcast_leading')
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "View.MemoryView":1222
 * 
 * @cname('__pyx_memoryview_copy_contents')
 * cdef int memoryview_copy_contents(__Pyx_memviewslice src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice dst,
 *                                   int src_ndim, int dst_ndim,
 */

  /* function exit code */
  __pyx_L1_error:;
  {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
    #endif
    __Pyx_AddTraceback("View.MemoryView.memoryview_copy_contents", __pyx_clineno, __pyx_lineno, __pyx_filename);
    #ifdef WITH_THREAD
    PyGILState_Release(__pyx_gilstate_save);
    #endif
  }
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1294
 * 
 * @cname('__pyx_memoryview_broadcast_leading')
 * cdef void broadcast_leading(__Pyx_memviewslice *slice,             # <<<<<<<<<<<<<<
 *                             int ndim,
 *                             int ndim_other) nogil:
 */

static void __pyx_memoryview_broadcast_leading(__Pyx_memviewslice *__pyx_v_slice, int __pyx_v_ndim, int __pyx_v_ndim_other) {
  int __pyx_v_i;
  int __pyx_v_offset;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "View.MemoryView":1298
 *                             int ndim_other) nogil:
 *     cdef int i
 *     cdef int offset = ndim_other - ndim             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim - 1, -1, -1):
 */
  __pyx_v_offset = (__pyx_v_ndim_other - __pyx_v_ndim);

  /* "View.MemoryView":1300
 *     cdef int offset = ndim_other - ndim
 * 
 *     for i in range(ndim - 1, -1, -1):             # <<<<<<<<<<<<<<
 *         slice.shape[i + offset] = slice.shape[i]
 *         slice.strides[i + offset] = slice.strides[i]
 */
  for (__pyx_t_1 = (__pyx_v_ndim - 1); __pyx_t_1 > -1; __pyx_t_1-=1) {
    __pyx_v_i = __pyx_t_1;

    /* "View.MemoryView":1301
 * 
 *     for i in range(ndim - 1, -1, -1):
 *         slice.shape[i + offset] = slice.shape[i]             # <<<<<<<<<<<<<<
 *         slice.strides[i + offset] = slice.strides[i]
 *         slice.suboffsets[i + offset] = slice.suboffsets[i]
 */
    (__pyx_v_slice->shape[(__pyx_v_i + __pyx_v_offset)]) = (__pyx_v_slice->shape[__pyx_v_i]);

    /* "View.MemoryView":1302
 *     for i in range(ndim - 1, -1, -1):
 *         slice.shape[i + offset] = slice.shape[i]
 *         slice.strides[i + offset] = slice.strides[i]             # <<<<<<<<<<<<<<
 *         slice.suboffsets[i + offset] = slice.suboffsets[i]
 * 
 */
    (__pyx_v_slice->strides[(__pyx_v_i + __pyx_v_offset)]) = (__pyx_v_slice->strides[__pyx_v_i]);

    /* "View.MemoryView":1303
 *         slice.shape[i + offset] = slice.shape[i]
 *         slice.strides[i + offset] = slice.strides[i]
 *         slice.suboffsets[i + offset] = slice.suboffsets[i]             # <<<<<<<<<<<<<<
 * 
 *     for i in range(offset):
 */
    (__pyx_v_slice->suboffsets[(__pyx_v_i + __pyx_v_offset)]) = (__pyx_v_slice->suboffsets[__pyx_v_i]);
  }

  /* "View.MemoryView":1305
 *         slice.suboffsets[i + offset] = slice.suboffsets[i]
 * 
 *     for i in range(offset):             # <<<<<<<<<<<<<<
 *         slice.shape[i] = 1
 *         slice.strides[i] = slice.strides[0]
 */
  __pyx_t_1 = __pyx_v_offset;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "View.MemoryView":1306
 * 
 *     for i in range(offset):
 *         slice.shape[i] = 1             # <<<<<<<<<<<<<<
 *         slice.strides[i] = slice.strides[0]
 *         slice.suboffsets[i] = -1
 */
    (__pyx_v_slice->shape[__pyx_v_i]) = 1;

    /* "View.MemoryView":1307
 *     for i in range(offset):
 *         slice.shape[i] = 1
 *         slice.strides[i] = slice.strides[0]             # <<<<<<<<<<<<<<
 *         slice.suboffsets[i] = -1
 * 
 */
    (__pyx_v_slice->strides[__pyx_v_i]) = (__pyx_v_slice->strides[0]);

    /* "View.MemoryView":1308
 *         slice.shape[i] = 1
 *         slice.strides[i] = slice.strides[0]
 *         slice.suboffsets[i] = -1             # <<<<<<<<<<<<<<
 * 
 * 
 */
    (__pyx_v_slice->suboffsets[__pyx_v_i]) = -1;
  }

  /* "View.MemoryView":1294
 * 
 * @cname('__pyx_memoryview_broadcast_leading')
 * cdef void broadcast_leading(__Pyx_memviewslice *slice,             # <<<<<<<<<<<<<<
 *                             int ndim,
 *                             int ndim_other) nogil:
 */

  /* function exit code */
}

/* "View.MemoryView":1316
 * 
 * @cname('__pyx_memoryview_refcount_copying')
 * cdef void refcount_copying(__Pyx_memviewslice *dst, bint dtype_is_object,             # <<<<<<<<<<<<<<
 *                            int ndim, bint inc) nogil:
 * 
 */

static void __pyx_memoryview_refcount_copying(__Pyx_memviewslice *__pyx_v_dst, int __pyx_v_dtype_is_object, int __pyx_v_ndim, int __pyx_v_inc) {
  int __pyx_t_1;

  /* "View.MemoryView":1320
 * 
 * 
 *     if dtype_is_object:             # <<<<<<<<<<<<<<
 *         refcount_objects_in_slice_with_gil(dst.data, dst.shape,
 *                                            dst.strides, ndim, inc)
 */
  __pyx_t_1 = (__pyx_v_dtype_is_object != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":1321
 * 
 *     if dtype_is_object:
 *         refcount_objects_in_slice_with_gil(dst.data, dst.shape,             # <<<<<<<<<<<<<<
 *                                            dst.strides, ndim, inc)
 * 
 */
    __pyx_memoryview_refcount_objects_in_slice_with_gil(__pyx_v_dst->data, __pyx_v_dst->shape, __pyx_v_dst->strides, __pyx_v_ndim, __pyx_v_inc);
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "View.MemoryView":1316
 * 
 * @cname('__pyx_memoryview_refcount_copying')
 * cdef void refcount_copying(__Pyx_memviewslice *dst, bint dtype_is_object,             # <<<<<<<<<<<<<<
 *                            int ndim, bint inc) nogil:
 * 
 */

  /* function exit code */
}

/* "View.MemoryView":1325
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice_with_gil')
 * cdef void refcount_objects_in_slice_with_gil(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                                              Py_ssize_t *strides, int ndim,
 *                                              bint inc) with gil:
 */

static void __pyx_memoryview_refcount_objects_in_slice_with_gil(char *__pyx_v_data, Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, int __pyx_v_ndim, int __pyx_v_inc) {
  __Pyx_RefNannyDeclarations
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("refcount_objects_in_slice_with_gil", 0);

  /* "View.MemoryView":1328
 *                                              Py_ssize_t *strides, int ndim,
 *                                              bint inc) with gil:
 *     refcount_objects_in_slice(data, shape, strides, ndim, inc)             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice')
 */
  __pyx_memoryview_refcount_objects_in_slice(__pyx_v_data, __pyx_v_shape, __pyx_v_strides, __pyx_v_ndim, __pyx_v_inc);

  /* "View.MemoryView":1325
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice_with_gil')
 * cdef void refcount_objects_in_slice_with_gil(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                                              Py_ssize_t *strides, int ndim,
 *                                              bint inc) with gil:
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "View.MemoryView":1331
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice')
 * cdef void refcount_objects_in_slice(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                                     Py_ssize_t *strides, int ndim, bint inc):
 *     cdef Py_ssize_t i
 */

static void __pyx_memoryview_refcount_objects_in_slice(char *__pyx_v_data, Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, int __pyx_v_ndim, int __pyx_v_inc) {
  CYTHON_UNUSED Py_ssize_t __pyx_v_i;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("refcount_objects_in_slice", 0);

  /* "View.MemoryView":1335
 *     cdef Py_ssize_t i
 * 
 *     for i in range(shape[0]):             # <<<<<<<<<<<<<<
 *         if ndim == 1:
 *             if inc:
 */
  __pyx_t_1 = (__pyx_v_shape[0]);
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "View.MemoryView":1336
 * 
 *     for i in range(shape[0]):
 *         if ndim == 1:             # <<<<<<<<<<<<<<
 *             if inc:
 *                 Py_INCREF((<PyObject **> data)[0])
 */
    __pyx_t_3 = ((__pyx_v_ndim == 1) != 0);
    if (__pyx_t_3) {

      /* "View.MemoryView":1337
 *     for i in range(shape[0]):
 *         if ndim == 1:
 *             if inc:             # <<<<<<<<<<<<<<
 *                 Py_INCREF((<PyObject **> data)[0])
 *             else:
 */
      __pyx_t_3 = (__pyx_v_inc != 0);
      if (__pyx_t_3) {

        /* "View.MemoryView":1338
 *         if ndim == 1:
 *             if inc:
 *                 Py_INCREF((<PyObject **> data)[0])             # <<<<<<<<<<<<<<
 *             else:
 *                 Py_DECREF((<PyObject **> data)[0])
 */
        Py_INCREF((((PyObject **)__pyx_v_data)[0]));
        goto __pyx_L6;
      }
      /*else*/ {

        /* "View.MemoryView":1340
 *                 Py_INCREF((<PyObject **> data)[0])
 *             else:
 *                 Py_DECREF((<PyObject **> data)[0])             # <<<<<<<<<<<<<<
 *         else:
 *             refcount_objects_in_slice(data, shape + 1, strides + 1,
 */
        Py_DECREF((((PyObject **)__pyx_v_data)[0]));
      }
      __pyx_L6:;
      goto __pyx_L5;
    }
    /*else*/ {

      /* "View.MemoryView":1342
 *                 Py_DECREF((<PyObject **> data)[0])
 *         else:
 *             refcount_objects_in_slice(data, shape + 1, strides + 1,             # <<<<<<<<<<<<<<
 *                                       ndim - 1, inc)
 * 
 */
      __pyx_memoryview_refcount_objects_in_slice(__pyx_v_data, (__pyx_v_shape + 1), (__pyx_v_strides + 1), (__pyx_v_ndim - 1), __pyx_v_inc);
    }
    __pyx_L5:;

    /* "View.MemoryView":1345
 *                                       ndim - 1, inc)
 * 
 *         data += strides[0]             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_v_data = (__pyx_v_data + (__pyx_v_strides[0]));
  }

  /* "View.MemoryView":1331
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice')
 * cdef void refcount_objects_in_slice(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                                     Py_ssize_t *strides, int ndim, bint inc):
 *     cdef Py_ssize_t i
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "View.MemoryView":1351
 * 
 * @cname('__pyx_memoryview_slice_assign_scalar')
 * cdef void slice_assign_scalar(__Pyx_memviewslice *dst, int ndim,             # <<<<<<<<<<<<<<
 *                               size_t itemsize, void *item,
 *                               bint dtype_is_object) nogil:
 */

static void __pyx_memoryview_slice_assign_scalar(__Pyx_memviewslice *__pyx_v_dst, int __pyx_v_ndim, size_t __pyx_v_itemsize, void *__pyx_v_item, int __pyx_v_dtype_is_object) {

  /* "View.MemoryView":1354
 *                               size_t itemsize, void *item,
 *                               bint dtype_is_object) nogil:
 *     refcount_copying(dst, dtype_is_object, ndim, False)             # <<<<<<<<<<<<<<
 *     _slice_assign_scalar(dst.data, dst.shape, dst.strides, ndim,
 *                          itemsize, item)
 */
  __pyx_memoryview_refcount_copying(__pyx_v_dst, __pyx_v_dtype_is_object, __pyx_v_ndim, 0);

  /* "View.MemoryView":1355
 *                               bint dtype_is_object) nogil:
 *     refcount_copying(dst, dtype_is_object, ndim, False)
 *     _slice_assign_scalar(dst.data, dst.shape, dst.strides, ndim,             # <<<<<<<<<<<<<<
 *                          itemsize, item)
 *     refcount_copying(dst, dtype_is_object, ndim, True)
 */
  __pyx_memoryview__slice_assign_scalar(__pyx_v_dst->data, __pyx_v_dst->shape, __pyx_v_dst->strides, __pyx_v_ndim, __pyx_v_itemsize, __pyx_v_item);

  /* "View.MemoryView":1357
 *     _slice_assign_scalar(dst.data, dst.shape, dst.strides, ndim,
 *                          itemsize, item)
 *     refcount_copying(dst, dtype_is_object, ndim, True)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_memoryview_refcount_copying(__pyx_v_dst, __pyx_v_dtype_is_object, __pyx_v_ndim, 1);

  /* "View.MemoryView":1351
 * 
 * @cname('__pyx_memoryview_slice_assign_scalar')
 * cdef void slice_assign_scalar(__Pyx_memviewslice *dst, int ndim,             # <<<<<<<<<<<<<<
 *                               size_t itemsize, void *item,
 *                               bint dtype_is_object) nogil:
 */

  /* function exit code */
}

/* "View.MemoryView":1361
 * 
 * @cname('__pyx_memoryview__slice_assign_scalar')
 * cdef void _slice_assign_scalar(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                               Py_ssize_t *strides, int ndim,
 *                               size_t itemsize, void *item) nogil:
 */

static void __pyx_memoryview__slice_assign_scalar(char *__pyx_v_data, Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, int __pyx_v_ndim, size_t __pyx_v_itemsize, void *__pyx_v_item) {
  CYTHON_UNUSED Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_stride;
  Py_ssize_t __pyx_v_extent;
  int __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;

  /* "View.MemoryView":1365
 *                               size_t itemsize, void *item) nogil:
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t stride = strides[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t extent = shape[0]
 * 
 */
  __pyx_v_stride = (__pyx_v_strides[0]);

  /* "View.MemoryView":1366
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t stride = strides[0]
 *     cdef Py_ssize_t extent = shape[0]             # <<<<<<<<<<<<<<
 * 
 *     if ndim == 1:
 */
  __pyx_v_extent = (__pyx_v_shape[0]);

  /* "View.MemoryView":1368
 *     cdef Py_ssize_t extent = shape[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)
 */
  __pyx_t_1 = ((__pyx_v_ndim == 1) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":1369
 * 
 *     if ndim == 1:
 *         for i in range(extent):             # <<<<<<<<<<<<<<
 *             memcpy(data, item, itemsize)
 *             data += stride
 */
    __pyx_t_2 = __pyx_v_extent;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "View.MemoryView":1370
 *     if ndim == 1:
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)             # <<<<<<<<<<<<<<
 *             data += stride
 *     else:
 */
      memcpy(__pyx_v_data, __pyx_v_item, __pyx_v_itemsize);

      /* "View.MemoryView":1371
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)
 *             data += stride             # <<<<<<<<<<<<<<
 *     else:
 *         for i in range(extent):
 */
      __pyx_v_data = (__pyx_v_data + __pyx_v_stride);
    }
    goto __pyx_L3;
  }
  /*else*/ {

    /* "View.MemoryView":1373
 *             data += stride
 *     else:
 *         for i in range(extent):             # <<<<<<<<<<<<<<
 *             _slice_assign_scalar(data, shape + 1, strides + 1,
 *                                 ndim - 1, itemsize, item)
 */
    __pyx_t_2 = __pyx_v_extent;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "View.MemoryView":1374
 *     else:
 *         for i in range(extent):
 *             _slice_assign_scalar(data, shape + 1, strides + 1,             # <<<<<<<<<<<<<<
 *                                 ndim - 1, itemsize, item)
 *             data += stride
 */
      __pyx_memoryview__slice_assign_scalar(__pyx_v_data, (__pyx_v_shape + 1), (__pyx_v_strides + 1), (__pyx_v_ndim - 1), __pyx_v_itemsize, __pyx_v_item);

      /* "View.MemoryView":1376
 *             _slice_assign_scalar(data, shape + 1, strides + 1,
 *                                 ndim - 1, itemsize, item)
 *             data += stride             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_v_data = (__pyx_v_data + __pyx_v_stride);
    }
  }
  __pyx_L3:;

  /* "View.MemoryView":1361
 * 
 * @cname('__pyx_memoryview__slice_assign_scalar')
 * cdef void _slice_assign_scalar(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                               Py_ssize_t *strides, int ndim,
 *                               size_t itemsize, void *item) nogil:
 */

  /* function exit code */
}

/* "BufferFormatFromTypeInfo":1417
 * 
 * @cname('__pyx_format_from_typeinfo')
 * cdef bytes format_from_typeinfo(__Pyx_TypeInfo *type):             # <<<<<<<<<<<<<<
 *     cdef __Pyx_StructField *field
 *     cdef __pyx_typeinfo_string fmt
 */

static PyObject *__pyx_format_from_typeinfo(__Pyx_TypeInfo *__pyx_v_type) {
  __Pyx_StructField *__pyx_v_field;
  struct __pyx_typeinfo_string __pyx_v_fmt;
  PyObject *__pyx_v_part = 0;
  PyObject *__pyx_v_result = 0;
  PyObject *__pyx_v_alignment = NULL;
  PyObject *__pyx_v_parts = NULL;
  PyObject *__pyx_v_extents = NULL;
  int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_StructField *__pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("format_from_typeinfo", 0);

  /* "BufferFormatFromTypeInfo":1422
 *     cdef bytes part, result
 * 
 *     if type.typegroup == 'S':             # <<<<<<<<<<<<<<
 *         assert type.fields != NULL and type.fields.type != NULL
 * 
 */
  __pyx_t_1 = ((__pyx_v_type->typegroup == 'S') != 0);
  if (__pyx_t_1) {

    /* "BufferFormatFromTypeInfo":1423
 * 
 *     if type.typegroup == 'S':
 *         assert type.fields != NULL and type.fields.type != NULL             # <<<<<<<<<<<<<<
 * 
 *         if type.flags & __PYX_BUF_FLAGS_PACKED_STRUCT:
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      __pyx_t_2 = ((__pyx_v_type->fields != NULL) != 0);
      if (__pyx_t_2) {
      } else {
        __pyx_t_1 = __pyx_t_2;
        goto __pyx_L4_bool_binop_done;
      }
      __pyx_t_2 = ((__pyx_v_type->fields->type != NULL) != 0);
      __pyx_t_1 = __pyx_t_2;
      __pyx_L4_bool_binop_done:;
      if (unlikely(!__pyx_t_1)) {
        PyErr_SetNone(PyExc_AssertionError);
        {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1423; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
    }
    #endif

    /* "BufferFormatFromTypeInfo":1425
 *         assert type.fields != NULL and type.fields.type != NULL
 * 
 *         if type.flags & __PYX_BUF_FLAGS_PACKED_STRUCT:             # <<<<<<<<<<<<<<
 *             alignment = b'^'
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_type->flags & __PYX_BUF_FLAGS_PACKED_STRUCT) != 0);
    if (__pyx_t_1) {

      /* "BufferFormatFromTypeInfo":1426
 * 
 *         if type.flags & __PYX_BUF_FLAGS_PACKED_STRUCT:
 *             alignment = b'^'             # <<<<<<<<<<<<<<
 *         else:
 *             alignment = b''
 */
      __Pyx_INCREF(__pyx_kp_b__50);
      __pyx_v_alignment = __pyx_kp_b__50;
      goto __pyx_L6;
    }
    /*else*/ {

      /* "BufferFormatFromTypeInfo":1428
 *             alignment = b'^'
 *         else:
 *             alignment = b''             # <<<<<<<<<<<<<<
 * 
 *         parts = [b"T{"]
 */
      __Pyx_INCREF(__pyx_kp_b__51);
      __pyx_v_alignment = __pyx_kp_b__51;
    }
    __pyx_L6:;

    /* "BufferFormatFromTypeInfo":1430
 *             alignment = b''
 * 
 *         parts = [b"T{"]             # <<<<<<<<<<<<<<
 *         field = type.fields
 * 
 */
    __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1430; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_kp_b_T);
    PyList_SET_ITEM(__pyx_t_3, 0, __pyx_kp_b_T);
    __Pyx_GIVEREF(__pyx_kp_b_T);
    __pyx_v_parts = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "BufferFormatFromTypeInfo":1431
 * 
 *         parts = [b"T{"]
 *         field = type.fields             # <<<<<<<<<<<<<<
 * 
 *         while field.type:
 */
    __pyx_t_4 = __pyx_v_type->fields;
    __pyx_v_field = __pyx_t_4;

    /* "BufferFormatFromTypeInfo":1433
 *         field = type.fields
 * 
 *         while field.type:             # <<<<<<<<<<<<<<
 *             part = format_from_typeinfo(field.type)
 *             parts.append(part + b':' + field.name + b':')
 */
    while (1) {
      __pyx_t_1 = (__pyx_v_field->type != 0);
      if (!__pyx_t_1) break;

      /* "BufferFormatFromTypeInfo":1434
 * 
 *         while field.type:
 *             part = format_from_typeinfo(field.type)             # <<<<<<<<<<<<<<
 *             parts.append(part + b':' + field.name + b':')
 *             field += 1
 */
      __pyx_t_3 = __pyx_format_from_typeinfo(__pyx_v_field->type); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_XDECREF_SET(__pyx_v_part, ((PyObject*)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "BufferFormatFromTypeInfo":1435
 *         while field.type:
 *             part = format_from_typeinfo(field.type)
 *             parts.append(part + b':' + field.name + b':')             # <<<<<<<<<<<<<<
 *             field += 1
 * 
 */
      __pyx_t_3 = PyNumber_Add(__pyx_v_part, __pyx_kp_b__52); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1435; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = __Pyx_PyBytes_FromString(__pyx_v_field->name); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1435; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = PyNumber_Add(__pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1435; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = PyNumber_Add(__pyx_t_6, __pyx_kp_b__52); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1435; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_parts, __pyx_t_5); if (unlikely(__pyx_t_7 == -1)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1435; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "BufferFormatFromTypeInfo":1436
 *             part = format_from_typeinfo(field.type)
 *             parts.append(part + b':' + field.name + b':')
 *             field += 1             # <<<<<<<<<<<<<<
 * 
 *         result = alignment.join(parts) + b'}'
 */
      __pyx_v_field = (__pyx_v_field + 1);
    }

    /* "BufferFormatFromTypeInfo":1438
 *             field += 1
 * 
 *         result = alignment.join(parts) + b'}'             # <<<<<<<<<<<<<<
 *     else:
 *         fmt = __Pyx_TypeInfoToFormat(type)
 */
    __pyx_t_5 = __Pyx_PyBytes_Join(__pyx_v_alignment, __pyx_v_parts); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1438; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyNumber_Add(__pyx_t_5, __pyx_kp_b__53); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1438; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (!(likely(PyBytes_CheckExact(__pyx_t_6))||((__pyx_t_6) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_6)->tp_name), 0))) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1438; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_result = ((PyObject*)__pyx_t_6);
    __pyx_t_6 = 0;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "BufferFormatFromTypeInfo":1440
 *         result = alignment.join(parts) + b'}'
 *     else:
 *         fmt = __Pyx_TypeInfoToFormat(type)             # <<<<<<<<<<<<<<
 *         if type.arraysize[0]:
 *             extents = [unicode(type.arraysize[i]) for i in range(type.ndim)]
 */
    __pyx_v_fmt = __Pyx_TypeInfoToFormat(__pyx_v_type);

    /* "BufferFormatFromTypeInfo":1441
 *     else:
 *         fmt = __Pyx_TypeInfoToFormat(type)
 *         if type.arraysize[0]:             # <<<<<<<<<<<<<<
 *             extents = [unicode(type.arraysize[i]) for i in range(type.ndim)]
 *             result = (u"(%s)" % u','.join(extents)).encode('ascii') + fmt.string
 */
    __pyx_t_1 = ((__pyx_v_type->arraysize[0]) != 0);
    if (__pyx_t_1) {

      /* "BufferFormatFromTypeInfo":1442
 *         fmt = __Pyx_TypeInfoToFormat(type)
 *         if type.arraysize[0]:
 *             extents = [unicode(type.arraysize[i]) for i in range(type.ndim)]             # <<<<<<<<<<<<<<
 *             result = (u"(%s)" % u','.join(extents)).encode('ascii') + fmt.string
 *         else:
 */
      __pyx_t_6 = PyList_New(0); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_8 = __pyx_v_type->ndim;
      for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
        __pyx_v_i = __pyx_t_9;
        __pyx_t_5 = __Pyx_PyInt_FromSize_t((__pyx_v_type->arraysize[__pyx_v_i])); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_5);
        __pyx_t_5 = 0;
        __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)((PyObject*)(&PyUnicode_Type))), __pyx_t_3, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_6, (PyObject*)__pyx_t_5))) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __pyx_v_extents = ((PyObject*)__pyx_t_6);
      __pyx_t_6 = 0;

      /* "BufferFormatFromTypeInfo":1443
 *         if type.arraysize[0]:
 *             extents = [unicode(type.arraysize[i]) for i in range(type.ndim)]
 *             result = (u"(%s)" % u','.join(extents)).encode('ascii') + fmt.string             # <<<<<<<<<<<<<<
 *         else:
 *             result = fmt.string
 */
      __pyx_t_6 = PyUnicode_Join(__pyx_kp_u__54, __pyx_v_extents); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1443; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_5 = PyUnicode_Format(__pyx_kp_u_s, __pyx_t_6); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1443; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = PyUnicode_AsASCIIString(((PyObject*)__pyx_t_5)); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1443; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyBytes_FromString(__pyx_v_fmt.string); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1443; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_3 = PyNumber_Add(__pyx_t_6, __pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1443; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (!(likely(PyBytes_CheckExact(__pyx_t_3))||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_3)->tp_name), 0))) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1443; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_v_result = ((PyObject*)__pyx_t_3);
      __pyx_t_3 = 0;
      goto __pyx_L9;
    }
    /*else*/ {

      /* "BufferFormatFromTypeInfo":1445
 *             result = (u"(%s)" % u','.join(extents)).encode('ascii') + fmt.string
 *         else:
 *             result = fmt.string             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
      __pyx_t_3 = __Pyx_PyBytes_FromString(__pyx_v_fmt.string); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1445; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_v_result = ((PyObject*)__pyx_t_3);
      __pyx_t_3 = 0;
    }
    __pyx_L9:;
  }
  __pyx_L3:;

  /* "BufferFormatFromTypeInfo":1447
 *             result = fmt.string
 * 
 *     return result             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "BufferFormatFromTypeInfo":1417
 * 
 * @cname('__pyx_format_from_typeinfo')
 * cdef bytes format_from_typeinfo(__Pyx_TypeInfo *type):             # <<<<<<<<<<<<<<
 *     cdef __Pyx_StructField *field
 *     cdef __pyx_typeinfo_string fmt
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("BufferFormatFromTypeInfo.format_from_typeinfo", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_part);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_alignment);
  __Pyx_XDECREF(__pyx_v_parts);
  __Pyx_XDECREF(__pyx_v_extents);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NeighborsHeap __pyx_vtable_7sklearn_9neighbors_7kd_tree_NeighborsHeap;

static PyObject *__pyx_tp_new_7sklearn_9neighbors_7kd_tree_NeighborsHeap(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *)o);
  p->__pyx_vtab = __pyx_vtabptr_7sklearn_9neighbors_7kd_tree_NeighborsHeap;
  p->distances_arr = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->indices_arr = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->distances.data = NULL;
  p->distances.memview = NULL;
  p->indices.data = NULL;
  p->indices.memview = NULL;
  if (unlikely(__pyx_pw_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_1__cinit__(o, __pyx_empty_tuple, NULL) < 0)) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_7sklearn_9neighbors_7kd_tree_NeighborsHeap(PyObject *o) {
  struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *p = (struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->distances_arr);
  Py_CLEAR(p->indices_arr);
  __PYX_XDEC_MEMVIEW(&p->distances, 1);
  __PYX_XDEC_MEMVIEW(&p->indices, 1);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_7sklearn_9neighbors_7kd_tree_NeighborsHeap(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *p = (struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *)o;
  if (p->distances_arr) {
    e = (*v)(((PyObject*)p->distances_arr), a); if (e) return e;
  }
  if (p->indices_arr) {
    e = (*v)(((PyObject*)p->indices_arr), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7sklearn_9neighbors_7kd_tree_NeighborsHeap(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *p = (struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *)o;
  tmp = ((PyObject*)p->distances_arr);
  p->distances_arr = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->indices_arr);
  p->indices_arr = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_7sklearn_9neighbors_7kd_tree_NeighborsHeap[] = {
  {"get_arrays", (PyCFunction)__pyx_pw_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_5get_arrays, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_4get_arrays},
  {"push", (PyCFunction)__pyx_pw_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_7push, METH_VARARGS|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_7sklearn_9neighbors_7kd_tree_NeighborsHeap = {
  PyVarObject_HEAD_INIT(0, 0)
  "sklearn.neighbors.kd_tree.NeighborsHeap", /*tp_name*/
  sizeof(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7sklearn_9neighbors_7kd_tree_NeighborsHeap, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "A max-heap structure to keep track of distances/indices of neighbors\n\n    This implements an efficient pre-allocated set of fixed-size heaps\n    for chasing neighbors, holding both an index and a distance.\n    When any row of the heap is full, adding an additional point will push\n    the furthest point off the heap.\n\n    Parameters\n    ----------\n    n_pts : int\n        the number of heaps to use\n    n_nbrs : int\n        the size of each heap.\n    ", /*tp_doc*/
  __pyx_tp_traverse_7sklearn_9neighbors_7kd_tree_NeighborsHeap, /*tp_traverse*/
  __pyx_tp_clear_7sklearn_9neighbors_7kd_tree_NeighborsHeap, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7sklearn_9neighbors_7kd_tree_NeighborsHeap, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_3__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7sklearn_9neighbors_7kd_tree_NeighborsHeap, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_NodeHeap __pyx_vtable_7sklearn_9neighbors_7kd_tree_NodeHeap;

static PyObject *__pyx_tp_new_7sklearn_9neighbors_7kd_tree_NodeHeap(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *)o);
  p->__pyx_vtab = __pyx_vtabptr_7sklearn_9neighbors_7kd_tree_NodeHeap;
  p->data_arr = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->data.data = NULL;
  p->data.memview = NULL;
  if (unlikely(__pyx_pw_7sklearn_9neighbors_7kd_tree_8NodeHeap_1__cinit__(o, __pyx_empty_tuple, NULL) < 0)) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_7sklearn_9neighbors_7kd_tree_NodeHeap(PyObject *o) {
  struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *p = (struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->data_arr);
  __PYX_XDEC_MEMVIEW(&p->data, 1);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_7sklearn_9neighbors_7kd_tree_NodeHeap(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *p = (struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *)o;
  if (p->data_arr) {
    e = (*v)(((PyObject*)p->data_arr), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7sklearn_9neighbors_7kd_tree_NodeHeap(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *p = (struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *)o;
  tmp = ((PyObject*)p->data_arr);
  p->data_arr = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_7sklearn_9neighbors_7kd_tree_NodeHeap[] = {
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_7sklearn_9neighbors_7kd_tree_NodeHeap = {
  PyVarObject_HEAD_INIT(0, 0)
  "sklearn.neighbors.kd_tree.NodeHeap", /*tp_name*/
  sizeof(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7sklearn_9neighbors_7kd_tree_NodeHeap, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "NodeHeap\n\n    This is a min-heap implementation for keeping track of nodes\n    during a breadth-first search.  Unlike the NeighborsHeap above,\n    the NodeHeap does not have a fixed size and must be able to grow\n    as elements are added.\n\n    Internally, the data is stored in a simple binary heap which meets\n    the min heap condition:\n\n        heap[i].val < min(heap[2 * i + 1].val, heap[2 * i + 2].val)\n    ", /*tp_doc*/
  __pyx_tp_traverse_7sklearn_9neighbors_7kd_tree_NodeHeap, /*tp_traverse*/
  __pyx_tp_clear_7sklearn_9neighbors_7kd_tree_NodeHeap, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7sklearn_9neighbors_7kd_tree_NodeHeap, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_7sklearn_9neighbors_7kd_tree_8NodeHeap_3__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7sklearn_9neighbors_7kd_tree_NodeHeap, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree __pyx_vtable_7sklearn_9neighbors_7kd_tree_BinaryTree;

static PyObject *__pyx_tp_new_7sklearn_9neighbors_7kd_tree_BinaryTree(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)o);
  p->__pyx_vtab = __pyx_vtabptr_7sklearn_9neighbors_7kd_tree_BinaryTree;
  p->data_arr = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->idx_array_arr = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->node_data_arr = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->node_bounds_arr = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->dist_metric = ((struct __pyx_obj_7sklearn_9neighbors_12dist_metrics_DistanceMetric *)Py_None); Py_INCREF(Py_None);
  p->data.data = NULL;
  p->data.memview = NULL;
  p->idx_array.data = NULL;
  p->idx_array.memview = NULL;
  p->node_data.data = NULL;
  p->node_data.memview = NULL;
  p->node_bounds.data = NULL;
  p->node_bounds.memview = NULL;
  if (unlikely(__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_1__cinit__(o, __pyx_empty_tuple, NULL) < 0)) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_7sklearn_9neighbors_7kd_tree_BinaryTree(PyObject *o) {
  struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *p = (struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->data_arr);
  Py_CLEAR(p->idx_array_arr);
  Py_CLEAR(p->node_data_arr);
  Py_CLEAR(p->node_bounds_arr);
  Py_CLEAR(p->dist_metric);
  __PYX_XDEC_MEMVIEW(&p->data, 1);
  __PYX_XDEC_MEMVIEW(&p->idx_array, 1);
  __PYX_XDEC_MEMVIEW(&p->node_data, 1);
  __PYX_XDEC_MEMVIEW(&p->node_bounds, 1);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_7sklearn_9neighbors_7kd_tree_BinaryTree(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *p = (struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)o;
  if (p->data_arr) {
    e = (*v)(((PyObject*)p->data_arr), a); if (e) return e;
  }
  if (p->idx_array_arr) {
    e = (*v)(((PyObject*)p->idx_array_arr), a); if (e) return e;
  }
  if (p->node_data_arr) {
    e = (*v)(((PyObject*)p->node_data_arr), a); if (e) return e;
  }
  if (p->node_bounds_arr) {
    e = (*v)(((PyObject*)p->node_bounds_arr), a); if (e) return e;
  }
  if (p->dist_metric) {
    e = (*v)(((PyObject*)p->dist_metric), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7sklearn_9neighbors_7kd_tree_BinaryTree(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *p = (struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *)o;
  tmp = ((PyObject*)p->data_arr);
  p->data_arr = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->idx_array_arr);
  p->idx_array_arr = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->node_data_arr);
  p->node_data_arr = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->node_bounds_arr);
  p->node_bounds_arr = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->dist_metric);
  p->dist_metric = ((struct __pyx_obj_7sklearn_9neighbors_12dist_metrics_DistanceMetric *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_7sklearn_9neighbors_7kd_tree_10BinaryTree_data(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_4data_1__get__(o);
}

static PyObject *__pyx_getprop_7sklearn_9neighbors_7kd_tree_10BinaryTree_idx_array(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_9idx_array_1__get__(o);
}

static int __pyx_setprop_7sklearn_9neighbors_7kd_tree_10BinaryTree_idx_array(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_9idx_array_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7sklearn_9neighbors_7kd_tree_10BinaryTree_node_data(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_9node_data_1__get__(o);
}

static int __pyx_setprop_7sklearn_9neighbors_7kd_tree_10BinaryTree_node_data(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_9node_data_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7sklearn_9neighbors_7kd_tree_10BinaryTree_node_bounds(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_11node_bounds_1__get__(o);
}

static int __pyx_setprop_7sklearn_9neighbors_7kd_tree_10BinaryTree_node_bounds(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_11node_bounds_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_7sklearn_9neighbors_7kd_tree_BinaryTree[] = {
  {"__reduce__", (PyCFunction)__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_5__reduce__, METH_NOARGS, __pyx_doc_7sklearn_9neighbors_7kd_tree_10BinaryTree_4__reduce__},
  {"__getstate__", (PyCFunction)__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_7__getstate__, METH_NOARGS, __pyx_doc_7sklearn_9neighbors_7kd_tree_10BinaryTree_6__getstate__},
  {"__setstate__", (PyCFunction)__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_9__setstate__, METH_O, __pyx_doc_7sklearn_9neighbors_7kd_tree_10BinaryTree_8__setstate__},
  {"get_tree_stats", (PyCFunction)__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_11get_tree_stats, METH_NOARGS, 0},
  {"reset_n_calls", (PyCFunction)__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_13reset_n_calls, METH_NOARGS, 0},
  {"get_n_calls", (PyCFunction)__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_15get_n_calls, METH_NOARGS, 0},
  {"get_arrays", (PyCFunction)__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_17get_arrays, METH_NOARGS, 0},
  {"query", (PyCFunction)__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_19query, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7sklearn_9neighbors_7kd_tree_10BinaryTree_18query},
  {"query_radius", (PyCFunction)__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_21query_radius, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7sklearn_9neighbors_7kd_tree_10BinaryTree_20query_radius},
  {"kernel_density", (PyCFunction)__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_23kernel_density, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7sklearn_9neighbors_7kd_tree_10BinaryTree_22kernel_density},
  {"two_point_correlation", (PyCFunction)__pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_25two_point_correlation, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7sklearn_9neighbors_7kd_tree_10BinaryTree_24two_point_correlation},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_7sklearn_9neighbors_7kd_tree_BinaryTree[] = {
  {(char *)"data", __pyx_getprop_7sklearn_9neighbors_7kd_tree_10BinaryTree_data, 0, 0, 0},
  {(char *)"idx_array", __pyx_getprop_7sklearn_9neighbors_7kd_tree_10BinaryTree_idx_array, __pyx_setprop_7sklearn_9neighbors_7kd_tree_10BinaryTree_idx_array, 0, 0},
  {(char *)"node_data", __pyx_getprop_7sklearn_9neighbors_7kd_tree_10BinaryTree_node_data, __pyx_setprop_7sklearn_9neighbors_7kd_tree_10BinaryTree_node_data, 0, 0},
  {(char *)"node_bounds", __pyx_getprop_7sklearn_9neighbors_7kd_tree_10BinaryTree_node_bounds, __pyx_setprop_7sklearn_9neighbors_7kd_tree_10BinaryTree_node_bounds, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_7sklearn_9neighbors_7kd_tree_BinaryTree = {
  PyVarObject_HEAD_INIT(0, 0)
  "sklearn.neighbors.kd_tree.BinaryTree", /*tp_name*/
  sizeof(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7sklearn_9neighbors_7kd_tree_BinaryTree, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7sklearn_9neighbors_7kd_tree_BinaryTree, /*tp_traverse*/
  __pyx_tp_clear_7sklearn_9neighbors_7kd_tree_BinaryTree, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7sklearn_9neighbors_7kd_tree_BinaryTree, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_7sklearn_9neighbors_7kd_tree_BinaryTree, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_3__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7sklearn_9neighbors_7kd_tree_BinaryTree, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_KDTree __pyx_vtable_7sklearn_9neighbors_7kd_tree_KDTree;

static PyObject *__pyx_tp_new_7sklearn_9neighbors_7kd_tree_KDTree(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7sklearn_9neighbors_7kd_tree_KDTree *p;
  PyObject *o = __pyx_tp_new_7sklearn_9neighbors_7kd_tree_BinaryTree(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7sklearn_9neighbors_7kd_tree_KDTree *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_7sklearn_9neighbors_7kd_tree_BinaryTree*)__pyx_vtabptr_7sklearn_9neighbors_7kd_tree_KDTree;
  return o;
}

static PyTypeObject __pyx_type_7sklearn_9neighbors_7kd_tree_KDTree = {
  PyVarObject_HEAD_INIT(0, 0)
  "sklearn.neighbors.kd_tree.KDTree", /*tp_name*/
  sizeof(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_KDTree), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7sklearn_9neighbors_7kd_tree_BinaryTree, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7sklearn_9neighbors_7kd_tree_BinaryTree, /*tp_traverse*/
  __pyx_tp_clear_7sklearn_9neighbors_7kd_tree_BinaryTree, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_7sklearn_9neighbors_7kd_tree_10BinaryTree_3__init__, /*tp_init*/
  #else
  0, /*tp_init*/
  #endif
  0, /*tp_alloc*/
  __pyx_tp_new_7sklearn_9neighbors_7kd_tree_KDTree, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_array(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_array_obj *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_array_obj *)o);
  p->mode = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_format = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_array___cinit__(o, a, k) < 0)) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_array(PyObject *o) {
  struct __pyx_array_obj *p = (struct __pyx_array_obj *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_array___dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->mode);
  Py_CLEAR(p->_format);
  (*Py_TYPE(o)->tp_free)(o);
}
static PyObject *__pyx_sq_item_array(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_array(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_array___setitem__(o, i, v);
  }
  else {
    PyErr_Format(PyExc_NotImplementedError,
      "Subscript deletion not supported by %.200s", Py_TYPE(o)->tp_name);
    return -1;
  }
}

static PyObject *__pyx_tp_getattro_array(PyObject *o, PyObject *n) {
  PyObject *v = PyObject_GenericGetAttr(o, n);
  if (!v && PyErr_ExceptionMatches(PyExc_AttributeError)) {
    PyErr_Clear();
    v = __pyx_array___getattr__(o, n);
  }
  return v;
}

static PyObject *__pyx_getprop___pyx_array_memview(PyObject *o, CYTHON_UNUSED void *x) {
  return get_memview(o);
}

static PyMethodDef __pyx_methods_array[] = {
  {"__getattr__", (PyCFunction)__pyx_array___getattr__, METH_O|METH_COEXIST, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_array[] = {
  {(char *)"memview", __pyx_getprop___pyx_array_memview, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_array = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_array, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_array = {
  0, /*mp_length*/
  __pyx_array___getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_array, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_array = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  __pyx_array_getbuffer, /*bf_getbuffer*/
  0, /*bf_releasebuffer*/
};

static PyTypeObject __pyx_type___pyx_array = {
  PyVarObject_HEAD_INIT(0, 0)
  "sklearn.neighbors.kd_tree.array", /*tp_name*/
  sizeof(struct __pyx_array_obj), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_array, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_array, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_array, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  __pyx_tp_getattro_array, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_array, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_array, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_array, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_array, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_Enum(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_MemviewEnum_obj *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_MemviewEnum_obj *)o);
  p->name = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_Enum(PyObject *o) {
  struct __pyx_MemviewEnum_obj *p = (struct __pyx_MemviewEnum_obj *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->name);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_Enum(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_MemviewEnum_obj *p = (struct __pyx_MemviewEnum_obj *)o;
  if (p->name) {
    e = (*v)(p->name, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_Enum(PyObject *o) {
  PyObject* tmp;
  struct __pyx_MemviewEnum_obj *p = (struct __pyx_MemviewEnum_obj *)o;
  tmp = ((PyObject*)p->name);
  p->name = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_Enum[] = {
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type___pyx_MemviewEnum = {
  PyVarObject_HEAD_INIT(0, 0)
  "sklearn.neighbors.kd_tree.Enum", /*tp_name*/
  sizeof(struct __pyx_MemviewEnum_obj), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_Enum, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  __pyx_MemviewEnum___repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_Enum, /*tp_traverse*/
  __pyx_tp_clear_Enum, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_Enum, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_MemviewEnum___init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_Enum, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_memoryview __pyx_vtable_memoryview;

static PyObject *__pyx_tp_new_memoryview(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_memoryview_obj *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_memoryview_obj *)o);
  p->__pyx_vtab = __pyx_vtabptr_memoryview;
  p->obj = Py_None; Py_INCREF(Py_None);
  p->_size = Py_None; Py_INCREF(Py_None);
  p->_array_interface = Py_None; Py_INCREF(Py_None);
  p->view.obj = NULL;
  if (unlikely(__pyx_memoryview___cinit__(o, a, k) < 0)) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_memoryview(PyObject *o) {
  struct __pyx_memoryview_obj *p = (struct __pyx_memoryview_obj *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_memoryview___dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->obj);
  Py_CLEAR(p->_size);
  Py_CLEAR(p->_array_interface);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_memoryview(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_memoryview_obj *p = (struct __pyx_memoryview_obj *)o;
  if (p->obj) {
    e = (*v)(p->obj, a); if (e) return e;
  }
  if (p->_size) {
    e = (*v)(p->_size, a); if (e) return e;
  }
  if (p->_array_interface) {
    e = (*v)(p->_array_interface, a); if (e) return e;
  }
  if (p->view.obj) {
    e = (*v)(p->view.obj, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_memoryview(PyObject *o) {
  PyObject* tmp;
  struct __pyx_memoryview_obj *p = (struct __pyx_memoryview_obj *)o;
  tmp = ((PyObject*)p->obj);
  p->obj = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_size);
  p->_size = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_array_interface);
  p->_array_interface = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  Py_CLEAR(p->view.obj);
  return 0;
}
static PyObject *__pyx_sq_item_memoryview(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_memoryview(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_memoryview___setitem__(o, i, v);
  }
  else {
    PyErr_Format(PyExc_NotImplementedError,
      "Subscript deletion not supported by %.200s", Py_TYPE(o)->tp_name);
    return -1;
  }
}

static PyObject *__pyx_getprop___pyx_memoryview_T(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_memoryview_transpose(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_base(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_memoryview__get__base(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_shape(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_memoryview_get_shape(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_strides(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_memoryview_get_strides(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_suboffsets(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_memoryview_get_suboffsets(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_ndim(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_memoryview_get_ndim(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_itemsize(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_memoryview_get_itemsize(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_nbytes(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_memoryview_get_nbytes(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_memoryview_get_size(o);
}

static PyMethodDef __pyx_methods_memoryview[] = {
  {"is_c_contig", (PyCFunction)__pyx_memoryview_is_c_contig, METH_NOARGS, 0},
  {"is_f_contig", (PyCFunction)__pyx_memoryview_is_f_contig, METH_NOARGS, 0},
  {"copy", (PyCFunction)__pyx_memoryview_copy, METH_NOARGS, 0},
  {"copy_fortran", (PyCFunction)__pyx_memoryview_copy_fortran, METH_NOARGS, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_memoryview[] = {
  {(char *)"T", __pyx_getprop___pyx_memoryview_T, 0, 0, 0},
  {(char *)"base", __pyx_getprop___pyx_memoryview_base, 0, 0, 0},
  {(char *)"shape", __pyx_getprop___pyx_memoryview_shape, 0, 0, 0},
  {(char *)"strides", __pyx_getprop___pyx_memoryview_strides, 0, 0, 0},
  {(char *)"suboffsets", __pyx_getprop___pyx_memoryview_suboffsets, 0, 0, 0},
  {(char *)"ndim", __pyx_getprop___pyx_memoryview_ndim, 0, 0, 0},
  {(char *)"itemsize", __pyx_getprop___pyx_memoryview_itemsize, 0, 0, 0},
  {(char *)"nbytes", __pyx_getprop___pyx_memoryview_nbytes, 0, 0, 0},
  {(char *)"size", __pyx_getprop___pyx_memoryview_size, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_memoryview = {
  __pyx_memoryview___len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_memoryview, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_memoryview = {
  __pyx_memoryview___len__, /*mp_length*/
  __pyx_memoryview___getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_memoryview, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_memoryview = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  __pyx_memoryview_getbuffer, /*bf_getbuffer*/
  0, /*bf_releasebuffer*/
};

static PyTypeObject __pyx_type___pyx_memoryview = {
  PyVarObject_HEAD_INIT(0, 0)
  "sklearn.neighbors.kd_tree.memoryview", /*tp_name*/
  sizeof(struct __pyx_memoryview_obj), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_memoryview, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  __pyx_memoryview___repr__, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_memoryview, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_memoryview, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_memoryview___str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_memoryview, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_memoryview, /*tp_traverse*/
  __pyx_tp_clear_memoryview, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_memoryview, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_memoryview, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_memoryview, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct__memoryviewslice __pyx_vtable__memoryviewslice;

static PyObject *__pyx_tp_new__memoryviewslice(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_memoryviewslice_obj *p;
  PyObject *o = __pyx_tp_new_memoryview(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_memoryviewslice_obj *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_memoryview*)__pyx_vtabptr__memoryviewslice;
  p->from_object = Py_None; Py_INCREF(Py_None);
  p->from_slice.memview = NULL;
  return o;
}

static void __pyx_tp_dealloc__memoryviewslice(PyObject *o) {
  struct __pyx_memoryviewslice_obj *p = (struct __pyx_memoryviewslice_obj *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_memoryviewslice___dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->from_object);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_memoryview(o);
}

static int __pyx_tp_traverse__memoryviewslice(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_memoryviewslice_obj *p = (struct __pyx_memoryviewslice_obj *)o;
  e = __pyx_tp_traverse_memoryview(o, v, a); if (e) return e;
  if (p->from_object) {
    e = (*v)(p->from_object, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear__memoryviewslice(PyObject *o) {
  PyObject* tmp;
  struct __pyx_memoryviewslice_obj *p = (struct __pyx_memoryviewslice_obj *)o;
  __pyx_tp_clear_memoryview(o);
  tmp = ((PyObject*)p->from_object);
  p->from_object = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  __PYX_XDEC_MEMVIEW(&p->from_slice, 1);
  return 0;
}

static PyObject *__pyx_getprop___pyx_memoryviewslice_base(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_memoryviewslice__get__base(o);
}

static PyMethodDef __pyx_methods__memoryviewslice[] = {
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets__memoryviewslice[] = {
  {(char *)"base", __pyx_getprop___pyx_memoryviewslice_base, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type___pyx_memoryviewslice = {
  PyVarObject_HEAD_INIT(0, 0)
  "sklearn.neighbors.kd_tree._memoryviewslice", /*tp_name*/
  sizeof(struct __pyx_memoryviewslice_obj), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc__memoryviewslice, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_memoryview___repr__, /*tp_repr*/
  #else
  0, /*tp_repr*/
  #endif
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_memoryview___str__, /*tp_str*/
  #else
  0, /*tp_str*/
  #endif
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Internal class for passing memoryview slices to Python", /*tp_doc*/
  __pyx_tp_traverse__memoryviewslice, /*tp_traverse*/
  __pyx_tp_clear__memoryviewslice, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods__memoryviewslice, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets__memoryviewslice, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new__memoryviewslice, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    "kd_tree",
    0, /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_BinaryTree, __pyx_k_BinaryTree, sizeof(__pyx_k_BinaryTree), 0, 0, 1, 1},
  {&__pyx_kp_s_BinaryTree_for_fast_generalized, __pyx_k_BinaryTree_for_fast_generalized, sizeof(__pyx_k_BinaryTree_for_fast_generalized), 0, 0, 1, 0},
  {&__pyx_kp_u_BinaryTree_kernel_density_line_1, __pyx_k_BinaryTree_kernel_density_line_1, sizeof(__pyx_k_BinaryTree_kernel_density_line_1), 0, 1, 0, 0},
  {&__pyx_kp_u_BinaryTree_query_line_1233, __pyx_k_BinaryTree_query_line_1233, sizeof(__pyx_k_BinaryTree_query_line_1233), 0, 1, 0, 0},
  {&__pyx_kp_u_BinaryTree_query_radius_line_135, __pyx_k_BinaryTree_query_radius_line_135, sizeof(__pyx_k_BinaryTree_query_radius_line_135), 0, 1, 0, 0},
  {&__pyx_kp_u_BinaryTree_two_point_correlation, __pyx_k_BinaryTree_two_point_correlation, sizeof(__pyx_k_BinaryTree_two_point_correlation), 0, 1, 0, 0},
  {&__pyx_kp_s_Buffer_view_does_not_expose_stri, __pyx_k_Buffer_view_does_not_expose_stri, sizeof(__pyx_k_Buffer_view_does_not_expose_stri), 0, 0, 1, 0},
  {&__pyx_n_s_C, __pyx_k_C, sizeof(__pyx_k_C), 0, 0, 1, 1},
  {&__pyx_n_s_CLASS_DOC, __pyx_k_CLASS_DOC, sizeof(__pyx_k_CLASS_DOC), 0, 0, 1, 1},
  {&__pyx_kp_s_Can_only_create_a_buffer_that_is, __pyx_k_Can_only_create_a_buffer_that_is, sizeof(__pyx_k_Can_only_create_a_buffer_that_is), 0, 0, 1, 0},
  {&__pyx_kp_s_Cannot_index_with_type_s, __pyx_k_Cannot_index_with_type_s, sizeof(__pyx_k_Cannot_index_with_type_s), 0, 0, 1, 0},
  {&__pyx_n_s_ChebyshevDistance, __pyx_k_ChebyshevDistance, sizeof(__pyx_k_ChebyshevDistance), 0, 0, 1, 1},
  {&__pyx_kp_u_Compute_the_two_point_correlatio, __pyx_k_Compute_the_two_point_correlatio, sizeof(__pyx_k_Compute_the_two_point_correlatio), 0, 1, 0, 0},
  {&__pyx_n_s_DOC_DICT, __pyx_k_DOC_DICT, sizeof(__pyx_k_DOC_DICT), 0, 0, 1, 1},
  {&__pyx_n_s_DTYPE, __pyx_k_DTYPE, sizeof(__pyx_k_DTYPE), 0, 0, 1, 1},
  {&__pyx_n_s_Ellipsis, __pyx_k_Ellipsis, sizeof(__pyx_k_Ellipsis), 0, 0, 1, 1},
  {&__pyx_kp_s_Empty_shape_tuple_for_cython_arr, __pyx_k_Empty_shape_tuple_for_cython_arr, sizeof(__pyx_k_Empty_shape_tuple_for_cython_arr), 0, 0, 1, 0},
  {&__pyx_n_s_EuclideanDistance, __pyx_k_EuclideanDistance, sizeof(__pyx_k_EuclideanDistance), 0, 0, 1, 1},
  {&__pyx_kp_s_Fatal_count_out_of_range_This_sh, __pyx_k_Fatal_count_out_of_range_This_sh, sizeof(__pyx_k_Fatal_count_out_of_range_This_sh), 0, 0, 1, 0},
  {&__pyx_kp_s_Fatal_count_too_big_this_should, __pyx_k_Fatal_count_too_big_this_should, sizeof(__pyx_k_Fatal_count_too_big_this_should), 0, 0, 1, 0},
  {&__pyx_kp_u_Format_string_allocated_too_shor, __pyx_k_Format_string_allocated_too_shor, sizeof(__pyx_k_Format_string_allocated_too_shor), 0, 1, 0, 0},
  {&__pyx_kp_u_Format_string_allocated_too_shor_2, __pyx_k_Format_string_allocated_too_shor_2, sizeof(__pyx_k_Format_string_allocated_too_shor_2), 0, 1, 0, 0},
  {&__pyx_n_s_ITYPE, __pyx_k_ITYPE, sizeof(__pyx_k_ITYPE), 0, 0, 1, 1},
  {&__pyx_n_s_IndexError, __pyx_k_IndexError, sizeof(__pyx_k_IndexError), 0, 0, 1, 1},
  {&__pyx_kp_s_Indirect_dimensions_not_supporte, __pyx_k_Indirect_dimensions_not_supporte, sizeof(__pyx_k_Indirect_dimensions_not_supporte), 0, 0, 1, 0},
  {&__pyx_kp_s_Internal_memory_layout_is_flawed, __pyx_k_Internal_memory_layout_is_flawed, sizeof(__pyx_k_Internal_memory_layout_is_flawed), 0, 0, 1, 0},
  {&__pyx_kp_s_Internal_memory_layout_is_flawed_2, __pyx_k_Internal_memory_layout_is_flawed_2, sizeof(__pyx_k_Internal_memory_layout_is_flawed_2), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_mode_expected_c_or_fortr, __pyx_k_Invalid_mode_expected_c_or_fortr, sizeof(__pyx_k_Invalid_mode_expected_c_or_fortr), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_shape_in_axis_d_d, __pyx_k_Invalid_shape_in_axis_d_d, sizeof(__pyx_k_Invalid_shape_in_axis_d_d), 0, 0, 1, 0},
  {&__pyx_n_s_KDTree, __pyx_k_KDTree, sizeof(__pyx_k_KDTree), 0, 0, 1, 1},
  {&__pyx_kp_s_Kernel_code_not_recognized, __pyx_k_Kernel_code_not_recognized, sizeof(__pyx_k_Kernel_code_not_recognized), 0, 0, 1, 0},
  {&__pyx_n_s_ManhattanDistance, __pyx_k_ManhattanDistance, sizeof(__pyx_k_ManhattanDistance), 0, 0, 1, 1},
  {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
  {&__pyx_kp_s_MemoryView_of_r_at_0x_x, __pyx_k_MemoryView_of_r_at_0x_x, sizeof(__pyx_k_MemoryView_of_r_at_0x_x), 0, 0, 1, 0},
  {&__pyx_kp_s_MemoryView_of_r_object, __pyx_k_MemoryView_of_r_object, sizeof(__pyx_k_MemoryView_of_r_object), 0, 0, 1, 0},
  {&__pyx_n_s_MinkowskiDistance, __pyx_k_MinkowskiDistance, sizeof(__pyx_k_MinkowskiDistance), 0, 0, 1, 1},
  {&__pyx_n_s_NodeData, __pyx_k_NodeData, sizeof(__pyx_k_NodeData), 0, 0, 1, 1},
  {&__pyx_n_s_NodeHeapData, __pyx_k_NodeHeapData, sizeof(__pyx_k_NodeHeapData), 0, 0, 1, 1},
  {&__pyx_kp_u_Non_native_byte_order_not_suppor, __pyx_k_Non_native_byte_order_not_suppor, sizeof(__pyx_k_Non_native_byte_order_not_suppor), 0, 1, 0, 0},
  {&__pyx_n_b_O, __pyx_k_O, sizeof(__pyx_k_O), 0, 0, 0, 1},
  {&__pyx_kp_s_Out_of_bounds_on_buffer_access_a, __pyx_k_Out_of_bounds_on_buffer_access_a, sizeof(__pyx_k_Out_of_bounds_on_buffer_access_a), 0, 0, 1, 0},
  {&__pyx_n_s_RuntimeError, __pyx_k_RuntimeError, sizeof(__pyx_k_RuntimeError), 0, 0, 1, 1},
  {&__pyx_kp_b_T, __pyx_k_T, sizeof(__pyx_k_T), 0, 0, 0, 0},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_kp_s_Unable_to_convert_item_to_object, __pyx_k_Unable_to_convert_item_to_object, sizeof(__pyx_k_Unable_to_convert_item_to_object), 0, 0, 1, 0},
  {&__pyx_n_s_VALID_METRICS, __pyx_k_VALID_METRICS, sizeof(__pyx_k_VALID_METRICS), 0, 0, 1, 1},
  {&__pyx_n_s_VALID_METRIC_IDS, __pyx_k_VALID_METRIC_IDS, sizeof(__pyx_k_VALID_METRIC_IDS), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_X, __pyx_k_X, sizeof(__pyx_k_X), 0, 0, 1, 1},
  {&__pyx_kp_s_X_is_an_empty_array, __pyx_k_X_is_an_empty_array, sizeof(__pyx_k_X_is_an_empty_array), 0, 0, 1, 0},
  {&__pyx_kp_b__50, __pyx_k__50, sizeof(__pyx_k__50), 0, 0, 0, 0},
  {&__pyx_kp_b__51, __pyx_k__51, sizeof(__pyx_k__51), 0, 0, 0, 0},
  {&__pyx_kp_b__52, __pyx_k__52, sizeof(__pyx_k__52), 0, 0, 0, 0},
  {&__pyx_kp_b__53, __pyx_k__53, sizeof(__pyx_k__53), 0, 0, 0, 0},
  {&__pyx_kp_u__54, __pyx_k__54, sizeof(__pyx_k__54), 0, 1, 0, 0},
  {&__pyx_n_s_all, __pyx_k_all, sizeof(__pyx_k_all), 0, 0, 1, 1},
  {&__pyx_n_s_allocate_buffer, __pyx_k_allocate_buffer, sizeof(__pyx_k_allocate_buffer), 0, 0, 1, 1},
  {&__pyx_n_s_arange, __pyx_k_arange, sizeof(__pyx_k_arange), 0, 0, 1, 1},
  {&__pyx_n_s_argsort, __pyx_k_argsort, sizeof(__pyx_k_argsort), 0, 0, 1, 1},
  {&__pyx_n_s_asarray, __pyx_k_asarray, sizeof(__pyx_k_asarray), 0, 0, 1, 1},
  {&__pyx_n_s_atleast_1d, __pyx_k_atleast_1d, sizeof(__pyx_k_atleast_1d), 0, 0, 1, 1},
  {&__pyx_n_s_atol, __pyx_k_atol, sizeof(__pyx_k_atol), 0, 0, 1, 1},
  {&__pyx_n_s_base, __pyx_k_base, sizeof(__pyx_k_base), 0, 0, 1, 1},
  {&__pyx_n_s_binary_tree, __pyx_k_binary_tree, sizeof(__pyx_k_binary_tree), 0, 0, 1, 1},
  {&__pyx_n_s_breadth_first, __pyx_k_breadth_first, sizeof(__pyx_k_breadth_first), 0, 0, 1, 1},
  {&__pyx_n_s_c, __pyx_k_c, sizeof(__pyx_k_c), 0, 0, 1, 1},
  {&__pyx_n_u_c, __pyx_k_c, sizeof(__pyx_k_c), 0, 1, 0, 1},
  {&__pyx_kp_s_cannot_pop_on_empty_heap, __pyx_k_cannot_pop_on_empty_heap, sizeof(__pyx_k_cannot_pop_on_empty_heap), 0, 0, 1, 0},
  {&__pyx_n_s_check_array, __pyx_k_check_array, sizeof(__pyx_k_check_array), 0, 0, 1, 1},
  {&__pyx_n_s_class, __pyx_k_class, sizeof(__pyx_k_class), 0, 0, 1, 1},
  {&__pyx_kp_s_contiguous_and_direct, __pyx_k_contiguous_and_direct, sizeof(__pyx_k_contiguous_and_direct), 0, 0, 1, 0},
  {&__pyx_kp_s_contiguous_and_indirect, __pyx_k_contiguous_and_indirect, sizeof(__pyx_k_contiguous_and_indirect), 0, 0, 1, 0},
  {&__pyx_n_s_copy, __pyx_k_copy, sizeof(__pyx_k_copy), 0, 0, 1, 1},
  {&__pyx_n_s_cosine, __pyx_k_cosine, sizeof(__pyx_k_cosine), 0, 0, 1, 1},
  {&__pyx_n_s_count_only, __pyx_k_count_only, sizeof(__pyx_k_count_only), 0, 0, 1, 1},
  {&__pyx_kp_s_count_only_and_return_distance_c, __pyx_k_count_only_and_return_distance_c, sizeof(__pyx_k_count_only_and_return_distance_c), 0, 0, 1, 0},
  {&__pyx_n_s_d, __pyx_k_d, sizeof(__pyx_k_d), 0, 0, 1, 1},
  {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
  {&__pyx_n_s_dist_metrics, __pyx_k_dist_metrics, sizeof(__pyx_k_dist_metrics), 0, 0, 1, 1},
  {&__pyx_n_s_distances, __pyx_k_distances, sizeof(__pyx_k_distances), 0, 0, 1, 1},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_dtype, __pyx_k_dtype, sizeof(__pyx_k_dtype), 0, 0, 1, 1},
  {&__pyx_n_s_dtype_is_object, __pyx_k_dtype_is_object, sizeof(__pyx_k_dtype_is_object), 0, 0, 1, 1},
  {&__pyx_n_s_dualtree, __pyx_k_dualtree, sizeof(__pyx_k_dualtree), 0, 0, 1, 1},
  {&__pyx_n_s_empty, __pyx_k_empty, sizeof(__pyx_k_empty), 0, 0, 1, 1},
  {&__pyx_n_s_enumerate, __pyx_k_enumerate, sizeof(__pyx_k_enumerate), 0, 0, 1, 1},
  {&__pyx_n_s_epanechnikov, __pyx_k_epanechnikov, sizeof(__pyx_k_epanechnikov), 0, 0, 1, 1},
  {&__pyx_n_s_error, __pyx_k_error, sizeof(__pyx_k_error), 0, 0, 1, 1},
  {&__pyx_n_s_exp, __pyx_k_exp, sizeof(__pyx_k_exp), 0, 0, 1, 1},
  {&__pyx_n_s_exponential, __pyx_k_exponential, sizeof(__pyx_k_exponential), 0, 0, 1, 1},
  {&__pyx_n_s_flags, __pyx_k_flags, sizeof(__pyx_k_flags), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_formats, __pyx_k_formats, sizeof(__pyx_k_formats), 0, 0, 1, 1},
  {&__pyx_n_s_fortran, __pyx_k_fortran, sizeof(__pyx_k_fortran), 0, 0, 1, 1},
  {&__pyx_n_u_fortran, __pyx_k_fortran, sizeof(__pyx_k_fortran), 0, 1, 0, 1},
  {&__pyx_n_s_gaussian, __pyx_k_gaussian, sizeof(__pyx_k_gaussian), 0, 0, 1, 1},
  {&__pyx_n_s_get_arrays, __pyx_k_get_arrays, sizeof(__pyx_k_get_arrays), 0, 0, 1, 1},
  {&__pyx_n_s_get_metric, __pyx_k_get_metric, sizeof(__pyx_k_get_metric), 0, 0, 1, 1},
  {&__pyx_n_s_get_valid_metric_ids, __pyx_k_get_valid_metric_ids, sizeof(__pyx_k_get_valid_metric_ids), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_kp_s_got_differing_extents_in_dimensi, __pyx_k_got_differing_extents_in_dimensi, sizeof(__pyx_k_got_differing_extents_in_dimensi), 0, 0, 1, 0},
  {&__pyx_n_s_h, __pyx_k_h, sizeof(__pyx_k_h), 0, 0, 1, 1},
  {&__pyx_n_s_heap, __pyx_k_heap, sizeof(__pyx_k_heap), 0, 0, 1, 1},
  {&__pyx_kp_s_home_varoquau_dev_scikit_learn, __pyx_k_home_varoquau_dev_scikit_learn, sizeof(__pyx_k_home_varoquau_dev_scikit_learn), 0, 0, 1, 0},
  {&__pyx_n_s_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 0, 1, 1},
  {&__pyx_n_s_i1, __pyx_k_i1, sizeof(__pyx_k_i1), 0, 0, 1, 1},
  {&__pyx_n_s_i2, __pyx_k_i2, sizeof(__pyx_k_i2), 0, 0, 1, 1},
  {&__pyx_n_s_i_val, __pyx_k_i_val, sizeof(__pyx_k_i_val), 0, 0, 1, 1},
  {&__pyx_n_s_id, __pyx_k_id, sizeof(__pyx_k_id), 0, 0, 1, 1},
  {&__pyx_n_s_idx_end, __pyx_k_idx_end, sizeof(__pyx_k_idx_end), 0, 0, 1, 1},
  {&__pyx_n_s_idx_start, __pyx_k_idx_start, sizeof(__pyx_k_idx_start), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_indices, __pyx_k_indices, sizeof(__pyx_k_indices), 0, 0, 1, 1},
  {&__pyx_n_s_inf, __pyx_k_inf, sizeof(__pyx_k_inf), 0, 0, 1, 1},
  {&__pyx_n_s_is_leaf, __pyx_k_is_leaf, sizeof(__pyx_k_is_leaf), 0, 0, 1, 1},
  {&__pyx_n_s_itemsize, __pyx_k_itemsize, sizeof(__pyx_k_itemsize), 0, 0, 1, 1},
  {&__pyx_kp_s_itemsize_0_for_cython_array, __pyx_k_itemsize_0_for_cython_array, sizeof(__pyx_k_itemsize_0_for_cython_array), 0, 0, 1, 0},
  {&__pyx_n_s_j, __pyx_k_j, sizeof(__pyx_k_j), 0, 0, 1, 1},
  {&__pyx_n_s_join, __pyx_k_join, sizeof(__pyx_k_join), 0, 0, 1, 1},
  {&__pyx_n_s_k, __pyx_k_k, sizeof(__pyx_k_k), 0, 0, 1, 1},
  {&__pyx_kp_s_k_must_be_less_than_or_equal_to, __pyx_k_k_must_be_less_than_or_equal_to, sizeof(__pyx_k_k_must_be_less_than_or_equal_to), 0, 0, 1, 0},
  {&__pyx_n_s_kd_tree, __pyx_k_kd_tree, sizeof(__pyx_k_kd_tree), 0, 0, 1, 1},
  {&__pyx_n_s_kernel, __pyx_k_kernel, sizeof(__pyx_k_kernel), 0, 0, 1, 1},
  {&__pyx_kp_u_kernel_density_self_X_h_kernel, __pyx_k_kernel_density_self_X_h_kernel, sizeof(__pyx_k_kernel_density_self_X_h_kernel), 0, 1, 0, 0},
  {&__pyx_n_s_kernel_norm, __pyx_k_kernel_norm, sizeof(__pyx_k_kernel_norm), 0, 0, 1, 1},
  {&__pyx_kp_s_kernel_not_recognized, __pyx_k_kernel_not_recognized, sizeof(__pyx_k_kernel_not_recognized), 0, 0, 1, 0},
  {&__pyx_kp_s_kernel_s_not_recognized, __pyx_k_kernel_s_not_recognized, sizeof(__pyx_k_kernel_s_not_recognized), 0, 0, 1, 0},
  {&__pyx_n_s_leaf_size, __pyx_k_leaf_size, sizeof(__pyx_k_leaf_size), 0, 0, 1, 1},
  {&__pyx_kp_s_leaf_size_must_be_greater_than_o, __pyx_k_leaf_size_must_be_greater_than_o, sizeof(__pyx_k_leaf_size_must_be_greater_than_o), 0, 0, 1, 0},
  {&__pyx_n_s_linear, __pyx_k_linear, sizeof(__pyx_k_linear), 0, 0, 1, 1},
  {&__pyx_n_s_load_heap, __pyx_k_load_heap, sizeof(__pyx_k_load_heap), 0, 0, 1, 1},
  {&__pyx_n_s_log2, __pyx_k_log2, sizeof(__pyx_k_log2), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_memview, __pyx_k_memview, sizeof(__pyx_k_memview), 0, 0, 1, 1},
  {&__pyx_n_s_metric, __pyx_k_metric, sizeof(__pyx_k_metric), 0, 0, 1, 1},
  {&__pyx_kp_s_metric_metric_is_not_valid_for_B, __pyx_k_metric_metric_is_not_valid_for_B, sizeof(__pyx_k_metric_metric_is_not_valid_for_B), 0, 0, 1, 0},
  {&__pyx_n_s_minkowski, __pyx_k_minkowski, sizeof(__pyx_k_minkowski), 0, 0, 1, 1},
  {&__pyx_n_s_mode, __pyx_k_mode, sizeof(__pyx_k_mode), 0, 0, 1, 1},
  {&__pyx_n_s_n_nbrs, __pyx_k_n_nbrs, sizeof(__pyx_k_n_nbrs), 0, 0, 1, 1},
  {&__pyx_n_s_n_pts, __pyx_k_n_pts, sizeof(__pyx_k_n_pts), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
  {&__pyx_n_s_names, __pyx_k_names, sizeof(__pyx_k_names), 0, 0, 1, 1},
  {&__pyx_kp_u_ndarray_is_not_C_contiguous, __pyx_k_ndarray_is_not_C_contiguous, sizeof(__pyx_k_ndarray_is_not_C_contiguous), 0, 1, 0, 0},
  {&__pyx_kp_u_ndarray_is_not_Fortran_contiguou, __pyx_k_ndarray_is_not_Fortran_contiguou, sizeof(__pyx_k_ndarray_is_not_Fortran_contiguou), 0, 1, 0, 0},
  {&__pyx_n_s_ndim, __pyx_k_ndim, sizeof(__pyx_k_ndim), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_newObj, __pyx_k_newObj, sizeof(__pyx_k_newObj), 0, 0, 1, 1},
  {&__pyx_n_s_node_data, __pyx_k_node_data, sizeof(__pyx_k_node_data), 0, 0, 1, 1},
  {&__pyx_n_s_nodeheap_sort, __pyx_k_nodeheap_sort, sizeof(__pyx_k_nodeheap_sort), 0, 0, 1, 1},
  {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_n_s_obj, __pyx_k_obj, sizeof(__pyx_k_obj), 0, 0, 1, 1},
  {&__pyx_n_s_object, __pyx_k_object, sizeof(__pyx_k_object), 0, 0, 1, 1},
  {&__pyx_n_s_offsets, __pyx_k_offsets, sizeof(__pyx_k_offsets), 0, 0, 1, 1},
  {&__pyx_n_s_order, __pyx_k_order, sizeof(__pyx_k_order), 0, 0, 1, 1},
  {&__pyx_n_s_pack, __pyx_k_pack, sizeof(__pyx_k_pack), 0, 0, 1, 1},
  {&__pyx_n_s_pi, __pyx_k_pi, sizeof(__pyx_k_pi), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_getbuffer, __pyx_k_pyx_getbuffer, sizeof(__pyx_k_pyx_getbuffer), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_kp_u_query_X_k_1_return_distance_Tru, __pyx_k_query_X_k_1_return_distance_Tru, sizeof(__pyx_k_query_X_k_1_return_distance_Tru), 0, 1, 0, 0},
  {&__pyx_kp_s_query_data_dimension_must_match, __pyx_k_query_data_dimension_must_match, sizeof(__pyx_k_query_data_dimension_must_match), 0, 0, 1, 0},
  {&__pyx_kp_u_query_radius_self_X_r_count_onl, __pyx_k_query_radius_self_X_r_count_onl, sizeof(__pyx_k_query_radius_self_X_r_count_onl), 0, 1, 0, 0},
  {&__pyx_n_s_r, __pyx_k_r, sizeof(__pyx_k_r), 0, 0, 1, 1},
  {&__pyx_kp_s_r_must_be_a_1_dimensional_array, __pyx_k_r_must_be_a_1_dimensional_array, sizeof(__pyx_k_r_must_be_a_1_dimensional_array), 0, 0, 1, 0},
  {&__pyx_kp_s_r_must_be_broadcastable_to_X_sha, __pyx_k_r_must_be_broadcastable_to_X_sha, sizeof(__pyx_k_r_must_be_broadcastable_to_X_sha), 0, 0, 1, 0},
  {&__pyx_n_s_radius, __pyx_k_radius, sizeof(__pyx_k_radius), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_rdist_to_dist, __pyx_k_rdist_to_dist, sizeof(__pyx_k_rdist_to_dist), 0, 0, 1, 1},
  {&__pyx_n_s_reshape, __pyx_k_reshape, sizeof(__pyx_k_reshape), 0, 0, 1, 1},
  {&__pyx_n_s_result, __pyx_k_result, sizeof(__pyx_k_result), 0, 0, 1, 1},
  {&__pyx_n_s_return_distance, __pyx_k_return_distance, sizeof(__pyx_k_return_distance), 0, 0, 1, 1},
  {&__pyx_kp_s_return_distance_must_be_True_if, __pyx_k_return_distance_must_be_True_if, sizeof(__pyx_k_return_distance_must_be_True_if), 0, 0, 1, 0},
  {&__pyx_n_s_return_log, __pyx_k_return_log, sizeof(__pyx_k_return_log), 0, 0, 1, 1},
  {&__pyx_n_s_row, __pyx_k_row, sizeof(__pyx_k_row), 0, 0, 1, 1},
  {&__pyx_n_s_rtol, __pyx_k_rtol, sizeof(__pyx_k_rtol), 0, 0, 1, 1},
  {&__pyx_kp_u_s, __pyx_k_s, sizeof(__pyx_k_s), 0, 1, 0, 0},
  {&__pyx_n_s_shape, __pyx_k_shape, sizeof(__pyx_k_shape), 0, 0, 1, 1},
  {&__pyx_n_s_simultaneous_sort, __pyx_k_simultaneous_sort, sizeof(__pyx_k_simultaneous_sort), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_n_s_size_guess, __pyx_k_size_guess, sizeof(__pyx_k_size_guess), 0, 0, 1, 1},
  {&__pyx_n_s_sklearn_neighbors_kd_tree, __pyx_k_sklearn_neighbors_kd_tree, sizeof(__pyx_k_sklearn_neighbors_kd_tree), 0, 0, 1, 1},
  {&__pyx_n_s_sort, __pyx_k_sort, sizeof(__pyx_k_sort), 0, 0, 1, 1},
  {&__pyx_n_s_sort_results, __pyx_k_sort_results, sizeof(__pyx_k_sort_results), 0, 0, 1, 1},
  {&__pyx_n_s_start, __pyx_k_start, sizeof(__pyx_k_start), 0, 0, 1, 1},
  {&__pyx_n_s_step, __pyx_k_step, sizeof(__pyx_k_step), 0, 0, 1, 1},
  {&__pyx_n_s_stop, __pyx_k_stop, sizeof(__pyx_k_stop), 0, 0, 1, 1},
  {&__pyx_kp_s_strided_and_direct, __pyx_k_strided_and_direct, sizeof(__pyx_k_strided_and_direct), 0, 0, 1, 0},
  {&__pyx_kp_s_strided_and_direct_or_indirect, __pyx_k_strided_and_direct_or_indirect, sizeof(__pyx_k_strided_and_direct_or_indirect), 0, 0, 1, 0},
  {&__pyx_kp_s_strided_and_indirect, __pyx_k_strided_and_indirect, sizeof(__pyx_k_strided_and_indirect), 0, 0, 1, 0},
  {&__pyx_n_s_struct, __pyx_k_struct, sizeof(__pyx_k_struct), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_tophat, __pyx_k_tophat, sizeof(__pyx_k_tophat), 0, 0, 1, 1},
  {&__pyx_n_s_typedefs, __pyx_k_typedefs, sizeof(__pyx_k_typedefs), 0, 0, 1, 1},
  {&__pyx_kp_s_unable_to_allocate_array_data, __pyx_k_unable_to_allocate_array_data, sizeof(__pyx_k_unable_to_allocate_array_data), 0, 0, 1, 0},
  {&__pyx_kp_s_unable_to_allocate_shape_and_str, __pyx_k_unable_to_allocate_shape_and_str, sizeof(__pyx_k_unable_to_allocate_shape_and_str), 0, 0, 1, 0},
  {&__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_k_unknown_dtype_code_in_numpy_pxd, sizeof(__pyx_k_unknown_dtype_code_in_numpy_pxd), 0, 1, 0, 0},
  {&__pyx_n_s_unpack, __pyx_k_unpack, sizeof(__pyx_k_unpack), 0, 0, 1, 1},
  {&__pyx_n_s_utils, __pyx_k_utils, sizeof(__pyx_k_utils), 0, 0, 1, 1},
  {&__pyx_n_s_val, __pyx_k_val, sizeof(__pyx_k_val), 0, 0, 1, 1},
  {&__pyx_n_s_valid_metrics, __pyx_k_valid_metrics, sizeof(__pyx_k_valid_metrics), 0, 0, 1, 1},
  {&__pyx_n_s_vals, __pyx_k_vals, sizeof(__pyx_k_vals), 0, 0, 1, 1},
  {&__pyx_n_s_vals_sorted, __pyx_k_vals_sorted, sizeof(__pyx_k_vals_sorted), 0, 0, 1, 1},
  {&__pyx_n_s_warn, __pyx_k_warn, sizeof(__pyx_k_warn), 0, 0, 1, 1},
  {&__pyx_n_s_warnings, __pyx_k_warnings, sizeof(__pyx_k_warnings), 0, 0, 1, 1},
  {&__pyx_n_s_xrange, __pyx_k_xrange, sizeof(__pyx_k_xrange), 0, 0, 1, 1},
  {&__pyx_n_s_zeros, __pyx_k_zeros, sizeof(__pyx_k_zeros), 0, 0, 1, 1},
  {&__pyx_n_s_zeros_like, __pyx_k_zeros_like, sizeof(__pyx_k_zeros_like), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 493; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_RuntimeError = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeError); if (!__pyx_builtin_RuntimeError) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 142; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_n_s_enumerate); if (!__pyx_builtin_enumerate) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 145; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_Ellipsis = __Pyx_GetBuiltinName(__pyx_n_s_Ellipsis); if (!__pyx_builtin_Ellipsis) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 386; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if PY_MAJOR_VERSION >= 3
  __pyx_builtin_xrange = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_xrange) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 514; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #else
  __pyx_builtin_xrange = __Pyx_GetBuiltinName(__pyx_n_s_xrange); if (!__pyx_builtin_xrange) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 514; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  __pyx_builtin_id = __Pyx_GetBuiltinName(__pyx_n_s_id); if (!__pyx_builtin_id) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 569; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_IndexError = __Pyx_GetBuiltinName(__pyx_n_s_IndexError); if (!__pyx_builtin_IndexError) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 789; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "binary_tree.pxi":493
 *         factor = log(factor) + logSn(d - 1)
 *     else:
 *         raise ValueError("Kernel code not recognized")             # <<<<<<<<<<<<<<
 *     return -factor - d * log(h)
 * 
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_Kernel_code_not_recognized); if (unlikely(!__pyx_tuple_)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 493; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "binary_tree.pxi":531
 *         result = _log_kernel_norm(h, d, COSINE_KERNEL)
 *     else:
 *         raise ValueError('kernel not recognized')             # <<<<<<<<<<<<<<
 * 
 *     if return_log:
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_kernel_not_recognized); if (unlikely(!__pyx_tuple__2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "binary_tree.pxi":582
 * 
 *     def __cinit__(self):
 *         self.distances_arr = np.zeros((1, 1), dtype=DTYPE, order='C')             # <<<<<<<<<<<<<<
 *         self.indices_arr = np.zeros((1, 1), dtype=ITYPE, order='C')
 *         self.distances = get_memview_DTYPE_2D(self.distances_arr)
 */
  __pyx_tuple__3 = PyTuple_Pack(2, __pyx_int_1, __pyx_int_1); if (unlikely(!__pyx_tuple__3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 582; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_tuple__3); if (unlikely(!__pyx_tuple__4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 582; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "binary_tree.pxi":583
 *     def __cinit__(self):
 *         self.distances_arr = np.zeros((1, 1), dtype=DTYPE, order='C')
 *         self.indices_arr = np.zeros((1, 1), dtype=ITYPE, order='C')             # <<<<<<<<<<<<<<
 *         self.distances = get_memview_DTYPE_2D(self.distances_arr)
 *         self.indices = get_memview_ITYPE_2D(self.indices_arr)
 */
  __pyx_tuple__5 = PyTuple_Pack(2, __pyx_int_1, __pyx_int_1); if (unlikely(!__pyx_tuple__5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_tuple__5); if (unlikely(!__pyx_tuple__6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 583; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "binary_tree.pxi":882
 * 
 *     def __cinit__(self):
 *         self.data_arr = np.zeros(1, dtype=NodeHeapData, order='C')             # <<<<<<<<<<<<<<
 *         self.data = get_memview_NodeHeapData_1D(self.data_arr)
 * 
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_int_1); if (unlikely(!__pyx_tuple__7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "binary_tree.pxi":946
 *         """Remove the root of the heap, and update the remaining nodes"""
 *         if self.n == 0:
 *             raise ValueError('cannot pop on empty heap')             # <<<<<<<<<<<<<<
 * 
 *         cdef ITYPE_t i, i_child1, i_child2, i_swap
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s_cannot_pop_on_empty_heap); if (unlikely(!__pyx_tuple__8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "binary_tree.pxi":1032
 *     # errors and seg-faults in rare cases where __init__ is not called
 *     def __cinit__(self):
 *         self.data_arr = np.empty((1, 1), dtype=DTYPE, order='C')             # <<<<<<<<<<<<<<
 *         self.idx_array_arr = np.empty(1, dtype=ITYPE, order='C')
 *         self.node_data_arr = np.empty(1, dtype=NodeData, order='C')
 */
  __pyx_tuple__9 = PyTuple_Pack(2, __pyx_int_1, __pyx_int_1); if (unlikely(!__pyx_tuple__9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1032; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_tuple__9); if (unlikely(!__pyx_tuple__10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1032; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "binary_tree.pxi":1033
 *     def __cinit__(self):
 *         self.data_arr = np.empty((1, 1), dtype=DTYPE, order='C')
 *         self.idx_array_arr = np.empty(1, dtype=ITYPE, order='C')             # <<<<<<<<<<<<<<
 *         self.node_data_arr = np.empty(1, dtype=NodeData, order='C')
 *         self.node_bounds_arr = np.empty((1, 1, 1), dtype=DTYPE)
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_int_1); if (unlikely(!__pyx_tuple__11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1033; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "binary_tree.pxi":1034
 *         self.data_arr = np.empty((1, 1), dtype=DTYPE, order='C')
 *         self.idx_array_arr = np.empty(1, dtype=ITYPE, order='C')
 *         self.node_data_arr = np.empty(1, dtype=NodeData, order='C')             # <<<<<<<<<<<<<<
 *         self.node_bounds_arr = np.empty((1, 1, 1), dtype=DTYPE)
 * 
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_int_1); if (unlikely(!__pyx_tuple__12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1034; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "binary_tree.pxi":1035
 *         self.idx_array_arr = np.empty(1, dtype=ITYPE, order='C')
 *         self.node_data_arr = np.empty(1, dtype=NodeData, order='C')
 *         self.node_bounds_arr = np.empty((1, 1, 1), dtype=DTYPE)             # <<<<<<<<<<<<<<
 * 
 *         self.data = get_memview_DTYPE_2D(self.data_arr)
 */
  __pyx_tuple__13 = PyTuple_Pack(3, __pyx_int_1, __pyx_int_1, __pyx_int_1); if (unlikely(!__pyx_tuple__13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_tuple__13); if (unlikely(!__pyx_tuple__14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "binary_tree.pxi":1071
 *         # validate data
 *         if self.data.size == 0:
 *             raise ValueError("X is an empty array")             # <<<<<<<<<<<<<<
 * 
 *         if leaf_size < 1:
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_kp_s_X_is_an_empty_array); if (unlikely(!__pyx_tuple__15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1071; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "binary_tree.pxi":1074
 * 
 *         if leaf_size < 1:
 *             raise ValueError("leaf_size must be greater than or equal to 1")             # <<<<<<<<<<<<<<
 * 
 *         n_samples = self.data.shape[0]
 */
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_kp_s_leaf_size_must_be_greater_than_o); if (unlikely(!__pyx_tuple__16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1074; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "binary_tree.pxi":1210
 *                 # warning saying we're doing so.
 *                 import warnings
 *                 warnings.warn("Internal: memory layout is flawed: "             # <<<<<<<<<<<<<<
 *                               "not enough nodes allocated")
 * 
 */
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_kp_s_Internal_memory_layout_is_flawed); if (unlikely(!__pyx_tuple__17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1210; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "binary_tree.pxi":1217
 *             # is correct.  Raise a warning.
 *             import warnings
 *             warnings.warn("Internal: memory layout is flawed: "             # <<<<<<<<<<<<<<
 *                           "too many nodes allocated")
 *             self.node_data[i_node].is_leaf = True
 */
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_kp_s_Internal_memory_layout_is_flawed_2); if (unlikely(!__pyx_tuple__18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1217; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "binary_tree.pxi":1295
 * 
 *         if X.shape[X.ndim - 1] != self.data.shape[1]:
 *             raise ValueError("query data dimension must "             # <<<<<<<<<<<<<<
 *                              "match training data dimension")
 * 
 */
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_kp_s_query_data_dimension_must_match); if (unlikely(!__pyx_tuple__19)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1295; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "binary_tree.pxi":1299
 * 
 *         if self.data.shape[0] < k:
 *             raise ValueError("k must be less than or equal "             # <<<<<<<<<<<<<<
 *                              "to the number of training points")
 * 
 */
  __pyx_tuple__20 = PyTuple_Pack(1, __pyx_kp_s_k_must_be_less_than_or_equal_to); if (unlikely(!__pyx_tuple__20)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);

  /* "binary_tree.pxi":1426
 *         """
 *         if count_only and return_distance:
 *             raise ValueError("count_only and return_distance "             # <<<<<<<<<<<<<<
 *                              "cannot both be true")
 * 
 */
  __pyx_tuple__21 = PyTuple_Pack(1, __pyx_kp_s_count_only_and_return_distance_c); if (unlikely(!__pyx_tuple__21)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1426; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);

  /* "binary_tree.pxi":1430
 * 
 *         if sort_results and not return_distance:
 *             raise ValueError("return_distance must be True "             # <<<<<<<<<<<<<<
 *                              "if sort_results is True")
 * 
 */
  __pyx_tuple__22 = PyTuple_Pack(1, __pyx_kp_s_return_distance_must_be_True_if); if (unlikely(!__pyx_tuple__22)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1430; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);

  /* "binary_tree.pxi":1443
 * 
 *         if X.shape[X.ndim - 1] != self.data.shape[1]:
 *             raise ValueError("query data dimension must "             # <<<<<<<<<<<<<<
 *                              "match training data dimension")
 * 
 */
  __pyx_tuple__23 = PyTuple_Pack(1, __pyx_kp_s_query_data_dimension_must_match); if (unlikely(!__pyx_tuple__23)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1443; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);

  /* "binary_tree.pxi":1452
 *         r = np.asarray(r, dtype=DTYPE, order='C')
 *         r = np.atleast_1d(r)
 *         if r.shape == (1,):             # <<<<<<<<<<<<<<
 *             r = r[0] + np.zeros(X.shape[:X.ndim - 1], dtype=DTYPE)
 *         else:
 */
  __pyx_tuple__24 = PyTuple_Pack(1, __pyx_int_1); if (unlikely(!__pyx_tuple__24)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1452; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);

  /* "binary_tree.pxi":1456
 *         else:
 *             if r.shape != X.shape[:X.ndim - 1]:
 *                 raise ValueError("r must be broadcastable to X.shape")             # <<<<<<<<<<<<<<
 * 
 *         rarr_np = r.reshape(-1)  # store explicitly to keep in scope
 */
  __pyx_tuple__25 = PyTuple_Pack(1, __pyx_kp_s_r_must_be_broadcastable_to_X_sha); if (unlikely(!__pyx_tuple__25)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1456; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);

  /* "binary_tree.pxi":1458
 *                 raise ValueError("r must be broadcastable to X.shape")
 * 
 *         rarr_np = r.reshape(-1)  # store explicitly to keep in scope             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t[::1] rarr = get_memview_DTYPE_1D(rarr_np)
 * 
 */
  __pyx_tuple__26 = PyTuple_Pack(1, __pyx_int_neg_1); if (unlikely(!__pyx_tuple__26)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1458; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);

  /* "binary_tree.pxi":1593
 * 
 *         if X.shape[X.ndim - 1] != n_features:
 *             raise ValueError("query data dimension must "             # <<<<<<<<<<<<<<
 *                              "match training data dimension")
 *         Xarr_np = X.reshape((-1, n_features))
 */
  __pyx_tuple__27 = PyTuple_Pack(1, __pyx_kp_s_query_data_dimension_must_match); if (unlikely(!__pyx_tuple__27)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1593; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);

  /* "binary_tree.pxi":1696
 * 
 *         if X.shape[X.ndim - 1] != self.data.shape[1]:
 *             raise ValueError("query data dimension must "             # <<<<<<<<<<<<<<
 *                              "match training data dimension")
 * 
 */
  __pyx_tuple__28 = PyTuple_Pack(1, __pyx_kp_s_query_data_dimension_must_match); if (unlikely(!__pyx_tuple__28)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);

  /* "binary_tree.pxi":1706
 *         r = np.atleast_1d(r)
 *         if r.ndim != 1:
 *             raise ValueError("r must be a 1-dimensional array")             # <<<<<<<<<<<<<<
 *         i_rsort = np.argsort(r)
 *         rarr_np = r[i_rsort]  # needed to keep memory in scope
 */
  __pyx_tuple__29 = PyTuple_Pack(1, __pyx_kp_s_r_must_be_a_1_dimensional_array); if (unlikely(!__pyx_tuple__29)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1706; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);

  /* "binary_tree.pxi":2054
 *                 for i in range(node_info.idx_start, node_info.idx_end):
 *                     if (count < 0) or (count >= self.data.shape[0]):
 *                         raise ValueError("Fatal: count too big: "             # <<<<<<<<<<<<<<
 *                                          "this should never happen")
 *                     indices[count] = idx_array[i]
 */
  __pyx_tuple__30 = PyTuple_Pack(1, __pyx_kp_s_Fatal_count_too_big_this_should); if (unlikely(!__pyx_tuple__30)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);

  /* "binary_tree.pxi":2074
 *                 if dist_pt <= reduced_r:
 *                     if (count < 0) or (count >= self.data.shape[0]):
 *                         raise ValueError("Fatal: count out of range. "             # <<<<<<<<<<<<<<
 *                                          "This should never happen.")
 *                     if count_only:
 */
  __pyx_tuple__31 = PyTuple_Pack(1, __pyx_kp_s_Fatal_count_out_of_range_This_sh); if (unlikely(!__pyx_tuple__31)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2074; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);

  /* "binary_tree.pxi":2506
 * 
 *     # use initial size 0 to check corner case
 *     cdef NodeHeap heap = NodeHeap(0)             # <<<<<<<<<<<<<<
 *     cdef NodeHeapData_t data
 *     cdef ITYPE_t i
 */
  __pyx_tuple__32 = PyTuple_Pack(1, __pyx_int_0); if (unlikely(!__pyx_tuple__32)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2506; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__32);
  __Pyx_GIVEREF(__pyx_tuple__32);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":215
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
  __pyx_tuple__33 = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_C_contiguous); if (unlikely(!__pyx_tuple__33)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__33);
  __Pyx_GIVEREF(__pyx_tuple__33);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":219
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
  __pyx_tuple__34 = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_Fortran_contiguou); if (unlikely(!__pyx_tuple__34)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":257
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
  __pyx_tuple__35 = PyTuple_Pack(1, __pyx_kp_u_Non_native_byte_order_not_suppor); if (unlikely(!__pyx_tuple__35)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__35);
  __Pyx_GIVEREF(__pyx_tuple__35);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":799
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
  __pyx_tuple__36 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor); if (unlikely(!__pyx_tuple__36)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__36);
  __Pyx_GIVEREF(__pyx_tuple__36);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":803
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *             # One could encode it in the format string and have Cython
 *             # complain instead, BUT: < and > in format strings also imply
 */
  __pyx_tuple__37 = PyTuple_Pack(1, __pyx_kp_u_Non_native_byte_order_not_suppor); if (unlikely(!__pyx_tuple__37)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__37);
  __Pyx_GIVEREF(__pyx_tuple__37);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":823
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
  __pyx_tuple__38 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor_2); if (unlikely(!__pyx_tuple__38)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__38);
  __Pyx_GIVEREF(__pyx_tuple__38);

  /* "View.MemoryView":127
 * 
 *         if not self.ndim:
 *             raise ValueError("Empty shape tuple for cython.array")             # <<<<<<<<<<<<<<
 * 
 *         if itemsize <= 0:
 */
  __pyx_tuple__39 = PyTuple_Pack(1, __pyx_kp_s_Empty_shape_tuple_for_cython_arr); if (unlikely(!__pyx_tuple__39)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__39);
  __Pyx_GIVEREF(__pyx_tuple__39);

  /* "View.MemoryView":130
 * 
 *         if itemsize <= 0:
 *             raise ValueError("itemsize <= 0 for cython.array")             # <<<<<<<<<<<<<<
 * 
 *         if isinstance(format, unicode):
 */
  __pyx_tuple__40 = PyTuple_Pack(1, __pyx_kp_s_itemsize_0_for_cython_array); if (unlikely(!__pyx_tuple__40)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 130; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__40);
  __Pyx_GIVEREF(__pyx_tuple__40);

  /* "View.MemoryView":142
 * 
 *         if not self._shape:
 *             raise MemoryError("unable to allocate shape and strides.")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__41 = PyTuple_Pack(1, __pyx_kp_s_unable_to_allocate_shape_and_str); if (unlikely(!__pyx_tuple__41)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 142; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__41);
  __Pyx_GIVEREF(__pyx_tuple__41);

  /* "View.MemoryView":170
 *             self.data = <char *>malloc(self.len)
 *             if not self.data:
 *                 raise MemoryError("unable to allocate array data.")             # <<<<<<<<<<<<<<
 * 
 *             if self.dtype_is_object:
 */
  __pyx_tuple__42 = PyTuple_Pack(1, __pyx_kp_s_unable_to_allocate_array_data); if (unlikely(!__pyx_tuple__42)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__42);
  __Pyx_GIVEREF(__pyx_tuple__42);

  /* "View.MemoryView":186
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")             # <<<<<<<<<<<<<<
 *         info.buf = self.data
 *         info.len = self.len
 */
  __pyx_tuple__43 = PyTuple_Pack(1, __pyx_kp_s_Can_only_create_a_buffer_that_is); if (unlikely(!__pyx_tuple__43)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__43);
  __Pyx_GIVEREF(__pyx_tuple__43);

  /* "View.MemoryView":445
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 *             raise ValueError("Unable to convert item to object")             # <<<<<<<<<<<<<<
 *         else:
 *             if len(self.view.format) == 1:
 */
  __pyx_tuple__44 = PyTuple_Pack(1, __pyx_kp_s_Unable_to_convert_item_to_object); if (unlikely(!__pyx_tuple__44)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__44);
  __Pyx_GIVEREF(__pyx_tuple__44);

  /* "View.MemoryView":521
 *             if self.view.strides == NULL:
 * 
 *                 raise ValueError("Buffer view does not expose strides")             # <<<<<<<<<<<<<<
 * 
 *             return tuple([self.view.strides[i] for i in xrange(self.view.ndim)])
 */
  __pyx_tuple__45 = PyTuple_Pack(1, __pyx_kp_s_Buffer_view_does_not_expose_stri); if (unlikely(!__pyx_tuple__45)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 521; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__45);
  __Pyx_GIVEREF(__pyx_tuple__45);

  /* "View.MemoryView":638
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))             # <<<<<<<<<<<<<<
 *                 seen_ellipsis = True
 *             else:
 */
  __pyx_slice__46 = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_slice__46)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 638; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_slice__46);
  __Pyx_GIVEREF(__pyx_slice__46);

  /* "View.MemoryView":641
 *                 seen_ellipsis = True
 *             else:
 *                 result.append(slice(None))             # <<<<<<<<<<<<<<
 *             have_slices = True
 *         else:
 */
  __pyx_slice__47 = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_slice__47)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 641; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_slice__47);
  __Pyx_GIVEREF(__pyx_slice__47);

  /* "View.MemoryView":652
 *     nslices = ndim - len(result)
 *     if nslices:
 *         result.extend([slice(None)] * nslices)             # <<<<<<<<<<<<<<
 * 
 *     return have_slices or nslices, tuple(result)
 */
  __pyx_slice__48 = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_slice__48)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 652; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_slice__48);
  __Pyx_GIVEREF(__pyx_slice__48);

  /* "View.MemoryView":660
 *     for i in range(ndim):
 *         if suboffsets[i] >= 0:
 *             raise ValueError("Indirect dimensions not supported")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__49 = PyTuple_Pack(1, __pyx_kp_s_Indirect_dimensions_not_supporte); if (unlikely(!__pyx_tuple__49)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 660; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__49);
  __Pyx_GIVEREF(__pyx_tuple__49);

  /* "binary_tree.pxi":497
 * 
 * 
 * def kernel_norm(h, d, kernel, return_log=False):             # <<<<<<<<<<<<<<
 *     """Given a string specification of a kernel, compute the normalization.
 * 
 */
  __pyx_tuple__55 = PyTuple_Pack(5, __pyx_n_s_h, __pyx_n_s_d, __pyx_n_s_kernel, __pyx_n_s_return_log, __pyx_n_s_result); if (unlikely(!__pyx_tuple__55)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 497; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__55);
  __Pyx_GIVEREF(__pyx_tuple__55);
  __pyx_codeobj__56 = (PyObject*)__Pyx_PyCode_New(4, 0, 5, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__55, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_varoquau_dev_scikit_learn, __pyx_n_s_kernel_norm, 497, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__56)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 497; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "binary_tree.pxi":990
 * # newObj function
 * #  this is a helper function for pickling
 * def newObj(obj):             # <<<<<<<<<<<<<<
 *     return obj.__new__(obj)
 * 
 */
  __pyx_tuple__57 = PyTuple_Pack(1, __pyx_n_s_obj); if (unlikely(!__pyx_tuple__57)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 990; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__57);
  __Pyx_GIVEREF(__pyx_tuple__57);
  __pyx_codeobj__58 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__57, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_varoquau_dev_scikit_learn, __pyx_n_s_newObj, 990, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__58)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 990; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "binary_tree.pxi":2474
 * # Python functions for benchmarking and testing C implementations
 * 
 * def load_heap(DTYPE_t[:, ::1] X, ITYPE_t k):             # <<<<<<<<<<<<<<
 *     """test fully loading the heap"""
 *     assert k <= X.shape[1]
 */
  __pyx_tuple__59 = PyTuple_Pack(5, __pyx_n_s_X, __pyx_n_s_k, __pyx_n_s_heap, __pyx_n_s_i, __pyx_n_s_j); if (unlikely(!__pyx_tuple__59)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__59);
  __Pyx_GIVEREF(__pyx_tuple__59);
  __pyx_codeobj__60 = (PyObject*)__Pyx_PyCode_New(2, 0, 5, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__59, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_varoquau_dev_scikit_learn, __pyx_n_s_load_heap, 2474, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__60)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "binary_tree.pxi":2485
 * 
 * 
 * def simultaneous_sort(DTYPE_t[:, ::1] distances, ITYPE_t[:, ::1] indices):             # <<<<<<<<<<<<<<
 *     """In-place simultaneous sort the given row of the arrays
 * 
 */
  __pyx_tuple__61 = PyTuple_Pack(3, __pyx_n_s_distances, __pyx_n_s_indices, __pyx_n_s_row); if (unlikely(!__pyx_tuple__61)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2485; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__61);
  __Pyx_GIVEREF(__pyx_tuple__61);
  __pyx_codeobj__62 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__61, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_varoquau_dev_scikit_learn, __pyx_n_s_simultaneous_sort, 2485, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__62)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2485; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "binary_tree.pxi":2500
 * 
 * 
 * def nodeheap_sort(DTYPE_t[::1] vals):             # <<<<<<<<<<<<<<
 *     """In-place reverse sort of vals using NodeHeap"""
 *     cdef ITYPE_t[::1] indices = np.zeros(vals.shape[0], dtype=ITYPE)
 */
  __pyx_tuple__63 = PyTuple_Pack(7, __pyx_n_s_vals, __pyx_n_s_vals, __pyx_n_s_indices, __pyx_n_s_vals_sorted, __pyx_n_s_heap, __pyx_n_s_data, __pyx_n_s_i); if (unlikely(!__pyx_tuple__63)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2500; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__63);
  __Pyx_GIVEREF(__pyx_tuple__63);
  __pyx_codeobj__64 = (PyObject*)__Pyx_PyCode_New(1, 0, 7, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__63, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_varoquau_dev_scikit_learn, __pyx_n_s_nodeheap_sort, 2500, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__64)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2500; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "View.MemoryView":276
 *         return self.name
 * 
 * cdef generic = Enum("<strided and direct or indirect>")             # <<<<<<<<<<<<<<
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")
 */
  __pyx_tuple__65 = PyTuple_Pack(1, __pyx_kp_s_strided_and_direct_or_indirect); if (unlikely(!__pyx_tuple__65)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 276; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__65);
  __Pyx_GIVEREF(__pyx_tuple__65);

  /* "View.MemoryView":277
 * 
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default             # <<<<<<<<<<<<<<
 * cdef indirect = Enum("<strided and indirect>")
 * 
 */
  __pyx_tuple__66 = PyTuple_Pack(1, __pyx_kp_s_strided_and_direct); if (unlikely(!__pyx_tuple__66)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 277; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__66);
  __Pyx_GIVEREF(__pyx_tuple__66);

  /* "View.MemoryView":278
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__67 = PyTuple_Pack(1, __pyx_kp_s_strided_and_indirect); if (unlikely(!__pyx_tuple__67)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__67);
  __Pyx_GIVEREF(__pyx_tuple__67);

  /* "View.MemoryView":281
 * 
 * 
 * cdef contiguous = Enum("<contiguous and direct>")             # <<<<<<<<<<<<<<
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")
 * 
 */
  __pyx_tuple__68 = PyTuple_Pack(1, __pyx_kp_s_contiguous_and_direct); if (unlikely(!__pyx_tuple__68)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 281; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__68);
  __Pyx_GIVEREF(__pyx_tuple__68);

  /* "View.MemoryView":282
 * 
 * cdef contiguous = Enum("<contiguous and direct>")
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__69 = PyTuple_Pack(1, __pyx_kp_s_contiguous_and_indirect); if (unlikely(!__pyx_tuple__69)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 282; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__69);
  __Pyx_GIVEREF(__pyx_tuple__69);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __pyx_float_1Eneg_8 = PyFloat_FromDouble(1E-8); if (unlikely(!__pyx_float_1Eneg_8)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_int_40 = PyInt_FromLong(40); if (unlikely(!__pyx_int_40)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_int_100 = PyInt_FromLong(100); if (unlikely(!__pyx_int_100)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initkd_tree(void); /*proto*/
PyMODINIT_FUNC initkd_tree(void)
#else
PyMODINIT_FUNC PyInit_kd_tree(void); /*proto*/
PyMODINIT_FUNC PyInit_kd_tree(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_kd_tree(void)", 0);
  if ( __Pyx_check_binary_version() < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #ifdef __Pyx_CyFunction_USED
  if (__Pyx_CyFunction_init() < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("kd_tree", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  /*--- Initialize various global constants etc. ---*/
  if (unlikely(__Pyx_InitGlobals() < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  if (__pyx_module_is_main_sklearn__neighbors__kd_tree) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!PyDict_GetItemString(modules, "sklearn.neighbors.kd_tree")) {
      if (unlikely(PyDict_SetItemString(modules, "sklearn.neighbors.kd_tree", __pyx_m) < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (unlikely(__Pyx_InitCachedBuiltins() < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Constants init code ---*/
  if (unlikely(__Pyx_InitCachedConstants() < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Global init code ---*/
  generic = Py_None; Py_INCREF(Py_None);
  strided = Py_None; Py_INCREF(Py_None);
  indirect = Py_None; Py_INCREF(Py_None);
  contiguous = Py_None; Py_INCREF(Py_None);
  indirect_contiguous = Py_None; Py_INCREF(Py_None);
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  __pyx_vtabptr_7sklearn_9neighbors_7kd_tree_NeighborsHeap = &__pyx_vtable_7sklearn_9neighbors_7kd_tree_NeighborsHeap;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_NeighborsHeap.largest = (__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t (*)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t))__pyx_f_7sklearn_9neighbors_7kd_tree_13NeighborsHeap_largest;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_NeighborsHeap._push = (int (*)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t))__pyx_f_7sklearn_9neighbors_7kd_tree_13NeighborsHeap__push;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_NeighborsHeap._sort = (int (*)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *))__pyx_f_7sklearn_9neighbors_7kd_tree_13NeighborsHeap__sort;
  if (PyType_Ready(&__pyx_type_7sklearn_9neighbors_7kd_tree_NeighborsHeap) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 560; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_7sklearn_9neighbors_7kd_tree_NeighborsHeap.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_7sklearn_9neighbors_7kd_tree_NeighborsHeap.tp_dict, __pyx_vtabptr_7sklearn_9neighbors_7kd_tree_NeighborsHeap) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 560; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "NeighborsHeap", (PyObject *)&__pyx_type_7sklearn_9neighbors_7kd_tree_NeighborsHeap) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 560; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_7sklearn_9neighbors_7kd_tree_NeighborsHeap = &__pyx_type_7sklearn_9neighbors_7kd_tree_NeighborsHeap;
  __pyx_vtabptr_7sklearn_9neighbors_7kd_tree_NodeHeap = &__pyx_vtable_7sklearn_9neighbors_7kd_tree_NodeHeap;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_NodeHeap.resize = (int (*)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t))__pyx_f_7sklearn_9neighbors_7kd_tree_8NodeHeap_resize;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_NodeHeap.push = (int (*)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *, struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t))__pyx_f_7sklearn_9neighbors_7kd_tree_8NodeHeap_push;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_NodeHeap.peek = (struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t (*)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *))__pyx_f_7sklearn_9neighbors_7kd_tree_8NodeHeap_peek;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_NodeHeap.pop = (struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t (*)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *))__pyx_f_7sklearn_9neighbors_7kd_tree_8NodeHeap_pop;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_NodeHeap.clear = (void (*)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *))__pyx_f_7sklearn_9neighbors_7kd_tree_8NodeHeap_clear;
  if (PyType_Ready(&__pyx_type_7sklearn_9neighbors_7kd_tree_NodeHeap) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 864; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_7sklearn_9neighbors_7kd_tree_NodeHeap.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_7sklearn_9neighbors_7kd_tree_NodeHeap.tp_dict, __pyx_vtabptr_7sklearn_9neighbors_7kd_tree_NodeHeap) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 864; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "NodeHeap", (PyObject *)&__pyx_type_7sklearn_9neighbors_7kd_tree_NodeHeap) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 864; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_7sklearn_9neighbors_7kd_tree_NodeHeap = &__pyx_type_7sklearn_9neighbors_7kd_tree_NodeHeap;
  __pyx_vtabptr_7sklearn_9neighbors_7kd_tree_BinaryTree = &__pyx_vtable_7sklearn_9neighbors_7kd_tree_BinaryTree;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_BinaryTree.dist = (__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t (*)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t))__pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree_dist;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_BinaryTree.rdist = (__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t (*)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t))__pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree_rdist;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_BinaryTree._recursive_build = (int (*)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t))__pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__recursive_build;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_BinaryTree._query_single_depthfirst = (int (*)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t))__pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__query_single_depthfirst;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_BinaryTree._query_single_breadthfirst = (int (*)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *))__pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__query_single_breadthfirst;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_BinaryTree._query_dual_depthfirst = (int (*)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __Pyx_memviewslice, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t))__pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__query_dual_depthfirst;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_BinaryTree._query_dual_breadthfirst = (int (*)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NeighborsHeap *, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *))__pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__query_dual_breadthfirst;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_BinaryTree._query_radius_single = (__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t (*)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, int, int))__pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__query_radius_single;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_BinaryTree._kde_single_breadthfirst = (__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t (*)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, enum __pyx_t_7sklearn_9neighbors_7kd_tree_KernelType, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_NodeHeap *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *))__pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__kde_single_breadthfirst;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_BinaryTree._kde_single_depthfirst = (int (*)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, enum __pyx_t_7sklearn_9neighbors_7kd_tree_KernelType, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *))__pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__kde_single_depthfirst;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_BinaryTree._two_point_single = (int (*)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t))__pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__two_point_single;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_BinaryTree._two_point_dual = (int (*)(struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, struct __pyx_obj_7sklearn_9neighbors_7kd_tree_BinaryTree *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t))__pyx_f_7sklearn_9neighbors_7kd_tree_10BinaryTree__two_point_dual;
  if (PyType_Ready(&__pyx_type_7sklearn_9neighbors_7kd_tree_BinaryTree) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1002; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_7sklearn_9neighbors_7kd_tree_BinaryTree.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_7sklearn_9neighbors_7kd_tree_BinaryTree.tp_dict, __pyx_vtabptr_7sklearn_9neighbors_7kd_tree_BinaryTree) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1002; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "BinaryTree", (PyObject *)&__pyx_type_7sklearn_9neighbors_7kd_tree_BinaryTree) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1002; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_7sklearn_9neighbors_7kd_tree_BinaryTree = &__pyx_type_7sklearn_9neighbors_7kd_tree_BinaryTree;
  __pyx_vtabptr_7sklearn_9neighbors_7kd_tree_KDTree = &__pyx_vtable_7sklearn_9neighbors_7kd_tree_KDTree;
  __pyx_vtable_7sklearn_9neighbors_7kd_tree_KDTree.__pyx_base = *__pyx_vtabptr_7sklearn_9neighbors_7kd_tree_BinaryTree;
  __pyx_type_7sklearn_9neighbors_7kd_tree_KDTree.tp_base = __pyx_ptype_7sklearn_9neighbors_7kd_tree_BinaryTree;
  if (PyType_Ready(&__pyx_type_7sklearn_9neighbors_7kd_tree_KDTree) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 21; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_7sklearn_9neighbors_7kd_tree_KDTree.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_7sklearn_9neighbors_7kd_tree_KDTree.tp_dict, __pyx_vtabptr_7sklearn_9neighbors_7kd_tree_KDTree) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 21; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "KDTree", (PyObject *)&__pyx_type_7sklearn_9neighbors_7kd_tree_KDTree) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 21; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_7sklearn_9neighbors_7kd_tree_KDTree = &__pyx_type_7sklearn_9neighbors_7kd_tree_KDTree;
  if (PyType_Ready(&__pyx_type___pyx_array) < 0) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 99; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type___pyx_array.tp_print = 0;
  __pyx_array_type = &__pyx_type___pyx_array;
  if (PyType_Ready(&__pyx_type___pyx_MemviewEnum) < 0) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 269; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type___pyx_MemviewEnum.tp_print = 0;
  __pyx_MemviewEnum_type = &__pyx_type___pyx_MemviewEnum;
  __pyx_vtabptr_memoryview = &__pyx_vtable_memoryview;
  __pyx_vtable_memoryview.get_item_pointer = (char *(*)(struct __pyx_memoryview_obj *, PyObject *))__pyx_memoryview_get_item_pointer;
  __pyx_vtable_memoryview.is_slice = (PyObject *(*)(struct __pyx_memoryview_obj *, PyObject *))__pyx_memoryview_is_slice;
  __pyx_vtable_memoryview.setitem_slice_assignment = (PyObject *(*)(struct __pyx_memoryview_obj *, PyObject *, PyObject *))__pyx_memoryview_setitem_slice_assignment;
  __pyx_vtable_memoryview.setitem_slice_assign_scalar = (PyObject *(*)(struct __pyx_memoryview_obj *, struct __pyx_memoryview_obj *, PyObject *))__pyx_memoryview_setitem_slice_assign_scalar;
  __pyx_vtable_memoryview.setitem_indexed = (PyObject *(*)(struct __pyx_memoryview_obj *, PyObject *, PyObject *))__pyx_memoryview_setitem_indexed;
  __pyx_vtable_memoryview.convert_item_to_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *))__pyx_memoryview_convert_item_to_object;
  __pyx_vtable_memoryview.assign_item_from_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *, PyObject *))__pyx_memoryview_assign_item_from_object;
  if (PyType_Ready(&__pyx_type___pyx_memoryview) < 0) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 302; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type___pyx_memoryview.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type___pyx_memoryview.tp_dict, __pyx_vtabptr_memoryview) < 0) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 302; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_memoryview_type = &__pyx_type___pyx_memoryview;
  __pyx_vtabptr__memoryviewslice = &__pyx_vtable__memoryviewslice;
  __pyx_vtable__memoryviewslice.__pyx_base = *__pyx_vtabptr_memoryview;
  __pyx_vtable__memoryviewslice.__pyx_base.convert_item_to_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *))__pyx_memoryviewslice_convert_item_to_object;
  __pyx_vtable__memoryviewslice.__pyx_base.assign_item_from_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *, PyObject *))__pyx_memoryviewslice_assign_item_from_object;
  __pyx_type___pyx_memoryviewslice.tp_base = __pyx_memoryview_type;
  if (PyType_Ready(&__pyx_type___pyx_memoryviewslice) < 0) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 922; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type___pyx_memoryviewslice.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type___pyx_memoryviewslice.tp_dict, __pyx_vtabptr__memoryviewslice) < 0) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 922; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_memoryviewslice_type = &__pyx_type___pyx_memoryviewslice;
  /*--- Type import code ---*/
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "type", 
  #if CYTHON_COMPILING_IN_PYPY
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  0); if (unlikely(!__pyx_ptype_7cpython_4type_type)) {__pyx_filename = __pyx_f[5]; __pyx_lineno = 9; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType("numpy", "dtype", sizeof(PyArray_Descr), 0); if (unlikely(!__pyx_ptype_5numpy_dtype)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType("numpy", "flatiter", sizeof(PyArrayIterObject), 0); if (unlikely(!__pyx_ptype_5numpy_flatiter)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 165; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType("numpy", "broadcast", sizeof(PyArrayMultiIterObject), 0); if (unlikely(!__pyx_ptype_5numpy_broadcast)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType("numpy", "ndarray", sizeof(PyArrayObject), 0); if (unlikely(!__pyx_ptype_5numpy_ndarray)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType("numpy", "ufunc", sizeof(PyUFuncObject), 0); if (unlikely(!__pyx_ptype_5numpy_ufunc)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 861; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_7sklearn_9neighbors_12dist_metrics_DistanceMetric = __Pyx_ImportType("sklearn.neighbors.dist_metrics", "DistanceMetric", sizeof(struct __pyx_obj_7sklearn_9neighbors_12dist_metrics_DistanceMetric), 1); if (unlikely(!__pyx_ptype_7sklearn_9neighbors_12dist_metrics_DistanceMetric)) {__pyx_filename = __pyx_f[6]; __pyx_lineno = 48; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_7sklearn_9neighbors_12dist_metrics_DistanceMetric = (struct __pyx_vtabstruct_7sklearn_9neighbors_12dist_metrics_DistanceMetric*)__Pyx_GetVtable(__pyx_ptype_7sklearn_9neighbors_12dist_metrics_DistanceMetric->tp_dict); if (unlikely(!__pyx_vtabptr_7sklearn_9neighbors_12dist_metrics_DistanceMetric)) {__pyx_filename = __pyx_f[6]; __pyx_lineno = 48; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  __pyx_t_1 = __Pyx_ImportModule("sklearn.utils.lgamma"); if (!__pyx_t_1) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "lgamma", (void (**)(void))&__pyx_f_7sklearn_5utils_6lgamma_lgamma, "double (double)") < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  /*--- Execution code ---*/

  /* "sklearn/neighbors/kd_tree.pyx":10
 * # License: BSD
 * 
 * __all__ = ['KDTree']             # <<<<<<<<<<<<<<
 * 
 * DOC_DICT = {'BinaryTree': 'KDTree', 'binary_tree': 'kd_tree'}
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 10; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_KDTree);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_KDTree);
  __Pyx_GIVEREF(__pyx_n_s_KDTree);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_all, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 10; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "sklearn/neighbors/kd_tree.pyx":12
 * __all__ = ['KDTree']
 * 
 * DOC_DICT = {'BinaryTree': 'KDTree', 'binary_tree': 'kd_tree'}             # <<<<<<<<<<<<<<
 * 
 * VALID_METRICS = ['EuclideanDistance', 'ManhattanDistance',
 */
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 12; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_BinaryTree, __pyx_n_s_KDTree) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 12; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_binary_tree, __pyx_n_s_kd_tree) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 12; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DOC_DICT, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 12; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "sklearn/neighbors/kd_tree.pyx":14
 * DOC_DICT = {'BinaryTree': 'KDTree', 'binary_tree': 'kd_tree'}
 * 
 * VALID_METRICS = ['EuclideanDistance', 'ManhattanDistance',             # <<<<<<<<<<<<<<
 *                  'ChebyshevDistance', 'MinkowskiDistance']
 * 
 */
  __pyx_t_2 = PyList_New(4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 14; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_EuclideanDistance);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_EuclideanDistance);
  __Pyx_GIVEREF(__pyx_n_s_EuclideanDistance);
  __Pyx_INCREF(__pyx_n_s_ManhattanDistance);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_s_ManhattanDistance);
  __Pyx_GIVEREF(__pyx_n_s_ManhattanDistance);
  __Pyx_INCREF(__pyx_n_s_ChebyshevDistance);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_s_ChebyshevDistance);
  __Pyx_GIVEREF(__pyx_n_s_ChebyshevDistance);
  __Pyx_INCREF(__pyx_n_s_MinkowskiDistance);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_s_MinkowskiDistance);
  __Pyx_GIVEREF(__pyx_n_s_MinkowskiDistance);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_VALID_METRICS, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 14; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "binary_tree.pxi":149
 * from sklearn.utils.lgamma cimport lgamma
 * 
 * import numpy as np             # <<<<<<<<<<<<<<
 * import warnings
 * from ..utils import check_array
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_numpy, 0, -1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 149; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 149; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "binary_tree.pxi":150
 * 
 * import numpy as np
 * import warnings             # <<<<<<<<<<<<<<
 * from ..utils import check_array
 * 
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_warnings, 0, -1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 150; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_warnings, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 150; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "binary_tree.pxi":151
 * import numpy as np
 * import warnings
 * from ..utils import check_array             # <<<<<<<<<<<<<<
 * 
 * from typedefs cimport DTYPE_t, ITYPE_t, DITYPE_t
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 151; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_check_array);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_check_array);
  __Pyx_GIVEREF(__pyx_n_s_check_array);
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_utils, __pyx_t_2, 2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 151; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_check_array); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 151; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_check_array, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 151; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "binary_tree.pxi":154
 * 
 * from typedefs cimport DTYPE_t, ITYPE_t, DITYPE_t
 * from typedefs import DTYPE, ITYPE             # <<<<<<<<<<<<<<
 * 
 * from dist_metrics cimport (DistanceMetric, euclidean_dist, euclidean_rdist,
 */
  __pyx_t_3 = PyList_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_n_s_DTYPE);
  PyList_SET_ITEM(__pyx_t_3, 0, __pyx_n_s_DTYPE);
  __Pyx_GIVEREF(__pyx_n_s_DTYPE);
  __Pyx_INCREF(__pyx_n_s_ITYPE);
  PyList_SET_ITEM(__pyx_t_3, 1, __pyx_n_s_ITYPE);
  __Pyx_GIVEREF(__pyx_n_s_ITYPE);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_typedefs, __pyx_t_3, -1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_DTYPE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DTYPE, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_ITYPE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ITYPE, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "binary_tree.pxi":160
 * 
 * # some handy constants
 * cdef DTYPE_t INF = np.inf             # <<<<<<<<<<<<<<
 * cdef DTYPE_t NEG_INF = -np.inf
 * cdef DTYPE_t PI = np.pi
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_inf); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_3); if (unlikely((__pyx_t_4 == (npy_float64)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_7sklearn_9neighbors_7kd_tree_INF = __pyx_t_4;

  /* "binary_tree.pxi":161
 * # some handy constants
 * cdef DTYPE_t INF = np.inf
 * cdef DTYPE_t NEG_INF = -np.inf             # <<<<<<<<<<<<<<
 * cdef DTYPE_t PI = np.pi
 * cdef DTYPE_t ROOT_2PI = sqrt(2 * PI)
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 161; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_inf); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 161; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_Negative(__pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 161; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_3); if (unlikely((__pyx_t_4 == (npy_float64)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 161; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_7sklearn_9neighbors_7kd_tree_NEG_INF = __pyx_t_4;

  /* "binary_tree.pxi":162
 * cdef DTYPE_t INF = np.inf
 * cdef DTYPE_t NEG_INF = -np.inf
 * cdef DTYPE_t PI = np.pi             # <<<<<<<<<<<<<<
 * cdef DTYPE_t ROOT_2PI = sqrt(2 * PI)
 * cdef DTYPE_t LOG_PI = log(PI)
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 162; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_pi); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 162; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_4 == (npy_float64)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 162; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_7sklearn_9neighbors_7kd_tree_PI = __pyx_t_4;

  /* "binary_tree.pxi":163
 * cdef DTYPE_t NEG_INF = -np.inf
 * cdef DTYPE_t PI = np.pi
 * cdef DTYPE_t ROOT_2PI = sqrt(2 * PI)             # <<<<<<<<<<<<<<
 * cdef DTYPE_t LOG_PI = log(PI)
 * cdef DTYPE_t LOG_2PI = log(2 * PI)
 */
  __pyx_v_7sklearn_9neighbors_7kd_tree_ROOT_2PI = sqrt((2.0 * __pyx_v_7sklearn_9neighbors_7kd_tree_PI));

  /* "binary_tree.pxi":164
 * cdef DTYPE_t PI = np.pi
 * cdef DTYPE_t ROOT_2PI = sqrt(2 * PI)
 * cdef DTYPE_t LOG_PI = log(PI)             # <<<<<<<<<<<<<<
 * cdef DTYPE_t LOG_2PI = log(2 * PI)
 * 
 */
  __pyx_v_7sklearn_9neighbors_7kd_tree_LOG_PI = log(__pyx_v_7sklearn_9neighbors_7kd_tree_PI);

  /* "binary_tree.pxi":165
 * cdef DTYPE_t ROOT_2PI = sqrt(2 * PI)
 * cdef DTYPE_t LOG_PI = log(PI)
 * cdef DTYPE_t LOG_2PI = log(2 * PI)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_7sklearn_9neighbors_7kd_tree_LOG_2PI = log((2.0 * __pyx_v_7sklearn_9neighbors_7kd_tree_PI));

  /* "binary_tree.pxi":180
 * #     NodeHeapData = np.asarray(<NodeHeapData_t[:1]>(&nhd_tmp)).dtype
 * cdef NodeHeapData_t nhd_tmp
 * offsets = [<np.intp_t>&(nhd_tmp.val) - <np.intp_t>&nhd_tmp,             # <<<<<<<<<<<<<<
 *            <np.intp_t>&(nhd_tmp.i1) - <np.intp_t>&nhd_tmp,
 *            <np.intp_t>&(nhd_tmp.i2) - <np.intp_t>&nhd_tmp]
 */
  __pyx_t_2 = __Pyx_PyInt_From_int((((__pyx_t_5numpy_intp_t)(&__pyx_v_7sklearn_9neighbors_7kd_tree_nhd_tmp.val)) - ((__pyx_t_5numpy_intp_t)(&__pyx_v_7sklearn_9neighbors_7kd_tree_nhd_tmp)))); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 180; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);

  /* "binary_tree.pxi":181
 * cdef NodeHeapData_t nhd_tmp
 * offsets = [<np.intp_t>&(nhd_tmp.val) - <np.intp_t>&nhd_tmp,
 *            <np.intp_t>&(nhd_tmp.i1) - <np.intp_t>&nhd_tmp,             # <<<<<<<<<<<<<<
 *            <np.intp_t>&(nhd_tmp.i2) - <np.intp_t>&nhd_tmp]
 * NodeHeapData = np.dtype({'names': ['val', 'i1', 'i2'],
 */
  __pyx_t_3 = __Pyx_PyInt_From_int((((__pyx_t_5numpy_intp_t)(&__pyx_v_7sklearn_9neighbors_7kd_tree_nhd_tmp.i1)) - ((__pyx_t_5numpy_intp_t)(&__pyx_v_7sklearn_9neighbors_7kd_tree_nhd_tmp)))); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);

  /* "binary_tree.pxi":182
 * offsets = [<np.intp_t>&(nhd_tmp.val) - <np.intp_t>&nhd_tmp,
 *            <np.intp_t>&(nhd_tmp.i1) - <np.intp_t>&nhd_tmp,
 *            <np.intp_t>&(nhd_tmp.i2) - <np.intp_t>&nhd_tmp]             # <<<<<<<<<<<<<<
 * NodeHeapData = np.dtype({'names': ['val', 'i1', 'i2'],
 *                          'formats': [DTYPE, ITYPE, ITYPE],
 */
  __pyx_t_5 = __Pyx_PyInt_From_int((((__pyx_t_5numpy_intp_t)(&__pyx_v_7sklearn_9neighbors_7kd_tree_nhd_tmp.i2)) - ((__pyx_t_5numpy_intp_t)(&__pyx_v_7sklearn_9neighbors_7kd_tree_nhd_tmp)))); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 182; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);

  /* "binary_tree.pxi":180
 * #     NodeHeapData = np.asarray(<NodeHeapData_t[:1]>(&nhd_tmp)).dtype
 * cdef NodeHeapData_t nhd_tmp
 * offsets = [<np.intp_t>&(nhd_tmp.val) - <np.intp_t>&nhd_tmp,             # <<<<<<<<<<<<<<
 *            <np.intp_t>&(nhd_tmp.i1) - <np.intp_t>&nhd_tmp,
 *            <np.intp_t>&(nhd_tmp.i2) - <np.intp_t>&nhd_tmp]
 */
  __pyx_t_6 = PyList_New(3); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 180; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  PyList_SET_ITEM(__pyx_t_6, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  PyList_SET_ITEM(__pyx_t_6, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  PyList_SET_ITEM(__pyx_t_6, 2, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_5);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_offsets, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 180; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "binary_tree.pxi":183
 *            <np.intp_t>&(nhd_tmp.i1) - <np.intp_t>&nhd_tmp,
 *            <np.intp_t>&(nhd_tmp.i2) - <np.intp_t>&nhd_tmp]
 * NodeHeapData = np.dtype({'names': ['val', 'i1', 'i2'],             # <<<<<<<<<<<<<<
 *                          'formats': [DTYPE, ITYPE, ITYPE],
 *                          'offsets': offsets,
 */
  __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = PyList_New(3); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_n_s_val);
  PyList_SET_ITEM(__pyx_t_5, 0, __pyx_n_s_val);
  __Pyx_GIVEREF(__pyx_n_s_val);
  __Pyx_INCREF(__pyx_n_s_i1);
  PyList_SET_ITEM(__pyx_t_5, 1, __pyx_n_s_i1);
  __Pyx_GIVEREF(__pyx_n_s_i1);
  __Pyx_INCREF(__pyx_n_s_i2);
  PyList_SET_ITEM(__pyx_t_5, 2, __pyx_n_s_i2);
  __Pyx_GIVEREF(__pyx_n_s_i2);
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_names, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "binary_tree.pxi":184
 *            <np.intp_t>&(nhd_tmp.i2) - <np.intp_t>&nhd_tmp]
 * NodeHeapData = np.dtype({'names': ['val', 'i1', 'i2'],
 *                          'formats': [DTYPE, ITYPE, ITYPE],             # <<<<<<<<<<<<<<
 *                          'offsets': offsets,
 *                          'itemsize': sizeof(NodeHeapData_t)})
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 184; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 184; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 184; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = PyList_New(3); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 184; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  PyList_SET_ITEM(__pyx_t_7, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_5);
  PyList_SET_ITEM(__pyx_t_7, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  PyList_SET_ITEM(__pyx_t_7, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_5 = 0;
  __pyx_t_3 = 0;
  __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_formats, __pyx_t_7) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "binary_tree.pxi":185
 * NodeHeapData = np.dtype({'names': ['val', 'i1', 'i2'],
 *                          'formats': [DTYPE, ITYPE, ITYPE],
 *                          'offsets': offsets,             # <<<<<<<<<<<<<<
 *                          'itemsize': sizeof(NodeHeapData_t)})
 * 
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_offsets); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_offsets, __pyx_t_7) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "binary_tree.pxi":186
 *                          'formats': [DTYPE, ITYPE, ITYPE],
 *                          'offsets': offsets,
 *                          'itemsize': sizeof(NodeHeapData_t)})             # <<<<<<<<<<<<<<
 * 
 * cdef struct NodeData_t:
 */
  __pyx_t_7 = __Pyx_PyInt_FromSize_t((sizeof(struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t))); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_itemsize, __pyx_t_7) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "binary_tree.pxi":183
 *            <np.intp_t>&(nhd_tmp.i1) - <np.intp_t>&nhd_tmp,
 *            <np.intp_t>&(nhd_tmp.i2) - <np.intp_t>&nhd_tmp]
 * NodeHeapData = np.dtype({'names': ['val', 'i1', 'i2'],             # <<<<<<<<<<<<<<
 *                          'formats': [DTYPE, ITYPE, ITYPE],
 *                          'offsets': offsets,
 */
  __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_6);
  __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_Call(((PyObject *)((PyObject*)__pyx_ptype_5numpy_dtype)), __pyx_t_7, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_NodeHeapData, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "binary_tree.pxi":200
 * #     NodeData = np.asarray(<NodeData_t[:1]>(&nd_tmp)).dtype
 * cdef NodeData_t nd_tmp
 * offsets = [<np.intp_t>&(nd_tmp.idx_start) - <np.intp_t>&nd_tmp,             # <<<<<<<<<<<<<<
 *            <np.intp_t>&(nd_tmp.idx_end) - <np.intp_t>&nd_tmp,
 *            <np.intp_t>&(nd_tmp.is_leaf) - <np.intp_t>&nd_tmp,
 */
  __pyx_t_6 = __Pyx_PyInt_From_int((((__pyx_t_5numpy_intp_t)(&__pyx_v_7sklearn_9neighbors_7kd_tree_nd_tmp.idx_start)) - ((__pyx_t_5numpy_intp_t)(&__pyx_v_7sklearn_9neighbors_7kd_tree_nd_tmp)))); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 200; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);

  /* "binary_tree.pxi":201
 * cdef NodeData_t nd_tmp
 * offsets = [<np.intp_t>&(nd_tmp.idx_start) - <np.intp_t>&nd_tmp,
 *            <np.intp_t>&(nd_tmp.idx_end) - <np.intp_t>&nd_tmp,             # <<<<<<<<<<<<<<
 *            <np.intp_t>&(nd_tmp.is_leaf) - <np.intp_t>&nd_tmp,
 *            <np.intp_t>&(nd_tmp.radius) - <np.intp_t>&nd_tmp]
 */
  __pyx_t_7 = __Pyx_PyInt_From_int((((__pyx_t_5numpy_intp_t)(&__pyx_v_7sklearn_9neighbors_7kd_tree_nd_tmp.idx_end)) - ((__pyx_t_5numpy_intp_t)(&__pyx_v_7sklearn_9neighbors_7kd_tree_nd_tmp)))); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 201; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);

  /* "binary_tree.pxi":202
 * offsets = [<np.intp_t>&(nd_tmp.idx_start) - <np.intp_t>&nd_tmp,
 *            <np.intp_t>&(nd_tmp.idx_end) - <np.intp_t>&nd_tmp,
 *            <np.intp_t>&(nd_tmp.is_leaf) - <np.intp_t>&nd_tmp,             # <<<<<<<<<<<<<<
 *            <np.intp_t>&(nd_tmp.radius) - <np.intp_t>&nd_tmp]
 * NodeData = np.dtype({'names': ['idx_start', 'idx_end', 'is_leaf', 'radius'],
 */
  __pyx_t_2 = __Pyx_PyInt_From_int((((__pyx_t_5numpy_intp_t)(&__pyx_v_7sklearn_9neighbors_7kd_tree_nd_tmp.is_leaf)) - ((__pyx_t_5numpy_intp_t)(&__pyx_v_7sklearn_9neighbors_7kd_tree_nd_tmp)))); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 202; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);

  /* "binary_tree.pxi":203
 *            <np.intp_t>&(nd_tmp.idx_end) - <np.intp_t>&nd_tmp,
 *            <np.intp_t>&(nd_tmp.is_leaf) - <np.intp_t>&nd_tmp,
 *            <np.intp_t>&(nd_tmp.radius) - <np.intp_t>&nd_tmp]             # <<<<<<<<<<<<<<
 * NodeData = np.dtype({'names': ['idx_start', 'idx_end', 'is_leaf', 'radius'],
 *                      'formats': [ITYPE, ITYPE, ITYPE, DTYPE],
 */
  __pyx_t_3 = __Pyx_PyInt_From_int((((__pyx_t_5numpy_intp_t)(&__pyx_v_7sklearn_9neighbors_7kd_tree_nd_tmp.radius)) - ((__pyx_t_5numpy_intp_t)(&__pyx_v_7sklearn_9neighbors_7kd_tree_nd_tmp)))); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 203; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);

  /* "binary_tree.pxi":200
 * #     NodeData = np.asarray(<NodeData_t[:1]>(&nd_tmp)).dtype
 * cdef NodeData_t nd_tmp
 * offsets = [<np.intp_t>&(nd_tmp.idx_start) - <np.intp_t>&nd_tmp,             # <<<<<<<<<<<<<<
 *            <np.intp_t>&(nd_tmp.idx_end) - <np.intp_t>&nd_tmp,
 *            <np.intp_t>&(nd_tmp.is_leaf) - <np.intp_t>&nd_tmp,
 */
  __pyx_t_5 = PyList_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 200; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  PyList_SET_ITEM(__pyx_t_5, 0, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_6);
  PyList_SET_ITEM(__pyx_t_5, 1, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_7);
  PyList_SET_ITEM(__pyx_t_5, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  PyList_SET_ITEM(__pyx_t_5, 3, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_offsets, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 200; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "binary_tree.pxi":204
 *            <np.intp_t>&(nd_tmp.is_leaf) - <np.intp_t>&nd_tmp,
 *            <np.intp_t>&(nd_tmp.radius) - <np.intp_t>&nd_tmp]
 * NodeData = np.dtype({'names': ['idx_start', 'idx_end', 'is_leaf', 'radius'],             # <<<<<<<<<<<<<<
 *                      'formats': [ITYPE, ITYPE, ITYPE, DTYPE],
 *                      'offsets': offsets,
 */
  __pyx_t_5 = PyDict_New(); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 204; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = PyList_New(4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 204; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_n_s_idx_start);
  PyList_SET_ITEM(__pyx_t_3, 0, __pyx_n_s_idx_start);
  __Pyx_GIVEREF(__pyx_n_s_idx_start);
  __Pyx_INCREF(__pyx_n_s_idx_end);
  PyList_SET_ITEM(__pyx_t_3, 1, __pyx_n_s_idx_end);
  __Pyx_GIVEREF(__pyx_n_s_idx_end);
  __Pyx_INCREF(__pyx_n_s_is_leaf);
  PyList_SET_ITEM(__pyx_t_3, 2, __pyx_n_s_is_leaf);
  __Pyx_GIVEREF(__pyx_n_s_is_leaf);
  __Pyx_INCREF(__pyx_n_s_radius);
  PyList_SET_ITEM(__pyx_t_3, 3, __pyx_n_s_radius);
  __Pyx_GIVEREF(__pyx_n_s_radius);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_names, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 204; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "binary_tree.pxi":205
 *            <np.intp_t>&(nd_tmp.radius) - <np.intp_t>&nd_tmp]
 * NodeData = np.dtype({'names': ['idx_start', 'idx_end', 'is_leaf', 'radius'],
 *                      'formats': [ITYPE, ITYPE, ITYPE, DTYPE],             # <<<<<<<<<<<<<<
 *                      'offsets': offsets,
 *                      'itemsize': sizeof(NodeData_t)})
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 205; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 205; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_ITYPE); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 205; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_DTYPE); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 205; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_8 = PyList_New(4); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 205; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  PyList_SET_ITEM(__pyx_t_8, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  PyList_SET_ITEM(__pyx_t_8, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  PyList_SET_ITEM(__pyx_t_8, 2, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_7);
  PyList_SET_ITEM(__pyx_t_8, 3, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_6);
  __pyx_t_3 = 0;
  __pyx_t_2 = 0;
  __pyx_t_7 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_formats, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 204; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "binary_tree.pxi":206
 * NodeData = np.dtype({'names': ['idx_start', 'idx_end', 'is_leaf', 'radius'],
 *                      'formats': [ITYPE, ITYPE, ITYPE, DTYPE],
 *                      'offsets': offsets,             # <<<<<<<<<<<<<<
 *                      'itemsize': sizeof(NodeData_t)})
 * 
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_offsets); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 206; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_offsets, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 204; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "binary_tree.pxi":207
 *                      'formats': [ITYPE, ITYPE, ITYPE, DTYPE],
 *                      'offsets': offsets,
 *                      'itemsize': sizeof(NodeData_t)})             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_8 = __Pyx_PyInt_FromSize_t((sizeof(struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t))); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 207; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_itemsize, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 204; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "binary_tree.pxi":204
 *            <np.intp_t>&(nd_tmp.is_leaf) - <np.intp_t>&nd_tmp,
 *            <np.intp_t>&(nd_tmp.radius) - <np.intp_t>&nd_tmp]
 * NodeData = np.dtype({'names': ['idx_start', 'idx_end', 'is_leaf', 'radius'],             # <<<<<<<<<<<<<<
 *                      'formats': [ITYPE, ITYPE, ITYPE, DTYPE],
 *                      'offsets': offsets,
 */
  __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 204; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_5);
  __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)((PyObject*)__pyx_ptype_5numpy_dtype)), __pyx_t_8, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 204; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_NodeData, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 204; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "binary_tree.pxi":255
 * # the DOC_DICT variable defined in the pyx files.
 * CLASS_DOC = \
 * """{BinaryTree} for fast generalized N-point problems             # <<<<<<<<<<<<<<
 * 
 * {BinaryTree}(X, leaf_size=40, metric='minkowski', \\**kwargs)
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CLASS_DOC, __pyx_kp_s_BinaryTree_for_fast_generalized) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 254; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "binary_tree.pxi":497
 * 
 * 
 * def kernel_norm(h, d, kernel, return_log=False):             # <<<<<<<<<<<<<<
 *     """Given a string specification of a kernel, compute the normalization.
 * 
 */
  __pyx_t_5 = PyCFunction_NewEx(&__pyx_mdef_7sklearn_9neighbors_7kd_tree_1kernel_norm, NULL, __pyx_n_s_sklearn_neighbors_kd_tree); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 497; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kernel_norm, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 497; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "binary_tree.pxi":990
 * # newObj function
 * #  this is a helper function for pickling
 * def newObj(obj):             # <<<<<<<<<<<<<<
 *     return obj.__new__(obj)
 * 
 */
  __pyx_t_5 = PyCFunction_NewEx(&__pyx_mdef_7sklearn_9neighbors_7kd_tree_3newObj, NULL, __pyx_n_s_sklearn_neighbors_kd_tree); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 990; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_newObj, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 990; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "binary_tree.pxi":996
 * ######################################################################
 * # define the reverse mapping of VALID_METRICS
 * from dist_metrics import get_valid_metric_ids             # <<<<<<<<<<<<<<
 * VALID_METRIC_IDS = get_valid_metric_ids(VALID_METRICS)
 * 
 */
  __pyx_t_5 = PyList_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 996; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_n_s_get_valid_metric_ids);
  PyList_SET_ITEM(__pyx_t_5, 0, __pyx_n_s_get_valid_metric_ids);
  __Pyx_GIVEREF(__pyx_n_s_get_valid_metric_ids);
  __pyx_t_8 = __Pyx_Import(__pyx_n_s_dist_metrics, __pyx_t_5, -1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 996; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_ImportFrom(__pyx_t_8, __pyx_n_s_get_valid_metric_ids); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 996; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_get_valid_metric_ids, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 996; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "binary_tree.pxi":997
 * # define the reverse mapping of VALID_METRICS
 * from dist_metrics import get_valid_metric_ids
 * VALID_METRIC_IDS = get_valid_metric_ids(VALID_METRICS)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_get_valid_metric_ids); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 997; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_VALID_METRICS); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 997; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  if (!__pyx_t_7) {
    __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 997; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_8);
  } else {
    __pyx_t_2 = PyTuple_New(1+1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 997; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_7); __Pyx_GIVEREF(__pyx_t_7); __pyx_t_7 = NULL;
    PyTuple_SET_ITEM(__pyx_t_2, 0+1, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_2, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 997; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_VALID_METRIC_IDS, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 997; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "binary_tree.pxi":1027
 *     cdef int n_calls
 * 
 *     valid_metrics = VALID_METRIC_IDS             # <<<<<<<<<<<<<<
 * 
 *     # Use cinit to initialize all arrays to empty: this will prevent memory
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_VALID_METRIC_IDS); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1027; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7sklearn_9neighbors_7kd_tree_BinaryTree->tp_dict, __pyx_n_s_valid_metrics, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1027; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  PyType_Modified(__pyx_ptype_7sklearn_9neighbors_7kd_tree_BinaryTree);

  /* "binary_tree.pxi":2474
 * # Python functions for benchmarking and testing C implementations
 * 
 * def load_heap(DTYPE_t[:, ::1] X, ITYPE_t k):             # <<<<<<<<<<<<<<
 *     """test fully loading the heap"""
 *     assert k <= X.shape[1]
 */
  __pyx_t_8 = PyCFunction_NewEx(&__pyx_mdef_7sklearn_9neighbors_7kd_tree_5load_heap, NULL, __pyx_n_s_sklearn_neighbors_kd_tree); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_load_heap, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "binary_tree.pxi":2485
 * 
 * 
 * def simultaneous_sort(DTYPE_t[:, ::1] distances, ITYPE_t[:, ::1] indices):             # <<<<<<<<<<<<<<
 *     """In-place simultaneous sort the given row of the arrays
 * 
 */
  __pyx_t_8 = PyCFunction_NewEx(&__pyx_mdef_7sklearn_9neighbors_7kd_tree_7simultaneous_sort, NULL, __pyx_n_s_sklearn_neighbors_kd_tree); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2485; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_simultaneous_sort, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2485; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "binary_tree.pxi":2500
 * 
 * 
 * def nodeheap_sort(DTYPE_t[::1] vals):             # <<<<<<<<<<<<<<
 *     """In-place reverse sort of vals using NodeHeap"""
 *     cdef ITYPE_t[::1] indices = np.zeros(vals.shape[0], dtype=ITYPE)
 */
  __pyx_t_8 = PyCFunction_NewEx(&__pyx_mdef_7sklearn_9neighbors_7kd_tree_9nodeheap_sort, NULL, __pyx_n_s_sklearn_neighbors_kd_tree); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2500; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nodeheap_sort, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2500; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "sklearn/neighbors/kd_tree.pyx":22
 * 
 * cdef class KDTree(BinaryTree):
 *     __doc__ = CLASS_DOC.format(**DOC_DICT)             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_CLASS_DOC); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 22; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 22; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_DOC_DICT); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 22; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (unlikely(__pyx_t_2 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "argument after ** must be a mapping, not NoneType");
    {__pyx_filename = __pyx_f[1]; __pyx_lineno = 22; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  if (likely(PyDict_Check(__pyx_t_2))) {
    __pyx_t_8 = __pyx_t_2;
    __Pyx_INCREF(__pyx_t_8);
  } else {
    __pyx_t_8 = PyObject_CallFunctionObjArgs((PyObject*)&PyDict_Type, __pyx_t_2, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 22; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_empty_tuple, __pyx_t_8); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 22; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7sklearn_9neighbors_7kd_tree_KDTree->tp_dict, __pyx_n_s_doc, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 22; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7sklearn_9neighbors_7kd_tree_KDTree);

  /* "sklearn/neighbors/kd_tree.pyx":1
 * #!python             # <<<<<<<<<<<<<<
 * #cython: boundscheck=False
 * #cython: wraparound=False
 */
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_kp_u_BinaryTree_query_line_1233, __pyx_kp_u_query_X_k_1_return_distance_Tru) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyDict_SetItem(__pyx_t_2, __pyx_kp_u_BinaryTree_query_radius_line_135, __pyx_kp_u_query_radius_self_X_r_count_onl) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyDict_SetItem(__pyx_t_2, __pyx_kp_u_BinaryTree_kernel_density_line_1, __pyx_kp_u_kernel_density_self_X_h_kernel) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyDict_SetItem(__pyx_t_2, __pyx_kp_u_BinaryTree_two_point_correlation, __pyx_kp_u_Compute_the_two_point_correlatio) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":203
 *         info.obj = self
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_array_getbuffer, "getbuffer(obj, view, flags)")             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(array self):
 */
  __pyx_t_2 = __pyx_capsule_create(((void *)(&__pyx_array_getbuffer)), __pyx_k_getbuffer_obj_view_flags); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 203; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_array_type->tp_dict, __pyx_n_s_pyx_getbuffer, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 203; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_array_type);

  /* "View.MemoryView":276
 *         return self.name
 * 
 * cdef generic = Enum("<strided and direct or indirect>")             # <<<<<<<<<<<<<<
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")
 */
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)((PyObject *)__pyx_MemviewEnum_type)), __pyx_tuple__65, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 276; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_XGOTREF(generic);
  __Pyx_DECREF_SET(generic, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "View.MemoryView":277
 * 
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default             # <<<<<<<<<<<<<<
 * cdef indirect = Enum("<strided and indirect>")
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)((PyObject *)__pyx_MemviewEnum_type)), __pyx_tuple__66, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 277; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_XGOTREF(strided);
  __Pyx_DECREF_SET(strided, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "View.MemoryView":278
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)((PyObject *)__pyx_MemviewEnum_type)), __pyx_tuple__67, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_XGOTREF(indirect);
  __Pyx_DECREF_SET(indirect, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "View.MemoryView":281
 * 
 * 
 * cdef contiguous = Enum("<contiguous and direct>")             # <<<<<<<<<<<<<<
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)((PyObject *)__pyx_MemviewEnum_type)), __pyx_tuple__68, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 281; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_XGOTREF(contiguous);
  __Pyx_DECREF_SET(contiguous, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "View.MemoryView":282
 * 
 * cdef contiguous = Enum("<contiguous and direct>")
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)((PyObject *)__pyx_MemviewEnum_type)), __pyx_tuple__69, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 282; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_XGOTREF(indirect_contiguous);
  __Pyx_DECREF_SET(indirect_contiguous, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "View.MemoryView":496
 *         info.obj = self
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_memoryview_getbuffer, "getbuffer(obj, view, flags)")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __pyx_capsule_create(((void *)(&__pyx_memoryview_getbuffer)), __pyx_k_getbuffer_obj_view_flags); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 496; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_memoryview_type->tp_dict, __pyx_n_s_pyx_getbuffer, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 496; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_memoryview_type);

  /* "View.MemoryView":953
 *             return self.from_object
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_memoryview_getbuffer, "getbuffer(obj, view, flags)")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __pyx_capsule_create(((void *)(&__pyx_memoryview_getbuffer)), __pyx_k_getbuffer_obj_view_flags); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 953; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_memoryviewslice_type->tp_dict, __pyx_n_s_pyx_getbuffer, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 953; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_memoryviewslice_type);

  /* "BufferFormatFromTypeInfo":1417
 * 
 * @cname('__pyx_format_from_typeinfo')
 * cdef bytes format_from_typeinfo(__Pyx_TypeInfo *type):             # <<<<<<<<<<<<<<
 *     cdef __Pyx_StructField *field
 *     cdef __pyx_typeinfo_string fmt
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init sklearn.neighbors.kd_tree", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init sklearn.neighbors.kd_tree");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* Runtime support code */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

static CYTHON_INLINE int __Pyx_IsLittleEndian(void) {
  unsigned int n = 1;
  return *(unsigned char*)(&n) != 0;
}
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type) {
  stack[0].field = &ctx->root;
  stack[0].parent_offset = 0;
  ctx->root.type = type;
  ctx->root.name = "buffer dtype";
  ctx->root.offset = 0;
  ctx->head = stack;
  ctx->head->field = &ctx->root;
  ctx->fmt_offset = 0;
  ctx->head->parent_offset = 0;
  ctx->new_packmode = '@';
  ctx->enc_packmode = '@';
  ctx->new_count = 1;
  ctx->enc_count = 0;
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  ctx->is_valid_array = 0;
  ctx->struct_alignment = 0;
  while (type->typegroup == 'S') {
    ++ctx->head;
    ctx->head->field = type->fields;
    ctx->head->parent_offset = 0;
    type = type->fields->type;
  }
}
static int __Pyx_BufFmt_ParseNumber(const char** ts) {
    int count;
    const char* t = *ts;
    if (*t < '0' || *t > '9') {
      return -1;
    } else {
        count = *t++ - '0';
        while (*t >= '0' && *t < '9') {
            count *= 10;
            count += *t++ - '0';
        }
    }
    *ts = t;
    return count;
}
static int __Pyx_BufFmt_ExpectNumber(const char **ts) {
    int number = __Pyx_BufFmt_ParseNumber(ts);
    if (number == -1)
        PyErr_Format(PyExc_ValueError,\
                     "Does not understand character buffer dtype format string ('%c')", **ts);
    return number;
}
static void __Pyx_BufFmt_RaiseUnexpectedChar(char ch) {
  PyErr_Format(PyExc_ValueError,
               "Unexpected format string character: '%c'", ch);
}
static const char* __Pyx_BufFmt_DescribeTypeChar(char ch, int is_complex) {
  switch (ch) {
    case 'c': return "'char'";
    case 'b': return "'signed char'";
    case 'B': return "'unsigned char'";
    case 'h': return "'short'";
    case 'H': return "'unsigned short'";
    case 'i': return "'int'";
    case 'I': return "'unsigned int'";
    case 'l': return "'long'";
    case 'L': return "'unsigned long'";
    case 'q': return "'long long'";
    case 'Q': return "'unsigned long long'";
    case 'f': return (is_complex ? "'complex float'" : "'float'");
    case 'd': return (is_complex ? "'complex double'" : "'double'");
    case 'g': return (is_complex ? "'complex long double'" : "'long double'");
    case 'T': return "a struct";
    case 'O': return "Python object";
    case 'P': return "a pointer";
    case 's': case 'p': return "a string";
    case 0: return "end";
    default: return "unparseable format string";
  }
}
static size_t __Pyx_BufFmt_TypeCharToStandardSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return 2;
    case 'i': case 'I': case 'l': case 'L': return 4;
    case 'q': case 'Q': return 8;
    case 'f': return (is_complex ? 8 : 4);
    case 'd': return (is_complex ? 16 : 8);
    case 'g': {
      PyErr_SetString(PyExc_ValueError, "Python does not define a standard format string size for long double ('g')..");
      return 0;
    }
    case 'O': case 'P': return sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static size_t __Pyx_BufFmt_TypeCharToNativeSize(char ch, int is_complex) {
  switch (ch) {
    case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(short);
    case 'i': case 'I': return sizeof(int);
    case 'l': case 'L': return sizeof(long);
    #ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(PY_LONG_LONG);
    #endif
    case 'f': return sizeof(float) * (is_complex ? 2 : 1);
    case 'd': return sizeof(double) * (is_complex ? 2 : 1);
    case 'g': return sizeof(long double) * (is_complex ? 2 : 1);
    case 'O': case 'P': return sizeof(void*);
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
typedef struct { char c; short x; } __Pyx_st_short;
typedef struct { char c; int x; } __Pyx_st_int;
typedef struct { char c; long x; } __Pyx_st_long;
typedef struct { char c; float x; } __Pyx_st_float;
typedef struct { char c; double x; } __Pyx_st_double;
typedef struct { char c; long double x; } __Pyx_st_longdouble;
typedef struct { char c; void *x; } __Pyx_st_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { char c; PY_LONG_LONG x; } __Pyx_st_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToAlignment(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_st_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_st_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_st_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_st_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_st_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_st_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_st_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_st_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
/* These are for computing the padding at the end of the struct to align
   on the first member of the struct. This will probably the same as above,
   but we don't have any guarantees.
 */
typedef struct { short x; char c; } __Pyx_pad_short;
typedef struct { int x; char c; } __Pyx_pad_int;
typedef struct { long x; char c; } __Pyx_pad_long;
typedef struct { float x; char c; } __Pyx_pad_float;
typedef struct { double x; char c; } __Pyx_pad_double;
typedef struct { long double x; char c; } __Pyx_pad_longdouble;
typedef struct { void *x; char c; } __Pyx_pad_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { PY_LONG_LONG x; char c; } __Pyx_pad_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToPadding(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_pad_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_pad_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_pad_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_pad_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_pad_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_pad_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_pad_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_pad_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static char __Pyx_BufFmt_TypeCharToGroup(char ch, int is_complex) {
  switch (ch) {
    case 'c':
        return 'H';
    case 'b': case 'h': case 'i':
    case 'l': case 'q': case 's': case 'p':
        return 'I';
    case 'B': case 'H': case 'I': case 'L': case 'Q':
        return 'U';
    case 'f': case 'd': case 'g':
        return (is_complex ? 'C' : 'R');
    case 'O':
        return 'O';
    case 'P':
        return 'P';
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
static void __Pyx_BufFmt_RaiseExpected(__Pyx_BufFmt_Context* ctx) {
  if (ctx->head == NULL || ctx->head->field == &ctx->root) {
    const char* expected;
    const char* quote;
    if (ctx->head == NULL) {
      expected = "end";
      quote = "";
    } else {
      expected = ctx->head->field->type->name;
      quote = "'";
    }
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected %s%s%s but got %s",
                 quote, expected, quote,
                 __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex));
  } else {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_StructField* parent = (ctx->head - 1)->field;
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected '%s' but got %s in '%s.%s'",
                 field->type->name, __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex),
                 parent->type->name, field->name);
  }
}
static int __Pyx_BufFmt_ProcessTypeChunk(__Pyx_BufFmt_Context* ctx) {
  char group;
  size_t size, offset, arraysize = 1;
  if (ctx->enc_type == 0) return 0;
  if (ctx->head->field->type->arraysize[0]) {
    int i, ndim = 0;
    if (ctx->enc_type == 's' || ctx->enc_type == 'p') {
        ctx->is_valid_array = ctx->head->field->type->ndim == 1;
        ndim = 1;
        if (ctx->enc_count != ctx->head->field->type->arraysize[0]) {
            PyErr_Format(PyExc_ValueError,
                         "Expected a dimension of size %zu, got %zu",
                         ctx->head->field->type->arraysize[0], ctx->enc_count);
            return -1;
        }
    }
    if (!ctx->is_valid_array) {
      PyErr_Format(PyExc_ValueError, "Expected %d dimensions, got %d",
                   ctx->head->field->type->ndim, ndim);
      return -1;
    }
    for (i = 0; i < ctx->head->field->type->ndim; i++) {
      arraysize *= ctx->head->field->type->arraysize[i];
    }
    ctx->is_valid_array = 0;
    ctx->enc_count = 1;
  }
  group = __Pyx_BufFmt_TypeCharToGroup(ctx->enc_type, ctx->is_complex);
  do {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_TypeInfo* type = field->type;
    if (ctx->enc_packmode == '@' || ctx->enc_packmode == '^') {
      size = __Pyx_BufFmt_TypeCharToNativeSize(ctx->enc_type, ctx->is_complex);
    } else {
      size = __Pyx_BufFmt_TypeCharToStandardSize(ctx->enc_type, ctx->is_complex);
    }
    if (ctx->enc_packmode == '@') {
      size_t align_at = __Pyx_BufFmt_TypeCharToAlignment(ctx->enc_type, ctx->is_complex);
      size_t align_mod_offset;
      if (align_at == 0) return -1;
      align_mod_offset = ctx->fmt_offset % align_at;
      if (align_mod_offset > 0) ctx->fmt_offset += align_at - align_mod_offset;
      if (ctx->struct_alignment == 0)
          ctx->struct_alignment = __Pyx_BufFmt_TypeCharToPadding(ctx->enc_type,
                                                                 ctx->is_complex);
    }
    if (type->size != size || type->typegroup != group) {
      if (type->typegroup == 'C' && type->fields != NULL) {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        ++ctx->head;
        ctx->head->field = type->fields;
        ctx->head->parent_offset = parent_offset;
        continue;
      }
      if ((type->typegroup == 'H' || group == 'H') && type->size == size) {
      } else {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
      }
    }
    offset = ctx->head->parent_offset + field->offset;
    if (ctx->fmt_offset != offset) {
      PyErr_Format(PyExc_ValueError,
                   "Buffer dtype mismatch; next field is at offset %" CYTHON_FORMAT_SSIZE_T "d but %" CYTHON_FORMAT_SSIZE_T "d expected",
                   (Py_ssize_t)ctx->fmt_offset, (Py_ssize_t)offset);
      return -1;
    }
    ctx->fmt_offset += size;
    if (arraysize)
      ctx->fmt_offset += (arraysize - 1) * size;
    --ctx->enc_count;
    while (1) {
      if (field == &ctx->root) {
        ctx->head = NULL;
        if (ctx->enc_count != 0) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
        }
        break;
      }
      ctx->head->field = ++field;
      if (field->type == NULL) {
        --ctx->head;
        field = ctx->head->field;
        continue;
      } else if (field->type->typegroup == 'S') {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        if (field->type->fields->type == NULL) continue;
        field = field->type->fields;
        ++ctx->head;
        ctx->head->field = field;
        ctx->head->parent_offset = parent_offset;
        break;
      } else {
        break;
      }
    }
  } while (ctx->enc_count);
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  return 0;
}
static CYTHON_INLINE PyObject *
__pyx_buffmt_parse_array(__Pyx_BufFmt_Context* ctx, const char** tsp)
{
    const char *ts = *tsp;
    int i = 0, number;
    int ndim = ctx->head->field->type->ndim;
;
    ++ts;
    if (ctx->new_count != 1) {
        PyErr_SetString(PyExc_ValueError,
                        "Cannot handle repeated arrays in format string");
        return NULL;
    }
    if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
    while (*ts && *ts != ')') {
        switch (*ts) {
            case ' ': case '\f': case '\r': case '\n': case '\t': case '\v':  continue;
            default:  break;
        }
        number = __Pyx_BufFmt_ExpectNumber(&ts);
        if (number == -1) return NULL;
        if (i < ndim && (size_t) number != ctx->head->field->type->arraysize[i])
            return PyErr_Format(PyExc_ValueError,
                        "Expected a dimension of size %zu, got %d",
                        ctx->head->field->type->arraysize[i], number);
        if (*ts != ',' && *ts != ')')
            return PyErr_Format(PyExc_ValueError,
                                "Expected a comma in format string, got '%c'", *ts);
        if (*ts == ',') ts++;
        i++;
    }
    if (i != ndim)
        return PyErr_Format(PyExc_ValueError, "Expected %d dimension(s), got %d",
                            ctx->head->field->type->ndim, i);
    if (!*ts) {
        PyErr_SetString(PyExc_ValueError,
                        "Unexpected end of format string, expected ')'");
        return NULL;
    }
    ctx->is_valid_array = 1;
    ctx->new_count = 1;
    *tsp = ++ts;
    return Py_None;
}
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts) {
  int got_Z = 0;
  while (1) {
    switch(*ts) {
      case 0:
        if (ctx->enc_type != 0 && ctx->head == NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        if (ctx->head != NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        return ts;
      case ' ':
      case '\r':
      case '\n':
        ++ts;
        break;
      case '<':
        if (!__Pyx_IsLittleEndian()) {
          PyErr_SetString(PyExc_ValueError, "Little-endian buffer not supported on big-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '>':
      case '!':
        if (__Pyx_IsLittleEndian()) {
          PyErr_SetString(PyExc_ValueError, "Big-endian buffer not supported on little-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '=':
      case '@':
      case '^':
        ctx->new_packmode = *ts++;
        break;
      case 'T':
        {
          const char* ts_after_sub;
          size_t i, struct_count = ctx->new_count;
          size_t struct_alignment = ctx->struct_alignment;
          ctx->new_count = 1;
          ++ts;
          if (*ts != '{') {
            PyErr_SetString(PyExc_ValueError, "Buffer acquisition: Expected '{' after 'T'");
            return NULL;
          }
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          ctx->enc_count = 0;
          ctx->struct_alignment = 0;
          ++ts;
          ts_after_sub = ts;
          for (i = 0; i != struct_count; ++i) {
            ts_after_sub = __Pyx_BufFmt_CheckString(ctx, ts);
            if (!ts_after_sub) return NULL;
          }
          ts = ts_after_sub;
          if (struct_alignment) ctx->struct_alignment = struct_alignment;
        }
        break;
      case '}':
        {
          size_t alignment = ctx->struct_alignment;
          ++ts;
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          if (alignment && ctx->fmt_offset % alignment) {
            ctx->fmt_offset += alignment - (ctx->fmt_offset % alignment);
          }
        }
        return ts;
      case 'x':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->fmt_offset += ctx->new_count;
        ctx->new_count = 1;
        ctx->enc_count = 0;
        ctx->enc_type = 0;
        ctx->enc_packmode = ctx->new_packmode;
        ++ts;
        break;
      case 'Z':
        got_Z = 1;
        ++ts;
        if (*ts != 'f' && *ts != 'd' && *ts != 'g') {
          __Pyx_BufFmt_RaiseUnexpectedChar('Z');
          return NULL;
        }
      case 'c': case 'b': case 'B': case 'h': case 'H': case 'i': case 'I':
      case 'l': case 'L': case 'q': case 'Q':
      case 'f': case 'd': case 'g':
      case 'O': case 'p':
        if (ctx->enc_type == *ts && got_Z == ctx->is_complex &&
            ctx->enc_packmode == ctx->new_packmode) {
          ctx->enc_count += ctx->new_count;
          ctx->new_count = 1;
          got_Z = 0;
          ++ts;
          break;
        }
      case 's':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->enc_count = ctx->new_count;
        ctx->enc_packmode = ctx->new_packmode;
        ctx->enc_type = *ts;
        ctx->is_complex = got_Z;
        ++ts;
        ctx->new_count = 1;
        got_Z = 0;
        break;
      case ':':
        ++ts;
        while(*ts != ':') ++ts;
        ++ts;
        break;
      case '(':
        if (!__pyx_buffmt_parse_array(ctx, &ts)) return NULL;
        break;
      default:
        {
          int number = __Pyx_BufFmt_ExpectNumber(&ts);
          if (number == -1) return NULL;
          ctx->new_count = (size_t)number;
        }
    }
  }
}
static CYTHON_INLINE void __Pyx_ZeroBuffer(Py_buffer* buf) {
  buf->buf = NULL;
  buf->obj = NULL;
  buf->strides = __Pyx_zeros;
  buf->shape = __Pyx_zeros;
  buf->suboffsets = __Pyx_minusones;
}
static CYTHON_INLINE int __Pyx_GetBufferAndValidate(
        Py_buffer* buf, PyObject* obj,  __Pyx_TypeInfo* dtype, int flags,
        int nd, int cast, __Pyx_BufFmt_StackElem* stack)
{
  if (obj == Py_None || obj == NULL) {
    __Pyx_ZeroBuffer(buf);
    return 0;
  }
  buf->buf = NULL;
  if (__Pyx_GetBuffer(obj, buf, flags) == -1) goto fail;
  if (buf->ndim != nd) {
    PyErr_Format(PyExc_ValueError,
                 "Buffer has wrong number of dimensions (expected %d, got %d)",
                 nd, buf->ndim);
    goto fail;
  }
  if (!cast) {
    __Pyx_BufFmt_Context ctx;
    __Pyx_BufFmt_Init(&ctx, stack, dtype);
    if (!__Pyx_BufFmt_CheckString(&ctx, buf->format)) goto fail;
  }
  if ((unsigned)buf->itemsize != dtype->size) {
    PyErr_Format(PyExc_ValueError,
      "Item size of buffer (%" CYTHON_FORMAT_SSIZE_T "d byte%s) does not match size of '%s' (%" CYTHON_FORMAT_SSIZE_T "d byte%s)",
      buf->itemsize, (buf->itemsize > 1) ? "s" : "",
      dtype->name, (Py_ssize_t)dtype->size, (dtype->size > 1) ? "s" : "");
    goto fail;
  }
  if (buf->suboffsets == NULL) buf->suboffsets = __Pyx_minusones;
  return 0;
fail:;
  __Pyx_ZeroBuffer(buf);
  return -1;
}
static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info) {
  if (info->buf == NULL) return;
  if (info->suboffsets == __Pyx_minusones) info->suboffsets = NULL;
  __Pyx_ReleaseBuffer(info);
}

static int
__Pyx_init_memviewslice(struct __pyx_memoryview_obj *memview,
                        int ndim,
                        __Pyx_memviewslice *memviewslice,
                        int memview_is_new_reference)
{
    __Pyx_RefNannyDeclarations
    int i, retval=-1;
    Py_buffer *buf = &memview->view;
    __Pyx_RefNannySetupContext("init_memviewslice", 0);
    if (!buf) {
        PyErr_SetString(PyExc_ValueError,
            "buf is NULL.");
        goto fail;
    } else if (memviewslice->memview || memviewslice->data) {
        PyErr_SetString(PyExc_ValueError,
            "memviewslice is already initialized!");
        goto fail;
    }
    if (buf->strides) {
        for (i = 0; i < ndim; i++) {
            memviewslice->strides[i] = buf->strides[i];
        }
    } else {
        Py_ssize_t stride = buf->itemsize;
        for (i = ndim - 1; i >= 0; i--) {
            memviewslice->strides[i] = stride;
            stride *= buf->shape[i];
        }
    }
    for (i = 0; i < ndim; i++) {
        memviewslice->shape[i]   = buf->shape[i];
        if (buf->suboffsets) {
            memviewslice->suboffsets[i] = buf->suboffsets[i];
        } else {
            memviewslice->suboffsets[i] = -1;
        }
    }
    memviewslice->memview = memview;
    memviewslice->data = (char *)buf->buf;
    if (__pyx_add_acquisition_count(memview) == 0 && !memview_is_new_reference) {
        Py_INCREF(memview);
    }
    retval = 0;
    goto no_fail;
fail:
    memviewslice->memview = 0;
    memviewslice->data = 0;
    retval = -1;
no_fail:
    __Pyx_RefNannyFinishContext();
    return retval;
}
static CYTHON_INLINE void __pyx_fatalerror(const char *fmt, ...) {
    va_list vargs;
    char msg[200];
    va_start(vargs, fmt);
#ifdef HAVE_STDARG_PROTOTYPES
    va_start(vargs, fmt);
#else
    va_start(vargs);
#endif
    vsnprintf(msg, 200, fmt, vargs);
    Py_FatalError(msg);
    va_end(vargs);
}
static CYTHON_INLINE int
__pyx_add_acquisition_count_locked(__pyx_atomic_int *acquisition_count,
                                   PyThread_type_lock lock)
{
    int result;
    PyThread_acquire_lock(lock, 1);
    result = (*acquisition_count)++;
    PyThread_release_lock(lock);
    return result;
}
static CYTHON_INLINE int
__pyx_sub_acquisition_count_locked(__pyx_atomic_int *acquisition_count,
                                   PyThread_type_lock lock)
{
    int result;
    PyThread_acquire_lock(lock, 1);
    result = (*acquisition_count)--;
    PyThread_release_lock(lock);
    return result;
}
static CYTHON_INLINE void
__Pyx_INC_MEMVIEW(__Pyx_memviewslice *memslice, int have_gil, int lineno)
{
    int first_time;
    struct __pyx_memoryview_obj *memview = memslice->memview;
    if (!memview || (PyObject *) memview == Py_None)
        return;
    if (__pyx_get_slice_count(memview) < 0)
        __pyx_fatalerror("Acquisition count is %d (line %d)",
                         __pyx_get_slice_count(memview), lineno);
    first_time = __pyx_add_acquisition_count(memview) == 0;
    if (first_time) {
        if (have_gil) {
            Py_INCREF((PyObject *) memview);
        } else {
            PyGILState_STATE _gilstate = PyGILState_Ensure();
            Py_INCREF((PyObject *) memview);
            PyGILState_Release(_gilstate);
        }
    }
}
static CYTHON_INLINE void __Pyx_XDEC_MEMVIEW(__Pyx_memviewslice *memslice,
                                             int have_gil, int lineno) {
    int last_time;
    struct __pyx_memoryview_obj *memview = memslice->memview;
    if (!memview ) {
        return;
    } else if ((PyObject *) memview == Py_None) {
        memslice->memview = NULL;
        return;
    }
    if (__pyx_get_slice_count(memview) <= 0)
        __pyx_fatalerror("Acquisition count is %d (line %d)",
                         __pyx_get_slice_count(memview), lineno);
    last_time = __pyx_sub_acquisition_count(memview) == 1;
    memslice->data = NULL;
    if (last_time) {
        if (have_gil) {
            Py_CLEAR(memslice->memview);
        } else {
            PyGILState_STATE _gilstate = PyGILState_Ensure();
            Py_CLEAR(memslice->memview);
            PyGILState_Release(_gilstate);
        }
    } else {
        memslice->memview = NULL;
    }
}

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_Restore(type, value, tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(type, value, tb);
#endif
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                if (PyObject_IsSubclass(instance_class, type)) {
                    type = instance_class;
                } else {
                    instance_class = NULL;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
#if PY_VERSION_HEX >= 0x03030000
    if (cause) {
#else
    if (cause && cause != Py_None) {
#endif
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(tmp_type, tmp_value, tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject* args = PyTuple_Pack(1, arg);
    return (likely(args)) ? __Pyx_PyObject_Call(func, args, NULL) : NULL;
}
#endif

static CYTHON_INLINE int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_CheckExact(key)) && unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(PyObject_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (wraparound & unlikely(i < 0)) i += PyList_GET_SIZE(o);
    if ((!boundscheck) || likely((0 <= i) & (i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (wraparound & unlikely(i < 0)) i += PyTuple_GET_SIZE(o);
    if ((!boundscheck) || likely((0 <= i) & (i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (PyErr_ExceptionMatches(PyExc_OverflowError))
                        PyErr_Clear();
                    else
                        return NULL;
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname) {
    PyErr_Format(PyExc_UnboundLocalError, "local variable '%s' referenced before assignment", varname);
}

static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(exc_type == PyExc_StopIteration) || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (PyErr_ExceptionMatches(PyExc_OverflowError))
                    PyErr_Clear();
                else
                    goto bad;
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_COMPILING_IN_CPYTHON
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

static CYTHON_INLINE int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (!j) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely((n >= 0) & (n < PyList_GET_SIZE(o)))) {
            PyObject* old = PyList_GET_ITEM(o, n);
            Py_INCREF(v);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
            return 1;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (PyErr_ExceptionMatches(PyExc_OverflowError))
                        PyErr_Clear();
                    else
                        return -1;
                }
            }
            return m->sq_ass_item(o, i, v);
        }
    }
#else
#if CYTHON_COMPILING_IN_PYPY
    if (is_list || (PySequence_Check(o) && !PyDict_Check(o))) {
#else
    if (is_list || PySequence_Check(o)) {
#endif
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyInt_FromSsize_t(i), v);
}

static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

static void __Pyx_RaiseArgumentTypeInvalid(const char* name, PyObject *obj, PyTypeObject *type) {
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
}
static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (none_allowed && obj == Py_None) return 1;
    else if (exact) {
        if (likely(Py_TYPE(obj) == type)) return 1;
        #if PY_MAJOR_VERSION == 2
        else if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(PyObject_TypeCheck(obj, type))) return 1;
    }
    __Pyx_RaiseArgumentTypeInvalid(name, obj, type);
    return 0;
}

static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_COMPILING_IN_CPYTHON
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    Py_ssize_t length;
    if (unlikely((start < 0) | (stop < 0))) {
        length = strlen(cstring);
        if (start < 0) {
            start += length;
            if (start < 0)
                start = 0;
        }
        if (stop < 0)
            stop += length;
    }
    length = stop - start;
    if (unlikely(length <= 0))
        return PyUnicode_FromUnicode(NULL, 0);
    cstring += start;
    if (decode_func) {
        return decode_func(cstring, length, errors);
    } else {
        return PyUnicode_Decode(cstring, length, encoding, errors);
    }
}

static CYTHON_INLINE void __Pyx_ExceptionSave(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
#else
    PyErr_GetExcInfo(type, value, tb);
#endif
}
static void __Pyx_ExceptionReset(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(type, value, tb);
#endif
}

static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_COMPILING_IN_CPYTHON
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
#else
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
#endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}

#if !CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values) {
    return PyObject_CallMethodObjArgs(sep, __pyx_n_s_join, values, NULL);
}
#endif

static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = (start + end) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        __pyx_d,      /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    py_frame->f_lineno = py_line;
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

static int
__pyx_typeinfo_cmp(__Pyx_TypeInfo *a, __Pyx_TypeInfo *b)
{
    int i;
    if (!a || !b)
        return 0;
    if (a == b)
        return 1;
    if (a->size != b->size || a->typegroup != b->typegroup ||
            a->is_unsigned != b->is_unsigned || a->ndim != b->ndim) {
        if (a->typegroup == 'H' || b->typegroup == 'H') {
            return a->size == b->size;
        } else {
            return 0;
        }
    }
    if (a->ndim) {
        for (i = 0; i < a->ndim; i++)
            if (a->arraysize[i] != b->arraysize[i])
                return 0;
    }
    if (a->typegroup == 'S') {
        if (a->flags != b->flags)
            return 0;
        if (a->fields || b->fields) {
            if (!(a->fields && b->fields))
                return 0;
            for (i = 0; a->fields[i].type && b->fields[i].type; i++) {
                __Pyx_StructField *field_a = a->fields + i;
                __Pyx_StructField *field_b = b->fields + i;
                if (field_a->offset != field_b->offset ||
                    !__pyx_typeinfo_cmp(field_a->type, field_b->type))
                    return 0;
            }
            return !a->fields[i].type && !b->fields[i].type;
        }
    }
    return 1;
}

static int
__pyx_check_strides(Py_buffer *buf, int dim, int ndim, int spec)
{
    if (buf->shape[dim] <= 1)
        return 1;
    if (buf->strides) {
        if (spec & __Pyx_MEMVIEW_CONTIG) {
            if (spec & (__Pyx_MEMVIEW_PTR|__Pyx_MEMVIEW_FULL)) {
                if (buf->strides[dim] != sizeof(void *)) {
                    PyErr_Format(PyExc_ValueError,
                                 "Buffer is not indirectly contiguous "
                                 "in dimension %d.", dim);
                    goto fail;
                }
            } else if (buf->strides[dim] != buf->itemsize) {
                PyErr_SetString(PyExc_ValueError,
                                "Buffer and memoryview are not contiguous "
                                "in the same dimension.");
                goto fail;
            }
        }
        if (spec & __Pyx_MEMVIEW_FOLLOW) {
            Py_ssize_t stride = buf->strides[dim];
            if (stride < 0)
                stride = -stride;
            if (stride < buf->itemsize) {
                PyErr_SetString(PyExc_ValueError,
                                "Buffer and memoryview are not contiguous "
                                "in the same dimension.");
                goto fail;
            }
        }
    } else {
        if (spec & __Pyx_MEMVIEW_CONTIG && dim != ndim - 1) {
            PyErr_Format(PyExc_ValueError,
                         "C-contiguous buffer is not contiguous in "
                         "dimension %d", dim);
            goto fail;
        } else if (spec & (__Pyx_MEMVIEW_PTR)) {
            PyErr_Format(PyExc_ValueError,
                         "C-contiguous buffer is not indirect in "
                         "dimension %d", dim);
            goto fail;
        } else if (buf->suboffsets) {
            PyErr_SetString(PyExc_ValueError,
                            "Buffer exposes suboffsets but no strides");
            goto fail;
        }
    }
    return 1;
fail:
    return 0;
}
static int
__pyx_check_suboffsets(Py_buffer *buf, int dim, CYTHON_UNUSED int ndim, int spec)
{
    if (spec & __Pyx_MEMVIEW_DIRECT) {
        if (buf->suboffsets && buf->suboffsets[dim] >= 0) {
            PyErr_Format(PyExc_ValueError,
                         "Buffer not compatible with direct access "
                         "in dimension %d.", dim);
            goto fail;
        }
    }
    if (spec & __Pyx_MEMVIEW_PTR) {
        if (!buf->suboffsets || (buf->suboffsets && buf->suboffsets[dim] < 0)) {
            PyErr_Format(PyExc_ValueError,
                         "Buffer is not indirectly accessible "
                         "in dimension %d.", dim);
            goto fail;
        }
    }
    return 1;
fail:
    return 0;
}
static int
__pyx_verify_contig(Py_buffer *buf, int ndim, int c_or_f_flag)
{
    int i;
    if (c_or_f_flag & __Pyx_IS_F_CONTIG) {
        Py_ssize_t stride = 1;
        for (i = 0; i < ndim; i++) {
            if (stride * buf->itemsize != buf->strides[i] &&
                    buf->shape[i] > 1)
            {
                PyErr_SetString(PyExc_ValueError,
                    "Buffer not fortran contiguous.");
                goto fail;
            }
            stride = stride * buf->shape[i];
        }
    } else if (c_or_f_flag & __Pyx_IS_C_CONTIG) {
        Py_ssize_t stride = 1;
        for (i = ndim - 1; i >- 1; i--) {
            if (stride * buf->itemsize != buf->strides[i] &&
                    buf->shape[i] > 1) {
                PyErr_SetString(PyExc_ValueError,
                    "Buffer not C contiguous.");
                goto fail;
            }
            stride = stride * buf->shape[i];
        }
    }
    return 1;
fail:
    return 0;
}
static int __Pyx_ValidateAndInit_memviewslice(
                int *axes_specs,
                int c_or_f_flag,
                int buf_flags,
                int ndim,
                __Pyx_TypeInfo *dtype,
                __Pyx_BufFmt_StackElem stack[],
                __Pyx_memviewslice *memviewslice,
                PyObject *original_obj)
{
    struct __pyx_memoryview_obj *memview, *new_memview;
    __Pyx_RefNannyDeclarations
    Py_buffer *buf;
    int i, spec = 0, retval = -1;
    __Pyx_BufFmt_Context ctx;
    int from_memoryview = __pyx_memoryview_check(original_obj);
    __Pyx_RefNannySetupContext("ValidateAndInit_memviewslice", 0);
    if (from_memoryview && __pyx_typeinfo_cmp(dtype, ((struct __pyx_memoryview_obj *)
                                                            original_obj)->typeinfo)) {
        memview = (struct __pyx_memoryview_obj *) original_obj;
        new_memview = NULL;
    } else {
        memview = (struct __pyx_memoryview_obj *) __pyx_memoryview_new(
                                            original_obj, buf_flags, 0, dtype);
        new_memview = memview;
        if (unlikely(!memview))
            goto fail;
    }
    buf = &memview->view;
    if (buf->ndim != ndim) {
        PyErr_Format(PyExc_ValueError,
                "Buffer has wrong number of dimensions (expected %d, got %d)",
                ndim, buf->ndim);
        goto fail;
    }
    if (new_memview) {
        __Pyx_BufFmt_Init(&ctx, stack, dtype);
        if (!__Pyx_BufFmt_CheckString(&ctx, buf->format)) goto fail;
    }
    if ((unsigned) buf->itemsize != dtype->size) {
        PyErr_Format(PyExc_ValueError,
                     "Item size of buffer (%" CYTHON_FORMAT_SSIZE_T "u byte%s) "
                     "does not match size of '%s' (%" CYTHON_FORMAT_SSIZE_T "u byte%s)",
                     buf->itemsize,
                     (buf->itemsize > 1) ? "s" : "",
                     dtype->name,
                     dtype->size,
                     (dtype->size > 1) ? "s" : "");
        goto fail;
    }
    for (i = 0; i < ndim; i++) {
        spec = axes_specs[i];
        if (!__pyx_check_strides(buf, i, ndim, spec))
            goto fail;
        if (!__pyx_check_suboffsets(buf, i, ndim, spec))
            goto fail;
    }
    if (buf->strides && !__pyx_verify_contig(buf, ndim, c_or_f_flag))
        goto fail;
    if (unlikely(__Pyx_init_memviewslice(memview, ndim, memviewslice,
                                         new_memview != NULL) == -1)) {
        goto fail;
    }
    retval = 0;
    goto no_fail;
fail:
    Py_XDECREF(new_memview);
    retval = -1;
no_fail:
    __Pyx_RefNannyFinishContext();
    return retval;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_CONTIG) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, __Pyx_IS_C_CONTIG,
                                                 (PyBUF_C_CONTIGUOUS | PyBUF_FORMAT | PyBUF_WRITABLE), 1,
                                                 &__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dc_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[2];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_CONTIG) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, __Pyx_IS_C_CONTIG,
                                                 (PyBUF_C_CONTIGUOUS | PyBUF_FORMAT | PyBUF_WRITABLE), 1,
                                                 &__Pyx_TypeInfo_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_d_d_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_FOLLOW), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_FOLLOW), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_CONTIG) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, __Pyx_IS_C_CONTIG,
                                                 (PyBUF_C_CONTIGUOUS | PyBUF_FORMAT | PyBUF_WRITABLE), 3,
                                                 &__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

#if PY_MAJOR_VERSION < 3
static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags) {
    if (PyObject_CheckBuffer(obj)) return PyObject_GetBuffer(obj, view, flags);
        if (PyObject_TypeCheck(obj, __pyx_ptype_5numpy_ndarray)) return __pyx_pw_5numpy_7ndarray_1__getbuffer__(obj, view, flags);
        if (PyObject_TypeCheck(obj, __pyx_array_type)) return __pyx_array_getbuffer(obj, view, flags);
        if (PyObject_TypeCheck(obj, __pyx_memoryview_type)) return __pyx_memoryview_getbuffer(obj, view, flags);
    PyErr_Format(PyExc_TypeError, "'%.200s' does not have the buffer interface", Py_TYPE(obj)->tp_name);
    return -1;
}
static void __Pyx_ReleaseBuffer(Py_buffer *view) {
    PyObject *obj = view->obj;
    if (!obj) return;
    if (PyObject_CheckBuffer(obj)) {
        PyBuffer_Release(view);
        return;
    }
        if (PyObject_TypeCheck(obj, __pyx_ptype_5numpy_ndarray)) { __pyx_pw_5numpy_7ndarray_3__releasebuffer__(obj, view); return; }
    Py_DECREF(obj);
    view->obj = NULL;
}
#endif


            static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_VERSION_HEX < 0x03030000
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                #if PY_VERSION_HEX < 0x03030000
                PyObject *py_level = PyInt_FromLong(1);
                if (!py_level)
                    goto bad;
                module = PyObject_CallFunctionObjArgs(py_import,
                    name, global_dict, empty_dict, list, py_level, NULL);
                Py_DECREF(py_level);
                #else
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                #endif
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_VERSION_HEX < 0x03030000
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_VERSION_HEX < 0x03030000
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(int) <= sizeof(unsigned long long)) {
            return PyLong_FromUnsignedLongLong((unsigned long long) value);
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(long long)) {
            return PyLong_FromLongLong((long long) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)       \
    {                                                                     \
        func_type value = func_value;                                     \
        if (sizeof(target_type) < sizeof(func_type)) {                    \
            if (unlikely(value != (func_type) (target_type) value)) {     \
                func_type zero = 0;                                       \
                if (is_unsigned && unlikely(value < zero))                \
                    goto raise_neg_overflow;                              \
                else                                                      \
                    goto raise_overflow;                                  \
            }                                                             \
        }                                                                 \
        return (target_type) value;                                       \
    }

#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
 #endif
#endif

static CYTHON_INLINE Py_intptr_t __Pyx_PyInt_As_Py_intptr_t(PyObject *x) {
    const Py_intptr_t neg_one = (Py_intptr_t) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(Py_intptr_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (Py_intptr_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(Py_intptr_t, digit, ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
            if (sizeof(Py_intptr_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(Py_intptr_t) <= sizeof(unsigned long long)) {
                __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long long, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(Py_intptr_t,  digit, +(((PyLongObject*)x)->ob_digit[0]));
                case -1: __PYX_VERIFY_RETURN_INT(Py_intptr_t, sdigit, -(sdigit) ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
            if (sizeof(Py_intptr_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, PyLong_AsLong(x))
            } else if (sizeof(Py_intptr_t) <= sizeof(long long)) {
                __PYX_VERIFY_RETURN_INT(Py_intptr_t, long long, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            Py_intptr_t val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (Py_intptr_t) -1;
        }
    } else {
        Py_intptr_t val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (Py_intptr_t) -1;
        val = __Pyx_PyInt_As_Py_intptr_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to Py_intptr_t");
    return (Py_intptr_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to Py_intptr_t");
    return (Py_intptr_t) -1;
}

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT(int, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(int) <= sizeof(unsigned long long)) {
                __PYX_VERIFY_RETURN_INT(int, unsigned long long, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +(((PyLongObject*)x)->ob_digit[0]));
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, -(sdigit) ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT(int, long, PyLong_AsLong(x))
            } else if (sizeof(int) <= sizeof(long long)) {
                __PYX_VERIFY_RETURN_INT(int, long long, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_d_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_FOLLOW), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_CONTIG) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, __Pyx_IS_C_CONTIG,
                                                 (PyBUF_C_CONTIGUOUS | PyBUF_FORMAT | PyBUF_WRITABLE), 2,
                                                 &__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_d_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_FOLLOW), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_CONTIG) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, __Pyx_IS_C_CONTIG,
                                                 (PyBUF_C_CONTIGUOUS | PyBUF_FORMAT | PyBUF_WRITABLE), 2,
                                                 &__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dc_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_CONTIG) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, __Pyx_IS_C_CONTIG,
                                                 (PyBUF_C_CONTIGUOUS | PyBUF_FORMAT | PyBUF_WRITABLE), 1,
                                                 &__Pyx_TypeInfo_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(long) <= sizeof(unsigned long long)) {
            return PyLong_FromUnsignedLongLong((unsigned long long) value);
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(long long)) {
            return PyLong_FromLongLong((long long) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Py_intptr_t(Py_intptr_t value) {
    const Py_intptr_t neg_one = (Py_intptr_t) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(Py_intptr_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(Py_intptr_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(Py_intptr_t) <= sizeof(unsigned long long)) {
            return PyLong_FromUnsignedLongLong((unsigned long long) value);
        }
    } else {
        if (sizeof(Py_intptr_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(Py_intptr_t) <= sizeof(long long)) {
            return PyLong_FromLongLong((long long) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(Py_intptr_t),
                                     little, !is_unsigned);
    }
}

static PyObject *__pyx_memview_get_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t(const char *itemp) {
    return (PyObject *) PyFloat_FromDouble(*(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) itemp);
}
static int __pyx_memview_set_nn___pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t(const char *itemp, PyObject *obj) {
    __pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t value = __pyx_PyFloat_AsDouble(obj);
    if ((value == (npy_float64)-1) && PyErr_Occurred())
        return 0;
    *(__pyx_t_7sklearn_9neighbors_8typedefs_DTYPE_t *) itemp = value;
    return 1;
}

static CYTHON_INLINE long __Pyx_pow_long(long b, long e) {
    long t = b;
    switch (e) {
        case 3:
            t *= b;
        case 2:
            t *= b;
        case 1:
            return t;
        case 0:
            return 1;
    }
    #if 1
    if (unlikely(e<0)) return 0;
    #endif
    t = 1;
    while (likely(e)) {
        t *= (b * (e&1)) | ((~e)&1);    /* 1 or b */
        b *= b;
        e >>= 1;
    }
    return t;
}

static PyObject *__pyx_memview_get_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t(const char *itemp) {
    return (PyObject *) __Pyx_PyInt_From_Py_intptr_t(*(__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) itemp);
}
static int __pyx_memview_set_nn___pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t(const char *itemp, PyObject *obj) {
    __pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t value = __Pyx_PyInt_As_Py_intptr_t(obj);
    if ((value == (npy_intp)-1) && PyErr_Occurred())
        return 0;
    *(__pyx_t_7sklearn_9neighbors_8typedefs_ITYPE_t *) itemp = value;
    return 1;
}

static PyObject* __pyx_convert__to_py___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t(struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t s) {
              PyObject* res;
              PyObject* member;
              res = PyDict_New(); if (res == NULL) return NULL;
              member = __Pyx_PyInt_From_Py_intptr_t(s.idx_start); if (member == NULL) goto bad;
              if (PyDict_SetItem(res, __pyx_n_s_idx_start, member) < 0) goto bad;
              Py_DECREF(member);
              member = __Pyx_PyInt_From_Py_intptr_t(s.idx_end); if (member == NULL) goto bad;
              if (PyDict_SetItem(res, __pyx_n_s_idx_end, member) < 0) goto bad;
              Py_DECREF(member);
              member = __Pyx_PyInt_From_Py_intptr_t(s.is_leaf); if (member == NULL) goto bad;
              if (PyDict_SetItem(res, __pyx_n_s_is_leaf, member) < 0) goto bad;
              Py_DECREF(member);
              member = PyFloat_FromDouble(s.radius); if (member == NULL) goto bad;
              if (PyDict_SetItem(res, __pyx_n_s_radius, member) < 0) goto bad;
              Py_DECREF(member);
              return res;
              bad:
              Py_XDECREF(member);
              Py_DECREF(res);
              return NULL;
            }
            static struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t __pyx_convert__from_py___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t(PyObject * o) {
    struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t result;
    PyObject *value = NULL;
    if (!PyMapping_Check(o)) {
        PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "a mapping", Py_TYPE(o)->tp_name);
        goto bad;
    }
        value = PyObject_GetItem(o, __pyx_n_s_idx_start);
        if (!value) {
            PyErr_Format(PyExc_ValueError, \
                "No value specified for struct attribute '%.200s'", "idx_start");
            goto bad;
        }
        result.idx_start = __Pyx_PyInt_As_Py_intptr_t(value);
        if ((result.idx_start == (npy_intp)-1) && PyErr_Occurred())
            goto bad;
        Py_DECREF(value);
        value = PyObject_GetItem(o, __pyx_n_s_idx_end);
        if (!value) {
            PyErr_Format(PyExc_ValueError, \
                "No value specified for struct attribute '%.200s'", "idx_end");
            goto bad;
        }
        result.idx_end = __Pyx_PyInt_As_Py_intptr_t(value);
        if ((result.idx_end == (npy_intp)-1) && PyErr_Occurred())
            goto bad;
        Py_DECREF(value);
        value = PyObject_GetItem(o, __pyx_n_s_is_leaf);
        if (!value) {
            PyErr_Format(PyExc_ValueError, \
                "No value specified for struct attribute '%.200s'", "is_leaf");
            goto bad;
        }
        result.is_leaf = __Pyx_PyInt_As_Py_intptr_t(value);
        if ((result.is_leaf == (npy_intp)-1) && PyErr_Occurred())
            goto bad;
        Py_DECREF(value);
        value = PyObject_GetItem(o, __pyx_n_s_radius);
        if (!value) {
            PyErr_Format(PyExc_ValueError, \
                "No value specified for struct attribute '%.200s'", "radius");
            goto bad;
        }
        result.radius = __pyx_PyFloat_AsDouble(value);
        if ((result.radius == (npy_float64)-1) && PyErr_Occurred())
            goto bad;
        Py_DECREF(value);
    return result;
bad:
    Py_XDECREF(value);
    return result;
}

static PyObject *__pyx_memview_get_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t(const char *itemp) {
    return (PyObject *) __pyx_convert__to_py___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t(*(struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) itemp);
}
static int __pyx_memview_set_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t(const char *itemp, PyObject *obj) {
    struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t value = __pyx_convert__from_py___pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t(obj);
    if (PyErr_Occurred())
        return 0;
    *(struct __pyx_t_7sklearn_9neighbors_7kd_tree_NodeData_t *) itemp = value;
    return 1;
}

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eqf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sumf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_difff(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prodf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quotf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        float denom = b.real * b.real + b.imag * b.imag;
        z.real = (a.real * b.real + a.imag * b.imag) / denom;
        z.imag = (a.imag * b.real - a.real * b.imag) / denom;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_negf(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zerof(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conjf(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_absf(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_powf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(a, a);
                    case 3:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(z, a);
                    case 4:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                }
                r = a.real;
                theta = 0;
            } else {
                r = __Pyx_c_absf(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        double denom = b.real * b.real + b.imag * b.imag;
        z.real = (a.real * b.real + a.imag * b.imag) / denom;
        z.imag = (a.imag * b.real - a.real * b.imag) / denom;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(a, a);
                    case 3:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(z, a);
                    case 4:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                }
                r = a.real;
                theta = 0;
            } else {
                r = __Pyx_c_abs(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

static int
__pyx_memviewslice_is_contig(const __Pyx_memviewslice *mvs,
                             char order, int ndim)
{
    int i, index, step, start;
    Py_ssize_t itemsize = mvs->memview->view.itemsize;
    if (order == 'F') {
        step = 1;
        start = 0;
    } else {
        step = -1;
        start = ndim - 1;
    }
    for (i = 0; i < ndim; i++) {
        index = start + step * i;
        if (mvs->suboffsets[index] >= 0 || mvs->strides[index] != itemsize)
            return 0;
        itemsize *= mvs->shape[index];
    }
    return 1;
}

static void
__pyx_get_array_memory_extents(__Pyx_memviewslice *slice,
                               void **out_start, void **out_end,
                               int ndim, size_t itemsize)
{
    char *start, *end;
    int i;
    start = end = slice->data;
    for (i = 0; i < ndim; i++) {
        Py_ssize_t stride = slice->strides[i];
        Py_ssize_t extent = slice->shape[i];
        if (extent == 0) {
            *out_start = *out_end = start;
            return;
        } else {
            if (stride > 0)
                end += stride * (extent - 1);
            else
                start += stride * (extent - 1);
        }
    }
    *out_start = start;
    *out_end = end + itemsize;
}
static int
__pyx_slices_overlap(__Pyx_memviewslice *slice1,
                     __Pyx_memviewslice *slice2,
                     int ndim, size_t itemsize)
{
    void *start1, *end1, *start2, *end2;
    __pyx_get_array_memory_extents(slice1, &start1, &end1, ndim, itemsize);
    __pyx_get_array_memory_extents(slice2, &start2, &end2, ndim, itemsize);
    return (start1 < end2) && (start2 < end1);
}

static __Pyx_memviewslice
__pyx_memoryview_copy_new_contig(const __Pyx_memviewslice *from_mvs,
                                 const char *mode, int ndim,
                                 size_t sizeof_dtype, int contig_flag,
                                 int dtype_is_object)
{
    __Pyx_RefNannyDeclarations
    int i;
    __Pyx_memviewslice new_mvs = { 0, 0, { 0 }, { 0 }, { 0 } };
    struct __pyx_memoryview_obj *from_memview = from_mvs->memview;
    Py_buffer *buf = &from_memview->view;
    PyObject *shape_tuple = NULL;
    PyObject *temp_int = NULL;
    struct __pyx_array_obj *array_obj = NULL;
    struct __pyx_memoryview_obj *memview_obj = NULL;
    __Pyx_RefNannySetupContext("__pyx_memoryview_copy_new_contig", 0);
    for (i = 0; i < ndim; i++) {
        if (from_mvs->suboffsets[i] >= 0) {
            PyErr_Format(PyExc_ValueError, "Cannot copy memoryview slice with "
                                           "indirect dimensions (axis %d)", i);
            goto fail;
        }
    }
    shape_tuple = PyTuple_New(ndim);
    if (unlikely(!shape_tuple)) {
        goto fail;
    }
    __Pyx_GOTREF(shape_tuple);
    for(i = 0; i < ndim; i++) {
        temp_int = PyInt_FromSsize_t(from_mvs->shape[i]);
        if(unlikely(!temp_int)) {
            goto fail;
        } else {
            PyTuple_SET_ITEM(shape_tuple, i, temp_int);
            temp_int = NULL;
        }
    }
    array_obj = __pyx_array_new(shape_tuple, sizeof_dtype, buf->format, (char *) mode, NULL);
    if (unlikely(!array_obj)) {
        goto fail;
    }
    __Pyx_GOTREF(array_obj);
    memview_obj = (struct __pyx_memoryview_obj *) __pyx_memoryview_new(
                                    (PyObject *) array_obj, contig_flag,
                                    dtype_is_object,
                                    from_mvs->memview->typeinfo);
    if (unlikely(!memview_obj))
        goto fail;
    if (unlikely(__Pyx_init_memviewslice(memview_obj, ndim, &new_mvs, 1) < 0))
        goto fail;
    if (unlikely(__pyx_memoryview_copy_contents(*from_mvs, new_mvs, ndim, ndim,
                                                dtype_is_object) < 0))
        goto fail;
    goto no_fail;
fail:
    __Pyx_XDECREF(new_mvs.memview);
    new_mvs.memview = NULL;
    new_mvs.data = NULL;
no_fail:
    __Pyx_XDECREF(shape_tuple);
    __Pyx_XDECREF(temp_int);
    __Pyx_XDECREF(array_obj);
    __Pyx_RefNannyFinishContext();
    return new_mvs;
}

static CYTHON_INLINE PyObject *
__pyx_capsule_create(void *p, CYTHON_UNUSED const char *sig)
{
    PyObject *cobj;
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(p, sig, NULL);
#else
    cobj = PyCObject_FromVoidPtr(p, NULL);
#endif
    return cobj;
}

static CYTHON_INLINE char __Pyx_PyInt_As_char(PyObject *x) {
    const char neg_one = (char) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(char) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(char, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (char) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(char, digit, ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
            if (sizeof(char) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT(char, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(char) <= sizeof(unsigned long long)) {
                __PYX_VERIFY_RETURN_INT(char, unsigned long long, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(char,  digit, +(((PyLongObject*)x)->ob_digit[0]));
                case -1: __PYX_VERIFY_RETURN_INT(char, sdigit, -(sdigit) ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
            if (sizeof(char) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT(char, long, PyLong_AsLong(x))
            } else if (sizeof(char) <= sizeof(long long)) {
                __PYX_VERIFY_RETURN_INT(char, long long, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            char val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (char) -1;
        }
    } else {
        char val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (char) -1;
        val = __Pyx_PyInt_As_char(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to char");
    return (char) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to char");
    return (char) -1;
}

static struct __pyx_typeinfo_string __Pyx_TypeInfoToFormat(__Pyx_TypeInfo *type) {
    struct __pyx_typeinfo_string result = { {0} };
    char *buf = (char *) result.string;
    size_t size = type->size;
    switch (type->typegroup) {
        case 'H':
            *buf = 'c';
            break;
        case 'I':
        case 'U':
            if (size == 1)
                *buf = 'b';
            else if (size == 2)
                *buf = 'h';
            else if (size == 4)
                *buf = 'i';
            else if (size == 8)
                *buf = 'q';
            if (type->is_unsigned)
                *buf = toupper(*buf);
            break;
        case 'P':
            *buf = 'P';
            break;
        case 'C':
         {
            __Pyx_TypeInfo complex_type = *type;
            complex_type.typegroup = 'R';
            complex_type.size /= 2;
            *buf++ = 'Z';
            *buf = __Pyx_TypeInfoToFormat(&complex_type).string[0];
            break;
         }
        case 'R':
            if (size == 4)
                *buf = 'f';
            else if (size == 8)
                *buf = 'd';
            else
                *buf = 'g';
            break;
    }
    return result;
}

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT(long, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(long) <= sizeof(unsigned long long)) {
                __PYX_VERIFY_RETURN_INT(long, unsigned long long, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +(((PyLongObject*)x)->ob_digit[0]));
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, -(sdigit) ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT(long, long, PyLong_AsLong(x))
            } else if (sizeof(long) <= sizeof(long long)) {
                __PYX_VERIFY_RETURN_INT(long, long long, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dc_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[2];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_CONTIG) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, __Pyx_IS_C_CONTIG,
                                                 (PyBUF_C_CONTIGUOUS | PyBUF_FORMAT | PyBUF_WRITABLE), 1,
                                                 &__Pyx_TypeInfo_nn_struct___pyx_t_7sklearn_9neighbors_7kd_tree_NodeHeapData_t, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

#ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility",
            module_name, class_name);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling",
            module_name, class_name);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

#ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
#if PY_VERSION_HEX < 0x03030000
        char* defenc_c;
        PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
        if (!defenc) return NULL;
        defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        {
            char* end = defenc_c + PyBytes_GET_SIZE(defenc);
            char* c;
            for (c = defenc_c; c < end; c++) {
                if ((unsigned char) (*c) >= 128) {
                    PyUnicode_AsASCIIString(o);
                    return NULL;
                }
            }
        }
#endif
        *length = PyBytes_GET_SIZE(defenc);
        return defenc_c;
#else
        if (__Pyx_PyUnicode_READY(o) == -1) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        if (PyUnicode_IS_ASCII(o)) {
            *length = PyUnicode_GET_LENGTH(o);
            return PyUnicode_AsUTF8(o);
        } else {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
#else
        return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
    } else
#endif
#if !CYTHON_COMPILING_IN_PYPY
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x) {
  PyNumberMethods *m;
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return Py_INCREF(x), x;
  m = Py_TYPE(x)->tp_as_number;
#if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
#else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
#endif
  if (res) {
#if PY_MAJOR_VERSION < 3
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%.4s__ returned non-%.4s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b)))
      return PyInt_AS_LONG(b);
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
     #if CYTHON_USE_PYLONG_INTERNALS
       switch (Py_SIZE(b)) {
       case -1: return -(sdigit)((PyLongObject*)b)->ob_digit[0];
       case  0: return 0;
       case  1: return ((PyLongObject*)b)->ob_digit[0];
       }
     #endif
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
