/* Generated by Cython 0.22.1 */

#define PY_SSIZE_T_CLEAN
#ifndef CYTHON_USE_PYLONG_INTERNALS
#ifdef PYLONG_BITS_IN_DIGIT
#define CYTHON_USE_PYLONG_INTERNALS 0
#else
#include "pyconfig.h"
#ifdef PYLONG_BITS_IN_DIGIT
#define CYTHON_USE_PYLONG_INTERNALS 1
#else
#define CYTHON_USE_PYLONG_INTERNALS 0
#endif
#endif
#endif
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03020000)
    #error Cython requires Python 2.6+ or Python 3.2+.
#else
#define CYTHON_ABI "0_22_1"
#include <stddef.h>
#ifndef offsetof
#define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
#define CYTHON_COMPILING_IN_PYPY 1
#define CYTHON_COMPILING_IN_CPYTHON 0
#else
#define CYTHON_COMPILING_IN_PYPY 0
#define CYTHON_COMPILING_IN_CPYTHON 1
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
#define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos) \
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos) \
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ? \
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
#else
  #define CYTHON_PEP393_ENABLED 0
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
  #define __Pyx_PyFrozenSet_Size(s)         PyObject_Size(s)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ? \
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
  #define __Pyx_PyFrozenSet_Size(s)         PySet_Size(s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#ifndef CYTHON_INLINE
  #if defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  /* Initialize NaN. The sign is irrelevant, an exponent with all bits 1 and
   a nonzero mantissa means NaN. If the first bit in the mantissa is 1, it is
   a quiet NaN. */
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#define __Pyx_void_to_None(void_result) (void_result, Py_INCREF(Py_None), Py_None)
#ifdef __cplusplus
template<typename T>
void __Pyx_call_destructor(T* x) {
    x->~T();
}
template<typename T>
class __Pyx_FakeReference {
  public:
    __Pyx_FakeReference() : ptr(NULL) { }
    __Pyx_FakeReference(T& ref) : ptr(&ref) { }
    T *operator->() { return ptr; }
    operator T&() { return *ptr; }
  private:
    T *ptr;
};
#endif


#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
#define _USE_MATH_DEFINES
#endif
#include <math.h>
#define __PYX_HAVE__sklearn__tree___splitter
#define __PYX_HAVE_API__sklearn__tree___splitter
#include "string.h"
#include "stdio.h"
#include "stdlib.h"
#include "numpy/arrayobject.h"
#include "numpy/ufuncobject.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
typedef struct {PyObject **p; char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (    \
    (sizeof(type) < sizeof(Py_ssize_t))  ||             \
    (sizeof(type) > sizeof(Py_ssize_t) &&               \
          likely(v < (type)PY_SSIZE_T_MAX ||            \
                 v == (type)PY_SSIZE_T_MAX)  &&         \
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||       \
                                v == (type)PY_SSIZE_T_MIN)))  ||  \
    (sizeof(type) == sizeof(Py_ssize_t) &&              \
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||        \
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyObject_AsSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if PY_MAJOR_VERSION < 3
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
#define __Pyx_Py_UNICODE_strlen Py_UNICODE_strlen
#endif
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_Owned_Py_None(b) (Py_INCREF(Py_None), Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? (Py_INCREF(Py_True), Py_True) : (Py_INCREF(Py_False), Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_COMPILING_IN_CPYTHON
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */

static PyObject *__pyx_m;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "sklearn/tree/_splitter.pyx",
  "sklearn/tree/_splitter.pxd",
  "__init__.pxd",
  "type.pxd",
  "sklearn/tree/_criterion.pxd",
  "sklearn/tree/_utils.pxd",
};
#define IS_UNSIGNED(type) (((type) -1) > 0)
struct __Pyx_StructField_;
#define __PYX_BUF_FLAGS_PACKED_STRUCT (1 << 0)
typedef struct {
  const char* name;
  struct __Pyx_StructField_* fields;
  size_t size;
  size_t arraysize[8];
  int ndim;
  char typegroup;
  char is_unsigned;
  int flags;
} __Pyx_TypeInfo;
typedef struct __Pyx_StructField_ {
  __Pyx_TypeInfo* type;
  const char* name;
  size_t offset;
} __Pyx_StructField;
typedef struct {
  __Pyx_StructField* field;
  size_t parent_offset;
} __Pyx_BufFmt_StackElem;
typedef struct {
  __Pyx_StructField root;
  __Pyx_BufFmt_StackElem* head;
  size_t fmt_offset;
  size_t new_count, enc_count;
  size_t struct_alignment;
  int is_complex;
  char enc_type;
  char new_packmode;
  char enc_packmode;
  char is_valid_array;
} __Pyx_BufFmt_Context;


/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":726
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":727
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":728
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":729
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":733
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":734
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":735
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":736
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":740
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":741
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":750
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":751
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":752
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":754
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":755
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":756
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":758
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":759
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":761
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":762
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":763
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;

/* "sklearn/tree/_criterion.pxd":15
 * cimport numpy as np
 * 
 * ctypedef np.npy_float32 DTYPE_t          # Type of X             # <<<<<<<<<<<<<<
 * ctypedef np.npy_float64 DOUBLE_t         # Type of y, sample_weight
 * ctypedef np.npy_intp SIZE_t              # Type for indices and counters
 */
typedef npy_float32 __pyx_t_7sklearn_4tree_10_criterion_DTYPE_t;

/* "sklearn/tree/_criterion.pxd":16
 * 
 * ctypedef np.npy_float32 DTYPE_t          # Type of X
 * ctypedef np.npy_float64 DOUBLE_t         # Type of y, sample_weight             # <<<<<<<<<<<<<<
 * ctypedef np.npy_intp SIZE_t              # Type for indices and counters
 * ctypedef np.npy_int32 INT32_t            # Signed 32 bit integer
 */
typedef npy_float64 __pyx_t_7sklearn_4tree_10_criterion_DOUBLE_t;

/* "sklearn/tree/_criterion.pxd":17
 * ctypedef np.npy_float32 DTYPE_t          # Type of X
 * ctypedef np.npy_float64 DOUBLE_t         # Type of y, sample_weight
 * ctypedef np.npy_intp SIZE_t              # Type for indices and counters             # <<<<<<<<<<<<<<
 * ctypedef np.npy_int32 INT32_t            # Signed 32 bit integer
 * ctypedef np.npy_uint32 UINT32_t          # Unsigned 32 bit integer
 */
typedef npy_intp __pyx_t_7sklearn_4tree_10_criterion_SIZE_t;

/* "sklearn/tree/_criterion.pxd":18
 * ctypedef np.npy_float64 DOUBLE_t         # Type of y, sample_weight
 * ctypedef np.npy_intp SIZE_t              # Type for indices and counters
 * ctypedef np.npy_int32 INT32_t            # Signed 32 bit integer             # <<<<<<<<<<<<<<
 * ctypedef np.npy_uint32 UINT32_t          # Unsigned 32 bit integer
 * 
 */
typedef npy_int32 __pyx_t_7sklearn_4tree_10_criterion_INT32_t;

/* "sklearn/tree/_criterion.pxd":19
 * ctypedef np.npy_intp SIZE_t              # Type for indices and counters
 * ctypedef np.npy_int32 INT32_t            # Signed 32 bit integer
 * ctypedef np.npy_uint32 UINT32_t          # Unsigned 32 bit integer             # <<<<<<<<<<<<<<
 * 
 * cdef class Criterion:
 */
typedef npy_uint32 __pyx_t_7sklearn_4tree_10_criterion_UINT32_t;

/* "sklearn/tree/_utils.pxd":13
 * cimport numpy as np
 * 
 * ctypedef np.npy_float32 DTYPE_t          # Type of X             # <<<<<<<<<<<<<<
 * ctypedef np.npy_float64 DOUBLE_t         # Type of y, sample_weight
 * ctypedef np.npy_intp SIZE_t              # Type for indices and counters
 */
typedef npy_float32 __pyx_t_7sklearn_4tree_6_utils_DTYPE_t;

/* "sklearn/tree/_utils.pxd":14
 * 
 * ctypedef np.npy_float32 DTYPE_t          # Type of X
 * ctypedef np.npy_float64 DOUBLE_t         # Type of y, sample_weight             # <<<<<<<<<<<<<<
 * ctypedef np.npy_intp SIZE_t              # Type for indices and counters
 * ctypedef np.npy_int32 INT32_t            # Signed 32 bit integer
 */
typedef npy_float64 __pyx_t_7sklearn_4tree_6_utils_DOUBLE_t;

/* "sklearn/tree/_utils.pxd":15
 * ctypedef np.npy_float32 DTYPE_t          # Type of X
 * ctypedef np.npy_float64 DOUBLE_t         # Type of y, sample_weight
 * ctypedef np.npy_intp SIZE_t              # Type for indices and counters             # <<<<<<<<<<<<<<
 * ctypedef np.npy_int32 INT32_t            # Signed 32 bit integer
 * ctypedef np.npy_uint32 UINT32_t          # Unsigned 32 bit integer
 */
typedef npy_intp __pyx_t_7sklearn_4tree_6_utils_SIZE_t;

/* "sklearn/tree/_utils.pxd":16
 * ctypedef np.npy_float64 DOUBLE_t         # Type of y, sample_weight
 * ctypedef np.npy_intp SIZE_t              # Type for indices and counters
 * ctypedef np.npy_int32 INT32_t            # Signed 32 bit integer             # <<<<<<<<<<<<<<
 * ctypedef np.npy_uint32 UINT32_t          # Unsigned 32 bit integer
 * 
 */
typedef npy_int32 __pyx_t_7sklearn_4tree_6_utils_INT32_t;

/* "sklearn/tree/_utils.pxd":17
 * ctypedef np.npy_intp SIZE_t              # Type for indices and counters
 * ctypedef np.npy_int32 INT32_t            # Signed 32 bit integer
 * ctypedef np.npy_uint32 UINT32_t          # Unsigned 32 bit integer             # <<<<<<<<<<<<<<
 * 
 * cdef enum:
 */
typedef npy_uint32 __pyx_t_7sklearn_4tree_6_utils_UINT32_t;
typedef npy_float32 __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t;

/* "sklearn/tree/_splitter.pxd":18
 * 
 * ctypedef np.npy_float32 DTYPE_t          # Type of X
 * ctypedef np.npy_float64 DOUBLE_t         # Type of y, sample_weight             # <<<<<<<<<<<<<<
 * ctypedef np.npy_intp SIZE_t              # Type for indices and counters
 * ctypedef np.npy_int32 INT32_t            # Signed 32 bit integer
 */
typedef npy_float64 __pyx_t_7sklearn_4tree_9_splitter_DOUBLE_t;

/* "sklearn/tree/_splitter.pxd":19
 * ctypedef np.npy_float32 DTYPE_t          # Type of X
 * ctypedef np.npy_float64 DOUBLE_t         # Type of y, sample_weight
 * ctypedef np.npy_intp SIZE_t              # Type for indices and counters             # <<<<<<<<<<<<<<
 * ctypedef np.npy_int32 INT32_t            # Signed 32 bit integer
 * ctypedef np.npy_uint32 UINT32_t          # Unsigned 32 bit integer
 */
typedef npy_intp __pyx_t_7sklearn_4tree_9_splitter_SIZE_t;

/* "sklearn/tree/_splitter.pxd":20
 * ctypedef np.npy_float64 DOUBLE_t         # Type of y, sample_weight
 * ctypedef np.npy_intp SIZE_t              # Type for indices and counters
 * ctypedef np.npy_int32 INT32_t            # Signed 32 bit integer             # <<<<<<<<<<<<<<
 * ctypedef np.npy_uint32 UINT32_t          # Unsigned 32 bit integer
 * 
 */
typedef npy_int32 __pyx_t_7sklearn_4tree_9_splitter_INT32_t;

/* "sklearn/tree/_splitter.pxd":21
 * ctypedef np.npy_intp SIZE_t              # Type for indices and counters
 * ctypedef np.npy_int32 INT32_t            # Signed 32 bit integer
 * ctypedef np.npy_uint32 UINT32_t          # Unsigned 32 bit integer             # <<<<<<<<<<<<<<
 * 
 * cdef struct SplitRecord:
 */
typedef npy_uint32 __pyx_t_7sklearn_4tree_9_splitter_UINT32_t;
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif


/*--- Type declarations ---*/
struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion;
struct __pyx_obj_7sklearn_4tree_6_utils_Stack;
struct __pyx_obj_7sklearn_4tree_6_utils_PriorityHeap;
struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter;
struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter;
struct __pyx_obj_7sklearn_4tree_9_splitter_BestSplitter;
struct __pyx_obj_7sklearn_4tree_9_splitter_RandomSplitter;
struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter;
struct __pyx_obj_7sklearn_4tree_9_splitter_BestSparseSplitter;
struct __pyx_obj_7sklearn_4tree_9_splitter_RandomSparseSplitter;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":765
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":766
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":767
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":769
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;
struct __pyx_t_7sklearn_4tree_6_utils_StackRecord;
struct __pyx_t_7sklearn_4tree_6_utils_PriorityHeapRecord;

/* "sklearn/tree/_utils.pxd":19
 * ctypedef np.npy_uint32 UINT32_t          # Unsigned 32 bit integer
 * 
 * cdef enum:             # <<<<<<<<<<<<<<
 *     # Max value for our rand_r replacement (near the bottom).
 *     # We don't use RAND_MAX because it's different across platforms and
 */
enum  {
  __pyx_e_7sklearn_4tree_6_utils_RAND_R_MAX = 0x7FFFFFFF
};

/* "sklearn/tree/_utils.pxd":58
 * 
 * # A record on the stack for depth-first tree growing
 * cdef struct StackRecord:             # <<<<<<<<<<<<<<
 *     SIZE_t start
 *     SIZE_t end
 */
struct __pyx_t_7sklearn_4tree_6_utils_StackRecord {
  __pyx_t_7sklearn_4tree_6_utils_SIZE_t start;
  __pyx_t_7sklearn_4tree_6_utils_SIZE_t end;
  __pyx_t_7sklearn_4tree_6_utils_SIZE_t depth;
  __pyx_t_7sklearn_4tree_6_utils_SIZE_t parent;
  int is_left;
  double impurity;
  __pyx_t_7sklearn_4tree_6_utils_SIZE_t n_constant_features;
};

/* "sklearn/tree/_utils.pxd":84
 * 
 * # A record on the frontier for best-first tree growing
 * cdef struct PriorityHeapRecord:             # <<<<<<<<<<<<<<
 *     SIZE_t node_id
 *     SIZE_t start
 */
struct __pyx_t_7sklearn_4tree_6_utils_PriorityHeapRecord {
  __pyx_t_7sklearn_4tree_6_utils_SIZE_t node_id;
  __pyx_t_7sklearn_4tree_6_utils_SIZE_t start;
  __pyx_t_7sklearn_4tree_6_utils_SIZE_t end;
  __pyx_t_7sklearn_4tree_6_utils_SIZE_t pos;
  __pyx_t_7sklearn_4tree_6_utils_SIZE_t depth;
  int is_leaf;
  double impurity;
  double impurity_left;
  double impurity_right;
  double improvement;
};
struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord;
struct __pyx_opt_args_7sklearn_4tree_9_splitter_8Splitter_init;
struct __pyx_opt_args_7sklearn_4tree_9_splitter_17BaseDenseSplitter_init;
struct __pyx_opt_args_7sklearn_4tree_9_splitter_18BaseSparseSplitter_init;

/* "sklearn/tree/_splitter.pxd":23
 * ctypedef np.npy_uint32 UINT32_t          # Unsigned 32 bit integer
 * 
 * cdef struct SplitRecord:             # <<<<<<<<<<<<<<
 *     # Data to track sample split
 *     SIZE_t feature         # Which feature to split on.
 */
struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord {
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t feature;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t pos;
  double threshold;
  double improvement;
  double impurity_left;
  double impurity_right;
};

/* "sklearn/tree/_splitter.pxd":84
 * 
 *     # Methods
 *     cdef void init(self, object X, np.ndarray y,             # <<<<<<<<<<<<<<
 *                    DOUBLE_t* sample_weight,
 *                    np.ndarray X_idx_sorted=*) except *
 */
struct __pyx_opt_args_7sklearn_4tree_9_splitter_8Splitter_init {
  int __pyx_n;
  PyArrayObject *X_idx_sorted;
};

/* "sklearn/tree/_splitter.pyx":260
 *             free(self.sample_mask)
 * 
 *     cdef void init(self,             # <<<<<<<<<<<<<<
 *                    object X,
 *                    np.ndarray[DOUBLE_t, ndim=2, mode="c"] y,
 */
struct __pyx_opt_args_7sklearn_4tree_9_splitter_17BaseDenseSplitter_init {
  int __pyx_n;
  PyArrayObject *X_idx_sorted;
};

/* "sklearn/tree/_splitter.pyx":868
 *         free(self.sorted_samples)
 * 
 *     cdef void init(self,             # <<<<<<<<<<<<<<
 *                    object X,
 *                    np.ndarray[DOUBLE_t, ndim=2, mode="c"] y,
 */
struct __pyx_opt_args_7sklearn_4tree_9_splitter_18BaseSparseSplitter_init {
  int __pyx_n;
  PyArrayObject *X_idx_sorted;
};

/* "sklearn/tree/_criterion.pxd":21
 * ctypedef np.npy_uint32 UINT32_t          # Unsigned 32 bit integer
 * 
 * cdef class Criterion:             # <<<<<<<<<<<<<<
 *     # The criterion computes the impurity of a node and the reduction of
 *     # impurity of a split on that node. It also computes the output statistics
 */
struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion {
  PyObject_HEAD
  struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *__pyx_vtab;
  __pyx_t_7sklearn_4tree_10_criterion_DOUBLE_t *y;
  __pyx_t_7sklearn_4tree_10_criterion_SIZE_t y_stride;
  __pyx_t_7sklearn_4tree_10_criterion_DOUBLE_t *sample_weight;
  __pyx_t_7sklearn_4tree_10_criterion_SIZE_t *samples;
  __pyx_t_7sklearn_4tree_10_criterion_SIZE_t start;
  __pyx_t_7sklearn_4tree_10_criterion_SIZE_t pos;
  __pyx_t_7sklearn_4tree_10_criterion_SIZE_t end;
  __pyx_t_7sklearn_4tree_10_criterion_SIZE_t n_outputs;
  __pyx_t_7sklearn_4tree_10_criterion_SIZE_t n_node_samples;
  double weighted_n_samples;
  double weighted_n_node_samples;
  double weighted_n_left;
  double weighted_n_right;
  double *sum_total;
  double *sum_left;
  double *sum_right;
};


/* "sklearn/tree/_utils.pxd":67
 *     SIZE_t n_constant_features
 * 
 * cdef class Stack:             # <<<<<<<<<<<<<<
 *     cdef SIZE_t capacity
 *     cdef SIZE_t top
 */
struct __pyx_obj_7sklearn_4tree_6_utils_Stack {
  PyObject_HEAD
  struct __pyx_vtabstruct_7sklearn_4tree_6_utils_Stack *__pyx_vtab;
  __pyx_t_7sklearn_4tree_6_utils_SIZE_t capacity;
  __pyx_t_7sklearn_4tree_6_utils_SIZE_t top;
  struct __pyx_t_7sklearn_4tree_6_utils_StackRecord *stack_;
};


/* "sklearn/tree/_utils.pxd":96
 *     double improvement
 * 
 * cdef class PriorityHeap:             # <<<<<<<<<<<<<<
 *     cdef SIZE_t capacity
 *     cdef SIZE_t heap_ptr
 */
struct __pyx_obj_7sklearn_4tree_6_utils_PriorityHeap {
  PyObject_HEAD
  struct __pyx_vtabstruct_7sklearn_4tree_6_utils_PriorityHeap *__pyx_vtab;
  __pyx_t_7sklearn_4tree_6_utils_SIZE_t capacity;
  __pyx_t_7sklearn_4tree_6_utils_SIZE_t heap_ptr;
  struct __pyx_t_7sklearn_4tree_6_utils_PriorityHeapRecord *heap_;
};


/* "sklearn/tree/_splitter.pxd":34
 *     double impurity_right  # Impurity of the right split.
 * 
 * cdef class Splitter:             # <<<<<<<<<<<<<<
 *     # The splitter searches in the input space for a feature and a threshold
 *     # to split the samples samples[start:end].
 */
struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter {
  PyObject_HEAD
  struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_Splitter *__pyx_vtab;
  struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *criterion;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t max_features;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t min_samples_leaf;
  double min_weight_leaf;
  PyObject *random_state;
  __pyx_t_7sklearn_4tree_9_splitter_UINT32_t rand_r_state;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t n_samples;
  double weighted_n_samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *features;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *constant_features;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t n_features;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *feature_values;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t start;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t end;
  int presort;
  __pyx_t_7sklearn_4tree_9_splitter_DOUBLE_t *y;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t y_stride;
  __pyx_t_7sklearn_4tree_9_splitter_DOUBLE_t *sample_weight;
};


/* "sklearn/tree/_splitter.pyx":232
 * 
 * 
 * cdef class BaseDenseSplitter(Splitter):             # <<<<<<<<<<<<<<
 *     cdef DTYPE_t* X
 *     cdef SIZE_t X_sample_stride
 */
struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter {
  struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter __pyx_base;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *X;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t X_sample_stride;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t X_feature_stride;
  PyArrayObject *X_idx_sorted;
  __pyx_t_7sklearn_4tree_9_splitter_INT32_t *X_idx_sorted_ptr;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t X_idx_sorted_stride;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t n_total_samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *sample_mask;
};


/* "sklearn/tree/_splitter.pyx":288
 * 
 * 
 * cdef class BestSplitter(BaseDenseSplitter):             # <<<<<<<<<<<<<<
 *     """Splitter for finding the best split."""
 *     def __reduce__(self):
 */
struct __pyx_obj_7sklearn_4tree_9_splitter_BestSplitter {
  struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter __pyx_base;
};


/* "sklearn/tree/_splitter.pyx":624
 * 
 * 
 * cdef class RandomSplitter(BaseDenseSplitter):             # <<<<<<<<<<<<<<
 *     """Splitter for finding the best random split."""
 *     def __reduce__(self):
 */
struct __pyx_obj_7sklearn_4tree_9_splitter_RandomSplitter {
  struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter __pyx_base;
};


/* "sklearn/tree/_splitter.pyx":838
 * 
 * 
 * cdef class BaseSparseSplitter(Splitter):             # <<<<<<<<<<<<<<
 *     # The sparse splitter works only with csc sparse matrix format
 *     cdef DTYPE_t* X_data
 */
struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter {
  struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter __pyx_base;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *X_data;
  __pyx_t_7sklearn_4tree_9_splitter_INT32_t *X_indices;
  __pyx_t_7sklearn_4tree_9_splitter_INT32_t *X_indptr;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t n_total_samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *index_to_samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *sorted_samples;
};


/* "sklearn/tree/_splitter.pyx":1158
 * 
 * 
 * cdef class BestSparseSplitter(BaseSparseSplitter):             # <<<<<<<<<<<<<<
 *     """Splitter for finding the best split, using the sparse data."""
 * 
 */
struct __pyx_obj_7sklearn_4tree_9_splitter_BestSparseSplitter {
  struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter __pyx_base;
};


/* "sklearn/tree/_splitter.pyx":1385
 * 
 * 
 * cdef class RandomSparseSplitter(BaseSparseSplitter):             # <<<<<<<<<<<<<<
 *     """Splitter for finding a random split, using the sparse data."""
 * 
 */
struct __pyx_obj_7sklearn_4tree_9_splitter_RandomSparseSplitter {
  struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter __pyx_base;
};



/* "sklearn/tree/_criterion.pxd":21
 * ctypedef np.npy_uint32 UINT32_t          # Unsigned 32 bit integer
 * 
 * cdef class Criterion:             # <<<<<<<<<<<<<<
 *     # The criterion computes the impurity of a node and the reduction of
 *     # impurity of a split on that node. It also computes the output statistics
 */

struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion {
  void (*init)(struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *, __pyx_t_7sklearn_4tree_10_criterion_DOUBLE_t *, __pyx_t_7sklearn_4tree_10_criterion_SIZE_t, __pyx_t_7sklearn_4tree_10_criterion_DOUBLE_t *, double, __pyx_t_7sklearn_4tree_10_criterion_SIZE_t *, __pyx_t_7sklearn_4tree_10_criterion_SIZE_t, __pyx_t_7sklearn_4tree_10_criterion_SIZE_t);
  void (*reset)(struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *);
  void (*reverse_reset)(struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *);
  void (*update)(struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *, __pyx_t_7sklearn_4tree_10_criterion_SIZE_t);
  double (*node_impurity)(struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *);
  void (*children_impurity)(struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *, double *, double *);
  void (*node_value)(struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *, double *);
  double (*impurity_improvement)(struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *, double);
  double (*proxy_impurity_improvement)(struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *);
};
static struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *__pyx_vtabptr_7sklearn_4tree_10_criterion_Criterion;


/* "sklearn/tree/_utils.pxd":67
 *     SIZE_t n_constant_features
 * 
 * cdef class Stack:             # <<<<<<<<<<<<<<
 *     cdef SIZE_t capacity
 *     cdef SIZE_t top
 */

struct __pyx_vtabstruct_7sklearn_4tree_6_utils_Stack {
  int (*is_empty)(struct __pyx_obj_7sklearn_4tree_6_utils_Stack *);
  int (*push)(struct __pyx_obj_7sklearn_4tree_6_utils_Stack *, __pyx_t_7sklearn_4tree_6_utils_SIZE_t, __pyx_t_7sklearn_4tree_6_utils_SIZE_t, __pyx_t_7sklearn_4tree_6_utils_SIZE_t, __pyx_t_7sklearn_4tree_6_utils_SIZE_t, int, double, __pyx_t_7sklearn_4tree_6_utils_SIZE_t);
  int (*pop)(struct __pyx_obj_7sklearn_4tree_6_utils_Stack *, struct __pyx_t_7sklearn_4tree_6_utils_StackRecord *);
};
static struct __pyx_vtabstruct_7sklearn_4tree_6_utils_Stack *__pyx_vtabptr_7sklearn_4tree_6_utils_Stack;


/* "sklearn/tree/_utils.pxd":96
 *     double improvement
 * 
 * cdef class PriorityHeap:             # <<<<<<<<<<<<<<
 *     cdef SIZE_t capacity
 *     cdef SIZE_t heap_ptr
 */

struct __pyx_vtabstruct_7sklearn_4tree_6_utils_PriorityHeap {
  int (*is_empty)(struct __pyx_obj_7sklearn_4tree_6_utils_PriorityHeap *);
  int (*push)(struct __pyx_obj_7sklearn_4tree_6_utils_PriorityHeap *, __pyx_t_7sklearn_4tree_6_utils_SIZE_t, __pyx_t_7sklearn_4tree_6_utils_SIZE_t, __pyx_t_7sklearn_4tree_6_utils_SIZE_t, __pyx_t_7sklearn_4tree_6_utils_SIZE_t, __pyx_t_7sklearn_4tree_6_utils_SIZE_t, int, double, double, double, double);
  int (*pop)(struct __pyx_obj_7sklearn_4tree_6_utils_PriorityHeap *, struct __pyx_t_7sklearn_4tree_6_utils_PriorityHeapRecord *);
};
static struct __pyx_vtabstruct_7sklearn_4tree_6_utils_PriorityHeap *__pyx_vtabptr_7sklearn_4tree_6_utils_PriorityHeap;


/* "sklearn/tree/_splitter.pyx":54
 *     self.improvement = -INFINITY
 * 
 * cdef class Splitter:             # <<<<<<<<<<<<<<
 *     """Abstract splitter class.
 * 
 */

struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_Splitter {
  void (*init)(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *, PyObject *, PyArrayObject *, __pyx_t_7sklearn_4tree_9_splitter_DOUBLE_t *, struct __pyx_opt_args_7sklearn_4tree_9_splitter_8Splitter_init *__pyx_optional_args);
  void (*node_reset)(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, double *);
  void (*node_split)(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *, double, struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *);
  void (*node_value)(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *, double *);
  double (*node_impurity)(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *);
};
static struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_Splitter *__pyx_vtabptr_7sklearn_4tree_9_splitter_Splitter;


/* "sklearn/tree/_splitter.pyx":232
 * 
 * 
 * cdef class BaseDenseSplitter(Splitter):             # <<<<<<<<<<<<<<
 *     cdef DTYPE_t* X
 *     cdef SIZE_t X_sample_stride
 */

struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_BaseDenseSplitter {
  struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_Splitter __pyx_base;
};
static struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_BaseDenseSplitter *__pyx_vtabptr_7sklearn_4tree_9_splitter_BaseDenseSplitter;


/* "sklearn/tree/_splitter.pyx":288
 * 
 * 
 * cdef class BestSplitter(BaseDenseSplitter):             # <<<<<<<<<<<<<<
 *     """Splitter for finding the best split."""
 *     def __reduce__(self):
 */

struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_BestSplitter {
  struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_BaseDenseSplitter __pyx_base;
};
static struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_BestSplitter *__pyx_vtabptr_7sklearn_4tree_9_splitter_BestSplitter;


/* "sklearn/tree/_splitter.pyx":624
 * 
 * 
 * cdef class RandomSplitter(BaseDenseSplitter):             # <<<<<<<<<<<<<<
 *     """Splitter for finding the best random split."""
 *     def __reduce__(self):
 */

struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_RandomSplitter {
  struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_BaseDenseSplitter __pyx_base;
};
static struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_RandomSplitter *__pyx_vtabptr_7sklearn_4tree_9_splitter_RandomSplitter;


/* "sklearn/tree/_splitter.pyx":838
 * 
 * 
 * cdef class BaseSparseSplitter(Splitter):             # <<<<<<<<<<<<<<
 *     # The sparse splitter works only with csc sparse matrix format
 *     cdef DTYPE_t* X_data
 */

struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_BaseSparseSplitter {
  struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_Splitter __pyx_base;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t (*_partition)(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *, double, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t);
  void (*extract_nnz)(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, int *);
};
static struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_BaseSparseSplitter *__pyx_vtabptr_7sklearn_4tree_9_splitter_BaseSparseSplitter;
static CYTHON_INLINE __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_f_7sklearn_4tree_9_splitter_18BaseSparseSplitter__partition(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *, double, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t);
static CYTHON_INLINE void __pyx_f_7sklearn_4tree_9_splitter_18BaseSparseSplitter_extract_nnz(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, int *);


/* "sklearn/tree/_splitter.pyx":1158
 * 
 * 
 * cdef class BestSparseSplitter(BaseSparseSplitter):             # <<<<<<<<<<<<<<
 *     """Splitter for finding the best split, using the sparse data."""
 * 
 */

struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_BestSparseSplitter {
  struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_BaseSparseSplitter __pyx_base;
};
static struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_BestSparseSplitter *__pyx_vtabptr_7sklearn_4tree_9_splitter_BestSparseSplitter;


/* "sklearn/tree/_splitter.pyx":1385
 * 
 * 
 * cdef class RandomSparseSplitter(BaseSparseSplitter):             # <<<<<<<<<<<<<<
 *     """Splitter for finding a random split, using the sparse data."""
 * 
 */

struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_RandomSparseSplitter {
  struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_BaseSparseSplitter __pyx_base;
};
static struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_RandomSparseSplitter *__pyx_vtabptr_7sklearn_4tree_9_splitter_RandomSparseSplitter;

/* --- Runtime support code (head) --- */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil) \
          if (acquire_gil) { \
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure(); \
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__); \
              PyGILState_Release(__pyx_gilstate_save); \
          } else { \
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__); \
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil) \
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext() \
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {                            \
        PyObject *tmp = (PyObject *) r;                         \
        r = v; __Pyx_XDECREF(tmp);                              \
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {                             \
        PyObject *tmp = (PyObject *) r;                         \
        r = v; __Pyx_DECREF(tmp);                               \
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name);

static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[], \
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args, \
    const char* function_name);

static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact);

static CYTHON_INLINE int  __Pyx_GetBufferAndValidate(Py_buffer* buf, PyObject* obj,
    __Pyx_TypeInfo* dtype, int flags, int nd, int cast, __Pyx_BufFmt_StackElem* stack);
static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info);

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck) \
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ? \
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) : \
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) : \
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck) \
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ? \
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) : \
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck) \
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ? \
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) : \
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb);

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args))
                PyErr_SetObject(PyExc_KeyError, args);
            Py_XDECREF(args);
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#else
    #define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#endif

static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

static int __Pyx_SetVtable(PyObject *dict, void *vtable);

static void* __Pyx_GetVtable(PyObject *dict);

static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

typedef struct {
    int code_line;
    PyCodeObject* code_object;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

typedef struct {
  Py_ssize_t shape, strides, suboffsets;
} __Pyx_Buf_DimInfo;
typedef struct {
  size_t refcount;
  Py_buffer pybuffer;
} __Pyx_Buffer;
typedef struct {
  __Pyx_Buffer *rcbuffer;
  char *data;
  __Pyx_Buf_DimInfo diminfo[8];
} __Pyx_LocalBuf_ND;

#if PY_MAJOR_VERSION < 3
    static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags);
    static void __Pyx_ReleaseBuffer(Py_buffer *view);
#else
    #define __Pyx_GetBuffer PyObject_GetBuffer
    #define __Pyx_ReleaseBuffer PyBuffer_Release
#endif


static Py_ssize_t __Pyx_zeros[] = {0, 0, 0, 0, 0, 0, 0, 0};
static Py_ssize_t __Pyx_minusones[] = {-1, -1, -1, -1, -1, -1, -1, -1};

static CYTHON_INLINE Py_intptr_t __Pyx_PyInt_As_Py_intptr_t(PyObject *);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

static CYTHON_INLINE npy_uint32 __Pyx_PyInt_As_npy_uint32(PyObject *);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Py_intptr_t(Py_intptr_t value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_npy_int32(npy_int32 value);

static CYTHON_INLINE npy_int32 __Pyx_PyInt_As_npy_int32(PyObject *);

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if (defined(_WIN32) || defined(__clang__)) && defined(__cplusplus) && CYTHON_CCOMPLEX
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

#if CYTHON_CCOMPLEX
    #define __Pyx_c_eqf(a, b)   ((a)==(b))
    #define __Pyx_c_sumf(a, b)  ((a)+(b))
    #define __Pyx_c_difff(a, b) ((a)-(b))
    #define __Pyx_c_prodf(a, b) ((a)*(b))
    #define __Pyx_c_quotf(a, b) ((a)/(b))
    #define __Pyx_c_negf(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zerof(z) ((z)==(float)0)
    #define __Pyx_c_conjf(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_absf(z)     (::std::abs(z))
        #define __Pyx_c_powf(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zerof(z) ((z)==0)
    #define __Pyx_c_conjf(z)    (conjf(z))
    #if 1
        #define __Pyx_c_absf(z)     (cabsf(z))
        #define __Pyx_c_powf(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eqf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sumf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_difff(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prodf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quotf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_negf(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zerof(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conjf(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_absf(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_powf(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);

#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq(a, b)   ((a)==(b))
    #define __Pyx_c_sum(a, b)  ((a)+(b))
    #define __Pyx_c_diff(a, b) ((a)-(b))
    #define __Pyx_c_prod(a, b) ((a)*(b))
    #define __Pyx_c_quot(a, b) ((a)/(b))
    #define __Pyx_c_neg(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero(z) ((z)==(double)0)
    #define __Pyx_c_conj(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs(z)     (::std::abs(z))
        #define __Pyx_c_pow(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero(z) ((z)==0)
    #define __Pyx_c_conj(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs(z)     (cabs(z))
        #define __Pyx_c_pow(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

static int __Pyx_check_binary_version(void);

#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

static PyObject *__Pyx_ImportModule(const char *name);

static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static void __pyx_f_7sklearn_4tree_9_splitter_8Splitter_init(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self, PyObject *__pyx_v_X, PyArrayObject *__pyx_v_y, __pyx_t_7sklearn_4tree_9_splitter_DOUBLE_t *__pyx_v_sample_weight, struct __pyx_opt_args_7sklearn_4tree_9_splitter_8Splitter_init *__pyx_optional_args); /* proto*/
static void __pyx_f_7sklearn_4tree_9_splitter_8Splitter_node_reset(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_start, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_end, double *__pyx_v_weighted_n_node_samples); /* proto*/
static void __pyx_f_7sklearn_4tree_9_splitter_8Splitter_node_split(CYTHON_UNUSED struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self, CYTHON_UNUSED double __pyx_v_impurity, CYTHON_UNUSED struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord *__pyx_v_split, CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_n_constant_features); /* proto*/
static void __pyx_f_7sklearn_4tree_9_splitter_8Splitter_node_value(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self, double *__pyx_v_dest); /* proto*/
static double __pyx_f_7sklearn_4tree_9_splitter_8Splitter_node_impurity(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self); /* proto*/
static void __pyx_f_7sklearn_4tree_9_splitter_17BaseDenseSplitter_init(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter *__pyx_v_self, PyObject *__pyx_v_X, PyArrayObject *__pyx_v_y, __pyx_t_7sklearn_4tree_9_splitter_DOUBLE_t *__pyx_v_sample_weight, struct __pyx_opt_args_7sklearn_4tree_9_splitter_17BaseDenseSplitter_init *__pyx_optional_args); /* proto*/
static void __pyx_f_7sklearn_4tree_9_splitter_12BestSplitter_node_split(struct __pyx_obj_7sklearn_4tree_9_splitter_BestSplitter *__pyx_v_self, double __pyx_v_impurity, struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord *__pyx_v_split, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_n_constant_features); /* proto*/
static void __pyx_f_7sklearn_4tree_9_splitter_14RandomSplitter_node_split(struct __pyx_obj_7sklearn_4tree_9_splitter_RandomSplitter *__pyx_v_self, double __pyx_v_impurity, struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord *__pyx_v_split, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_n_constant_features); /* proto*/
static void __pyx_f_7sklearn_4tree_9_splitter_18BaseSparseSplitter_init(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *__pyx_v_self, PyObject *__pyx_v_X, PyArrayObject *__pyx_v_y, __pyx_t_7sklearn_4tree_9_splitter_DOUBLE_t *__pyx_v_sample_weight, struct __pyx_opt_args_7sklearn_4tree_9_splitter_18BaseSparseSplitter_init *__pyx_optional_args); /* proto*/
static CYTHON_INLINE __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_f_7sklearn_4tree_9_splitter_18BaseSparseSplitter__partition(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *__pyx_v_self, double __pyx_v_threshold, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_end_negative, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_start_positive, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_zero_pos); /* proto*/
static CYTHON_INLINE void __pyx_f_7sklearn_4tree_9_splitter_18BaseSparseSplitter_extract_nnz(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *__pyx_v_self, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_feature, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_end_negative, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_start_positive, int *__pyx_v_is_samples_sorted); /* proto*/
static void __pyx_f_7sklearn_4tree_9_splitter_18BestSparseSplitter_node_split(struct __pyx_obj_7sklearn_4tree_9_splitter_BestSparseSplitter *__pyx_v_self, double __pyx_v_impurity, struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord *__pyx_v_split, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_n_constant_features); /* proto*/
static void __pyx_f_7sklearn_4tree_9_splitter_20RandomSparseSplitter_node_split(struct __pyx_obj_7sklearn_4tree_9_splitter_RandomSparseSplitter *__pyx_v_self, double __pyx_v_impurity, struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord *__pyx_v_split, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_n_constant_features); /* proto*/

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'cpython.object' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;
static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *, char *, char *, int *); /*proto*/

/* Module declarations from 'sklearn.tree._criterion' */
static PyTypeObject *__pyx_ptype_7sklearn_4tree_10_criterion_Criterion = 0;

/* Module declarations from 'sklearn.tree._utils' */
static PyTypeObject *__pyx_ptype_7sklearn_4tree_6_utils_Stack = 0;
static PyTypeObject *__pyx_ptype_7sklearn_4tree_6_utils_PriorityHeap = 0;
static __pyx_t_7sklearn_4tree_6_utils_SIZE_t (*__pyx_f_7sklearn_4tree_6_utils_rand_int)(__pyx_t_7sklearn_4tree_6_utils_SIZE_t, __pyx_t_7sklearn_4tree_6_utils_SIZE_t, __pyx_t_7sklearn_4tree_6_utils_UINT32_t *); /*proto*/
static double (*__pyx_f_7sklearn_4tree_6_utils_rand_uniform)(double, double, __pyx_t_7sklearn_4tree_6_utils_UINT32_t *); /*proto*/
static double (*__pyx_f_7sklearn_4tree_6_utils_log)(double); /*proto*/
static __pyx_t_7sklearn_4tree_6_utils_DTYPE_t *(*__pyx_fuse_0__pyx_f_7sklearn_4tree_6_utils_safe_realloc)(__pyx_t_7sklearn_4tree_6_utils_DTYPE_t **, size_t); /*proto*/
static __pyx_t_7sklearn_4tree_6_utils_SIZE_t *(*__pyx_fuse_1__pyx_f_7sklearn_4tree_6_utils_safe_realloc)(__pyx_t_7sklearn_4tree_6_utils_SIZE_t **, size_t); /*proto*/
static unsigned char *(*__pyx_fuse_2__pyx_f_7sklearn_4tree_6_utils_safe_realloc)(unsigned char **, size_t); /*proto*/

/* Module declarations from 'sklearn.tree._splitter' */
static PyTypeObject *__pyx_ptype_7sklearn_4tree_9_splitter_Splitter = 0;
static PyTypeObject *__pyx_ptype_7sklearn_4tree_9_splitter_BaseDenseSplitter = 0;
static PyTypeObject *__pyx_ptype_7sklearn_4tree_9_splitter_BestSplitter = 0;
static PyTypeObject *__pyx_ptype_7sklearn_4tree_9_splitter_RandomSplitter = 0;
static PyTypeObject *__pyx_ptype_7sklearn_4tree_9_splitter_BaseSparseSplitter = 0;
static PyTypeObject *__pyx_ptype_7sklearn_4tree_9_splitter_BestSparseSplitter = 0;
static PyTypeObject *__pyx_ptype_7sklearn_4tree_9_splitter_RandomSparseSplitter = 0;
static double __pyx_v_7sklearn_4tree_9_splitter_INFINITY;
static __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t __pyx_v_7sklearn_4tree_9_splitter_FEATURE_THRESHOLD;
static __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t __pyx_v_7sklearn_4tree_9_splitter_EXTRACT_NNZ_SWITCH;
static CYTHON_INLINE void __pyx_f_7sklearn_4tree_9_splitter__init_split(struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t); /*proto*/
static CYTHON_INLINE void __pyx_f_7sklearn_4tree_9_splitter_sort(__pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t); /*proto*/
static CYTHON_INLINE void __pyx_f_7sklearn_4tree_9_splitter_swap(__pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t); /*proto*/
static CYTHON_INLINE __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t __pyx_f_7sklearn_4tree_9_splitter_median3(__pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t); /*proto*/
static void __pyx_f_7sklearn_4tree_9_splitter_introsort(__pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, int); /*proto*/
static CYTHON_INLINE void __pyx_f_7sklearn_4tree_9_splitter_sift_down(__pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t); /*proto*/
static void __pyx_f_7sklearn_4tree_9_splitter_heapsort(__pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t); /*proto*/
static int __pyx_f_7sklearn_4tree_9_splitter_compare_SIZE_t(void const *, void const *); /*proto*/
static CYTHON_INLINE void __pyx_f_7sklearn_4tree_9_splitter_binary_search(__pyx_t_7sklearn_4tree_9_splitter_INT32_t *, __pyx_t_7sklearn_4tree_9_splitter_INT32_t, __pyx_t_7sklearn_4tree_9_splitter_INT32_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, __pyx_t_7sklearn_4tree_9_splitter_INT32_t *); /*proto*/
static CYTHON_INLINE void __pyx_f_7sklearn_4tree_9_splitter_extract_nnz_index_to_samples(__pyx_t_7sklearn_4tree_9_splitter_INT32_t *, __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *, __pyx_t_7sklearn_4tree_9_splitter_INT32_t, __pyx_t_7sklearn_4tree_9_splitter_INT32_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *); /*proto*/
static CYTHON_INLINE void __pyx_f_7sklearn_4tree_9_splitter_extract_nnz_binary_search(__pyx_t_7sklearn_4tree_9_splitter_INT32_t *, __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *, __pyx_t_7sklearn_4tree_9_splitter_INT32_t, __pyx_t_7sklearn_4tree_9_splitter_INT32_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, int *); /*proto*/
static CYTHON_INLINE void __pyx_f_7sklearn_4tree_9_splitter_sparse_swap(__pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t); /*proto*/
static __Pyx_TypeInfo __Pyx_TypeInfo_nn___pyx_t_7sklearn_4tree_9_splitter_DOUBLE_t = { "DOUBLE_t", NULL, sizeof(__pyx_t_7sklearn_4tree_9_splitter_DOUBLE_t), { 0 }, 0, 'R', 0, 0 };
static __Pyx_TypeInfo __Pyx_TypeInfo_nn___pyx_t_7sklearn_4tree_9_splitter_DTYPE_t = { "DTYPE_t", NULL, sizeof(__pyx_t_7sklearn_4tree_9_splitter_DTYPE_t), { 0 }, 0, 'R', 0, 0 };
static __Pyx_TypeInfo __Pyx_TypeInfo_nn___pyx_t_7sklearn_4tree_9_splitter_INT32_t = { "INT32_t", NULL, sizeof(__pyx_t_7sklearn_4tree_9_splitter_INT32_t), { 0 }, 0, IS_UNSIGNED(__pyx_t_7sklearn_4tree_9_splitter_INT32_t) ? 'U' : 'I', IS_UNSIGNED(__pyx_t_7sklearn_4tree_9_splitter_INT32_t), 0 };
#define __Pyx_MODULE_NAME "sklearn.tree._splitter"
int __pyx_module_is_main_sklearn__tree___splitter = 0;

/* Implementation of 'sklearn.tree._splitter' */
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_RuntimeError;
static int __pyx_pf_7sklearn_4tree_9_splitter_8Splitter___cinit__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self, struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *__pyx_v_criterion, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_max_features, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_min_samples_leaf, double __pyx_v_min_weight_leaf, PyObject *__pyx_v_random_state, int __pyx_v_presort); /* proto */
static void __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_2__dealloc__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7sklearn_4tree_9_splitter_8Splitter_4__getstate__(CYTHON_UNUSED struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7sklearn_4tree_9_splitter_8Splitter_6__setstate__(CYTHON_UNUSED struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_d); /* proto */
static PyObject *__pyx_pf_7sklearn_4tree_9_splitter_8Splitter_9criterion___get__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self); /* proto */
static int __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_9criterion_2__set__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_9criterion_4__del__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7sklearn_4tree_9_splitter_8Splitter_12max_features___get__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self); /* proto */
static int __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_12max_features_2__set__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_7sklearn_4tree_9_splitter_8Splitter_16min_samples_leaf___get__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self); /* proto */
static int __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_16min_samples_leaf_2__set__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_7sklearn_4tree_9_splitter_8Splitter_15min_weight_leaf___get__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self); /* proto */
static int __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_15min_weight_leaf_2__set__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_7sklearn_4tree_9_splitter_17BaseDenseSplitter___cinit__(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *__pyx_v_criterion, CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_max_features, CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_min_samples_leaf, CYTHON_UNUSED double __pyx_v_min_weight_leaf, CYTHON_UNUSED PyObject *__pyx_v_random_state, int __pyx_v_presort); /* proto */
static void __pyx_pf_7sklearn_4tree_9_splitter_17BaseDenseSplitter_2__dealloc__(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7sklearn_4tree_9_splitter_12BestSplitter___reduce__(struct __pyx_obj_7sklearn_4tree_9_splitter_BestSplitter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7sklearn_4tree_9_splitter_14RandomSplitter___reduce__(struct __pyx_obj_7sklearn_4tree_9_splitter_RandomSplitter *__pyx_v_self); /* proto */
static int __pyx_pf_7sklearn_4tree_9_splitter_18BaseSparseSplitter___cinit__(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *__pyx_v_criterion, CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_max_features, CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_min_samples_leaf, CYTHON_UNUSED double __pyx_v_min_weight_leaf, CYTHON_UNUSED PyObject *__pyx_v_random_state, CYTHON_UNUSED int __pyx_v_presort); /* proto */
static void __pyx_pf_7sklearn_4tree_9_splitter_18BaseSparseSplitter_2__dealloc__(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7sklearn_4tree_9_splitter_18BestSparseSplitter___reduce__(struct __pyx_obj_7sklearn_4tree_9_splitter_BestSparseSplitter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7sklearn_4tree_9_splitter_20RandomSparseSplitter___reduce__(struct __pyx_obj_7sklearn_4tree_9_splitter_RandomSparseSplitter *__pyx_v_self); /* proto */
static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info); /* proto */
static PyObject *__pyx_tp_new_7sklearn_4tree_9_splitter_Splitter(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7sklearn_4tree_9_splitter_BaseDenseSplitter(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7sklearn_4tree_9_splitter_BestSplitter(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7sklearn_4tree_9_splitter_RandomSplitter(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7sklearn_4tree_9_splitter_BaseSparseSplitter(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7sklearn_4tree_9_splitter_BestSparseSplitter(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7sklearn_4tree_9_splitter_RandomSparseSplitter(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static char __pyx_k_B[] = "B";
static char __pyx_k_H[] = "H";
static char __pyx_k_I[] = "I";
static char __pyx_k_L[] = "L";
static char __pyx_k_O[] = "O";
static char __pyx_k_Q[] = "Q";
static char __pyx_k_b[] = "b";
static char __pyx_k_d[] = "d";
static char __pyx_k_f[] = "f";
static char __pyx_k_g[] = "g";
static char __pyx_k_h[] = "h";
static char __pyx_k_i[] = "i";
static char __pyx_k_l[] = "l";
static char __pyx_k_q[] = "q";
static char __pyx_k_Zd[] = "Zd";
static char __pyx_k_Zf[] = "Zf";
static char __pyx_k_Zg[] = "Zg";
static char __pyx_k_np[] = "np";
static char __pyx_k_inf[] = "inf";
static char __pyx_k_data[] = "data";
static char __pyx_k_main[] = "__main__";
static char __pyx_k_test[] = "__test__";
static char __pyx_k_numpy[] = "numpy";
static char __pyx_k_range[] = "range";
static char __pyx_k_shape[] = "shape";
static char __pyx_k_import[] = "__import__";
static char __pyx_k_indptr[] = "indptr";
static char __pyx_k_indices[] = "indices";
static char __pyx_k_presort[] = "presort";
static char __pyx_k_randint[] = "randint";
static char __pyx_k_strides[] = "strides";
static char __pyx_k_getstate[] = "__getstate__";
static char __pyx_k_itemsize[] = "itemsize";
static char __pyx_k_criterion[] = "criterion";
static char __pyx_k_ValueError[] = "ValueError";
static char __pyx_k_csc_matrix[] = "csc_matrix";
static char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static char __pyx_k_RuntimeError[] = "RuntimeError";
static char __pyx_k_max_features[] = "max_features";
static char __pyx_k_random_state[] = "random_state";
static char __pyx_k_scipy_sparse[] = "scipy.sparse";
static char __pyx_k_min_weight_leaf[] = "min_weight_leaf";
static char __pyx_k_min_samples_leaf[] = "min_samples_leaf";
static char __pyx_k_X_should_be_in_csc_format[] = "X should be in csc format";
static char __pyx_k_ndarray_is_not_C_contiguous[] = "ndarray is not C contiguous";
static char __pyx_k_unknown_dtype_code_in_numpy_pxd[] = "unknown dtype code in numpy.pxd (%d)";
static char __pyx_k_Format_string_allocated_too_shor[] = "Format string allocated too short, see comment in numpy.pxd";
static char __pyx_k_Non_native_byte_order_not_suppor[] = "Non-native byte order not supported";
static char __pyx_k_ndarray_is_not_Fortran_contiguou[] = "ndarray is not Fortran contiguous";
static char __pyx_k_Format_string_allocated_too_shor_2[] = "Format string allocated too short.";
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor;
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor_2;
static PyObject *__pyx_kp_u_Non_native_byte_order_not_suppor;
static PyObject *__pyx_n_s_RuntimeError;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_kp_s_X_should_be_in_csc_format;
static PyObject *__pyx_n_s_criterion;
static PyObject *__pyx_n_s_csc_matrix;
static PyObject *__pyx_n_s_data;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_indices;
static PyObject *__pyx_n_s_indptr;
static PyObject *__pyx_n_s_inf;
static PyObject *__pyx_n_s_itemsize;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_max_features;
static PyObject *__pyx_n_s_min_samples_leaf;
static PyObject *__pyx_n_s_min_weight_leaf;
static PyObject *__pyx_kp_u_ndarray_is_not_C_contiguous;
static PyObject *__pyx_kp_u_ndarray_is_not_Fortran_contiguou;
static PyObject *__pyx_n_s_np;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_n_s_presort;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_randint;
static PyObject *__pyx_n_s_random_state;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_scipy_sparse;
static PyObject *__pyx_n_s_shape;
static PyObject *__pyx_n_s_strides;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_kp_u_unknown_dtype_code_in_numpy_pxd;
static PyObject *__pyx_int_0;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;

/* "sklearn/tree/_splitter.pyx":46
 * cdef DTYPE_t EXTRACT_NNZ_SWITCH = 0.1
 * 
 * cdef inline void _init_split(SplitRecord* self, SIZE_t start_pos) nogil:             # <<<<<<<<<<<<<<
 *     self.impurity_left = INFINITY
 *     self.impurity_right = INFINITY
 */

static CYTHON_INLINE void __pyx_f_7sklearn_4tree_9_splitter__init_split(struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord *__pyx_v_self, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_start_pos) {

  /* "sklearn/tree/_splitter.pyx":47
 * 
 * cdef inline void _init_split(SplitRecord* self, SIZE_t start_pos) nogil:
 *     self.impurity_left = INFINITY             # <<<<<<<<<<<<<<
 *     self.impurity_right = INFINITY
 *     self.pos = start_pos
 */
  __pyx_v_self->impurity_left = __pyx_v_7sklearn_4tree_9_splitter_INFINITY;

  /* "sklearn/tree/_splitter.pyx":48
 * cdef inline void _init_split(SplitRecord* self, SIZE_t start_pos) nogil:
 *     self.impurity_left = INFINITY
 *     self.impurity_right = INFINITY             # <<<<<<<<<<<<<<
 *     self.pos = start_pos
 *     self.feature = 0
 */
  __pyx_v_self->impurity_right = __pyx_v_7sklearn_4tree_9_splitter_INFINITY;

  /* "sklearn/tree/_splitter.pyx":49
 *     self.impurity_left = INFINITY
 *     self.impurity_right = INFINITY
 *     self.pos = start_pos             # <<<<<<<<<<<<<<
 *     self.feature = 0
 *     self.threshold = 0.
 */
  __pyx_v_self->pos = __pyx_v_start_pos;

  /* "sklearn/tree/_splitter.pyx":50
 *     self.impurity_right = INFINITY
 *     self.pos = start_pos
 *     self.feature = 0             # <<<<<<<<<<<<<<
 *     self.threshold = 0.
 *     self.improvement = -INFINITY
 */
  __pyx_v_self->feature = 0;

  /* "sklearn/tree/_splitter.pyx":51
 *     self.pos = start_pos
 *     self.feature = 0
 *     self.threshold = 0.             # <<<<<<<<<<<<<<
 *     self.improvement = -INFINITY
 * 
 */
  __pyx_v_self->threshold = 0.;

  /* "sklearn/tree/_splitter.pyx":52
 *     self.feature = 0
 *     self.threshold = 0.
 *     self.improvement = -INFINITY             # <<<<<<<<<<<<<<
 * 
 * cdef class Splitter:
 */
  __pyx_v_self->improvement = (-__pyx_v_7sklearn_4tree_9_splitter_INFINITY);

  /* "sklearn/tree/_splitter.pyx":46
 * cdef DTYPE_t EXTRACT_NNZ_SWITCH = 0.1
 * 
 * cdef inline void _init_split(SplitRecord* self, SIZE_t start_pos) nogil:             # <<<<<<<<<<<<<<
 *     self.impurity_left = INFINITY
 *     self.impurity_right = INFINITY
 */

  /* function exit code */
}

/* "sklearn/tree/_splitter.pyx":61
 *     """
 * 
 *     def __cinit__(self, Criterion criterion, SIZE_t max_features,             # <<<<<<<<<<<<<<
 *                   SIZE_t min_samples_leaf, double min_weight_leaf,
 *                   object random_state, bint presort):
 */

/* Python wrapper */
static int __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *__pyx_v_criterion = 0;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_max_features;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_min_samples_leaf;
  double __pyx_v_min_weight_leaf;
  PyObject *__pyx_v_random_state = 0;
  int __pyx_v_presort;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_criterion,&__pyx_n_s_max_features,&__pyx_n_s_min_samples_leaf,&__pyx_n_s_min_weight_leaf,&__pyx_n_s_random_state,&__pyx_n_s_presort,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_criterion)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_max_features)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 61; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_min_samples_leaf)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 61; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_min_weight_leaf)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 61; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_random_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 61; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_presort)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, 5); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 61; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 61; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 6) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
    }
    __pyx_v_criterion = ((struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *)values[0]);
    __pyx_v_max_features = __Pyx_PyInt_As_Py_intptr_t(values[1]); if (unlikely((__pyx_v_max_features == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 61; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_min_samples_leaf = __Pyx_PyInt_As_Py_intptr_t(values[2]); if (unlikely((__pyx_v_min_samples_leaf == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 62; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_min_weight_leaf = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_min_weight_leaf == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 62; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_random_state = values[4];
    __pyx_v_presort = __Pyx_PyObject_IsTrue(values[5]); if (unlikely((__pyx_v_presort == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 63; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 61; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sklearn.tree._splitter.Splitter.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_criterion), __pyx_ptype_7sklearn_4tree_10_criterion_Criterion, 1, "criterion", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 61; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_7sklearn_4tree_9_splitter_8Splitter___cinit__(((struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *)__pyx_v_self), __pyx_v_criterion, __pyx_v_max_features, __pyx_v_min_samples_leaf, __pyx_v_min_weight_leaf, __pyx_v_random_state, __pyx_v_presort);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7sklearn_4tree_9_splitter_8Splitter___cinit__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self, struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *__pyx_v_criterion, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_max_features, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_min_samples_leaf, double __pyx_v_min_weight_leaf, PyObject *__pyx_v_random_state, int __pyx_v_presort) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "sklearn/tree/_splitter.pyx":87
 *         """
 * 
 *         self.criterion = criterion             # <<<<<<<<<<<<<<
 * 
 *         self.samples = NULL
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_criterion));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_criterion));
  __Pyx_GOTREF(__pyx_v_self->criterion);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->criterion));
  __pyx_v_self->criterion = __pyx_v_criterion;

  /* "sklearn/tree/_splitter.pyx":89
 *         self.criterion = criterion
 * 
 *         self.samples = NULL             # <<<<<<<<<<<<<<
 *         self.n_samples = 0
 *         self.features = NULL
 */
  __pyx_v_self->samples = NULL;

  /* "sklearn/tree/_splitter.pyx":90
 * 
 *         self.samples = NULL
 *         self.n_samples = 0             # <<<<<<<<<<<<<<
 *         self.features = NULL
 *         self.n_features = 0
 */
  __pyx_v_self->n_samples = 0;

  /* "sklearn/tree/_splitter.pyx":91
 *         self.samples = NULL
 *         self.n_samples = 0
 *         self.features = NULL             # <<<<<<<<<<<<<<
 *         self.n_features = 0
 *         self.feature_values = NULL
 */
  __pyx_v_self->features = NULL;

  /* "sklearn/tree/_splitter.pyx":92
 *         self.n_samples = 0
 *         self.features = NULL
 *         self.n_features = 0             # <<<<<<<<<<<<<<
 *         self.feature_values = NULL
 * 
 */
  __pyx_v_self->n_features = 0;

  /* "sklearn/tree/_splitter.pyx":93
 *         self.features = NULL
 *         self.n_features = 0
 *         self.feature_values = NULL             # <<<<<<<<<<<<<<
 * 
 *         self.y = NULL
 */
  __pyx_v_self->feature_values = NULL;

  /* "sklearn/tree/_splitter.pyx":95
 *         self.feature_values = NULL
 * 
 *         self.y = NULL             # <<<<<<<<<<<<<<
 *         self.y_stride = 0
 *         self.sample_weight = NULL
 */
  __pyx_v_self->y = NULL;

  /* "sklearn/tree/_splitter.pyx":96
 * 
 *         self.y = NULL
 *         self.y_stride = 0             # <<<<<<<<<<<<<<
 *         self.sample_weight = NULL
 * 
 */
  __pyx_v_self->y_stride = 0;

  /* "sklearn/tree/_splitter.pyx":97
 *         self.y = NULL
 *         self.y_stride = 0
 *         self.sample_weight = NULL             # <<<<<<<<<<<<<<
 * 
 *         self.max_features = max_features
 */
  __pyx_v_self->sample_weight = NULL;

  /* "sklearn/tree/_splitter.pyx":99
 *         self.sample_weight = NULL
 * 
 *         self.max_features = max_features             # <<<<<<<<<<<<<<
 *         self.min_samples_leaf = min_samples_leaf
 *         self.min_weight_leaf = min_weight_leaf
 */
  __pyx_v_self->max_features = __pyx_v_max_features;

  /* "sklearn/tree/_splitter.pyx":100
 * 
 *         self.max_features = max_features
 *         self.min_samples_leaf = min_samples_leaf             # <<<<<<<<<<<<<<
 *         self.min_weight_leaf = min_weight_leaf
 *         self.random_state = random_state
 */
  __pyx_v_self->min_samples_leaf = __pyx_v_min_samples_leaf;

  /* "sklearn/tree/_splitter.pyx":101
 *         self.max_features = max_features
 *         self.min_samples_leaf = min_samples_leaf
 *         self.min_weight_leaf = min_weight_leaf             # <<<<<<<<<<<<<<
 *         self.random_state = random_state
 *         self.presort = presort
 */
  __pyx_v_self->min_weight_leaf = __pyx_v_min_weight_leaf;

  /* "sklearn/tree/_splitter.pyx":102
 *         self.min_samples_leaf = min_samples_leaf
 *         self.min_weight_leaf = min_weight_leaf
 *         self.random_state = random_state             # <<<<<<<<<<<<<<
 *         self.presort = presort
 * 
 */
  __Pyx_INCREF(__pyx_v_random_state);
  __Pyx_GIVEREF(__pyx_v_random_state);
  __Pyx_GOTREF(__pyx_v_self->random_state);
  __Pyx_DECREF(__pyx_v_self->random_state);
  __pyx_v_self->random_state = __pyx_v_random_state;

  /* "sklearn/tree/_splitter.pyx":103
 *         self.min_weight_leaf = min_weight_leaf
 *         self.random_state = random_state
 *         self.presort = presort             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_v_self->presort = __pyx_v_presort;

  /* "sklearn/tree/_splitter.pyx":61
 *     """
 * 
 *     def __cinit__(self, Criterion criterion, SIZE_t max_features,             # <<<<<<<<<<<<<<
 *                   SIZE_t min_samples_leaf, double min_weight_leaf,
 *                   object random_state, bint presort):
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/tree/_splitter.pyx":105
 *         self.presort = presort
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         """Destructor."""
 * 
 */

/* Python wrapper */
static void __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_2__dealloc__(((struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_2__dealloc__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "sklearn/tree/_splitter.pyx":108
 *         """Destructor."""
 * 
 *         free(self.samples)             # <<<<<<<<<<<<<<
 *         free(self.features)
 *         free(self.constant_features)
 */
  free(__pyx_v_self->samples);

  /* "sklearn/tree/_splitter.pyx":109
 * 
 *         free(self.samples)
 *         free(self.features)             # <<<<<<<<<<<<<<
 *         free(self.constant_features)
 *         free(self.feature_values)
 */
  free(__pyx_v_self->features);

  /* "sklearn/tree/_splitter.pyx":110
 *         free(self.samples)
 *         free(self.features)
 *         free(self.constant_features)             # <<<<<<<<<<<<<<
 *         free(self.feature_values)
 * 
 */
  free(__pyx_v_self->constant_features);

  /* "sklearn/tree/_splitter.pyx":111
 *         free(self.features)
 *         free(self.constant_features)
 *         free(self.feature_values)             # <<<<<<<<<<<<<<
 * 
 *     def __getstate__(self):
 */
  free(__pyx_v_self->feature_values);

  /* "sklearn/tree/_splitter.pyx":105
 *         self.presort = presort
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         """Destructor."""
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "sklearn/tree/_splitter.pyx":113
 *         free(self.feature_values)
 * 
 *     def __getstate__(self):             # <<<<<<<<<<<<<<
 *         return {}
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_4tree_9_splitter_8Splitter_5__getstate__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7sklearn_4tree_9_splitter_8Splitter_5__getstate__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getstate__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_4__getstate__(((struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_4tree_9_splitter_8Splitter_4__getstate__(CYTHON_UNUSED struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getstate__", 0);

  /* "sklearn/tree/_splitter.pyx":114
 * 
 *     def __getstate__(self):
 *         return {}             # <<<<<<<<<<<<<<
 * 
 *     def __setstate__(self, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 114; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "sklearn/tree/_splitter.pyx":113
 *         free(self.feature_values)
 * 
 *     def __getstate__(self):             # <<<<<<<<<<<<<<
 *         return {}
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sklearn.tree._splitter.Splitter.__getstate__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/tree/_splitter.pyx":116
 *         return {}
 * 
 *     def __setstate__(self, d):             # <<<<<<<<<<<<<<
 *         pass
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_4tree_9_splitter_8Splitter_7__setstate__(PyObject *__pyx_v_self, PyObject *__pyx_v_d); /*proto*/
static PyObject *__pyx_pw_7sklearn_4tree_9_splitter_8Splitter_7__setstate__(PyObject *__pyx_v_self, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_6__setstate__(((struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *)__pyx_v_self), ((PyObject *)__pyx_v_d));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_4tree_9_splitter_8Splitter_6__setstate__(CYTHON_UNUSED struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate__", 0);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/tree/_splitter.pyx":119
 *         pass
 * 
 *     cdef void init(self,             # <<<<<<<<<<<<<<
 *                    object X,
 *                    np.ndarray[DOUBLE_t, ndim=2, mode="c"] y,
 */

static void __pyx_f_7sklearn_4tree_9_splitter_8Splitter_init(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self, PyObject *__pyx_v_X, PyArrayObject *__pyx_v_y, __pyx_t_7sklearn_4tree_9_splitter_DOUBLE_t *__pyx_v_sample_weight, struct __pyx_opt_args_7sklearn_4tree_9_splitter_8Splitter_init *__pyx_optional_args) {
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_i;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_j;
  double __pyx_v_weighted_n_samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_features;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_features;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_y;
  __Pyx_Buffer __pyx_pybuffer_y;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __pyx_t_7sklearn_4tree_9_splitter_UINT32_t __pyx_t_7;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_8;
  __pyx_t_7sklearn_4tree_6_utils_SIZE_t *__pyx_t_9;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("init", 0);
  if (__pyx_optional_args) {
  }
  __pyx_pybuffer_y.pybuffer.buf = NULL;
  __pyx_pybuffer_y.refcount = 0;
  __pyx_pybuffernd_y.data = NULL;
  __pyx_pybuffernd_y.rcbuffer = &__pyx_pybuffer_y;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_y.rcbuffer->pybuffer, (PyObject*)__pyx_v_y, &__Pyx_TypeInfo_nn___pyx_t_7sklearn_4tree_9_splitter_DOUBLE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 119; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_y.diminfo[0].strides = __pyx_pybuffernd_y.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_y.diminfo[0].shape = __pyx_pybuffernd_y.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_y.diminfo[1].strides = __pyx_pybuffernd_y.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_y.diminfo[1].shape = __pyx_pybuffernd_y.rcbuffer->pybuffer.shape[1];

  /* "sklearn/tree/_splitter.pyx":142
 *         """
 * 
 *         self.rand_r_state = self.random_state.randint(0, RAND_R_MAX)             # <<<<<<<<<<<<<<
 *         cdef SIZE_t n_samples = X.shape[0]
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->random_state, __pyx_n_s_randint); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 142; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_e_7sklearn_4tree_6_utils_RAND_R_MAX); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 142; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 142; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_int_0);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 142; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyInt_As_npy_uint32(__pyx_t_1); if (unlikely((__pyx_t_7 == (npy_uint32)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 142; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->rand_r_state = __pyx_t_7;

  /* "sklearn/tree/_splitter.pyx":143
 * 
 *         self.rand_r_state = self.random_state.randint(0, RAND_R_MAX)
 *         cdef SIZE_t n_samples = X.shape[0]             # <<<<<<<<<<<<<<
 * 
 *         # Create a new array which will be used to store nonzero
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_shape); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 143; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 143; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_2); if (unlikely((__pyx_t_8 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 143; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_n_samples = __pyx_t_8;

  /* "sklearn/tree/_splitter.pyx":147
 *         # Create a new array which will be used to store nonzero
 *         # samples from the feature of interest
 *         cdef SIZE_t* samples = safe_realloc(&self.samples, n_samples)             # <<<<<<<<<<<<<<
 * 
 *         cdef SIZE_t i, j
 */
  __pyx_t_9 = __pyx_fuse_1__pyx_f_7sklearn_4tree_6_utils_safe_realloc((&__pyx_v_self->samples), __pyx_v_n_samples); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_samples = __pyx_t_9;

  /* "sklearn/tree/_splitter.pyx":150
 * 
 *         cdef SIZE_t i, j
 *         cdef double weighted_n_samples = 0.0             # <<<<<<<<<<<<<<
 *         j = 0
 * 
 */
  __pyx_v_weighted_n_samples = 0.0;

  /* "sklearn/tree/_splitter.pyx":151
 *         cdef SIZE_t i, j
 *         cdef double weighted_n_samples = 0.0
 *         j = 0             # <<<<<<<<<<<<<<
 * 
 *         for i in range(n_samples):
 */
  __pyx_v_j = 0;

  /* "sklearn/tree/_splitter.pyx":153
 *         j = 0
 * 
 *         for i in range(n_samples):             # <<<<<<<<<<<<<<
 *             # Only work with positively weighted samples
 *             if sample_weight == NULL or sample_weight[i] != 0.0:
 */
  __pyx_t_8 = __pyx_v_n_samples;
  for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_8; __pyx_t_10+=1) {
    __pyx_v_i = __pyx_t_10;

    /* "sklearn/tree/_splitter.pyx":155
 *         for i in range(n_samples):
 *             # Only work with positively weighted samples
 *             if sample_weight == NULL or sample_weight[i] != 0.0:             # <<<<<<<<<<<<<<
 *                 samples[j] = i
 *                 j += 1
 */
    __pyx_t_12 = ((__pyx_v_sample_weight == NULL) != 0);
    if (!__pyx_t_12) {
    } else {
      __pyx_t_11 = __pyx_t_12;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_12 = (((__pyx_v_sample_weight[__pyx_v_i]) != 0.0) != 0);
    __pyx_t_11 = __pyx_t_12;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_11) {

      /* "sklearn/tree/_splitter.pyx":156
 *             # Only work with positively weighted samples
 *             if sample_weight == NULL or sample_weight[i] != 0.0:
 *                 samples[j] = i             # <<<<<<<<<<<<<<
 *                 j += 1
 * 
 */
      (__pyx_v_samples[__pyx_v_j]) = __pyx_v_i;

      /* "sklearn/tree/_splitter.pyx":157
 *             if sample_weight == NULL or sample_weight[i] != 0.0:
 *                 samples[j] = i
 *                 j += 1             # <<<<<<<<<<<<<<
 * 
 *             if sample_weight != NULL:
 */
      __pyx_v_j = (__pyx_v_j + 1);
      goto __pyx_L5;
    }
    __pyx_L5:;

    /* "sklearn/tree/_splitter.pyx":159
 *                 j += 1
 * 
 *             if sample_weight != NULL:             # <<<<<<<<<<<<<<
 *                 weighted_n_samples += sample_weight[i]
 *             else:
 */
    __pyx_t_11 = ((__pyx_v_sample_weight != NULL) != 0);
    if (__pyx_t_11) {

      /* "sklearn/tree/_splitter.pyx":160
 * 
 *             if sample_weight != NULL:
 *                 weighted_n_samples += sample_weight[i]             # <<<<<<<<<<<<<<
 *             else:
 *                 weighted_n_samples += 1.0
 */
      __pyx_v_weighted_n_samples = (__pyx_v_weighted_n_samples + (__pyx_v_sample_weight[__pyx_v_i]));
      goto __pyx_L8;
    }
    /*else*/ {

      /* "sklearn/tree/_splitter.pyx":162
 *                 weighted_n_samples += sample_weight[i]
 *             else:
 *                 weighted_n_samples += 1.0             # <<<<<<<<<<<<<<
 * 
 *         # Number of samples is number of positively weighted samples
 */
      __pyx_v_weighted_n_samples = (__pyx_v_weighted_n_samples + 1.0);
    }
    __pyx_L8:;
  }

  /* "sklearn/tree/_splitter.pyx":165
 * 
 *         # Number of samples is number of positively weighted samples
 *         self.n_samples = j             # <<<<<<<<<<<<<<
 *         self.weighted_n_samples = weighted_n_samples
 * 
 */
  __pyx_v_self->n_samples = __pyx_v_j;

  /* "sklearn/tree/_splitter.pyx":166
 *         # Number of samples is number of positively weighted samples
 *         self.n_samples = j
 *         self.weighted_n_samples = weighted_n_samples             # <<<<<<<<<<<<<<
 * 
 *         cdef SIZE_t n_features = X.shape[1]
 */
  __pyx_v_self->weighted_n_samples = __pyx_v_weighted_n_samples;

  /* "sklearn/tree/_splitter.pyx":168
 *         self.weighted_n_samples = weighted_n_samples
 * 
 *         cdef SIZE_t n_features = X.shape[1]             # <<<<<<<<<<<<<<
 *         cdef SIZE_t* features = safe_realloc(&self.features, n_features)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_shape); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_t_2, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_1); if (unlikely((__pyx_t_8 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_n_features = __pyx_t_8;

  /* "sklearn/tree/_splitter.pyx":169
 * 
 *         cdef SIZE_t n_features = X.shape[1]
 *         cdef SIZE_t* features = safe_realloc(&self.features, n_features)             # <<<<<<<<<<<<<<
 * 
 *         for i in range(n_features):
 */
  __pyx_t_9 = __pyx_fuse_1__pyx_f_7sklearn_4tree_6_utils_safe_realloc((&__pyx_v_self->features), __pyx_v_n_features); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_features = __pyx_t_9;

  /* "sklearn/tree/_splitter.pyx":171
 *         cdef SIZE_t* features = safe_realloc(&self.features, n_features)
 * 
 *         for i in range(n_features):             # <<<<<<<<<<<<<<
 *             features[i] = i
 * 
 */
  __pyx_t_8 = __pyx_v_n_features;
  for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_8; __pyx_t_10+=1) {
    __pyx_v_i = __pyx_t_10;

    /* "sklearn/tree/_splitter.pyx":172
 * 
 *         for i in range(n_features):
 *             features[i] = i             # <<<<<<<<<<<<<<
 * 
 *         self.n_features = n_features
 */
    (__pyx_v_features[__pyx_v_i]) = __pyx_v_i;
  }

  /* "sklearn/tree/_splitter.pyx":174
 *             features[i] = i
 * 
 *         self.n_features = n_features             # <<<<<<<<<<<<<<
 * 
 *         safe_realloc(&self.feature_values, n_samples)
 */
  __pyx_v_self->n_features = __pyx_v_n_features;

  /* "sklearn/tree/_splitter.pyx":176
 *         self.n_features = n_features
 * 
 *         safe_realloc(&self.feature_values, n_samples)             # <<<<<<<<<<<<<<
 *         safe_realloc(&self.constant_features, n_features)
 * 
 */
  __pyx_fuse_0__pyx_f_7sklearn_4tree_6_utils_safe_realloc((&__pyx_v_self->feature_values), __pyx_v_n_samples); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 176; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sklearn/tree/_splitter.pyx":177
 * 
 *         safe_realloc(&self.feature_values, n_samples)
 *         safe_realloc(&self.constant_features, n_features)             # <<<<<<<<<<<<<<
 * 
 *         self.y = <DOUBLE_t*> y.data
 */
  __pyx_fuse_1__pyx_f_7sklearn_4tree_6_utils_safe_realloc((&__pyx_v_self->constant_features), __pyx_v_n_features); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 177; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sklearn/tree/_splitter.pyx":179
 *         safe_realloc(&self.constant_features, n_features)
 * 
 *         self.y = <DOUBLE_t*> y.data             # <<<<<<<<<<<<<<
 *         self.y_stride = <SIZE_t> y.strides[0] / <SIZE_t> y.itemsize
 * 
 */
  __pyx_v_self->y = ((__pyx_t_7sklearn_4tree_9_splitter_DOUBLE_t *)__pyx_v_y->data);

  /* "sklearn/tree/_splitter.pyx":180
 * 
 *         self.y = <DOUBLE_t*> y.data
 *         self.y_stride = <SIZE_t> y.strides[0] / <SIZE_t> y.itemsize             # <<<<<<<<<<<<<<
 * 
 *         self.sample_weight = sample_weight
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_y), __pyx_n_s_itemsize); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 180; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_1); if (unlikely((__pyx_t_8 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 180; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->y_stride = (((__pyx_t_7sklearn_4tree_9_splitter_SIZE_t)(__pyx_v_y->strides[0])) / ((__pyx_t_7sklearn_4tree_9_splitter_SIZE_t)__pyx_t_8));

  /* "sklearn/tree/_splitter.pyx":182
 *         self.y_stride = <SIZE_t> y.strides[0] / <SIZE_t> y.itemsize
 * 
 *         self.sample_weight = sample_weight             # <<<<<<<<<<<<<<
 * 
 *     cdef void node_reset(self, SIZE_t start, SIZE_t end,
 */
  __pyx_v_self->sample_weight = __pyx_v_sample_weight;

  /* "sklearn/tree/_splitter.pyx":119
 *         pass
 * 
 *     cdef void init(self,             # <<<<<<<<<<<<<<
 *                    object X,
 *                    np.ndarray[DOUBLE_t, ndim=2, mode="c"] y,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_y.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("sklearn.tree._splitter.Splitter.init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_y.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_RefNannyFinishContext();
}

/* "sklearn/tree/_splitter.pyx":184
 *         self.sample_weight = sample_weight
 * 
 *     cdef void node_reset(self, SIZE_t start, SIZE_t end,             # <<<<<<<<<<<<<<
 *                          double* weighted_n_node_samples) nogil:
 *         """Reset splitter on node samples[start:end].
 */

static void __pyx_f_7sklearn_4tree_9_splitter_8Splitter_node_reset(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_start, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_end, double *__pyx_v_weighted_n_node_samples) {
  double __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":198
 *         """
 * 
 *         self.start = start             # <<<<<<<<<<<<<<
 *         self.end = end
 * 
 */
  __pyx_v_self->start = __pyx_v_start;

  /* "sklearn/tree/_splitter.pyx":199
 * 
 *         self.start = start
 *         self.end = end             # <<<<<<<<<<<<<<
 * 
 *         self.criterion.init(self.y,
 */
  __pyx_v_self->end = __pyx_v_end;

  /* "sklearn/tree/_splitter.pyx":201
 *         self.end = end
 * 
 *         self.criterion.init(self.y,             # <<<<<<<<<<<<<<
 *                             self.y_stride,
 *                             self.sample_weight,
 */
  ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->criterion->__pyx_vtab)->init(__pyx_v_self->criterion, __pyx_v_self->y, __pyx_v_self->y_stride, __pyx_v_self->sample_weight, __pyx_v_self->weighted_n_samples, __pyx_v_self->samples, __pyx_v_start, __pyx_v_end);

  /* "sklearn/tree/_splitter.pyx":209
 *                             end)
 * 
 *         weighted_n_node_samples[0] = self.criterion.weighted_n_node_samples             # <<<<<<<<<<<<<<
 * 
 *     cdef void node_split(self, double impurity, SplitRecord* split,
 */
  __pyx_t_1 = __pyx_v_self->criterion->weighted_n_node_samples;
  (__pyx_v_weighted_n_node_samples[0]) = __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":184
 *         self.sample_weight = sample_weight
 * 
 *     cdef void node_reset(self, SIZE_t start, SIZE_t end,             # <<<<<<<<<<<<<<
 *                          double* weighted_n_node_samples) nogil:
 *         """Reset splitter on node samples[start:end].
 */

  /* function exit code */
}

/* "sklearn/tree/_splitter.pyx":211
 *         weighted_n_node_samples[0] = self.criterion.weighted_n_node_samples
 * 
 *     cdef void node_split(self, double impurity, SplitRecord* split,             # <<<<<<<<<<<<<<
 *                          SIZE_t* n_constant_features) nogil:
 *         """Find the best split on node samples[start:end].
 */

static void __pyx_f_7sklearn_4tree_9_splitter_8Splitter_node_split(CYTHON_UNUSED struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self, CYTHON_UNUSED double __pyx_v_impurity, CYTHON_UNUSED struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord *__pyx_v_split, CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_n_constant_features) {

  /* function exit code */
}

/* "sklearn/tree/_splitter.pyx":221
 *         pass
 * 
 *     cdef void node_value(self, double* dest) nogil:             # <<<<<<<<<<<<<<
 *         """Copy the value of node samples[start:end] into dest."""
 * 
 */

static void __pyx_f_7sklearn_4tree_9_splitter_8Splitter_node_value(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self, double *__pyx_v_dest) {

  /* "sklearn/tree/_splitter.pyx":224
 *         """Copy the value of node samples[start:end] into dest."""
 * 
 *         self.criterion.node_value(dest)             # <<<<<<<<<<<<<<
 * 
 *     cdef double node_impurity(self) nogil:
 */
  ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->criterion->__pyx_vtab)->node_value(__pyx_v_self->criterion, __pyx_v_dest);

  /* "sklearn/tree/_splitter.pyx":221
 *         pass
 * 
 *     cdef void node_value(self, double* dest) nogil:             # <<<<<<<<<<<<<<
 *         """Copy the value of node samples[start:end] into dest."""
 * 
 */

  /* function exit code */
}

/* "sklearn/tree/_splitter.pyx":226
 *         self.criterion.node_value(dest)
 * 
 *     cdef double node_impurity(self) nogil:             # <<<<<<<<<<<<<<
 *         """Return the impurity of the current node."""
 * 
 */

static double __pyx_f_7sklearn_4tree_9_splitter_8Splitter_node_impurity(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self) {
  double __pyx_r;

  /* "sklearn/tree/_splitter.pyx":229
 *         """Return the impurity of the current node."""
 * 
 *         return self.criterion.node_impurity()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->criterion->__pyx_vtab)->node_impurity(__pyx_v_self->criterion);
  goto __pyx_L0;

  /* "sklearn/tree/_splitter.pyx":226
 *         self.criterion.node_value(dest)
 * 
 *     cdef double node_impurity(self) nogil:             # <<<<<<<<<<<<<<
 *         """Return the impurity of the current node."""
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "sklearn/tree/_splitter.pxd":41
 * 
 *     # Internal structures
 *     cdef public Criterion criterion      # Impurity criterion             # <<<<<<<<<<<<<<
 *     cdef public SIZE_t max_features      # Number of features to test
 *     cdef public SIZE_t min_samples_leaf  # Min samples in a leaf
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_4tree_9_splitter_8Splitter_9criterion_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7sklearn_4tree_9_splitter_8Splitter_9criterion_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_9criterion___get__(((struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_4tree_9_splitter_8Splitter_9criterion___get__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->criterion));
  __pyx_r = ((PyObject *)__pyx_v_self->criterion);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_9criterion_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_9criterion_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_9criterion_2__set__(((struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_9criterion_2__set__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_7sklearn_4tree_10_criterion_Criterion))))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 41; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->criterion);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->criterion));
  __pyx_v_self->criterion = ((struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sklearn.tree._splitter.Splitter.criterion.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_9criterion_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_9criterion_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_9criterion_4__del__(((struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_9criterion_4__del__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->criterion);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->criterion));
  __pyx_v_self->criterion = ((struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/tree/_splitter.pxd":42
 *     # Internal structures
 *     cdef public Criterion criterion      # Impurity criterion
 *     cdef public SIZE_t max_features      # Number of features to test             # <<<<<<<<<<<<<<
 *     cdef public SIZE_t min_samples_leaf  # Min samples in a leaf
 *     cdef public double min_weight_leaf   # Minimum weight in a leaf
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_4tree_9_splitter_8Splitter_12max_features_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7sklearn_4tree_9_splitter_8Splitter_12max_features_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_12max_features___get__(((struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_4tree_9_splitter_8Splitter_12max_features___get__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->max_features); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 42; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sklearn.tree._splitter.Splitter.max_features.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_12max_features_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_12max_features_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_12max_features_2__set__(((struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_12max_features_2__set__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_Py_intptr_t(__pyx_v_value); if (unlikely((__pyx_t_1 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 42; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_self->max_features = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("sklearn.tree._splitter.Splitter.max_features.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/tree/_splitter.pxd":43
 *     cdef public Criterion criterion      # Impurity criterion
 *     cdef public SIZE_t max_features      # Number of features to test
 *     cdef public SIZE_t min_samples_leaf  # Min samples in a leaf             # <<<<<<<<<<<<<<
 *     cdef public double min_weight_leaf   # Minimum weight in a leaf
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_4tree_9_splitter_8Splitter_16min_samples_leaf_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7sklearn_4tree_9_splitter_8Splitter_16min_samples_leaf_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_16min_samples_leaf___get__(((struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_4tree_9_splitter_8Splitter_16min_samples_leaf___get__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->min_samples_leaf); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 43; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sklearn.tree._splitter.Splitter.min_samples_leaf.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_16min_samples_leaf_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_16min_samples_leaf_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_16min_samples_leaf_2__set__(((struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_16min_samples_leaf_2__set__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_Py_intptr_t(__pyx_v_value); if (unlikely((__pyx_t_1 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 43; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_self->min_samples_leaf = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("sklearn.tree._splitter.Splitter.min_samples_leaf.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/tree/_splitter.pxd":44
 *     cdef public SIZE_t max_features      # Number of features to test
 *     cdef public SIZE_t min_samples_leaf  # Min samples in a leaf
 *     cdef public double min_weight_leaf   # Minimum weight in a leaf             # <<<<<<<<<<<<<<
 * 
 *     cdef object random_state             # Random state
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_4tree_9_splitter_8Splitter_15min_weight_leaf_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7sklearn_4tree_9_splitter_8Splitter_15min_weight_leaf_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_15min_weight_leaf___get__(((struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_4tree_9_splitter_8Splitter_15min_weight_leaf___get__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->min_weight_leaf); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 44; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("sklearn.tree._splitter.Splitter.min_weight_leaf.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_15min_weight_leaf_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_15min_weight_leaf_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_15min_weight_leaf_2__set__(((struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7sklearn_4tree_9_splitter_8Splitter_15min_weight_leaf_2__set__(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 44; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_self->min_weight_leaf = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("sklearn.tree._splitter.Splitter.min_weight_leaf.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/tree/_splitter.pyx":243
 *     cdef SIZE_t* sample_mask
 * 
 *     def __cinit__(self, Criterion criterion, SIZE_t max_features,             # <<<<<<<<<<<<<<
 *                   SIZE_t min_samples_leaf, double min_weight_leaf,
 *                   object random_state, bint presort):
 */

/* Python wrapper */
static int __pyx_pw_7sklearn_4tree_9_splitter_17BaseDenseSplitter_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7sklearn_4tree_9_splitter_17BaseDenseSplitter_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *__pyx_v_criterion = 0;
  CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_max_features;
  CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_min_samples_leaf;
  CYTHON_UNUSED double __pyx_v_min_weight_leaf;
  CYTHON_UNUSED PyObject *__pyx_v_random_state = 0;
  int __pyx_v_presort;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_criterion,&__pyx_n_s_max_features,&__pyx_n_s_min_samples_leaf,&__pyx_n_s_min_weight_leaf,&__pyx_n_s_random_state,&__pyx_n_s_presort,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_criterion)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_max_features)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_min_samples_leaf)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_min_weight_leaf)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_random_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_presort)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, 5); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 6) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
    }
    __pyx_v_criterion = ((struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *)values[0]);
    __pyx_v_max_features = __Pyx_PyInt_As_Py_intptr_t(values[1]); if (unlikely((__pyx_v_max_features == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_min_samples_leaf = __Pyx_PyInt_As_Py_intptr_t(values[2]); if (unlikely((__pyx_v_min_samples_leaf == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 244; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_min_weight_leaf = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_min_weight_leaf == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 244; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_random_state = values[4];
    __pyx_v_presort = __Pyx_PyObject_IsTrue(values[5]); if (unlikely((__pyx_v_presort == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 245; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sklearn.tree._splitter.BaseDenseSplitter.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_criterion), __pyx_ptype_7sklearn_4tree_10_criterion_Criterion, 1, "criterion", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_7sklearn_4tree_9_splitter_17BaseDenseSplitter___cinit__(((struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter *)__pyx_v_self), __pyx_v_criterion, __pyx_v_max_features, __pyx_v_min_samples_leaf, __pyx_v_min_weight_leaf, __pyx_v_random_state, __pyx_v_presort);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7sklearn_4tree_9_splitter_17BaseDenseSplitter___cinit__(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *__pyx_v_criterion, CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_max_features, CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_min_samples_leaf, CYTHON_UNUSED double __pyx_v_min_weight_leaf, CYTHON_UNUSED PyObject *__pyx_v_random_state, int __pyx_v_presort) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "sklearn/tree/_splitter.pyx":247
 *                   object random_state, bint presort):
 * 
 *         self.X = NULL             # <<<<<<<<<<<<<<
 *         self.X_sample_stride = 0
 *         self.X_feature_stride = 0
 */
  __pyx_v_self->X = NULL;

  /* "sklearn/tree/_splitter.pyx":248
 * 
 *         self.X = NULL
 *         self.X_sample_stride = 0             # <<<<<<<<<<<<<<
 *         self.X_feature_stride = 0
 *         self.X_idx_sorted_ptr = NULL
 */
  __pyx_v_self->X_sample_stride = 0;

  /* "sklearn/tree/_splitter.pyx":249
 *         self.X = NULL
 *         self.X_sample_stride = 0
 *         self.X_feature_stride = 0             # <<<<<<<<<<<<<<
 *         self.X_idx_sorted_ptr = NULL
 *         self.X_idx_sorted_stride = 0
 */
  __pyx_v_self->X_feature_stride = 0;

  /* "sklearn/tree/_splitter.pyx":250
 *         self.X_sample_stride = 0
 *         self.X_feature_stride = 0
 *         self.X_idx_sorted_ptr = NULL             # <<<<<<<<<<<<<<
 *         self.X_idx_sorted_stride = 0
 *         self.sample_mask = NULL
 */
  __pyx_v_self->X_idx_sorted_ptr = NULL;

  /* "sklearn/tree/_splitter.pyx":251
 *         self.X_feature_stride = 0
 *         self.X_idx_sorted_ptr = NULL
 *         self.X_idx_sorted_stride = 0             # <<<<<<<<<<<<<<
 *         self.sample_mask = NULL
 *         self.presort = presort
 */
  __pyx_v_self->X_idx_sorted_stride = 0;

  /* "sklearn/tree/_splitter.pyx":252
 *         self.X_idx_sorted_ptr = NULL
 *         self.X_idx_sorted_stride = 0
 *         self.sample_mask = NULL             # <<<<<<<<<<<<<<
 *         self.presort = presort
 * 
 */
  __pyx_v_self->sample_mask = NULL;

  /* "sklearn/tree/_splitter.pyx":253
 *         self.X_idx_sorted_stride = 0
 *         self.sample_mask = NULL
 *         self.presort = presort             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_v_self->__pyx_base.presort = __pyx_v_presort;

  /* "sklearn/tree/_splitter.pyx":243
 *     cdef SIZE_t* sample_mask
 * 
 *     def __cinit__(self, Criterion criterion, SIZE_t max_features,             # <<<<<<<<<<<<<<
 *                   SIZE_t min_samples_leaf, double min_weight_leaf,
 *                   object random_state, bint presort):
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/tree/_splitter.pyx":255
 *         self.presort = presort
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         """Destructor."""
 *         if self.presort == 1:
 */

/* Python wrapper */
static void __pyx_pw_7sklearn_4tree_9_splitter_17BaseDenseSplitter_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_7sklearn_4tree_9_splitter_17BaseDenseSplitter_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_7sklearn_4tree_9_splitter_17BaseDenseSplitter_2__dealloc__(((struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_7sklearn_4tree_9_splitter_17BaseDenseSplitter_2__dealloc__(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "sklearn/tree/_splitter.pyx":257
 *     def __dealloc__(self):
 *         """Destructor."""
 *         if self.presort == 1:             # <<<<<<<<<<<<<<
 *             free(self.sample_mask)
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->__pyx_base.presort == 1) != 0);
  if (__pyx_t_1) {

    /* "sklearn/tree/_splitter.pyx":258
 *         """Destructor."""
 *         if self.presort == 1:
 *             free(self.sample_mask)             # <<<<<<<<<<<<<<
 * 
 *     cdef void init(self,
 */
    free(__pyx_v_self->sample_mask);
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "sklearn/tree/_splitter.pyx":255
 *         self.presort = presort
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         """Destructor."""
 *         if self.presort == 1:
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "sklearn/tree/_splitter.pyx":260
 *             free(self.sample_mask)
 * 
 *     cdef void init(self,             # <<<<<<<<<<<<<<
 *                    object X,
 *                    np.ndarray[DOUBLE_t, ndim=2, mode="c"] y,
 */

static void __pyx_f_7sklearn_4tree_9_splitter_17BaseDenseSplitter_init(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter *__pyx_v_self, PyObject *__pyx_v_X, PyArrayObject *__pyx_v_y, __pyx_t_7sklearn_4tree_9_splitter_DOUBLE_t *__pyx_v_sample_weight, struct __pyx_opt_args_7sklearn_4tree_9_splitter_17BaseDenseSplitter_init *__pyx_optional_args) {

  /* "sklearn/tree/_splitter.pyx":264
 *                    np.ndarray[DOUBLE_t, ndim=2, mode="c"] y,
 *                    DOUBLE_t* sample_weight,
 *                    np.ndarray X_idx_sorted=None) except *:             # <<<<<<<<<<<<<<
 *         """Initialize the splitter."""
 * 
 */
  PyArrayObject *__pyx_v_X_idx_sorted = ((PyArrayObject *)Py_None);
  PyArrayObject *__pyx_v_X_ndarray = 0;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_y;
  __Pyx_Buffer __pyx_pybuffer_y;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_3;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("init", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_X_idx_sorted = __pyx_optional_args->X_idx_sorted;
    }
  }
  __pyx_pybuffer_y.pybuffer.buf = NULL;
  __pyx_pybuffer_y.refcount = 0;
  __pyx_pybuffernd_y.data = NULL;
  __pyx_pybuffernd_y.rcbuffer = &__pyx_pybuffer_y;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_y.rcbuffer->pybuffer, (PyObject*)__pyx_v_y, &__Pyx_TypeInfo_nn___pyx_t_7sklearn_4tree_9_splitter_DOUBLE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 260; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_y.diminfo[0].strides = __pyx_pybuffernd_y.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_y.diminfo[0].shape = __pyx_pybuffernd_y.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_y.diminfo[1].strides = __pyx_pybuffernd_y.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_y.diminfo[1].shape = __pyx_pybuffernd_y.rcbuffer->pybuffer.shape[1];

  /* "sklearn/tree/_splitter.pyx":268
 * 
 *         # Call parent init
 *         Splitter.init(self, X, y, sample_weight)             # <<<<<<<<<<<<<<
 * 
 *         # Initialize X
 */
  __pyx_vtabptr_7sklearn_4tree_9_splitter_Splitter->init(((struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *)__pyx_v_self), __pyx_v_X, ((PyArrayObject *)__pyx_v_y), __pyx_v_sample_weight, NULL); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sklearn/tree/_splitter.pyx":271
 * 
 *         # Initialize X
 *         cdef np.ndarray X_ndarray = X             # <<<<<<<<<<<<<<
 * 
 *         self.X = <DTYPE_t*> X_ndarray.data
 */
  if (!(likely(((__pyx_v_X) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_X, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 271; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __pyx_v_X;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_X_ndarray = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sklearn/tree/_splitter.pyx":273
 *         cdef np.ndarray X_ndarray = X
 * 
 *         self.X = <DTYPE_t*> X_ndarray.data             # <<<<<<<<<<<<<<
 *         self.X_sample_stride = <SIZE_t> X.strides[0] / <SIZE_t> X.itemsize
 *         self.X_feature_stride = <SIZE_t> X.strides[1] / <SIZE_t> X.itemsize
 */
  __pyx_v_self->X = ((__pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *)__pyx_v_X_ndarray->data);

  /* "sklearn/tree/_splitter.pyx":274
 * 
 *         self.X = <DTYPE_t*> X_ndarray.data
 *         self.X_sample_stride = <SIZE_t> X.strides[0] / <SIZE_t> X.itemsize             # <<<<<<<<<<<<<<
 *         self.X_feature_stride = <SIZE_t> X.strides[1] / <SIZE_t> X.itemsize
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_strides); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 274; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 274; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_2); if (unlikely((__pyx_t_3 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 274; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 274; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_2); if (unlikely((__pyx_t_4 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 274; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_self->X_sample_stride = (((__pyx_t_7sklearn_4tree_9_splitter_SIZE_t)__pyx_t_3) / ((__pyx_t_7sklearn_4tree_9_splitter_SIZE_t)__pyx_t_4));

  /* "sklearn/tree/_splitter.pyx":275
 *         self.X = <DTYPE_t*> X_ndarray.data
 *         self.X_sample_stride = <SIZE_t> X.strides[0] / <SIZE_t> X.itemsize
 *         self.X_feature_stride = <SIZE_t> X.strides[1] / <SIZE_t> X.itemsize             # <<<<<<<<<<<<<<
 * 
 *         if self.presort == 1:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_strides); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 275; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_t_2, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 275; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 275; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 275; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 275; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->X_feature_stride = (((__pyx_t_7sklearn_4tree_9_splitter_SIZE_t)__pyx_t_4) / ((__pyx_t_7sklearn_4tree_9_splitter_SIZE_t)__pyx_t_3));

  /* "sklearn/tree/_splitter.pyx":277
 *         self.X_feature_stride = <SIZE_t> X.strides[1] / <SIZE_t> X.itemsize
 * 
 *         if self.presort == 1:             # <<<<<<<<<<<<<<
 *             self.X_idx_sorted = X_idx_sorted
 *             self.X_idx_sorted_ptr = <INT32_t*> self.X_idx_sorted.data
 */
  __pyx_t_5 = ((__pyx_v_self->__pyx_base.presort == 1) != 0);
  if (__pyx_t_5) {

    /* "sklearn/tree/_splitter.pyx":278
 * 
 *         if self.presort == 1:
 *             self.X_idx_sorted = X_idx_sorted             # <<<<<<<<<<<<<<
 *             self.X_idx_sorted_ptr = <INT32_t*> self.X_idx_sorted.data
 *             self.X_idx_sorted_stride = (<SIZE_t> self.X_idx_sorted.strides[1] /
 */
    __Pyx_INCREF(((PyObject *)__pyx_v_X_idx_sorted));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_X_idx_sorted));
    __Pyx_GOTREF(__pyx_v_self->X_idx_sorted);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->X_idx_sorted));
    __pyx_v_self->X_idx_sorted = __pyx_v_X_idx_sorted;

    /* "sklearn/tree/_splitter.pyx":279
 *         if self.presort == 1:
 *             self.X_idx_sorted = X_idx_sorted
 *             self.X_idx_sorted_ptr = <INT32_t*> self.X_idx_sorted.data             # <<<<<<<<<<<<<<
 *             self.X_idx_sorted_stride = (<SIZE_t> self.X_idx_sorted.strides[1] /
 *                                         <SIZE_t> self.X_idx_sorted.itemsize)
 */
    __pyx_v_self->X_idx_sorted_ptr = ((__pyx_t_7sklearn_4tree_9_splitter_INT32_t *)__pyx_v_self->X_idx_sorted->data);

    /* "sklearn/tree/_splitter.pyx":281
 *             self.X_idx_sorted_ptr = <INT32_t*> self.X_idx_sorted.data
 *             self.X_idx_sorted_stride = (<SIZE_t> self.X_idx_sorted.strides[1] /
 *                                         <SIZE_t> self.X_idx_sorted.itemsize)             # <<<<<<<<<<<<<<
 * 
 *             self.n_total_samples = X.shape[0]
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->X_idx_sorted), __pyx_n_s_itemsize); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 281; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 281; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "sklearn/tree/_splitter.pyx":280
 *             self.X_idx_sorted = X_idx_sorted
 *             self.X_idx_sorted_ptr = <INT32_t*> self.X_idx_sorted.data
 *             self.X_idx_sorted_stride = (<SIZE_t> self.X_idx_sorted.strides[1] /             # <<<<<<<<<<<<<<
 *                                         <SIZE_t> self.X_idx_sorted.itemsize)
 * 
 */
    __pyx_v_self->X_idx_sorted_stride = (((__pyx_t_7sklearn_4tree_9_splitter_SIZE_t)(__pyx_v_self->X_idx_sorted->strides[1])) / ((__pyx_t_7sklearn_4tree_9_splitter_SIZE_t)__pyx_t_3));

    /* "sklearn/tree/_splitter.pyx":283
 *                                         <SIZE_t> self.X_idx_sorted.itemsize)
 * 
 *             self.n_total_samples = X.shape[0]             # <<<<<<<<<<<<<<
 *             safe_realloc(&self.sample_mask, self.n_total_samples)
 *             memset(self.sample_mask, 0, self.n_total_samples*sizeof(SIZE_t))
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_shape); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 283; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 283; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_3 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_2); if (unlikely((__pyx_t_3 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 283; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_self->n_total_samples = __pyx_t_3;

    /* "sklearn/tree/_splitter.pyx":284
 * 
 *             self.n_total_samples = X.shape[0]
 *             safe_realloc(&self.sample_mask, self.n_total_samples)             # <<<<<<<<<<<<<<
 *             memset(self.sample_mask, 0, self.n_total_samples*sizeof(SIZE_t))
 * 
 */
    __pyx_fuse_1__pyx_f_7sklearn_4tree_6_utils_safe_realloc((&__pyx_v_self->sample_mask), __pyx_v_self->n_total_samples); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 284; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "sklearn/tree/_splitter.pyx":285
 *             self.n_total_samples = X.shape[0]
 *             safe_realloc(&self.sample_mask, self.n_total_samples)
 *             memset(self.sample_mask, 0, self.n_total_samples*sizeof(SIZE_t))             # <<<<<<<<<<<<<<
 * 
 * 
 */
    memset(__pyx_v_self->sample_mask, 0, (__pyx_v_self->n_total_samples * (sizeof(__pyx_t_7sklearn_4tree_9_splitter_SIZE_t))));
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "sklearn/tree/_splitter.pyx":260
 *             free(self.sample_mask)
 * 
 *     cdef void init(self,             # <<<<<<<<<<<<<<
 *                    object X,
 *                    np.ndarray[DOUBLE_t, ndim=2, mode="c"] y,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_y.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("sklearn.tree._splitter.BaseDenseSplitter.init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_y.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_X_ndarray);
  __Pyx_RefNannyFinishContext();
}

/* "sklearn/tree/_splitter.pyx":290
 * cdef class BestSplitter(BaseDenseSplitter):
 *     """Splitter for finding the best split."""
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return (BestSplitter, (self.criterion,
 *                                self.max_features,
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_4tree_9_splitter_12BestSplitter_1__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7sklearn_4tree_9_splitter_12BestSplitter_1__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_4tree_9_splitter_12BestSplitter___reduce__(((struct __pyx_obj_7sklearn_4tree_9_splitter_BestSplitter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_4tree_9_splitter_12BestSplitter___reduce__(struct __pyx_obj_7sklearn_4tree_9_splitter_BestSplitter *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "sklearn/tree/_splitter.pyx":291
 *     """Splitter for finding the best split."""
 *     def __reduce__(self):
 *         return (BestSplitter, (self.criterion,             # <<<<<<<<<<<<<<
 *                                self.max_features,
 *                                self.min_samples_leaf,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "sklearn/tree/_splitter.pyx":292
 *     def __reduce__(self):
 *         return (BestSplitter, (self.criterion,
 *                                self.max_features,             # <<<<<<<<<<<<<<
 *                                self.min_samples_leaf,
 *                                self.min_weight_leaf,
 */
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->__pyx_base.__pyx_base.max_features); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);

  /* "sklearn/tree/_splitter.pyx":293
 *         return (BestSplitter, (self.criterion,
 *                                self.max_features,
 *                                self.min_samples_leaf,             # <<<<<<<<<<<<<<
 *                                self.min_weight_leaf,
 *                                self.random_state,
 */
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->__pyx_base.__pyx_base.min_samples_leaf); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);

  /* "sklearn/tree/_splitter.pyx":294
 *                                self.max_features,
 *                                self.min_samples_leaf,
 *                                self.min_weight_leaf,             # <<<<<<<<<<<<<<
 *                                self.random_state,
 *                                self.presort), self.__getstate__())
 */
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.min_weight_leaf); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);

  /* "sklearn/tree/_splitter.pyx":296
 *                                self.min_weight_leaf,
 *                                self.random_state,
 *                                self.presort), self.__getstate__())             # <<<<<<<<<<<<<<
 * 
 *     cdef void node_split(self, double impurity, SplitRecord* split,
 */
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.presort); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);

  /* "sklearn/tree/_splitter.pyx":291
 *     """Splitter for finding the best split."""
 *     def __reduce__(self):
 *         return (BestSplitter, (self.criterion,             # <<<<<<<<<<<<<<
 *                                self.max_features,
 *                                self.min_samples_leaf,
 */
  __pyx_t_5 = PyTuple_New(6); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.criterion));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.criterion));
  PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.criterion));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.random_state);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.random_state);
  PyTuple_SET_ITEM(__pyx_t_5, 4, __pyx_v_self->__pyx_base.__pyx_base.random_state);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 5, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;

  /* "sklearn/tree/_splitter.pyx":296
 *                                self.min_weight_leaf,
 *                                self.random_state,
 *                                self.presort), self.__getstate__())             # <<<<<<<<<<<<<<
 * 
 *     cdef void node_split(self, double impurity, SplitRecord* split,
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getstate); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sklearn/tree/_splitter.pyx":291
 *     """Splitter for finding the best split."""
 *     def __reduce__(self):
 *         return (BestSplitter, (self.criterion,             # <<<<<<<<<<<<<<
 *                                self.max_features,
 *                                self.min_samples_leaf,
 */
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)((PyObject*)__pyx_ptype_7sklearn_4tree_9_splitter_BestSplitter)));
  __Pyx_GIVEREF(((PyObject *)((PyObject*)__pyx_ptype_7sklearn_4tree_9_splitter_BestSplitter)));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)((PyObject*)__pyx_ptype_7sklearn_4tree_9_splitter_BestSplitter)));
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_4);
  __pyx_t_5 = 0;
  __pyx_t_4 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "sklearn/tree/_splitter.pyx":290
 * cdef class BestSplitter(BaseDenseSplitter):
 *     """Splitter for finding the best split."""
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return (BestSplitter, (self.criterion,
 *                                self.max_features,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("sklearn.tree._splitter.BestSplitter.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/tree/_splitter.pyx":298
 *                                self.presort), self.__getstate__())
 * 
 *     cdef void node_split(self, double impurity, SplitRecord* split,             # <<<<<<<<<<<<<<
 *                          SIZE_t* n_constant_features) nogil:
 *         """Find the best split on node samples[start:end]."""
 */

static void __pyx_f_7sklearn_4tree_9_splitter_12BestSplitter_node_split(struct __pyx_obj_7sklearn_4tree_9_splitter_BestSplitter *__pyx_v_self, double __pyx_v_impurity, struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord *__pyx_v_split, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_n_constant_features) {
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_start;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_end;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_features;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_constant_features;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_features;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_v_X;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_v_Xf;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_X_sample_stride;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_X_feature_stride;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_max_features;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_min_samples_leaf;
  double __pyx_v_min_weight_leaf;
  __pyx_t_7sklearn_4tree_9_splitter_UINT32_t *__pyx_v_random_state;
  __pyx_t_7sklearn_4tree_9_splitter_INT32_t *__pyx_v_X_idx_sorted;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_sample_mask;
  struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord __pyx_v_best;
  struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord __pyx_v_current;
  double __pyx_v_current_proxy_improvement;
  double __pyx_v_best_proxy_improvement;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_f_i;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_f_j;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_tmp;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_p;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_feature_idx_offset;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_feature_offset;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_i;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_j;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_visited_features;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_found_constants;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_drawn_constants;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_known_constants;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_total_constants;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_partition_end;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_t_1;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_2;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_t_3;
  double __pyx_t_4;
  __pyx_t_7sklearn_4tree_9_splitter_INT32_t *__pyx_t_5;
  int __pyx_t_6;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_7;
  int __pyx_t_8;

  /* "sklearn/tree/_splitter.pyx":302
 *         """Find the best split on node samples[start:end]."""
 *         # Find the best split
 *         cdef SIZE_t* samples = self.samples             # <<<<<<<<<<<<<<
 *         cdef SIZE_t start = self.start
 *         cdef SIZE_t end = self.end
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.__pyx_base.samples;
  __pyx_v_samples = __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":303
 *         # Find the best split
 *         cdef SIZE_t* samples = self.samples
 *         cdef SIZE_t start = self.start             # <<<<<<<<<<<<<<
 *         cdef SIZE_t end = self.end
 * 
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.start;
  __pyx_v_start = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":304
 *         cdef SIZE_t* samples = self.samples
 *         cdef SIZE_t start = self.start
 *         cdef SIZE_t end = self.end             # <<<<<<<<<<<<<<
 * 
 *         cdef SIZE_t* features = self.features
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.end;
  __pyx_v_end = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":306
 *         cdef SIZE_t end = self.end
 * 
 *         cdef SIZE_t* features = self.features             # <<<<<<<<<<<<<<
 *         cdef SIZE_t* constant_features = self.constant_features
 *         cdef SIZE_t n_features = self.n_features
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.__pyx_base.features;
  __pyx_v_features = __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":307
 * 
 *         cdef SIZE_t* features = self.features
 *         cdef SIZE_t* constant_features = self.constant_features             # <<<<<<<<<<<<<<
 *         cdef SIZE_t n_features = self.n_features
 * 
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.__pyx_base.constant_features;
  __pyx_v_constant_features = __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":308
 *         cdef SIZE_t* features = self.features
 *         cdef SIZE_t* constant_features = self.constant_features
 *         cdef SIZE_t n_features = self.n_features             # <<<<<<<<<<<<<<
 * 
 *         cdef DTYPE_t* X = self.X
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.n_features;
  __pyx_v_n_features = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":310
 *         cdef SIZE_t n_features = self.n_features
 * 
 *         cdef DTYPE_t* X = self.X             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t* Xf = self.feature_values
 *         cdef SIZE_t X_sample_stride = self.X_sample_stride
 */
  __pyx_t_3 = __pyx_v_self->__pyx_base.X;
  __pyx_v_X = __pyx_t_3;

  /* "sklearn/tree/_splitter.pyx":311
 * 
 *         cdef DTYPE_t* X = self.X
 *         cdef DTYPE_t* Xf = self.feature_values             # <<<<<<<<<<<<<<
 *         cdef SIZE_t X_sample_stride = self.X_sample_stride
 *         cdef SIZE_t X_feature_stride = self.X_feature_stride
 */
  __pyx_t_3 = __pyx_v_self->__pyx_base.__pyx_base.feature_values;
  __pyx_v_Xf = __pyx_t_3;

  /* "sklearn/tree/_splitter.pyx":312
 *         cdef DTYPE_t* X = self.X
 *         cdef DTYPE_t* Xf = self.feature_values
 *         cdef SIZE_t X_sample_stride = self.X_sample_stride             # <<<<<<<<<<<<<<
 *         cdef SIZE_t X_feature_stride = self.X_feature_stride
 *         cdef SIZE_t max_features = self.max_features
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.X_sample_stride;
  __pyx_v_X_sample_stride = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":313
 *         cdef DTYPE_t* Xf = self.feature_values
 *         cdef SIZE_t X_sample_stride = self.X_sample_stride
 *         cdef SIZE_t X_feature_stride = self.X_feature_stride             # <<<<<<<<<<<<<<
 *         cdef SIZE_t max_features = self.max_features
 *         cdef SIZE_t min_samples_leaf = self.min_samples_leaf
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.X_feature_stride;
  __pyx_v_X_feature_stride = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":314
 *         cdef SIZE_t X_sample_stride = self.X_sample_stride
 *         cdef SIZE_t X_feature_stride = self.X_feature_stride
 *         cdef SIZE_t max_features = self.max_features             # <<<<<<<<<<<<<<
 *         cdef SIZE_t min_samples_leaf = self.min_samples_leaf
 *         cdef double min_weight_leaf = self.min_weight_leaf
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.max_features;
  __pyx_v_max_features = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":315
 *         cdef SIZE_t X_feature_stride = self.X_feature_stride
 *         cdef SIZE_t max_features = self.max_features
 *         cdef SIZE_t min_samples_leaf = self.min_samples_leaf             # <<<<<<<<<<<<<<
 *         cdef double min_weight_leaf = self.min_weight_leaf
 *         cdef UINT32_t* random_state = &self.rand_r_state
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.min_samples_leaf;
  __pyx_v_min_samples_leaf = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":316
 *         cdef SIZE_t max_features = self.max_features
 *         cdef SIZE_t min_samples_leaf = self.min_samples_leaf
 *         cdef double min_weight_leaf = self.min_weight_leaf             # <<<<<<<<<<<<<<
 *         cdef UINT32_t* random_state = &self.rand_r_state
 * 
 */
  __pyx_t_4 = __pyx_v_self->__pyx_base.__pyx_base.min_weight_leaf;
  __pyx_v_min_weight_leaf = __pyx_t_4;

  /* "sklearn/tree/_splitter.pyx":317
 *         cdef SIZE_t min_samples_leaf = self.min_samples_leaf
 *         cdef double min_weight_leaf = self.min_weight_leaf
 *         cdef UINT32_t* random_state = &self.rand_r_state             # <<<<<<<<<<<<<<
 * 
 *         cdef INT32_t* X_idx_sorted = self.X_idx_sorted_ptr
 */
  __pyx_v_random_state = (&__pyx_v_self->__pyx_base.__pyx_base.rand_r_state);

  /* "sklearn/tree/_splitter.pyx":319
 *         cdef UINT32_t* random_state = &self.rand_r_state
 * 
 *         cdef INT32_t* X_idx_sorted = self.X_idx_sorted_ptr             # <<<<<<<<<<<<<<
 *         cdef SIZE_t* sample_mask = self.sample_mask
 * 
 */
  __pyx_t_5 = __pyx_v_self->__pyx_base.X_idx_sorted_ptr;
  __pyx_v_X_idx_sorted = __pyx_t_5;

  /* "sklearn/tree/_splitter.pyx":320
 * 
 *         cdef INT32_t* X_idx_sorted = self.X_idx_sorted_ptr
 *         cdef SIZE_t* sample_mask = self.sample_mask             # <<<<<<<<<<<<<<
 * 
 *         cdef SplitRecord best, current
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.sample_mask;
  __pyx_v_sample_mask = __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":323
 * 
 *         cdef SplitRecord best, current
 *         cdef double current_proxy_improvement = -INFINITY             # <<<<<<<<<<<<<<
 *         cdef double best_proxy_improvement = -INFINITY
 * 
 */
  __pyx_v_current_proxy_improvement = (-__pyx_v_7sklearn_4tree_9_splitter_INFINITY);

  /* "sklearn/tree/_splitter.pyx":324
 *         cdef SplitRecord best, current
 *         cdef double current_proxy_improvement = -INFINITY
 *         cdef double best_proxy_improvement = -INFINITY             # <<<<<<<<<<<<<<
 * 
 *         cdef SIZE_t f_i = n_features
 */
  __pyx_v_best_proxy_improvement = (-__pyx_v_7sklearn_4tree_9_splitter_INFINITY);

  /* "sklearn/tree/_splitter.pyx":326
 *         cdef double best_proxy_improvement = -INFINITY
 * 
 *         cdef SIZE_t f_i = n_features             # <<<<<<<<<<<<<<
 *         cdef SIZE_t f_j
 *         cdef SIZE_t tmp
 */
  __pyx_v_f_i = __pyx_v_n_features;

  /* "sklearn/tree/_splitter.pyx":335
 *         cdef SIZE_t j
 * 
 *         cdef SIZE_t n_visited_features = 0             # <<<<<<<<<<<<<<
 *         # Number of features discovered to be constant during the split search
 *         cdef SIZE_t n_found_constants = 0
 */
  __pyx_v_n_visited_features = 0;

  /* "sklearn/tree/_splitter.pyx":337
 *         cdef SIZE_t n_visited_features = 0
 *         # Number of features discovered to be constant during the split search
 *         cdef SIZE_t n_found_constants = 0             # <<<<<<<<<<<<<<
 *         # Number of features known to be constant and drawn without replacement
 *         cdef SIZE_t n_drawn_constants = 0
 */
  __pyx_v_n_found_constants = 0;

  /* "sklearn/tree/_splitter.pyx":339
 *         cdef SIZE_t n_found_constants = 0
 *         # Number of features known to be constant and drawn without replacement
 *         cdef SIZE_t n_drawn_constants = 0             # <<<<<<<<<<<<<<
 *         cdef SIZE_t n_known_constants = n_constant_features[0]
 *         # n_total_constants = n_known_constants + n_found_constants
 */
  __pyx_v_n_drawn_constants = 0;

  /* "sklearn/tree/_splitter.pyx":340
 *         # Number of features known to be constant and drawn without replacement
 *         cdef SIZE_t n_drawn_constants = 0
 *         cdef SIZE_t n_known_constants = n_constant_features[0]             # <<<<<<<<<<<<<<
 *         # n_total_constants = n_known_constants + n_found_constants
 *         cdef SIZE_t n_total_constants = n_known_constants
 */
  __pyx_v_n_known_constants = (__pyx_v_n_constant_features[0]);

  /* "sklearn/tree/_splitter.pyx":342
 *         cdef SIZE_t n_known_constants = n_constant_features[0]
 *         # n_total_constants = n_known_constants + n_found_constants
 *         cdef SIZE_t n_total_constants = n_known_constants             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t current_feature_value
 *         cdef SIZE_t partition_end
 */
  __pyx_v_n_total_constants = __pyx_v_n_known_constants;

  /* "sklearn/tree/_splitter.pyx":346
 *         cdef SIZE_t partition_end
 * 
 *         _init_split(&best, end)             # <<<<<<<<<<<<<<
 * 
 *         if self.presort == 1:
 */
  __pyx_f_7sklearn_4tree_9_splitter__init_split((&__pyx_v_best), __pyx_v_end);

  /* "sklearn/tree/_splitter.pyx":348
 *         _init_split(&best, end)
 * 
 *         if self.presort == 1:             # <<<<<<<<<<<<<<
 *             for p in range(start, end):
 *                 sample_mask[samples[p]] = 1
 */
  __pyx_t_6 = ((__pyx_v_self->__pyx_base.__pyx_base.presort == 1) != 0);
  if (__pyx_t_6) {

    /* "sklearn/tree/_splitter.pyx":349
 * 
 *         if self.presort == 1:
 *             for p in range(start, end):             # <<<<<<<<<<<<<<
 *                 sample_mask[samples[p]] = 1
 * 
 */
    __pyx_t_2 = __pyx_v_end;
    for (__pyx_t_7 = __pyx_v_start; __pyx_t_7 < __pyx_t_2; __pyx_t_7+=1) {
      __pyx_v_p = __pyx_t_7;

      /* "sklearn/tree/_splitter.pyx":350
 *         if self.presort == 1:
 *             for p in range(start, end):
 *                 sample_mask[samples[p]] = 1             # <<<<<<<<<<<<<<
 * 
 *         # Sample up to max_features without replacement using a
 */
      (__pyx_v_sample_mask[(__pyx_v_samples[__pyx_v_p])]) = 1;
    }
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "sklearn/tree/_splitter.pyx":361
 *         # newly discovered constant features to spare computation on descendant
 *         # nodes.
 *         while (f_i > n_total_constants and  # Stop early if remaining features             # <<<<<<<<<<<<<<
 *                                             # are constant
 *                 (n_visited_features < max_features or
 */
  while (1) {
    __pyx_t_8 = ((__pyx_v_f_i > __pyx_v_n_total_constants) != 0);
    if (__pyx_t_8) {
    } else {
      __pyx_t_6 = __pyx_t_8;
      goto __pyx_L8_bool_binop_done;
    }

    /* "sklearn/tree/_splitter.pyx":363
 *         while (f_i > n_total_constants and  # Stop early if remaining features
 *                                             # are constant
 *                 (n_visited_features < max_features or             # <<<<<<<<<<<<<<
 *                  # At least one drawn features must be non constant
 *                  n_visited_features <= n_found_constants + n_drawn_constants)):
 */
    __pyx_t_8 = ((__pyx_v_n_visited_features < __pyx_v_max_features) != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_6 = __pyx_t_8;
      goto __pyx_L8_bool_binop_done;
    }

    /* "sklearn/tree/_splitter.pyx":365
 *                 (n_visited_features < max_features or
 *                  # At least one drawn features must be non constant
 *                  n_visited_features <= n_found_constants + n_drawn_constants)):             # <<<<<<<<<<<<<<
 * 
 *             n_visited_features += 1
 */
    __pyx_t_8 = ((__pyx_v_n_visited_features <= (__pyx_v_n_found_constants + __pyx_v_n_drawn_constants)) != 0);
    __pyx_t_6 = __pyx_t_8;
    __pyx_L8_bool_binop_done:;
    if (!__pyx_t_6) break;

    /* "sklearn/tree/_splitter.pyx":367
 *                  n_visited_features <= n_found_constants + n_drawn_constants)):
 * 
 *             n_visited_features += 1             # <<<<<<<<<<<<<<
 * 
 *             # Loop invariant: elements of features in
 */
    __pyx_v_n_visited_features = (__pyx_v_n_visited_features + 1);

    /* "sklearn/tree/_splitter.pyx":381
 * 
 *             # Draw a feature at random
 *             f_j = rand_int(n_drawn_constants, f_i - n_found_constants,             # <<<<<<<<<<<<<<
 *                            random_state)
 * 
 */
    __pyx_v_f_j = __pyx_f_7sklearn_4tree_6_utils_rand_int(__pyx_v_n_drawn_constants, (__pyx_v_f_i - __pyx_v_n_found_constants), __pyx_v_random_state);

    /* "sklearn/tree/_splitter.pyx":384
 *                            random_state)
 * 
 *             if f_j < n_known_constants:             # <<<<<<<<<<<<<<
 *                 # f_j in the interval [n_drawn_constants, n_known_constants[
 *                 tmp = features[f_j]
 */
    __pyx_t_6 = ((__pyx_v_f_j < __pyx_v_n_known_constants) != 0);
    if (__pyx_t_6) {

      /* "sklearn/tree/_splitter.pyx":386
 *             if f_j < n_known_constants:
 *                 # f_j in the interval [n_drawn_constants, n_known_constants[
 *                 tmp = features[f_j]             # <<<<<<<<<<<<<<
 *                 features[f_j] = features[n_drawn_constants]
 *                 features[n_drawn_constants] = tmp
 */
      __pyx_v_tmp = (__pyx_v_features[__pyx_v_f_j]);

      /* "sklearn/tree/_splitter.pyx":387
 *                 # f_j in the interval [n_drawn_constants, n_known_constants[
 *                 tmp = features[f_j]
 *                 features[f_j] = features[n_drawn_constants]             # <<<<<<<<<<<<<<
 *                 features[n_drawn_constants] = tmp
 * 
 */
      (__pyx_v_features[__pyx_v_f_j]) = (__pyx_v_features[__pyx_v_n_drawn_constants]);

      /* "sklearn/tree/_splitter.pyx":388
 *                 tmp = features[f_j]
 *                 features[f_j] = features[n_drawn_constants]
 *                 features[n_drawn_constants] = tmp             # <<<<<<<<<<<<<<
 * 
 *                 n_drawn_constants += 1
 */
      (__pyx_v_features[__pyx_v_n_drawn_constants]) = __pyx_v_tmp;

      /* "sklearn/tree/_splitter.pyx":390
 *                 features[n_drawn_constants] = tmp
 * 
 *                 n_drawn_constants += 1             # <<<<<<<<<<<<<<
 * 
 *             else:
 */
      __pyx_v_n_drawn_constants = (__pyx_v_n_drawn_constants + 1);
      goto __pyx_L11;
    }
    /*else*/ {

      /* "sklearn/tree/_splitter.pyx":394
 *             else:
 *                 # f_j in the interval [n_known_constants, f_i - n_found_constants[
 *                 f_j += n_found_constants             # <<<<<<<<<<<<<<
 *                 # f_j in the interval [n_total_constants, f_i[
 *                 current.feature = features[f_j]
 */
      __pyx_v_f_j = (__pyx_v_f_j + __pyx_v_n_found_constants);

      /* "sklearn/tree/_splitter.pyx":396
 *                 f_j += n_found_constants
 *                 # f_j in the interval [n_total_constants, f_i[
 *                 current.feature = features[f_j]             # <<<<<<<<<<<<<<
 *                 feature_offset = self.X_feature_stride * current.feature
 * 
 */
      __pyx_v_current.feature = (__pyx_v_features[__pyx_v_f_j]);

      /* "sklearn/tree/_splitter.pyx":397
 *                 # f_j in the interval [n_total_constants, f_i[
 *                 current.feature = features[f_j]
 *                 feature_offset = self.X_feature_stride * current.feature             # <<<<<<<<<<<<<<
 * 
 *                 # Sort samples along that feature; either by utilizing
 */
      __pyx_v_feature_offset = (__pyx_v_self->__pyx_base.X_feature_stride * __pyx_v_current.feature);

      /* "sklearn/tree/_splitter.pyx":403
 *                 # sorting the array in a manner which utilizes the cache more
 *                 # effectively.
 *                 if self.presort == 1:             # <<<<<<<<<<<<<<
 *                     p = start
 *                     feature_idx_offset = self.X_idx_sorted_stride * current.feature
 */
      __pyx_t_6 = ((__pyx_v_self->__pyx_base.__pyx_base.presort == 1) != 0);
      if (__pyx_t_6) {

        /* "sklearn/tree/_splitter.pyx":404
 *                 # effectively.
 *                 if self.presort == 1:
 *                     p = start             # <<<<<<<<<<<<<<
 *                     feature_idx_offset = self.X_idx_sorted_stride * current.feature
 * 
 */
        __pyx_v_p = __pyx_v_start;

        /* "sklearn/tree/_splitter.pyx":405
 *                 if self.presort == 1:
 *                     p = start
 *                     feature_idx_offset = self.X_idx_sorted_stride * current.feature             # <<<<<<<<<<<<<<
 * 
 *                     for i in range(self.n_total_samples):
 */
        __pyx_v_feature_idx_offset = (__pyx_v_self->__pyx_base.X_idx_sorted_stride * __pyx_v_current.feature);

        /* "sklearn/tree/_splitter.pyx":407
 *                     feature_idx_offset = self.X_idx_sorted_stride * current.feature
 * 
 *                     for i in range(self.n_total_samples):             # <<<<<<<<<<<<<<
 *                         j = X_idx_sorted[i + feature_idx_offset]
 *                         if sample_mask[j] == 1:
 */
        __pyx_t_2 = __pyx_v_self->__pyx_base.n_total_samples;
        for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_2; __pyx_t_7+=1) {
          __pyx_v_i = __pyx_t_7;

          /* "sklearn/tree/_splitter.pyx":408
 * 
 *                     for i in range(self.n_total_samples):
 *                         j = X_idx_sorted[i + feature_idx_offset]             # <<<<<<<<<<<<<<
 *                         if sample_mask[j] == 1:
 *                             samples[p] = j
 */
          __pyx_v_j = (__pyx_v_X_idx_sorted[(__pyx_v_i + __pyx_v_feature_idx_offset)]);

          /* "sklearn/tree/_splitter.pyx":409
 *                     for i in range(self.n_total_samples):
 *                         j = X_idx_sorted[i + feature_idx_offset]
 *                         if sample_mask[j] == 1:             # <<<<<<<<<<<<<<
 *                             samples[p] = j
 *                             Xf[p] = X[self.X_sample_stride * j + feature_offset]
 */
          __pyx_t_6 = (((__pyx_v_sample_mask[__pyx_v_j]) == 1) != 0);
          if (__pyx_t_6) {

            /* "sklearn/tree/_splitter.pyx":410
 *                         j = X_idx_sorted[i + feature_idx_offset]
 *                         if sample_mask[j] == 1:
 *                             samples[p] = j             # <<<<<<<<<<<<<<
 *                             Xf[p] = X[self.X_sample_stride * j + feature_offset]
 *                             p += 1
 */
            (__pyx_v_samples[__pyx_v_p]) = __pyx_v_j;

            /* "sklearn/tree/_splitter.pyx":411
 *                         if sample_mask[j] == 1:
 *                             samples[p] = j
 *                             Xf[p] = X[self.X_sample_stride * j + feature_offset]             # <<<<<<<<<<<<<<
 *                             p += 1
 *                 else:
 */
            (__pyx_v_Xf[__pyx_v_p]) = (__pyx_v_X[((__pyx_v_self->__pyx_base.X_sample_stride * __pyx_v_j) + __pyx_v_feature_offset)]);

            /* "sklearn/tree/_splitter.pyx":412
 *                             samples[p] = j
 *                             Xf[p] = X[self.X_sample_stride * j + feature_offset]
 *                             p += 1             # <<<<<<<<<<<<<<
 *                 else:
 *                     for i in range(start, end):
 */
            __pyx_v_p = (__pyx_v_p + 1);
            goto __pyx_L15;
          }
          __pyx_L15:;
        }
        goto __pyx_L12;
      }
      /*else*/ {

        /* "sklearn/tree/_splitter.pyx":414
 *                             p += 1
 *                 else:
 *                     for i in range(start, end):             # <<<<<<<<<<<<<<
 *                         Xf[i] = X[self.X_sample_stride * samples[i] + feature_offset]
 * 
 */
        __pyx_t_2 = __pyx_v_end;
        for (__pyx_t_7 = __pyx_v_start; __pyx_t_7 < __pyx_t_2; __pyx_t_7+=1) {
          __pyx_v_i = __pyx_t_7;

          /* "sklearn/tree/_splitter.pyx":415
 *                 else:
 *                     for i in range(start, end):
 *                         Xf[i] = X[self.X_sample_stride * samples[i] + feature_offset]             # <<<<<<<<<<<<<<
 * 
 *                     sort(Xf + start, samples + start, end - start)
 */
          (__pyx_v_Xf[__pyx_v_i]) = (__pyx_v_X[((__pyx_v_self->__pyx_base.X_sample_stride * (__pyx_v_samples[__pyx_v_i])) + __pyx_v_feature_offset)]);
        }

        /* "sklearn/tree/_splitter.pyx":417
 *                         Xf[i] = X[self.X_sample_stride * samples[i] + feature_offset]
 * 
 *                     sort(Xf + start, samples + start, end - start)             # <<<<<<<<<<<<<<
 * 
 *                 if Xf[end - 1] <= Xf[start] + FEATURE_THRESHOLD:
 */
        __pyx_f_7sklearn_4tree_9_splitter_sort((__pyx_v_Xf + __pyx_v_start), (__pyx_v_samples + __pyx_v_start), (__pyx_v_end - __pyx_v_start));
      }
      __pyx_L12:;

      /* "sklearn/tree/_splitter.pyx":419
 *                     sort(Xf + start, samples + start, end - start)
 * 
 *                 if Xf[end - 1] <= Xf[start] + FEATURE_THRESHOLD:             # <<<<<<<<<<<<<<
 *                     features[f_j] = features[n_total_constants]
 *                     features[n_total_constants] = current.feature
 */
      __pyx_t_6 = (((__pyx_v_Xf[(__pyx_v_end - 1)]) <= ((__pyx_v_Xf[__pyx_v_start]) + __pyx_v_7sklearn_4tree_9_splitter_FEATURE_THRESHOLD)) != 0);
      if (__pyx_t_6) {

        /* "sklearn/tree/_splitter.pyx":420
 * 
 *                 if Xf[end - 1] <= Xf[start] + FEATURE_THRESHOLD:
 *                     features[f_j] = features[n_total_constants]             # <<<<<<<<<<<<<<
 *                     features[n_total_constants] = current.feature
 * 
 */
        (__pyx_v_features[__pyx_v_f_j]) = (__pyx_v_features[__pyx_v_n_total_constants]);

        /* "sklearn/tree/_splitter.pyx":421
 *                 if Xf[end - 1] <= Xf[start] + FEATURE_THRESHOLD:
 *                     features[f_j] = features[n_total_constants]
 *                     features[n_total_constants] = current.feature             # <<<<<<<<<<<<<<
 * 
 *                     n_found_constants += 1
 */
        __pyx_t_2 = __pyx_v_current.feature;
        (__pyx_v_features[__pyx_v_n_total_constants]) = __pyx_t_2;

        /* "sklearn/tree/_splitter.pyx":423
 *                     features[n_total_constants] = current.feature
 * 
 *                     n_found_constants += 1             # <<<<<<<<<<<<<<
 *                     n_total_constants += 1
 * 
 */
        __pyx_v_n_found_constants = (__pyx_v_n_found_constants + 1);

        /* "sklearn/tree/_splitter.pyx":424
 * 
 *                     n_found_constants += 1
 *                     n_total_constants += 1             # <<<<<<<<<<<<<<
 * 
 *                 else:
 */
        __pyx_v_n_total_constants = (__pyx_v_n_total_constants + 1);
        goto __pyx_L18;
      }
      /*else*/ {

        /* "sklearn/tree/_splitter.pyx":427
 * 
 *                 else:
 *                     f_i -= 1             # <<<<<<<<<<<<<<
 *                     features[f_i], features[f_j] = features[f_j], features[f_i]
 * 
 */
        __pyx_v_f_i = (__pyx_v_f_i - 1);

        /* "sklearn/tree/_splitter.pyx":428
 *                 else:
 *                     f_i -= 1
 *                     features[f_i], features[f_j] = features[f_j], features[f_i]             # <<<<<<<<<<<<<<
 * 
 *                     # Evaluate all splits
 */
        __pyx_t_2 = (__pyx_v_features[__pyx_v_f_j]);
        __pyx_t_7 = (__pyx_v_features[__pyx_v_f_i]);
        (__pyx_v_features[__pyx_v_f_i]) = __pyx_t_2;
        (__pyx_v_features[__pyx_v_f_j]) = __pyx_t_7;

        /* "sklearn/tree/_splitter.pyx":431
 * 
 *                     # Evaluate all splits
 *                     self.criterion.reset()             # <<<<<<<<<<<<<<
 *                     p = start
 * 
 */
        ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->reset(__pyx_v_self->__pyx_base.__pyx_base.criterion);

        /* "sklearn/tree/_splitter.pyx":432
 *                     # Evaluate all splits
 *                     self.criterion.reset()
 *                     p = start             # <<<<<<<<<<<<<<
 * 
 *                     while p < end:
 */
        __pyx_v_p = __pyx_v_start;

        /* "sklearn/tree/_splitter.pyx":434
 *                     p = start
 * 
 *                     while p < end:             # <<<<<<<<<<<<<<
 *                         while (p + 1 < end and
 *                                Xf[p + 1] <= Xf[p] + FEATURE_THRESHOLD):
 */
        while (1) {
          __pyx_t_6 = ((__pyx_v_p < __pyx_v_end) != 0);
          if (!__pyx_t_6) break;

          /* "sklearn/tree/_splitter.pyx":435
 * 
 *                     while p < end:
 *                         while (p + 1 < end and             # <<<<<<<<<<<<<<
 *                                Xf[p + 1] <= Xf[p] + FEATURE_THRESHOLD):
 *                             p += 1
 */
          while (1) {
            __pyx_t_8 = (((__pyx_v_p + 1) < __pyx_v_end) != 0);
            if (__pyx_t_8) {
            } else {
              __pyx_t_6 = __pyx_t_8;
              goto __pyx_L23_bool_binop_done;
            }

            /* "sklearn/tree/_splitter.pyx":436
 *                     while p < end:
 *                         while (p + 1 < end and
 *                                Xf[p + 1] <= Xf[p] + FEATURE_THRESHOLD):             # <<<<<<<<<<<<<<
 *                             p += 1
 * 
 */
            __pyx_t_8 = (((__pyx_v_Xf[(__pyx_v_p + 1)]) <= ((__pyx_v_Xf[__pyx_v_p]) + __pyx_v_7sklearn_4tree_9_splitter_FEATURE_THRESHOLD)) != 0);
            __pyx_t_6 = __pyx_t_8;
            __pyx_L23_bool_binop_done:;
            if (!__pyx_t_6) break;

            /* "sklearn/tree/_splitter.pyx":437
 *                         while (p + 1 < end and
 *                                Xf[p + 1] <= Xf[p] + FEATURE_THRESHOLD):
 *                             p += 1             # <<<<<<<<<<<<<<
 * 
 *                         # (p + 1 >= end) or (X[samples[p + 1], current.feature] >
 */
            __pyx_v_p = (__pyx_v_p + 1);
          }

          /* "sklearn/tree/_splitter.pyx":441
 *                         # (p + 1 >= end) or (X[samples[p + 1], current.feature] >
 *                         #                    X[samples[p], current.feature])
 *                         p += 1             # <<<<<<<<<<<<<<
 *                         # (p >= end) or (X[samples[p], current.feature] >
 *                         #                X[samples[p - 1], current.feature])
 */
          __pyx_v_p = (__pyx_v_p + 1);

          /* "sklearn/tree/_splitter.pyx":445
 *                         #                X[samples[p - 1], current.feature])
 * 
 *                         if p < end:             # <<<<<<<<<<<<<<
 *                             current.pos = p
 * 
 */
          __pyx_t_6 = ((__pyx_v_p < __pyx_v_end) != 0);
          if (__pyx_t_6) {

            /* "sklearn/tree/_splitter.pyx":446
 * 
 *                         if p < end:
 *                             current.pos = p             # <<<<<<<<<<<<<<
 * 
 *                             # Reject if min_samples_leaf is not guaranteed
 */
            __pyx_v_current.pos = __pyx_v_p;

            /* "sklearn/tree/_splitter.pyx":449
 * 
 *                             # Reject if min_samples_leaf is not guaranteed
 *                             if (((current.pos - start) < min_samples_leaf) or             # <<<<<<<<<<<<<<
 *                                     ((end - current.pos) < min_samples_leaf)):
 *                                 continue
 */
            __pyx_t_8 = (((__pyx_v_current.pos - __pyx_v_start) < __pyx_v_min_samples_leaf) != 0);
            if (!__pyx_t_8) {
            } else {
              __pyx_t_6 = __pyx_t_8;
              goto __pyx_L27_bool_binop_done;
            }

            /* "sklearn/tree/_splitter.pyx":450
 *                             # Reject if min_samples_leaf is not guaranteed
 *                             if (((current.pos - start) < min_samples_leaf) or
 *                                     ((end - current.pos) < min_samples_leaf)):             # <<<<<<<<<<<<<<
 *                                 continue
 * 
 */
            __pyx_t_8 = (((__pyx_v_end - __pyx_v_current.pos) < __pyx_v_min_samples_leaf) != 0);
            __pyx_t_6 = __pyx_t_8;
            __pyx_L27_bool_binop_done:;
            if (__pyx_t_6) {

              /* "sklearn/tree/_splitter.pyx":451
 *                             if (((current.pos - start) < min_samples_leaf) or
 *                                     ((end - current.pos) < min_samples_leaf)):
 *                                 continue             # <<<<<<<<<<<<<<
 * 
 *                             self.criterion.update(current.pos)
 */
              goto __pyx_L19_continue;
            }

            /* "sklearn/tree/_splitter.pyx":453
 *                                 continue
 * 
 *                             self.criterion.update(current.pos)             # <<<<<<<<<<<<<<
 * 
 *                             # Reject if min_weight_leaf is not satisfied
 */
            ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->update(__pyx_v_self->__pyx_base.__pyx_base.criterion, __pyx_v_current.pos);

            /* "sklearn/tree/_splitter.pyx":456
 * 
 *                             # Reject if min_weight_leaf is not satisfied
 *                             if ((self.criterion.weighted_n_left < min_weight_leaf) or             # <<<<<<<<<<<<<<
 *                                     (self.criterion.weighted_n_right < min_weight_leaf)):
 *                                 continue
 */
            __pyx_t_8 = ((__pyx_v_self->__pyx_base.__pyx_base.criterion->weighted_n_left < __pyx_v_min_weight_leaf) != 0);
            if (!__pyx_t_8) {
            } else {
              __pyx_t_6 = __pyx_t_8;
              goto __pyx_L30_bool_binop_done;
            }

            /* "sklearn/tree/_splitter.pyx":457
 *                             # Reject if min_weight_leaf is not satisfied
 *                             if ((self.criterion.weighted_n_left < min_weight_leaf) or
 *                                     (self.criterion.weighted_n_right < min_weight_leaf)):             # <<<<<<<<<<<<<<
 *                                 continue
 * 
 */
            __pyx_t_8 = ((__pyx_v_self->__pyx_base.__pyx_base.criterion->weighted_n_right < __pyx_v_min_weight_leaf) != 0);
            __pyx_t_6 = __pyx_t_8;
            __pyx_L30_bool_binop_done:;
            if (__pyx_t_6) {

              /* "sklearn/tree/_splitter.pyx":458
 *                             if ((self.criterion.weighted_n_left < min_weight_leaf) or
 *                                     (self.criterion.weighted_n_right < min_weight_leaf)):
 *                                 continue             # <<<<<<<<<<<<<<
 * 
 *                             current_proxy_improvement = self.criterion.proxy_impurity_improvement()
 */
              goto __pyx_L19_continue;
            }

            /* "sklearn/tree/_splitter.pyx":460
 *                                 continue
 * 
 *                             current_proxy_improvement = self.criterion.proxy_impurity_improvement()             # <<<<<<<<<<<<<<
 * 
 *                             if current_proxy_improvement > best_proxy_improvement:
 */
            __pyx_v_current_proxy_improvement = ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->proxy_impurity_improvement(__pyx_v_self->__pyx_base.__pyx_base.criterion);

            /* "sklearn/tree/_splitter.pyx":462
 *                             current_proxy_improvement = self.criterion.proxy_impurity_improvement()
 * 
 *                             if current_proxy_improvement > best_proxy_improvement:             # <<<<<<<<<<<<<<
 *                                 best_proxy_improvement = current_proxy_improvement
 *                                 current.threshold = (Xf[p - 1] + Xf[p]) / 2.0
 */
            __pyx_t_6 = ((__pyx_v_current_proxy_improvement > __pyx_v_best_proxy_improvement) != 0);
            if (__pyx_t_6) {

              /* "sklearn/tree/_splitter.pyx":463
 * 
 *                             if current_proxy_improvement > best_proxy_improvement:
 *                                 best_proxy_improvement = current_proxy_improvement             # <<<<<<<<<<<<<<
 *                                 current.threshold = (Xf[p - 1] + Xf[p]) / 2.0
 * 
 */
              __pyx_v_best_proxy_improvement = __pyx_v_current_proxy_improvement;

              /* "sklearn/tree/_splitter.pyx":464
 *                             if current_proxy_improvement > best_proxy_improvement:
 *                                 best_proxy_improvement = current_proxy_improvement
 *                                 current.threshold = (Xf[p - 1] + Xf[p]) / 2.0             # <<<<<<<<<<<<<<
 * 
 *                                 if current.threshold == Xf[p]:
 */
              __pyx_v_current.threshold = (((__pyx_v_Xf[(__pyx_v_p - 1)]) + (__pyx_v_Xf[__pyx_v_p])) / 2.0);

              /* "sklearn/tree/_splitter.pyx":466
 *                                 current.threshold = (Xf[p - 1] + Xf[p]) / 2.0
 * 
 *                                 if current.threshold == Xf[p]:             # <<<<<<<<<<<<<<
 *                                     current.threshold = Xf[p - 1]
 * 
 */
              __pyx_t_6 = ((__pyx_v_current.threshold == (__pyx_v_Xf[__pyx_v_p])) != 0);
              if (__pyx_t_6) {

                /* "sklearn/tree/_splitter.pyx":467
 * 
 *                                 if current.threshold == Xf[p]:
 *                                     current.threshold = Xf[p - 1]             # <<<<<<<<<<<<<<
 * 
 *                                 best = current  # copy
 */
                __pyx_v_current.threshold = (__pyx_v_Xf[(__pyx_v_p - 1)]);
                goto __pyx_L33;
              }
              __pyx_L33:;

              /* "sklearn/tree/_splitter.pyx":469
 *                                     current.threshold = Xf[p - 1]
 * 
 *                                 best = current  # copy             # <<<<<<<<<<<<<<
 * 
 *         # Reorganize into samples[start:best.pos] + samples[best.pos:end]
 */
              __pyx_v_best = __pyx_v_current;
              goto __pyx_L32;
            }
            __pyx_L32:;
            goto __pyx_L25;
          }
          __pyx_L25:;
          __pyx_L19_continue:;
        }
      }
      __pyx_L18:;
    }
    __pyx_L11:;
  }

  /* "sklearn/tree/_splitter.pyx":472
 * 
 *         # Reorganize into samples[start:best.pos] + samples[best.pos:end]
 *         if best.pos < end:             # <<<<<<<<<<<<<<
 *             feature_offset = X_feature_stride * best.feature
 *             partition_end = end
 */
  __pyx_t_6 = ((__pyx_v_best.pos < __pyx_v_end) != 0);
  if (__pyx_t_6) {

    /* "sklearn/tree/_splitter.pyx":473
 *         # Reorganize into samples[start:best.pos] + samples[best.pos:end]
 *         if best.pos < end:
 *             feature_offset = X_feature_stride * best.feature             # <<<<<<<<<<<<<<
 *             partition_end = end
 *             p = start
 */
    __pyx_v_feature_offset = (__pyx_v_X_feature_stride * __pyx_v_best.feature);

    /* "sklearn/tree/_splitter.pyx":474
 *         if best.pos < end:
 *             feature_offset = X_feature_stride * best.feature
 *             partition_end = end             # <<<<<<<<<<<<<<
 *             p = start
 * 
 */
    __pyx_v_partition_end = __pyx_v_end;

    /* "sklearn/tree/_splitter.pyx":475
 *             feature_offset = X_feature_stride * best.feature
 *             partition_end = end
 *             p = start             # <<<<<<<<<<<<<<
 * 
 *             while p < partition_end:
 */
    __pyx_v_p = __pyx_v_start;

    /* "sklearn/tree/_splitter.pyx":477
 *             p = start
 * 
 *             while p < partition_end:             # <<<<<<<<<<<<<<
 *                 if X[X_sample_stride * samples[p] + feature_offset] <= best.threshold:
 *                     p += 1
 */
    while (1) {
      __pyx_t_6 = ((__pyx_v_p < __pyx_v_partition_end) != 0);
      if (!__pyx_t_6) break;

      /* "sklearn/tree/_splitter.pyx":478
 * 
 *             while p < partition_end:
 *                 if X[X_sample_stride * samples[p] + feature_offset] <= best.threshold:             # <<<<<<<<<<<<<<
 *                     p += 1
 * 
 */
      __pyx_t_6 = (((__pyx_v_X[((__pyx_v_X_sample_stride * (__pyx_v_samples[__pyx_v_p])) + __pyx_v_feature_offset)]) <= __pyx_v_best.threshold) != 0);
      if (__pyx_t_6) {

        /* "sklearn/tree/_splitter.pyx":479
 *             while p < partition_end:
 *                 if X[X_sample_stride * samples[p] + feature_offset] <= best.threshold:
 *                     p += 1             # <<<<<<<<<<<<<<
 * 
 *                 else:
 */
        __pyx_v_p = (__pyx_v_p + 1);
        goto __pyx_L37;
      }
      /*else*/ {

        /* "sklearn/tree/_splitter.pyx":482
 * 
 *                 else:
 *                     partition_end -= 1             # <<<<<<<<<<<<<<
 * 
 *                     tmp = samples[partition_end]
 */
        __pyx_v_partition_end = (__pyx_v_partition_end - 1);

        /* "sklearn/tree/_splitter.pyx":484
 *                     partition_end -= 1
 * 
 *                     tmp = samples[partition_end]             # <<<<<<<<<<<<<<
 *                     samples[partition_end] = samples[p]
 *                     samples[p] = tmp
 */
        __pyx_v_tmp = (__pyx_v_samples[__pyx_v_partition_end]);

        /* "sklearn/tree/_splitter.pyx":485
 * 
 *                     tmp = samples[partition_end]
 *                     samples[partition_end] = samples[p]             # <<<<<<<<<<<<<<
 *                     samples[p] = tmp
 * 
 */
        (__pyx_v_samples[__pyx_v_partition_end]) = (__pyx_v_samples[__pyx_v_p]);

        /* "sklearn/tree/_splitter.pyx":486
 *                     tmp = samples[partition_end]
 *                     samples[partition_end] = samples[p]
 *                     samples[p] = tmp             # <<<<<<<<<<<<<<
 * 
 *             self.criterion.reset()
 */
        (__pyx_v_samples[__pyx_v_p]) = __pyx_v_tmp;
      }
      __pyx_L37:;
    }

    /* "sklearn/tree/_splitter.pyx":488
 *                     samples[p] = tmp
 * 
 *             self.criterion.reset()             # <<<<<<<<<<<<<<
 *             self.criterion.update(best.pos)
 *             best.improvement = self.criterion.impurity_improvement(impurity)
 */
    ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->reset(__pyx_v_self->__pyx_base.__pyx_base.criterion);

    /* "sklearn/tree/_splitter.pyx":489
 * 
 *             self.criterion.reset()
 *             self.criterion.update(best.pos)             # <<<<<<<<<<<<<<
 *             best.improvement = self.criterion.impurity_improvement(impurity)
 *             self.criterion.children_impurity(&best.impurity_left,
 */
    ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->update(__pyx_v_self->__pyx_base.__pyx_base.criterion, __pyx_v_best.pos);

    /* "sklearn/tree/_splitter.pyx":490
 *             self.criterion.reset()
 *             self.criterion.update(best.pos)
 *             best.improvement = self.criterion.impurity_improvement(impurity)             # <<<<<<<<<<<<<<
 *             self.criterion.children_impurity(&best.impurity_left,
 *                                              &best.impurity_right)
 */
    __pyx_v_best.improvement = ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->impurity_improvement(__pyx_v_self->__pyx_base.__pyx_base.criterion, __pyx_v_impurity);

    /* "sklearn/tree/_splitter.pyx":491
 *             self.criterion.update(best.pos)
 *             best.improvement = self.criterion.impurity_improvement(impurity)
 *             self.criterion.children_impurity(&best.impurity_left,             # <<<<<<<<<<<<<<
 *                                              &best.impurity_right)
 * 
 */
    ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->children_impurity(__pyx_v_self->__pyx_base.__pyx_base.criterion, (&__pyx_v_best.impurity_left), (&__pyx_v_best.impurity_right));
    goto __pyx_L34;
  }
  __pyx_L34:;

  /* "sklearn/tree/_splitter.pyx":495
 * 
 *         # Reset sample mask
 *         if self.presort == 1:             # <<<<<<<<<<<<<<
 *             for p in range(start, end):
 *                 sample_mask[samples[p]] = 0
 */
  __pyx_t_6 = ((__pyx_v_self->__pyx_base.__pyx_base.presort == 1) != 0);
  if (__pyx_t_6) {

    /* "sklearn/tree/_splitter.pyx":496
 *         # Reset sample mask
 *         if self.presort == 1:
 *             for p in range(start, end):             # <<<<<<<<<<<<<<
 *                 sample_mask[samples[p]] = 0
 * 
 */
    __pyx_t_7 = __pyx_v_end;
    for (__pyx_t_2 = __pyx_v_start; __pyx_t_2 < __pyx_t_7; __pyx_t_2+=1) {
      __pyx_v_p = __pyx_t_2;

      /* "sklearn/tree/_splitter.pyx":497
 *         if self.presort == 1:
 *             for p in range(start, end):
 *                 sample_mask[samples[p]] = 0             # <<<<<<<<<<<<<<
 * 
 *         # Respect invariant for constant features: the original order of
 */
      (__pyx_v_sample_mask[(__pyx_v_samples[__pyx_v_p])]) = 0;
    }
    goto __pyx_L38;
  }
  __pyx_L38:;

  /* "sklearn/tree/_splitter.pyx":502
 *         # element in features[:n_known_constants] must be preserved for sibling
 *         # and child nodes
 *         memcpy(features, constant_features, sizeof(SIZE_t) * n_known_constants)             # <<<<<<<<<<<<<<
 * 
 *         # Copy newly found constant features
 */
  memcpy(__pyx_v_features, __pyx_v_constant_features, ((sizeof(__pyx_t_7sklearn_4tree_9_splitter_SIZE_t)) * __pyx_v_n_known_constants));

  /* "sklearn/tree/_splitter.pyx":505
 * 
 *         # Copy newly found constant features
 *         memcpy(constant_features + n_known_constants,             # <<<<<<<<<<<<<<
 *                features + n_known_constants,
 *                sizeof(SIZE_t) * n_found_constants)
 */
  memcpy((__pyx_v_constant_features + __pyx_v_n_known_constants), (__pyx_v_features + __pyx_v_n_known_constants), ((sizeof(__pyx_t_7sklearn_4tree_9_splitter_SIZE_t)) * __pyx_v_n_found_constants));

  /* "sklearn/tree/_splitter.pyx":510
 * 
 *         # Return values
 *         split[0] = best             # <<<<<<<<<<<<<<
 *         n_constant_features[0] = n_total_constants
 * 
 */
  (__pyx_v_split[0]) = __pyx_v_best;

  /* "sklearn/tree/_splitter.pyx":511
 *         # Return values
 *         split[0] = best
 *         n_constant_features[0] = n_total_constants             # <<<<<<<<<<<<<<
 * 
 * 
 */
  (__pyx_v_n_constant_features[0]) = __pyx_v_n_total_constants;

  /* "sklearn/tree/_splitter.pyx":298
 *                                self.presort), self.__getstate__())
 * 
 *     cdef void node_split(self, double impurity, SplitRecord* split,             # <<<<<<<<<<<<<<
 *                          SIZE_t* n_constant_features) nogil:
 *         """Find the best split on node samples[start:end]."""
 */

  /* function exit code */
}

/* "sklearn/tree/_splitter.pyx":516
 * # Sort n-element arrays pointed to by Xf and samples, simultaneously,
 * # by the values in Xf. Algorithm: Introsort (Musser, SP&E, 1997).
 * cdef inline void sort(DTYPE_t* Xf, SIZE_t* samples, SIZE_t n) nogil:             # <<<<<<<<<<<<<<
 *     cdef int maxd = 2 * <int>log(n)
 *     introsort(Xf, samples, n, maxd)
 */

static CYTHON_INLINE void __pyx_f_7sklearn_4tree_9_splitter_sort(__pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_v_Xf, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_samples, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n) {
  int __pyx_v_maxd;

  /* "sklearn/tree/_splitter.pyx":517
 * # by the values in Xf. Algorithm: Introsort (Musser, SP&E, 1997).
 * cdef inline void sort(DTYPE_t* Xf, SIZE_t* samples, SIZE_t n) nogil:
 *     cdef int maxd = 2 * <int>log(n)             # <<<<<<<<<<<<<<
 *     introsort(Xf, samples, n, maxd)
 * 
 */
  __pyx_v_maxd = (2 * ((int)__pyx_f_7sklearn_4tree_6_utils_log(__pyx_v_n)));

  /* "sklearn/tree/_splitter.pyx":518
 * cdef inline void sort(DTYPE_t* Xf, SIZE_t* samples, SIZE_t n) nogil:
 *     cdef int maxd = 2 * <int>log(n)
 *     introsort(Xf, samples, n, maxd)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_f_7sklearn_4tree_9_splitter_introsort(__pyx_v_Xf, __pyx_v_samples, __pyx_v_n, __pyx_v_maxd);

  /* "sklearn/tree/_splitter.pyx":516
 * # Sort n-element arrays pointed to by Xf and samples, simultaneously,
 * # by the values in Xf. Algorithm: Introsort (Musser, SP&E, 1997).
 * cdef inline void sort(DTYPE_t* Xf, SIZE_t* samples, SIZE_t n) nogil:             # <<<<<<<<<<<<<<
 *     cdef int maxd = 2 * <int>log(n)
 *     introsort(Xf, samples, n, maxd)
 */

  /* function exit code */
}

/* "sklearn/tree/_splitter.pyx":521
 * 
 * 
 * cdef inline void swap(DTYPE_t* Xf, SIZE_t* samples, SIZE_t i, SIZE_t j) nogil:             # <<<<<<<<<<<<<<
 *     # Helper for sort
 *     Xf[i], Xf[j] = Xf[j], Xf[i]
 */

static CYTHON_INLINE void __pyx_f_7sklearn_4tree_9_splitter_swap(__pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_v_Xf, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_samples, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_i, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_j) {
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t __pyx_t_1;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t __pyx_t_2;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_3;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_4;

  /* "sklearn/tree/_splitter.pyx":523
 * cdef inline void swap(DTYPE_t* Xf, SIZE_t* samples, SIZE_t i, SIZE_t j) nogil:
 *     # Helper for sort
 *     Xf[i], Xf[j] = Xf[j], Xf[i]             # <<<<<<<<<<<<<<
 *     samples[i], samples[j] = samples[j], samples[i]
 * 
 */
  __pyx_t_1 = (__pyx_v_Xf[__pyx_v_j]);
  __pyx_t_2 = (__pyx_v_Xf[__pyx_v_i]);
  (__pyx_v_Xf[__pyx_v_i]) = __pyx_t_1;
  (__pyx_v_Xf[__pyx_v_j]) = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":524
 *     # Helper for sort
 *     Xf[i], Xf[j] = Xf[j], Xf[i]
 *     samples[i], samples[j] = samples[j], samples[i]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = (__pyx_v_samples[__pyx_v_j]);
  __pyx_t_4 = (__pyx_v_samples[__pyx_v_i]);
  (__pyx_v_samples[__pyx_v_i]) = __pyx_t_3;
  (__pyx_v_samples[__pyx_v_j]) = __pyx_t_4;

  /* "sklearn/tree/_splitter.pyx":521
 * 
 * 
 * cdef inline void swap(DTYPE_t* Xf, SIZE_t* samples, SIZE_t i, SIZE_t j) nogil:             # <<<<<<<<<<<<<<
 *     # Helper for sort
 *     Xf[i], Xf[j] = Xf[j], Xf[i]
 */

  /* function exit code */
}

/* "sklearn/tree/_splitter.pyx":527
 * 
 * 
 * cdef inline DTYPE_t median3(DTYPE_t* Xf, SIZE_t n) nogil:             # <<<<<<<<<<<<<<
 *     # Median of three pivot selection, after Bentley and McIlroy (1993).
 *     # Engineering a sort function. SP&E. Requires 8/3 comparisons on average.
 */

static CYTHON_INLINE __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t __pyx_f_7sklearn_4tree_9_splitter_median3(__pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_v_Xf, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n) {
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t __pyx_v_a;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t __pyx_v_b;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t __pyx_v_c;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t __pyx_r;
  int __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":530
 *     # Median of three pivot selection, after Bentley and McIlroy (1993).
 *     # Engineering a sort function. SP&E. Requires 8/3 comparisons on average.
 *     cdef DTYPE_t a = Xf[0], b = Xf[n / 2], c = Xf[n - 1]             # <<<<<<<<<<<<<<
 *     if a < b:
 *         if b < c:
 */
  __pyx_v_a = (__pyx_v_Xf[0]);
  __pyx_v_b = (__pyx_v_Xf[(__pyx_v_n / 2)]);
  __pyx_v_c = (__pyx_v_Xf[(__pyx_v_n - 1)]);

  /* "sklearn/tree/_splitter.pyx":531
 *     # Engineering a sort function. SP&E. Requires 8/3 comparisons on average.
 *     cdef DTYPE_t a = Xf[0], b = Xf[n / 2], c = Xf[n - 1]
 *     if a < b:             # <<<<<<<<<<<<<<
 *         if b < c:
 *             return b
 */
  __pyx_t_1 = ((__pyx_v_a < __pyx_v_b) != 0);
  if (__pyx_t_1) {

    /* "sklearn/tree/_splitter.pyx":532
 *     cdef DTYPE_t a = Xf[0], b = Xf[n / 2], c = Xf[n - 1]
 *     if a < b:
 *         if b < c:             # <<<<<<<<<<<<<<
 *             return b
 *         elif a < c:
 */
    __pyx_t_1 = ((__pyx_v_b < __pyx_v_c) != 0);
    if (__pyx_t_1) {

      /* "sklearn/tree/_splitter.pyx":533
 *     if a < b:
 *         if b < c:
 *             return b             # <<<<<<<<<<<<<<
 *         elif a < c:
 *             return c
 */
      __pyx_r = __pyx_v_b;
      goto __pyx_L0;
    }

    /* "sklearn/tree/_splitter.pyx":534
 *         if b < c:
 *             return b
 *         elif a < c:             # <<<<<<<<<<<<<<
 *             return c
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_a < __pyx_v_c) != 0);
    if (__pyx_t_1) {

      /* "sklearn/tree/_splitter.pyx":535
 *             return b
 *         elif a < c:
 *             return c             # <<<<<<<<<<<<<<
 *         else:
 *             return a
 */
      __pyx_r = __pyx_v_c;
      goto __pyx_L0;
    }
    /*else*/ {

      /* "sklearn/tree/_splitter.pyx":537
 *             return c
 *         else:
 *             return a             # <<<<<<<<<<<<<<
 *     elif b < c:
 *         if a < c:
 */
      __pyx_r = __pyx_v_a;
      goto __pyx_L0;
    }
  }

  /* "sklearn/tree/_splitter.pyx":538
 *         else:
 *             return a
 *     elif b < c:             # <<<<<<<<<<<<<<
 *         if a < c:
 *             return a
 */
  __pyx_t_1 = ((__pyx_v_b < __pyx_v_c) != 0);
  if (__pyx_t_1) {

    /* "sklearn/tree/_splitter.pyx":539
 *             return a
 *     elif b < c:
 *         if a < c:             # <<<<<<<<<<<<<<
 *             return a
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_a < __pyx_v_c) != 0);
    if (__pyx_t_1) {

      /* "sklearn/tree/_splitter.pyx":540
 *     elif b < c:
 *         if a < c:
 *             return a             # <<<<<<<<<<<<<<
 *         else:
 *             return c
 */
      __pyx_r = __pyx_v_a;
      goto __pyx_L0;
    }
    /*else*/ {

      /* "sklearn/tree/_splitter.pyx":542
 *             return a
 *         else:
 *             return c             # <<<<<<<<<<<<<<
 *     else:
 *         return b
 */
      __pyx_r = __pyx_v_c;
      goto __pyx_L0;
    }
  }
  /*else*/ {

    /* "sklearn/tree/_splitter.pyx":544
 *             return c
 *     else:
 *         return b             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_r = __pyx_v_b;
    goto __pyx_L0;
  }

  /* "sklearn/tree/_splitter.pyx":527
 * 
 * 
 * cdef inline DTYPE_t median3(DTYPE_t* Xf, SIZE_t n) nogil:             # <<<<<<<<<<<<<<
 *     # Median of three pivot selection, after Bentley and McIlroy (1993).
 *     # Engineering a sort function. SP&E. Requires 8/3 comparisons on average.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "sklearn/tree/_splitter.pyx":549
 * # Introsort with median of 3 pivot selection and 3-way partition function
 * # (robust to repeated elements, e.g. lots of zero features).
 * cdef void introsort(DTYPE_t* Xf, SIZE_t *samples, SIZE_t n, int maxd) nogil:             # <<<<<<<<<<<<<<
 *     cdef DTYPE_t pivot
 *     cdef SIZE_t i, l, r
 */

static void __pyx_f_7sklearn_4tree_9_splitter_introsort(__pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_v_Xf, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_samples, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n, int __pyx_v_maxd) {
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t __pyx_v_pivot;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_i;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_l;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_r;
  int __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":553
 *     cdef SIZE_t i, l, r
 * 
 *     while n > 1:             # <<<<<<<<<<<<<<
 *         if maxd <= 0:   # max depth limit exceeded ("gone quadratic")
 *             heapsort(Xf, samples, n)
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_n > 1) != 0);
    if (!__pyx_t_1) break;

    /* "sklearn/tree/_splitter.pyx":554
 * 
 *     while n > 1:
 *         if maxd <= 0:   # max depth limit exceeded ("gone quadratic")             # <<<<<<<<<<<<<<
 *             heapsort(Xf, samples, n)
 *             return
 */
    __pyx_t_1 = ((__pyx_v_maxd <= 0) != 0);
    if (__pyx_t_1) {

      /* "sklearn/tree/_splitter.pyx":555
 *     while n > 1:
 *         if maxd <= 0:   # max depth limit exceeded ("gone quadratic")
 *             heapsort(Xf, samples, n)             # <<<<<<<<<<<<<<
 *             return
 *         maxd -= 1
 */
      __pyx_f_7sklearn_4tree_9_splitter_heapsort(__pyx_v_Xf, __pyx_v_samples, __pyx_v_n);

      /* "sklearn/tree/_splitter.pyx":556
 *         if maxd <= 0:   # max depth limit exceeded ("gone quadratic")
 *             heapsort(Xf, samples, n)
 *             return             # <<<<<<<<<<<<<<
 *         maxd -= 1
 * 
 */
      goto __pyx_L0;
    }

    /* "sklearn/tree/_splitter.pyx":557
 *             heapsort(Xf, samples, n)
 *             return
 *         maxd -= 1             # <<<<<<<<<<<<<<
 * 
 *         pivot = median3(Xf, n)
 */
    __pyx_v_maxd = (__pyx_v_maxd - 1);

    /* "sklearn/tree/_splitter.pyx":559
 *         maxd -= 1
 * 
 *         pivot = median3(Xf, n)             # <<<<<<<<<<<<<<
 * 
 *         # Three-way partition.
 */
    __pyx_v_pivot = __pyx_f_7sklearn_4tree_9_splitter_median3(__pyx_v_Xf, __pyx_v_n);

    /* "sklearn/tree/_splitter.pyx":562
 * 
 *         # Three-way partition.
 *         i = l = 0             # <<<<<<<<<<<<<<
 *         r = n
 *         while i < r:
 */
    __pyx_v_i = 0;
    __pyx_v_l = 0;

    /* "sklearn/tree/_splitter.pyx":563
 *         # Three-way partition.
 *         i = l = 0
 *         r = n             # <<<<<<<<<<<<<<
 *         while i < r:
 *             if Xf[i] < pivot:
 */
    __pyx_v_r = __pyx_v_n;

    /* "sklearn/tree/_splitter.pyx":564
 *         i = l = 0
 *         r = n
 *         while i < r:             # <<<<<<<<<<<<<<
 *             if Xf[i] < pivot:
 *                 swap(Xf, samples, i, l)
 */
    while (1) {
      __pyx_t_1 = ((__pyx_v_i < __pyx_v_r) != 0);
      if (!__pyx_t_1) break;

      /* "sklearn/tree/_splitter.pyx":565
 *         r = n
 *         while i < r:
 *             if Xf[i] < pivot:             # <<<<<<<<<<<<<<
 *                 swap(Xf, samples, i, l)
 *                 i += 1
 */
      __pyx_t_1 = (((__pyx_v_Xf[__pyx_v_i]) < __pyx_v_pivot) != 0);
      if (__pyx_t_1) {

        /* "sklearn/tree/_splitter.pyx":566
 *         while i < r:
 *             if Xf[i] < pivot:
 *                 swap(Xf, samples, i, l)             # <<<<<<<<<<<<<<
 *                 i += 1
 *                 l += 1
 */
        __pyx_f_7sklearn_4tree_9_splitter_swap(__pyx_v_Xf, __pyx_v_samples, __pyx_v_i, __pyx_v_l);

        /* "sklearn/tree/_splitter.pyx":567
 *             if Xf[i] < pivot:
 *                 swap(Xf, samples, i, l)
 *                 i += 1             # <<<<<<<<<<<<<<
 *                 l += 1
 *             elif Xf[i] > pivot:
 */
        __pyx_v_i = (__pyx_v_i + 1);

        /* "sklearn/tree/_splitter.pyx":568
 *                 swap(Xf, samples, i, l)
 *                 i += 1
 *                 l += 1             # <<<<<<<<<<<<<<
 *             elif Xf[i] > pivot:
 *                 r -= 1
 */
        __pyx_v_l = (__pyx_v_l + 1);
        goto __pyx_L8;
      }

      /* "sklearn/tree/_splitter.pyx":569
 *                 i += 1
 *                 l += 1
 *             elif Xf[i] > pivot:             # <<<<<<<<<<<<<<
 *                 r -= 1
 *                 swap(Xf, samples, i, r)
 */
      __pyx_t_1 = (((__pyx_v_Xf[__pyx_v_i]) > __pyx_v_pivot) != 0);
      if (__pyx_t_1) {

        /* "sklearn/tree/_splitter.pyx":570
 *                 l += 1
 *             elif Xf[i] > pivot:
 *                 r -= 1             # <<<<<<<<<<<<<<
 *                 swap(Xf, samples, i, r)
 *             else:
 */
        __pyx_v_r = (__pyx_v_r - 1);

        /* "sklearn/tree/_splitter.pyx":571
 *             elif Xf[i] > pivot:
 *                 r -= 1
 *                 swap(Xf, samples, i, r)             # <<<<<<<<<<<<<<
 *             else:
 *                 i += 1
 */
        __pyx_f_7sklearn_4tree_9_splitter_swap(__pyx_v_Xf, __pyx_v_samples, __pyx_v_i, __pyx_v_r);
        goto __pyx_L8;
      }
      /*else*/ {

        /* "sklearn/tree/_splitter.pyx":573
 *                 swap(Xf, samples, i, r)
 *             else:
 *                 i += 1             # <<<<<<<<<<<<<<
 * 
 *         introsort(Xf, samples, l, maxd)
 */
        __pyx_v_i = (__pyx_v_i + 1);
      }
      __pyx_L8:;
    }

    /* "sklearn/tree/_splitter.pyx":575
 *                 i += 1
 * 
 *         introsort(Xf, samples, l, maxd)             # <<<<<<<<<<<<<<
 *         Xf += r
 *         samples += r
 */
    __pyx_f_7sklearn_4tree_9_splitter_introsort(__pyx_v_Xf, __pyx_v_samples, __pyx_v_l, __pyx_v_maxd);

    /* "sklearn/tree/_splitter.pyx":576
 * 
 *         introsort(Xf, samples, l, maxd)
 *         Xf += r             # <<<<<<<<<<<<<<
 *         samples += r
 *         n -= r
 */
    __pyx_v_Xf = (__pyx_v_Xf + __pyx_v_r);

    /* "sklearn/tree/_splitter.pyx":577
 *         introsort(Xf, samples, l, maxd)
 *         Xf += r
 *         samples += r             # <<<<<<<<<<<<<<
 *         n -= r
 * 
 */
    __pyx_v_samples = (__pyx_v_samples + __pyx_v_r);

    /* "sklearn/tree/_splitter.pyx":578
 *         Xf += r
 *         samples += r
 *         n -= r             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_v_n = (__pyx_v_n - __pyx_v_r);
  }

  /* "sklearn/tree/_splitter.pyx":549
 * # Introsort with median of 3 pivot selection and 3-way partition function
 * # (robust to repeated elements, e.g. lots of zero features).
 * cdef void introsort(DTYPE_t* Xf, SIZE_t *samples, SIZE_t n, int maxd) nogil:             # <<<<<<<<<<<<<<
 *     cdef DTYPE_t pivot
 *     cdef SIZE_t i, l, r
 */

  /* function exit code */
  __pyx_L0:;
}

/* "sklearn/tree/_splitter.pyx":581
 * 
 * 
 * cdef inline void sift_down(DTYPE_t* Xf, SIZE_t* samples,             # <<<<<<<<<<<<<<
 *                            SIZE_t start, SIZE_t end) nogil:
 *     # Restore heap order in Xf[start:end] by moving the max element to start.
 */

static CYTHON_INLINE void __pyx_f_7sklearn_4tree_9_splitter_sift_down(__pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_v_Xf, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_samples, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_start, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_end) {
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_child;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_maxind;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_root;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":586
 *     cdef SIZE_t child, maxind, root
 * 
 *     root = start             # <<<<<<<<<<<<<<
 *     while True:
 *         child = root * 2 + 1
 */
  __pyx_v_root = __pyx_v_start;

  /* "sklearn/tree/_splitter.pyx":587
 * 
 *     root = start
 *     while True:             # <<<<<<<<<<<<<<
 *         child = root * 2 + 1
 * 
 */
  while (1) {

    /* "sklearn/tree/_splitter.pyx":588
 *     root = start
 *     while True:
 *         child = root * 2 + 1             # <<<<<<<<<<<<<<
 * 
 *         # find max of root, left child, right child
 */
    __pyx_v_child = ((__pyx_v_root * 2) + 1);

    /* "sklearn/tree/_splitter.pyx":591
 * 
 *         # find max of root, left child, right child
 *         maxind = root             # <<<<<<<<<<<<<<
 *         if child < end and Xf[maxind] < Xf[child]:
 *             maxind = child
 */
    __pyx_v_maxind = __pyx_v_root;

    /* "sklearn/tree/_splitter.pyx":592
 *         # find max of root, left child, right child
 *         maxind = root
 *         if child < end and Xf[maxind] < Xf[child]:             # <<<<<<<<<<<<<<
 *             maxind = child
 *         if child + 1 < end and Xf[maxind] < Xf[child + 1]:
 */
    __pyx_t_2 = ((__pyx_v_child < __pyx_v_end) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_2 = (((__pyx_v_Xf[__pyx_v_maxind]) < (__pyx_v_Xf[__pyx_v_child])) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_1) {

      /* "sklearn/tree/_splitter.pyx":593
 *         maxind = root
 *         if child < end and Xf[maxind] < Xf[child]:
 *             maxind = child             # <<<<<<<<<<<<<<
 *         if child + 1 < end and Xf[maxind] < Xf[child + 1]:
 *             maxind = child + 1
 */
      __pyx_v_maxind = __pyx_v_child;
      goto __pyx_L5;
    }
    __pyx_L5:;

    /* "sklearn/tree/_splitter.pyx":594
 *         if child < end and Xf[maxind] < Xf[child]:
 *             maxind = child
 *         if child + 1 < end and Xf[maxind] < Xf[child + 1]:             # <<<<<<<<<<<<<<
 *             maxind = child + 1
 * 
 */
    __pyx_t_2 = (((__pyx_v_child + 1) < __pyx_v_end) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L9_bool_binop_done;
    }
    __pyx_t_2 = (((__pyx_v_Xf[__pyx_v_maxind]) < (__pyx_v_Xf[(__pyx_v_child + 1)])) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L9_bool_binop_done:;
    if (__pyx_t_1) {

      /* "sklearn/tree/_splitter.pyx":595
 *             maxind = child
 *         if child + 1 < end and Xf[maxind] < Xf[child + 1]:
 *             maxind = child + 1             # <<<<<<<<<<<<<<
 * 
 *         if maxind == root:
 */
      __pyx_v_maxind = (__pyx_v_child + 1);
      goto __pyx_L8;
    }
    __pyx_L8:;

    /* "sklearn/tree/_splitter.pyx":597
 *             maxind = child + 1
 * 
 *         if maxind == root:             # <<<<<<<<<<<<<<
 *             break
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_maxind == __pyx_v_root) != 0);
    if (__pyx_t_1) {

      /* "sklearn/tree/_splitter.pyx":598
 * 
 *         if maxind == root:
 *             break             # <<<<<<<<<<<<<<
 *         else:
 *             swap(Xf, samples, root, maxind)
 */
      goto __pyx_L4_break;
    }
    /*else*/ {

      /* "sklearn/tree/_splitter.pyx":600
 *             break
 *         else:
 *             swap(Xf, samples, root, maxind)             # <<<<<<<<<<<<<<
 *             root = maxind
 * 
 */
      __pyx_f_7sklearn_4tree_9_splitter_swap(__pyx_v_Xf, __pyx_v_samples, __pyx_v_root, __pyx_v_maxind);

      /* "sklearn/tree/_splitter.pyx":601
 *         else:
 *             swap(Xf, samples, root, maxind)
 *             root = maxind             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_v_root = __pyx_v_maxind;
    }
  }
  __pyx_L4_break:;

  /* "sklearn/tree/_splitter.pyx":581
 * 
 * 
 * cdef inline void sift_down(DTYPE_t* Xf, SIZE_t* samples,             # <<<<<<<<<<<<<<
 *                            SIZE_t start, SIZE_t end) nogil:
 *     # Restore heap order in Xf[start:end] by moving the max element to start.
 */

  /* function exit code */
}

/* "sklearn/tree/_splitter.pyx":604
 * 
 * 
 * cdef void heapsort(DTYPE_t* Xf, SIZE_t* samples, SIZE_t n) nogil:             # <<<<<<<<<<<<<<
 *     cdef SIZE_t start, end
 * 
 */

static void __pyx_f_7sklearn_4tree_9_splitter_heapsort(__pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_v_Xf, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_samples, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n) {
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_start;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_end;
  int __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":608
 * 
 *     # heapify
 *     start = (n - 2) / 2             # <<<<<<<<<<<<<<
 *     end = n
 *     while True:
 */
  __pyx_v_start = ((__pyx_v_n - 2) / 2);

  /* "sklearn/tree/_splitter.pyx":609
 *     # heapify
 *     start = (n - 2) / 2
 *     end = n             # <<<<<<<<<<<<<<
 *     while True:
 *         sift_down(Xf, samples, start, end)
 */
  __pyx_v_end = __pyx_v_n;

  /* "sklearn/tree/_splitter.pyx":610
 *     start = (n - 2) / 2
 *     end = n
 *     while True:             # <<<<<<<<<<<<<<
 *         sift_down(Xf, samples, start, end)
 *         if start == 0:
 */
  while (1) {

    /* "sklearn/tree/_splitter.pyx":611
 *     end = n
 *     while True:
 *         sift_down(Xf, samples, start, end)             # <<<<<<<<<<<<<<
 *         if start == 0:
 *             break
 */
    __pyx_f_7sklearn_4tree_9_splitter_sift_down(__pyx_v_Xf, __pyx_v_samples, __pyx_v_start, __pyx_v_end);

    /* "sklearn/tree/_splitter.pyx":612
 *     while True:
 *         sift_down(Xf, samples, start, end)
 *         if start == 0:             # <<<<<<<<<<<<<<
 *             break
 *         start -= 1
 */
    __pyx_t_1 = ((__pyx_v_start == 0) != 0);
    if (__pyx_t_1) {

      /* "sklearn/tree/_splitter.pyx":613
 *         sift_down(Xf, samples, start, end)
 *         if start == 0:
 *             break             # <<<<<<<<<<<<<<
 *         start -= 1
 * 
 */
      goto __pyx_L4_break;
    }

    /* "sklearn/tree/_splitter.pyx":614
 *         if start == 0:
 *             break
 *         start -= 1             # <<<<<<<<<<<<<<
 * 
 *     # sort by shrinking the heap, putting the max element immediately after it
 */
    __pyx_v_start = (__pyx_v_start - 1);
  }
  __pyx_L4_break:;

  /* "sklearn/tree/_splitter.pyx":617
 * 
 *     # sort by shrinking the heap, putting the max element immediately after it
 *     end = n - 1             # <<<<<<<<<<<<<<
 *     while end > 0:
 *         swap(Xf, samples, 0, end)
 */
  __pyx_v_end = (__pyx_v_n - 1);

  /* "sklearn/tree/_splitter.pyx":618
 *     # sort by shrinking the heap, putting the max element immediately after it
 *     end = n - 1
 *     while end > 0:             # <<<<<<<<<<<<<<
 *         swap(Xf, samples, 0, end)
 *         sift_down(Xf, samples, 0, end)
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_end > 0) != 0);
    if (!__pyx_t_1) break;

    /* "sklearn/tree/_splitter.pyx":619
 *     end = n - 1
 *     while end > 0:
 *         swap(Xf, samples, 0, end)             # <<<<<<<<<<<<<<
 *         sift_down(Xf, samples, 0, end)
 *         end = end - 1
 */
    __pyx_f_7sklearn_4tree_9_splitter_swap(__pyx_v_Xf, __pyx_v_samples, 0, __pyx_v_end);

    /* "sklearn/tree/_splitter.pyx":620
 *     while end > 0:
 *         swap(Xf, samples, 0, end)
 *         sift_down(Xf, samples, 0, end)             # <<<<<<<<<<<<<<
 *         end = end - 1
 * 
 */
    __pyx_f_7sklearn_4tree_9_splitter_sift_down(__pyx_v_Xf, __pyx_v_samples, 0, __pyx_v_end);

    /* "sklearn/tree/_splitter.pyx":621
 *         swap(Xf, samples, 0, end)
 *         sift_down(Xf, samples, 0, end)
 *         end = end - 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_v_end = (__pyx_v_end - 1);
  }

  /* "sklearn/tree/_splitter.pyx":604
 * 
 * 
 * cdef void heapsort(DTYPE_t* Xf, SIZE_t* samples, SIZE_t n) nogil:             # <<<<<<<<<<<<<<
 *     cdef SIZE_t start, end
 * 
 */

  /* function exit code */
}

/* "sklearn/tree/_splitter.pyx":626
 * cdef class RandomSplitter(BaseDenseSplitter):
 *     """Splitter for finding the best random split."""
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return (RandomSplitter, (self.criterion,
 *                                  self.max_features,
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_4tree_9_splitter_14RandomSplitter_1__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7sklearn_4tree_9_splitter_14RandomSplitter_1__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_4tree_9_splitter_14RandomSplitter___reduce__(((struct __pyx_obj_7sklearn_4tree_9_splitter_RandomSplitter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_4tree_9_splitter_14RandomSplitter___reduce__(struct __pyx_obj_7sklearn_4tree_9_splitter_RandomSplitter *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "sklearn/tree/_splitter.pyx":627
 *     """Splitter for finding the best random split."""
 *     def __reduce__(self):
 *         return (RandomSplitter, (self.criterion,             # <<<<<<<<<<<<<<
 *                                  self.max_features,
 *                                  self.min_samples_leaf,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "sklearn/tree/_splitter.pyx":628
 *     def __reduce__(self):
 *         return (RandomSplitter, (self.criterion,
 *                                  self.max_features,             # <<<<<<<<<<<<<<
 *                                  self.min_samples_leaf,
 *                                  self.min_weight_leaf,
 */
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->__pyx_base.__pyx_base.max_features); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 628; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);

  /* "sklearn/tree/_splitter.pyx":629
 *         return (RandomSplitter, (self.criterion,
 *                                  self.max_features,
 *                                  self.min_samples_leaf,             # <<<<<<<<<<<<<<
 *                                  self.min_weight_leaf,
 *                                  self.random_state,
 */
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->__pyx_base.__pyx_base.min_samples_leaf); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 629; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);

  /* "sklearn/tree/_splitter.pyx":630
 *                                  self.max_features,
 *                                  self.min_samples_leaf,
 *                                  self.min_weight_leaf,             # <<<<<<<<<<<<<<
 *                                  self.random_state,
 *                                  self.presort), self.__getstate__())
 */
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.min_weight_leaf); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 630; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);

  /* "sklearn/tree/_splitter.pyx":632
 *                                  self.min_weight_leaf,
 *                                  self.random_state,
 *                                  self.presort), self.__getstate__())             # <<<<<<<<<<<<<<
 * 
 *     cdef void node_split(self, double impurity, SplitRecord* split,
 */
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.presort); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 632; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);

  /* "sklearn/tree/_splitter.pyx":627
 *     """Splitter for finding the best random split."""
 *     def __reduce__(self):
 *         return (RandomSplitter, (self.criterion,             # <<<<<<<<<<<<<<
 *                                  self.max_features,
 *                                  self.min_samples_leaf,
 */
  __pyx_t_5 = PyTuple_New(6); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 627; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.criterion));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.criterion));
  PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.criterion));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.random_state);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.random_state);
  PyTuple_SET_ITEM(__pyx_t_5, 4, __pyx_v_self->__pyx_base.__pyx_base.random_state);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 5, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;

  /* "sklearn/tree/_splitter.pyx":632
 *                                  self.min_weight_leaf,
 *                                  self.random_state,
 *                                  self.presort), self.__getstate__())             # <<<<<<<<<<<<<<
 * 
 *     cdef void node_split(self, double impurity, SplitRecord* split,
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getstate); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 632; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 632; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 632; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sklearn/tree/_splitter.pyx":627
 *     """Splitter for finding the best random split."""
 *     def __reduce__(self):
 *         return (RandomSplitter, (self.criterion,             # <<<<<<<<<<<<<<
 *                                  self.max_features,
 *                                  self.min_samples_leaf,
 */
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 627; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)((PyObject*)__pyx_ptype_7sklearn_4tree_9_splitter_RandomSplitter)));
  __Pyx_GIVEREF(((PyObject *)((PyObject*)__pyx_ptype_7sklearn_4tree_9_splitter_RandomSplitter)));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)((PyObject*)__pyx_ptype_7sklearn_4tree_9_splitter_RandomSplitter)));
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_4);
  __pyx_t_5 = 0;
  __pyx_t_4 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "sklearn/tree/_splitter.pyx":626
 * cdef class RandomSplitter(BaseDenseSplitter):
 *     """Splitter for finding the best random split."""
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return (RandomSplitter, (self.criterion,
 *                                  self.max_features,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("sklearn.tree._splitter.RandomSplitter.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/tree/_splitter.pyx":634
 *                                  self.presort), self.__getstate__())
 * 
 *     cdef void node_split(self, double impurity, SplitRecord* split,             # <<<<<<<<<<<<<<
 *                          SIZE_t* n_constant_features) nogil:
 *         """Find the best random split on node samples[start:end]."""
 */

static void __pyx_f_7sklearn_4tree_9_splitter_14RandomSplitter_node_split(struct __pyx_obj_7sklearn_4tree_9_splitter_RandomSplitter *__pyx_v_self, double __pyx_v_impurity, struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord *__pyx_v_split, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_n_constant_features) {
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_start;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_end;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_features;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_constant_features;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_features;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_v_X;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_v_Xf;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_X_sample_stride;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_X_feature_stride;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_max_features;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_min_samples_leaf;
  double __pyx_v_min_weight_leaf;
  __pyx_t_7sklearn_4tree_9_splitter_UINT32_t *__pyx_v_random_state;
  struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord __pyx_v_best;
  struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord __pyx_v_current;
  double __pyx_v_current_proxy_improvement;
  double __pyx_v_best_proxy_improvement;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_f_i;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_f_j;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_p;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_tmp;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_feature_stride;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_found_constants;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_drawn_constants;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_known_constants;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_total_constants;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_visited_features;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t __pyx_v_min_feature_value;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t __pyx_v_max_feature_value;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t __pyx_v_current_feature_value;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_partition_end;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_t_1;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_2;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_t_3;
  double __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_7;

  /* "sklearn/tree/_splitter.pyx":638
 *         """Find the best random split on node samples[start:end]."""
 *         # Draw random splits and pick the best
 *         cdef SIZE_t* samples = self.samples             # <<<<<<<<<<<<<<
 *         cdef SIZE_t start = self.start
 *         cdef SIZE_t end = self.end
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.__pyx_base.samples;
  __pyx_v_samples = __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":639
 *         # Draw random splits and pick the best
 *         cdef SIZE_t* samples = self.samples
 *         cdef SIZE_t start = self.start             # <<<<<<<<<<<<<<
 *         cdef SIZE_t end = self.end
 * 
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.start;
  __pyx_v_start = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":640
 *         cdef SIZE_t* samples = self.samples
 *         cdef SIZE_t start = self.start
 *         cdef SIZE_t end = self.end             # <<<<<<<<<<<<<<
 * 
 *         cdef SIZE_t* features = self.features
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.end;
  __pyx_v_end = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":642
 *         cdef SIZE_t end = self.end
 * 
 *         cdef SIZE_t* features = self.features             # <<<<<<<<<<<<<<
 *         cdef SIZE_t* constant_features = self.constant_features
 *         cdef SIZE_t n_features = self.n_features
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.__pyx_base.features;
  __pyx_v_features = __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":643
 * 
 *         cdef SIZE_t* features = self.features
 *         cdef SIZE_t* constant_features = self.constant_features             # <<<<<<<<<<<<<<
 *         cdef SIZE_t n_features = self.n_features
 * 
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.__pyx_base.constant_features;
  __pyx_v_constant_features = __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":644
 *         cdef SIZE_t* features = self.features
 *         cdef SIZE_t* constant_features = self.constant_features
 *         cdef SIZE_t n_features = self.n_features             # <<<<<<<<<<<<<<
 * 
 *         cdef DTYPE_t* X = self.X
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.n_features;
  __pyx_v_n_features = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":646
 *         cdef SIZE_t n_features = self.n_features
 * 
 *         cdef DTYPE_t* X = self.X             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t* Xf = self.feature_values
 *         cdef SIZE_t X_sample_stride = self.X_sample_stride
 */
  __pyx_t_3 = __pyx_v_self->__pyx_base.X;
  __pyx_v_X = __pyx_t_3;

  /* "sklearn/tree/_splitter.pyx":647
 * 
 *         cdef DTYPE_t* X = self.X
 *         cdef DTYPE_t* Xf = self.feature_values             # <<<<<<<<<<<<<<
 *         cdef SIZE_t X_sample_stride = self.X_sample_stride
 *         cdef SIZE_t X_feature_stride = self.X_feature_stride
 */
  __pyx_t_3 = __pyx_v_self->__pyx_base.__pyx_base.feature_values;
  __pyx_v_Xf = __pyx_t_3;

  /* "sklearn/tree/_splitter.pyx":648
 *         cdef DTYPE_t* X = self.X
 *         cdef DTYPE_t* Xf = self.feature_values
 *         cdef SIZE_t X_sample_stride = self.X_sample_stride             # <<<<<<<<<<<<<<
 *         cdef SIZE_t X_feature_stride = self.X_feature_stride
 *         cdef SIZE_t max_features = self.max_features
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.X_sample_stride;
  __pyx_v_X_sample_stride = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":649
 *         cdef DTYPE_t* Xf = self.feature_values
 *         cdef SIZE_t X_sample_stride = self.X_sample_stride
 *         cdef SIZE_t X_feature_stride = self.X_feature_stride             # <<<<<<<<<<<<<<
 *         cdef SIZE_t max_features = self.max_features
 *         cdef SIZE_t min_samples_leaf = self.min_samples_leaf
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.X_feature_stride;
  __pyx_v_X_feature_stride = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":650
 *         cdef SIZE_t X_sample_stride = self.X_sample_stride
 *         cdef SIZE_t X_feature_stride = self.X_feature_stride
 *         cdef SIZE_t max_features = self.max_features             # <<<<<<<<<<<<<<
 *         cdef SIZE_t min_samples_leaf = self.min_samples_leaf
 *         cdef double min_weight_leaf = self.min_weight_leaf
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.max_features;
  __pyx_v_max_features = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":651
 *         cdef SIZE_t X_feature_stride = self.X_feature_stride
 *         cdef SIZE_t max_features = self.max_features
 *         cdef SIZE_t min_samples_leaf = self.min_samples_leaf             # <<<<<<<<<<<<<<
 *         cdef double min_weight_leaf = self.min_weight_leaf
 *         cdef UINT32_t* random_state = &self.rand_r_state
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.min_samples_leaf;
  __pyx_v_min_samples_leaf = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":652
 *         cdef SIZE_t max_features = self.max_features
 *         cdef SIZE_t min_samples_leaf = self.min_samples_leaf
 *         cdef double min_weight_leaf = self.min_weight_leaf             # <<<<<<<<<<<<<<
 *         cdef UINT32_t* random_state = &self.rand_r_state
 * 
 */
  __pyx_t_4 = __pyx_v_self->__pyx_base.__pyx_base.min_weight_leaf;
  __pyx_v_min_weight_leaf = __pyx_t_4;

  /* "sklearn/tree/_splitter.pyx":653
 *         cdef SIZE_t min_samples_leaf = self.min_samples_leaf
 *         cdef double min_weight_leaf = self.min_weight_leaf
 *         cdef UINT32_t* random_state = &self.rand_r_state             # <<<<<<<<<<<<<<
 * 
 *         cdef SplitRecord best, current
 */
  __pyx_v_random_state = (&__pyx_v_self->__pyx_base.__pyx_base.rand_r_state);

  /* "sklearn/tree/_splitter.pyx":656
 * 
 *         cdef SplitRecord best, current
 *         cdef double current_proxy_improvement = - INFINITY             # <<<<<<<<<<<<<<
 *         cdef double best_proxy_improvement = - INFINITY
 * 
 */
  __pyx_v_current_proxy_improvement = (-__pyx_v_7sklearn_4tree_9_splitter_INFINITY);

  /* "sklearn/tree/_splitter.pyx":657
 *         cdef SplitRecord best, current
 *         cdef double current_proxy_improvement = - INFINITY
 *         cdef double best_proxy_improvement = - INFINITY             # <<<<<<<<<<<<<<
 * 
 *         cdef SIZE_t f_i = n_features
 */
  __pyx_v_best_proxy_improvement = (-__pyx_v_7sklearn_4tree_9_splitter_INFINITY);

  /* "sklearn/tree/_splitter.pyx":659
 *         cdef double best_proxy_improvement = - INFINITY
 * 
 *         cdef SIZE_t f_i = n_features             # <<<<<<<<<<<<<<
 *         cdef SIZE_t f_j
 *         cdef SIZE_t p
 */
  __pyx_v_f_i = __pyx_v_n_features;

  /* "sklearn/tree/_splitter.pyx":665
 *         cdef SIZE_t feature_stride
 *         # Number of features discovered to be constant during the split search
 *         cdef SIZE_t n_found_constants = 0             # <<<<<<<<<<<<<<
 *         # Number of features known to be constant and drawn without replacement
 *         cdef SIZE_t n_drawn_constants = 0
 */
  __pyx_v_n_found_constants = 0;

  /* "sklearn/tree/_splitter.pyx":667
 *         cdef SIZE_t n_found_constants = 0
 *         # Number of features known to be constant and drawn without replacement
 *         cdef SIZE_t n_drawn_constants = 0             # <<<<<<<<<<<<<<
 *         cdef SIZE_t n_known_constants = n_constant_features[0]
 *         # n_total_constants = n_known_constants + n_found_constants
 */
  __pyx_v_n_drawn_constants = 0;

  /* "sklearn/tree/_splitter.pyx":668
 *         # Number of features known to be constant and drawn without replacement
 *         cdef SIZE_t n_drawn_constants = 0
 *         cdef SIZE_t n_known_constants = n_constant_features[0]             # <<<<<<<<<<<<<<
 *         # n_total_constants = n_known_constants + n_found_constants
 *         cdef SIZE_t n_total_constants = n_known_constants
 */
  __pyx_v_n_known_constants = (__pyx_v_n_constant_features[0]);

  /* "sklearn/tree/_splitter.pyx":670
 *         cdef SIZE_t n_known_constants = n_constant_features[0]
 *         # n_total_constants = n_known_constants + n_found_constants
 *         cdef SIZE_t n_total_constants = n_known_constants             # <<<<<<<<<<<<<<
 *         cdef SIZE_t n_visited_features = 0
 *         cdef DTYPE_t min_feature_value
 */
  __pyx_v_n_total_constants = __pyx_v_n_known_constants;

  /* "sklearn/tree/_splitter.pyx":671
 *         # n_total_constants = n_known_constants + n_found_constants
 *         cdef SIZE_t n_total_constants = n_known_constants
 *         cdef SIZE_t n_visited_features = 0             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t min_feature_value
 *         cdef DTYPE_t max_feature_value
 */
  __pyx_v_n_visited_features = 0;

  /* "sklearn/tree/_splitter.pyx":677
 *         cdef SIZE_t partition_end
 * 
 *         _init_split(&best, end)             # <<<<<<<<<<<<<<
 * 
 *         # Sample up to max_features without replacement using a
 */
  __pyx_f_7sklearn_4tree_9_splitter__init_split((&__pyx_v_best), __pyx_v_end);

  /* "sklearn/tree/_splitter.pyx":688
 *         # newly discovered constant features to spare computation on descendant
 *         # nodes.
 *         while (f_i > n_total_constants and  # Stop early if remaining features             # <<<<<<<<<<<<<<
 *                                             # are constant
 *                 (n_visited_features < max_features or
 */
  while (1) {
    __pyx_t_6 = ((__pyx_v_f_i > __pyx_v_n_total_constants) != 0);
    if (__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L5_bool_binop_done;
    }

    /* "sklearn/tree/_splitter.pyx":690
 *         while (f_i > n_total_constants and  # Stop early if remaining features
 *                                             # are constant
 *                 (n_visited_features < max_features or             # <<<<<<<<<<<<<<
 *                  # At least one drawn features must be non constant
 *                  n_visited_features <= n_found_constants + n_drawn_constants)):
 */
    __pyx_t_6 = ((__pyx_v_n_visited_features < __pyx_v_max_features) != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L5_bool_binop_done;
    }

    /* "sklearn/tree/_splitter.pyx":692
 *                 (n_visited_features < max_features or
 *                  # At least one drawn features must be non constant
 *                  n_visited_features <= n_found_constants + n_drawn_constants)):             # <<<<<<<<<<<<<<
 *             n_visited_features += 1
 * 
 */
    __pyx_t_6 = ((__pyx_v_n_visited_features <= (__pyx_v_n_found_constants + __pyx_v_n_drawn_constants)) != 0);
    __pyx_t_5 = __pyx_t_6;
    __pyx_L5_bool_binop_done:;
    if (!__pyx_t_5) break;

    /* "sklearn/tree/_splitter.pyx":693
 *                  # At least one drawn features must be non constant
 *                  n_visited_features <= n_found_constants + n_drawn_constants)):
 *             n_visited_features += 1             # <<<<<<<<<<<<<<
 * 
 *             # Loop invariant: elements of features in
 */
    __pyx_v_n_visited_features = (__pyx_v_n_visited_features + 1);

    /* "sklearn/tree/_splitter.pyx":707
 * 
 *             # Draw a feature at random
 *             f_j = rand_int(n_drawn_constants, f_i - n_found_constants,             # <<<<<<<<<<<<<<
 *                            random_state)
 * 
 */
    __pyx_v_f_j = __pyx_f_7sklearn_4tree_6_utils_rand_int(__pyx_v_n_drawn_constants, (__pyx_v_f_i - __pyx_v_n_found_constants), __pyx_v_random_state);

    /* "sklearn/tree/_splitter.pyx":710
 *                            random_state)
 * 
 *             if f_j < n_known_constants:             # <<<<<<<<<<<<<<
 *                 # f_j in the interval [n_drawn_constants, n_known_constants[
 *                 tmp = features[f_j]
 */
    __pyx_t_5 = ((__pyx_v_f_j < __pyx_v_n_known_constants) != 0);
    if (__pyx_t_5) {

      /* "sklearn/tree/_splitter.pyx":712
 *             if f_j < n_known_constants:
 *                 # f_j in the interval [n_drawn_constants, n_known_constants[
 *                 tmp = features[f_j]             # <<<<<<<<<<<<<<
 *                 features[f_j] = features[n_drawn_constants]
 *                 features[n_drawn_constants] = tmp
 */
      __pyx_v_tmp = (__pyx_v_features[__pyx_v_f_j]);

      /* "sklearn/tree/_splitter.pyx":713
 *                 # f_j in the interval [n_drawn_constants, n_known_constants[
 *                 tmp = features[f_j]
 *                 features[f_j] = features[n_drawn_constants]             # <<<<<<<<<<<<<<
 *                 features[n_drawn_constants] = tmp
 * 
 */
      (__pyx_v_features[__pyx_v_f_j]) = (__pyx_v_features[__pyx_v_n_drawn_constants]);

      /* "sklearn/tree/_splitter.pyx":714
 *                 tmp = features[f_j]
 *                 features[f_j] = features[n_drawn_constants]
 *                 features[n_drawn_constants] = tmp             # <<<<<<<<<<<<<<
 * 
 *                 n_drawn_constants += 1
 */
      (__pyx_v_features[__pyx_v_n_drawn_constants]) = __pyx_v_tmp;

      /* "sklearn/tree/_splitter.pyx":716
 *                 features[n_drawn_constants] = tmp
 * 
 *                 n_drawn_constants += 1             # <<<<<<<<<<<<<<
 * 
 *             else:
 */
      __pyx_v_n_drawn_constants = (__pyx_v_n_drawn_constants + 1);
      goto __pyx_L8;
    }
    /*else*/ {

      /* "sklearn/tree/_splitter.pyx":720
 *             else:
 *                 # f_j in the interval [n_known_constants, f_i - n_found_constants[
 *                 f_j += n_found_constants             # <<<<<<<<<<<<<<
 *                 # f_j in the interval [n_total_constants, f_i[
 * 
 */
      __pyx_v_f_j = (__pyx_v_f_j + __pyx_v_n_found_constants);

      /* "sklearn/tree/_splitter.pyx":723
 *                 # f_j in the interval [n_total_constants, f_i[
 * 
 *                 current.feature = features[f_j]             # <<<<<<<<<<<<<<
 *                 feature_stride = X_feature_stride * current.feature
 * 
 */
      __pyx_v_current.feature = (__pyx_v_features[__pyx_v_f_j]);

      /* "sklearn/tree/_splitter.pyx":724
 * 
 *                 current.feature = features[f_j]
 *                 feature_stride = X_feature_stride * current.feature             # <<<<<<<<<<<<<<
 * 
 *                 # Find min, max
 */
      __pyx_v_feature_stride = (__pyx_v_X_feature_stride * __pyx_v_current.feature);

      /* "sklearn/tree/_splitter.pyx":727
 * 
 *                 # Find min, max
 *                 min_feature_value = X[X_sample_stride * samples[start] + feature_stride]             # <<<<<<<<<<<<<<
 *                 max_feature_value = min_feature_value
 *                 Xf[start] = min_feature_value
 */
      __pyx_v_min_feature_value = (__pyx_v_X[((__pyx_v_X_sample_stride * (__pyx_v_samples[__pyx_v_start])) + __pyx_v_feature_stride)]);

      /* "sklearn/tree/_splitter.pyx":728
 *                 # Find min, max
 *                 min_feature_value = X[X_sample_stride * samples[start] + feature_stride]
 *                 max_feature_value = min_feature_value             # <<<<<<<<<<<<<<
 *                 Xf[start] = min_feature_value
 * 
 */
      __pyx_v_max_feature_value = __pyx_v_min_feature_value;

      /* "sklearn/tree/_splitter.pyx":729
 *                 min_feature_value = X[X_sample_stride * samples[start] + feature_stride]
 *                 max_feature_value = min_feature_value
 *                 Xf[start] = min_feature_value             # <<<<<<<<<<<<<<
 * 
 *                 for p in range(start + 1, end):
 */
      (__pyx_v_Xf[__pyx_v_start]) = __pyx_v_min_feature_value;

      /* "sklearn/tree/_splitter.pyx":731
 *                 Xf[start] = min_feature_value
 * 
 *                 for p in range(start + 1, end):             # <<<<<<<<<<<<<<
 *                     current_feature_value = X[X_sample_stride * samples[p] + feature_stride]
 *                     Xf[p] = current_feature_value
 */
      __pyx_t_2 = __pyx_v_end;
      for (__pyx_t_7 = (__pyx_v_start + 1); __pyx_t_7 < __pyx_t_2; __pyx_t_7+=1) {
        __pyx_v_p = __pyx_t_7;

        /* "sklearn/tree/_splitter.pyx":732
 * 
 *                 for p in range(start + 1, end):
 *                     current_feature_value = X[X_sample_stride * samples[p] + feature_stride]             # <<<<<<<<<<<<<<
 *                     Xf[p] = current_feature_value
 * 
 */
        __pyx_v_current_feature_value = (__pyx_v_X[((__pyx_v_X_sample_stride * (__pyx_v_samples[__pyx_v_p])) + __pyx_v_feature_stride)]);

        /* "sklearn/tree/_splitter.pyx":733
 *                 for p in range(start + 1, end):
 *                     current_feature_value = X[X_sample_stride * samples[p] + feature_stride]
 *                     Xf[p] = current_feature_value             # <<<<<<<<<<<<<<
 * 
 *                     if current_feature_value < min_feature_value:
 */
        (__pyx_v_Xf[__pyx_v_p]) = __pyx_v_current_feature_value;

        /* "sklearn/tree/_splitter.pyx":735
 *                     Xf[p] = current_feature_value
 * 
 *                     if current_feature_value < min_feature_value:             # <<<<<<<<<<<<<<
 *                         min_feature_value = current_feature_value
 *                     elif current_feature_value > max_feature_value:
 */
        __pyx_t_5 = ((__pyx_v_current_feature_value < __pyx_v_min_feature_value) != 0);
        if (__pyx_t_5) {

          /* "sklearn/tree/_splitter.pyx":736
 * 
 *                     if current_feature_value < min_feature_value:
 *                         min_feature_value = current_feature_value             # <<<<<<<<<<<<<<
 *                     elif current_feature_value > max_feature_value:
 *                         max_feature_value = current_feature_value
 */
          __pyx_v_min_feature_value = __pyx_v_current_feature_value;
          goto __pyx_L11;
        }

        /* "sklearn/tree/_splitter.pyx":737
 *                     if current_feature_value < min_feature_value:
 *                         min_feature_value = current_feature_value
 *                     elif current_feature_value > max_feature_value:             # <<<<<<<<<<<<<<
 *                         max_feature_value = current_feature_value
 * 
 */
        __pyx_t_5 = ((__pyx_v_current_feature_value > __pyx_v_max_feature_value) != 0);
        if (__pyx_t_5) {

          /* "sklearn/tree/_splitter.pyx":738
 *                         min_feature_value = current_feature_value
 *                     elif current_feature_value > max_feature_value:
 *                         max_feature_value = current_feature_value             # <<<<<<<<<<<<<<
 * 
 *                 if max_feature_value <= min_feature_value + FEATURE_THRESHOLD:
 */
          __pyx_v_max_feature_value = __pyx_v_current_feature_value;
          goto __pyx_L11;
        }
        __pyx_L11:;
      }

      /* "sklearn/tree/_splitter.pyx":740
 *                         max_feature_value = current_feature_value
 * 
 *                 if max_feature_value <= min_feature_value + FEATURE_THRESHOLD:             # <<<<<<<<<<<<<<
 *                     features[f_j] = features[n_total_constants]
 *                     features[n_total_constants] = current.feature
 */
      __pyx_t_5 = ((__pyx_v_max_feature_value <= (__pyx_v_min_feature_value + __pyx_v_7sklearn_4tree_9_splitter_FEATURE_THRESHOLD)) != 0);
      if (__pyx_t_5) {

        /* "sklearn/tree/_splitter.pyx":741
 * 
 *                 if max_feature_value <= min_feature_value + FEATURE_THRESHOLD:
 *                     features[f_j] = features[n_total_constants]             # <<<<<<<<<<<<<<
 *                     features[n_total_constants] = current.feature
 * 
 */
        (__pyx_v_features[__pyx_v_f_j]) = (__pyx_v_features[__pyx_v_n_total_constants]);

        /* "sklearn/tree/_splitter.pyx":742
 *                 if max_feature_value <= min_feature_value + FEATURE_THRESHOLD:
 *                     features[f_j] = features[n_total_constants]
 *                     features[n_total_constants] = current.feature             # <<<<<<<<<<<<<<
 * 
 *                     n_found_constants += 1
 */
        __pyx_t_2 = __pyx_v_current.feature;
        (__pyx_v_features[__pyx_v_n_total_constants]) = __pyx_t_2;

        /* "sklearn/tree/_splitter.pyx":744
 *                     features[n_total_constants] = current.feature
 * 
 *                     n_found_constants += 1             # <<<<<<<<<<<<<<
 *                     n_total_constants += 1
 * 
 */
        __pyx_v_n_found_constants = (__pyx_v_n_found_constants + 1);

        /* "sklearn/tree/_splitter.pyx":745
 * 
 *                     n_found_constants += 1
 *                     n_total_constants += 1             # <<<<<<<<<<<<<<
 * 
 *                 else:
 */
        __pyx_v_n_total_constants = (__pyx_v_n_total_constants + 1);
        goto __pyx_L12;
      }
      /*else*/ {

        /* "sklearn/tree/_splitter.pyx":748
 * 
 *                 else:
 *                     f_i -= 1             # <<<<<<<<<<<<<<
 *                     features[f_i], features[f_j] = features[f_j], features[f_i]
 * 
 */
        __pyx_v_f_i = (__pyx_v_f_i - 1);

        /* "sklearn/tree/_splitter.pyx":749
 *                 else:
 *                     f_i -= 1
 *                     features[f_i], features[f_j] = features[f_j], features[f_i]             # <<<<<<<<<<<<<<
 * 
 *                     # Draw a random threshold
 */
        __pyx_t_2 = (__pyx_v_features[__pyx_v_f_j]);
        __pyx_t_7 = (__pyx_v_features[__pyx_v_f_i]);
        (__pyx_v_features[__pyx_v_f_i]) = __pyx_t_2;
        (__pyx_v_features[__pyx_v_f_j]) = __pyx_t_7;

        /* "sklearn/tree/_splitter.pyx":752
 * 
 *                     # Draw a random threshold
 *                     current.threshold = rand_uniform(min_feature_value,             # <<<<<<<<<<<<<<
 *                                                      max_feature_value,
 *                                                      random_state)
 */
        __pyx_v_current.threshold = __pyx_f_7sklearn_4tree_6_utils_rand_uniform(__pyx_v_min_feature_value, __pyx_v_max_feature_value, __pyx_v_random_state);

        /* "sklearn/tree/_splitter.pyx":756
 *                                                      random_state)
 * 
 *                     if current.threshold == max_feature_value:             # <<<<<<<<<<<<<<
 *                         current.threshold = min_feature_value
 * 
 */
        __pyx_t_5 = ((__pyx_v_current.threshold == __pyx_v_max_feature_value) != 0);
        if (__pyx_t_5) {

          /* "sklearn/tree/_splitter.pyx":757
 * 
 *                     if current.threshold == max_feature_value:
 *                         current.threshold = min_feature_value             # <<<<<<<<<<<<<<
 * 
 *                     # Partition
 */
          __pyx_v_current.threshold = __pyx_v_min_feature_value;
          goto __pyx_L13;
        }
        __pyx_L13:;

        /* "sklearn/tree/_splitter.pyx":760
 * 
 *                     # Partition
 *                     partition_end = end             # <<<<<<<<<<<<<<
 *                     p = start
 *                     while p < partition_end:
 */
        __pyx_v_partition_end = __pyx_v_end;

        /* "sklearn/tree/_splitter.pyx":761
 *                     # Partition
 *                     partition_end = end
 *                     p = start             # <<<<<<<<<<<<<<
 *                     while p < partition_end:
 *                         current_feature_value = Xf[p]
 */
        __pyx_v_p = __pyx_v_start;

        /* "sklearn/tree/_splitter.pyx":762
 *                     partition_end = end
 *                     p = start
 *                     while p < partition_end:             # <<<<<<<<<<<<<<
 *                         current_feature_value = Xf[p]
 *                         if current_feature_value <= current.threshold:
 */
        while (1) {
          __pyx_t_5 = ((__pyx_v_p < __pyx_v_partition_end) != 0);
          if (!__pyx_t_5) break;

          /* "sklearn/tree/_splitter.pyx":763
 *                     p = start
 *                     while p < partition_end:
 *                         current_feature_value = Xf[p]             # <<<<<<<<<<<<<<
 *                         if current_feature_value <= current.threshold:
 *                             p += 1
 */
          __pyx_v_current_feature_value = (__pyx_v_Xf[__pyx_v_p]);

          /* "sklearn/tree/_splitter.pyx":764
 *                     while p < partition_end:
 *                         current_feature_value = Xf[p]
 *                         if current_feature_value <= current.threshold:             # <<<<<<<<<<<<<<
 *                             p += 1
 *                         else:
 */
          __pyx_t_5 = ((__pyx_v_current_feature_value <= __pyx_v_current.threshold) != 0);
          if (__pyx_t_5) {

            /* "sklearn/tree/_splitter.pyx":765
 *                         current_feature_value = Xf[p]
 *                         if current_feature_value <= current.threshold:
 *                             p += 1             # <<<<<<<<<<<<<<
 *                         else:
 *                             partition_end -= 1
 */
            __pyx_v_p = (__pyx_v_p + 1);
            goto __pyx_L16;
          }
          /*else*/ {

            /* "sklearn/tree/_splitter.pyx":767
 *                             p += 1
 *                         else:
 *                             partition_end -= 1             # <<<<<<<<<<<<<<
 * 
 *                             Xf[p] = Xf[partition_end]
 */
            __pyx_v_partition_end = (__pyx_v_partition_end - 1);

            /* "sklearn/tree/_splitter.pyx":769
 *                             partition_end -= 1
 * 
 *                             Xf[p] = Xf[partition_end]             # <<<<<<<<<<<<<<
 *                             Xf[partition_end] = current_feature_value
 * 
 */
            (__pyx_v_Xf[__pyx_v_p]) = (__pyx_v_Xf[__pyx_v_partition_end]);

            /* "sklearn/tree/_splitter.pyx":770
 * 
 *                             Xf[p] = Xf[partition_end]
 *                             Xf[partition_end] = current_feature_value             # <<<<<<<<<<<<<<
 * 
 *                             tmp = samples[partition_end]
 */
            (__pyx_v_Xf[__pyx_v_partition_end]) = __pyx_v_current_feature_value;

            /* "sklearn/tree/_splitter.pyx":772
 *                             Xf[partition_end] = current_feature_value
 * 
 *                             tmp = samples[partition_end]             # <<<<<<<<<<<<<<
 *                             samples[partition_end] = samples[p]
 *                             samples[p] = tmp
 */
            __pyx_v_tmp = (__pyx_v_samples[__pyx_v_partition_end]);

            /* "sklearn/tree/_splitter.pyx":773
 * 
 *                             tmp = samples[partition_end]
 *                             samples[partition_end] = samples[p]             # <<<<<<<<<<<<<<
 *                             samples[p] = tmp
 * 
 */
            (__pyx_v_samples[__pyx_v_partition_end]) = (__pyx_v_samples[__pyx_v_p]);

            /* "sklearn/tree/_splitter.pyx":774
 *                             tmp = samples[partition_end]
 *                             samples[partition_end] = samples[p]
 *                             samples[p] = tmp             # <<<<<<<<<<<<<<
 * 
 *                     current.pos = partition_end
 */
            (__pyx_v_samples[__pyx_v_p]) = __pyx_v_tmp;
          }
          __pyx_L16:;
        }

        /* "sklearn/tree/_splitter.pyx":776
 *                             samples[p] = tmp
 * 
 *                     current.pos = partition_end             # <<<<<<<<<<<<<<
 * 
 *                     # Reject if min_samples_leaf is not guaranteed
 */
        __pyx_v_current.pos = __pyx_v_partition_end;

        /* "sklearn/tree/_splitter.pyx":779
 * 
 *                     # Reject if min_samples_leaf is not guaranteed
 *                     if (((current.pos - start) < min_samples_leaf) or             # <<<<<<<<<<<<<<
 *                             ((end - current.pos) < min_samples_leaf)):
 *                         continue
 */
        __pyx_t_6 = (((__pyx_v_current.pos - __pyx_v_start) < __pyx_v_min_samples_leaf) != 0);
        if (!__pyx_t_6) {
        } else {
          __pyx_t_5 = __pyx_t_6;
          goto __pyx_L18_bool_binop_done;
        }

        /* "sklearn/tree/_splitter.pyx":780
 *                     # Reject if min_samples_leaf is not guaranteed
 *                     if (((current.pos - start) < min_samples_leaf) or
 *                             ((end - current.pos) < min_samples_leaf)):             # <<<<<<<<<<<<<<
 *                         continue
 * 
 */
        __pyx_t_6 = (((__pyx_v_end - __pyx_v_current.pos) < __pyx_v_min_samples_leaf) != 0);
        __pyx_t_5 = __pyx_t_6;
        __pyx_L18_bool_binop_done:;
        if (__pyx_t_5) {

          /* "sklearn/tree/_splitter.pyx":781
 *                     if (((current.pos - start) < min_samples_leaf) or
 *                             ((end - current.pos) < min_samples_leaf)):
 *                         continue             # <<<<<<<<<<<<<<
 * 
 *                     # Evaluate split
 */
          goto __pyx_L3_continue;
        }

        /* "sklearn/tree/_splitter.pyx":784
 * 
 *                     # Evaluate split
 *                     self.criterion.reset()             # <<<<<<<<<<<<<<
 *                     self.criterion.update(current.pos)
 * 
 */
        ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->reset(__pyx_v_self->__pyx_base.__pyx_base.criterion);

        /* "sklearn/tree/_splitter.pyx":785
 *                     # Evaluate split
 *                     self.criterion.reset()
 *                     self.criterion.update(current.pos)             # <<<<<<<<<<<<<<
 * 
 *                     # Reject if min_weight_leaf is not satisfied
 */
        ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->update(__pyx_v_self->__pyx_base.__pyx_base.criterion, __pyx_v_current.pos);

        /* "sklearn/tree/_splitter.pyx":788
 * 
 *                     # Reject if min_weight_leaf is not satisfied
 *                     if ((self.criterion.weighted_n_left < min_weight_leaf) or             # <<<<<<<<<<<<<<
 *                             (self.criterion.weighted_n_right < min_weight_leaf)):
 *                         continue
 */
        __pyx_t_6 = ((__pyx_v_self->__pyx_base.__pyx_base.criterion->weighted_n_left < __pyx_v_min_weight_leaf) != 0);
        if (!__pyx_t_6) {
        } else {
          __pyx_t_5 = __pyx_t_6;
          goto __pyx_L21_bool_binop_done;
        }

        /* "sklearn/tree/_splitter.pyx":789
 *                     # Reject if min_weight_leaf is not satisfied
 *                     if ((self.criterion.weighted_n_left < min_weight_leaf) or
 *                             (self.criterion.weighted_n_right < min_weight_leaf)):             # <<<<<<<<<<<<<<
 *                         continue
 * 
 */
        __pyx_t_6 = ((__pyx_v_self->__pyx_base.__pyx_base.criterion->weighted_n_right < __pyx_v_min_weight_leaf) != 0);
        __pyx_t_5 = __pyx_t_6;
        __pyx_L21_bool_binop_done:;
        if (__pyx_t_5) {

          /* "sklearn/tree/_splitter.pyx":790
 *                     if ((self.criterion.weighted_n_left < min_weight_leaf) or
 *                             (self.criterion.weighted_n_right < min_weight_leaf)):
 *                         continue             # <<<<<<<<<<<<<<
 * 
 *                     current_proxy_improvement = self.criterion.proxy_impurity_improvement()
 */
          goto __pyx_L3_continue;
        }

        /* "sklearn/tree/_splitter.pyx":792
 *                         continue
 * 
 *                     current_proxy_improvement = self.criterion.proxy_impurity_improvement()             # <<<<<<<<<<<<<<
 * 
 *                     if current_proxy_improvement > best_proxy_improvement:
 */
        __pyx_v_current_proxy_improvement = ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->proxy_impurity_improvement(__pyx_v_self->__pyx_base.__pyx_base.criterion);

        /* "sklearn/tree/_splitter.pyx":794
 *                     current_proxy_improvement = self.criterion.proxy_impurity_improvement()
 * 
 *                     if current_proxy_improvement > best_proxy_improvement:             # <<<<<<<<<<<<<<
 *                         best_proxy_improvement = current_proxy_improvement
 *                         best = current  # copy
 */
        __pyx_t_5 = ((__pyx_v_current_proxy_improvement > __pyx_v_best_proxy_improvement) != 0);
        if (__pyx_t_5) {

          /* "sklearn/tree/_splitter.pyx":795
 * 
 *                     if current_proxy_improvement > best_proxy_improvement:
 *                         best_proxy_improvement = current_proxy_improvement             # <<<<<<<<<<<<<<
 *                         best = current  # copy
 * 
 */
          __pyx_v_best_proxy_improvement = __pyx_v_current_proxy_improvement;

          /* "sklearn/tree/_splitter.pyx":796
 *                     if current_proxy_improvement > best_proxy_improvement:
 *                         best_proxy_improvement = current_proxy_improvement
 *                         best = current  # copy             # <<<<<<<<<<<<<<
 * 
 *         # Reorganize into samples[start:best.pos] + samples[best.pos:end]
 */
          __pyx_v_best = __pyx_v_current;
          goto __pyx_L23;
        }
        __pyx_L23:;
      }
      __pyx_L12:;
    }
    __pyx_L8:;
    __pyx_L3_continue:;
  }

  /* "sklearn/tree/_splitter.pyx":799
 * 
 *         # Reorganize into samples[start:best.pos] + samples[best.pos:end]
 *         feature_stride = X_feature_stride * best.feature             # <<<<<<<<<<<<<<
 *         if best.pos < end:
 *             if current.feature != best.feature:
 */
  __pyx_v_feature_stride = (__pyx_v_X_feature_stride * __pyx_v_best.feature);

  /* "sklearn/tree/_splitter.pyx":800
 *         # Reorganize into samples[start:best.pos] + samples[best.pos:end]
 *         feature_stride = X_feature_stride * best.feature
 *         if best.pos < end:             # <<<<<<<<<<<<<<
 *             if current.feature != best.feature:
 *                 partition_end = end
 */
  __pyx_t_5 = ((__pyx_v_best.pos < __pyx_v_end) != 0);
  if (__pyx_t_5) {

    /* "sklearn/tree/_splitter.pyx":801
 *         feature_stride = X_feature_stride * best.feature
 *         if best.pos < end:
 *             if current.feature != best.feature:             # <<<<<<<<<<<<<<
 *                 partition_end = end
 *                 p = start
 */
    __pyx_t_5 = ((__pyx_v_current.feature != __pyx_v_best.feature) != 0);
    if (__pyx_t_5) {

      /* "sklearn/tree/_splitter.pyx":802
 *         if best.pos < end:
 *             if current.feature != best.feature:
 *                 partition_end = end             # <<<<<<<<<<<<<<
 *                 p = start
 * 
 */
      __pyx_v_partition_end = __pyx_v_end;

      /* "sklearn/tree/_splitter.pyx":803
 *             if current.feature != best.feature:
 *                 partition_end = end
 *                 p = start             # <<<<<<<<<<<<<<
 * 
 *                 while p < partition_end:
 */
      __pyx_v_p = __pyx_v_start;

      /* "sklearn/tree/_splitter.pyx":805
 *                 p = start
 * 
 *                 while p < partition_end:             # <<<<<<<<<<<<<<
 *                     if X[X_sample_stride * samples[p] + feature_stride] <= best.threshold:
 *                         p += 1
 */
      while (1) {
        __pyx_t_5 = ((__pyx_v_p < __pyx_v_partition_end) != 0);
        if (!__pyx_t_5) break;

        /* "sklearn/tree/_splitter.pyx":806
 * 
 *                 while p < partition_end:
 *                     if X[X_sample_stride * samples[p] + feature_stride] <= best.threshold:             # <<<<<<<<<<<<<<
 *                         p += 1
 * 
 */
        __pyx_t_5 = (((__pyx_v_X[((__pyx_v_X_sample_stride * (__pyx_v_samples[__pyx_v_p])) + __pyx_v_feature_stride)]) <= __pyx_v_best.threshold) != 0);
        if (__pyx_t_5) {

          /* "sklearn/tree/_splitter.pyx":807
 *                 while p < partition_end:
 *                     if X[X_sample_stride * samples[p] + feature_stride] <= best.threshold:
 *                         p += 1             # <<<<<<<<<<<<<<
 * 
 *                     else:
 */
          __pyx_v_p = (__pyx_v_p + 1);
          goto __pyx_L28;
        }
        /*else*/ {

          /* "sklearn/tree/_splitter.pyx":810
 * 
 *                     else:
 *                         partition_end -= 1             # <<<<<<<<<<<<<<
 * 
 *                         tmp = samples[partition_end]
 */
          __pyx_v_partition_end = (__pyx_v_partition_end - 1);

          /* "sklearn/tree/_splitter.pyx":812
 *                         partition_end -= 1
 * 
 *                         tmp = samples[partition_end]             # <<<<<<<<<<<<<<
 *                         samples[partition_end] = samples[p]
 *                         samples[p] = tmp
 */
          __pyx_v_tmp = (__pyx_v_samples[__pyx_v_partition_end]);

          /* "sklearn/tree/_splitter.pyx":813
 * 
 *                         tmp = samples[partition_end]
 *                         samples[partition_end] = samples[p]             # <<<<<<<<<<<<<<
 *                         samples[p] = tmp
 * 
 */
          (__pyx_v_samples[__pyx_v_partition_end]) = (__pyx_v_samples[__pyx_v_p]);

          /* "sklearn/tree/_splitter.pyx":814
 *                         tmp = samples[partition_end]
 *                         samples[partition_end] = samples[p]
 *                         samples[p] = tmp             # <<<<<<<<<<<<<<
 * 
 * 
 */
          (__pyx_v_samples[__pyx_v_p]) = __pyx_v_tmp;
        }
        __pyx_L28:;
      }
      goto __pyx_L25;
    }
    __pyx_L25:;

    /* "sklearn/tree/_splitter.pyx":817
 * 
 * 
 *             self.criterion.reset()             # <<<<<<<<<<<<<<
 *             self.criterion.update(best.pos)
 *             best.improvement = self.criterion.impurity_improvement(impurity)
 */
    ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->reset(__pyx_v_self->__pyx_base.__pyx_base.criterion);

    /* "sklearn/tree/_splitter.pyx":818
 * 
 *             self.criterion.reset()
 *             self.criterion.update(best.pos)             # <<<<<<<<<<<<<<
 *             best.improvement = self.criterion.impurity_improvement(impurity)
 *             self.criterion.children_impurity(&best.impurity_left,
 */
    ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->update(__pyx_v_self->__pyx_base.__pyx_base.criterion, __pyx_v_best.pos);

    /* "sklearn/tree/_splitter.pyx":819
 *             self.criterion.reset()
 *             self.criterion.update(best.pos)
 *             best.improvement = self.criterion.impurity_improvement(impurity)             # <<<<<<<<<<<<<<
 *             self.criterion.children_impurity(&best.impurity_left,
 *                                              &best.impurity_right)
 */
    __pyx_v_best.improvement = ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->impurity_improvement(__pyx_v_self->__pyx_base.__pyx_base.criterion, __pyx_v_impurity);

    /* "sklearn/tree/_splitter.pyx":820
 *             self.criterion.update(best.pos)
 *             best.improvement = self.criterion.impurity_improvement(impurity)
 *             self.criterion.children_impurity(&best.impurity_left,             # <<<<<<<<<<<<<<
 *                                              &best.impurity_right)
 * 
 */
    ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->children_impurity(__pyx_v_self->__pyx_base.__pyx_base.criterion, (&__pyx_v_best.impurity_left), (&__pyx_v_best.impurity_right));
    goto __pyx_L24;
  }
  __pyx_L24:;

  /* "sklearn/tree/_splitter.pyx":826
 *         # element in features[:n_known_constants] must be preserved for sibling
 *         # and child nodes
 *         memcpy(features, constant_features, sizeof(SIZE_t) * n_known_constants)             # <<<<<<<<<<<<<<
 * 
 *         # Copy newly found constant features
 */
  memcpy(__pyx_v_features, __pyx_v_constant_features, ((sizeof(__pyx_t_7sklearn_4tree_9_splitter_SIZE_t)) * __pyx_v_n_known_constants));

  /* "sklearn/tree/_splitter.pyx":829
 * 
 *         # Copy newly found constant features
 *         memcpy(constant_features + n_known_constants,             # <<<<<<<<<<<<<<
 *                features + n_known_constants,
 *                sizeof(SIZE_t) * n_found_constants)
 */
  memcpy((__pyx_v_constant_features + __pyx_v_n_known_constants), (__pyx_v_features + __pyx_v_n_known_constants), ((sizeof(__pyx_t_7sklearn_4tree_9_splitter_SIZE_t)) * __pyx_v_n_found_constants));

  /* "sklearn/tree/_splitter.pyx":834
 * 
 *         # Return values
 *         split[0] = best             # <<<<<<<<<<<<<<
 *         n_constant_features[0] = n_total_constants
 * 
 */
  (__pyx_v_split[0]) = __pyx_v_best;

  /* "sklearn/tree/_splitter.pyx":835
 *         # Return values
 *         split[0] = best
 *         n_constant_features[0] = n_total_constants             # <<<<<<<<<<<<<<
 * 
 * 
 */
  (__pyx_v_n_constant_features[0]) = __pyx_v_n_total_constants;

  /* "sklearn/tree/_splitter.pyx":634
 *                                  self.presort), self.__getstate__())
 * 
 *     cdef void node_split(self, double impurity, SplitRecord* split,             # <<<<<<<<<<<<<<
 *                          SIZE_t* n_constant_features) nogil:
 *         """Find the best random split on node samples[start:end]."""
 */

  /* function exit code */
}

/* "sklearn/tree/_splitter.pyx":849
 *     cdef SIZE_t* sorted_samples
 * 
 *     def __cinit__(self, Criterion criterion, SIZE_t max_features,             # <<<<<<<<<<<<<<
 *                   SIZE_t min_samples_leaf, double min_weight_leaf,
 *                   object random_state, bint presort):
 */

/* Python wrapper */
static int __pyx_pw_7sklearn_4tree_9_splitter_18BaseSparseSplitter_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7sklearn_4tree_9_splitter_18BaseSparseSplitter_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *__pyx_v_criterion = 0;
  CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_max_features;
  CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_min_samples_leaf;
  CYTHON_UNUSED double __pyx_v_min_weight_leaf;
  CYTHON_UNUSED PyObject *__pyx_v_random_state = 0;
  CYTHON_UNUSED int __pyx_v_presort;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_criterion,&__pyx_n_s_max_features,&__pyx_n_s_min_samples_leaf,&__pyx_n_s_min_weight_leaf,&__pyx_n_s_random_state,&__pyx_n_s_presort,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_criterion)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_max_features)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 849; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_min_samples_leaf)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 849; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_min_weight_leaf)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 849; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_random_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 849; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_presort)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, 5); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 849; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 849; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 6) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
    }
    __pyx_v_criterion = ((struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *)values[0]);
    __pyx_v_max_features = __Pyx_PyInt_As_Py_intptr_t(values[1]); if (unlikely((__pyx_v_max_features == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 849; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_min_samples_leaf = __Pyx_PyInt_As_Py_intptr_t(values[2]); if (unlikely((__pyx_v_min_samples_leaf == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 850; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_min_weight_leaf = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_min_weight_leaf == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 850; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_random_state = values[4];
    __pyx_v_presort = __Pyx_PyObject_IsTrue(values[5]); if (unlikely((__pyx_v_presort == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 851; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 849; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("sklearn.tree._splitter.BaseSparseSplitter.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_criterion), __pyx_ptype_7sklearn_4tree_10_criterion_Criterion, 1, "criterion", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 849; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_7sklearn_4tree_9_splitter_18BaseSparseSplitter___cinit__(((struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *)__pyx_v_self), __pyx_v_criterion, __pyx_v_max_features, __pyx_v_min_samples_leaf, __pyx_v_min_weight_leaf, __pyx_v_random_state, __pyx_v_presort);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7sklearn_4tree_9_splitter_18BaseSparseSplitter___cinit__(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *__pyx_v_criterion, CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_max_features, CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_min_samples_leaf, CYTHON_UNUSED double __pyx_v_min_weight_leaf, CYTHON_UNUSED PyObject *__pyx_v_random_state, CYTHON_UNUSED int __pyx_v_presort) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "sklearn/tree/_splitter.pyx":854
 *         # Parent __cinit__ is automatically called
 * 
 *         self.X_data = NULL             # <<<<<<<<<<<<<<
 *         self.X_indices = NULL
 *         self.X_indptr = NULL
 */
  __pyx_v_self->X_data = NULL;

  /* "sklearn/tree/_splitter.pyx":855
 * 
 *         self.X_data = NULL
 *         self.X_indices = NULL             # <<<<<<<<<<<<<<
 *         self.X_indptr = NULL
 * 
 */
  __pyx_v_self->X_indices = NULL;

  /* "sklearn/tree/_splitter.pyx":856
 *         self.X_data = NULL
 *         self.X_indices = NULL
 *         self.X_indptr = NULL             # <<<<<<<<<<<<<<
 * 
 *         self.n_total_samples = 0
 */
  __pyx_v_self->X_indptr = NULL;

  /* "sklearn/tree/_splitter.pyx":858
 *         self.X_indptr = NULL
 * 
 *         self.n_total_samples = 0             # <<<<<<<<<<<<<<
 * 
 *         self.index_to_samples = NULL
 */
  __pyx_v_self->n_total_samples = 0;

  /* "sklearn/tree/_splitter.pyx":860
 *         self.n_total_samples = 0
 * 
 *         self.index_to_samples = NULL             # <<<<<<<<<<<<<<
 *         self.sorted_samples = NULL
 * 
 */
  __pyx_v_self->index_to_samples = NULL;

  /* "sklearn/tree/_splitter.pyx":861
 * 
 *         self.index_to_samples = NULL
 *         self.sorted_samples = NULL             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_v_self->sorted_samples = NULL;

  /* "sklearn/tree/_splitter.pyx":849
 *     cdef SIZE_t* sorted_samples
 * 
 *     def __cinit__(self, Criterion criterion, SIZE_t max_features,             # <<<<<<<<<<<<<<
 *                   SIZE_t min_samples_leaf, double min_weight_leaf,
 *                   object random_state, bint presort):
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/tree/_splitter.pyx":863
 *         self.sorted_samples = NULL
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         """Deallocate memory."""
 *         free(self.index_to_samples)
 */

/* Python wrapper */
static void __pyx_pw_7sklearn_4tree_9_splitter_18BaseSparseSplitter_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_7sklearn_4tree_9_splitter_18BaseSparseSplitter_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_7sklearn_4tree_9_splitter_18BaseSparseSplitter_2__dealloc__(((struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_7sklearn_4tree_9_splitter_18BaseSparseSplitter_2__dealloc__(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "sklearn/tree/_splitter.pyx":865
 *     def __dealloc__(self):
 *         """Deallocate memory."""
 *         free(self.index_to_samples)             # <<<<<<<<<<<<<<
 *         free(self.sorted_samples)
 * 
 */
  free(__pyx_v_self->index_to_samples);

  /* "sklearn/tree/_splitter.pyx":866
 *         """Deallocate memory."""
 *         free(self.index_to_samples)
 *         free(self.sorted_samples)             # <<<<<<<<<<<<<<
 * 
 *     cdef void init(self,
 */
  free(__pyx_v_self->sorted_samples);

  /* "sklearn/tree/_splitter.pyx":863
 *         self.sorted_samples = NULL
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         """Deallocate memory."""
 *         free(self.index_to_samples)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "sklearn/tree/_splitter.pyx":868
 *         free(self.sorted_samples)
 * 
 *     cdef void init(self,             # <<<<<<<<<<<<<<
 *                    object X,
 *                    np.ndarray[DOUBLE_t, ndim=2, mode="c"] y,
 */

static void __pyx_f_7sklearn_4tree_9_splitter_18BaseSparseSplitter_init(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *__pyx_v_self, PyObject *__pyx_v_X, PyArrayObject *__pyx_v_y, __pyx_t_7sklearn_4tree_9_splitter_DOUBLE_t *__pyx_v_sample_weight, struct __pyx_opt_args_7sklearn_4tree_9_splitter_18BaseSparseSplitter_init *__pyx_optional_args) {
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_samples;
  PyArrayObject *__pyx_v_data = 0;
  PyArrayObject *__pyx_v_indices = 0;
  PyArrayObject *__pyx_v_indptr = 0;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_total_samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_index_to_samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_p;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_data;
  __Pyx_Buffer __pyx_pybuffer_data;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_indices;
  __Pyx_Buffer __pyx_pybuffer_indices;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_indptr;
  __Pyx_Buffer __pyx_pybuffer_indptr;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_y;
  __Pyx_Buffer __pyx_pybuffer_y;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_t_4;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_5;
  PyArrayObject *__pyx_t_6 = NULL;
  PyArrayObject *__pyx_t_7 = NULL;
  PyArrayObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("init", 0);
  if (__pyx_optional_args) {
  }
  __pyx_pybuffer_data.pybuffer.buf = NULL;
  __pyx_pybuffer_data.refcount = 0;
  __pyx_pybuffernd_data.data = NULL;
  __pyx_pybuffernd_data.rcbuffer = &__pyx_pybuffer_data;
  __pyx_pybuffer_indices.pybuffer.buf = NULL;
  __pyx_pybuffer_indices.refcount = 0;
  __pyx_pybuffernd_indices.data = NULL;
  __pyx_pybuffernd_indices.rcbuffer = &__pyx_pybuffer_indices;
  __pyx_pybuffer_indptr.pybuffer.buf = NULL;
  __pyx_pybuffer_indptr.refcount = 0;
  __pyx_pybuffernd_indptr.data = NULL;
  __pyx_pybuffernd_indptr.rcbuffer = &__pyx_pybuffer_indptr;
  __pyx_pybuffer_y.pybuffer.buf = NULL;
  __pyx_pybuffer_y.refcount = 0;
  __pyx_pybuffernd_y.data = NULL;
  __pyx_pybuffernd_y.rcbuffer = &__pyx_pybuffer_y;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_y.rcbuffer->pybuffer, (PyObject*)__pyx_v_y, &__Pyx_TypeInfo_nn___pyx_t_7sklearn_4tree_9_splitter_DOUBLE_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 868; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_y.diminfo[0].strides = __pyx_pybuffernd_y.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_y.diminfo[0].shape = __pyx_pybuffernd_y.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_y.diminfo[1].strides = __pyx_pybuffernd_y.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_y.diminfo[1].shape = __pyx_pybuffernd_y.rcbuffer->pybuffer.shape[1];

  /* "sklearn/tree/_splitter.pyx":876
 * 
 *         # Call parent init
 *         Splitter.init(self, X, y, sample_weight)             # <<<<<<<<<<<<<<
 * 
 *         if not isinstance(X, csc_matrix):
 */
  __pyx_vtabptr_7sklearn_4tree_9_splitter_Splitter->init(((struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *)__pyx_v_self), __pyx_v_X, ((PyArrayObject *)__pyx_v_y), __pyx_v_sample_weight, NULL); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 876; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sklearn/tree/_splitter.pyx":878
 *         Splitter.init(self, X, y, sample_weight)
 * 
 *         if not isinstance(X, csc_matrix):             # <<<<<<<<<<<<<<
 *             raise ValueError("X should be in csc format")
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_csc_matrix); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 878; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_IsInstance(__pyx_v_X, __pyx_t_1); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 878; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = ((!(__pyx_t_2 != 0)) != 0);
  if (__pyx_t_3) {

    /* "sklearn/tree/_splitter.pyx":879
 * 
 *         if not isinstance(X, csc_matrix):
 *             raise ValueError("X should be in csc format")             # <<<<<<<<<<<<<<
 * 
 *         cdef SIZE_t* samples = self.samples
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 879; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 879; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "sklearn/tree/_splitter.pyx":881
 *             raise ValueError("X should be in csc format")
 * 
 *         cdef SIZE_t* samples = self.samples             # <<<<<<<<<<<<<<
 *         cdef SIZE_t n_samples = self.n_samples
 * 
 */
  __pyx_t_4 = __pyx_v_self->__pyx_base.samples;
  __pyx_v_samples = __pyx_t_4;

  /* "sklearn/tree/_splitter.pyx":882
 * 
 *         cdef SIZE_t* samples = self.samples
 *         cdef SIZE_t n_samples = self.n_samples             # <<<<<<<<<<<<<<
 * 
 *         # Initialize X
 */
  __pyx_t_5 = __pyx_v_self->__pyx_base.n_samples;
  __pyx_v_n_samples = __pyx_t_5;

  /* "sklearn/tree/_splitter.pyx":885
 * 
 *         # Initialize X
 *         cdef np.ndarray[dtype=DTYPE_t, ndim=1] data = X.data             # <<<<<<<<<<<<<<
 *         cdef np.ndarray[dtype=INT32_t, ndim=1] indices = X.indices
 *         cdef np.ndarray[dtype=INT32_t, ndim=1] indptr = X.indptr
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_data); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 885; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 885; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = ((PyArrayObject *)__pyx_t_1);
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_data.rcbuffer->pybuffer, (PyObject*)__pyx_t_6, &__Pyx_TypeInfo_nn___pyx_t_7sklearn_4tree_9_splitter_DTYPE_t, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack) == -1)) {
      __pyx_v_data = ((PyArrayObject *)Py_None); __Pyx_INCREF(Py_None); __pyx_pybuffernd_data.rcbuffer->pybuffer.buf = NULL;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 885; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    } else {__pyx_pybuffernd_data.diminfo[0].strides = __pyx_pybuffernd_data.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_data.diminfo[0].shape = __pyx_pybuffernd_data.rcbuffer->pybuffer.shape[0];
    }
  }
  __pyx_t_6 = 0;
  __pyx_v_data = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sklearn/tree/_splitter.pyx":886
 *         # Initialize X
 *         cdef np.ndarray[dtype=DTYPE_t, ndim=1] data = X.data
 *         cdef np.ndarray[dtype=INT32_t, ndim=1] indices = X.indices             # <<<<<<<<<<<<<<
 *         cdef np.ndarray[dtype=INT32_t, ndim=1] indptr = X.indptr
 *         cdef SIZE_t n_total_samples = X.shape[0]
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_indices); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 886; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 886; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_7 = ((PyArrayObject *)__pyx_t_1);
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_indices.rcbuffer->pybuffer, (PyObject*)__pyx_t_7, &__Pyx_TypeInfo_nn___pyx_t_7sklearn_4tree_9_splitter_INT32_t, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack) == -1)) {
      __pyx_v_indices = ((PyArrayObject *)Py_None); __Pyx_INCREF(Py_None); __pyx_pybuffernd_indices.rcbuffer->pybuffer.buf = NULL;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 886; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    } else {__pyx_pybuffernd_indices.diminfo[0].strides = __pyx_pybuffernd_indices.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_indices.diminfo[0].shape = __pyx_pybuffernd_indices.rcbuffer->pybuffer.shape[0];
    }
  }
  __pyx_t_7 = 0;
  __pyx_v_indices = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sklearn/tree/_splitter.pyx":887
 *         cdef np.ndarray[dtype=DTYPE_t, ndim=1] data = X.data
 *         cdef np.ndarray[dtype=INT32_t, ndim=1] indices = X.indices
 *         cdef np.ndarray[dtype=INT32_t, ndim=1] indptr = X.indptr             # <<<<<<<<<<<<<<
 *         cdef SIZE_t n_total_samples = X.shape[0]
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_indptr); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 887; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 887; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_8 = ((PyArrayObject *)__pyx_t_1);
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_indptr.rcbuffer->pybuffer, (PyObject*)__pyx_t_8, &__Pyx_TypeInfo_nn___pyx_t_7sklearn_4tree_9_splitter_INT32_t, PyBUF_FORMAT| PyBUF_STRIDES, 1, 0, __pyx_stack) == -1)) {
      __pyx_v_indptr = ((PyArrayObject *)Py_None); __Pyx_INCREF(Py_None); __pyx_pybuffernd_indptr.rcbuffer->pybuffer.buf = NULL;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 887; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    } else {__pyx_pybuffernd_indptr.diminfo[0].strides = __pyx_pybuffernd_indptr.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_indptr.diminfo[0].shape = __pyx_pybuffernd_indptr.rcbuffer->pybuffer.shape[0];
    }
  }
  __pyx_t_8 = 0;
  __pyx_v_indptr = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "sklearn/tree/_splitter.pyx":888
 *         cdef np.ndarray[dtype=INT32_t, ndim=1] indices = X.indices
 *         cdef np.ndarray[dtype=INT32_t, ndim=1] indptr = X.indptr
 *         cdef SIZE_t n_total_samples = X.shape[0]             # <<<<<<<<<<<<<<
 * 
 *         self.X_data = <DTYPE_t*> data.data
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_X, __pyx_n_s_shape); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_9 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 888; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_9); if (unlikely((__pyx_t_5 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_n_total_samples = __pyx_t_5;

  /* "sklearn/tree/_splitter.pyx":890
 *         cdef SIZE_t n_total_samples = X.shape[0]
 * 
 *         self.X_data = <DTYPE_t*> data.data             # <<<<<<<<<<<<<<
 *         self.X_indices = <INT32_t*> indices.data
 *         self.X_indptr = <INT32_t*> indptr.data
 */
  __pyx_v_self->X_data = ((__pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *)__pyx_v_data->data);

  /* "sklearn/tree/_splitter.pyx":891
 * 
 *         self.X_data = <DTYPE_t*> data.data
 *         self.X_indices = <INT32_t*> indices.data             # <<<<<<<<<<<<<<
 *         self.X_indptr = <INT32_t*> indptr.data
 *         self.n_total_samples = n_total_samples
 */
  __pyx_v_self->X_indices = ((__pyx_t_7sklearn_4tree_9_splitter_INT32_t *)__pyx_v_indices->data);

  /* "sklearn/tree/_splitter.pyx":892
 *         self.X_data = <DTYPE_t*> data.data
 *         self.X_indices = <INT32_t*> indices.data
 *         self.X_indptr = <INT32_t*> indptr.data             # <<<<<<<<<<<<<<
 *         self.n_total_samples = n_total_samples
 * 
 */
  __pyx_v_self->X_indptr = ((__pyx_t_7sklearn_4tree_9_splitter_INT32_t *)__pyx_v_indptr->data);

  /* "sklearn/tree/_splitter.pyx":893
 *         self.X_indices = <INT32_t*> indices.data
 *         self.X_indptr = <INT32_t*> indptr.data
 *         self.n_total_samples = n_total_samples             # <<<<<<<<<<<<<<
 * 
 *         # Initialize auxiliary array used to perform split
 */
  __pyx_v_self->n_total_samples = __pyx_v_n_total_samples;

  /* "sklearn/tree/_splitter.pyx":896
 * 
 *         # Initialize auxiliary array used to perform split
 *         safe_realloc(&self.index_to_samples, n_total_samples)             # <<<<<<<<<<<<<<
 *         safe_realloc(&self.sorted_samples, n_samples)
 * 
 */
  __pyx_fuse_1__pyx_f_7sklearn_4tree_6_utils_safe_realloc((&__pyx_v_self->index_to_samples), __pyx_v_n_total_samples); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 896; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sklearn/tree/_splitter.pyx":897
 *         # Initialize auxiliary array used to perform split
 *         safe_realloc(&self.index_to_samples, n_total_samples)
 *         safe_realloc(&self.sorted_samples, n_samples)             # <<<<<<<<<<<<<<
 * 
 *         cdef SIZE_t* index_to_samples = self.index_to_samples
 */
  __pyx_fuse_1__pyx_f_7sklearn_4tree_6_utils_safe_realloc((&__pyx_v_self->sorted_samples), __pyx_v_n_samples); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 897; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "sklearn/tree/_splitter.pyx":899
 *         safe_realloc(&self.sorted_samples, n_samples)
 * 
 *         cdef SIZE_t* index_to_samples = self.index_to_samples             # <<<<<<<<<<<<<<
 *         cdef SIZE_t p
 *         for p in range(n_total_samples):
 */
  __pyx_t_4 = __pyx_v_self->index_to_samples;
  __pyx_v_index_to_samples = __pyx_t_4;

  /* "sklearn/tree/_splitter.pyx":901
 *         cdef SIZE_t* index_to_samples = self.index_to_samples
 *         cdef SIZE_t p
 *         for p in range(n_total_samples):             # <<<<<<<<<<<<<<
 *             index_to_samples[p] = -1
 * 
 */
  __pyx_t_5 = __pyx_v_n_total_samples;
  for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_5; __pyx_t_10+=1) {
    __pyx_v_p = __pyx_t_10;

    /* "sklearn/tree/_splitter.pyx":902
 *         cdef SIZE_t p
 *         for p in range(n_total_samples):
 *             index_to_samples[p] = -1             # <<<<<<<<<<<<<<
 * 
 *         for p in range(n_samples):
 */
    (__pyx_v_index_to_samples[__pyx_v_p]) = -1;
  }

  /* "sklearn/tree/_splitter.pyx":904
 *             index_to_samples[p] = -1
 * 
 *         for p in range(n_samples):             # <<<<<<<<<<<<<<
 *             index_to_samples[samples[p]] = p
 * 
 */
  __pyx_t_5 = __pyx_v_n_samples;
  for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_5; __pyx_t_10+=1) {
    __pyx_v_p = __pyx_t_10;

    /* "sklearn/tree/_splitter.pyx":905
 * 
 *         for p in range(n_samples):
 *             index_to_samples[samples[p]] = p             # <<<<<<<<<<<<<<
 * 
 *     cdef inline SIZE_t _partition(self, double threshold,
 */
    (__pyx_v_index_to_samples[(__pyx_v_samples[__pyx_v_p])]) = __pyx_v_p;
  }

  /* "sklearn/tree/_splitter.pyx":868
 *         free(self.sorted_samples)
 * 
 *     cdef void init(self,             # <<<<<<<<<<<<<<
 *                    object X,
 *                    np.ndarray[DOUBLE_t, ndim=2, mode="c"] y,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_9);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_data.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_indices.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_indptr.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_y.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("sklearn.tree._splitter.BaseSparseSplitter.init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_data.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_indices.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_indptr.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_y.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_data);
  __Pyx_XDECREF((PyObject *)__pyx_v_indices);
  __Pyx_XDECREF((PyObject *)__pyx_v_indptr);
  __Pyx_RefNannyFinishContext();
}

/* "sklearn/tree/_splitter.pyx":907
 *             index_to_samples[samples[p]] = p
 * 
 *     cdef inline SIZE_t _partition(self, double threshold,             # <<<<<<<<<<<<<<
 *                                   SIZE_t end_negative, SIZE_t start_positive,
 *                                   SIZE_t zero_pos) nogil:
 */

static CYTHON_INLINE __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_f_7sklearn_4tree_9_splitter_18BaseSparseSplitter__partition(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *__pyx_v_self, double __pyx_v_threshold, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_end_negative, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_start_positive, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_zero_pos) {
  double __pyx_v_value;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_partition_end;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_p;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_v_Xf;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_index_to_samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_r;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_t_1;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_t_2;
  int __pyx_t_3;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_4;

  /* "sklearn/tree/_splitter.pyx":916
 *         cdef SIZE_t p
 * 
 *         cdef DTYPE_t* Xf = self.feature_values             # <<<<<<<<<<<<<<
 *         cdef SIZE_t* samples = self.samples
 *         cdef SIZE_t* index_to_samples = self.index_to_samples
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.feature_values;
  __pyx_v_Xf = __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":917
 * 
 *         cdef DTYPE_t* Xf = self.feature_values
 *         cdef SIZE_t* samples = self.samples             # <<<<<<<<<<<<<<
 *         cdef SIZE_t* index_to_samples = self.index_to_samples
 * 
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.samples;
  __pyx_v_samples = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":918
 *         cdef DTYPE_t* Xf = self.feature_values
 *         cdef SIZE_t* samples = self.samples
 *         cdef SIZE_t* index_to_samples = self.index_to_samples             # <<<<<<<<<<<<<<
 * 
 *         if threshold < 0.:
 */
  __pyx_t_2 = __pyx_v_self->index_to_samples;
  __pyx_v_index_to_samples = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":920
 *         cdef SIZE_t* index_to_samples = self.index_to_samples
 * 
 *         if threshold < 0.:             # <<<<<<<<<<<<<<
 *             p = self.start
 *             partition_end = end_negative
 */
  __pyx_t_3 = ((__pyx_v_threshold < 0.) != 0);
  if (__pyx_t_3) {

    /* "sklearn/tree/_splitter.pyx":921
 * 
 *         if threshold < 0.:
 *             p = self.start             # <<<<<<<<<<<<<<
 *             partition_end = end_negative
 *         elif threshold > 0.:
 */
    __pyx_t_4 = __pyx_v_self->__pyx_base.start;
    __pyx_v_p = __pyx_t_4;

    /* "sklearn/tree/_splitter.pyx":922
 *         if threshold < 0.:
 *             p = self.start
 *             partition_end = end_negative             # <<<<<<<<<<<<<<
 *         elif threshold > 0.:
 *             p = start_positive
 */
    __pyx_v_partition_end = __pyx_v_end_negative;
    goto __pyx_L3;
  }

  /* "sklearn/tree/_splitter.pyx":923
 *             p = self.start
 *             partition_end = end_negative
 *         elif threshold > 0.:             # <<<<<<<<<<<<<<
 *             p = start_positive
 *             partition_end = self.end
 */
  __pyx_t_3 = ((__pyx_v_threshold > 0.) != 0);
  if (__pyx_t_3) {

    /* "sklearn/tree/_splitter.pyx":924
 *             partition_end = end_negative
 *         elif threshold > 0.:
 *             p = start_positive             # <<<<<<<<<<<<<<
 *             partition_end = self.end
 *         else:
 */
    __pyx_v_p = __pyx_v_start_positive;

    /* "sklearn/tree/_splitter.pyx":925
 *         elif threshold > 0.:
 *             p = start_positive
 *             partition_end = self.end             # <<<<<<<<<<<<<<
 *         else:
 *             # Data are already split
 */
    __pyx_t_4 = __pyx_v_self->__pyx_base.end;
    __pyx_v_partition_end = __pyx_t_4;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "sklearn/tree/_splitter.pyx":928
 *         else:
 *             # Data are already split
 *             return zero_pos             # <<<<<<<<<<<<<<
 * 
 *         while p < partition_end:
 */
    __pyx_r = __pyx_v_zero_pos;
    goto __pyx_L0;
  }
  __pyx_L3:;

  /* "sklearn/tree/_splitter.pyx":930
 *             return zero_pos
 * 
 *         while p < partition_end:             # <<<<<<<<<<<<<<
 *             value = Xf[p]
 * 
 */
  while (1) {
    __pyx_t_3 = ((__pyx_v_p < __pyx_v_partition_end) != 0);
    if (!__pyx_t_3) break;

    /* "sklearn/tree/_splitter.pyx":931
 * 
 *         while p < partition_end:
 *             value = Xf[p]             # <<<<<<<<<<<<<<
 * 
 *             if value <= threshold:
 */
    __pyx_v_value = (__pyx_v_Xf[__pyx_v_p]);

    /* "sklearn/tree/_splitter.pyx":933
 *             value = Xf[p]
 * 
 *             if value <= threshold:             # <<<<<<<<<<<<<<
 *                 p += 1
 * 
 */
    __pyx_t_3 = ((__pyx_v_value <= __pyx_v_threshold) != 0);
    if (__pyx_t_3) {

      /* "sklearn/tree/_splitter.pyx":934
 * 
 *             if value <= threshold:
 *                 p += 1             # <<<<<<<<<<<<<<
 * 
 *             else:
 */
      __pyx_v_p = (__pyx_v_p + 1);
      goto __pyx_L6;
    }
    /*else*/ {

      /* "sklearn/tree/_splitter.pyx":937
 * 
 *             else:
 *                 partition_end -= 1             # <<<<<<<<<<<<<<
 * 
 *                 Xf[p] = Xf[partition_end]
 */
      __pyx_v_partition_end = (__pyx_v_partition_end - 1);

      /* "sklearn/tree/_splitter.pyx":939
 *                 partition_end -= 1
 * 
 *                 Xf[p] = Xf[partition_end]             # <<<<<<<<<<<<<<
 *                 Xf[partition_end] = value
 *                 sparse_swap(index_to_samples, samples, p, partition_end)
 */
      (__pyx_v_Xf[__pyx_v_p]) = (__pyx_v_Xf[__pyx_v_partition_end]);

      /* "sklearn/tree/_splitter.pyx":940
 * 
 *                 Xf[p] = Xf[partition_end]
 *                 Xf[partition_end] = value             # <<<<<<<<<<<<<<
 *                 sparse_swap(index_to_samples, samples, p, partition_end)
 * 
 */
      (__pyx_v_Xf[__pyx_v_partition_end]) = __pyx_v_value;

      /* "sklearn/tree/_splitter.pyx":941
 *                 Xf[p] = Xf[partition_end]
 *                 Xf[partition_end] = value
 *                 sparse_swap(index_to_samples, samples, p, partition_end)             # <<<<<<<<<<<<<<
 * 
 *         return partition_end
 */
      __pyx_f_7sklearn_4tree_9_splitter_sparse_swap(__pyx_v_index_to_samples, __pyx_v_samples, __pyx_v_p, __pyx_v_partition_end);
    }
    __pyx_L6:;
  }

  /* "sklearn/tree/_splitter.pyx":943
 *                 sparse_swap(index_to_samples, samples, p, partition_end)
 * 
 *         return partition_end             # <<<<<<<<<<<<<<
 * 
 *     cdef inline void extract_nnz(self, SIZE_t feature,
 */
  __pyx_r = __pyx_v_partition_end;
  goto __pyx_L0;

  /* "sklearn/tree/_splitter.pyx":907
 *             index_to_samples[samples[p]] = p
 * 
 *     cdef inline SIZE_t _partition(self, double threshold,             # <<<<<<<<<<<<<<
 *                                   SIZE_t end_negative, SIZE_t start_positive,
 *                                   SIZE_t zero_pos) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "sklearn/tree/_splitter.pyx":945
 *         return partition_end
 * 
 *     cdef inline void extract_nnz(self, SIZE_t feature,             # <<<<<<<<<<<<<<
 *                                  SIZE_t* end_negative, SIZE_t* start_positive,
 *                                  bint* is_samples_sorted) nogil:
 */

static CYTHON_INLINE void __pyx_f_7sklearn_4tree_9_splitter_18BaseSparseSplitter_extract_nnz(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *__pyx_v_self, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_feature, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_end_negative, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_start_positive, int *__pyx_v_is_samples_sorted) {
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_indptr_start;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_indptr_end;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_indices;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_samples;
  int __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":977
 * 
 *         """
 *         cdef SIZE_t indptr_start = self.X_indptr[feature],             # <<<<<<<<<<<<<<
 *         cdef SIZE_t indptr_end = self.X_indptr[feature + 1]
 *         cdef SIZE_t n_indices = <SIZE_t>(indptr_end - indptr_start)
 */
  __pyx_v_indptr_start = (__pyx_v_self->X_indptr[__pyx_v_feature]);

  /* "sklearn/tree/_splitter.pyx":978
 *         """
 *         cdef SIZE_t indptr_start = self.X_indptr[feature],
 *         cdef SIZE_t indptr_end = self.X_indptr[feature + 1]             # <<<<<<<<<<<<<<
 *         cdef SIZE_t n_indices = <SIZE_t>(indptr_end - indptr_start)
 *         cdef SIZE_t n_samples = self.end - self.start
 */
  __pyx_v_indptr_end = (__pyx_v_self->X_indptr[(__pyx_v_feature + 1)]);

  /* "sklearn/tree/_splitter.pyx":979
 *         cdef SIZE_t indptr_start = self.X_indptr[feature],
 *         cdef SIZE_t indptr_end = self.X_indptr[feature + 1]
 *         cdef SIZE_t n_indices = <SIZE_t>(indptr_end - indptr_start)             # <<<<<<<<<<<<<<
 *         cdef SIZE_t n_samples = self.end - self.start
 * 
 */
  __pyx_v_n_indices = ((__pyx_t_7sklearn_4tree_9_splitter_SIZE_t)(__pyx_v_indptr_end - __pyx_v_indptr_start));

  /* "sklearn/tree/_splitter.pyx":980
 *         cdef SIZE_t indptr_end = self.X_indptr[feature + 1]
 *         cdef SIZE_t n_indices = <SIZE_t>(indptr_end - indptr_start)
 *         cdef SIZE_t n_samples = self.end - self.start             # <<<<<<<<<<<<<<
 * 
 *         # Use binary search if n_samples * log(n_indices) <
 */
  __pyx_v_n_samples = (__pyx_v_self->__pyx_base.end - __pyx_v_self->__pyx_base.start);

  /* "sklearn/tree/_splitter.pyx":988
 *         # approach.
 *         if ((1 - is_samples_sorted[0]) * n_samples * log(n_samples) +
 *                 n_samples * log(n_indices) < EXTRACT_NNZ_SWITCH * n_indices):             # <<<<<<<<<<<<<<
 *             extract_nnz_binary_search(self.X_indices, self.X_data,
 *                                       indptr_start, indptr_end,
 */
  __pyx_t_1 = ((((((1 - (__pyx_v_is_samples_sorted[0])) * __pyx_v_n_samples) * __pyx_f_7sklearn_4tree_6_utils_log(__pyx_v_n_samples)) + (__pyx_v_n_samples * __pyx_f_7sklearn_4tree_6_utils_log(__pyx_v_n_indices))) < (__pyx_v_7sklearn_4tree_9_splitter_EXTRACT_NNZ_SWITCH * __pyx_v_n_indices)) != 0);
  if (__pyx_t_1) {

    /* "sklearn/tree/_splitter.pyx":989
 *         if ((1 - is_samples_sorted[0]) * n_samples * log(n_samples) +
 *                 n_samples * log(n_indices) < EXTRACT_NNZ_SWITCH * n_indices):
 *             extract_nnz_binary_search(self.X_indices, self.X_data,             # <<<<<<<<<<<<<<
 *                                       indptr_start, indptr_end,
 *                                       self.samples, self.start, self.end,
 */
    __pyx_f_7sklearn_4tree_9_splitter_extract_nnz_binary_search(__pyx_v_self->X_indices, __pyx_v_self->X_data, __pyx_v_indptr_start, __pyx_v_indptr_end, __pyx_v_self->__pyx_base.samples, __pyx_v_self->__pyx_base.start, __pyx_v_self->__pyx_base.end, __pyx_v_self->index_to_samples, __pyx_v_self->__pyx_base.feature_values, __pyx_v_end_negative, __pyx_v_start_positive, __pyx_v_self->sorted_samples, __pyx_v_is_samples_sorted);
    goto __pyx_L3;
  }
  /*else*/ {

    /* "sklearn/tree/_splitter.pyx":1000
 *         # index_to_samples is a mapping from X_indices to samples
 *         else:
 *             extract_nnz_index_to_samples(self.X_indices, self.X_data,             # <<<<<<<<<<<<<<
 *                                          indptr_start, indptr_end,
 *                                          self.samples, self.start, self.end,
 */
    __pyx_f_7sklearn_4tree_9_splitter_extract_nnz_index_to_samples(__pyx_v_self->X_indices, __pyx_v_self->X_data, __pyx_v_indptr_start, __pyx_v_indptr_end, __pyx_v_self->__pyx_base.samples, __pyx_v_self->__pyx_base.start, __pyx_v_self->__pyx_base.end, __pyx_v_self->index_to_samples, __pyx_v_self->__pyx_base.feature_values, __pyx_v_end_negative, __pyx_v_start_positive);
  }
  __pyx_L3:;

  /* "sklearn/tree/_splitter.pyx":945
 *         return partition_end
 * 
 *     cdef inline void extract_nnz(self, SIZE_t feature,             # <<<<<<<<<<<<<<
 *                                  SIZE_t* end_negative, SIZE_t* start_positive,
 *                                  bint* is_samples_sorted) nogil:
 */

  /* function exit code */
}

/* "sklearn/tree/_splitter.pyx":1008
 * 
 * 
 * cdef int compare_SIZE_t(const void* a, const void* b) nogil:             # <<<<<<<<<<<<<<
 *     """Comparison function for sort."""
 *     return <int>((<SIZE_t*>a)[0] - (<SIZE_t*>b)[0])
 */

static int __pyx_f_7sklearn_4tree_9_splitter_compare_SIZE_t(void const *__pyx_v_a, void const *__pyx_v_b) {
  int __pyx_r;

  /* "sklearn/tree/_splitter.pyx":1010
 * cdef int compare_SIZE_t(const void* a, const void* b) nogil:
 *     """Comparison function for sort."""
 *     return <int>((<SIZE_t*>a)[0] - (<SIZE_t*>b)[0])             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((int)((((__pyx_t_7sklearn_4tree_9_splitter_SIZE_t *)__pyx_v_a)[0]) - (((__pyx_t_7sklearn_4tree_9_splitter_SIZE_t *)__pyx_v_b)[0])));
  goto __pyx_L0;

  /* "sklearn/tree/_splitter.pyx":1008
 * 
 * 
 * cdef int compare_SIZE_t(const void* a, const void* b) nogil:             # <<<<<<<<<<<<<<
 *     """Comparison function for sort."""
 *     return <int>((<SIZE_t*>a)[0] - (<SIZE_t*>b)[0])
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "sklearn/tree/_splitter.pyx":1013
 * 
 * 
 * cdef inline void binary_search(INT32_t* sorted_array,             # <<<<<<<<<<<<<<
 *                                INT32_t start, INT32_t end,
 *                                SIZE_t value, SIZE_t* index,
 */

static CYTHON_INLINE void __pyx_f_7sklearn_4tree_9_splitter_binary_search(__pyx_t_7sklearn_4tree_9_splitter_INT32_t *__pyx_v_sorted_array, __pyx_t_7sklearn_4tree_9_splitter_INT32_t __pyx_v_start, __pyx_t_7sklearn_4tree_9_splitter_INT32_t __pyx_v_end, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_value, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_index, __pyx_t_7sklearn_4tree_9_splitter_INT32_t *__pyx_v_new_start) {
  __pyx_t_7sklearn_4tree_9_splitter_INT32_t __pyx_v_pivot;
  int __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":1022
 *     """
 *     cdef INT32_t pivot
 *     index[0] = -1             # <<<<<<<<<<<<<<
 *     while start < end:
 *         pivot = start + (end - start) / 2
 */
  (__pyx_v_index[0]) = -1;

  /* "sklearn/tree/_splitter.pyx":1023
 *     cdef INT32_t pivot
 *     index[0] = -1
 *     while start < end:             # <<<<<<<<<<<<<<
 *         pivot = start + (end - start) / 2
 * 
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_start < __pyx_v_end) != 0);
    if (!__pyx_t_1) break;

    /* "sklearn/tree/_splitter.pyx":1024
 *     index[0] = -1
 *     while start < end:
 *         pivot = start + (end - start) / 2             # <<<<<<<<<<<<<<
 * 
 *         if sorted_array[pivot] == value:
 */
    __pyx_v_pivot = (__pyx_v_start + ((__pyx_v_end - __pyx_v_start) / 2));

    /* "sklearn/tree/_splitter.pyx":1026
 *         pivot = start + (end - start) / 2
 * 
 *         if sorted_array[pivot] == value:             # <<<<<<<<<<<<<<
 *             index[0] = pivot
 *             start = pivot + 1
 */
    __pyx_t_1 = (((__pyx_v_sorted_array[__pyx_v_pivot]) == __pyx_v_value) != 0);
    if (__pyx_t_1) {

      /* "sklearn/tree/_splitter.pyx":1027
 * 
 *         if sorted_array[pivot] == value:
 *             index[0] = pivot             # <<<<<<<<<<<<<<
 *             start = pivot + 1
 *             break
 */
      (__pyx_v_index[0]) = __pyx_v_pivot;

      /* "sklearn/tree/_splitter.pyx":1028
 *         if sorted_array[pivot] == value:
 *             index[0] = pivot
 *             start = pivot + 1             # <<<<<<<<<<<<<<
 *             break
 * 
 */
      __pyx_v_start = (__pyx_v_pivot + 1);

      /* "sklearn/tree/_splitter.pyx":1029
 *             index[0] = pivot
 *             start = pivot + 1
 *             break             # <<<<<<<<<<<<<<
 * 
 *         if sorted_array[pivot] < value:
 */
      goto __pyx_L4_break;
    }

    /* "sklearn/tree/_splitter.pyx":1031
 *             break
 * 
 *         if sorted_array[pivot] < value:             # <<<<<<<<<<<<<<
 *             start = pivot + 1
 *         else:
 */
    __pyx_t_1 = (((__pyx_v_sorted_array[__pyx_v_pivot]) < __pyx_v_value) != 0);
    if (__pyx_t_1) {

      /* "sklearn/tree/_splitter.pyx":1032
 * 
 *         if sorted_array[pivot] < value:
 *             start = pivot + 1             # <<<<<<<<<<<<<<
 *         else:
 *             end = pivot
 */
      __pyx_v_start = (__pyx_v_pivot + 1);
      goto __pyx_L6;
    }
    /*else*/ {

      /* "sklearn/tree/_splitter.pyx":1034
 *             start = pivot + 1
 *         else:
 *             end = pivot             # <<<<<<<<<<<<<<
 *     new_start[0] = start
 * 
 */
      __pyx_v_end = __pyx_v_pivot;
    }
    __pyx_L6:;
  }
  __pyx_L4_break:;

  /* "sklearn/tree/_splitter.pyx":1035
 *         else:
 *             end = pivot
 *     new_start[0] = start             # <<<<<<<<<<<<<<
 * 
 * 
 */
  (__pyx_v_new_start[0]) = __pyx_v_start;

  /* "sklearn/tree/_splitter.pyx":1013
 * 
 * 
 * cdef inline void binary_search(INT32_t* sorted_array,             # <<<<<<<<<<<<<<
 *                                INT32_t start, INT32_t end,
 *                                SIZE_t value, SIZE_t* index,
 */

  /* function exit code */
}

/* "sklearn/tree/_splitter.pyx":1038
 * 
 * 
 * cdef inline void extract_nnz_index_to_samples(INT32_t* X_indices,             # <<<<<<<<<<<<<<
 *                                               DTYPE_t* X_data,
 *                                               INT32_t indptr_start,
 */

static CYTHON_INLINE void __pyx_f_7sklearn_4tree_9_splitter_extract_nnz_index_to_samples(__pyx_t_7sklearn_4tree_9_splitter_INT32_t *__pyx_v_X_indices, __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_v_X_data, __pyx_t_7sklearn_4tree_9_splitter_INT32_t __pyx_v_indptr_start, __pyx_t_7sklearn_4tree_9_splitter_INT32_t __pyx_v_indptr_end, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_samples, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_start, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_end, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_index_to_samples, __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_v_Xf, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_end_negative, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_start_positive) {
  __pyx_t_7sklearn_4tree_9_splitter_INT32_t __pyx_v_k;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_index;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_end_negative_;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_start_positive_;
  __pyx_t_7sklearn_4tree_9_splitter_INT32_t __pyx_t_1;
  __pyx_t_7sklearn_4tree_9_splitter_INT32_t __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "sklearn/tree/_splitter.pyx":1055
 *     cdef INT32_t k
 *     cdef SIZE_t index
 *     cdef SIZE_t end_negative_ = start             # <<<<<<<<<<<<<<
 *     cdef SIZE_t start_positive_ = end
 * 
 */
  __pyx_v_end_negative_ = __pyx_v_start;

  /* "sklearn/tree/_splitter.pyx":1056
 *     cdef SIZE_t index
 *     cdef SIZE_t end_negative_ = start
 *     cdef SIZE_t start_positive_ = end             # <<<<<<<<<<<<<<
 * 
 *     for k in range(indptr_start, indptr_end):
 */
  __pyx_v_start_positive_ = __pyx_v_end;

  /* "sklearn/tree/_splitter.pyx":1058
 *     cdef SIZE_t start_positive_ = end
 * 
 *     for k in range(indptr_start, indptr_end):             # <<<<<<<<<<<<<<
 *         if start <= index_to_samples[X_indices[k]] < end:
 *             if X_data[k] > 0:
 */
  __pyx_t_1 = __pyx_v_indptr_end;
  for (__pyx_t_2 = __pyx_v_indptr_start; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_k = __pyx_t_2;

    /* "sklearn/tree/_splitter.pyx":1059
 * 
 *     for k in range(indptr_start, indptr_end):
 *         if start <= index_to_samples[X_indices[k]] < end:             # <<<<<<<<<<<<<<
 *             if X_data[k] > 0:
 *                 start_positive_ -= 1
 */
    __pyx_t_3 = (__pyx_v_start <= (__pyx_v_index_to_samples[(__pyx_v_X_indices[__pyx_v_k])]));
    if (__pyx_t_3) {
      __pyx_t_3 = ((__pyx_v_index_to_samples[(__pyx_v_X_indices[__pyx_v_k])]) < __pyx_v_end);
    }
    __pyx_t_4 = (__pyx_t_3 != 0);
    if (__pyx_t_4) {

      /* "sklearn/tree/_splitter.pyx":1060
 *     for k in range(indptr_start, indptr_end):
 *         if start <= index_to_samples[X_indices[k]] < end:
 *             if X_data[k] > 0:             # <<<<<<<<<<<<<<
 *                 start_positive_ -= 1
 *                 Xf[start_positive_] = X_data[k]
 */
      __pyx_t_4 = (((__pyx_v_X_data[__pyx_v_k]) > 0.0) != 0);
      if (__pyx_t_4) {

        /* "sklearn/tree/_splitter.pyx":1061
 *         if start <= index_to_samples[X_indices[k]] < end:
 *             if X_data[k] > 0:
 *                 start_positive_ -= 1             # <<<<<<<<<<<<<<
 *                 Xf[start_positive_] = X_data[k]
 *                 index = index_to_samples[X_indices[k]]
 */
        __pyx_v_start_positive_ = (__pyx_v_start_positive_ - 1);

        /* "sklearn/tree/_splitter.pyx":1062
 *             if X_data[k] > 0:
 *                 start_positive_ -= 1
 *                 Xf[start_positive_] = X_data[k]             # <<<<<<<<<<<<<<
 *                 index = index_to_samples[X_indices[k]]
 *                 sparse_swap(index_to_samples, samples, index, start_positive_)
 */
        (__pyx_v_Xf[__pyx_v_start_positive_]) = (__pyx_v_X_data[__pyx_v_k]);

        /* "sklearn/tree/_splitter.pyx":1063
 *                 start_positive_ -= 1
 *                 Xf[start_positive_] = X_data[k]
 *                 index = index_to_samples[X_indices[k]]             # <<<<<<<<<<<<<<
 *                 sparse_swap(index_to_samples, samples, index, start_positive_)
 * 
 */
        __pyx_v_index = (__pyx_v_index_to_samples[(__pyx_v_X_indices[__pyx_v_k])]);

        /* "sklearn/tree/_splitter.pyx":1064
 *                 Xf[start_positive_] = X_data[k]
 *                 index = index_to_samples[X_indices[k]]
 *                 sparse_swap(index_to_samples, samples, index, start_positive_)             # <<<<<<<<<<<<<<
 * 
 * 
 */
        __pyx_f_7sklearn_4tree_9_splitter_sparse_swap(__pyx_v_index_to_samples, __pyx_v_samples, __pyx_v_index, __pyx_v_start_positive_);
        goto __pyx_L6;
      }

      /* "sklearn/tree/_splitter.pyx":1067
 * 
 * 
 *             elif X_data[k] < 0:             # <<<<<<<<<<<<<<
 *                 Xf[end_negative_] = X_data[k]
 *                 index = index_to_samples[X_indices[k]]
 */
      __pyx_t_4 = (((__pyx_v_X_data[__pyx_v_k]) < 0.0) != 0);
      if (__pyx_t_4) {

        /* "sklearn/tree/_splitter.pyx":1068
 * 
 *             elif X_data[k] < 0:
 *                 Xf[end_negative_] = X_data[k]             # <<<<<<<<<<<<<<
 *                 index = index_to_samples[X_indices[k]]
 *                 sparse_swap(index_to_samples, samples, index, end_negative_)
 */
        (__pyx_v_Xf[__pyx_v_end_negative_]) = (__pyx_v_X_data[__pyx_v_k]);

        /* "sklearn/tree/_splitter.pyx":1069
 *             elif X_data[k] < 0:
 *                 Xf[end_negative_] = X_data[k]
 *                 index = index_to_samples[X_indices[k]]             # <<<<<<<<<<<<<<
 *                 sparse_swap(index_to_samples, samples, index, end_negative_)
 *                 end_negative_ += 1
 */
        __pyx_v_index = (__pyx_v_index_to_samples[(__pyx_v_X_indices[__pyx_v_k])]);

        /* "sklearn/tree/_splitter.pyx":1070
 *                 Xf[end_negative_] = X_data[k]
 *                 index = index_to_samples[X_indices[k]]
 *                 sparse_swap(index_to_samples, samples, index, end_negative_)             # <<<<<<<<<<<<<<
 *                 end_negative_ += 1
 * 
 */
        __pyx_f_7sklearn_4tree_9_splitter_sparse_swap(__pyx_v_index_to_samples, __pyx_v_samples, __pyx_v_index, __pyx_v_end_negative_);

        /* "sklearn/tree/_splitter.pyx":1071
 *                 index = index_to_samples[X_indices[k]]
 *                 sparse_swap(index_to_samples, samples, index, end_negative_)
 *                 end_negative_ += 1             # <<<<<<<<<<<<<<
 * 
 *     # Returned values
 */
        __pyx_v_end_negative_ = (__pyx_v_end_negative_ + 1);
        goto __pyx_L6;
      }
      __pyx_L6:;
      goto __pyx_L5;
    }
    __pyx_L5:;
  }

  /* "sklearn/tree/_splitter.pyx":1074
 * 
 *     # Returned values
 *     end_negative[0] = end_negative_             # <<<<<<<<<<<<<<
 *     start_positive[0] = start_positive_
 * 
 */
  (__pyx_v_end_negative[0]) = __pyx_v_end_negative_;

  /* "sklearn/tree/_splitter.pyx":1075
 *     # Returned values
 *     end_negative[0] = end_negative_
 *     start_positive[0] = start_positive_             # <<<<<<<<<<<<<<
 * 
 * 
 */
  (__pyx_v_start_positive[0]) = __pyx_v_start_positive_;

  /* "sklearn/tree/_splitter.pyx":1038
 * 
 * 
 * cdef inline void extract_nnz_index_to_samples(INT32_t* X_indices,             # <<<<<<<<<<<<<<
 *                                               DTYPE_t* X_data,
 *                                               INT32_t indptr_start,
 */

  /* function exit code */
}

/* "sklearn/tree/_splitter.pyx":1078
 * 
 * 
 * cdef inline void extract_nnz_binary_search(INT32_t* X_indices,             # <<<<<<<<<<<<<<
 *                                            DTYPE_t* X_data,
 *                                            INT32_t indptr_start,
 */

static CYTHON_INLINE void __pyx_f_7sklearn_4tree_9_splitter_extract_nnz_binary_search(__pyx_t_7sklearn_4tree_9_splitter_INT32_t *__pyx_v_X_indices, __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_v_X_data, __pyx_t_7sklearn_4tree_9_splitter_INT32_t __pyx_v_indptr_start, __pyx_t_7sklearn_4tree_9_splitter_INT32_t __pyx_v_indptr_end, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_samples, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_start, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_end, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_index_to_samples, __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_v_Xf, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_end_negative, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_start_positive, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_sorted_samples, int *__pyx_v_is_samples_sorted) {
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_p;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_index;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_k;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_end_negative_;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_start_positive_;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":1101
 *     cdef SIZE_t n_samples
 * 
 *     if not is_samples_sorted[0]:             # <<<<<<<<<<<<<<
 *         n_samples = end - start
 *         memcpy(sorted_samples + start, samples + start,
 */
  __pyx_t_1 = ((!((__pyx_v_is_samples_sorted[0]) != 0)) != 0);
  if (__pyx_t_1) {

    /* "sklearn/tree/_splitter.pyx":1102
 * 
 *     if not is_samples_sorted[0]:
 *         n_samples = end - start             # <<<<<<<<<<<<<<
 *         memcpy(sorted_samples + start, samples + start,
 *                n_samples * sizeof(SIZE_t))
 */
    __pyx_v_n_samples = (__pyx_v_end - __pyx_v_start);

    /* "sklearn/tree/_splitter.pyx":1103
 *     if not is_samples_sorted[0]:
 *         n_samples = end - start
 *         memcpy(sorted_samples + start, samples + start,             # <<<<<<<<<<<<<<
 *                n_samples * sizeof(SIZE_t))
 *         qsort(sorted_samples + start, n_samples, sizeof(SIZE_t),
 */
    memcpy((__pyx_v_sorted_samples + __pyx_v_start), (__pyx_v_samples + __pyx_v_start), (__pyx_v_n_samples * (sizeof(__pyx_t_7sklearn_4tree_9_splitter_SIZE_t))));

    /* "sklearn/tree/_splitter.pyx":1105
 *         memcpy(sorted_samples + start, samples + start,
 *                n_samples * sizeof(SIZE_t))
 *         qsort(sorted_samples + start, n_samples, sizeof(SIZE_t),             # <<<<<<<<<<<<<<
 *               compare_SIZE_t)
 *         is_samples_sorted[0] = 1
 */
    qsort((__pyx_v_sorted_samples + __pyx_v_start), __pyx_v_n_samples, (sizeof(__pyx_t_7sklearn_4tree_9_splitter_SIZE_t)), __pyx_f_7sklearn_4tree_9_splitter_compare_SIZE_t);

    /* "sklearn/tree/_splitter.pyx":1107
 *         qsort(sorted_samples + start, n_samples, sizeof(SIZE_t),
 *               compare_SIZE_t)
 *         is_samples_sorted[0] = 1             # <<<<<<<<<<<<<<
 * 
 *     while (indptr_start < indptr_end and
 */
    (__pyx_v_is_samples_sorted[0]) = 1;
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "sklearn/tree/_splitter.pyx":1109
 *         is_samples_sorted[0] = 1
 * 
 *     while (indptr_start < indptr_end and             # <<<<<<<<<<<<<<
 *            sorted_samples[start] > X_indices[indptr_start]):
 *         indptr_start += 1
 */
  while (1) {
    __pyx_t_2 = ((__pyx_v_indptr_start < __pyx_v_indptr_end) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L6_bool_binop_done;
    }

    /* "sklearn/tree/_splitter.pyx":1110
 * 
 *     while (indptr_start < indptr_end and
 *            sorted_samples[start] > X_indices[indptr_start]):             # <<<<<<<<<<<<<<
 *         indptr_start += 1
 * 
 */
    __pyx_t_2 = (((__pyx_v_sorted_samples[__pyx_v_start]) > (__pyx_v_X_indices[__pyx_v_indptr_start])) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L6_bool_binop_done:;
    if (!__pyx_t_1) break;

    /* "sklearn/tree/_splitter.pyx":1111
 *     while (indptr_start < indptr_end and
 *            sorted_samples[start] > X_indices[indptr_start]):
 *         indptr_start += 1             # <<<<<<<<<<<<<<
 * 
 *     while (indptr_start < indptr_end and
 */
    __pyx_v_indptr_start = (__pyx_v_indptr_start + 1);
  }

  /* "sklearn/tree/_splitter.pyx":1113
 *         indptr_start += 1
 * 
 *     while (indptr_start < indptr_end and             # <<<<<<<<<<<<<<
 *            sorted_samples[end - 1] < X_indices[indptr_end - 1]):
 *         indptr_end -= 1
 */
  while (1) {
    __pyx_t_2 = ((__pyx_v_indptr_start < __pyx_v_indptr_end) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L10_bool_binop_done;
    }

    /* "sklearn/tree/_splitter.pyx":1114
 * 
 *     while (indptr_start < indptr_end and
 *            sorted_samples[end - 1] < X_indices[indptr_end - 1]):             # <<<<<<<<<<<<<<
 *         indptr_end -= 1
 * 
 */
    __pyx_t_2 = (((__pyx_v_sorted_samples[(__pyx_v_end - 1)]) < (__pyx_v_X_indices[(__pyx_v_indptr_end - 1)])) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L10_bool_binop_done:;
    if (!__pyx_t_1) break;

    /* "sklearn/tree/_splitter.pyx":1115
 *     while (indptr_start < indptr_end and
 *            sorted_samples[end - 1] < X_indices[indptr_end - 1]):
 *         indptr_end -= 1             # <<<<<<<<<<<<<<
 * 
 *     cdef SIZE_t p = start
 */
    __pyx_v_indptr_end = (__pyx_v_indptr_end - 1);
  }

  /* "sklearn/tree/_splitter.pyx":1117
 *         indptr_end -= 1
 * 
 *     cdef SIZE_t p = start             # <<<<<<<<<<<<<<
 *     cdef SIZE_t index
 *     cdef SIZE_t k
 */
  __pyx_v_p = __pyx_v_start;

  /* "sklearn/tree/_splitter.pyx":1120
 *     cdef SIZE_t index
 *     cdef SIZE_t k
 *     cdef SIZE_t end_negative_ = start             # <<<<<<<<<<<<<<
 *     cdef SIZE_t start_positive_ = end
 * 
 */
  __pyx_v_end_negative_ = __pyx_v_start;

  /* "sklearn/tree/_splitter.pyx":1121
 *     cdef SIZE_t k
 *     cdef SIZE_t end_negative_ = start
 *     cdef SIZE_t start_positive_ = end             # <<<<<<<<<<<<<<
 * 
 *     while (p < end and indptr_start < indptr_end):
 */
  __pyx_v_start_positive_ = __pyx_v_end;

  /* "sklearn/tree/_splitter.pyx":1123
 *     cdef SIZE_t start_positive_ = end
 * 
 *     while (p < end and indptr_start < indptr_end):             # <<<<<<<<<<<<<<
 *         # Find index of sorted_samples[p] in X_indices
 *         binary_search(X_indices, indptr_start, indptr_end,
 */
  while (1) {
    __pyx_t_2 = ((__pyx_v_p < __pyx_v_end) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L14_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_indptr_start < __pyx_v_indptr_end) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L14_bool_binop_done:;
    if (!__pyx_t_1) break;

    /* "sklearn/tree/_splitter.pyx":1125
 *     while (p < end and indptr_start < indptr_end):
 *         # Find index of sorted_samples[p] in X_indices
 *         binary_search(X_indices, indptr_start, indptr_end,             # <<<<<<<<<<<<<<
 *                       sorted_samples[p], &k, &indptr_start)
 * 
 */
    __pyx_f_7sklearn_4tree_9_splitter_binary_search(__pyx_v_X_indices, __pyx_v_indptr_start, __pyx_v_indptr_end, (__pyx_v_sorted_samples[__pyx_v_p]), (&__pyx_v_k), (&__pyx_v_indptr_start));

    /* "sklearn/tree/_splitter.pyx":1128
 *                       sorted_samples[p], &k, &indptr_start)
 * 
 *         if k != -1:             # <<<<<<<<<<<<<<
 *              # If k != -1, we have found a non zero value
 * 
 */
    __pyx_t_1 = ((__pyx_v_k != -1) != 0);
    if (__pyx_t_1) {

      /* "sklearn/tree/_splitter.pyx":1131
 *              # If k != -1, we have found a non zero value
 * 
 *             if X_data[k] > 0:             # <<<<<<<<<<<<<<
 *                 start_positive_ -= 1
 *                 Xf[start_positive_] = X_data[k]
 */
      __pyx_t_1 = (((__pyx_v_X_data[__pyx_v_k]) > 0.0) != 0);
      if (__pyx_t_1) {

        /* "sklearn/tree/_splitter.pyx":1132
 * 
 *             if X_data[k] > 0:
 *                 start_positive_ -= 1             # <<<<<<<<<<<<<<
 *                 Xf[start_positive_] = X_data[k]
 *                 index = index_to_samples[X_indices[k]]
 */
        __pyx_v_start_positive_ = (__pyx_v_start_positive_ - 1);

        /* "sklearn/tree/_splitter.pyx":1133
 *             if X_data[k] > 0:
 *                 start_positive_ -= 1
 *                 Xf[start_positive_] = X_data[k]             # <<<<<<<<<<<<<<
 *                 index = index_to_samples[X_indices[k]]
 *                 sparse_swap(index_to_samples, samples, index, start_positive_)
 */
        (__pyx_v_Xf[__pyx_v_start_positive_]) = (__pyx_v_X_data[__pyx_v_k]);

        /* "sklearn/tree/_splitter.pyx":1134
 *                 start_positive_ -= 1
 *                 Xf[start_positive_] = X_data[k]
 *                 index = index_to_samples[X_indices[k]]             # <<<<<<<<<<<<<<
 *                 sparse_swap(index_to_samples, samples, index, start_positive_)
 * 
 */
        __pyx_v_index = (__pyx_v_index_to_samples[(__pyx_v_X_indices[__pyx_v_k])]);

        /* "sklearn/tree/_splitter.pyx":1135
 *                 Xf[start_positive_] = X_data[k]
 *                 index = index_to_samples[X_indices[k]]
 *                 sparse_swap(index_to_samples, samples, index, start_positive_)             # <<<<<<<<<<<<<<
 * 
 * 
 */
        __pyx_f_7sklearn_4tree_9_splitter_sparse_swap(__pyx_v_index_to_samples, __pyx_v_samples, __pyx_v_index, __pyx_v_start_positive_);
        goto __pyx_L17;
      }

      /* "sklearn/tree/_splitter.pyx":1138
 * 
 * 
 *             elif X_data[k] < 0:             # <<<<<<<<<<<<<<
 *                 Xf[end_negative_] = X_data[k]
 *                 index = index_to_samples[X_indices[k]]
 */
      __pyx_t_1 = (((__pyx_v_X_data[__pyx_v_k]) < 0.0) != 0);
      if (__pyx_t_1) {

        /* "sklearn/tree/_splitter.pyx":1139
 * 
 *             elif X_data[k] < 0:
 *                 Xf[end_negative_] = X_data[k]             # <<<<<<<<<<<<<<
 *                 index = index_to_samples[X_indices[k]]
 *                 sparse_swap(index_to_samples, samples, index, end_negative_)
 */
        (__pyx_v_Xf[__pyx_v_end_negative_]) = (__pyx_v_X_data[__pyx_v_k]);

        /* "sklearn/tree/_splitter.pyx":1140
 *             elif X_data[k] < 0:
 *                 Xf[end_negative_] = X_data[k]
 *                 index = index_to_samples[X_indices[k]]             # <<<<<<<<<<<<<<
 *                 sparse_swap(index_to_samples, samples, index, end_negative_)
 *                 end_negative_ += 1
 */
        __pyx_v_index = (__pyx_v_index_to_samples[(__pyx_v_X_indices[__pyx_v_k])]);

        /* "sklearn/tree/_splitter.pyx":1141
 *                 Xf[end_negative_] = X_data[k]
 *                 index = index_to_samples[X_indices[k]]
 *                 sparse_swap(index_to_samples, samples, index, end_negative_)             # <<<<<<<<<<<<<<
 *                 end_negative_ += 1
 *         p += 1
 */
        __pyx_f_7sklearn_4tree_9_splitter_sparse_swap(__pyx_v_index_to_samples, __pyx_v_samples, __pyx_v_index, __pyx_v_end_negative_);

        /* "sklearn/tree/_splitter.pyx":1142
 *                 index = index_to_samples[X_indices[k]]
 *                 sparse_swap(index_to_samples, samples, index, end_negative_)
 *                 end_negative_ += 1             # <<<<<<<<<<<<<<
 *         p += 1
 * 
 */
        __pyx_v_end_negative_ = (__pyx_v_end_negative_ + 1);
        goto __pyx_L17;
      }
      __pyx_L17:;
      goto __pyx_L16;
    }
    __pyx_L16:;

    /* "sklearn/tree/_splitter.pyx":1143
 *                 sparse_swap(index_to_samples, samples, index, end_negative_)
 *                 end_negative_ += 1
 *         p += 1             # <<<<<<<<<<<<<<
 * 
 *     # Returned values
 */
    __pyx_v_p = (__pyx_v_p + 1);
  }

  /* "sklearn/tree/_splitter.pyx":1146
 * 
 *     # Returned values
 *     end_negative[0] = end_negative_             # <<<<<<<<<<<<<<
 *     start_positive[0] = start_positive_
 * 
 */
  (__pyx_v_end_negative[0]) = __pyx_v_end_negative_;

  /* "sklearn/tree/_splitter.pyx":1147
 *     # Returned values
 *     end_negative[0] = end_negative_
 *     start_positive[0] = start_positive_             # <<<<<<<<<<<<<<
 * 
 * 
 */
  (__pyx_v_start_positive[0]) = __pyx_v_start_positive_;

  /* "sklearn/tree/_splitter.pyx":1078
 * 
 * 
 * cdef inline void extract_nnz_binary_search(INT32_t* X_indices,             # <<<<<<<<<<<<<<
 *                                            DTYPE_t* X_data,
 *                                            INT32_t indptr_start,
 */

  /* function exit code */
}

/* "sklearn/tree/_splitter.pyx":1150
 * 
 * 
 * cdef inline void sparse_swap(SIZE_t* index_to_samples, SIZE_t* samples,             # <<<<<<<<<<<<<<
 *                              SIZE_t pos_1, SIZE_t pos_2) nogil  :
 *     """Swap sample pos_1 and pos_2 preserving sparse invariant."""
 */

static CYTHON_INLINE void __pyx_f_7sklearn_4tree_9_splitter_sparse_swap(__pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_index_to_samples, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_samples, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_pos_1, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_pos_2) {
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_1;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":1153
 *                              SIZE_t pos_1, SIZE_t pos_2) nogil  :
 *     """Swap sample pos_1 and pos_2 preserving sparse invariant."""
 *     samples[pos_1], samples[pos_2] =  samples[pos_2], samples[pos_1]             # <<<<<<<<<<<<<<
 *     index_to_samples[samples[pos_1]] = pos_1
 *     index_to_samples[samples[pos_2]] = pos_2
 */
  __pyx_t_1 = (__pyx_v_samples[__pyx_v_pos_2]);
  __pyx_t_2 = (__pyx_v_samples[__pyx_v_pos_1]);
  (__pyx_v_samples[__pyx_v_pos_1]) = __pyx_t_1;
  (__pyx_v_samples[__pyx_v_pos_2]) = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":1154
 *     """Swap sample pos_1 and pos_2 preserving sparse invariant."""
 *     samples[pos_1], samples[pos_2] =  samples[pos_2], samples[pos_1]
 *     index_to_samples[samples[pos_1]] = pos_1             # <<<<<<<<<<<<<<
 *     index_to_samples[samples[pos_2]] = pos_2
 * 
 */
  (__pyx_v_index_to_samples[(__pyx_v_samples[__pyx_v_pos_1])]) = __pyx_v_pos_1;

  /* "sklearn/tree/_splitter.pyx":1155
 *     samples[pos_1], samples[pos_2] =  samples[pos_2], samples[pos_1]
 *     index_to_samples[samples[pos_1]] = pos_1
 *     index_to_samples[samples[pos_2]] = pos_2             # <<<<<<<<<<<<<<
 * 
 * 
 */
  (__pyx_v_index_to_samples[(__pyx_v_samples[__pyx_v_pos_2])]) = __pyx_v_pos_2;

  /* "sklearn/tree/_splitter.pyx":1150
 * 
 * 
 * cdef inline void sparse_swap(SIZE_t* index_to_samples, SIZE_t* samples,             # <<<<<<<<<<<<<<
 *                              SIZE_t pos_1, SIZE_t pos_2) nogil  :
 *     """Swap sample pos_1 and pos_2 preserving sparse invariant."""
 */

  /* function exit code */
}

/* "sklearn/tree/_splitter.pyx":1161
 *     """Splitter for finding the best split, using the sparse data."""
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return (BestSparseSplitter, (self.criterion,
 *                                      self.max_features,
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_4tree_9_splitter_18BestSparseSplitter_1__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7sklearn_4tree_9_splitter_18BestSparseSplitter_1__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_4tree_9_splitter_18BestSparseSplitter___reduce__(((struct __pyx_obj_7sklearn_4tree_9_splitter_BestSparseSplitter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_4tree_9_splitter_18BestSparseSplitter___reduce__(struct __pyx_obj_7sklearn_4tree_9_splitter_BestSparseSplitter *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "sklearn/tree/_splitter.pyx":1162
 * 
 *     def __reduce__(self):
 *         return (BestSparseSplitter, (self.criterion,             # <<<<<<<<<<<<<<
 *                                      self.max_features,
 *                                      self.min_samples_leaf,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "sklearn/tree/_splitter.pyx":1163
 *     def __reduce__(self):
 *         return (BestSparseSplitter, (self.criterion,
 *                                      self.max_features,             # <<<<<<<<<<<<<<
 *                                      self.min_samples_leaf,
 *                                      self.min_weight_leaf,
 */
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->__pyx_base.__pyx_base.max_features); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);

  /* "sklearn/tree/_splitter.pyx":1164
 *         return (BestSparseSplitter, (self.criterion,
 *                                      self.max_features,
 *                                      self.min_samples_leaf,             # <<<<<<<<<<<<<<
 *                                      self.min_weight_leaf,
 *                                      self.random_state,
 */
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->__pyx_base.__pyx_base.min_samples_leaf); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);

  /* "sklearn/tree/_splitter.pyx":1165
 *                                      self.max_features,
 *                                      self.min_samples_leaf,
 *                                      self.min_weight_leaf,             # <<<<<<<<<<<<<<
 *                                      self.random_state,
 *                                      self.presort), self.__getstate__())
 */
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.min_weight_leaf); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1165; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);

  /* "sklearn/tree/_splitter.pyx":1167
 *                                      self.min_weight_leaf,
 *                                      self.random_state,
 *                                      self.presort), self.__getstate__())             # <<<<<<<<<<<<<<
 * 
 *     cdef void node_split(self, double impurity, SplitRecord* split,
 */
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.presort); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1167; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);

  /* "sklearn/tree/_splitter.pyx":1162
 * 
 *     def __reduce__(self):
 *         return (BestSparseSplitter, (self.criterion,             # <<<<<<<<<<<<<<
 *                                      self.max_features,
 *                                      self.min_samples_leaf,
 */
  __pyx_t_5 = PyTuple_New(6); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1162; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.criterion));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.criterion));
  PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.criterion));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.random_state);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.random_state);
  PyTuple_SET_ITEM(__pyx_t_5, 4, __pyx_v_self->__pyx_base.__pyx_base.random_state);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 5, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;

  /* "sklearn/tree/_splitter.pyx":1167
 *                                      self.min_weight_leaf,
 *                                      self.random_state,
 *                                      self.presort), self.__getstate__())             # <<<<<<<<<<<<<<
 * 
 *     cdef void node_split(self, double impurity, SplitRecord* split,
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getstate); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1167; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1167; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1167; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sklearn/tree/_splitter.pyx":1162
 * 
 *     def __reduce__(self):
 *         return (BestSparseSplitter, (self.criterion,             # <<<<<<<<<<<<<<
 *                                      self.max_features,
 *                                      self.min_samples_leaf,
 */
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1162; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)((PyObject*)__pyx_ptype_7sklearn_4tree_9_splitter_BestSparseSplitter)));
  __Pyx_GIVEREF(((PyObject *)((PyObject*)__pyx_ptype_7sklearn_4tree_9_splitter_BestSparseSplitter)));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)((PyObject*)__pyx_ptype_7sklearn_4tree_9_splitter_BestSparseSplitter)));
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_4);
  __pyx_t_5 = 0;
  __pyx_t_4 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "sklearn/tree/_splitter.pyx":1161
 *     """Splitter for finding the best split, using the sparse data."""
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return (BestSparseSplitter, (self.criterion,
 *                                      self.max_features,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("sklearn.tree._splitter.BestSparseSplitter.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/tree/_splitter.pyx":1169
 *                                      self.presort), self.__getstate__())
 * 
 *     cdef void node_split(self, double impurity, SplitRecord* split,             # <<<<<<<<<<<<<<
 *                          SIZE_t* n_constant_features) nogil:
 *         """Find the best split on node samples[start:end], using sparse
 */

static void __pyx_f_7sklearn_4tree_9_splitter_18BestSparseSplitter_node_split(struct __pyx_obj_7sklearn_4tree_9_splitter_BestSparseSplitter *__pyx_v_self, double __pyx_v_impurity, struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord *__pyx_v_split, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_n_constant_features) {
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_start;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_end;
  CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_INT32_t *__pyx_v_X_indices;
  CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_INT32_t *__pyx_v_X_indptr;
  CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_v_X_data;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_features;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_constant_features;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_features;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_v_Xf;
  CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_sorted_samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_index_to_samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_max_features;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_min_samples_leaf;
  double __pyx_v_min_weight_leaf;
  __pyx_t_7sklearn_4tree_9_splitter_UINT32_t *__pyx_v_random_state;
  struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord __pyx_v_best;
  struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord __pyx_v_current;
  double __pyx_v_current_proxy_improvement;
  double __pyx_v_best_proxy_improvement;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_f_i;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_f_j;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_p;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_tmp;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_visited_features;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_found_constants;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_drawn_constants;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_known_constants;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_total_constants;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_p_next;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_p_prev;
  int __pyx_v_is_samples_sorted;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_start_positive;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_end_negative;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_t_1;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_2;
  __pyx_t_7sklearn_4tree_9_splitter_INT32_t *__pyx_t_3;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_t_4;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_8;

  /* "sklearn/tree/_splitter.pyx":1175
 *         """
 *         # Find the best split
 *         cdef SIZE_t* samples = self.samples             # <<<<<<<<<<<<<<
 *         cdef SIZE_t start = self.start
 *         cdef SIZE_t end = self.end
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.__pyx_base.samples;
  __pyx_v_samples = __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":1176
 *         # Find the best split
 *         cdef SIZE_t* samples = self.samples
 *         cdef SIZE_t start = self.start             # <<<<<<<<<<<<<<
 *         cdef SIZE_t end = self.end
 * 
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.start;
  __pyx_v_start = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":1177
 *         cdef SIZE_t* samples = self.samples
 *         cdef SIZE_t start = self.start
 *         cdef SIZE_t end = self.end             # <<<<<<<<<<<<<<
 * 
 *         cdef INT32_t* X_indices = self.X_indices
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.end;
  __pyx_v_end = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":1179
 *         cdef SIZE_t end = self.end
 * 
 *         cdef INT32_t* X_indices = self.X_indices             # <<<<<<<<<<<<<<
 *         cdef INT32_t* X_indptr = self.X_indptr
 *         cdef DTYPE_t* X_data = self.X_data
 */
  __pyx_t_3 = __pyx_v_self->__pyx_base.X_indices;
  __pyx_v_X_indices = __pyx_t_3;

  /* "sklearn/tree/_splitter.pyx":1180
 * 
 *         cdef INT32_t* X_indices = self.X_indices
 *         cdef INT32_t* X_indptr = self.X_indptr             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t* X_data = self.X_data
 * 
 */
  __pyx_t_3 = __pyx_v_self->__pyx_base.X_indptr;
  __pyx_v_X_indptr = __pyx_t_3;

  /* "sklearn/tree/_splitter.pyx":1181
 *         cdef INT32_t* X_indices = self.X_indices
 *         cdef INT32_t* X_indptr = self.X_indptr
 *         cdef DTYPE_t* X_data = self.X_data             # <<<<<<<<<<<<<<
 * 
 *         cdef SIZE_t* features = self.features
 */
  __pyx_t_4 = __pyx_v_self->__pyx_base.X_data;
  __pyx_v_X_data = __pyx_t_4;

  /* "sklearn/tree/_splitter.pyx":1183
 *         cdef DTYPE_t* X_data = self.X_data
 * 
 *         cdef SIZE_t* features = self.features             # <<<<<<<<<<<<<<
 *         cdef SIZE_t* constant_features = self.constant_features
 *         cdef SIZE_t n_features = self.n_features
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.__pyx_base.features;
  __pyx_v_features = __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":1184
 * 
 *         cdef SIZE_t* features = self.features
 *         cdef SIZE_t* constant_features = self.constant_features             # <<<<<<<<<<<<<<
 *         cdef SIZE_t n_features = self.n_features
 * 
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.__pyx_base.constant_features;
  __pyx_v_constant_features = __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":1185
 *         cdef SIZE_t* features = self.features
 *         cdef SIZE_t* constant_features = self.constant_features
 *         cdef SIZE_t n_features = self.n_features             # <<<<<<<<<<<<<<
 * 
 *         cdef DTYPE_t* Xf = self.feature_values
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.n_features;
  __pyx_v_n_features = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":1187
 *         cdef SIZE_t n_features = self.n_features
 * 
 *         cdef DTYPE_t* Xf = self.feature_values             # <<<<<<<<<<<<<<
 *         cdef SIZE_t* sorted_samples = self.sorted_samples
 *         cdef SIZE_t* index_to_samples = self.index_to_samples
 */
  __pyx_t_4 = __pyx_v_self->__pyx_base.__pyx_base.feature_values;
  __pyx_v_Xf = __pyx_t_4;

  /* "sklearn/tree/_splitter.pyx":1188
 * 
 *         cdef DTYPE_t* Xf = self.feature_values
 *         cdef SIZE_t* sorted_samples = self.sorted_samples             # <<<<<<<<<<<<<<
 *         cdef SIZE_t* index_to_samples = self.index_to_samples
 *         cdef SIZE_t max_features = self.max_features
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.sorted_samples;
  __pyx_v_sorted_samples = __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":1189
 *         cdef DTYPE_t* Xf = self.feature_values
 *         cdef SIZE_t* sorted_samples = self.sorted_samples
 *         cdef SIZE_t* index_to_samples = self.index_to_samples             # <<<<<<<<<<<<<<
 *         cdef SIZE_t max_features = self.max_features
 *         cdef SIZE_t min_samples_leaf = self.min_samples_leaf
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.index_to_samples;
  __pyx_v_index_to_samples = __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":1190
 *         cdef SIZE_t* sorted_samples = self.sorted_samples
 *         cdef SIZE_t* index_to_samples = self.index_to_samples
 *         cdef SIZE_t max_features = self.max_features             # <<<<<<<<<<<<<<
 *         cdef SIZE_t min_samples_leaf = self.min_samples_leaf
 *         cdef double min_weight_leaf = self.min_weight_leaf
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.max_features;
  __pyx_v_max_features = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":1191
 *         cdef SIZE_t* index_to_samples = self.index_to_samples
 *         cdef SIZE_t max_features = self.max_features
 *         cdef SIZE_t min_samples_leaf = self.min_samples_leaf             # <<<<<<<<<<<<<<
 *         cdef double min_weight_leaf = self.min_weight_leaf
 *         cdef UINT32_t* random_state = &self.rand_r_state
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.min_samples_leaf;
  __pyx_v_min_samples_leaf = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":1192
 *         cdef SIZE_t max_features = self.max_features
 *         cdef SIZE_t min_samples_leaf = self.min_samples_leaf
 *         cdef double min_weight_leaf = self.min_weight_leaf             # <<<<<<<<<<<<<<
 *         cdef UINT32_t* random_state = &self.rand_r_state
 * 
 */
  __pyx_t_5 = __pyx_v_self->__pyx_base.__pyx_base.min_weight_leaf;
  __pyx_v_min_weight_leaf = __pyx_t_5;

  /* "sklearn/tree/_splitter.pyx":1193
 *         cdef SIZE_t min_samples_leaf = self.min_samples_leaf
 *         cdef double min_weight_leaf = self.min_weight_leaf
 *         cdef UINT32_t* random_state = &self.rand_r_state             # <<<<<<<<<<<<<<
 * 
 *         cdef SplitRecord best, current
 */
  __pyx_v_random_state = (&__pyx_v_self->__pyx_base.__pyx_base.rand_r_state);

  /* "sklearn/tree/_splitter.pyx":1196
 * 
 *         cdef SplitRecord best, current
 *         _init_split(&best, end)             # <<<<<<<<<<<<<<
 *         cdef double current_proxy_improvement = - INFINITY
 *         cdef double best_proxy_improvement = - INFINITY
 */
  __pyx_f_7sklearn_4tree_9_splitter__init_split((&__pyx_v_best), __pyx_v_end);

  /* "sklearn/tree/_splitter.pyx":1197
 *         cdef SplitRecord best, current
 *         _init_split(&best, end)
 *         cdef double current_proxy_improvement = - INFINITY             # <<<<<<<<<<<<<<
 *         cdef double best_proxy_improvement = - INFINITY
 * 
 */
  __pyx_v_current_proxy_improvement = (-__pyx_v_7sklearn_4tree_9_splitter_INFINITY);

  /* "sklearn/tree/_splitter.pyx":1198
 *         _init_split(&best, end)
 *         cdef double current_proxy_improvement = - INFINITY
 *         cdef double best_proxy_improvement = - INFINITY             # <<<<<<<<<<<<<<
 * 
 *         cdef SIZE_t f_i = n_features
 */
  __pyx_v_best_proxy_improvement = (-__pyx_v_7sklearn_4tree_9_splitter_INFINITY);

  /* "sklearn/tree/_splitter.pyx":1200
 *         cdef double best_proxy_improvement = - INFINITY
 * 
 *         cdef SIZE_t f_i = n_features             # <<<<<<<<<<<<<<
 *         cdef SIZE_t f_j, p, tmp
 *         cdef SIZE_t n_visited_features = 0
 */
  __pyx_v_f_i = __pyx_v_n_features;

  /* "sklearn/tree/_splitter.pyx":1202
 *         cdef SIZE_t f_i = n_features
 *         cdef SIZE_t f_j, p, tmp
 *         cdef SIZE_t n_visited_features = 0             # <<<<<<<<<<<<<<
 *         # Number of features discovered to be constant during the split search
 *         cdef SIZE_t n_found_constants = 0
 */
  __pyx_v_n_visited_features = 0;

  /* "sklearn/tree/_splitter.pyx":1204
 *         cdef SIZE_t n_visited_features = 0
 *         # Number of features discovered to be constant during the split search
 *         cdef SIZE_t n_found_constants = 0             # <<<<<<<<<<<<<<
 *         # Number of features known to be constant and drawn without replacement
 *         cdef SIZE_t n_drawn_constants = 0
 */
  __pyx_v_n_found_constants = 0;

  /* "sklearn/tree/_splitter.pyx":1206
 *         cdef SIZE_t n_found_constants = 0
 *         # Number of features known to be constant and drawn without replacement
 *         cdef SIZE_t n_drawn_constants = 0             # <<<<<<<<<<<<<<
 *         cdef SIZE_t n_known_constants = n_constant_features[0]
 *         # n_total_constants = n_known_constants + n_found_constants
 */
  __pyx_v_n_drawn_constants = 0;

  /* "sklearn/tree/_splitter.pyx":1207
 *         # Number of features known to be constant and drawn without replacement
 *         cdef SIZE_t n_drawn_constants = 0
 *         cdef SIZE_t n_known_constants = n_constant_features[0]             # <<<<<<<<<<<<<<
 *         # n_total_constants = n_known_constants + n_found_constants
 *         cdef SIZE_t n_total_constants = n_known_constants
 */
  __pyx_v_n_known_constants = (__pyx_v_n_constant_features[0]);

  /* "sklearn/tree/_splitter.pyx":1209
 *         cdef SIZE_t n_known_constants = n_constant_features[0]
 *         # n_total_constants = n_known_constants + n_found_constants
 *         cdef SIZE_t n_total_constants = n_known_constants             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t current_feature_value
 * 
 */
  __pyx_v_n_total_constants = __pyx_v_n_known_constants;

  /* "sklearn/tree/_splitter.pyx":1214
 *         cdef SIZE_t p_next
 *         cdef SIZE_t p_prev
 *         cdef bint is_samples_sorted = 0  # indicate is sorted_samples is             # <<<<<<<<<<<<<<
 *                                          # inititialized
 * 
 */
  __pyx_v_is_samples_sorted = 0;

  /* "sklearn/tree/_splitter.pyx":1231
 *         # newly discovered constant features to spare computation on descendant
 *         # nodes.
 *         while (f_i > n_total_constants and  # Stop early if remaining features             # <<<<<<<<<<<<<<
 *                                             # are constant
 *                 (n_visited_features < max_features or
 */
  while (1) {
    __pyx_t_7 = ((__pyx_v_f_i > __pyx_v_n_total_constants) != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L5_bool_binop_done;
    }

    /* "sklearn/tree/_splitter.pyx":1233
 *         while (f_i > n_total_constants and  # Stop early if remaining features
 *                                             # are constant
 *                 (n_visited_features < max_features or             # <<<<<<<<<<<<<<
 *                  # At least one drawn features must be non constant
 *                  n_visited_features <= n_found_constants + n_drawn_constants)):
 */
    __pyx_t_7 = ((__pyx_v_n_visited_features < __pyx_v_max_features) != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L5_bool_binop_done;
    }

    /* "sklearn/tree/_splitter.pyx":1235
 *                 (n_visited_features < max_features or
 *                  # At least one drawn features must be non constant
 *                  n_visited_features <= n_found_constants + n_drawn_constants)):             # <<<<<<<<<<<<<<
 * 
 *             n_visited_features += 1
 */
    __pyx_t_7 = ((__pyx_v_n_visited_features <= (__pyx_v_n_found_constants + __pyx_v_n_drawn_constants)) != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L5_bool_binop_done:;
    if (!__pyx_t_6) break;

    /* "sklearn/tree/_splitter.pyx":1237
 *                  n_visited_features <= n_found_constants + n_drawn_constants)):
 * 
 *             n_visited_features += 1             # <<<<<<<<<<<<<<
 * 
 *             # Loop invariant: elements of features in
 */
    __pyx_v_n_visited_features = (__pyx_v_n_visited_features + 1);

    /* "sklearn/tree/_splitter.pyx":1251
 * 
 *             # Draw a feature at random
 *             f_j = rand_int(n_drawn_constants, f_i - n_found_constants,             # <<<<<<<<<<<<<<
 *                            random_state)
 * 
 */
    __pyx_v_f_j = __pyx_f_7sklearn_4tree_6_utils_rand_int(__pyx_v_n_drawn_constants, (__pyx_v_f_i - __pyx_v_n_found_constants), __pyx_v_random_state);

    /* "sklearn/tree/_splitter.pyx":1254
 *                            random_state)
 * 
 *             if f_j < n_known_constants:             # <<<<<<<<<<<<<<
 *                 # f_j in the interval [n_drawn_constants, n_known_constants[
 *                 tmp = features[f_j]
 */
    __pyx_t_6 = ((__pyx_v_f_j < __pyx_v_n_known_constants) != 0);
    if (__pyx_t_6) {

      /* "sklearn/tree/_splitter.pyx":1256
 *             if f_j < n_known_constants:
 *                 # f_j in the interval [n_drawn_constants, n_known_constants[
 *                 tmp = features[f_j]             # <<<<<<<<<<<<<<
 *                 features[f_j] = features[n_drawn_constants]
 *                 features[n_drawn_constants] = tmp
 */
      __pyx_v_tmp = (__pyx_v_features[__pyx_v_f_j]);

      /* "sklearn/tree/_splitter.pyx":1257
 *                 # f_j in the interval [n_drawn_constants, n_known_constants[
 *                 tmp = features[f_j]
 *                 features[f_j] = features[n_drawn_constants]             # <<<<<<<<<<<<<<
 *                 features[n_drawn_constants] = tmp
 * 
 */
      (__pyx_v_features[__pyx_v_f_j]) = (__pyx_v_features[__pyx_v_n_drawn_constants]);

      /* "sklearn/tree/_splitter.pyx":1258
 *                 tmp = features[f_j]
 *                 features[f_j] = features[n_drawn_constants]
 *                 features[n_drawn_constants] = tmp             # <<<<<<<<<<<<<<
 * 
 *                 n_drawn_constants += 1
 */
      (__pyx_v_features[__pyx_v_n_drawn_constants]) = __pyx_v_tmp;

      /* "sklearn/tree/_splitter.pyx":1260
 *                 features[n_drawn_constants] = tmp
 * 
 *                 n_drawn_constants += 1             # <<<<<<<<<<<<<<
 * 
 *             else:
 */
      __pyx_v_n_drawn_constants = (__pyx_v_n_drawn_constants + 1);
      goto __pyx_L8;
    }
    /*else*/ {

      /* "sklearn/tree/_splitter.pyx":1264
 *             else:
 *                 # f_j in the interval [n_known_constants, f_i - n_found_constants[
 *                 f_j += n_found_constants             # <<<<<<<<<<<<<<
 *                 # f_j in the interval [n_total_constants, f_i[
 * 
 */
      __pyx_v_f_j = (__pyx_v_f_j + __pyx_v_n_found_constants);

      /* "sklearn/tree/_splitter.pyx":1267
 *                 # f_j in the interval [n_total_constants, f_i[
 * 
 *                 current.feature = features[f_j]             # <<<<<<<<<<<<<<
 *                 self.extract_nnz(current.feature,
 *                                  &end_negative, &start_positive,
 */
      __pyx_v_current.feature = (__pyx_v_features[__pyx_v_f_j]);

      /* "sklearn/tree/_splitter.pyx":1268
 * 
 *                 current.feature = features[f_j]
 *                 self.extract_nnz(current.feature,             # <<<<<<<<<<<<<<
 *                                  &end_negative, &start_positive,
 *                                  &is_samples_sorted)
 */
      __pyx_f_7sklearn_4tree_9_splitter_18BaseSparseSplitter_extract_nnz(((struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *)__pyx_v_self), __pyx_v_current.feature, (&__pyx_v_end_negative), (&__pyx_v_start_positive), (&__pyx_v_is_samples_sorted));

      /* "sklearn/tree/_splitter.pyx":1273
 * 
 *                 # Sort the positive and negative parts of `Xf`
 *                 sort(Xf + start, samples + start, end_negative - start)             # <<<<<<<<<<<<<<
 *                 sort(Xf + start_positive, samples + start_positive,
 *                      end - start_positive)
 */
      __pyx_f_7sklearn_4tree_9_splitter_sort((__pyx_v_Xf + __pyx_v_start), (__pyx_v_samples + __pyx_v_start), (__pyx_v_end_negative - __pyx_v_start));

      /* "sklearn/tree/_splitter.pyx":1274
 *                 # Sort the positive and negative parts of `Xf`
 *                 sort(Xf + start, samples + start, end_negative - start)
 *                 sort(Xf + start_positive, samples + start_positive,             # <<<<<<<<<<<<<<
 *                      end - start_positive)
 * 
 */
      __pyx_f_7sklearn_4tree_9_splitter_sort((__pyx_v_Xf + __pyx_v_start_positive), (__pyx_v_samples + __pyx_v_start_positive), (__pyx_v_end - __pyx_v_start_positive));

      /* "sklearn/tree/_splitter.pyx":1278
 * 
 *                 # Update index_to_samples to take into account the sort
 *                 for p in range(start, end_negative):             # <<<<<<<<<<<<<<
 *                     index_to_samples[samples[p]] = p
 *                 for p in range(start_positive, end):
 */
      __pyx_t_2 = __pyx_v_end_negative;
      for (__pyx_t_8 = __pyx_v_start; __pyx_t_8 < __pyx_t_2; __pyx_t_8+=1) {
        __pyx_v_p = __pyx_t_8;

        /* "sklearn/tree/_splitter.pyx":1279
 *                 # Update index_to_samples to take into account the sort
 *                 for p in range(start, end_negative):
 *                     index_to_samples[samples[p]] = p             # <<<<<<<<<<<<<<
 *                 for p in range(start_positive, end):
 *                     index_to_samples[samples[p]] = p
 */
        (__pyx_v_index_to_samples[(__pyx_v_samples[__pyx_v_p])]) = __pyx_v_p;
      }

      /* "sklearn/tree/_splitter.pyx":1280
 *                 for p in range(start, end_negative):
 *                     index_to_samples[samples[p]] = p
 *                 for p in range(start_positive, end):             # <<<<<<<<<<<<<<
 *                     index_to_samples[samples[p]] = p
 * 
 */
      __pyx_t_2 = __pyx_v_end;
      for (__pyx_t_8 = __pyx_v_start_positive; __pyx_t_8 < __pyx_t_2; __pyx_t_8+=1) {
        __pyx_v_p = __pyx_t_8;

        /* "sklearn/tree/_splitter.pyx":1281
 *                     index_to_samples[samples[p]] = p
 *                 for p in range(start_positive, end):
 *                     index_to_samples[samples[p]] = p             # <<<<<<<<<<<<<<
 * 
 *                 # Add one or two zeros in Xf, if there is any
 */
        (__pyx_v_index_to_samples[(__pyx_v_samples[__pyx_v_p])]) = __pyx_v_p;
      }

      /* "sklearn/tree/_splitter.pyx":1284
 * 
 *                 # Add one or two zeros in Xf, if there is any
 *                 if end_negative < start_positive:             # <<<<<<<<<<<<<<
 *                     start_positive -= 1
 *                     Xf[start_positive] = 0.
 */
      __pyx_t_6 = ((__pyx_v_end_negative < __pyx_v_start_positive) != 0);
      if (__pyx_t_6) {

        /* "sklearn/tree/_splitter.pyx":1285
 *                 # Add one or two zeros in Xf, if there is any
 *                 if end_negative < start_positive:
 *                     start_positive -= 1             # <<<<<<<<<<<<<<
 *                     Xf[start_positive] = 0.
 * 
 */
        __pyx_v_start_positive = (__pyx_v_start_positive - 1);

        /* "sklearn/tree/_splitter.pyx":1286
 *                 if end_negative < start_positive:
 *                     start_positive -= 1
 *                     Xf[start_positive] = 0.             # <<<<<<<<<<<<<<
 * 
 *                     if end_negative != start_positive:
 */
        (__pyx_v_Xf[__pyx_v_start_positive]) = 0.;

        /* "sklearn/tree/_splitter.pyx":1288
 *                     Xf[start_positive] = 0.
 * 
 *                     if end_negative != start_positive:             # <<<<<<<<<<<<<<
 *                         Xf[end_negative] = 0.
 *                         end_negative += 1
 */
        __pyx_t_6 = ((__pyx_v_end_negative != __pyx_v_start_positive) != 0);
        if (__pyx_t_6) {

          /* "sklearn/tree/_splitter.pyx":1289
 * 
 *                     if end_negative != start_positive:
 *                         Xf[end_negative] = 0.             # <<<<<<<<<<<<<<
 *                         end_negative += 1
 * 
 */
          (__pyx_v_Xf[__pyx_v_end_negative]) = 0.;

          /* "sklearn/tree/_splitter.pyx":1290
 *                     if end_negative != start_positive:
 *                         Xf[end_negative] = 0.
 *                         end_negative += 1             # <<<<<<<<<<<<<<
 * 
 *                 if Xf[end - 1] <= Xf[start] + FEATURE_THRESHOLD:
 */
          __pyx_v_end_negative = (__pyx_v_end_negative + 1);
          goto __pyx_L14;
        }
        __pyx_L14:;
        goto __pyx_L13;
      }
      __pyx_L13:;

      /* "sklearn/tree/_splitter.pyx":1292
 *                         end_negative += 1
 * 
 *                 if Xf[end - 1] <= Xf[start] + FEATURE_THRESHOLD:             # <<<<<<<<<<<<<<
 *                     features[f_j] = features[n_total_constants]
 *                     features[n_total_constants] = current.feature
 */
      __pyx_t_6 = (((__pyx_v_Xf[(__pyx_v_end - 1)]) <= ((__pyx_v_Xf[__pyx_v_start]) + __pyx_v_7sklearn_4tree_9_splitter_FEATURE_THRESHOLD)) != 0);
      if (__pyx_t_6) {

        /* "sklearn/tree/_splitter.pyx":1293
 * 
 *                 if Xf[end - 1] <= Xf[start] + FEATURE_THRESHOLD:
 *                     features[f_j] = features[n_total_constants]             # <<<<<<<<<<<<<<
 *                     features[n_total_constants] = current.feature
 * 
 */
        (__pyx_v_features[__pyx_v_f_j]) = (__pyx_v_features[__pyx_v_n_total_constants]);

        /* "sklearn/tree/_splitter.pyx":1294
 *                 if Xf[end - 1] <= Xf[start] + FEATURE_THRESHOLD:
 *                     features[f_j] = features[n_total_constants]
 *                     features[n_total_constants] = current.feature             # <<<<<<<<<<<<<<
 * 
 *                     n_found_constants += 1
 */
        __pyx_t_2 = __pyx_v_current.feature;
        (__pyx_v_features[__pyx_v_n_total_constants]) = __pyx_t_2;

        /* "sklearn/tree/_splitter.pyx":1296
 *                     features[n_total_constants] = current.feature
 * 
 *                     n_found_constants += 1             # <<<<<<<<<<<<<<
 *                     n_total_constants += 1
 * 
 */
        __pyx_v_n_found_constants = (__pyx_v_n_found_constants + 1);

        /* "sklearn/tree/_splitter.pyx":1297
 * 
 *                     n_found_constants += 1
 *                     n_total_constants += 1             # <<<<<<<<<<<<<<
 * 
 *                 else:
 */
        __pyx_v_n_total_constants = (__pyx_v_n_total_constants + 1);
        goto __pyx_L15;
      }
      /*else*/ {

        /* "sklearn/tree/_splitter.pyx":1300
 * 
 *                 else:
 *                     f_i -= 1             # <<<<<<<<<<<<<<
 *                     features[f_i], features[f_j] = features[f_j], features[f_i]
 * 
 */
        __pyx_v_f_i = (__pyx_v_f_i - 1);

        /* "sklearn/tree/_splitter.pyx":1301
 *                 else:
 *                     f_i -= 1
 *                     features[f_i], features[f_j] = features[f_j], features[f_i]             # <<<<<<<<<<<<<<
 * 
 *                     # Evaluate all splits
 */
        __pyx_t_2 = (__pyx_v_features[__pyx_v_f_j]);
        __pyx_t_8 = (__pyx_v_features[__pyx_v_f_i]);
        (__pyx_v_features[__pyx_v_f_i]) = __pyx_t_2;
        (__pyx_v_features[__pyx_v_f_j]) = __pyx_t_8;

        /* "sklearn/tree/_splitter.pyx":1304
 * 
 *                     # Evaluate all splits
 *                     self.criterion.reset()             # <<<<<<<<<<<<<<
 *                     p = start
 * 
 */
        ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->reset(__pyx_v_self->__pyx_base.__pyx_base.criterion);

        /* "sklearn/tree/_splitter.pyx":1305
 *                     # Evaluate all splits
 *                     self.criterion.reset()
 *                     p = start             # <<<<<<<<<<<<<<
 * 
 *                     while p < end:
 */
        __pyx_v_p = __pyx_v_start;

        /* "sklearn/tree/_splitter.pyx":1307
 *                     p = start
 * 
 *                     while p < end:             # <<<<<<<<<<<<<<
 *                         if p + 1 != end_negative:
 *                             p_next = p + 1
 */
        while (1) {
          __pyx_t_6 = ((__pyx_v_p < __pyx_v_end) != 0);
          if (!__pyx_t_6) break;

          /* "sklearn/tree/_splitter.pyx":1308
 * 
 *                     while p < end:
 *                         if p + 1 != end_negative:             # <<<<<<<<<<<<<<
 *                             p_next = p + 1
 *                         else:
 */
          __pyx_t_6 = (((__pyx_v_p + 1) != __pyx_v_end_negative) != 0);
          if (__pyx_t_6) {

            /* "sklearn/tree/_splitter.pyx":1309
 *                     while p < end:
 *                         if p + 1 != end_negative:
 *                             p_next = p + 1             # <<<<<<<<<<<<<<
 *                         else:
 *                             p_next = start_positive
 */
            __pyx_v_p_next = (__pyx_v_p + 1);
            goto __pyx_L18;
          }
          /*else*/ {

            /* "sklearn/tree/_splitter.pyx":1311
 *                             p_next = p + 1
 *                         else:
 *                             p_next = start_positive             # <<<<<<<<<<<<<<
 * 
 *                         while (p_next < end and
 */
            __pyx_v_p_next = __pyx_v_start_positive;
          }
          __pyx_L18:;

          /* "sklearn/tree/_splitter.pyx":1313
 *                             p_next = start_positive
 * 
 *                         while (p_next < end and             # <<<<<<<<<<<<<<
 *                                Xf[p_next] <= Xf[p] + FEATURE_THRESHOLD):
 *                             p = p_next
 */
          while (1) {
            __pyx_t_7 = ((__pyx_v_p_next < __pyx_v_end) != 0);
            if (__pyx_t_7) {
            } else {
              __pyx_t_6 = __pyx_t_7;
              goto __pyx_L21_bool_binop_done;
            }

            /* "sklearn/tree/_splitter.pyx":1314
 * 
 *                         while (p_next < end and
 *                                Xf[p_next] <= Xf[p] + FEATURE_THRESHOLD):             # <<<<<<<<<<<<<<
 *                             p = p_next
 *                             if p + 1 != end_negative:
 */
            __pyx_t_7 = (((__pyx_v_Xf[__pyx_v_p_next]) <= ((__pyx_v_Xf[__pyx_v_p]) + __pyx_v_7sklearn_4tree_9_splitter_FEATURE_THRESHOLD)) != 0);
            __pyx_t_6 = __pyx_t_7;
            __pyx_L21_bool_binop_done:;
            if (!__pyx_t_6) break;

            /* "sklearn/tree/_splitter.pyx":1315
 *                         while (p_next < end and
 *                                Xf[p_next] <= Xf[p] + FEATURE_THRESHOLD):
 *                             p = p_next             # <<<<<<<<<<<<<<
 *                             if p + 1 != end_negative:
 *                                 p_next = p + 1
 */
            __pyx_v_p = __pyx_v_p_next;

            /* "sklearn/tree/_splitter.pyx":1316
 *                                Xf[p_next] <= Xf[p] + FEATURE_THRESHOLD):
 *                             p = p_next
 *                             if p + 1 != end_negative:             # <<<<<<<<<<<<<<
 *                                 p_next = p + 1
 *                             else:
 */
            __pyx_t_6 = (((__pyx_v_p + 1) != __pyx_v_end_negative) != 0);
            if (__pyx_t_6) {

              /* "sklearn/tree/_splitter.pyx":1317
 *                             p = p_next
 *                             if p + 1 != end_negative:
 *                                 p_next = p + 1             # <<<<<<<<<<<<<<
 *                             else:
 *                                 p_next = start_positive
 */
              __pyx_v_p_next = (__pyx_v_p + 1);
              goto __pyx_L23;
            }
            /*else*/ {

              /* "sklearn/tree/_splitter.pyx":1319
 *                                 p_next = p + 1
 *                             else:
 *                                 p_next = start_positive             # <<<<<<<<<<<<<<
 * 
 * 
 */
              __pyx_v_p_next = __pyx_v_start_positive;
            }
            __pyx_L23:;
          }

          /* "sklearn/tree/_splitter.pyx":1324
 *                         # (p_next >= end) or (X[samples[p_next], current.feature] >
 *                         #                     X[samples[p], current.feature])
 *                         p_prev = p             # <<<<<<<<<<<<<<
 *                         p = p_next
 *                         # (p >= end) or (X[samples[p], current.feature] >
 */
          __pyx_v_p_prev = __pyx_v_p;

          /* "sklearn/tree/_splitter.pyx":1325
 *                         #                     X[samples[p], current.feature])
 *                         p_prev = p
 *                         p = p_next             # <<<<<<<<<<<<<<
 *                         # (p >= end) or (X[samples[p], current.feature] >
 *                         #                X[samples[p_prev], current.feature])
 */
          __pyx_v_p = __pyx_v_p_next;

          /* "sklearn/tree/_splitter.pyx":1330
 * 
 * 
 *                         if p < end:             # <<<<<<<<<<<<<<
 *                             current.pos = p
 * 
 */
          __pyx_t_6 = ((__pyx_v_p < __pyx_v_end) != 0);
          if (__pyx_t_6) {

            /* "sklearn/tree/_splitter.pyx":1331
 * 
 *                         if p < end:
 *                             current.pos = p             # <<<<<<<<<<<<<<
 * 
 *                             # Reject if min_samples_leaf is not guaranteed
 */
            __pyx_v_current.pos = __pyx_v_p;

            /* "sklearn/tree/_splitter.pyx":1334
 * 
 *                             # Reject if min_samples_leaf is not guaranteed
 *                             if (((current.pos - start) < min_samples_leaf) or             # <<<<<<<<<<<<<<
 *                                     ((end - current.pos) < min_samples_leaf)):
 *                                 continue
 */
            __pyx_t_7 = (((__pyx_v_current.pos - __pyx_v_start) < __pyx_v_min_samples_leaf) != 0);
            if (!__pyx_t_7) {
            } else {
              __pyx_t_6 = __pyx_t_7;
              goto __pyx_L26_bool_binop_done;
            }

            /* "sklearn/tree/_splitter.pyx":1335
 *                             # Reject if min_samples_leaf is not guaranteed
 *                             if (((current.pos - start) < min_samples_leaf) or
 *                                     ((end - current.pos) < min_samples_leaf)):             # <<<<<<<<<<<<<<
 *                                 continue
 * 
 */
            __pyx_t_7 = (((__pyx_v_end - __pyx_v_current.pos) < __pyx_v_min_samples_leaf) != 0);
            __pyx_t_6 = __pyx_t_7;
            __pyx_L26_bool_binop_done:;
            if (__pyx_t_6) {

              /* "sklearn/tree/_splitter.pyx":1336
 *                             if (((current.pos - start) < min_samples_leaf) or
 *                                     ((end - current.pos) < min_samples_leaf)):
 *                                 continue             # <<<<<<<<<<<<<<
 * 
 *                             self.criterion.update(current.pos)
 */
              goto __pyx_L16_continue;
            }

            /* "sklearn/tree/_splitter.pyx":1338
 *                                 continue
 * 
 *                             self.criterion.update(current.pos)             # <<<<<<<<<<<<<<
 * 
 *                             # Reject if min_weight_leaf is not satisfied
 */
            ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->update(__pyx_v_self->__pyx_base.__pyx_base.criterion, __pyx_v_current.pos);

            /* "sklearn/tree/_splitter.pyx":1341
 * 
 *                             # Reject if min_weight_leaf is not satisfied
 *                             if ((self.criterion.weighted_n_left < min_weight_leaf) or             # <<<<<<<<<<<<<<
 *                                     (self.criterion.weighted_n_right < min_weight_leaf)):
 *                                 continue
 */
            __pyx_t_7 = ((__pyx_v_self->__pyx_base.__pyx_base.criterion->weighted_n_left < __pyx_v_min_weight_leaf) != 0);
            if (!__pyx_t_7) {
            } else {
              __pyx_t_6 = __pyx_t_7;
              goto __pyx_L29_bool_binop_done;
            }

            /* "sklearn/tree/_splitter.pyx":1342
 *                             # Reject if min_weight_leaf is not satisfied
 *                             if ((self.criterion.weighted_n_left < min_weight_leaf) or
 *                                     (self.criterion.weighted_n_right < min_weight_leaf)):             # <<<<<<<<<<<<<<
 *                                 continue
 * 
 */
            __pyx_t_7 = ((__pyx_v_self->__pyx_base.__pyx_base.criterion->weighted_n_right < __pyx_v_min_weight_leaf) != 0);
            __pyx_t_6 = __pyx_t_7;
            __pyx_L29_bool_binop_done:;
            if (__pyx_t_6) {

              /* "sklearn/tree/_splitter.pyx":1343
 *                             if ((self.criterion.weighted_n_left < min_weight_leaf) or
 *                                     (self.criterion.weighted_n_right < min_weight_leaf)):
 *                                 continue             # <<<<<<<<<<<<<<
 * 
 *                             current_proxy_improvement = self.criterion.proxy_impurity_improvement()
 */
              goto __pyx_L16_continue;
            }

            /* "sklearn/tree/_splitter.pyx":1345
 *                                 continue
 * 
 *                             current_proxy_improvement = self.criterion.proxy_impurity_improvement()             # <<<<<<<<<<<<<<
 * 
 *                             if current_proxy_improvement > best_proxy_improvement:
 */
            __pyx_v_current_proxy_improvement = ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->proxy_impurity_improvement(__pyx_v_self->__pyx_base.__pyx_base.criterion);

            /* "sklearn/tree/_splitter.pyx":1347
 *                             current_proxy_improvement = self.criterion.proxy_impurity_improvement()
 * 
 *                             if current_proxy_improvement > best_proxy_improvement:             # <<<<<<<<<<<<<<
 *                                 best_proxy_improvement = current_proxy_improvement
 * 
 */
            __pyx_t_6 = ((__pyx_v_current_proxy_improvement > __pyx_v_best_proxy_improvement) != 0);
            if (__pyx_t_6) {

              /* "sklearn/tree/_splitter.pyx":1348
 * 
 *                             if current_proxy_improvement > best_proxy_improvement:
 *                                 best_proxy_improvement = current_proxy_improvement             # <<<<<<<<<<<<<<
 * 
 *                                 current.threshold = (Xf[p_prev] + Xf[p]) / 2.0
 */
              __pyx_v_best_proxy_improvement = __pyx_v_current_proxy_improvement;

              /* "sklearn/tree/_splitter.pyx":1350
 *                                 best_proxy_improvement = current_proxy_improvement
 * 
 *                                 current.threshold = (Xf[p_prev] + Xf[p]) / 2.0             # <<<<<<<<<<<<<<
 *                                 if current.threshold == Xf[p]:
 *                                     current.threshold = Xf[p_prev]
 */
              __pyx_v_current.threshold = (((__pyx_v_Xf[__pyx_v_p_prev]) + (__pyx_v_Xf[__pyx_v_p])) / 2.0);

              /* "sklearn/tree/_splitter.pyx":1351
 * 
 *                                 current.threshold = (Xf[p_prev] + Xf[p]) / 2.0
 *                                 if current.threshold == Xf[p]:             # <<<<<<<<<<<<<<
 *                                     current.threshold = Xf[p_prev]
 * 
 */
              __pyx_t_6 = ((__pyx_v_current.threshold == (__pyx_v_Xf[__pyx_v_p])) != 0);
              if (__pyx_t_6) {

                /* "sklearn/tree/_splitter.pyx":1352
 *                                 current.threshold = (Xf[p_prev] + Xf[p]) / 2.0
 *                                 if current.threshold == Xf[p]:
 *                                     current.threshold = Xf[p_prev]             # <<<<<<<<<<<<<<
 * 
 *                                 best = current
 */
                __pyx_v_current.threshold = (__pyx_v_Xf[__pyx_v_p_prev]);
                goto __pyx_L32;
              }
              __pyx_L32:;

              /* "sklearn/tree/_splitter.pyx":1354
 *                                     current.threshold = Xf[p_prev]
 * 
 *                                 best = current             # <<<<<<<<<<<<<<
 * 
 *         # Reorganize into samples[start:best.pos] + samples[best.pos:end]
 */
              __pyx_v_best = __pyx_v_current;
              goto __pyx_L31;
            }
            __pyx_L31:;
            goto __pyx_L24;
          }
          __pyx_L24:;
          __pyx_L16_continue:;
        }
      }
      __pyx_L15:;
    }
    __pyx_L8:;
  }

  /* "sklearn/tree/_splitter.pyx":1357
 * 
 *         # Reorganize into samples[start:best.pos] + samples[best.pos:end]
 *         if best.pos < end:             # <<<<<<<<<<<<<<
 *             self.extract_nnz(best.feature, &end_negative, &start_positive,
 *                              &is_samples_sorted)
 */
  __pyx_t_6 = ((__pyx_v_best.pos < __pyx_v_end) != 0);
  if (__pyx_t_6) {

    /* "sklearn/tree/_splitter.pyx":1358
 *         # Reorganize into samples[start:best.pos] + samples[best.pos:end]
 *         if best.pos < end:
 *             self.extract_nnz(best.feature, &end_negative, &start_positive,             # <<<<<<<<<<<<<<
 *                              &is_samples_sorted)
 * 
 */
    __pyx_f_7sklearn_4tree_9_splitter_18BaseSparseSplitter_extract_nnz(((struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *)__pyx_v_self), __pyx_v_best.feature, (&__pyx_v_end_negative), (&__pyx_v_start_positive), (&__pyx_v_is_samples_sorted));

    /* "sklearn/tree/_splitter.pyx":1361
 *                              &is_samples_sorted)
 * 
 *             self._partition(best.threshold, end_negative, start_positive,             # <<<<<<<<<<<<<<
 *                             best.pos)
 * 
 */
    __pyx_f_7sklearn_4tree_9_splitter_18BaseSparseSplitter__partition(((struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *)__pyx_v_self), __pyx_v_best.threshold, __pyx_v_end_negative, __pyx_v_start_positive, __pyx_v_best.pos);

    /* "sklearn/tree/_splitter.pyx":1364
 *                             best.pos)
 * 
 *             self.criterion.reset()             # <<<<<<<<<<<<<<
 *             self.criterion.update(best.pos)
 *             best.improvement = self.criterion.impurity_improvement(impurity)
 */
    ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->reset(__pyx_v_self->__pyx_base.__pyx_base.criterion);

    /* "sklearn/tree/_splitter.pyx":1365
 * 
 *             self.criterion.reset()
 *             self.criterion.update(best.pos)             # <<<<<<<<<<<<<<
 *             best.improvement = self.criterion.impurity_improvement(impurity)
 *             self.criterion.children_impurity(&best.impurity_left,
 */
    ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->update(__pyx_v_self->__pyx_base.__pyx_base.criterion, __pyx_v_best.pos);

    /* "sklearn/tree/_splitter.pyx":1366
 *             self.criterion.reset()
 *             self.criterion.update(best.pos)
 *             best.improvement = self.criterion.impurity_improvement(impurity)             # <<<<<<<<<<<<<<
 *             self.criterion.children_impurity(&best.impurity_left,
 *                                              &best.impurity_right)
 */
    __pyx_v_best.improvement = ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->impurity_improvement(__pyx_v_self->__pyx_base.__pyx_base.criterion, __pyx_v_impurity);

    /* "sklearn/tree/_splitter.pyx":1367
 *             self.criterion.update(best.pos)
 *             best.improvement = self.criterion.impurity_improvement(impurity)
 *             self.criterion.children_impurity(&best.impurity_left,             # <<<<<<<<<<<<<<
 *                                              &best.impurity_right)
 * 
 */
    ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->children_impurity(__pyx_v_self->__pyx_base.__pyx_base.criterion, (&__pyx_v_best.impurity_left), (&__pyx_v_best.impurity_right));
    goto __pyx_L33;
  }
  __pyx_L33:;

  /* "sklearn/tree/_splitter.pyx":1373
 *         # element in features[:n_known_constants] must be preserved for sibling
 *         # and child nodes
 *         memcpy(features, constant_features, sizeof(SIZE_t) * n_known_constants)             # <<<<<<<<<<<<<<
 * 
 *         # Copy newly found constant features
 */
  memcpy(__pyx_v_features, __pyx_v_constant_features, ((sizeof(__pyx_t_7sklearn_4tree_9_splitter_SIZE_t)) * __pyx_v_n_known_constants));

  /* "sklearn/tree/_splitter.pyx":1376
 * 
 *         # Copy newly found constant features
 *         memcpy(constant_features + n_known_constants,             # <<<<<<<<<<<<<<
 *                features + n_known_constants,
 *                sizeof(SIZE_t) * n_found_constants)
 */
  memcpy((__pyx_v_constant_features + __pyx_v_n_known_constants), (__pyx_v_features + __pyx_v_n_known_constants), ((sizeof(__pyx_t_7sklearn_4tree_9_splitter_SIZE_t)) * __pyx_v_n_found_constants));

  /* "sklearn/tree/_splitter.pyx":1381
 * 
 *         # Return values
 *         split[0] = best             # <<<<<<<<<<<<<<
 *         n_constant_features[0] = n_total_constants
 * 
 */
  (__pyx_v_split[0]) = __pyx_v_best;

  /* "sklearn/tree/_splitter.pyx":1382
 *         # Return values
 *         split[0] = best
 *         n_constant_features[0] = n_total_constants             # <<<<<<<<<<<<<<
 * 
 * 
 */
  (__pyx_v_n_constant_features[0]) = __pyx_v_n_total_constants;

  /* "sklearn/tree/_splitter.pyx":1169
 *                                      self.presort), self.__getstate__())
 * 
 *     cdef void node_split(self, double impurity, SplitRecord* split,             # <<<<<<<<<<<<<<
 *                          SIZE_t* n_constant_features) nogil:
 *         """Find the best split on node samples[start:end], using sparse
 */

  /* function exit code */
}

/* "sklearn/tree/_splitter.pyx":1388
 *     """Splitter for finding a random split, using the sparse data."""
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return (RandomSparseSplitter, (self.criterion,
 *                                        self.max_features,
 */

/* Python wrapper */
static PyObject *__pyx_pw_7sklearn_4tree_9_splitter_20RandomSparseSplitter_1__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7sklearn_4tree_9_splitter_20RandomSparseSplitter_1__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7sklearn_4tree_9_splitter_20RandomSparseSplitter___reduce__(((struct __pyx_obj_7sklearn_4tree_9_splitter_RandomSparseSplitter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7sklearn_4tree_9_splitter_20RandomSparseSplitter___reduce__(struct __pyx_obj_7sklearn_4tree_9_splitter_RandomSparseSplitter *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "sklearn/tree/_splitter.pyx":1389
 * 
 *     def __reduce__(self):
 *         return (RandomSparseSplitter, (self.criterion,             # <<<<<<<<<<<<<<
 *                                        self.max_features,
 *                                        self.min_samples_leaf,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "sklearn/tree/_splitter.pyx":1390
 *     def __reduce__(self):
 *         return (RandomSparseSplitter, (self.criterion,
 *                                        self.max_features,             # <<<<<<<<<<<<<<
 *                                        self.min_samples_leaf,
 *                                        self.min_weight_leaf,
 */
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->__pyx_base.__pyx_base.max_features); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1390; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);

  /* "sklearn/tree/_splitter.pyx":1391
 *         return (RandomSparseSplitter, (self.criterion,
 *                                        self.max_features,
 *                                        self.min_samples_leaf,             # <<<<<<<<<<<<<<
 *                                        self.min_weight_leaf,
 *                                        self.random_state,
 */
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_self->__pyx_base.__pyx_base.min_samples_leaf); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1391; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);

  /* "sklearn/tree/_splitter.pyx":1392
 *                                        self.max_features,
 *                                        self.min_samples_leaf,
 *                                        self.min_weight_leaf,             # <<<<<<<<<<<<<<
 *                                        self.random_state,
 *                                        self.presort), self.__getstate__())
 */
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->__pyx_base.__pyx_base.min_weight_leaf); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1392; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);

  /* "sklearn/tree/_splitter.pyx":1394
 *                                        self.min_weight_leaf,
 *                                        self.random_state,
 *                                        self.presort), self.__getstate__())             # <<<<<<<<<<<<<<
 * 
 *     cdef void node_split(self, double impurity, SplitRecord* split,
 */
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.presort); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1394; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);

  /* "sklearn/tree/_splitter.pyx":1389
 * 
 *     def __reduce__(self):
 *         return (RandomSparseSplitter, (self.criterion,             # <<<<<<<<<<<<<<
 *                                        self.max_features,
 *                                        self.min_samples_leaf,
 */
  __pyx_t_5 = PyTuple_New(6); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1389; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.criterion));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.criterion));
  PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.criterion));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.random_state);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.random_state);
  PyTuple_SET_ITEM(__pyx_t_5, 4, __pyx_v_self->__pyx_base.__pyx_base.random_state);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 5, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;

  /* "sklearn/tree/_splitter.pyx":1394
 *                                        self.min_weight_leaf,
 *                                        self.random_state,
 *                                        self.presort), self.__getstate__())             # <<<<<<<<<<<<<<
 * 
 *     cdef void node_split(self, double impurity, SplitRecord* split,
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getstate); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1394; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1394; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1394; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sklearn/tree/_splitter.pyx":1389
 * 
 *     def __reduce__(self):
 *         return (RandomSparseSplitter, (self.criterion,             # <<<<<<<<<<<<<<
 *                                        self.max_features,
 *                                        self.min_samples_leaf,
 */
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1389; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)((PyObject*)__pyx_ptype_7sklearn_4tree_9_splitter_RandomSparseSplitter)));
  __Pyx_GIVEREF(((PyObject *)((PyObject*)__pyx_ptype_7sklearn_4tree_9_splitter_RandomSparseSplitter)));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)((PyObject*)__pyx_ptype_7sklearn_4tree_9_splitter_RandomSparseSplitter)));
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_4);
  __pyx_t_5 = 0;
  __pyx_t_4 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "sklearn/tree/_splitter.pyx":1388
 *     """Splitter for finding a random split, using the sparse data."""
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return (RandomSparseSplitter, (self.criterion,
 *                                        self.max_features,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("sklearn.tree._splitter.RandomSparseSplitter.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sklearn/tree/_splitter.pyx":1396
 *                                        self.presort), self.__getstate__())
 * 
 *     cdef void node_split(self, double impurity, SplitRecord* split,             # <<<<<<<<<<<<<<
 *                          SIZE_t* n_constant_features) nogil:
 *         """Find a random split on node samples[start:end], using sparse
 */

static void __pyx_f_7sklearn_4tree_9_splitter_20RandomSparseSplitter_node_split(struct __pyx_obj_7sklearn_4tree_9_splitter_RandomSparseSplitter *__pyx_v_self, double __pyx_v_impurity, struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord *__pyx_v_split, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_n_constant_features) {
  CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_start;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_end;
  CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_INT32_t *__pyx_v_X_indices;
  CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_INT32_t *__pyx_v_X_indptr;
  CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_v_X_data;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_features;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_constant_features;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_features;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_v_Xf;
  CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_sorted_samples;
  CYTHON_UNUSED __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_v_index_to_samples;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_max_features;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_min_samples_leaf;
  double __pyx_v_min_weight_leaf;
  __pyx_t_7sklearn_4tree_9_splitter_UINT32_t *__pyx_v_random_state;
  struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord __pyx_v_best;
  struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord __pyx_v_current;
  double __pyx_v_current_proxy_improvement;
  double __pyx_v_best_proxy_improvement;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t __pyx_v_current_feature_value;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_f_i;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_f_j;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_p;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_tmp;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_visited_features;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_found_constants;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_drawn_constants;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_known_constants;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_n_total_constants;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t __pyx_v_min_feature_value;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t __pyx_v_max_feature_value;
  int __pyx_v_is_samples_sorted;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_start_positive;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_v_end_negative;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *__pyx_t_1;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_2;
  __pyx_t_7sklearn_4tree_9_splitter_INT32_t *__pyx_t_3;
  __pyx_t_7sklearn_4tree_9_splitter_DTYPE_t *__pyx_t_4;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  __pyx_t_7sklearn_4tree_9_splitter_SIZE_t __pyx_t_8;

  /* "sklearn/tree/_splitter.pyx":1402
 *         """
 *         # Find the best split
 *         cdef SIZE_t* samples = self.samples             # <<<<<<<<<<<<<<
 *         cdef SIZE_t start = self.start
 *         cdef SIZE_t end = self.end
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.__pyx_base.samples;
  __pyx_v_samples = __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":1403
 *         # Find the best split
 *         cdef SIZE_t* samples = self.samples
 *         cdef SIZE_t start = self.start             # <<<<<<<<<<<<<<
 *         cdef SIZE_t end = self.end
 * 
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.start;
  __pyx_v_start = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":1404
 *         cdef SIZE_t* samples = self.samples
 *         cdef SIZE_t start = self.start
 *         cdef SIZE_t end = self.end             # <<<<<<<<<<<<<<
 * 
 *         cdef INT32_t* X_indices = self.X_indices
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.end;
  __pyx_v_end = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":1406
 *         cdef SIZE_t end = self.end
 * 
 *         cdef INT32_t* X_indices = self.X_indices             # <<<<<<<<<<<<<<
 *         cdef INT32_t* X_indptr = self.X_indptr
 *         cdef DTYPE_t* X_data = self.X_data
 */
  __pyx_t_3 = __pyx_v_self->__pyx_base.X_indices;
  __pyx_v_X_indices = __pyx_t_3;

  /* "sklearn/tree/_splitter.pyx":1407
 * 
 *         cdef INT32_t* X_indices = self.X_indices
 *         cdef INT32_t* X_indptr = self.X_indptr             # <<<<<<<<<<<<<<
 *         cdef DTYPE_t* X_data = self.X_data
 * 
 */
  __pyx_t_3 = __pyx_v_self->__pyx_base.X_indptr;
  __pyx_v_X_indptr = __pyx_t_3;

  /* "sklearn/tree/_splitter.pyx":1408
 *         cdef INT32_t* X_indices = self.X_indices
 *         cdef INT32_t* X_indptr = self.X_indptr
 *         cdef DTYPE_t* X_data = self.X_data             # <<<<<<<<<<<<<<
 * 
 *         cdef SIZE_t* features = self.features
 */
  __pyx_t_4 = __pyx_v_self->__pyx_base.X_data;
  __pyx_v_X_data = __pyx_t_4;

  /* "sklearn/tree/_splitter.pyx":1410
 *         cdef DTYPE_t* X_data = self.X_data
 * 
 *         cdef SIZE_t* features = self.features             # <<<<<<<<<<<<<<
 *         cdef SIZE_t* constant_features = self.constant_features
 *         cdef SIZE_t n_features = self.n_features
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.__pyx_base.features;
  __pyx_v_features = __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":1411
 * 
 *         cdef SIZE_t* features = self.features
 *         cdef SIZE_t* constant_features = self.constant_features             # <<<<<<<<<<<<<<
 *         cdef SIZE_t n_features = self.n_features
 * 
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.__pyx_base.constant_features;
  __pyx_v_constant_features = __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":1412
 *         cdef SIZE_t* features = self.features
 *         cdef SIZE_t* constant_features = self.constant_features
 *         cdef SIZE_t n_features = self.n_features             # <<<<<<<<<<<<<<
 * 
 *         cdef DTYPE_t* Xf = self.feature_values
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.n_features;
  __pyx_v_n_features = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":1414
 *         cdef SIZE_t n_features = self.n_features
 * 
 *         cdef DTYPE_t* Xf = self.feature_values             # <<<<<<<<<<<<<<
 *         cdef SIZE_t* sorted_samples = self.sorted_samples
 *         cdef SIZE_t* index_to_samples = self.index_to_samples
 */
  __pyx_t_4 = __pyx_v_self->__pyx_base.__pyx_base.feature_values;
  __pyx_v_Xf = __pyx_t_4;

  /* "sklearn/tree/_splitter.pyx":1415
 * 
 *         cdef DTYPE_t* Xf = self.feature_values
 *         cdef SIZE_t* sorted_samples = self.sorted_samples             # <<<<<<<<<<<<<<
 *         cdef SIZE_t* index_to_samples = self.index_to_samples
 *         cdef SIZE_t max_features = self.max_features
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.sorted_samples;
  __pyx_v_sorted_samples = __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":1416
 *         cdef DTYPE_t* Xf = self.feature_values
 *         cdef SIZE_t* sorted_samples = self.sorted_samples
 *         cdef SIZE_t* index_to_samples = self.index_to_samples             # <<<<<<<<<<<<<<
 *         cdef SIZE_t max_features = self.max_features
 *         cdef SIZE_t min_samples_leaf = self.min_samples_leaf
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.index_to_samples;
  __pyx_v_index_to_samples = __pyx_t_1;

  /* "sklearn/tree/_splitter.pyx":1417
 *         cdef SIZE_t* sorted_samples = self.sorted_samples
 *         cdef SIZE_t* index_to_samples = self.index_to_samples
 *         cdef SIZE_t max_features = self.max_features             # <<<<<<<<<<<<<<
 *         cdef SIZE_t min_samples_leaf = self.min_samples_leaf
 *         cdef double min_weight_leaf = self.min_weight_leaf
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.max_features;
  __pyx_v_max_features = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":1418
 *         cdef SIZE_t* index_to_samples = self.index_to_samples
 *         cdef SIZE_t max_features = self.max_features
 *         cdef SIZE_t min_samples_leaf = self.min_samples_leaf             # <<<<<<<<<<<<<<
 *         cdef double min_weight_leaf = self.min_weight_leaf
 *         cdef UINT32_t* random_state = &self.rand_r_state
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base.__pyx_base.min_samples_leaf;
  __pyx_v_min_samples_leaf = __pyx_t_2;

  /* "sklearn/tree/_splitter.pyx":1419
 *         cdef SIZE_t max_features = self.max_features
 *         cdef SIZE_t min_samples_leaf = self.min_samples_leaf
 *         cdef double min_weight_leaf = self.min_weight_leaf             # <<<<<<<<<<<<<<
 *         cdef UINT32_t* random_state = &self.rand_r_state
 * 
 */
  __pyx_t_5 = __pyx_v_self->__pyx_base.__pyx_base.min_weight_leaf;
  __pyx_v_min_weight_leaf = __pyx_t_5;

  /* "sklearn/tree/_splitter.pyx":1420
 *         cdef SIZE_t min_samples_leaf = self.min_samples_leaf
 *         cdef double min_weight_leaf = self.min_weight_leaf
 *         cdef UINT32_t* random_state = &self.rand_r_state             # <<<<<<<<<<<<<<
 * 
 *         cdef SplitRecord best, current
 */
  __pyx_v_random_state = (&__pyx_v_self->__pyx_base.__pyx_base.rand_r_state);

  /* "sklearn/tree/_splitter.pyx":1423
 * 
 *         cdef SplitRecord best, current
 *         _init_split(&best, end)             # <<<<<<<<<<<<<<
 *         cdef double current_proxy_improvement = - INFINITY
 *         cdef double best_proxy_improvement = - INFINITY
 */
  __pyx_f_7sklearn_4tree_9_splitter__init_split((&__pyx_v_best), __pyx_v_end);

  /* "sklearn/tree/_splitter.pyx":1424
 *         cdef SplitRecord best, current
 *         _init_split(&best, end)
 *         cdef double current_proxy_improvement = - INFINITY             # <<<<<<<<<<<<<<
 *         cdef double best_proxy_improvement = - INFINITY
 * 
 */
  __pyx_v_current_proxy_improvement = (-__pyx_v_7sklearn_4tree_9_splitter_INFINITY);

  /* "sklearn/tree/_splitter.pyx":1425
 *         _init_split(&best, end)
 *         cdef double current_proxy_improvement = - INFINITY
 *         cdef double best_proxy_improvement = - INFINITY             # <<<<<<<<<<<<<<
 * 
 *         cdef DTYPE_t current_feature_value
 */
  __pyx_v_best_proxy_improvement = (-__pyx_v_7sklearn_4tree_9_splitter_INFINITY);

  /* "sklearn/tree/_splitter.pyx":1429
 *         cdef DTYPE_t current_feature_value
 * 
 *         cdef SIZE_t f_i = n_features             # <<<<<<<<<<<<<<
 *         cdef SIZE_t f_j, p, tmp
 *         cdef SIZE_t n_visited_features = 0
 */
  __pyx_v_f_i = __pyx_v_n_features;

  /* "sklearn/tree/_splitter.pyx":1431
 *         cdef SIZE_t f_i = n_features
 *         cdef SIZE_t f_j, p, tmp
 *         cdef SIZE_t n_visited_features = 0             # <<<<<<<<<<<<<<
 *         # Number of features discovered to be constant during the split search
 *         cdef SIZE_t n_found_constants = 0
 */
  __pyx_v_n_visited_features = 0;

  /* "sklearn/tree/_splitter.pyx":1433
 *         cdef SIZE_t n_visited_features = 0
 *         # Number of features discovered to be constant during the split search
 *         cdef SIZE_t n_found_constants = 0             # <<<<<<<<<<<<<<
 *         # Number of features known to be constant and drawn without replacement
 *         cdef SIZE_t n_drawn_constants = 0
 */
  __pyx_v_n_found_constants = 0;

  /* "sklearn/tree/_splitter.pyx":1435
 *         cdef SIZE_t n_found_constants = 0
 *         # Number of features known to be constant and drawn without replacement
 *         cdef SIZE_t n_drawn_constants = 0             # <<<<<<<<<<<<<<
 *         cdef SIZE_t n_known_constants = n_constant_features[0]
 *         # n_total_constants = n_known_constants + n_found_constants
 */
  __pyx_v_n_drawn_constants = 0;

  /* "sklearn/tree/_splitter.pyx":1436
 *         # Number of features known to be constant and drawn without replacement
 *         cdef SIZE_t n_drawn_constants = 0
 *         cdef SIZE_t n_known_constants = n_constant_features[0]             # <<<<<<<<<<<<<<
 *         # n_total_constants = n_known_constants + n_found_constants
 *         cdef SIZE_t n_total_constants = n_known_constants
 */
  __pyx_v_n_known_constants = (__pyx_v_n_constant_features[0]);

  /* "sklearn/tree/_splitter.pyx":1438
 *         cdef SIZE_t n_known_constants = n_constant_features[0]
 *         # n_total_constants = n_known_constants + n_found_constants
 *         cdef SIZE_t n_total_constants = n_known_constants             # <<<<<<<<<<<<<<
 *         cdef SIZE_t partition_end
 * 
 */
  __pyx_v_n_total_constants = __pyx_v_n_known_constants;

  /* "sklearn/tree/_splitter.pyx":1444
 *         cdef DTYPE_t max_feature_value
 * 
 *         cdef bint is_samples_sorted = 0  # indicate that sorted_samples is             # <<<<<<<<<<<<<<
 *                                          # inititialized
 * 
 */
  __pyx_v_is_samples_sorted = 0;

  /* "sklearn/tree/_splitter.pyx":1461
 *         # newly discovered constant features to spare computation on descendant
 *         # nodes.
 *         while (f_i > n_total_constants and  # Stop early if remaining features             # <<<<<<<<<<<<<<
 *                                             # are constant
 *                 (n_visited_features < max_features or
 */
  while (1) {
    __pyx_t_7 = ((__pyx_v_f_i > __pyx_v_n_total_constants) != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L5_bool_binop_done;
    }

    /* "sklearn/tree/_splitter.pyx":1463
 *         while (f_i > n_total_constants and  # Stop early if remaining features
 *                                             # are constant
 *                 (n_visited_features < max_features or             # <<<<<<<<<<<<<<
 *                  # At least one drawn features must be non constant
 *                  n_visited_features <= n_found_constants + n_drawn_constants)):
 */
    __pyx_t_7 = ((__pyx_v_n_visited_features < __pyx_v_max_features) != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L5_bool_binop_done;
    }

    /* "sklearn/tree/_splitter.pyx":1465
 *                 (n_visited_features < max_features or
 *                  # At least one drawn features must be non constant
 *                  n_visited_features <= n_found_constants + n_drawn_constants)):             # <<<<<<<<<<<<<<
 * 
 *             n_visited_features += 1
 */
    __pyx_t_7 = ((__pyx_v_n_visited_features <= (__pyx_v_n_found_constants + __pyx_v_n_drawn_constants)) != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L5_bool_binop_done:;
    if (!__pyx_t_6) break;

    /* "sklearn/tree/_splitter.pyx":1467
 *                  n_visited_features <= n_found_constants + n_drawn_constants)):
 * 
 *             n_visited_features += 1             # <<<<<<<<<<<<<<
 * 
 *             # Loop invariant: elements of features in
 */
    __pyx_v_n_visited_features = (__pyx_v_n_visited_features + 1);

    /* "sklearn/tree/_splitter.pyx":1481
 * 
 *             # Draw a feature at random
 *             f_j = rand_int(n_drawn_constants, f_i - n_found_constants,             # <<<<<<<<<<<<<<
 *                            random_state)
 * 
 */
    __pyx_v_f_j = __pyx_f_7sklearn_4tree_6_utils_rand_int(__pyx_v_n_drawn_constants, (__pyx_v_f_i - __pyx_v_n_found_constants), __pyx_v_random_state);

    /* "sklearn/tree/_splitter.pyx":1484
 *                            random_state)
 * 
 *             if f_j < n_known_constants:             # <<<<<<<<<<<<<<
 *                 # f_j in the interval [n_drawn_constants, n_known_constants[
 *                 tmp = features[f_j]
 */
    __pyx_t_6 = ((__pyx_v_f_j < __pyx_v_n_known_constants) != 0);
    if (__pyx_t_6) {

      /* "sklearn/tree/_splitter.pyx":1486
 *             if f_j < n_known_constants:
 *                 # f_j in the interval [n_drawn_constants, n_known_constants[
 *                 tmp = features[f_j]             # <<<<<<<<<<<<<<
 *                 features[f_j] = features[n_drawn_constants]
 *                 features[n_drawn_constants] = tmp
 */
      __pyx_v_tmp = (__pyx_v_features[__pyx_v_f_j]);

      /* "sklearn/tree/_splitter.pyx":1487
 *                 # f_j in the interval [n_drawn_constants, n_known_constants[
 *                 tmp = features[f_j]
 *                 features[f_j] = features[n_drawn_constants]             # <<<<<<<<<<<<<<
 *                 features[n_drawn_constants] = tmp
 * 
 */
      (__pyx_v_features[__pyx_v_f_j]) = (__pyx_v_features[__pyx_v_n_drawn_constants]);

      /* "sklearn/tree/_splitter.pyx":1488
 *                 tmp = features[f_j]
 *                 features[f_j] = features[n_drawn_constants]
 *                 features[n_drawn_constants] = tmp             # <<<<<<<<<<<<<<
 * 
 *                 n_drawn_constants += 1
 */
      (__pyx_v_features[__pyx_v_n_drawn_constants]) = __pyx_v_tmp;

      /* "sklearn/tree/_splitter.pyx":1490
 *                 features[n_drawn_constants] = tmp
 * 
 *                 n_drawn_constants += 1             # <<<<<<<<<<<<<<
 * 
 *             else:
 */
      __pyx_v_n_drawn_constants = (__pyx_v_n_drawn_constants + 1);
      goto __pyx_L8;
    }
    /*else*/ {

      /* "sklearn/tree/_splitter.pyx":1494
 *             else:
 *                 # f_j in the interval [n_known_constants, f_i - n_found_constants[
 *                 f_j += n_found_constants             # <<<<<<<<<<<<<<
 *                 # f_j in the interval [n_total_constants, f_i[
 * 
 */
      __pyx_v_f_j = (__pyx_v_f_j + __pyx_v_n_found_constants);

      /* "sklearn/tree/_splitter.pyx":1497
 *                 # f_j in the interval [n_total_constants, f_i[
 * 
 *                 current.feature = features[f_j]             # <<<<<<<<<<<<<<
 * 
 *                 self.extract_nnz(current.feature,
 */
      __pyx_v_current.feature = (__pyx_v_features[__pyx_v_f_j]);

      /* "sklearn/tree/_splitter.pyx":1499
 *                 current.feature = features[f_j]
 * 
 *                 self.extract_nnz(current.feature,             # <<<<<<<<<<<<<<
 *                                  &end_negative, &start_positive,
 *                                  &is_samples_sorted)
 */
      __pyx_f_7sklearn_4tree_9_splitter_18BaseSparseSplitter_extract_nnz(((struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *)__pyx_v_self), __pyx_v_current.feature, (&__pyx_v_end_negative), (&__pyx_v_start_positive), (&__pyx_v_is_samples_sorted));

      /* "sklearn/tree/_splitter.pyx":1504
 * 
 *                 # Add one or two zeros in Xf, if there is any
 *                 if end_negative < start_positive:             # <<<<<<<<<<<<<<
 *                     start_positive -= 1
 *                     Xf[start_positive] = 0.
 */
      __pyx_t_6 = ((__pyx_v_end_negative < __pyx_v_start_positive) != 0);
      if (__pyx_t_6) {

        /* "sklearn/tree/_splitter.pyx":1505
 *                 # Add one or two zeros in Xf, if there is any
 *                 if end_negative < start_positive:
 *                     start_positive -= 1             # <<<<<<<<<<<<<<
 *                     Xf[start_positive] = 0.
 * 
 */
        __pyx_v_start_positive = (__pyx_v_start_positive - 1);

        /* "sklearn/tree/_splitter.pyx":1506
 *                 if end_negative < start_positive:
 *                     start_positive -= 1
 *                     Xf[start_positive] = 0.             # <<<<<<<<<<<<<<
 * 
 *                     if end_negative != start_positive:
 */
        (__pyx_v_Xf[__pyx_v_start_positive]) = 0.;

        /* "sklearn/tree/_splitter.pyx":1508
 *                     Xf[start_positive] = 0.
 * 
 *                     if end_negative != start_positive:             # <<<<<<<<<<<<<<
 *                         Xf[end_negative] = 0.
 *                         end_negative += 1
 */
        __pyx_t_6 = ((__pyx_v_end_negative != __pyx_v_start_positive) != 0);
        if (__pyx_t_6) {

          /* "sklearn/tree/_splitter.pyx":1509
 * 
 *                     if end_negative != start_positive:
 *                         Xf[end_negative] = 0.             # <<<<<<<<<<<<<<
 *                         end_negative += 1
 * 
 */
          (__pyx_v_Xf[__pyx_v_end_negative]) = 0.;

          /* "sklearn/tree/_splitter.pyx":1510
 *                     if end_negative != start_positive:
 *                         Xf[end_negative] = 0.
 *                         end_negative += 1             # <<<<<<<<<<<<<<
 * 
 *                 # Find min, max in Xf[start:end_negative]
 */
          __pyx_v_end_negative = (__pyx_v_end_negative + 1);
          goto __pyx_L10;
        }
        __pyx_L10:;
        goto __pyx_L9;
      }
      __pyx_L9:;

      /* "sklearn/tree/_splitter.pyx":1513
 * 
 *                 # Find min, max in Xf[start:end_negative]
 *                 min_feature_value = Xf[start]             # <<<<<<<<<<<<<<
 *                 max_feature_value = min_feature_value
 * 
 */
      __pyx_v_min_feature_value = (__pyx_v_Xf[__pyx_v_start]);

      /* "sklearn/tree/_splitter.pyx":1514
 *                 # Find min, max in Xf[start:end_negative]
 *                 min_feature_value = Xf[start]
 *                 max_feature_value = min_feature_value             # <<<<<<<<<<<<<<
 * 
 *                 for p in range(start, end_negative):
 */
      __pyx_v_max_feature_value = __pyx_v_min_feature_value;

      /* "sklearn/tree/_splitter.pyx":1516
 *                 max_feature_value = min_feature_value
 * 
 *                 for p in range(start, end_negative):             # <<<<<<<<<<<<<<
 *                     current_feature_value = Xf[p]
 * 
 */
      __pyx_t_2 = __pyx_v_end_negative;
      for (__pyx_t_8 = __pyx_v_start; __pyx_t_8 < __pyx_t_2; __pyx_t_8+=1) {
        __pyx_v_p = __pyx_t_8;

        /* "sklearn/tree/_splitter.pyx":1517
 * 
 *                 for p in range(start, end_negative):
 *                     current_feature_value = Xf[p]             # <<<<<<<<<<<<<<
 * 
 *                     if current_feature_value < min_feature_value:
 */
        __pyx_v_current_feature_value = (__pyx_v_Xf[__pyx_v_p]);

        /* "sklearn/tree/_splitter.pyx":1519
 *                     current_feature_value = Xf[p]
 * 
 *                     if current_feature_value < min_feature_value:             # <<<<<<<<<<<<<<
 *                         min_feature_value = current_feature_value
 *                     elif current_feature_value > max_feature_value:
 */
        __pyx_t_6 = ((__pyx_v_current_feature_value < __pyx_v_min_feature_value) != 0);
        if (__pyx_t_6) {

          /* "sklearn/tree/_splitter.pyx":1520
 * 
 *                     if current_feature_value < min_feature_value:
 *                         min_feature_value = current_feature_value             # <<<<<<<<<<<<<<
 *                     elif current_feature_value > max_feature_value:
 *                         max_feature_value = current_feature_value
 */
          __pyx_v_min_feature_value = __pyx_v_current_feature_value;
          goto __pyx_L13;
        }

        /* "sklearn/tree/_splitter.pyx":1521
 *                     if current_feature_value < min_feature_value:
 *                         min_feature_value = current_feature_value
 *                     elif current_feature_value > max_feature_value:             # <<<<<<<<<<<<<<
 *                         max_feature_value = current_feature_value
 * 
 */
        __pyx_t_6 = ((__pyx_v_current_feature_value > __pyx_v_max_feature_value) != 0);
        if (__pyx_t_6) {

          /* "sklearn/tree/_splitter.pyx":1522
 *                         min_feature_value = current_feature_value
 *                     elif current_feature_value > max_feature_value:
 *                         max_feature_value = current_feature_value             # <<<<<<<<<<<<<<
 * 
 *                 # Update min, max given Xf[start_positive:end]
 */
          __pyx_v_max_feature_value = __pyx_v_current_feature_value;
          goto __pyx_L13;
        }
        __pyx_L13:;
      }

      /* "sklearn/tree/_splitter.pyx":1525
 * 
 *                 # Update min, max given Xf[start_positive:end]
 *                 for p in range(start_positive, end):             # <<<<<<<<<<<<<<
 *                     current_feature_value = Xf[p]
 * 
 */
      __pyx_t_2 = __pyx_v_end;
      for (__pyx_t_8 = __pyx_v_start_positive; __pyx_t_8 < __pyx_t_2; __pyx_t_8+=1) {
        __pyx_v_p = __pyx_t_8;

        /* "sklearn/tree/_splitter.pyx":1526
 *                 # Update min, max given Xf[start_positive:end]
 *                 for p in range(start_positive, end):
 *                     current_feature_value = Xf[p]             # <<<<<<<<<<<<<<
 * 
 *                     if current_feature_value < min_feature_value:
 */
        __pyx_v_current_feature_value = (__pyx_v_Xf[__pyx_v_p]);

        /* "sklearn/tree/_splitter.pyx":1528
 *                     current_feature_value = Xf[p]
 * 
 *                     if current_feature_value < min_feature_value:             # <<<<<<<<<<<<<<
 *                         min_feature_value = current_feature_value
 *                     elif current_feature_value > max_feature_value:
 */
        __pyx_t_6 = ((__pyx_v_current_feature_value < __pyx_v_min_feature_value) != 0);
        if (__pyx_t_6) {

          /* "sklearn/tree/_splitter.pyx":1529
 * 
 *                     if current_feature_value < min_feature_value:
 *                         min_feature_value = current_feature_value             # <<<<<<<<<<<<<<
 *                     elif current_feature_value > max_feature_value:
 *                         max_feature_value = current_feature_value
 */
          __pyx_v_min_feature_value = __pyx_v_current_feature_value;
          goto __pyx_L16;
        }

        /* "sklearn/tree/_splitter.pyx":1530
 *                     if current_feature_value < min_feature_value:
 *                         min_feature_value = current_feature_value
 *                     elif current_feature_value > max_feature_value:             # <<<<<<<<<<<<<<
 *                         max_feature_value = current_feature_value
 * 
 */
        __pyx_t_6 = ((__pyx_v_current_feature_value > __pyx_v_max_feature_value) != 0);
        if (__pyx_t_6) {

          /* "sklearn/tree/_splitter.pyx":1531
 *                         min_feature_value = current_feature_value
 *                     elif current_feature_value > max_feature_value:
 *                         max_feature_value = current_feature_value             # <<<<<<<<<<<<<<
 * 
 *                 if max_feature_value <= min_feature_value + FEATURE_THRESHOLD:
 */
          __pyx_v_max_feature_value = __pyx_v_current_feature_value;
          goto __pyx_L16;
        }
        __pyx_L16:;
      }

      /* "sklearn/tree/_splitter.pyx":1533
 *                         max_feature_value = current_feature_value
 * 
 *                 if max_feature_value <= min_feature_value + FEATURE_THRESHOLD:             # <<<<<<<<<<<<<<
 *                     features[f_j] = features[n_total_constants]
 *                     features[n_total_constants] = current.feature
 */
      __pyx_t_6 = ((__pyx_v_max_feature_value <= (__pyx_v_min_feature_value + __pyx_v_7sklearn_4tree_9_splitter_FEATURE_THRESHOLD)) != 0);
      if (__pyx_t_6) {

        /* "sklearn/tree/_splitter.pyx":1534
 * 
 *                 if max_feature_value <= min_feature_value + FEATURE_THRESHOLD:
 *                     features[f_j] = features[n_total_constants]             # <<<<<<<<<<<<<<
 *                     features[n_total_constants] = current.feature
 * 
 */
        (__pyx_v_features[__pyx_v_f_j]) = (__pyx_v_features[__pyx_v_n_total_constants]);

        /* "sklearn/tree/_splitter.pyx":1535
 *                 if max_feature_value <= min_feature_value + FEATURE_THRESHOLD:
 *                     features[f_j] = features[n_total_constants]
 *                     features[n_total_constants] = current.feature             # <<<<<<<<<<<<<<
 * 
 *                     n_found_constants += 1
 */
        __pyx_t_2 = __pyx_v_current.feature;
        (__pyx_v_features[__pyx_v_n_total_constants]) = __pyx_t_2;

        /* "sklearn/tree/_splitter.pyx":1537
 *                     features[n_total_constants] = current.feature
 * 
 *                     n_found_constants += 1             # <<<<<<<<<<<<<<
 *                     n_total_constants += 1
 * 
 */
        __pyx_v_n_found_constants = (__pyx_v_n_found_constants + 1);

        /* "sklearn/tree/_splitter.pyx":1538
 * 
 *                     n_found_constants += 1
 *                     n_total_constants += 1             # <<<<<<<<<<<<<<
 * 
 *                 else:
 */
        __pyx_v_n_total_constants = (__pyx_v_n_total_constants + 1);
        goto __pyx_L17;
      }
      /*else*/ {

        /* "sklearn/tree/_splitter.pyx":1541
 * 
 *                 else:
 *                     f_i -= 1             # <<<<<<<<<<<<<<
 *                     features[f_i], features[f_j] = features[f_j], features[f_i]
 * 
 */
        __pyx_v_f_i = (__pyx_v_f_i - 1);

        /* "sklearn/tree/_splitter.pyx":1542
 *                 else:
 *                     f_i -= 1
 *                     features[f_i], features[f_j] = features[f_j], features[f_i]             # <<<<<<<<<<<<<<
 * 
 *                     # Draw a random threshold
 */
        __pyx_t_2 = (__pyx_v_features[__pyx_v_f_j]);
        __pyx_t_8 = (__pyx_v_features[__pyx_v_f_i]);
        (__pyx_v_features[__pyx_v_f_i]) = __pyx_t_2;
        (__pyx_v_features[__pyx_v_f_j]) = __pyx_t_8;

        /* "sklearn/tree/_splitter.pyx":1545
 * 
 *                     # Draw a random threshold
 *                     current.threshold = rand_uniform(min_feature_value,             # <<<<<<<<<<<<<<
 *                                                      max_feature_value,
 *                                                      random_state)
 */
        __pyx_v_current.threshold = __pyx_f_7sklearn_4tree_6_utils_rand_uniform(__pyx_v_min_feature_value, __pyx_v_max_feature_value, __pyx_v_random_state);

        /* "sklearn/tree/_splitter.pyx":1549
 *                                                      random_state)
 * 
 *                     if current.threshold == max_feature_value:             # <<<<<<<<<<<<<<
 *                         current.threshold = min_feature_value
 * 
 */
        __pyx_t_6 = ((__pyx_v_current.threshold == __pyx_v_max_feature_value) != 0);
        if (__pyx_t_6) {

          /* "sklearn/tree/_splitter.pyx":1550
 * 
 *                     if current.threshold == max_feature_value:
 *                         current.threshold = min_feature_value             # <<<<<<<<<<<<<<
 * 
 *                     # Partition
 */
          __pyx_v_current.threshold = __pyx_v_min_feature_value;
          goto __pyx_L18;
        }
        __pyx_L18:;

        /* "sklearn/tree/_splitter.pyx":1553
 * 
 *                     # Partition
 *                     current.pos = self._partition(current.threshold,             # <<<<<<<<<<<<<<
 *                                                   end_negative,
 *                                                   start_positive,
 */
        __pyx_v_current.pos = __pyx_f_7sklearn_4tree_9_splitter_18BaseSparseSplitter__partition(((struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *)__pyx_v_self), __pyx_v_current.threshold, __pyx_v_end_negative, __pyx_v_start_positive, (__pyx_v_start_positive + ((__pyx_v_Xf[__pyx_v_start_positive]) == 0.)));

        /* "sklearn/tree/_splitter.pyx":1560
 * 
 *                     # Reject if min_samples_leaf is not guaranteed
 *                     if (((current.pos - start) < min_samples_leaf) or             # <<<<<<<<<<<<<<
 *                             ((end - current.pos) < min_samples_leaf)):
 *                         continue
 */
        __pyx_t_7 = (((__pyx_v_current.pos - __pyx_v_start) < __pyx_v_min_samples_leaf) != 0);
        if (!__pyx_t_7) {
        } else {
          __pyx_t_6 = __pyx_t_7;
          goto __pyx_L20_bool_binop_done;
        }

        /* "sklearn/tree/_splitter.pyx":1561
 *                     # Reject if min_samples_leaf is not guaranteed
 *                     if (((current.pos - start) < min_samples_leaf) or
 *                             ((end - current.pos) < min_samples_leaf)):             # <<<<<<<<<<<<<<
 *                         continue
 * 
 */
        __pyx_t_7 = (((__pyx_v_end - __pyx_v_current.pos) < __pyx_v_min_samples_leaf) != 0);
        __pyx_t_6 = __pyx_t_7;
        __pyx_L20_bool_binop_done:;
        if (__pyx_t_6) {

          /* "sklearn/tree/_splitter.pyx":1562
 *                     if (((current.pos - start) < min_samples_leaf) or
 *                             ((end - current.pos) < min_samples_leaf)):
 *                         continue             # <<<<<<<<<<<<<<
 * 
 *                     # Evaluate split
 */
          goto __pyx_L3_continue;
        }

        /* "sklearn/tree/_splitter.pyx":1565
 * 
 *                     # Evaluate split
 *                     self.criterion.reset()             # <<<<<<<<<<<<<<
 *                     self.criterion.update(current.pos)
 * 
 */
        ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->reset(__pyx_v_self->__pyx_base.__pyx_base.criterion);

        /* "sklearn/tree/_splitter.pyx":1566
 *                     # Evaluate split
 *                     self.criterion.reset()
 *                     self.criterion.update(current.pos)             # <<<<<<<<<<<<<<
 * 
 *                     # Reject if min_weight_leaf is not satisfied
 */
        ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->update(__pyx_v_self->__pyx_base.__pyx_base.criterion, __pyx_v_current.pos);

        /* "sklearn/tree/_splitter.pyx":1569
 * 
 *                     # Reject if min_weight_leaf is not satisfied
 *                     if ((self.criterion.weighted_n_left < min_weight_leaf) or             # <<<<<<<<<<<<<<
 *                             (self.criterion.weighted_n_right < min_weight_leaf)):
 *                         continue
 */
        __pyx_t_7 = ((__pyx_v_self->__pyx_base.__pyx_base.criterion->weighted_n_left < __pyx_v_min_weight_leaf) != 0);
        if (!__pyx_t_7) {
        } else {
          __pyx_t_6 = __pyx_t_7;
          goto __pyx_L23_bool_binop_done;
        }

        /* "sklearn/tree/_splitter.pyx":1570
 *                     # Reject if min_weight_leaf is not satisfied
 *                     if ((self.criterion.weighted_n_left < min_weight_leaf) or
 *                             (self.criterion.weighted_n_right < min_weight_leaf)):             # <<<<<<<<<<<<<<
 *                         continue
 * 
 */
        __pyx_t_7 = ((__pyx_v_self->__pyx_base.__pyx_base.criterion->weighted_n_right < __pyx_v_min_weight_leaf) != 0);
        __pyx_t_6 = __pyx_t_7;
        __pyx_L23_bool_binop_done:;
        if (__pyx_t_6) {

          /* "sklearn/tree/_splitter.pyx":1571
 *                     if ((self.criterion.weighted_n_left < min_weight_leaf) or
 *                             (self.criterion.weighted_n_right < min_weight_leaf)):
 *                         continue             # <<<<<<<<<<<<<<
 * 
 *                     current_proxy_improvement = self.criterion.proxy_impurity_improvement()
 */
          goto __pyx_L3_continue;
        }

        /* "sklearn/tree/_splitter.pyx":1573
 *                         continue
 * 
 *                     current_proxy_improvement = self.criterion.proxy_impurity_improvement()             # <<<<<<<<<<<<<<
 * 
 *                     if current_proxy_improvement > best_proxy_improvement:
 */
        __pyx_v_current_proxy_improvement = ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->proxy_impurity_improvement(__pyx_v_self->__pyx_base.__pyx_base.criterion);

        /* "sklearn/tree/_splitter.pyx":1575
 *                     current_proxy_improvement = self.criterion.proxy_impurity_improvement()
 * 
 *                     if current_proxy_improvement > best_proxy_improvement:             # <<<<<<<<<<<<<<
 *                         best_proxy_improvement = current_proxy_improvement
 *                         current.improvement = self.criterion.impurity_improvement(impurity)
 */
        __pyx_t_6 = ((__pyx_v_current_proxy_improvement > __pyx_v_best_proxy_improvement) != 0);
        if (__pyx_t_6) {

          /* "sklearn/tree/_splitter.pyx":1576
 * 
 *                     if current_proxy_improvement > best_proxy_improvement:
 *                         best_proxy_improvement = current_proxy_improvement             # <<<<<<<<<<<<<<
 *                         current.improvement = self.criterion.impurity_improvement(impurity)
 * 
 */
          __pyx_v_best_proxy_improvement = __pyx_v_current_proxy_improvement;

          /* "sklearn/tree/_splitter.pyx":1577
 *                     if current_proxy_improvement > best_proxy_improvement:
 *                         best_proxy_improvement = current_proxy_improvement
 *                         current.improvement = self.criterion.impurity_improvement(impurity)             # <<<<<<<<<<<<<<
 * 
 *                         self.criterion.children_impurity(&current.impurity_left,
 */
          __pyx_v_current.improvement = ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->impurity_improvement(__pyx_v_self->__pyx_base.__pyx_base.criterion, __pyx_v_impurity);

          /* "sklearn/tree/_splitter.pyx":1579
 *                         current.improvement = self.criterion.impurity_improvement(impurity)
 * 
 *                         self.criterion.children_impurity(&current.impurity_left,             # <<<<<<<<<<<<<<
 *                                                          &current.impurity_right)
 *                         best = current
 */
          ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->children_impurity(__pyx_v_self->__pyx_base.__pyx_base.criterion, (&__pyx_v_current.impurity_left), (&__pyx_v_current.impurity_right));

          /* "sklearn/tree/_splitter.pyx":1581
 *                         self.criterion.children_impurity(&current.impurity_left,
 *                                                          &current.impurity_right)
 *                         best = current             # <<<<<<<<<<<<<<
 * 
 *         # Reorganize into samples[start:best.pos] + samples[best.pos:end]
 */
          __pyx_v_best = __pyx_v_current;
          goto __pyx_L25;
        }
        __pyx_L25:;
      }
      __pyx_L17:;
    }
    __pyx_L8:;
    __pyx_L3_continue:;
  }

  /* "sklearn/tree/_splitter.pyx":1584
 * 
 *         # Reorganize into samples[start:best.pos] + samples[best.pos:end]
 *         if best.pos < end:             # <<<<<<<<<<<<<<
 *             if current.feature != best.feature:
 *                 self.extract_nnz(best.feature, &end_negative, &start_positive,
 */
  __pyx_t_6 = ((__pyx_v_best.pos < __pyx_v_end) != 0);
  if (__pyx_t_6) {

    /* "sklearn/tree/_splitter.pyx":1585
 *         # Reorganize into samples[start:best.pos] + samples[best.pos:end]
 *         if best.pos < end:
 *             if current.feature != best.feature:             # <<<<<<<<<<<<<<
 *                 self.extract_nnz(best.feature, &end_negative, &start_positive,
 *                                  &is_samples_sorted)
 */
    __pyx_t_6 = ((__pyx_v_current.feature != __pyx_v_best.feature) != 0);
    if (__pyx_t_6) {

      /* "sklearn/tree/_splitter.pyx":1586
 *         if best.pos < end:
 *             if current.feature != best.feature:
 *                 self.extract_nnz(best.feature, &end_negative, &start_positive,             # <<<<<<<<<<<<<<
 *                                  &is_samples_sorted)
 * 
 */
      __pyx_f_7sklearn_4tree_9_splitter_18BaseSparseSplitter_extract_nnz(((struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *)__pyx_v_self), __pyx_v_best.feature, (&__pyx_v_end_negative), (&__pyx_v_start_positive), (&__pyx_v_is_samples_sorted));

      /* "sklearn/tree/_splitter.pyx":1589
 *                                  &is_samples_sorted)
 * 
 *                 self._partition(best.threshold, end_negative, start_positive,             # <<<<<<<<<<<<<<
 *                                 best.pos)
 * 
 */
      __pyx_f_7sklearn_4tree_9_splitter_18BaseSparseSplitter__partition(((struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *)__pyx_v_self), __pyx_v_best.threshold, __pyx_v_end_negative, __pyx_v_start_positive, __pyx_v_best.pos);
      goto __pyx_L27;
    }
    __pyx_L27:;

    /* "sklearn/tree/_splitter.pyx":1592
 *                                 best.pos)
 * 
 *             self.criterion.reset()             # <<<<<<<<<<<<<<
 *             self.criterion.update(best.pos)
 *             best.improvement = self.criterion.impurity_improvement(impurity)
 */
    ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->reset(__pyx_v_self->__pyx_base.__pyx_base.criterion);

    /* "sklearn/tree/_splitter.pyx":1593
 * 
 *             self.criterion.reset()
 *             self.criterion.update(best.pos)             # <<<<<<<<<<<<<<
 *             best.improvement = self.criterion.impurity_improvement(impurity)
 *             self.criterion.children_impurity(&best.impurity_left,
 */
    ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->update(__pyx_v_self->__pyx_base.__pyx_base.criterion, __pyx_v_best.pos);

    /* "sklearn/tree/_splitter.pyx":1594
 *             self.criterion.reset()
 *             self.criterion.update(best.pos)
 *             best.improvement = self.criterion.impurity_improvement(impurity)             # <<<<<<<<<<<<<<
 *             self.criterion.children_impurity(&best.impurity_left,
 *                                              &best.impurity_right)
 */
    __pyx_v_best.improvement = ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->impurity_improvement(__pyx_v_self->__pyx_base.__pyx_base.criterion, __pyx_v_impurity);

    /* "sklearn/tree/_splitter.pyx":1595
 *             self.criterion.update(best.pos)
 *             best.improvement = self.criterion.impurity_improvement(impurity)
 *             self.criterion.children_impurity(&best.impurity_left,             # <<<<<<<<<<<<<<
 *                                              &best.impurity_right)
 * 
 */
    ((struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion *)__pyx_v_self->__pyx_base.__pyx_base.criterion->__pyx_vtab)->children_impurity(__pyx_v_self->__pyx_base.__pyx_base.criterion, (&__pyx_v_best.impurity_left), (&__pyx_v_best.impurity_right));
    goto __pyx_L26;
  }
  __pyx_L26:;

  /* "sklearn/tree/_splitter.pyx":1601
 *         # element in features[:n_known_constants] must be preserved for sibling
 *         # and child nodes
 *         memcpy(features, constant_features, sizeof(SIZE_t) * n_known_constants)             # <<<<<<<<<<<<<<
 * 
 *         # Copy newly found constant features
 */
  memcpy(__pyx_v_features, __pyx_v_constant_features, ((sizeof(__pyx_t_7sklearn_4tree_9_splitter_SIZE_t)) * __pyx_v_n_known_constants));

  /* "sklearn/tree/_splitter.pyx":1604
 * 
 *         # Copy newly found constant features
 *         memcpy(constant_features + n_known_constants,             # <<<<<<<<<<<<<<
 *                features + n_known_constants,
 *                sizeof(SIZE_t) * n_found_constants)
 */
  memcpy((__pyx_v_constant_features + __pyx_v_n_known_constants), (__pyx_v_features + __pyx_v_n_known_constants), ((sizeof(__pyx_t_7sklearn_4tree_9_splitter_SIZE_t)) * __pyx_v_n_found_constants));

  /* "sklearn/tree/_splitter.pyx":1609
 * 
 *         # Return values
 *         split[0] = best             # <<<<<<<<<<<<<<
 *         n_constant_features[0] = n_total_constants
 */
  (__pyx_v_split[0]) = __pyx_v_best;

  /* "sklearn/tree/_splitter.pyx":1610
 *         # Return values
 *         split[0] = best
 *         n_constant_features[0] = n_total_constants             # <<<<<<<<<<<<<<
 */
  (__pyx_v_n_constant_features[0]) = __pyx_v_n_total_constants;

  /* "sklearn/tree/_splitter.pyx":1396
 *                                        self.presort), self.__getstate__())
 * 
 *     cdef void node_split(self, double impurity, SplitRecord* split,             # <<<<<<<<<<<<<<
 *                          SIZE_t* n_constant_features) nogil:
 *         """Find a random split on node samples[start:end], using sparse
 */

  /* function exit code */
}

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":197
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fullfill the PEP.
 */

/* Python wrapper */
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5numpy_7ndarray___getbuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_v_copy_shape;
  int __pyx_v_i;
  int __pyx_v_ndim;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  int __pyx_v_t;
  char *__pyx_v_f;
  PyArray_Descr *__pyx_v_descr = 0;
  int __pyx_v_offset;
  int __pyx_v_hasfields;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  char *__pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  if (__pyx_v_info != NULL) {
    __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(__pyx_v_info->obj);
  }

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":203
 *             # of flags
 * 
 *             if info == NULL: return             # <<<<<<<<<<<<<<
 * 
 *             cdef int copy_shape, i, ndim
 */
  __pyx_t_1 = ((__pyx_v_info == NULL) != 0);
  if (__pyx_t_1) {
    __pyx_r = 0;
    goto __pyx_L0;
  }

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":206
 * 
 *             cdef int copy_shape, i, ndim
 *             cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 */
  __pyx_v_endian_detector = 1;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":207
 *             cdef int copy_shape, i, ndim
 *             cdef int endian_detector = 1
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 * 
 *             ndim = PyArray_NDIM(self)
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":209
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 *             ndim = PyArray_NDIM(self)             # <<<<<<<<<<<<<<
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_v_ndim = PyArray_NDIM(__pyx_v_self);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":211
 *             ndim = PyArray_NDIM(self)
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 copy_shape = 1
 *             else:
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":212
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 copy_shape = 1             # <<<<<<<<<<<<<<
 *             else:
 *                 copy_shape = 0
 */
    __pyx_v_copy_shape = 1;
    goto __pyx_L4;
  }
  /*else*/ {

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":214
 *                 copy_shape = 1
 *             else:
 *                 copy_shape = 0             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 */
    __pyx_v_copy_shape = 0;
  }
  __pyx_L4:;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":216
 *                 copy_shape = 0
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_C_CONTIGUOUS) == PyBUF_C_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L6_bool_binop_done;
  }

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":217
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_C_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_1) {

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":218
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[2]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":220
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_F_CONTIGUOUS) == PyBUF_F_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":221
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_F_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_1) {

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":222
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[2]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":224
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 *             info.buf = PyArray_DATA(self)             # <<<<<<<<<<<<<<
 *             info.ndim = ndim
 *             if copy_shape:
 */
  __pyx_v_info->buf = PyArray_DATA(__pyx_v_self);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":225
 * 
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim             # <<<<<<<<<<<<<<
 *             if copy_shape:
 *                 # Allocate new buffer for strides and shape info.
 */
  __pyx_v_info->ndim = __pyx_v_ndim;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":226
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim
 *             if copy_shape:             # <<<<<<<<<<<<<<
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 */
  __pyx_t_1 = (__pyx_v_copy_shape != 0);
  if (__pyx_t_1) {

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":229
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)             # <<<<<<<<<<<<<<
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 */
    __pyx_v_info->strides = ((Py_ssize_t *)malloc((((sizeof(Py_ssize_t)) * ((size_t)__pyx_v_ndim)) * 2)));

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":230
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)
 *                 info.shape = info.strides + ndim             # <<<<<<<<<<<<<<
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 */
    __pyx_v_info->shape = (__pyx_v_info->strides + __pyx_v_ndim);

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":231
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):             # <<<<<<<<<<<<<<
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 */
    __pyx_t_4 = __pyx_v_ndim;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_i = __pyx_t_5;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":232
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]             # <<<<<<<<<<<<<<
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 */
      (__pyx_v_info->strides[__pyx_v_i]) = (PyArray_STRIDES(__pyx_v_self)[__pyx_v_i]);

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":233
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]             # <<<<<<<<<<<<<<
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 */
      (__pyx_v_info->shape[__pyx_v_i]) = (PyArray_DIMS(__pyx_v_self)[__pyx_v_i]);
    }
    goto __pyx_L11;
  }
  /*else*/ {

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":235
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)             # <<<<<<<<<<<<<<
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 */
    __pyx_v_info->strides = ((Py_ssize_t *)PyArray_STRIDES(__pyx_v_self));

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":236
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)             # <<<<<<<<<<<<<<
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 */
    __pyx_v_info->shape = ((Py_ssize_t *)PyArray_DIMS(__pyx_v_self));
  }
  __pyx_L11:;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":237
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL             # <<<<<<<<<<<<<<
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 */
  __pyx_v_info->suboffsets = NULL;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":238
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)             # <<<<<<<<<<<<<<
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 * 
 */
  __pyx_v_info->itemsize = PyArray_ITEMSIZE(__pyx_v_self);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":239
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)             # <<<<<<<<<<<<<<
 * 
 *             cdef int t
 */
  __pyx_v_info->readonly = (!(PyArray_ISWRITEABLE(__pyx_v_self) != 0));

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":242
 * 
 *             cdef int t
 *             cdef char* f = NULL             # <<<<<<<<<<<<<<
 *             cdef dtype descr = self.descr
 *             cdef list stack
 */
  __pyx_v_f = NULL;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":243
 *             cdef int t
 *             cdef char* f = NULL
 *             cdef dtype descr = self.descr             # <<<<<<<<<<<<<<
 *             cdef list stack
 *             cdef int offset
 */
  __pyx_t_3 = ((PyObject *)__pyx_v_self->descr);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_v_descr = ((PyArray_Descr *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":247
 *             cdef int offset
 * 
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)             # <<<<<<<<<<<<<<
 * 
 *             if not hasfields and not copy_shape:
 */
  __pyx_v_hasfields = PyDataType_HASFIELDS(__pyx_v_descr);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":249
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)
 * 
 *             if not hasfields and not copy_shape:             # <<<<<<<<<<<<<<
 *                 # do not call releasebuffer
 *                 info.obj = None
 */
  __pyx_t_2 = ((!(__pyx_v_hasfields != 0)) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_2 = ((!(__pyx_v_copy_shape != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L15_bool_binop_done:;
  if (__pyx_t_1) {

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":251
 *             if not hasfields and not copy_shape:
 *                 # do not call releasebuffer
 *                 info.obj = None             # <<<<<<<<<<<<<<
 *             else:
 *                 # need to call releasebuffer
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj);
    __pyx_v_info->obj = Py_None;
    goto __pyx_L14;
  }
  /*else*/ {

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":254
 *             else:
 *                 # need to call releasebuffer
 *                 info.obj = self             # <<<<<<<<<<<<<<
 * 
 *             if not hasfields:
 */
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj);
    __pyx_v_info->obj = ((PyObject *)__pyx_v_self);
  }
  __pyx_L14:;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":256
 *                 info.obj = self
 * 
 *             if not hasfields:             # <<<<<<<<<<<<<<
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 */
  __pyx_t_1 = ((!(__pyx_v_hasfields != 0)) != 0);
  if (__pyx_t_1) {

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":257
 * 
 *             if not hasfields:
 *                 t = descr.type_num             # <<<<<<<<<<<<<<
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 */
    __pyx_t_4 = __pyx_v_descr->type_num;
    __pyx_v_t = __pyx_t_4;

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":258
 *             if not hasfields:
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '>') != 0);
    if (!__pyx_t_2) {
      goto __pyx_L20_next_or;
    } else {
    }
    __pyx_t_2 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L19_bool_binop_done;
    }
    __pyx_L20_next_or:;

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":259
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '<') != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L19_bool_binop_done;
    }
    __pyx_t_2 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L19_bool_binop_done:;
    if (__pyx_t_1) {

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":260
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 260; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[2]; __pyx_lineno = 260; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":277
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"             # <<<<<<<<<<<<<<
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
    switch (__pyx_v_t) {

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":261
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 */
      case NPY_BYTE:
      __pyx_v_f = __pyx_k_b;
      break;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":262
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 */
      case NPY_UBYTE:
      __pyx_v_f = __pyx_k_B;
      break;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":263
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 */
      case NPY_SHORT:
      __pyx_v_f = __pyx_k_h;
      break;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":264
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 */
      case NPY_USHORT:
      __pyx_v_f = __pyx_k_H;
      break;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":265
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 */
      case NPY_INT:
      __pyx_v_f = __pyx_k_i;
      break;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":266
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 */
      case NPY_UINT:
      __pyx_v_f = __pyx_k_I;
      break;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":267
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 */
      case NPY_LONG:
      __pyx_v_f = __pyx_k_l;
      break;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":268
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 */
      case NPY_ULONG:
      __pyx_v_f = __pyx_k_L;
      break;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":269
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 */
      case NPY_LONGLONG:
      __pyx_v_f = __pyx_k_q;
      break;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":270
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 */
      case NPY_ULONGLONG:
      __pyx_v_f = __pyx_k_Q;
      break;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":271
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 */
      case NPY_FLOAT:
      __pyx_v_f = __pyx_k_f;
      break;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":272
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 */
      case NPY_DOUBLE:
      __pyx_v_f = __pyx_k_d;
      break;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":273
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 */
      case NPY_LONGDOUBLE:
      __pyx_v_f = __pyx_k_g;
      break;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":274
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 */
      case NPY_CFLOAT:
      __pyx_v_f = __pyx_k_Zf;
      break;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":275
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"
 */
      case NPY_CDOUBLE:
      __pyx_v_f = __pyx_k_Zd;
      break;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":276
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 */
      case NPY_CLONGDOUBLE:
      __pyx_v_f = __pyx_k_Zg;
      break;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":277
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"             # <<<<<<<<<<<<<<
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      case NPY_OBJECT:
      __pyx_v_f = __pyx_k_O;
      break;
      default:

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":279
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *                 info.format = f
 *                 return
 */
      __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_t); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 279; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_6 = PyUnicode_Format(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_t_3); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 279; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 279; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_3, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 279; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      {__pyx_filename = __pyx_f[2]; __pyx_lineno = 279; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      break;
    }

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":280
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f             # <<<<<<<<<<<<<<
 *                 return
 *             else:
 */
    __pyx_v_info->format = __pyx_v_f;

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":281
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f
 *                 return             # <<<<<<<<<<<<<<
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 */
    __pyx_r = 0;
    goto __pyx_L0;
  }
  /*else*/ {

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":283
 *                 return
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)             # <<<<<<<<<<<<<<
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 */
    __pyx_v_info->format = ((char *)malloc(255));

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":284
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment             # <<<<<<<<<<<<<<
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,
 */
    (__pyx_v_info->format[0]) = '^';

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":285
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0             # <<<<<<<<<<<<<<
 *                 f = _util_dtypestring(descr, info.format + 1,
 *                                       info.format + _buffer_format_string_len,
 */
    __pyx_v_offset = 0;

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":286
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,             # <<<<<<<<<<<<<<
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 */
    __pyx_t_7 = __pyx_f_5numpy__util_dtypestring(__pyx_v_descr, (__pyx_v_info->format + 1), (__pyx_v_info->format + 255), (&__pyx_v_offset)); if (unlikely(__pyx_t_7 == NULL)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 286; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_f = __pyx_t_7;

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":289
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 *                 f[0] = c'\0' # Terminate format string             # <<<<<<<<<<<<<<
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 */
    (__pyx_v_f[0]) = '\x00';
  }

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":197
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fullfill the PEP.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("numpy.ndarray.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  if (__pyx_v_info != NULL && __pyx_v_info->obj != NULL) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = NULL;
  }
  goto __pyx_L2;
  __pyx_L0:;
  if (__pyx_v_info != NULL && __pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(Py_None);
    __Pyx_DECREF(Py_None); __pyx_v_info->obj = NULL;
  }
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_descr);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":291
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 */

/* Python wrapper */
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info); /*proto*/
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__releasebuffer__ (wrapper)", 0);
  __pyx_pf_5numpy_7ndarray_2__releasebuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__releasebuffer__", 0);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":292
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_t_1 = (PyArray_HASFIELDS(__pyx_v_self) != 0);
  if (__pyx_t_1) {

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":293
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)             # <<<<<<<<<<<<<<
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 stdlib.free(info.strides)
 */
    free(__pyx_v_info->format);
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":294
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.strides)
 *                 # info.shape was stored after info.strides in the same block
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":295
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 stdlib.free(info.strides)             # <<<<<<<<<<<<<<
 *                 # info.shape was stored after info.strides in the same block
 * 
 */
    free(__pyx_v_info->strides);
    goto __pyx_L4;
  }
  __pyx_L4:;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":291
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":771
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":772
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 772; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":771
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":774
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":775
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 775; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":774
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":777
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":778
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 778; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":777
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":780
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":781
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 781; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":780
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":783
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":784
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 784; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":783
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":786
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *__pyx_v_descr, char *__pyx_v_f, char *__pyx_v_end, int *__pyx_v_offset) {
  PyArray_Descr *__pyx_v_child = 0;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  PyObject *__pyx_v_fields = 0;
  PyObject *__pyx_v_childname = NULL;
  PyObject *__pyx_v_new_offset = NULL;
  PyObject *__pyx_v_t = NULL;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  long __pyx_t_8;
  char *__pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_util_dtypestring", 0);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":793
 *     cdef int delta_offset
 *     cdef tuple i
 *     cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 *     cdef tuple fields
 */
  __pyx_v_endian_detector = 1;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":794
 *     cdef tuple i
 *     cdef int endian_detector = 1
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 *     cdef tuple fields
 * 
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":797
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  if (unlikely(__pyx_v_descr->names == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[2]; __pyx_lineno = 797; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __pyx_v_descr->names; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 797; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 797; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_childname, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":798
 * 
 *     for childname in descr.names:
 *         fields = descr.fields[childname]             # <<<<<<<<<<<<<<
 *         child, new_offset = fields
 * 
 */
    if (unlikely(__pyx_v_descr->fields == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[2]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_descr->fields, __pyx_v_childname); if (unlikely(__pyx_t_3 == NULL)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyTuple_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_t_3)->tp_name), 0))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_fields, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":799
 *     for childname in descr.names:
 *         fields = descr.fields[childname]
 *         child, new_offset = fields             # <<<<<<<<<<<<<<
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 */
    if (likely(__pyx_v_fields != Py_None)) {
      PyObject* sequence = __pyx_v_fields;
      #if CYTHON_COMPILING_IN_CPYTHON
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        {__pyx_filename = __pyx_f[2]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      #if CYTHON_COMPILING_IN_CPYTHON
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      #else
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      #endif
    } else {
      __Pyx_RaiseNoneNotIterableError(); {__pyx_filename = __pyx_f[2]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_dtype))))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_child, ((PyArray_Descr *)__pyx_t_3));
    __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_new_offset, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":801
 *         child, new_offset = fields
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 */
    __pyx_t_4 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 801; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = PyNumber_Subtract(__pyx_v_new_offset, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 801; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 801; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = ((((__pyx_v_end - __pyx_v_f) - ((int)__pyx_t_5)) < 15) != 0);
    if (__pyx_t_6) {

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":802
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 802; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[2]; __pyx_lineno = 802; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":804
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '>') != 0);
    if (!__pyx_t_7) {
      goto __pyx_L8_next_or;
    } else {
    }
    __pyx_t_7 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_L8_next_or:;

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":805
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *             raise ValueError(u"Non-native byte order not supported")
 *             # One could encode it in the format string and have Cython
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '<') != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_7 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L7_bool_binop_done:;
    if (__pyx_t_6) {

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":806
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *             # One could encode it in the format string and have Cython
 *             # complain instead, BUT: < and > in format strings also imply
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 806; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[2]; __pyx_lineno = 806; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":816
 * 
 *         # Output padding bytes
 *         while offset[0] < new_offset:             # <<<<<<<<<<<<<<
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 */
    while (1) {
      __pyx_t_3 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 816; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_v_new_offset, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 816; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 816; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (!__pyx_t_6) break;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":817
 *         # Output padding bytes
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte             # <<<<<<<<<<<<<<
 *             f += 1
 *             offset[0] += 1
 */
      (__pyx_v_f[0]) = 120;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":818
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte
 *             f += 1             # <<<<<<<<<<<<<<
 *             offset[0] += 1
 * 
 */
      __pyx_v_f = (__pyx_v_f + 1);

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":819
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 *             offset[0] += 1             # <<<<<<<<<<<<<<
 * 
 *         offset[0] += child.itemsize
 */
      __pyx_t_8 = 0;
      (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + 1);
    }

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":821
 *             offset[0] += 1
 * 
 *         offset[0] += child.itemsize             # <<<<<<<<<<<<<<
 * 
 *         if not PyDataType_HASFIELDS(child):
 */
    __pyx_t_8 = 0;
    (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + __pyx_v_child->elsize);

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":823
 *         offset[0] += child.itemsize
 * 
 *         if not PyDataType_HASFIELDS(child):             # <<<<<<<<<<<<<<
 *             t = child.type_num
 *             if end - f < 5:
 */
    __pyx_t_6 = ((!(PyDataType_HASFIELDS(__pyx_v_child) != 0)) != 0);
    if (__pyx_t_6) {

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":824
 * 
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num             # <<<<<<<<<<<<<<
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")
 */
      __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_child->type_num); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 824; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_XDECREF_SET(__pyx_v_t, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":825
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num
 *             if end - f < 5:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError(u"Format string allocated too short.")
 * 
 */
      __pyx_t_6 = (((__pyx_v_end - __pyx_v_f) < 5) != 0);
      if (__pyx_t_6) {

        /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":826
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_Raise(__pyx_t_4, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        {__pyx_filename = __pyx_f[2]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":829
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 */
      __pyx_t_4 = PyInt_FromLong(NPY_BYTE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 98;
        goto __pyx_L15;
      }

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":830
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"             # <<<<<<<<<<<<<<
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 */
      __pyx_t_3 = PyInt_FromLong(NPY_UBYTE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 66;
        goto __pyx_L15;
      }

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":831
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"             # <<<<<<<<<<<<<<
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 */
      __pyx_t_4 = PyInt_FromLong(NPY_SHORT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 104;
        goto __pyx_L15;
      }

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":832
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"             # <<<<<<<<<<<<<<
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 */
      __pyx_t_3 = PyInt_FromLong(NPY_USHORT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 72;
        goto __pyx_L15;
      }

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":833
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 */
      __pyx_t_4 = PyInt_FromLong(NPY_INT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 105;
        goto __pyx_L15;
      }

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":834
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 */
      __pyx_t_3 = PyInt_FromLong(NPY_UINT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 73;
        goto __pyx_L15;
      }

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":835
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 */
      __pyx_t_4 = PyInt_FromLong(NPY_LONG); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 108;
        goto __pyx_L15;
      }

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":836
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 */
      __pyx_t_3 = PyInt_FromLong(NPY_ULONG); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 76;
        goto __pyx_L15;
      }

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":837
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 */
      __pyx_t_4 = PyInt_FromLong(NPY_LONGLONG); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 113;
        goto __pyx_L15;
      }

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":838
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 */
      __pyx_t_3 = PyInt_FromLong(NPY_ULONGLONG); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 81;
        goto __pyx_L15;
      }

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":839
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"             # <<<<<<<<<<<<<<
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 */
      __pyx_t_4 = PyInt_FromLong(NPY_FLOAT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 102;
        goto __pyx_L15;
      }

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":840
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 */
      __pyx_t_3 = PyInt_FromLong(NPY_DOUBLE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 100;
        goto __pyx_L15;
      }

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":841
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"             # <<<<<<<<<<<<<<
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 */
      __pyx_t_4 = PyInt_FromLong(NPY_LONGDOUBLE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 103;
        goto __pyx_L15;
      }

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":842
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf             # <<<<<<<<<<<<<<
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 */
      __pyx_t_3 = PyInt_FromLong(NPY_CFLOAT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 102;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":843
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd             # <<<<<<<<<<<<<<
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 */
      __pyx_t_4 = PyInt_FromLong(NPY_CDOUBLE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 843; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 843; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 843; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 100;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":844
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg             # <<<<<<<<<<<<<<
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 */
      __pyx_t_3 = PyInt_FromLong(NPY_CLONGDOUBLE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 103;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":845
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"             # <<<<<<<<<<<<<<
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      __pyx_t_4 = PyInt_FromLong(NPY_OBJECT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 845; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 845; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 845; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 79;
        goto __pyx_L15;
      }
      /*else*/ {

        /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":847
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *             f += 1
 *         else:
 */
        __pyx_t_3 = PyUnicode_Format(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_v_t); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_Raise(__pyx_t_3, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        {__pyx_filename = __pyx_f[2]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_L15:;

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":848
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *             f += 1             # <<<<<<<<<<<<<<
 *         else:
 *             # Cython ignores struct boundary information ("T{...}"),
 */
      __pyx_v_f = (__pyx_v_f + 1);
      goto __pyx_L13;
    }
    /*else*/ {

      /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":852
 *             # Cython ignores struct boundary information ("T{...}"),
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)             # <<<<<<<<<<<<<<
 *     return f
 * 
 */
      __pyx_t_9 = __pyx_f_5numpy__util_dtypestring(__pyx_v_child, __pyx_v_f, __pyx_v_end, __pyx_v_offset); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 852; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_v_f = __pyx_t_9;
    }
    __pyx_L13:;

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":797
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":853
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)
 *     return f             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_f;
  goto __pyx_L0;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":786
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("numpy._util_dtypestring", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child);
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XDECREF(__pyx_v_childname);
  __Pyx_XDECREF(__pyx_v_new_offset);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":969
 * 
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *      cdef PyObject* baseptr
 *      if base is None:
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  PyObject *__pyx_v_baseptr;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":971
 * cdef inline void set_array_base(ndarray arr, object base):
 *      cdef PyObject* baseptr
 *      if base is None:             # <<<<<<<<<<<<<<
 *          baseptr = NULL
 *      else:
 */
  __pyx_t_1 = (__pyx_v_base == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":972
 *      cdef PyObject* baseptr
 *      if base is None:
 *          baseptr = NULL             # <<<<<<<<<<<<<<
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!
 */
    __pyx_v_baseptr = NULL;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":974
 *          baseptr = NULL
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!             # <<<<<<<<<<<<<<
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)
 */
    Py_INCREF(__pyx_v_base);

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":975
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!
 *          baseptr = <PyObject*>base             # <<<<<<<<<<<<<<
 *      Py_XDECREF(arr.base)
 *      arr.base = baseptr
 */
    __pyx_v_baseptr = ((PyObject *)__pyx_v_base);
  }
  __pyx_L3:;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":976
 *          Py_INCREF(base) # important to do this before decref below!
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)             # <<<<<<<<<<<<<<
 *      arr.base = baseptr
 * 
 */
  Py_XDECREF(__pyx_v_arr->base);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":977
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)
 *      arr.base = baseptr             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  __pyx_v_arr->base = __pyx_v_baseptr;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":969
 * 
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *      cdef PyObject* baseptr
 *      if base is None:
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":979
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":980
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_arr->base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":981
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     else:
 *         return <object>arr.base
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_None);
    __pyx_r = Py_None;
    goto __pyx_L0;
  }
  /*else*/ {

    /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":983
 *         return None
 *     else:
 *         return <object>arr.base             # <<<<<<<<<<<<<<
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_arr->base));
    __pyx_r = ((PyObject *)__pyx_v_arr->base);
    goto __pyx_L0;
  }

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":979
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_Splitter __pyx_vtable_7sklearn_4tree_9_splitter_Splitter;

static PyObject *__pyx_tp_new_7sklearn_4tree_9_splitter_Splitter(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *)o);
  p->__pyx_vtab = __pyx_vtabptr_7sklearn_4tree_9_splitter_Splitter;
  p->criterion = ((struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *)Py_None); Py_INCREF(Py_None);
  p->random_state = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_7sklearn_4tree_9_splitter_8Splitter_1__cinit__(o, a, k) < 0)) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_7sklearn_4tree_9_splitter_Splitter(PyObject *o) {
  struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *p = (struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->criterion);
  Py_CLEAR(p->random_state);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_7sklearn_4tree_9_splitter_Splitter(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *p = (struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *)o;
  if (p->criterion) {
    e = (*v)(((PyObject*)p->criterion), a); if (e) return e;
  }
  if (p->random_state) {
    e = (*v)(p->random_state, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7sklearn_4tree_9_splitter_Splitter(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *p = (struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *)o;
  tmp = ((PyObject*)p->criterion);
  p->criterion = ((struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->random_state);
  p->random_state = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_7sklearn_4tree_9_splitter_8Splitter_criterion(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_9criterion_1__get__(o);
}

static int __pyx_setprop_7sklearn_4tree_9_splitter_8Splitter_criterion(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_9criterion_3__set__(o, v);
  }
  else {
    return __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_9criterion_5__del__(o);
  }
}

static PyObject *__pyx_getprop_7sklearn_4tree_9_splitter_8Splitter_max_features(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_12max_features_1__get__(o);
}

static int __pyx_setprop_7sklearn_4tree_9_splitter_8Splitter_max_features(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_12max_features_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7sklearn_4tree_9_splitter_8Splitter_min_samples_leaf(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_16min_samples_leaf_1__get__(o);
}

static int __pyx_setprop_7sklearn_4tree_9_splitter_8Splitter_min_samples_leaf(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_16min_samples_leaf_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7sklearn_4tree_9_splitter_8Splitter_min_weight_leaf(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_15min_weight_leaf_1__get__(o);
}

static int __pyx_setprop_7sklearn_4tree_9_splitter_8Splitter_min_weight_leaf(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7sklearn_4tree_9_splitter_8Splitter_15min_weight_leaf_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_7sklearn_4tree_9_splitter_Splitter[] = {
  {"__getstate__", (PyCFunction)__pyx_pw_7sklearn_4tree_9_splitter_8Splitter_5__getstate__, METH_NOARGS, 0},
  {"__setstate__", (PyCFunction)__pyx_pw_7sklearn_4tree_9_splitter_8Splitter_7__setstate__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_7sklearn_4tree_9_splitter_Splitter[] = {
  {(char *)"criterion", __pyx_getprop_7sklearn_4tree_9_splitter_8Splitter_criterion, __pyx_setprop_7sklearn_4tree_9_splitter_8Splitter_criterion, 0, 0},
  {(char *)"max_features", __pyx_getprop_7sklearn_4tree_9_splitter_8Splitter_max_features, __pyx_setprop_7sklearn_4tree_9_splitter_8Splitter_max_features, 0, 0},
  {(char *)"min_samples_leaf", __pyx_getprop_7sklearn_4tree_9_splitter_8Splitter_min_samples_leaf, __pyx_setprop_7sklearn_4tree_9_splitter_8Splitter_min_samples_leaf, 0, 0},
  {(char *)"min_weight_leaf", __pyx_getprop_7sklearn_4tree_9_splitter_8Splitter_min_weight_leaf, __pyx_setprop_7sklearn_4tree_9_splitter_8Splitter_min_weight_leaf, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_7sklearn_4tree_9_splitter_Splitter = {
  PyVarObject_HEAD_INIT(0, 0)
  "sklearn.tree._splitter.Splitter", /*tp_name*/
  sizeof(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7sklearn_4tree_9_splitter_Splitter, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Abstract splitter class.\n\n    Splitters are called by tree builders to find the best splits on both\n    sparse and dense data, one split at a time.\n    ", /*tp_doc*/
  __pyx_tp_traverse_7sklearn_4tree_9_splitter_Splitter, /*tp_traverse*/
  __pyx_tp_clear_7sklearn_4tree_9_splitter_Splitter, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7sklearn_4tree_9_splitter_Splitter, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_7sklearn_4tree_9_splitter_Splitter, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7sklearn_4tree_9_splitter_Splitter, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_BaseDenseSplitter __pyx_vtable_7sklearn_4tree_9_splitter_BaseDenseSplitter;

static PyObject *__pyx_tp_new_7sklearn_4tree_9_splitter_BaseDenseSplitter(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter *p;
  PyObject *o = __pyx_tp_new_7sklearn_4tree_9_splitter_Splitter(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_Splitter*)__pyx_vtabptr_7sklearn_4tree_9_splitter_BaseDenseSplitter;
  p->X_idx_sorted = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_7sklearn_4tree_9_splitter_17BaseDenseSplitter_1__cinit__(o, a, k) < 0)) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_7sklearn_4tree_9_splitter_BaseDenseSplitter(PyObject *o) {
  struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter *p = (struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_7sklearn_4tree_9_splitter_17BaseDenseSplitter_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->X_idx_sorted);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_7sklearn_4tree_9_splitter_Splitter(o);
}

static int __pyx_tp_traverse_7sklearn_4tree_9_splitter_BaseDenseSplitter(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter *p = (struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter *)o;
  e = __pyx_tp_traverse_7sklearn_4tree_9_splitter_Splitter(o, v, a); if (e) return e;
  if (p->X_idx_sorted) {
    e = (*v)(((PyObject*)p->X_idx_sorted), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7sklearn_4tree_9_splitter_BaseDenseSplitter(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter *p = (struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter *)o;
  __pyx_tp_clear_7sklearn_4tree_9_splitter_Splitter(o);
  tmp = ((PyObject*)p->X_idx_sorted);
  p->X_idx_sorted = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_7sklearn_4tree_9_splitter_BaseDenseSplitter[] = {
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_7sklearn_4tree_9_splitter_BaseDenseSplitter = {
  PyVarObject_HEAD_INIT(0, 0)
  "sklearn.tree._splitter.BaseDenseSplitter", /*tp_name*/
  sizeof(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseDenseSplitter), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7sklearn_4tree_9_splitter_BaseDenseSplitter, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7sklearn_4tree_9_splitter_BaseDenseSplitter, /*tp_traverse*/
  __pyx_tp_clear_7sklearn_4tree_9_splitter_BaseDenseSplitter, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7sklearn_4tree_9_splitter_BaseDenseSplitter, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7sklearn_4tree_9_splitter_BaseDenseSplitter, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_BestSplitter __pyx_vtable_7sklearn_4tree_9_splitter_BestSplitter;

static PyObject *__pyx_tp_new_7sklearn_4tree_9_splitter_BestSplitter(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7sklearn_4tree_9_splitter_BestSplitter *p;
  PyObject *o = __pyx_tp_new_7sklearn_4tree_9_splitter_BaseDenseSplitter(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7sklearn_4tree_9_splitter_BestSplitter *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_Splitter*)__pyx_vtabptr_7sklearn_4tree_9_splitter_BestSplitter;
  return o;
}

static PyMethodDef __pyx_methods_7sklearn_4tree_9_splitter_BestSplitter[] = {
  {"__reduce__", (PyCFunction)__pyx_pw_7sklearn_4tree_9_splitter_12BestSplitter_1__reduce__, METH_NOARGS, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_7sklearn_4tree_9_splitter_BestSplitter = {
  PyVarObject_HEAD_INIT(0, 0)
  "sklearn.tree._splitter.BestSplitter", /*tp_name*/
  sizeof(struct __pyx_obj_7sklearn_4tree_9_splitter_BestSplitter), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7sklearn_4tree_9_splitter_BaseDenseSplitter, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Splitter for finding the best split.", /*tp_doc*/
  __pyx_tp_traverse_7sklearn_4tree_9_splitter_BaseDenseSplitter, /*tp_traverse*/
  __pyx_tp_clear_7sklearn_4tree_9_splitter_BaseDenseSplitter, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7sklearn_4tree_9_splitter_BestSplitter, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7sklearn_4tree_9_splitter_BestSplitter, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_RandomSplitter __pyx_vtable_7sklearn_4tree_9_splitter_RandomSplitter;

static PyObject *__pyx_tp_new_7sklearn_4tree_9_splitter_RandomSplitter(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7sklearn_4tree_9_splitter_RandomSplitter *p;
  PyObject *o = __pyx_tp_new_7sklearn_4tree_9_splitter_BaseDenseSplitter(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7sklearn_4tree_9_splitter_RandomSplitter *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_Splitter*)__pyx_vtabptr_7sklearn_4tree_9_splitter_RandomSplitter;
  return o;
}

static PyMethodDef __pyx_methods_7sklearn_4tree_9_splitter_RandomSplitter[] = {
  {"__reduce__", (PyCFunction)__pyx_pw_7sklearn_4tree_9_splitter_14RandomSplitter_1__reduce__, METH_NOARGS, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_7sklearn_4tree_9_splitter_RandomSplitter = {
  PyVarObject_HEAD_INIT(0, 0)
  "sklearn.tree._splitter.RandomSplitter", /*tp_name*/
  sizeof(struct __pyx_obj_7sklearn_4tree_9_splitter_RandomSplitter), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7sklearn_4tree_9_splitter_BaseDenseSplitter, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Splitter for finding the best random split.", /*tp_doc*/
  __pyx_tp_traverse_7sklearn_4tree_9_splitter_BaseDenseSplitter, /*tp_traverse*/
  __pyx_tp_clear_7sklearn_4tree_9_splitter_BaseDenseSplitter, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7sklearn_4tree_9_splitter_RandomSplitter, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7sklearn_4tree_9_splitter_RandomSplitter, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_BaseSparseSplitter __pyx_vtable_7sklearn_4tree_9_splitter_BaseSparseSplitter;

static PyObject *__pyx_tp_new_7sklearn_4tree_9_splitter_BaseSparseSplitter(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *p;
  PyObject *o = __pyx_tp_new_7sklearn_4tree_9_splitter_Splitter(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_Splitter*)__pyx_vtabptr_7sklearn_4tree_9_splitter_BaseSparseSplitter;
  if (unlikely(__pyx_pw_7sklearn_4tree_9_splitter_18BaseSparseSplitter_1__cinit__(o, a, k) < 0)) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_7sklearn_4tree_9_splitter_BaseSparseSplitter(PyObject *o) {
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_7sklearn_4tree_9_splitter_18BaseSparseSplitter_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_7sklearn_4tree_9_splitter_Splitter(o);
}

static PyMethodDef __pyx_methods_7sklearn_4tree_9_splitter_BaseSparseSplitter[] = {
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_7sklearn_4tree_9_splitter_BaseSparseSplitter = {
  PyVarObject_HEAD_INIT(0, 0)
  "sklearn.tree._splitter.BaseSparseSplitter", /*tp_name*/
  sizeof(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7sklearn_4tree_9_splitter_BaseSparseSplitter, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7sklearn_4tree_9_splitter_Splitter, /*tp_traverse*/
  __pyx_tp_clear_7sklearn_4tree_9_splitter_Splitter, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7sklearn_4tree_9_splitter_BaseSparseSplitter, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7sklearn_4tree_9_splitter_BaseSparseSplitter, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_BestSparseSplitter __pyx_vtable_7sklearn_4tree_9_splitter_BestSparseSplitter;

static PyObject *__pyx_tp_new_7sklearn_4tree_9_splitter_BestSparseSplitter(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7sklearn_4tree_9_splitter_BestSparseSplitter *p;
  PyObject *o = __pyx_tp_new_7sklearn_4tree_9_splitter_BaseSparseSplitter(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7sklearn_4tree_9_splitter_BestSparseSplitter *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_Splitter*)__pyx_vtabptr_7sklearn_4tree_9_splitter_BestSparseSplitter;
  return o;
}

static PyMethodDef __pyx_methods_7sklearn_4tree_9_splitter_BestSparseSplitter[] = {
  {"__reduce__", (PyCFunction)__pyx_pw_7sklearn_4tree_9_splitter_18BestSparseSplitter_1__reduce__, METH_NOARGS, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_7sklearn_4tree_9_splitter_BestSparseSplitter = {
  PyVarObject_HEAD_INIT(0, 0)
  "sklearn.tree._splitter.BestSparseSplitter", /*tp_name*/
  sizeof(struct __pyx_obj_7sklearn_4tree_9_splitter_BestSparseSplitter), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7sklearn_4tree_9_splitter_BaseSparseSplitter, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Splitter for finding the best split, using the sparse data.", /*tp_doc*/
  __pyx_tp_traverse_7sklearn_4tree_9_splitter_Splitter, /*tp_traverse*/
  __pyx_tp_clear_7sklearn_4tree_9_splitter_Splitter, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7sklearn_4tree_9_splitter_BestSparseSplitter, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7sklearn_4tree_9_splitter_BestSparseSplitter, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_RandomSparseSplitter __pyx_vtable_7sklearn_4tree_9_splitter_RandomSparseSplitter;

static PyObject *__pyx_tp_new_7sklearn_4tree_9_splitter_RandomSparseSplitter(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7sklearn_4tree_9_splitter_RandomSparseSplitter *p;
  PyObject *o = __pyx_tp_new_7sklearn_4tree_9_splitter_BaseSparseSplitter(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7sklearn_4tree_9_splitter_RandomSparseSplitter *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_7sklearn_4tree_9_splitter_Splitter*)__pyx_vtabptr_7sklearn_4tree_9_splitter_RandomSparseSplitter;
  return o;
}

static PyMethodDef __pyx_methods_7sklearn_4tree_9_splitter_RandomSparseSplitter[] = {
  {"__reduce__", (PyCFunction)__pyx_pw_7sklearn_4tree_9_splitter_20RandomSparseSplitter_1__reduce__, METH_NOARGS, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_7sklearn_4tree_9_splitter_RandomSparseSplitter = {
  PyVarObject_HEAD_INIT(0, 0)
  "sklearn.tree._splitter.RandomSparseSplitter", /*tp_name*/
  sizeof(struct __pyx_obj_7sklearn_4tree_9_splitter_RandomSparseSplitter), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7sklearn_4tree_9_splitter_BaseSparseSplitter, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Splitter for finding a random split, using the sparse data.", /*tp_doc*/
  __pyx_tp_traverse_7sklearn_4tree_9_splitter_Splitter, /*tp_traverse*/
  __pyx_tp_clear_7sklearn_4tree_9_splitter_Splitter, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7sklearn_4tree_9_splitter_RandomSparseSplitter, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7sklearn_4tree_9_splitter_RandomSparseSplitter, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    "_splitter",
    0, /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_u_Format_string_allocated_too_shor, __pyx_k_Format_string_allocated_too_shor, sizeof(__pyx_k_Format_string_allocated_too_shor), 0, 1, 0, 0},
  {&__pyx_kp_u_Format_string_allocated_too_shor_2, __pyx_k_Format_string_allocated_too_shor_2, sizeof(__pyx_k_Format_string_allocated_too_shor_2), 0, 1, 0, 0},
  {&__pyx_kp_u_Non_native_byte_order_not_suppor, __pyx_k_Non_native_byte_order_not_suppor, sizeof(__pyx_k_Non_native_byte_order_not_suppor), 0, 1, 0, 0},
  {&__pyx_n_s_RuntimeError, __pyx_k_RuntimeError, sizeof(__pyx_k_RuntimeError), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_kp_s_X_should_be_in_csc_format, __pyx_k_X_should_be_in_csc_format, sizeof(__pyx_k_X_should_be_in_csc_format), 0, 0, 1, 0},
  {&__pyx_n_s_criterion, __pyx_k_criterion, sizeof(__pyx_k_criterion), 0, 0, 1, 1},
  {&__pyx_n_s_csc_matrix, __pyx_k_csc_matrix, sizeof(__pyx_k_csc_matrix), 0, 0, 1, 1},
  {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_indices, __pyx_k_indices, sizeof(__pyx_k_indices), 0, 0, 1, 1},
  {&__pyx_n_s_indptr, __pyx_k_indptr, sizeof(__pyx_k_indptr), 0, 0, 1, 1},
  {&__pyx_n_s_inf, __pyx_k_inf, sizeof(__pyx_k_inf), 0, 0, 1, 1},
  {&__pyx_n_s_itemsize, __pyx_k_itemsize, sizeof(__pyx_k_itemsize), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_max_features, __pyx_k_max_features, sizeof(__pyx_k_max_features), 0, 0, 1, 1},
  {&__pyx_n_s_min_samples_leaf, __pyx_k_min_samples_leaf, sizeof(__pyx_k_min_samples_leaf), 0, 0, 1, 1},
  {&__pyx_n_s_min_weight_leaf, __pyx_k_min_weight_leaf, sizeof(__pyx_k_min_weight_leaf), 0, 0, 1, 1},
  {&__pyx_kp_u_ndarray_is_not_C_contiguous, __pyx_k_ndarray_is_not_C_contiguous, sizeof(__pyx_k_ndarray_is_not_C_contiguous), 0, 1, 0, 0},
  {&__pyx_kp_u_ndarray_is_not_Fortran_contiguou, __pyx_k_ndarray_is_not_Fortran_contiguou, sizeof(__pyx_k_ndarray_is_not_Fortran_contiguou), 0, 1, 0, 0},
  {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_n_s_presort, __pyx_k_presort, sizeof(__pyx_k_presort), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_randint, __pyx_k_randint, sizeof(__pyx_k_randint), 0, 0, 1, 1},
  {&__pyx_n_s_random_state, __pyx_k_random_state, sizeof(__pyx_k_random_state), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_scipy_sparse, __pyx_k_scipy_sparse, sizeof(__pyx_k_scipy_sparse), 0, 0, 1, 1},
  {&__pyx_n_s_shape, __pyx_k_shape, sizeof(__pyx_k_shape), 0, 0, 1, 1},
  {&__pyx_n_s_strides, __pyx_k_strides, sizeof(__pyx_k_strides), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_k_unknown_dtype_code_in_numpy_pxd, sizeof(__pyx_k_unknown_dtype_code_in_numpy_pxd), 0, 1, 0, 0},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 879; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_RuntimeError = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeError); if (!__pyx_builtin_RuntimeError) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 802; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "sklearn/tree/_splitter.pyx":879
 * 
 *         if not isinstance(X, csc_matrix):
 *             raise ValueError("X should be in csc format")             # <<<<<<<<<<<<<<
 * 
 *         cdef SIZE_t* samples = self.samples
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_X_should_be_in_csc_format); if (unlikely(!__pyx_tuple_)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 879; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":218
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_C_contiguous); if (unlikely(!__pyx_tuple__2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":222
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_Fortran_contiguou); if (unlikely(!__pyx_tuple__3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":260
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_u_Non_native_byte_order_not_suppor); if (unlikely(!__pyx_tuple__4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 260; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":802
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor); if (unlikely(!__pyx_tuple__5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 802; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":806
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *             # One could encode it in the format string and have Cython
 *             # complain instead, BUT: < and > in format strings also imply
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_u_Non_native_byte_order_not_suppor); if (unlikely(!__pyx_tuple__6)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 806; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":826
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor_2); if (unlikely(!__pyx_tuple__7)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC init_splitter(void); /*proto*/
PyMODINIT_FUNC init_splitter(void)
#else
PyMODINIT_FUNC PyInit__splitter(void); /*proto*/
PyMODINIT_FUNC PyInit__splitter(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  double __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit__splitter(void)", 0);
  if ( __Pyx_check_binary_version() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #ifdef __Pyx_CyFunction_USED
  if (__Pyx_CyFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("_splitter", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  /*--- Initialize various global constants etc. ---*/
  if (unlikely(__Pyx_InitGlobals() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  if (__pyx_module_is_main_sklearn__tree___splitter) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!PyDict_GetItemString(modules, "sklearn.tree._splitter")) {
      if (unlikely(PyDict_SetItemString(modules, "sklearn.tree._splitter", __pyx_m) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (unlikely(__Pyx_InitCachedBuiltins() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Constants init code ---*/
  if (unlikely(__Pyx_InitCachedConstants() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  __pyx_vtabptr_7sklearn_4tree_9_splitter_Splitter = &__pyx_vtable_7sklearn_4tree_9_splitter_Splitter;
  __pyx_vtable_7sklearn_4tree_9_splitter_Splitter.init = (void (*)(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *, PyObject *, PyArrayObject *, __pyx_t_7sklearn_4tree_9_splitter_DOUBLE_t *, struct __pyx_opt_args_7sklearn_4tree_9_splitter_8Splitter_init *__pyx_optional_args))__pyx_f_7sklearn_4tree_9_splitter_8Splitter_init;
  __pyx_vtable_7sklearn_4tree_9_splitter_Splitter.node_reset = (void (*)(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, double *))__pyx_f_7sklearn_4tree_9_splitter_8Splitter_node_reset;
  __pyx_vtable_7sklearn_4tree_9_splitter_Splitter.node_split = (void (*)(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *, double, struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *))__pyx_f_7sklearn_4tree_9_splitter_8Splitter_node_split;
  __pyx_vtable_7sklearn_4tree_9_splitter_Splitter.node_value = (void (*)(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *, double *))__pyx_f_7sklearn_4tree_9_splitter_8Splitter_node_value;
  __pyx_vtable_7sklearn_4tree_9_splitter_Splitter.node_impurity = (double (*)(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *))__pyx_f_7sklearn_4tree_9_splitter_8Splitter_node_impurity;
  if (PyType_Ready(&__pyx_type_7sklearn_4tree_9_splitter_Splitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_7sklearn_4tree_9_splitter_Splitter.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_7sklearn_4tree_9_splitter_Splitter.tp_dict, __pyx_vtabptr_7sklearn_4tree_9_splitter_Splitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "Splitter", (PyObject *)&__pyx_type_7sklearn_4tree_9_splitter_Splitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_7sklearn_4tree_9_splitter_Splitter = &__pyx_type_7sklearn_4tree_9_splitter_Splitter;
  __pyx_vtabptr_7sklearn_4tree_9_splitter_BaseDenseSplitter = &__pyx_vtable_7sklearn_4tree_9_splitter_BaseDenseSplitter;
  __pyx_vtable_7sklearn_4tree_9_splitter_BaseDenseSplitter.__pyx_base = *__pyx_vtabptr_7sklearn_4tree_9_splitter_Splitter;
  __pyx_vtable_7sklearn_4tree_9_splitter_BaseDenseSplitter.__pyx_base.init = (void (*)(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *, PyObject *, PyArrayObject *, __pyx_t_7sklearn_4tree_9_splitter_DOUBLE_t *, struct __pyx_opt_args_7sklearn_4tree_9_splitter_8Splitter_init *__pyx_optional_args))__pyx_f_7sklearn_4tree_9_splitter_17BaseDenseSplitter_init;
  __pyx_type_7sklearn_4tree_9_splitter_BaseDenseSplitter.tp_base = __pyx_ptype_7sklearn_4tree_9_splitter_Splitter;
  if (PyType_Ready(&__pyx_type_7sklearn_4tree_9_splitter_BaseDenseSplitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 232; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_7sklearn_4tree_9_splitter_BaseDenseSplitter.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_7sklearn_4tree_9_splitter_BaseDenseSplitter.tp_dict, __pyx_vtabptr_7sklearn_4tree_9_splitter_BaseDenseSplitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 232; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "BaseDenseSplitter", (PyObject *)&__pyx_type_7sklearn_4tree_9_splitter_BaseDenseSplitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 232; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_7sklearn_4tree_9_splitter_BaseDenseSplitter = &__pyx_type_7sklearn_4tree_9_splitter_BaseDenseSplitter;
  __pyx_vtabptr_7sklearn_4tree_9_splitter_BestSplitter = &__pyx_vtable_7sklearn_4tree_9_splitter_BestSplitter;
  __pyx_vtable_7sklearn_4tree_9_splitter_BestSplitter.__pyx_base = *__pyx_vtabptr_7sklearn_4tree_9_splitter_BaseDenseSplitter;
  __pyx_vtable_7sklearn_4tree_9_splitter_BestSplitter.__pyx_base.__pyx_base.node_split = (void (*)(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *, double, struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *))__pyx_f_7sklearn_4tree_9_splitter_12BestSplitter_node_split;
  __pyx_type_7sklearn_4tree_9_splitter_BestSplitter.tp_base = __pyx_ptype_7sklearn_4tree_9_splitter_BaseDenseSplitter;
  if (PyType_Ready(&__pyx_type_7sklearn_4tree_9_splitter_BestSplitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_7sklearn_4tree_9_splitter_BestSplitter.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_7sklearn_4tree_9_splitter_BestSplitter.tp_dict, __pyx_vtabptr_7sklearn_4tree_9_splitter_BestSplitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "BestSplitter", (PyObject *)&__pyx_type_7sklearn_4tree_9_splitter_BestSplitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_7sklearn_4tree_9_splitter_BestSplitter = &__pyx_type_7sklearn_4tree_9_splitter_BestSplitter;
  __pyx_vtabptr_7sklearn_4tree_9_splitter_RandomSplitter = &__pyx_vtable_7sklearn_4tree_9_splitter_RandomSplitter;
  __pyx_vtable_7sklearn_4tree_9_splitter_RandomSplitter.__pyx_base = *__pyx_vtabptr_7sklearn_4tree_9_splitter_BaseDenseSplitter;
  __pyx_vtable_7sklearn_4tree_9_splitter_RandomSplitter.__pyx_base.__pyx_base.node_split = (void (*)(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *, double, struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *))__pyx_f_7sklearn_4tree_9_splitter_14RandomSplitter_node_split;
  __pyx_type_7sklearn_4tree_9_splitter_RandomSplitter.tp_base = __pyx_ptype_7sklearn_4tree_9_splitter_BaseDenseSplitter;
  if (PyType_Ready(&__pyx_type_7sklearn_4tree_9_splitter_RandomSplitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 624; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_7sklearn_4tree_9_splitter_RandomSplitter.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_7sklearn_4tree_9_splitter_RandomSplitter.tp_dict, __pyx_vtabptr_7sklearn_4tree_9_splitter_RandomSplitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 624; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "RandomSplitter", (PyObject *)&__pyx_type_7sklearn_4tree_9_splitter_RandomSplitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 624; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_7sklearn_4tree_9_splitter_RandomSplitter = &__pyx_type_7sklearn_4tree_9_splitter_RandomSplitter;
  __pyx_vtabptr_7sklearn_4tree_9_splitter_BaseSparseSplitter = &__pyx_vtable_7sklearn_4tree_9_splitter_BaseSparseSplitter;
  __pyx_vtable_7sklearn_4tree_9_splitter_BaseSparseSplitter.__pyx_base = *__pyx_vtabptr_7sklearn_4tree_9_splitter_Splitter;
  __pyx_vtable_7sklearn_4tree_9_splitter_BaseSparseSplitter.__pyx_base.init = (void (*)(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *, PyObject *, PyArrayObject *, __pyx_t_7sklearn_4tree_9_splitter_DOUBLE_t *, struct __pyx_opt_args_7sklearn_4tree_9_splitter_8Splitter_init *__pyx_optional_args))__pyx_f_7sklearn_4tree_9_splitter_18BaseSparseSplitter_init;
  __pyx_vtable_7sklearn_4tree_9_splitter_BaseSparseSplitter._partition = (__pyx_t_7sklearn_4tree_9_splitter_SIZE_t (*)(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *, double, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t))__pyx_f_7sklearn_4tree_9_splitter_18BaseSparseSplitter__partition;
  __pyx_vtable_7sklearn_4tree_9_splitter_BaseSparseSplitter.extract_nnz = (void (*)(struct __pyx_obj_7sklearn_4tree_9_splitter_BaseSparseSplitter *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *, int *))__pyx_f_7sklearn_4tree_9_splitter_18BaseSparseSplitter_extract_nnz;
  __pyx_type_7sklearn_4tree_9_splitter_BaseSparseSplitter.tp_base = __pyx_ptype_7sklearn_4tree_9_splitter_Splitter;
  if (PyType_Ready(&__pyx_type_7sklearn_4tree_9_splitter_BaseSparseSplitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_7sklearn_4tree_9_splitter_BaseSparseSplitter.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_7sklearn_4tree_9_splitter_BaseSparseSplitter.tp_dict, __pyx_vtabptr_7sklearn_4tree_9_splitter_BaseSparseSplitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "BaseSparseSplitter", (PyObject *)&__pyx_type_7sklearn_4tree_9_splitter_BaseSparseSplitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_7sklearn_4tree_9_splitter_BaseSparseSplitter = &__pyx_type_7sklearn_4tree_9_splitter_BaseSparseSplitter;
  __pyx_vtabptr_7sklearn_4tree_9_splitter_BestSparseSplitter = &__pyx_vtable_7sklearn_4tree_9_splitter_BestSparseSplitter;
  __pyx_vtable_7sklearn_4tree_9_splitter_BestSparseSplitter.__pyx_base = *__pyx_vtabptr_7sklearn_4tree_9_splitter_BaseSparseSplitter;
  __pyx_vtable_7sklearn_4tree_9_splitter_BestSparseSplitter.__pyx_base.__pyx_base.node_split = (void (*)(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *, double, struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *))__pyx_f_7sklearn_4tree_9_splitter_18BestSparseSplitter_node_split;
  __pyx_type_7sklearn_4tree_9_splitter_BestSparseSplitter.tp_base = __pyx_ptype_7sklearn_4tree_9_splitter_BaseSparseSplitter;
  if (PyType_Ready(&__pyx_type_7sklearn_4tree_9_splitter_BestSparseSplitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1158; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_7sklearn_4tree_9_splitter_BestSparseSplitter.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_7sklearn_4tree_9_splitter_BestSparseSplitter.tp_dict, __pyx_vtabptr_7sklearn_4tree_9_splitter_BestSparseSplitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1158; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "BestSparseSplitter", (PyObject *)&__pyx_type_7sklearn_4tree_9_splitter_BestSparseSplitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1158; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_7sklearn_4tree_9_splitter_BestSparseSplitter = &__pyx_type_7sklearn_4tree_9_splitter_BestSparseSplitter;
  __pyx_vtabptr_7sklearn_4tree_9_splitter_RandomSparseSplitter = &__pyx_vtable_7sklearn_4tree_9_splitter_RandomSparseSplitter;
  __pyx_vtable_7sklearn_4tree_9_splitter_RandomSparseSplitter.__pyx_base = *__pyx_vtabptr_7sklearn_4tree_9_splitter_BaseSparseSplitter;
  __pyx_vtable_7sklearn_4tree_9_splitter_RandomSparseSplitter.__pyx_base.__pyx_base.node_split = (void (*)(struct __pyx_obj_7sklearn_4tree_9_splitter_Splitter *, double, struct __pyx_t_7sklearn_4tree_9_splitter_SplitRecord *, __pyx_t_7sklearn_4tree_9_splitter_SIZE_t *))__pyx_f_7sklearn_4tree_9_splitter_20RandomSparseSplitter_node_split;
  __pyx_type_7sklearn_4tree_9_splitter_RandomSparseSplitter.tp_base = __pyx_ptype_7sklearn_4tree_9_splitter_BaseSparseSplitter;
  if (PyType_Ready(&__pyx_type_7sklearn_4tree_9_splitter_RandomSparseSplitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1385; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_7sklearn_4tree_9_splitter_RandomSparseSplitter.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_7sklearn_4tree_9_splitter_RandomSparseSplitter.tp_dict, __pyx_vtabptr_7sklearn_4tree_9_splitter_RandomSparseSplitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1385; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "RandomSparseSplitter", (PyObject *)&__pyx_type_7sklearn_4tree_9_splitter_RandomSparseSplitter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1385; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_7sklearn_4tree_9_splitter_RandomSparseSplitter = &__pyx_type_7sklearn_4tree_9_splitter_RandomSparseSplitter;
  /*--- Type import code ---*/
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "type", 
  #if CYTHON_COMPILING_IN_PYPY
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  0); if (unlikely(!__pyx_ptype_7cpython_4type_type)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 9; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType("numpy", "dtype", sizeof(PyArray_Descr), 0); if (unlikely(!__pyx_ptype_5numpy_dtype)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType("numpy", "flatiter", sizeof(PyArrayIterObject), 0); if (unlikely(!__pyx_ptype_5numpy_flatiter)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType("numpy", "broadcast", sizeof(PyArrayMultiIterObject), 0); if (unlikely(!__pyx_ptype_5numpy_broadcast)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType("numpy", "ndarray", sizeof(PyArrayObject), 0); if (unlikely(!__pyx_ptype_5numpy_ndarray)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType("numpy", "ufunc", sizeof(PyUFuncObject), 0); if (unlikely(!__pyx_ptype_5numpy_ufunc)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 864; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_7sklearn_4tree_10_criterion_Criterion = __Pyx_ImportType("sklearn.tree._criterion", "Criterion", sizeof(struct __pyx_obj_7sklearn_4tree_10_criterion_Criterion), 1); if (unlikely(!__pyx_ptype_7sklearn_4tree_10_criterion_Criterion)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 21; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_7sklearn_4tree_10_criterion_Criterion = (struct __pyx_vtabstruct_7sklearn_4tree_10_criterion_Criterion*)__Pyx_GetVtable(__pyx_ptype_7sklearn_4tree_10_criterion_Criterion->tp_dict); if (unlikely(!__pyx_vtabptr_7sklearn_4tree_10_criterion_Criterion)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 21; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_7sklearn_4tree_6_utils_Stack = __Pyx_ImportType("sklearn.tree._utils", "Stack", sizeof(struct __pyx_obj_7sklearn_4tree_6_utils_Stack), 1); if (unlikely(!__pyx_ptype_7sklearn_4tree_6_utils_Stack)) {__pyx_filename = __pyx_f[5]; __pyx_lineno = 67; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_7sklearn_4tree_6_utils_Stack = (struct __pyx_vtabstruct_7sklearn_4tree_6_utils_Stack*)__Pyx_GetVtable(__pyx_ptype_7sklearn_4tree_6_utils_Stack->tp_dict); if (unlikely(!__pyx_vtabptr_7sklearn_4tree_6_utils_Stack)) {__pyx_filename = __pyx_f[5]; __pyx_lineno = 67; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_7sklearn_4tree_6_utils_PriorityHeap = __Pyx_ImportType("sklearn.tree._utils", "PriorityHeap", sizeof(struct __pyx_obj_7sklearn_4tree_6_utils_PriorityHeap), 1); if (unlikely(!__pyx_ptype_7sklearn_4tree_6_utils_PriorityHeap)) {__pyx_filename = __pyx_f[5]; __pyx_lineno = 96; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_7sklearn_4tree_6_utils_PriorityHeap = (struct __pyx_vtabstruct_7sklearn_4tree_6_utils_PriorityHeap*)__Pyx_GetVtable(__pyx_ptype_7sklearn_4tree_6_utils_PriorityHeap->tp_dict); if (unlikely(!__pyx_vtabptr_7sklearn_4tree_6_utils_PriorityHeap)) {__pyx_filename = __pyx_f[5]; __pyx_lineno = 96; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  __pyx_t_1 = __Pyx_ImportModule("sklearn.tree._utils"); if (!__pyx_t_1) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "rand_int", (void (**)(void))&__pyx_f_7sklearn_4tree_6_utils_rand_int, "__pyx_t_7sklearn_4tree_6_utils_SIZE_t (__pyx_t_7sklearn_4tree_6_utils_SIZE_t, __pyx_t_7sklearn_4tree_6_utils_SIZE_t, __pyx_t_7sklearn_4tree_6_utils_UINT32_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "rand_uniform", (void (**)(void))&__pyx_f_7sklearn_4tree_6_utils_rand_uniform, "double (double, double, __pyx_t_7sklearn_4tree_6_utils_UINT32_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "log", (void (**)(void))&__pyx_f_7sklearn_4tree_6_utils_log, "double (double)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "__pyx_fuse_0safe_realloc", (void (**)(void))&__pyx_fuse_0__pyx_f_7sklearn_4tree_6_utils_safe_realloc, "__pyx_t_7sklearn_4tree_6_utils_DTYPE_t *(__pyx_t_7sklearn_4tree_6_utils_DTYPE_t **, size_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "__pyx_fuse_1safe_realloc", (void (**)(void))&__pyx_fuse_1__pyx_f_7sklearn_4tree_6_utils_safe_realloc, "__pyx_t_7sklearn_4tree_6_utils_SIZE_t *(__pyx_t_7sklearn_4tree_6_utils_SIZE_t **, size_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "__pyx_fuse_2safe_realloc", (void (**)(void))&__pyx_fuse_2__pyx_f_7sklearn_4tree_6_utils_safe_realloc, "unsigned char *(unsigned char **, size_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  /*--- Execution code ---*/

  /* "sklearn/tree/_splitter.pyx":25
 * from libc.string cimport memset
 * 
 * import numpy as np             # <<<<<<<<<<<<<<
 * cimport numpy as np
 * np.import_array()
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_numpy, 0, -1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "sklearn/tree/_splitter.pyx":27
 * import numpy as np
 * cimport numpy as np
 * np.import_array()             # <<<<<<<<<<<<<<
 * 
 * from scipy.sparse import csc_matrix
 */
  import_array();

  /* "sklearn/tree/_splitter.pyx":29
 * np.import_array()
 * 
 * from scipy.sparse import csc_matrix             # <<<<<<<<<<<<<<
 * 
 * from ._utils cimport log
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 29; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_csc_matrix);
  __Pyx_GIVEREF(__pyx_n_s_csc_matrix);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_csc_matrix);
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_scipy_sparse, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 29; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_csc_matrix); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 29; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_csc_matrix, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 29; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "sklearn/tree/_splitter.pyx":37
 * from ._utils cimport safe_realloc
 * 
 * cdef double INFINITY = np.inf             # <<<<<<<<<<<<<<
 * 
 * # Mitigate precision differences between 32 bit and 64 bit
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 37; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_inf); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 37; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 37; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_7sklearn_4tree_9_splitter_INFINITY = __pyx_t_4;

  /* "sklearn/tree/_splitter.pyx":40
 * 
 * # Mitigate precision differences between 32 bit and 64 bit
 * cdef DTYPE_t FEATURE_THRESHOLD = 1e-7             # <<<<<<<<<<<<<<
 * 
 * # Constant to switch between algorithm non zero value extract algorithm
 */
  __pyx_v_7sklearn_4tree_9_splitter_FEATURE_THRESHOLD = 1e-7;

  /* "sklearn/tree/_splitter.pyx":44
 * # Constant to switch between algorithm non zero value extract algorithm
 * # in SparseSplitter
 * cdef DTYPE_t EXTRACT_NNZ_SWITCH = 0.1             # <<<<<<<<<<<<<<
 * 
 * cdef inline void _init_split(SplitRecord* self, SIZE_t start_pos) nogil:
 */
  __pyx_v_7sklearn_4tree_9_splitter_EXTRACT_NNZ_SWITCH = 0.1;

  /* "sklearn/tree/_splitter.pyx":1
 * # cython: cdivision=True             # <<<<<<<<<<<<<<
 * # cython: boundscheck=False
 * # cython: wraparound=False
 */
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "../../anaconda/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":979
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init sklearn.tree._splitter", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init sklearn.tree._splitter");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* --- Runtime support code --- */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

static void __Pyx_RaiseArgumentTypeInvalid(const char* name, PyObject *obj, PyTypeObject *type) {
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
}
static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (none_allowed && obj == Py_None) return 1;
    else if (exact) {
        if (likely(Py_TYPE(obj) == type)) return 1;
        #if PY_MAJOR_VERSION == 2
        else if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(PyObject_TypeCheck(obj, type))) return 1;
    }
    __Pyx_RaiseArgumentTypeInvalid(name, obj, type);
    return 0;
}

static CYTHON_INLINE int __Pyx_IsLittleEndian(void) {
  unsigned int n = 1;
  return *(unsigned char*)(&n) != 0;
}
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type) {
  stack[0].field = &ctx->root;
  stack[0].parent_offset = 0;
  ctx->root.type = type;
  ctx->root.name = "buffer dtype";
  ctx->root.offset = 0;
  ctx->head = stack;
  ctx->head->field = &ctx->root;
  ctx->fmt_offset = 0;
  ctx->head->parent_offset = 0;
  ctx->new_packmode = '@';
  ctx->enc_packmode = '@';
  ctx->new_count = 1;
  ctx->enc_count = 0;
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  ctx->is_valid_array = 0;
  ctx->struct_alignment = 0;
  while (type->typegroup == 'S') {
    ++ctx->head;
    ctx->head->field = type->fields;
    ctx->head->parent_offset = 0;
    type = type->fields->type;
  }
}
static int __Pyx_BufFmt_ParseNumber(const char** ts) {
    int count;
    const char* t = *ts;
    if (*t < '0' || *t > '9') {
      return -1;
    } else {
        count = *t++ - '0';
        while (*t >= '0' && *t < '9') {
            count *= 10;
            count += *t++ - '0';
        }
    }
    *ts = t;
    return count;
}
static int __Pyx_BufFmt_ExpectNumber(const char **ts) {
    int number = __Pyx_BufFmt_ParseNumber(ts);
    if (number == -1)
        PyErr_Format(PyExc_ValueError,\
                     "Does not understand character buffer dtype format string ('%c')", **ts);
    return number;
}
static void __Pyx_BufFmt_RaiseUnexpectedChar(char ch) {
  PyErr_Format(PyExc_ValueError,
               "Unexpected format string character: '%c'", ch);
}
static const char* __Pyx_BufFmt_DescribeTypeChar(char ch, int is_complex) {
  switch (ch) {
    case 'c': return "'char'";
    case 'b': return "'signed char'";
    case 'B': return "'unsigned char'";
    case 'h': return "'short'";
    case 'H': return "'unsigned short'";
    case 'i': return "'int'";
    case 'I': return "'unsigned int'";
    case 'l': return "'long'";
    case 'L': return "'unsigned long'";
    case 'q': return "'long long'";
    case 'Q': return "'unsigned long long'";
    case 'f': return (is_complex ? "'complex float'" : "'float'");
    case 'd': return (is_complex ? "'complex double'" : "'double'");
    case 'g': return (is_complex ? "'complex long double'" : "'long double'");
    case 'T': return "a struct";
    case 'O': return "Python object";
    case 'P': return "a pointer";
    case 's': case 'p': return "a string";
    case 0: return "end";
    default: return "unparseable format string";
  }
}
static size_t __Pyx_BufFmt_TypeCharToStandardSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return 2;
    case 'i': case 'I': case 'l': case 'L': return 4;
    case 'q': case 'Q': return 8;
    case 'f': return (is_complex ? 8 : 4);
    case 'd': return (is_complex ? 16 : 8);
    case 'g': {
      PyErr_SetString(PyExc_ValueError, "Python does not define a standard format string size for long double ('g')..");
      return 0;
    }
    case 'O': case 'P': return sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static size_t __Pyx_BufFmt_TypeCharToNativeSize(char ch, int is_complex) {
  switch (ch) {
    case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(short);
    case 'i': case 'I': return sizeof(int);
    case 'l': case 'L': return sizeof(long);
    #ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(PY_LONG_LONG);
    #endif
    case 'f': return sizeof(float) * (is_complex ? 2 : 1);
    case 'd': return sizeof(double) * (is_complex ? 2 : 1);
    case 'g': return sizeof(long double) * (is_complex ? 2 : 1);
    case 'O': case 'P': return sizeof(void*);
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
typedef struct { char c; short x; } __Pyx_st_short;
typedef struct { char c; int x; } __Pyx_st_int;
typedef struct { char c; long x; } __Pyx_st_long;
typedef struct { char c; float x; } __Pyx_st_float;
typedef struct { char c; double x; } __Pyx_st_double;
typedef struct { char c; long double x; } __Pyx_st_longdouble;
typedef struct { char c; void *x; } __Pyx_st_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { char c; PY_LONG_LONG x; } __Pyx_st_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToAlignment(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_st_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_st_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_st_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_st_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_st_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_st_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_st_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_st_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
/* These are for computing the padding at the end of the struct to align
   on the first member of the struct. This will probably the same as above,
   but we don't have any guarantees.
 */
typedef struct { short x; char c; } __Pyx_pad_short;
typedef struct { int x; char c; } __Pyx_pad_int;
typedef struct { long x; char c; } __Pyx_pad_long;
typedef struct { float x; char c; } __Pyx_pad_float;
typedef struct { double x; char c; } __Pyx_pad_double;
typedef struct { long double x; char c; } __Pyx_pad_longdouble;
typedef struct { void *x; char c; } __Pyx_pad_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { PY_LONG_LONG x; char c; } __Pyx_pad_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToPadding(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_pad_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_pad_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_pad_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_pad_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_pad_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_pad_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_pad_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_pad_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static char __Pyx_BufFmt_TypeCharToGroup(char ch, int is_complex) {
  switch (ch) {
    case 'c':
        return 'H';
    case 'b': case 'h': case 'i':
    case 'l': case 'q': case 's': case 'p':
        return 'I';
    case 'B': case 'H': case 'I': case 'L': case 'Q':
        return 'U';
    case 'f': case 'd': case 'g':
        return (is_complex ? 'C' : 'R');
    case 'O':
        return 'O';
    case 'P':
        return 'P';
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
static void __Pyx_BufFmt_RaiseExpected(__Pyx_BufFmt_Context* ctx) {
  if (ctx->head == NULL || ctx->head->field == &ctx->root) {
    const char* expected;
    const char* quote;
    if (ctx->head == NULL) {
      expected = "end";
      quote = "";
    } else {
      expected = ctx->head->field->type->name;
      quote = "'";
    }
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected %s%s%s but got %s",
                 quote, expected, quote,
                 __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex));
  } else {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_StructField* parent = (ctx->head - 1)->field;
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected '%s' but got %s in '%s.%s'",
                 field->type->name, __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex),
                 parent->type->name, field->name);
  }
}
static int __Pyx_BufFmt_ProcessTypeChunk(__Pyx_BufFmt_Context* ctx) {
  char group;
  size_t size, offset, arraysize = 1;
  if (ctx->enc_type == 0) return 0;
  if (ctx->head->field->type->arraysize[0]) {
    int i, ndim = 0;
    if (ctx->enc_type == 's' || ctx->enc_type == 'p') {
        ctx->is_valid_array = ctx->head->field->type->ndim == 1;
        ndim = 1;
        if (ctx->enc_count != ctx->head->field->type->arraysize[0]) {
            PyErr_Format(PyExc_ValueError,
                         "Expected a dimension of size %zu, got %zu",
                         ctx->head->field->type->arraysize[0], ctx->enc_count);
            return -1;
        }
    }
    if (!ctx->is_valid_array) {
      PyErr_Format(PyExc_ValueError, "Expected %d dimensions, got %d",
                   ctx->head->field->type->ndim, ndim);
      return -1;
    }
    for (i = 0; i < ctx->head->field->type->ndim; i++) {
      arraysize *= ctx->head->field->type->arraysize[i];
    }
    ctx->is_valid_array = 0;
    ctx->enc_count = 1;
  }
  group = __Pyx_BufFmt_TypeCharToGroup(ctx->enc_type, ctx->is_complex);
  do {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_TypeInfo* type = field->type;
    if (ctx->enc_packmode == '@' || ctx->enc_packmode == '^') {
      size = __Pyx_BufFmt_TypeCharToNativeSize(ctx->enc_type, ctx->is_complex);
    } else {
      size = __Pyx_BufFmt_TypeCharToStandardSize(ctx->enc_type, ctx->is_complex);
    }
    if (ctx->enc_packmode == '@') {
      size_t align_at = __Pyx_BufFmt_TypeCharToAlignment(ctx->enc_type, ctx->is_complex);
      size_t align_mod_offset;
      if (align_at == 0) return -1;
      align_mod_offset = ctx->fmt_offset % align_at;
      if (align_mod_offset > 0) ctx->fmt_offset += align_at - align_mod_offset;
      if (ctx->struct_alignment == 0)
          ctx->struct_alignment = __Pyx_BufFmt_TypeCharToPadding(ctx->enc_type,
                                                                 ctx->is_complex);
    }
    if (type->size != size || type->typegroup != group) {
      if (type->typegroup == 'C' && type->fields != NULL) {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        ++ctx->head;
        ctx->head->field = type->fields;
        ctx->head->parent_offset = parent_offset;
        continue;
      }
      if ((type->typegroup == 'H' || group == 'H') && type->size == size) {
      } else {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
      }
    }
    offset = ctx->head->parent_offset + field->offset;
    if (ctx->fmt_offset != offset) {
      PyErr_Format(PyExc_ValueError,
                   "Buffer dtype mismatch; next field is at offset %" CYTHON_FORMAT_SSIZE_T "d but %" CYTHON_FORMAT_SSIZE_T "d expected",
                   (Py_ssize_t)ctx->fmt_offset, (Py_ssize_t)offset);
      return -1;
    }
    ctx->fmt_offset += size;
    if (arraysize)
      ctx->fmt_offset += (arraysize - 1) * size;
    --ctx->enc_count;
    while (1) {
      if (field == &ctx->root) {
        ctx->head = NULL;
        if (ctx->enc_count != 0) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
        }
        break;
      }
      ctx->head->field = ++field;
      if (field->type == NULL) {
        --ctx->head;
        field = ctx->head->field;
        continue;
      } else if (field->type->typegroup == 'S') {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        if (field->type->fields->type == NULL) continue;
        field = field->type->fields;
        ++ctx->head;
        ctx->head->field = field;
        ctx->head->parent_offset = parent_offset;
        break;
      } else {
        break;
      }
    }
  } while (ctx->enc_count);
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  return 0;
}
static CYTHON_INLINE PyObject *
__pyx_buffmt_parse_array(__Pyx_BufFmt_Context* ctx, const char** tsp)
{
    const char *ts = *tsp;
    int i = 0, number;
    int ndim = ctx->head->field->type->ndim;
;
    ++ts;
    if (ctx->new_count != 1) {
        PyErr_SetString(PyExc_ValueError,
                        "Cannot handle repeated arrays in format string");
        return NULL;
    }
    if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
    while (*ts && *ts != ')') {
        switch (*ts) {
            case ' ': case '\f': case '\r': case '\n': case '\t': case '\v':  continue;
            default:  break;
        }
        number = __Pyx_BufFmt_ExpectNumber(&ts);
        if (number == -1) return NULL;
        if (i < ndim && (size_t) number != ctx->head->field->type->arraysize[i])
            return PyErr_Format(PyExc_ValueError,
                        "Expected a dimension of size %zu, got %d",
                        ctx->head->field->type->arraysize[i], number);
        if (*ts != ',' && *ts != ')')
            return PyErr_Format(PyExc_ValueError,
                                "Expected a comma in format string, got '%c'", *ts);
        if (*ts == ',') ts++;
        i++;
    }
    if (i != ndim)
        return PyErr_Format(PyExc_ValueError, "Expected %d dimension(s), got %d",
                            ctx->head->field->type->ndim, i);
    if (!*ts) {
        PyErr_SetString(PyExc_ValueError,
                        "Unexpected end of format string, expected ')'");
        return NULL;
    }
    ctx->is_valid_array = 1;
    ctx->new_count = 1;
    *tsp = ++ts;
    return Py_None;
}
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts) {
  int got_Z = 0;
  while (1) {
    switch(*ts) {
      case 0:
        if (ctx->enc_type != 0 && ctx->head == NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        if (ctx->head != NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        return ts;
      case ' ':
      case '\r':
      case '\n':
        ++ts;
        break;
      case '<':
        if (!__Pyx_IsLittleEndian()) {
          PyErr_SetString(PyExc_ValueError, "Little-endian buffer not supported on big-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '>':
      case '!':
        if (__Pyx_IsLittleEndian()) {
          PyErr_SetString(PyExc_ValueError, "Big-endian buffer not supported on little-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '=':
      case '@':
      case '^':
        ctx->new_packmode = *ts++;
        break;
      case 'T':
        {
          const char* ts_after_sub;
          size_t i, struct_count = ctx->new_count;
          size_t struct_alignment = ctx->struct_alignment;
          ctx->new_count = 1;
          ++ts;
          if (*ts != '{') {
            PyErr_SetString(PyExc_ValueError, "Buffer acquisition: Expected '{' after 'T'");
            return NULL;
          }
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          ctx->enc_count = 0;
          ctx->struct_alignment = 0;
          ++ts;
          ts_after_sub = ts;
          for (i = 0; i != struct_count; ++i) {
            ts_after_sub = __Pyx_BufFmt_CheckString(ctx, ts);
            if (!ts_after_sub) return NULL;
          }
          ts = ts_after_sub;
          if (struct_alignment) ctx->struct_alignment = struct_alignment;
        }
        break;
      case '}':
        {
          size_t alignment = ctx->struct_alignment;
          ++ts;
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          if (alignment && ctx->fmt_offset % alignment) {
            ctx->fmt_offset += alignment - (ctx->fmt_offset % alignment);
          }
        }
        return ts;
      case 'x':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->fmt_offset += ctx->new_count;
        ctx->new_count = 1;
        ctx->enc_count = 0;
        ctx->enc_type = 0;
        ctx->enc_packmode = ctx->new_packmode;
        ++ts;
        break;
      case 'Z':
        got_Z = 1;
        ++ts;
        if (*ts != 'f' && *ts != 'd' && *ts != 'g') {
          __Pyx_BufFmt_RaiseUnexpectedChar('Z');
          return NULL;
        }
      case 'c': case 'b': case 'B': case 'h': case 'H': case 'i': case 'I':
      case 'l': case 'L': case 'q': case 'Q':
      case 'f': case 'd': case 'g':
      case 'O': case 'p':
        if (ctx->enc_type == *ts && got_Z == ctx->is_complex &&
            ctx->enc_packmode == ctx->new_packmode) {
          ctx->enc_count += ctx->new_count;
          ctx->new_count = 1;
          got_Z = 0;
          ++ts;
          break;
        }
      case 's':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->enc_count = ctx->new_count;
        ctx->enc_packmode = ctx->new_packmode;
        ctx->enc_type = *ts;
        ctx->is_complex = got_Z;
        ++ts;
        ctx->new_count = 1;
        got_Z = 0;
        break;
      case ':':
        ++ts;
        while(*ts != ':') ++ts;
        ++ts;
        break;
      case '(':
        if (!__pyx_buffmt_parse_array(ctx, &ts)) return NULL;
        break;
      default:
        {
          int number = __Pyx_BufFmt_ExpectNumber(&ts);
          if (number == -1) return NULL;
          ctx->new_count = (size_t)number;
        }
    }
  }
}
static CYTHON_INLINE void __Pyx_ZeroBuffer(Py_buffer* buf) {
  buf->buf = NULL;
  buf->obj = NULL;
  buf->strides = __Pyx_zeros;
  buf->shape = __Pyx_zeros;
  buf->suboffsets = __Pyx_minusones;
}
static CYTHON_INLINE int __Pyx_GetBufferAndValidate(
        Py_buffer* buf, PyObject* obj,  __Pyx_TypeInfo* dtype, int flags,
        int nd, int cast, __Pyx_BufFmt_StackElem* stack)
{
  if (obj == Py_None || obj == NULL) {
    __Pyx_ZeroBuffer(buf);
    return 0;
  }
  buf->buf = NULL;
  if (__Pyx_GetBuffer(obj, buf, flags) == -1) goto fail;
  if (buf->ndim != nd) {
    PyErr_Format(PyExc_ValueError,
                 "Buffer has wrong number of dimensions (expected %d, got %d)",
                 nd, buf->ndim);
    goto fail;
  }
  if (!cast) {
    __Pyx_BufFmt_Context ctx;
    __Pyx_BufFmt_Init(&ctx, stack, dtype);
    if (!__Pyx_BufFmt_CheckString(&ctx, buf->format)) goto fail;
  }
  if ((unsigned)buf->itemsize != dtype->size) {
    PyErr_Format(PyExc_ValueError,
      "Item size of buffer (%" CYTHON_FORMAT_SSIZE_T "d byte%s) does not match size of '%s' (%" CYTHON_FORMAT_SSIZE_T "d byte%s)",
      buf->itemsize, (buf->itemsize > 1) ? "s" : "",
      dtype->name, (Py_ssize_t)dtype->size, (dtype->size > 1) ? "s" : "");
    goto fail;
  }
  if (buf->suboffsets == NULL) buf->suboffsets = __Pyx_minusones;
  return 0;
fail:;
  __Pyx_ZeroBuffer(buf);
  return -1;
}
static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info) {
  if (info->buf == NULL) return;
  if (info->suboffsets == __Pyx_minusones) info->suboffsets = NULL;
  __Pyx_ReleaseBuffer(info);
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (wraparound & unlikely(i < 0)) i += PyList_GET_SIZE(o);
    if ((!boundscheck) || likely((0 <= i) & (i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (wraparound & unlikely(i < 0)) i += PyTuple_GET_SIZE(o);
    if ((!boundscheck) || likely((0 <= i) & (i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (PyErr_ExceptionMatches(PyExc_OverflowError))
                        PyErr_Clear();
                    else
                        return NULL;
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_Restore(type, value, tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(type, value, tb);
#endif
}

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(PyObject_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject* args = PyTuple_Pack(1, arg);
    return (likely(args)) ? __Pyx_PyObject_Call(func, args, NULL) : NULL;
}
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
#if PY_VERSION_HEX >= 0x03030000
    if (cause) {
#else
    if (cause && cause != Py_None) {
#endif
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = (start + end) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        __pyx_d,      /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    py_frame->f_lineno = py_line;
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_VERSION_HEX < 0x03030000
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                #if PY_VERSION_HEX < 0x03030000
                PyObject *py_level = PyInt_FromLong(1);
                if (!py_level)
                    goto bad;
                module = PyObject_CallFunctionObjArgs(py_import,
                    name, global_dict, empty_dict, list, py_level, NULL);
                Py_DECREF(py_level);
                #else
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                #endif
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_VERSION_HEX < 0x03030000
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_VERSION_HEX < 0x03030000
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

#if PY_MAJOR_VERSION < 3
static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags) {
    if (PyObject_CheckBuffer(obj)) return PyObject_GetBuffer(obj, view, flags);
        if (PyObject_TypeCheck(obj, __pyx_ptype_5numpy_ndarray)) return __pyx_pw_5numpy_7ndarray_1__getbuffer__(obj, view, flags);
    PyErr_Format(PyExc_TypeError, "'%.200s' does not have the buffer interface", Py_TYPE(obj)->tp_name);
    return -1;
}
static void __Pyx_ReleaseBuffer(Py_buffer *view) {
    PyObject *obj = view->obj;
    if (!obj) return;
    if (PyObject_CheckBuffer(obj)) {
        PyBuffer_Release(view);
        return;
    }
        if (PyObject_TypeCheck(obj, __pyx_ptype_5numpy_ndarray)) { __pyx_pw_5numpy_7ndarray_3__releasebuffer__(obj, view); return; }
    Py_DECREF(obj);
    view->obj = NULL;
}
#endif


          #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)       \
    {                                                                     \
        func_type value = func_value;                                     \
        if (sizeof(target_type) < sizeof(func_type)) {                    \
            if (unlikely(value != (func_type) (target_type) value)) {     \
                func_type zero = 0;                                       \
                if (is_unsigned && unlikely(value < zero))                \
                    goto raise_neg_overflow;                              \
                else                                                      \
                    goto raise_overflow;                                  \
            }                                                             \
        }                                                                 \
        return (target_type) value;                                       \
    }

#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
 #endif
#endif

static CYTHON_INLINE Py_intptr_t __Pyx_PyInt_As_Py_intptr_t(PyObject *x) {
    const Py_intptr_t neg_one = (Py_intptr_t) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(Py_intptr_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (Py_intptr_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(Py_intptr_t, digit, ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (Py_intptr_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(Py_intptr_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(Py_intptr_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(Py_intptr_t,  digit, +(((PyLongObject*)x)->ob_digit[0]));
                case -1: __PYX_VERIFY_RETURN_INT(Py_intptr_t, sdigit, -(sdigit) ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
            if (sizeof(Py_intptr_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, PyLong_AsLong(x))
            } else if (sizeof(Py_intptr_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT(Py_intptr_t, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            Py_intptr_t val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (Py_intptr_t) -1;
        }
    } else {
        Py_intptr_t val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (Py_intptr_t) -1;
        val = __Pyx_PyInt_As_Py_intptr_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to Py_intptr_t");
    return (Py_intptr_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to Py_intptr_t");
    return (Py_intptr_t) -1;
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE npy_uint32 __Pyx_PyInt_As_npy_uint32(PyObject *x) {
    const npy_uint32 neg_one = (npy_uint32) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(npy_uint32) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(npy_uint32, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (npy_uint32) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(npy_uint32, digit, ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (npy_uint32) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(npy_uint32) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT(npy_uint32, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(npy_uint32) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT(npy_uint32, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(npy_uint32,  digit, +(((PyLongObject*)x)->ob_digit[0]));
                case -1: __PYX_VERIFY_RETURN_INT(npy_uint32, sdigit, -(sdigit) ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
            if (sizeof(npy_uint32) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT(npy_uint32, long, PyLong_AsLong(x))
            } else if (sizeof(npy_uint32) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT(npy_uint32, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            npy_uint32 val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (npy_uint32) -1;
        }
    } else {
        npy_uint32 val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (npy_uint32) -1;
        val = __Pyx_PyInt_As_npy_uint32(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to npy_uint32");
    return (npy_uint32) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to npy_uint32");
    return (npy_uint32) -1;
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Py_intptr_t(Py_intptr_t value) {
    const Py_intptr_t neg_one = (Py_intptr_t) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(Py_intptr_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(Py_intptr_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(Py_intptr_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(Py_intptr_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(Py_intptr_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(Py_intptr_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_npy_int32(npy_int32 value) {
    const npy_int32 neg_one = (npy_int32) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(npy_int32) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(npy_int32) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(npy_int32) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(npy_int32) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(npy_int32) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(npy_int32),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE npy_int32 __Pyx_PyInt_As_npy_int32(PyObject *x) {
    const npy_int32 neg_one = (npy_int32) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(npy_int32) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(npy_int32, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (npy_int32) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(npy_int32, digit, ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (npy_int32) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(npy_int32) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT(npy_int32, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(npy_int32) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT(npy_int32, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(npy_int32,  digit, +(((PyLongObject*)x)->ob_digit[0]));
                case -1: __PYX_VERIFY_RETURN_INT(npy_int32, sdigit, -(sdigit) ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
            if (sizeof(npy_int32) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT(npy_int32, long, PyLong_AsLong(x))
            } else if (sizeof(npy_int32) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT(npy_int32, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            npy_int32 val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (npy_int32) -1;
        }
    } else {
        npy_int32 val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (npy_int32) -1;
        val = __Pyx_PyInt_As_npy_int32(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to npy_int32");
    return (npy_int32) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to npy_int32");
    return (npy_int32) -1;
}

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eqf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sumf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_difff(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prodf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quotf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        float denom = b.real * b.real + b.imag * b.imag;
        z.real = (a.real * b.real + a.imag * b.imag) / denom;
        z.imag = (a.imag * b.real - a.real * b.imag) / denom;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_negf(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zerof(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conjf(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_absf(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_powf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(a, a);
                    case 3:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(z, a);
                    case 4:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                }
                r = a.real;
                theta = 0;
            } else {
                r = __Pyx_c_absf(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        double denom = b.real * b.real + b.imag * b.imag;
        z.real = (a.real * b.real + a.imag * b.imag) / denom;
        z.imag = (a.imag * b.real - a.real * b.imag) / denom;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(a, a);
                    case 3:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(z, a);
                    case 4:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                }
                r = a.real;
                theta = 0;
            } else {
                r = __Pyx_c_abs(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT(int, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +(((PyLongObject*)x)->ob_digit[0]));
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, -(sdigit) ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT(int, long, PyLong_AsLong(x))
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT(int, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT(long, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +(((PyLongObject*)x)->ob_digit[0]));
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, -(sdigit) ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT(long, long, PyLong_AsLong(x))
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT(long, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

#ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility",
            module_name, class_name);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling",
            module_name, class_name);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

#ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
#if PY_VERSION_HEX < 0x03030000
        char* defenc_c;
        PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
        if (!defenc) return NULL;
        defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        {
            char* end = defenc_c + PyBytes_GET_SIZE(defenc);
            char* c;
            for (c = defenc_c; c < end; c++) {
                if ((unsigned char) (*c) >= 128) {
                    PyUnicode_AsASCIIString(o);
                    return NULL;
                }
            }
        }
#endif
        *length = PyBytes_GET_SIZE(defenc);
        return defenc_c;
#else
        if (__Pyx_PyUnicode_READY(o) == -1) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        if (PyUnicode_IS_ASCII(o)) {
            *length = PyUnicode_GET_LENGTH(o);
            return PyUnicode_AsUTF8(o);
        } else {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
#else
        return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
    } else
#endif
#if !CYTHON_COMPILING_IN_PYPY
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x) {
  PyNumberMethods *m;
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return Py_INCREF(x), x;
  m = Py_TYPE(x)->tp_as_number;
#if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
#else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
#endif
  if (res) {
#if PY_MAJOR_VERSION < 3
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%.4s__ returned non-%.4s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b)))
      return PyInt_AS_LONG(b);
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
     #if CYTHON_USE_PYLONG_INTERNALS
       switch (Py_SIZE(b)) {
       case -1: return -(sdigit)((PyLongObject*)b)->ob_digit[0];
       case  0: return 0;
       case  1: return ((PyLongObject*)b)->ob_digit[0];
       }
     #endif
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
